/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	nfx-defs.h
 *
 * RCS:  $Id: nfx-defs.h,v 1.74 2002/11/21 19:26:34 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

#ifndef _NFX_DEFS_H
#define _NFX_DEFS_H

/*
 * The *most* important piece of information is this entire file is this:
 * The name of the product.
 */
#define PRODUCT_NAME "ClariFiler"

#ifndef NULL
#define NULL				0
#endif

#ifndef TRUE
#define TRUE                1
#define FALSE               0
#endif

/*
 * Generic return codes, see nfx-error.h for module specific ranges
 */
#define NFX_OK				0
#define NFX_ERR				-1

/*
 * MIN/MAX
 */
#if !defined(MIN)
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define MAX(a,b) (((a) < (b)) ? (b) : (a))
#endif


/* Ethernet types */
#define EEE_ETH_TYPE 		0x0022
#define EEE_ETH_TYPE_SAC 	0x0F22

/*
 * Class of Service levels
 */
#define NFX_COS_HIGH		0
#define NFX_COS_LOW			1

/*
 * Card type definitions
 */
#define NFX_CARDTYPE_NUM	3

#define NFX_CARDTYPE_SSC	0
#define NFX_CARDTYPE_NFP	1
#define NFX_CARDTYPE_FCNIM	2

/*
 * Slot definitions
 */
#define NFX_NUM_SLOTS_PER_CHASSIS	12
#define NFX_SLOT_MIN				0
#define NFX_SLOT_MAX				11

#define NFX_SLOTTYPE_SSC            0
#define NFX_SLOTTYPE_NFP            1
#define NFX_SLOTTYPE_FC   	        2
#define NFX_SLOTTYPE_UNKNOWN        (-1)

/*
 * Chassis definitions
 */
#define NFX_NUM_CH_PER_CLUSTER      2

/*
 * CPU identifiers
 */
#define NFX_MAX_CPUS_PER_SLOT       16

/* NFP cpu types */
#define NFX_NFP_NUM_CPUS            4   /* on physical card */
#define NFX_NFP_TXRX1               0   /* for mgmt plane traffic */
#define NFX_NFP_TXRX2               1   /* for mgmt plane traffic */
#define NFX_NFP_FP1                 2   /* for mgmt plane traffic */
#define NFX_NFP_FP2                 3   /* for mgmt plane traffic */
#define NFX_NFP_TXRX_ANY            4   /* for data plane traffic */
#define NFX_NFP_FP_ANY              5   /* for data plane traffic */

/* FC cpu types */
#define NFX_FCNIM_NUM_CPUS          4   /* on physical card */
/* nfxMyCPUId=3/fc slice 0, is the control cpu */
#define NFX_FCNIM_CTRL_CPU          NFX_FCNIM_CPUID3
#define NFX_FCNIM_CPUID3            3   /* nfxMyCPUId=3 is fc slice 0 */
#define NFX_FCNIM_CPUID2            2   /* nfxMyCPUId=2 is fc slice 1 */
#define NFX_FCNIM_CPUID1            1   /* nfxMyCPUId=1 is fc slice 2 */
#define NFX_FCNIM_CPUID0            0   /* nfxMyCPUId=0 is fc slice 3 */

/* SSC cpu types */
#define NFX_SSC_NUM_CPUS            2   /* on physical card */
#define NFX_SSC_UNIX                0
#define NFX_SSC_MGMT                1

#define NFX_CPUID_UNKNOWN           (-1)

/*
 * Port Identifiers
 */
#define PORT_BEST_FIT				0
#define PORT_LOCAL_APP				1
/*
#if defined(NFX_MOD_NFP)
*/
#define NFP_PORT_TXRX_ETH0			2
#define NFP_PORT_TXRX_ETH1			3
#define NFP_PORT_TXRX_ETH2			4
#define NFP_PORT_FP_ETH0			5
#define NFP_PORT_FP_ETH1			6
#define NFP_PORT_FP_ETH2			7
#define NFP_PORT_MGMT				8
#define NFP_PORT_LOCAL_QUEUE		9
/*
#elif defined(NFX_MOD_FCNIM)
*/
#define FCNIM_PORT_FC				2
#define FCNIM_PORT_BP_DATA			3
#define FCNIM_PORT_BP_MGMT			4
#define FCNIM_PORT_BP_CONTROL		5
/*
#elif defined(NFX_MOD_SSC)
*/
#define SSC_PORT_BP_DATA			2
#define SSC_PORT_BP_MGMT			3
#define SSC_PORT_LOCAL_QUEUE		4
/*
#elif defined(PROM)
*/
#define PROM_PORT_MGMT			2
#define PROM_PORT_NET0			3
#define PROM_PORT_NET1			4
#define PROM_PORT_FP0			5
#define PROM_PORT_FP1			6
#define PROM_PORT_TXRX0			7
#define PROM_PORT_TXRX1			8
#define PROM_PORT_TXRX2			9
#define PROM_PORT_FP2			10
/*
#endif
*/

/*
 * Application Message Type Base Number Assignment
 *  The first word of the application's messages shall be in the range
 *  assigned to that application. This will enable applications to 
 *  differentiate their messages from those of other applications.
 */
#define FS_MSG_BASE		0
#define SCSI_MSG_BASE	1024
#define EVM_MSG_BASE	2048
#define SPM_MSG_BASE	2560
#define NFS_MSG_BASE	3072
#define CIFS_MSG_BASE	4096
#define SBM_MSG_BASE	5120
#define ELAYER_MSG_BASE	6144
#define SDM_MSG_BASE	7168
#define EA_MSG_BASE     8192
#define EA_SPM_MSG_BASE		  8200
#define EA_NTFY_MSG_BASE      8250
#define EA_FSYS_MSG_BASE      8300
#define EA_FSET_MSG_BASE      8350
#define EA_EXPORT_MSG_BASE    8400
#define EA_SNAPS_MSG_BASE     8450
#define EA_CHASSIS_MSG_BASE   8480
#define EA_SDM_MSG_BASE		  8490
#define EA_RAID_MSG_BASE      8500
#define EL_MSG_BASE		8600
#define EV_MSG_BASE     8704
#define REGM_MSG_BASE   9216
#define ELOG_MSG_BASE   9248
#define CCM_MSG_BASE    9312
#define MOUNTD_MSG_BASE 9344 
#define AM_MSG_BASE     9376
#define TAPE_MSG_BASE   9504
#define NDMP_MSG_BASE   9604
#define FILE_MSG_BASE   9704
#define IPMD_MSG_BASE   9804
#define DM_MSG_BASE     9904
#define RCONN_MSG_BASE	9920
#define NCM_MSG_BASE	9950
#define CLUSTER_MSG_BASE    9960
#define EVENT_MSG_BASE  9970
#define FSSF_MSG_BASE	9990
#define BCETH_MSG_BASE	10030
#define NFSSSC_MSG_BASE	10050
#define RAID_MSG_BASE	10112
#define CM_MSG_BASE		10208
#define PROM_UPGRADE_MSG_BASE 10240
#define STATS_MSG_BASE 10250
#define NEXT_MSG_BASE 10300

/*
 * NFX spinlock definitions.
 */
#ifdef NFX_SMP /* TODO - define these functions, just placeholders now. */
#ifdef NFX_SPINLOCK_DEBUG
#define NFX_SPIN_INIT(_lockp, _level)	spinlock_init(_lockp, _level)
#define NFX_SPIN_LOCK(_lockp)			spinlock_getlock(_lockp)
#define NFX_SPIN_UNLOCK(_lockp)			spinlock_unlock(_lockp)
#define NFX_SPIN_TRYLOCK(_lockp)		spinlock_trylock(_lockp)
#else
#define NFX_SPIN_INIT(_lockp, _level)	(_lockp)->lock = 0
#define NFX_SPIN_LOCK(_lockp)			SPINLOCK_LOCK(_lockp)
#define NFX_SPIN_UNLOCK(_lockp)			SPINLOCK_UNLOCK(_lockp)
#define NFX_SPIN_TRYLOCK(_lockp)		SPINLOCK_TRYLOCK(_lockp)
#endif
#else
#define NFX_SPIN_INIT(_lockp, _level)
#define NFX_SPIN_LOCK(_lockp)
#define NFX_SPIN_UNLOCK(_lockp)
#define NFX_SPIN_TRYLOCK(_lockp)		1
#endif

/*------------------------------------------------------------------------------
 * System timeouts and periods.
 *------------------------------------------------------------------------------
 */

#define NFX_GENERAL_IO_TIMOUT   55  /* 10s for link up + */
                                    /* 30s for drive start time + */
                                    /* (1 IO + 2 retries) at 5s each */
#define NFX_EXPRESS_IO_TIMOUT   15  /* 10s for link up + 1 IO at 5s */
#define NFX_VOL_TSTAMP_PERIOD    5  /* >= EXPRESS_IO_TO / 2 to simplify... */
#define NFX_WRITE_GATE_PERIOD   10  /* TSTAMP_PERIOD < x <= 2*TSTAMP_PERIOD */

/*----------------------------------------------------------------------------*/

#define NFX_SW_VER 2

#ifdef PROM
#define NFX_MAX_CONN	128		/* max. # of connections supported */
#else
#define NFX_MAX_CONN	4096	/* max. # of connections supported */
#endif

/*------------------------------------------------------------------------------
 * HW workaround definition.
 *------------------------------------------------------------------------------
 */
/* Temporary WorkARound for fake seep in crasdump are in flash memory */
#define FAKE_SEEP_ON_FLASH_WAR  1

/*------------------------------------------------------------------------------
 * Global Clustering Constants
 *------------------------------------------------------------------------------
 */

#define CLUST_NODE_FAIL_TIMEOUT		60	/* (seconds) */

#endif	/* _NFX_DEFS_H */

