/*
 *-----------------------------------------------------------------
 *
 * Name:	tx3927.h
 *
 * RCS:  $Id: tx3927.h,v 1.4 2002/04/06 02:50:02 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Toshiba TX3927 register map
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _TX3927_H
#define _TX3927_H

/*
 *-----------------------------------------------------------------
 * PIO
 *-----------------------------------------------------------------
 */
#define TX3927_XPIOMASKEXT		0xFFFEF524
#define TX3927_XPIOMASKCPU		0xFFFEF520
#define TX3927_XPIOINT			0xFFFEF51C
#define TX3927_XPIOPOL			0xFFFEF518
#define TX3927_XPIOFLAG1		0xFFFEF514
#define TX3927_XPIOFLAG0		0xFFFEF510
#define TX3927_XPIOOD			0xFFFEF50C
#define TX3927_XPIODIR			0xFFFEF508
#define TX3927_XPIODI			0xFFFEF504
#define TX3927_XPIODO			0xFFFEF500

/*
 *-----------------------------------------------------------------
 * SIO1
 *-----------------------------------------------------------------
 */
#define TX3927_SIRFIFO1			0xFFFEF420
#define TX3927_SITFIFO1			0xFFFEF41C
#define TX3927_SIBGR1			0xFFFEF418
#define TX3927_SIFLCR1			0xFFFEF414
#define TX3927_SIFCR1			0xFFFEF410
#define TX3927_SICISR1			0xFFFEF40C
#define TX3927_SIDISR1			0xFFFEF408
#define TX3927_SIDICR1			0xFFFEF404
#define TX3927_SILCR1			0xFFFEF400

/*
 *-----------------------------------------------------------------
 * SIO0
 *-----------------------------------------------------------------
 */
#define TX3927_SIRFIFO0			0xFFFEF320
#define TX3927_SITFIFO0			0xFFFEF31C
#define TX3927_SIBGR0			0xFFFEF318
#define TX3927_SIFLCR0			0xFFFEF314
#define TX3927_SIFCR0			0xFFFEF310
#define TX3927_SICISR0			0xFFFEF30C
#define TX3927_SIDISR0			0xFFFEF308
#define TX3927_SIDICR0			0xFFFEF304
#define TX3927_SILCR0			0xFFFEF300

/*------------------------------------------------------------------*/
#define TX3927_SIO0_BASE		0xFFFEF300
#define TX3927_SIO1_BASE		0xFFFEF400
#define TX3927_SIO_LCR_OFF		0x0
#define TX3927_SIO_DICR_OFF		0x4
#define TX3927_SIO_DISR_OFF		0x8
#define TX3927_SIO_CISR_OFF		0xC
#define TX3927_SIO_FCR_OFF		0x10
#define TX3927_SIO_FLCR_OFF		0x14
#define TX3927_SIO_BGR_OFF		0x18
#define TX3927_SIO_TFIFO_OFF	0x1C
#define TX3927_SIO_RFIFO_OFF	0x20

#ifdef LANGUAGE_C
typedef struct sio {
	uint32	lcr;
	uint32	dicr;
	uint32	disr;
	uint32	cisr;
	uint32	fcr;
	uint32	flcr;
	uint32	bgr;
	uint32	tfifo;
	uint32	rfifo;
} tx3927_sio_t;
#endif

/*
 *-----------------------------------------------------------------
 * TMR2
 *-----------------------------------------------------------------
 */
#define TX3927_TMTRR2			0xFFFEF2F0
#define TX3927_TMWTMR2			0xFFFEF240
#define TX3927_TMPGMR2			0xFFFEF230
#define TX3927_TMCCDR2			0xFFFEF220
#define TX3927_TMITMR2			0xFFFEF210
#define TX3927_TMCPRB2			0xFFFEF20C
#define TX3927_TMCPRA2			0xFFFEF208
#define TX3927_TMTISR2			0xFFFEF204
#define TX3927_TMTCR2			0xFFFEF200

/*
 *-----------------------------------------------------------------
 * TMR1
 *-----------------------------------------------------------------
 */
#define TX3927_TMTRR1			0xFFFEF1F0
#define TX3927_TMWTMR1			0xFFFEF140
#define TX3927_TMPGMR1			0xFFFEF130
#define TX3927_TMCCDR1			0xFFFEF120
#define TX3927_TMITMR1			0xFFFEF110
#define TX3927_TMCPRB1			0xFFFEF10C
#define TX3927_TMCPRA1			0xFFFEF108
#define TX3927_TMTISR1			0xFFFEF104
#define TX3927_TMTCR1			0xFFFEF100

/*
 *-----------------------------------------------------------------
 * TMR0
 *-----------------------------------------------------------------
 */
#define TX3927_TMTRR0			0xFFFEF0F0
#define TX3927_TMWTMR0			0xFFFEF040
#define TX3927_TMPGMR0			0xFFFEF030
#define TX3927_TMCCDR0			0xFFFEF020
#define TX3927_TMITMR0			0xFFFEF010
#define TX3927_TMCPRB0			0xFFFEF00C
#define TX3927_TMCPRA0			0xFFFEF008
#define TX3927_TMTISR0			0xFFFEF004
#define TX3927_TMTCR0			0xFFFEF000

#define TX3927_TMR0_BASE		0xFFFEF000
#define TX3927_TMR1_BASE		0xFFFEF100
#define TX3927_TMR2_BASE		0xFFFEF200

#ifdef LANGUAGE_C
typedef struct tmr {
	uint32	tcr;
	uint32	tisr;
	uint32	cpra;
	uint32	cprb;
	uint32	itmr;
	uint32	res[3];
	uint32	ccdr;
	uint32	res2[3];
	uint32	pgmr;
	uint32	res3[3];
	uint32	wtmr;
	uint32	res4[3];
	uint32	res5[40];
	uint32	trr;
} tx3927_tmr_t;
#endif

/*
 *-----------------------------------------------------------------
 * CCFG
 *-----------------------------------------------------------------
 */
#define TX3927_PDCR				0xFFFEE010
#define TX3927_TEAR				0xFFFEE00C
#define TX3927_PCFG				0xFFFEE008
#define TX3927_CRIR				0xFFFEE004
#define TX3927_CCFG				0xFFFEE000

/*
 *-----------------------------------------------------------------
 * PCIC
 *-----------------------------------------------------------------
 */
#define TX3927_IPCICBE			0xFFFED158
#define TX3927_IPCIDATA			0xFFFED154
#define TX3927_IPCIADDR			0xFFFED150
#define TX3927_IOMAS			0xFFFED14C
#define TX3927_MMAS				0xFFFED148
#define TX3927_ISCDP			0xFFFED144
#define TX3927_IIADP			0xFFFED140
#define TX3927_ICRD				0xFFFED13C
#define TX3927_ICA				0xFFFED138
#define TX3927_PCISTATIM		0xFFFED134
#define TX3927_LBIM				0xFFFED130
#define TX3927_LBSTAT			0xFFFED12C
#define TX3927_LBC				0xFFFED128
#define TX3927_MBAS				0xFFFED124
#define TX3927_IOBAS			0xFFFED120
#define TX3927_PBACS			0xFFFED11C
#define TX3927_CPCIBGS			0xFFFED118
#define TX3927_CPCIBRS			0xFFFED114
#define TX3927_BM				0xFFFED110
#define TX3927_PBAPMIM			0xFFFED10C
#define TX3927_PBAPMS			0xFFFED108
#define TX3927_PBAPMC			0xFFFED104
#define TX3927_REQ_TRACE		0xFFFED100
#define TX3927_PWMNGSR			0xFFFED0E4
#define TX3927_PWMNGR			0xFFFED0E0
#define TX3927_TBL				0xFFFED0D0
#define TX3927_SC_BE			0xFFFED0CC
#define TX3927_SC_MSG			0xFFFED0C8
#define TX3927_TLBIOMAR			0xFFFED0C4
#define TX3927_TLBMMAR			0xFFFED0C0
#define TX3927_TLBIAP			0xFFFED0BC
#define TX3927_TLBOAP			0xFFFED0B8
#define TX3927_PCIRRDT			0xFFFED0A8
#define TX3927_PCIRRT_CMD		0xFFFED0A4
#define TX3927_PCIRRT			0xFFFED0A0
#define TX3927_TCCMD			0xFFFED09C
#define TX3927_TIM				0xFFFED098
#define TX3927_TSTAT			0xFFFED094
#define TX3927_TC				0xFFFED090
#define TX3927_ILBIOMA			0xFFFED068
#define TX3927_ILBMMAR			0xFFFED064
#define TX3927_IPBIOMAR			0xFFFED060
#define TX3927_IPBMMAR			0xFFFED05C
#define TX3927_RRT				0xFFFED04C
#define TX3927_IIM				0xFFFED048
#define TX3927_ISTAT			0xFFFED044
#define TX3927_IL				0xFFFED03F
#define TX3927_IP				0xFFFED03E
#define TX3927_MG				0xFFFED03D
#define TX3927_ML				0xFFFED03C
#define TX3927_CAPPTR			0xFFFED034
#define TX3927_SSVID			0xFFFED02E
#define TX3927_SVID				0xFFFED02C
#define TX3927_MBA				0xFFFED014
#define TX3927_IOBA				0xFFFED010
#define TX3927_CLS				0xFFFED00F
#define TX3927_MLT				0xFFFED00E
#define TX3927_HT				0xFFFED00D
#define TX3927_RID				0xFFFED00B
#define TX3927_RLPI				0xFFFED00A
#define TX3927_SCC				0xFFFED009
#define TX3927_CC				0xFFFED008
#define TX3927_PCICMD			0xFFFED006
#define TX3927_PCISTAT			0xFFFED004
#define TX3927_VID				0xFFFED002
#define TX3927_DID				0xFFFED000

/*
 *-----------------------------------------------------------------
 * IRC
 *-----------------------------------------------------------------
 */
#define TX3927_IRCSR			0xFFFEC0A0
#define TX3927_IRSSR			0xFFFEC080
#define TX3927_IRSCR			0xFFFEC060
#define TX3927_IRIMR			0xFFFEC040
#define TX3927_IRILR7			0xFFFEC02C
#define TX3927_IRILR6			0xFFFEC028
#define TX3927_IRILR5			0xFFFEC024
#define TX3927_IRILR4			0xFFFEC020
#define TX3927_IRILR3			0xFFFEC01C
#define TX3927_IRILR2			0xFFFEC018
#define TX3927_IRILR1			0xFFFEC014
#define TX3927_IRILR0			0xFFFEC010
#define TX3927_IRDMR1			0xFFFEC008
#define TX3927_IRDMR0			0xFFFEC004
#define TX3927_IRDER			0xFFFEC000

#define TX3927_INT_CONTROL_BASE	0xFFFEC000

#ifdef LANGUAGE_C
typedef struct intcntrl {
	uint32	der;
	uint32	dmr0;
	uint32	dmr1;
	uint32	res;
	uint32	ilr0;
	uint32	ilr1;
	uint32	ilr2;
	uint32	ilr3;
	uint32	ilr4;
	uint32	ilr5;
	uint32	ilr6;
	uint32	ilr7;
	uint32	res2[4];
	uint32	imr;
	uint32	res3[7];
	uint32	scr;
	uint32	res4[7];
	uint32	ssr;
	uint32	res5[7];
	uint32	csr;
} tx3927_intcntrl_t;
#endif

/*
 *-----------------------------------------------------------------
 * DMA
 *-----------------------------------------------------------------
 */
#define TX3927_MCR				0xFFFEB0A4
#define TX3927_TDHR				0xFFFEB0A0
#define TX3927_DBR7				0xFFFEB09C
#define TX3927_DBR6				0xFFFEB098
#define TX3927_DBR5				0xFFFEB094
#define TX3927_DBR4				0xFFFEB090
#define TX3927_DBR3				0xFFFEB08C
#define TX3927_DBR2				0xFFFEB088
#define TX3927_DBR1				0xFFFEB084
#define TX3927_DBR0				0xFFFEB080
#define TX3927_CSR3				0xFFFEB07C
#define TX3927_CCR3				0xFFFEB078
#define TX3927_DAI3				0xFFFEB074
#define TX3927_SAI3				0xFFFEB070
#define TX3927_CNAR3			0xFFFEB06C
#define TX3927_DAR3				0xFFFEB068
#define TX3927_SAR3				0xFFFEB064
#define TX3927_CHAR3			0xFFFEB060
#define TX3927_CSR2				0xFFFEB05C
#define TX3927_CCR2				0xFFFEB058
#define TX3927_DAI2				0xFFFEB054
#define TX3927_SAI2				0xFFFEB050
#define TX3927_CNAR2			0xFFFEB04C
#define TX3927_DAR2				0xFFFEB048
#define TX3927_SAR2				0xFFFEB044
#define TX3927_CHAR2			0xFFFEB040
#define TX3927_CSR1				0xFFFEB03C
#define TX3927_CCR1				0xFFFEB038
#define TX3927_DAI1				0xFFFEB034
#define TX3927_SAI1				0xFFFEB030
#define TX3927_CNAR1			0xFFFEB02C
#define TX3927_DAR1				0xFFFEB028
#define TX3927_SAR1				0xFFFEB024
#define TX3927_CHAR1			0xFFFEB020
#define TX3927_CSR0				0xFFFEB01C
#define TX3927_CCR0				0xFFFEB018
#define TX3927_DAI0				0xFFFEB014
#define TX3927_SAI0				0xFFFEB010
#define TX3927_CNAR0			0xFFFEB00C
#define TX3927_DAR0				0xFFFEB008
#define TX3927_SAR0				0xFFFEB004
#define TX3927_CHAR0			0xFFFEB000

/*
 *-----------------------------------------------------------------
 * ROMC
 *-----------------------------------------------------------------
 */
#define TX3927_RCCR7			0xFFFE901C
#define TX3927_RCCR6			0xFFFE9018
#define TX3927_RCCR5			0xFFFE9014
#define TX3927_RCCR4			0xFFFE9010
#define TX3927_RCCR3			0xFFFE900C
#define TX3927_RCCR2			0xFFFE9008
#define TX3927_RCCR1			0xFFFE9004
#define TX3927_RCCR0			0xFFFE9000

/*
 *-----------------------------------------------------------------
 * SDRAMC
 *-----------------------------------------------------------------
 */
#define TX3927_SDCSMRS2			0xFFFE8034
#define TX3927_SDCSMRS1			0xFFFE8030
#define TX3927_SDCCMD			0xFFFE802C
#define TX3927_SDCTR3			0xFFFE8028
#define TX3927_SDCTR2			0xFFFE8024
#define TX3927_SDCTR1			0xFFFE8020
#define TX3927_SDCCR7			0xFFFE801C
#define TX3927_SDCCR6			0xFFFE8018
#define TX3927_SDCCR5			0xFFFE8014
#define TX3927_SDCCR4			0xFFFE8010
#define TX3927_SDCCR3			0xFFFE800C
#define TX3927_SDCCR2			0xFFFE8008
#define TX3927_SDCCR1			0xFFFE8004
#define TX3927_SDCCR0			0xFFFE8000

#endif	/* _TX3927_H */

