/*
 *-----------------------------------------------------------------
 *
 * Name: makerom.c
 *
 * RCS:  $Id: makerom.c,v 1.6 2002/12/05 21:51:15 lipikab Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: utility for building compressed PROM image.  Also includes
 *				decompression code for verifying build.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/28/01
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#define LANGUAGE_C
#include "../../Includes/sys/amd29f040.h"
#include "../../code/sm-prom-upgrade/prom-part-api.h"

#if defined(PROM_R9000) && defined(EVAL_BOARD)
#define PROM_BASE	0xffc00000LL
#define PROM_BASE_DIAG	0xffcc0000LL
#else
#define PROM_BASE	0xbfc00000LL
#define PROM_BASE_DIAG	0xbfcc0000LL
#endif

/******************************************************************************/
/*
 * descriptor for the 4MB sibyte prom
 */
prom_part_desc_t make_full_sibyte_part_desc[] = PART_DESC_SIBYTE_LAYOUT; 

/*
 * descriptor for the 2MB r7k prom
 */

prom_part_desc_t make_full_r7k_part_desc[] = PART_DESC_R7K_LAYOUT; 

/*
 * descriptor for the 4MB r9k prom
 */

prom_part_desc_t make_full_r9k_part_desc[] = PART_DESC_R9K_LAYOUT; 

/*
 * descriptor for the 0.5 MB ssc_mgmt (toshiba) prom
 */
prom_part_desc_t make_full_mgmt_part_desc[] = PART_DESC_MGMT_LAYOUT;

/*
 * descriptor for the 4MB sibyte prom
 */
prom_part_desc_t make_full_sibyte_rprom_part_desc[] = PART_DESC_SIBYTE_RPROM_LAYOUT; 

/*
 * descriptor for the 0.5MB r7k rprom
 */

prom_part_desc_t make_full_r7k_rprom_part_desc[] = PART_DESC_R7K_RPROM_LAYOUT; 

/*
 * descriptor for the 0.5MB r9k rprom
 */

prom_part_desc_t make_full_r9k_rprom_part_desc[] = PART_DESC_R9K_RPROM_LAYOUT; 


long prom_base;
long prom_base_diag;

/******************************************************************************/

#define BOOTROM_BIG_MAX_SIZE	0x500000
#define COMPR_SIZE				0x500000
#define MAX_BIN_FILE_SIZE		0x400000


#define OLD_BOOTROM_BIN_SIZE	0x80000
#define BOOTROM_SECTOR_SIZE		0x10000

#define BOOTROM_BIN_SIZE        bin_file_size
int bin_file_size;


char	compr[COMPR_SIZE];
char	bin[MAX_BIN_FILE_SIZE];
char	big[BOOTROM_BIG_MAX_SIZE];

int		big_compress_start_offset;
int		bin_compress_start_offset;
int		big_compress_len;
int		bin_compress_len;
int		compr_len = COMPR_SIZE;
int		bin_raw_len;
int		bin_padding;
int		bin_reserve_size = BOOTROM_SECTOR_SIZE;

FILE	* bigfile;
FILE	* binfile;
FILE	* nmfile;

#define COMPRESSED_START_LABEL	"_compressed_image_start"

char *
findstr(
    const char  *str,
    const char  *substr,
    const char  **newpos
)
{
    const char *a;
    const char *b;

    b = substr;
    if (*b == 0)
        return str;
    for ( ; *str != 0; str += 1)
    {
        if (*str != *b)
            continue;
        a = str;
        while (1)
        {
            if (*b == 0) {
                *newpos = a;
                return str;
            }
            if (*a++ != *b++)
                break;
        }
        b = substr;
    }
    *newpos = str + 1;
    return NULL;

} /* findstr() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */
int
getline (char s[], int lim, FILE *stream)
{
	int c=0, i=0;

	for (i=0; i<lim-1 && (c=getc(stream))!=EOF && c!='\n' && c!='\r'; ++i)
		s[i] = c;

	if (c == '\n' || c == '\r') {
		s[i] = '\n';
		++i;
	}
	s[i] = '\0';
	return i;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/28/01
 *
 *-----------------------------------------------------------------
 */
int
find_compressed_start(
	char	* fname_nm
)
{
	char	line[256];
	char	* address, * type, * symbol;
	long long	start_address,a;

	/*
	 * Read .nm file, looking for COMPRESSED_START_LABEL
	 */
	nmfile = fopen(fname_nm, "r");
	if (!nmfile) {
		printf("Error opening file, %s\n", fname_nm);
		exit(1);
	}

	/*
	 * nm file is in form: 'path:address type symbol'
	 */
	while (!feof(nmfile)) {
		/*
		 * read a line
		 */
		getline(line, 256, nmfile);

		/*
		 * skip past the path
		 */
		strtok(line, ":");
	
		/*
		 * get address
		 */
		address = strtok(NULL, " ");

		/*
		 * get type
		 */
		type = strtok(NULL, " ");

		/*
		 * get symbol
		 */
		symbol = strtok(NULL, "\n");

		/*
		 * See if it's the one we want
		 */
		if ((!address) || (!type) || (!symbol))
			continue;

		if ((strcmp(type, "A") == 0) &&
							(strcmp(symbol, COMPRESSED_START_LABEL) == 0)) {
			printf("found _compressed_image_start at %s\n", address);
			sscanf(address, "%llx", &start_address);
			printf("got address = 0x%llx\n", start_address);
	    		if(strstr(fname_nm,"diag.nm") == 0)
				a = start_address - PROM_BASE;
	    		else
				a = start_address - PROM_BASE_DIAG;

			return(a);
		}
		else
			continue;
	}

	return 0;
}


/*-----------------------------------------------------------------
 * Name: get_prom_type
 *
 * Description: map the valid prom type string to prom type code 
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

unsigned short
get_prom_type(char * type_str)
{
    unsigned short rv;

    if (strcmp(type_str, "r7k") == 0)
        rv = PROM_PART_R7K;
    else if (strcmp(type_str, "r9k") == 0)
        rv = PROM_PART_R9K;
    else if (strcmp(type_str, "sibyte") == 0)
        rv = PROM_PART_SIBYTE;
    else if (strcmp(type_str, "sibyte_bc") == 0)
        rv = PROM_PART_SIBYTE;
    else if (strcmp(type_str, "ssc_mgmt") == 0)
        rv = PROM_PART_MGMT;
    else if (strcmp(type_str, "diag_r7k") == 0)
        rv = PROM_PART_R7K_DIAG;
    else if (strcmp(type_str, "diag_r9k") == 0)
        rv = PROM_PART_R9K_DIAG;
    else if (strcmp(type_str, "diag_sibyte") == 0)
        rv = PROM_PART_SIBYTE_DIAG;
    else if (strcmp(type_str, "diag_sibyte_bc") == 0)
        rv = PROM_PART_SIBYTE_DIAG;
    else if (strcmp(type_str, "lmux_hex") == 0)
        rv = PROM_PART_LMUX_HEX;
    else if (strcmp(type_str, "luc_hex") == 0)
        rv = PROM_PART_LUC_HEX;
    else if (strcmp(type_str, "rprom_r7k") == 0)
        rv = PROM_PART_R7K_RPROM;
    else if (strcmp(type_str, "rprom_r9k") == 0)
        rv = PROM_PART_R9K_RPROM;
    else if (strcmp(type_str, "rprom_sibyte") == 0)
        rv = PROM_PART_SIBYTE_RPROM;
    else if (strcmp(type_str, "rprom_sibyte_bc") == 0)
        rv = PROM_PART_SIBYTE_RPROM;
    else
        rv = PROM_PART_UNKNOWN;

    return(rv);
}

/*-----------------------------------------------------------------
 * Name: get_full_prom_size(char * type_str)
 *
 * Description: return size of flash prom based on prom type
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

int
get_full_prom_size(char * type_str)
{
    unsigned int type;
    unsigned int rv;

    type = get_prom_type(type_str);

    switch (type) {
        case PROM_PART_R7K:
            rv = AMD_29LV017_SIZE;
            break;

        case PROM_PART_SIBYTE:
        case PROM_PART_R9K:
            rv = AMD_29LV033_SIZE;
            break;

        case PROM_PART_MGMT:
        case PROM_PART_R7K_RPROM:
        case PROM_PART_R9K_RPROM:
        case PROM_PART_SIBYTE_RPROM:
            rv = AMD_29LV040_SIZE;
            break;

        case PROM_PART_R7K_DIAG:
        case PROM_PART_R9K_DIAG:
        case PROM_PART_SIBYTE_DIAG:
        case PROM_PART_SYS:
        case PROM_PART_LUC_HEX:
        case PROM_PART_LMUX_HEX:
        case PROM_PART_LUC_BIN:
        case PROM_PART_LMUX_BIN:
        default:
            rv = 0;
            break;
    }
    return(rv);
}

/*-----------------------------------------------------------------
 * Name: get_full_part_desc(char * type_str)
 *
 * Description: return partition descriptor table for full prom builds
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

prom_part_desc_p
get_full_part_desc(char * type_str)
{
    unsigned short type;
    prom_part_desc_t * rv;

    type = get_prom_type(type_str);

    switch (type) {
        case PROM_PART_R7K:
            rv = make_full_r7k_part_desc;
            break;

        case PROM_PART_R9K:
            rv = make_full_r9k_part_desc;
            break;

        case PROM_PART_SIBYTE:
            rv = make_full_sibyte_part_desc;
            break;

        case PROM_PART_MGMT:
            rv = make_full_mgmt_part_desc;
            break;

        case PROM_PART_R9K_RPROM:
            rv = make_full_r9k_rprom_part_desc;
            break;

        case PROM_PART_SIBYTE_RPROM:
            rv = make_full_sibyte_rprom_part_desc;
            break;


        case PROM_PART_SYS:
        case PROM_PART_LUC_HEX:
        case PROM_PART_LMUX_HEX:
        case PROM_PART_LUC_BIN:
        case PROM_PART_LMUX_BIN:
        default:
            rv = NULL;
            break;
    }
    return(rv);
}

/*-----------------------------------------------------------------
 * Name: get_single_part_desc(char * type_str)
 *
 * Description: return partition descriptor for single partition prom builds
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

prom_part_desc_p
get_single_part_desc(char * type_str)
{
    static prom_part_desc_t single_part_desc[2]; 
    unsigned short type;
    prom_part_desc_t *rv;

    type = get_prom_type(type_str);

    switch (type) {
        case PROM_PART_R7K:
        case PROM_PART_R9K:
        case PROM_PART_SIBYTE:
        case PROM_PART_MGMT:
        case PROM_PART_R7K_RPROM:
        case PROM_PART_R9K_RPROM:
        case PROM_PART_SIBYTE_RPROM:
        case PROM_PART_R7K_DIAG:
        case PROM_PART_R9K_DIAG:
        case PROM_PART_SIBYTE_DIAG:
        case PROM_PART_SYS:
        case PROM_PART_LUC_HEX:
        case PROM_PART_LMUX_HEX:
            memset((void *)single_part_desc, 0, sizeof(single_part_desc));
            rv = &single_part_desc[0];
            rv->part_type = type;
            rv->part_name = type_str;
            rv->part_offs = 0;
            rv->part_size = 0;
            break;

        case PROM_PART_LUC_BIN:
        case PROM_PART_LMUX_BIN:
        default:
            rv = NULL;
            break;
    }
    return(rv);
}
/*-----------------------------------------------------------------
 * Name: write_file 
 *
 * Description: open, write out the entire contents to memory buf 
 *              and close it.
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

void
write_file (
    char * fname_str,
    char * write_buf_p,
    int    write_len
)
{
    int     rc;
    FILE *  file_p;
	/*
	 * Write bootrom.bin file from bin[]
	 */
	printf("writing %d bytes to %s file\n", write_len, fname_str);
	file_p = fopen(fname_str, "w");
	rc = fwrite(write_buf_p, write_len, 1, file_p);
	if (rc != 1) {
		printf("Error writing file %s\n", fname_str);
		fclose(file_p);
		exit(1);
	}
    fclose(file_p);
}

/*-----------------------------------------------------------------
 * Name: read_file 
 *
 * Description: open and read in the entire contents to memory buf
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */

void
read_file (
    char *  fname_str,
    char *  read_buf_p,
    int  *  read_len
)
{
    int     rc;
    int     file_len;
    FILE *  file_p;
	/*
	 * Read bootrom.big file into big[]
	 */
	file_p = fopen(fname_str, "r");
	if (!file_p) {
		printf("ERR: opening file, %s\n", fname_str);
		exit(0);
	}
	rc = fseek(file_p, 0, SEEK_END);
	if (rc == -1) {
		printf("ERR: could not find eof, errno = %d\n", errno);
		fclose(file_p);
		exit(0);
	}
	file_len = ftell(file_p);
	printf("%s: len = %d\n", fname_str, file_len);

    if (file_len > *read_len) {
		printf("ERR: file \'%s\' size %d over buffer size %d\n",
                fname_str, file_len, *read_len);
        exit(0);
    }

	rc = fseek(file_p, 0, SEEK_SET);
	if (rc == -1) {
		printf("ERR: could not find beginning of file, errno = %d\n", errno);
		fclose(file_p);
		exit(0);
	}
	rc = fread(read_buf_p, file_len, 1, file_p);
	if (rc != 1) {
		printf("ERR: fread returned %d\n", rc);
		fclose(file_p);
		exit(0);
	}
	fclose(file_p);

    *read_len = file_len;
}


/*-----------------------------------------------------------------
 * Name: get_info_str
 *
 * Description:
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */
void
get_info_str(
    char *          file_info_str,
    prom_part_cb_p  cb_p)
{
    FILE *  file_p;
    char *  info_str;
    int     info_len;

	file_p = fopen(file_info_str, "r");
	if (!file_p) {
        // no file use the file name as info
        info_str = file_info_str;
        info_len = strlen(info_str);
        if (info_len >= PROM_PART_INFO_STR_LEN) {
            printf("ERR info string \'%s\' over %d bytes\n",
                    file_info_str, PROM_PART_INFO_STR_LEN - 1);
            exit(0);
        }
        memcpy(cb_p->info_str, file_info_str, info_len);
    }
    else {
        // there is a file, close it and call read fct
        fclose(file_p);
        info_len = PROM_PART_INFO_STR_LEN - 1;
        read_file(file_info_str, cb_p->info_str, &info_len);
    }
    // terminate the string
    cb_p->info_str[info_len] = 0;
}

/*-----------------------------------------------------------------
 * Name: make_prom
 *
 * Description:
 *
 * Created by:	John Hahn
 *
 * Date Created:	07/18/02
 *
 *-----------------------------------------------------------------
 */
int
make_prom(
    prom_part_desc_p desc_p,
    char *           binfile_str,
    char *           swfile_str,
    char *           swinfo_str,
    char *           comp_start_str,
    char *           diagfile_str,
    char *           diaginfo_str,
    char *           diag_start_str,
    char *           lmuxfile_str,
    char *           lmuxinfo_str,
    char *           lucfile_str,
    char *           lucinfo_str,
    unsigned int     full_prom_size
)
{
    int             err;
    char *          bin_buf_p;
    char *          info_str;
    int             total_len;
    unsigned int    srcfile_len;        // size of src file
    unsigned int    srcfile_comp_offs;  // offset in buffer to compress data
    unsigned int    srcfile_raw_len;    // raw/uncompressed len
    prom_part_cb_p  cb_p;
    long            compressed_start;
    int             valid_compressed_start;
    long            diag_compressed_start;
    int             valid_diag_compressed_start;
    int             to_be_compressed_len;


    valid_compressed_start = 0;
    valid_diag_compressed_start = 0;
    total_len = 0;
    // initialize with 1s, flash friendly
	memset(bin, 0xff, BOOTROM_BIN_SIZE);

	printf("Creating %s file\n", binfile_str);

    /*
     * comp_start_str can either be a offset value or name of the boot*.nm file
     */
    if (comp_start_str != NULL && strlen(comp_start_str)) {
        char * endptr;
        /*
         * Check first for user specified offset of data to be compressed
         */
        compressed_start = strtol(comp_start_str, &endptr, 0);
        if (&comp_start_str[strlen(comp_start_str)] != endptr) {
            /*
             * comp_start_str is not a number,
             * find start of data to be compressed from the *.nm file
             */
            compressed_start = find_compressed_start(comp_start_str);
            if (compressed_start == 0) {
                printf("Error, Invalid number or no compressed_image_start\n");
                exit(1);
            }
        }
        valid_compressed_start = 1;
    }
    /*
     * diag_start_str name of the diag*.nm file
     */
    if (diag_start_str != NULL && strlen(diag_start_str)) {
        char * endptr;

        diag_compressed_start = strtol(diag_start_str, &endptr, 0);
        if (&diag_start_str[strlen(diag_start_str)] != endptr) {
            /*
             * diag_start_str is not a number,
             * find start of data to be compressed from the *.nm file
             */
            diag_compressed_start = find_compressed_start(diag_start_str);
            if (diag_compressed_start == 0) {
                printf("Error, Invalid number or no diag compressed_image_start\n");
                exit(1);
            }
        }
        valid_diag_compressed_start = 1;
    }

    while (desc_p->part_name != NULL) {
        memset(big, 0, BOOTROM_BIG_MAX_SIZE);
        memset(compr, 0, COMPR_SIZE);

        bin_buf_p = &bin[desc_p->part_offs];

        srcfile_len = BOOTROM_BIG_MAX_SIZE;
        /*
         * Read bootrom.big file into big[]
         */
        switch(desc_p->part_type) {
            case PROM_PART_R7K:
            case PROM_PART_R9K:
            case PROM_PART_SIBYTE:
            case PROM_PART_MGMT:
            case PROM_PART_R7K_RPROM:
            case PROM_PART_R9K_RPROM:
            case PROM_PART_SIBYTE_RPROM:
                read_file(swfile_str, big, &srcfile_len);
                info_str = swinfo_str;
                if (valid_compressed_start)
                    srcfile_comp_offs = compressed_start;
                else
                    srcfile_comp_offs = srcfile_len;
                break;

            case PROM_PART_R7K_DIAG:
            case PROM_PART_R9K_DIAG:
            case PROM_PART_SIBYTE_DIAG:
                read_file(diagfile_str, big, &srcfile_len);
                info_str = diaginfo_str;
                if (valid_diag_compressed_start)
                    srcfile_comp_offs = diag_compressed_start;
                else
                    srcfile_comp_offs = srcfile_len;
                break;

            case PROM_PART_LUC_HEX:
            case PROM_PART_LMUX_HEX:
                if (desc_p->part_type == PROM_PART_LMUX_HEX) {
                    read_file(lmuxfile_str, big, &srcfile_len);
                    info_str = lmuxinfo_str;
                }
                else {
                    read_file(lucfile_str, big, &srcfile_len);
                    info_str = lucinfo_str;
                }
                if (full_prom_size) {
                    // always full compression
                    srcfile_comp_offs = 0;
                }
                else {
                    // check user specified compression start offset
                    if (valid_compressed_start)
                        srcfile_comp_offs = compressed_start;
                    else    // no compression
                        srcfile_comp_offs = srcfile_len;
                }
                break;

            case PROM_PART_SYS:
                srcfile_len = 0;
                srcfile_comp_offs = srcfile_len;
                info_str = PROM_PART_SYS_INFO_STR; 
                break;

            case PROM_PART_LUC_BIN:
            case PROM_PART_LMUX_BIN:
            default:
                printf("Error unsupported prom type %d\n", desc_p->part_type);
                exit(1);
                break;
        }

        printf("compressed_start_offset = %d (0x%x)\n",
                srcfile_comp_offs,
                srcfile_comp_offs);

        if (srcfile_comp_offs > srcfile_len) {
            printf("Error, invalid compress offset 0x%x srcfile_len %08x diag_com_st %08x \n",
                    srcfile_comp_offs, srcfile_len, diag_compressed_start);
            exit(1);
        }
        /*
         * now that we have big file len and offset of compress data
         * derive the raw_len in big file
         */
        srcfile_raw_len = srcfile_comp_offs;

        to_be_compressed_len = srcfile_len - srcfile_comp_offs;
        if (to_be_compressed_len) {
            /*
             * Compress data from big[] file into compr[]
             */
            compr_len = COMPR_SIZE;
            err = compress(compr,
                    &compr_len,
                    big+srcfile_comp_offs,
                    srcfile_len - srcfile_comp_offs);
            if (err) {
                printf("compress error, err = %d\n", err);
                exit(0);
            }
        }
        else {
            compr_len = 0;
        }
        printf("big_compress_len = %d\n", to_be_compressed_len); 
        printf("compressed length = %d\n", compr_len);

        /*
         * Check size
         */
//WTG need to check partiion size also
        if ((desc_p->part_type == PROM_PART_SIBYTE_RPROM)|| 
            (desc_p->part_type == PROM_PART_R9K_RPROM))  {
            if( (srcfile_comp_offs + compr_len) > PROM_RPRM_CODE_SIZE) {
                printf(" Error, - Image and Compressed Image does NOT fit in partition. \n");
                printf("  Compresed offset Start=0x%x + Compressed Len=0x%x > than Total Size of 0x%x\n",
                        srcfile_comp_offs, compr_len, PROM_RPRM_CODE_SIZE);
                printf("  Over by 0x%08x \n",((srcfile_comp_offs + compr_len) - PROM_RPRM_CODE_SIZE));
                exit(1);
            }
            else {
                printf("  RPROM Space available is 0x%08x or %d \n",
                         (PROM_RPRM_CODE_SIZE - (srcfile_comp_offs + compr_len)),
                         (PROM_RPRM_CODE_SIZE - (srcfile_comp_offs + compr_len)));
            }
        } 
// -----------------------------------
        if (desc_p->part_size) {
            
            if (desc_p->part_size < 
                (srcfile_raw_len + compr_len + PROM_PART_CB_SIZE)) {
                printf("Image too big raw=0x%x + comp=0x%x + rec=0x%x > 0x%x\n",
                        srcfile_raw_len,
                        compr_len,
                        PROM_PART_CB_SIZE,
                        desc_p->part_size);
                exit(0);
            }
            total_len += desc_p->part_size;
        }
        else
            total_len += srcfile_raw_len + compr_len + PROM_PART_CB_SIZE;

        if (total_len > BOOTROM_BIN_SIZE) {
            printf("Total Image too big 0x%x for working buffer size 0x%x\n",
                    total_len, BOOTROM_BIN_SIZE);
            exit(0);
        }

        /*
         * Copy raw data from big[] to bin[]
         */
        printf("copying %u bytes from big to bin[%u]\n",
                srcfile_raw_len,
                (unsigned int) (bin_buf_p - bin));
        memcpy(bin_buf_p, big, srcfile_raw_len);

        bin_buf_p += srcfile_raw_len;

        /*
         * Copy compressed data from compr[] to bin[]
         */
        printf("copying %u bytes from compr to bin[%u]\n",
                compr_len,
                (unsigned int) (bin_buf_p - bin));
        memcpy(bin_buf_p, compr, compr_len);


        /*
         * to the end of data or the next partition
         * use the desc part_size to determine single item
         * or full prom layout with valid part_size 
         */ 
        if (desc_p->part_size)
            bin_buf_p = &bin[desc_p->part_offs + desc_p->part_size];
        else
            bin_buf_p += ((compr_len + PROM_PART_CB_SIZE + 0x3) & ~0x3);

        /*
         * Add signature
         */
        cb_p = (prom_part_cb_p) (bin_buf_p - PROM_PART_CB_SIZE);

        // initialize with 1s, flash friendly
        memset(cb_p, 0xff, PROM_PART_CB_SIZE);
                
        get_info_str(info_str, cb_p);
        cb_p->compressed_start = srcfile_raw_len;
        cb_p->compressed_size = compr_len;
        cb_p->decompressed_size = to_be_compressed_len;
        cb_p->prom_type = desc_p->part_type;
        cb_p->flags = 0;
        if (to_be_compressed_len != 0) {
            cb_p->flags |= PROM_PART_COMPRESSION_FLAG;
        }
        if (cb_p->compressed_start + cb_p->compressed_size) {
            cb_p->partition_crc32 = crc32(0,
                    &bin[desc_p->part_offs],
                    cb_p->compressed_start + cb_p->compressed_size);
            cb_p->flags |= PROM_PART_CRC32_FLAG;
        }
        cb_p->signature = PROM_PART_SIGNATURE;
        cb_p->header_crc32 = crc32(0, cb_p, PROM_PART_CB_CRC32_SIZE);
        printf("%s partition: hdr crc32 0x%x data crc32 0x%x\n",
                desc_p->part_name, cb_p->header_crc32, cb_p->partition_crc32);
        cb_p++;
        desc_p++;
    }

    /*
     * Write bootrom.bin file from bin[]
     */
    if (full_prom_size == 0)
        full_prom_size = ((char *)cb_p - (char *)bin);
    write_file(binfile_str, bin, full_prom_size);
            
    return 0;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/28/01
 *
 *-----------------------------------------------------------------
 */
int
compress_rom(
	char	* fname_big,
	char	* fname_bin,
	char	* fname_nm,
	int		prom_type
)
{
	int	err;
	int	biglen;
	comp_rec_t	*rec;

    memset(bin, 0, BOOTROM_BIN_SIZE);
	memset(big, 0, BOOTROM_BIG_MAX_SIZE);
	memset(compr, 0, COMPR_SIZE);

	printf("Creating %s file\n", fname_bin);

	/*
	 * Read bootrom.big file into big[]
	 */
    biglen = BOOTROM_BIG_MAX_SIZE;
    read_file(fname_big, big, &biglen);

	/*
	 * Find start of data to compress within big[]
	 */
	big_compress_start_offset = find_compressed_start(fname_nm);
	if (!big_compress_start_offset) {
		if (prom_type == 3)			/* SSC_MGMT prom image */
			big_compress_start_offset = biglen;
		else {
			printf("Error, could not find compressed_image_start\n");
			exit(1);
		}
	}
	printf("compressed_start_offset = 0x%x\n", big_compress_start_offset);

	bin_raw_len = big_compress_start_offset;
	big_compress_len = biglen - big_compress_start_offset;

	/*
	 * Compress data from big[] file into compr[]
	 */
	err = compress(compr, &compr_len, big+big_compress_start_offset,
														big_compress_len);
	if (err) {
		printf("compress error, err = %d\n", err);
		exit(0);
	}
	printf("big_compress_len = %d\n", big_compress_len);
	printf("compressed length = %d\n", compr_len);

	/*
	 * Verify sizes
	 */
	if (compr_len + bin_raw_len + bin_reserve_size + COMPRESS_RECORD_SIZE >
															 BOOTROM_BIN_SIZE) {
		printf("Image too big, 0x%x + 0x%x + 0x%x + 0x%x = 0x%x\n",
									compr_len, bin_raw_len, bin_reserve_size,
									COMPRESS_RECORD_SIZE, BOOTROM_BIN_SIZE);
		exit(0);
	}

	/*
	 * Copy raw data from big[] to bin[]
	 */
	printf("copying %d bytes from big to bin\n", bin_raw_len);
	memcpy(bin, big, bin_raw_len);

	/*
	 * Copy compressed data from compr[] to bin[]
	 */
	printf("copying %d bytes from compr to bin[%d]\n", compr_len, bin_raw_len);
	memcpy(&bin[bin_raw_len], compr, compr_len);

	/*
	 * Add signature
	 */
	rec = (comp_rec_t *)
			&(bin[BOOTROM_BIN_SIZE - bin_reserve_size - COMPRESS_RECORD_SIZE]);
	rec->signature = COMPRESS_SIGNATURE;
	rec->compressed_start = bin_raw_len;
	rec->compressed_size = compr_len;
	rec->prom_type = prom_type;

	/*
	 * Write bootrom.bin file from bin[]
	 */
    write_file(fname_bin, bin, BOOTROM_BIN_SIZE);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/28/01
 *
 *-----------------------------------------------------------------
 */
int
decompress_rom(
	char	* fname_big,
	char	* fname_bin
)
{
	int	err, rc;
	int	binlen, biglen;
	comp_rec_t	*rec;

	printf("decompress rom\n");

	memset(bin, 0, BOOTROM_BIN_SIZE);
	memset(big, 0, BOOTROM_BIG_MAX_SIZE);
	memset(compr, 0, COMPR_SIZE);

	printf("Creating %s file\n", fname_big);

	/*
	 * Read bootrom.bin file into bin[]
	 */
	binfile = fopen(fname_bin, "r");
	if (!binfile) {
		printf("Error opening file, %s\n", fname_bin);
		exit(1);
	}
	rc = fseek(binfile, 0, SEEK_END);
	if (rc == -1) {
		printf("could not find eof, errno = %d\n", errno);
		fclose(binfile);
		exit(1);
	}
	binlen = ftell(binfile);
	printf("binlen = %d\n", binlen);

	rc = fseek(binfile, 0, SEEK_SET);
	if (rc == -1) {
		printf("could not find beginning of file, errno = %d\n", errno);
		fclose(binfile);
		exit(1);
	}
	rc = fread(bin, binlen, 1, binfile);
	if (rc != 1) {
		printf("fread returned %d\n", rc);
		fclose(binfile);
		exit(1);
	}
	fclose(binfile);

	/*
	 * Verirfy signature
	 */
	rec = (comp_rec_t *)
			&(bin[BOOTROM_BIN_SIZE - bin_reserve_size - COMPRESS_RECORD_SIZE]);
	if (rec->signature != COMPRESS_SIGNATURE) {
		printf("Error in signature, read 0x%x\n", rec->signature);
		exit(1);
	}

	/*
	 * Find start of compress data within bin[]
	 */
	bin_compress_start_offset = rec->compressed_start;

	bin_raw_len = bin_compress_start_offset;
	bin_compress_len = rec->compressed_size;

	/*
	 * Uncompress data from bin[] into compr[]
	 */
	err = uncompress(compr, &compr_len, bin+bin_compress_start_offset,
														bin_compress_len);
	if (err) {
		printf("uncompress error, err = %d\n", err);
		exit(0);
	}
	printf("bin_compress_len = %d\n", bin_compress_len);
	printf("uncompressed length = %d\n", compr_len);

	biglen = compr_len + bin_raw_len;

	/*
	 * Copy raw data from bin[] to big[]
	 */
	printf("copying %d bytes from bin to big\n", bin_raw_len);
	memcpy(big, bin, bin_raw_len);

	/*
	 * Copy uncompressed data from compr[] to big[]
	 */
	printf("copying %d bytes from compr to big[%d]\n", compr_len, bin_raw_len);
	memcpy(&big[bin_raw_len], compr, compr_len);

	/*
	 * Write bootrom.big file from big[]
	 */
	printf("writing %s file, biglen = %d\n", fname_big, biglen);
	bigfile = fopen(fname_big, "w");
	rc = fwrite(big, biglen, 1, bigfile);
	if (rc != 1) {
		printf("Error writing file %s\n", fname_big);
		fclose(bigfile);
		exit(1);
	}


	return 0;
}



/*
 * following are verified on the partition:
 * in the control block: signature, header crc32, partition data crc32
 */                
int
prom_part_verify(
        prom_part_cb_p  cb_p,
        char *          base_p,
        int             verbose)
{
    unsigned int crc;

    if (cb_p->signature != PROM_PART_SIGNATURE) {
        if (verbose)
            printf("Error partition control block bad signature 0x%x, expect 0x%x\n",
                    cb_p->signature, PROM_PART_SIGNATURE);
        return -1;
    }
    crc = crc32(0, cb_p, PROM_PART_CB_CRC32_SIZE);
    if (crc != cb_p->header_crc32) {
        if (verbose)
            printf("Error partition control block header corrupt crc32 0x%x hdr 0x%x\n",
                    crc, cb_p->header_crc32);
        return -1;
    }
    switch (cb_p->prom_type) {
        case PROM_PART_SIBYTE:
        case PROM_PART_SIBYTE_DIAG:
        case PROM_PART_SIBYTE_RPROM:
        case PROM_PART_LMUX_HEX:
        case PROM_PART_LUC_HEX:
        case PROM_PART_R7K:
        case PROM_PART_R7K_DIAG:
        case PROM_PART_R7K_RPROM:
        case PROM_PART_R9K:
        case PROM_PART_R9K_DIAG:
        case PROM_PART_R9K_RPROM:
        case PROM_PART_MGMT:
        case PROM_PART_SYS:
            // partition known type, proceed ...
            break;

        default:
            // if we got here then the partition doesnt belong
            if (verbose)
                printf("Error prom partition type %d doesnt belong here\n", cb_p->prom_type);
            return -1;
    }
    if (cb_p->flags | PROM_PART_CRC32_FLAG) {
        if (cb_p->compressed_start + cb_p->compressed_size) {
            crc = crc32(0, base_p,
                    cb_p->compressed_start + cb_p->compressed_size);
            if (crc != cb_p->partition_crc32) {
                if (verbose)
                    printf("Error, partition data corrupt crc32 0x%x data 0x%x\n", crc, cb_p->partition_crc32);
                return -1;
            }
        }
    }
    return 0;
}

/*
 * print info and verify partition(s) in file
 *
 * for each partition:
 *    verify the data in partition
 *    hdr and data crc32 checksum
 *    if compressed data verify it decompresses
 */
void
dump_file(char * file_str)
{
    int rv;
    int len;
    int uncomp_len;
    char *              base_p;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;
    comp_rec_t *        cr_p;

    len = BOOTROM_BIG_MAX_SIZE;
    read_file(file_str, big, &len);
    printf("file: %s, size=%d (0x%x)\n", file_str, len, len);

    switch (len) {
    case AMD_29LV017_SIZE:
    case AMD_29LV033_SIZE:
        if (len == AMD_29LV033_SIZE){
            if(strstr(file_str,"r9k")) // file_str is for r9k
                desc_p = make_full_r9k_part_desc;
	    else
                desc_p = make_full_sibyte_part_desc;
	}    
        else 
            desc_p = make_full_r7k_part_desc;
full_prom_dump:
        while (desc_p->part_name != NULL) {
            PART_DESC_PRINT(desc_p);
            base_p = &big[desc_p->part_offs];   // set partition base_p
            cb_p = (prom_part_cb_p)&base_p[desc_p->part_size - PROM_PART_CB_SIZE];
            PART_CB_PRINT(cb_p, base_p);

            if (prom_part_verify(cb_p, base_p, 1/*verbose*/))
                printf("\nERR: Partition failed to verify\n");
            else {
                if (cb_p->flags & PROM_PART_COMPRESSION_FLAG) {
                    uncomp_len = BOOTROM_BIG_MAX_SIZE;
                    rv = uncompress(compr,
                                 &uncomp_len,
                                 &base_p[cb_p->compressed_start],
                                 cb_p->compressed_size);
                    if (rv)
                        printf("\nuncompress error, err = %d\n", rv);
                    else
                        printf(" compressed data uncompresses to : 0x%x (%d)\n",
                                uncomp_len, uncomp_len);
                }
            }

            desc_p++;
        }
        break;

    //case AMD_29F040_SIZE:
    case AMD_29LV040_SIZE:
        /* ssc-mgmt or recovery prom or old style prom */

        cr_p = (comp_rec_t *) &big[OLD_PROM_COMP_REC_OFFSET];
        if (cr_p->signature == COMPRESS_SIGNATURE) {
            /*
             * if compressed data verify it decompresses
             */
            printf("Old Style Prom for:");
            switch(cr_p->prom_type) {
            case PROM_PART_R7K:
                printf(" PROM_R7000");
                break;
            case PROM_PART_R9K:
                printf(" PROM_R9000");
                break;
            case PROM_PART_SIBYTE:
                printf(" PROM_SIBYTE");
                break;
            case PROM_PART_MGMT:
                printf(" PROM_SSC_MGMT");
                break;
            default:
                printf(" Unknown Prom type");
                break;
            }
            printf("\n");

            printf("\tsignature         : 0x%x\n", cr_p->signature);
            printf("\tcompressed_start  : 0x%x (%d)\n",
                    cr_p->compressed_start,
                    cr_p->compressed_start);
            printf("\tcompressed_size   : 0x%x (%d)\n",
                    cr_p->compressed_size,
                    cr_p->compressed_size);
            printf("\tprom_type         : 0x%x\n", cr_p->prom_type);

            uncomp_len = BOOTROM_BIG_MAX_SIZE;
            rv = uncompress(compr,
                         &uncomp_len,
                         &big[cr_p->compressed_start],
                         cr_p->compressed_size);
            if (rv)
                printf("\nuncompress error, err = %d\n", rv);
            else {
                const char * vstr;
                const char * str;

                printf(" compressed data uncompresses to : 0x%x (%d)\n",
                        uncomp_len, uncomp_len);

                /* search for and print version string */
                str = compr;
                while (((unsigned long)str - (unsigned long)compr) < uncomp_len) {
                    vstr = findstr(str, "PROM_", &str);
                    if (vstr != 0) {
                        printf("\nVersion strings is:\n%s\n\n", vstr);
                    }
                }
            }
            return;
        }

        /* may be rprom or ssc_mgmt fall thru */
        cb_p = (prom_part_cb_p) &big[PROM_RPRM_SIZE - PROM_PART_CB_SIZE];
        if (cb_p->prom_type == PROM_PART_MGMT)
            desc_p = make_full_mgmt_part_desc;
        else if (cb_p->prom_type == PROM_PART_R7K_RPROM)
            desc_p = make_full_r7k_rprom_part_desc;
        else if (cb_p->prom_type == PROM_PART_R9K_RPROM)
            desc_p = make_full_r9k_rprom_part_desc;
        else if (cb_p->prom_type == PROM_PART_SIBYTE_RPROM)
            desc_p = make_full_sibyte_rprom_part_desc;
        else
            desc_p = NULL;

        if (desc_p != NULL)
            goto full_prom_dump;

        /* single partition? fall thru */

    default:
        /* single prom partition file */ 

        cb_p = (prom_part_cb_p) &big[len - PROM_PART_CB_SIZE];
        PART_CB_PRINT(cb_p, (void *)0);

        if (prom_part_verify(cb_p, big, 1/*verbose*/))
            printf("\nERR: Partition failed to verify\n");
        else {
            if (cb_p->flags & PROM_PART_COMPRESSION_FLAG) {
                uncomp_len = BOOTROM_BIG_MAX_SIZE;
                rv = uncompress(compr,
                             &uncomp_len,
                             &big[cb_p->compressed_start],
                             cb_p->compressed_size);
                if (rv)
                    printf("\nuncompress error, err = %d\n", rv);
                else
                    printf(" compressed data uncompresses to : 0x%x (%d)\n",
                            uncomp_len, uncomp_len);
            }
        }
        break;
    }
}



void
print_usage(void)
{
    printf("Usage: makerom\n");
    printf("\t   <bootrom.big> <bootrom.bin> <bootrom.nm> <r7k|sibyte|ssc_mgmt>\n");
    printf("\t-d <bootrom.big> <bootrom.bin>\n");
    printf("\t-f <r7k|sibyte|ssc_mgmt|rprom_r7k|rprom_sibyte> <bootrom.bin> <bootrom.big> <bootrom.version> <bootrom.nm> [<diag.big> <diag.version> <diag.nm> [<lmux.hex> <lmux.info> <luc.hex> <luc.info>]]\n");
    printf("\t-s <r7k|sibyte|rprom_r7k|rprom_sibyte|ssc_mgmt|diag_r7k|diag_sibyte|luc_hex|lmux_hex> <file.bin> <file.big> <file.version> [<file.nm>|<comp_offset>]\n");
    printf("\t-v <file.bin>\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/28/01
 *
 *-----------------------------------------------------------------
 */
int
main(
	int		argc,
	char	* argv[]
)
{
    int i;
	int	rc;
	int prom_type;
    int dflag = 0;
    int mode = 0;
    int full_prom_size = 0;

    char * arg_promtype_str;
    char * arg_binfile_str;
    char * arg_srcfile_str = NULL;
    char * arg_srcinfo_str = NULL;
    char * arg_diagfile_str = NULL;
    char * arg_diaginfo_str = NULL;
    char * arg_diagnmfile_str = NULL;
    char * arg_nmfile_str = NULL;
    char * arg_lmuxfile_str = NULL;
    char * arg_lmuxinfo_str = NULL;
    char * arg_lucfile_str = NULL;
    char * arg_lucinfo_str = NULL;

    prom_part_desc_p part_desc_p;

    if (argc < 3) {
        print_usage();
        exit(0);
    }

    for (i=1; i < argc; i++) {
        if (strcmp(argv[i], "-d") == 0) {
            dflag = 1;
        }
        else if (strcmp(argv[i], "-v") == 0) {
            arg_binfile_str  = argv[i+1];

            dump_file(arg_binfile_str);
            exit(0);
        }
        else if (strcmp(argv[i], "-f") == 0) {
            if (mode || argc < 7) {
                printf("Command line syntax Error for option -f.\n");
                print_usage();
                exit(1);
            }
            mode = 1;
            arg_promtype_str = argv[i+1];
            arg_binfile_str  = argv[i+2];
            arg_srcfile_str  = argv[i+3];
            arg_srcinfo_str  = argv[i+4];
            arg_nmfile_str   = argv[i+5];
            if (argc >= 10) {
                arg_diagfile_str   = argv[i+6];
                arg_diaginfo_str   = argv[i+7];
                arg_diagnmfile_str = argv[i+8];
            }
            if (argc == 14) {
                arg_lmuxfile_str = argv[i+9];
                arg_lmuxinfo_str = argv[i+10];
                arg_lucfile_str  = argv[i+11];
                arg_lucinfo_str  = argv[i+12];
            }
            part_desc_p = get_full_part_desc(arg_promtype_str);
            full_prom_size = get_full_prom_size(arg_promtype_str);
        }
        else if (strcmp(argv[i], "-s") == 0) {
            if (mode || argc < 6) {
                printf("Command line syntax Error for option -s.\n");
                print_usage();
                exit(1);
            }
            mode = 2;
            arg_promtype_str = argv[i+1];
            arg_binfile_str  = argv[i+2];
            arg_srcfile_str  = argv[i+3];
            arg_srcinfo_str  = argv[i+4];
            part_desc_p = get_single_part_desc(arg_promtype_str);
            if (argc == 7) {
                if (part_desc_p->part_type == PROM_PART_LMUX_HEX) {
                    arg_lmuxfile_str   = argv[i+3];
                    arg_lmuxinfo_str   = argv[i+4];
                    arg_nmfile_str     = argv[i+5];
                }
                else if (part_desc_p->part_type == PROM_PART_LUC_HEX) {
                    arg_lucfile_str    = argv[i+3];
                    arg_lucinfo_str    = argv[i+4];
                    arg_nmfile_str     = argv[i+5];
                }
                else if (part_desc_p->part_type == PROM_PART_R7K_DIAG ||
                	 part_desc_p->part_type == PROM_PART_R9K_DIAG ||
                         part_desc_p->part_type == PROM_PART_SIBYTE_DIAG) {
                    arg_diagfile_str   = argv[i+3];
                    arg_diaginfo_str   = argv[i+4];
                    arg_diagnmfile_str = argv[i+5];
                    arg_srcfile_str    = NULL;
                    arg_nmfile_str     = NULL;
                }
                else { /* its boot prom */
                    arg_srcfile_str    = argv[i+3];
                    arg_srcinfo_str    = argv[i+4];
                    arg_nmfile_str     = argv[i+5];
                    arg_diagfile_str   = NULL;
                    arg_diagnmfile_str = NULL;
                }
            }
        }
    }

	if (dflag == 1) {
		if (argc != 4) {
			printf("usage: makerom -d <bootrom.big> <bootrom.bin>\n");
            print_usage();
			exit(0);
		}
        BOOTROM_BIN_SIZE = OLD_BOOTROM_BIN_SIZE;
		rc = decompress_rom(argv[2], argv[3]);
	}
	else if (mode == 1 || mode == 2) {
        if (part_desc_p == NULL) {
            printf("ERR: Invalid prom type\n");
            print_usage();
            exit(0);
        }
        BOOTROM_BIN_SIZE = MAX_BIN_FILE_SIZE;
        rc = make_prom(part_desc_p,
                    arg_binfile_str,
                    arg_srcfile_str,
                    arg_srcinfo_str,
                    arg_nmfile_str,
                    arg_diagfile_str,
                    arg_diaginfo_str,
                    arg_diagnmfile_str,
                    arg_lmuxfile_str,
                    arg_lmuxinfo_str,
                    arg_lucfile_str,
                    arg_lucinfo_str,
                    full_prom_size);
	}
    else {
        /*
         * legacy support
         */
		if (argc != 5) {
            print_usage();
			exit(0);
		}
		if (strcmp(argv[4], "r7k") == 0)
			prom_type = 1;
		else if (strcmp(argv[4], "r9k") == 0)
			prom_type = 1;
		else if (strcmp(argv[4], "sibyte") == 0)
			prom_type = 2;
		else if (strcmp(argv[4], "ssc_mgmt") == 0)
			prom_type = 3;
		else {
			printf("Invalid prom_type, %s\n", argv[4]);
			exit(0);
		}
        BOOTROM_BIN_SIZE = OLD_BOOTROM_BIN_SIZE;
		rc = compress_rom(argv[1], argv[2], argv[3], prom_type);
	}
    return 0;
}
