/*
 *-----------------------------------------------------------------
 *
 * Name:	meminit.S
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description:	Initiliaze memory sections
 *
 * Created by:	Rick Lund
 *
 * Date Created:6/18/01
 *
 * History: 03/18/04  Shaun: Taken from prom-r7k-init for r9k.
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-target.h>
#include <sys/mips.h>
#include <exports/hwMap.h>

#include "serio-io.h"

	.text

	.globl	meminit_data
	.globl	meminit_bss
	.globl	meminit_decompression_data
	.globl	meminit_decompression_bss

/*-----------------------------------------------------------------
 * Name:	meminit_data()
 *
 * Description:	Initialize the data section of the PROM in memory
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	meminit_data
meminit_data:
	la		t0,etext
	addu	t0,15
	and		t0,~15
	la		t1,_fdata
	la		t2,edata

	.set noreorder
1:
	lw		t3,0(t0)
	sw		t3,0(t1)
	addu	t0,4
	addu	t1,4
	bltu	t1,t2,1b
	nop

	j		ra
	nop

	.set reorder

	.end	meminit_data

#ifdef USE_DECOMPRESS
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/29/01
 *
 *-----------------------------------------------------------------
 */
	.ent	meminit_decompression_data
meminit_decompression_data:
    move    t4, ra
	la		t0,etext
	addu	t0,15
	and		t0,~15
	la		t1,_decompression_text_start
	la		t2,_decompression_text_end

	.set noreorder
1:
	lw		t3,0(t0)
	sw		t3,0(t1)
	addu	t0,4
	addu	t1,4
	bltu	t1,t2,1b
	nop


	jr		t4
	nop

	.set reorder

	.end	meminit_decompression_data
#endif

/*-----------------------------------------------------------------
 * Name:	meminit_bss()
 *
 * Description:	Initialize the bss section of the PROM in memory
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	meminit_bss
meminit_bss:
	la		v0,_fbss
	la		v1,end

    .set noreorder
1:
	sw		zero,0(v0)
    sw		zero,4(v0)
    sw		zero,8(v0)
	addu	v0,16
	bltu	v0,v1,1b
    sw		zero,-4(v0)			# BDSLOT

	j		ra
	nop

	.set	reorder

	.end	meminit_bss

#ifdef USE_DECOMPRESS
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/29/01
 *
 *-----------------------------------------------------------------
 */
	.ent	meminit_decompression_bss
meminit_decompression_bss:
	la		v0,_decompression_bss_start
	la		v1,_decompression_bss_end

    .set noreorder
1:
	sw		zero,0(v0)
    sw		zero,4(v0)
    sw		zero,8(v0)
	addu	v0,16
	bltu	v0,v1,1b
    sw		zero,-4(v0)			# BDSLOT

	j		ra
	nop

	.set	reorder

	.end	meminit_decompression_bss
#endif

#endif	/* PROM */

