/*
 *-----------------------------------------------------------------
 *
 * Name:	sdram-init.S
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description:	low level SDRAM init code
 *
 * Created by:	Shaun Hui
 *
 * Date Created:03/18/04
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>

#include <exports/hwMap.h>
#include <sys/gt64440r.h>
#include <sys/r9k-regs.h>

#include "serio-io.h"

#define FPGA_BASE_ADDR  FPGA_BASE
#define BOARD_STAT      0x4     /* Board Status Register */

/*
 * Serial Presence Detect Bytes for Micron SDRAM DIMM
 */
#define SPD_NUM_OF_BANKS        5
#define SPD_SDRAM_WIDTH         13
#define SPD_BANKS_ON_DEV        17
#define SPD_SDRAM_ATTR          21
#define SPD_TRP                 27
#define SPD_TRCD                29
#define SPD_BANK_DENSITY        31
                                                                                

#define RM9K_WRITE_REG(offset, reg)           \
        sw      reg, RM9K_BASE_ADDR+(offset)
                                                                                
#define RM9K_WRITE(offset, value)           \
        li      t0, HTOLE32(value);     \
        sw      t0, RM9K_BASE_ADDR+(offset)
                                                                                
#define RM9K_INIT(offset, value) \
        .word   RM9K_BASE_ADDR+(offset), value

	.set    noreorder
/*
 *  Initialize SDRAM controllers.
 *
 *  We have two SDRAM controllers, one on the RM9000 itself and
 *  one on the GT64340. Depending on the "user jumper" setting we
 *  either place the RM9000 modules or the GT64340 modules first.
 */
       .globl  sdram_init
sdram_init:

#ifdef	EVAL_BOARD
        TTYDBG("Setting up SDRAM controllers\r\n")
                                                                                
        li      s1, 0                   /* Clear memory accumulator */
                                                                               
        jal     boot_i2c_init           /* Init SPD read interface */
        nop
                                                                                
                                                                                
        TTYDBG("Back from boot_i2c_init\r\n")
/*
 *  Dig the memory modules for information.
 *  RM9K has module address 0 and 1, MV64340 has 3 and 4.
 */
                                                                                
        /*
         * Check for user jumper -- if not present, normal setup.
         * Otherwise, swap the order of memory, Marvel @ 0x0.
         */
        la      v0, FPGA_BASE_ADDR
        lb      t0, BOARD_STAT(v0)
        and     t0, 0x80
        beqz    t0, sdram_normal
        nop
        TTYDBG("SDRAM: Inverse module order\r\n")
                                                                                
        jal     init_mv64340_sdram
        nop
        jal     init_gemini_sdram
        nop
        li      s1, 0x08000000
        j       sdram_init_done
        nop


sdram_normal:
        jal     init_gemini_sdram
        nop
        li      s1, 0x10000000
        jal     init_mv64340_sdram
        nop
        j       sdram_init_done
        nop
                                                                                
#else

/*
 *  Probe the size of a SDRAM DIMM module base
 *  on Micron Serial Presence-Detect Matrix.
 *  Memory size = # of banks * banks on device * bank density (MB)
 *
 *  a0 - device address
 *  a1 - SPD byte to read
 */


        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_NUM_OF_BANKS	/* Read # of banks */
        jal     boot_i2c_read
        nop

        bltz    v0, init_sdram_r9k_err     /* Empty slot? */
        move    t0, v0
        move    t3, v0

        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_BANKS_ON_DEV	/* */
        jal     boot_i2c_read
        nop

        mult    t3, v0
        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_BANK_DENSITY	/* */
        jal     boot_i2c_read
        nop

        mflo    t3
        mult    t3, v0
        mflo    v0
        sll     v0, 20          /* times 1M to get byte count */
                                                                                
        move    t3, v0
        PRINTSTR("R9K MEMORY SIZE=")
        move    a0, t3
        jal     hexserial
        nop
        PRINTSTR("\n\r")


        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_SDRAM_ATTR	/* registered sdram ? */
        jal     boot_i2c_read
        nop

        move    t1, v0

        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_SDRAM_WIDTH	/* Read device width */
        jal     boot_i2c_read
        nop

        move    t2, v0


		beq		t2, 32, bus_width_32
		nop

		beq		t2, 16, bus_width_16
		nop

bus_width_8:
		li		v0, R9K_SDM_WIDTH_8_BITS
		b		1f
		nop

bus_width_16:
		li		v0, R9K_SDM_WIDTH_16_BITS
		b		1f
		nop

bus_width_32:
		li		v0, R9K_SDM_WIDTH_32_BITS
		b		1f
		nop

1:
		beq		t3, 0x40000000, mem_size_1G
		nop

		ori		v0, R9K_SDM_DENSITY_256M
		b		2f
		nop

mem_size_1G:
		ori		v0, R9K_SDM_DENSITY_1G
		b		2f
		nop

2:
		beq		t1, 0x26, registered_dimm
		nop

		ori		v0, R9K_SDM_UNREG_DIMM
		b		3f
		nop

registered_dimm:
		ori		v0, R9K_SDM_REG_DIMM
		b		3f
		nop

3:
		ori		v0, (R9K_SDM_RANK_ENA | R9K_SDM_ECC_CHK_ENA)

        move    t1, v0
        PRINTSTR("SDM  REG=")
        move    a0, t1
        jal     hexserial
        nop
        PRINTSTR("\n\r")

        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_TRCD				/* tRCD */
        jal     boot_i2c_read
        nop

        move    t2, v0


        li      a0, I2C_ADDR_PMC_MEM_SEEP
        li      a1, SPD_TRP					/* tRP */
        jal     boot_i2c_read
        nop

        move    t3, v0

		lw		a0, BMFPGA_BASE
		andi	a0, 0x8000
		beqz	a0,	clk_100Mhz
		nop
clk_133Mhz:
		li		a0, 7		/* clk cycle time (1/133Mhz) */
		b		1f
		nop

clk_100Mhz:
		li		a0, 10		/* clk cycle time (1/100Mhz) */
1:
		li		v0, (0x3f<<16)
        /* RM9k errata -- must set to CL3 to get 2.5 */
		ori		v0, R9K_SDT_TCAC_3
		srl		t3, 2
		div		t3, a0
		addi	t3, 1			/* round up */
		beq		t3, 2, trp_2
		nop
		
		beq		t3, 3, trp_3
		nop

trp_4:
		ori		v0, R9K_SDT_TRP_4
		b		4f
		nop
trp_3:
		ori		v0, R9K_SDT_TRP_3
		b		4f
		nop
trp_2:
		ori		v0, R9K_SDT_TRP_2
		b		4f
		nop
4:
		srl		t2, 2
		div		t2, a0
		addi	t2, 1			/* round up */
		beq		t2, 2, trcd_2
		nop

		beq		t2, 3, trcd_3
		nop

trcd_4:
		ori		v0, R9K_SDT_TRCD_4
		b		5f
		nop
trcd_3:
		ori		v0, R9K_SDT_TRCD_3
		b		5f
		nop
trcd_2:
		ori		v0, R9K_SDT_TRCD_2
		b		5f
		nop
5:
		add		t3, t2			/* tRC = tRP + tRCD + 1 */
		addi	t3, 1		
		addi	t3, 1			/* round up */
		sll		t3, R9K_SDT_TRC_SHIFT
		or		v0, t3
	
	or	v0, R9K_SDT_TRFC_10

        move    t3, v0
        PRINTSTR("SDT  REG=")
        move    a0, t3
        jal     hexserial
        nop
        PRINTSTR("\n\r")

		/* Check # of banks (t0). 
		 * If banks=2, init ranks 0 & 1
		 * If banks=1, init ranks 0 
		 */
		blt		t0, 2, init_sdram_rank0		
		nop

init_sdram_rank1:

        RM9K_WRITE(R9K_LKB2, HW_LOCAL_MEM_BASE>>4 | R9K_LKB_ENABLE)    
        RM9K_WRITE(R9K_LKM2, R9K_LKM_SIZE_256M) 
        RM9K_WRITE_REG(R9K_SDT1, t3)
        RM9K_WRITE_REG(R9K_SDM1, t1)
        RM9K_WRITE(R9K_SDMW, R9K_SDMW_EXT_RANK_1 | \
							 R9K_SDMW_DLL_ENABLE | \
							 R9K_SDMW_DS_NORMAL  | \
							 R9K_SDMW_QFC_DISABLE)

                                                                                

init_sdram_rank0:

        RM9K_WRITE(R9K_LKB1, HW_RAM_BASE | R9K_LKB_ENABLE)   
        RM9K_WRITE(R9K_LKM1, R9K_LKM_SIZE_256M)    
        RM9K_WRITE_REG(R9K_SDT0, t3)
        RM9K_WRITE_REG(R9K_SDM0, t1)
        RM9K_WRITE(R9K_SDMW,R9K_SDMW_EXT_RANK_0 | \
							R9K_SDMW_DLL_ENABLE | \
							R9K_SDMW_DS_NORMAL  | \
							R9K_SDMW_QFC_DISABLE)

                                                                                
        RM9K_WRITE(R9K_SDCFG, R9K_SDC_CLKDSC_CL1_STRONG  | \
							  R9K_SDC_DATADSC_CL2_STRONG | \
							  R9K_SDC_CHKDSC_CL2_STRONG  | \
							  R9K_SDC_CMDDSC_CL1_STRONG  | \
							  R9K_SDC_ECC_ENABLE)

		/* Enable CPU-SDRAM write error & CPU-Any Read error 
		 * in Extended fault group register to generate OCD
		 * interrput during ECC errors.
		 */
		RM9K_WRITE(R9K_XCPUEBLFA0,  R9K_XCPUEBLFA0_CSEWEN | \
									R9K_XCPUEBLFA0_CAREN);

#ifdef xxx
		/* Enable OCD interrupt - Int15 */
        cfc0    t0, C0C_ICR
        ori     t0, 0x8000
        ctc0    t0, C0C_ICR

		/* Enable global interrupt */
        mfc0    t0, C0_SR
        ori     t0, SR_IE
        mtc0    t0, C0_SR
#endif

        j       sdram_init_done
        nop

init_sdram_r9k_err:
        TTYDBG("No R9K SDRAM detected. Booting stopped!\r\n")
1:
		b		1b
		nop

        j       sdram_init_done
        nop

#endif

#ifdef	EVAL_BOARD
/*
 * Set up SDRAM on the Gemeni. Various models depending on
 * processor version and user jumper.
 */
LEAF(init_gemini_sdram)
        move    t9, ra
                                                                                
        bal     probe_sdram_size
        li      a0, 0x000
                                                                                
        bltz    v0, gem_no_mem
        nop
                                                                                
        jal     boot_i2c_read           /* Get number of rows from primary */
        li      a0, 0x003
                                                                                
        li      v1, 12
        beq     v0, v1, m0init_16x8
        li      v1, 13
        bne     v0, v1, gem_no_mem
        nop
                                                                                
m0init_32x8:
        TTYDBG("Using 32x8 Configuration\r\n")
                                                                                
        la      v0, FPGA_BASE_ADDR
        lb      t0, BOARD_STAT(v0)
        and     t0, 0x80
        bnez    t0, gem_no_mem
        nop
        bal     doinit
        nop
                                                                                
        RM9K_INIT(0x300, 0x000302d9)
        RM9K_INIT(0x108, 0x00000001)    /* LKB1 */
        RM9K_INIT(0x10c, 0x00ffff00)
        RM9K_INIT(0x344, 0x022f9481)
        RM9K_INIT(0x340, 0x00004025)
        RM9K_INIT(0x304, 0x00000000)    /* Rank 0 */
        /* RM9k errata -- must set to CL3 to get 2.5 */
        RM9K_INIT(0x344, 0x022f9482)
        .word 0, 0
                                                                                
m0init_16x8:
        TTYDBG("Using 16x8 Configuration\r\n")
                                                                                
        la      v0, FPGA_BASE_ADDR
        lb      t0, BOARD_STAT(v0)
        and     t0, 0x80
        bnez    t0, m0init_16x8_swap
        nop
        bal     doinit
        nop
                                                                                
        RM9K_INIT(0x300, 0x000302d9)
        RM9K_INIT(0x108, 0x00000001)    /* LKB1 */
        RM9K_INIT(0x10c, 0x007fff00)
        RM9K_INIT(0x110, 0x00800001)    /* LKB2 */
        RM9K_INIT(0x114, 0x007fff00)
        RM9K_INIT(0x344, 0x022f9481)
        RM9K_INIT(0x354, 0x022f9481)
        RM9K_INIT(0x340, 0x00004015)
        RM9K_INIT(0x350, 0x00004015)
        RM9K_INIT(0x304, 0x00000000)    /* Rank 0 */
        RM9K_INIT(0x304, 0x00000001)    /* Rank 1 */
                                                                                
        /* RM9k errata -- must set to CL3 to get 2.5 */
        RM9K_INIT(0x344, 0x022f9482)
        RM9K_INIT(0x354, 0x022f9482)
        .word 0, 0
                                                                                
m0init_16x8_swap:
        bal     doinit
        nop
                                                                                
        RM9K_INIT(0x300, 0x000302d9)
        RM9K_INIT(0x108, 0x00800001)    /* LKB1 */
        RM9K_INIT(0x10c, 0x007fff00)
        RM9K_INIT(0x110, 0x01000001)    /* LKB2 */
        RM9K_INIT(0x114, 0x007fff00)
        RM9K_INIT(0x344, 0x022f9481)
        RM9K_INIT(0x354, 0x022f9481)
        RM9K_INIT(0x340, 0x00004015)
        RM9K_INIT(0x350, 0x00004015)
        RM9K_INIT(0x304, 0x00000000)    /* Rank 0 */
        RM9K_INIT(0x304, 0x00000001)    /* Rank 1 */
                                                                                
        /* RM9k errata -- must set to CL3 to get 2.5 */
        RM9K_INIT(0x344, 0x022f9482)
        RM9K_INIT(0x354, 0x022f9482)
        .word 0, 0
                                                                                
doinit:
        b       2f
        nop
                                                                                
1:
        sw      v1, 0(v0)
2:
        lw      v0, 0(ra)               /* Address */
        lw      v1, 4(ra)               /* Data */
        bnez    v0, 1b
        addiu   ra, 8
                                                                                
gem_no_mem:
        jr      t9
        nop
END(init_gemini_sdram)
                                                                                

/*
 *  Set up any modules on the MV64340. This code assumes
 *  both modules to be the same size and type. Either
 *  slot may be fitted. We use parameters from first found.
 */
LEAF(init_mv64340_sdram)
        move    t9, ra
        TTYDBG("SDRAM Setting up MV64340\r\n")
                                                                                
        /* Disable LK6 in case there are no memory on the GT64340 */
        la      v0, RM9K_BASE_ADDR
        sw      zero, 0x130(v0)
        lw      t0, 0x130(v0)
                                                                                
        bal     probe_sdram_size
        li      a0, 0x200
        bgtz    v0, 1f
        nop
        bal     probe_sdram_size
        li      a0, 0x300
        bltz    v0, mv_no_modules
        nop
                                                                                
1:
        /* set LKM6 for 128MByte */
        la      v0, RM9K_BASE_ADDR
        li      t0, 0x007fff00
        sw      t0, 0x134(v0)
        lw      t0, 0x134(v0)
        sync
                                                                                
        /* set LKB6 for 0x0000_0000 base */
        srl     t0, s1, 4
        ori     t0, 0x00000001
        sw      t0, 0x130(v0)
        lw      t0, 0x130(v0)
        sync
        bal     makele32
        srl     a0, s1, 16
        sw      v0, SCS_0_BASE_ADDRESS(s2)
        li      v0, HTOLE32(0x07ff)
        sw      v0, SCS_0_SIZE(s2)              /* Map 128Mb */
                                                                                
        lw      v0, CPU_BASE_ADDRESS_ENABLE(s2)
        and     v0, HTOLE32(0xfffffff0)
        or      v0, HTOLE32(0x0000000e)
        sw      v0, CPU_BASE_ADDRESS_ENABLE(s2)
                                                                                
        addu    s1, 0x08000000          /* XXX Should use real size! */
                                                                                
        jal     boot_i2c_read           /* Get module type from primary */
        li      a0, 0x215
        move    t0, v0
        jal     boot_i2c_read           /* Address size */
        li      a0, 0x203
        move    t1, v0
        jal     boot_i2c_read           /* Latency */
        li      a0, 0x212
                                                                                
        b       mv_init_dll
        move    t2, v0
                                                                                
        jal     boot_i2c_read           /* Get module type from secondary */
        li      a0, 0x315
                                                                                
        move    t0, v0
        jal     boot_i2c_read           /* Address size */
        li      a0, 0x303
        move    t1, v0
        jal     boot_i2c_read           /* Latency */
        li      a0, 0x312
        move    t2, v0
                                                                                
mv_init_dll:
        sw      zero, SRAM_ADDRESS(s2)
        bal     1f
        li      v0, 64
                                                                                
        .word HTOLE32(0x00000000), HTOLE32(0x00000041), HTOLE32(0x00000082)
        .word HTOLE32(0x000000c3), HTOLE32(0x00000104), HTOLE32(0x00000145)
        .word HTOLE32(0x00000186), HTOLE32(0x000001c7), HTOLE32(0x00000208)
        .word HTOLE32(0x00000249), HTOLE32(0x0000028a), HTOLE32(0x000002cb)
        .word HTOLE32(0x0000030c), HTOLE32(0x0000034d), HTOLE32(0x0000038e)
        .word HTOLE32(0x000003cf), HTOLE32(0x00000410), HTOLE32(0x00000451)
        .word HTOLE32(0x00000492), HTOLE32(0x000004d3), HTOLE32(0x00000514)
        .word HTOLE32(0x00000555), HTOLE32(0x00000596), HTOLE32(0x000005d7)
        .word HTOLE32(0x00000618), HTOLE32(0x00000659), HTOLE32(0x0000069a)
        .word HTOLE32(0x000006db), HTOLE32(0x0000071c), HTOLE32(0x0000075d)
        .word HTOLE32(0x0000079e), HTOLE32(0x000007df), HTOLE32(0x00000820)
        .word HTOLE32(0x00000861), HTOLE32(0x000008a2), HTOLE32(0x000008e3)
        .word HTOLE32(0x00000924), HTOLE32(0x00000965), HTOLE32(0x000009a6)
        .word HTOLE32(0x000009e7), HTOLE32(0x00000a28), HTOLE32(0x00000a69)
        .word HTOLE32(0x00000aaa), HTOLE32(0x00000aeb), HTOLE32(0x00000b2c)
        .word HTOLE32(0x00000b6d), HTOLE32(0x00000bae), HTOLE32(0x00000bef)
        .word HTOLE32(0x00000c30), HTOLE32(0x00000c71), HTOLE32(0x00000cb2)
        .word HTOLE32(0x00000cf3), HTOLE32(0x00000d34), HTOLE32(0x00000d75)
        .word HTOLE32(0x00000db6), HTOLE32(0x00000df7), HTOLE32(0x00000e38)
        .word HTOLE32(0x00000e79), HTOLE32(0x00000eba), HTOLE32(0x00000efb)
        .word HTOLE32(0x00000f3c), HTOLE32(0x00000f7d), HTOLE32(0x00000fbe)
        .word HTOLE32(0x00000fff)
                                                                                
1:
        lw      v1, 0(ra)               /* Data */
        sw      v1, SRAM_DATA0(s2)
        addiu   v0, -1
        bnez    v0, 1b
        addiu   ra, 4
                                                                                
        li      v0, HTOLE32(0x00300000)
        sw      v0, DFCDL_CONFIGURATION0(s2)
                                                                                
        /* Send a NOP */
        li      v0, HTOLE32(0x00000005)
        sw      v0, SDRAM_OPERATION(s2)
        lw      v0, SDRAM_OPERATION(s2)         /* this flushes the store */
        li      v0, HTOLE32(0x00000007)
1:
        lw      v1, SDRAM_OPERATION(s2)
        and     v1, v0
        bnez    v1, 1b
        nop
                                                                                
/*
 *  Configure for registred or non registred modules.
 */
        andi    t0, 0x02
        beqz    t0, mv_sdram_nonreg
        nop
                                                                                
        TTYDBG("SDRAM registred\r\n")
        li      v0, HTOLE32(0x58220400)
        b       mv_sdram_config
        nop
                                                                                
mv_sdram_nonreg:
        TTYDBG("SDRAM non-registred\r\n")
        li      v0, HTOLE32(0x58200400)
                                                                                
mv_sdram_config:
        sw      v0, SDRAM_CONFIGURATION(s2)
        sw      zero, SDRAM_OPEN_PAGES_CONTROL(s2)
                                                                                
/*
 *  Select CAS latency.
 */
        lw      t4, DUNIT_CONTROL_LOW(s2)
        and     t4, HTOLE32(0x00000001)
/*
 *  Now t0 = 0 if non-reg, t4 = 0 if clocks are synchronized in Dunit.
 */
                                                                                
        srl     t2, 2
        li      t3, HTOLE32(0x00000052)
        beqz    t2, mv_cas_mode
        li      v0, 0                   # 1.5
        srl     t2, 1
        li      t3, HTOLE32(0x00000022)
        beqz    t2, mv_cas_mode
        li      v0, 16                  # 2.0
        srl     t2, 1
        li      t3, HTOLE32(0x00000062)
        beqz    t2, mv_cas_mode
        li      v0, 32                  # 2.5
                                                                                
        li      t3, HTOLE32(0x00000032)
        li      v0, 48                  # 3.0
                                                                                
mv_cas_mode:
        bnez    t4, 1f
        nop
        ori     v0, 8
1:
        bnez    t0, 2f
        nop
        ori     v0, 4
2:
        bal     mv_setcas
        nop
                                                                                
/*  Table for different cas latency settings.
 *  Values are in order StBurstDelay for
 *  Sync/Reg, Sync/Non-Reg, Async/Reg, Async/NonReg
 */
/* Cas 1.5 */   .word   HTOLE32(0x24110000), HTOLE32(0x23110000)
                .word   HTOLE32(0x0b110000), HTOLE32(0x1a110780)
/* Cas 2.0 */   .word   HTOLE32(0x04110000), HTOLE32(0x03110000)
                .word   HTOLE32(0x3c110000), HTOLE32(0x3b110780)
/* Cas 2.5 */   .word   HTOLE32(0x25110000), HTOLE32(0x24110000)
                .word   HTOLE32(0x0c110000), HTOLE32(0x1b110780)
/* Cas 3.0 */   .word   HTOLE32(0x05110000), HTOLE32(0x04110000)
                .word   HTOLE32(0x3d110000), HTOLE32(0x3c110780)
/* Cas 3.5 */   .word   HTOLE32(0x26110000), HTOLE32(0x25110000)
                .word   HTOLE32(0x0d110000), HTOLE32(0x1c110780)
/* Cas 4.0 */   .word   HTOLE32(0x06110000), HTOLE32(0x05110000)
                .word   HTOLE32(0x3d110000), HTOLE32(0x3e110780)
                                                                                
mv_setcas:
        /* issue Mode Register Set command */
        li      t0, HTOLE32(0x00000003)
        sw      t0, SDRAM_OPERATION(s2)
        lw      t0, SDRAM_OPERATION(s2)         /* this flushes the store */
        li      t0, HTOLE32(0x00000007)
1:
        lw      v1, SDRAM_OPERATION(s2)
        and     v1, t0
        bnez    v1, 1b
        nop
                                                                                
        addu    v1, v0, ra
        lw      v1, 0(v1)
        lw      t0, DUNIT_CONTROL_LOW(s2)
        and     t0, HTOLE32(0x0000007f)
        or      t0, v1
        sw      t0, DUNIT_CONTROL_LOW(s2)
        TTYDBG("SDRAM Cas settings: ")
        TTYDBGHEX(t0)
        TTYDBG("\r\n")
                                                                                
        sw      t3, SDRAM_MODE(s2)
                                                                                
        /* issue Mode Register Set command */
        li      t0, HTOLE32(0x00000003)
        sw      t0, SDRAM_OPERATION(s2)
        lw      t0, SDRAM_OPERATION(s2)         /* this flushes the store */
        li      t0, HTOLE32(0x00000007)
1:
        lw      v1, SDRAM_OPERATION(s2)
        and     v1, t0
        bnez    v1, 1b
        nop
                                                                                
/*
 *  XXX, This needs more combinations.
 */
        li      v1, HTOLE32(0x00000002)
        li      t0, 12
        beq     t0, t1, mv_setdens
        nop
                                                                                
        li      v1, HTOLE32(0x00000012)
        li      t0, 13
        beq     t0, t1, mv_setdens
        nop
                                                                                
        li      v1, HTOLE32(0x00000022)
                                                                                
mv_setdens:
        sw      v1, SDRAM_ADDRESS_CONTROL(s2)
                                                                                
        li      v1, HTOLE32(0x11511220)
        b       mv_mem_done
        sw      v1, SDRAM_TIMING_LOW(s2)
                                                                                
mv_no_modules:
        TTYDBG("SDRAM Slot 1 empty.\r\n")
        PRINTSTR("WARN: No PCI DMA memory. PCI devices may not work!\r\n")
                                                                                
mv_mem_done:
        jr      t9
        nop
END(init_mv64340_sdram)
                                                                                

LEAF(makele32)
        move    v0, a0
        jr      ra
        nop
END(makele32)
                                                                                

                                                                                
/*
 *  Probe the size of a module.
 */
                                                                                
probe_sdram_size:
        move    t1, ra
        and     t2, a0, 0x0700
        jal     boot_i2c_read
        ori     a0, t2, 5
                                                                                
        bltz    v0, probe_sdram_err     /* Empty slot? */
        move    t3, v0
                                                                                
        jal     boot_i2c_read
        ori     a0, t2, 31
                                                                                
        mult    t3, v0
        jal     boot_i2c_read
        ori     a0, t2, 17
                                                                                
        mflo    t3
        mult    t3, v0
        mflo    v0
        sll     v0, 20
                                                                                
probe_sdram_err:
        jr      t1
        nop


#endif	/* EVAL_BOARD */
                                                                                
#endif	/* PROM */

