/*
 *-----------------------------------------------------------------
 *
 * Name:	sbdreset.S
 *
 * RCS:  $Id: sbdreset.S,v 1.8 2002/11/21 17:24:47 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	low level code for GT 64240 board
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/21/01
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM

#include <nfx-target.h>
#include <sys/mips.h>
#include "../prom-pmon/pmon.h"

#include <exports/promio-api.h>
#include "sbd.h"
#include <exports/hwMap.h>
#include <sys/sb1250/sb1250_scd.h>
#include "mipsmacros.h"

#if !__mips64 && __mips >= 3 && __mips != 32
	/* force 64-bit register support */
#undef __mips64	
#define __mips64 1	
	.set	gp64
#endif
	
#define MEG	0x100000		

#define CFE_CACHE_FLUSH_D   1
#define CFE_CACHE_INVAL_I   2
#define CFE_CACHE_INVAL_D   4
#define CFE_CACHE_INVAL_L2  8
#define CFE_CACHE_FLUSH_L2  16

	.globl	nfx_init_done
	.globl	nsdb_done

/*
 * Basic board initialisation, called straight from RESET.
 * It is only called if this program is built for ROM.  
 * It can use any registers except s8, k0 and k1.
 *
 * Note that s8 holds a "relocation factor" (see ../share/romlow.sx)
 * which must be added to any address before it is used.  This
 * is to support relocatable roms.
 */
	
#define msize		s4


LEAF(sbdreset)
	
	jal	sb1250_altcpu_reset
	nop
	
	jal	board_earlyinit
	nop

	SETLEDS('P','R','I','O')
	nop
	jal	promio_init
	nop

#ifdef USE_NSDB
	SETLEDS('N','S','D','B')
	nop
	jal	nsdb
	nop
#endif

	SETLEDS('C','P','U','0')
	nop
	jal	sb1250_cpuinit
	nop

	SETLEDS('A','L','T','1')
	nop
	jal	sb1250_altcpu_start1
	nop

	SETLEDS('D','R','A','M')
	nop
	li	a0,0
	jal	sb1250_dram_init
	nop

	move	s4,v0

	SETLEDS('B','A','C','K')
	nop

#if 0
	move	a0,k0
	li		a1,8
	jal		promio_prx
	nop
#endif

	bne	s4,zero,have_ram
	SETLEDS('R','A','M','X')

die1:	b	die1

have_ram:
#ifdef __long64
	mfc0	t0,C0_SR
	or		t0,t0,SR_KX
	mtc0	t0,C0_SR
#endif

#if defined(USE_DECOMPRESS) && !defined(SB1250_SIMULATOR)

#if LUC_HAS_A_BUG_THIS_RUNS_TOO_FAST
	li  a0,CFE_CACHE_INVAL_I
	jal sb1250_cacheops

	.set    noreorder
	.set    nomacro
	bal 1f
	lui a0,0xDFFF
1:	or  a0,0xFFFF
	and ra,a0
	addiu   ra,20
	j       ra
	nop
	.set    macro
	.set    reorder
#endif

	jal	meminit_decompression_bss
	jal	meminit_decompression_data

	jal	meminit_bss

#if LUC_HAS_A_BUG_THIS_RUNS_TOO_FAST
	.set    noreorder
	.set    nomacro
	bal 1f
	lui a0,0xA000
1:	or  ra,a0
	addiu   ra,16
	j   ra
	nop
	.set    macro
	.set    reorder
#endif

#else
	SETLEDS('Z','B','S','S')
	jal	meminit_bss

	SETLEDS('D','A','T','A')
	jal	meminit_data
#endif

	SETLEDS('L','1','2','F')
	la	a0,CFE_CACHE_FLUSH_D | CFE_CACHE_FLUSH_L2
	jal	sb1250_cacheops

	li	a0,CFE_CACHE_INVAL_I
	jal	sb1250_cacheops

	SETLEDS('C','P','U','2')
	jal	sb1250_altcpu_start2

	SETLEDS('T','E','S','T')
#ifdef _SB1250_PASS1_WORKAROUNDS_
    /*
     * Okay, it's safe now to be coherent.
     * Flush the D cache to invalidate all the lines we have,
     * then change the config register back.
     */
    li  a0,CFE_CACHE_FLUSH_D
	jal	sb1250_cacheops

    SETCCAMODE(v0,CFG_C_COHERENTXCLW) /* cacheable coherent */
#endif

#if 0
	la	ARG1,leaving_str
	jal	promio_prs
#endif

	SETLEDS('D','O','N','E')
	j	sbdreset_done
END(sbdreset)


	.lcomm	wbfltmp,4

LEAF(mips_wbflush)
	sync
	la	t0,wbfltmp
	or	t0,K1BASE
	lw	zero,0(t0)
	j	ra
END(mips_wbflush)


LEAF(sbddelay)
	mfc0	t0,C0_CONFIG
	li	t1,CACHEMISS+ROMCYCLE
	and	t0,CFG_K0MASK
	beq	t0,CFG_C_UNCACHED,1f
	and	t0,ra,0x20000000
	bnez	t0,1f
	li	t1,CACHECYCLE
1:	mul	a0,1000
	addu	a0,t1
	sll	t1,1
	divu	a0,t1
	subu	a0,48		# approx number of loops so far
	.set	noreorder	
	.set	nomacro
	nop
2:	bgtz	a0,2b
	subu	a0,1
	.set	macro
	.set	reorder
	j	ra
END(sbddelay)


LEAF(mips_cycle)
	.set	noreorder	
	.set	nomacro
1:	bgtz	a0,1b
	subu	a0,1
	.set	macro
	.set	reorder
	j	ra
END(mips_cycle)

leaving_str:
	.asciiz	"Leaving sbdreset\r\n"

#endif	/* PROM */

