/*
 *-----------------------------------------------------------------
 *
 * Name:	tlbinit.S
 *
 * RCS:  $Id: tlbinit.S,v 1.2 2002/03/15 21:17:32 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Initialize the TLB entries
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>

	.text

	.globl	init_tlb
	.globl	tlb_inc
	.globl	tlb_error

#define INDEX		t8
#define SIZE		t9
#define PHYSICAL	a0
#define VIRTUAL		a1
#define TABLE		a2

/*-----------------------------------------------------------------
 * Name:	init_tlb()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	init_tlb
init_tlb:
	move	v0,ra

	/*
	 * initialise tlb
	 */
	dmtc0	zero,C0_TLBLO0		/* tlblo0 = invalid */
	dmtc0	zero,C0_TLBLO1		/* tlblo1 = invalid */
	dmtc0	zero,C0_PGMASK
	li		t8,K1BASE			/* tlbhi  = impossible vpn */
	li		t9,(NTLBENTRIES-1)	/* index */

	.set noreorder
	nop
1:
	dmtc0	t8,C0_TLBHI
	mtc0	t9,C0_INX
	addu	t8,0x2000		/* inc vpn */
	tlbwi
	bnez	t9,1b
	subu	t9,1			# BDSLOT


#if 0
	/*
	 * Set up mappings for our board
	 */
	li		PHYSICAL,0x20000000
	li		VIRTUAL,0x20000000
	la		TABLE,tlb_mem_0

	move	INDEX,zero
tlb_install_entries:
	lw		SIZE,0(TABLE)
	beq		SIZE,-1,tlb_done
	nop
	mtc0	INDEX,C0_INX
	mtc0	SIZE,C0_PGMASK
	move	t1,VIRTUAL
	and		t1,TLBHI_ADDR_MASK
	mtc0	t1,C0_TLBHI
	move	t1,PHYSICAL
	and		t1,TLBLO_ADDR_MASK
	srl		t1,TLBLO_ADDR_SHIFT
	or		t1,TLBLO_G
	or		t1,TLBLO_V
	or		t1,TLBLO_D
	or		t1,TLBLO_CALG_WBACK
	mtc0	t1,C0_TLBLO0

	jal		tlb_inc
	nop

	move	t1,PHYSICAL
	and		t1,TLBLO_ADDR_MASK
	srl		t1,TLBLO_ADDR_SHIFT
	or		t1,TLBLO_G
	or		t1,TLBLO_V
	or		t1,TLBLO_D
	or		t1,TLBLO_CALG_WBACK
	mtc0	t1,C0_TLBLO1

	jal		tlb_inc
	nop

	tlbwi
	nop; nop; nop

	add		INDEX,1
	add		TABLE,4
	b		tlb_install_entries

#endif

	.set	reorder

tlb_done:
	move	ra,v0
	j		ra

	.end	init_tlb

#if 0
/*-----------------------------------------------------------------
 * Name:	tlb_inc()
 *
 * Description:	Increment the physical and virtual addresses by the
 *				page size
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	tlb_inc
tlb_inc:
	beq		SIZE,TLBMASK_MASK256kb,tlb_add_256K
	beq		SIZE,TLBMASK_MASK1Mb,tlb_add_1
	beq		SIZE,TLBMASK_MASK4Mb,tlb_add_4
	beq		SIZE,TLBMASK_MASK16Mb,tlb_add_16
	j		tlb_error

tlb_add_256K:
	add		PHYSICAL,0x40000
	add		VIRTUAL,0x40000
	j		ra
tlb_add_1:
	add		PHYSICAL,0x100000
	add		VIRTUAL,0x100000
	j		ra
tlb_add_4:
	add		PHYSICAL,0x400000
	add		VIRTUAL,0x400000
	j		ra
tlb_add_16:
	add		PHYSICAL,0x1000000
	add		VIRTUAL,0x1000000
	j		ra

	.end	tlb_inc

/*-----------------------------------------------------------------
 * Name:	tlb_error()
 *
 * Description:	There was an error during TLB initialization.  Do something.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	tlb_error
tlb_error:

1:	b		1b

	.end	tlb_error

/*------------------------------------------------------------------*/
/*
 * Tables for memory sizes
 */

tlb_mem_1024:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_896:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_768:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_640:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_512:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_384:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_256:
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
	.word	TLBMASK_MASK16Mb
tlb_mem_128:
	.word	TLBMASK_MASK16Mb
tlb_mem_96:
	.word	TLBMASK_MASK16Mb
tlb_mem_64:
	.word	TLBMASK_MASK16Mb
tlb_mem_32:
	.word	TLBMASK_MASK16Mb
	.word	-1

tlb_mem_16:
	.word	TLBMASK_MASK4Mb
tlb_mem_8:
	.word	TLBMASK_MASK4Mb
	.word	-1

tlb_mem_4:
	.word	TLBMASK_MASK1Mb
	.word	TLBMASK_MASK1Mb
	.word	-1

tlb_mem_1:
	.word	TLBMASK_MASK256kb
	.word	TLBMASK_MASK256kb
tlb_mem_0:
	.word	-1

#endif
#endif	/* PROM */

