/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: agent-cron.c
 *
 * RCS:  $Id: agent-cron.c,v 1.6 2002/09/28 00:09:34 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Used by cron job to start mirror operations
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()
#include <unistd.h>     // close()

#include <sys/time.h>
#include <sys/socket.h>
#include <signal.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <errno.h>

#include "exports/elog-api.h"

#include "exports/anpssc-api.h"
#include "exports/ncm-api.h"
#include "exports/evm-api.h"
#include "exports/fs-api.h"
#include "agent-api.h"

/*
 * Constants.
 */

// Wait for Agent to respond (10s)
static struct timeval waitAmTo = {10, 0};

int32 amAppId;

/*
 *------------------------------------------------------------------------------
 * Name         : main
 *
 * Description  : Start a mirror operation
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

int32 main(int argc, char **argv)
{
	int32			rspLen;
	am_msg_cmd_t	amReq;
    am_msg_cmd_t    amRsp;
	am_srcConfig_t	*srcCfgPtr;

	amAppId = eee_getApplicationId("am");

    bzero((void *) &amReq, sizeof(am_msg_cmd_t));
    bzero((void *) &amRsp, sizeof(am_msg_cmd_t));

    amReq.msgType = AM_USER_START_MIRROR_REQ;
	srcCfgPtr = amReq.srcCfg;
    strcpy(srcCfgPtr->amName, argv[1]);

	AM_FCT_PRINT(("AM Cron: Sending mirror start request for %s", 
						srcCfgPtr->amName));

    /*
     * Send message to AM.
     */
    rspLen = sendAgileMsg(-1, &amReq, sizeof(am_msg_cmd_t),
                          "am", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                          &waitAmTo, &amRsp, sizeof(am_msg_cmd_t));

    /*
     * Receive response from AM.
     */
    if (rspLen < sizeof(am_msg_cmd_t))
    {
		AM_ERR_PRINT(("AM Cron: Mirror %s No response from - am -",
				srcCfgPtr->amName));
		return NFX_ERR;
    }
    else if (amRsp.msgType != AM_USER_START_MIRROR_RSP)
    {
		AM_ERR_PRINT(("AM Cron: Incorrect message type from - am -"));
		return NFX_ERR;
    }

	if (amRsp.result != NFX_OK)
	{
		AM_ERR_PRINT(("AM Cron: Result %d Start failed", amRsp.result));
		return NFX_ERR;
	}

	AM_FCT_PRINT(("AM Cron: Started mirror operation for %s",
				srcCfgPtr->amName));

    return NFX_OK;
}
