/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : sim-ctl-rx.c
 *
 * Description  :
 * Simulates Agile Mirror (AM) controller.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 05-NOV-2001
 *
 *------------------------------------------------------------------------------
 */
#ifdef SSC

#include <stdlib.h>         // exit
#include <stdio.h>          // fprintf
#include <signal.h>

#include "exports/anpssc-api.h"
#include "am-agent-api.h"

/*------------------------------------------------------------------------------
 * Data structures definition
 *------------------------------------------------------------------------------
 */

/* Basic message structure. */
typedef struct
{
    char    buf[500];  // optimize!!!
    uint32  len;
} am_msg_t;

/*------------------------------------------------------------------------------
 * Prototype of local functions.
 *------------------------------------------------------------------------------
 */

static void
am_terminate(int sig);

/*------------------------------------------------------------------------------
 * Global variables
 *------------------------------------------------------------------------------
 */

/* AM's file descriptor */
static int fd;

/*------------------------------------------------------------------------------
 * Name         : am_terminate()
 *
 * Description  : Signal handler which provides orderly termination.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 05-NOV-2001
 *
 *------------------------------------------------------------------------------
 */

static void
am_terminate(int sig)

{
    printf("> am_terminate()\n");

    /*
     * Unregister EV.
     */
    unRegAgileApp(fd);
    
    printf("< am_terminate()\n");
    exit(NFX_OK);
}

/*------------------------------------------------------------------------------
 * Name         : main
 *
 * Description  : AM's entry point.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 05-NOV-2001
 *
 *------------------------------------------------------------------------------
 */

int main(int argc, char **argv, char **envp) 
{
    printf("> main()\n");

    /*
     * Register to receive messages for AM controller.
     */
	fd = regAgileApp("am");
	if (fd < 0)
    {
        perror("regAgileApp() failed");
		exit(NFX_ERR);
	}
    /*
     * Sets the procedure to follow in case of request for program termination
     * or CTRL-C interrupt.
     */
    signal(SIGTERM, am_terminate);
    signal(SIGINT, am_terminate);
    /*
	 * Receive messages and process them.
     */
	while (TRUE)
    {
        am_msg_t        msg;

        /*
         * Wait for messages...
         */
		msg.len = recvAgileMsg(fd,
                               msg.buf, sizeof(msg.buf),
                               NULL,
                               NULL,
                               NULL,
                               NULL,
                               NULL);
        /*
         * Something has been received, check it.
         */
		if (msg.len < 0)
        {
			printf("recvAgileMsg() failed\n");
		} 
		else
        {
            am_msgHdr_t * hdr;

            hdr = (am_msgHdr_t *) &(msg.buf);
            
            printf("A message (%d) has been received!\n", hdr->msgType);
            printf("agentHdl = %d\n", hdr->agentHdl);
            printf("ctldmHdl = %d\n", hdr->ctldmHdl);

            switch(hdr->msgType)
            {
            case AM_SRC_CREAT_AGENT_RSP:
            {
                printf("AM_SRC_CREAT_AGENT_RSP\n");
                break;
            }
    
            case AM_SRC_KILL_AGENT_RSP:
            {
                printf("AM_SRC_KILL_AGENT_RSP\n");
                break;
            }
    
            case AM_SRC_START_MIRROR_RSP:
            {
                printf("AM_SRC_START_MIRROR_RSP\n");
                break;
            }
    
            case AM_TGT_CREAT_AGENT_RSP:
            {
                printf("AM_TGT_CREAT_AGENT_RSP\n");
                break;
            }
    
            case AM_TGT_KILL_AGENT_RSP:
            {
                printf("AM_TGT_KILL_AGENT_RSP\n");
                break;
            }
    
            case AM_TGT_START_MIRROR_RSP:
            {
                printf("AM_TGT_START_MIRROR_RSP\n");
                break;
            }

            case AM_TGT_MIRROR_DONE_IND:
            {
                printf("AM_TGT_MIRROR_DONE_IND\n");
                break;
            }
            
            default:
            {
                printf("Unknow message type (%d)\n", hdr->msgType);
                break;
            }
            } // switch
        }
    } /* while */
    
    printf("< main()\n");
    exit(NFX_ERR);  // should never exit by this way!
}

/*----------------------------------------------------------------------------*/
#endif  /* SSC */
