/*
 *-----------------------------------------------------------------
 *
 * Name:	bp_init.c
 *
 * RCS:  $Id: bp_init.c,v 1.8 2002/11/07 01:23:36 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Initialize devices related to the backplane
 *
 * Created by:		Tony Wong
 *
 * Date Created:	01/04/02
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/backplane-api.h>

#ifdef AGILE_HARDWARE
#include <sys/mips.h>
#include <exports/hwMap.h>

typedef int *common_func_mii(uint32, uint32, uint32, uint32);

extern void bp_init_ssc(void);
extern void bp_init_fc(void);
extern void bp_init_fp(void);
extern void bp_init_txrx(void);

void
bp_init()
{
	if (isSSC()) {
		bp_init_ssc();
	}
	else
	if (isFC()) {
		bp_init_fc();
	}
	else
	if (isFP()) {
		bp_init_fp();
	}
	if (isTXRX()) {
		bp_init_txrx();
	}
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   common_init_phy8353
 *
 * Description:     Initialization routine for PMC-8353 PHY
 *
 *                  1. Errata: reg 0x1e = 0x4090, then 0x1e = 0090 for PLL lock
 *                  2. Soft Reset
 *                  3. IDECENEN = 1, PCS_EN = 1, TXCLK4 = 1
 *
 *                  To be called in all images with proper MII PHY access
 *                  function, unit id, and device id.
 *                  Example:
 *                      common_init_phy8353(gt_mii_write_SMI_common, 0, 0x10);
 *
 * Created by:      Tony Wong
 *
 * Data Created:    02/05/02
 *
 * Returns:
 *
 *------------------------------------------------------------------------------ */
void
common_init_phy8353(common_func_mii *fp, uint32 unit, uint32 dev_id)
{
	int dummy;

	/*
	 *  Support write verify with mask if unit == 0xff 
	 * "unit" won't be needed when 5600 switch and rev 1 SSC 
     * is no longer supported 
	 */

	if (unit == 0xff) {
	    dummy = *fp(0xffff, dev_id, 0x1e, 0x4090);
	    dummy = *fp(0xffff, dev_id, 0x1e, 0x0090);
	    dummy = *fp(0xffff, dev_id, 0x11, 0x6308);
	    dummy = *fp(0xffff, dev_id, 0x11, 0xe384);
	    dummy = *fp(0xffff, dev_id, 0x1d, 0x0ff0);

	    dummy = *fp(0xffff, dev_id, 0x10, 0xaae8);
	   	dummy = *fp(0x00ff, dev_id, 0x1f, 0x0028);

		/* disable auto-negotiate, this is especially for backplane 
         * switch on ssc
         */
 	   	dummy = *fp(0xffff, dev_id, 0x00, 0x0140);
	}
	else {
	    dummy = *fp(unit, dev_id, 0x1e, 0x4090);
	    dummy = *fp(unit, dev_id, 0x1e, 0x0090);
	    dummy = *fp(unit, dev_id, 0x11, 0x6308);
	    dummy = *fp(unit, dev_id, 0x11, 0xe384);
	    dummy = *fp(unit, dev_id, 0x1d, 0x0ff0);

	    dummy = *fp(unit, dev_id, 0x10, 0xaae8);
	    dummy = *fp(unit, dev_id, 0x1f, 0x0028);

		/* disable auto-negotiate, this is especially for backplane 
         * switch on ssc
         */
 	   	dummy = *fp(unit, dev_id, 0x00, 0x0140);
	}
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   common_init_phy8354
 *
 * Description:     Initialization routine for PMC-8354 PHY
 *
 *                  Same as for 8353 except don't write to register 0x1f
 *
 * Created by:      Tony Wong
 *
 * Data Created:    07/11/03
 *
 * Returns:
 *
 *------------------------------------------------------------------------------ */
void
common_init_phy8354(common_func_mii *fp, uint32 unit, uint32 dev_id)
{
	int dummy;

	/*
	 *  Support write verify with mask if unit == 0xff 
	 * "unit" won't be needed when 5600 switch and rev 1 SSC 
     * is no longer supported 
	 */

	if (unit == 0xff) {
	    dummy = *fp(0xffff, dev_id, 0x1e, 0x4090);
	    dummy = *fp(0xffff, dev_id, 0x1e, 0x0090);
	    dummy = *fp(0xffff, dev_id, 0x11, 0x6308);
	    dummy = *fp(0xffff, dev_id, 0x11, 0xe384);
	    dummy = *fp(0xffff, dev_id, 0x1d, 0x0ff0);

	    dummy = *fp(0xffff, dev_id, 0x10, 0xaae8);

		/* disable auto-negotiate, this is especially for backplane 
         * switch on ssc
         */
 	   	dummy = *fp(0xffff, dev_id, 0x00, 0x0140);
	}
	else {
	    dummy = *fp(unit, dev_id, 0x1e, 0x4090);
	    dummy = *fp(unit, dev_id, 0x1e, 0x0090);
	    dummy = *fp(unit, dev_id, 0x11, 0x6308);
	    dummy = *fp(unit, dev_id, 0x11, 0xe384);
	    dummy = *fp(unit, dev_id, 0x1d, 0x0ff0);

	    dummy = *fp(unit, dev_id, 0x10, 0xaae8);

		/* disable auto-negotiate, this is especially for backplane 
         * switch on ssc
         */
 	   	dummy = *fp(unit, dev_id, 0x00, 0x0140);
	}
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   common_init_phy5228
 *
 * Description:     Initialization routine for BCM-5228 PHY
 *
 *                  Full-duplex, no auto-neg, force 100MBS
 *
 *                  To be called in all images with proper MII PHY access
 *                  function, unit id, and device id.
 *                  Example:
 *                      common_init_phy5228(gt_mii_write_SMI_common, 0, 0x10);
 *
 * Created by:      Tony Wong
 *
 * Data Created:    02/05/02
 *
 * Returns:
 *
 *------------------------------------------------------------------------------ */
void
common_init_phy5228(common_func_mii *fp, uint32 unit, uint32 dev_id)
{
	int dummy;

    dummy = *fp(unit, dev_id, 0x00, 0x2100);
}

#endif	/* AGILE_HARDWARE */
