/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: bp_ui.c
 *
 * RCS:  $Id: bp_ui.c,v 1.13 2002/11/14 20:59:28 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Contains routines to handle backplane UI commands.
 *
 * Created by:	Tony Wong
 *
 * Date Created: 01/17/2002
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/backplane-api.h>
#ifndef PROM
#ifdef AGILE_HARDWARE
#include <exports/hwMap.h>
#if defined(FCNIM) || defined(SSC_MGMT)
#include <exports/gt64240mii-regs.h>
#endif
#include <exports/cm-api.h>

int32 bp_cmdHelp( int32 argc, char *argv[] );
int32 bp_cmdShow( int32 argc, char *argv[] );

typedef struct
{
	int32	(*cmdFunc)( int32 argc, char *argv[] );
	char	*cmd;
	char	*helpTxt;
} bp_cmdTbl_t;

bp_cmdTbl_t	bp_cmdTable[ ] =
{
#if defined(FCNIM)
	{ bp_cmdShow, 	"show", 	"bp show data|mgmt" }
#elif defined(SSC_MGMT)
	{ bp_cmdShow, 	"show", 	"bp show data|mgmt <port id>" }
#elif defined(NFP_TXRX)
	{ bp_cmdShow, 	"show", 	"bp show data|mgmt" }
#elif defined(NFP_FP)
	{ bp_cmdShow, 	"show", 	"bp show data|mgmt" }
#endif
};

#define BP_CMD_MAX		sizeof(bp_cmdTable)/sizeof(bp_cmdTbl_t)

/*-----------------------------------------------------------------
 * Name: bp_cmdHelpAll
 *
 * Description: Display help-texts for all backplane commands.
 *
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmdHelpAll( void )
{
	int32 i;

	printf("Usage:");
	for (i=0; i < BP_CMD_MAX; i++)
	{
		printf("\t%s\n", bp_cmdTable[i].helpTxt);
	}
	return 0;
}

/*-----------------------------------------------------------------
 * Name: bp_cmdHelpCmd
 *
 * Description: Display help-texts for individual backplane commands.
 *
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmdHelpCmd( char *pCmd )
{
	int32 i;

	for (i=0; i < BP_CMD_MAX; i++)
	{
		if (strequ(pCmd, bp_cmdTable[i].cmd))
		{
			printf("Usage:");
			printf("\t%s\n", bp_cmdTable[i].helpTxt);
			return 0;
		}
	}
	return (bp_cmdHelpAll( ));
}

/*-----------------------------------------------------------------
 * Name: bp_cmdHelp
 *
 * Description: Display help-texts.  
 *
 *				bp help [show | trace]
 *  
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmdHelp( int32 argc, char *argv[] )
{
	if (argc < 3)
		return (bp_cmdHelpAll());
	else 
		return (bp_cmdHelpCmd( argv[2] ));
}

#ifdef FCNIM
int32 bp_showStatsData_fc()
{
	extern int dp83820_bpStats(void);

	/* this is only available on the same slice the request is made from */
    printf("FC Data Plane Statistics\n");
    printf("========================\n");
	dp83820_bpStats();
	return 0;
}

int32 bp_showStatsCtrl_fc()
{
	uint32 port;
	extern int gt_mii_bpStats(uint32);

    printf("FC Control Plane Statistics\n");
    printf("===========================\n");
	for (port = 0; port <= 2; port += 2) {
    	printf("FE Port %d\n", port/2 + 1);
		gt_mii_bpStats(port);
	}
	return 0;
}

int32 bp_showStatsMgmt_fc()
{
	extern int gt_mii_bpStats(uint32);

    printf("FC Management Plane Statistics\n");
    printf("==============================\n");
	gt_mii_bpStats(1);
	return 0;
}

int32 bp_showStatsSwitch_fc(uint32 slice)
{
	uint32 i, port[2];
	extern void bcm5600_bpStats_FE(uint32, uint32);
	extern void bcm5600_bpStats_GE(uint32, uint32);

    switch (slice) {
    case 0: port[0] = 19;
            port[1] = 23;
            break;
    case 1: port[0] = 18;
            port[1] = 22;
            break;
    case 2: port[0] = 17;
            port[1] = 21;
            break;
    case 3: port[0] = 16;
            port[1] = 20;
            break;
    default:break;
    }

    printf("FC Switch Statistics\n");
    printf("====================\n");
	for (i = 0; i < 2; i++) {
		printf("FE Port %d\n", i+1);
		bcm5600_bpStats_FE(0, port[i]);
		printf("\n");
	}
	for (i = 0; i < 2; i++) {
    	printf("GE Port %d\n", i+1);
		bcm5600_bpStats_GE(0, 24+i);
		printf("\n");
	}
	return 0;
}
#endif

/*-----------------------------------------------------------------
 * Name: bp_cmdShow_fc
 *
 * Description: Dispatch for backplane SHOW commands.  
 *
 *				bp show [data|mgmt]
 *  
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmdShow_fc( int32 argc, char *argv[] )
{
#ifdef FCNIM

	if (argc < 3)
	{
		bp_cmdHelpCmd("show");
		return 0;
	}
	if (strequ(argv[2], "data"))
	{
		return (bp_showStatsData_fc());	
	}
	else if (strequ(argv[2], "mgmt"))
	{
		return (bp_showStatsMgmt_fc());	
	}
	else
		return (bp_cmdHelpCmd("show"));
#endif
	return 0;
}

#ifdef SSC_MGMT
int32 bp_showStatsDataCtrl_ssc(uint32 port)
{
	extern int bcm5632_bpStats(uint32, uint32);

    printf("SSC Control/Data Plane Statistics\n");
    printf("=================================\n");
   	printf("GE Port %d\n", port);
	bcm5632_bpStats(0, port);
	return 0;
}

int32 bp_showStatsMgmt_ssc(uint32 port)
{
	extern void bcm5600_bpStats_FE(uint32, uint32);

    printf("SSC Management Plane Statistics\n");
    printf("===============================\n");
   	printf("FE Port %d\n", port);
	bcm5600_bpStats_FE(0, port);
	return 0;
}
#endif

int32 bp_cmdShow_ssc( int32 argc, char *argv[] )
{
#ifdef SSC_MGMT
	uint32 port;

	if (argc < 4)
	{
		bp_cmdHelpCmd("show");
		return 0;
	}
	port = atoi(argv[3]);
	if (port > 11) {
		printf("Port Id must be 0-11\n"); 
		return 0;
	}
	if (strequ(argv[2], "ctrl"))
	{
		return (bp_showStatsDataCtrl_ssc(port));	
	}
	else if (strequ(argv[2], "data"))
	{
		return (bp_showStatsDataCtrl_ssc(port));	
	}
	else if (strequ(argv[2], "mgmt"))
	{
		return (bp_showStatsMgmt_ssc(port));	
	}
#endif
	return 0;
}

#ifdef NFP_TXRX
int32 bp_showStatsMgmt_txrx()
{
	extern int dp83820_bpStats(void);

    printf("TXRX Management Plane Statistics\n");
    printf("================================\n");
	dp83820_bpStats();
	return 0;
}

#endif

int32 bp_cmdShow_txrx( int32 argc, char *argv[] )
{
#ifdef NFP_TXRX
	if (argc < 3)
	{
		bp_cmdHelpCmd("show");
		return 0;
	}
	if (strequ(argv[2], "data"))
	{
		printf("Data plane stats are on FP!\n");
	}
	else if (strequ(argv[2], "mgmt"))
	{
		return (bp_showStatsMgmt_txrx());	
	}
#endif
	return 0;
}

#ifdef NFP_FP
int32 bp_showStatsCtrl_fp()
{
	extern int bmc12500Eth_bpStats(uint32);
	extern void diag_lsi_viewStats(int);
	extern void diag_lsi_clearStats(int);

    printf("FP Control Plane Statistics\n");
    printf("===========================\n");
	/* display stats for both control plane ports */
	printf("GE Port 1\n");
	diag_lsi_viewStats(2);
	printf("\n");
	bmc12500Eth_bpStats(0);
	printf("\n");
	printf("GE Port 2\n");
	diag_lsi_viewStats(5);
	printf("\n");
	bmc12500Eth_bpStats(2);
	diag_lsi_clearStats(2);
	diag_lsi_clearStats(5);
	return 0;
}

int32 bp_showStatsData_fp()
{
	extern int bmc12500Eth_bpStats(uint32);
	extern void bmc12500Eth_clear_stats(uint32);
	extern void diag_lsi_viewStats(int);
	extern void diag_lsi_clearStats(int);

    printf("FP Data Plane Statistics\n");
    printf("========================\n");
	/* display stats for four data plane ports */
	printf("GE Port 1\n");
	diag_lsi_viewStats(0);
	printf("\n");
	printf("GE Port 2\n");
	diag_lsi_viewStats(1);
	printf("\n");
	printf("GE Port 1 and 2\n");
	bmc12500Eth_bpStats(0);
	printf("GE Port 3\n");
	diag_lsi_viewStats(3);
	printf("\n");
	printf("GE Port 4\n");
	diag_lsi_viewStats(4);
	printf("\n");
	printf("GE Port 3 and 4\n");
	bmc12500Eth_bpStats(2);

	diag_lsi_clearStats(0);
	diag_lsi_clearStats(1);
	diag_lsi_clearStats(3);
	diag_lsi_clearStats(4);
	bmc12500Eth_clear_stats(0);
	bmc12500Eth_clear_stats(2);
	return 0;
}

int32 bp_showStatsDataCtrl_fp()
{
	uint32 port;
	extern int bmc12500Eth_bpStats(uint32);
	extern void bmc12500Eth_clear_stats(uint32);

    printf("FP Control/Data Plane Statistics\n");
    printf("================================\n");
	for (port = 0; port < 2; port++) {
		printf("GE Port %d\n", port+1);
		bmc12500Eth_bpStats(port);
		bmc12500Eth_clear_stats(port);
	}
	return 0;
}
#endif

int32 bp_cmdShow_fp( int32 argc, char *argv[] )
{
#ifdef NFP_FP
	if (argc < 3)
	{
		bp_cmdHelpCmd("show");
		return 0;
	}
	if (strequ(argv[2], "data"))
	{
		if (isCheetah())
			return (bp_showStatsData_fp());	
		else
			return (bp_showStatsDataCtrl_fp());	
	}
	else if (strequ(argv[2], "mgmt"))
	{
		printf("Management plane stats are on TXRX!\n");
	}
#endif
	return 0;
}

/*-----------------------------------------------------------------
 * Name: bp_cmdShow
 *
 * Description: Dispatch for backplane SHOW commands.  
 *
 *				bp show [data|ctrl|mgmt|switch]
 *  
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmdShow( int32 argc, char *argv[] )
{
	if (isFC()) {
		bp_cmdShow_fc(argc, argv);
	}
	else
	if (isSSC()) {
		bp_cmdShow_ssc(argc, argv);
	}
	else
	if (isTXRX()) {
		bp_cmdShow_txrx(argc, argv);
	}
	else
	if (isFP()) {
		bp_cmdShow_fp(argc, argv);
	}
	return 0;
}

/*-----------------------------------------------------------------
 * Name: bp_cmd
 *
 * Description: Main dispatch for backplane commands.  Called from 
 *				command table.
 *
 *				bp show [data|ctrl|mgmt|switch]
 *  
 * Created by : Tony Wong
 *  
 * Date Created: 01/17/2002
 *-----------------------------------------------------------------
 */
int32 bp_cmd( int32 argc, char *argv[] )
{
	int32	i;

	if (argc < 2)
	{
		return (bp_cmdHelp(argc, argv));
	}

	for (i=0; i < BP_CMD_MAX; i++)
	{
		if (strequ(argv[1], bp_cmdTable[i].cmd))
		{
			return ((*bp_cmdTable[i].cmdFunc)(argc, argv));
		}
	}
	return (bp_cmdHelp(argc, argv));
}
#endif /* AGILE_HARDWARE */
#endif /* PROM */
