/*
 *=============================================================================
 * File Name   :    bcm5600-diag.c
 *
 *  RCS: $Id: bcm5600-diag.c,v 1.19 2002/11/07 01:23:17 rick Exp $
 *
 *  Description:	Diagnostics menu
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#ifndef SSC_MGMT
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#endif

#include "bcm5600-soctypes.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"
#include "bcm5600-port.h"
#include "bcm5600-l2.h"
#include "bcm5600-ll.h"
#include "bcm5600-phy.h"

extern int soc_reset_init(int);
extern int bcm_init_5600(int);
extern int bcm5600_port_monitor_ingress_set(int, soc_port_t, int);
extern void watchdog_disable(void);
extern void watchdog_enable(int32);
extern int get_cmd(char8 *);

int
diag5600_uiGetInput( char *msg )
{
   int sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   return sel;
} /* diag5600_uiGetInput */

int
diag5600_uiGetHex( char *msg )
{
   uint32 sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%x", &sel);
   return sel;
} /* diag5600_uiGetHex */


void
diag5600_showPCI()
{
#ifndef SSC_MGMT
   uint32 reg_val;

   printf("PCI registers\n");
   printf("=============\n");
   pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x00, &reg_val);
   printf("Vendor ID = %0x\n", reg_val);

   pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x04, &reg_val);
   printf("Command/Status = %0x\n", reg_val);

   pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x08, &reg_val);
   printf("Class Code/Revision ID = %0x\n", reg_val);

   pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x0c, &reg_val);
   printf("Header/Latency/Cacheline Size = %0x\n", reg_val);

   pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x10, &reg_val);
   printf("Base Addr = %0x\n", reg_val);
#else
	printf("Not supported!\n");
#endif
}

void
diag5600_writePCI()
{
#ifndef SSC_MGMT
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = diag5600_uiGetHex("Enter reg addr(hex): ");

   reg_val = diag5600_uiGetHex("Enter reg value(hex): ");

   pci_cfg_write(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, reg_addr, reg_val);
#else
	printf("Not supported!\n");
#endif
}

void
diag5600_readReg()
{
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = diag5600_uiGetHex("Enter reg addr(hex): ");

   reg_val = cmic_pci_getreg(0, reg_addr);

   printf("Value at %0x: %0x\n", reg_addr, reg_val);
}

void
diag5600_writeReg()
{
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = diag5600_uiGetHex("Enter reg addr(hex): ");

   reg_val = diag5600_uiGetHex("Enter reg value(hex): ");

   cmic_pci_setreg(0, reg_addr, reg_val);
}

void
diag5600_readIndReg()
{
   uint32 reg_addr;
   uint32 reg_val;
   uint32 rc = SOC_E_INTERNAL;

   reg_addr = diag5600_uiGetHex("Enter indirect reg addr(hex): ");

   rc = cmic_read_reg(0, reg_addr, &reg_val);

   if (rc == SOC_E_NONE)
       printf("Value at %0x: %0x\n", reg_addr, reg_val);
   else
       printf("Error reading from indirect reg at %0x\n", reg_addr);
}

void
diag5600_writeIndReg()
{
   uint32 reg_addr;
   uint32 reg_val;
   uint32 rc = SOC_E_INTERNAL;

   reg_addr = diag5600_uiGetHex("Enter indirect reg addr(hex): ");

   reg_val = diag5600_uiGetHex("Enter reg value(hex): ");

   rc = cmic_write_reg(0, reg_addr, reg_val);

   if (rc != SOC_E_NONE)
       printf("Error writing to indirect reg at %0x\n", reg_addr);
}

void
diag5600_viewStatus()
{
   uint32 status, i, loopback, loopmac;

   status = cmic_pci_getreg(0, CMIC_LINK_STAT);
   for (i = 0; i < NUM_FE_PORT_5600; i++)
   {
       printf("Port #%d status: ", i);
	   loopback = 0;
/*
       phy_fe_ge_lb_get(0, i, &loopback);
*/
       READ_FE_MAC1r(0, i, &loopmac);
       if (loopback || GET_FE_MAC1r_LBACK(loopmac)) 
           printf("Loopback\n");
       else
       if (status & (1 << i))
           printf("Up\n");
       else
           printf("Down\n");
   }
}

/* 
 * view/clear MIB for one port if display=1
 * clear MIB for all FE ports if display=0 
 */
void
diag5600_viewMIB(int32 display) 
{
   uint32 data;
   uint32 port = 0;

   if (display) {
      port = diag5600_uiGetInput("Enter port number(0-25): ");
      printf("MIB counters\n");
      printf("============\n");
   }
   if (port > 23)
   {
      READ_GTR64r(0,port,&data);
      printf("GE Tx/Rx 64 byte frames: %d\n", data);

      READ_GTR127r(0,port,&data);
      printf("GE Tx/Rx 65-127 byte frames: %d\n", data);

      READ_GTR255r(0,port,&data);
      printf("GE Tx/Rx 128-255 byte frames: %d\n", data);

      READ_GTR511r(0,port,&data);
      printf("GE Tx/Rx 256-511 byte frames: %d\n", data);

      READ_GTR1023r(0,port,&data);
      printf("GE Tx/Rx 512-1023 byte frames: %d\n", data);

      READ_GTRMAXr(0,port,&data);
      printf("GE Tx/Rx 1024-max byte frames: %d\n", data);

      READ_GRBCAr(0,port,&data);
      printf("GE Rx broadcast frames: %d\n", data);

      READ_GRMCAr(0,port,&data);
      printf("GE Rx multicast frames: %d\n", data);

      READ_GRBYTr(0,port,&data);
      printf("GE Rx byte count: %d\n", data);

      READ_GRPKTr(0,port,&data);
      printf("GE Rx frame count: %d\n", data);

      READ_GRCDEr(0,port,&data);
      printf("GE Rx code error count: %d\n", data);

      READ_GRALNr(0,port,&data);
      printf("GE Rx align error frames: %d\n", data);

      READ_GRDISCr(0,port,&data);
      printf("GE Rx discard packet count: %d\n", data);

      READ_GRFCRr(0,port,&data);
      printf("GE Rx false carrier count: %d\n", data);

      READ_GRFCSr(0,port,&data);
      printf("GE Rx FCS error count: %d\n", data);

      READ_GRFLRr(0,port,&data);
      printf("GE Rx length out of range count: %d\n", data);

      READ_GROVRr(0,port,&data);
      printf("GE Rx oversize frame count: %d\n", data);

      READ_GRUNDr(0,port,&data);
      printf("GE Rx undersize frame count: %d\n", data);

      READ_GRJBRr(0,port,&data);
      printf("GE Rx jabber frame count: %d\n", data);


      READ_GTPKTr(0,port,&data);
      printf("GE Tx frame count: %d\n", data);

      READ_GTBYTr(0,port,&data);
      printf("GE Tx byte count: %d\n", data);

      READ_GTBCAr(0,port,&data);
      printf("GE Tx broadcast frames: %d\n", data);

      READ_GTMCAr(0,port,&data);
      printf("GE Tx multicast frames: %d\n", data);

      READ_GTVLANr(0,port,&data);
      printf("GE Tx tagged VLAN packet count: %d\n", data);

      READ_GTFCSr(0,port,&data);
      printf("GE Tx FCS error count: %d\n", data);

      READ_GTABRTr(0,port,&data);
      printf("GE Tx aborted packet count: %d\n", data);

      READ_GTOVRr(0,port,&data);
      printf("GE Tx oversize frame count: %d\n", data);

      READ_GTJBRr(0,port,&data);
      printf("GE Tx jabber frame count: %d\n", data);

   }
   else
   {
again:
      READ_TR64r(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 64 byte frames: %d\n", data);

      READ_TR127r(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 65-127 byte frames: %d\n", data);

      READ_TR255r(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 128-255 byte frames: %d\n", data);

      READ_TR511r(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 256-511 byte frames: %d\n", data);

      READ_TR1023r(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 512-1023 byte frames: %d\n", data);

      READ_TRMAXr(0,port,&data);
	  if (display)
      printf("FE Tx/Rx 1024-max byte frames: %d\n", data);

      READ_RBCAr(0,port,&data);
	  if (display)
      printf("FE Rx broadcast frames: %d\n", data);

      READ_RMCAr(0,port,&data);
	  if (display)
      printf("FE Rx multicast frames: %d\n", data);

      READ_RBYTr(0,port,&data);
	  if (display)
      printf("FE Rx byte count: %d\n", data);

      READ_RPKTr(0,port,&data);
	  if (display)
      printf("FE Rx frame count: %d\n", data);

      READ_RCDEr(0,port,&data);
	  if (display)
      printf("FE Rx code error count: %d\n", data);

      READ_RALNr(0,port,&data);
	  if (display)
      printf("FE Rx align error frames: %d\n", data);

      READ_RDISCr(0,port,&data);
	  if (display)
      printf("FE Rx discard packet count: %d\n", data);

      READ_RFCRr(0,port,&data);
	  if (display)
      printf("FE Rx false carrier count: %d\n", data);

      READ_RFCSr(0,port,&data);
	  if (display)
      printf("FE Rx FCS error count: %d\n", data);

      READ_RFLRr(0,port,&data);
	  if (display)
      printf("FE Rx length out of range count: %d\n", data);

      READ_ROVRr(0,port,&data);
	  if (display)
      printf("FE Rx oversize frame count: %d\n", data);

      READ_RUNDr(0,port,&data);
	  if (display)
      printf("FE Rx undersize frame count: %d\n", data);

      READ_RJBRr(0,port,&data);
	  if (display)
      printf("FE Rx jabber frame count: %d\n", data);

      READ_RFRGr(0,port,&data);
	  if (display)
      printf("FE Rx fragment count: %d\n", data);

	  READ_RXPFr(0,port,&data);
	  if (display)
	  printf("FE Rx pause frame count: %d\n", data);

      READ_TPKTr(0,port,&data);
	  if (display)
      printf("FE Tx frame count: %d\n", data);

      READ_TBYTr(0,port,&data);
	  if (display)
      printf("FE Tx byte count: %d\n", data);

      READ_TBCAr(0,port,&data);
	  if (display)
      printf("FE Tx broadcast frames: %d\n", data);

      READ_TMCAr(0,port,&data);
	  if (display)
      printf("FE Tx multicast frames: %d\n", data);

      READ_TVLANr(0,port,&data);
	  if (display)
      printf("FE Tx tagged VLAN packet count: %d\n", data);

      READ_TFCSr(0,port,&data);
	  if (display)
      printf("FE Tx FCS error count: %d\n", data);

      READ_TABRTr(0,port,&data);
	  if (display)
      printf("FE Tx aborted packet count: %d\n", data);

      READ_TOVRr(0,port,&data);
	  if (display)
      printf("FE Tx oversize frame count: %d\n", data);

      READ_TFRGr(0,port,&data);
	  if (display)
      printf("FE Tx fragment packet count: %d\n", data);

      READ_TJBRr(0,port,&data);
	  if (display)
         printf("FE Tx jabber frame count: %d\n", data);

	  READ_TXPFr(0,port,&data);
	  if (display)
		 printf("FE Tx pause frame count: %d\n", data);
	  else if (++port < 24)
         goto again;
   }
}

void
diag5600_viewARL()
{
   uint32 num, index;
   arl_entry_t entry;

   num = diag5600_uiGetInput("Enter number of entries to display(1-8192): ");
   if (num > BCM5600_MAX_ARL_ENTRIES)
   {
       printf("Invalid entry!\n");
       return;
   }
   watchdog_disable();
   for (index = 1; index <= num; index++) {
       if (cmic_read_memory(0, ARL_m, 0, index, &entry) < 0)
           printf("Failed to read ARL entry #%d\n", index);
       else
       {
			if ((entry.mac_addr_b0 == 0xff) && (entry.mac_addr_b1 == 0xff) &&
				(entry.mac_addr_b2 == 0xff) && (entry.mac_addr_b3 == 0xff) &&
				(entry.mac_addr_b4 == 0xff) && (entry.mac_addr_b5 == 0xff))
				;
			else {
           	  printf("ARL entry #%d, port #%d, ", index, entry.port);
           	  printf("MAC addr: %0x %0x %0x %0x %0x %0x\n", entry.mac_addr_b0,
                    entry.mac_addr_b1, entry.mac_addr_b2, entry.mac_addr_b3, 
                    entry.mac_addr_b4, entry.mac_addr_b5);
			  if (entry.st == 1)
				 printf("Static entry\n");
			}
       }
   }
   watchdog_enable(8);
}
#if 0
void
diag5600_setLoopPhy()
{
   uint32 port, enable;

   enable = diag5600_uiGetInput("Enter loopback mode (1=Enable, 0=Disable): ");
   if (enable > 1)
   {
       printf("Invalid entry!\n");
       return;
   }
   port = diag5600_uiGetInput("Enter port number(0-23): ");
   if (port > 23)
   {
       printf("Invalid entry!\n");
       return;
   }
   if (enable)
       cmic_link_scan_pause(0);
   else
       cmic_link_scan_enable(0);

   PHY_LOOPBACK_SET(&phy_default_fe, 0, port, enable);
}
#endif

void
diag5600_setLoopMac()
{
   uint32 port, enable;

   enable = diag5600_uiGetInput("Enter loopback mode (1=Enable, 0=Disable): ");
   if (enable > 1)
   {
       printf("Invalid entry!\n");
       return;
   }
   port = diag5600_uiGetInput("Enter port number(0-25): ");
   if (port >= NUM_E_PORT_5600)
   {
       printf("Invalid entry!\n");
       return;
   }
#if 0
   if (enable)
       cmic_link_scan_pause(0);
   else
       cmic_link_scan_enable(0);
#endif
   if (port < NUM_FE_PORT_5600)
   {
      MAC_LOOPBACK_SET(&mac_fe_5600, 0, port, enable);
   }
   else
   {
      MAC_LOOPBACK_SET(&mac_ge_5600, 0, port, enable);
   }
}

void
diag5600_sys_dvt_mgmt_port_mac_loop_set()
{
	uint32 loopmac;

	/* set port 7 to enable mac lpbk*/
	MAC_LOOPBACK_SET(&mac_fe_5600, 0, 7, 1);
    READ_FE_MAC1r(0, 7, &loopmac);

    if (GET_FE_MAC1r_LBACK(loopmac)) 
       printf("Port 7: %dLoopback\n", loopmac);
}

void
diag5600_sys_dvt_mgmt_port_mac_loop_unset()
{
	uint32 loopmac;
	/* set port 7 to enable mac lpbk*/
	MAC_LOOPBACK_SET(&mac_fe_5600, 0, 7, 0);
       READ_FE_MAC1r(0, 7, &loopmac);

    if (GET_FE_MAC1r_LBACK(loopmac)) 
       printf("Loopmac %d\n", loopmac);
}

void
diag5600_readMIIM(unit)
{
   uint32 dev, reg;
   uint16 data;

   dev = diag5600_uiGetHex("Enter dev addr(hex): ");
   dev &= 0x1f;
   reg = diag5600_uiGetHex("Enter reg addr(hex): ");
   reg &= 0x1f;
   cmic_read_miim(unit, dev, reg, &data);
   printf("Data read: %0x\n", data);
}

void
diag5600_writeMIIM(unit)
{
   uint32 dev, reg, data;

   dev = diag5600_uiGetHex("Enter dev addr(hex): ");
   dev &= 0x1f;
   reg = diag5600_uiGetHex("Enter reg addr(hex): ");
   reg &= 0x1f;
   data = diag5600_uiGetHex("Enter data(hex): ");
   cmic_write_miim(unit, dev, reg, data);
}

/*
 *  This routine can be polled to gather stats for SNMP. All counters are
 *  cleared after each call. Counters returned are sequential as listed below.
 *  Caller must pass pointer to buffer with space for 16 32-bit values.
 */
void
bcm5600_get_stats(uint32 port, uint32 *stat_buf)
{
	uint32 unit = 0;

	if (port > 23) {
		printf("bcm5600_get_stats: invalid port=%d\n", port);
		return;
	}
    READ_RBYTr(unit,port,stat_buf++); // rx bytes

    READ_RPKTr(unit,port,stat_buf++); // rx packets

    READ_RCDEr(unit,port,stat_buf++); // rx symbol error 

    READ_RALNr(unit,port,stat_buf++); // rx align error

    READ_RDISCr(unit,port,stat_buf++); // rx discard

    READ_RFCRr(unit,port,stat_buf++); // rx false carrier

    READ_RFCSr(unit,port,stat_buf++); // rx fcs error

    READ_RUNDr(unit,port,stat_buf++); // rx undersized

    READ_RJBRr(unit,port,stat_buf++); // rx jabber

    READ_RFRGr(unit,port,stat_buf++); // rx fragments

    READ_TPKTr(unit,port,stat_buf++); // tx packets

    READ_TBYTr(unit,port,stat_buf++); // tx bytes

    READ_TFCSr(unit,port,stat_buf++); // tx fcs error

    READ_TABRTr(unit,port,stat_buf++); // tx abort

    READ_TFRGr(unit,port,stat_buf++); // tx fragment

    READ_TJBRr(unit,port,stat_buf++); // tx jabber 
}

/*
 *------------------------------------------------------------------------------ * Function Name:       bcm5600_bpStats_FE
 *
 * Description:         Display stats for Fast Ethernet ports
 *
 * Created by:          Tony Wong
 *
 * Data Created:        01/25/02
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------ */
void
bcm5600_bpStats_FE(uint32 unit, uint32 port)
{
    uint32  data;

    READ_RBYTr(unit,port,&data);
    printf("FE Rx byte count: %d\n", data);

    READ_RPKTr(unit,port,&data);
    printf("FE Rx frame count: %d\n", data);

    READ_RCDEr(unit,port,&data);
    printf("FE Rx code error count: %d\n", data);

    READ_RALNr(unit,port,&data);
    printf("FE Rx align error frames: %d\n", data);

    READ_RDISCr(unit,port,&data);
    printf("FE Rx discard packet count: %d\n", data);

    READ_RFCRr(unit,port,&data);
    printf("FE Rx false carrier count: %d\n", data);

    READ_RFCSr(unit,port,&data);
    printf("FE Rx FCS error count: %d\n", data);

    READ_RFLRr(unit,port,&data);
    printf("FE Rx length out of range count: %d\n", data);

    READ_ROVRr(unit,port,&data);
    printf("FE Rx oversize frame count: %d\n", data);

    READ_RUNDr(unit,port,&data);
    printf("FE Rx undersize frame count: %d\n", data);

    READ_RJBRr(unit,port,&data);
    printf("FE Rx jabber frame count: %d\n", data);

    READ_RFRGr(unit,port,&data);
    printf("FE Rx fragment count: %d\n", data);

    READ_TPKTr(unit,port,&data);
    printf("FE Tx frame count: %d\n", data);

    READ_TBYTr(unit,port,&data);
    printf("FE Tx byte count: %d\n", data);

    READ_TFCSr(unit,port,&data);
    printf("FE Tx FCS error count: %d\n", data);

    READ_TABRTr(unit,port,&data);
    printf("FE Tx aborted packet count: %d\n", data);

    READ_TOVRr(unit,port,&data);
    printf("FE Tx oversize frame count: %d\n", data);

    READ_TFRGr(unit,port,&data);
    printf("FE Tx fragment packet count: %d\n", data);

    READ_TJBRr(unit,port,&data);
    printf("FE Tx jabber frame count: %d\n", data);
}

/*
 *------------------------------------------------------------------------------ * Function Name:       bcm5600_bpStats_GE
 *
 * Description:         Display stats for GIG Ethernet ports
 *
 * Created by:          Tony Wong
 *
 * Data Created:        01/25/02
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------ */
int
bcm5600_bpStats_GE(uint32 unit, uint32 port)
{
    uint32 data;

    if ((port != 24) && (port != 25))
        return NFX_ERR;

    READ_GRBYTr(unit,port,&data);
    printf("GE Rx byte count: %d\n", data);

    READ_GRPKTr(unit,port,&data);
    printf("GE Rx frame count: %d\n", data);

    READ_GRCDEr(unit,port,&data);
    printf("GE Rx code error count: %d\n", data);

    READ_GRALNr(unit,port,&data);
    printf("GE Rx align error frames: %d\n", data);

    READ_GRDISCr(unit,port,&data);
    printf("GE Rx discard packet count: %d\n", data);

    READ_GRFCRr(unit,port,&data);
    printf("GE Rx false carrier count: %d\n", data);

    READ_GRFCSr(unit,port,&data);
    printf("GE Rx FCS error count: %d\n", data);

    READ_GRFLRr(unit,port,&data);
    printf("GE Rx length out of range count: %d\n", data);

    READ_GROVRr(unit,port,&data);
    printf("GE Rx oversize frame count: %d\n", data);

    READ_GRUNDr(unit,port,&data);
    printf("GE Rx undersize frame count: %d\n", data);

    READ_GRJBRr(unit,port,&data);
    printf("GE Rx jabber frame count: %d\n", data);

    READ_GTPKTr(unit,port,&data);
    printf("GE Tx frame count: %d\n", data);

    READ_GTBYTr(unit,port,&data);
    printf("GE Tx byte count: %d\n", data);

    READ_GTFCSr(unit,port,&data);
    printf("GE Tx FCS error count: %d\n", data);

    READ_GTABRTr(unit,port,&data);
    printf("GE Tx aborted packet count: %d\n", data);

    READ_GTOVRr(unit,port,&data);
    printf("GE Tx oversize frame count: %d\n", data);

    READ_GTJBRr(unit,port,&data);
    printf("GE Tx jabber frame count: %d\n", data);
    return 0;
}

void
bcm5600_port_disable(int unit, int port)
{
    bcm5600_port_egress_enable_set(unit, port, 0);
    FE_WRITE(FE_MAC1r, unit, port, 0);
}

void
diag5600_monitorPortEnable(uint32 unit)
{
	uint32 port1, port2, reg_val;

	port1 = diag5600_uiGetInput("Enter port to be monitored(0-25): ");
	if (port1 > 25) {
		printf("Invalid port entry!\n");
		return;
	}
	port2 = diag5600_uiGetInput("Enter port to monitor with(0-25): ");
	if (port2 > 25) {
		printf("Invalid port entry!\n");
		return;
	}

	bcm5600_port_monitor_ingress_set(0, port1, 1); /* monitor ingress */

	reg_val = 1 << port1;
	WRITE_DEST_MIRRr(unit, port2block(port1), reg_val); /* monitor egress */

	reg_val = 0;
	SET_MIRRORr_MIRR_EN(reg_val, 1);
	SET_MIRRORr_MPORT(reg_val, port2);
	WRITE_MIRRORr(unit, port2block(port2), reg_val);
}

void
diag5600_monitorPortDisable(uint32 unit)
{
	uint32 port;

	port = diag5600_uiGetInput("Enter port to stop monitoring(0-25): ");
	bcm5600_port_monitor_ingress_set(0, port, 0);

	WRITE_DEST_MIRRr(unit, port2block(0), 0);
	WRITE_DEST_MIRRr(unit, port2block(8), 0);
	WRITE_DEST_MIRRr(unit, port2block(16), 0);
	WRITE_DEST_MIRRr(unit, port2block(24), 0);
	WRITE_DEST_MIRRr(unit, port2block(25), 0);

	WRITE_MIRRORr(unit, port2block(0), 0);
	WRITE_MIRRORr(unit, port2block(8), 0);
	WRITE_MIRRORr(unit, port2block(16), 0);
	WRITE_MIRRORr(unit, port2block(24), 0);
	WRITE_MIRRORr(unit, port2block(25), 0);
}

boolean bcm5600_enable_debug_ports;     /* default is 0, 'off' */

static void
diag5600_toggleDebugPorts(uint32 unit)
{
    /* toggle */
    if (bcm5600_enable_debug_ports) {
        bcm5600_enable_debug_ports = 0;
        printf("initialzing bcm5600 with debug ports disabled\n");
    }
    else {
        bcm5600_enable_debug_ports = 1;
        printf("initialzing bcm5600 with debug ports enabled\n");
    }
    bcm_init_5600(unit);
}

/*
 *------------------------------------------------------------------------------
 * Function Name:       bcm5600_uiShowDiagMenu
 *
 * Description:         Prints BCM5600 diagnostic menu items.
 *
 * Created by:          Tony Wong
 *
 * Data Created:        OCT 09, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bcm5600_uiShowDiagMenu()
{
	int sel;
	char cmd[80];

	sel = 0;

	printf("BCM 5600 Menu\n");
	printf("=============\n");
	printf("1 : Read PCI mapped reg\n");
	printf("2 : Write PCI mapped reg\n");
	printf("3 : Read indirect mapped reg\n");
	printf("4 : Write indirect mapped reg\n");
	printf("5 : Show PCI regs\n");
	printf("6 : Write PCI reg\n");
	printf("7 : View link status\n");
	printf("8 : View MIB counters\n");
	printf("9 : View ARL table\n");
	printf("10: Reset BCM5600\n");
	printf("11: Set MAC loopback\n");
	printf("12: Re-init BCM5600\n");
	printf("13: Clear MIB counters\n");
	printf("14: Enable port monitoring\n");
	printf("15: Disable port monitoring\n");
	if (isSSC()) {
	printf("16: Read MIIM reg\n");
	printf("17: Write MIIM reg\n");
	}
	printf("18: toggle debug port enable flag and Re-init\n");
	printf("99: Quit\n");
	printf("Enter Selection: ");

	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
} 

/*
 *------------------------------------------------------------------------------
 * Function Name:       bcm5600_uiDiag
 *
 * Description:         BCM 5600 UI entry function.
 *
 * Created by:          Tony Wong
 *
 * Data Created:        OCT 08, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bcm5600_uiDiag()
{
   int sel ;

   for(;;) {
      sel = bcm5600_uiShowDiagMenu();
      switch(sel)
      {
        case 1: diag5600_readReg();
                break;
        case 2: diag5600_writeReg();
                break;
        case 3: diag5600_readIndReg();
                break;
        case 4: diag5600_writeIndReg();
                break;
		case 5: diag5600_showPCI();
				break;
		case 6: diag5600_writePCI();
				break;
		case 7: diag5600_viewStatus();
 				break;
		case 8: diag5600_viewMIB(1);
 				break;
        case 9: diag5600_viewARL();
                break;
		case 10:soc_reset_init(0);
				break;
        case 11:diag5600_setLoopMac();
                break;
		case 12:bcm_init_5600(0);
				break;
		case 13:diag5600_viewMIB(0);
				break;
		case 14:diag5600_monitorPortEnable(0);
				break;
		case 15:diag5600_monitorPortDisable(0);
				break;
		case 16:diag5600_readMIIM(0);
				break;
		case 17:diag5600_writeMIIM(0);
				break;
		case 18:diag5600_toggleDebugPorts(0);
				break;
        case 99:
 		  		return(NFX_OK);
        default:
                break;
      }
   }
   return(NFX_OK);
}
