/*
 *=============================================================================
 * File Name   :    bcm5600-mac.c
 *
 *  RCS: $Id: bcm5600-mac.c,v 1.5 2002/10/09 01:22:02 tonyw Exp $		Copied and modified from Broadcom
 *
 *  Description:
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>

#include "bcm5600-soctypes.h"
#include "bcm5600-socregs.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"
#include "bcm5600-port.h"
#include "bcm5600-ll.h"

#define	FE_FULL_DUPLEX	0x15
#define	FE_HALF_DUPLEX	0x12
#define	GE_FULL_DUPLEX	0x0c
#define	GE_HALF_DUPLEX	0x0c

/* Forward Declarations */

static int	mac_fe_init_5600(int unit, soc_port_t port);
static int	mac_ge_init_5600(int unit, soc_port_t port);

static int	mac_ge_duplex_set_5600(int unit, soc_port_t port, int duplex);
static int	mac_ge_duplex_get_5600(int unit, soc_port_t port, int *duplex);
static int	mac_ge_pause_set_5600(int unit, soc_port_t port, 
				 int pause_tx, int pause_rx);
static int	mac_ge_pause_get_5600(int unit, soc_port_t port, 
				 int *pause_tx, int *pause_rx);
static int	mac_ge_pause_addr_set_5600(int unit, soc_port_t port, mac_addr_t);
static int	mac_ge_pause_addr_get_5600(int unit, soc_port_t port, mac_addr_t);
static int	mac_ge_interface_set_5600(int unit, soc_port_t port, int mii);

/*
 * Function:
 *	_in_fe_mode_5600
 *
 * Purpose:
 *      Boolean function indicating if this gig port is in fe mode.
 *      No error checking (eg, if called on FE port).
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 * Returns:
 *      Boolean value.  True means this is a _gig_ port in 10/100 mode.
 */
static int
_in_fe_mode_5600(int unit, soc_port_t port)
{
    return(0);
}

int
soc_gmac_in_fe_mode(int unit, soc_port_t port)
{
    return(_in_fe_mode_5600(unit, port));
}

typedef enum _mac_mode {
    _MM_unknown = 0,			/* Force unknown to 0 */
    _MM_1000,
    _MM_10_100
} _mac_mode_t;

#if 0
/*
 * Function:
 *	_current_mac_mode
 *
 * Purpose:
 *      Return the current active mac.  Important for GSL and later
 *      where the same port can have multiple macs of which one is
 *      active.
 */
static _mac_mode_t
_mac_mode_get(int unit, soc_port_t port)
{
    if (PBMP_PORT(port) & PBMP_GE_ALL) {
        return(_MM_1000);
    }

    return(_MM_10_100);
}
#endif

/*
 * Function:
 *	mac_fe_init_5600
 *
 * Purpose:
 *	Initialize the MAC into a known good state.
 *  4/24/00: Initialization from rc.soc copied.
 *
 *  Assumes no packets are flowing in mac currently
 *  Assumes for gsl, etc., that FX100 bit has been set in mac control.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *
 * Returns:
 *	SOC_E_NONE
 */
static int
mac_fe_init_5600(int unit, soc_port_t port)
{
    uint32 regval;

    regval = 0;   /* put part in reset. */
    FE_SET(port, FE_MAC1r_RST_TFUN, regval, 1);
    FE_SET(port, FE_MAC1r_RST_TMCS, regval, 1);
    FE_SET(port, FE_MAC1r_RST_RFUN, regval, 1);
    FE_SET(port, FE_MAC1r_RST_RMCS, regval, 1);
    FE_SET(port, FE_MAC1r_RST_SIM, regval, 1);
    FE_SET(port, FE_MAC1r_SOFT_RST, regval, 1);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit,port,regval));

    /*
     * Perform length check on Ethernet frames and do not allow huge frames
     */
    regval = 0;
    FE_SET(port, FE_MAC2r_FULL_DUP, regval, 1);
    FE_SET(port, FE_MAC2r_EXC_DEF, regval, 1);
    FE_SET(port, FE_MAC2r_AUTOZ, regval, 1);
    FE_SET(port, FE_MAC2r_LG_CHK, regval, 0);
    FE_SET(port, FE_MAC2r_HUGE_FR, regval, 0);
    FE_SET(port, FE_MAC2r_DEL_CRC, regval, 0);
    FE_SET(port, FE_MAC2r_CRC_EN, regval, 0);
    FE_SET(port, FE_MAC2r_PAD_EN, regval, 0);
    FE_SET(port, FE_MAC2r_VLAN_PAD, regval, 0);
    FE_SET(port, FE_MAC2r_AUTO_PAD, regval, 0);
    FE_SET(port, FE_MAC2r_PURE_PAD, regval, 0);
    FE_SET(port, FE_MAC2r_LONG_PRE, regval, 0);
    FE_SET(port, FE_MAC2r_NO_BOFF, regval, 0);
    FE_SET(port, FE_MAC2r_BP_NO_BOFF, regval, 0);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC2r, unit,port,regval));

    /*  # FE_CLRT has reversed fields in SA */
    regval = 0;
    FE_SET(port, FE_CLRTr_COL_WIN, regval, 0x37);
    FE_SET(port, FE_CLRTr_RETRY, regval, 0xf);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_CLRTr, unit, port, regval));

    SOC_IF_ERROR_RETURN(FE_WRITE(FE_IPGTr, unit, port, FE_FULL_DUPLEX));
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAXFr, unit, port, 0x05f2));

    regval = 0;
    FE_SET(port, FE_IPGRr_IPGR2, regval, 0x12);
    FE_SET(port, FE_IPGRr_IPGR1, regval, 0xc);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_IPGRr, unit, port, regval));

    regval = 0;
    FE_SET(port, FE_SUPPr_SPEED, regval, 1);
    FE_SET(port, FE_SUPPr_BIT_MODE, regval, 0);
    FE_SET(port, FE_SUPPr_ENBL_JABBR, regval, 0);
    FE_SET(port, FE_SUPPr_RESET_10T, regval, 0);
    FE_SET(port, FE_SUPPr_LNK_FAIL, regval, 0);
    FE_SET(port, FE_SUPPr_NO_CIPHER, regval, 0);
    FE_SET(port, FE_SUPPr_FORCE_QUIET, regval, 0);
    FE_SET(port, FE_SUPPr_RESET_100X, regval, 0);
    FE_SET(port, FE_SUPPr_RESET_RMII, regval, 0);
#if defined(BCM5605)
    FE_SET(port, FE_SUPPr_PHYMOD, regval, 1);
#endif
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_SUPPr, unit, port, regval));

    regval = 0;
    FE_SET(port, FE_TESTr_SHORT_QNTA, regval, 0);
    FE_SET(port, FE_TESTr_TEST_PAUSE, regval, 0);
    FE_SET(port, FE_TESTr_TEST_BACK, regval, 0);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_TESTr, unit, port, regval));

    regval = 0;  /* bring mac out of reset. enable rx. */
    FE_SET(port, FE_MAC1r_RST_TFUN, regval, 0);
    FE_SET(port, FE_MAC1r_RST_TMCS, regval, 0);
    FE_SET(port, FE_MAC1r_RST_RFUN, regval, 0);
    FE_SET(port, FE_MAC1r_RST_RMCS, regval, 0);
    FE_SET(port, FE_MAC1r_RST_SIM, regval, 0);
    FE_SET(port, FE_MAC1r_SOFT_RST, regval, 0);
FE_SET(port, FE_MAC1r_TX_PAU, regval, 1);
FE_SET(port, FE_MAC1r_RX_PAU, regval, 1);
    FE_SET(port, FE_MAC1r_RX_EN, regval, 1);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit,port,regval));

    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_fe_duplex_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	duplex - Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_fe_duplex_set_5600(int unit, soc_port_t port, int duplex)
{
    uint32	fe_mac1, fe_mac2;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC2r, unit, port, &fe_mac2));
    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC1r, unit, port, &fe_mac1));

    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 1);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit, port, fe_mac1)); /* In reset */

    FE_SET(port, FE_MAC2r_FULL_DUP, fe_mac2, duplex ? 1 : 0);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC2r, unit, port, fe_mac2));
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_IPGTr, unit, port,
				       duplex ?
				       FE_FULL_DUPLEX :
				       FE_HALF_DUPLEX));
    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 0);

    return(FE_WRITE(FE_MAC1r, unit, port, fe_mac1)); /* Out of reset */
}

/*
 * Function:
 *	mac_fe_duplex_get_5600
 *
 * Purpose:
 *	Get MAC duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	duplex - (OUT) Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 *	-1 - MII operation failed.
 */
static int
mac_fe_duplex_get_5600(int unit, soc_port_t port, int *duplex)
{
    uint32	fe_mac2;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC2r, unit, port, &fe_mac2));
    *duplex = FE_GET(port, FE_MAC2r_FULL_DUP, fe_mac2) ? 1 : 0;
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_fe_speed_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified speed.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	speed - 10,100,1000 for speed.
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_fe_speed_set_5600(int unit, soc_port_t port, int speed)
{
    uint32	fe_mac1, fe_supp;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC1r, unit, port, &fe_mac1));
    SOC_IF_ERROR_RETURN(FE_READ(FE_SUPPr, unit, port, &fe_supp));

    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 1);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit, port, fe_mac1)); /* RESET */

    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 0);
    switch(speed) {
    case 10:
	FE_SET(port, FE_SUPPr_SPEED, fe_supp, 0);
	break;
    case 100:
	FE_SET(port, FE_SUPPr_SPEED, fe_supp, 1);
	break;
    case 1000:
    default:
	return(-1);
    }
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_SUPPr, unit, port, fe_supp));
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit, port, fe_mac1));

    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_fe_speed_get_5600
 *
 * Purpose:
 *	Get MAC speed mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	speed - (OUT) speed in Mb (10/100/1000)
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_fe_speed_get_5600(int unit, soc_port_t port, int *speed)
{
    uint32	fe_supp;

    SOC_IF_ERROR_RETURN(FE_READ(FE_SUPPr, unit, port, &fe_supp));
    if (FE_GET(port, FE_SUPPr_SPEED, fe_supp) == 1) {
	*speed = 100;
    } else {
	*speed = 10;
    }
    return(SOC_E_NONE);
}

/*
 * Function: 	mac_fe_pause_set_5600
 * Purpose:
 *	Configure mac to transmit/receive pause frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - Boolean: transmit pause
 *	pause_rx - Boolean: receive pause
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_fe_pause_set_5600(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    uint32	fe_mac1;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC1r, unit, port, &fe_mac1));
    FE_SET(port, FE_MAC1r_RX_PAU, fe_mac1, pause_rx ? 1 : 0);
    FE_SET(port, FE_MAC1r_TX_PAU, fe_mac1, pause_tx ? 1 : 0);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit, port, fe_mac1));

    return(SOC_E_NONE);
}

/*
 * Function: 	mac_fe_pause_addr_set_5600
 * Purpose:
 *	Configure PAUSE frame source address.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_fe_pause_addr_set_5600(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	esa0 = 0, esa1 = 0, esa2 = 0;

    FE_SET(port, ESA0r_STAD0, esa0, pause_mac[4] | (pause_mac[5] << 8));
    FE_SET(port, ESA1r_STAD0, esa1, pause_mac[2] | (pause_mac[4] << 8));
    FE_SET(port, ESA2r_STAD0, esa2, pause_mac[0] | (pause_mac[1] << 8));

    SOC_IF_ERROR_RETURN(FE_WRITE(ESA0r, unit, port, esa0));
    SOC_IF_ERROR_RETURN(FE_WRITE(ESA1r, unit, port, esa1));
    SOC_IF_ERROR_RETURN(FE_WRITE(ESA2r, unit, port, esa2));

    return(SOC_E_NONE);
}    

/*
 * Function: 	mac_fe_pause_get_5600
 * Purpose:
 *	Return the ability of the Fast-etherner macs on the 5600.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - Boolean: transmit pause
 *	pause_rx - Boolean: receive pause
 *	pause_mac - MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX.
 */
static	int
mac_fe_pause_get_5600(int unit, soc_port_t port, int *pause_tx, int *pause_rx)
{
    uint32	fe_mac1;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC1r, unit, port, &fe_mac1));
    *pause_rx = FE_GET(port, FE_MAC1r_RX_PAU, fe_mac1) ? 1 : 0;
    *pause_tx = FE_GET(port, FE_MAC1r_TX_PAU, fe_mac1) ? 1 : 0;

    return(SOC_E_NONE);
}

/*
 * Function: 	mac_fe_pause_addr_set_5600
 * Purpose:
 *	Configure PAUSE frame source address.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - (OUT) MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_fe_pause_addr_get_5600(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	esa0, esa1, esa2;
    uint32	b0_15, b16_31, b32_47;

    SOC_IF_ERROR_RETURN(FE_READ(ESA0r, unit, port, &esa0));
    SOC_IF_ERROR_RETURN(FE_READ(ESA1r, unit, port, &esa1));
    SOC_IF_ERROR_RETURN(FE_READ(ESA2r, unit, port, &esa2));

    b0_15  = FE_GET(port, ESA0r_STAD0, esa0);
    b16_31 = FE_GET(port, ESA1r_STAD0, esa1);
    b32_47 = FE_GET(port, ESA2r_STAD0, esa2);

    pause_mac[0] = (uint8)(b32_47 >> 8);
    pause_mac[1] = (uint8)(b32_47 >> 0);
    pause_mac[2] = (uint8)(b16_31 >> 8);
    pause_mac[3] = (uint8)(b16_31 >> 0);
    pause_mac[4] = (uint8)(b0_15  >> 8);
    pause_mac[5] = (uint8)(b0_15  >> 0);

    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_fe_loopback_set_5600
 *
 * Purpose:
 *	Set a MAC into/out-of loopback mode
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch unit # on unit.
 *	loopback - Boolean: true -> loopback mode, false -> normal operation
 *
 * Returns:
 *	SOC_E_XXXX.
 */
static int
mac_fe_loopback_set_5600(int unit, soc_port_t port, int loopback)
{
    uint32	fe_mac1;

    SOC_IF_ERROR_RETURN(FE_READ(FE_MAC1r, unit, port, &fe_mac1));

    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 1);
    SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC1r, unit, port, fe_mac1));
    FE_SET(port, FE_MAC1r_SOFT_RST, fe_mac1, 0);
    FE_SET(port, FE_MAC1r_LBACK, fe_mac1, loopback ? 1 : 0);
    return(FE_WRITE(FE_MAC1r, unit, port, fe_mac1));
}

/*
 * Function:
 *	mac_fe_loopback_get_5600
 *
 * Purpose:
 *	Get current MAC loopback mode setting.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	loopback - (OUT) Boolean: true = loopback, false = normal
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_fe_loopback_get_5600(int unit, soc_port_t port, int *loopback)
{
    int		rv;
    uint32	fe_mac1;

    if ((rv = FE_READ(FE_MAC1r, unit, port, &fe_mac1)) == SOC_E_NONE) {
	*loopback = FE_GET(port, FE_MAC1r_LBACK, fe_mac1);
    }
    return(rv);
}

/*
 * Function:
 *	mac_fe_interface_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified mode, MII or 10B, ONLY MIi supported
 *	on FE ports.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	mii  - Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	SOC_E_NONE
 *	SOC_E_NOT_IMPL - requested mode not supported.
 */
static int
mac_fe_interface_set_5600(int unit, soc_port_t port, int mii)
{
    if (!mii) {
	return(SOC_E_NOT_IMPL);
    }
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_fe_interface_get_5600
 *
 * Purpose:
 *	Retrieve MAC mode, MII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	mii  - (OUT) Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 */
static int
mac_fe_interface_get_5600(int unit, soc_port_t port, int *mii)
{
    *mii = TRUE;
    return(SOC_E_NONE);
}

/*
 * Function: 	mac_fe_ability_5600
 * Purpose:
 *	Return the ability of the Fast-etherner macs on the 5600.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	mode - (OUT) Suppported operating modes as a mask of abilities.
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_fe_ability_5600(int unit, soc_port_t port, port_mode_t *mode)
{
    if (PBMP_PORT(port) & PBMP_FE_ALL) {
	*mode = SOC_PM_10MB | SOC_PM_100MB | SOC_PM_PAUSE_TX |
	        SOC_PM_PAUSE_RX | SOC_PM_MII | SOC_PM_LB_MAC;
	return(SOC_E_NONE);
    } else {
	*mode = 0;
	return(SOC_E_NOT_IMPL);
    }
}

/* Exported MAC driver structure */

mac_driver_t mac_fe_5600 = {
    "10/100 MAC Driver",
    mac_fe_init_5600,
    mac_fe_duplex_set_5600,
    mac_fe_duplex_get_5600,
    mac_fe_speed_set_5600,
    mac_fe_speed_get_5600,
    mac_fe_pause_set_5600,
    mac_fe_pause_get_5600,
    mac_fe_pause_addr_set_5600,
    mac_fe_pause_addr_get_5600,
    mac_fe_loopback_set_5600,
    mac_fe_loopback_get_5600,
    mac_fe_interface_set_5600,
    mac_fe_interface_get_5600,
    mac_fe_ability_5600
};

#if NUM_GE_PORT_5600 > 0
/*
 * Function:
 *	mac_ge_init_5600
 *
 * Purpose:
 *	Initialize the MAC into a known good state.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *
 * Returns:
 *	SOC_E_XXXX
 */
static 	int
mac_ge_init_5600(int unit, soc_port_t port)
{
    uint32	gmacc0, gmacc1, gmacc2;

    gmacc0 = 0;

    SET_GMACC0r_SRST(gmacc0, 1);
    SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));
    SET_GMACC0r_SRST(gmacc0, 0);
    SET_GMACC0r_L10B(gmacc0, 0);
    SET_GMACC0r_L32B(gmacc0, 0);
    SET_GMACC0r_TMDS(gmacc0, 0);

    gmacc1 = 0;
    SET_GMACC1r_FULLD(gmacc1, 1);
    SET_GMACC1r_FCTX(gmacc1, 1);	/* Flow control */
    SET_GMACC1r_FCRX(gmacc1, 1);	/* Flow control */
    SET_GMACC1r_MIFG(gmacc1, 2);
    SET_GMACC1r_GLVR(gmacc1, 1);
    SET_GMACC1r_TXEN0(gmacc1, 1);	/* TX enable */
    SET_GMACC1r_RXEN0(gmacc1, 1);	/* RX enable */
    SET_GMACC1r_AUTOZ(gmacc1, 1);	/* autoz on */
	if (isFC()) {
    SET_GMACC1r_LONGP(gmacc1, 1);   /* must accept long preamble on FC-NIM */
	}
    gmacc2 = 0;
    SET_GMACC2r_IPGT(gmacc2, 0xc);
    SET_GMACC2r_MCLKFQ(gmacc2, 1);

     /* LAST - clears reset */

    SOC_IF_ERROR_RETURN(WRITE_GMACC1r(unit, port, gmacc1));
    SOC_IF_ERROR_RETURN(WRITE_GMACC2r(unit, port, gmacc2));
    SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));

    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_interface_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	mii  - Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
static int
mac_ge_interface_set_5600(int unit, soc_port_t port, int mii)
{
    uint32	gmacc0, gpcsc;

    SOC_IF_ERROR_RETURN(READ_GMACC0r(unit, port, &gmacc0));
    SOC_IF_ERROR_RETURN(READ_GPCSCr(unit, port, &gpcsc));

    SET_GMACC0r_SRST(gmacc0, 1);	/* Put into reset */
    SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));
    SET_GMACC0r_SRST(gmacc0, 0);	/* Clear reset for later */

    if (1) {
	SET_GMACC0r_TMDS(gmacc0, 1);	/* GMII Mode */
	SET_GPCSCr_RCSEL(gpcsc, 1);	/* Use 125 MHZ for GMII */
    } else {
	SET_GMACC0r_TMDS(gmacc0, 0);	/* 10B Mode */
	SET_GPCSCr_RCSEL(gpcsc, 0);	/* Use 62.5. MHZ for 10B */
    }

    SOC_IF_ERROR_RETURN(WRITE_GPCSCr(unit, port, gpcsc));
    SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_interface_get_5600
 *
 * Purpose:
 *	Retrieve MAC mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch unit #.
 *	mii  - (OUT) Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 *
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
static int
mac_ge_interface_get_5600(int unit, soc_port_t port, int *mii)
{
    uint32	gmacc0;

    SOC_IF_ERROR_RETURN(READ_GMACC0r(unit, port, &gmacc0));
    *mii = GET_GMACC0r_TMDS(gmacc0) ? TRUE : FALSE;
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_duplex_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port# on unit.
 *	duplex - Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 *
 * Note:
 *      For gsl type parts, ge_speed_set_5600 _must_ be called before
 *      duplex set to ensure that the proper mac is being updated.
 */
static int
mac_ge_duplex_set_5600(int unit, soc_port_t port, int duplex)
{
    uint32	gmacc1, gmacc2;

    if (_in_fe_mode_5600(unit, port)) {
        return(mac_fe_duplex_set_5600(unit, port, duplex));
    } else {
        SOC_IF_ERROR_RETURN(READ_GMACC1r(unit, port, &gmacc1));
        SOC_IF_ERROR_RETURN(READ_GMACC2r(unit, port, &gmacc2));

        SET_GMACC1r_FULLD(gmacc1, duplex ? 1 : 0);
        SET_GMACC2r_IPGT(gmacc2,
                         duplex ? GE_FULL_DUPLEX : GE_HALF_DUPLEX);
        SOC_IF_ERROR_RETURN(WRITE_GMACC2r(unit, port, gmacc2));
        SOC_IF_ERROR_RETURN(WRITE_GMACC1r(unit, port, gmacc1));
        return(SOC_E_NONE);
    }
}

/*
 * Function:
 *	mac_ge_duplex_get_5600
 *
 * Purpose:
 *	Get MAC duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	duplex - (OUT) Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_ge_duplex_get_5600(int unit, soc_port_t port, int *duplex)
{
    uint32	gmacc1;

    if (_in_fe_mode_5600(unit, port)) {
        return mac_fe_duplex_get_5600(unit, port, duplex);
    } else {
        SOC_IF_ERROR_RETURN(READ_GMACC1r(unit, port, &gmacc1));
        *duplex = GET_GMACC1r_FULLD(gmacc1);
        return(SOC_E_NONE);
    }
}

/*
 * Function:
 *	mac_ge_speed_set_5600
 *
 * Purpose:
 *	Set a MAC in the specified speed.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - 10,100,1000 for speed.
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_ge_speed_set_5600(int unit, soc_port_t port, int speed)
{
    int rv = SOC_E_NONE;

    if (speed != 1000) {
        rv = SOC_E_NOT_IMPL;
    }
    return rv;
}

/*
 * Function:
 *	mac_ge_speed_get_5600
 *
 * Purpose:
 *	Get MAC speed mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - (OUT) speed in Mb (10/100/1000)
 *
 * Returns:
 *	SOC_E_NONE
 */
static int
mac_ge_speed_get_5600(int unit, soc_port_t port, int *speed)
{
    int rv = SOC_E_NONE;

    if (_in_fe_mode_5600(unit, port)) {
        return mac_fe_speed_get_5600(unit, port, speed);
    } else {                    /* mac in gig mode */
        *speed = 1000;
    }

    return rv;
}

/*
 * Function: 	mac_ge_pause_set_5600
 * Purpose:
 *	Configure mac to transmit/receive pause frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - Boolean: transmit pause
 *	pause_rx - Boolean: receive pause
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_ge_pause_set_5600(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    uint32	gmacc1;

    if (_in_fe_mode_5600(unit, port)) {
        return(mac_fe_pause_set_5600(unit, port, pause_tx, pause_rx));
    } else {

        SOC_IF_ERROR_RETURN(READ_GMACC1r(unit, port, &gmacc1));
        SET_GMACC1r_FCTX(gmacc1, pause_tx ? 1 : 0);
        SET_GMACC1r_FCRX(gmacc1, pause_rx ? 1 : 0);
        SOC_IF_ERROR_RETURN(WRITE_GMACC1r(unit, port, gmacc1));

        return(SOC_E_NONE);
    }
}

/*
 * Function: 	mac_ge_pause_addr_set_5600
 * Purpose:
 *	Configure PAUSE frame source address.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_ge_pause_addr_set_5600(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	gsa0 = 0, gsa1 = 0;

    memcpy((void *)&gsa0, pause_mac, sizeof(gsa0));
    memcpy((void *)&gsa1, pause_mac + sizeof(gsa1),
	   sizeof(mac_addr_t) - sizeof(gsa0));
    SOC_IF_ERROR_RETURN(WRITE_GSA0r(unit, port, gsa0));
    SOC_IF_ERROR_RETURN(WRITE_GSA1r(unit, port, gsa1));

    return(SOC_E_NONE);
}    

/*
 * Function: 	mac_ge_pause_get_5600
 * Purpose:
 *	Return the current PAUSE configuration on the specified port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - (OUT) TRUE is TX pause enabled.
 *	pause_rx - (OUT) TRUE if obey RX pause enabled.
 * Returns:
 *	SOC_E_XXXX.
 */
static	int
mac_ge_pause_get_5600(int unit, soc_port_t port, int *pause_tx, int *pause_rx)
{
    uint32	gmacc1;
    
    if (_in_fe_mode_5600(unit, port)) {
        return(mac_fe_pause_get_5600(unit, port, pause_tx, pause_rx));
    } else {                    /* mac in gig mode */
	SOC_IF_ERROR_RETURN(READ_GMACC1r(unit, port, &gmacc1));
	*pause_tx = GET_GMACC1r_FCTX(gmacc1) ? TRUE : FALSE;
	*pause_rx = GET_GMACC1r_FCRX(gmacc1) ? TRUE : FALSE;
	return(SOC_E_NONE);
    }
}

/*
 * Function: 	mac_ge_pause_addr_get_5600
 * Purpose:
 *	Retrieve the source addrtess for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - (OUT) MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
static	int
mac_ge_pause_addr_get_5600(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	t_mac;

    if (_in_fe_mode_5600(unit, port)) {
        return(mac_fe_pause_addr_get_5600(unit, port, pause_mac));
    } else {                    /* mac in gig mode */
	SOC_IF_ERROR_RETURN(READ_GSA0r(unit, port, &t_mac));
	memcpy(pause_mac, (void *)&t_mac, sizeof(t_mac));
	SOC_IF_ERROR_RETURN(READ_GSA1r(unit, port, &t_mac));
	memcpy(pause_mac + sizeof(t_mac), (void *)&t_mac, sizeof(t_mac));
	return(SOC_E_NONE);
    }
}

/*
 * Function:
 *	mac_ge_loopback_set_5600
 *
 * Purpose:
 *	Set a MAC into/out-of loopback mode
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	loopback - Boolean: true -> loopback mode, false -> normal operation
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_ge_loopback_set_5600(int unit, soc_port_t port, int loopback)
{
    uint32	gmacc0;

    if (_in_fe_mode_5600(unit, port)) {
        return mac_fe_loopback_set_5600(unit, port, loopback);
    } else {                    /* mac in gig mode */
        SOC_IF_ERROR_RETURN(READ_GMACC0r(unit, port, &gmacc0));
        SET_GMACC0r_SRST(gmacc0, 1); /* Into Reset */
        SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));
        SET_GMACC0r_SRST(gmacc0, 0); /* Out of reset + LB mode */
        SET_GMACC0r_L10B(gmacc0, loopback ? 1 : 0);
        SOC_IF_ERROR_RETURN(WRITE_GMACC0r(unit, port, gmacc0));

        return(SOC_E_NONE);
    }
}

/*
 * Function:
 *	mac_ge_loopback_get_5600
 *
 * Purpose:
 *	Get current MAC loopback mode setting.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit
 *	loopback - (OUT) Boolean: true = loopback, false = normal
 *
 * Returns:
 *	SOC_E_XXXX
 */
static int
mac_ge_loopback_get_5600(int unit, soc_port_t port, int *loopback)
{
    uint32	gmacc0;

    if (_in_fe_mode_5600(unit, port)) {
        return mac_fe_loopback_get_5600(unit, port, loopback);
    } else {                    /* mac in gig mode */
        SOC_IF_ERROR_RETURN(READ_GMACC0r(unit, port, &gmacc0));
        *loopback = GET_GMACC0r_L10B(gmacc0);
        return(SOC_E_NONE);
    }
}

/*
 * Function:
 *	mac_ge_ability_5600
 * Purpose:
 *	Return the ability of the gigabit macs on the 5600.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch Port # on unit.
 *	mode - (OUT) Mac abilities on return.
 *
 * Returns:
 *	SOC_E_NONE
 */
static	int
mac_ge_ability_5600(int unit, soc_port_t port, port_mode_t *mode)
{
    *mode = SOC_PM_1000MB_FD | SOC_PM_MII | SOC_PM_10B |
        SOC_PM_LB_MAC | SOC_PM_PAUSE_TX | SOC_PM_PAUSE_RX;

    return(SOC_E_NONE);
}

mac_driver_t mac_ge_5600 = {
    "1000MB MAC Driver",
    mac_ge_init_5600,
    /*    mac_ge_enable,*/
    mac_ge_duplex_set_5600,
    mac_ge_duplex_get_5600,
    mac_ge_speed_set_5600,
    mac_ge_speed_get_5600,
    mac_ge_pause_set_5600,
    mac_ge_pause_get_5600,
    mac_ge_pause_addr_set_5600,
    mac_ge_pause_addr_get_5600,
    mac_ge_loopback_set_5600,
    mac_ge_loopback_get_5600,
    mac_ge_interface_set_5600,
    mac_ge_interface_get_5600,
    mac_ge_ability_5600
};

#endif /* NUM_GE_PORT_5600 > 0 */

mac_driver_t *
soc_mac_probe(int unit, soc_port_t port)
/*
 * Function: 	soc_mac_probe
 * Purpose:	Probe the specified unit/port for MAC type and return
 *		a mac_driver_t pointer used to drive the MAC.
 * Parameters:	unit - SOC unit #
 *		port - port number on device.
 * Returns:	NULL - no port/driver
 *		!NULL - pointer to MAC driver to use.
 */
{
#if NUM_FE_PORT_5600 > 0
    if (PBMP_PORT(port) & PBMP_FE_ALL) {
	return(&mac_fe_5600);
    }
#endif /* NUM_FE_PORT_5600 > 0 */

#if NUM_GE_PORT_5600 > 0
    if (PBMP_PORT(port) & PBMP_GE_ALL) {
	return(&mac_ge_5600);
    }
#endif
    return(&mac_fe_5600);  /* default is mac_fe_5600 */
}

#if 0
/*
 * Function: 	mac_set_autoz
 * Purpose:	Set the autoz bit in the active mac 
 * Parameters:	unit - SOC unit #
 *		port - port number on device.
 *              enable - Boolean value to set state
 * Returns:	SOC_E_xxxx
 */
int
mac_set_autoz(int unit, soc_port_t port, int enable)
{
    uint32 reg;

    switch (_mac_mode_get(unit, port)) {
    case _MM_1000:
	SOC_IF_ERROR_RETURN(READ_GMACC1r(unit, port, &reg));
	SET_GMACC1r_AUTOZ(reg, enable ? 1 : 0);
	SOC_IF_ERROR_RETURN(WRITE_GMACC1r(unit, port, reg));
        break;
    case _MM_10_100:
	SOC_IF_ERROR_RETURN(FE_READ(FE_MAC2r, unit, port, &reg));
	FE_SET(port, FE_MAC2r_AUTOZ, reg, enable ? 1 : 0);
	SOC_IF_ERROR_RETURN(FE_WRITE(FE_MAC2r, unit, port, reg));
        break;
    default:
        return(SOC_E_INTERNAL);
    }

    return(SOC_E_NONE);
}
#endif
