/*
 *=============================================================================
 * File Name   :    bcm5632-init.c
 *
 *  RCS: $Id: bcm5632-init.c,v 1.6 2002/05/08 17:50:39 rick Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-cmic.h"
#include "bcm5632-socregs.h"
#include "bcm5632-error.h"

uint32 BCM5632_PCI_BASE_ADDR[BCM5632_NUM_DEV] = {
       BCM5632_PCI_BASE_ADDR0,
       BCM5632_PCI_BASE_ADDR1 };

uint32 BCM5632_DEV_ID[BCM5632_NUM_DEV] = { 
       BCM5632_DEV_ID0, 
       BCM5632_DEV_ID1 };

#define CHK(r)  if ((r) < 0) return -1;

void bcm5632_delay()
{
    uint32 i,j,delay;

    for (i = 0; i < 200; i++)  /* kill some time */
        for (j = 0; j < 100; j++)
            delay += i * j;
}

void 
bcm5632_poll()
{
	/* manage link states/status */
	bcm5632_linkscan_update(0); 
}

/*
 * Function:
 *	bcm5632_dma_init
 * Purpose:
 * 	Initialize DMA memory allocation.
 * Parameters:
 *	unit - StrataSwitch unit #.
 * Returns:
 *	BCM_E_NONE
 */
void
bcm5632_dma_init(int unit)
{
    uint32  *addr, channel, data;
    char    st[20];

    for( channel=0; channel<32; channel++) {
        cmic_read_reg_5632(unit, RX_CH_0r+channel, &data);
		if( !data) {
	    	sprintf(st, "RxDMA_%02d", channel);
	    	addr = (uint32 *)eee_ramAlloc(1024*9, st);
	    	cmic_write_reg_5632(unit, RX_CH_0r+channel, (uint32) addr);
		}
		cmic_read_reg_5632(unit, TX_CH_0r+channel, &data);
		if( !data) {
	    	sprintf(st, "TxDMA_%02d", channel);
	    	addr = (uint32 *)eee_ramAlloc(1024*9, st);
	    	cmic_write_reg_5632(unit, TX_CH_0r+channel, (uint32) addr);
		}
    }
}

/*
 * Function:
 *	bcm_system_init
 * Purpose:
 * 	Initialize bcm software system.
 * Parameters:
 *	unit - StrataSwitch unit #.
 * Returns:
 *	BCM_E_NONE
 */
static int
bcm_system_init_5632(int unit)
{
#if 0 /* not needed right now */	
    bcm5632_dma_init(unit);
#endif
   	cmic_write_reg_5632(unit, PHY_CONFr, 0x4000); /* set slow clock for MIIM */
    return BCM_E_NONE;
}

/*
 * Function:
 *	bcm_init_5632
 * Purpose:
 * 	Reset the switch chip, re-initialize, and initialize the BCM API layer.
 * Parameters:
 *	unit - StrataSwitch unit #.
 * Returns:
 *	BCM_E_XXX
 */
int 
bcm_init_5632(int unit)
{
	int	rc;

#if 0  /* already done by PROM */
    pci_cfg_write(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x10, 
                  BCM5632_PCI_BASE_ADDR[unit]);
#endif

    /*
     * PMC-8353 init
     * 1. errata: reg 0x1e = 0x4000, then 0x1e = 0x0000
     * 2. soft reset
     * 3. IDECENEN = 1, PCS_EN = 1, TXCLK4 = 1
     */
#if 0  /* done by 5600 for now, until 5632 can do MIIM */
	for (i = 0; i <= 7; i++) {
    	cmic_write_miim_5632(unit, i, 0x1e, 0x4000);
    	cmic_write_miim_5632(unit, i, 0x1e, 0x0000);
    	cmic_write_miim_5632(unit, i, 0x11, 0x6308);
    	cmic_write_miim_5632(unit, i, 0x11, 0xe384);
	}
	for (i = 0x1c; i <= 0x1f; i++) {
    	cmic_write_miim_5632(unit, i, 0x1e, 0x4000);
    	cmic_write_miim_5632(unit, i, 0x1e, 0x0000);
    	cmic_write_miim_5632(unit, i, 0x11, 0x6308);
    	cmic_write_miim_5632(unit, i, 0x11, 0xe384);
	}
#endif
    BCM_IF_ERROR_RETURN(bcm_system_init_5632(unit));
    BCM_IF_ERROR_RETURN(bcm5632_port_init(unit));
    BCM_IF_ERROR_RETURN(bcm5632_vlan_init(unit));
    BCM_IF_ERROR_RETURN(bcm_l2_init_5632(unit));
#if 0
	rc = eee_registerPollingFunc(EEE_LOW_PRIORITY, (void *)bcm5632_poll, NULL);
	if (rc != NFX_OK)
		printf("bcm_init_5632: couldn't register polling function\n");
#endif

    /*
     * RAL - for initial testing, enable port 0,2,3,4,5,6,7,8,9,10,11 on
     *      switch 0.
     */
    printf("Enabling 5632(0) ports: 0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11\n");
    WRITE_P_CONF_1r(0, 0, (0 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 1, (0 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 2, (2 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 3, (3 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 4, (4 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 5, (5 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 6, (6 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 8, (8 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 9, (9 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 10, (10 << 12) | 0xc);
    WRITE_P_CONF_1r(0, 11, (11 << 12) | 0xc);

	/*
	 * RAL - buffer management register 1: enable jumbo frames, sets
	 * 		 watermark 1
	 */
//	cmic_write_reg_5632(0, 0x17, 0xe014);
	cmic_write_reg_5632(0, 0x17, 0xe014);

	/*
	 * RAL - buffer management register 2: sets watermark 2 and 3
	 */
//	cmic_write_reg_5632(0, 0x18, (52 << 7) | 68);
	cmic_write_reg_5632(0, 0x18, (40 << 7) | 60);

    return BCM_E_NONE;
}

/*
 * Function: bcm_vlan_port_add_5632
 *
 * Description:
 *      Add a port to the specified vlan.
 *      Note that VLAN table is already automatically created by chip.
 *
 * Parameters:
 *      unit - StrataSwitch PCI device unit number (driver internal).
 *      vid - VLAN ID to add port to as a member.
 *      pbmp - port bitmap for members of VLAN
 *      ubmp - untagged members of VLAN
 *
 * Returns:
 *      BCM_E_NONE              Success.
 *      BCM_E_INTERNAL          Chip access failure.
 *      BCM_E_VLAN_NOT_FOUND    VLAN ID not in use.
 *
 * Notes:
 *      None.
 */
int
bcm_vlan_port_add_5632(int unit, vlan_id_t vid, pbmp_t pbmp, pbmp_t ubmp)
{
    _u32 buf[MAX_MEM_WORDS];
    int rc;
    VLAN_m_entry_t entry;

    ubmp &= pbmp;  /* only allow untagged ports on the vlan. */
    rc = cmic_read_memory_5632(unit, VLAN_m, COPYNO_ALL, vid, buf);
    if (rc != SOC_E_NONE) {
       return(BCM_E_INTERNAL);
    }

    Buf2Struct(VLAN_m, (unsigned char *)&buf[0], (unsigned char *)&entry, 1);

    /*
     * Update the entry for this VLAN.
     */
    entry.PortMap |= pbmp;
    entry.OutputTagUntag |= ubmp;
    Struct2Buf(VLAN_m, (unsigned char *)&entry, (unsigned char *)&buf[0], 1);
    rc = cmic_write_memory_5632(unit, VLAN_m, COPYNO_ALL, vid, buf);
    if (rc != SOC_E_NONE) {
       return(BCM_E_INTERNAL);
    }

    return BCM_E_NONE;
}
