/*
 *=============================================================================
 * File Name   :    bcm5632-mac.c
 *
 *  RCS: $Id: bcm5632-mac.c,v 1.2 2002/10/09 19:27:21 tonyw Exp $      Copied and modified from Broadcom
 *
 *  Description:
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-socregs.h"
#include "bcm5632-cmic.h"
#include "bcm5632-error.h"
#include "bcm5632-port.h"
#include "bcm5632-phy.h"

/* Forward Declarations */

int	mac_ge_init5632(int unit, soc_port_t port);

int	mac_ge_duplex_set5632(int unit, soc_port_t port, int duplex);
int	mac_ge_duplex_get5632(int unit, soc_port_t port, int *duplex);
int	mac_ge_pause_set5632(int unit, soc_port_t port, 
				 int pause_tx, int pause_rx);
int	mac_ge_pause_get5632(int unit, soc_port_t port, 
				 int *pause_tx, int *pause_rx);
int	mac_ge_pause_addr_set5632(int unit, soc_port_t port, mac_addr_t);
int	mac_ge_pause_addr_get5632(int unit, soc_port_t port, mac_addr_t);
int	mac_ge_interface_set5632(int unit, soc_port_t port, int mii);
int mac_ge_speed_set5632(int unit, soc_port_t port, int speed);
int mac_ge_speed_get5632(int unit, soc_port_t port, int *speed);

extern uint32 reg_addr_5632(soc_reg_t reg, ...);

typedef enum _mac_mode {
    _MM_unknown = 0,			/* Force unknown to 0 */
    _MM_1000,
    _MM_10_100
} _mac_mode_t;

#if 0
/*
 * Function:
 *	_current_mac_mode
 *
 * Purpose:
 *      Return the current active mac.  Important for GSL and later
 *      where the same port can have multiple macs of which one is
 *      active.
 */
static _mac_mode_t
_mac_mode_get(int unit, soc_port_t port)
{
    if (PBMP_PORT(port) & PBMP_GE_ALL) {
        return(_MM_1000);
    }
    return(_MM_10_100);
}
#endif

#if NUM_GE_PORT_5632 > 0
/*
 * Function:
 *	mac_ge_init5632
 *
 * Purpose:
 *	Initialize the MAC into a known good state.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_init5632(int unit, soc_port_t port)
{
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_interface_set5632
 *
 * Purpose:
 *	Set a MAC in the specified mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	mii  - Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
int mac_ge_interface_set5632(int unit, soc_port_t port, int mii)
{
    uint32 data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (mii) {
		SET_P_CONF_3r_EnGMII(data, 1);	/* GMII Mode */
    } 
	else {
		SET_P_CONF_3r_EnGMII(data, 0);	/* GMII Mode */
    }
    SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), data));
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_interface_get5632
 *
 * Purpose:
 *	Retrieve MAC mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch unit #.
 *	mii  - (OUT) Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 *
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
int mac_ge_interface_get5632(int unit, soc_port_t port, int *mii)
{
    uint32	gmacc0;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &gmacc0));
    *mii = GET_P_CONF_3r_EnGMII(gmacc0) ? TRUE : FALSE;
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_duplex_set5632
 *
 * Purpose:
 *	Set a MAC in the specified duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port# on unit.
 *	duplex - Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 *
 * Note:
 *      For gsl type parts, ge_speed_set _must_ be called before
 *      duplex set to ensure that the proper mac is being updated.
 */
int mac_ge_duplex_set5632(int unit, soc_port_t port, int duplex)
{
    uint32    data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (duplex) { /* set full duplex */
        SET_P_CONF_3r_Duplex_cfg(data, 1);
    } 
	else {
        SET_P_CONF_3r_Duplex_cfg(data, 0);
    }
    return(cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_3r, port), data));
}

/*
 * Function:
 *	mac_ge_duplex_get5632
 *
 * Purpose:
 *	Get MAC duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	duplex - (OUT) Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_duplex_get5632(int unit, soc_port_t port, int *duplex)
{
    uint32	gmacc1;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &gmacc1));
    *duplex = GET_P_CONF_3r_Duplex_cfg(gmacc1);
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_speed_set5632
 *
 * Purpose:
 *	Set a MAC in the specified speed.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - 10,100,1000 for speed.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_speed_set5632(int unit, soc_port_t port, int speed)
{ 
    int rv = SOC_E_NONE;
    uint32 data, duplex_bit, speed_bit;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    speed_bit = GET_P_CONF_3r_Speedselect_cfg(data);
    duplex_bit = GET_P_CONF_3r_Duplex_cfg(data);
    /* if speed is 1000 than set the duplex mode to full-duplex */
    if (speed == 1000) {
        if (speed_bit == 2) { /* already in 1000 speed */
            return(SOC_E_NONE);
        } 
        speed_bit  = 2;    /* set to 1000 speed */
        duplex_bit = 1;    /* set to full duplex */
    } else if (speed == 100) {
        if (speed_bit == 1) { /* already in 100 speed */
            return(SOC_E_NONE);
        }
        speed_bit = 1; /* set to 100 speed */
    } else if (speed == 10) {
        if (speed_bit == 0) { /* already in 10 speed */
            return(SOC_E_NONE);
        }    
        speed_bit = 0; /* set to 10 speed */
    } else {
        return(SOC_E_NONE);
    }

    SET_P_CONF_3r_Speedselect_cfg(data, speed_bit);
    SET_P_CONF_3r_Duplex_cfg(data, duplex_bit);
    rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_3r, port), data);
    return rv;
}

/*
 * Function:
 *	mac_ge_speed_get5632
 *
 * Purpose:
 *	Get MAC speed mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - (OUT) speed in Mb (10/100/1000)
 *
 * Returns:
 *	SOC_E_NONE
 */
int mac_ge_speed_get5632(int unit, soc_port_t port, int *speed)
{
    int rv = SOC_E_NONE;
    uint32 data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (GET_P_CONF_3r_Speedselect_cfg(data) == 2) {
        *speed = 1000; /* 1000 Mbps */
    } else if (GET_P_CONF_3r_Speedselect_cfg(data) == 1) {
        *speed = 100; /* 100 Mbps */
    } else {
        *speed = 10; /* 10 Mbps */
    }
        
    return rv;
}

/*
 * Function: 	mac_ge_pause_set5632
 * Purpose:
 *	Configure mac to transmit/receive pause frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - Boolean: transmit pause
 *	pause_rx - Boolean: receive pause
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_pause_set5632(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    uint32	gmacc1;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port), &gmacc1));
    /* 
     *  if any tx/rx is enable, we need to set Pause capability on to enable it
	 */
    if (pause_tx || pause_rx) { 
        SET_P_CONF_2r_PAUSE_CAP(gmacc1, 1);
    }
    SET_P_CONF_2r_TXPAUSE(gmacc1, pause_tx ? 1 : 0);
    SET_P_CONF_2r_RXPAUSE(gmacc1, pause_rx ? 1 : 0);
    SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port), gmacc1));
    return(SOC_E_NONE);
}

/*
 * Function: 	mac_ge_pause_addr_set5632
 * Purpose:
 *	Configure PAUSE frame source address.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_pause_addr_set5632(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    /* BCM5632 hardcode to use mac address 0 in chip */
    return(SOC_E_NONE);
}    

/*
 * Function: 	mac_ge_pause_get5632
 * Purpose:
 *	Return the current PAUSE configuration on the specified port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - (OUT) TRUE is TX pause enabled.
 *	pause_rx - (OUT) TRUE if obey RX pause enabled.
 * Returns:
 *	SOC_E_XXXX.
 */
int mac_ge_pause_get5632(int unit, soc_port_t port, int *pause_tx,int *pause_rx)
{
    uint32	gmacc1;
    uint32   i;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port),&gmacc1));
    i = GET_P_CONF_2r_PAUSE_CAP(gmacc1);        
    /* 
     BCM5632 needs both Pause capablity and TxPause/RxPause enable to enable it 
    */
    *pause_tx = (i && GET_P_CONF_2r_TXPAUSE(gmacc1)) ? TRUE : FALSE;
    *pause_rx = (i && GET_P_CONF_2r_RXPAUSE(gmacc1)) ? TRUE : FALSE;

    return(SOC_E_NONE);
}

/*
 * Function: 	mac_ge_pause_addr_get5632
 * Purpose:
 *	Retrieve the source addrtess for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - (OUT) MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_pause_addr_get5632(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	t_mac;

    /* BCM5632 hardcode in chip to use mac address 0*/
    t_mac = 0x0;
    memcpy(pause_mac, (void *)&t_mac, sizeof(t_mac));
    memcpy(pause_mac + sizeof(t_mac), (void *)&t_mac, sizeof(t_mac));

    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_loopback_set5632
 *
 * Purpose:
 *	Set a MAC into/out-of loopback mode
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	loopback - Boolean: true -> loopback mode, false -> normal operation
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_loopback_set5632(int unit, soc_port_t port, int loopback)
{
    /* BCM5632 does not support loopback */ 
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_loopback_get5632
 *
 * Purpose:
 *	Get current MAC loopback mode setting.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit
 *	loopback - (OUT) Boolean: true = loopback, false = normal
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_ge_loopback_get5632(int unit, soc_port_t port, int *loopback)
{
    /* BCM5632 does not support loopback */
    *loopback = 0; 
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_ge_ability5632
 * Purpose:
 *	Return the ability of the gigabit macs on the 5632.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch Port # on unit.
 *	mode - (OUT) Mac abilities on return.
 *
 * Returns:
 *	SOC_E_NONE
 */
int mac_ge_ability5632(int unit, soc_port_t port, port_mode_t *mode)
{
    *mode = SOC_PM_1000MB_FD | SOC_PM_MII | SOC_PM_10B |
        	SOC_PM_LB_MAC | SOC_PM_PAUSE_TX | SOC_PM_PAUSE_RX;

    return(SOC_E_NONE);
}

#endif /* NUM_GE_PORT_5632 > 0 */


#if NUM_XGE_PORT_5632 > 0
/*
 * Function:
 *	mac_xge_init5632
 *
 * Purpose:
 *	Initialize the MAC into a known good state.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_init5632(int unit, soc_port_t port)
{
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_interface_set5632
 *
 * Purpose:
 *	Set a MAC in the specified mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	mii  - Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
int mac_xge_interface_set5632(int unit, soc_port_t port, int mii)
{
    uint32 data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (mii) {
		SET_P_CONF_3r_EnGMII(data, 1);	/* GMII Mode */
    } 
	else {
		SET_P_CONF_3r_EnGMII(data, 0);	/* GMII Mode */
    }

    SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), data));
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_interface_get5632
 *
 * Purpose:
 *	Retrieve MAC mode, GMII or 10B.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch unit #.
 *	mii  - (OUT) Boolean: true --> MII/GMII, false --> 10B.
 *
 * Returns:
 *	0
 *	-1 - MII operation failed.
 *
 * Note:
 *      RESOLVE:  Does interface change when in _fe_ mode?
 */
int mac_xge_interface_get5632(int unit, soc_port_t port, int *mii)
{
    uint32	gmacc0;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &gmacc0));
    *mii = GET_P_CONF_3r_EnGMII(gmacc0) ? TRUE : FALSE;
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_duplex_set5632
 *
 * Purpose:
 *	Set a MAC in the specified duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port# on unit.
 *	duplex - Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 *
 * Note:
 *      For gsl type parts, ge_speed_set _must_ be called before
 *      duplex set to ensure that the proper mac is being updated.
 */
int mac_xge_duplex_set5632(int unit, soc_port_t port, int duplex)
{
    uint32    data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (duplex) { /* set full duplex */
        SET_P_CONF_3r_Duplex_cfg(data, 1);
    } 
	else {
        SET_P_CONF_3r_Duplex_cfg(data, 0);
    }
    return(cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_3r, port), data));
}

/*
 * Function:
 *	mac_xge_duplex_get5632
 *
 * Purpose:
 *	Get MAC duplex mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	duplex - (OUT) Boolean: true --> full duplex, false --> half duplex.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_duplex_get5632(int unit, soc_port_t port, int *duplex)
{
    uint32	gmacc1;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &gmacc1));
    *duplex = GET_P_CONF_3r_Duplex_cfg(gmacc1);
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_speed_set5632
 *
 * Purpose:
 *	Set a MAC in the specified speed.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - 10,100,1000 for speed.
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_speed_set5632(int unit, soc_port_t port, int speed)
{ 
    int rv = SOC_E_NONE;
    uint32 data, duplex_bit, speed_bit;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    speed_bit = GET_P_CONF_3r_Speedselect_cfg(data);
    duplex_bit = GET_P_CONF_3r_Duplex_cfg(data);
    /* if speed is 1000 than set the duplex mode to full-duplex */
    if (speed == 1000) {
        if (speed_bit == 2) { /* already in 1000 speed */
            return(SOC_E_NONE);
        } 
        speed_bit  = 2;    /* set to 1000 speed */
        duplex_bit = 1;    /* set to full duplex */
    } else if (speed == 100) {
        if (speed_bit == 1) { /* already in 100 speed */
            return(SOC_E_NONE);
        }
        speed_bit = 1; /* set to 100 speed */
    } else if (speed == 10) {
        if (speed_bit == 0) { /* already in 10 speed */
            return(SOC_E_NONE);
        }    
        speed_bit = 0; /* set to 10 speed */
    } else {
        return(SOC_E_NONE);
    }
    SET_P_CONF_3r_Speedselect_cfg(data, speed_bit);
    SET_P_CONF_3r_Duplex_cfg(data, duplex_bit);
    rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_3r, port), data);
    return rv;
}

/*
 * Function:
 *	mac_xge_speed_get5632
 *
 * Purpose:
 *	Get MAC speed mode.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	speed - (OUT) speed in Mb (10/100/1000)
 *
 * Returns:
 *	SOC_E_NONE
 */
int mac_xge_speed_get5632(int unit, soc_port_t port, int *speed)
{
    int rv = SOC_E_NONE;
    uint32 data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));
    if (GET_P_CONF_3r_Speedselect_cfg(data) == 2) {
        *speed = 1000; /* 1000 Mbps */
    } else if (GET_P_CONF_3r_Speedselect_cfg(data) == 1) {
        *speed = 100; /* 100 Mbps */
    } else {
        *speed = 10; /* 10 Mbps */
    }
    return rv;
}

/*
 * Function: 	mac_xge_pause_set5632
 * Purpose:
 *	Configure mac to transmit/receive pause frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - Boolean: transmit pause
 *	pause_rx - Boolean: receive pause
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_pause_set5632(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    uint32	gmacc1;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port), &gmacc1));
    /* 
     * if any tx/rx is enable, we need to set Pause capability on to enable it 
	 */
    if (pause_tx || pause_rx) { 
        SET_P_CONF_2r_PAUSE_CAP(gmacc1, 1);
    }
    SET_P_CONF_2r_TXPAUSE(gmacc1, pause_tx ? 1 : 0);
    SET_P_CONF_2r_RXPAUSE(gmacc1, pause_rx ? 1 : 0);
    SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port), gmacc1));
    return(SOC_E_NONE);
}

/*
 * Function: 	mac_xge_pause_addr_set5632
 * Purpose:
 *	Configure PAUSE frame source address.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_pause_addr_set5632(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    /* BCM5632 hardcode to use mac address 0 in chip */
    return(SOC_E_NONE);
}    

/*
 * Function: 	mac_xge_pause_get5632
 * Purpose:
 *	Return the current PAUSE configuration on the specified port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_tx - (OUT) TRUE is TX pause enabled.
 *	pause_rx - (OUT) TRUE if obey RX pause enabled.
 * Returns:
 *	SOC_E_XXXX.
 */
int mac_xge_pause_get5632(int unit, soc_port_t port,int *pause_tx,int *pause_rx)
{
    uint32	gmacc1;
    uint32   i;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_2r, port),&gmacc1));
    i = GET_P_CONF_2r_PAUSE_CAP(gmacc1);        
    /* 
      BCM5632 needs both Pause capablity and TxPause/RxPause enable to enable it
    */
    *pause_tx = (i && GET_P_CONF_2r_TXPAUSE(gmacc1)) ? TRUE : FALSE;
    *pause_rx = (i && GET_P_CONF_2r_RXPAUSE(gmacc1)) ? TRUE : FALSE;
    return(SOC_E_NONE);
}

/*
 * Function: 	mac_xge_pause_addr_get5632
 * Purpose:
 *	Retrieve the source addrtess for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit.
 *	pause_mac - (OUT) MAC address used for pause transmission.
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_pause_addr_get5632(int unit, soc_port_t port, mac_addr_t pause_mac)
{
    uint32	t_mac;

    /* BCM5632 hardcode in chip to use mac address 0*/
    t_mac = 0x0;
    memcpy(pause_mac, (void *)&t_mac, sizeof(t_mac));
    memcpy(pause_mac + sizeof(t_mac), (void *)&t_mac, sizeof(t_mac));
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_loopback_set5632
 *
 * Purpose:
 *	Set a MAC into/out-of loopback mode
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - Port number on unit.
 *	loopback - Boolean: true -> loopback mode, false -> normal operation
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_loopback_set5632(int unit, soc_port_t port, int loopback)
{
    /* BCM5632 does not support loopback */ 
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_loopback_get5632
 *
 * Purpose:
 *	Get current MAC loopback mode setting.
 *
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port # on unit
 *	loopback - (OUT) Boolean: true = loopback, false = normal
 *
 * Returns:
 *	SOC_E_XXXX
 */
int mac_xge_loopback_get5632(int unit, soc_port_t port, int *loopback)
{
    /* BCM5632 does not support loopback */
    *loopback = 0; 
    return(SOC_E_NONE);
}

/*
 * Function:
 *	mac_xge_ability5632
 * Purpose:
 *	Return the ability of the gigabit macs on the 5632.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch Port # on unit.
 *	mode - (OUT) Mac abilities on return.
 *
 * Returns:
 *	SOC_E_NONE
 */
int mac_xge_ability5632(int unit, soc_port_t port, port_mode_t *mode)
{
    *mode = SOC_PM_1000MB_FD | SOC_PM_MII | SOC_PM_10B |
        SOC_PM_LB_MAC | SOC_PM_PAUSE_TX | SOC_PM_PAUSE_RX;

    return(SOC_E_NONE);
}

#endif /* NUM_XGE_PORT_5632 > 0 */

