/*
 *=============================================================================
 * File Name   :    bcm5632-phy.h
 *
 *  RCS: $Id: bcm5632-phy.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:    For PM8353 PHY
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#ifndef BCM5632_PHY_H
#define BCM5632_PHY_H

#define PORT_TO_XPHY_ID(_u, _p)	((uchar8)(0x1c))

/* Standard MII Registers */

#define MII_CTRL_REG          	0x00	/* MII Control Register : r/w */
#define MII_STAT_REG          	0x01	/* MII Status Register: ro */ 
#define MII_PHY_ID0_REG       	0x02	/* MII PHY ID register: r/w */
#define MII_PHY_ID1_REG       	0x03	/* MII PHY ID register: r/w */
#define MII_ANA_REG     	0x04	/* MII Auto-Neg Advertisement: r/w */
#define MII_ANP_REG       	0x05	/* MII Auto-Neg Link Partner: ro */
#define MII_AN_EXP_REG        	0x06	/* MII Auto-Neg Expansion: ro */
#define MII_GB_CTRL_REG         0x09    /* MII 1000Base-T control register */
#define MII_GB_STAT_REG         0x0a    /* MII 1000Base-T Status register */
#define	MII_ESR_REG		0x0f	/* MII Extended Status register */

/* Non-standard MII Registers */

#define MII_ECR_REG			0x10	/* MII Extended Control Register */
#define	MII_PMC1_REG		0x10	/* MII Extended Control Register */
#define	MII_PMC2_REG		0x11	/* MII Extended Control Register */

/* MII Control Register: bit definitions */

#define MII_CTRL_SS_MSB	(1 << 6) /* Speed select, MSb */
#define MII_CTRL_CST	(1 << 7) /* Collision Signal test */
#define MII_CTRL_FD		(1 << 8) /* Full Duplex */
#define MII_CTRL_RAN	(1 << 9) /* Restart Autonegotiation */
#define MII_CTRL_IP		(1 << 10) /* Isolate Phy */
#define MII_CTRL_PD		(1 << 11) /* Power Down */
#define MII_CTRL_AE		(1 << 12) /* Autonegotiation enable */
#define MII_CTRL_SS_LSB	(1 << 13) /* Speed select, LSb */
#define MII_CTRL_LE		(1 << 14) /* Loopback enable */
#define MII_CTRL_RESET	(1 << 15) /* PHY reset */
#define MII_ECR_10B		(1 << 15) /* 1 == 10B, 0 == GMII */

#define	MII_CTRL_SS(_x)		((_x) & (MII_CTRL_SS_LSB|MII_CTRL_SS_MSB))
#define	MII_CTRL_SS_10		0
#define	MII_CTRL_SS_100		(MII_CTRL_SS_LSB)
#define	MII_CTRL_SS_1000	(MII_CTRL_SS_MSB)
#define	MII_CTRL_SS_INVALID	(MII_CTRL_SS_LSB | MII_CTRL_SS_MSB)

/* 
 * MII Status Register: See 802.3, 1998 pg 544 
 */
#define MII_STAT_EXT		(1 << 0) /* Extended Registers */
#define MII_STAT_JBBR		(1 << 1) /* Jabber Detected */
#define MII_STAT_LA		(1 << 2) /* Link Active */
#define MII_STAT_AN_CAP		(1 << 3) /* Autoneg capable */
#define MII_STAT_RF		(1 << 4) /* Remote Fault */
#define MII_STAT_AN_DONE	(1 << 5) /* Autoneg complete */
#define MII_STAT_MF_PS		(1 << 6) /* Preamble suppression */
#define	MII_STAT_ES		(1 << 8) /* Extended status (R15) */
#define	MII_STAT_HD_100_T2	(1 << 9) /* Half duplex 100Mb/s supported */
#define	MII_STAT_FD_100_T2	(1 << 10)/* Full duplex 100Mb/s supported */
#define	MII_STAT_HD_10		(1 << 11)/* Half duplex 100Mb/s supported */
#define	MII_STAT_FD_10		(1 << 12)/* Full duplex 100Mb/s supported */
#define	MII_STAT_HD_100		(1 << 13)/* Half duplex 100Mb/s supported */
#define	MII_STAT_FD_100		(1 << 14)/* Full duplex 100Mb/s supported */
#define	MII_STAT_100_T4		(1 << 15)/* Full duplex 100Mb/s supported */
#define XMII_STAT_LA		(1 << 4) /* Link Active */

/*
 * MII Link Advertisment
 */
#define	MII_ANA_ASF		(1 << 0)/* Advertise Selector Field */
#define	MII_ANA_FD		(1 << 5)/* Half duplex 10Mb/s supported */
#define	MII_ANA_HD		(1 << 6)/* Full duplex 10Mb/s supported */
#define	MII_ANA_PAUSE	(1 << 8)/* Half duplex 100Mb/s supported */
#define	MII_ANA_RF		(1 << 13)/* Remote fault */
#define MII_ANA_ASYM_PAUSE (1 << 11) /* Asymmetric pause supported */
#define	MII_ANA_NP		(1 << 15)/* Next Page */

#define	MII_ANA_ASF_802_3	(1)	/* 802.3 PHY */

/*
 * 1000Base-T Control Register
 */
#define	MII_GB_CTRL_MS_MAN	(1 << 12) /* Manual Master/Slave mode */
#define	MII_GB_CTRL_MS		(1 << 11) /* Master/Slave negotiation mode */
#define	MII_GB_CTRL_PT		(1 << 10) /* Port type */
#define	MII_GB_CTRL_ADV_1000FD	(1 << 9) /* Advertise 1000Base-T FD */
#define	MII_GB_CTRL_ADV_1000HD	(1 << 8) /* Advertise 1000Base-T HD */

/*
 * 1000Base-T Status Register
 */
#define MII_GB_STAT_MS_FAULT	(1 << 15) /* Master/Slave Fault */
#define MII_GB_STAT_MS		(1 << 14) /* Master/Slave, 1 == Master */
#define MII_GB_STAT_LRS		(1 << 13) /* Local receiver status */
#define	MII_GB_STAT_RRS		(1 << 12) /* Remote receiver status */
#define	MII_GB_STAT_LP_1000FD	(1 << 11) /* Link partner 1000FD capable */
#define	MII_GB_STAT_LP_1000HD	(1 << 12) /* Link partner 1000HD capable */
#define MII_GB_STAT_IDE		(0xff << 0) /* Idle error count */

/*
 * IEEE Extended Status Register
 */
#define	MII_ESR_1000_X_FD	(1 << 15) /* 1000Base-T FD capable */
#define	MII_ESR_1000_X_HD	(1 << 14) /* 1000Base-T HD capable */
#define	MII_ESR_1000_T_FD	(1 << 13) /* 1000Base-T FD capable */
#define	MII_ESR_1000_T_HD	(1 << 12) /* 1000Base-T FD capable */

#define PHY_MIN_REG		0
#define PHY_MAX_REG		0x1f

#define PHY_OUI_LOW_MASK        0xfc00
#define PHY_ID_MASK             0x03f0
#define PHY_ID_REV_MASK         0x000f

#define SOC_PM_10MB_HD          (1<<0)
#define SOC_PM_10MB_FD          (1<<1)
#define SOC_PM_100MB_HD         (1<<2)
#define SOC_PM_100MB_FD         (1<<3)
#define SOC_PM_1000MB_HD        (1<<4)
#define SOC_PM_1000MB_FD        (1<<5)
#define SOC_PM_10000MB_FD		(1<<13)

#define SOC_PM_PAUSE_TX         (1<<6)  /* TX pause capable */
#define SOC_PM_PAUSE_RX         (1<<7)  /* RX pause capable */
#define SOC_PM_MII              (1<<8)  /* MII mode supported */
#define SOC_PM_10B              (1<<9)  /* 10B mode supported */
#define SOC_PM_LB_MAC           (1<<10) /* MAC loopback supported */
#define SOC_PM_LB_PHY           (1<<11) /* PHY loopback supported */

#define SOC_PM_AN               (1<<12) /* Auto-negotiation */

#define SOC_PM_PAUSE            (SOC_PM_PAUSE_TX|SOC_PM_PAUSE_RX)
#define SOC_PM_10MB             (SOC_PM_10MB_HD|SOC_PM_10MB_FD)
#define SOC_PM_100MB            (SOC_PM_100MB_HD|SOC_PM_100MB_FD)
#define SOC_PM_1000MB           (SOC_PM_1000MB_HD|SOC_PM_1000MB_FD)
#define SOC_PM_FD               (SOC_PM_10MB_FD |       \
                                 SOC_PM_100MB_FD |      \
                                 SOC_PM_1000MB_FD)
#define SOC_PM_HD               (SOC_PM_10MB_HD |       \
                                 SOC_PM_100MB_HD |      \
                                 SOC_PM_1000MB_HD)

#define SOC_PM_ABILITY_PHY		(SOC_PM_LB_MAC|SOC_PM_AN)
#define SOC_PM_ABILITY_MAC		(SOC_PM_LB_MAC)

extern int phy_ge_enable_get5632(int unit, uint32 port, int *enable);
extern int phy_ge_enable_set5632(int unit, uint32 port, int enable);
extern int phy_ge_init5632(int unit, uint32 port);
extern int phy_ge_link_get5632(int unit, uint32 port, int *link);
extern int phy_ge_duplex_set5632(int unit, uint32 port, int duplex);
extern int phy_ge_duplex_get5632(int unit, uint32 port, int *duplex);
extern int phy_ge_speed_set5632(int unit, uint32 port, int speed);
extern int phy_ge_speed_get5632(int unit, uint32 port, int *speed);
extern int phy_ge_an_set5632(int unit, uint32 port, int an);
extern int phy_ge_an_get5632(int unit, uint32 port, int *an, int *an_done);
extern int phy_ge_adv_local_set5632(int unit, uint32 port, uint32 mode);
extern int phy_ge_adv_local_get5632(int unit, uint32 port, uint32 *mode);
extern int phy_ge_adv_remote_get5632(int unit, uint32 port, uint32 *mode);
extern int phy_ge_lb_set5632(int unit, uint32 port, int enable);
extern int phy_ge_lb_get5632(int unit, uint32 port, int *enable);
extern int phy_ge_interface_set5632(int unit, uint32 port, int mii);
extern int phy_ge_interface_get5632(int unit, uint32 port, int *mii);
extern int phy_ge_ability5632(int unit, uint32 port, uint32 *mode);

extern int mac_ge_init5632(int unit, soc_port_t port);
extern int mac_ge_duplex_set5632(int unit, soc_port_t port, int duplex);
extern int mac_ge_duplex_get5632(int unit, soc_port_t port, int *duplex);
extern int mac_ge_pause_set5632(int unit, soc_port_t port, int pause_tx, int pause_rx);
extern int mac_ge_pause_get5632(int unit, soc_port_t port, int *pause_tx, int *pause_rx);
extern int mac_ge_pause_addr_set5632(int unit, soc_port_t port, mac_addr_t);
extern int mac_ge_pause_addr_get5632(int unit, soc_port_t port, mac_addr_t);
extern int mac_ge_interface_set5632(int unit, soc_port_t port, int mii);
extern int mac_ge_speed_set5632(int unit, soc_port_t port, int speed);
extern int mac_ge_speed_get5632(int unit, soc_port_t port, int *speed);

#endif /* BCM5632_PHY_H */
