/*
 *=============================================================================
 * File Name   :    bcm5632-soctypes.h
 *
 *  RCS: $Id: bcm5632-soctypes.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef BCM5632_SOCTYPES_H
#define BCM5632_SOCTYPES_H

#include "bcm5632-socregs.h"

/*
 * Port bitmap used throughout the BCM5632 switch chip family
 *
 * BCM5632:
 *
 *     31               14    13     12    11       0
 *   +---------------------+------+------+------------+
 *   |          0          | CMIC | GE12 | GE11 - GE0 |
 *   +---------------------+------+------+------------+
 *
 */
typedef uint32 physaddr_t;

typedef uint32 pbmp_t;

#define PBMP_NONE		0
#define PBMP_PORT(port)		(1U << (port))
#define PBMP_RANGE(lp, hp)	(PBMP_PORT((hp) + 1) - PBMP_PORT(lp))
#define PBMP_FE(port)		PBMP_PORT(MIN_FE_PORT_5632 + (port))
#define PBMP_FE_ALL		(PBMP_FE(NUM_FE_PORT_5632) - PBMP_FE(0))
#define PBMP_GE(port)		PBMP_PORT(MIN_GE_PORT_5632 + (port))
#define PBMP_GE_ALL		(PBMP_GE(NUM_GE_PORT_5632) - PBMP_GE(0))
#define PBMP_E(port)		PBMP_PORT(port)		/* HACK */
#define PBMP_E_ALL		(PBMP_FE_ALL | PBMP_GE_ALL)
#define PBMP_IPIC		PBMP_PORT(26)		/* No IPIC yet */
#define PBMP_CMIC		PBMP_PORT(CMIC_PORT_5632)
#define PBMP_ALL		(PBMP_E_ALL | PBMP_CMIC)    /* No IPIC yet */
#define PBMP_MEMBER(bmp, port)	(((bmp) & PBMP_PORT(port)) != 0)
#define PBMP_INVALID		0xffffffff
#define PBMP_ITER(bmp, _port)	for (_port = MIN_PORT_5632; \
					_port <= MAX_PORT_5632; _port++) \
						if (PBMP_MEMBER((bmp), _port))

typedef int dma_chan_t;

/*
 * vlan_id_t		12-bit VLAN ID
 * vlan_ctrl_t		16-bit VLAN control (prio + cfi + id)
 * vlan_tag_t		32-bit VLAN tag (structure)
 */

typedef uint16 vlan_id_t;

#define VLAN_ID_NONE			((vlan_id_t) 0x0000)
#define VLAN_ID_DEFAULT			((vlan_id_t) 0x0001)
#define VLAN_ID_SHARED_BASE		((vlan_id_t) 0x0f80)
#define VLAN_ID_SHARED_COUNT		((vlan_id_t) 0x0020)
#define VLAN_ID_INVALID			((vlan_id_t) 0x0fff)
#define VLAN_ID_VALID(id)		((id) >= VLAN_ID_DEFAULT && \
					 (id) < VLAN_ID_INVALID)

typedef uint16 vlan_ctrl_t;

#define VLAN_CTRL(prio, cfi, id)	(((prio) & 0x007) << 13 | \
					 ((cfi ) & 0x001) << 12 | \
					 ((id  ) & 0xfff) << 0)

#define VLAN_CTRL_PRIO(c)		((c) >> 13 & 0x007)
#define VLAN_CTRL_CFI(c)		((c) >> 12 & 0x001)
#define VLAN_CTRL_ID(c)			((c) >>  0 & 0xfff)

#define VLAN_ENTRY_COUNT	    4096

typedef union vlan_tag_u {
    uint16		vt_tpid;	/* Tag protocol ID = 0x8100 */
    vlan_ctrl_t		vt_ctrl;	/* Tag control */
} vlan_tag_t;

/*
 * Trunking definitions
 */

typedef int trunk_id_t;

#define TRUNK_ID_MIN		0
#define TRUNK_ID_MAX		5
#define TRUNK_ID_COUNT		(TRUNK_ID_MAX + 1 - TRUNK_ID_MIN)

/*
 * Class-of-service definitions
 */

#define COS_QUEUE_COUNT		NUM_COS
#define COS_PRIO_COUNT		8

typedef int cos_prio_t;
typedef int cos_queue_t;

/*
 * ARL definitions
 */

#define ARL_ENTRY_VALID(ae)	((ae)->vlan_tag != 0xfff)

#define ARL_MATCH_VLAN(p1, p2)	((p1)->vlan_tag == (p2)->vlan_tag)
#define ARL_MATCH_MAC(p1, p2)	((p1)->mac_addr_b0 == (p2)->mac_addr_b0 && \
				 (p1)->mac_addr_b1 == (p2)->mac_addr_b1 && \
				 (p1)->mac_addr_b2 == (p2)->mac_addr_b2 && \
				 (p1)->mac_addr_b3 == (p2)->mac_addr_b3 && \
				 (p1)->mac_addr_b4 == (p2)->mac_addr_b4 && \
				 (p1)->mac_addr_b5 == (p2)->mac_addr_b5)

#define ARL_MATCH(p1, p2)	(ARL_MATCH_MAC(p1, p2) && \
				 ARL_MATCH_VLAN(p1, p2))

/*
 * Macros to get and set the MAC address in an ARL entry from an array
 * of bytes or mac_addr_t (useful in many places).  Also works for any
 * of the other table types whose entries have MAC addresses.
 */

#define	ARL_MAC_GET(_a, _m) 			\
    do { /* Look like single 'C' stmt */	\
        (_m)[0] = (_a).mac_addr_b5;		\
	(_m)[1] = (_a).mac_addr_b4;		\
	(_m)[2] = (_a).mac_addr_b3;		\
	(_m)[3] = (_a).mac_addr_b2;		\
	(_m)[4] = (_a).mac_addr_b1;		\
	(_m)[5] = (_a).mac_addr_b0;		\
    } while (0)

#define	ARL_MAC_GET_5632(_a, _m) 		\
	do { /* Look like single 'C' stmt */	\
        (_m)[0] = (_a).MacAddr5;		\
		(_m)[1] = (_a).MacAddr4;	\
		(_m)[2] = (_a).MacAddr3;	\
		(_m)[3] = (_a).MacAddr2;	\
		(_m)[4] = (_a).MacAddr1;	\
		(_m)[5] = (_a).MacAddr0;	\
    } while (0)

#define	ARL_MAC_SET_5632(_a, _m)		\
    do { /* Look like single 'C' stmt */	\
	(_a).MacAddr5 = (_m)[0];		\
	(_a).MacAddr4 = (_m)[1];		\
	(_a).MacAddr3 = (_m)[2];		\
	(_a).MacAddr2 = (_m)[3];		\
	(_a).MacAddr1 = (_m)[4];		\
	(_a).MacAddr0 = (_m)[5];		\
    } while (0)

#define	ARL_MAC_SET(_a, _m)			\
    do { /* Look like single 'C' stmt */	\
	(_a).mac_addr_b0 = (_m)[5];		\
	(_a).mac_addr_b1 = (_m)[4];		\
	(_a).mac_addr_b2 = (_m)[3];		\
	(_a).mac_addr_b3 = (_m)[2];		\
	(_a).mac_addr_b4 = (_m)[1];		\
	(_a).mac_addr_b5 = (_m)[0];		\
    } while (0)

#define	L3INTF_MAC_GET(_a, _m) 			\
    do { /* Look like single 'C' stmt */	\
        (_m)[0] = (_a).rout_addr_b5;		\
	(_m)[1] = (_a).rout_addr_b4;		\
	(_m)[2] = (_a).rout_addr_b3;		\
	(_m)[3] = (_a).rout_addr_b2;		\
	(_m)[4] = (_a).rout_addr_b1;		\
	(_m)[5] = (_a).rout_addr_b0;		\
    } while (0)

#define	L3INTF_MAC_SET(_a, _m)			\
    do { /* Look like single 'C' stmt */	\
	(_a).rout_addr_b0 = (_m)[5];		\
	(_a).rout_addr_b1 = (_m)[4];		\
	(_a).rout_addr_b2 = (_m)[3];		\
	(_a).rout_addr_b3 = (_m)[2];		\
	(_a).rout_addr_b4 = (_m)[1];		\
	(_a).rout_addr_b5 = (_m)[0];		\
    } while (0)

#define	L3_MAC_SET ARL_MAC_SET
#define	L3_MAC_GET ARL_MAC_GET
#define	DEFIP_MAC_SET ARL_MAC_SET
#define	DEFIP_MAC_GET ARL_MAC_GET

/*
 * PTABLE bit definitions
 */

/* Values for SPSSTATE */

#define PVP_STP_DISABLED	0	/* Disabled */
#define PVP_STP_BLOCKING	1	/* Blocking/Listening */
#define PVP_STP_LEARNING	2	/* Learning */
#define PVP_STP_FORWARDING	3	/* Forwarding */

/* Values for CML */

#define	PVP_CML_SWITCH		0	/* Switch learns addresses */
#define	PVP_CML_CPU		1	/* CPU learning */
#define	PVP_CML_FORWARD		2	/* No Learning, switch packet */
#define	PVP_CML_DROP		3	/* No learning, drop packet */

/*
 * Rule definitions
 *
 * NOTE:  In socregs, filter_irule_entry_t and filter_imask_entry_t are
 *	  big-endian formats.  This was done so the filter data would
 *	  align with network packet data.  Unfortunately, the entire
 *	  structure is reversed, not just the 64-byte field.
 *
 *	  The high byte of filter[0] corresponds to the first byte in
 *        the network packet.  This is true regardless of host endian
 *        since the lower level routines swap appropriately when copying
 *        the data out to the hardware as 32-bit words.
 */

#define FILTER_ACTION_INSERT_PRIO		0x01
#define FILTER_ACTION_SET_COS_QUEUE		0x02
#define FILTER_ACTION_INSERT_IP_TOS		0x04
#define FILTER_ACTION_COPY_TO_CPU		0x08
#define FILTER_ACTION_DO_NOT_SWITCH		0x10
#define FILTER_ACTION_SET_OUTPUT_PORT		0x20
#define FILTER_ACTION_COPY_TO_MIRROR		0x40

#endif /* BCM5632_SOCTYPES_H */
