/*  
 *=============================================================================
 * File Name   :    bmc12500-mac-regs.h
 *
 *  Description:    
 *
 *  Created    :    09 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  RCS		   :	$Id: bmc12500-eth-regs.h,v 1.5 2002/08/01 17:55:51 sandrineb Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#ifndef BMC12500_ETH_REGS_H_INCLUDED
#define BMC12500_ETH_REGS_H_INCLUDED

/* Option flags for Ethernet Transmit Channels */
#define BMC12500_ETH_TXOPT_APPEND_CRC			0x1
#define BMC12500_ETH_TXOPT_REPLACE_CRC			0x2
#define BMC12500_ETH_TXOPT_APPEND_CRC_PAD		0x3
#define BMC12500_ETH_TXOPT_NO_MOD			0x7

typedef struct _bc1250Eth_regs
{
	uint64 *dma_config0_rx[2];	
	uint64 *dma_config0_tx[2];	

	uint64 *dma_config1_rx[2];	
	uint64 *dma_config1_tx[2];	
	
	uint64 *dma_dscr_base_rx[2];
	uint64 *dma_dscr_base_tx[2];

	uint64 *dma_asic_addr;

	uint64 *dma_dscr_cnt_rx[2];
	uint64 *dma_dscr_cnt_tx[2];

	uint64 *dma_cur_dscra_rx[2];
	uint64 *dma_cur_dscra_tx[2];
	
	uint64 *dma_cur_dscrb_rx[2];
	uint64 *dma_cur_dscrb_tx[2];

	uint64 *dma_cur_dscr_addr_rx[2];
	uint64 *dma_cur_dscr_addr_tx[2];

	uint64 *mac_cfg;
	uint64 *mac_enable;
	uint64 *mac_txd_ctl;
	uint64 *mac_thrsh_cfg;
	uint64 *mac_frame_cfg;
	uint64 *mac_vlantag;
	uint64 *mac_status;
	uint64 *mac_debug_status;
	uint64 *mac_int_mask;
	uint64 *mac_fifo_ptrs;			// RO: Limited to broadcom use only
	uint64 *mac_eopcnt;				// RO: Limited to broadcom use only
	uint64 *mac_addr[8];
	uint64 *mac_hash[8];
	uint64 *mac_ethernet_addr;		// Src addr inserted into outgoing pkts
	uint64 *mac_type_cfg;
	uint64 *mac_adfilter_cfg;
	uint64 *mac_chlo[4];
	uint64 *mac_chup[4];
	uint64 *mac_mdio;

	uint64 *rmon;
} bc1250Eth_regs_t;

//------------------------------------------------
// BMC12500 ETHERNET MAC CONFIGURATION REGISTER 0
//------------------------------------------------
typedef union _bmc12500_mac_cfg_reg
{
	uint64 wd;
	struct {
		uint64 ss_tmode			:1;
		uint64 tx_hold_sop_en	:1;
		uint64 retry_en			:1;
		uint64 ret_drpreq_en	:1;
		uint64 ret_ufl_en		:1;
		uint64 burst_en			:1;
		uint64 tx_pause_cnt		:3;
		uint64 resv1			:8;
		uint64 ap_stat_en		:1;
		uint64 resv2			:1;
		uint64 drp_errpkt_en	:1;
		uint64 drp_fcserrpkt_en	:1;
		uint64 drp_codeerrpkt_en:1;
		uint64 drp_drblerrpkt_en:1;
		uint64 drp_rntpkt_en	:1;
		uint64 drp_oszpkt_en	:1;
		uint64 drp_lenerrpkt_en	:1;
		uint64 resv3			:6;
		uint64 bypass_sel		:1;
		uint64 hdx_en			:1;
		uint64 speed_sel		:2;
		uint64 tx_clk_edge		:1;
		uint64 loopback_sel		:1;	
		uint64 fast_sync		:1;
		uint64 ss_en			:1;
		uint64 bypass_cfg		:2;
		uint64 bypass_16		:1;
		uint64 bypass_fcs_chk	:1;
		uint64 resv4			:2;
		uint64 bypass_ifg		:8;
		uint64 fc_sel			:1;
		uint64 fc_cmd			:2;
		uint64 rx_ch_sel		:7;
	} bits;
} bmc12500_macCfgr_t;

//---------------------------------------------
// BMC12500 MAC ENABLE REGISTER 
//---------------------------------------------
typedef union _bmc12500_mac_enable_reg 
{
	uint64 wd;
	struct {
		uint64 rxdma_en			:2;
		uint64 resv1			:2;
		uint64 txdma_en			:2;
		uint64 resv2			:2;
		uint64 p_reset			:1;
		uint64 resv3			:1;
		uint64 mac_rx_en		:1;
		uint64 mac_tx_en		:1;
		uint64 byp_rx_en		:1;
		uint64 byp_tx_en		:1;
		uint64 resv4			:50;
	} bits;
} bmc12500_macEnar_t;

//---------------------------------------------
// BMC12500 MAC FIFO THRESHOLD REGISTER 
//---------------------------------------------
typedef union _bmc12500_mac_fifo_thresh_reg
{
	uint64 wd;
	struct {
		uint64 tx_wr_thresh		:7;
		uint64 resv1			:1;
		uint64 tx_rd_thresh		:7;
		uint64 resv2			:1;
		uint64 tx_rl_thresh		:6;
		uint64 resv3			:10;
		uint64 rx_rd_thresh		:6;
		uint64 resv4			:2;
		uint64 rx_rl_thresh		:6;
		uint64 resv5			:10;
		uint64 enc_fc_thresh	:6;
		uint64 resv6			:2;
	} bits;
} bmc12500_mac_fifo_thresh_r_t;

//---------------------------------------------
// BMC12500 MAC FRAME CONFIGURATION REGISTER 
//---------------------------------------------
typedef union _bmc12500_mac_frame_config_reg
{
	uint64 wd;
	struct {
		uint64 ifg_rx			:6;
		uint64 ifg_tx			:6;
		uint64 ifg_thresh		:6;
		uint64 backoff_sel		:4;
		uint64 ifsr_seed		:8;
		uint64 slot_offset		:10;
		uint64 min_framesz		:8;
		uint64 max_framesz		:16;
	} bits;
} bmc12500_macFrmCfgr_t;

//---------------------------------------------
// BMC12500 MAC DMA CONFIGURATION REGISTER 0
//---------------------------------------------
typedef union _bmc12500Eth_mac_dma_config0_reg
{
	uint64 wd;
	struct {
		uint64 drop       :1;
		uint64 chain_sel  :1;
		uint64 resv1      :1;
		uint64 eop_int_en :1;
		uint64 hwm_int_en :1;
		uint64 lwm_int_en :1;
		uint64 tbx_en     :1;
		uint64 tdx_en     :1;
		uint64 int_pktcnt :8;
		uint64 ringsz     :16;
		uint64 hi_watermk :16;
		uint64 lo_watermk :16;
	} bits;
} bmc12500_mac_dma_cr0_t;

//---------------------------------------------
// BMC12500 MAC DMA CONFIGURATION REGISTER 1
//---------------------------------------------
typedef union _bmc12500Eth_mac_dma_config1_reg
{
	uint64 wd;
	struct {
		uint64 hdr_cf_ena   :1;
		uint64 asic_xfr_ena :1;
		uint64 pre_addr_ena :1;
		uint64 flow_ctl_en  :1;
		uint64 no_dscr_updt :1;
		uint64 dscr_l2ca	:1;
		uint64 reserved		:10;
		uint64 zero			:5;
		uint64 hdr_size		:9;
		uint64 reserved1	:2;
		uint64 zero1		:5;
		uint64 asicxfr_size	:9;
		uint64 reserved2	:2;
		uint64 int_timeout	:16;
	} bits;
} bmc12500_mac_dma_cr1_t;

typedef struct _bc1250Eth_rmon
{
	uint64 tx_byte_cnt;			// Total tx bytes
	uint64 tx_coll_cnt;			// Collision 
	uint64 tx_lcol_cnt;			// Late Collision 
	uint64 tx_xcol_cnt;			// Excessive Collisions
	uint64 tx_fcse_cnt;			// FCS Errors
	uint64 tx_abrt_cnt;			// Abort Errors
	uint64 resv1;
	uint64 tx_badp_cnt;			// Bad Packets
	uint64 tx_good_cnt;			// Good Packets
	uint64 tx_runt_cnt;			// Runt Packets
	uint64 tx_ovsz_cnt;			// Over sized Packets
	uint64 resv2[5];
	uint64 rx_byte_cnt;			// Total Rx bytes
	uint64 rx_mcst_cnt;			// Multicast Packets
	uint64 rx_bcst_cnt;			// Broadcast Packets
	uint64 rx_badp_cnt;			// Bad Packets
	uint64 rx_good_cnt;			// Good Packets
	uint64 rx_runt_cnt;			// Runt Packets
	uint64 rx_ovsz_cnt;			// Over sized Packets
	uint64 rx_fcse_cnt;			// FCS Errors
	uint64 rx_lene_cnt;			// Length Errors
	uint64 rx_code_cnt;			// Code Errors
	uint64 rx_drbl_cnt;			// Dribble Errors
} bc1250Eth_rmon_t;
#endif /* BMC12500_MAC_REGS_H_INCLUDED */
