/* vi:ts=4  
 *-----------------------------------------------------------------
 *
 * Name: bsd-stats.c
 *
 * RCS:  $Id
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: all tcp/ip stats. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/28/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "exports/bsd-api.h"
#include "exports/cm-api.h"
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/domain.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"
#include "../sm-bsd-sys/syslog.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/if_var.h"
#include "../sm-bsd-net/if_dl.h"
#include "../sm-bsd-net/route.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/udp.h"
#include "../sm-bsd-netinet/udp_var.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "../sm-bsd-netinet/ip_icmp.h"
#include "../sm-bsd-netinet/ip_icmp.h"
#include "../sm-bsd-netinet/icmp_var.h"
#include "../sm-bsd-netinet/if_ether.h"
#include "../sm-ipm/ipm.h"
#include "../sm-luc/luc-reas.h"
#include "../sm-luc/luc.h"
#include "bsd-arpsniff.h"
#include "bsd-vif.h"

/*-----------------------------------------------------------------
 * Name:                bsd_FillIpmStatsMsg
 *
 * Description: IP packet stats.
 *
 * Created by : Su Wang 
 *
 * Date Created: 12/12/01
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_FillIpmStatsMsg(int mtype, void *msg, void	*parm)
{
  ipm_ifstat_t	*ifsp;
  ipm_ipstat_t	*ipsp;
  ipm_icpstat_t	*icmpsp;
  ipm_tcpstat_t	*tcpsp;
  ipm_udpstat_t	*udpsp;
  struct ifnet	*ifp = (struct ifnet*)parm;
  int			link;
  luc_stats_t	*pRxStats, *pTxStats;
  int			extrasize;		/* size of extra stuff */
								/* besides bsd stats */



  switch(mtype)
	{
	case IPM_MSG_INTF_STATS:
	  ifsp = (ipm_ifstat_t*)msg;
	  
	  link = GET_ULINK_FROM_CPULINK(ifp->if_cpu, ifp->if_unit);
	  sprintf(ifsp->ifi_ifname,
			  "%s%d.%d", 
			  ifp->if_name, ifp->if_slot, link); 
				
	  ifsp->ifi_speed = 100000000;
	  ifsp->ifi_oQlen = ifp->if_snd.ifq_len;
	  
	  /* fill in the stats field */
	  pRxStats   = &pLucCfg->linkCfg[link].rxStats;
	  pTxStats   = &pLucCfg->linkCfg[link].txStats;
	  
	  ifsp->ifi_ipackets = pRxStats->pktCnt;
	  ifsp->ifi_ibytes = pRxStats->byteCnt;
	  ifsp->ifi_iqdrops = pRxStats->discards;
	  ifsp->ifi_ierrors = pRxStats->noEeeDesc + pRxStats->noPktDesc +
		  pRxStats->noBuffErr + pRxStats->badPktDesc + 
		  pRxStats->hwBadIpVer + pRxStats->hwBadIpCsum +
		  pRxStats->hwBadIpHlen + pRxStats->hwBadTplCsum +
		  pRxStats->hwBadTcpHlen + pRxStats->linkErr;
	  
	  
	  ifsp->ifi_opackets = pTxStats->pktCnt;
	  ifsp->ifi_obytes = pTxStats->byteCnt;
	  ifsp->ifi_oerrors = pRxStats->noEeeDesc + pRxStats->noPktDesc +
		  pRxStats->noBuffErr + pRxStats->badPktDesc;
	  return NFX_OK;

	case IPM_MSG_IP_STATS:
	  ipsp = (ipm_ipstat_t*)msg;
	  
	  ipsp->ips_card = nfxMySlotId;
	  ipsp->ips_defaultTTL = IPDEFTTL;
	  extrasize =  (int)((address_t)&(ipsp->ips_total) - (address_t)ipsp);
	  if (sizeof(ipm_ipstat_t) != extrasize + sizeof(struct ipstat)) {
	    printf("ipm_ipstat %d, ipstat %d, extrasize %d\n",
		   (int)sizeof(ipm_ipstat_t), (int)sizeof(struct ipstat), extrasize);
	    return NFX_ERR;
	  }

	  memcpy(&(ipsp->ips_total), &ipstat, sizeof(struct ipstat));
	  return NFX_OK;

	case IPM_MSG_ICMP_STATS:
	  icmpsp = (ipm_icpstat_t*)msg;
	  icmpsp->icps_card = nfxMySlotId;
	  extrasize = (int)((address_t)&(icmpsp->icps_error) - (address_t)icmpsp);
	  if (sizeof(ipm_icpstat_t) != extrasize + sizeof(struct icmpstat)) {
	    printf("ipm_icpstat_t %d, icmpstat %d, extrasize %d\n",
		   (int)sizeof(ipm_icpstat_t), (int)sizeof(struct icmpstat), extrasize);
	    return NFX_ERR;
	  }

	  memcpy(&(icmpsp->icps_error), &icmpstat, sizeof(struct icmpstat));
	  return NFX_OK;
	  
	case IPM_MSG_TCP_STATS:
	  tcpsp = (ipm_tcpstat_t*)msg;
	  tcpsp->tcps_card = nfxMySlotId;
	  extrasize = (int)((address_t)&(tcpsp->tcps_connattempt) - (address_t)tcpsp);
	  if (sizeof(ipm_tcpstat_t) != extrasize + sizeof(struct tcpstat)) {
	    printf("ipm_tcpstat_t %d, tcp_stat %d, extrasize %d\n",
		   (int)sizeof(ipm_tcpstat_t), (int)sizeof(struct tcpstat), extrasize);
	    return NFX_ERR;
	  }

	  memcpy(&(tcpsp->tcps_connattempt), &tcpstat, sizeof(struct tcpstat));
	  return NFX_OK;
	  
	  
	case IPM_MSG_UDP_STATS:
	  udpsp = (ipm_udpstat_t*)msg;
	  udpsp->udps_card = nfxMySlotId;
	  extrasize = (int)((address_t)&(udpsp->udps_ipackets) - (address_t)udpsp);
	  if (sizeof(ipm_udpstat_t) != extrasize + sizeof(struct udpstat)) {
	    printf("ipm_udpstat_t %d, udpstat %d, extrasize %d\n",
		   (int)sizeof(ipm_udpstat_t),  (int)sizeof(struct udpstat), extrasize);
	    return NFX_ERR;
	  }

	  memcpy(&(udpsp->udps_ipackets), &udpstat, sizeof(struct udpstat));
	  return NFX_OK;	  
	}

  return NFX_ERR;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ShowIpStats
 *
 * Description: IP packet stats.
 *
 * Created by : Su Wang 
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowIpStats(struct ipstat *ips)
{
  uiPrintf("IP packets statistics \n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Tot Pkts Rcvd :  %-12d		Pkts Delivered:  %-12d\n",
		   ips->ips_total, ips->ips_delivered);
  uiPrintf("Pkts Fwd      :  %-12d		Pkts Cant Fwd :  %-12d\n",
		   ips->ips_forward, ips->ips_cantforward);
  uiPrintf("Pkts Bad Hlen :  %-12d		Pkts Bad Len  :  %-12d\n",
		   ips->ips_badhlen, ips->ips_badlen);
  uiPrintf("Pkts Bad Opts :  %-12d		Pkts Bad Csum :  %-12d\n",
		   ips->ips_badoptions, ips->ips_badsum);
  uiPrintf("Pkts Bad Ver  :  %-12d		Pkts Bad Proto:  %-12d\n",
		   ips->ips_badvers, ips->ips_noproto);
  uiPrintf("Pkts Short    :  %-12d		Pkts Small    :  %-12d\n",
		   ips->ips_tooshort, ips->ips_toosmall);
  uiPrintf("Pkts Frag Rcvd:	 %-12d		Pkts Frag Drop:  %-12d\n",
		    ips->ips_fragments, ips->ips_fragdropped);
  uiPrintf("Pkts Frag Tout:	 %-12d		Pkts Frag reas:  %-12d\n",
		   ips->ips_fragtimeout, ips->ips_reassembled);
  uiPrintf("Pkts RawIP In :  %-12d\n",
		   ips->ips_rawin);

  uiPrintf("\n");
  uiPrintf("Tot Pkts Out  :  %-12d\n",
		   ips->ips_localout);
  uiPrintf("Pkts Frag Pkt :  %-12d		Pkts Frag Frag:  %-12d\n",
		   ips->ips_fragmented, ips->ips_ofragments);
  uiPrintf("Pkts Cant Frag:  %-12d		Pkts Frag Drop:  %-12d\n",
		   ips->ips_cantfrag, ips->ips_odropped);
  uiPrintf("Pkts No Rt    :  %-12d		Pkts Redir    :  %-12d\n",
		   ips->ips_noroute, ips->ips_redirectsent);
  uiPrintf("Pkts RawIP Out:	 %-12d\n",
		   ips->ips_rawout);
  
  uiPrintf("\n");
  uiPrintf("ARP Rl Fwd    :	 %-12d		IP Rl Fwd     :  %-12d\n",
		   ips->rl_arpfwd, ips->rl_ipfwd);
  uiPrintf("ARP Rl Rcvd   :	 %-12d		IP Rl Rcvd    :  %-12d\n",
		   ips->rl_arprcv, ips->rl_iprcv);
  uiPrintf("ARP Rl Fwd Err:	 %-12d		IP Rl Fwd Err :  %-12d\n",
		   ips->rl_arpfwderr, ips->rl_ipfwderr);
  uiPrintf("ARP Rl Rcv Err:	 %-12d		IP Rl Rcv Err :  %-12d\n",
		   ips->rl_arprcverr, ips->rl_iprcverr);
  uiPrintf("Rl Rcv No Prot:	 %-12d\n",
		   ips->rl_noproto);

  uiPrintf("\n");
  uiPrintf("Frags Rl Fwd  :	 %-12d		IP Rl Frag fwd:  %-12d\n",
		   ips->rl_fragsfwd, ips->rl_fragedfwd);
  uiPrintf("Frags Rl F_Err:	 %-12d\n",
		   ips->rl_fragsfwderr);
  uiPrintf("Frags Rl Rcv  :	 %-12d		IP Rl Reas Rcv:  %-12d\n",
		   ips->rl_fragsrcv, ips->rl_reasrcv);
  uiPrintf("Frags Rl R_Err:	 %-12d\n",
		   ips->rl_fragsrcverr);
  

  return;
}


/*-----------------------------------------------------------------
 * Name:                bsd_ClearIpStats
 *
 * Description: Clear IP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ClearIpStats(struct ipstat *ips)
{
  memset(ips, 0, sizeof(*ips));
  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ShowUdpStats
 *
 * Description: UDP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowUdpStats(struct udpstat *ups)
{
  uiPrintf("UDP packet statistics\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Tot Pkts Rcvd :  %-12d		Tot Pkts Sent :  %-12d\n",
		   ups->udps_ipackets, ups->udps_opackets);
  uiPrintf("Pkts Bad Len  :  %-12d		Pkts Bad Hdr  :  %-12d\n",
		   ups->udps_badlen, ups->udps_hdrops);
  uiPrintf("Pkts Bad Csum :  %-12d		Pkts Sock Full:  %-12d\n",
		   ups->udps_badsum, ups->udps_fullsock);
  uiPrintf("Pkts No UPort :  %-12d		Pkts No BPort :  %-12d\n",
		   ups->udps_noport, ups->udps_noportbcast);

  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ClearUdpStats
 *
 * Description: Clear UDP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ClearUdpStats(struct udpstat *ups)
{
  memset(ups, 0, sizeof(*ups));
  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ShowTcpStats
 *
 * Description: TCP connection and  packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowTcpStats(struct tcpstat *tps)
{
  uiPrintf("TCP connection and packet statistics\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Conn. Attempt :  %-12d		Conn. Accept  :  %-12d\n",
		   tps->tcps_connattempt, tps->tcps_accepts);
  uiPrintf("Conn. Estab   :  %-12d		Conn. Dropped :  %-12d\n",
		   tps->tcps_connects, tps->tcps_drops);
  uiPrintf("Conn. Em  Drop:  %-12d		Conn. Closed  :  %-12d\n",
		   tps->tcps_conndrops, tps->tcps_closed);
  uiPrintf("Conn. Rex Drop:  %-12d		Conn. Kp Drop :  %-12d\n",
		   tps->tcps_timeoutdrop, tps->tcps_keepdrops);
  uiPrintf("Conn. Per Drop:  %-12d		Conn. Lis Drop:  %-12d\n",
		   tps->tcps_persistdrop, tps->tcps_listendrop);
  uiPrintf("Rtt Timed     :  %-12d		Rtt Updated   :  %-12d\n",
		   tps->tcps_segstimed, tps->tcps_rttupdated);
  uiPrintf("Delay Ack Sent:  %-12d		Rex Timeout   :  %-12d\n",
		   tps->tcps_delack, tps->tcps_rexmttimeo);
  uiPrintf("Perst Timeout :  %-12d		Keep Timeout  :  %-12d\n",
		   tps->tcps_persisttimeo, tps->tcps_keeptimeo);
  uiPrintf("Keep Sent     :  %-12d\n",
		   tps->tcps_keepprobe);


  uiPrintf("\n");
  uiPrintf("Tot Pkts Sent :  %-12u		Tot Dgram Sent:  %-12u\n",
		   tps->tcps_sndtotal, tps->tcps_sndpack);
  uiPrintf("Tot Byte Sent :  %-12u		Uge Pkt Sent  :  %-12u\n",
		   tps->tcps_sndbyte, tps->tcps_sndurg);		
  uiPrintf("Rex Pkts      :  %-12u		Rex Bytes     :  %-12u\n",
		   tps->tcps_sndrexmitpack, tps->tcps_sndrexmitbyte);
  uiPrintf("Ack Only Sent :  %-12u		Win Prob Sent :  %-12u\n",
		   tps->tcps_sndacks, tps->tcps_sndprobe);
  uiPrintf("Win Udt Sent  :  %-12u		Contr Sent    :  %-12u\n",
		   tps->tcps_sndwinup, tps->tcps_sndctrl);
		   
  uiPrintf("\n");
  uiPrintf("Tot Pkts Rcvd :  %-12u		Pkts Rcvd Ord :  %-12u\n",
		   tps->tcps_rcvtotal, tps->tcps_rcvpack);
  uiPrintf("Bytes Rcvd Ord:  %-12u		Pkts Too Short:  %-12u\n",
		   tps->tcps_rcvbyte, tps->tcps_rcvshort);
  uiPrintf("Pkts Bad Csum :  %-12u		Pkts Bad Off  :  %-12u\n",
		   tps->tcps_rcvbadsum, tps->tcps_rcvbadoff);
  uiPrintf("Pkts Dup      :  %-12u		Bytes Dup     :  %-12u\n",
		   tps->tcps_rcvduppack, tps->tcps_rcvdupbyte);
  uiPrintf("Pkts Part Dup :  %-12u		Bytes Part Dup:  %-12u\n",
		   tps->tcps_rcvpartduppack, tps->tcps_rcvpartdupbyte);
  uiPrintf("Pkts Out Ord  :  %-12u		Bytes Out Ord :  %-12u\n",
		   tps->tcps_rcvoopack, tps->tcps_rcvoobyte);
  uiPrintf("Pkts Aft Win  :  %-12u		Bytes Aft Win :  %-12u\n",
		   tps->tcps_rcvpackafterwin, tps->tcps_rcvbyteafterwin);
  uiPrintf("Pkts Aft Close:  %-12u\n",
		   tps->tcps_rcvafterclose);
  uiPrintf("Win Prob Rcvd :  %-12u		Ack Dup Rcvd  :  %-12u\n",
		   tps->tcps_rcvwinprobe, tps->tcps_rcvdupack);
  uiPrintf("Ack Too Much  :  %-12u\n",
		   tps->tcps_rcvacktoomuch);
  uiPrintf("Ack Pkts Rcvd :  %-12u		Bytes Acked   :  %-12u\n",
		   tps->tcps_rcvackpack, tps->tcps_rcvackbyte);
  
  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ClearTcpStats
 *
 * Description: Clear TCP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ClearTcpStats(struct tcpstat *tps)
{
  memset(tps, 0, sizeof(*tps));
  return;
}

static char *icmp_code[] = {
  "EchoReply",
  0,
  0,
  "Unreach",
  "SrcQuench",
  "Redirect",
  0,
  0,
  "Echo",
  "RtAdver",
  "RtSolic",
  "Timxeed",
  "BadParam",
  "Tstamp",
  "TstampRply",
  "InfoReq",
  "InfoReply",
  "MaskReq",
  "MaskReply"
};
  
/*-----------------------------------------------------------------
 * Name:                bsd_ShowIcmpStats
 *
 * Description: ICMP  packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowIcmpStats(struct icmpstat *cps)
{
  int i;

  uiPrintf("ICMP Packets Statistics\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Icmp Err      :  %-12u		Icmp Reflects :  %-12u\n",
		   cps->icps_error, cps->icps_reflect);
  uiPrintf("Icmp Old Icmp :  %-12u		Icmp Bad Code :  %-12u\n",
		   cps->icps_oldicmp, cps->icps_badcode);
  uiPrintf("Icmp Too Short:  %-12u		Icmp Bad Csum :  %-12u\n",
		   cps->icps_tooshort, cps->icps_checksum);
  uiPrintf("Icmp Bad Len  :  %-12u		Icmp Bcast Ech:  %-12u\n",
		   cps->icps_badlen, cps->icps_bmcastecho);
  uiPrintf("Icmp Bcast Tst:  %-12u\n",
		   cps->icps_bmcasttstamp);

  uiPrintf("\n");
  
  for (i = 0; i < ICMP_MAXTYPE; i++)
	{
	  if (icmp_code[i] == 0)
		continue;

	  uiPrintf("In %-11s:  %-12u		out %-10s:  %-12u\n",
			   icmp_code[i], cps->icps_inhist[i], 
			   icmp_code[i], cps->icps_outhist[i]
			   );
	}
  
  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ClearIcmpStats
 *
 * Description: Clear ICMP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ClearIcmpStats(struct icmpstat *cps)
{
  memset(cps, 0, sizeof(*cps));
  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ShowArpStats
 *
 * Description: Arp  packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowArpStats(struct arpstat *aps)
{

  uiPrintf("ARP packets statistics\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Rcvd Request  :  %-12u		Snd Request   :  %-12u\n",
		   aps->rcvreq, aps->sndreq);
  uiPrintf("Rcvd Reply    :  %-12u		Snd Reply     :  %-12u\n",
		   aps->rcvreply, aps->sndreply);

  return;
}

/*-----------------------------------------------------------------
 * Name:                bsd_ClearArpStats
 *
 * Description: Clear ARP packet stats.
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ClearArpStats(struct arpstat *aps)
{
  memset(aps, 0, sizeof(*aps));
  return;
}

extern int32 ifa_allocated;
extern int32 ifa_freed;
extern int32 ifa_trash;
extern int32 rt_allocated;
extern int32 rt_freed;
extern int32 rttrash;
extern int32 pcb_allocated;
extern int32 pcb_freed;
extern int32 tcb_allocated;
extern int32 tcb_freed;
extern int32 so_allocated;
extern int32 so_freed;
/*-----------------------------------------------------------------
 * Name:                bsd_ShowMemStats
 *
 * Description: Show mem alloc stats for structures..
 *
 * Created by : Su Wang
 *
 * Date Created: 08/28/01
 *-----------------------------------------------------------------
 */
void
bsd_ShowMemStats()
{
  uiPrintf("BSD Memory Allocation Summary\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Struct		alloc	  		free			trash\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("IP addr     %-16d%-16d%d\n",
		   ifa_allocated, ifa_freed, ifa_trash);
  uiPrintf("IP Route    %-16d%-16d%d\n",
		   rt_allocated, rt_freed, rttrash);
  uiPrintf("PCB         %-16d%-16dN/A\n",
		   pcb_allocated, pcb_freed);
  uiPrintf("TCB         %-16d%-16dN/A\n",
		   tcb_allocated, tcb_freed);
  uiPrintf("Socket      %-16d%-16dN/A\n",
		   so_allocated, so_freed);

  uiPrintf("\n\n Fastpath Reassembly  Memory Allocation Summary\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Struct             alloc         free\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("srcq               %-16d%-16d\n", 
		   luc_memstat.src_alloc, luc_memstat.src_free);
  uiPrintf("reasq              %-16d%-16d\n", 
		   luc_memstat.reas_alloc, luc_memstat.reas_free);
  uiPrintf("frag               %-16d%-16d\n", 
		   luc_memstat.frag_alloc, luc_memstat.frag_free);
  uiPrintf("force/timeout free %-16d%-16d\n", 
		   luc_memstat.force_free, luc_memstat.timeout_free);
  uiPrintf("alloc/fatal err    %-16d%-16d\n",
		   luc_memstat.alloc_err,luc_memstat.fatal);

#if !defined(PROM)
  uiPrintf("\n\n ARP Sniffing Stats\n");
  uiPrintf("------------------------------------------------------------\n");
  uiPrintf("Ent. Alloc         : %u\n", arpsCfgPtr->stat.alloc);
  uiPrintf("Ent. Dealloc       : %u\n", arpsCfgPtr->stat.dealloc);
  uiPrintf("Ent. Alloc Fail    : %u\n", arpsCfgPtr->stat.allocFail);
#endif

#if defined(LINK_AGG)
  printf("\n\n Virtual Interface Stats\n");
  printf("------------------------------------------------------------\n");
  printf("Vif Configured     :    %d\n",		VifStats->vif_cnt);
  printf("Active LInk changes:    %d\n",		VifStats->vif_actLinkChange);
  printf("Up LInk changes    :    %d\n",		VifStats->vif_upLinkChange);
  
  printf("tmp alloc/free/err :    %d/%d/%d\n",	VifStats->vif_alloc,
		 VifStats->vif_dealloc, VifStats->vif_allocErr);
  printf("Vif add/del err    :    %d/%d\n",		VifStats->vif_addErr,
		 VifStats->vif_delErr);
  printf("VifLink add/del err:    %d/%d\n",		VifStats->vif_addLinkErr,
		 VifStats->vif_delLinkErr);
  printf("VifIP add/del err  :    %d/%d\n",		VifStats->vif_addIpErr,
		 VifStats->vif_delIpErr);
  printf("Vif pref err       :    %d\n",		VifStats->vif_prefErr);
#endif /* LINK_AGG */

  return;
}
