#include <nfx-incl.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/macros.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/domain.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"
#include "../sm-bsd-sys/syslog.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/route.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/in_pcb.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/tcp_fsm.h"
#include "../sm-bsd-netinet/tcp_seq.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "../sm-bsd-netinet/tcpip.h"
#include "../sm-bsd-netinet/tcp_debug.h"
#include "../sm-bsd-api/bsdrelay.h"
#include "exports/bsdsock-api.h"
#include "exports/bsdnet-api.h"
#include "exports/tpl-api.h"




#define MAX_TCP_TEST   0xffff

void *loopConn1 = 0;
void *loopConn2 = 0;
extern struct ifnet		loif[];
static int				cont=0;
static uint32			pkt_cnt;
//static int				drop = 1;
//static int				change_ack = 1;



extern q_headNodePtr_t tst_pkt_queue;

void bsd_RcvTcpPkt(pkt_desc_t *pdesc, void *ifp,
				   void	*parm1, void *parm2)
{
  struct tcpiphdr *ti;
  pkt_desc_t	  *mydesc;  
  int			  ret = TPL_HOLD_PKT;		
  int			  chge_ack_now = 0;

  /* drop pkt to test retransmission */
  /*  if ((*pdesc->pHdr == 2) && drop)
	{
	  printf("pkt %u from conn %lx dropped\n", *pdesc->pHdr, ifp);
	  drop = 0;
	  return; 
	}
  */



  /* need to allocate new desc since input pdesc */
  /* belongs to the sender */
  if (!(mydesc = PKT_ALLOC_PACKET(PKT_USR_PKT)))
	{
	  panic("bsd_RcvTcpPkt: no pkt allocated \n");
	  return;
	}

  mydesc->pHdr += PKT_HDR_INDENT;

  /* set new IP hdr ptr */
  mydesc->pIpHdr = mydesc->pHdr;

  mydesc->flags |= PKT_FAST_PATH;
  
  /* set msg size */
  mydesc->hdrSize = pdesc->hdrSize;
  mydesc->dataSize = pdesc->dataSize;

  /* copy ip pkt */
  memcpy(mydesc->pIpHdr, pdesc->pIpHdr, 
		 (mydesc->hdrSize+40));

  /* set new pHdr ptr */
  mydesc->pTplHdr = mydesc->pIpHdr + 20;
  mydesc->pHdr = mydesc->pIpHdr+40;
  

  /* hdrSize only indicates the data porttion */ 
  /* need to expand to cover all hdrs */

  /* if pkt not from a TPL queue, release it */
  if (!(pdesc->flags & PKT_USR_TCP_TX_Q))
	{

	  /* change ack num to ack partially pkt */
	  /*  
		if (change_ack)
		{
		  chge_ack_now = 1;
		  change_ack = 0;
		}
	  */
	  pdesc->flags |= PKT_USR_TPL;
	  TPL_FREE_PACKET(ifp, pdesc);
	}
  
  
  ti = (struct tcpiphdr*)mydesc->pIpHdr;
  if ( ti->ti_flags & (TH_RST | TH_FIN))
	/* termination go through slowpath */
	bsd_RcvIpPkt(mydesc, loif, 0, 0);
  else
	{
	  /* change seq to emulate over lapping */
	  /* offset by 36 */
	  /*	  if (*mydesc->pHdr)
		{
		  ti->ti_seq = NTOHL(ti->ti_seq) - 0x24;
		  HTONL(ti->ti_seq);
		  ((struct ip *)ti)->ip_len = 
			NTOHS(((struct ip *)ti)->ip_len) + 0x24;
		  HTONS(((struct ip *)ti)->ip_len);
		  mydesc->hdrSize += 0x24;
		  *(mydesc->pHdr + 0x24) = *mydesc->pHdr;
		}
	  */
	  
	  /* change ack num to ack partially pkt */
	  if (chge_ack_now)
		{
		  ti->ti_ack = NTOHL(ti->ti_ack) - 0x40;
		  printf("ack change from %x to %x\n", ti->ti_ack+0x40,
				 ti->ti_ack);
		  NTOHL(ti->ti_ack);
		} 
	  if (ifp == loopConn1)
		ret = tpl_rcvPkt(loopConn2, mydesc);
	  else
		ret = tpl_rcvPkt(loopConn1, mydesc);
	}

  if (ret == TPL_FREE_PKT)
	PKT_FREE_PACKET(mydesc);

  return;
}

int32 fast_addconn(tpl_bindCb_t *pBind, void *pConnId, 
				void **ppAppHdl, uint32 tmp1, ushort16 tmp2)
{
  printf("connection %lx set\n", (address_t)pConnId);
  if (!loopConn1)
	loopConn1 = pConnId;
  else
	loopConn2 = pConnId;

  *ppAppHdl = pConnId;
  return NFX_OK;
}

int32 fast_delconn(void *pAppHdl, void *tmp)
{
  printf("connection %lx closed \n", (address_t)pAppHdl);
  return NFX_OK; 
}

/* data format: 1 byte non zero: data pkt seq num */
/*				1 byte zero:     ack pkt */
/*				2 byte :		 seq num acked */

int32 fast_rcv(void *pAppHdl, pkt_desc_t *pDesc)
{
  pkt_desc_t *pp, *np;


again:
  np = pDesc->pNext;
  pDesc->pNext = 0;
  
  if (cont)
	{
	  if (*pDesc->pHdr)			/* data pkt */
		{
		  printf("pkt rcved at contineous conn %lx, size %d(%u) \n", 
				 (address_t)pAppHdl, pDesc->hdrSize+pDesc->dataSize, 
				 (u_char)*pDesc->pHdr);		  
		  if ((u_char)(*pDesc->pHdr) % 3 == 0)
			{
			  /* send ack bck */
			  pp = TPL_ALLOC_PACKET(pAppHdl, 0);
			  pp->hdrSize = 2;
			  pp->dataSize = 0;
			  *(pp->pHdr) = 0;
			  *(pp->pHdr +1) = *pDesc->pHdr;
			  printf("ack send at contineous conn %lx, ack %u\n",(address_t)pAppHdl,
					 (u_char)*(pp->pHdr+1));
			  tpl_xmtPkt(pAppHdl, pp);
			}
		}
	  else						/* ack */
		{
		  printf("ack rcved at contineous conn %lx, ack %u \n", 
				 (address_t)pAppHdl, (u_char)*(pDesc->pHdr+1));		  
		  if ((u_char)*(pDesc->pHdr+1) != pkt_cnt%256)
			panic("bsd tcp test: pkt %d ack not matching\n", pkt_cnt);

		  /* ready to send to next three pkt */
		  if (pkt_cnt < MAX_TCP_TEST)
			{
			  int num =3;
			  while (num--)
				{
				  if (!(pp = TPL_ALLOC_PACKET(pAppHdl, 0)))
					{
					  printf("tpl %lx pkt pool depleted\n", (address_t)pAppHdl);
					  return NFX_OK;
					}

				  if (!pp->pIpHdr)
					panic("tcp test send: hdr not set correctly\n"); 

				  pkt_cnt++;
				  /* still mod 0 to distinguish from test ack */
				  if (pkt_cnt%256 == 0)
					pkt_cnt++;
				  pp->hdrSize  = 20;
				  pp->dataSize = 600;
				  *(pp->pHdr)  = pkt_cnt%256;
	  
				  printf("conn %lx send continues %d data(%d, %u)\n",
						 (address_t)pAppHdl, pp->dataSize + 20, pkt_cnt, 
						 (u_char)*(pp->pHdr));
				  tpl_xmtPkt(pAppHdl, pp);
				}
			}
		}
	}
  else
	printf("pkt rcved at burst conn %lx, size %d(%d) \n", (address_t)pAppHdl,
		   pDesc->hdrSize+pDesc->dataSize, *pDesc->pHdr);

  PKT_FREE_PACKET(pDesc);

  if ((pDesc = np))
	goto again; 

  /* chk if the pkt is ack */
    return TPL_HOLD_PKT;
}

int32 fast_send_1(void* pConn, int num)
{
  pkt_desc_t* pdesc;

  pkt_cnt = 0;

  if (num)
	cont = 0;
  else
	  cont = 1;


  /* two send mode: contineous, or single burst */
  /* cont mode, send 3 pkt at time, */
  /* snd more only if acked */
  if (cont) 
	num =3;
	  
  while (num--)
	{
	  if (!(pdesc = TPL_ALLOC_PACKET(pConn, 0)))
		return NFX_OK;

	  pkt_cnt++;
	  pdesc->hdrSize  = 20;
	  pdesc->dataSize = 600;
	  *(pdesc->pHdr)  = pkt_cnt%256;

	  
	  printf("conn %lx send burst %d data(%d)\n", 
			 (address_t)pConn, pdesc->dataSize + 20, pkt_cnt);
	  tpl_xmtPkt(pConn, pdesc);
	}

  return 0;
}

int32 fast_xmtCmp(void *pAppHdl, pkt_desc_t *pDesc)
{
  return NFX_OK;
}


int  bsd_IsTcpLoopBck(void *pdesc, void *pConn)
{
  	/* bck to loopbck intf */
  if (pConn == loopConn1 || pConn == loopConn2)
	{
	  q_enQueue(tst_pkt_queue, (void *)pdesc, (void *)pConn);
	  return 1;
	}
  else
	return 0;
}

void bsd_SetupFastTcp(int type, uint32 ip, ushort16 port )
{

  tpl_bindCb_t tpl;
  tpl_bindCb_t *tmp;
  int           mode;
  
  bzero(&tpl, sizeof(tpl));
  if (type == BSD_SO_SERVER)
	{
	  tpl.laddr = (ip);
	  tpl.lport = (port);
	  mode = 0;
	}
  else
	{
	  tpl.lport = htons(6000);
	  tpl.laddr = (ip);
	  tpl.raddr = (ip);
	  tpl.rport = (port);
	  mode = 1;	  
	}
  tpl.tplProt = BSD_IPPROTO_TCP;
  tpl.ul_addConnInd = fast_addconn;
  tpl.ul_delConnInd = fast_delconn;
  tpl.ul_rcvPktInd = fast_rcv;
  tpl.ul_xmtCmpInd = fast_xmtCmp;

  tpl_bindReq(&tpl, &tmp, mode);
  return;
}
