/* vi:ts=4  
 *-----------------------------------------------------------------
 *
 * Name: bsd-util.c
 *
 * RCS:  $Id: bsd-util.c,v 1.22 2002/10/30 22:18:28 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: bsd utilities. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/28/71
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <nfx-ports.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/macros.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"
#include "../sm-bsd-sys/sockio.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/syslog.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/route.h"
#include "../sm-bsd-net/if_dl.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/in_pcb.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/tcp_fsm.h"
#include "../sm-bsd-netinet/tcp_seq.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "../sm-bsd-netinet/tcpip.h"

#include "exports/libc.h"
#include "exports/bsdnet-api.h"
#include "exports/bsdsock-api.h"
#include "exports/ipm-api.h"
#include "exports/cm-api.h"
#include "../sm-bsd-api/bsdrelay.h"
#include "exports/luc-api.h"
#include "exports/pkt-api.h"
#include "bsd-vif.h"

char prnt_mac[64];
char prnt_ip[64];
char prnt_dev[64];

// extern char8* strchr (char8*,	char8);

#if defined(PROM)
/*
 *-----------------------------------------------------------------
 *
 * NAME : bsd_ShowPkt
 *
 * Description: Display a packet  
 *
 * Created by: Su Wang
 *
 * Data Created: 02/13/01
 *
 *-----------------------------------------------------------------
 */
void
bsd_ShowPkt(char* pkt, int len)
{

  int	 new_line = 1;
  uint32 count = 0;

  if (!pkt || !len)
    return;

  printf("\n");
  printf("Print packet size %d\n", len);
  while (len > count)
    {
      if (new_line)
	{
	  printf("0x%08X:       ", count);
	  new_line = 0;
	}
      
      printf("%02X ", (*pkt++) & 0xff);
      count++;
      
      if (count % 16 == 0)
	{
	  printf("\n");
	  new_line = 1;
	}
    }

  printf("\n\n");
  return;

}
#endif /* PROM */

/*-----------------------------------------------------------------
 * Name: bsd_adjBuffer
 *
 * Description: adjust hdr buf ptr of a pkt descriptor that 
 *				associated with a mbuf.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_AdjBuffer(struct mbuf *m, 
			  int offset		/* + forward, - bckward */
			  )
{
  pkt_desc_t *pdesc;

  if ( m == 0 || (m->m_flags & M_PKTHDR) == 0 ||
	   m->m_pktdesc == 0)
	{
	  debugPrintf(("bsd_AdjBuffer: invalid mbuf\n")); 
	  return NFX_ERR;
	}

  pdesc = (pkt_desc_t*)m->m_pktdesc;
  return (pkt_AdjPkt(pdesc, offset));
}

/*-----------------------------------------------------------------
 * Name: bsd_ModBuffer
 *
 * Description: modify header buffer a pkt descriptor that 
 *				associated with a mbuf.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsd_ModBuffer(struct mbuf *m,
			  uint32	  where,/* where in buffer */
			  char		  *ptr,	/* content */
			  uint32	  size
			  )
{
  pkt_desc_t *pdesc;
  address_t  start;
 
  if ( m == 0 || (m->m_flags & M_PKTHDR) == 0 ||
	   m->m_pktdesc == 0 || ptr == 0 || size == 0)
	return NFX_ERR;  
  
  pdesc = (pkt_desc_t*)m->m_pktdesc;
  
  /* make sure range is valid */
  start = (address_t)pdesc->pHdr;
  if (((start & ~(PKT_BUFF_SIZE-1))!=((start + where) & ~(PKT_BUFF_SIZE-1))) ||
	  ((start & ~(PKT_BUFF_SIZE-1))!=((start+where+size)& ~(PKT_BUFF_SIZE-1))))
	return NFX_ERR;

  memcpy((pdesc->pHdr + where), ptr, size);
  return NFX_OK;
}
			  

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32 bsdMasklen(void* in_data)
{
  uint32 data = *(uint32*)in_data;
  int i;


  data = ntohl(data);
  for (i = 0 ; i<32 ; i++)
    {
      if (data & 1)
	return (32-i);
      else
	data >>= 1;
    }

  return 0;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32 bsd_IsSupernet(uint32 net, uint32 in_mask)
{
  uint32 mask;

  if (IN_CLASSA(net))
    mask = IN_CLASSA_NET;
  else if (IN_CLASSB(net))
    mask = IN_CLASSB_NET;
  else if (IN_CLASSC(net))
    mask = IN_CLASSC_NET;
  else
    return -1;  /* bad net */
  
  /* supernet is true, if assigned netmask is shorter
     then classful mask */
  return ((in_mask >= mask) ? 0:1);
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert to ifp number to string.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char*  bsdPrintIntf(struct ifnet* ifp)
{
  if (ifp)
    sprintf(prnt_dev, "%s:%d:%d:%d", 
			ifp->if_name,
			ifp->if_slot,
			ifp->if_cpu,
			ifp->if_unit);
  else
    *prnt_dev='\0';

  return prnt_dev;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert to ifp number to string, with assoc. with
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char*  bsdPrintIntfWithVif(struct ifnet* ifp)
{
	char	*vifName = NULL;

#if defined(LINK_AGG)
	if (ifp->if_vif)
		vifName = ((bsd_vif_t*)ifp->if_vif)->bsd_vifName;
#endif /* LINK_AGG */
	
  if (ifp) 
    sprintf(prnt_dev, "%s:%d:%d:%d(%s)", 
			ifp->if_name,
			ifp->if_slot,
			ifp->if_cpu,
			ifp->if_unit,
			vifName?vifName:"NULL");
  else
    *prnt_dev='\0';

  return prnt_dev;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert IP address string to numbers, network order
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32 bsdIpStoN(char8* str, ipa_ptr_t addr)
{
  char *np = (char*)addr, *qp = str;
  uint32 tmp = 0;
  int i = 4;

  /* scan through any invalid characters */
  while (*qp != '\0')
    {
      if (((*qp < '0') || (*qp > '9')) &&
		  (*qp != '.'))
		{
		  uiPrintf("invalid IP address format 1 \n");
		  return NFX_ERR;
		}

      qp++;
    }

  while (i)
    {
      qp = (char*)strchr(str, '.');
      
      if (((!qp) && (i > 1))   ||   /* first three byte, have '.' */ 
		  (qp && (i == 1)))         /* last byte, through all '.' */
		{
		  uiPrintf("invalid IP address format 2\n");
		  return NFX_ERR;	  
		}
      
      /* atoi does not work properly otherwise */
      if (qp)
		*qp = '\0';

      tmp = atoi(str) & 0xff;
      
      if (qp)
		*qp = '.';

      *np++ = tmp;
      str = qp + 1;
      i--;
    }

  return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert IP address numbers to string, network order
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
bsdIpNtoS(ipa_ptr_t addr, char* str, int str_sz)
{
  char* np = (char*)addr;

  if (str_sz <= 16)
    return (NFX_ERR);

  sprintf(str, "%u.%u.%u.%u", 0xff&np[0], 0xff&np[1], 0xff&np[2], 0xff&np[3]);
  return NFX_OK;
  
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char* 
bsdPrintIpa(ipa_t addr)
{
  bsdIpNtoS(&addr, prnt_ip, sizeof(prnt_ip));
  return prnt_ip;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert MAC address string to numbers, network order
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
bsdMacStoN(char* str, char* mac)
{
  char* np = mac;
  char* qp = str;
  int tmp;
  int i = 6;
  

  /* scan through any invalid characters */
  while (*qp != '\0')
    {
      if (!((*qp >= '0' && *qp <= '9') ||
			(*qp >= 'a' && *qp <= 'f') ||
			(*qp >= 'A' && *qp <= 'F') ||
			(*qp == '-')))
		{
		  uiPrintf("invalid MAC address format 1\n");
		  return NFX_ERR;
		}
      qp++;
    }
  
  while (i)
    {
      qp = (char*) strchr(str, '-');
      
      if (( !qp && (i > 1))   ||   /* first five byte, have '-' */ 
		  (qp && (i == 1)))         /* last byte, through all '-' */
		{
		  uiPrintf("invalid MAC address format 2\n");
		  return NFX_ERR;	  
		}

      if (qp)
		*qp = '\0';

      tmp = atox(str) & 0xff;

      if (qp)
		*qp = '-';

      *np++ = tmp;
      str = qp + 1;
      i--;
    }

  return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert MAC address numbers to string, network order
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
bsdMacNtoS(char* addr, char* str, int str_sz)
{
  char* np = (char*)addr;

  if (str_sz <= 18)
    return (NFX_ERR);

  sprintf(str, "%02X-%02X-%02X-%02X-%02X-%02X", 0xff&np[0], 0xff&np[1], 
	  0xff&np[2], 0xff&np[3], 0xff&np[4], 0xff&np[5]);
  return NFX_OK;
  
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char* 
bsdPrintMac(char* mac)
{
  bsdMacNtoS(mac, prnt_mac, sizeof(prnt_mac));
  return prnt_mac;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert a local port string to a global port string
 *				if src already global, simply copy
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#ifdef xxxx
void bsdPortToString(char* dest, const char* src)
{
  int i = IFNAMSIZ;
  
  while (i && (*src != '\0'))
    {
      *dest++ = *src++;
      i--;
    }

  *dest='\0';

  return;
}
#else
void bsdPortToString(char* dest, char* src)
{
  int			i = IFNAMSIZ;
  char			dtype[IFNAMSIZ];
  int			link;
  char			*tc;
  int			glob = 0;

  /* is format global */
  tc = src;
  while (*tc != '\0')
	{
	  if (*tc == ':')
		{
		  glob = 1;
		  break;
		}
	  tc++;
	}

  if (!glob)
	{
	  char		*t_ptr = dtype;
	  /* copy device type */
	  while (i && (*src < '0' || *src > '9'))
		{
		  *t_ptr++ = *src++;
		  i--;
		}
	  *t_ptr='\0';
	  
	  
	  /* copy link */
	  for (link = 0; (*src >= '0') && (*src <= '9'); src++)
		link = (link*10)+(*src - '0');
	  
	  /* if src not finished bad format */
	  if (*src != '\0')
		*dest = '\0';
	  else
		sprintf(dest, "%s:%d:%d:%d", dtype, 
				nfxMySlotId, nfxMyCPUId, link);
	}
  else
	{
	  strcpy(dest, src);
	}
  return;
}
#endif /* xxxx */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert input to string format and stored in dest
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void bsdPortToString2(int		slot, 
					  int		cpu,
					  int		link, 
					  int		dtype, 
					  char		*dest,
					  int		size)
{
  
  /* sanity check */
  if ((slot < 0 || slot > NFX_SLOT_MAX) ||
	  (cpu <0 || cpu > NFX_NFP_NUM_CPUS)|| 
	  (link < 0 || link > IPM_LINK_MAX) ||
	  (dtype < IPM_INTF_TYPE_MIN || dtype > IPM_INTF_TYPE_MAX))
	panic("bsdPortToString2: invalid port, slot %d, link %d, type %d\n",
		  slot, link, dtype);


  sprintf(dest, "%s:%d:%d:%d", bsd_ifnamebuf[dtype], slot, cpu, link);

  return;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char* 
bsdPrintPort(int slot, 
			 int cpu,
			 int link, 
			 int dtype)
{
  bsdPortToString2(slot, cpu, link, dtype, prnt_dev, sizeof(prnt_dev));
  return prnt_dev;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert port name from ssc.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
char*
bsdPrintPortFromSSC(char	*port)
{
	char		node[32];
	boardType_t	board;
	int			slot;
	int			link;
	
	getPortInfo(port, node, &board, &slot, &link);
	return bsdPrintPort(slot, nfxMyCPUId, link, board);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: convert type string to value
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int
bsdDTypeStoN(char *name)
{
  char	**t_name;
  int	i;

  for (t_name = bsd_ifnamebuf, i=0; i <= IPM_INTF_TYPE_MAX; i++, t_name++)
	{
	  if (strcmp(*t_name, name) == 0)
		return i;
	}

  /* nothing found, return 0 reflect for "dummy" */
  return 0;
}

/*
 *-----------------------------------------------------------------
 *
 * Name : bsd_ChkSum
 *
 * Description: chksum for flat bufer  
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *-----------------------------------------------------------------
 */
ushort16
bsd_ChkSum(ushort16 *ptr, int len)
{
  int nleft = len;
  int sum = 0;

  // debugPrintf(("chksum len %d\n", len));
  while (nleft > 1)
    {
      sum += *ptr++;
      nleft -= 2;
    }
    
  if (nleft == 1)
    sum += (ushort16)*((char*)ptr);

  // debugPrintf(("raw chksum val %lx\n", sum));
  sum = ((sum >> 16) & 0xffff) + (sum & 0xffff);
  sum += (sum >> 16) & 0xffff;
  
  //  debugPrintf(("chksum val %lx\n", ~sum & 0xffff));
  return (~sum & 0xffff);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: The following routine returns corresponding field 
 *				pcb block. All port and IP/MAC address returned in
 *				network order.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
ipa_t							/* local address in PCB */
bsd_GetPcbLaddr(void *inp)
{
  return ((struct inpcb*)inp)->inp_laddr.s_addr;
}

ipa_t							/* remote address in PCB */
bsd_GetPcbFaddr(void *inp)
{
  return ((struct inpcb*)inp)->inp_faddr.s_addr;
}

ushort16						/* local port addr in PCB */
bsd_GetPcbLport(void *inp)
{
  return ((struct inpcb*)inp)->inp_lport;
}

ushort16						/* remote port addr in PCB */
bsd_GetPcbFport(void *inp)	
{
  return ((struct inpcb*)inp)->inp_fport;
}

void*							/* ptr to corresponding socket */
bsd_GetPcbSo(void *inp)	
{
  return (void*)((struct inpcb*)inp)->inp_socket;
}

ipa_t							/* nexthop IP in PCB */
bsd_GetPcbNHIp(void *inp)
{
  return ((struct inpcb*)inp)->inp_nhip;  
}

char*							/* ptr to nexthop mac in PCB */
bsd_GetPcbNHMac(void *inp)
{
  return ((struct inpcb*)inp)->inp_nhmac;  
}

void
bsd_SetPcbNHIp(void *inp, uint32 ip)
{
	((struct inpcb*)inp)->inp_nhip = ip;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description: The following routine returns corresponding field 
 *				of socket block. All port and IP/MAC address returned in
 *				network order.
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void*
bsd_GetSoPcb(void *so)	
{
  return ((struct socket*)so)->so_pcb;
}

/*
 * is TCP time stamp option set 
 */
int32
bsd_IsTcpTimeStamp(void *ptr)
{
	struct tcpcb	*tcb = (struct tcpcb*)ptr;

	if (tcb &&
		(tcb->t_flags & TF_RCVD_TSTMP) &&
		(tcb->t_flags & TF_REQ_TSTMP))
		return 1;
	else
		return 0;
}


static char8	log_buf[256];

/*-----------------------------------------------------------------
 * Name:                bsd_log_f
 *
 * Description: redirect bsd log messages to bsd_logErr.
 *
 * Created by : 
 *
 * Date Created: 
 *-----------------------------------------------------------------
 */
void 
bsd_log_f(int32 level, char8 *fmt,
		...
		)
{
  va_list		ap;
  int32			len;  
  int			panicFlag = 0;

  va_start(ap, fmt);
  
  len = vsprintf(log_buf, fmt, ap);
  
  va_end(ap);  

  if (level == BSD_ERR_GEN_PANIC)
	  panicFlag = 1;
  
  /* convert errcode to level */
  if (level < 0) {
	  level -= BSD_ERROR_BASE;
	  level /= 10;
  }

#if defined(PROM)
  printf("%s\n", log_buf);
#else
  E_LOG(class_1, level, bsdrl_appid, 0, 0, 0, (log_buf));
#endif /* PROM */
  
  if (panicFlag)
	  BSD_PANIC( );
  
  return;
}

/*-----------------------------------------------------------------
 * Name:      bsd_GetFreePort()  
 *
 * Description: get a free local port
 *
 * Created by :  Su Wang
 * 
 * Date Created: 02/25/2002 
 *-----------------------------------------------------------------
 */
ushort16
bsd_GetFreePort(ushort proto)
{
	bsd_sarg_t		sockArg;
	struct socket   *so=0;
	uint32			port;

	bzero(&sockArg, sizeof(bsd_sarg_t));
	sockArg.bufType = BSD_BUF_DESCR;

	/* open a socket to get a dynamic port */
	so = (struct socket*)bsd_OpenSock(proto, BSD_SO_SERVER, &sockArg);
	if (!so) {
		bsd_log(BSD_ERR_SOCK, "bsd_GetFreePort failed\n");
		return 0;
	}
	
	port = ((struct inpcb*)so->so_pcb)->inp_lport;

	/* close the socket */
	bsd_CloseSock(so);
	
	return port;
}


short16
bsd_getLinkId( struct inpcb *pcb)
{
#if defined(PROM)
	if (!pcb 							|| 
		!(pcb->inp_route.ro_rt) 		|| 
		!(pcb->inp_route.ro_rt->rt_ifp))
		return -1;

	return ( pcb->inp_route.ro_rt->rt_ifp->if_unit );
#else
	return (pcb->inp_link >= lucMaxLinkNum || pcb->inp_link < 0) ? 
		-1 : pcb->inp_link;
#endif
}

int32
bsd_setLinkId( void *in_pcb, int link)
{
	struct inpcb	*pcb = (struct inpcb*)in_pcb;
	
	if (link != -1) {
		pcb->inp_link = link;
		return NFX_OK;
	}
	
	if (!pcb 							|| 
		!(pcb->inp_route.ro_rt) 		|| 
		!(pcb->inp_route.ro_rt->rt_ifp)) {
		pcb->inp_link = -1;
		return NFX_ERR;
	}
	
	pcb->inp_link = pcb->inp_route.ro_rt->rt_ifp->if_unit;
	// printf("link id assigned %d to so %lx\n", 
	//	   pcb->inp_link, (address_t)pcb->inp_socket);
	return NFX_OK;
}
