/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: bsdrelay.h
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: contains IP pkt relay functions
 *
 * Created by:   Su Wang
 *
 * Date Created: 10/19/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _BSDRELAY_H
#define _BSDRELAY_H

#include "exports/eee-api.h"
#include "exports/pkt-api.h"

/* port range allocated to txrx */
#define BSD_PORT_TXRX_FIRST 48152	/* corresponding to port gap specified on ssc */	
#define BSD_PORT_TXRX_LAST  49151   /* last port before free port high at ssc */	


/* well known port processed locally on txrx, */
/* mostly used for server socket */
#define			BSD_WELLKNOWN_PORT_TFTP		69	
#define			BSD_WELLKNOWN_PORT_TFTP1    2086
#define			BSD_WELLKNOWN_PORT_NFS		2049
#define			BSD_WELLKNOWN_PORT_NFS1	    2088	
#define			BSD_WELLKNOWN_PORT_MNT		2087
#define			BSD_WELLKNOWN_PORT_NLM		2090
#define			BSD_WELLKNOWN_PORT_NLM_PMAP	2091
#define			BSD_WELLKNOWN_PORT_NSM		2092
#define			BSD_WELLKNOWN_PORT_NSM_PMAP	2093
#define			BSD_WELLKNOWN_PORT_NTTCP	5037	
#define         BSD_WELLKNOWN_PORT_DM       11000

#define			BSD_WELLKNOWN_PORTS(_port)				\
((_port == BSD_WELLKNOWN_PORT_NFS) ||					\
 (_port == BSD_WELLKNOWN_PORT_TFTP)||						\
 (_port == BSD_WELLKNOWN_PORT_TFTP1)||					\
 (_port == BSD_WELLKNOWN_PORT_NFS1)	||					\
 (_port == BSD_WELLKNOWN_PORT_NTTCP)	||					\
 (_port == BSD_WELLKNOWN_PORT_NLM)	||					\
 (_port == BSD_WELLKNOWN_PORT_NSM)	||					\
 (_port == BSD_WELLKNOWN_PORT_NLM_PMAP)	||				\
 (_port == BSD_WELLKNOWN_PORT_NSM_PMAP)	||				\
 (_port == BSD_WELLKNOWN_PORT_DM)	||					\
 (_port == BSD_WELLKNOWN_PORT_MNT))						

/* Su, 09/23/02
 * Normally all broadcast pkts are processed locally, the
 * following ports are exceptions and forwarded to SSC
 */
#define		BSD_BCAST_SSC_PORT_RPC			111
#define		BSD_BCAST_SSC_PORT_NTP			123

#define		BSD_BCAST_TO_SSC(_port)					\
((_port == BSD_BCAST_SSC_PORT_RPC) ||				\
 (_port == BSD_BCAST_SSC_PORT_NTP))

extern int bsdrl_appid;

typedef struct {				/* relay addr */
  ushort16		rl_slot;		/* dest slot, max+1, to all slot of cpu type */
  ushort16		rl_cpu;			/* dest cpu, max+1, to all cpu of slot  */
#define			BSD_RL_PKT_OWN			0x1
								/* pkt is generated within the chassis, directly */
								/* forwarded to interface, no need to run through bsd stack*/
#define			BSD_RL_PKT_BCAST		0x2
								/* pkt should be bcasted  */
#define			BSD_RL_PKT_MORE			0x4
								/* reach end of complete pkt, */
								/* flag always set for unfragmented, for last fragmented*/
  ushort16		rl_ctrl;
  void			*rl_ifp;		/* intf that received pkt */
} rl_addr_t; 

/* relay header */
typedef struct {
  ushort16		rh_dtype;		/* device type of intf */
  ushort16		rh_slot;		/* slot num of intf */
  ushort16		rh_cpu;			/* cpu num of intf */
  ushort16		rh_link;		/* link num of intf */
  ushort16		rh_ctrl;		/* ctrl flag for this pkt */
  ushort16		rh_offset;		/* fragment offset */
  ushort16		rh_pad;			/* keep 16 bits aligned */
} rl_hdr_t;

/* backplane reassemble queue */
typedef struct {
  uint32		offset;
  eee_descPtr_t edescp[EEE_MAX_BUFS]; /* assume one buf per descp */
  int			position;
  int			ttl;			/* time to live */
} rl_rq_t;

extern rl_rq_t bsdrl_ReassemQ[NFX_NUM_SLOTS_PER_CHASSIS];
void bsdrl_ClrReassemQ(rl_rq_t  *q);

/* maximum frame size to backplane */
#define BSD_BP_MTU		(ETHERMTU - sizeof(agile_hdr_t) - sizeof(rl_hdr_t))	



/* 
 *-----------------------------------------------------------------
 *
 * Name: bsdrl_Init
 *
 * Description: initialize relay layer.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/22/2001
 *
 *-----------------------------------------------------------------
 */
void
bsdrl_Init();


/*-----------------------------------------------------------------
 * Name:        bsdrl_DstPkt
 *
 * Description: determine the pkt destination, if pkt is to be 
 *				forwarded to another, the rl_adddr_t struct to 
 *				filled to specify destination.
 *
 * Created by : Su Wang
 *
 * Date Created: 10/19/01
 *-----------------------------------------------------------------
 */
#define			BSD_RELAY		1
#define			BSD_NRELAY		2
#define			BSD_RELAY_ERR	3
int32							/* BSD_RELAY, pkt relayed via eee */
								/* BSD_NREPLAY, pkt is for loc card*/
bsdrl_DstPkt(pkt_desc_t	*pdesc, 
			 rl_addr_t	*rla,
			 void		*parm);

/*-----------------------------------------------------------------
 * Name:        bsd_RelayPkt
 *
 * Description: relay an pkt to destination specified by rl_addr_t 
 *				struct..
 *
 * Created by : Su Wang
 *
 * Date Created: 10/19/01
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
bsdrl_RelayPkt(pkt_desc_t	*pdesc,
			   rl_addr_t	*rla,  
			   void			*parm);

/*-----------------------------------------------------------------
 * Name:        bsdrl_RcvPkt
 *
 * Description: rcv pkt at  back plane.  
 *
 *
 * Created by : Su Wang
 *
 * Date Created: 10/22/01
 *-----------------------------------------------------------------
 */
void
bsdrl_RcvPkt(eee_descPtr_t epdesc);

#endif /* _BSDRELAY_H */
