/* vi:ts=4  
 *-----------------------------------------------------------------
 *
 * Name: bsdsock-api.c
 *
 * RCS:  $Id: bsdsock-api.c,v 1.18 2002/10/30 22:18:28 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: provide socket interface for application access to TCP/IP stack
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/09/71
 *
 *-----------------------------------------------------------------
 */
#define TCPSTATES

#include <nfx-incl.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/macros.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/domain.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/uio.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/route.h"
#include "../sm-bsd-net/if_dl.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/in_pcb.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_fsm.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "exports/libc.h"
#include "exports/bsdsock-api.h"
#include "exports/bsdnet-api.h"
#include "exports/pkt-api.h" 
#include "exports/ipm-api.h"

extern	struct	inpcbhead ripcb;
extern  struct	inpcbhead udb;
extern  struct	inpcbhead tcb;


q_headNodePtr_t bsd_sockcmd_queue;

/******************************************************************************
 * Name : bsd_socket 
 * 
 * Descrition: create a connection socket for TCP/IP usage by application
 *
 * Created by: Su Wang
 *
 * Date created: 07/16/01
 *****************************************************************************/ 
socket_ptr_t /* ptr to the socket structure allocated */
bsd_socket(
       int				dom, 
								/* protocol domain ID, always PF_INET */
       int				type, 
								/* socket type */
       int				proto, 
								/* protocol, TCP/UDP/ICMP etc .. */
       int				bufType, 
								/* rcv buffer type of the the socket */
       connevent_calbck_t connevent, 
								/* callback function when connection */
								/*   has been established */
       rcvdata_calbck_t   rcvdata 
								/* callback function when data need to */
								/* forward to application */
	   )
{
  socket_ptr_t so;
  int error;

  if ((error = socreate(dom, &so, type, proto, 0)) != 0)
    {
      bsd_log(BSD_ERR_SOCK, "socket create failed errno %d\n", error);
      return NULL; /* socket create failed */
    }
		   
  so->so_connevent = connevent;
  so->so_rcvdata = rcvdata;
  so->so_buftype = bufType;

  return so;
}

/******************************************************************************
 * Name : bsd_bind 
 *
 * Description: bind local address and/or port to a socket.
 *              raw ip only bind IP address. TCP, UDP bind address and port.
 *              if no port specified, stack allocate one. 
 *
 * Created by: Su Wang
 *
 * Date created: 07/16/01 
 *****************************************************************************/
int								/* 0 fail, 1 success */
bsd_bind (socket_ptr_t		so,    
								/* socket that bining performed on */
      ipa_t				laddr, 
								/* local ip network order, 0 for wild card */
      u_short			lport 
								/* local port num, network order */
      )
{
  struct sockaddr_in soaddr;
  int error;

  if (!so)
    return 0;

  bzero(&soaddr, sizeof(struct sockaddr_in));
  soaddr.sin_len = 16;
  soaddr.sin_family = AF_INET;
  soaddr.sin_port = lport;
  soaddr.sin_addr.s_addr = laddr;

      
  if ((error = sobind(so, (struct sockaddr*)&soaddr, 0)))
    {
      bsd_log(BSD_ERR_SOCK, "socket binding failure, errno %d\n", error);
      return 0;
    }
  

  return 1;
}

/*****************************************************************************
 * Name : bsd_listen 
 *
 * Description: listen to a connection socket for any incoming connection 
 *				or data.
 *
 * Created by: Su Wang
 *
 * Date created: 07/16/01 
 *****************************************************************************/
int								/* 0 fail, 1 success */
bsd_listen( socket_ptr_t	so, 
								/* listening socket */
		int				backlog 
								/* max. number of connections */ 
								/* allowed in waiting Q. */
		)
{
  int error;

  if ((error = solisten(so, backlog, 0)))
    {
      bsd_log(BSD_ERR_SOCK, "socket listen failed, errno %d\n", error);
      return 0;
    }
  else
    return 1;
}

/******************************************************************************
 * Name : bsd_connect 
 *
 * Description: actively connect to a remote server.
 *
 * Created by: Su Wang
 *
 * Date created: 07/16/01 
 ******************************************************************************/
int  /* 0 fail, 1 success */
bsd_connect(socket_ptr_t	so, 
								/* local socket */
		ipa_t			faddr, 
								/* remote ip */
		u_short			fport 
								/* server port num. */
		)
{
  struct sockaddr_in soaddr;
  int error;

  
  if (!so)
    return 0;

  bzero(&soaddr, sizeof(struct sockaddr_in));

  soaddr.sin_len = 16;
  soaddr.sin_family = AF_INET;
  soaddr.sin_port = fport;
  soaddr.sin_addr.s_addr = faddr;

      
  if ((error = soconnect(so, (struct sockaddr*)&soaddr, 0)))
    {			
      bsd_log(BSD_ERR_SOCK, "socket connect failure 2, errno %d\n", error);
      return 0;
    }
  
  return 1;
}


/*****************************************************************************
 * Name : bsd_ShutSock
 *
 * Description: terminate read and/write part of connection.
 *
 * Created by: Su Wang
 *
 * Date created: 09/16/01 
 *****************************************************************************/
int32								/* NFX_OK, NFX_ERR */
bsd_ShutSock(
		 void	*so,
		 int	how				/* terminate incoming and/or outgoing */
								/* 0 read, 1 write, 2 both */
		 )
{

  if (!so)
	return NFX_ERR;

  if (how > 2)
	return NFX_ERR;
  
  soshutdown((socket_ptr_t)so, how); 
  return NFX_OK;
}

/**********************************************************************
 * Name : bsd_CloseSock
 *
 * Description: close a socket.
 *        
 * Created by : Su Wang
 *
 * Date created: 07/18/01
 **********************************************************************/        
int32							/* NFX_OK, NFX_ERR */
bsd_CloseSock(void*  so)
{
  if (so)
	{
	  /* update ssc */
#if !defined(PROM)
	  struct inpcb *inp = sotoinpcb((struct socket*)so);
	  if (inp && (inp->inp_ip_p == IPPROTO_UDP))
		ipm_DelUdpConn(*(uint32*)&(inp->inp_laddr), inp->inp_lport,
					   *(uint32*)&(inp->inp_faddr), inp->inp_fport);
#endif

	  if (soclose((socket_ptr_t)so))
		return NFX_ERR;
	  else 
		return NFX_OK;
	}

  return NFX_ERR;
}

/******************************************************************
 * Name : bsd_AbortSock 
 *
 * Description: completely remove connection including socket structure,
 *				immidiately.
 *
 * Created by: Su Wang
 * 
 * Date created: 09/16/01
 ******************************************************************/
int32							/* NFX_ERR, NFX_OK */		
bsd_AbortSock(void* so)
{
  if (!so)
	return NFX_ERR;

  /* set SS_NOFDREF such that socket structure can be removed */
  return (soabort((socket_ptr_t)so))?NFX_ERR:NFX_OK;
}

/******************************************************************
 * Name : bsd_OpenSock 
 *
 * Description: open a socket structure. When using raw socket, 
 *				the received data will be entire IP datagram;
 *				when using tcp/udp socket, received data will be
 *				data portion of the ip datagram.
 *
 * Created by: Su Wang
 * 
 * Date created: 07/16/01
 ******************************************************************/
void*							/* NULL failed, ptr to socket */
bsd_OpenSock(
			 int			proto, /* see bsd-api.h */
			 int			type,  /* server or client */
			 bsd_sarg_ptr_t arg    /*socket creation arguments */
			 )
{
  int					so_type;	
  struct socket*		so=0;

  if (proto >= BSD_IPPROTO_MAX)
	return NULL;

  /* det. socket type */
  if (proto == BSD_IPPROTO_TCP)
	so_type = SOCK_STREAM;
  else if (proto == BSD_IPPROTO_UDP)
	so_type = SOCK_DGRAM;
  else
	so_type = SOCK_RAW;


  /* create socket */
  if (!(so=bsd_socket(PF_INET, so_type,  proto, arg->bufType,
					  arg->cb, arg->db)))
	return NULL;

  /* setting up nessary options */		
  if ((arg->flags & BSD_SO_FASTPATH))
	{
	  /* flags socket rcv queue to process pkt */
	  so->so_options |= SO_FASTPATH;
	  so->so_snd.sb_flags |= SB_PKT;
	}
  so->so_rcv.sb_flags |= SB_PKT;
  so->so_handle = arg->handle;


  if (type == BSD_SO_CLIENT)
	{
	  if (arg->laddr || arg->lport)
		if (!bsd_bind(so, arg->laddr, arg->lport))
		  goto bad;

	  if (!arg->faddr)
		goto bad;
	  else
		{
		  if (!bsd_connect(so, arg->faddr, arg->fport))
			goto bad;
		}
	}
  else if (type == BSD_SO_SERVER)
	{
	  /* raw ip, local ip must be provided */ 
	  /* tcp/udp, local port must be provided */
	  if ((so_type == SOCK_RAW && !arg->laddr) ||
		  (!arg->lport && so_type != SOCK_RAW))
		goto bad;
	  else
		{
		  if (!bsd_bind(so, arg->laddr, arg->lport))
			goto bad;
		}

	  if (proto == BSD_IPPROTO_TCP)
		{
		  if (!bsd_listen(so, SOMAXCONN))
			goto bad;		  
		}
	  /* does server want to spawn off connections */
	  if ((arg->flags & BSD_SO_NO_SPAWN) == 0)
		so->so_options |= SO_SPAWN;
	}
  else
	goto bad;

  return so;

 bad:
  if (so)	
  		soclose(so);
  return NULL;    
}


/*****************************************************************************
 *
 * Name : bsd_OpenTcpClient
 * 
 * Description: request to open a TCP connection, this function should be 
 *				called the a TCP client. All ip address and port number 
 *				is in network order. Note: connection is not established
 *              after this function call.
 *
 * Created by : Su Wang
 *
 * Date created: 07/17/01
 *****************************************************************************/
void* /* ptr to openning socket, NULL error */
bsd_OpenTcpClient(				  
				  ipa_t			lAddr,  
								/* local IP, wild card */
				  u_short		lPort, 
								/* local port, wild card */
				  ipa_t			fAddr, 
								/* remote IP, must have */
				  u_short		fPort, 
								/* remote port, must have */
				  int			bufType, 
				  void*			handle, 
								/* handle passed by application */
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t   rcvdata
				  )
{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.lport = lPort;
  arg.fport = fPort;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return(bsd_OpenSock(BSD_IPPROTO_TCP, BSD_SO_CLIENT, &arg));
}

/*****************************************************************************
 *
 * Name : bsd_OpenTcpServer
 * 
 * Description: open a TCP listening socket, this function should be called 
 *				by  a TCP server.
 *              all ip address and port number is in network order. 
 *
 * Created by : Su Wang
 *
 * Date created: 07/17/01
 *****************************************************************************/
void* /* ptr to listenning socket, 0 error */
bsd_OpenTcpServer(
				  ipa_t			lAddr,  
								/* local IP, wild card */
				  u_short		lPort, 
								/* local port, must have */
				  ipa_t			fAddr, 
								/* not used */
				  u_short		fPort, 
								/* not used */
				  int			bufType,  
								/* buffer type */
				  void*			handle, 
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t rcvdata
				  )  
{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.lport = lPort;
  arg.fport = fPort;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return(bsd_OpenSock(BSD_IPPROTO_TCP, BSD_SO_SERVER, &arg));  
}  

/*****************************************************************************
 * Name : bsd_OpenUdpClient
 *
 * Description: open an udp socket client.
 *
 * Created by: Su Wang
 *
 * Date created: 07/18/01
 *****************************************************************************/
void* /* ptr to opened socket,  NULL failed */
bsd_OpenUdpClient(
				  ipa_t			lAddr,  
								/* local IP, wildcard */
				  u_short		lPort, 
								/* local port, wildcard */
				  ipa_t			fAddr, 
								/* remote ip, must have */
				  u_short		fPort, 
								/* remote port, must have */
				  int			bufType, 
								/* pkt_ptr or flatbuf */
				  void			*handle,
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t rcvdata
				  )  	

{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.lport = lPort;
  arg.fport = fPort;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return(bsd_OpenSock(BSD_IPPROTO_UDP, BSD_SO_CLIENT, &arg));  
}  

/*****************************************************************************
 * Name : bsd_OpenUdpServer
 *
 * Description: open an udp socket server.
 *
 * Created by: Su Wang
 *
 * Date created: 07/18/01
 *****************************************************************************/
void* /* ptr to opened socket, 0 socket openning failed */
bsd_OpenUdpServer(
				  ipa_t			lAddr, 
								/* local IP, wildcard */
				  u_short		lPort, 
								/* local port, must have */
				  ipa_t			fAddr, 
								/* not used */
				  u_short		fPort, 
								/* not used */
				  int			bufType, 
								/* BSD_BUF_FLAT or BSD_BUF_DESCR */
				  void*			handle,
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t rcvdata
				  )  
{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.lport = lPort;
  arg.fport = fPort;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return (bsd_OpenSock(BSD_IPPROTO_UDP, BSD_SO_SERVER, &arg));
}  

/***************************************************************************
 * Name : bsd_OpenRipClient
 *
 * Description: open a raw ip socket client.
 *
 * Created by: Su Wang
 *
 * Date created: 07/18/01
 **************************************************************************/
void*							/* ptr to open socket, 0 openning failed */
bsd_OpenRipClient(
				  int			protocol, 
								/* IP protocols using raw ip:ICMP, OSPF.. */
				  ipa_t			lAddr, 
								/* local IP, wildcard */
				  ipa_t			fAddr, 
								/* remote IP, must have */
				  int			bufType, 
				  void			*handle, 
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t   rcvdata
				  )
{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return(bsd_OpenSock(protocol, BSD_SO_CLIENT, &arg));  
}  

/*********************************************************************
 * Name : bsd_OpenRipServer
 *
 * Description: open a raw ip server socket. 
 *
 * Created by: Su Wang
 *
 * Date created: 07/18/01
 *********************************************************************/
void*							/* ptr to open socket, 0 failed */
bsd_OpenRipServer(
				  int			protocol, 
								/* ICMP, OSPF, etc .. */
				  ipa_t			lAddr, 
								/* local IP, should have */
				  ipa_t			fAddr,	
								/* not used,*/
				  int			bufType,
				  void*			handle, 
				  connevent_calbck_t connevent,
				  rcvdata_calbck_t   rcvdata
				  )
{
  bsd_sarg_t  arg;

  memset(&arg, 0, sizeof(bsd_sarg_t));
  arg.laddr = lAddr;
  arg.faddr = fAddr;
  arg.handle = handle;
  arg.bufType = bufType;
  arg.cb = connevent;
  arg.db = rcvdata;

  return(bsd_OpenSock(protocol, BSD_SO_SERVER, &arg));  
}  

/*******************************************************************
 * 
 * Name : bsd_SetSoOption
 *
 * Description : set socket options.
 *
 * Created by : Su Wang
 *
 * Date created: 07/18/01
 ********************************************************************/
void
bsd_SetSoOption(void *so, int flag, int value)
{
  switch(flag)
	{
	case BSD_SO_FASTPATH:
	  if (value)
		((struct socket*)so)->so_options |= SO_FASTPATH;
	  else
		((struct socket*)so)->so_options &= ~SO_FASTPATH;
	}

  return;
}


/*******************************************************************
 * 
 * Name : bsd_SendSock
 *
 * Description : api to send traffic on to socket.
 *				  Assuming flat buffer only 
 *
 * Created by : Su Wang
 *
 * Date created: 07/18/01
 ********************************************************************/
int32 /* NFX_OK, NFX_ERR */
bsd_SendSock(
			 void*				so1, 
								/* socket to send traffic */
			 ipa_t				faddr, 
								/* remote ip, */
								/* sometimes needed if socket is RAW or UDP */
			 ipa_t				fport, 
								/* remote port */
								/* may be needed for some UDP socket */
			 void				*data_ptr,	
								/* pointer to data */
			 int				type, 
			 uint32				size,		
								/* data size */
			 int32				flags		
								/* MSG_OOB, MSG_EOF */
			 ) 
{
  struct mbuf *m;
  struct sockaddr_in soaddr;
  socket_ptr_t so = (socket_ptr_t)so1;
  int error;
  
  
  /* sanity check */
  if (! (so && size && data_ptr))
    return NFX_ERR;

  /* eligibility to send */
  if (so->so_state & SS_CANTSENDMORE)
	return NFX_ERR;
  
  if ((so->so_proto->pr_flags & PR_CONNREQUIRED) &&
	  (so->so_state & SS_ISCONNECTED) == 0)
	return NFX_ERR;

  if (sbspace(&so->so_snd) < size)
	return NFX_ERR;

  if (type != BSD_BUF_FLAT)
    {
      /* do something to reset data_ptr and size. 
		 save current header buffer */
      return NFX_ERR;
    }
  
  if ((m = m_devget(data_ptr, size, 0, 0, 0)) == NULL)
	return NFX_ERR;

  m->m_pkthdr.rcvif = (struct ifnet *)0;

  if (faddr || fport)
    {
      soaddr.sin_len = sizeof(soaddr);
      soaddr.sin_family = AF_INET;
      soaddr.sin_port = fport;
      soaddr.sin_addr.s_addr = faddr;
    }
  
  error = (*so->so_proto->pr_usrreqs->pru_send)
	(so,
	 (flags & MSG_OOB) ? PRUS_OOB :
	 (flags & MSG_EOF) ? PRUS_EOF : 0,m,
	 (faddr|| fport)?(struct sockaddr*)&soaddr:0, 
	 0, 0);

  if (error)
	{
	  bsd_log(BSD_ERR_SOCK, "sosend failed, err num %d\n", error);
	}
  
  return error ? NFX_ERR:NFX_OK;
}

/**************************************************************************** 
 * Name : bsd_RcvSock
 *
 * Description: This function send data to upper 
 *              layer.
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
int32 bsd_RcvSock(struct socket*		so, 
				  int					flags, 
				  uint32				offset  
				  )
{

  char   *data_ptr = 0;  
  struct mbuf *m = so->so_rcv.sb_mb, *n;
  struct protosw *pr = so->so_proto;
  uint32 srcAddr = 0;
  ushort srcPort = 0;
  int len = 0;
  pkt_desc_t  *pdesc, *prev_desc=0, *sav_desc=0;
  char		  *tmp_buf = 0;
  int		  ok_flag = 1;
  
  
  
  if (!m)
	return NFX_OK;

  if (!so->so_rcvdata) {/* add something to check q size */
	ok_flag = 0;
	goto bad;
  }
  
  if (so->so_state & SS_CANTRCVMORE) {
	ok_flag = 0;
	goto bad;
  }
  

  /* UDP/RIP  buffer, first mbuf contains src addr */
  if (pr->pr_flags & PR_ADDR) 
	{
	  srcAddr = ((struct sockaddr_in*)(m->m_data))->sin_addr.s_addr;
	  srcPort = ((struct sockaddr_in*)(m->m_data))->sin_port;
	  m = m->m_next;
	}

  /* The next buffer must be PKTHDR buff where pkt desc is located */
  if (!((m->m_flags & M_PKTHDR) && (pdesc = (pkt_desc_t*)m->m_pktdesc)))
	{
	  bsd_log(BSD_ERR_SOCK, "bsdIpRcv: buf unexpected\n");
	  ok_flag = 0;
	  goto bad;
	}

  if (!(flags & MSG_MULTI))
    {
 
      /* data len */
      len = pdesc->hdrSize + pdesc->dataSize;
      if (len <= 0) {
		ok_flag = 0;
		goto bad;
	  }

      if (so->so_buftype == BSD_BUF_FLAT)
		{
		  data_ptr = pdesc->pHdr + offset;
		}
      else 
		{
		  /* the descriptor is wanted, we simply pass the descriptor */
		  data_ptr = (char*)pdesc;
		  pdesc->pHdr = pdesc->pHdr + offset;
		}
    }
  else
    {
      /* 
		 two cases arrive here, IP reassembly and TCP segment out of
		 order.
      */
	  if (so->so_buftype == BSD_BUF_FLAT)
		{
		  /* calculate size of buffer needed */
		  len = 0;
		  for (m = so->so_rcv.sb_mb; m; m = m->m_next)
			{
				pdesc = m->m_pktdesc;
				while (pdesc) {
					len += pdesc->hdrSize + pdesc->dataSize;
					pdesc = pdesc->pNext;
				}
			}
		  /* taking offset into account */
		  len -= offset;
			
		  /* copy header buffers into tmp_buff */
		  tmp_buf = (char*)malloc(len, 0, 0);
		  data_ptr = tmp_buf;
		  m = so->so_rcv.sb_mb;
		  for (m = m->m_next; m; m = m->m_next)
			{
			  if ((pdesc = m->m_pktdesc))
				{
				  while (pdesc)
					{
					  memcpy(tmp_buf, pdesc->pHdr+offset, pdesc->hdrSize-offset);
					  tmp_buf += pdesc->hdrSize-offset;
					  pdesc = pdesc->pNext;
					}
				  /* offset only for first pkt */
				  if (offset)
					offset = 0;
				} 
			}		  
		}
	  else
		{
		  len = 0;	

		  for (m = so->so_rcv.sb_mb; m; m = m->m_next)
			{
				pdesc = m->m_pktdesc;
				while (pdesc) {
					len += pdesc->hdrSize + pdesc->dataSize;
					pdesc = pdesc->pNext;
				}
			}

		  len -= offset;

		//  printf("multi pkt rcved \n");
		  if (len <= 0) {
			ok_flag = 0;
			goto bad;
		  }
		  
		  /* just need to chain all pkt descriptors together */
		  m = so->so_rcv.sb_mb;
		  while (m->m_pktdesc == 0)
			m = m->m_next;
		  pdesc = m->m_pktdesc;
		  m->m_pktdesc = 0;
		  data_ptr = (char*)pdesc;
		  pdesc->hdrSize -= offset;
		  pdesc->pHdr += offset;
		
		  /* find last pkt in the chain */
		  while (pdesc)
			{
			  prev_desc = pdesc;
			  pdesc = pdesc->pNext;
			}

		  for (m = m->m_next; m; m = m->m_next)
			{
			  if ((pdesc = m->m_pktdesc))
				{
				  m->m_pktdesc = 0;
				  prev_desc->pNext = pdesc;
				  while (pdesc)
					{
					  prev_desc = pdesc;
					  pdesc = pdesc->pNext;
					}
				}
			}
		}
    }


  /* if rcv buff type is descriptor, pass the desc ownership to upper layer,
     else free desc now */
  if (so->so_buftype == BSD_BUF_DESCR)
    {
      for (m = so->so_rcv.sb_mb; m; m = m->m_nextpkt)
		for (n = m; n; n = n->m_next)
		  {
			if (n->m_pktdesc)
			  n->m_flags |= M_PROTO1;
		  }
    }
  else if (!(flags & MSG_MULTI) &&
		   (so->so_buftype == BSD_BUF_FLAT))	{ /* single flat buf case */
	  m->m_flags |= M_PROTO1;
	  sav_desc = pdesc;
  }

  /* free mbuf before call application, for application may decides to */
  /* close socket before returning. */
bad:
  if (tmp_buf)
	free(tmp_buf, MT_DATA);
  sbflush(&so->so_rcv);


  if (ok_flag)
	  so->so_rcvdata((void*)so, (void*)data_ptr, srcAddr, srcPort, 
					 len, so->so_handle);

  if (sav_desc)
	  PKT_FREE_PACKET(sav_desc);  

  return NFX_OK;
}

/**************************************************************************** 
 * Name : bsd_ShowSock
 *
 * Description: This function displays socket configuration 
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
void bsd_ShowSock(int	 what,	/* single, udp, tcp. all, etc */ 
				   void* id,	/* socket address */   
				   void* param1,
				   void* param2
				   )
{
  struct inpcb *inp;
  struct inpcbhead head;
  int again = 0;
  char* type;
  ushort16 port;

  if (!id) /* multiple display */ 
    {
      uiPrintf("Summary of Sockets\n");
      uiPrintf("---------------------------------------------------------------------------------\n");
      uiPrintf("ID        Type    SrcIP               SrcPort   DstIP               DstPort   Mac\n");
	loop_again:
      switch (what)
		{
		case 1: /* udp */
		  type = "udp";
		  head = udb;
		  break;

		case 2: /* tcp */
		  type = "tcp";
		  head = tcb;
		  break;

		case 3: /* rip */
		  type = "rip";
		  head = ripcb; 
		  break;

		case 4: /* all */
		  type = "udp";
		  head = udb;
		  again = 1;
		  break;
      
		default:
		  return;
		}
      
      for (inp = head.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
		{
		  bsdIpNtoS(&(inp->inp_laddr.s_addr), prnt_ip, sizeof(prnt_ip));
		  port = ntohs(inp->inp_lport);
		  uiPrintf("%-10lx%s%-5d%-20s%-10u", 
				   (address_t)(inp->inp_socket), type, 
				   inp->inp_ip_p, prnt_ip, port);
		  bsdIpNtoS(&(inp->inp_faddr.s_addr), prnt_ip, sizeof(prnt_ip));
		  bsdMacNtoS(inp->inp_nhmac, prnt_mac, sizeof(prnt_mac));
		  port = ntohs(inp->inp_fport);
		  uiPrintf("%-20s%-10u%s\n", prnt_ip, port, prnt_mac);
		}

      if ((again > 0) && (again < 3))
		{
		  what = ++again;
		  goto loop_again;
		}
    }
  else /* single case */
    {
    }
  
  return;
}

/**************************************************************************** 
 * Name : bsd_MakeConnQ
 *
 * Description: build TCP/UDP connection update queue. 
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
void
bsd_MakeConnQ(all)
{
  struct inpcb *inp;
  struct tcpcb *tp;
 
  for (inp = tcb.lh_first; inp != NULL; inp = inp->inp_list.le_next)
	{
	  tp = (struct tcpcb*)inp->inp_ppcb;
	  if (!tp)
		continue;

	  /* connection is newly added */
	  if ((inp->inp_state == 0) || all)
		{
#if !defined(PROM)
		  ipm_AddTcpConn(*(uint32*)&(inp->inp_laddr), inp->inp_lport,
						 *(uint32*)&(inp->inp_faddr), inp->inp_fport, 
						 tp->t_state);
#endif
		  inp->inp_state = tp->t_state;
		}

	  /* conection state has changed */
	  if ( inp->inp_state != tp->t_state)
		{
#if !defined(PROM)
		  ipm_ChgeStateTcpConn(*(uint32*)&(inp->inp_laddr), inp->inp_lport,
							   *(uint32*)&(inp->inp_faddr), inp->inp_fport, 
						 tp->t_state);
#endif
		  inp->inp_state = tp->t_state;
		}
	}

    for (inp = udb.lh_first; inp != NULL; inp = inp->inp_list.le_next)
	{

	  /* connection is newly added */
	  if ((inp->inp_state == 0) || all)
		{
#if !defined(PROM)
		  ipm_AddUdpConn(*(uint32*)&(inp->inp_laddr), inp->inp_lport,
						 *(uint32*)&(inp->inp_faddr), inp->inp_fport);
#endif
		  inp->inp_state = 1;
		}
	}

  return;
}

/**************************************************************************** 
 * Name : bsd_ResetTcp
 *
 * Description: abort all tcp connections, useful system shutdown.
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 03/22/02 
 ****************************************************************************/
void
bsd_ResetTcp()
{
	struct tcpcb *tp;
	struct inpcb *inp, *nxtInp;

	inp = tcb.lh_first;
	while (inp) {
		nxtInp = inp->inp_list.le_next;
		
		tp = (struct tcpcb*)inp->inp_ppcb;
		if (!tp)
			continue;
		
		/* terminate connections  */
		if (tp->t_state > TCPS_LISTEN)
			bsd_AbortSock(inp->inp_socket);

		inp = nxtInp;
	}

	return;
}


/**************************************************************************** 
 * Name : bsd_ShowTcp
 *
 * Description: This function displays TCP CB configuration 
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
void bsd_ShowTcp(  void* id,	/* socket address */   
				   void* param1,
				   void* param2
				   )
{
  struct inpcb *inp;
  struct tcpcb *tp;

  for (inp = tcb.lh_first; inp != NULL; inp = inp->inp_list.le_next)
	{
	  if (inp->inp_socket != (struct socket*)id)
		continue;
	  
	  tp = (struct tcpcb*)inp->inp_ppcb;
	  if (!tp)
		continue;

	  uiPrintf("TCP						:		%lx\n", (address_t)tp);
	  uiPrintf("State					:		%s\n", 
			   tcpstates[tp->t_state]);
	  uiPrintf("Timers					:		%d:%d:%d:%d\n",
			   tp->t_timer[0], tp->t_timer[1], tp->t_timer[2],
			   tp->t_timer[3]);
	  uiPrintf("Max Seg 				:		%d\n", tp->t_maxseg);
	  uiPrintf("Flags					:		%x\n\n", tp->t_flags);
	  uiPrintf("Snd Unack				:		%x\n", tp->snd_una);
	  uiPrintf("Snd Max					:		%x\n", tp->snd_max);
	  uiPrintf("Snd Next				:		%x\n", tp->snd_nxt);
	  uiPrintf("Snd Urg					:		%x\n", tp->snd_up);
	  uiPrintf("Snd Win					:		%x\n", tp->snd_wnd);
	  uiPrintf("Snd Con.Win				:		%x\n", tp->snd_cwnd);
	  uiPrintf("Snd Thresh				:		%x\n", tp->snd_ssthresh);
	  uiPrintf("Snd Win Seq				:		%x\n", tp->snd_wl1);
	  uiPrintf("Snd Win Ack				:		%x\n", tp->snd_wl2);
	  uiPrintf("Rcv Next				:		%x\n", tp->rcv_nxt);
	  uiPrintf("Rcv Adv					:		%x\n", tp->rcv_adv);
	  uiPrintf("Rcv Urg					:		%x\n", tp->rcv_up);
	  uiPrintf("Rcv Win					:		%x\n\n", tp->rcv_wnd);

	  uiPrintf("Idle Time				:		%u\n", tp->t_idle);
	  uiPrintf("Up Time					:		%u\n", tp->t_duration);
	  uiPrintf("RTrip Time				:		%u\n", tp->t_rtt);
	  uiPrintf("Rex Timeout				:		%u\n", tp->t_rxtcur);
	  
	}
 
  return;
}


#if defined(LINUX_TEST)
int bsd_TstSock(void* so, ipa_t da, ushort dport)
{
  static char msg_buf[]= {      
	0x13, 0xa0, 0x74, 0x45,
	0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x40, 0x3f,
	0x00, 0x00, 0x00, 0x0b, 0x00, 0x00, 0xea, 0x61,
	0x00, 0x00, 0xff, 0xfe, 0x00, 0x00, 0x02, 0x00,
	0x00, 0x00, 0x20, 0x00, 0x80, 0x00, 0x0c, 0x01,
	0x00, 0x00, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00,
	0x00, 0x00, 0x09, 0x00, 0x3b, 0x73, 0x95, 0xa8,
	0x00, 0x0a, 0xae, 0x7c, 0x3b, 0x4b, 0x44, 0xb9,
	0x00, 0x03, 0x0d, 0x41, 0x3a, 0xae, 0x85, 0x4d,
	0x00, 0x07, 0xa1, 0x02
    
  };
  struct inpcb *inp;
  int find = 0;

  for (inp = udb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == so)
		{
		  find = 1;
		  goto find;
		}
    }

  for (inp = ripcb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == so)
		{
		  find = 1;
		  goto find;
		}
    }

  for (inp = tcb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == so)
		{
		  find = 1;
		  goto find;
		}
    }

 find:
  if (!find)
    return NFX_ERR;

  bsd_SendSock(so, da, dport, msg_buf, BSD_BUF_FLAT, sizeof(msg_buf), 0);

  return NFX_OK;

}
#endif /* LINUX_TEST */

static void data_dummy(void* so, void* dataptr, ipa_t sa, 
					   ushort sp, uint32 len, void* handle)
{

  printf("pkt received from %s, port %u\n", bsdPrintIpa(sa), ntohs(sp));
  bsd_ShowPkt(dataptr, len);
  return;
}

static int conn_dummy(void* so0, void* inp0, int type, void** handptr)
{
  struct socket	*so = (struct socket*)so0;

  printf("so %lx rcv evnt %d\n", (address_t)so0, type);
  *handptr = so0;

  switch(type)
		{
		case SO_CONN_DOWN_IND:
		  bsd_CloseSock(so);
		}
  return 0;
}

int32 
bsd_AddSock(char* proto, int type, ipa_t addr, ushort port)
{

  struct socket* so = 0;

  if (strcmp(proto, "TCP") == 0 || strcmp(proto, "tcp") == 0)
    {
      if (type == 1) /* server */
		{
		  if (bsd_OpenTcpServer(addr, port, 0, 0, BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: TCP open server failed\n");
			  goto bad;
			}
		}
	  else /* client */
		{
		  if (bsd_OpenTcpClient(0, 0, addr, port, BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: TCP open client failed\n");
			  goto bad;
			}
		}
    }
  else if (strcmp(proto, "UDP") == 0 || strcmp(proto, "udp") == 0)
    {
      if (type == 1) /* server */
		{
		  if (bsd_OpenUdpServer(addr, port, 0, 0, BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: UDP open server failed\n");
			  goto bad;
			}
		}
      else /* client */
		{
		  if (bsd_OpenUdpClient(0, 0, addr, port, BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: UDP open client failed\n");	
			  goto bad;
			}
		}
    }
  else if (strcmp(proto, "ICMP") == 0 || strcmp(proto, "icmp") == 0)
    {
      if (type == 1)  /* server */
		{
		  if (bsd_OpenRipServer(IPPROTO_ICMP, addr, 0, BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: icmp open server failed\n");
			  goto bad;
			}
		}
      else /* client */
		{
		  if (bsd_OpenRipClient(IPPROTO_ICMP, 0, addr,BSD_BUF_FLAT, 0, &conn_dummy, &data_dummy) 
			  == 0)
			{
			  printf("bsd_addSock: icmp open client failed\n");
			  goto bad;
			}
		}
    }
  else
    goto bad;

  return NFX_OK;

 bad:  
  if(so)
    soclose(so);

  return NFX_ERR;
}

int bsd_DeleteSock(void* so)
{
  struct inpcb *inp;
  int find = 0;

  for (inp = udb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == (struct socket*)so)
		{
		  find = 1;
		  goto find;
		}
    }

  for (inp = ripcb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == (struct socket*)so)
		{
		  find = 1;
		  goto find;
		}
    }

  for (inp = tcb.lh_first; inp != NULL; inp = inp->inp_list.le_next) 
    {
      if (inp->inp_socket == (struct socket*)so)
		{
		  find = 1;
		  goto find;
		}
    }

 find:
  if (!find)
    return NFX_ERR;

  bsd_CloseSock((struct socket*)so);

  return NFX_OK;

}

