#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <pwd.h>

#include <sys/wait.h>
#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "bsdhelper.h"

#define MAX_ARGS 20


/* SY: This routine is shamelessly copied from ssc-nfxsh/vtysh.c */
/* Execute command in child process. */
int
execute_command(char *command, int argc, ...)
{
va_list ap;
int ret;
pid_t pid;
int status;
char *a[MAX_ARGS];
int i;
int return_code;

	va_start(ap, argc);

	/* Call fork(). */
	pid = fork ();

	if (pid < 0)
	{
		/* Failure of fork(). */
		return -1;
	}
	else if (pid == 0)
	{
		/* This is child process. */
		/* This would be much easier if exec*() accepted an va_list.  */
		if (argc > MAX_ARGS)
			argc = MAX_ARGS;

		memset(a, 0, sizeof(a));
		for (i = 0; i < argc; i++)
		{
			a[i] = va_arg(ap, char *);
			if (!a[i])
				break;
		}
		va_end(ap);

		ret = execlp(command, command, a[0], a[1], a[2], a[3], a[4],
				a[5], a[6], a[7], a[8], a[9], a[10], a[11], a[12],
				a[13], a[14], a[15], a[16], a[17], a[18], a[19], NULL);

		/* When execlp suceed, this part is not executed. */
		fprintf(stderr, "Can't execute %s.\n", command);
		exit (1);
	}
	else
	{
		/* This is parent. */
		ret = wait4(pid, &status, 0, NULL);
		if (WIFEXITED(status))
			return_code = WEXITSTATUS(status);
		else if (WIFSIGNALED(status))
			return_code = WTERMSIG(status);
		else
			return_code = -1;
	}
	return return_code;
}


/* This only work with "," delimitered string */
/* return the next starting point, which can  */
/* be used to pass back in "from" parameter   */
int GetDelimiteredParam(int start, char *srcbuf, char *dstbuf)
{
int first = start;

	if (dstbuf)
		dstbuf[0] = '\0';

	if ((start < 0) || !srcbuf)		/* no buffer? */
		return 0;

	if (!srcbuf[start])		/* already end of buffer */
		return 0;

	while (srcbuf[start] && (srcbuf[start] != ','))
		start++;

	if (start > first)
	{
		strncpy(dstbuf, &srcbuf[first], start - first);
		dstbuf[start-first] = '\0';
	}

	if (srcbuf[start])
		start++;

	return start;
}


char *GetIpAddressStr(u_int32_t src)
{
struct in_addr addr;

	addr.s_addr = src;
	return inet_ntoa(addr);
}

u_int32_t GetIpAddressFromStr(char* buf)
{
struct in_addr addr;

	inet_aton(buf, &addr);
	return addr.s_addr;
}


u_int32_t GetDefaultRoute(void)
{
struct rt_msghdr *rtm;
char *rtbuf = NULL, *next = NULL, *lim = NULL;
size_t needed;
int mib[6] = { CTL_NET, PF_ROUTE, 0, AF_INET, NET_RT_DUMP, 0 };
struct in_addr defroute;

	/* add default route to the end of buffer */

	defroute.s_addr = 0;

	/* first, retrieve the routing table from system */
	if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
		goto done_getdefroute;

	if (needed > 0)
	{
		if ((rtbuf = malloc(needed)) == 0)
			goto done_getdefroute; 		/* no space for routing table */
		if (sysctl(mib, 6, rtbuf, &needed, NULL, 0) < 0)
			goto done_getdefroute; 		/* can't retrieve it?? */
		lim  = rtbuf + needed;
	}

	/* now go find the default route */
	if (rtbuf)
	{
		int found = 0;
		struct sockaddr *sa;

		for (next = rtbuf; next < lim; next += rtm->rtm_msglen) {
			char *rtmend;
			int bit;

			rtm = (struct rt_msghdr *) next;
			rtmend = (char *) rtm + rtm->rtm_msglen;
			sa = (struct sockaddr *) (rtm + 1);

			for (bit = 1; (char *)sa < (char *)rtmend; bit <<= 1)
			{
				if ((rtm->rtm_addrs & bit) == 0)
					continue;

				switch (bit)
				{
				case RTA_DST:
					if (((struct sockaddr_in *) sa)->sin_addr.s_addr == 0)
						found++;
					break;
				case RTA_GATEWAY:
					if (sa->sa_family == AF_INET)
					{
						defroute = ((struct sockaddr_in *) sa)->sin_addr;
						found++;
					}
					break;
				}
				if (found >= 2)		/* found it!! */
					goto done_getdefroute;

#define ROUNDUP(a) \
		((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
				sa = (struct sockaddr *)((char *)sa + ROUNDUP(sa->sa_len));
			}

			/* found the default route? if yes, no more search */
			if (found >= 2)
				goto done_getdefroute;
			else
				found = 0;	/* reset found flag for next rtm entry */
		}
	}

done_getdefroute:
	if (rtbuf)
		free(rtbuf);
	return defroute.s_addr;
}


int SetDefaultRoute(u_int32_t defRoute)
{
pid_t pid, ret_pid;
int status, i;
int options = WNOHANG | WUNTRACED;
struct in_addr addr;
struct timespec tout = {0, 200000};

	/* it's easier to just use command "route" to */
	/* delete and add the default route than deal */
	/* with the kernel route socket....           */

	/* now fork and execute "route" command to delete route */
	pid = fork();
	if (pid == 0)		/* this is child process */
	{
		/* delete the default route first */
		status = execlp("/sbin/route", "/sbin/route",
							"delete", "0.0.0.0", NULL);

		/* shouldn't return from above call, if it return */
		/* failed to execute command...                   */
		exit(1);
	}

	/* the original process continue here... */
	status = 0;
	if (pid > 0)
	{
		for (i = 0; i < 10; i++)
		{
			ret_pid = waitpid(pid, &status, options);
			if (ret_pid == 0)
				nanosleep(&tout, NULL);
			else
				break;
		}
	}
	else
		return 0;

	/* now fork and execute "route" command to add route */
	pid = fork();
	if (pid == 0)		/* this is child process */
	{
		/* add the new default route */
		addr.s_addr = defRoute;
		status = execlp("/sbin/route", "/sbin/route",
				"add", "0.0.0.0", inet_ntoa(addr), NULL);

		/* shouldn't return from above call, if it return */
		/* failed to execute command...                   */
		exit(1);
	}

	/* the original process continue here... */
	status = 0;
	if (pid > 0)
	{
		for (i = 0; i < 10; i++)
		{
			ret_pid = waitpid(pid, &status, options);
			if (ret_pid == 0)
				nanosleep(&tout, NULL);
			else
				break;
		}
	}
	else
		return 0;

	return 1;
}


int SetSystemTime(void *data)
{
struct tm			*tmval, settmval;
struct timeval		tval;
struct timezone 	tzval;

	tmval = (struct tm *) data;

	gettimeofday(&tval, &tzval);
	localtime_r((time_t *) &(tval.tv_sec), &settmval);

	settmval.tm_min  = tmval->tm_min;
	settmval.tm_hour = tmval->tm_hour;
	settmval.tm_mday = tmval->tm_mday;
	settmval.tm_mon  = tmval->tm_mon;
	settmval.tm_year = tmval->tm_year;

	tval.tv_sec = (long int) mktime(&settmval);
	tval.tv_usec = 0;

	return settimeofday(&tval, &tzval);
}


int VerifyUserPass(char *user, char *passwd)
{
struct passwd *pwd;

	/* do login check here and set login_ok properly */
	pwd = getpwnam(user);
	if (!pwd)
		return -1;
	else
	{
		if (strcmp(crypt(passwd, pwd->pw_passwd), pwd->pw_passwd))
			return -1;
		else
			return 1;
	}

	return -1;
}


int RebootBSD()
{
pid_t pid;
int status;

	pid = fork();
	if (pid == 0)		/* this is child process */
	{
		status = execlp("/sbin/reboot", "/sbin/reboot", NULL);
		exit(1);
	}

	return 1;
}

