/*  File Name  :    agilediskaddvol.c
 *
 *  Description:    HTTPD action functions for add volume - from pool.
 *
 *  Created    :    3/15/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <spm-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_MAKE			1
#define VOL_MODEL			2
#define VOL_RAID			3
#define VOL_RAIDSTR			4
#define NUM_OF_TAGS			4

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_MAKE,		"args.make"},
	{VOL_MODEL,		"args.model"},
	{VOL_RAID,		"args.rl"},
	{VOL_RAIDSTR,	"args.rlstr"},
};


#define PARAM_MAKE			1
#define PARAM_MODEL			2
#define PARAM_RAID			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_MAKE,	"make"},
	{PARAM_MODEL,	"model"},
	{PARAM_RAID,	"rl"},
};


/* HERE ARE VARIABLES FOR LUN INFORMATION */
static char make[SDM_VENDOR_NAME_LEN+1];
static char model[SDM_MODEL_NAME_LEN+1];
static int raid;


/***********************************************************
 * routines to retrieve volume information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_MAKE:
			strcpy(make, argval[argind]);
			break;
		case PARAM_MODEL:
			strcpy(model, argval[argind]);
			break;
		case PARAM_RAID:
			sscanf(argval[argind], "%d", &raid);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_MAKE:
		strcpy(value, make);
		break;
	case VOL_MODEL:
		strcpy(value, model);
		break;
	case VOL_RAID:
		sprintf(value, "%d", raid);
		break;
	case VOL_RAIDSTR:
		GetRaidLevelStr(raid, value);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionDiskAddVol(void *r, int count, char **argname, char **argval)
{
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
