/*  File Name  :    agilediskmain.c 
 *
 *  Description:    HTTPD action functions for main Disk Page.
 *
 *  Created    :    3/14/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <spm-api.h>

#include "agilepost.h"
#include "agileutil.h"

#if 0
#define DISK_WWN			1
#define DISK_WWNSTR			11
#endif
#define INVALID_TAG			0
#define DISK_CTRL			1
#define DISK_MAKE			2
#define DISK_MODEL			3
#define DISK_MODELURL		4
#define DISK_RAIDLVL		5
#define DISK_RAIDSTR		6
#define DISK_USED			7
#define DISK_FREE			8
#define DISK_FORE			9
#define DISK_OUTC			10
#define NUM_OF_TAGS			10

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
#if 0
	{DISK_WWN,		"spm_poolNode_t.wwn"},
	{DISK_WWNSTR,	"spm_poolNode_t.wwnstr"},
#endif
	{DISK_CTRL,		"spm_poolNode_t.ctrl"},
	{DISK_MAKE,		"spm_poolNode_t.make"},
	{DISK_MODEL,	"spm_poolNode_t.model"},
	{DISK_MODELURL,	"spm_poolNode_t.modelurl"},
	{DISK_RAIDLVL,	"spm_poolNode_t.rL"},
	{DISK_RAIDSTR,	"spm_poolNode_t.rLstr"},
	{DISK_USED,		"spm_poolNode_t.usedSize"},
	{DISK_FREE,		"spm_poolNode_t.freeSize"},
	{DISK_FORE,		"spm_poolNode_t.foreignSize"},
	{DISK_OUTC,		"spm_poolNode_t.ocSize"},
};


/* HERE ARE VARIABLES FOR DISK INFORMATION */
static int dataCount;
static int dataCurr;
static spm_poolNode_t stData;
static spm_poolNode_t *diskList;


/***********************************************************
 * routines to get disk information thru anpssc
 **********************************************************/

static int getTableRowCount(void)
{
    int						stat;
    char					respbuff[4096];
    spm_MsgPoolListReq_t    req;
    spm_MsgPoolListRsp_t    *rsp;
	struct timeval			timeout = {10, 0};
            
    /* Fill the request. */
    bzero((void *) &req, sizeof(spm_MsgPoolListReq_t));
    req.hdr.msgType = SPM_GET_POOL_LIST_REQ;
	strcpy(req.hdr.node, "\0");

    /* Send message to SPM. */
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "spm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, respbuff, sizeof(respbuff));

    /* Check response from SPM. */
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;
    rsp = (spm_MsgPoolListRsp_t *)respbuff;
    if (rsp->hdr.msgType != SPM_GET_POOL_LIST_RSP)
		return -1;
	if (rsp->numEntries <= 0)
		return -1;

	diskList = malloc((rsp->numEntries) * sizeof(spm_poolNode_t));
	if (diskList == NULL)
		return -1;

	memcpy(diskList, rsp->poolList, (rsp->numEntries) * sizeof(spm_poolNode_t));
	return (rsp->numEntries);
}

static int getTableRowData(int *index, spm_poolNode_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (diskList == NULL)
		return -1;

	memcpy(data, &diskList[(*index)-1], sizeof(spm_poolNode_t));
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case DISK_CTRL:
		strcpy(value, stData.ctrl);
		break;
	case DISK_MAKE:
		strcpy(value, stData.make);
		removeTrailingBlank(value);
		convertSpecCharToUrlChar(value);
		break;
	case DISK_MODEL:
		strcpy(value, stData.model);
		removeTrailingBlank(value);
		break;
	case DISK_MODELURL:
		strcpy(value, stData.model);
		removeTrailingBlank(value);
		convertSpecCharToUrlChar(value);
		break;
	case DISK_RAIDLVL:
		sprintf(value, "%d", stData.rL);
		break;
	case DISK_RAIDSTR:
		GetRaidLevelStr(stData.rL, value);
		break;
	case DISK_USED:
		sprintf(value, "%lld MB", GetMBfromBlock(stData.usedSize));
		break;
	case DISK_FREE:
		sprintf(value, "%lld MB", GetMBfromBlock(stData.freeSize));
		break;
	case DISK_FORE:
		sprintf(value, "%lld MB", GetMBfromBlock(stData.foreignSize));
		break;
	case DISK_OUTC:
		sprintf(value, "%lld MB", GetMBfromBlock(stData.ocSize));
		break;
	default:
		return -1;
	}

	if (value && !value[0])
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionDiskMain(void *r, int count, char **argname, char **argval)
{
	prepare_agile_template(r, NULL);

	/* initialize disk information variables */
	dataCount = 0;
	dataCurr = 0;
	diskList = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (diskList != NULL)
		free(diskList);
	return 1;
}
