/*  File Name  :    agilefsetdquomod.c 
 *
 *  Description:    HTTPD action functions for Modify Fileset Default Quota Action Page.
 *
 *  Created    :    3/21/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>
#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define FS_TYPE				1
#define NUM_OF_TAGS			1

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{FS_TYPE,		"args.typestr"},
};


#define PARAM_VOLID			1
#define PARAM_VOLNAME		2
#define PARAM_PATH			3
#define PARAM_ISOFT			4
#define PARAM_IHARD			5
#define PARAM_ITIME			6
#define PARAM_BSOFT			7
#define PARAM_BHARD			8
#define PARAM_BTIME			9
#define PARAM_TYPE			10
#define NUM_OF_PARAMS		10

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_VOLID,	"id"},
	{PARAM_VOLNAME,	"vn"},
	{PARAM_PATH,	"path"},
	{PARAM_ISOFT,	"isoft"},
	{PARAM_IHARD,	"ihard"},
	{PARAM_ITIME,	"itime"},
	{PARAM_BSOFT,	"bsoft"},
	{PARAM_BHARD,	"bhard"},
	{PARAM_BTIME,	"btime"},
	{PARAM_TYPE,	"type"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
static char bufpath[FS_PATH_MAX+1];
static char bufvol[VOL_MAX_NAME+1];
static char inType[10];
static uint64 inId;
static fs_QuotaBlk_t data;


/***********************************************************
 *  Main routines to handle Add Volume actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_TYPE:
			strcpy(inType, argval[argind]);
			break;
		case PARAM_VOLID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VOLNAME:
			strcpy(bufvol, argval[argind]);
			break;
		case PARAM_PATH:
			strcpy(bufpath, argval[argind]);
			break;
		case PARAM_ISOFT:
			sscanf(argval[argind], "%lld", &data.inodeSoftLimit);
			break;
		case PARAM_IHARD:
			sscanf(argval[argind], "%lld", &data.inodeHardLimit);
			break;
		case PARAM_ITIME:
			sscanf(argval[argind], "%d", &data.inodeTimeLimit);
			break;
		case PARAM_BSOFT:
			sscanf(argval[argind], "%lld", &data.blockSoftLimit);
			break;
		case PARAM_BHARD:
			sscanf(argval[argind], "%lld", &data.blockHardLimit);
			break;
		case PARAM_BTIME:
			sscanf(argval[argind], "%d", &data.blockTimeLimit);
			break;
		}
	}

	return 0;
}

static int DoSetQuota()
{
    int						stat;
    fs_MsgSetQuotaReq_t		req;
    fs_MsgSetQuotaResp_t	rsp;
	struct timeval			timeout = {60, 0};
    
    /* Fill the request. */
    bzero((void *) &req, sizeof(fs_MsgSetQuotaReq_t));
	req.msgHdr.requestType = FS_REQ_SET_QUOTA;
	req.fsid.whole_thing = inId;
	strcpy(req.path, bufpath);
	req.numberLimits = 1;
	data.id = 0;
	data.flag = 0;
	if (strcmp(inType, "grp") == 0)
		data.flag |= FS_QUOTA_DEFAULT | FS_QUOTA_GROUP;
	else if (strcmp(inType, "usr") == 0)
		data.flag |= FS_QUOTA_DEFAULT | FS_QUOTA_USER;
	else
		data.flag |= FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE;
	memcpy(&req.limits[0], &data, sizeof(fs_QuotaBlk_t));

    /* Send message to FS. */
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE, &timeout,
                        &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;
    if (rsp.msgHdr.requestType != FS_REQ_SET_QUOTA)
		return -1;
	if (rsp.msgHdr.fsStatus != FS_ERR_NONE)
		return -1;
	if (rsp.responses[0].status != FS_ERR_NONE)
		return -1;

	return 1;
}

static void HandleFSAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);
	res = DoSetQuota();
	if (res <= 0)
		strcpy(newfname, "error_fssetdefquota.html");
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case FS_TYPE:
		if (strcmp(inType, "grp") == 0)
			sprintf(value, "Group");
		else if (strcmp(inType, "usr") == 0)
			sprintf(value, "User");
		else
			strcpy(value, "");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionFsetDQuoMod(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle FileSet action */
	HandleFSAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
