/*  File Name  :    agilelunmain.c 
 *
 *  Description:    HTTPD action functions for LUN main page.
 *
 *  Created    :    3/4/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <eee-desc.h>
#include <scsi-api.h>
#include <sdm-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define STGDEV_SCSIID		1
#define STGDEV_WWN			2
#define STGDEV_LUN			3
#define STGDEV_WWNLUN		4
#define STGDEV_VENDOR		5
#define STGDEV_MODEL		6
#define STGDEV_STATE		7
#define STGDEV_ARGWWN		8
#define STGDEV_ARGLUN		9
//#define STGDEV_INDEX		10
//#define STGDEV_TYPE		11
//#define STGDEV_CATEG		12
//#define STGDEV_COLSPAN	13
#define NUM_OF_TAGS			9

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{STGDEV_SCSIID,	"sdm_devInfoRspMsg_t.id"},
	{STGDEV_WWN,	"sdm_devInfoRspMsg_t.wwn"},
	{STGDEV_LUN,	"sdm_devInfoRspMsg_t.lun"},
	{STGDEV_WWNLUN,	"sdm_devInfoRspMsg_t.wwn_lun"},
	{STGDEV_VENDOR,	"sdm_devInfoRspMsg_t.vendorName"},
	{STGDEV_MODEL,	"sdm_devInfoRspMsg_t.modelName"},
	{STGDEV_STATE,	"sdm_devInfoRspMsg_t.state"},
	{STGDEV_ARGWWN,	"args.wwn"},
	{STGDEV_ARGLUN,	"args.lun"},
	/*
	{STGDEV_INDEX,	"sdm_devInfoRspMsg_t.index"},
	{STGDEV_TYPE,	"sdm_devInfoRspMsg_t.devType"},
	{STGDEV_CATEG,	"sdm_devInfoRspMsg_t.category"},
	{STGDEV_COLSPAN,"priv.colspan"},
	*/
};


#define PARAM_WWN			1
#define PARAM_LUN			2
#define PARAM_ACT			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_WWN,		"wwn"},
	{PARAM_LUN,		"lun"},
	{PARAM_ACT,		"act"},
};


/* HERE ARE VARIABLES FOR STORAGE DEVICE LIST INFORMATION */
static uint64 inWwn;
static uint32 inLun;
static char inAct[10];
static int dataCount;
static int dataCurr;
static sdm_devInfoRspMsg_t stData;
static scsi_id_t	*idList;

#if 0
static char *devType_str[] = {"Disk",		"Tape",
					   "printer",	"processor",
					   "worm",		"cdrom",
					   "scanner",	"optical_disk",
					   "Changer",	"comm_dev",
					   "printer1",	"printer2",
					   "raid",		"ses",
					   "unknown",	"unknown1"
};
#endif

static char *devState_str[] = {"Closed", "Opened", "Closing", "Opening" };


/***********************************************************
 *  Main routines to handle Reset, Label actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_WWN:
			sscanf(argval[argind], "%lld", &inWwn);
			break;
		case PARAM_LUN:
			sscanf(argval[argind], "%d", &inLun);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		}
	}

	return 0;
}

static int DoResetDevice(int64 wwn, int32 lun)
{
    int                       stat;
    uint32                    rc=0;
    char                	  respbuff[1024];
    sdm_getSdmHandleReqMsg_t  hmsg;
    sdm_getSdmHandleRspMsg_t *hrsp;
    sdm_devResetReqMsg_t	  msg;
    sdm_devResetRspMsg_t     *rsp;
	uint32 				      sdmHandle;
	struct timeval			timeout = {5, 0};

	/* get SDM handle */
	hmsg.hdr.mid = SDM_MID_GET_SDMHANDLE_REQ;
    hmsg.wwn      = wwn;
    hmsg.lun      = lun;

    stat = sendAgileMsg(-1, &hmsg, sizeof(hmsg),
						"sdm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&timeout, respbuff, sizeof(respbuff));

	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;

    hrsp = (sdm_getSdmHandleRspMsg_t *)respbuff;
    if( hrsp->hdr.mid != SDM_MID_GET_SDMHANDLE_RSP ) 
		return -1;
    else if (hrsp->rCode == SDM_FAIL) 
		return -1;
	else 
		sdmHandle = hrsp->sdmHandle;
	if (sdmHandle == 0)
		return -1;

	/* reset device */
	bzero(&msg, sizeof(sdm_devResetReqMsg_t));
	msg.hdr.mid = SDM_MID_DEV_RESET_REQ;
	msg.handle  = sdmHandle;
	
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_devResetReqMsg_t),
						"sdm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&timeout, respbuff, sizeof(respbuff));
	if (rc < 0)
		return -1;
	else 
	if (rc == 0)
		return -1;

	rsp = (sdm_devResetRspMsg_t *)respbuff;
	if (rsp->hdr.mid != SDM_MID_DEV_RESET_RSP)
		return -1;
	else 
		return -1;

	return 1;
}

/*
static int DoLabelDevice(int64 wwn, int32 lun, char* label)
{
	return 1;
}
*/

static int HandleDevAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);

	if (!strcmp(inAct, "reset"))
	{
		res = DoResetDevice(inWwn, inLun);
		if (res <= 0)
			strcpy(newfname, "error_resetstgdev.html");
	}
	/*
	if (!strcmp(inAct, "label"))
	{
		res = DoLabelDevice(inWwn, inLun, inLabel);
		if (res <= 0)
			strcpy(newfname, "error_labelstgdev.html");
	}
	*/
	return 1;
}


/***********************************************************
 * routines to retrieve storage information thru anpssc
 **********************************************************/

static int getTableRowCount(void)
{
    int                 stat;
    int                 num_entries;
    char                respbuff[4096];
    sdm_devListReqMsg_t msg;
    sdm_devListRspMsg_t *rsp;
	struct timeval		timeout = {5, 0};

	memset(&msg, 0, sizeof(msg));
    msg.hdr.mid = SDM_MID_DEV_LIST_REQ;
	msg.type = SDM_DEVLIST_TAPE; //SDM_DEVLIST_ALL;
	msg.port = SDM_DEVLIST_PORTALL;

    stat = sendAgileMsg(-1, &msg, sizeof(msg),
						"sdm", NFX_SSC_UNIX, SSC_SLOT, 0,
                        &timeout, respbuff, sizeof(respbuff));

	if (stat < 0) {
		return -1;
	}
	else if (!stat) {
		return -1;
	}

    rsp = (sdm_devListRspMsg_t *)respbuff;
    if (rsp->hdr.mid != SDM_MID_DEV_LIST_RSP) {
		return -1;
	}

	num_entries = rsp->numEntries;
	if (num_entries == 0)
		return 0;

	if (idList != 0)
		free(idList);
	idList = malloc(num_entries * sizeof(scsi_id_t));
	memcpy(idList, rsp->idList, num_entries * sizeof(scsi_id_t));

	return num_entries;
}

static int getTableRowData(int *index, sdm_devInfoRspMsg_t *devInfo)
{
    int                 stat;
    sdm_devInfoReqMsg_t msg;
	struct timeval		timeout = {5, 0};

	if (*index < 1 || *index > dataCount)
		return -1;
	if (idList == NULL)
		return -1;

    msg.hdr.mid = SDM_MID_DEV_INFO_REQ;
    msg.id      = idList[(*index) - 1];

	memset(devInfo, 0, sizeof(sdm_devInfoRspMsg_t));
    stat = sendAgileMsg(-1, &msg, sizeof(msg),
						"sdm", NFX_SSC_UNIX, SSC_SLOT, 0,
                        &timeout, devInfo, sizeof(sdm_devInfoRspMsg_t));

	if (stat < 0) {
		return -1;
	}
	else if (!stat) {
		return -1;
	}

    if (devInfo->hdr.mid != SDM_MID_DEV_INFO_RSP) {
		return -1;
    }
	else if(devInfo->rc == SDM_FAIL) {
		return -1;
    }

	return 0;
}



/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case STGDEV_SCSIID:
		sprintf(value, "%lld", stData.id);
		break;
	case STGDEV_WWN:
		sprintf(value, "%lld", stData.wwn);
		break;
#if 0
	case STGDEV_LUN:
		lun = SCSI_ID_GET_LUN(stData.id); 
		sprintf(value, "%d", lun);
		break;
	case STGDEV_WWNLUN:
		lun = SCSI_ID_GET_LUN(stData.id); 
		sprintf(tempbuf, ":%d", lun);
		GetWwnStr(stData.wwn, value);
		strcat(value, tempbuf);
		break;
#endif
	case STGDEV_VENDOR:
		strcpy(value, stData.st.vendor);
		break;
	case STGDEV_MODEL:
		strcpy(value, stData.st.model);
		break;
	case STGDEV_STATE:
		strcpy(value, devState_str[stData.state]);
		break;
	case STGDEV_ARGWWN:
		sprintf(value, "%lld", inWwn);
		break;
	case STGDEV_ARGLUN:
		sprintf(value, "%d", inLun);
		break;
	/*
	case STGDEV_INDEX:
		sprintf(value, "%d", dataCurr-1);
		break;
	case STGDEV_TYPE:
		strcpy(value, devType_str[stData.devType]);
		break;
	case STGDEV_CATEG:
		strcpy(value, "N/A");
		break;
	case STGDEV_COLSPAN:
		pri = GetUserPrivilege();
		if (pri == PRIV_DIAG)
			strcpy(value, "2");
		else if (pri == PRIV_FULL)
			strcpy(value, "1");
		break;
	*/
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionLunMain(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle Reset actions */
	if (count > 0)
		HandleDevAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;
	idList = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (idList != NULL)
		free(idList);
	return 1;
}
