/*  File Name  :    agilenetworkarp.c
 *
 *  Description:    HTTPD action functions for displaying network arp table.
 *
 *  Created    :    2/1/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <nfx-defs.h>
#include <nfx-types.h>

#include <ipmd-lib.h>
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NWARP_ADDR			1
#define NWARP_MAC			2
#define NWARP_AGE			3
#define NWARP_IFNAME		4
#define NUM_OF_TAGS			4

#define NUM_ARP_ENTRIES		20

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWARP_ADDR,		"ipm_arp_t.arp_addr"},
	{NWARP_MAC,		"ipm_arp_t.arp_mac"},
	{NWARP_AGE,		"ipm_arp_t.arp_age"},
	{NWARP_IFNAME,	"ipm_arp_t.arp_ifname"},
};


/* HERE ARE VARIABLES FOR STORAGE DEVICE LIST INFORMATION */
static int dataCount;
static int dataCurr;
static ipm_arp_t *stArpTable;
static ipm_arp_t stData;


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getTableRowCount(void)
{
	int32	res;
	int count, need;

	/* Try to get ARP Table with maxi. 20 entries */
	stArpTable = (ipm_arp_t *) malloc(NUM_ARP_ENTRIES * sizeof(ipm_arp_t));
	if (stArpTable == NULL)
		return -1;
	res = ipmd_GetArpTbl(stArpTable, NUM_ARP_ENTRIES*sizeof(ipm_arp_t), 
							&count, &need);
	if (res == IPM_GET_ERR)
		return -1;
	else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
	{
		// Need a larger buffer
		free(stArpTable);
		stArpTable = (ipm_arp_t *) malloc(need);
		if (stArpTable == NULL)
			return -1;

		res = ipmd_GetArpTbl(stArpTable, need, &count, &need);
		if (res == IPM_GET_ERR)
			return -1;
		else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
			return -1;
	}

	if (count < 0)
		return -1;
	return count;
}

static int getTableRowData(int *index, ipm_arp_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (stArpTable == NULL)
		return -1;

	memcpy(data, &stArpTable[(*index)-1], sizeof(ipm_arp_t));
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NWARP_ADDR:
		GetIpAddressStr(stData.arp_addr, value);
		break;
	case NWARP_MAC:
		GetMacAddressStr(stData.arp_mac, value);
		break;
	case NWARP_AGE:
		sprintf(value, "%d", stData.arp_age);
		break;
	case NWARP_IFNAME:
		strcpy(value, stData.arp_ifname);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) < 0) {
				return -1;
			}
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNetworkARP(void *r)
{
	prepare_agile_template(r, NULL);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;
	stArpTable = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (stArpTable != NULL)
		free(stArpTable);
	return 1;
}
