/*  File Name  :    agilenfsdel.c
 *
 *  Description:    HTTPD action functions for delete share/export.
 *
 *  Created    :    3/22/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>

#include <anpssc-api.h>

#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>
#include <rpc.h>
#include <mount.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_ID				1
#define VOL_NAME			2
#define PATH_NAME			3
#define PATH_IP				4
#define NUM_OF_TAGS			4

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_ID,			"args.vol_id"},
	{VOL_NAME,			"args.vol_name"},
	{PATH_NAME,			"args.path"},
	{PATH_IP,			"args.ip"},
};


#define PARAM_VOLID			1
#define PARAM_VOLNAME		2
#define PARAM_PATH			3
#define PARAM_IP			4
#define NUM_OF_PARAMS		4

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_VOLID,	"id"},
	{PARAM_VOLNAME,	"vn"},
	{PARAM_PATH,	"path"},
	{PARAM_IP,		"ip"},
};


/* HERE ARE VARIABLES FOR SHARE/EXPORT INFORMATION */
static char bufpath[MAX_NAME_LEN+1];
static char bufvol[VOL_MAX_NAME+1];
static char bufip[32];
static uint64 inId;


/***********************************************************
 * routines to retrieve nfs information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_VOLID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VOLNAME:
			strcpy(bufvol, argval[argind]);
			break;
		case PARAM_PATH:
			strcpy(bufpath, argval[argind]);
			break;
		case PARAM_IP:
			strcpy(bufip, argval[argind]);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_ID:
		sprintf(value, "%lld", inId);
		break;
	case VOL_NAME:
		strcpy(value, bufvol);
		break;
	case PATH_NAME:
		strcpy(value, bufpath);
		break;
	case PATH_IP:
		strcpy(value, bufip);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNfsDel(void *r, int count, char **argname, char **argval)
{
	/* initialize export/share information variables */
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
