/*  File Name  :    agilesshotadd.c
 *
 *  Description:    HTTPD action functions for add snapshot.
 *
 *  Created    :    3/18/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_NAME			1
#define VOL_ID				2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_NAME,	"args.vol_name"},
	{VOL_ID,	"args.vol_id"},
};


#define PARAM_ID			1
#define PARAM_VN			2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ID,		"id"},
	{PARAM_VN,		"vn"},
};


/* HERE ARE VARIABLES FOR SNAPSHOT INFORMATION */
static int64 inId;
static char inName[VOL_MAX_NAME+1];


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VN:
			strcpy(inName, argval[argind]);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_NAME:
		strcpy(value, inName);
		break;
	case VOL_ID:
		sprintf(value, "%lld", inId);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSshotAdd(void *r, int count, char **argname, char **argval)
{
	prepare_agile_template(r, NULL);

	getInputParams(count, argname, argval);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
