/*  File Name  :    agilesyswz1.c
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 1.
 *
 *  Created    :    4/5/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <dirent.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SYS_HOST			1
#define SYS_ADMIN			2
#define SYS_PWD				3
#define SYS_ADMPRVG			4
#define SYS_TZONELST		5
#define SYS_LANGLST			6
#define SYS_TM_MIN			7
#define SYS_TM_HOUR			8
#define SYS_TM_MDAY			9
#define SYS_TM_MON			10
#define SYS_TM_YEAR			11
#define NUM_OF_TAGS			11

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_HOST,		"system.hostname"},
	{SYS_ADMIN,		"system.adminname"},
	{SYS_PWD,		"system.adminpwd"},
	{SYS_ADMPRVG,	"system.adminprvg"},
	{SYS_TZONELST,	"system.timezone.list"},
	{SYS_LANGLST,	"system.language.list"},
	{SYS_TM_MIN,	"system.tm.min"},
	{SYS_TM_HOUR,	"system.tm.hour"},
	{SYS_TM_MDAY,	"system.tm.mday"},
	{SYS_TM_MON,	"system.tm.mon"},
	{SYS_TM_YEAR,	"system.tm.year"},
};


#define PARAM_ACT			1
#define NUM_OF_PARAMS		1

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
};


/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
typedef struct _system_config_t
{
    char		name[128];
    char		adminName[128];
    char		adminPwd[128];
	char		timeZone[256];
	char		language[32];
	int			adminPrvg;
#if 0
	int			timeZone;
	int			language;
#endif
	int			tm_min;
	int			tm_hour;
	int			tm_mday;
	int			tm_mon;
	int			tm_year;
} system_config_t;

typedef struct _time_zone_t
{
    char		zone[256];
	int			type;
} time_zone_t;

typedef struct _language_t
{
    char		lang[256];
	int			type;
} language_t;

static char inAct[10];
static int	res;
static system_config_t stData;
int numTimeZone = 0;
int numTZAlloc = 0;
static time_zone_t* timeZones = NULL;
int numLanguage = 0;
int numLangAlloc = 0;
static language_t* languages = NULL;
static char *monthname[12] =
{
	"Jaunary",
	"February",
	"March",
	"Arpil",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
};

#define TIMEZONEPATH "/usr/share/zoneinfo"
#define TIMEZONELINK "/etc/localtime"


static char *groupNames[] =
{
	"View", 
	"Add", 
	"Change", 
	"Full", 
	"Diags"
};


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	memset(inAct, 0, 10);
	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType)
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetAdminConfigs(char* buf)
{
	int from = 0;
	char tmpbuf[64];

	from = GetDelimiteredParam(from, buf, stData.adminName);
	from = GetDelimiteredParam(from, buf, stData.adminPwd);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* privilege */
	stData.adminPrvg = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, stData.timeZone);
	from = GetDelimiteredParam(from, buf, stData.language);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* month */
	stData.tm_mon = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* day */
	stData.tm_mday = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* year */
	stData.tm_year = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* hour */
	stData.tm_hour = atoi(tmpbuf);
	from = GetDelimiteredParam(from, buf, tmpbuf);		/* minute */
	stData.tm_min = atoi(tmpbuf);
}

static void GetSystemConfig(void* r, int from)
{
	char ip[20];
	int rc;
	char reqbuf[256];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			tval;

	res = -1;

	memset(&stData, 0, sizeof(system_config_t));

	// Get System Name
	GetSystemHostInfo(r, ip, stData.name);

	// Get Admin Name, Pwd
    //...... Fill the request.
    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;

	cfgtype = (haSystemConfigType_t *) req->data;

	cfgtype->from = from;
	cfgtype->step = 1;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

	tval.tv_sec = 15;
	tval.tv_usec = 0;

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &tval, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if (rsp->rspstatus != HARSP_OK)
		return;

	config = (haSystemConfig_t *) rsp->data;
	GetAdminConfigs(config->configs);

	res = 1;
}

static int AddNewZoneName(char *name)
{
time_zone_t *newTimeZones = NULL;

	if (numTZAlloc == 0)
	{
		timeZones = malloc(16 * sizeof(time_zone_t));
		if (timeZones == NULL)
			return -1;
		numTZAlloc = 16;
	}
	else if (numTimeZone >= numTZAlloc)
	{

		newTimeZones = realloc(timeZones, (numTZAlloc + 16) * sizeof(time_zone_t));
		if (newTimeZones == NULL)
			return -1;

		numTZAlloc += 16;
		timeZones = newTimeZones;
	}

	newTimeZones = &(timeZones[numTimeZone]);
	strcpy(newTimeZones->zone, name);
	newTimeZones->type = ++numTimeZone;

	return 1;
}

static int BuildZoneInfoList(char *path, char *prepend)
{
struct dirent *dp;
DIR *dirp;
char newpath[2560]; /* should handle 10 level deep... */
char newprepend[2560];
int count;

	dirp = opendir(path);
	if (dirp == NULL)
		return 0;

	count = 0;
	while ((dp = readdir(dirp)) != NULL)
	{
		if (dp->d_type & DT_DIR)
		{
			if (!strcmp(dp->d_name, ".") || !strcmp(dp->d_name, ".."))
				continue;

			sprintf(newpath, "%s/%s", path, dp->d_name);
			if (prepend[0])
				sprintf(newprepend, "%s/%s", prepend, dp->d_name);
			else
				sprintf(newprepend, "%s", dp->d_name);

			count += BuildZoneInfoList(newpath, newprepend);
		}
		else
		{
			if(prepend[0])
				sprintf(newpath, "%s/%s", prepend, dp->d_name);
			else
				sprintf(newpath, "%s", dp->d_name);
			if (AddNewZoneName(newpath) > 0)
				count++;
		}
	}

	closedir(dirp);

	return count;
}

static int BuildLanguageList()
{
	/* let's just do English for now... */
	languages = malloc(sizeof(language_t));
	if (languages == NULL)
		return -1;

	strcpy(languages->lang, "English");
	languages->type = 1;

	numLanguage = 1;
	numLangAlloc = 1;

	return 1;
}

static void GetSystemData(void* r)
{
	int from;

#if 0
	if (!strcmp(inAct, "prev"))
		from = 0;	// prev action, get data from temp file that we just created
	else
		from = 1;	// init or cancel action, get data from perminate file
#endif
	from = 0;		// always get from temp file (should I do this?)

	// Get TimeZone List
	//..........
	numTimeZone = 0;
	BuildZoneInfoList("/usr/share/zoneinfo", "");
	//timeZones = malloc(numTimeZone * sizeof(timeZones));

	// Get Supported Language List
	//..........
	numLanguage = 0;
	BuildLanguageList();
	//languages = malloc(numLanguage * sizeof(language_t));

	// Get System Configuration
	// This has to be done last after timezone list and
	// Language list are built...
	GetSystemConfig(r, from);

}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
    char   tempbuf[64];

#if 0
	if (res <= 0)
		return -1;
#endif

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType)
	{
	case SYS_HOST:
		strcpy(value, stData.name);
		break;
	case SYS_ADMIN:
		strcpy(value, stData.adminName);
		break;
	case SYS_PWD:
		strcpy(value, stData.adminPwd);
		break;
	case SYS_ADMPRVG:
		value[0] = '\0';
		for (i = 0; i < 5; i++)
		{
			if (i == stData.adminPrvg)
				sprintf(tempbuf, "<option value=\"%d\" selected>%s\n",
												i, groupNames[i]);
			else
				sprintf(tempbuf, "<option value=\"%d\">%s\n",
												i, groupNames[i]);
			strcat(value, tempbuf);
		}
		break;
	case SYS_TZONELST:
		value[0] = '\0';
		if (numTimeZone <= 0 || timeZones == NULL)
			strcpy(value, "&nbsp;");
		else
		{
			for (i = 0; i < numTimeZone; i ++)
			{
#if 0
/* discard this 4/23/02 */
				sprintf(tempbuf, "<option value=\"%d\"", timeZones[i].type);
				if (timeZones[i].type == stData.timeZone)
					strcat(tempbuf, " selected>");
				else
					strcat(tempbuf, ">");
				strcat(value, tempbuf);
				strcat(value, timeZones[i].zone);
				strcat(value, "</option>\n");
#endif
				if (!strcmp(timeZones[i].zone, stData.timeZone))
					sprintf(tempbuf, "<option selected>%s\n", timeZones[i].zone);
				else
					sprintf(tempbuf, "<option>%s\n", timeZones[i].zone);
				strcat(value, tempbuf);
			}
		}
		break;
	case SYS_LANGLST:
		if (numLanguage <= 0 || languages == NULL)
			strcpy(value, "&nbsp;");
		else
		{
			strcpy(value, "");
			for (i = 0; i < numLanguage; i ++)
			{
#if 0
				sprintf(tempbuf, "<option value=\"%d\"", languages[i].type);
				if (languages[i].type == stData.language)
					strcat(tempbuf, " selected>");
				else
					strcat(tempbuf, ">");
				strcat(value, tempbuf);
				strcat(value, languages[i].lang);
				strcat(value, "</option>\n");
#endif
				if (!strcmp(languages[i].lang, stData.language))
					sprintf(tempbuf, "<option selected>%s\n", languages[i].lang);
				else
					sprintf(tempbuf, "<option>%s\n", languages[i].lang);
				strcat(value, tempbuf);
			}
		}
		break;
	case SYS_TM_MIN:
		value[0] = '\0';
		for (i = 0; i < 60; i++)
		{
			sprintf(tempbuf, "<option value=\"%02d\"", i);
			if (i == stData.tm_min)
				strcat(tempbuf, " selected>");
			else
				strcat(tempbuf, ">");
			strcat(value, tempbuf);

			// sprintf(tempbuf, "%02d</option>\n", i);
			sprintf(tempbuf, "%02d\n", i);
			strcat(value, tempbuf);
		}
		break;
	case SYS_TM_HOUR:
		value[0] = '\0';
		for (i = 0; i < 24; i++)
		{
			sprintf(tempbuf, "<option value=\"%02d\"", i);
			if (i == stData.tm_hour)
				strcat(tempbuf, " selected>");
			else
				strcat(tempbuf, ">");
			strcat(value, tempbuf);

			// sprintf(tempbuf, "%02d</option>\n", i);
			sprintf(tempbuf, "%02d\n", i);
			strcat(value, tempbuf);
		}
		break;
	case SYS_TM_MDAY:
		value[0] = '\0';
		for (i = 1; i <= 31; i++)
		{
			sprintf(tempbuf, "<option value=\"%d\"", i);
			if (i == stData.tm_mday)
				strcat(tempbuf, " selected>");
			else
				strcat(tempbuf, ">");
			strcat(value, tempbuf);

			// sprintf(tempbuf, "%02d</option>\n", i);
			sprintf(tempbuf, "%02d\n", i);
			strcat(value, tempbuf);
		}
		break;
	case SYS_TM_MON:
		value[0] = '\0';
		for (i = 0; i < 12; i++)
		{
			sprintf(tempbuf, "<option value=\"%d\"", i);
			if (i == stData.tm_mon)
				strcat(tempbuf, " selected>");
			else
				strcat(tempbuf, ">");
			strcat(value, tempbuf);

			// sprintf(tempbuf, "%s</option>\n", monthname[i]);
			sprintf(tempbuf, "%s\n", monthname[i]);
			strcat(value, tempbuf);
		}
		break;
	case SYS_TM_YEAR:
		value[0] = '\0';
		for (i = 2000; i < 2020; i++)
		{
			sprintf(tempbuf, "<option value=\"%d\"", i);
			if (i == stData.tm_year)
				strcat(tempbuf, " selected>");
			else
				strcat(tempbuf, ">");
			strcat(value, tempbuf);

			// sprintf(tempbuf, "%d</option>\n", i);
			sprintf(tempbuf, "%d\n", i);
			strcat(value, tempbuf);
		}
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizard1(void *r, int count, char **argname, char **argval)
{
	/* initialize system information variables */
	getInputParams(count, argname, argval);
	GetSystemData(r);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	if (timeZones != NULL)
	{
		free(timeZones);
		timeZones = NULL;
		numTimeZone = 0;
		numTZAlloc = 0;
	}
	if (languages != NULL)
	{
		free(languages);
		languages = NULL;
		numLanguage = 0;
		numLangAlloc = 0;
	}

	return 1;
}
