/*  File Name  :    agilevoladd.c
 *
 *  Description:    HTTPD action functions for add volume.
 *
 *  Created    :    3/5/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <spm-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_LUNLIST			1
#define VOL_VENLIST			2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_LUNLIST,	"luntypelist"},
	{VOL_VENLIST,	"vendorlist"},
};

static spm_poolNode_t *poolList;
static int numPool;


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int InPool(int count, spm_poolNode_t* node)
{
	int i;
	spm_poolNode_t *curr;

	for (i = 0; i < count; i ++)
	{
		curr = &poolList[i];
		if (strcmp(node->make, curr->make))
			continue;
		if (strcmp(node->model, curr->model))
			return 1;
		/*
			continue;
		if (node->rL == curr->rL)
			return 1;
		*/
	}
	return 0;
}

static int GetLunTypeList()
{
    int						stat, i;
    char					respbuff[4096];
    spm_MsgPoolListReq_t	req;
    spm_MsgPoolListRsp_t	*rsp;
	spm_poolNode_t			*node;
	numPool = 0;
            
    //Fill the request. 
    bzero((void *) &req, sizeof(spm_MsgPoolListReq_t));
    req.hdr.msgType = SPM_GET_POOL_LIST_REQ;
    strcpy(req.hdr.node, "\0");

    // Send message to SPM. 
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "spm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        NULL,
                        respbuff, sizeof(respbuff));

    // Check response from SPM. 
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;
    rsp = (spm_MsgPoolListRsp_t *)respbuff;
    if (rsp->hdr.msgType != SPM_GET_POOL_LIST_RSP)
		return -1;
	if (rsp->numEntries <= 0)
		return -1;

	poolList = malloc((rsp->numEntries) * sizeof(spm_poolNode_t));
	for (i = 0; i < rsp->numEntries; i ++)
	{
		node = &(rsp->poolList[i]);
		if (!InPool(numPool, node))
		{
			memcpy(&poolList[numPool], node, sizeof(spm_poolNode_t));
			numPool ++;
		}
	}
	return 1;
}

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	char buf[128], buf2[10];
	spm_poolNode_t	*node;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_LUNLIST:
		strcpy(value, "");
		for (i = 0; i < numPool; i ++)
		{
			node = &poolList[i];
			strcpy(buf, node->make);
			removeTrailingBlank(buf);
			strcat(buf, ",");

			strcat(buf, node->model);
			removeTrailingBlank(buf);
			strcat(buf, ",");

			GetRaidLevelStr(node->rL, buf2);
			strcat(buf, buf2);
fprintf(stderr, "LUN=%s\n", buf);

			strcat(value, "<option value=\"");
			strcat(value, buf);
			if (i == 0)
				strcat(value, "\" selected>");
			else
				strcat(value, "\">");
			strcat(value, buf);
			strcat(value, "</option>");
		}
		break;
	case VOL_VENLIST:
        strcpy(value, "<option value=\"any\" selected>Any</option>");
		for (i = 0; i < numPool; i ++)
		{
			node = &poolList[i];
			strcpy(buf, node->make);
			removeTrailingBlank(buf);
			strcat(buf, ",");
			strcat(buf, node->model);
			removeTrailingBlank(buf);

			strcat(value, "<option value=\"");
			strcat(value, buf);
			strcat(value, "\">");
			strcat(value, buf);
			strcat(value, "</option>");
		}
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionVolAdd(void *r)
{
	prepare_agile_template(r, NULL);

	GetLunTypeList();

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
