/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cm-bmfpga.c,v 1.21 2002/11/04 18:10:29 maximk Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <exports/cm-api.h>
#include <sys/bmfpga.h>
#include "cm.h"
#if !defined(PROM) && !defined(FCNIM_TM)
#include "exports/event-api.h"
#endif  /* !PROM */

#undef DEBUG_PRINTF

#ifdef DEBUG_PRINTF
#define DPRINTF(_x)         printf _x 
#else
#define DPRINTF(_x)
#endif

#ifdef SSC_MGMT
volatile bmfpga_master_t	* bmfpga;
#else
volatile bmfpga_slave_t		* bmfpga;
#endif

uchar8	bmfpga_str[CM_VER_STRING_LEN];

uint32	bmfpga_status = 0;			/* slave status register is write only */

/*-----------------------------------------------------------------
 * Name:	bmfpga_swap()
 *
 * Description: sibyte runs generic bus in big endian, must swap
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
uint32
bmfpga_swap(
	uint32	t
)
{
#if defined(SIBYTE_BC)
	uint32	a,b,c,d;

	a = (t & 0x000000FF) << 24;
	b = (t & 0x0000FF00) << 8;
	c = (t & 0x00FF0000) >> 8;
	d = (t & 0xFF000000) >> 24;

	return(a | b | c | d);
#else
	return(t);
#endif
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_init()
{
	bzero(bmfpga_str, CM_VER_STRING_LEN);

#ifdef SSC_MGMT
	bmfpga = (bmfpga_master_t *)HW_BMFPGA_BASE;
    sprintf(bmfpga_str, "BMFPGA : Version %d, Chassis Id: %d",
            (bmfpga->board_status & BMFPGA_BOARD_STATUS_VERSION_MASK) >>
                                    BMFPGA_BOARD_STATUS_VERSION_SHIFT,
            ((bmfpga->board_status & BMFPGA_BOARD_STATUS_CHASSIS_ID_MASK) >>
                                    BMFPGA_BOARD_STATUS_CHASSIS_ID_SHIT));
#endif

#if defined(FCNIM) || defined(PROM_R7000)
	if (CM_IS_FC_CTRL) {      // control cpu is cpuid 3 (fc slice 0)
		bmfpga = (bmfpga_slave_t *)PHYS_TO_K1(HW_BMFPGA_BASE);
		sprintf(bmfpga_str, "BMFPGA : Version %d",
					(bmfpga->board_status & BMFPGA_BOARD_STATUS_VERSION_MASK) >>
											BMFPGA_BOARD_STATUS_VERSION_SHIFT);
	}
#endif

#if defined(NFP_TXRX) || defined(PROM_SIBYTE_BC)
	if (isTXRX()) {
		bmfpga = (bmfpga_slave_t *)PHYS_TO_K1(HW_BMFPGA_BASE);
		sprintf(bmfpga_str, "BMFPGA : Version %d",
											(bmfpga_swap(bmfpga->version)));
	}
#endif
}

#if defined(NFP_TXRX) || defined(FCNIM) || defined(PROM)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_setCardTemp(
	uint32	temp
)
{
	DPRINTF(("bmfpgaSetCardTemp, %d\n", temp));

	bmfpga_status &= ~BMFPGA_SLOT_STATUS_TEMP_MASK;

	temp &= (BMFPGA_SLOT_STATUS_TEMP_MASK >> BMFPGA_SLOT_STATUS_TEMP_SHIFT);

	bmfpga_status |= (temp << BMFPGA_SLOT_STATUS_TEMP_SHIFT);

#if 0
	if (temp >= CM_TEMP_CRITICAL)
		bmfpga_status |= (BMFPGA_SLOT_STATUS_TEMP_CRITICAL <<
												BMFPGA_SLOT_STATUS_TEMP_SHIFT);
	else if (temp >= CM_TEMP_HIGH)
		bmfpga_status |= (BMFPGA_SLOT_STATUS_TEMP_ABOVE_NORMAL <<
												BMFPGA_SLOT_STATUS_TEMP_SHIFT);
	else if (temp >= CM_TEMP_NORMAL)
		bmfpga_status |= (BMFPGA_SLOT_STATUS_TEMP_NORMAL <<
												BMFPGA_SLOT_STATUS_TEMP_SHIFT);
	else
		bmfpga_status |= (BMFPGA_SLOT_STATUS_TEMP_BELOW_NORMAL <<
												BMFPGA_SLOT_STATUS_TEMP_SHIFT);
#endif

	bmfpga->status = bmfpga_swap(bmfpga_status);
	delay_us(1000);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_setCardPower(
	uint32	power
)
{
	DPRINTF(("bmfpgaSetCardPower, %d\n", power));

	bmfpga->power_out = bmfpga_swap(power);
	delay_us(1000);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_setCardType(
	uint32	type
)
{
	DPRINTF(("bmfpgaSetCardType, %d\n", type));

	bmfpga_status &= ~BMFPGA_SLOT_STATUS_BOARD_TYPE_MASK;

	if (type == CM_CARD_TYPE_FCNIM)
		bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_FCNIM <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_CHEETAH)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_CHEETAH <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_GARFIELD)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_GARFIELD <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_SSC)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_SSC <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);

	bmfpga->status = bmfpga_swap(bmfpga_status);
	delay_us(1000);
}


void
bmfpga_initCardType(
	uint32	type
)
{
	DPRINTF(("bmfpgaInitCardType, %d\n", type));

	bmfpga_status &= ~BMFPGA_SLOT_STATUS_BOARD_TYPE_MASK;

	if (type == CM_CARD_TYPE_FCNIM)
		bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_FCNIM <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_CHEETAH)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_CHEETAH <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_GARFIELD)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_GARFIELD <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
	else if (type == CM_CARD_TYPE_SSC)
			bmfpga_status |= (BMFPGA_SLOT_STATUS_BOARD_TYPE_SSC <<
											BMFPGA_SLOT_STATUS_BOARD_TYPE_SHIFT);
}

#endif	/* NFP_TXRX || FCNIM || PROM */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpgaSetCPUState(
	uint32	state
)
{
	DPRINTF(("bmfpgaSetCPUState, 0x%x\n", state));

#ifdef FCNIM
	bmfpga_status &= ~(BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK);

    bmfpga_status |= state;

	bmfpga->status = bmfpga_status;

	cm_slot_status = bmfpga_status;
#endif

#if defined(NFP_TXRX)
	bmfpga_status &= ~(BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK);

	bmfpga_status |= state;
	bmfpga->status = bmfpga_swap(bmfpga_status);

	cm_slot_status = bmfpga_status;
#endif

#ifdef PROM_R7000
	bmfpga_status &= ~(BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK);

	bmfpga_status |= state;
	bmfpga->status = bmfpga_status;

	cm_slot_status = bmfpga_status;
#endif

#ifdef PROM_SIBYTE_BC
        uint32 cpu;        
        if(isFP()) 
            cpu = NFX_NFP_FP1;
        else
            cpu = NFX_NFP_TXRX1;

	bmfpga_status &= ~(BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK << (cpu*4));

	bmfpga_status |= (state << (cpu*4));
	bmfpga->status = bmfpga_swap(bmfpga_status);

	cm_slot_status = bmfpga_status;
#endif
}

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
uint32
bmfpgaGetNimSlot(
	uint32	slot
)
{
	return slot-1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpgaResetCard(
	uint32	slot
)
{
	volatile uint32	* control;
	uint32			nim_slot;
	uint32			cpu;

	DPRINTF(("bmfpgaResetCard, slot %d\n", slot));

	nim_slot = bmfpgaGetNimSlot(slot);

	control = &(bmfpga->nim_slot[nim_slot].control);

	*control = BMFPGA_SLOT_CONTROL_GLOBAL_ENABLE;
	delay_us(1000);
	*control = BMFPGA_SLOT_CONTROL_SLOT_COLD_START_MASK;
	delay_us(1000);
	*control = BMFPGA_SLOT_CONTROL_SLOT_COLD_START_MASK |
									BMFPGA_SLOT_CONTROL_SLOT_WARM_START_MASK;

	cm.slot[slot].control = BMFPGA_SLOT_CONTROL_SLOT_COLD_START_MASK |
							BMFPGA_SLOT_CONTROL_SLOT_WARM_START_MASK;

	for (cpu = 0; cpu < cm.slot[slot].num_cpus; cpu++)
	{
#ifndef PROM
		if (cm.slot[slot].cpu[cpu].state == CM_SLICE_STATE_RUNTIME)
            cmSendCpuEvent(slot, cpu, EVENT_STATE_DOWN);
#endif  /* !PROM */

		cm.slot[slot].cpu[cpu].state = CM_SLICE_STATE_RUNTIME_REBOOT;
	}
	cmSendInfoResp(0, 0);

	delay_us(1000);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpgaResetCPU(
	uint32	slot,
	uint32	cpu
)
{
	volatile uint32	* control;
	uint32			nim_slot;
    uint32          slice;

	DPRINTF(("bmfpgaResetCPU, slot %d, cpu %d\n", slot, cpu));

	nim_slot = bmfpgaGetNimSlot(slot);

	control = &(bmfpga->nim_slot[nim_slot].control);

    if (slot == NFX_SLOTTYPE_FC)
        slice = CM_FC_CPUID2SLICE(cpu); 
    else
        slice = cpu;

	cm.slot[slot].control &= ~((BMFPGA_SLOT_CONTROL_SLICE_COLD_START_MASK |
                    BMFPGA_SLOT_CONTROL_SLICE_WARM_START_MASK) << (slice*2));
	*control = cm.slot[slot].control;
	delay_us(1000);
	cm.slot[slot].control |=
                    (BMFPGA_SLOT_CONTROL_SLICE_COLD_START_MASK << (slice*2));
	*control = cm.slot[slot].control;
	delay_us(1000);
	cm.slot[slot].control |=
                    (BMFPGA_SLOT_CONTROL_SLICE_WARM_START_MASK << (slice*2));
	*control = cm.slot[slot].control;

#ifndef PROM
	if (cm.slot[slot].cpu[cpu].state == CM_SLICE_STATE_RUNTIME)
		cmSendCpuEvent(slot, cpu, EVENT_STATE_DOWN);
#endif  /* !PROM */

	cm.slot[slot].cpu[cpu].state = CM_SLICE_STATE_RUNTIME_REBOOT;
	cmSendInfoResp(0, 0);

	delay_us(1000);
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

void
bmfpga_nmi_clear(uint32 val)
{
#if !defined(SIBYTE_BC) && !defined(SSC_MGMT)
	*(volatile uint32 *)PHYS_TO_K1(HW_BMFPGA_NMI_CLEAR) = val;
#endif
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_sendMessage(
	uint32	dest_slot,
	uint32	dest_cpu,
	uint32	* msg,
	uint32	num_bytes
)
{
#if 0
	uint32	num_words = (num_bytes + 3)/4;
	uint32	i;

	for (i=0; i<num_words; i++) {
		if (msg[i] & BMFPGA_OWN)
			panic("Error sending bmfpga message, w = 0x%x\n", msg[i]);
#ifdef SSC_MGMT
		bmfpga->nim_slot[dest_slot].message_out = BMFPGA_OWN | msg[i];
		while (bmfpga->nim_slot[dest_slot].message_out & BMFPGA_OWN);
#else
		bmfpga->message_out = BMFPGA_OWN | msg[i];
		while (bmfpga->message_out & BMFPGA_OWN);
#endif
	}
#else
#if 0
	printf("bmfpga_sendMessage, slot %d, cpu %d!!!!!!!!!!!!!!!!!!!!!!!!!!\n",
														dest_slot, dest_cpu);
#endif
#endif
}

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
bmfpgaIsModulePresent(
	int32	slot
)
{
	uint32	nim_slot;

	nim_slot = bmfpgaGetNimSlot(slot);
	
	return (bmfpga->module_presence & (BMFPGA_MOD_PRESENT_BIT << nim_slot));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
void
bmfpga_clearPendingInts()
{
	uint32	garbage;
	uint32	i;

	for (i=0; i<BMFPGA_NIM_SLOTS; i++) {
		garbage = bmfpga->nim_slot[i].status;
		garbage = bmfpga->nim_slot[i].message_in;
	}
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
uint32
bmfpga_get_busclock()
{
	if (isSSC()) {
		volatile  bmfpga_master_t * bmfpga = (volatile bmfpga_master_t *)
													PHYS_TO_K1(HW_BMFPGA_BASE);

		if (((bmfpga->board_status & BMFPGA_BOARD_STATUS_CLK_MODE_MASK) >>
										BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT) !=
											BMFPGA_BOARD_STATUS_CLK_MODE_100MHz)
#ifdef	R9000
			return 133;
#else
			return 125;
#endif
		else
			return 100;
	}
	else if (isFC()) {
		volatile  bmfpga_slave_t * bmfpga = (volatile bmfpga_slave_t *)
													PHYS_TO_K1(HW_BMFPGA_BASE);

		if (((bmfpga->board_status & BMFPGA_BOARD_STATUS_CLK_MODE_MASK) >>
										BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT) !=
											BMFPGA_BOARD_STATUS_CLK_MODE_100MHz)
#ifdef	R9000
			return 133;
#else
			return 125;
#endif
		else
			return 100;
	}
	else
		return 100;
}

#ifdef PROM_R9000
int
bmfpga_getBoardRev( void )
{
    return ((BMFPGA_READ(BMFPGA_REG_BOARD_STATUS) &
        BMFPGA_BOARD_STATUS_BOARD_REV ) >>
        BMFPGA_BOARD_STATUS_BOARD_REV_SHIFT );
                                                                                
}
#endif
                                                                                

#endif	/* !SSC */
