// vi:ts=4
 /*-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rpc-lsarpc.h,v 1.2 2002/12/10 18:13:23 laurentb Exp $
 *
 * Name			: cifs-rpc-lsarpc.h
 *
 * Description	: CIFS DCE/RPC Local Security Authority services definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/17/2002
 *
 *-----------------------------------------------------------------*/
   
#ifndef _CIFS_RPC_LSARPC_H
#define _CIFS_RPC_LSARPC_H

//LSARPC services operation numbers

#define LSARPC_CLOSE              0x00
#define LSARPC_DELETE             0x01
#define LSARPC_ENUM_PRIVS         0x02
#define LSARPC_QUERYSECOBJ        0x03
#define LSARPC_SETSECOBJ          0x04
#define LSARPC_CHANGEPASSWORD     0x05
#define LSARPC_OPENPOLICY         0x06
#define LSARPC_QUERYINFOPOLICY    0x07
#define LSARPC_SETINFOPOLICY      0x08
#define LSARPC_CLEARAUDITLOG      0x09
#define LSARPC_CREATEACCOUNT      0x0a
#define LSARPC_ENUM_ACCOUNTS      0x0b
#define LSARPC_CREATETRUSTDOM     0x0c
#define LSARPC_ENUMTRUSTDOM       0x0d
#define LSARPC_LOOKUPNAMES        0x0e
#define LSARPC_LOOKUPSIDS         0x0f
#define LSARPC_CREATESECRET       0x10
#define LSARPC_OPENACCOUNT	      0x11
#define LSARPC_ENUMPRIVSACCOUNT   0x12
#define LSARPC_ADDPRIVS           0x13
#define LSARPC_REMOVEPRIVS        0x14
#define LSARPC_GETQUOTAS          0x15
#define LSARPC_SETQUOTAS          0x16
#define LSARPC_GETSYSTEMACCOUNT   0x17
#define LSARPC_SETSYSTEMACCOUNT   0x18
#define LSARPC_OPENTRUSTDOM       0x19
#define LSARPC_QUERYTRUSTDOM      0x1a
#define LSARPC_SETINFOTRUSTDOM    0x1b
#define LSARPC_OPENSECRET         0x1c
#define LSARPC_SETSECRET          0x1d
#define LSARPC_QUERYSECRET        0x1e
#define LSARPC_LOOKUPPRIVVALUE    0x1f
#define LSARPC_LOOKUPPRIVNAME     0x20
#define LSARPC_PRIV_GET_DISPNAME  0x21
#define LSARPC_DELETEOBJECT       0x22
#define LSARPC_ENUMACCTWITHRIGHT  0x23
#define LSARPC_ENUMACCTRIGHTS     0x24
#define LSARPC_ADDACCTRIGHTS      0x25
#define LSARPC_REMOVEACCTRIGHTS   0x26
#define LSARPC_QUERYTRUSTDOMINFO  0x27
#define LSARPC_SETTRUSTDOMINFO    0x28
#define LSARPC_DELETETRUSTDOM     0x29
#define LSARPC_STOREPRIVDATA      0x2a
#define LSARPC_RETRPRIVDATA       0x2b
#define LSARPC_OPENPOLICY2        0x2c
//#define LSARPC_UNK_GET_CONNUSER   0x2d // LsaGetConnectedCredentials ? 


#define CIFS_POLICY_VIEW_LOCAL_INFORMATION    0x00000001
#define CIFS_POLICY_VIEW_AUDIT_INFORMATION    0x00000002
#define CIFS_POLICY_GET_PRIVATE_INFORMATION   0x00000004
#define CIFS_POLICY_TRUST_ADMIN               0x00000008
#define CIFS_POLICY_CREATE_ACCOUNT            0x00000010
#define CIFS_POLICY_CREATE_SECRET             0x00000020
#define CIFS_POLICY_CREATE_PRIVILEGE          0x00000040
#define CIFS_POLICY_SET_DEFAULT_QUOTA_LIMITS  0x00000080
#define CIFS_POLICY_SET_AUDIT_REQUIREMENTS    0x00000100
#define CIFS_POLICY_AUDIT_LOG_ADMIN           0x00000200
#define CIFS_POLICY_SERVER_ADMIN              0x00000400
#define CIFS_POLICY_LOOKUP_NAMES              0x00000800




#define SEC_GENR_POLICY_ALL		(SEC_STDR_REQUIRED_ACCESS			  |\
		                            CIFS_POLICY_VIEW_LOCAL_INFORMATION    |\
        		                    CIFS_POLICY_VIEW_AUDIT_INFORMATION    |\
                		            CIFS_POLICY_GET_PRIVATE_INFORMATION   |\
                        		    CIFS_POLICY_TRUST_ADMIN               |\
		                            CIFS_POLICY_CREATE_ACCOUNT            |\
        		                    CIFS_POLICY_CREATE_SECRET             |\
                		            CIFS_POLICY_CREATE_PRIVILEGE          |\
                        		    CIFS_POLICY_SET_DEFAULT_QUOTA_LIMITS  |\
									CIFS_POLICY_SET_AUDIT_REQUIREMENTS    |\
		                            CIFS_POLICY_AUDIT_LOG_ADMIN           |\
		                            CIFS_POLICY_SERVER_ADMIN              |\
		                            CIFS_POLICY_LOOKUP_NAMES )


#define SEC_GENR_POLICY_READ       (SEC_STDR_READ_ACCESS				  |\
        		                    CIFS_POLICY_VIEW_AUDIT_INFORMATION    |\
                		            CIFS_POLICY_GET_PRIVATE_INFORMATION)

#define SEC_GENR_POLICY_WRITE      (SEC_STDR_WRITE_ACCESS				  |\
        		                    CIFS_POLICY_TRUST_ADMIN               |\
                		            CIFS_POLICY_CREATE_ACCOUNT            |\
                        		    CIFS_POLICY_CREATE_SECRET             |\
		                            CIFS_POLICY_CREATE_PRIVILEGE          |\
        		                    CIFS_POLICY_SET_DEFAULT_QUOTA_LIMITS  |\
                		            CIFS_POLICY_SET_AUDIT_REQUIREMENTS    |\
                        		    CIFS_POLICY_AUDIT_LOG_ADMIN           |\
		                            CIFS_POLICY_SERVER_ADMIN)

#define SEC_GENR_POLICY_EXECUTE    (SEC_STDR_EXECUTE_ACCESS			  |\
		                            CIFS_POLICY_VIEW_LOCAL_INFORMATION    |\
		                            CIFS_POLICY_LOOKUP_NAMES )







typedef struct lsaSecQos
{
  uint32	length; 
  ushort16	impersoLvl;				//impersonation level 
  uchar8	ctxTrackMod;			//context tracking mode 
  uchar8	effectiOnly;			//effective only 
}lsaSecQos;

typedef struct lsaObjAttrs
{
  uint32	length;					//including the length field. 
  uint32	rootDirp;				//root directory pointer
  uint32	objNamep;				//object name pointer 
  uint32	attributes;				//attributes
  uint32	secDescp;				//security descriptior pointer 
  uint32	secQosp;				//security quality of service pointer
  lsaSecQos secQos;
}lsaObjAttrs;


//LSA Open Policy function call
typedef struct lsaOpenPolicyCall
{
  uint32			unknown; 
  ushort16			systemName;		//system name 
  lsaObjAttrs		attrs;			//object attributes 
  sec_accessMask	desiredAccess;	// desired access attributes 
}lsaOpenPolicyCall;

//LSA Open Policy function response
typedef struct lsaOpenPolicyResp
{
  cifsrpc_lsaPolHnd	pol;			//policy handle 
  uint32			returnValue;	//return code 
}lsaOpenPolicyResp;

//LSA Open Policy2 function call
typedef struct lsaOpenPolicy2Call
{
  uint32			unknown; 
  cifsrpc_uniStruc	serverName;		//server name 
  lsaObjAttrs		attrs;			//object attributes 
  sec_accessMask	desiredAccess;	//desired access
}lsaOpenPolicy2Call;

//LSA Open Policy2 function response
#define lsaOpenPolicy2Resp	lsaOpenPolicyResp

//LSA close function call 
typedef struct lsaCloseCall
{
  cifsrpc_lsaPolHnd pol;			//policy handle 
}lsaCloseCall;

//LSA Close function response 
typedef struct lsaCloseResp
{
  cifsrpc_lsaPolHnd pol;			//policy handle 
  uint32			returnValue;	//return code 
}lsaCloseResp;


//domInfo - info class 3 and 5 LSA Query response 
typedef struct domInfo
{
  ushort16			uniLen; 
  ushort16			uniMaxLen; 
  uint32			domNamep; 
  uint32			domSidp; 
  cifsrpc_uniStruc	domName; 
  cifsrpc_sid		domSid;
} domInfo;

//level 5 = level 3 
typedef domInfo domInfo3;
typedef domInfo domInfo5;

//auditing settings 
typedef struct domInfo2
{
  uint32	auditEnabled;
  uint32	count1; 
  uint32	count2; 
  //TODO see what to do here
} domInfo2;

//server role information 
typedef struct domInfo6
{
  ushort16		serverRole;			//2=backup, 3=primary 
} domInfo6;

//LSA QueryInformationPolicy function call 
typedef struct lsaQueryInfoPolicyCall
{
  cifsrpc_lsaPolHnd pol;			//policy handle 
  ushort16			infoClass;
}lsaQueryInfoPolicyCall;

//LSA QueryInformationPolicy function call 
typedef struct lsaQueryInfoPolicyResp
{
  uint32		infop; 
  ushort16		infoClass;	 
  union {   
	domInfo2	cl2;
	domInfo3	cl3;
	domInfo5	cl5;
	domInfo6	cl6;
  }info; 
  uint32			returnValue;	//return code 
}lsaQueryInfoPolicyResp;


/* 
// XXXX these are here to get a compile! 
#define LSA_LOOKUPRIDS      0xFD

#define LSA_MAX_GROUPS 96
#define LSA_MAX_SIDS 128


// LSA_Q_QUERY_SEC_OBJ - LSA query security 
typedef struct lsa_query_sec_obj_info
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 sec_info;

} LSA_Q_QUERY_SEC_OBJ;

// LSA_R_QUERY_SEC_OBJ - probably an open 
typedef struct r_lsa_query_sec_obj_info
{
	uint32 ptr;
	SEC_DESC_BUF *buf;

	NTSTATUS status;         // return status 

} LSA_R_QUERY_SEC_OBJ;



// LSA_Q_ENUM_TRUST_DOM - LSA enumerate trusted domains 
typedef struct lsa_enum_trust_dom_info
{
	cifsrpc_lsaPolHnd pol; // policy handle 
    uint32 enum_context; // enumeration context handle 
    uint32 preferred_len; // preferred maximum length 

} LSA_Q_ENUM_TRUST_DOM;

// LSA_R_ENUM_TRUST_DOM - response to LSA enumerate trusted domains 
typedef struct lsa_r_enum_trust_dom_info
{
	uint32 enum_context; // enumeration context handle 
	uint32 num_domains; // number of domains 
	uint32 ptr_enum_domains; // buffer pointer to num domains 

	// this lot is only added if ptr_enum_domains is non-NULL 
	uint32 num_domains2; // number of domains 
	UNIHDR2 *hdr_domain_name;
	UNISTR2 *uni_domain_name;
	DOM_SID2 *domain_sid;

	NTSTATUS status; // return code 

} LSA_R_ENUM_TRUST_DOM;

#define MAX_REF_DOMAINS 32

// DOM_TRUST_HDR 
typedef struct dom_trust_hdr
{
	UNIHDR hdr_dom_name; // referenced domain unicode string headers 
	uint32 ptr_dom_sid;

} DOM_TRUST_HDR;
	
// DOM_TRUST_INFO 
typedef struct dom_trust_info
{
	UNISTR2  uni_dom_name; // domain name unicode string 
	DOM_SID2 ref_dom     ; // referenced domain SID 

} DOM_TRUST_INFO;
	
// DOM_R_REF 
typedef struct dom_ref_info
{
    uint32 num_ref_doms_1; // num referenced domains 
    uint32 ptr_ref_dom; // pointer to referenced domains 
    uint32 max_entries; // 32 - max number of entries 
    uint32 num_ref_doms_2; // num referenced domains 

    DOM_TRUST_HDR  hdr_ref_dom[MAX_REF_DOMAINS]; // referenced domains 
    DOM_TRUST_INFO ref_dom    [MAX_REF_DOMAINS]; // referenced domains 

} DOM_R_REF;

// the domain_idx points to a SID associated with the name 

// LSA_TRANS_NAME - translated name 
typedef struct lsa_trans_name_info
{
	ushort16 sid_name_use; // value is 5 for a well-known group; 2 for a domain group; 1 for a user... 
	UNIHDR hdr_name; 
	uint32 domain_idx; // index into DOM_R_REF array of SIDs 

} LSA_TRANS_NAME;

#define MAX_LOOKUP_SIDS 30

// LSA_TRANS_NAME_ENUM - LSA Translated Name Enumeration container 
typedef struct lsa_trans_name_enum_info
{
	uint32 num_entries;
	uint32 ptr_trans_names;
	uint32 num_entries2;
	
	LSA_TRANS_NAME *name; // translated names  
	UNISTR2 *uni_name;

} LSA_TRANS_NAME_ENUM;

// LSA_SID_ENUM - LSA SID enumeration container 
typedef struct lsa_sid_enum_info
{
	uint32 num_entries;
	uint32 ptr_sid_enum;
	uint32 num_entries2;
	
	uint32 *ptr_sid; // domain SID pointers to be looked up. 
	DOM_SID2 *sid; // domain SIDs to be looked up. 

} LSA_SID_ENUM;

// LSA_Q_LOOKUP_SIDS - LSA Lookup SIDs 
typedef struct lsa_q_lookup_sids
{
	cifsrpc_lsaPolHnd          pol; // policy handle 
	LSA_SID_ENUM        sids;
	LSA_TRANS_NAME_ENUM names;
	LOOKUP_LEVEL        level;
	uint32              mapped_count;

} LSA_Q_LOOKUP_SIDS;

// LSA_R_LOOKUP_SIDS - response to LSA Lookup SIDs 
typedef struct lsa_r_lookup_sids
{
	uint32              ptr_dom_ref;
	DOM_R_REF           *dom_ref; // domain reference info 

	LSA_TRANS_NAME_ENUM *names;
	uint32              mapped_count;

	NTSTATUS            status; // return code 

} LSA_R_LOOKUP_SIDS;

// LSA_Q_LOOKUP_NAMES - LSA Lookup NAMEs 
typedef struct lsa_q_lookup_names
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 num_entries;
	uint32 num_entries2;
	UNIHDR  *hdr_name; // name buffer pointers 
	UNISTR2 *uni_name; // names to be looked up 

	uint32 num_trans_entries;
	uint32 ptr_trans_sids; // undocumented domain SID buffer pointer 
	uint32 lookup_level;
	uint32 mapped_count;

} LSA_Q_LOOKUP_NAMES;

// LSA_R_LOOKUP_NAMES - response to LSA Lookup NAMEs by name 
typedef struct lsa_r_lookup_names
{
	uint32 ptr_dom_ref;
	DOM_R_REF *dom_ref; // domain reference info 

	uint32 num_entries;
	uint32 ptr_entries;
	uint32 num_entries2;
	DOM_RID2 *dom_rid; // domain RIDs being looked up 

	uint32 mapped_count;

	NTSTATUS status; // return code 
} LSA_R_LOOKUP_NAMES;

// This is probably a policy handle but at the moment we
   never read it - so use a dummy struct. 

typedef struct lsa_q_open_secret
{
	uint32 dummy;
} LSA_Q_OPEN_SECRET;

// We always return "not found" at present - so just marshal the minimum. 

typedef struct lsa_r_open_secret
{
	uint32 dummy1;
	uint32 dummy2;
	uint32 dummy3;
	uint32 dummy4;
	NTSTATUS status;
} LSA_R_OPEN_SECRET;

typedef struct lsa_enum_priv_entry
{
	UNIHDR hdr_name;
	uint32 luid_low;
	uint32 luid_high;
	UNISTR2 name;
	
} LSA_PRIV_ENTRY;

// LSA_Q_ENUM_PRIVS - LSA enum privileges 
typedef struct lsa_q_enum_privs
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 enum_context;
	uint32 pref_max_length;
} LSA_Q_ENUM_PRIVS;

typedef struct lsa_r_enum_privs
{
	uint32 enum_context;
	uint32 count;
	uint32 ptr;
	uint32 count1;

	LSA_PRIV_ENTRY *privs;

	NTSTATUS status;
} LSA_R_ENUM_PRIVS;

// LSA_Q_PRIV_GET_DISPNAME - LSA get privilege display name 
typedef struct lsa_q_priv_get_dispname
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	UNIHDR hdr_name;
	UNISTR2 name;
	ushort16 lang_id;
	ushort16 lang_id_sys;
} LSA_Q_PRIV_GET_DISPNAME;

typedef struct lsa_r_priv_get_dispname
{
	uint32 ptr_info;
	UNIHDR hdr_desc;
	UNISTR2 desc;
	// Don't align ! 
	ushort16 lang_id;
	// align 
	NTSTATUS status;
} LSA_R_PRIV_GET_DISPNAME;

// LSA_Q_ENUM_ACCOUNTS 
typedef struct lsa_q_enum_accounts
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 enum_context;
	uint32 pref_max_length;
} LSA_Q_ENUM_ACCOUNTS;

// LSA_R_ENUM_ACCOUNTS 
typedef struct lsa_r_enum_accounts
{
	uint32 enum_context;
	LSA_SID_ENUM sids;
	NTSTATUS status;
} LSA_R_ENUM_ACCOUNTS;

// LSA_Q_UNK_GET_CONNUSER - gets username\domain of connected user
                  called when "Take Ownership" is clicked -SK 
typedef struct lsa_q_unk_get_connuser
{
  uint32 ptr_srvname;
  UNISTR2 uni2_srvname;
  uint32 unk1; // 3 unknown uint32's are seen right after uni2_srvname 
  uint32 unk2; // unk2 appears to be a ptr, unk1 = unk3 = 0 usually 
  uint32 unk3; 
} LSA_Q_UNK_GET_CONNUSER;

// LSA_R_UNK_GET_CONNUSER 
typedef struct lsa_r_unk_get_connuser
{
  uint32 ptr_user_name;
  UNIHDR hdr_user_name;
  UNISTR2 uni2_user_name;
  
  uint32 unk1;
  
  uint32 ptr_dom_name;
  UNIHDR hdr_dom_name;
  UNISTR2 uni2_dom_name;

  NTSTATUS status;
} LSA_R_UNK_GET_CONNUSER;


typedef struct lsa_q_openaccount
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	DOM_SID2 sid;
	uint32 access; // desired access 
} LSA_Q_OPENACCOUNT;

typedef struct lsa_r_openaccount
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	NTSTATUS status;
} LSA_R_OPENACCOUNT;

typedef struct lsa_q_enumprivsaccount
{
	cifsrpc_lsaPolHnd pol; // policy handle 
} LSA_Q_ENUMPRIVSACCOUNT;


typedef struct LUID
{
	uint32 low;
	uint32 high;
} LUID;

typedef struct LUID_ATTR
{
	LUID luid;
	uint32 attr;
} LUID_ATTR ;

typedef struct privilege_set
{
	uint32 count;
	uint32 control;
	LUID_ATTR *set;
} PRIVILEGE_SET;

typedef struct lsa_r_enumprivsaccount
{
	uint32 ptr;
	uint32 count;
	PRIVILEGE_SET set;
	NTSTATUS status;
} LSA_R_ENUMPRIVSACCOUNT;

typedef struct lsa_q_getsystemaccount
{
	cifsrpc_lsaPolHnd pol; // policy handle 
} LSA_Q_GETSYSTEMACCOUNT;

typedef struct lsa_r_getsystemaccount
{
	uint32 access;
	NTSTATUS status;
} LSA_R_GETSYSTEMACCOUNT;


typedef struct lsa_q_setsystemaccount
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 access;
} LSA_Q_SETSYSTEMACCOUNT;

typedef struct lsa_r_setsystemaccount
{
	NTSTATUS status;
} LSA_R_SETSYSTEMACCOUNT;


typedef struct lsa_q_lookupprivvalue
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	UNIHDR hdr_right;
	UNISTR2 uni2_right;
} LSA_Q_LOOKUPPRIVVALUE;

typedef struct lsa_r_lookupprivvalue
{
	LUID luid;
	NTSTATUS status;
} LSA_R_LOOKUPPRIVVALUE;


typedef struct lsa_q_addprivs
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 count;
	PRIVILEGE_SET set;
} LSA_Q_ADDPRIVS;

typedef struct lsa_r_addprivs
{
	NTSTATUS status;
} LSA_R_ADDPRIVS;


typedef struct lsa_q_removeprivs
{
	cifsrpc_lsaPolHnd pol; // policy handle 
	uint32 allrights;
	uint32 ptr;
	uint32 count;
	PRIVILEGE_SET set;
} LSA_Q_REMOVEPRIVS;

typedef struct lsa_r_removeprivs
{
	NTSTATUS status;
} LSA_R_REMOVEPRIVS;
*/

//Server services api entry point function
int32 cifsrpc_lsarpc(cifsrpc_pipe *rpip);

#endif // #ifndef _CIFS_RPC_LSARPC_H 

