/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-mem.h,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 *
 * Name			: cifs-rpc-mem.h
 *
 * Description	: Contains CIFS DCE/RPC memory management definitions.
 * 				  Keeps track of all the memory used for a particular data
 * 				  processing and allows to free it in a single operation
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/04/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_RPC_MEM_H
#define _CIFS_RPC_MEM_H

#define	RELEASE(ptr)	{ if (ptr) { CIFS_RAM_DEALLOC(ptr); ptr = NULL; } }



typedef struct cifsrpc_memItem	//Memory Item
{
  struct cifsrpc_memItem *nextp;//next memory item pointer
  uint32	size;				//memory item size
  uchar8	data[1];			//memory item data
}cifsrpc_memItem;


typedef struct cifsrpc_memList	//Memory items list
{
  cifsrpc_memItem	*items;		//list of memory items allocated
  uint32			size;		//sum of all the memory items sizes
}cifsrpc_memList;



/*-----------------------------------------------------------------
 * Name         : cifsrpc_initMemList
 *
 * Description  : Initialize a memory items list 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void cifsrpc_initMemList(cifsrpc_memList *ml);


/*-----------------------------------------------------------------
 * Name         : cifsrpc_allocMemItem
 *
 * Description  : Allocate a memory item and add it to the memory list
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void *cifsrpc_allocMemItem(cifsrpc_memList *ml, uint32 size);


/*-----------------------------------------------------------------
 * Name         : cifsrpc_reallocMemItem
 *
 * Description  : Reallocate a memory item
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void *cifsrpc_reallocMemItem(cifsrpc_memList *ml, void *datap, uint32 size);


/*-----------------------------------------------------------------
 * Name         : cifsrpc_freeMemList
 *
 * Description  : free all the memory allocated in the memory list
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/04/2002
 *
 *-----------------------------------------------------------------
 */
void cifsrpc_freeMemList(cifsrpc_memList *ml);



#endif // #ifndef _CIFS_RPC_MEM_H 

