/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rpc.h,v 1.2 2002/12/10 18:13:23 laurentb Exp $
 *
 * Name			: cifs-rpc.h
 *
 * Description	: Contains CIFS DCE/RPC general definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 12/09/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_RPC_H
#define _CIFS_RPC_H


//Sizes
#define CIFSRPC_S_PDUFRAGMENT		5680
#define CIFSRPC_S_PDU				64*1024	//Only accept PDU up to 64k for now
#define CIFSRPC_S_HEADER			16
#define CIFSRPC_S_REQ_HDR			8
#define CIFSRPC_S_RESP_HDR			8
#define CIFSRPC_S_RESP_FULLHDR		(CIFSRPC_S_HEADER+CIFSRPC_S_RESP_HDR)
#define CIFSRPC_S_FAULT_HDR			8

/*
#define CIFSRPC_S_UUID				16
#define CIFSRPC_S_INTERF			(CIFSRPC_S_UUID + 4)
#define CIFSRPC_S_AUTHA_HDR			12
#define CIFSRPC_S_AUTH_HDR			8
#define NTLMSSP_S_AUTH_CHK			16
*/

//RPC version
#define CIFSRPC_MAJORVERSION		0x05
#define CIFSRPC_MINORVERSION		0x00

//RPC packet types
#define	CIFSRPC_REQUEST				0x00
#define CIFSRPC_RESPONSE			0x02
#define CIFSRPC_FAULT				0x03
#define CIFSRPC_BIND				0x0B
#define CIFSRPC_BINDACK				0x0C
#define CIFSRPC_BINDNACK			0x0D
#define CIFSRPC_ALTCONT				0x0E
#define CIFSRPC_ALTCONTRESP			0x0F

//RPC flags
#define CIFSRPC_FLG_FIRST			0x01
#define CIFSRPC_FLG_LAST			0x02
#define CIFSRPC_FLG_NOCALL			0x20

/*
//NTLMSSP message types
#define NTLMSSP_NEGOTIATE			0x01
#define NTLMSSP_CHALLENGE			0x02
#define NTLMSSP_AUTH				0x03
#define NTLMSSP_UNKNOWN				0x04

//NTLMSSP negotiation flags
#define NTLMSSP_NEG_UNICODE			0x00000001
#define NTLMSSP_NEG_OEM				0x00000002
#define NTLMSSP_REQ_TARGET			0x00000004
#define NTLMSSP_NEG_SIGN			0x00000010
#define NTLMSSP_NEG_SEAL			0x00000020
#define NTLMSSP_NEG_LM_KEY			0x00000080
#define NTLMSSP_NEG_NTLM			0x00000200
//#define NTLMSSP_NEG_00001000		0x00001000
//#define NTLMSSP_NEG_00002000		0x00002000
#define NTLMSSP_NEG_ALWAYS_SIGN		0x00008000
#define NTLMSSP_NEG_NTLM2			0x00080000
#define NTLMSSP_NEG_TARGET_INFO		0x00800000
#define NTLMSSP_NEG_128				0x20000000
#define NTLMSSP_NEG_KEY_EXCHANGE	0x40000000

//#define NTLMSSP_NEG_FLAGS			0x000082b1

//NTLMSSP signature version
#define NTLMSSP_SIGN_VERSION		0x01

//NTLMSSP authentication type and level
#define NTLMSSP_AUTH_TYPE			0xa
#define NTLMSSP_AUTH_LEVEL			0x6
*/


// Linked definitions pre-definitions
struct cifs_client; 
struct cifs_session; 

//************************** RPC related types ******************************

// MS Unicode (2 bytes)
typedef ushort16 uni16;

//Structure used to carry an unicode string over an RPC pipe
typedef struct cifsrpc_uniStruc
{
  uint32	maxLen;					//max length in uni16 including ending 0
  uint32	unknown;
  uint32	len;					//length in uni16 including ending 0
  uni16		*str;					//unicode string (little endian)
}cifsrpc_uniStruc;

//RPC Enumeration handle
typedef struct cifsrpc_enumHandle
{
  uint32	handlep;				//handle pointer
  uint32	handle;					//handle
}cifsrpc_enumHandle;

//UUID structure
typedef struct cifsrpc_uuid
{
  uint32	d1;						//Internal temperature of the sun in Kelvin
  ushort16	d2;						//Population of Saint Cyr Sur Loire
  ushort16	d3;						//...
  uchar8	d4[2];
  uchar8	d5[6];
}cifsrpc_uuid;

//RPC Interface
typedef struct cifsrpc_intf
{
  cifsrpc_uuid uuid;				//identification
  uint32 version;					//version number
}cifsrpc_intf;

//Policy handle
typedef struct cifsrpc_polHnd
{
  uint32		zero;
  cifsrpc_uuid	uuid;				//identification
}cifsrpc_lsaPolHnd;

//Sid
typedef struct {
  uint32		nbSubAuthy;
  sec_cifsSid	sid;
}cifsrpc_sid;

//************************* RPC messages structures *************************


//RPC Header
typedef struct cifsrpc_hdr
{
  uchar8	packetType;
  uchar8	flags;			
  boolean	bigEndian;				//TRUE if RPC BIG ENDIAN received
  ushort16	fragLen;				//fragment length
  ushort16	authLen;				//authentication length
  uint32	callId;					//call identifier
}cifsrpc_hdr;


//RPC bind request
typedef struct cifsrpc_bindReq
{
  ushort16 		maxTFragSize;		//maximum transmission fragment size
  ushort16 		maxRFragsize;		//maximum receive fragment size
  uint32 		assocGroupId;		//associated group identifier
  uint32		nbCtxElts;			//number of context elements
  ushort16		contextId;			//presentation context identifier
  uchar8		nbSyntaxes;			//number of transfer syntaxs
  cifsrpc_intf	abstract;			//interface used by the client
  cifsrpc_intf	transfer;			//interface to be used by us
}cifsrpc_bindReq;


//RPC bind acknowledge
typedef struct cifsrpc_bindAck
{
  ushort16 		maxTFragSize;		//maximum transmission fragment size
  ushort16 		maxRFragsize;		//maximum receive fragment size
  uint32 		assocGroupId;		//associated group identifier
  //Secondary Address
  ushort16		secAddrlen;			//secondary address length
  char			secAddrPort[20];  	//secondary address port (full pipe name)
  //Result List (Aligned)
  uchar8		nbResults;			//number of results
  ushort16		result;				//result (Aligned)
  ushort16		reason;
  cifsrpc_intf	transfer;			//interface used by us (from request)
}cifsrpc_bindAck;


//RPC request header
typedef struct cifsrpc_reqHdr
{
  uint32		allocHint;			//data size in bytes w/o header and tail.
  ushort16		contextId;			//presentation context identifier
  ushort16		opNb;				//operation number
}cifsrpc_reqHdr;


//RPC response header
typedef struct cifsrpc_respHdr
{
  uint32		allocHint;			//data size in bytes w/o header and tail.
  ushort16		contextId;			//presentation context identifier
  uchar8		cancelCnt;			//cancel count
  uchar8		reserved;			//alignment ?
}cifsrpc_respHdr;


//RPC fault Header
typedef struct cifsrpc_faultHdr
{
  cifsrpc_respHdr	respHdr;
  uint32			status;			//NT status		
  uint32			reserved;
}cifsrpc_faultHdr;


//************************ RPC handling structures ************************

typedef struct cifsrpc_outPdu
{
  uint32	fragLen;					//fragment length
  uint32	fragSentLen;				//size of fragment data already sent
  uchar8	frag[CIFSRPC_S_PDUFRAGMENT];//fragment data
  
  cifsrpc_ed	enc;					//Encoding structure
  uint32	pduSentLen;					//size of pdu data already sent
}cifsrpc_outPdu;


typedef struct cifsrpc_inPdu
{
  uint32	fragLen;					//fragment length (without header)
  uint32	fragRecLen;					//size of data already received(w/o hdr)
  uchar8	frag[CIFSRPC_S_PDUFRAGMENT];//fragment data
  boolean	bigEndian;					//TRUE if RPC BIG ENDIAN received

  cifsrpc_hdr		hdr;				//header information
  cifsrpc_reqHdr	reqHdr;				//request header information
  cifsrpc_ed		dec;				//Decoding structure
}cifsrpc_inPdu;


typedef struct cifsrpc_pipe {
  uchar8	index;						//index in the supported pipes list
  boolean	fault;						//Fault detected, always return fault
  boolean	rpcBind;					//RPC bind has been received

  cifsrpc_memList		memList;		//List of the memory used on this pipe
  cifsrpc_inPdu	 		inPdu;			//Incoming PDU data
  cifsrpc_outPdu 		outPdu;			//Outgoing PDU data
  struct cifs_client	*clp;			// associated client 
  struct cifs_session	*ssp;			// associated user's session 
}cifsrpc_pipe;



typedef struct cifsrpc_function
{
  ushort16	opNb;						//operation number (from RPC req header)
  char		*name;						//function name
  int32		(*func)(cifsrpc_pipe *rpip);//pointer to the function
}cifsrpc_function;


//********************** RPC internal functions ***************************

int32   //NFX_OK or NFX_ERR
cifsrpc_callFunction(cifsrpc_pipe *rpip, cifsrpc_function *pipeFns);

int32   //NFX_OK or NFX_ERR
cifsrpc_encFaultPdu(cifsrpc_pipe *rpip, uint32 status);



  
#endif // #ifndef _CIFS_RPC_H 

