/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-transportFSA.c,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-transport.c
 *
 * Description	: Contains CIFS transport interface. 
 *				  First step NetBIOS, then directly TCP (W2K)
 *				  So, some functions are very simple now (could be just #define)
 *				  but will be enhanced later for W2K
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 10/31/2001
 *
 *-----------------------------------------------------------------
 */

#include "cifs-api.h"
#include "cifs-utils.h"
#include "cifs-transport.h"
#include "cifs-encdec.h"
#include "cifs-ipc.h"
#include "cifs-intmsg.h"

#define IFDBG(lvl)  if (lvl <= cifs_debugLevel)


void	 cifs_nbNameRegCfmInd(void *svp, int32 status);
int32	 cifs_nbSessionInd(void *svp, nb_ssnCB_t *nbssp, void **ssp);
void	 cifs_nbSessionMsgInd(void *ssp, pkt_desc_t *msgp, void *smbp,
									uint32 smbl);
void	 cifs_nbSessionXmtCmpInd(void *ssp, void *xmtp, pkt_desc_t *pktp);
void	 cifs_nbSessionEndInd(void *ssp);



/*-----------------------------------------------------------------
 * Name			: cifs_initTransportLayer 
 *
 * Description	: Initialize CIFS transport layer
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/06/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifs_initTransportLayer(void)
{

//	nb_init(CIFS_N_SERVERS, CIFS_N_CLIENTS, cifs_globalCfgp->eee_cifsAppId);
	
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name			: cifs_initServerTransportLayer 
 *
 * Description	: Initialize CIFS transport layer for a CIFS server
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/06/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifs_initServerTransportLayer(cifs_server *svp)
{
/*	static nb_srvRegCB_t srvReg = {
		0,
		0,
		0,
		NULL,
		{ 0, "" },
		{ 0, "" },
		cifs_nbNameRegCfmInd,
		cifs_nbSessionInd,
		cifs_nbSessionMsgInd,
		cifs_nbSessionXmtCmpInd,
		cifs_nbSessionEndInd
	};
	
	//Register CIFS server with NetBIOS
	srvReg.cifsSrvIpAddr= svp->cfg.ipAddr;
	srvReg.subnetMask	= svp->cfg.subnetMask;
	srvReg.winsSrvIpAddr= svp->cfg.winsSrvIpAddr;
	srvReg.cifsSrvCbP	= svp;
	strcpy(srvReg.cifsSrvName.name,	svp->cfg.name);
	strcpy(srvReg.cifsDomName.name,	svp->cfg.domainName);
	NFX_ASSERT(nb_srvRegisterReq(&srvReg, &svp->transportCBp) == NFX_OK);
	//	panic("cifs_initTransportLayer: failed to register %s%c%s server",
	//			svp->cfg.domainName, 0x5C, svp->cfg.name);
*/
	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name			: cifs_getDatagBuf 
 *
 * Description	: Returns a datagram buffer
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/06/2001
 *
 *-----------------------------------------------------------------
 */
pkt_desc_t *	//NULL if no buffer available
cifs_getDatagBuf(cifs_server *svp)
{
	
	//Waiting for outgoing packets interface from Shaun
	static pkt_desc_t pktDesc;
	static char msgBuf[2048];
	pktDesc.pHdr = msgBuf + 0x80;//dummy value ~= smb offset in datag msg
	return &pktDesc;
}


/*-----------------------------------------------------------------
 * Name			: cifs_getClientBuf 
 *
 * Description	: Returns a buffer for the specified client
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/06/2001
 *
 *-----------------------------------------------------------------
 */
pkt_desc_t *	//NULL if no buffer available
cifs_getClientBuf(cifs_client *clp, uchar8 **smbp)
{
//	return (nb_allocOutgoingPkt(clp->transportCBp, (void**)smbp));
	
	static pkt_desc_t pktDesc;
	static char msgBuf[2048];
	static uint32 count = 0;

	pktDesc.pHdr = &msgBuf[0x80];//dummy value ~= smb offset in datag msg
	*smbp = &msgBuf[0x84];
	pktDesc.pNext = NULL;

#ifdef CIFSTEST
//DBG Simulate no buffer every 3 calls
//if ((count++) % 3 == 2) return (NULL);
#endif
	
	return &pktDesc;
}


/*-----------------------------------------------------------------
 * Name			: cifs_sendClientMsg
 *
 * Description	: Sends a message for the specified client
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/06/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifs_sendClientMsg(cifs_client *clp, cifs_operation *opp)
{
	int32 rc;

	opp->status = cifs_opstatus_sending;
	
	CIFS_PRINT(CIFS_DBGLV_DBG, ("cifs_sendClientMsg: "
				"send msg (l=%d)", opp->respMsg.smbLen));
	IFDBG(CIFS_DBGLV_DBG)
	{
		memDump(opp->respMsg.smbp, opp->respMsg.smbLen);
	}
	
//	rc = nb_ssnXmtPktReq(clp->transportCBp, opp, opp->respMsg.tplPktDescp,
//							opp->respMsg.smbLen);
	rc = NFX_OK;
	
	//Make the incoming buffer available since no more needed
	cifs_freeInBuf(clp, &opp->reqMsg.tplPktDescp);

   	if (rc != NFX_OK)
	{	//Drop the client connection
		cifs_closeOperation(opp);
		cifs_abortClientConnection(clp);
	}
	
	return rc;
}


/*-----------------------------------------------------------------
 * Name			: cifs_sendClientMsgWithoutOp
 *
 * Description	: Sends a message for the specified client without
 * 				  operation
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 2/14/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifs_sendClientMsgWithoutOp(cifs_client *clp, pkt_desc_t *pktp, uint32 length)
{
	CIFS_PRINT(CIFS_DBGLV_DBG, ("cifs_sendClientMsgWithoutOp: "
				"send msg (l=%d)", length));
	
//	return (nb_ssnXmtPktReq(clp->transportCBp, NULL, pktp, length));
	return (NFX_OK);
}



/*-----------------------------------------------------------------
 * Name			: cifs_freeInBuf 
 *
 * Description	: Returns an incoming buffer
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/12/2001
 *
 *-----------------------------------------------------------------
 */
void
cifs_freeInBuf(cifs_client *clp, pkt_desc_t **pktpp)
{
	if (*pktpp == NULL)
		return;
//	Free linked buffers (lower layer will do it)
//	nb_freeIncomingPkt(cifs_client *clp, *pktpp);
	*pktpp = NULL;
}


/*-----------------------------------------------------------------
 * Name			: cifs_freeOutBuf 
 *
 * Description	: Returns a buffer for the specified client
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/12/2001
 *
 *-----------------------------------------------------------------
 */
void
cifs_freeOutBuf(cifs_client *clp, cifs_operation *opp, pkt_desc_t **pktpp)
{
	pkt_desc_t	*prevp, *currp;

	if (*pktpp == NULL)
		return;

	*pktpp = NULL;
}


/*-----------------------------------------------------------------
 * Name			: cifs_closeSessionLayer 
 *
 * Description	: close the session layer associated with the client
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/12/2001
 *
 *-----------------------------------------------------------------
 */
void
cifs_closeSessionLayer(cifs_client *clp)
{
//	nb_ssnEndReq (clp->transportCBp);
}


/*-----------------------------------------------------------------
 * Name			: cifs_testServerInitCompletion
 * 
 * Description	: test if server initialization is completed, if yes
 * 					start browse announcement
 * 
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/14/2002
 *-----------------------------------------------------------------
 */
int32 	//NFX_OK or NFX_ERR
cifs_testServerInitCompletion(cifs_server *svp)
{	
	cifs_cmdDesc msgd;
		
	if ((svp->flags & CIFS_SERVER_INITCOMPLETIONMASK) == 
		CIFS_SERVER_INITCOMPLETIONMASK)
	{
	}
	return (NFX_OK);
}




/*-----------------------------------------------------------------
 *
 * Description	: callback functions from the session layer.
 * 				(NetBIOS for now)
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/10/2001
 *
 *-----------------------------------------------------------------
 */



/*-----------------------------------------------------------------
 * Name			: cifs_nbNameRegCfmInd 
 *
 * Description	: Server name registration confirmation
 * 
 * Created by	: Laurent Brard
 *
 * Date Created : 11/10/2001
 *
 *-----------------------------------------------------------------
 */
void cifs_nbNameRegCfmInd(void *serverp, int32 status)
{
	cifs_server *svp = (cifs_server*)serverp;
	
	svp->flags |= CIFS_SERVER_NAMEREGISTERED;	

	cifs_testServerInitCompletion(svp);
}


/*-----------------------------------------------------------------
 * Name			: cifs_nbSessionInd 
 *
 * Description	: New NetBIOS session
 *
 * Created by	: Laurent Brard
 *
 * Date Created : 11/06/2001
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK if session accepted, NFX_ERR otherwise
cifs_nbSessionInd(void *serverp, nb_ssnCB_t *nbssp, void **ssp)
{
	cifs_server *svp = (cifs_server*)serverp;
	cifs_client *clp;
//DBG
extern void *cifsssp;
	
	CIFS_PRINT(CIFS_DBGLV_DBG, ("cifs_nbSessionInd: new cifs client !"));
	
	//Test if server's shares are initialized 
	if (!(svp->flags & CIFS_SERVER_SHARESINITIALIZED))
		return (NFX_ERR);
	
	clp = cifs_addClient(svp);
	if (clp == NULL) {
		CIFS_PRINT(CIFS_DBGLV_ERR, ("cifs_nbSessionInd: "
					"no client object, refuse session"));
		return (NFX_ERR);
	}
	clp->transportCBp = nbssp;
	
//DGB
cifsssp = clp;
	
	*ssp = clp;
	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name			: cifs_nbSessionMsgInd 
 *
 * Description	: Message reception on a netBIOS session
 * 
 * Created by	: Laurent Brard
 *
 * Date Created : 11/10/2001
 *
 *-----------------------------------------------------------------
 */
void cifs_nbSessionMsgInd(void *clp, pkt_desc_t *msgp, void *smbp,
									uint32 smbl)
{
	CIFS_PRINT(CIFS_DBGLV_DBG, ("cifs_nbSessionMsgInd: "
				"msg (l=%d) for client %p", smbl, clp));
	IFDBG(CIFS_DBGLV_DBG) 
	{
//		memDump(smbp, smbl);
	}
	cifs_processMsg((cifs_client*)clp, msgp, (char*)smbp, smbl);
}


/*-----------------------------------------------------------------
 * Name			: cifs_nbSessionXmtCmpInd
 *
 * Description	: Message transmission completion
 * 
 * Created by	: Laurent Brard
 *
 * Date Created : 11/10/2001
 *
 *-----------------------------------------------------------------
 */
void cifs_nbSessionXmtCmpInd(void *nbclp, void *xmtp, pkt_desc_t *xmtPktDescp)
{
	cifs_client *clp = (cifs_client*)nbclp;
	cifs_operation *opp = (cifs_operation*)xmtp;
	
	CIFS_PRINT(CIFS_DBGLV_DBG, ("> cifs_nbSessionXmtCmpInd"));

	//Test if the message was sent out with an operation attached
	if (opp != NULL)
	{
		if ((opp->status != cifs_opstatus_sending) &&
			(opp->status != cifs_opstatus_sendPending))
		{
			CIFS_PRINT(CIFS_DBGLV_ERR, ("cifs_nbSessionXmtCmpInd: status err"));
		}
	
		if ((opp->status == cifs_opstatus_sendPending) ||
			((opp->status == cifs_opstatus_sending) && 
			 (opp->flags & CIFS_OPFLAG_MORE_TO_SEND)))
		{
			// Operation has more message to send
			// Only free outgoing buffer
			cifs_freeOutBuf(clp, opp, &xmtPktDescp);
		}
		else
		{
			if (opp->flags & CIFS_OPFLAG_READ_REQ)
			{
				// Send Xmt complete message to FS inorder for FS to free COD
				cifs_msgXmtCmpToFs(opp);
			}

			//TODO whatever is needed to free other allocated stuff
		
			cifs_closeOperation(opp);
		}
	}

	//See if there are any operations waiting for an outgoing buffer
	if (!DLL_IS_EMPTY(&clp->waitOutBufOps))
	{
		pkt_desc_t *pktp;
		uchar8* osmbp;
		esm_event_t evt;
		pktp = cifs_getClientBuf(clp, &osmbp);
		if (pktp != NULL)
		{	//There is a free outgoing buffer
			//(there should always be since we just freed one)
			DLL_REMOVE_HEAD(&clp->waitOutBufOps, opp, entry);

			switch (opp->status)
			{
				case cifs_opstatus_waitOutBuffer:
					cifs_procMsgWithXmtBuf(clp, opp, pktp, osmbp);
					break;
				case cifs_opstatus_sendPending:
					evt.event = CIFS_THREADEVT_BUF_AVAILABLE;
					evt.p1.u.p = pktp;
					evt.p2.u.p = osmbp;
					thread_event(opp->threadId, &evt);
					break;
				default:
					cifs_freeOutBuf(clp, opp, &pktp);
			}
		}
	}

	CIFS_PRINT(CIFS_DBGLV_DBG, ("< cifs_nbSessionXmtCmpInd"));
}


/*-----------------------------------------------------------------
 * Name			: cifs_nbSessionEndInd
 *
 * Description	: End of a NetBIOS session
 * 
 * Created by	: Laurent Brard
 *
 * Date Created : 11/10/2001
 *
 *-----------------------------------------------------------------
 */
void cifs_nbSessionEndInd(void *ssp)
{
	cifs_closeClient((cifs_client*)ssp);
}


