/*  
 *=============================================================================
 * File Name  :    bmc12500-eth-rx.c
 *
 * Description:    Receive driver code for the BMC12500 Ethernet Mac.    
 *
 * Created    :    18 October 2001
 *
 * Author     :    Frank Salzmann
 *
 * RCS        :    $Id: bmc12500-eth-rx.c,v 1.62 2002/12/07 01:50:42 tonyw Exp $
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#define COREDUMP

#include "bmc12500-eth.h"
#include <exports/mem-api.h>
#include "../sm-eee/eee-test.h"

extern int bmc12500Eth_rx_pkt_trace;
#ifndef PROM
extern uint32 sysDVTrunning;
extern uint32 txrxdebug; /* This flag is used for txrx pkt dump.*/
#endif
#define LMUX_DEBUG
#define LUC_DEBUG
#ifdef LMUX_DEBUG
extern uint32 lmux_rxPkts[];
extern uint32 lmux_rxBytes[];
#endif
#ifdef LUC_DEBUG
extern uint32 luc_rxPkts[];
extern uint32 luc_rxBytes[];
#endif
extern uint32 lmux_loopback_test;
extern uint32 luc_loopback_test;
extern uint32 bc1250_diagCPU;
extern uint32 bc1250_FP2CPU_active;
extern uint32 bc1250_FP1CPU_active;
extern dvt_front_cfg dvtFrontCfg;
extern eee_test_hdr_t  * test_hdr;

#if !defined(COREDUMP)
uint32 get_word(char *buf)
{
   int i;
   uint32 tmp = 0;
   for(i=0;i<4;i++)
   {
      tmp |= ((*(buf + i)) << (i*8));
   }
   return tmp;
}
#else
extern uint32 get_word(char *);
#endif

void crash_bmc12500Eth_decodeHdr(char*);
int bmc1250_rxPktCapture(char*, uint32, int32);
int crash_bmc12500Eth_printBuf(char*, int, int);
int crash_bmc12500Eth_decodeScsi(char *); 

extern void luc_rcvPkt(eee_desc_t *);

#define EEE_MAC_HDR_LEN		16
#define EEE_AGILE_HDR_LEN	8

extern uint32 lmux_get_rev;
extern uint32 lmux_rev[3];

/*
 *------------------------------------------------------------------------------
 * Function Name:	crash_bmc12500Eth_rxPkt_net
 *
 * Description: 	Receives a unicast packet.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
crash_bmc12500Eth_rxPkt_net( bc1250Eth_rxCtrl_t *rxc, 
					   bc1250EthCfg_t     *cfg, 
					   uint32             loopCnt )
{
#ifdef NFP_TXRX
	int 			i = 0;
	uint32			hdrLen;
	uchar8 			*pDstBuff;
	uchar8 			*dvt_tmp;
	volatile uchar8	*bptr;
#endif
	uint32			procCnt;		// Keep track of number of desc processed.
	uint32 			pktLen;
	uint32			bufCnt;
	uchar8 			*newBuf;
	uint64 			status;
	eee_desc_t     	*edesc;
	volatile eee_descBuf_t	*bd;
	volatile bc1250_rxdesc_t*rxCur;
#ifndef NFP_TXRX
	volatile agile_hdr_t	*eeeHdr;
	uchar8 *lmux_hdr = NULL;
	int hdrlen;
#endif
	int	err_flag;
#ifndef NFP_FP
	uchar8 *luc_hdr;
	uchar8 idx;
#endif
uint64 tmp;

	procCnt = 0;
	rxCur = rxc->rxCur;
	
	while( procCnt < loopCnt ) {

		// Make sure that the descriptor we are looking at indicates
		// that the MAC	has completed receiving the entire packet.
		status = rxCur->dw1;
		if( !(status & BC1250_DESC_STATUS_SOP) ) 
			goto bmc12500_rx_finish;

		err_flag = 0;
		if (status & (BC1250_DESC_STATUS_BAD | BC1250_DESC_STATUS_ERR)) {
		//	printf("RX Error, status = 0x%lx\n", status);
		//	printf("MAC Status = 0x%lx\n", *(cfg->regs.mac_status));
			rxc->rxBadErr++;
			err_flag = 1;
		}

		//-- Allocate EEE packet descriptor.
		edesc = eee_crash_allocateDesc(rxc->bufPoolId, NFX_COS_HIGH);
		if( !edesc ) {
			rxc->noDescErr++;
			goto bmc12500_rx_finish;
		}
        eee_attrAssignDesc(edesc, EEE_ATTR_MAC_RX);

		// Get packet len from 2nd descriptor word.
		pktLen = BC1250_GET_PKT_SIZE(rxCur);
		// Calculate the buffer count for this packet.
#if defined(NFP_FP)
        if (rxc->rxqId == 1) {
            if (pktLen <= EEE_BUFSIZE_LARGE)
                bufCnt = 1;
            else {
                bufCnt = pktLen / EEE_BUFSIZE_LARGE;
                if (pktLen % EEE_BUFSIZE_LARGE)
                    bufCnt++;
            }
        }
        else {
#if defined(EEE_NEW_BUFSIZE)
            if (pktLen + 128 <= EEE_BUFSIZE_ULTRA)
                bufCnt = 1;
            else {
                bufCnt = (pktLen + 64) / (EEE_BUFSIZE_ULTRA-64);
                if ((pktLen + 64) % (EEE_BUFSIZE_ULTRA-64))
                    bufCnt++;
            }
#else
            if (pktLen + 256 <= EEE_BUFSIZE_ULTRA)
                bufCnt = 1;
            else {
                bufCnt = (pktLen + 128) / (EEE_BUFSIZE_ULTRA-128);
                if ((pktLen + 128) % (EEE_BUFSIZE_ULTRA-128))
                    bufCnt++;
            }
#endif
        }
#elif defined(NFP_TXRX)
        if (isCheetah()) {
            if (pktLen <= EEE_BUFSIZE_LARGE)
                bufCnt = 1;
            else {
                bufCnt = pktLen / EEE_BUFSIZE_LARGE;
                if (pktLen % EEE_BUFSIZE_LARGE)
                    bufCnt++;
                }
        }
        else {
            bufCnt = (pktLen + rxc->bufOffset) / rxc->descBufSize;
            if( (pktLen + rxc->bufOffset) % rxc->descBufSize )
                bufCnt++;
        }
#else
		bufCnt = (pktLen + rxc->bufOffset) / rxc->descBufSize;
		if( (pktLen + rxc->bufOffset) % rxc->descBufSize )
			bufCnt++;
#endif
		//-- Strip CRC bytes 
		if (!isCheetah())
			pktLen -= 4;

		//-- Mark first buffer as unused.
		edesc->bd[0].control = 0;
		bd = &(edesc->bd[1]);

		if( bmc12500Eth_rx_pkt_trace) {
			printf("BMC Rx: pktLen: %d, BufCnt: %d, Status: %016llx\n", 
					pktLen, bufCnt, status & BC1250_DESC_STATUS );
		}

		if( bufCnt == 1 ) {
			//-- Allocate a new receive buffer and add to rx queue.
			newBuf = eee_crash_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
			if( !newBuf ) {
				rxc->noBufErr++;
				eee_crash_deallocateDesc(edesc);
				goto bmc12500_rx_finish;      // Wait for more bufs.
			}

            eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

			//-- Grab buffer pointer from rx desc.
			BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);
			bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
			
			//-- Data len equal to packet len minus the 4-byte CRC.
			bd->len = pktLen;

#ifdef BC1250_RX_PKT_CAPTURE
#ifdef NFP_TXRX
			if( err_flag && bc1250_pcb[cfg->macId].rxCapEnabled )
		   		bmc1250_rxPktCapture( bd->buf, bd->len, cfg->macId );
#else
			if( bc1250_pcb[cfg->macId].rxCapEnabled )
				bmc1250_rxPktCapture( bd->buf, bd->len, cfg->macId );
#endif // NFP_TXRX
#endif // BC1250_RX_PKT_CAPTURE

			if( bmc12500Eth_rx_pkt_trace) {
				printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
						(address_t)bd->buf, pktLen);
				crash_bmc12500Eth_printBuf( bd->buf, pktLen, 1);  
			}
#ifndef NFP_FP
#ifdef LUC_DEBUG 
		if (isCheetah() && !err_flag) {
			luc_hdr = bd->buf;
			idx = (luc_hdr[4] >> 2) & 0x03;
			luc_rxPkts[idx] += 1;
			luc_rxBytes[idx] += pktLen;
		}
#endif
#endif
			//-- Replenish buffer
			BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);

			//-- FSDEBUG: Clear pkt size
			tmp = rxCur->dw2;
			tmp &= ~BC1250_DESC_PKT_SIZE;
			rxCur->dw2 = tmp;

			//-- Keep track of number of rx descriptors processed.
			procCnt++;

			if( ++rxCur >= rxc->rxqEnd )
				rxCur = rxc->rxqStart;
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
			rxc->rxCur = (bc1250_rxdesc_t *)rxCur;
//			NFX_SPIN_UNLOCK(&rxc->lock);  
//			rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
		}
		else //-- Each rx desc can hold 2 buf ptrs.
		if( bufCnt == 2 ) {
			int32 buf2Len;
			char *newBuf2 = NULL;

			//-- Alloc first new receive buffer.
			newBuf = eee_crash_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
			if( !newBuf ) {
				rxc->noBufErr++;
				eee_crash_deallocateDesc(edesc);
				goto bmc12500_rx_finish;      // Wait for more bufs.
			}

            eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

			//-- Calculate number of data bytes in second buffer.
			buf2Len = pktLen - (rxc->descBufSize - rxc->bufOffset);

			if( bmc12500Eth_rx_pkt_trace) {
				printf("buf2Len: %d\n", buf2Len);
			}

			//-- Alloc second new receive buffer.
			if(buf2Len > 0 ) {
				newBuf2 = eee_crash_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
				if( !newBuf2 ) {
					rxc->noBufErr++;
					eee_crash_deallocateDesc(edesc);
					eee_crash_deallocateBuffer(newBuf); // Release first buffer.
					goto bmc12500_rx_finish;      // Wait for more bufs.
				}
                eee_attrAssignBuf(newBuf2, EEE_ATTR_MAC_RX);
			}
			else {
				bufCnt=1;
			}

			//-- Get first buffer pointer and len.
			BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);
			bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
			bd->len     = (rxc->descBufSize - rxc->bufOffset);

#ifdef BC1250_RX_PKT_CAPTURE
#ifdef NFP_TXRX
			if( err_flag  && bc1250_pcb[cfg->macId].rxCapEnabled)
		   		bmc1250_rxPktCapture( bd->buf, pktLen, cfg->macId );
#endif // NFP_TXRX
#endif // BC1250_RX_PKT_CAPTURE

			if( bmc12500Eth_rx_pkt_trace) {
				printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
						(address_t)bd->buf, bd->len);
				crash_bmc12500Eth_printBuf( bd->buf, bd->len, 1);   
			}
#ifndef NFP_FP
#ifdef LUC_DEBUG 
			if (isCheetah() && !err_flag) {
				luc_hdr = bd->buf;
				idx = (luc_hdr[4] >> 2) & 0x03;
				luc_rxPkts[idx] += 1;
				luc_rxBytes[idx] += pktLen;
			}
#endif
#endif
			//-- Replenish first buffer
			BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);
			if( buf2Len > 0 ) {
				//-- Get second buffer pointer and len.
				bd++;
				BC1250_DESC_GET_B_ADDR(rxCur, (address_t)bd->buf);
				bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
				bd->len     = buf2Len;
							  
				if( bmc12500Eth_rx_pkt_trace) {
					printf("BMC Rx: bufp: %lx, bufLen: %d\n", 
							(address_t)bd->buf, bd->len);
					crash_bmc12500Eth_printBuf( bd->buf, bd->len, 0);   
				}
				//-- Replenish second buffer
				BC1250_DESC_SET_B_ADDR_AND_SIZE(rxCur, (address_t)newBuf2, 
												rxc->descBufSize);
			}
			//-- Keep track of number of rx descriptors processed.
			procCnt++;

			if( ++rxCur >= rxc->rxqEnd )
				rxCur = rxc->rxqStart;
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
			rxc->rxCur = (bc1250_rxdesc_t *)rxCur;
//			NFX_SPIN_UNLOCK(&rxc->lock);
//			rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
		}
		else { //-- Packet is contained in multiple descriptors.
			int32 remain = pktLen;
			uint32 i     = bufCnt;
			uint32 buf_a = 1;
			bc1250_rxdesc_t *rxq_save = (bc1250_rxdesc_t *)rxCur;

			while( i-- ) {
				
				//-- Allocate a new receive buffer and add to rx queue.
				newBuf = eee_crash_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
				if( !newBuf ) {
					rxc->noBufErr++;

					//-- Release bufs and edesc.
					edesc->hdr.control = (pktLen & EDESC_TOTAL_LEN) |
			          (((bufCnt-(i-1))<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
					eee_crash_freePacket( edesc );	

					//-- Put rxq pointer back to where it was.
					rxCur = rxq_save;
					goto bmc12500_rx_finish;	// Wait for more bufs.
				}

                eee_attrAssignBuf(newBuf, EEE_ATTR_MAC_RX);

				if( bmc12500Eth_rx_pkt_trace) {
					printf("BMC RxCur: %lx  ", (address_t)rxCur);
				}
				//-- Get and replenish data buffer.
				if( buf_a ) {
					BC1250_DESC_GET_A_ADDR(rxCur, (address_t)bd->buf);

					//-- Replenish buffer A.
					BC1250_DESC_SET_A_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
											rxc->bufOffset, rxc->descBufSize);
					buf_a = 0;
				}
				else {
					BC1250_DESC_GET_B_ADDR(rxCur, (address_t)bd->buf);

					//-- Replenish buffer B.
					BC1250_DESC_SET_B_ADDR_AND_SIZE(rxCur, (address_t)newBuf, 
													rxc->descBufSize);

					//-- Keep track of number of rx descriptors processed.
					procCnt++;

					//-- Get next descriptor.
					if( ++rxCur >= rxc->rxqEnd )
						rxCur = rxc->rxqStart;
					buf_a = 1;
				}
				//-- Process the last buffer.
				if( i == 0 ) {
					if( remain > 0 ) {
					    bd->len = remain; 
					    bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					}
					else { // Last buffer only contains CRC bytes, so free it.
						eee_crash_deallocateBuffer(bd->buf);
						bd->buf = 0;
						bufCnt--;
					}

					//-- Make sure procCnt gets updated if only first
					//-- buffer is used.
					if( buf_a == 0 )  {
						procCnt++;
						if( ++rxCur >= rxc->rxqEnd )
							rxCur = rxc->rxqStart;
					}
				}
				else
			    if( i == bufCnt-1 ) // First buffer
				{
					bd->len = rxc->descBufSize - rxc->bufOffset;
					bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					remain -= bd->len;
#ifdef BC1250_RX_PKT_CAPTURE
#ifdef NFP_TXRX
			if( err_flag && bc1250_pcb[cfg->macId].rxCapEnabled )
		   		bmc1250_rxPktCapture( bd->buf, pktLen, cfg->macId );
#endif // NFP_TXRX
#endif // BC1250_RX_PKT_CAPTURE

#ifndef NFP_FP
#ifdef LUC_DEBUG 
					if (isCheetah() && !err_flag) {
						luc_hdr = bd->buf;
						idx = (luc_hdr[4] >> 2) & 0x03;
						luc_rxPkts[idx] += 1;
						luc_rxBytes[idx] += pktLen;
					}
#endif
#endif
				}
				else {
					remain -= rxc->descBufSize;
					bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
					if( remain > 0 )
						bd->len = rxc->descBufSize;
					else
					    bd->len = (remain + rxc->descBufSize);
				}
				if( bmc12500Eth_rx_pkt_trace) {
					printf("Rx: bufp: %lx, bufLen: %d\n", 
							(address_t)bd->buf, bd->len);
					if( i== bufCnt-1 )
						crash_bmc12500Eth_printBuf( bd->buf, bd->len, 1);   
					else
						crash_bmc12500Eth_printBuf( bd->buf, bd->len, 0);   
				}
				//-- Next eee buf desc.
				bd++;
			} 
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
			rxc->rxCur = (bc1250_rxdesc_t *)rxCur;
//			NFX_SPIN_UNLOCK(&rxc->lock);
//			rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
		}
		//-- Setup dest and src port information.
#if defined(NFP_TXRX)	
		//-- packet to the LUC application.
		if((cfg->link.loopback == lb_ext) && (sysDVTrunning)) {
				bptr = (uchar8 *)bd->buf + 14;	// Point to padlen field.
				hdrLen = *(ushort16 *)bptr;
				bptr += 2;
				edesc->hdr.dest_port = get_word((char *)bptr);
				edesc->hdr.src_port  = get_word((char *)(bptr+4));
				bd->buf+=(EEE_MAC_HDR_LEN+EEE_AGILE_HDR_LEN+hdrLen+8);
				bd->len-=(EEE_MAC_HDR_LEN+EEE_AGILE_HDR_LEN+hdrLen+8);
				pktLen -= (EEE_MAC_HDR_LEN+EEE_AGILE_HDR_LEN+hdrLen+8);
		}
		else if(dvtFrontCfg.test_started == 1)  {
			uint32 cmd;
			bc1250_rxdesc_t *rxq_save = (bc1250_rxdesc_t *)rxCur;

			pDstBuff = eee_crash_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize,0);
			if(!pDstBuff)
			{
				rxc->noBufErr++;
printf("No more buffers\n");
				//-- Release bufs and edesc.
				edesc->hdr.control = (pktLen & EDESC_TOTAL_LEN) |
			  (((bufCnt-(i-1))<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
				eee_crash_freePacket( edesc );	

				//-- Put rxq pointer back to where it was.
				rxCur = rxq_save;
				goto bmc12500_rx_finish;	// Wait for more bufs.
			}
            eee_attrAssignBuf(pDstBuff, EEE_ATTR_MAC_RX);
			//printf("Allocated new buf %08x\n",pDstBuff);
			//pDstBuff += 2;
			bptr = bd->buf;
			(ushort16 *)dvt_tmp = (ushort16 *)bd->buf+29; /* 56 bytes hdr + 2 bytes alignment*/
			//printf("bptr %08x\n",*(ushort16 *)dvt_tmp);
			//memcpy(pDstBuff, (ushort16 *)dvt_tmp, 28);
			//(ushort16 *)dvt_tmp++;
			memcpy(pDstBuff, (ushort16 *)dvt_tmp, pktLen-29);
			test_hdr = (eee_test_hdr_t  *)pDstBuff;
			cmd = test_hdr->cmd;
			if(cmd == EEE_MGMT_TEST_CMD_DATA)
				dvtFrontCfg.rcvdDataPkt[test_hdr->spin] = 1;
			eee_crash_deallocateBuffer((char *)bptr);
			(uchar8 *)bd->buf = pDstBuff; /* This is the bare data - no hdrs */
			
    		edesc->hdr.dest_port = dvtFrontCfg.sp[test_hdr->spin];
		 	edesc->hdr.src_port = dvtFrontCfg.sp[test_hdr->spin]; 
			//bd->buf+=56;
			bd->len-=58;
			pktLen-=58;
			//crash_bmc12500Eth_printBuf( bd->buf, pktLen, 0);  
		}	
		else {
			edesc->hdr.src_port  = cfg->myPortAddr;		// BMC12500 App Id.
			edesc->hdr.dest_port = cfg->lucRcvAddr; 	// LUC application 
		}	
#elif defined(NFP_FP)
		/* Used as a backplane driver */
		if (isCheetah()) {
#ifdef LMUX_DEBUG
			if (!err_flag) {
				lmux_hdr = edesc->bd[1].buf;
				if (cfg->macId == 0) {
#if defined(NFX_SMP) && !defined(COREDUMP)
					rxc->lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					rxc->lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#else
					lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#endif
				}
				else {
#if defined(NFX_SMP) && !defined(COREDUMP)
					rxc->lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					rxc->lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#else
					lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#endif
				}
				if (lmux_get_rev == 1) { // get lmux revision from header
					lmux_get_rev = 0;
					lmux_rev[0] = lmux_hdr[4];					
					lmux_rev[1] = lmux_hdr[5];					
					lmux_rev[2] = lmux_hdr[6];					
				}
			}
#endif
			eeeHdr = (agile_hdr_t *)(edesc->bd[1].buf + sizeof(enet_hdr_t) + 8);
    		hdrlen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + eeeHdr->pad + 8;
    		edesc->hdr.dest_port = eeeHdr->dest_port;
		    edesc->hdr.src_port = eeeHdr->src_port;
#if defined(NETEEE_FRAGMENT)
    		edesc->hdr.offset = eeeHdr->offset;
#endif     
    		edesc->bd[1].buf += hdrlen;
		    edesc->bd[1].len -= hdrlen;
			pktLen -= hdrlen;
		}
		else {
			pktLen -= eee_removeHeader(edesc);
		}
#elif defined(PROM)
		if( isTXRX() ) {
			if(cfg->link.loopback == lb_none) {
				//-- packet to the LUC application.
				edesc->hdr.src_port  = cfg->myPortAddr;		// BMC12500 App Id.
				edesc->hdr.dest_port = cfg->lucRcvAddr; 	// LUC application 
			}
			else {
				edesc->hdr.src_port  = cfg->myPortAddr;		// BMC12500 App Id.
				edesc->hdr.dest_port = cfg->myPortAddr; 	// BMC12500 App Id.
			}
		}
		else {
			/* Used as a backplane driver */
		if (isCheetah()) {
#ifdef LMUX_DEBUG
			if (!err_flag) {
				lmux_hdr = edesc->bd[1].buf;
				if (cfg->macId == 0) {
#if defined(NFX_SMP) && !defined(COREDUMP)
					rxc->lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					rxc->lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#else
					lmux_rxPkts[lmux_hdr[7] & 0x03]++;
					lmux_rxBytes[lmux_hdr[7] & 0x03] += pktLen;
#endif
				}
				else {
#if defined(NFX_SMP) && !defined(COREDUMP)
					rxc->lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					rxc->lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#else
					lmux_rxPkts[(lmux_hdr[7] & 0x03)+3]++;
					lmux_rxBytes[(lmux_hdr[7] & 0x03)+3] += pktLen;
#endif
				}
			}
#endif
			if (lmux_get_rev == 1) { // get lmux revision from header
				lmux_get_rev = 0;
				lmux_rev[0] = lmux_hdr[4];					
				lmux_rev[1] = lmux_hdr[5];					
				lmux_rev[2] = lmux_hdr[6];					
			}
			eeeHdr = (agile_hdr_t *)(edesc->bd[1].buf + sizeof(enet_hdr_t) + 8);
    		hdrlen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + eeeHdr->pad + 8;
    		edesc->hdr.dest_port = eeeHdr->dest_port;
		    edesc->hdr.src_port = eeeHdr->src_port;
#if defined(NETEEE_FRAGMENT)
    		edesc->hdr.offset = eeeHdr->offset;
#endif     
    		edesc->bd[1].buf += hdrlen;
		    edesc->bd[1].len -= hdrlen;
			pktLen -= hdrlen;
		}
		else {
			pktLen -= eee_removeHeader(edesc);
		}
		}
#endif
		//printf("sp %08x dp %08x\n",edesc->hdr.src_port, edesc->hdr.dest_port);
		//-- Finish setting up the packet.
		edesc->hdr.control = (pktLen & EDESC_TOTAL_LEN) |
			                 ((bufCnt<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

		//-- Drop pkt if rx error detected.
		if( err_flag ) {
#if defined(PROM_SIBYTE)
            // XXX for FP in prom we can get huged (>25KB) garbage pkts on
            //     SSC/chassis powerup. So Ignore any bogus packets received,
            //     this is a temp workaround until hw power-up un-reset sequence
            //     is properly done in HW
            if(isTXRX())
                eee_crash_freePacket(edesc);
            else
                printf("crash_bmc12500Eth_rxPkt_net: bad pkt, ignore desc %p\n", edesc);
#else
            eee_crash_freePacket(edesc);
#endif
			goto bmc12500_rx_next_pkt;
		}

		//-- Send packet up to the E-Layer for further processing.
#if defined(NFP_TXRX) 
		if((dvtFrontCfg.test_started == 1) && (dvtFrontCfg.rcvdDataPkt[test_hdr->spin] == 1)){
				eee_receivePacket(edesc);
				dvtFrontCfg.rcvdDataPkt[test_hdr->spin] = 0;
		}		
		else if(( cfg->link.loopback == lb_none ) && !(luc_loopback_test)) {
            eee_attrAssignPkt(edesc, EEE_ATTR_LUC_RX);
			luc_rcvPkt(edesc);
        }
		else {
			if (luc_loopback_test)
				eee_crash_freePacket(edesc);
			else
				eee_receivePacket(edesc);
		}
#elif defined(NFP_FP)
			if( rxc->rxqId == 1 ) {
				edesc->hdr.dest_port = cfg->sacDestPort |
                    (EEE_GET_APP_ID(edesc->hdr.dest_port) << EDESC2_APP_SHIFT);
			    eee_forwardPacket(edesc);
			}
			else {
				if (lmux_loopback_test)
					eee_crash_freePacket(edesc);
				else {
				    eee_crash_receivePacket(edesc);
				}
			}
#elif defined(PROM)
			if( isTXRX() ) {
				if( cfg->link.loopback == lb_none ) 
					luc_rcvPkt(edesc);
				else {
					if (luc_loopback_test)
						eee_crash_freePacket(edesc);
					else
						eee_receivePacket(edesc);
				}
			}
			else { // NFP_FP
                if (lmux_loopback_test)
                    eee_crash_freePacket(edesc);
                else
                    eee_crash_receivePacket(desc);
			}
#else
			eee_receivePacket(edesc);
#endif

bmc12500_rx_next_pkt:
#if 0
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
		if (!NFX_SPIN_TRYLOCK(&rxc->lock))
			return(procCnt);
//		rxc->lock_cnt[cmGetMyCPUId() - 2]++;
		rxCur = rxc->rxCur;	// get latest queue ptr
#endif
#endif
	; // avoids compiler warning
	}

bmc12500_rx_finish:
	rxc->rxCur = (bc1250_rxdesc_t *)rxCur;
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
//	NFX_SPIN_UNLOCK(&rxc->lock);
//	rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
	return(procCnt);

} /* crash_bmc12500Eth_rxPkt_net */

/*
 *------------------------------------------------------------------------------
 * Function Name:	crash_bmc12500Eth_rxPktPoll
 *
 * Description: 	Called by the eee layer and in turn calls dp83820_rxPacket
 *					for each receive priority queue to collect packet buffers
 *					and pass them up to the eee layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 18, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
extern int32 bmc_rxHWM CPU_PRIVATE;
#ifdef NFP_FP
extern int32 bmc_rxMax;
#else
extern int32 bmc_rxMax;
#endif

void
crash_bmc12500Eth_rxPktPoll( void *cb, uint32 tlimit )
{
	int i;
	uint32 mac_descCnt;
	uint32 dirty_descCnt;
	uint32 descProcCnt;
	bc1250EthCfg_t *cfg;
	
	volatile uint64 *cntReg;
	bc1250Eth_rxCtrl_t *rxc;

#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
	// check for disabled mode 
	if (!bc1250_FP2CPU_active) {
		if (cmGetMyCPUId() == NFX_NFP_FP2)
			return;
	}
	if (!bc1250_FP1CPU_active) {
		if (cmGetMyCPUId() == NFX_NFP_FP1)
			return;
	}
#endif
	cfg = (bc1250EthCfg_t *)cb;
	
	for(i=0; i < BC1250_MAC_RX_Q_CNT; i++) {
		
		// Get the driver rx control block.
		rxc = &(cfg->rxc[i]);

#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
		if (!NFX_SPIN_TRYLOCK(&rxc->lock))
			continue;
//		rxc->lock_cnt[cmGetMyCPUId() - 2]++;
#endif
		// Get the MAC rx desc count.
		cntReg = cfg->regs.dma_dscr_cnt_rx[i];
		if (*cntReg <= 4) // check for low descriptor queue
			rxc->lowDesc++;

		mac_descCnt = *cntReg;
		
		// The difference should give the count of descriptors
		// ready to be processed. Note: mac_descCnt should never be
		// greater than BMC12500_MAC_RX_Q_LEN.
		dirty_descCnt = BC1250_MAC_RX_Q_LEN - mac_descCnt;
		if( !dirty_descCnt ) {
#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
			NFX_SPIN_UNLOCK(&rxc->lock);
//			rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
			continue;
		}

		if( dirty_descCnt > bmc_rxMax )
			dirty_descCnt = bmc_rxMax;
			
		// Call the receive function to collect the completed packets.
		// The function will return the number of descriptors actually
		// processed. 
		descProcCnt = crash_bmc12500Eth_rxPkt_net(rxc, cfg, dirty_descCnt);

#if defined(NFX_SMP) && defined(NFP_FP) && !defined(COREDUMP)
		NFX_SPIN_UNLOCK(&rxc->lock); // preserve sequentiality
//		rxc->unlock_cnt[cmGetMyCPUId() - 2]++;
#endif
		if( descProcCnt > bmc_rxHWM )
			bmc_rxHWM = descProcCnt;

		// Give the MAC back the number of descriptors processed by 
		// the receive function.
		*cntReg = descProcCnt;
	}
	return;
} /* crash_bmc12500Eth_rxPktPoll */

int
crash_bmc12500Eth_printBuf( char *buf, int len, int fDecode )
{
    int i, j;
    char ch;
	char *data;

	data = buf;
	if( !data || !len )
		return 0;

	if( fDecode ) {
		crash_bmc12500Eth_decodeHdr(data);
	}

#if 0
	if(len > 64)
		len = 64;
#endif
	
    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");
	return 0;
} // crash_bmc12500Eth_printBuf

void
crash_bmc12500Eth_decodeHdr( char *data )
{
	uint32 tmp;

	printf("\n");
	printf("MAC DA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[0]&0xff, data[1]&0xff, data[2]&0xff, 
		   data[3]&0xff, data[4]&0xff, data[5]&0xff);
	printf("MAC SA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[6]&0xff, data[7]&0xff, data[8]&0xff, 
		   data[9]&0xff, data[10]&0xff, data[11]&0xff);

	
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+12) & 0xFFFF;
	else
#endif
		tmp = get_word(data+12) & 0xFFFF;
	printf("TYPE    : %04x\n", tmp);

	if((tmp == EEE_ETH_TYPE) || (tmp==EEE_ETH_TYPE_SAC)) {		// Only if Agile Protocol type.
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+16);
	else
#endif
		tmp = get_word(data+16);
		printf("EEE DEST: %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));
#ifndef PROM
	if(txrxdebug)
		tmp = get_word(data+20);
	else
#endif
		tmp = get_word(data+20);
		printf("EEE SRC : %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));

#ifdef NFP_FP
		crash_bmc12500Eth_decodeScsi(data);
#endif
	}
	printf("\n............HEX DUMP.................\n");
} // crash_bmc12500Eth_decodeHdr

int
crash_bmc12500Eth_decodeScsi(char *data) 
{
	uint32 tmp;
	uint32 *msg;

	msg = (uint32 *)(data+32);

	tmp = *msg;
	if( tmp == 0x0401 || tmp == 0x0403 ) {
		printf("\n............SCSI HDR.................\n\n");
		if( tmp == 0x0401 )
			printf("SCSI READ RSP : ");
		else
			printf("SCSI WRITE RSP: ");

		printf("CHANDLE: %x, LBA: %d, LBCNT: %d\n",  
			   *(msg+1), *(msg+5), *(msg+6));
	} 
	return 0;
} // crash_bmc12500Eth_decodeScsi

void
crash_bmc12500Eth_rx()
{
	int i;
	for (i = 0; i < BC1250_MAC_CNT; i++) {
		if (crash_bcEthCfg[i] != NULL) {
			crash_bmc12500Eth_rxPktPoll(crash_bcEthCfg[i], 1000);
		}
	}
}
