/*  
 *=============================================================================
 * File Name   :    bmc12500-eth.h
 *
 *  Description:    
 *
 *  Created    :    04 October 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 *	RCS        : $Id: bmc12500-eth.h,v 1.52 2002/11/27 20:08:22 tonyw Exp $
 *=============================================================================
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/timer-api.h>
#include <exports/cm-api.h>
#include "bmc12500-eth-regs.h"
#include "bmc12500-eth-mii.h"
#include "bmc12500-eth-api.h"
#include <exports/hwMap.h>

#ifndef BMC12500_ETH_H_INCLUDED
#define BMC12500_ETH_H_INCLUDED

#define BC1250_MAX_PKT_SIZE	9800

#define BC1250ETH_BACKPLANE 0
#define BC1250ETH_NETWORK	1
#define BC1250ETH_TXQ_LIST	1

#define BC1250_MAC_CNT		2		// Number of ethernet interfaces
#define BC1250_PF_CNT		2		// Number of pkt fifo interfaces

#if defined(NFP_FP)
#define BC1250_MAC_TX_Q_CNT		2	
#define BC1250_MAC_RX_Q_CNT	 	2
#define BC1250_MAC_RX_Q_LEN		100
#define BC1250_MAC_TX_Q_LEN		100
#define BC1250_LINK_TX_Q_CNT	4
#define BC1250_LINK_TX_Q_LEN	100

#elif defined(NFP_TXRX)

#define BC1250_MAC_TX_Q_CNT		1	
#define BC1250_MAC_RX_Q_CNT	 	1
#define BC1250_MAC_RX_Q_LEN		2048
#define BC1250_MAC_TX_Q_LEN		2048
#define BC1250_LINK_TX_Q_CNT	4
#define BC1250_LINK_TX_Q_LEN	1024

#elif defined(PROM)

#define BC1250_MAC_TX_Q_CNT		1	
#define BC1250_MAC_RX_Q_CNT	 	1
#define BC1250_MAC_RX_Q_LEN	    128	
#define BC1250_MAC_TX_Q_LEN	    128	
#define BC1250_LINK_TX_Q_CNT	4
#define BC1250_LINK_TX_Q_LEN	64

#endif

#define BC1250_MAC_LB_ENABLE		1
#define BC1250_MAC_LB_DISABLE		2

#define BC1250_LINK_TIMER			2000		// Two Seconds

#define BC1250_CLEAR				0	
#define BC1250_SET					1

//--------------------------------------------------
// BC1250 MAC Tx Descriptor Format
//--------------------------------------------------
#if BC1250ETH_TXQ_LIST
typedef struct _bc1250_txdesc {
	uint64		dw1;
	uint64		dw2;
	uint64		edesc;
	uint64		resv;
} bc1250_txdesc_t;
#else
typedef struct _bc1250_txdesc {
	uint64		dw1;
	uint64		dw2;
} bc1250_txdesc_t;
#endif // BC1250ETH_TXQ_LIST

//--------------------------------------------------
// BC1250 MAC Rx Descriptor Format
//--------------------------------------------------
typedef struct _bc1250_rxdesc {
	uint64		dw1;
	uint64		dw2;
} bc1250_rxdesc_t;



//--------------------------------------------------
// Definitions for first double word of descriptor.
//--------------------------------------------------
#define BC1250_DESC_OFFSETA			0x000000000000001Fll
#define BC1250_DESC_A_ADDR			0x000000FFFFFFFFE0ll
#define BC1250_DESC_A_SIZE			0x0001FF0000000000ll
#define BC1250_DESC_INTERRUPT		0x0002000000000000ll
#define BC1250_DESC_OFFSET_B		0x0004000000000000ll
#define BC1250_DESC_STATUS			0xFFF8000000000000ll

#define BC1250_DESC_A_ADDR_SHIFT	5
#define BC1250_DESC_A_SIZE_SHIFT	40
#define BC1250_DESC_STATUS_SHIFT	51

#define BC1250_DESC_SET_A_SIZE(_d_, _sz_)				\
{														\
	uint64 _x_ = ((uint64)(_sz_) / 32LL);				\
    ((_d_)->dw1) &= ~BC1250_DESC_A_SIZE;				\
    ((_d_)->dw1) |= ((_x_)<<BC1250_DESC_A_SIZE_SHIFT)&BC1250_DESC_A_SIZE;\
}

#define BC1250_DESC_GET_A_SIZE(_d_)				\
    ((((_d_)->dw1)&BC1250_DESC_A_SIZE) >> BC1250_DESC_A_SIZE_SHIFT)

#define BC1250_DESC_RX_OFFSET	(isCheetah() ? 0: isTXRX() ? 2:0)	


#define BC1250_DESC_SET_A_ADDR(_d_, _pbuf_, _offset_)	\
{														\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);				\
    ((_d_)->dw1) = ((_buf_) | (_offset_));	            \
}	

#define BC1250_DESC_SET_A_ADDR_AND_SIZE(_d_, _pbuf_, _offset_, _sz_)	\
{														\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);				\
	uint64 _x_ = ((uint64)(_sz_) / 32LL);				\
    ((_d_)->dw1) = ((_buf_) | (_offset_) | (((_x_)<<BC1250_DESC_A_SIZE_SHIFT)&BC1250_DESC_A_SIZE));	            \
}	

#define BC1250_DESC_SET_A_ADDR_TX(_d_, _pbuf_)			\
{														\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);				\
    ((_d_)->dw1) = ((_buf_));							\
}	

#define BC1250_DESC_GET_A_ADDR(_d_, _addr_)			\
{                                                 	\
    uint64 _pbuf_;                                  \
    _pbuf_ = (((_d_)->dw1) & 0x000000FFFFFFFFFFll);	\
    _addr_ = PHYS_TO_VIRT(_pbuf_);					\
}

#define BC1250_DESC_SET_WD_A_TX(_d_, _pbuf_, _sz_, _sop_)			\
{																	\
    uint64 _val_ = VIRT_TO_PHYS(_pbuf_);							\
    _val_ |= ((_sz_)<<BC1250_DESC_A_SIZE_SHIFT)&BC1250_DESC_A_SIZE;	\
    _val_ |= (_sop_);												\
    ((_d_)->dw1) = (_val_);											\
}	


//--------------------------------------------------
// Definitions for second double word of descriptor.
//--------------------------------------------------
#define BC1250_DESC_OPTIONS			0x000000000000000Fll
#define BC1250_DESC_NEXT_DESC		0x000000FFFFFFFFF0ll
#define BC1250_DESC_PKT_SIZE		0xFFFC000000000000ll
#define BC1250_DESC_B_ADDR			0x000000FFFFFFFFE0ll
#define BC1250_DESC_B_SIZE			0x0001FF0000000000ll

#define BC1250_DESC_NEXT_DEST_SHIFT	4	
#define BC1250_DESC_PKT_SIZE_SHIFT	50
#define BC1250_DESC_B_ADDR_SHIFT	5
#define BC1250_DESC_B_SIZE_SHIFT	40

#define BC1250_DESC_B_ADDR_VALID	0x0002000000000000ll

#define BC1250_DESC_SET_B_SIZE(_d_, _sz_)				\
{														\
	uint64 _x_ = ((uint64)(_sz_) / 32LL);				\
    ((_d_)->dw2) &= ~BC1250_DESC_B_SIZE;				\
    ((_d_)->dw2) |= ((_x_)<<BC1250_DESC_B_SIZE_SHIFT)&BC1250_DESC_B_SIZE;\
}

#define BC1250_GET_NEXT_DESC(_d_)	\
    (((_d_)->dw2) & BC1250_DESC_NEXT_DESC)		

#define BC1250_GET_PKT_SIZE(_d_)	\
  ((((_d_)->dw2) & BC1250_DESC_PKT_SIZE) >> BC1250_DESC_PKT_SIZE_SHIFT) 

#define BC1250_DESC_SET_B_ADDR(_d_, _pbuf_)					\
{															\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);					\
	(_buf_) &= BC1250_DESC_B_ADDR;							\
    (((_d_)->dw2) = ((_buf_)) | BC1250_DESC_B_ADDR_VALID);	\
}	

#define BC1250_DESC_SET_B_ADDR_AND_SIZE(_d_, _pbuf_, _sz_)	\
{															\
    uint64 _buf_ = VIRT_TO_PHYS(_pbuf_);					\
	uint64 _x_ = ((uint64)(_sz_) / 32LL);				\
	(_buf_) &= BC1250_DESC_B_ADDR;							\
    (((_d_)->dw2) = ((_buf_)) | BC1250_DESC_B_ADDR_VALID | (((_x_)<<BC1250_DESC_B_SIZE_SHIFT)&BC1250_DESC_B_SIZE));	\
}	

#define BC1250_DESC_GET_B_ADDR(_d_, _addr_)			\
{                                                 	\
    uint64 pbuf;                                    \
    pbuf = (((_d_)->dw2) & BC1250_DESC_B_ADDR);		\
    _addr_ = PHYS_TO_VIRT(pbuf);					\
}

// Compatible with both Cheetah and Garfield
#define BC1250_DESC_SET_WD_B_TX(_d_, _sz_, _txOpt_)			 			\
{																		\
    uint64 val;															\
    val = (_d_)->dw2;                                   				\
    val &= ~BC1250_DESC_PKT_SIZE;				        				\
    val |= ((_sz_)<<BC1250_DESC_PKT_SIZE_SHIFT)&BC1250_DESC_PKT_SIZE;	\
    ((_d_)->dw2) = (val | _txOpt_);										\
}

#define BC1250_DESC_SET_WD_B_TX_CHEETAH(_d_, _sz_)						\
{                                                                       \
    uint64 val;                                                         \
    val = (_d_)->dw2;                                                   \
    val &= ~BC1250_DESC_PKT_SIZE;                                       \
    val |= ((_sz_)<<BC1250_DESC_PKT_SIZE_SHIFT)&BC1250_DESC_PKT_SIZE;   \
    ((_d_)->dw2) = (val | 0x7);                                         \
}

//--------------------------------------------------
// Bit definitions for descriptor status field.
//--------------------------------------------------
#define BC1250_DESC_STATUS_IP4CS	0x0008000000000000ll
#define BC1250_DESC_STATUS_ERR		0x0010000000000000ll
#define BC1250_DESC_STATUS_CHNL		0x0060000000000000ll
#define BC1250_DESC_STATUS_TYPE		0x0380000000000000ll
#define BC1250_DESC_STATUS_HASH		0x0400000000000000ll
#define BC1250_DESC_STATUS_EXACT	0x0800000000000000ll
#define BC1250_DESC_STATUS_BCAST	0x1000000000000000ll
#define BC1250_DESC_STATUS_MCAST	0x2000000000000000ll
#define BC1250_DESC_STATUS_BAD		0x4000000000000000ll
#define BC1250_DESC_STATUS_SOP		0x8000000000000000ll


#define ETHER_SPEED_UNKNOWN	0	
#define ETHER_SPEED_10HDX	1	/* 10MB hdx and fdx */
#define ETHER_SPEED_10FDX	2
#define ETHER_SPEED_100HDX	3	/* 100MB hdx and fdx */
#define ETHER_SPEED_100FDX	4
#define ETHER_SPEED_1000HDX	5	/* 1000MB hdx and fdx */
#define ETHER_SPEED_1000FDX	6


typedef struct _bmc12500Eth_stats {
	uint32 rxPkts;
	uint32 rxOctets;
	uint32 txPkts;
	uint32 txOctets;
	uint32 rxCrc;
	uint32 rxNoBuf;
} bc1250Eth_stats_t;

//-- Device Driver Link Tx Descriptor Format
typedef struct _bc1250_linkDesc {
	uint32		control;
#define BC1250_LINK_TX_EDESC	0
#define BC1250_LINK_TX_RAW		1
	uint32      pktLen;
	uint32      dest;
	union {
		char        *txbuf;
		eee_desc_t  *edesc;
	}o;
} bc1250_ld_t;

//-- Per link transmit fifo control structure.
typedef struct _bc1250_txLinkCtrl {
	bc1250_ld_t *qStart;
	bc1250_ld_t *qEnd;
	bc1250_ld_t *qAddCur;
	bc1250_ld_t *qRemCur;
	uint64      zcycle_cnt;
	uint64      zdiff;
	int32       dCnt;			// Deficit Counter
	uint32      qdCnt;          // Num Pkts currently on the queue.
	uint32		qdTotal;		// Total Num Pkts added to Q.
	uint32      octets;			// Num bytes tx'd from queue.
	uint32      qFullErr;		// Num times q was full.
} bc1250_ltc_t;

//-- BC1250 transmit queue control structure. 
typedef struct _bmc12500Eth_txCtrl {
	bc1250_txdesc_t	*txqStart;
	bc1250_txdesc_t	*txqEnd;
	bc1250_txdesc_t	*txCur;
	bc1250_txdesc_t	*txCln;
	uint32			txqId;
	uint32			qd;
	uint32			pkts;
	uint32			octets;
	uint32			qFull;
	uint32			lmux_txPkts[6];
	uint32			lmux_txBytes[6];
} bc1250Eth_txCtrl_t;

//-- BC1250 ethernet link control structure. 
typedef struct _bmc12500Eth_link {
	fc_t     fc;
	uint32	 timer;
	speed_t  speed;
	duplex_t duplex;
	uint32   linkstat;
	loop_t	 loopback;
} bc1250Eth_link_t;

//-- BC1250 physical device control structure. 
typedef struct _bmc12500Eth_phy {

	uint32	addr;
	union {
		struct {
			uint16 bmsr;
			uint16 bmcr;
			uint16 k1stsr;
			uint16 anlpar;
		}bmc5411;
	}p;
} bc1250Eth_phy_t;

//-- BC1250 receive queue control structure. 
typedef struct _bmc12500Eth_rxCtrl {
	bc1250_rxdesc_t	*rxqStart;
	bc1250_rxdesc_t	*rxqEnd;
	bc1250_rxdesc_t	*rxCur;
	uint32			rxqId;
	uint32			bufPoolId;
	uint32			eeeBufSize;
	uint64			descBufSize;
	uint32          bufOffset;
	uint32          avail;
	uint32			pkts;
	uint32			octets;
	uint32 			noBufErr;
	uint32 			noDescErr;
	uint32			rxBadErr;
	uint32			lowDesc;
	uint32			lmux_rxPkts[6];
	uint32			lmux_rxBytes[6];
	nfx_spinlock_t	lock; // mutex
	uint32			lock_cnt[2]; // count per cpu
	uint32			unlock_cnt[2]; // count per cpu
} bc1250Eth_rxCtrl_t;

//-- Main device driver control structure.
typedef struct _bc1250Eth_cfg {
	uint32				macId;					// Which MAC/PF am I?
	uint32				appId;
	uint32				sacDestPort;
	uint32				myPortAddr;				// Local EEE Port Address.
	uint32				lucRcvAddr;
	uint32				txSeqId;				// Tx Sequence Id for lmux
	char                macAddr[6];		     	// Local Station Mac Addr
	char                macAddr_fp_ctrl[6];	 	// Local Station Mac Addr
	bc1250Eth_regs_t	regs;
	bc1250_rxdesc_t		*rxQ[BC1250_MAC_RX_Q_CNT];

	bc1250Eth_txCtrl_t	txc[BC1250_MAC_TX_Q_CNT];
	bc1250Eth_rxCtrl_t	rxc[BC1250_MAC_RX_Q_CNT];

	bc1250Eth_stats_t	stats;
	bc1250Eth_link_t	link;
	bc1250Eth_phy_t		phy;
} bc1250EthCfg_t;

typedef union _luc_rx_hdr {
    uchar8 data[14];    /* msb is at data[13] */
    struct {
        uint16 pkt_size;        /* lsb */
        uint32 rsvd         :8;
        uint32 l5_offset    :8;
        uint32 l4_offset    :8;
        uint32 l3_offset    :8;
        uint32 cam_index    :16;
        uint32 class_code   :8;
        uint32 link_id      :4;
        uint32 error_code   :4;
        uint32 txerr_seq    :16;
        uint32 tx_seq       :16; /* msb */
    } field;
} luc_rx_hdr_t;

typedef union _luc_tx_hdr {
    uchar8 data[20];    /* msb is at data[19] */
    struct {
        uint32 ip_csum      :8; /* lsb */
        uint32 tcp_udp_csum :8;
        uint32 l4_offset    :8;
        uint32 l3_offset    :8;
        uint32 dest_ip;
        uint32 src_ip;
        uint32 tcp_udp_len  :16;
        uint32 protocol     :8;
        uint32 control      :4;
        uint32 link_id      :4;
        uint32 pkt_len      :16;
        uint32 seq_num      :16; /* msb */
    } field;
} luc_tx_hdr_t;

//-- Defines for the LMUX header.
#define BC1250_LMUX_LINK_ID		0xFF000000

#define BC1250_LMUX_TXERR_SEQID	0x0000FFFF
#define BC1250_LMUX_TX_SEQID	0xFFFF0000
#define BC1250_TX_SEQID_START	0x00010000

/*
 * Macros to convert virtual address to phy and back again.
 */
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#define V2PHYS_ADDR_MASK	(int)0xA0000000
#define BC1250_ETH_2UNCHACHED(_x_)	((_x_) |= 0x20000000)
#else
#define V2PHYS_ADDR_MASK	0x00000000
#define BC1250_ETH_2UNCHACHED(_x_)	((_x_)) 
#endif

//-- Defines for packet capturing.
//-- Pkt tracing built into the runtime
#ifdef NFP_FP
#define BC1250_RX_PKT_CAPTURE		 1
#define BC1250_TX_PKT_CAPTURE		 1
#endif

#ifdef NFP_TXRX
#define BC1250_RX_PKT_CAPTURE		 1
#define BC1250_TX_PKT_CAPTURE		 1
#endif

#define BC1250_CAPTURE_ENTRIES_MAX   512
#define BC1250_BYTES_CAPTURE_MAX     256
 
typedef struct _bc1250_pkt_capBufEntry {
	eeeRTC_t t;
	uint32 pktLen;
	uint32 capLen;
	char   *bufPtr;
    uchar8 bytes[BC1250_BYTES_CAPTURE_MAX];
} bc1250_pcbe_t;
        
typedef struct _bc1250_pkt_capBuf {
	uint32 rxCapEnabled;
	uint32 txCapEnabled;
	uint32 curEntry_tx;
	uint32 curEntry_rx;
	bc1250_pcbe_t pcbe_rx[BC1250_CAPTURE_ENTRIES_MAX];
	bc1250_pcbe_t pcbe_tx[BC1250_CAPTURE_ENTRIES_MAX];
} bc1250_pcb_t;

extern bc1250_pcb_t bc1250_pcb[3];		// packet capture buffer


//-- External Variable Declarations
extern bc1250EthCfg_t *crash_bcEthCfg[BC1250_MAC_CNT+1];
extern bc1250_ltc_t   *crash_bcLinkTxqs[BC1250_LINK_TX_Q_CNT];
#ifdef NFX_SMP
extern bc1250_ltc_t   *bcLinkTxqs_2ndCPU[BC1250_LINK_TX_Q_CNT];
#endif

/*
 * Function Prototypes
 */
int    crash_bmc12500Eth_writeReg(volatile uint64 *, uint64);
uint64 crash_bmc12500Eth_readReg(volatile uint64 *);
void   crash_bmc12500Eth_rxPktPoll(void *, uint32);
void   crash_bmc12500Eth_txCleanPoll(void *, uint32);
void   crash_bmc12500Eth_rxAppPkt(eee_desc_t *);
int crash_bc1250_other_CPUId(void);
int crash_bmc12500Eth_tx_raw(char *, uint32, uint32);
extern void eee_crash_deallocateBuffer(uchar8 *);
extern void eee_crash_deallocateDesc(eee_desc_t *);
extern eee_desc_t *eee_crash_allocateDesc(uint32, uint32);
extern void *eee_crash_allocateBuffer(uint32, uint32, uint32);
extern void *eee_crash_ramAlloc(uint32);
extern void *eee_crash_ramAllocAligned(uint32);
extern void eee_crash_receivePacket(eee_desc_t *);
extern void eee_crash_freePacket(eee_desc_t *);

#ifdef NFP_FP
#define BC1250ETH_GET_FP2FC_LINK_TXQ(_d_)       CM_FC_CPUID2SLICE(EEE_GET_DEST_CPU(_d_))
#endif /* NFP_FP */

#ifdef NFP_TXRX
#define BC1250ETH_GET_TX_PORT( _dest_, _cfg_)			\
{														\
    int _port_;											\
    _port_ = EEE_GET_PORT_ID(_dest_);					\
    if( _port_ == NFP_PORT_TXRX_ETH0 )					\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
	if( _port_ == NFP_PORT_TXRX_ETH1 )					\
		_cfg_ = crash_bcEthCfg[1];							\
    else 												\
	if( _port_ == NFP_PORT_TXRX_ETH2 )					\
		_cfg_ = crash_bcEthCfg[2];							\
    else 												\
	if( _port_ == 0 )									\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
		_cfg_ = 0;										\
}
#elif defined(NFP_FP)
#define BC1250ETH_GET_TX_PORT( _dest_, _cfg_)			\
{														\
    int _port_;											\
    int _slot_;											\
    _port_ = EEE_GET_PORT_ID(_dest_);					\
    _slot_ = EEE_GET_SLOT_NUM(_dest_);				    \
    if( _slot_ == NFX_SLOTTYPE_FC ) {                   \
        int _cpu_;                                      \
        _cpu_ = EEE_GET_DEST_CPU(_dest_);				\
        if( (_cpu_ ==NFX_FCNIM_CPUID3 ) || (_cpu_ == NFX_FCNIM_CPUID2 )) \
		    _cfg_ = crash_bcEthCfg[0];						\
         else {                                         \
			if (isCheetah())							\
		    	_cfg_ = crash_bcEthCfg[2];					\
			else										\
		    	_cfg_ = crash_bcEthCfg[1];					\
		 }												\
    }                                                   \
    else                                                \
    if( _port_ == NFP_PORT_FP_ETH0 )					\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
	if( _port_ == NFP_PORT_FP_ETH1 )					\
		_cfg_ = crash_bcEthCfg[1];							\
    else 												\
	if( _port_ == NFP_PORT_FP_ETH2 )					\
		_cfg_ = crash_bcEthCfg[2];							\
    else 												\
	if( _port_ == 0 )									\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
		_cfg_ = 0;										\
}
#elif defined(PROM)
#define BC1250ETH_GET_TX_PORT( _dest_, _cfg_)			\
{														\
    int _port_;											\
    _port_ = EEE_GET_PORT_ID(_dest_);					\
    if( _port_ == PROM_PORT_FP0 )						\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
	if( _port_ == PROM_PORT_FP1 )						\
		_cfg_ = crash_bcEthCfg[1];							\
    else 												\
	if( _port_ == PROM_PORT_FP2 )						\
		_cfg_ = crash_bcEthCfg[2];							\
    else 												\
	if( _port_ == PROM_PORT_TXRX0 )						\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
	if( _port_ == PROM_PORT_TXRX1 )						\
		_cfg_ = crash_bcEthCfg[1];							\
    else 												\
    if( _port_ == PROM_PORT_TXRX2 )                     \
        _cfg_ = crash_bcEthCfg[2];                            \
    else                                                \
	if( _port_ == 0 )									\
		_cfg_ = crash_bcEthCfg[0];							\
    else 												\
		_cfg_ = 0;										\
}
#endif

/* LUC registers */
#define LUC_CMD_REG                 0xffffffffbf000000ll
#define LUC_SW_REV_REG              0xffffffffbf00000cll

#define LUC_TX0_TCP_LEN				0xffffffffbf000018ll
#define LUC_TX1_TCP_LEN				0xffffffffbf00001cll
#define LUC_TX2_TCP_LEN				0xffffffffbf000020ll
#define LUC_TX3_TCP_LEN				0xffffffffbf000024ll
#define LUC_TX0_IPF_DIAG			0xffffffffbf000028ll
#define LUC_TX1_IPF_DIAG			0xffffffffbf00002cll
#define LUC_TX2_IPF_DIAG			0xffffffffbf000030ll
#define LUC_TX3_IPF_DIAG			0xffffffffbf000034ll
#define LUC_TX_IPF_DIAG_CLR			0xffffffffbf000038ll

#define LUC_HW_REV_REG              0xffffffffbf000010ll
#define LUC_GIO_REG                 0xffffffffbf000154ll
#define LUC_CNTCLR0_REG             0xffffffffbf00016cll
#define LUC_CNTCLR1_REG             0xffffffffbf000170ll
#define LUC_INGRESS0_REG            0xffffffffbf000174ll
#define LUC_INGRESS1_REG            0xffffffffbf000178ll
#define LUC_INGRESS2_REG            0xffffffffbf00017cll
#define LUC_INGRESS3_REG            0xffffffffbf000180ll
#define LUC_EGRESS0_REG             0xffffffffbf000184ll
#define LUC_EGRESS1_REG             0xffffffffbf000188ll
#define LUC_EGRESS2_REG             0xffffffffbf00018cll
#define LUC_EGRESS3_REG             0xffffffffbf000190ll
#define LUC_TO_SIBYTE0_REG          0xffffffffbf000194ll
#define LUC_TO_SIBYTE1_REG          0xffffffffbf000198ll
#define LUC_FROM_SIBYTE0_REG        0xffffffffbf00019cll
#define LUC_FROM_SIBYTE1_REG        0xffffffffbf0001a0ll

#define LUC_RXBYTES0_LO_REG         0xffffffffbf0001a4ll
#define LUC_RXBYTES0_HI_REG         0xffffffffbf0001a8ll
#define LUC_RXBYTES1_LO_REG         0xffffffffbf0001acll
#define LUC_RXBYTES1_HI_REG         0xffffffffbf0001b0ll
#define LUC_RXBYTES2_LO_REG         0xffffffffbf0001b4ll
#define LUC_RXBYTES2_HI_REG         0xffffffffbf0001b8ll
#define LUC_RXBYTES3_LO_REG         0xffffffffbf0001bcll
#define LUC_RXBYTES3_HI_REG         0xffffffffbf0001c0ll

#define LUC_TXBYTES0_LO_REG         0xffffffffbf0001c4ll
#define LUC_TXBYTES0_HI_REG         0xffffffffbf0001c8ll
#define LUC_TXBYTES1_LO_REG         0xffffffffbf0001ccll
#define LUC_TXBYTES1_HI_REG         0xffffffffbf0001d0ll
#define LUC_TXBYTES2_LO_REG         0xffffffffbf0001d4ll
#define LUC_TXBYTES2_HI_REG         0xffffffffbf0001d8ll
#define LUC_TXBYTES3_LO_REG         0xffffffffbf0001dcll
#define LUC_TXBYTES3_HI_REG         0xffffffffbf0001e0ll

#define LUC_TO_SIBYTES0_LO_REG      0xffffffffbf0001e4ll
#define LUC_TO_SIBYTES0_HI_REG      0xffffffffbf0001e8ll
#define LUC_TO_SIBYTES1_LO_REG      0xffffffffbf0001ecll
#define LUC_TO_SIBYTES1_HI_REG      0xffffffffbf0001f0ll

#define LUC_FROM_SIBYTES0_LO_REG    0xffffffffbf0001f4ll
#define LUC_FROM_SIBYTES0_HI_REG    0xffffffffbf0001f8ll
#define LUC_FROM_SIBYTES1_LO_REG    0xffffffffbf0001fcll
#define LUC_FROM_SIBYTES1_HI_REG    0xffffffffbf000200ll

#define LUC_ECC_ERRORS				0xffffffffbf000204ll
#define LUC_TX0_ERRORS				0xffffffffbf000208ll
#define LUC_TX1_ERRORS				0xffffffffbf00020cll
#define LUC_TX2_ERRORS				0xffffffffbf000210ll
#define LUC_TX3_ERRORS				0xffffffffbf000214ll
#define LUC_TO_SIBYTE_EOP			0xffffffffbf000218ll
#define LUC_FROM_SIBYTE_EOP			0xffffffffbf00021cll

#endif // BMC12500_MAC_H_INCLUDED 
