/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage Inc.
 * RCS          :  $Id: datamover-api.h,v 1.3 2002/04/18 03:15:17 pankajg Exp $
 *
 * Name         : datamover-api.h
 *
 * Description  : Interface and API for the Data Mover (DM).
 *
 * Important    : All IP addresses and ports are in network byte order. !!!
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 30-NOV-2001
 *
 *------------------------------------------------------------------------------
 */
#ifndef _DATAMOVER_API_H_
#define _DATAMOVER_API_H_

/*------------------------------------------------------------------------------
 * Header of the message.
 *------------------------------------------------------------------------------
 */

/* Message type
 */
typedef enum
{
    DM_LISTEN_REQ = DM_MSG_BASE,
    DM_LISTEN_RSP,

    DM_ACCEPT_REQ,
    DM_ACCEPT_RSP,

    DM_OPEN_REQ,
    DM_OPEN_RSP,

    DM_SEND_REQ,
    DM_SEND_RSP,

    DM_RCV_IND,

    DM_CLOSE_REQ,
    DM_CLOSE_RSP,
    
    DM_KILL_ME
} dm_msgType_t;

/* Header of the message.
 */
typedef struct
{
    dm_msgType_t    msgType;    // the message type SHALL always be first
    uint64          appHdl;     // handle to user application
    uint64          dmHdl;      // handle to the datamover instance
} dm_msgHdr_t;

/*------------------------------------------------------------------------------
 * Message
 *------------------------------------------------------------------------------
 */

/* Start DM on server (listen for all clients).
 * -------------------------------------------
 * (server only)
 */
typedef struct
{
    dm_msgHdr_t     hdr;        // message header
	ushort16        protocol;   // protocol UDP/TCP
    uint64          lclAddr;    // local IP @ (server)
    uint64        	lclPort;    // local IP @ (server)
} dm_listenReq_t;               // DM_LISTEN_REQ

typedef struct
{
    dm_msgHdr_t     hdr;        // message header
    int32           result;     // operation's result
    uint64          lclAddr;    // local IP @ (server)
    uint64        	lclPort;    // local IP @ (server)
} dm_listenRsp_t;               // DM_LISTEN_RSP

/* Accept client connection.
 * ------------------------
 * (server only)
 */
typedef struct
{
    dm_msgHdr_t hdr;            // message header
    uint64          lclAddr;    // local IP @ (client)  ???
    uint64        	lclPort;    // local IP @ (client)  ???
    uint64          rmtAddr;    // remote IP @ (client)
    uint64        	rmtPort;    // remote IP @ (client)
} dm_acceptReq_t;               // DM_ACCEPT_REQ

typedef struct
{
    dm_msgHdr_t     hdr;        // message header
    int32           result;     // operation's result
} dm_acceptRsp_t;               // DM_ACCEPT_RSP

/* Open connection to the server.
 * -----------------------------
 * (client only)
 */
typedef struct
{
    dm_msgHdr_t     hdr;        // message header
	ushort16        protocol;   // protocol UDP/TCP
    uint64          lclAddr;    // local IP @ (server)
    uint64        	lclPort;    // local IP @ (server)
    uint64          rmtAddr;    // remote IP @ (client)
    uint64        	rmtPort;    // remote IP @ (client)
} dm_openReq_t;                 // DM_OPEN_REQ

typedef struct
{
    dm_msgHdr_t     hdr;        // message header
    int32           result;     // operation's result
    uint64          lclAddr;    // local IP @ (server)
    uint64        	lclPort;    // local IP @ (server)
} dm_openRsp_t;                 // DM_OPEN_RSP

/* Send message/data to the other side.
 * -----------------------------------
 * (server and client)
 */
typedef struct
{
    dm_msgHdr_t     hdr;        // message header
	ushort16        dataSize;	// COD data size
	nfx_cod_t       cod;		// COD
	int32           offset;	    // COD Offset
	ushort16        appHdrSize;	// size of the application header
    //char            appHdr[1];  // application header
} dm_sendReq_t;                 // DM_SEND_REQ

typedef struct
{
    dm_msgHdr_t hdr;            // message header
    int32       result;         // operation's result
} dm_sendRsp_t;                 // DM_SEND_RSP

/* Receive message/data from the other side.
 * ----------------------------------------
 * (server and client)
 */
typedef struct
{
    dm_msgHdr_t     hdr;        // message header
	ushort16        dataSize;	// COD data size
	nfx_cod_t       cod;		// COD
	int32           offset;	    // COD Offset
	ushort16        appHdrSize;	// size of the application header
    //char            appHdr[1];  // application header
} dm_rcvInd_t;                  // DM_RCV_IND

/* Close connection.
 * ----------------
 * (server and client)
 */
typedef struct
{
    dm_msgHdr_t     hdr;        // message header
} dm_closeReq_t;                // DM_CLOSE_REQ

typedef struct
{
    dm_msgHdr_t     hdr;        // message header
    int32           result;     // operation's result
} dm_closeRsp_t;                // DM_CLOSE_RSP

/*----------------------------------------------------------------------------*/

#endif  // _DATAMOVER_API_H_
