/*
 *------------------------------------------------------------------------------
 *
 * Name:    ispfc_debug.h
 *
 * RCS:     $Id: debug.h,v 1.3 2001/08/31 19:01:22 bob Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Runtime assertions (and more) definitions
 *
 * Created by:  Radek Aster
 *
 * Date Created: 8/24/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __DEBUG_H__
#define __DEBUG_H__

/**
 * Print an assertion failure message.
 */
extern void __assert_fail(
    char *__assertion,
    char *__file,
    uint32 __line,
    char *__func);

/**
 * Verify an assertion.  VERIFY checks are always enabled.
 */
#ifdef __GNUC__
#define VERIFY(expr) \
    ((void) ((expr) ? 0 : \
              (__assert_fail (#expr, \
                      __FILE__, __LINE__, __FUNCTION__), 0)))
#else
#define VERIFY(expr) \
    ((void) ((expr) ? 0 : \
              (__assert_fail (#expr, \
                      __FILE__, __LINE__, NULL), 0)))
#endif


/**
 * Assert a condition.  ASSERT checks are enabled when the DEBUG
 * compile-time flag is on.
 */
#ifdef DEBUG
#define ASSERT(expr) VERIFY(expr)
#else
#define ASSERT(expr) ((void) 0)
#endif

#endif /* __DEBUG_H__ */
