/*
 *=============================================================================
 * File Name   :    dp83816-regs.h
 *
 *  Description:    Register definitions for DP83816 (NS 10/100 Ethernet MAC)
 *                  Ported from sm-dp83820 module.
 *
 *  Created    :    05/24/04
 *
 *  Author     :    Shaun Hui
 *
 *  Copyright(c)  2004, OnStor Inc
 *=============================================================================
 */
#ifndef DP83816_REGS_H_INCLUDED
#define DP83816_REGS_H_INCLUDED

typedef struct _dp83816_regs
{
	uint32 cr;					/* 0x00:		Cmd Reg						*/
	uint32 cfg;					/* 0x04:		Config Reg					*/
	uint32 mear;				/* 0x08:		EEProm Access Reg			*/
	uint32 ptscr;				/* 0x0C:		PCI Test Ctrl Reg			*/
	uint32 isr;					/* 0x10:		Interrupt Status Reg		*/
	uint32 imr;					/* 0x14:		Interrupt Mask Reg			*/
	uint32 ier;					/* 0x18:		Interrupt Enable Reg		*/
	uint32 ihr;					/* 0x1C:		Interrupt Holdoff Reg		*/
	uint32 txdp;				/* 0x20:		Tx Desc Pointer Reg			*/

	uint32 txcfg;				/* 0x24:		Tx Config Reg				*/
	uint32 reserved1;			/* 0x28:		Reserved 0x28           	*/
	uint32 reserved2;			/* 0x2c:		Reserved 0x2C           	*/
	uint32 rxdp;				/* 0x30:		Rx Desc Pointer Reg			*/
	uint32 rxcfg;				/* 0x34:		Rx Config Reg				*/
	uint32 reserved3;			/* 0x38:		Reserved 0x38           	*/
	uint32 ccsr;				/* 0x3C:		CLKRUN control/status Reg	*/
	uint32 wcsr;				/* 0x40:		Wake on LAN ctrl/status Reg	*/
	uint32 pcr;					/* 0x44:		Pause Ctrl/Status Reg		*/
	uint32 rfcr;				/* 0x48:		Rx Filter/Match Ctrl Reg	*/
	uint32 rfdr;				/* 0x4C:		Rx Filter/Match Data Reg	*/
	uint32 brar;				/* 0x50:		Boot ROM Address			*/
	uint32 brdr;				/* 0x54:		Boot ROM Data				*/
	uint32 srr;					/* 0x58:		Silicon Revision Reg		*/
	uint32 mibc;				/* 0x5C:		MIB Ctrl Reg				*/
	uint32 mib[8];				/* 0x60-0x78:	MIB Data Regs				*/
	uint32 reserved4;			/* 0x7C:     	Reserved					*/

	uint32 bmcr;				/* 0x80:		Basic Mode Control Reg    	*/
	uint32 bmsr;				/* 0x84:		Basic Mode Status Reg    	*/
	uint32 phyidr1;				/* 0x88:		PHY identifier Reg 1    	*/
	uint32 phyidr2;				/* 0x8C:		PHY identifier Reg 2    	*/
	uint32 anar;				/* 0x90:		Auto-Neg Advertisement Reg	*/
	uint32 anlpar;				/* 0x94:		Auto-Neg Link Pt Ab Reg		*/
	uint32 aner;				/* 0x98:		Auto-Neg Expansion Reg		*/
	uint32 annptr;				/* 0x9C:		Auto-Neg Next Page Tx Reg	*/
	uint32 reserved5[8];		/* 0xA0-0xBC: 	Reserved					*/

	uint32 physts;				/* 0xC0:		PHY Status Reg     			*/
	uint32 micr;				/* 0xC4:		MII Int Ctrl Reg   			*/
	uint32 misr;				/* 0xC8:		MII Int Status Reg			*/
	uint32 reserved6;			/* 0xCC:     	Reserved					*/

	uint32 fcsr;				/* 0xD0:		False carrier sense Reg		*/
	uint32 recr;				/* 0xD4:		Recv error counter Reg		*/
	uint32 pcsr;				/* 0xD4:		100mbps PCS cfg/status Reg	*/
	uint32 resverved7[2];		/* 0xDC-0xE0:	Reserved					*/
	uint32 phycr;				/* 0xE4:		PHY control Reg				*/
	uint32 resverved8[5];		/* 0xEC-0xFC:	Reserved					*/
} dp83816_regs_t;

/*
 *--------------------------------
 * DP83816 COMMAND REGISTER
 *--------------------------------
 */
typedef union _dp83816_cr
{
	uint32 wd;
	struct {
		uint32 txe    :1;
		uint32 txd	  :1;
        uint32 rxe    :1;
		uint32 rxd    :1;
		uint32 txr    :1;
        uint32 rxr    :1;
		uint32 unused2:1;
		uint32 swi	  :1;
		uint32 rst    :1;
		uint32 unused1:23;
	} bits;
} dp83816_cr_t;

/*
 *--------------------------------
 * DP83816 CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83816_cfgr
{
	uint32 wd;
	struct {
		uint32 bem      :1;
		uint32 rsvd1    :1;
		uint32 brom_dis :1;
		uint32 pesel    :1;
		uint32 exd      :1;
		uint32 pow      :1;
		uint32 sb       :1;
		uint32 reqalg   :1;
		uint32 euphcomp :1;
		uint32 phy_dis  :1;
		uint32 phy_rst  :1;
		uint32 rsvd2    :1;
		uint32 ext_phy  :1;
		uint32 aneg_sel :3;
		uint32 pause_adv:1;
		uint32 pint_acen:1;
		uint32 phy_cfg  :6;
		uint32 unused2  :3;
		uint32 aneg_dn  :1;
		uint32 pol      :1;
		uint32 fdup     :1;
		uint32 speed100 :1;
		uint32 lnksts   :1;
	} bits;
} dp83816_cfgr_t;

/*
 *--------------------------------
 * DP83816 RECEIVE CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83816_rxcfgr
{
	uint32 wd;
	struct {
		uint32 unused1  :1;
	    uint32 drth     :5;		
		uint32 unused2  :14;
		uint32 mxdma	:3;
		uint32 unused3  :4;
		uint32 alp		:1;
		uint32 atx   	:1;
		uint32 unused4  :1;
		uint32 arp      :1;
		uint32 aep      :1;
	}bits;
} dp83816_rxcfgr_t;

/*
 *--------------------------------
 * DP83816 TRANSMIT CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83816_txcfgr
{
	uint32 wd;
	struct {
	    uint32 drth     :6;		
		uint32 unused1  :2;
		uint32 flth     :6;
		uint32 unused2  :4;
		uint32 nsc_rsvd :1;
		uint32 unused3  :1;
		uint32 mxdma    :3;
		uint32 ecretry  :1;
		uint32 unused4  :2;
		uint32 ifg      :2;
		uint32 atp      :1;
		uint32 mlb      :1;
		uint32 hbi      :1;
		uint32 csi      :1;
	}bits;
} dp83816_txcfgr_t;

/*
 *----------------------------------------
 * DP83816 RECEIVE FILTER CONTROL REGISTER
 *----------------------------------------
 */
typedef union _dp83816_rfcr
{
	uint32 wd;
	struct {
		uint32 rfaddr  :10;
		uint32 unused  :9;
		uint32 ulm     :1;
		uint32 uhen    :1;
		uint32 mhen    :1;
		uint32 aarp    :1;
		uint32 apat    :4;
		uint32 apm     :1;
		uint32 aau     :1;
		uint32 aam     :1;
		uint32 aab     :1;
		uint32 rfen    :1;
	}bits;
} dp83816_rfcr_t;

/*
 *----------------------------------------
 * DP83816 PAUSE CONTROL REGISTER
 *----------------------------------------
 */
typedef union _dp83816_pcr
{
	uint32 wd;
	struct {
		uint32 pause_cnt	:16;
		uint32 mld_en		:1;
		uint32 unused1		:4;
		uint32 psneg		:1;
		uint32 ps_rcvd		:1;
		uint32 ps_act		:1;
		uint32 unused2		:5;
		uint32 ps_da		:1;
		uint32 ps_mcast		:1;
		uint32 psen			:1;
	}bits;
} dp83816_pcr_t;

#endif /* DP83816_REGS_H_INCLUDED */
