/* =============================================================================
 * File Name  :	dp83820-ui.c
 *
 * Description:    
 *
 * Created    :	08 May 2001
 *
 * Author     :	Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 * =============================================================================
 */
#include "dp83820.h"

extern int dp_rxHWM;

extern int dp83820_printBuf(char *, int, int);
extern int dp83820_diagSetLbMode(dp83820_cfg_t *, uint32);
extern int dp83820_readReg(volatile uint32 *);
extern int dp83820_diagTxPkt(dp83820_cfg_t *);
extern int dp83820_diagTxEna(dp83820_cfg_t *);
extern int dp83820_diagRxEna(dp83820_cfg_t *);
extern int dp83820_diagLBTest(dp83820_cfg_t *);
extern int dp83820_diagViewTestStats(dp83820_cfg_t *);
extern int get_cmd(char8 *);

int dp83820_uiShowRxPTB(dp83820_cfg_t *);
int dp83820_uiSetPktCapture(dp83820_cfg_t *);
int dp83820_findChandle(dp83820_cfg_t *);
int dp83820_uiRxSniffOnOff(dp83820_cfg_t *);
int dp83820_uiShowRxPSB(dp83820_cfg_t *);
int dp83820_uiShowTxPTB(dp83820_cfg_t *);
int dp83820_uiClearStats(dp83820_cfg_t *);
int dp83820_uiSetPktTrace(dp83820_cfg_t *);
int dp83820_uiShowRxq(dp83820_cfg_t *);
int dp83820_uiShowTxq(dp83820_cfg_t *);
int dp83820_uiConfigLBMode(dp83820_cfg_t *);
int dp83820_uiShowOpRegs(dp83820_cfg_t *);
int dp83820_uiViewCfg(dp83820_cfg_t *);
int dp83820_uiViewStats(dp83820_cfg_t *);
int dp83820_uiShowPciRegs(dp83820_cfg_t *);
int dp83820_uiDiagMenu(dp83820_cfg_t *);

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiDiag
 *
 * Description: 	DP83820 UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiDiag()
{
	dp83820_uiDiagMenu( dp83820 );
	return(NFX_OK);
} /* dp83820_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiShowDiagMenu
 *
 * Description: 	Prints DP83820 diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiShowDiagMenu()
{
	int sel;
	char cmd[80];

	sel = 0;
	printf("\n============= DP83820 MENU =============\n");
	printf("1  : Show Driver Cfg        ");
	printf("2  : Show Driver Stats\n");
	printf("3  : Show Op Regs           ");
	printf("4  : Show PCI Regs\n");
	printf("5  : Set/Clear LB Mode      ");
	printf("6  : Show RX Desc Q\n");
	printf("7  : Show TX Desc Q         ");
	printf("8  : Transmit Packet\n");
	printf("9  : Transmit Enable        ");
	printf("10 : Receive Enable\n");
	printf("11 : Run LB Test            ");
	printf("12 : View Test Results\n");
	printf("13 : Ena/Dis Pkt Trace      ");
	printf("14 : Clear Stats\n");
	printf("15 : Show Tx Pkt Trace Buf  ");
	printf("16 : Show Rx Pkt Snif Buf\n");
	printf("17 : Turn Sniff On/Off      ");
	printf("18 : Search For Chandle\n");
	printf("19 : Ena/Dis Pkt Capturing\n");
	printf("20 : Show Rx Pkt Trace Buf  ");
	printf("Enter Selection [99:quit]:  ");
#ifdef LINUX_TEST
	fflush(0);
#endif
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
} /* dp83820_uiShowDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiGetInput( char *msg )
{
	int sel;
	char cmd[80];

	printf("%s: ", msg);
#ifdef LINUX_TEST
	fflush(0);
#else
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
#endif
	return sel; 
} /* dp83820_uiGetInput */
/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiDiagMenu	
 *
 * Description: 	Runs the DP83820 diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiDiagMenu(dp83820_cfg_t *dp)
{
	int sel ;
	for(;;) {
		sel = dp83820_uiShowDiagMenu();
		switch(sel)
		{
			case 1:
				dp83820_uiViewCfg(dp);
				break;
			case 2:
				dp83820_uiViewStats(dp);
				break;
			case 3:
				dp83820_uiShowOpRegs(dp);
				break;
			case 4:
				dp83820_uiShowPciRegs(dp);
				break;
			case 5:
				dp83820_uiConfigLBMode(dp);
				break;
			case 6:
				dp83820_uiShowRxq(dp);
				break;
			case 7:
				dp83820_uiShowTxq(dp);
				break;
			case 8:
				dp83820_diagTxPkt(dp);
				break;
			case 9:
				dp83820_diagTxEna(dp);
				break;
			case 10:
				dp83820_diagRxEna(dp);
				break;
			case 11:
				dp83820_diagLBTest(dp);
				break;
			case 12:
				dp83820_diagViewTestStats(dp);
				break;
			case 13:
				dp83820_uiSetPktTrace(dp);
				break;
			case 14:
				dp83820_uiClearStats(dp);
				break;
			case 15:
				dp83820_uiShowTxPTB(dp);
				break;
			case 16:
				dp83820_uiShowRxPSB(dp);
				break;
			case 17:
				dp83820_uiRxSniffOnOff(dp);
				break;
			case 18:
				dp83820_findChandle(dp);
				break;
			case 19:
				dp83820_uiSetPktCapture(dp);
				break;
			case 20:
				dp83820_uiShowRxPTB(dp);
				break;
			case 99:
			case 999:
				return 0;
			default:
				break;
		}
	} 
	return(NFX_OK);
} /* dp83820_uiDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiShowOpRegs( dp83820_cfg_t *dp )
{
	dp83820_regs_t *r;

	r = dp->regs;
	printf("DP83820 OPERATIONAL REGISTER DUMP: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("========================================================== \n\n");
	printf("CMD                     :%08x\n", dp83820_readReg(&r->cr));
	printf("CFG                     :%08x\n", dp83820_readReg(&r->cfg));
	printf("EEProm Access           :%08x\n", dp83820_readReg(&r->mear));
	printf("PCI Test Control        :%08x\n", dp83820_readReg(&r->ptscr));
	printf("Interrupt Status        :%08x\n", dp83820_readReg(&r->isr));
	printf("Interrupt Mask          :%08x\n", dp83820_readReg(&r->imr));
	printf("Interrupt Enable        :%08x\n", dp83820_readReg(&r->ier));
	printf("Interrupt Holdoff       :%08x\n", dp83820_readReg(&r->ihr));
	printf("Tx Descriptor Ptr       :%08x\n", dp83820_readReg(&r->txdp));
	printf("Tx Descriptor Ptr Hi    :%08x\n", dp83820_readReg(&r->txdp_hi));
	printf("Tx Config               :%08x\n", dp83820_readReg(&r->txcfg));
	printf("Gen Purpose I/O Ctrl    :%08x\n", dp83820_readReg(&r->gpior));
	printf("Rx Descriptor Ptr       :%08x\n", dp83820_readReg(&r->rxdp));
	printf("Rx Descriptor Ptr Hi    :%08x\n", dp83820_readReg(&r->rxdp_hi));
	printf("Rx Config               :%08x\n", dp83820_readReg(&r->rxcfg));
	printf("Priority Q Ctrl         :%08x\n", dp83820_readReg(&r->pqcr));
	printf("Wake Ctrl/Status        :%08x\n", dp83820_readReg(&r->wcsr));
	printf("Pause Ctrl/Status       :%08x\n", dp83820_readReg(&r->pcr));
	printf("Rx Filter/Match Ctrl    :%08x\n", dp83820_readReg(&r->rfcr));
	printf("Rx Filter/Match Data    :%08x\n", dp83820_readReg(&r->rfdr));
	printf("Boot ROM Address        :%08x\n", dp83820_readReg(&r->brar));
	printf("Boot ROM Data           :%08x\n", dp83820_readReg(&r->brdr));
	printf("Silicon Revision        :%08x\n", dp83820_readReg(&r->srr));
	printf("MIB Ctrl                :%08x\n", dp83820_readReg(&r->mibc));
	printf("MIB0 RXErroredPkts      :%08x\n", dp83820_readReg(&r->mib[0]));
	printf("MIB1 RXFCSErrors        :%08x\n", dp83820_readReg(&r->mib[1]));
	printf("MIB2 RxFifoOverrun      :%08x\n", dp83820_readReg(&r->mib[2]));
	printf("MIB3 RXFAErrors         :%08x\n", dp83820_readReg(&r->mib[3]));
	printf("MIB4 RXSymbolErrors     :%08x\n", dp83820_readReg(&r->mib[4]));
	printf("MIB5 RXBig              :%08x\n", dp83820_readReg(&r->mib[5]));
	printf("MIB6 RXIRLErrors        :%08x\n", dp83820_readReg(&r->mib[6]));
	printf("MIB7 RXBadOpcodes       :%08x\n", dp83820_readReg(&r->mib[7]));
	printf("MIB8 RxPauseFrames      :%08x\n", dp83820_readReg(&r->mib[8]));
	printf("MIB9 TxPauseFrames      :%08x\n", dp83820_readReg(&r->mib[9]));
	printf("MIB10 TXSQEErrors       :%08x\n", dp83820_readReg(&r->mib[10]));
	printf("Tx Desc Ptr Priority 1  :%08x\n", dp83820_readReg(&r->txdp1));
	printf("Tx Desc Ptr Priority 2  :%08x\n", dp83820_readReg(&r->txdp2));
	printf("Tx Desc Ptr Priority 3  :%08x\n", dp83820_readReg(&r->txdp3));
	printf("Rx Desc Ptr Priority 1  :%08x\n", dp83820_readReg(&r->rxdp1));
	printf("Rx Desc Ptr Priority 2  :%08x\n", dp83820_readReg(&r->rxdp2));
	printf("Rx Desc Ptr Priority 3  :%08x\n", dp83820_readReg(&r->rxdp3));
	printf("VLAN/IP Rx Ctrl         :%08x\n", dp83820_readReg(&r->vrcr));
	printf("VLAN/IP Tx Ctrl         :%08x\n", dp83820_readReg(&r->vtcr));
	printf("VLAN Data               :%08x\n", dp83820_readReg(&r->vdr));
	printf("Clock Ctrl/Status       :%08x\n", dp83820_readReg(&r->ccsr));
	printf("TBI Ctrl                :%08x\n", dp83820_readReg(&r->tbicr));
	printf("TBI Status              :%08x\n", dp83820_readReg(&r->tbisr));
	printf("TBI Auto-Neg Ad         :%08x\n", dp83820_readReg(&r->tanar));
	printf("TBI Auto-Neg Link Pt    :%08x\n", dp83820_readReg(&r->tanlpar));
	printf("TBI Auto-Neg Exp        :%08x\n", dp83820_readReg(&r->taner));
	printf("TBI Ext Status          :%08x\n", dp83820_readReg(&r->tesr));
	printf("\n");

#if 0
	printf("PHY: BMCR               :%04x\n", dp83820_readMiiReg(dp, 0));
	printf("PHY: BMSR               :%04x\n", dp83820_readMiiReg(dp, 1));
	printf("PHY: ANAR               :%04x\n", dp83820_readMiiReg(dp, 4));
	printf("PHY: ANLPAR             :%04x\n", dp83820_readMiiReg(dp, 5));
	printf("PHY: ANER               :%04x\n", dp83820_readMiiReg(dp, 6));
	printf("PHY: 1KTCR              :%04x\n", dp83820_readMiiReg(dp, 9));
	printf("PHY: 1KSTSR             :%04x\n", dp83820_readMiiReg(dp, 10));
	printf("PHY: Strap_Opt          :%04x\n", dp83820_readMiiReg(dp, 16));
	printf("PHY: PHY_SUP            :%04x\n", dp83820_readMiiReg(dp, 17));
	printf("PHY: MDIX_sel           :%04x\n", dp83820_readMiiReg(dp, 21));
#endif
	return(NFX_OK);
} /* dp83820_uiShowAllRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiShowPciRegs( dp83820_cfg_t *dp )
{
	dp83820_regs_t *r;

	r = dp->regs;
	printf("DP83820 PCI REGISTER DUMP: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("================================================== \n\n");
	printf("Config Id                   :%08x\n", 
					dp83820_readReg(&r->pciCfgRegs[0]));
	printf("Config Cmd and Status       :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[1]));
	printf("Config Revision Id          :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[2]));
	printf("Config Latency Timer        :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[3]));
	printf("Config IO Base Address      :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[4]));
	printf("Config Memory Address Lo    :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[5]));
	printf("Config Memory Address Hi    :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[6]));
	printf("Config Subsystem Id         :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[11]));
	printf("Boot ROM Config             :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[12]));
	printf("Capabilities Pointer        :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[13]));
	printf("Config Interrupt Select     :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[15]));
	printf("Power Mgmt Capabilities     :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[16]));
	printf("Power Mgmt Ctrl/Status      :%08x\n", 
			dp83820_readReg(&r->pciCfgRegs[17]));
	printf("\n");
	return(NFX_OK);
} /* dp83820_uiShowPciRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiViewStats	
 *
 * Description: 	Display device/driver statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiViewStats(dp83820_cfg_t *dp)
{
	uint32 i;
	dp83820_regs_t *r;

	r = dp->regs;

	printf("DP83820 STATISTICS: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("================== Receive Q Statistics ================= \n\n");
	for(i=0; i < DP83820_RX_Q_CNT; i++) {
		printf("RxQ %d avail       : %8u       ", i, dp->rxc[i].avail);
		printf("RxQ %d pkts        : %8u\n", i, dp->rxc[i].pkts);
		printf("RxQ %d octets      : %8u       ", i, dp->rxc[i].octets);
		printf("RxQ %d rxo err     : %8u\n", i, dp->rxc[i].rxoErr);
		printf("RxQ %d rxa err     : %8u       ", i, dp->rxc[i].rxaErr);
		printf("RxQ %d no buf err  : %8u\n", i, dp->rxc[i].noBufErr);
		printf("RxQ %d no desc err : %8u       ", i, dp->rxc[i].noDescErr);
		printf("RxQ %d too long err: %8u\n", i, dp->rxc[i].tooLongErr);
		printf("RxQ %d runt err    : %8u       ", i, dp->rxc[i].runtErr);
		printf("RxQ %d is err      : %8u\n", i, dp->rxc[i].isErr);
		printf("RxQ %d crc err     : %8u       ", i, dp->rxc[i].crcErr);
		printf("RxQ %d fa err      : %8u\n", i, dp->rxc[i].faErr);
		printf("RxQ %d lbp err     : %8u       ", i, dp->rxc[i].lbpErr);
		printf("RxQ %d irl err     : %8u\n", i, dp->rxc[i].irlErr);
	}
	printf("HWM Rx desc Qd    : %8u\n", dp_rxHWM);
	printf("\n");
	printf("================== Transmit Q Statistics ================= \n\n");
	for(i=0; i < DP83820_TX_Q_CNT; i++) {
		printf("TxQ %d avail       : %8u       ", i, dp->txc[i].avail);
		printf("TxQ %d pkts        : %8u\n", i, dp->txc[i].pkts);
		printf("TxQ %d octets      : %8u       ", i, dp->txc[i].octets);
		printf("TxQ %d qFull err   : %8u\n", i, dp->txc[i].qFull);
		printf("TxQ %d txa err     : %8u       ", i, dp->txc[i].txaErr);
		printf("TxQ %d tfu err     : %8u\n", i, dp->txc[i].tfuErr);
		printf("TxQ %d crs err     : %8u       ", i, dp->txc[i].crsErr);
		printf("TxQ %d td err      : %8u\n", i, dp->txc[i].tdErr);
		printf("TxQ %d ed err      : %8u       ", i, dp->txc[i].edErr);
		printf("TxQ %d owc err     : %8u\n", i, dp->txc[i].owcErr);
		printf("TxQ %d ec err      : %8u       ", i, dp->txc[i].ecErr);
		printf("TxQ %d bufs Qd     : %8u\n", i, dp->txc[i].qd);
		printf("TxQ %d bufs un Qd  : %8u\n", i, dp->txc[i].unqd);
	}
	printf("\n");
	printf("================== DP83820 MIB Counters ================= \n\n");
	printf("MIB0 RXErroredPkts: %8u       ", dp83820_readReg(&r->mib[0]));
	printf("MIB1 RXFCSErrors  : %8u\n", dp83820_readReg(&r->mib[1]));
	printf("MIB2 RxFifoOverrun: %8u       ", dp83820_readReg(&r->mib[2]));
	printf("MIB3 RXFAErrors   : %8u\n", dp83820_readReg(&r->mib[3]));
	printf("MIB4 RXSymbolErrs : %8u       ", dp83820_readReg(&r->mib[4]));
	printf("MIB5 RXBig        : %8u\n", dp83820_readReg(&r->mib[5]));
	printf("MIB6 RXIRLErrors  : %8u       ", dp83820_readReg(&r->mib[6]));
	printf("MIB7 RXBadOpcodes : %8u\n", dp83820_readReg(&r->mib[7]));
	printf("MIB8 RxPauseFrames: %8u       ", dp83820_readReg(&r->mib[8]));
	printf("MIB9 TxPauseFrames: %8u\n", dp83820_readReg(&r->mib[9]));
	printf("MIB10 TXSQEErrors : %8u\n", dp83820_readReg(&r->mib[10]));
	printf("\n");
	printf("================== DP83820 Per Slot ================= \n\n");
	printf("Slot0 Cpu0 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[0][0]);
	printf("Slot0 Cpu0 Tx Pkts: %8u\n", dp->txc[0].txpkts[0][0]);
	printf("Slot1 Cpu0 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[1][0]);
	printf("Slot1 Cpu0 Tx Pkts: %8u\n", dp->txc[0].txpkts[1][0]);
	printf("Slot1 Cpu1 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[1][1]);
	printf("Slot1 Cpu1 Tx Pkts: %8u\n", dp->txc[0].txpkts[1][1]);
	printf("Slot1 Cpu2 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[1][2]);
	printf("Slot1 Cpu2 Tx Pkts: %8u\n", dp->txc[0].txpkts[1][2]);
	printf("Slot1 Cpu3 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[1][3]);
	printf("Slot1 Cpu3 Tx Pkts: %8u\n", dp->txc[0].txpkts[1][3]);
	printf("Slot2 Cpu0 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[2][0]);
	printf("Slot2 Cpu0 Tx Pkts: %8u\n", dp->txc[0].txpkts[2][0]);
	printf("Slot2 Cpu1 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[2][1]);
	printf("Slot2 Cpu1 Tx Pkts: %8u\n", dp->txc[0].txpkts[2][1]);
	printf("Slot2 Cpu2 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[2][2]);
	printf("Slot2 Cpu2 Tx Pkts: %8u\n", dp->txc[0].txpkts[2][2]);
	printf("Slot2 Cpu3 Rx Pkts: %8u       ", dp->rxc[0].rxpkts[2][3]);
	printf("Slot2 Cpu3 Tx Pkts: %8u\n", dp->txc[0].txpkts[2][3]);
	return(NFX_OK);
} /* dp83820_uiViewStats */

char *dp_link_speed_str[] = {"10Mbps-","100Mbps-","1000Mbps-","unknown"};
char *dp_link_duplex_str[] = {"hdx", "fdx", ""};
char *dp_link_fc_str[] = {"disabled", "frame", "collision", "carrier","unknown"};
char *dp_link_lb_str[] = {"none", "internal", "phy", "external"};

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 20, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiViewCfg(dp83820_cfg_t *dp)
{
	uint32 i;

	printf("DP83820 CONFIGURATION: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("=============== General Config ============== \n\n");
	printf("Application Id             : %08d\n", dp->appId);
	printf("Base Addr                  : %x\n", dp->baseAddr);
	printf("Phy Addr                   : %08x\n", dp->phyAddr);
	printf("\n");
	printf("=============== Link Control ================ \n\n");
	printf("Link State:                : %s\n", 
		   dp->link.linkstat == ETHER_SPEED_UNKNOWN ? "down":"up");
	printf("Link Speed:                : %s%s\n", dp_link_speed_str[dp->link.speed],
		                                           dp_link_duplex_str[dp->link.duplex]);
	printf("Link Flow Control Mode     : %s\n", dp_link_fc_str[dp->link.fc]);
	printf("Link Loopback Mode         : %s\n", dp_link_lb_str[dp->link.loopback]);
	printf("\n");
	printf("=============== Transmit Control ============== \n\n");
	for(i=0; i < DP83820_TX_Q_CNT; i++) {
		printf("TxQ %d Start                : %p\n", i, dp->txc[i].txqStart);
		printf("TxQ %d End                  : %p\n", i, dp->txc[i].txqEnd);
		printf("TxQ %d Current Addr         : %p\n", i, dp->txc[i].txCur);
		printf("TxQ %d Cleanup Addr         : %p\n", i, dp->txc[i].txCln);
	}
	printf("\n");
	printf("=============== Receive Control ============== \n\n");
	for(i=0; i < DP83820_RX_Q_CNT; i++) {
		printf("Rx Desc Pool %d Base Addr   : %p\n", i, dp->rxQ[i]);
		printf("RxQ %d Start                : %p\n", i, dp->rxc[i].rxqStart);
		printf("RxQ %d End                  : %p\n", i, dp->rxc[i].rxqEnd);
		printf("RxQ %d Current Addr         : %p\n", i, dp->rxc[i].rxCur);
		printf("RxQ %d Buffer Count         : %08d\n", i, dp->rxc[i].bufCnt);
	}
	printf("\n");
	return(NFX_OK);
} /* dp83820_uiViewCfg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiSetLoopbackMode
 *
 * Description: 	Allows user to enable/disable device loopback.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiConfigLBMode( dp83820_cfg_t *dp )
{
	int32 mode;

	mode = dp83820_uiGetInput("Set Loopback Mode [ON:1 | OFF:2]: \n");
	if((mode != 1) && (mode != 2)) {
        printf("Invalid Mode: %d. Exiting....\n", mode);
        return(NFX_OK);
    }    
	if( mode == 1 )
		dp83820_diagSetLbMode(dp83820, DP83820_LB_ENABLE); 
	else
		dp83820_diagSetLbMode(dp83820, DP83820_LB_DISABLE); 

	return(NFX_OK);
} /* dp83820_uiConfigLBMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiShowRxq
 *
 * Description: 	Dump the chip receive descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiShowRxq( dp83820_cfg_t *dp)
{
	int i, quit;
	volatile dp83820_desc_t *descp;

	descp = dp->rxc[0].rxqStart;

	printf("DP83820 Receive Descriptor Queue Dump\n");
	printf("=====================================\n");
	for(i=0; i < DP83820_RX_Q_LEN; i++) {
		printf("%03d   Link: %p   Buf: %p   Cmdsts: %08x   Edesc: %p\n",
			      i, descp->link, descp->bufptr, descp->cmdsts, descp->pktDesc);
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = dp83820_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} /* dp83820_uiShowRxq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_uiShowTxq
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_uiShowTxq( dp83820_cfg_t *dp)
{
	int i, quit;
	volatile dp83820_desc_t *descp;

	descp = dp->txc[0].txqStart;

	printf("DP83820 Transmit Descriptor Queue Dump\n");
	printf("======================================\n");
	for(i=0; i < DP83820_TX_Q_LEN; i++) {
		printf("%03d   Link: %p   Buf: %p   Cmdsts: %08x   Edesc: %p\n",
			      i, descp->link, descp->bufptr, descp->cmdsts, descp->pktDesc);
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = dp83820_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} /* dp83820_uiShowTxq */

/*
 *-----------------------------------------------------------------------------
 * Function Name:   dp83820_uiSetPktTrace
 *
 * Description:     Enables/Disables packet tracing in dp83820 driver.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    August 08, 2001
 *
 * Returns: NFX_OK
 *
 *-----------------------------------------------------------------------------
 */
int
dp83820_uiSetPktTrace(dp83820_cfg_t *dp)
{
	uint32        sel;
	extern uint32 dp83820_rx_pkt_trace;
	extern uint32 dp83820_tx_pkt_trace;

	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(dp83820_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(dp83820_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    sel = dp83820_uiGetInput("Rx Trace Dis/Ena [0|1], Tx Trace Dis/Ena [2|3]: ");
	switch(sel) {
	
		case 0:
			dp83820_rx_pkt_trace = 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			break;
		case 2:
			dp83820_tx_pkt_trace = 0;
			break;
		case 3:
			dp83820_tx_pkt_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(dp83820_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(dp83820_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    printf("\n");
    return 0;
} /* dp83820_uiSetPktTrace */

/*
 *-----------------------------------------------------------------------------
 * Function Name:   dp83820_bpStats
 *
 * Description:     Display quick summary stats
 *
 * Created by:      Tony Wong
 *
 * Data Created:    01/25/02
 *
 * Returns: NFX_OK
 *
 *-----------------------------------------------------------------------------
 */
int dp83820_bpStats()
{
    dp83820_cfg_t *dp;

    dp = dp83820;
	dp83820_uiViewStats(dp);
	dp83820_uiClearStats(dp);
	return(0);
}

int
dp83820_uiClearStats(dp83820_cfg_t *dp)
{
	int i;
	dp83820_regs_t *r;

	//-- Clear chip MIB register counters.
	r = dp->regs;
	dp83820_writeReg(&r->mibc, 0x4);

	//-- Clear device driver software counters.
	for(i=0; i < DP83820_RX_Q_CNT; i++) {
		dp->rxc[i].pkts	    = 0;
		dp->rxc[i].octets	= 0;
		dp->rxc[i].rxoErr	= 0;
		dp->rxc[i].rxaErr	= 0;
		dp->rxc[i].noBufErr	= 0;
		dp->rxc[i].noDescErr= 0;
		dp->rxc[i].tooLongErr= 0;
		dp->rxc[i].runtErr	= 0;
		dp->rxc[i].isErr	= 0;
		dp->rxc[i].crcErr	= 0;
		dp->rxc[i].faErr	= 0;
		dp->rxc[i].lbpErr	= 0;
		dp->rxc[i].irlErr	= 0;
		memset(&dp->rxc[i].rxpkts, 0, sizeof(uint32) * 3 * 5);
	}
	for(i=0; i < DP83820_TX_Q_CNT; i++) {
		dp->txc[i].pkts	    = 0;
		dp->txc[i].octets	= 0;
		dp->txc[i].qFull	= 0;
		dp->txc[i].txaErr	= 0;
		dp->txc[i].tfuErr	= 0;
		dp->txc[i].crsErr	= 0;
		dp->txc[i].tdErr	= 0;
		dp->txc[i].edErr	= 0;
		dp->txc[i].owcErr	= 0;
		dp->txc[i].ecErr	= 0;
		dp->txc[i].qd	    = 0;
		dp->txc[i].unqd	    = 0;
		memset(&dp->txc[i].txpkts, 0, sizeof(uint32) * 3 * 5);
	}
	dp_rxHWM = 0;
	return 0;
} /* dp83820_uiClearStats */

int
dp83820_uiShowTxPTB(dp83820_cfg_t *dp)
{
#ifdef DP83820_TX_PKT_CAPTURE
	int i, j, quit;
	char *timestr;
	dp83820_ptb_t *ptb;

	j = dp83820_curTxTraceEntry - 1;
	for(i=0; i < DP83820_TRACE_ENTRIES_MAX; i++ ) {

		ptb = &dp83820_tx_ptb[j];

		timestr = eee_humandate((uint32)eee_rtcToSec(&ptb->t));
        printf("\n== Time: %s == BufSize: %d == OrgBuf: %lx == cmdsts: %x == (Entry: %d)\n", 
				timestr, ptb->pktLen, (address_t)ptb->orgBuf, ptb->cmdsts, j);

		dp83820_printBuf( ptb->bytes, ptb->capLen, 1);

		if( --j <= 0 )
			j = DP83820_TRACE_ENTRIES_MAX-1;

		if( !(i % 5) && i != 0 ) {
			quit = dp83820_uiGetInput("Quit [99]: ");
		    if( quit == 99 )
				return 0;
		}
	}
#else
	printf("Tx capture buf not compiled into this build.\n");
#endif
	return 0;
} // dp83820_uiShowTxPTB

int
dp83820_uiShowRxPTB(dp83820_cfg_t *dp)
{
#ifdef DP83820_RX_PKT_CAPTURE
	int i, j, quit;
	char *timestr;
	dp83820_ptb_t *ptb;

	j = dp83820_curRxTraceEntry - 1;
	for(i=0; i < DP83820_TRACE_ENTRIES_MAX; i++ ) {

		ptb = &dp83820_rx_ptb[j];

		timestr = eee_humandate(eee_rtcToSec(&ptb->t));
        printf("\n== Time: %s == BufSize: %d == OrgBuf: %lx == cmdsts: %x == (Entry: %d)\n", 
				timestr, ptb->pktLen, (address_t)ptb->orgBuf, ptb->cmdsts, j);

		dp83820_printBuf( ptb->bytes, ptb->capLen, 1);

		if( --j <= 0 )
			j = DP83820_TRACE_ENTRIES_MAX-1;

		if( !(i % 2) && i != 0 ) {
			quit = dp83820_uiGetInput("Quit [99]: ");
		    if( quit == 99 )
				return 0;
		}
	}
#else
	printf("Rx capture buf not compiled into this build.\n");
#endif
	return 0;
} // dp83820_uiShowRxPTB
int
dp83820_uiShowRxPSB(dp83820_cfg_t *dp)
{
#ifdef DP83820_SNIFFER
	int i, j, quit;
	char *timestr;
	int display_len;
	int display_entry;
	dp83820_psb_t *psb;

    if(!CM_IS_FC_CPUID0) {   // cpuid 0 is fc slice 3 on slot 2
		printf("This feature only available on SP_NIM slice 3.\n");
		return 0;
	}
		
    display_len   = dp83820_uiGetInput("Enter Num Bytes To Display : [16-9000]");
    display_entry = dp83820_uiGetInput("Enter Entry To Display From: [0-2047, Last=9999] ");


	if( display_entry > 2047 || display_entry < 0 )
		j = dp83820_curRxSnifEntry - 1;
	else
		j = display_entry;
		
	if( j < 0 )
		j = 0;

	for(i=0; i < DP83820_SNIF_ENTRIES_MAX; i++ ) {

		psb = &dp83820_rx_psb[j];

		timestr = eee_humandate((uint32)eee_rtcToSec(&psb->t));
        printf("\n== Time: %s == BufSize: %d == cmdsts: %x == (Entry: %d)\n", timestr, 
			   psb->bufLen, psb->cmdsts, j);

		if( psb->capLen < display_len )
			display_len = psb->capLen;

		if( psb->buf )
			dp83820_printBuf( psb->buf, display_len, 1);

		if( --j <= 0 )
			j = DP83820_SNIF_ENTRIES_MAX-1;

		quit = dp83820_uiGetInput("Quit [99]: ");
		if( quit == 99 )
			return 0;
	}
#else
	printf("Rx Sniffer feature not compiled into this build.\n");
#endif
	return 0;
} //dp83820_uiShowRxPSB

//-----------------------------------------------------------------------------
// Function Name:   dp83820_uiRxSniffOnOff
//
// Description:     Enables/Disables packet sniffing in dp83820 driver.
//
// Created by:      Frank Salzmann
//
// Data Created:    June 10, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
dp83820_uiRxSniffOnOff(dp83820_cfg_t *dp)
{
#ifdef DP83820_SNIFFER
	uint32        sel;
	extern uint32 dp83820_snifferOn;

    if(!CM_IS_FC_CPUID0) {   // cpuid 0 is fc slice 3 on slot 2
		printf("This feature only available on SP_NIM slice 3.\n");
		return 0;
	}

	printf("Rx Pkt Sniffer: %s\n", (dp83820_snifferOn == 0) ? "OFF" : "ON");
    sel = dp83820_uiGetInput("Pkt Sniffer Off/On [0|1] : ");
	switch(sel) {
	
		case 0:
			dp83820_snifferOn = 0;
			break;
		case 1:
			dp83820_snifferOn = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Sniffer: %s\n", (dp83820_snifferOn == 0) ? "OFF" : "ON");
    printf("\n");
#else
	printf("Rx Sniffer feature not compiled into this build.\n");
#endif
    return 0;
}  // dp83820_uiRxSniffOnOff

int
dp83820_findChandle(dp83820_cfg_t *dp)
{
#ifdef DP83820_SNIFFER
	uint32 i, hdrLen;
    uint32 tmp,chandle, chandle_find;
    uint32 *msg;
	char *data;
	int display_len;
	dp83820_psb_t *psb;
	char *timestr;
	agile_hdr_t *eeeHdr; 

    if(!CM_IS_FC_CPUID0) {   // cpuid 0 is fc slice 3 on slot 2
		printf("This feature only available on SP_NIM slice 3.\n");
		return 0;
	}

    chandle_find = dp83820_uiGetInput("Enter chandle (hex): ");
	printf("Looking for: %x\n", chandle_find);

	display_len = 64;
	for(i=0; i < DP83820_SNIF_ENTRIES_MAX; i++ ) {

		psb = &dp83820_rx_psb[i];
		data = psb->buf;

		if( !data )
			continue;

		eeeHdr = (agile_hdr_t *)(data + sizeof(enet_hdr_t)); 
		hdrLen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + eeeHdr->pad; 
		msg = (uint32 *)(data+hdrLen);

		tmp = *msg;
		if( tmp == 0x0401 || tmp == 0x0403 ) {
			chandle = *(msg+1);

			if( chandle == chandle_find ) {

				timestr = eee_humandate((uint32)eee_rtcToSec(&psb->t));
				printf("\n=== Time: %s === BufSize: %d === cmdsts: %x === (Entry: %d)\n", timestr, 
					   psb->bufLen, psb->cmdsts, i);

				if( psb->capLen < display_len )
					display_len = psb->capLen;

				if( psb->buf )
					dp83820_printBuf( psb->buf, display_len, 1);

				return 0;
			}
		}
	}
	printf("Did not find pkt with chandle: %x\n", chandle_find);
#else
	printf("Rx Sniffer feature not compiled into this build.\n");
#endif
    return 0;
} // dp83820_findChandle

//-----------------------------------------------------------------------------
// Function Name:   dp83820_uiSetPktCapture
//
// Description:     Enables/Disables packet capturing in dp83820 driver.
//
// Created by:      Frank Salzmann
//
// Data Created:    June 18, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
dp83820_uiSetPktCapture(dp83820_cfg_t *dp)
{
	uint32        sel;

	printf("Rx Pkt Capture: %s, Tx Pkt Capture: %s\n",
					(dp83820_pc.rxEna == 0) ? "DISABLED" : "ENABLED",
					(dp83820_pc.txEna == 0) ? "DISABLED" : "ENABLED");
    sel = dp83820_uiGetInput("Rx Capture Dis/Ena [0|1], Tx Capture Dis/Ena [2|3]: ");
	switch(sel) {
	
		case 0:
			dp83820_pc.rxEna = 0;
			break;
		case 1:
			dp83820_pc.rxEna = 1;
			break;
		case 2:
			dp83820_pc.txEna = 0;
			break;
		case 3:
			dp83820_pc.txEna = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Capture: %s, Tx Pkt Capture: %s\n",
					(dp83820_pc.rxEna == 0) ? "DISABLED" : "ENABLED",
					(dp83820_pc.txEna == 0) ? "DISABLED" : "ENABLED");
    printf("\n");
    return 0;
} // dp83820_uiSetPktCapture
