/*
 *-----------------------------------------------------------------
 *
 * Name:	eee-inlines.h
 *
 * RCS:  $Id: eee-inlines.h,v 1.12 2002/11/22 21:16:46 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	eee inline functions
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/26/2002
 *
 *-----------------------------------------------------------------
 */

#if !defined(__EEE_INLINES__)
#define __EEE_INLINES__

#include <exports/spinlock-api.h>
#include <exports/eee.h>
#include <exports/eee-desc.h>
#include <nfx-defs.h>

extern address_t get_pc(void);

#if defined(NFP_FP)
#define EEE_REMOTE_POOL EEE_TXRX_POOL
#elif defined(NFP_TXRX)
#define EEE_REMOTE_POOL EEE_FP_POOL
#endif


#if defined(__EEE_DESC_COMPILE)
/*-----------------------------------------------------------------
 * Name:	eee_allocateDesc(uint32 mempool, uint32 cos)
 *
 * Description:	Allocate an EEE descriptor.
 *				For optimal performance, the caller should know where
 *				the packet will be destined.  This knowledge will
 *				allow the allocation routine to get the descriptor
 *				from the best pool, according to the module's
 *				architecture.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
eee_descPtr_t
eee_allocateDesc(uint32 mempool, uint32 cos)
#else
static inline eee_descPtr_t
EEE_ALLOCATE_DESC(
	uint32	mempool,	/* mem_pool_id */
	uint32	cos			/* NFX_COS_HIGH, NFX_COS_LOW */
)
#endif	
{
#if defined(FCNIM) && (EEE_NUM_SM_POOLS == 0)
	return test_eee_allocateDesc(mempool, cos);
#else
	address_t		ptr;
	eee_descPtr_t	desc;

#if defined(EEE_VALIDATE)
#if defined(__NO_INLINE__) || defined(__EEE_DESC_COMPILE)
	address_t ra = get_ra();
#else
	address_t ra = get_pc();
#endif
#endif

	NFX_SPIN_LOCK(&eee_descpool_spin);

	if (mempool == EEE_SHARED_POOL || mempool == EEE_MY_POOL_ID) {
	try_shared:
		{
			eee_stack_pool_cntl_t *stack =
				&eee.desc_pool[EEE_MY_SHARED_POOL].stack;
			if (stack->num_avail) {
				--stack->num_avail;
				ptr = *stack->top++;
#if defined(EEE_VALIDATE)
				EEE_VALIDATE_ALLOC_DESC(ptr, EEE_MY_SHARED_POOL, ra);
				if (stack->num_avail <
					eee_stats->low_watermark_desc[EEE_MY_SHARED_POOL])
					eee_stats->low_watermark_desc[EEE_MY_SHARED_POOL] =
						stack->num_avail;
#endif			
				eee_attrAssignDesc((void *)ptr, EEE_ATTR_ALLOCATE);
			}
			else {
				eee_stats->no_desc_avail[EEE_MY_SHARED_POOL]++;
				return NULL;
			}
		}
	}
#if defined(NFP_FP) || defined(NFP_TXRX)		
	else if (mempool == EEE_REMOTE_POOL) {
		eee_queue_pool_cntl_t *queue =
			&eee.desc_pool[EEE_REMOTE_POOL - SHARED_POOL_OFFSET].queue;
		ptr = *queue->head;
		if ((void*)ptr != NULL) {
			(void*)*queue->head = NULL;
			if (((address_t)queue->head & CACHELINE_MASK) == 0)
				*(queue->tail_ptr) = queue->head;
			queue->head++;
			if (queue->head == queue->end)
				queue->head = queue->start;
			eee_attrAssignDesc((void *)ptr, EEE_ATTR_RQ_DEQUEUE);
		}
		else {
			eee_stats->no_desc_avail[EEE_REMOTE_POOL - SHARED_POOL_OFFSET]++;
			goto try_shared;
		}
	}
#endif
	else if (mempool == EEE_LOCAL_POOL) {
		eee_stack_pool_cntl_t *stack = &eee.desc_pool[0].stack;
		if (stack->num_avail) {
			stack->num_avail--;
			ptr = *stack->top++;
#if defined(EEE_VALIDATE)
			EEE_VALIDATE_ALLOC_DESC(ptr, EEE_LOCAL_POOL, ra);
#endif	
			eee_attrAssignDesc((void *)ptr, EEE_ATTR_ALLOCATE);
		}
		else {
			eee_stats->no_desc_avail[0]++;
			goto try_shared;
		}
	}
	else {
		*(volatile uint64 *)0 = 0;
		return NULL;
	}
	
	NFX_SPIN_UNLOCK(&eee_descpool_spin);
	
	desc = (eee_descPtr_t)ptr;

	return desc;
#endif
}	/* eee_allocateDesc() */


#if defined(__EEE_DESC_COMPILE)
/*-----------------------------------------------------------------
 * Name:	eee_allocateBuffer(uint32 buf_size, uint32 cos)
 *
 * Description:	Allocates a system buffer from the free pools
 *				Like the descriptor allocation routine, for optimal
 *				performance, the calling function should know where
 *				the buffer is to be ultimately used.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
void *				/* returns the buffer pointer, or NULL if none
					 * available from the given cos
					 */
eee_allocateBuffer(
	uint32	mempool,		/* mem_pool_id */
	uint16	buf_size,		/* EEE_BUF_SMALL
							 * EEE_BUF_LARGE
							 * EEE_BUF_ULTRA
							 */
	uint32	cos				/* NFX_COS_HIGH, NFX_COS_LOW */
)
#else
static inline void *
EEE_ALLOCATE_BUFFER(uint32 mempool,
					uint16 buf_size, uint32 cos)
#endif
{
#if defined(FCNIM) && (EEE_NUM_SM_POOLS == 0)
    return test_eee_allocateBuffer(mempool, 2048, cos);
#else
	address_t		ptr;

#if defined(EEE_VALIDATE)
#if defined(__NO_INLINE__) || defined(__EEE_DESC_COMPILE) 
	address_t ra = get_ra();
#else
	address_t ra = get_pc();
#endif
#endif

	NFX_SPIN_LOCK(&eee_bufpool_spin);

	if (buf_size >= EEE_NUM_BUF_SIZES) {
		*(volatile uint64 *)0 = 0;
		return NULL;
	}

	if (mempool == EEE_SHARED_POOL || mempool == EEE_MY_POOL_ID) {
	try_shared:
		{
			eee_stack_pool_cntl_t *stack =
				&eee.buf_pool[EEE_MY_SHARED_POOL][buf_size].stack;
			if (stack->num_avail) {
				--stack->num_avail;
				ptr = *stack->top++;
#if defined(EEE_VALIDATE)
				EEE_VALIDATE_ALLOC_BUF(ptr, EEE_MY_SHARED_POOL, buf_size, ra);
				if (stack->num_avail <
					eee_stats->low_watermark_buf[EEE_MY_SHARED_POOL][buf_size])
					eee_stats->low_watermark_buf[EEE_MY_SHARED_POOL][buf_size] =
						stack->num_avail;
#endif			
				eee_attrAssignBuf((void *)ptr, EEE_ATTR_ALLOCATE);
			}
			else {
				NFX_SPIN_UNLOCK(&eee_bufpool_spin);
				ATOMIC_INC_INT(
					&eee_stats->no_buf_avail[EEE_MY_SHARED_POOL][buf_size], 1);
				return NULL;
			}
		}
	}
#if defined(NFP_FP) || defined(NFP_TXRX)		
	else if (mempool == EEE_REMOTE_POOL) {
		eee_queue_pool_cntl_t *queue =
			&eee.buf_pool[EEE_REMOTE_POOL - SHARED_POOL_OFFSET][buf_size].queue;
		ptr = *queue->head;
		if ((void*)ptr != NULL) {
			(void*)*queue->head = NULL;
			if (((address_t)queue->head & CACHELINE_MASK) == 0)
				*(queue->tail_ptr) = queue->head;
			queue->head++;
			if (queue->head == queue->end)
				queue->head = queue->start;
			eee_attrAssignBuf((void *)ptr, EEE_ATTR_RQ_DEQUEUE);
		}
		else {
			ATOMIC_INC_INT(&eee_stats->no_buf_avail
					   [EEE_REMOTE_POOL - SHARED_POOL_OFFSET][buf_size], 1);
			goto try_shared;
		}
	}
#endif
	else if (mempool == EEE_LOCAL_POOL) {
		eee_stack_pool_cntl_t *stack = &eee.buf_pool[0][buf_size].stack;
		if (stack->num_avail) {
			stack->num_avail--;
			ptr = *stack->top++;
#if defined(EEE_VALIDATE)
			EEE_VALIDATE_ALLOC_BUF(ptr, EEE_LOCAL_POOL, buf_size, ra);
#endif	
			eee_attrAssignBuf((void *)ptr, EEE_ATTR_ALLOCATE);
		}
		else {
			eee_stats->no_buf_avail[0][buf_size]++;
			goto try_shared;
		}
	}
	else {
		*(volatile uint64 *)0 = 0;
		return NULL;
	}

	NFX_SPIN_UNLOCK(&eee_bufpool_spin);

	return (void *)ptr;
#endif
}	/* eee_allocateBuffer() */



/*-----------------------------------------------------------------
 * Name :	eee_deallocateSharedDesc()
 *
 * Description: Deallocate descriptor into shared pool
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/27/02
 *
 *-----------------------------------------------------------------
 */
static inline void
eee_deallocateSharedDesc(eee_descPtr_t edesc)
{
#if defined(EEE_VALIDATE)
#if defined(__NO_INLINE__) || defined(__EEE_DESC_COMPILE) 
	address_t ra = get_ra();
#else
	address_t ra = get_pc();
#endif
#endif	
	eee_stack_pool_cntl_t *pool = &eee.desc_pool[EEE_MY_SHARED_POOL].stack;

	edesc->hdr.next = 0;
	edesc->hdr.offset = 0;

	NFX_SPIN_LOCK(&eee_descpool_spin);
#ifdef EEE_ATTR
	if (!(eee_attrOkToFreeDesc(edesc))) {
		eee_attrDisplay((address_t)edesc);
		panic("eee_attr failed in deallocateDescRA, edesc = %p, ra = %p\n", edesc, ra);
	}
	eee_attrSetDescValidFreeAttr(edesc, 0, 0, 0, 0);
#endif

#ifdef EEE_VALIDATE
	if (edesc < eee.desc_base[EEE_MY_SHARED_POOL]
		|| edesc >= eee.desc_end[EEE_MY_SHARED_POOL]) {
		panic("descriptor is not shared descriptor\n");
	}
	EEE_VALIDATE_FREE_DESC(edesc, EEE_MY_SHARED_POOL, ra);
#endif

	*(--pool->top) = (address_t)edesc;
	++pool->num_avail;

	NFX_SPIN_UNLOCK(&eee_descpool_spin);

#ifdef EEE_ATTR
	eee_attrAssignDesc(edesc, EEE_ATTR_FREE);
#endif
}
	


/*-----------------------------------------------------------------
 * Name :	eee_deallocateSharedBuffer()
 *
 * Description: Deallocate buffer into shared pool.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/27/02
 *
 *-----------------------------------------------------------------
 */
static inline void
eee_deallocateSharedBuffer(void *buf, uint16 size)
{
#if defined(EEE_VALIDATE)
#if defined(__NO_INLINE__) || defined(__EEE_DESC_COMPILE) 
	address_t ra = get_ra();
#else
	address_t ra = get_pc();
#endif
#endif	
	eee_stack_pool_cntl_t *pool =
		&eee.buf_pool[EEE_MY_SHARED_POOL][size].stack;

	NFX_SPIN_LOCK(&eee_bufpool_spin);

#ifdef EEE_ATTR
	if (!(eee_attrOkToFreeBuf(buf))) {
		eee_attrDisplay((address_t)buf);
		panic("eee_attr failed in deallocateBufferRA, buf = %p, ra = %p\n", buf, ra);
	}
	eee_attrSetBufValidFreeAttr(buf, 0, 0, 0, 0);
#endif

#ifdef EEE_VALIDATE
	EEE_VALIDATE_FREE_BUF(buf, EEE_MY_SHARED_POOL, size, ra);
#endif

	*(--pool->top) = (address_t)buf;
	++pool->num_avail;

#ifdef EEE_ATTR
	eee_attrAssignBuf(buf, EEE_ATTR_FREE);
#endif
	NFX_SPIN_UNLOCK(&eee_bufpool_spin);
}


#if (defined(NFP_FP) || defined(NFP_TXRX)) && !defined(FS_STANDALONE)
static inline int
eee_isLocalAddr(address_t addr)
{
	return (addr >> 36) == (uint16)(LOCAL_MEM >> 36);
}

static inline int
eee_isRemoteAddr(address_t addr)
{
	return (addr >> 36) == (uint16)(REMOTE_MEM >> 36);
}

static inline int
eee_isRemoteCachedAddr(address_t addr)
{
	return (addr >> 36) == (uint16)(REMOTE_CACHED_MEM >> 36);
}
#else
#define eee_isLocalAddr(addr) 1
#define eee_isRemoteAddr(addr) 0
#define eee_isRemoteCachedAddr(addr) 0
#endif


/*-----------------------------------------------------------------
 * Name :	eee_deallocateRemoteDesc()
 *
 * Description: Deallocate descriptor into forwarding queue
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/27/02
 *
 *-----------------------------------------------------------------
 */
#if defined(NFP_FP) || defined(NFP_TXRX)
static inline void
eee_deallocateRemoteDesc(eee_descPtr_t edesc)
{
#if defined(EEE_ATTR)
#if defined(__NO_INLINE__) || defined(__EEE_DESC_COMPILE) 
	address_t ra = get_ra();
#else
	address_t ra = get_pc();
#endif
#endif	
#ifdef EEE_ATTR
	if (!(eee_attrOkToFreeDesc(edesc))) {
		eee_attrDisplay((address_t)edesc);
		panic("eee_attr failed in deallocateDescRA, edesc = %p, ra = %p\n",
			  edesc, ra);
	}
	
	eee_attrSetDescValidFreeAttr(edesc, 0, 0, 0, 0);
#endif
	ATOMIC_INC_INT(&eee_stats->fwd_free_desc[EEE_REMOTE_QUEUE_INDEX], 1);
#ifdef EEE_ATTR
	eee_attrAssignDesc(edesc, EEE_ATTR_FWD_TO_FREE);
#endif
	NFX_SPIN_LOCK(&eee_fwdqueue_spin);
	FWD_PACKET((address_t)edesc | EEE_FREE_DESC, EEE_REMOTE_QUEUE_INDEX);
	NFX_SPIN_UNLOCK(&eee_fwdqueue_spin);
}


static inline void
eee_deallocateDescMaybeCached(eee_descPtr_t edesc)
{
#if !defined(FS_STANDALONE)
	/* Done with the descriptor */
	if (eee_isRemoteCachedAddr((address_t)edesc)) {
		__asm__("cache %0,0(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,32(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,64(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,96(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		eee_deallocateRemoteDesc((eee_descPtr_t)
			(((address_t)edesc - REMOTE_CACHED_MEM) + REMOTE_MEM));
	}
	else {
		eee_deallocateDesc(edesc);
	}
#endif
}

#else
#define eee_deallocateRemoteDesc(edesc) 
#endif

/*-----------------------------------------------------------------
 * Name:    eee_getNbrOfBufs()
 *
 * Description: 
 *
 * Created by:  Jan Vanhoutte
 *
 * Date Created:    5/3/02
 *
 *-----------------------------------------------------------------
 */
static inline const uint32
eee_getTotalNbrOfBufs(int buf_size)
{
#ifdef FS_STANDALONE
	return 8192;
#else
	return eee.sm_num_buf[buf_size];
#endif
}


/*-----------------------------------------------------------------
 * Name:    eee_getNbrOfBufs()
 *
 * Description: 
 *
 * Created by:  Jan Vanhoutte
 *
 * Date Created:    5/3/02
 *
 *-----------------------------------------------------------------
 */
static inline uint32
eee_getAvailableNbrOfBufs(
	int		buf_size 		/* EEE_BUF_SMALL
							 * EEE_BUF_LARGE
							 * EEE_BUF_ULTRA
							 */
)
{
	return eee.buf_pool[EEE_MY_SHARED_POOL][buf_size].stack.num_avail;
}



#endif
