/*
 *-----------------------------------------------------------------
 *
 * Name:	eee-rq.c
 *
 * RCS:  $Id: eee-rq.c,v 1.10 2002/11/22 21:16:46 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Resource Queue routines
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/16/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/mem-api.h>

#if (EEE_NUM_SM_POOLS > 1)

#define REMOTE_POOL_INDEX	(EEE_REMOTE_POOL-SHARED_POOL_OFFSET)

nfx_spinlock_t eee_rq_spin;

/*-----------------------------------------------------------------
 * Name: eee_rqResupply()
 *
 * Description:	Supply resources to queues in shared memory pools in other
 *				CPUs memory.  In our write-only scheme, check the tail
 *				pointers for the last entry read by the receiving CPU.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/17/01
 *
 *-----------------------------------------------------------------
 */
void
eee_rqResupply(
	void	* cb,
	uint32	tref
)
{
	eee_descPtr_t	desc;
	address_t		buf;
	void			* tp;

	if (!NFX_SPIN_TRYLOCK(&eee_rq_spin))
		return;
	
	/*
	 * Check descriptor resource queues
	 */
	if ((tp = *(eee.far_desc_pool[REMOTE_POOL_INDEX].tail_ptr))) {
		address_t	* hp = eee.far_desc_pool[REMOTE_POOL_INDEX].head;
		uint32		cnt = 0;
		while (hp != tp) {
			desc = EEE_ALLOCATE_DESC(EEE_SHARED_POOL, NFX_COS_HIGH);
			if (desc) {
/*
				printf("allocated descriptor %p\n", desc);
				printf("writing to %p\n", hp);
*/
#ifdef EEE_ATTR
				eee_attrAssignDesc(desc, EEE_ATTR_RQ_ENQUEUE);
#endif
				*hp = (address_t)desc;
				cnt++;
				QUEUE_PTR_INC(	hp,
								eee.far_desc_pool[REMOTE_POOL_INDEX].end,
								eee.far_desc_pool[REMOTE_POOL_INDEX].start);
			}
			else {
/*
				printf("No descriptor available\n");
*/
				break;
			}
		}
		if (cnt) {
			eee_stats->rq_desc_allocated[REMOTE_POOL_INDEX] += cnt;
			eee.far_desc_pool[REMOTE_POOL_INDEX].head = hp;
		}
	}

	/*
	 * Check buffer resource queues
	 */
	if ((tp = *(eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_SMALL].tail_ptr))) {
		address_t	* hp = eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_SMALL].head;
		uint32		cnt = 0;

		while (hp != tp) {
			buf = (address_t)EEE_ALLOCATE_BUFFER(EEE_SHARED_POOL,
												EEE_BUF_SMALL,
												NFX_COS_HIGH);
			if (buf) {
/*
				printf("allocated buf %p to %p\n", buf, hp);
*/
#ifdef EEE_ATTR
				eee_attrAssignBuf((void *)buf, EEE_ATTR_RQ_ENQUEUE);
#endif
				*hp = buf;
				cnt++;
				QUEUE_PTR_INC(	hp,
								eee.far_buf_pool[REMOTE_POOL_INDEX]
													[EEE_BUF_SMALL].end,
								eee.far_buf_pool[REMOTE_POOL_INDEX]
													[EEE_BUF_SMALL].start);
			}
			else {
/*
				printf("No buffer available\n");
*/
				break;
			}
		}
		if (cnt) {
			eee_stats->rq_buf_allocated[REMOTE_POOL_INDEX][EEE_BUF_SMALL] += cnt;
			eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_SMALL].head = hp;
		}
	}
	if ((tp = *(eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_LARGE].tail_ptr))) {
		address_t	* hp = eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_LARGE].head;
		uint32		cnt = 0;

		while (hp != tp) {
			buf = (address_t)eee_allocateBuffer(EEE_SHARED_POOL,
												EEE_BUF_LARGE,
												NFX_COS_HIGH);
			if (buf) {
/*
				printf("allocated buf %p to %p\n", buf, hp);
*/
#ifdef EEE_ATTR
				eee_attrAssignBuf((void *)buf, EEE_ATTR_RQ_ENQUEUE);
#endif
				*hp = buf;
				cnt++;
				QUEUE_PTR_INC(	hp,
								eee.far_buf_pool[REMOTE_POOL_INDEX]
													[EEE_BUF_LARGE].end,
								eee.far_buf_pool[REMOTE_POOL_INDEX]
													[EEE_BUF_LARGE].start);
			}
			else {
/*
				printf("No buffer available\n");
*/
				break;
			}
		}
		if (cnt) {
			eee_stats->rq_buf_allocated[REMOTE_POOL_INDEX][EEE_BUF_LARGE] += cnt;
			eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_LARGE].head = hp;
		}
	}
	if (eee.sm_num_buf[EEE_BUF_ULTRA]) {
		if ((tp =
			*(eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_ULTRA].tail_ptr))) {
			address_t	* hp = eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_ULTRA].head;
			uint32		cnt = 0;

			while (hp != tp) {
				buf = (address_t)eee_allocateBuffer(EEE_SHARED_POOL,
													EEE_BUF_ULTRA,
													NFX_COS_HIGH);
				if (buf) {
/*
					printf("allocated buf %p to %p\n", buf, hp);
*/
#ifdef EEE_ATTR
					eee_attrAssignBuf((void *)buf, EEE_ATTR_RQ_ENQUEUE);
#endif
					*hp = buf;
					cnt++;
					QUEUE_PTR_INC(	hp,
									eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_ULTRA].end,
									eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_ULTRA].start);
				}
				else {
/*
					printf("No buffer available\n");
*/
					break;
				}
			}
			if (cnt) {
				eee_stats->rq_buf_allocated[REMOTE_POOL_INDEX][EEE_BUF_ULTRA] += cnt;
				eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_ULTRA].head = hp;
			}
		}
	}
	if (eee.sm_num_buf[EEE_BUF_MGMT]) {
		if ((tp =
			*(eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_MGMT].tail_ptr))) {
			address_t	* hp = eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_MGMT].head;
			uint32		cnt = 0;

			while (hp != tp) {
				buf = (address_t)eee_allocateBuffer(EEE_SHARED_POOL,
													EEE_BUF_MGMT,
													NFX_COS_HIGH);
				if (buf) {
/*
					printf("allocated buf %p to %p\n", buf, hp);
*/
#ifdef EEE_ATTR
					eee_attrAssignBuf((void *)buf, EEE_ATTR_RQ_ENQUEUE);
#endif
					*hp = buf;
					cnt++;
					QUEUE_PTR_INC(	hp,
									eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_MGMT].end,
									eee.far_buf_pool[REMOTE_POOL_INDEX]
														[EEE_BUF_MGMT].start);
				}
				else {
/*
					printf("No buffer available\n");
*/
					break;
				}
			}
			if (cnt) {
				eee_stats->rq_buf_allocated[REMOTE_POOL_INDEX][EEE_BUF_MGMT] += cnt;
				eee.far_buf_pool[REMOTE_POOL_INDEX][EEE_BUF_MGMT].head = hp;
			}
		}
	}

	NFX_SPIN_UNLOCK(&eee_rq_spin);
}	/* eee_rqResupply() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
eee_rqInit()
{
	NFX_SPIN_INIT(&eee_rq_spin, SPINLOCK_LEVEL_UNKNOWN);
	eee_registerPollingFunc(EEE_HIGH_PRIORITY, eee_rqResupply, NULL);
}	/* eee_rqInit() */

#endif
