/*
 * Name:  elog-shell.c
 *
 * RCS:  $Id: elog-shell.c,v 1.5 2002/08/01 17:56:04 sandrineb Exp $
 *
 * Description:  Command line interface functions for elog_embedded
 * 
 * Created By: Jan Vanhoutte
 *             Pankaj Gupta
 *
 * Date Created:  9/09/2001
 */

/*
gcc -I ../../Includes/ -I ../../Includes/exports/SSC -D LANGUAGE_C -Werror  -lanpssc -L../../Install/SSC/lib -o elog-shell elog-shell.c ../sm-anpssc/eee-stub.c

*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>

#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"


#define SSC_ARG_MAX      10
#define MAX_APP_ID       64

#define USAGE_STR_SIZE   256
/* 
 * see elog-api.h for message request
 * e_log_configure_msg_t
 * 
 * No response is foreseen.
 * No mesg foreseen to dump conf of each CPU
 *
 * The SSC needs to keep track of the config.
 */

// dummy request Hdr
typedef struct
{
    int command;
    int len;
} MsgReqHdr;

// dummy response Hdr
typedef struct
{
    int command;
    int len;
    int status;
} MsgRespHdr;


static MsgReqHdr *elog_cmd(int argc, char *argv[]);

typedef struct 
{
    char *name;
    uint32 flag;
} tab_t;


tab_t elogtab[] = {
    {"enable",           ELOG_CONFIGURE_ENABLE},
    {"severity",         ELOG_CONFIGURE_SEVERITY},
    {"test",             ELOG_CONFIGURE_TEST},

    {"def-sev",          ELOG_CONFIGURE_APP_DEFAULT_SEVERITY},
    {"def-bitmap",       ELOG_CONFIGURE_APP_DEFAULT_BITMAP},
    {"app-sev",          ELOG_CONFIGURE_APP_SEVERITY},
//    {"app-set-bitmap",   ELOG_CONFIGURE_APP_SET_BITMAP},
//    {"app-clear-bitmap", ELOG_CONFIGURE_APP_CLEAR_BITMAP},
    {"trace",            ELOG_CONFIGURE_APP_SET_BITMAP},
    {"untrace",          ELOG_CONFIGURE_APP_CLEAR_BITMAP},

    {"show-date",        ELOG_CONFIGURE_SHOW_DATE},
    {"format",           ELOG_CONFIGURE_FORMAT},
    {"show-line-file",   ELOG_CONFIGURE_SHOW_LINE},
    {"show-seq",         ELOG_CONFIGURE_SHOW_SEQ},
    {"show-name",        ELOG_CONFIGURE_SHOW_APP_NAME},
    {"show-severity",    ELOG_CONFIGURE_SHOW_SEVERITY},
    {"show-local",       ELOG_CONFIGURE_SHOW_LOCAL},

    {0, 0}
};

tab_t severitytab[] = {
    {"emerg",     LOG_EMERG},
    {"alert",     LOG_ALERT},
    {"crit",      LOG_CRIT},
    {"err",       LOG_ERR},
    {"warning",   LOG_WARNING},
    {"notice",    LOG_NOTICE},
    {"info",      LOG_INFO},
    {"debug",     LOG_DEBUG},
    {0, 0}
};

tab_t onofftab[] = {
    {"on",        1},
    {"off",       0},
    {0, 0}
};

tab_t showlocaltab[] = {
    {"off",          ELOG_DISABLED},
    {"stdout",       ELOG_LOCAL_STDOUT},
    {"stderr",       ELOG_LOCAL_STDERR},
    {"local",        ELOG_ONLY_LOCAL_STDOUT},
    {0, 0}
};

tab_t datetab[] = {
    {"on",        ELOG_SHOW_DATE},
    {"off",       ELOG_DISABLED},
    {"msec",      ELOG_SHOW_DATE_MSEC},
    {"usec",      ELOG_SHOW_DATE_USEC},
    {0, 0}
};

tab_t formattab[] = {
    {"one",       ELOG_ONE_LINE},
    {"multi",     ELOG_MULTIPLE_LINE},
    {"min",       ELOG_FORMAT_MIN_INFO},
    {"max",       ELOG_FORMAT_MAX_INFO},
    {0, 0}
};

tab_t tracetab[] = {
    {"all",   ~0},
    {"0",     1<<0},
    {"1",     1<<1},
    {"2",     1<<2},
    {"3",     1<<3},
    {"4",     1<<4},
    {"5",     1<<5},
    {"6",     1<<6},
    {"7",     1<<7},
    {"8",     1<<8},
    {0, 0}
};

/* 
 * Some very application specific stuff
 * Needs cleanup, and some redesign (*.h files)
 */

tab_t apptab[MAX_APP_ID];

typedef char app_name_t [64];

app_name_t appname[MAX_APP_ID];

// to store known applications
int fs_applicationId;
int nvram_applicationId;

typedef enum fs_traceLvlNum
{
    FS_TRACE_REQ,
    FS_TRACE_BUF,
    FS_TRACE_INO,
    FS_TRACE_LCK,
    FS_TRACE_LOG,
    FS_TRACE_IO,
    FS_TRACE_SPACE,
    FS_TRACE_SNAP,
    FS_TRACE_COD,
    FS_TRACE_VERBOSE,
    FS_TRACE_VOLLOG,
    FS_TRACE_READDIR,
    FS_TRACE_DUMP_RESTORE,
    FS_TRACE_LVL_MAX
} fs_TraceLvlNum;

tab_t fs_tracetab[] = {
    {"all",   ~0},
    {"req",   1<<FS_TRACE_REQ},
    {"buf",   1<<FS_TRACE_BUF},
    {"ino",   1<<FS_TRACE_INO},
    {"lck",   1<<FS_TRACE_LCK},
    {"log",   1<<FS_TRACE_LOG},
    {"ior",   1<<FS_TRACE_IO},
    {"space", 1<<FS_TRACE_SPACE},
    {"snap",  1<<FS_TRACE_SNAP},
    {"cod",   1<<FS_TRACE_COD},
    {"verbose",   1<<FS_TRACE_VERBOSE},
    {"vollog",    1<<FS_TRACE_VOLLOG},
    {"dir",    1<<FS_TRACE_READDIR},
    {"dumprestore",    1<<FS_TRACE_DUMP_RESTORE},
    {0, 0}
};


typedef enum nv_traceLvlNum
{
    NV_TRACE_MNGT,
    NV_TRACE_WRITE,
    NV_TRACE_READ,
    NV_TRACE_LVL_MAX
} nv_TraceLvlNum;

tab_t nv_tracetab[] = {
    {"all",    ~0},
    {"mngt",   1<<NV_TRACE_MNGT},
    {"write",  1<<NV_TRACE_WRITE},
    {"read",   1<<NV_TRACE_READ},
    {0, 0}
};




#define eee_ramAlloc(_size_)  (void*)malloc(_size_)
#define eee_ramDealloc(_ptr_) free(_ptr_)

char *eee_getNameFromAppId (int32 appId);

// #define abort()  exit(2)

#define RESP_MAX 1024
// static char RespBuf[RESP_MAX];

/*
 * caller must provide the 
 * - tab: table with name and flags (terminated with {0,0}.
 * - name: the name to look for
 * - list: an char array, big enough to contain all data, as help string
 * return 
 * - NFX_OK  : then the flag is valid
 * - NFX_ERR : then the list is valid
 */
  
static int
shell_parse_parameter(tab_t *tab, char *name, char *list, int *flag)
{
    int loop;
    
    for (loop = 0; tab->name != NULL; loop++, tab++) 
    {
        if (strcmp(name, tab->name) == 0) 
        {
            *flag = tab->flag;
            return (NFX_OK);
        }
        list += sprintf(list,"%s|", tab->name);
    }
    list--;
    *list = 0;
    
    return(NFX_ERR);
}

static void *
alloc_msg(int type, uint32 size)
{
    MsgReqHdr *hdr;
    void *msg;

    msg = (MsgReqHdr*)eee_ramAlloc(size);
    bzero(msg, size);

    hdr = msg;
    hdr->command = type;
    hdr->len = size;

    return msg;
}

char tablist[USAGE_STR_SIZE];
char usage_cmd[USAGE_STR_SIZE];
char usage_name[] = "elog-shell <slot> <cpu>" ;
char null_name[] = "" ;

static MsgReqHdr *
elog_cmd(int argc, char *argv[])
{
    e_log_configure_msg_t *msg;
    int command = 0 ;
    int value_1 = 0 ;
    int value_2 = 0 ;
    int rc = NFX_ERR ;
    char *cmd_name = NULL ;
    char *val_1_name = NULL ;
    char *val_2_name = NULL ;
    int tmp, val_argc = 0 ;
    
    if (argc >= 2)
        cmd_name = argv[1] ;
    else
        cmd_name = null_name ;

    if (argc >= 3)
        val_1_name = argv[2] ;
    else
        val_1_name = null_name ;
    
    if (argc >= 4)
    {
        val_2_name = argv[3] ;
        val_argc = 3 ;
    }
    else
        val_2_name = null_name ;
    
    if ((rc = shell_parse_parameter (elogtab, cmd_name, tablist, &command)) != NFX_OK)
    {
        printf("Usage: %s <%s>\n", usage_name, tablist);
        return NULL;
    }

    switch(command)
    {
    case ELOG_CONFIGURE_SEVERITY:
    case ELOG_CONFIGURE_TEST:
    case ELOG_CONFIGURE_APP_DEFAULT_SEVERITY:
        rc = shell_parse_parameter (severitytab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_ENABLE:
    case ELOG_CONFIGURE_SHOW_LINE:
    case ELOG_CONFIGURE_SHOW_SEQ:
    case ELOG_CONFIGURE_SHOW_SEVERITY:
    case ELOG_CONFIGURE_SHOW_APP_NAME:
        rc = shell_parse_parameter (onofftab, val_1_name, tablist, &value_1);
        break;
        
    case ELOG_CONFIGURE_SHOW_LOCAL:
        rc = shell_parse_parameter (showlocaltab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_APP_DEFAULT_BITMAP:
        if (sscanf (val_1_name, "0x%x", &value_1) == 1)
        {
            rc = NFX_OK;
        }
        else
        {
            sprintf(tablist, "0xVAL");
            rc = NFX_ERR;
        }
        break;

    case ELOG_CONFIGURE_APP_SEVERITY:
        if ((rc = shell_parse_parameter (apptab, val_1_name, tablist, &value_1)) == NFX_OK)
        {
            rc = shell_parse_parameter (severitytab, val_2_name, tablist, &value_2);
        }
        if (rc != NFX_OK)
        {
            sprintf(usage_cmd, "app-sev <appId> <severity>, choice:  ");
            cmd_name = usage_cmd;
        }
        break;
        
    case ELOG_CONFIGURE_APP_SET_BITMAP:
    case ELOG_CONFIGURE_APP_CLEAR_BITMAP:
        if ((rc = shell_parse_parameter (apptab, val_1_name, tablist, &value_1)) == NFX_OK)
        {
            if (value_1 == fs_applicationId)
            {
                while ((rc == NFX_OK) && (val_argc < argc))
                {
                    rc = shell_parse_parameter (fs_tracetab, argv[val_argc], tablist, &tmp);
                    value_2 |= tmp;
                    val_argc++;
                }
            }
            else
            if (value_1 == nvram_applicationId)
            {
                while ((rc == NFX_OK) && (val_argc < argc))
                {
                    rc = shell_parse_parameter (nv_tracetab, argv[val_argc], tablist, &tmp);
                    value_2 |= tmp;
                    val_argc++;
                }
            }
            else
            {
                while ((rc == NFX_OK) && (val_argc < argc))
                {
                    rc = shell_parse_parameter (tracetab, argv[val_argc], tablist, &tmp);
                    value_2 |= tmp;
                    val_argc++;
                }
            }
        }
        if (rc != NFX_OK)
        {
            sprintf(usage_cmd, "[un]trace <app> <type>, choice:  ");
            cmd_name = usage_cmd;
        }
        break;

    case ELOG_CONFIGURE_SHOW_DATE:
        rc = shell_parse_parameter (datetab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_FORMAT:
        rc = shell_parse_parameter (formattab, val_1_name, tablist, &value_1);
        break;
    }
    
    if (rc == NFX_OK)
    {
        msg = alloc_msg(command, sizeof *msg);
        // msg->command = command;
        // msg->len = sizeof(e_log_configure_msg_t);
        msg->value_1 = value_1 ;
        msg->value_2 = value_2 ;        
        return (MsgReqHdr *)msg;
    }
    else 
    {
        printf("Usage: %s %s <%s>\n", usage_name, cmd_name, tablist);
        return NULL;
    }
}

static void
get_applications (void)
{
    int app_tab = 0;
    int loop;
    char *app_name;
    
    fs_applicationId = eee_getApplicationId("efs");
    nvram_applicationId = eee_getApplicationId("nvram");

    for (loop = 1; loop < MAX_APP_ID; loop++)
    {
        if ((app_name = eee_getAppNameFromId (loop)) != NULL)
        {
            if (strlen(app_name) > 0)
            {
                strcpy(appname[app_tab], app_name);
                apptab[app_tab].name = appname[app_tab];
                apptab[app_tab].flag = loop ;
                app_tab++;
            }
        }
    }
    apptab[app_tab].name = 0 ;
    apptab[app_tab].flag = 0 ;
}


int
main(int argc, char **argv)
{
    MsgReqHdr        *reqHdr;
    MsgRespHdr       *respHdr;
    int                 rc, cpu, slot;
#define RESP_SIZE_MAX   9000
    void                *respBuf;
    struct timeval      timeout = {0, 0};

    // This is where the SSC command line interface command for
    // the fs begins.   There's only a single binary, fscmd, which
    // takes arguments to direct its actions.  It is intended that
    // shell scripts (eg. mkfs) will provide the appropriate args
    // to fscmd.

    if (argc < 3)
    {
        printf ("Usage: %s <slot> <cpu> <commmand>\n", argv[0]);
        exit (0);
    }
    
    get_applications();

    slot = atoi(argv[1]);
    cpu = atoi(argv[2]);

    argc -= 2 ;
    argv++ ;
    argv++ ;    

    // reqHdr = cmd_parse_opts(argc, argv);
    reqHdr = elog_cmd(argc, argv);
    if (reqHdr == NULL)
        exit(2);

    respBuf = (char *)malloc(RESP_SIZE_MAX);
    if (respBuf == NULL)
    {
        fprintf(stderr, "failed to allocate response buffer\n");
        exit(2);
    }

    rc = sendAgileMsg(  -1,                 // system assigned tmp socket
                        reqHdr,             // the msg
                        reqHdr->len,        // the length
                        "e_log",            // application
                        cpu,                  // Todo: cpuId
                        slot,                  // Todo: slot
                        MGMTPLANE,          // Mgmt plane
                        &timeout,           // == 0: no response expected
                        // NULL,               // no resp message for e-log
                        respBuf,
                        RESP_SIZE_MAX);
    if (rc < 0)
    {
        perror("sendAgileMsg");
        fprintf(stderr, "command failed to complete (rc: %d)\n", rc);
        exit(2);
    }

    printf ("Send to cpu:%d slot:%d App: e_log rc:%d\n", cpu, slot, rc);
    printf ("Send cmd %d, len %d, val1 %d, val2 %d\n",
            reqHdr->command, reqHdr->len, 
            ((e_log_configure_msg_t*)reqHdr)->value_1,
            ((e_log_configure_msg_t*)reqHdr)->value_2);


    respHdr = respBuf;
//    if (respHdr->fsStatus != NFX_OK)
//    {
//        fprintf(stderr, "command completed with error status (%d)\n",
//                respHdr->fsStatus);
//        exit(2);
//    }

    exit(0);

} /* main */


