/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	sbd.c
 *
 * RCS:  $Id: sbd.c,v 1.6 2002/04/22 18:13:03 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/19/01
 *
 *-----------------------------------------------------------------
 */

#ifdef EVAL_BOARD

#include <nfx-target.h>
#include <sys/mips.h>
#include "../prom-pmon/pmon.h"
#include <pmon/termio.h>
#include "sbd.h"
#include <mips/prid.h>

extern int ns16550();

static volatile p5064softc * const softc = PA_TO_KVA1 (SOFTC_BASE);

ConfigEntry     ConfigTable[] =
{
/*
	{(Addr)PHYS_TO_K1(UART_BASE1), 0, ns16550, 256, B57600},
	{(Addr)PHYS_TO_K1(UART_BASE0), 0, ns16550, 256, B57600},
*/
	{0}
};


void
sbdpoll()
{
}

/* display a message (four chars packed into one word),
   for very low-level code. */
void
sbddisplay (unsigned long msg, unsigned int x)
{
}


/* display a long message on the display, scrolling if necessary. */
void
sbdmessage (int line, const char *msg)
{
}

/* display a message and then pause so that it can be read */
static void
display (const char *msg)
{
}

/* display a register name and its value, then pause */
static void
xdisplay (char *s, unsigned int x)
{
}

char *
sbddbgintr (unsigned int cr)
{
	return 0;
}

char *
sbdexception (epc, cause, ra, extra, exc)
	unsigned long   epc, cause, ra, extra;
	char *exc;
{
}

_fpstatesz ()
{
	return 0;
}

_fpstate ()
{
	return -1;
}

cop1 ()
{
	printf ("\ncop1 called\n");
	cliexit ();
}

getmachtype ()
{
	extern int mips_scache_size;
	extern int mips_icache_size;

	switch ((Prid >> 8) & 0xff) {
		case PRID_R4000:
			return 4000;
		case PRID_R4600:
			return 4600;
		case PRID_R4700:
			return 4700;
		case PRID_R5000:
			return 5000;
		case PRID_RM52XX:
			/* blech */
			if (mips_scache_size > 0)
				return 5270;
			if (mips_icache_size == 32*1024)
				return (5261);
			return 5260;
		case PRID_RM7000:
			return 7000;
		case PRID_R5400:
			return 5464;
		case PRID_JADE:
			return 4032;        /* XXXX think of a number */
		case PRID_RC6457X:
			return 64574;
		case PRID_RC6447X:
			return 64474;
		default:
			return 0;
	}
}

#ifndef INET
/* gsignal() is called when a keyboard interrupt (^C) is detected */
gsignal (jmp_buf *jb, int sig)
{
	if (jb) longjmp (jb, 1);
}

netopen()	{return -1;}
netread()	{return -1;}
netwrite()	{return -1;}
netclose()	{return -1;}
netlseek()	{return -1;}
netioctl()	{return -1;}
netpoll()	{}


unsigned long __time (void)
{
#ifdef RTC
	return (sbd_gettime ());
#else
	return 0;
#endif
}
#endif

static unsigned long pipefreq = 0;
int cpuspeed = 100;

/* hand tune delay */
static void
probefreq(void)
{
#ifdef RTC
	unsigned long start, cnt, ohead, timeout;
	int i;

	SBD_DISPLAY ("FREQ", CHKPNT_FREQ);

	pipefreq = 133333333;   /* default value */

	/* select interrupt register */
	outb (RTC_ADDR_PORT, RTC_INTR);

	/* run more than once to be sure that we're in the cache */
	for (i = 2; i != 0; i--) {
	start = get_count ();
	/* clear any pending rtc i/u */
	(void) inb (RTC_DATA_PORT);
	/* wait until we see an RTC i/u */
	timeout = 10000000 / RTC_HZ;
	while ((icu->irr.dev & ICU_DEV_RTC) == 0)
		if (--timeout == 0) goto done;
	cnt = get_count ();
	}
	cnt -= start;

	/* discover icu read overhead */

	for (i = 2; i != 0; i--) {
	start = get_count ();
	timeout = 10000000 / RTC_HZ;
	while ((icu->irr.dev & ICU_DEV_RTC) == 0)
		if (--timeout == 0) goto done;
	ohead = get_count ();
	}
	ohead -= start;
	ohead /= 2;         /* average */

	/* work out number of cpu ticks per sec, the cpu pipeline */
	pipefreq = cnt * RTC_HZ;
	ohead *= RTC_HZ;        /* XXX is this right? */
	/*printf ("(%d-%d->", pipefreq, ohead);*/
	pipefreq -= ohead;

	/* All R5000 processors clock the count register at 1/2 of PClock */
	pipefreq *= 2;

	/* round frequency to 3 decimal places */
	pipefreq += 500;
	pipefreq -= pipefreq % 1000;
	/*printf ("%d)", pipefreq);*/
#else
	pipefreq = 133333333;
#endif

	/* how many times round a 2 instruction loop gives 1us delay */
done:
	if (IS_KVA0 (&probefreq))
	cpuspeed = (pipefreq/2) / 1000000;
	else
	cpuspeed = 1;
}

unsigned long sbdpipefreq (void)
{
	if (pipefreq == 0)
		probefreq ();
	return pipefreq;
}

/* SysClk to PipeClk ratio table */
struct ratio {char mult; char div;};
static const struct ratio ratios_rm7k[8] = {{2,1}, {3,1}, {4,1}, {5,2},
											{6,1}, {7,2}, {8,1}, {9,2}};

unsigned long sbdcpufreq (void)
{
	unsigned int ec = (Config & CFG_ECMASK) >> CFG_ECSHIFT;
	const struct ratio *ratio;

	switch (getmachtype ()) {
		case 7000:
		case 7010:
			ratio = &ratios_rm7k[ec];
			break;
		default:
			printf("sbdcpufreq, invalid machtype, %d\n", getmachtype());
			break;
	}

	/* For PipeClk to SysClk reverse the ratio calculation */
	return sbdpipefreq () * ratio->div / ratio->mult;
}

void
sbdmachinfo (void)
{
}

unsigned int
sbdenable (int machtype)
{
	unsigned int sr;

#ifdef FLASH
	{
		/* now is the chance to report any early eerom failures */
		extern const char *_sbd_envinit (void);
		const char * err = _sbd_envinit ();
		if (err)
			printf ("WARNING: flash environment: %s\n", err);
	}
#endif

#if RAL_XXX
	/* clear interrupt latches */
	icu->ctrl.clear = ~0;

	/* all external interrupts masked initially */
	imask_set (0);
#endif

	/* enable panic errors */
	sr = SR_IBIT7 | SR_IE;

	(void) bis_sr (sr);
	return sr;
}


#define WDELAY()							\
{											\
	register int i;							\
	wbflush();								\
	for (i = DELAY; i != 0; i--) continue;	\
}

void
sbddevinit ()
{
/*
	extern int	_pmon_in_ram;
	if (_pmon_in_ram) {
*/
		pci_init();

		eee_init(1);

        mbinit();
        pkt_init();
        luc_init();
        tpl_init();

		eee_init(2);

		q_initQueue();
		eee_timerInit();
		timer_ticInit();

		eee_initFragmentTimeout();
		
		gt64240mii_initModule();

        tftp_init();
        domaininit();
        luc_postInit();

/*
	}
*/
}


const char *
sbdgetname ()
{
	return "GT64240";
}


/* early low-level machine initialisation */
void
sbdmachinit ()
{
#define MEG (1024*1024)

	serio_init();

	/* size memory (look for sbdreset's computed value) */
	memorysize = softc->memsz * 4 * MEG;

	if (memorysize < 4*MEG || memorysize > 256*MEG)
	memorysize = 4 * MEG;   /* minimum possible */

#ifdef RTC
	/* set RTC periodic interrupt frequency */
	rtc_bic (RTC_STATUSA, RTC_RATE_MASK);
	rtc_bis (RTC_STATUSA, RTC_RATE);
	/* enable periodic interrupts */
	rtc_bis (RTC_STATUSB, RTCSB_PIE);
#endif

}

#ifndef RTC
long
sbd_gettime (void)
{
	return 0;
}
#endif


#ifndef FLASH
/*
 * dummy nvram handler for systems without nvram.
 */

char *
sbd_getenv (name)
	char *name;
{
	return 0;
}


int
sbd_setenv (char *name, char *value)
{
	return 1;
}


int
sbd_unsetenv (char *name)
{
	return 1;
}


void
sbd_mapenv (int (*func)(char *, char *))
{
}

void
sbd_flashinfo (void **buf, int *size)
{
	buf = 0;
	size = 0;
}


int
sbd_flashprogram (void *buf, int size)
{
	return 0;
}
#endif


#endif	/* EVAL_BOARD */
