/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-hash.c,v 1.18 2002/12/10 23:30:39 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-hash.c
 *
 * Description  : Volume Manager (EVM) - hash management
 *
 * Created by   : Lan Chin
 *                Isabelle Cnudde
 *
 * Date Created : 28-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

#include "nfx-incl.h"
#include "exports/hash-api.h"
#include "evm-hash.h"
#include "evm-lv.h"
#include "evm-debug.h"

/*------------------------------------------------------------------------------
 * Volume hash table.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : evm_vol_hash
 *
 * Description  : hash function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 13-AUG-2002
 *
 *------------------------------------------------------------------------------
 */

uint32
evm_vol_hash(const void * key)
    
{
    evm_volId_t * id = (evm_volId_t *) key;

    //printf("evm_vol_hash(0x%llx) - key = %d\n", id->whole_thing,
    //id->details.u.parts.clusterUnique % EVM_VOL_BUCKETS);
    
    return(id->details.u.parts.clusterUnique & EVM_VOL_BUCKETS);
}

/*------------------------------------------------------------------------------
 * Name         : evm_vol_compare
 *
 * Description  : compare function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 13-AUG-2002
 *
 *------------------------------------------------------------------------------
 */

int32
evm_vol_compare(const void * obj1,
                const void * obj2)
    
{
    evm_vol_t * vol1 = (evm_vol_t *) obj1;
    evm_vol_t * vol2 = (evm_vol_t *) obj2;
    
    //printf("evm_vol_compare(0x%llx/0x%llx)\n",
    //vol1->id.whole_thing, vol2->id.whole_thing);
    
    if (vol1->id.whole_thing == vol2->id.whole_thing)
        return(0);
    else
        return(1);
}

/*------------------------------------------------------------------------------
 * Name         : evm_vol_release
 *
 * Description  : release function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 13-AUG-2002
 *
 *------------------------------------------------------------------------------
 */

void
evm_vol_release(void * key,
                void * obj)
    
{
    // do nothing
    return;
}

/*------------------------------------------------------------------------------
 * LUN hash table
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : evm_lun_hash
 *
 * Description  : hash function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-SEP-2002
 *
 *------------------------------------------------------------------------------
 */

uint32
evm_lun_hash(const void * key)
    
{
    uint32 * idx = (uint32 *) key;
    return(*idx & EVM_LUN_BUCKETS);
}

/*------------------------------------------------------------------------------
 * Name         : evm_lun_compare
 *
 * Description  : compare function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-SEP-2002
 *
 *------------------------------------------------------------------------------
 */

int32
evm_lun_compare(const void * obj1,
                const void * obj2)
    
{
    evm_lun_t * lun1 = (evm_lun_t *) obj1;
    evm_lun_t * lun2 = (evm_lun_t *) obj2;
    
    if (lun1->idx == lun2->idx)
        return(0);
    else
        return(1);
}

/*------------------------------------------------------------------------------
 * Name         : evm_lun_release
 *
 * Description  : release function for the general hash routine
 *                (see hash-api.h)
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-SEP-2002
 *
 *------------------------------------------------------------------------------
 */

void
evm_lun_release(void * key,
                void * obj)
    
{
    // do nothing
    return;
}

/*----------------------------------------------------------------------------*/
