/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-io.h
 *
 * Description  : Volume Manager (EVM) - I/O management
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 18-DEC-2001
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_IO_H_
#define _EVM_IO_H_

#include "nfx-incl.h"
#include "exports/evm-api.h"
#include "evm-srvr-includes.h"
#include "evm-srvr-defines.h"
#include "evm-srvr-structs.h"

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Type definition.
 *------------------------------------------------------------------------------
 */

/* Logical I/O control block.
 *
 * Plain type   : 1 logical I/O request <=> 1 physical I/O request
 * RAID 1       : 1 logical I/O request <=> n physical I/O requests
 */
typedef struct
{
    uint32              id;             // number of the I/O request
                                        // (for debugging purpose only)
    evm_IOReq_t         req;            // user request
    void (*userCb)(evm_IOStat_t *stat); // user callback function
    boolean             accessPrivate;  // TRUE if data to access are in the
                                        // private areas (LUN, VOLUME, TAKEOVER)
    evm_pReq_t          type;           // request type
                                        // combination of request and data types
    evm_vol_t *         lv;             // logical volume descriptor
    evm_IOStat_t        iostat;         // status returned to caller
} evm_lioCb_t;

/* Physical I/O control block.
 */
typedef struct
{
    evm_qhandle_t       handle;         // pass to the SCSI layers
    evm_lun_t *         dev;            // physical device
    uint32              offset;         // offset - where to access
    uint32              size;           // number of blocks to access
    evm_handle_t        dataHandle;     // data to write or read
    int32               errCode;        // saved code for ERROR event
	uint32              nbRetry;        // number of retry
    uint32              checksum;       // checksum on data  if meta write req
} evm_pioCb_t;

/*------------------------------------------------------------------------------
 * Function prototypes.
 *------------------------------------------------------------------------------
 */

int32
evm_io_init(void);

int32
evm_io_terminate(void);

int32
evm_io_req(evm_IOReq_t *    req,
           void (*cmplt)    (evm_IOStat_t * stat),
           boolean          accessPrivate);

int32
evm_io_clockTic(uint32  tmr_handle,
                void *  usr_param1,
                void *  usr_param2,
                void *  usr_param3,
                void *  usr_param4);

void
evm_io_scsiReadRsp(char *   msgp,
                   uint32   msgLen,
                   uint32   srcPort);

void
evm_io_scsiWriteRsp(scsi_writeRspMsg_t *    wp,
                   uint32                   srcPort);

void
evm_io_scsiErrRsp(scsi_errRspMsg_t *    rsp);

void
evm_io_markOutstandingIO(evm_volId_t * volId);

/*----------------------------------------------------------------------------*/

#endif // _EVM_IO_H_
