/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-srvr-defines.h,v 1.39 2002/12/10 23:30:39 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-srvr-defines.h
 *
 * Description  : Volume Manager (EVM) - defines for the EVM server
 *
 * Created by   : Lan Chin
 *                Isabelle Cnudde
 *
 * Date Created : 28-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_SRVR_DEFINES_H_
#define _EVM_SRVR_DEFINES_H_

//#define SY_IO_PERFORMANCE_TEST

/* EVM I/O queues
 */
#define EVM_MAX_META_PIO_PEND   (2*1024)
#define EVM_MAX_META_PIO_WAIT   (8*1024)
#define EVM_MAX_USER_PIO_PEND   DP_PROXY_NUM_BUFS
#define EVM_MAX_USER_PIO_WAIT   (1024)

/* EVM SRVR hash buckets
 */
#define EVM_VOL_BUCKETS     1023
#define EVM_LUN_BUCKETS       63

/* EVM IO timer
 */
#define EVM_IO_TICK         1000    // 1 second
#define EVM_IO_TIMEOUT      30000   // 30 seconds for I/O timeouts, in ms
#define EVM_MAX_IO_RETRY    1       // 1 retry
#define EVM_MAX_IO          128     // max of 128 IO to be sent in a loop

/* EVM LV timer
 */
#define EVM_LV_TICK         (NFX_VOL_TSTAMP_PERIOD * 1000)

/* EVM SRVR trace levels
 */
#define EVM_TRACE_ERRORS    0x0001  // error trace                  -> LOG_ERR
#define EVM_TRACE_INFO      0x0002  // important info               -> LOG_INFO
#define EVM_TRACE_IO        0x0004  // IO trace                     -> LOG_DBG
#define EVM_TRACE_FUNC      0x0008  // function call trace          -> LOG_DBG
#define EVM_TRACE_SRVR_MSG  0x0010  // IPC control message trace    -> LOG_DBG
#define EVM_TRACE_EEE       0x0020  // EEE (eg packet) trace        -> LOG_DBG
#define EVM_TRACE_DETAIL    0x0040  // detail trace                 -> LOG_DBG
#define EVM_TRACE_MEMORY    0x0080  // memory trace                 -> LOG_DBG
#define EVM_TRACE_HISTORY   0x0100  // EVM history
#define EVM_TRACE_ALL       0xff
//#define EVM_TRACE_DEBUG     (EVM_TRACE_ERRORS|EVM_TRACE_INFO|EVM_TRACE_HISTORY)
#define EVM_TRACE_DEBUG     (EVM_TRACE_ERRORS|EVM_TRACE_INFO)
#define EVM_TRACE_DEFAULT   (EVM_TRACE_ERRORS|EVM_TRACE_INFO)

#define EVM_TRACE_MASK      0xfc    // mask for all debug traces

/*------------------------------------------------------------------------------
 * EVM SRVR statistics
 *
 * use the following macros to increment the global statistic
 * (see evm-srvr-structs.h for evm_stats_t)
 *
 * to turn on global evm srvr statistic gathering: define EVM_DO_STATS
 *------------------------------------------------------------------------------
 */

#define EVM_DO_STATS

#ifdef  EVM_DO_STATS

#define EVM_STAT_IOREQ          ATOMIC_INC_DWORD(&(evm_stats.nIOReqs), 1);
#define EVM_STAT_IOCOMPLETE     ATOMIC_INC_DWORD(&(evm_stats.nIOCompletes), 1);
#define EVM_STAT_IO_REJECTED    ATOMIC_INC_DWORD(&(evm_stats.nIORejected), 1);
#define EVM_STAT_MREADS         ATOMIC_INC_DWORD(&(evm_stats.nMReads), 1);
#define EVM_STAT_MWRITES        ATOMIC_INC_DWORD(&(evm_stats.nMWrites), 1);
#define EVM_STAT_UREADS         ATOMIC_INC_DWORD(&(evm_stats.nUReads), 1);
#define EVM_STAT_UWRITES        ATOMIC_INC_DWORD(&(evm_stats.nUWrites), 1);
#define EVM_STAT_MREADS_RETRY   ATOMIC_INC_DWORD(&(evm_stats.nMReadsRetry), 1);
#define EVM_STAT_MWRITES_RETRY  ATOMIC_INC_DWORD(&(evm_stats.nMWritesRetry), 1);
#define EVM_STAT_UREADS_RETRY   ATOMIC_INC_DWORD(&(evm_stats.nUReadsRetry), 1);
#define EVM_STAT_UWRITES_RETRY  ATOMIC_INC_DWORD(&(evm_stats.nUWritesRetry), 1);
#define EVM_STAT_IOEVENTS       ATOMIC_INC_DWORD(&(evm_stats.nIOEvents), 1);
#define EVM_STAT_CODRCVS        ATOMIC_INC_DWORD(&(evm_stats.nCodRcvs), 1);
#define EVM_STAT_SCSIMREADRSPS  ATOMIC_INC_DWORD(&(evm_stats.nScsiMReadRsps), 1);
#define EVM_STAT_SCSIMWRITERSPS ATOMIC_INC_DWORD(&(evm_stats.nScsiMWriteRsps), 1);
#define EVM_STAT_SCSIUREADRSPS  ATOMIC_INC_DWORD(&(evm_stats.nScsiUReadRsps), 1);
#define EVM_STAT_SCSIUWRITERSPS ATOMIC_INC_DWORD(&(evm_stats.nScsiUWriteRsps), 1);
#define EVM_STAT_SCSIDEVERRS    ATOMIC_INC_DWORD(&(evm_stats.nScsiDevErrs), 1);
#define EVM_STAT_ALLOCS         ATOMIC_INC_DWORD(&(evm_stats.nAllocs), 1);
#define EVM_STAT_DEALLOCS       ATOMIC_INC_DWORD(&(evm_stats.nDeallocs), 1);
#define EVM_STAT_TIMERCREATES   ATOMIC_INC_DWORD(&(evm_stats.nTimerCreates), 1);
#define EVM_STAT_TIMERDESTROYS  ATOMIC_INC_DWORD(&(evm_stats.nTimerDestroys), 1);
#define EVM_STAT_ERRORS         ATOMIC_INC_DWORD(&(evm_stats.nErrors), 1);
#define EVM_STAT_TIMEOUTS       ATOMIC_INC_DWORD(&(evm_stats.nTimeouts), 1);
#define EVM_STAT_STALES         ATOMIC_INC_DWORD(&(evm_stats.nStales), 1);
#define EVM_STAT_EEEDROPPACKET  ATOMIC_INC_DWORD(&(evm_stats.nEEEDropPacket), 1);
#define EVM_STAT_EEETXQUEUEFULL ATOMIC_INC_DWORD(&(evm_stats.nEEETxQueueFull), 1);

#define EVM_STAT_Q_INSERT_FREE                                  \
        if (qPtr->dataType == EVM_META_DATA_REQ)                \
            ATOMIC_INC_INT(&(evm_stats.qCnt.metaFree), 1);      \
        else                                                    \
            ATOMIC_INC_INT(&(evm_stats.qCnt.userFree), 1);

#define EVM_STAT_Q_REMOVE_FREE                                  \
        if (qPtr->dataType == EVM_META_DATA_REQ)                \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.metaFree), 1);      \
        else                                                    \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.userFree), 1);

#define EVM_STAT_Q_INSERT_PIOWAIT                                       \
        if (qPtr->dataType == EVM_META_DATA_REQ)                        \
        {                                                               \
            ATOMIC_INC_INT(&(evm_stats.qCnt.metaPioWait), 1);           \
            evm_stats.nMetaMaxPioWait = MAX(evm_stats.qCnt.metaPioWait, \
                                            evm_stats.nMetaMaxPioWait); \
        }                                                               \
        else                                                            \
        {                                                               \
            ATOMIC_INC_INT(&(evm_stats.qCnt.userPioWait), 1);           \
            evm_stats.nUserMaxPioWait = MAX(evm_stats.qCnt.userPioWait, \
                                            evm_stats.nUserMaxPioWait); \
        }                                                               \
        evm_stats.nMaxPioWait = MAX(evm_stats.qCnt.metaPioWait +        \
                                    evm_stats.qCnt.userPioWait,         \
                                    evm_stats.nMaxPioWait);

#define EVM_STAT_Q_REMOVE_PIOWAIT                               \
        if (qPtr->dataType == EVM_META_DATA_REQ)                \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.metaPioWait), 1);   \
        else                                                    \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.userPioWait), 1);

#define EVM_STAT_Q_INSERT_PIOPEND                                       \
        if (qPtr->dataType == EVM_META_DATA_REQ)                        \
        {                                                               \
            ATOMIC_INC_INT(&(evm_stats.qCnt.metaPioPend), 1);           \
            evm_stats.nMetaMaxPioPend = MAX(evm_stats.qCnt.metaPioPend, \
                                            evm_stats.nMetaMaxPioPend); \
        }                                                               \
        else                                                            \
        {                                                               \
            ATOMIC_INC_INT(&(evm_stats.qCnt.userPioPend), 1);           \
            evm_stats.nUserMaxPioPend = MAX(evm_stats.qCnt.userPioPend, \
                                            evm_stats.nUserMaxPioPend); \
        }                                                               \
        evm_stats.nMaxPioPend = MAX(evm_stats.qCnt.metaPioPend +        \
                                    evm_stats.qCnt.userPioPend,         \
                                    evm_stats.nMaxPioPend);

#define EVM_STAT_Q_REMOVE_PIOPEND                               \
        if (qPtr->dataType == EVM_META_DATA_REQ)                \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.metaPioPend), 1);   \
        else                                                    \
            ATOMIC_DEC_INT(&(evm_stats.qCnt.userPioPend), 1);
#else

#define EVM_STAT_IOREQ                  
#define EVM_STAT_IOCOMPLETE             
#define EVM_STAT_IO_REJECTED
#define EVM_STAT_MREADS                 
#define EVM_STAT_MWRITES                
#define EVM_STAT_UREADS                 
#define EVM_STAT_UWRITES               
#define EVM_STAT_MREADS_RETRY
#define EVM_STAT_MWRITES_RETRY
#define EVM_STAT_UREADS_RETRY
#define EVM_STAT_UWRITES_RETRY
#define EVM_STAT_IOEVENTS             
#define EVM_STAT_CODRCVS             
#define EVM_STAT_CODCONCATACKS          
#define EVM_STAT_SCSIMREADRSPS          
#define EVM_STAT_SCSIMWRITERSPS         
#define EVM_STAT_SCSIUREADRSPS          
#define EVM_STAT_SCSIUWRITERSPS         
#define EVM_STAT_SCSIDEVERRS            
#define EVM_STAT_ALLOCS                 
#define EVM_STAT_DEALLOCS               
#define EVM_STAT_TIMERCREATES          
#define EVM_STAT_TIMERDESTROYS        
#define EVM_STAT_ERRORS              
#define EVM_STAT_TIMEOUTS               
#define EVM_STAT_STALES                
#define EVM_STAT_EEEDROPPACKET        
#define EVM_STAT_EEETXQUEUEFULL      
#define EVM_STAT_Q_INSERT_FREE
#define EVM_STAT_Q_REMOVE_FREE
#define EVM_STAT_Q_INSERT_PIOWAIT
#define EVM_STAT_Q_REMOVE_PIOWAIT
#define EVM_STAT_Q_INSERT_PIOPEND
#define EVM_STAT_Q_REMOVE_PIOPEND
#endif

/*
 * EVM SRVR unit test defines
 */

//#define EVM_SRVR_UNIT_TEST        1
//#define EVM_SRVR_TEST_TIMEOUT     1
//#define EVM_SRVR_TEST_ERR         1


/*------------------------------------------------------------------------------
 * Macros for memory (de)allocation.
 *------------------------------------------------------------------------------
 */
           
#define EVM_RAM_ALLOC(_p_, _type_, _size_)          \
    _p_ = (_type_ *) eee_ramAlloc(_size_);          \
    if (_p_ != NULL)                                \
    {                                               \
        EVM_STAT_ALLOCS;                            \
    }

#define EVM_RAM_ALLOC_ZERO(_p_, _type_, _size_)     \
    _p_ = (_type_ *) eee_ramAllocZero(_size_);      \
    if (_p_ != NULL)                                \
    {                                               \
        EVM_STAT_ALLOCS;                            \
    }

#define EVM_RAM_ALLOC_ALIGNED(_p_, _type_, _size_)  \
    _p_ = (_type_ *) eee_ramAllocAligned(_size_);   \
    if (_p_ != NULL)                                \
    {                                               \
        EVM_STAT_ALLOCS;                            \
    }

#define EVM_RAM_ALLOC_ALIGNED_ZERO(_p_, _type_, _size_) \
    _p_ = (_type_ *) eee_ramAllocAlignedZero(_size_);   \
    if (_p_ != NULL)                                    \
    {                                                   \
        EVM_STAT_ALLOCS;                                \
    }

#define EVM_RAM_DEALLOC(_p_)                                \
    if (_p_ != NULL)                                        \
    {                                                       \
        int32 rc = NFX_OK;                                  \
                                                            \
        rc = eee_ramDealloc(_p_);                           \
        if (rc != NFX_OK)                                   \
        {                                                   \
            evm_trace(EVM_TRACE_ERRORS,                     \
                      "%s:%s:%d: DEALLOC failed\n",         \
                      __FILE__, __FUNCTION__, __LINE__);    \
        }                                                   \
        _p_ = NULL;                                         \
        EVM_STAT_DEALLOCS;                                  \
    }

#define EVM_DEALLOC_BUF(_p_)                        \
    if (_p_ != NULL)                                \
    {                                               \
        eee_deallocateBuffer(_p_);                  \
        _p_ = NULL;                                 \
    }

/*----------------------------------------------------------------------------*/

#endif // _EVM_SRVR_DEFINES_H_
