/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: evm_loc_defs.h
 *
 * RCS:  $Id: evm_loc_defs.h,v 1.20 2002/09/19 00:38:20 isabellec Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Inter-module data defintions internal to EVM
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 27 Apr 2001
 *
 *-----------------------------------------------------------------
 */
#if !defined(__evm_loc_defs_h)
#define __evm_loc_defs_h

#include "exports/evm_com_defs.h"

// evm_LVCntrl: message between evm_cfgd and evm_lvmgr
#define EVM_LVMGR_RX_KEY (key_t)0xbeef0001

#define EVM_USE_REG 1                   // use agile registry mgr

// This LV has reached it high water mark
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_CNTRL_LVFULL
    evm_volId_t         lVol;		// This one has reached high water
} evm_MsgLVFull_t;


// EVM server is starting - it requests to be synchronized.
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_CNTRL_SYNC
} evm_MsgSync_t;

// evm_LVAct: messages between evm_cfgd and evm_srvr

// Create and serve this logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_LVACT_CREATELV
    evm_volId_t			lVol;		// Which logical volume to serve
    uint32				nInitSAU;	// Initial storage allocation
    uint32              pad;
    evm_nfxId_t         nfxId;      // unique NFX identifier
    evm_strgConfig_t	initSAU[1];	// Variable length message
} evm_MsgCreateLV_t;
#define CREATELV_SIZE(x) \
         (sizeof(evm_MsgCreateLV_t) + \
         (((x)->nInitSAU - 1) * sizeof(evm_strgConfig_t)))

// Append this storage to this existing logical volume
typedef	struct {
			evm_msgHdr_t		hdr;		// EVM_LVACT_ADDSTRG
			evm_volId_t			lVol;		// Which LV to add
			evm_strgConfig_t	strg;		// this physical storage to
		} evm_MsgAddStrg_t;

typedef	struct {
			evm_msgHdr_t		hdr;		// EVM_LVACT_REMSTRG
			evm_volId_t			lVol;		// Which LV to remove from
			uint64				numBlks;	// how many blocks (should be
		} evm_MsgRemStrg_t;					// whole allocation unit).

typedef	struct {
			evm_msgHdr_t		hdr;		// EVM_LVACT_ONLINELV
			evm_volId_t			lVol;		// Which LV to put online
		} evm_MsgOnLinLV_t;

typedef	struct {
			evm_msgHdr_t		hdr;		// EVM_LVACT_OFFLINELV
			evm_volId_t			lVol;		// Which LV to take offline
		} evm_MsgOffLinLV_t;

typedef	struct {
			evm_msgHdr_t		hdr;		// EVM_LVACT_DESTRLV
			evm_volId_t			lVol;		// Which LV to destroy
		} evm_MsgDestLV_t;

#ifdef DEBUG
typedef struct
{
    evm_msgHdr_t    hdr;
    uint64          offsetTab[1024];
} evm_MsgOffsetTab_t;
#endif // DEBUG

/*----------------------------------------------------------------------------*/
#endif
