/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: crashdump-ui.c,v 1.5 2002/10/16 03:03:14 johnh Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/crashdump-api.h>
#include "crashdump.h"

static void crashdump_test_panic(uchar8	* str);
static void crashdump_test_nmi(void);
static void crashdump_test_stackoverflow(int);

#if defined(NFP_FP)
// these fct are NOT really 'crashdump' modules but sm-coredump modules
// piggyback-ing on crashdump-ui
void crashdump_test_sendpacket(void);
void crashdump_test_receivepacket(void);
void savecore(void);
#endif

int32
crashdump_cmd(
	int32	argc,
	uchar8	* argv[]
)
{
	if (argc < 2)
		return -1;

	if (!strncmp(argv[1], "view", 2)) {
		if (argc == 2)
			exc_crashdump_view(EXC_CD_SAVED);
		else {
			if (!strncmp(argv[2], "saved", 3))
				exc_crashdump_view(EXC_CD_SAVED);
			else if (!strncmp(argv[2], "pending", 3))
				exc_crashdump_view(EXC_CD_PENDING);
			else
                return NFX_ERR;
		}
	}
	else if (!strncmp(argv[1], "store", 3)) {
		if (argc == 2 || !strncmp(argv[2], "saved", 3)) {
            /*
             * store off the saved crashdump, to SCC file system
             * and then remove to make room for next dump
             */
            exc_crashdump_store(EXC_CD_SAVED);
        }
        else {
			if (!strncmp(argv[2], "pending", 3)) {
                /*
                 * not supported
                 */
				printf("pending crashdump store is not supported\n");
            }
            return NFX_ERR;
		}
	}
	else if (!strncmp(argv[1], "remove", 3)) {
		exc_crashdump_remove();
	}
	else if (!strncmp(argv[1], "panic", 5)) {
		crashdump_test_panic(argv[2]);
	}
	else if (!strncmp(argv[1], "nmi", 3)) {
		crashdump_test_nmi();
	}
	else if (!strncmp(argv[1], "stackoverflow", 3)) {
        int n = 0;
        if (argc == 3)
            n = atoi(argv[2]);
		crashdump_test_stackoverflow(n);
	}
#if defined(NFP_FP)
    else if (!strncmp(argv[1], "core", sizeof("core"))) {
        crashdump_test_core();
    }
    else if (!strncmp(argv[1], "send", sizeof("send"))) {
        crashdump_test_sendpacket();
    }
    else if (!strncmp(argv[1], "receive", sizeof("receive"))) {
        crashdump_test_receivepacket();
    }
    else if (!strncmp(argv[1], "save", sizeof("save"))) {
        savecore();
    }
#endif
	else
		return NFX_ERR;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
static void
crashdump_test_panic(
	uchar8	* str
)
{
	printf("test Panic\n");

	//panic(str);
    if( str == NULL)
        panic("WTG Test Panic\n");
    else 
	panic(str);
      
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
static void
crashdump_test_nmi(void)
{
	printf("test NMI\n");

	wd_init();

	watchdog_disable();
	watchdog_enable(5);
	delay_us(1000000*5);
	delay_us(1000000*5);
	delay_us(1000000*5);
	printf("back from 15 sec delay?\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	John Hahn
 *
 * Date Created:	10/14/02
 *
 *-----------------------------------------------------------------
 */
static void
crashdump_test_stackoverflow(
	int n
)
{
    volatile int dummy[128];

    dummy[0] = n;
	printf("test stackoverflow, nesting %d, sp=%p\n",
            dummy[0], (void *)_mips_get_address_reg(R_SP));

    if (--n)
        crashdump_test_stackoverflow(n);
    else
        panic("crashdump_test_stackoverflow()\n");
}
