/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: debug.h,v 1.5 2002/11/05 00:44:05 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _DEBUG_H
#define _DEBUG_H

extern int32	trace_mode;
extern int32	runtime_exception;
#define EXCEPTION_NONE			0
#define EXCEPTION_GEN_EXCEPT	1
#define EXCEPTION_NMI			2

#define MAX_BPT  64     /* max number of user breakpoints */
#define LINESZ  200     /* max command line length */
#define MAX_AC  100     /* max number of args to commands */
#define PATSZ    80     /* max pattern length for search & fill */

typedef struct Bps {
	unsigned long	addr;
	unsigned long	len;
	unsigned int	value;
	unsigned long	wpt;
#if 0
	char			*cmdstr;
#endif
} Bps;

extern Bps	Bpt[MAX_AC];	/* user break points                    */
extern Bps	BptTmp;			/* tmp bpt used for continue cmd        */
extern Bps	BptTrc;			/* bpt for tracing                      */
extern Bps	BptTrcb;		/* bpt for tracing through branches     */

#define NO_BPT      -1L
#define BPT_CODE    0x0000000dL /* BREAK instruction */
#define SIGNATURE   0x4572696e

/* trace_mode states */
#define TRACE_NO    0   /* no trace/go */
#define TRACE_TB    1   /* trace & break */
#define TRACE_TG    2   /* trace & go */
#define TRACE_GB    3   /* go & break */
#define TRACE_DG    4   /* debug mode step and go */
#define TRACE_DC    5   /* debug mode continue */
#define TRACE_DS    6   /* debug mode step */
#define TRACE_TN    7   /* multistep trace */
#define TRACE_TW    8   /* trace watchpoint step */
#define TRACE_DW    9   /* debug mode watchpoint step */

#define EOF		(-1)

#define NCC 23
struct termio {
    unsigned short c_iflag;
    unsigned short c_oflag;
    unsigned short c_cflag;
    unsigned short c_lflag;
    unsigned char c_cc[NCC];
};

/* operation codes for ioctl */
#define TCGETA      1
#define TCSETAF     2
#define TCSETAW     3
#define SETINTR     4
#define SETSANE     5
#define FIONREAD    6
#define GETINTR     7
#define GETTERM     8
#define SETTERM     9
#define SETNCNE     10
#define CBREAK      11
#define TERMTYPE    12

/* iflags */
#define ISTRIP      0x0020  /* strip 8th bit off chars */
#define ICRNL       0x0040  /* map CR to NL */
#define IXON        0x0400  /* enable output flow control */
#define IXANY       0x0800  /* any char restarts after stop */
#define IXOFF       0x1000  /* enable input flow control */
#define ICTS_OFLOW  0x2000  /* CTS flow control of output */
#define IRTS_IFLOW  0x4000  /* RTS flow control of input */

/* cc definitions */
#define VINTR   0
#define VERASE  2
#define VEOL    5
#define VEOL2   6
#define V_START 8
#define V_STOP  9
#define VDEL    10

#if 0
/* operation codes for device specific driver */
#define OP_INIT     1
#define OP_TX       2
#define OP_RX       3
#define OP_RXRDY    4
#define OP_TXRDY    5
#define OP_BAUD     6
#define OP_RXSTOP   7
#define OP_FLUSH    8
#define OP_RESET    9
#define OP_OPEN     10
#define OP_CLOSE    11
#define OP_XBAUD    12
#define OP_TCSET    13
#endif

/* watchpoint capabilities and type (when setting) */
#define MIPS_WATCHPOINT_X   0x0004  /* instruction fetch wp */
#define MIPS_WATCHPOINT_R   0x0002  /* data read wp */
#define MIPS_WATCHPOINT_W   0x0001  /* data write wp */

/* flush_cache types */
#define ICACHE  0
#define DCACHE  1
#define IADDR   2

#define CNTRL(x) (x & 0x1f)
#define GDB_QUIT	(-3)

void        store_dword (address_t, unsigned long long);
void        store_word (address_t, unsigned long);
void        store_half (address_t, ushort16);
void        store_int (address_t, int);
void        store_byte (address_t, byte);

void	clrbpts();
int		clrbpt(int wpt, long adr, long len);
int		setbpt(int wpt, long adr, long len);
void	debug_exit();
void	godebug();
int		sstep();
void	exception_return();
void	panic_prompt();

#include <sys/mips.h>

#define SIBYTE_CACHEOP(_op, _addr) {                   \
    unsigned int  _r;                                 \
    __asm volatile (".set noreorder\n\tand %0,%2,~0x1F\n\tand %0,%0,%0\n\tcache %1,0(%0)\n\tssnop\n\tssnop\n\tssnop\n\tssnop\n\t.set reorder"                       \
    : "=r" (_r) : "i" (_op), "r" (_addr));      \
}

#ifdef SIBYTE
#define RAL_store_word(_addr, _data)    {       \
    store_word(_addr, _data);                   \
    SIBYTE_CACHEOP(Hit_Invalidate_I,_addr);     \
    SIBYTE_CACHEOP(Hit_Writeback_D,_addr);      \
}
#define RAL_store_int(_addr, _data) {       \
    store_int(_addr, _data);                    \
    SIBYTE_CACHEOP(Hit_Invalidate_I,_addr);     \
    SIBYTE_CACHEOP(Hit_Writeback_D,_addr);      \
}
#else
#define RAL_store_int(_addr, _data) store_int(_addr, _data)
#endif

int is_branch(address_t adr);
int is_jr(address_t adr);
int is_jal(address_t adr);
int is_conditional_branch(address_t adr);
address_t branch_target_address(address_t adr);
int is_writeable(address_t adr);

typedef enum {
	RD_RS_RT, RT_RS_IMM, OFF, RS_RT_OFF, RS_OFF, NONE, RT_RD, COFUN, RD_RT,
	RS_RT, TARGET, JALR, RS, RD_RT_SFT, LOAD_STORE, RT_IMM, RD, RD_RT_RS, RD_RS,
	RT_RS_SIMM, RS_SIMM, RT_SIMM, JR, RT_C0, RT_C1, RT_CN, RT_CC1, LDSTC0, LDSTC1,
	LDSTCN, WORD, RT_C2, RT_CC2, BPCODE, CACHE_OP,
	CP_OFF, STORE, STOREC1, STORECN,
	FT_FS_FD_D, FT_FS_FD_S, FS_FD_D, FS_FD_S, FS_FD_W, FS_FD_L, FT_FS_D, FT_FS_S,
	RT_RD_TO, RT_CC1_TO, RT_CN_TO, RT_C0_TO, RT_C1_TO
} TYPE;

typedef struct {
	const char     *str;
	long            mask, code;
	TYPE            type;
} DISTBL;

#endif	/* _DEBUG_H */

