/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: stacktrace.c,v 1.9 2002/11/05 00:44:05 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/crashdump-api.h>
#include "crashdump.h"
#include "except.h"
#include "debug.h"

#include <mips/mips_opcode.h> 
#define MIPS_JR_RA	0x03e00008	/* instruction code for jr ra */
#define MIPS_DADDIU_SP_SP_MASK	0xffff8000
#define MIPS_DADDIU_SP_SP	0x67bd8000
#define MAX_PROLOGUE	32		/* max insns in prologue */

char	prnbuf[LINESZ + 8]; /* commonly used print buffer */

static struct savedreg {
	void	* addr;
	short	base;
	short	size;
	sreg_t	val;
} regs[32];


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
static int
is_validpc(
	address_t	adr
)
{
	extern uchar8	_ftext[], _fdata[];

	if (adr & 3)
		return 0;

	if ((adr >= (address_t)_ftext) && (adr < (address_t)_fdata))
		return 1;

	return 0;
}

static int
#if RAL_XXX
getsavedreg (int rn, sreg_t *rv)
#else
getsavedreg (crashdump_t * cptr, int rn, sreg_t *rv)
#endif
{
	int saved = 0;
	
	if (regs[rn].addr) {
#if RAL_XXX
		if (regs[rn].size == 8)
			regs[rn].val = load_dword (regs[rn].addr);
		else
			regs[rn].val = load_word (regs[rn].addr);
#else
		int index = ((uint32*)regs[rn].addr)-((uint32*)((address_t)cptr->regs[R_SP]));
	
		/** if we underrun on the stored stack, give up. **/
		if (index < 0 || index >= CRASHDUMP_STACK_LEN ||
					(regs[rn].size == 8 && index >= CRASHDUMP_STACK_LEN-1)) {
			*rv = 0;
			return 0;
		}
	
		if (regs[rn].size == 8)
			regs[rn].val = ((uint64)cptr->stack[index]) +
										(((uint64)cptr->stack[index+1]) << 32);
		else
			regs[rn].val = cptr->stack[index];
#endif
		regs[rn].addr = 0;
		saved = 1;
	}

	*rv = regs[rn].val;

	return saved;
}
	
	
	
static void
setregsave (int rn, int base, void *addr, int size)
{
	regs[rn].addr = addr;
	regs[rn].base = base;
	regs[rn].size = size;
}
	
	
static void
adjregsaves (int obase, int nbase, int adjust)
{
	int r;
	
	for (r = 0; r < 32; r++) {
		if (regs[r].addr && regs[r].base == obase) {
			regs[r].addr += adjust;
			regs[r].base = nbase;
		}
	}
}


#if 0
/*
 * Print a stack backtrace.
 */
int
stacktrace(
	crashdump_t	* cptr
)
{
#if RAL_XXX
	uword			pc, sp, fp, ra, va, subr;
#else
	address_t		pc, sp, fp, ra, va, subr;
#endif
	long			base;
	sreg_t			rv;
	unsigned long	mask;
	InstFmt			i;
	int				cont, stksize, adjust, r;
#if RAL_XXX
	uword			cnt, siz;
#else
	uint32			cnt, siz;
#endif
	char			* p;
#if __mips >= 3
	int				regsize = REGSZ_64;
#else
	int				regsize = REGSZ_32;
#endif
	int				vflag = 0;
	extern int		optind;
	extern char		*optarg;
	int				c, rn;

#if RAL_XXX
	optind = 0;
	while ((c = getopt (ac, av, "v")) != EOF)
		switch (c) {
			case 'v':
				vflag++;
				break;
			default:
				return (-1);
		}

	cnt = siz = moresz;
	if (optind < ac) {
		if (!get_rsa (&cnt, av[optind++]))
			return (-1);
		siz = 0;
	}

	if (optind != ac)
		return (-1);

	ioctl (STDIN, CBREAK, NULL);
#endif

	bzero (regs, sizeof (regs));
	sp = cptr->regs[R_SP];
	fp = cptr->regs[R_FP];
	ra = cptr->regs[R_RA];
#ifndef SSC_MGMT
	if (cptr->flags & CRASHDUMP_FLAGS_CAUSE_NMI)
		pc = cptr->regs[R_ERREPC];
	else
#endif
		pc = cptr->regs[R_EPC];

stkloop:

	stksize = adjust = 0;

	if ((sp & 3) || sp < K0BASE || sp >= K2BASE) {
		printf("invalid SP=%p\n", sp);
		ra = subr = 0;
		goto stkdone;
	}

	/*
	 * Find the beginning of the current subroutine by scanning backwards
	 * from the current PC for the end of the previous subroutine.
	 */
	va = pc;
	do {
		va -= sizeof(int);
		if (!is_validpc (va)) {
			printf("invalid PC=%p\n", va);
			ra = subr = 0;
			goto stkdone;
		}
#ifdef SIBYTE
	} while ((load_int(va) & MIPS_DADDIU_SP_SP_MASK) != MIPS_DADDIU_SP_SP);
#else
	} while (load_int(va) != MIPS_JR_RA);

	va += 2 * sizeof(int);	/* skip back over "jr ra" & delay slot */
#endif

	/* skip over nulls which might separate .o files */
	while (load_int(va) == 0 && va < pc)
		va += sizeof(int);
	subr = va;

	/* scan forwards to find stack size and any saved registers */
	mask = 0;

	cont = (pc - va) / sizeof(int);
	for ( ; cont > 0 ; va += sizeof(int), cont--) {

		i.word = load_int(va);
		switch (i.JType.op) {
			case OP_SPECIAL:
				switch (i.RType.func) {
					case OP_JR:
					case OP_JALR:
						if (cont > 2)
							cont = 2; /* stop after next instruction */
						break;
	
					case OP_SYSCALL:
					case OP_BREAK:
						cont = 1; /* stop now */
						break;

					case OP_OR:
					case OP_DADDU:
					case OP_ADDU:
						/* one of RS or RT must be $0, for this to be a MOVE */
						if (i.RType.rt == 0)
							rn = i.RType.rs;
						else if (i.RType.rs == 0) 
							rn = i.RType.rt;
						else
							break;

						if (i.RType.rd == 30 && rn == 29) {
							/* move $fp,$sp */
							/* check for use of alloca ($sp != $fp) */
							if (fp > sp) {
						/* base previous register-saves on more reliable $fp */
								adjregsaves (29, 30, adjust = fp - sp);
						/* get the correct value of $sp at this point */
								sp += adjust;
							}
						}
						break;
				}
				break;

			case OP_REGIMM:
				switch (i.IType.rt) {
					case OP_BLTZ:
					case OP_BGEZ:
					case OP_BLTZL:
					case OP_BGEZL:
					case OP_BLTZAL:
					case OP_BGEZAL:
					case OP_BLTZALL:
					case OP_BGEZALL:
						if (cont > 2)
							cont = 2;		/* stop after next instruction */
						break;

					case OP_TGEI:
					case OP_TGEIU:
					case OP_TLTI:
					case OP_TLTIU:
					case OP_TEQI:
					case OP_TNEI:
						cont = 1;		/* stop now */
						break;
				}
				break;

			case OP_COP0:
			case OP_COP1:
			case OP_COP2:
			case OP_COP3:
				switch (i.RType.rs) {
					case OP_BCx:
					case OP_BCy:
						if (cont > 2)
							cont = 2;		/* stop after next instruction */
				};
				break;

			case OP_SW:
			case OP_SD:
				/* look for saved registers and store a pointer to them */
				if (i.IType.rs == 29)
					base = sp;
				else if (i.IType.rs == 30)
					base = fp;
				else
					break;
				if (!(mask & (1 << i.IType.rt))) {
					mask |= 1 << i.IType.rt;
					setregsave (i.IType.rt, i.IType.rs,
										(void *)base + (short)i.IType.imm,
												(i.JType.op == OP_SW) ? 4 : 8);
				}
				break;

			case OP_ADDIU:
			case OP_DADDIU:
#ifdef SIBYTE
/*
 * Sibyte compiler puts return in middle of function.  Algorithmics puts return
 * at end of function (much nicer).
 */
if (load_int(va-4) == MIPS_JR_RA)
	break;
#endif
				/* stack or frame pointer adjustment? */
				if (i.IType.rs == 29) {
					if (i.IType.rt == 29) {		/* addu $sp,size */
					/* adjust previous saves, which should be based on old val */
						adjregsaves (29, 29, -(short)i.IType.imm);
					/* but we assume that current sp is now correct */
						stksize += -(short)i.IType.imm;
					} else if (i.IType.rt == 30) {	/* addu $fp,$sp,size */
						/* check for use of alloca ($sp + size != $fp) */
						if (fp > sp + (short)i.IType.imm) {
						/* base previous register-saves on more reliable $fp */
							adjust = fp - (sp + (short)i.IType.imm);
							adjregsaves (29, 30, adjust);
						/* we also get the correct value of $sp at this point */
							sp += adjust;
						}
					}
				}
			break;
		}
	}

	/* fetch saved fp and ra registers from stack */
#if RAL_XXX
	if (getsavedreg (30, &rv))
		fp = rv;
	if (getsavedreg (31, &rv))
		ra = rv;
#else
	if (getsavedreg (cptr, 30, &rv))
		fp = rv;
	if (getsavedreg (cptr, 31, &rv))
		ra = rv;
#endif

stkdone:
	p = prnbuf;
#if RAL_XXX
	if (!adr2symoff (p, pc, 24)) {
		if (subr) 
			sprintf (p, "          0x%08x+0x%-4x", subr, pc - subr);
		else
			sprintf (p, "                 0x%08x", pc);

	}
#else
	if (subr)
		sprintf (p, " pc == %p, function @: %p", pc, subr);
	else
		sprintf (p, " pc == %p, no function context found ", pc);
#endif
	p += strlen(p);

	/* print out argument reg values, if they were saved */
	*p++ = '(';
	for (r = R_A0; r <= R_A3; r++) {
#if RAL_XXX
		if (!getsavedreg (r, &rv))
#else
		if (!getsavedreg (cptr, r, &rv))
#endif
			/* reg not saved (optimised code or unused arg) */
			break;
		if (r != R_A0)
			*p++ = ',';
		if (regsize == REGSZ_64)
			sprintf (p, "0x%llx", rv);
		else
			sprintf (p, "0x%lx", (uint32)rv);
		p += strlen(p);
	}
	*p++ = ')'; *p++ = '\0';
#if RAL_XXX
	if (more (prnbuf, &cnt, siz))
		return (0);
#else
	printf("%s\n", prnbuf);
#endif

	if (vflag) {
		p = prnbuf;
		strcpy (p, "            from "); 
		p += strlen (p);;
#if RAL_XXX
		if (!adr2symoff (p, ra, 12))
#endif
			sprintf (p, "         %p", ra);
		p += strlen(p);
		sprintf (p, " frame=%p size=%-4d", sp, stksize);
		if (adjust) {
			p += strlen(p);
			sprintf (p, " (alloca=%-4d)", adjust);
		}
#if RAL_XXX
		if (more (prnbuf, &cnt, siz))
			return (0);
#else
		printf("%s\n", prnbuf);
#endif
	}

	if (ra) {
		if (pc == ra && stksize == 0)
#if RAL_XXX
			more ("stacktrace loop!", &cnt, siz);
#else
			printf ("stacktrace loop!");
#endif
		else {
			pc = ra; ra = 0;
			sp += stksize;
			goto stkloop;
		}
	}

	return 0;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/23/02
 *
 *-----------------------------------------------------------------
 */
static int
is_validsp(
	address_t	sp
)
{
	if ((sp >= K0BASE) && (sp <= K2BASE) && !(sp & 3))
		return 1;
#ifdef USE_THREAD_STACKS
	if ((sp & 0xF000000000) == 0x5000000000)
		return 1;
#endif
	return 0;

}

/*
 * Print a stack backtrace.
 */
uint32
format_stacktrace(
	crashdump_t	*   cptr,
    uint8 *         buf_p
)
{
	address_t		pc, sp, fp, ra, va, subr;
	long			base;
	sreg_t			rv;
	unsigned long	mask;
	InstFmt			i;
	int				cont, stksize, adjust, r;
#if __mips >= 3
	int				regsize = REGSZ_64;
#else
	int				regsize = REGSZ_32;
#endif
	int				vflag = 0;
	int				rn;
    uint8 *         bp;

    bp = buf_p;

	bzero (regs, sizeof (regs));
	sp = cptr->regs[R_SP];
	fp = cptr->regs[R_FP];
	ra = cptr->regs[R_RA];
#ifndef SSC_MGMT
	if (cptr->flags & CRASHDUMP_FLAGS_CAUSE_NMI)
		pc = cptr->regs[R_ERREPC];
	else
#endif
		pc = cptr->regs[R_EPC];

stkloop:

	stksize = adjust = 0;

	if (!is_validsp(sp)) {
		bp += snprintf(bp, exc_fblen(bp), "invalid SP=%p\n", sp);
		ra = subr = 0;
		goto stkdone;
	}

	/*
	 * Find the beginning of the current subroutine by scanning backwards
	 * from the current PC for the end of the previous subroutine.
	 */
	va = pc;
	do {
		va -= sizeof(int);
		if (!is_validpc (va)) {
			bp += snprintf(bp, exc_fblen(bp), "invalid PC=%p\n", va);
			ra = subr = 0;
			goto stkdone;
		}
#ifdef SIBYTE
	} while ((load_int(va) & MIPS_DADDIU_SP_SP_MASK) != MIPS_DADDIU_SP_SP);
#else
	} while (load_int(va) != MIPS_JR_RA);

	va += 2 * sizeof(int);	/* skip back over "jr ra" & delay slot */
#endif

	/* skip over nulls which might separate .o files */
	while (load_int(va) == 0 && va < pc)
		va += sizeof(int);
	subr = va;

	/* scan forwards to find stack size and any saved registers */
	mask = 0;

	cont = (pc - va) / sizeof(int);
	for ( ; cont > 0 ; va += sizeof(int), cont--) {

		i.word = load_int(va);
		switch (i.JType.op) {
			case OP_SPECIAL:
				switch (i.RType.func) {
					case OP_JR:
					case OP_JALR:
						if (cont > 2)
							cont = 2; /* stop after next instruction */
						break;
	
					case OP_SYSCALL:
					case OP_BREAK:
						cont = 1; /* stop now */
						break;

					case OP_OR:
					case OP_DADDU:
					case OP_ADDU:
						/* one of RS or RT must be $0, for this to be a MOVE */
						if (i.RType.rt == 0)
							rn = i.RType.rs;
						else if (i.RType.rs == 0) 
							rn = i.RType.rt;
						else
							break;

						if (i.RType.rd == 30 && rn == 29) {
							/* move $fp,$sp */
							/* check for use of alloca ($sp != $fp) */
							if (fp > sp) {
						/* base previous register-saves on more reliable $fp */
								adjregsaves (29, 30, adjust = fp - sp);
						/* get the correct value of $sp at this point */
								sp += adjust;
							}
						}
						break;
				}
				break;

			case OP_REGIMM:
				switch (i.IType.rt) {
					case OP_BLTZ:
					case OP_BGEZ:
					case OP_BLTZL:
					case OP_BGEZL:
					case OP_BLTZAL:
					case OP_BGEZAL:
					case OP_BLTZALL:
					case OP_BGEZALL:
						if (cont > 2)
							cont = 2;		/* stop after next instruction */
						break;

					case OP_TGEI:
					case OP_TGEIU:
					case OP_TLTI:
					case OP_TLTIU:
					case OP_TEQI:
					case OP_TNEI:
						cont = 1;		/* stop now */
						break;
				}
				break;

			case OP_COP0:
			case OP_COP1:
			case OP_COP2:
			case OP_COP3:
				switch (i.RType.rs) {
					case OP_BCx:
					case OP_BCy:
						if (cont > 2)
							cont = 2;		/* stop after next instruction */
				};
				break;

			case OP_SW:
			case OP_SD:
				/* look for saved registers and store a pointer to them */
				if (i.IType.rs == 29)
					base = sp;
				else if (i.IType.rs == 30)
					base = fp;
				else
					break;
				if (!(mask & (1 << i.IType.rt))) {
					mask |= 1 << i.IType.rt;
					setregsave (i.IType.rt, i.IType.rs,
										(void *)base + (short)i.IType.imm,
												(i.JType.op == OP_SW) ? 4 : 8);
				}
				break;

			case OP_ADDIU:
			case OP_DADDIU:
#ifdef SIBYTE
                /*
                 * Sibyte compiler puts return in middle of function.  Algorithmics puts return
                 * at end of function (much nicer).
                 */
                if (load_int(va-4) == MIPS_JR_RA)
                    break;
#endif
				/* stack or frame pointer adjustment? */
				if (i.IType.rs == 29) {
					if (i.IType.rt == 29) {		/* addu $sp,size */
					/* adjust previous saves, which should be based on old val */
						adjregsaves (29, 29, -(short)i.IType.imm);
					/* but we assume that current sp is now correct */
						stksize += -(short)i.IType.imm;
					} else if (i.IType.rt == 30) {	/* addu $fp,$sp,size */
						/* check for use of alloca ($sp + size != $fp) */
						if (fp > sp + (short)i.IType.imm) {
						/* base previous register-saves on more reliable $fp */
							adjust = fp - (sp + (short)i.IType.imm);
							adjregsaves (29, 30, adjust);
						/* we also get the correct value of $sp at this point */
							sp += adjust;
						}
					}
				}
			break;
		}
	}

	/* fetch saved fp and ra registers from stack */
	if (getsavedreg (cptr, 30, &rv))
		fp = rv;
	if (getsavedreg (cptr, 31, &rv))
		ra = rv;

stkdone:
	if (subr)
		bp += snprintf (bp, exc_fblen(bp), " pc == %p, function @: %p", pc, subr);
	else
		bp += snprintf (bp, exc_fblen(bp), " pc == %p, no function context found ", pc);

	/* print out argument reg values, if they were saved */
	bp += snprintf(bp, exc_fblen(bp), "(");
	for (r = R_A0; r <= R_A3; r++) {
		if (!getsavedreg (cptr, r, &rv))
			/* reg not saved (optimised code or unused arg) */
			break;
		if (r != R_A0)
            bp += snprintf(bp, exc_fblen(bp), ",");
		if (regsize == REGSZ_64)
			bp += snprintf (bp, exc_fblen(bp), "0x%llx", rv);
		else
			bp += snprintf (bp, exc_fblen(bp), "0x%lx", (uint32)rv);
	}
	bp += snprintf(bp, exc_fblen(bp), ")\n");

	if (vflag) {
		bp += snprintf (bp, exc_fblen(bp), "            from "); 
        bp += snprintf (bp, exc_fblen(bp), "         %p", ra);
		bp += snprintf (bp, exc_fblen(bp), " frame=%p size=%-4d", sp, stksize);
		if (adjust) {
            bp += snprintf (bp, exc_fblen(bp), " (alloca=%-4d)", adjust);
        }
		bp += snprintf (bp, exc_fblen(bp), "\n");
	}

	if (ra) {
		if (pc == ra && stksize == 0)
			bp += snprintf (bp, exc_fblen(bp), "stacktrace loop!");
		else {
			pc = ra; ra = 0;
			sp += stksize;
			goto stkloop;
		}
	}

	return((uint32) (bp - buf_p));
}

