/*
 *-----------------------------------------------------------------
 *
 * Name:	file-api.h
 *
 * RCS:  $Id: file-api.h,v 1.5 2002/08/28 17:43:15 bob Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	UNIX-like file I/O library for eagle filesystem.
 *
 * Created by:	Bob Fozard
 *
 * Date Created:	11/07/01
 *
 *-----------------------------------------------------------------
 */



/**
 * This is the maximum size supported per
 * read/write request.  TODO - this right????
 */
#define FILE_BLOCK_SIZE		(8 * 1024)

/**
 * create a file, returns fd on success, < 0 on error.
 */
int32
file_create(
    char        *name,      // eg. "volumeName:/path1/.../pathN/fileName"
    uint32      mode        // eg. unix modes, eg. 0666
);

/**
 * open an existing file for reading/writing, returns fd on success,
 * < 0 on error.
 */
int32
file_open(
    char        *name      // eg. "volumeName:/path1/.../pathN/fileName"
);

/**
 * close an open file.
 */
int32
file_close(
    int32   fd
);

/**
 * write to a file.  Returns num bytes written or < 0 on error.
 */
int32
file_write(
    int32   fd,
    char    *buf,
    int32   length
);

/**
 * write to a file, using preallocated buffer to send messages.
 * headerBuf should be of size length + sizeof(file_WriteHeader).
 * This can be used to save on malloc/free calls if it is necessary
 * call call file_write several times.
 */
#define FILE_HEADER_MAX    128
typedef struct file_WriteHeader
{
    union {
        gmc_app_hdr_t   gmcHdr;
        byte            fileHdr[FILE_HEADER_MAX];
    } u;
} file_WriteHeader;

int32
file_write_use_prealloc(
    int32   fd,
    char    *buf,
    int32   length,
    char  	*headerBuf
);

/**
 * read from a file.  Returns num bytes read, or < 0 on error.
 */
int32
file_read(
    int32   fd,
    char    *buf,
    int32   length
);

/**
 * Unlink (remove) a file.
 */
int32
file_unlink(
    char    *name       // eg. "volumeName:/path1/.../pathN/fileName"
);

/**
 * Create a directory.
 */
int32
file_mkdir(
    char    *name,      // eg. "volumeName:/path1/.../pathN/fileName"
    uint32  mode        // eg. unix modes, eg. 0666
);

/**
 * Remove a directory.
 */
int32
file_rmdir(
    char    *name       // eg. "volumeName:/path1/.../pathN/fileName"
);

/**
 * Rename a file.
 */
int32
file_rename(
	char *nameFrom,
	char *nameTo
);

/**
 * Seek on a file.  Returns new offset or -1 on error.
 */
int32
file_lseek(
    int32   fd,
    int32   whence,
    int64   offset
);

/**
 * whence argument for file_lseek.
 */
enum {
    FILE_SEEK_SET,  // The offset is set to offset bytes.

    FILE_SEEK_CUR,  // The offset is set to its current location plus
                    // offset bytes.

    FILE_SEEK_END,  // The offset is set to the size of the file plus
                    // offset bytes.
};
