#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

typedef unsigned long long uint64;
typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;
typedef struct fs_Time  fs_Time;

struct fs_Time
{
    uint32      secs;
    uint32      nsecs;
};

#include "fs-dumprestore.h"


int     fd;
char  *progname;
char  *filename;
char  buf[1024];
char  nameBuf[1024 + 1];
fs_DumpHeader_t *header = (fs_DumpHeader_t *)buf;

static void
usage()
{
    fprintf(stderr, "%s <dumpfilename>\n", progname);
    fprintf(stderr, "where dumpfilename is the full path name of the "
            " file generated by dump\n");
}

static void
printStartHeader()
{
    fs_DumpStartHeader_t    *startHeader = (fs_DumpStartHeader_t *)header;

    printf("\tmagic is %llx\n", startHeader->magic);
    printf("\tsnapshot is %llu\n", startHeader->snapshot);
    printf("\tprevSnapShot is %llu\n", startHeader->prevSnapshot);
    printf("\tdumpTime is %ld\n", startHeader->dumpTime.secs);
    printf("\tprevDumpTime is %llu\n", startHeader->prevDumpTime.secs);
    printf("\tlevel is %d\n", (int)startHeader->level);
    printf("\ttype is %s\n",
            startHeader->type == FS_DR_FULL ? "full" : "incrmental");
    printf("\tpathSep is %c\n", startHeader->pathSeparator);
    printf("\tflags is %x\n", (int)startHeader->flags);
    printf("\tvolid is %lld\n", startHeader->volid);
    printf("\tchecksum is %lld\n", startHeader->checkSum);
}

static void
printDirCreat()
{
    fs_DumpOperation_t  *obj = &header->u.operation;
    int                 len = obj->len;
    int                 objsize = sizeof (fs_DumpOperation_t);
    char                *dummy = header->u.dummy;
    char                *name = dummy + objsize;
    int                 nBytes;

    strncpy(nameBuf, name, len);
    if (objsize + len > FS_DR_TBSIZE) {
        nameBuf[FS_DR_TBSIZE - objsize] = '\0';
        printf("\tCreate %s", name);
        nBytes = read(fd, nameBuf, FS_DR_TBSIZE);
    } else {
        nBytes = len;
        printf("\tCreate ");
    }
    nameBuf[nBytes] = '\0';
    name = nameBuf;
    printf("%s\n", name);
    
}
static void
printFileCreat()
{
    fs_DumpOperation_t  *obj = &header->u.operation;
    int                 len = obj->len;
    int                 objsize = sizeof (fs_DumpOperation_t);
    char                *dummy = header->u.dummy;
    char                *name = dummy + objsize;
    int                 nBytes;
    int                 nExtents = 0;
    int                 i;
    fs_DumpExtent_t     *drExtent;
    uint64              blocks = 0;

    strncpy(nameBuf, name, len);
    if (objsize + len > FS_DR_TBSIZE) {
        nameBuf[FS_DR_TBSIZE - objsize] = '\0';
        printf("\tCreate %s", name);
        nBytes = read(fd, nameBuf, FS_DR_TBSIZE);
    } else {
        nBytes = len;
        printf("\tCreate ");
        nExtents = obj->header.dataExtents;
    }
    nameBuf[nBytes] = '\0';
    name = nameBuf;
    printf("%s\n", name);

    if (nExtents != 0) {

        /*
         * Find out number of blocks to drain
         */

        objsize += len;
        objsize += (objsize & 7);
        drExtent = (fs_DumpExtent_t *)(dummy + objsize);
        for (i = 0; i < nExtents; i++) {
            blocks += drExtent->len;
            drExtent++;  
        }

        blocks <<= 8;
        while (blocks != 0) {
            nBytes = read(fd, nameBuf, FS_DR_TBSIZE);
            write(2, nameBuf, FS_DR_TBSIZE);
            blocks--;
        }
    }
    
}
static void
printSymlCreat()
{
    fs_DumpOperation_t  *obj = &header->u.operation;
    int                 len = obj->len;
    int                 objsize = sizeof (fs_DumpOperation_t);
    char                *dummy = header->u.dummy;
    char                *name = dummy + objsize;
    int                 nBytes;
    int                 nExtents = 0;
    int                 i;
    fs_DumpExtent_t     *drExtent;
    uint64              blocks = 0;

    strncpy(nameBuf, name, len);
    if (objsize + len > FS_DR_TBSIZE) {
        nameBuf[FS_DR_TBSIZE - objsize] = '\0';
        printf("\tCreate %s", name);
        nBytes = read(fd, nameBuf, FS_DR_TBSIZE);
    } else {
        nBytes = len;
        printf("\tCreate ");
        nExtents = obj->header.dataExtents;
    }
    nameBuf[nBytes] = '\0';
    name = nameBuf;
    printf("%s\n", name);
    if (nExtents != 0) {

        /*
         * Find out number of blocks to drain
         */

        objsize += len;
        objsize += (objsize & 7);
        drExtent = (fs_DumpExtent_t *)(dummy + objsize);
        for (i = 0; i < nExtents; i++) {
            blocks += drExtent->len;
            drExtent++;  
        }

        blocks <= 8;
        while (blocks != 0) {
            nBytes = read(fd, nameBuf, FS_DR_TBSIZE);
            blocks--;
        }
    }
    
}
static void
printGenericHeader()
{
    fs_DumpGenericHeader_t  *genHeader = (fs_DumpGenericHeader_t *)header;
    printf("\nStart of new record \n\n");
    printf("header id is %d\n", genHeader->headerId);
    printf("header sub id is %d\n", genHeader->headerSubId);
    printf("header blocks is %d\n", genHeader->headerBlocks);
    printf("header dataExtents is %d\n", genHeader->dataExtents);
    printf("header flags are %x\n", genHeader->flags);
    switch (genHeader->op) {

    case FS_DR_CREATE_FILE:
        printf("header operation is create file\n");
        printFileCreat();
        break;

    case FS_DR_CREATE_SYML:
        printf("header operation is create symlink\n");
        printSymlCreat();
        break;

    case FS_DR_CREATE_BLOCK:
        printf("header operation is create block\n");
        break;

    case FS_DR_CREATE_CHAR:
        printf("header operation is create char\n");
        break;

    case FS_DR_CREATE_SOCK:
        printf("header operation is create sock\n");
        break;

    case FS_DR_CREATE_FIFO:
        printf("header operation is create fifo\n");
        break;

    case FS_DR_CREATE_DIR:
        printf("header operation is create Dir\n");
        printDirCreat();
        break;

    case FS_DR_CREATE_FSET:
        printf("header operation is create fset\n");
        break;

    case FS_DR_CREATE_LINK:
        printf("header operation is create link\n");
        break;

    case FS_DR_RENAME:
        printf("header operation is  rename\n");
        break;

    case FS_DR_REMOVE:
        printf("header operation is  remove\n");
        break;

    case FS_DR_REMOVE_DIR:
        printf("header operation is  rmdir\n");
        break;

    case FS_DR_DATA:
        printf("header operation is  data for file\n");
        break;

    case FS_DR_DIR_END:
        printf("header operation is  endo of dir\n");
        break;

    case FS_DR_TAPE_START:
        printf("header operation is  start of tape\n");
        printStartHeader();
        break;

    case FS_DR_TAPE_END:
        printf("header operation is  end of tape\n");
        printStartHeader();
        break;

    case FS_DR_NAME_ONLY:
        printf("header operation is  name only\n");
        break;
    }

    printf("Size of the object is %llu\n", genHeader->attr.size);
    printf("ctime of the object is %u\n", genHeader->attr.ctime);
    printf("mtime of the object is %u\n", genHeader->attr.mtime);
    printf("atime of the object is %u\n", genHeader->attr.atime);
    printf("uid of the object is %u\n", genHeader->attr.uid);
    printf("gid of the object is %u\n", genHeader->attr.gid);
    printf("mode of the object is %o\n", genHeader->attr.mode);

    return;
}

int
main(int argc, char **argv)
{
    int     nBytes;
    progname = argv[0];
    if (argc != 2) {
        usage();
        exit(1);
    }

    filename = argv[1];

    fd = open(filename, 0);
    if (fd == -1) {
        perror("cannot open dumpfile");
        exit(2);
    }   

    while ((nBytes = read(fd, buf, 1024)) != 0) {
        printGenericHeader();
    }
}
