/*
 * Name		: fs-am-api.h
 * RCS		: $ID$
 * Description	: FS-Agile Mirror interface
 * Created by	: Rahul Khona
 * Date Created	: 18/1/2002
 */
#ifndef _FS_AM_API_H_
#define _FS_AM_API_H_

typedef struct fs_AMExtent {
    uint64          blknum; // physical block on the volume
    uint32          len;    // length of the extent
} fs_AMExtent_t;
/*
 * Messages sent by Agile Mirror to setup and start delta
 * process
 */

typedef struct fs_MsgAmDeltaSetupReq {
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    uint64              cookie; // Identifies Am Session - unique ID
                                // provided by Am
    char                oldName[FS_SNAPNAME_MAX]; // 0 len string if no
                                                    // previous snapshots
    char                newName[FS_SNAPNAME_MAX];
} fs_MsgAmDeltaSetupReq_t;

typedef struct fs_MsgAmDeltaSetupReq    fs_MsgAmDeltaSetupReq;

typedef struct fs_MsgAmDeltaSetupResp {
    fs_MsgRespHdr hdr;
    uint64          cookie;     // identifies Am session
/**
    fs_Status       status;     //  FS_ERR_NONE on Success
                                // FS_ERR_EXISTS if new snapshot exists
                                // FS_ERR_NOENT if old snapshot does not exist
                                // FS_ERR_GENERIC if snapshot creation failed
                                // due to variety of reasons
**/
    uint64          fsCookie;   // cookie to be sent to fs for each
                                // subsequent transfer
    uint32          snapNum;    // snapshot id of new snapshot
} fs_MsgAmDeltaSetupResp_t;
typedef struct fs_MsgAmDeltaSetupResp fs_MsgAmDeltaSetupResp;

typedef struct fs_MsgAmDeltaGetExtentReq {
    fs_MsgReqHdr      hdr;
    evm_volId_t         volId;
    uint64              cookie;     // identifies Am session
    uint64              fsCookie;   // fs cookie returned in previous
                                    // setup or get request
    uint32              maxExtents; // maximum num of extents possible
} fs_MsgAmDeltaGetExtentReq_t;
typedef struct fs_MsgAmDeltaGetExtentReq fs_MsgAmDeltaGetExtentReq;

#define FS_Am_MAX_EXTENT_TRANSFER   512
typedef struct fs_MsgAmDeltaGetExtentResp {
    fs_MsgRespHdr     hdr;
    uint64              cookie;     // Am session
    uint64              fsCookie;   // Cookie to be returned in next
                                    // get request
/**
    fs_Status           status;     // FS_ERR_NONE for successful transfer
                                    // FS_ERR_NOENT if end of block
                                    // FS_ERR_IO if i/o error occured
**/
    uint32              numExtents; // number of extents returned 
                                    // <= requested max extents
    fs_AMExtent_t           extents[FS_Am_MAX_EXTENT_TRANSFER];
} fs_MsgAmDeltaGetExtentResp_t;
typedef struct fs_MsgAmDeltaGetExtentResp fs_MsgAmDeltaGetExtentResp;

typedef struct fs_MsgAmDeltaCompleteReq {
    fs_MsgReqHdr     hdr;
    evm_volId_t         volId;
    uint64             cookie;  // Am session
	char               snapName[FS_SNAPNAME_MAX];
/**
    fs_Status          status;  // FS_ERR_NONE for successfull complete
                                // FS_ERR_GENERIC for abort
**/
} fs_MsgAmDeltaCompleteReq_t;
typedef struct fs_MsgAmDeltaCompleteReq fs_MsgAmDeltaCompleteReq;

typedef struct fs_MsgAmDeltaCompleteResp {
    fs_MsgRespHdr     hdr;     
    uint64             cookie;  // Am session
/**
    fs_Status          status;  // FS_ERR_NONE always (? why keep it)
**/
} fs_MsgAmDeltaCompleteResp_t;
typedef struct fs_MsgAmDeltaCompleteResp fs_MsgAmDeltaCompleteResp;

/*
 * Messages sent by Agile Mirror to install snapshot
 */

typedef struct fs_MsgAmDeltaInstallSetupReq {
    fs_MsgReqHdr      hdr;
    evm_volId_t         volId;
    uint64              cookie;     // Unique ID provided by Am
    uint32              snapNum;    // snapshot num
    char                newName[FS_SNAPNAME_MAX];
} fs_MsgAmDeltaInstallSetupReq_t;
typedef struct fs_MsgAmDeltaInstallSetupReq fs_MsgAmDeltaInstallSetupReq;

typedef struct fs_MsgAmDeltaInstallSetupResp {
    fs_MsgRespHdr     hdr;
    uint64              cookie; // Am Session cookie
/**
    fs_Status           status; // FS_ERR_NONE for success
                                // FS_ERR_EXISTS if snapshot exists
**/
} fs_MsgAmDeltaInstallSetupResp_t;
typedef struct fs_MsgAmDeltaInstallSetupResp fs_MsgAmDeltaInstallSetupResp;

typedef struct fs_MsgAmDeltaInstallExtentReq {
    fs_MsgReqHdr      hdr;
    evm_volId_t         volId;
    uint64              cookie; // Am session cookie
    uint64              extentCookie;   // unique id for each set extent
                                        // request
    uint32              numExtents; // number of extent contained in this
                                    // message
    fs_AMExtent_t           extents[FS_Am_MAX_EXTENT_TRANSFER];
} fs_MsgAmDeltaInstallExtentReq_t;
typedef struct fs_MsgAmDeltaInstallExtentReq fs_MsgAmDeltaInstallExtentReq;

typedef struct fs_MsgAmDeltaInstallExtentResp {
    fs_MsgRespHdr     hdr;
    uint64              cookie; // session cookie
    uint64              extentCookie;   // extent cookie returned
/**
    fs_Status           status; // FS_ERR_NONE for success
                                // FS_ERR_IO for i/o error
                                // FS_ERR_GENERIC if updating fs bits failed
                                // FS_ERR_NXIO for devices temporarily not
                                // available
                                // FS_ERR_INVAL if any block in the extent
                                // is already in use
**/
} fs_MsgAmDeltaInstallExtentResp_t;
typedef struct fs_MsgAmDeltaInstallExtentResp fs_MsgAmDeltaInstallExtentResp;

typedef struct fs_MsgAmLastSnapshotReq {
    fs_MsgReqHdr      hdr;
    evm_volId_t         volId;
    uint64              cookie; // identifies Am session
} fs_MsgAmLastSnapshotReq_t;
typedef struct fs_MsgAmLastSnapshotReq fs_MsgAmLastSnapshotReq;

typedef struct fs_MsgAmLastSnapshotResp {
    fs_MsgRespHdr     hdr;
    uint64              cookie; // cookie (?why do we need it)
    char                name[FS_SNAPNAME_MAX];
} fs_MsgAmLastSnapshotResp_t;
typedef struct fs_MsgAmLastSnapshotResp fs_MsgAmLastSnapshotResp;

typedef struct fs_MsgAmDeltaInstallCommitReq {
    fs_MsgReqHdr      hdr;
    evm_volId_t         volId;
    uint64              cookie; // identifies Am session
} fs_MsgAmDeltaInstallCommitReq_t;
typedef struct fs_MsgAmDeltaInstallCommitReq fs_MsgAmDeltaInstallCommitReq;

typedef struct fs_MsgAmDeltaInstallCommitResp {
    fs_MsgRespHdr     hdr;
    uint64              cookie; // identifies Am session
/**
    fs_Status           status; // FS_ERR_NONE for success
                                // FS_ERR_IO i/o errors
                                // FS_ERR_NXIO devices temporarily not avail
                                // FS_ERR_GENERIC if snapshot creation failed
**/
} fs_MsgAmDeltaInstallCommitResp_t;
typedef struct fs_MsgAmDeltaInstallCommitResp fs_MsgAmDeltaInstallCommitResp;

#endif  /* _FS_AM_API_H_ */
