/*
 * Name:  fs-benchmark.c
 *
 * RCS:  $Id: fs-benchmark.c,v 1.4 2002/10/22 01:30:33 jong Exp $
 *
 * Description:  Benchmark I/O Routines
 * 
 * Created By:  Rahul Khona
 *
 * Date Created:  7/22/2002
 */
#include "fs-includes.h"

void
fs_benchmarkIODone(evm_IOStat_t *stat)
{
    fs_BenchMarkIoDesc_t    *desc =
                (fs_BenchMarkIoDesc_t *)(address_t)stat->clientPrivate;
    NFX_SPIN_LOCK(&desc->spin);
    desc->ioCount--;
    if (desc->ioCount == 0) {
        esm_event_t event;
        NFX_SPIN_UNLOCK(&desc->spin);
        event.event = FS_THREAD_EVENT_IO_DONE;
        event.p1.u.p = NULL;
        event.p2.u.p = NULL;
        eee_rtcDuration(&desc->startTime, &desc->duration);

        /*
         * Wake up the i/o issuer
         */

        thread_highPriorityEvent(desc->wakeupHandle, &event);
        return;
    }
    NFX_SPIN_UNLOCK(&desc->spin);
    return;
}

#define MAX_IO  32 // never more than 64
static eeeRTC_t
fs_readBenchMark(fs_Context *context, uint64 off, uint32 len)
{
    int i;
    fs_BenchMarkIoDesc_t    *desc;
    evm_IOReq_t             reqs[MAX_IO];   // large stack
    esm_event_t             wakeupEvent;

    FS_ASSERT(len < MAX_IO);
    desc  = fs_ramAlloc(FS_RAM_BENCHMARK_DESC, sizeof (fs_BenchMarkIoDesc_t));
    FS_ASSERT(desc != NULL);
    desc->ioCount = len;

    /*
     * Build the i/o requests
     */
    for (i = 0; i < len; i++) {
       reqs[i].id = context->volId;
       reqs[i].reqSize = FS_BLOCK_SIZE / EVM_LBLK_SIZE;
       reqs[i].dataType = EVM_META_DATA_REQ;
       reqs[i].reqType = EVM_READ_REQ;
       reqs[i].offset = off / EVM_LBLK_SIZE; 
       reqs[i].clientPrivate = (address_t)desc;
       reqs[i].attributes = 0;
    }

    desc->wakeupHandle = context->smHandle;
    eee_rtc(&desc->startTime);
    for (i = 0; i < len; i++) {
        evm_IOReq(&reqs[i], fs_benchmarkIODone);
    }
    fs_yield(context, &wakeupEvent, FS_YIELD_TIME_IO, FS_YIELD_TYPE_IO_MR);
    fs_ramDealloc(FS_RAM_BENCHMARK_DESC, desc);

    return desc->duration;
}

char    globalBenchmarkBuffer[FS_BLOCK_SIZE];

static eeeRTC_t
fs_writeBenchMark(fs_Context    *context, uint64 off, uint32 len)
{
    int i;
    fs_BenchMarkIoDesc_t    *desc;
    evm_IOReq_t             reqs[MAX_IO];   // large stack
    esm_event_t             wakeupEvent;

    FS_ASSERT(len < MAX_IO);
    desc  = fs_ramAlloc(FS_RAM_BENCHMARK_DESC, sizeof (fs_BenchMarkIoDesc_t));
    FS_ASSERT(desc != NULL);
    desc->ioCount = len;

    /*
     * Build the i/o requests
     */
    for (i = 0; i < len; i++) {
       reqs[i].id = context->volId;
       reqs[i].reqSize = FS_BLOCK_SIZE / EVM_LBLK_SIZE;
       reqs[i].dataType = EVM_META_DATA_REQ;
       reqs[i].reqType = EVM_WRITE_REQ;
       reqs[i].offset = off / EVM_LBLK_SIZE; 
       reqs[i].wDataHandle.dataPtr = globalBenchmarkBuffer;
       reqs[i].clientPrivate = (address_t)desc;
       reqs[i].attributes = 0;
    }

    desc->wakeupHandle = context->smHandle;
    eee_rtc(&desc->startTime);
    for (i = 0; i < len; i++) {
        evm_IOReq(&reqs[i], fs_benchmarkIODone);
    }
    fs_yield(context, &wakeupEvent, FS_YIELD_TIME_IO, FS_YIELD_TYPE_IO_MW);

    fs_ramDealloc(FS_RAM_BENCHMARK_DESC, desc);

	return desc->duration;
}

int32
fs_benchMarkThread(
    fs_Context  *context)
{
    fs_MsgBenchMarkReq    *req = (fs_MsgBenchMarkReq *) context->reqHdr;
    fs_MsgBenchMarkResp   *resp = (fs_MsgBenchMarkResp *) context->respHdr;

    /*
     * If the filesystem is mounted, don't run the benchmark
     */

    if (fs_fileSystemForVolumeLocked(context->volId)) {
        printf("Cannot run benchmark as filesystem is mounted\n");
        return NFX_ERR;
    }

    if (req->read) {
        resp->time = fs_readBenchMark(context, req->off, req->len);
    } else {
		resp->time = fs_writeBenchMark(context, req->off, req->len);
    }
	resp->count_per_usec = count_per_usec;
    resp->hdr.fsStatus = FS_ERR_NONE;
    return 0;

}
