/*
 * Name:  fs-benchmarkDriver.c
 *
 * RCS:  $Id: fs-benchmarkDriver.c,v 1.3 2002/10/22 01:30:33 jong Exp $
 *
 * Description:  Drives the benchmark and has i/o load knowledge
 * 
 * Created By:  Rahul Khona
 *
 * Date Created:  7/22/2002
 */
#include "fs-includes.h"
#include "fs-benchmark.h"
#if defined (SSC) || defined (FS_STANDALONE)
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <exports/anpssc-api.h>
#endif


#define MAX_THREADS 200
#define MAX_LEN 65536
#define IOSIZE  FS_BLOCK_SIZE   // change this to suitable value for non fs
#define IOSHIFT FS_BLOCK_SHIFT  // ---- do -----------
#define MIN_READ_PERCENTAGE 0

/*
 * Globals
 */

/* Declare globals used by eee_rtc calls.  The actual values will be returned
 * as part of the responses.
 */
uint64 count_per_usec = 1;
uint64 count_per_msec = 1;

int nUsers = 1;
int nReadPercentage = 0;
int shmId;
int nIterations = 0;
int nTimeout = 0;
int nReadSize = 0;
int nWriteSize = 0;
char    *volume = NULL;

struct ioStats {
    int offset;
    int len;
    int read;
    eeeRTC_t ioTimes;
};

struct ioStats  *ioStats;
evm_volId_t     volId;

int           pids[MAX_THREADS];
int counter;

unsigned long respBuf;

static void
usage(char *progname)
{
    fprintf(stderr, "%s: volume=name percentRead=count thread=count "
           "iterations=count time=seconds "
           "readsize=size writesize=size\nMax Threads= 200\n"
           "0 <= percentRead <=100\n"
   "readsize if specified will do all the reads of this size (in fs blocks)\n"
"writesize if specified will do all the writes of this size (in fs blocks)\n",
            progname);
}

#define MAX_OPTS    7
char *opts[] = {"volume", "percentRead", "thread", "iterations", "time",
                "readsize", "writesize", 0};

static void
parseOptions(int argc, char *argv[])
{
    char    *c, *t, *endPtr;
    int     index = 1, i;
    int     value;

    while (index < argc) {
        c = strchr(argv[index], '=');
        if (!c) {
            fprintf(stderr, "Invalid Argument %s index %d\n",
                    argv[index], index);
            usage(argv[0]);
            exit(1);
        }
        t = c + 1;

        *c = '\0';
        for (i = 0; i < MAX_OPTS; i++) {
            if (strcmp(argv[index], opts[i]) == 0) {
                switch (i) {

                case 0:
                    volume = t;    
                    break;
                case 1:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nReadPercentage = value;
                    break;
                case 2:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nUsers = value;
                    break;
                case 3:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nIterations = value;
                    break;
                case 4:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nTimeout = value;
                    break;
                case 5:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nReadSize = value;
                    break;
                case 6:
                    value = strtol(t, &endPtr, 10);
                    if (*endPtr != '\0') {
                        fprintf(stderr, "Invalid Argument %s index %d\n",
                                argv[index], index);
                        usage(argv[0]);
                        exit(1);
                    }
                    nWriteSize = value;
                    break;
                }
                break;
            }
        }
        
        index++;
    }
}

#if defined (SSC)
#define EVM_RESP_MAX 1024
static char evmRespBuf[EVM_RESP_MAX];
static int
send_evm_message(void *msg, int msgLen)
{
    struct timeval      timeout = { 10, 0};
    int                 rc;

    rc = sendAgileMsg(  -1,     // system assigned tmp socket
                        msg,
                        msgLen,
                        "evm",
                        NFX_SSC_UNIX,
                        0,  // TODO -replace with correct cm api call for SSC slot
                        MGMTPLANE,
                        &timeout,
                        evmRespBuf,
                        EVM_RESP_MAX);
    if (rc <= 0) {
        perror("sendAgileMessage");
        fprintf(stderr, "Failed to communicate with EVM\n");
        return NFX_ERR;
    }
    return NFX_OK;
}
#endif

static eeeRTC_t
sendRequest(int offset, int len, int read)
{
    fs_MsgBenchMarkReq  req;
    fs_MsgBenchMarkResp  resp;
    int                 rc;
    struct timeval      timeout = { 10, 0};
    char                *r;
    
    bzero(&req, sizeof (req));
    req.hdr.requestType = FS_REQ_BENCH_MARK;
    req.hdr.len = sizeof (req);
    req.volId = volId;
    req.off = offset / FS_BLOCK_SIZE;
    req.len = len;
    req.read = read;
#if defined (FS_STANDALONE)
    fprintf(stderr, "benchmark not support in standalone\n");
    rc =  -1;
#else
    rc = sendAgileMsg(  -1,                 // system assigned tmp socket
                        &req.hdr,
                        req.hdr.len,
                        "efs",
                        //NFX_NFP_FP,         // Either cpu
                        NFX_NFP_FP2,         // LINUX_TEST use this fp now
                        1,                  // TODO - identify correct slot
                        MGMTPLANE,          // Mgmt plane
                        &timeout,
                        &resp,
                        sizeof (fs_MsgBenchMarkResp));
#endif
    if (rc < 0) {
        printf("Cannot send message to fp, rc %d\n", rc);
        resp.time = *(eee_rtcZero());;
    } else {
		/* Set the value of the rtc to usec conversion factor as
		 * returned by the server.
		 */
		count_per_usec = resp.count_per_usec;
		count_per_msec = count_per_usec * MSEC_PER_SEC;

	}
    return resp.time;
}

static void
work()
{
    int randomNumber;
    int read = 0;
    int len;
    srand(getpid());
    randomNumber = rand();
    read = ((MIN_READ_PERCENTAGE +
            (int)(1.0 * 100 * randomNumber / (RAND_MAX + 1.0)))
            <= nReadPercentage);
    len = 1 + (int)(1.0 * MAX_LEN * randomNumber / (RAND_MAX + 1.0));

    /*
     * Convert the request to number of blocks
     */

    len = (len + (IOSIZE - 1)) >> IOSHIFT;

    /*
     * Record the times in our slot
     */

    ioStats[counter].offset = randomNumber;
    if (read) {
        if (nReadSize) {
            len = nReadSize;
        }
    } else {
        if (nWriteSize) {
            len = nWriteSize;
        }
    }
    ioStats[counter].len = len;
    ioStats[counter].read = read;
    ioStats[counter].ioTimes = sendRequest(randomNumber, len, read);
}

static void
printStats()
{
    int i;
    uint64  readBytes = 0, writeBytes = 0;
    eeeRTC_t  readTime, writeTime;  
    uint64  readVariance = 0, writeVariance = 0;
    uint32  readCount = 0, writeCount = 0;
    uint32  readAverage, writeAverage;
    uint32  readMin = 0 , writeMin = 0 , readMax = 0 , writeMax = 0;

	readTime = *(eee_rtcZero());
	writeTime = *(eee_rtcZero());
	printf("Read\t\tOffset\t\tLen\t\tMilliseconds\n");
    for (i = 0; i < counter; i++) {
        printf("%s\t\t%d\t\t%d\t\t%lld\n",
               ioStats[i].read ? "TRUE" : "FALSE",
               ioStats[i].offset, ioStats[i].len,
			   eee_rtcToMS(&ioStats[i].ioTimes));
        if (ioStats[i].read)  {
            readBytes += ioStats[i].len;
            eee_rtcIncr(&readTime, &ioStats[i].ioTimes);
            readCount++;
        } else {
            writeBytes += ioStats[i].len;
            eee_rtcIncr(&writeTime, &ioStats[i].ioTimes);
            writeCount++;
        }
    }

    readAverage = (uint32)((readBytes * FS_BLOCK_SIZE * 1000) /
						   eee_rtcToMS(&readTime));
    writeAverage = (uint32)((writeBytes * FS_BLOCK_SIZE * 1000) /
							eee_rtcToMS(&writeTime));
    printf("\n");
    printf("Average Read Throughput %d Bytes/sec\n", readAverage);
    printf("Average Write Throughput %d Bytes/sec\n", writeAverage);

    for (i = 0; i < counter; i++) {
        uint32 tmp;

		tmp = (uint32)((ioStats[i].len * FS_BLOCK_SIZE * 1000) /
					   eee_rtcToMS(&ioStats[i].ioTimes));
		if (ioStats[i].read)  {
            if (tmp < readMin) {
                readMin = tmp;
            } else if (tmp > readMax) {
                readMax = tmp;
            }
            readVariance += ((readAverage -
              ((ioStats[i].len * FS_BLOCK_SIZE * 1000) / 
			   eee_rtcToMS(&ioStats[i].ioTimes)))^2);
        } else {
            if (tmp < writeMin) {
                writeMin = tmp;
            } else if (tmp > writeMax) {
                writeMax = tmp;
            }
            writeVariance += ((writeAverage -
              ((ioStats[i].len * FS_BLOCK_SIZE * 1000) /
			   eee_rtcToMS(&ioStats[i].ioTimes)))^2);
        }
    }

    printf("\n");
    printf("Read sample size %d - Variance %lld - min %d "
            "Bytes/sec max %d Bytes/sec\n",
            readCount, readVariance/(readCount^2), readMin, readMax);
    printf("Write sample size %d - Variance %lld - min %d Bytes/sec "
            "max %d Bytes/sec\n",
            writeCount, writeVariance/(writeCount^2), writeMin, writeMax);
}

static boolean
is_vol_online(evm_volId_t  volId)
{
#if defined(FS_STANDALONE)
    return TRUE;
#elif defined(SSC)

    evm_MsgVolGetAttrib_t   evmReq;
    evm_MsgVolCreate_t      evmRsp;
    struct timeval waitEvmTo = {10, 0};
    int                     rc;

    /* Reset memory for request and response.
     */
    bzero((void *) &evmReq, sizeof(evmReq));
    bzero((void *) &evmRsp, sizeof(evmRsp));

    /* Fill request.
     */
    evmReq.hdr.msgType = EVM_VOL_GET_ATTRIB;
    evmReq.hdr.clientPrivate = 0;   // not used
    memcpy((void *) &(evmReq.id), (const void *) &(volId), sizeof(evm_volId_t));

    /* Send request to EVM.
     */
    rc = sendAgileMsg(-1,
                      &evmReq, sizeof(evmReq), 
                      "evm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                      &waitEvmTo,
                      &evmRsp, sizeof(evmRsp));
    
	if ((rc <  sizeof(evm_msgHdr_t)) && (rc != EVM_VOL_CONFIG))
		return FALSE;

    fprintf(stderr, "vol 0x%llx is %s\n", volId.whole_thing,
           (evmRsp.config.onLine ? "online" : "offline"));
    
    if (evmRsp.config.onLine)
        return TRUE;
    else
        return FALSE;
#endif
}

static boolean
get_volid(char *id_str, evm_volId_t *volId)
{

#if defined (FS_STANDALONE)
    volId->whole_thing = strtol(id_str, NULL, 10);
    if (volId->whole_thing <= 0 || volId->whole_thing > FS_SIM_VOL_MAX) {
        fprintf(stderr, "volId %lld out of range\n", volId->whole_thing);
        return FALSE;
    }
#elif defined (SSC)
    evm_MsgGetId_t      msg;
    evm_MsgStat_t       *respMsg;
    int                 rc;

    msg.hdr.msgType = EVM_VOL_GET_ID;
    strncpy(msg.name, id_str, VOL_MAX_NAME);

    rc = send_evm_message(&msg, sizeof msg);
    if (rc != NFX_OK) {
        fprintf(stderr, "Failed to obtain volume ID\n");
        return FALSE;
    }
    respMsg = (evm_MsgStat_t *) evmRespBuf;
    if (respMsg->stat != NFX_OK)
    {
        fprintf(stderr, "EVM returned error status (%d)\n", respMsg->stat);
        return FALSE;
    }
    *volId = respMsg->payload.lVolId;

    return TRUE;
#endif
    return FALSE;
}

#if defined(SSC)
int
main(int argc, char *argv[])
{
    int pid;
    int doIterations = 0;
    time_t  start;

    parseOptions(argc, argv);
    if (volume == NULL) {
        fprintf(stderr, "Volume is NULL\n");
        usage(argv[0]);
        exit(1);
    }

    if (!get_volid(volume, &volId)) {
        fprintf(stderr, "Invalid volume %s\n", volume);
        usage(argv[0]);
        exit(2);
    }

    if (!is_vol_online(volId)) {
        fprintf(stderr, "Volume Name %s is not online\n", argv[1]);
        usage(argv[0]);
        exit(2);
    }

    if (nUsers > MAX_THREADS || nUsers < 0) {
        fprintf(stderr, "Invalid Number of threads\n");
        usage("benchmark");
        exit(2);
    }

    if (nReadPercentage < 0 || nReadPercentage > 100) {
        fprintf(stderr, "Invalid Read percentage\n");
        usage("benchmark");
        exit(2);
    }

    if (nIterations <= 0 && nTimeout <= 0) {
        fprintf(stderr, "Specify either iterations or  timeout 1\n");
        usage("benchmark");
        exit(2);
    }

    if (nIterations > 0 && nTimeout > 0) {
        fprintf(stderr, "Specify either iterations or  timeout 2\n");
        usage("benchmark");
        exit(2);
    }

    if (nReadSize < 0) {
        fprintf(stderr, "Read Size cannot be negative\n");
        usage("benchmark");
        exit(2);
    }
    if (nWriteSize < 0) {
        fprintf(stderr, "Write Size cannot be negative\n");
        usage("benchmark");
        exit(2);
    }

    if (nIterations) {
        doIterations = 1;
    }

    start = time(NULL);
    if (start < 0) {
        perror("time returned failure:");
        exit(6);
    }
more:
    /*
     * Create shared memory segment which would be used to keep
     * track of results from all processes
     */

    shmId = shmget(IPC_PRIVATE, sizeof (struct ioStats) * MAX_THREADS,
                    IPC_CREAT | 0777);
    if (shmId == -1) {
        perror("cannot get shared memory");
        exit(1);
    }
    ioStats = (struct ioStats *)shmat(shmId, 0, 0);
    if (ioStats == NULL) {
        exit(3);
    }

    /*
     * For the requested number of threads, fork off a process, which
     * does the real work
     */

    for (counter = 0; counter < nUsers; counter++) {
        pid = fork();
        if (pid != 0) {
            pids[counter] = pid;
            continue;
        } else {
            work();
            break;
        }
    }

    /*
     * If we are the master thread, wait for all the processes to
     * complete and then print the statistics
     */

    if (pid != 0) {
        int status;
        int i;

        /*
         * Wait for all the children
         */
        for (i = 0; i < counter; i++) {
            wait(&status);
        }
        printStats();
        shmdt(ioStats);
        if (doIterations) {
            nIterations--;
            if (nIterations > 0) {
                goto more;
            }
        } else {
            time_t  t = time(NULL);
            if (t < 0) {
                perror("time returned failure:");
                exit(6);
            }
            if ((t - start) <= nTimeout) {
                shmdt(ioStats);
                goto more;
            }
        }
    } else {

        /*
         * Detach the inherited shared memory segment
         */

        shmdt(ioStats);
    }
    exit(0);
    
}
#endif
