/*
 * Name		: fs-dump.h
 * RCS		: $ID$
 * Description	: Dump Implementation
 * Created by	: Rahul Khona
 * Date Created	: 7/11/2001
 */

#ifndef _FS_DUMP_H_
#define _FS_DUMP_H_
#include "fs-dumprestore.h"
#include <exports/ndmp-fs-api.h>

#if 0
typedef struct fs_DumpPathNameList {
    DLL_ENTRY(fs_DumpPathNameList)  link;
    char     buf[FS_BLOCK_SIZE];
} fs_DumpPathNameList_t;
#endif
/*
 * Structures to hold Renamed inodes incore
 */

#define FS_DUMP_INO_RENAMED 1   // dir inode was renamed
#define FS_DUMP_INO_CREATED 2   // For links when inode is create first time
#define FS_DUMP_INO_LINKED  4   // inode is linked
#define FS_DUMP_INO_TEMP    8   // inode is created in tmp dir

typedef struct fs_dumpRenameLink   fs_dumpRenameLink_t;
struct fs_dumpRenameLink {
    fs_Inumber      inum;    
    uint32          flags;
    uint32          pathLen;
    DLL_ENTRY(fs_dumpRenameLink_t)  link;
    char            path[1];    // used to record creation of inode
                                // for links only
//    char            *path;      // used to record creation of inode
//                                // for links only
};

#define FS_DUMP_FILEHIST_FILE   1
#define FS_DUMP_FILEHIST_DIR    2
#define FS_DUMP_FILEHIST_NODE   3

typedef struct fs_DumpNameList fs_DumpNameList_t;
struct fs_DumpNameList {
    DLL_ENTRY(fs_DumpNameList_t)    link;
    char                            *name;
};
#define FS_MAXPATHLEN FS_PATH_MAX

/*
 * Tape i/o Queue
 */

typedef struct fs_DumpTapeIoQueue fs_DumpTapeIoQueue_t;
struct fs_DumpTapeIoQueue {
        DLL_ENTRY(fs_DumpTapeIoQueue_t) link;
        DLL_ENTRY(fs_DumpTapeIoQueue_t) pendingIoLink;
        uint64                      ioSequence;
        char                        *ioBuf;
};

/*
 * Maintain a stack of directory inodes that we have been processing
 * while walking the tree
 */

/*
 * Worst case is that we have a deep tree with 1 byte entries, which
 * would mean we have 2 bytes per name in the path including /.
 */

#define FS_DUMP_MAX_DIR_DEPTH   FS_MAXPATHLEN / 2
typedef struct fs_inodeAttr {
    fs_Inumber      inum;
    uint8           doneHistory;      
    int64           block;
    int16           boff;
} fs_inodeAttr_t;

/*
 * Dump Context
 */

#define FS_DUMP_MAX_INUM_ARRAY  256
#define FS_DUMP_KILL_INUM   (fs_Inumber)0
#define FS_DUMP_TAPE_IO_BUFFERS 30
#define FS_HISTORY_BUF_SIZE 8192

#ifdef FS_HIST_WAIT
struct fs_DumpSession;
#endif

typedef DLL_HDR(fs_dumpRenameLink_t) fs_dumpRenameLinkDLL;
typedef struct fs_DumpContext   fs_DumpContext_t;
struct fs_DumpContext {
    DLL_ENTRY(fs_DumpContext_t) dumpList;   // hangs of fileSystem
    fs_DumpHeader_t  record;
    uint32           cookie;                // ndmp cookie
    uint32           ndmpSessId;            // ndmp session id
    uint32          ndmp_ssc_src_port;
    uint64          currentId;
    int32           snapnum;
    int             doFileHistory;
    int32           prevSnapNum;
    fs_Time         prevDumpTime;
    fs_Time         dumpTime;
    int             files;
    char            *label;
    char            *prevSnapName;
    char            *snapName;
    int             level;
    char            buf[FS_DR_TBSIZE];
    char            namebuf[2 * FS_MAXPATHLEN + 2];
    char            currentName[FS_MAXPATHLEN + 1];
    char            *cwd;                   //current working dir (partial dump)
    uint64          partialTreeRootInum;    //root of partial tree inode number
    int             recursiveDescent;       //recursive descent specified or
    fs_dumpRenameLinkDLL renames[1023];
    fs_dumpRenameLinkDLL links[127];
    DLL_HDR(fs_DumpNameList_t)  nameList;
    esm_handle_t    smHandle;               // handle of thread doing
                                            // dump
    int             abort;                  // abort dump

    fs_inodeAttr_t  parentStack[FS_DUMP_MAX_DIR_DEPTH];
    int32           stackIndex;
	uint8			pendingIo;
    uint8           activeThreads;
	nfx_spinlock_t	spinLock;
    uint64          nodeInum;
    uint64          nameInum;
    fs_Inumber      inumArray[FS_DUMP_MAX_INUM_ARRAY];
    volatile int    head;
    volatile int    tail;
    int             killedThreads;  // number of threads killed
    fs_Lock         tapeLock;
    fs_Status       threadStatus;
    esm_handle_t    tapeWaiter; // thread waiting for tape resources
    esm_handle_t    workerThread; // worker thread
    esm_handle_t    workWaiter;     // worker thread
	uint64			ioSequence;
	uint64			neSequence;
    uint32          ioSize;
    DLL_HDR(fs_DumpTapeIoQueue_t)   freeList;
    DLL_HDR(fs_DumpTapeIoQueue_t)   ioList;
    DLL_HDR(fs_DumpTapeIoQueue_t)   pendingIoList;
    fs_DumpTapeIoQueue_t queueHdrs[FS_DUMP_TAPE_IO_BUFFERS];    // preallocated
                                                                // hdrs
    fs_DumpTapeIoQueue_t    *currentBuffer; // current io buffer
    uint32                  currOffset;     // current offset in the buffer
    int                     needBuffer;
    fs_Status               ioStatus;       // io status returned by async io
    uint32                  dirCount;
    uint32                  nodeCount;      
    fs_DirHistory_t         *curDirHist;    // current dir history pointer
    fs_NodeHistory_t        *curNodeHist;    // current node history pointer
    char                    *historyBuf;    // For File and Dir Histories
#ifdef FS_HIST_WAIT
    int                     histWait;       // wait for availspace to send
                                            // filehistories
    struct fs_DumpSession   *session;       // back pointer to session
    fs_Lock                 histLock;       // lock to write to history buffer
    esm_handle_t            histWaiter;     // thread waiting for flow control
#endif

#ifdef FS_DEBUG
    /*
     * Some statistics
     */
    uint64                  totTapeIo;
    uint64                  totTapeWait;
    int                     maxTapeIosInTransit;                  
    int                     totTapeQueueLen;
    uint64                  totTapeWaitTime;
    eeeRTC_t                startTime;
    uint64                  objectsProcessed;
    uint64                  workerThreadsPaused;
    uint64                  objectsProcessedByThreads;
#endif

#ifdef FS_STANDALONE
    int             fd;                     // file descriptor where dump
                                            // output is written
#endif  /* FS_STANDALONE */

};


//Structure for non recursive dumpTree function
typedef struct fs_DumpTreeContext	fs_DumpTreeContext_t;
struct fs_DumpTreeContext	{
	DLL_ENTRY(fs_DumpTreeContext_t)	dumpTreeList;//Attached to dumpTree function
    char                *name;
    int                 nameLen;
    int                 isFset;
    fs_DirEntry         *dirp;
	fs_Inode			*inode;
    int                 eof;
    int                 numEntries;
    uint64              cookie;
    fs_Inumber          inum;
    int                 i;
    char                *cp;
    char                buf[FS_MAXPATHLEN];
	uint64				nodeId;
};

typedef struct fs_fullDumpFileHistoryContext {
        fs_DumpContext_t    *dumpContext;
        uint64              startInum;
        uint64              endInum;
} fs_fullDumpFileHistoryContext_t;

#ifdef FS_HIST_WAIT
typedef struct fs_DumpSession fs_DumpSession_t;

struct fs_DumpSession {
        DLL_ENTRY(fs_DumpSession_t)    link;
        fs_DumpContext_t                    *dumpContext;
        uint32                              charSpace;
        uint32                              mbufSpace;
};

typedef struct fs_DumpSessionsHdr
{                                               
    fs_DumpSession_t *head;                               
    fs_DumpSession_t *tail;                               
} fs_DumpSessionsHdr_t;

extern fs_DumpSessionsHdr_t fs_dumpSessionsHdr;
extern nfx_spinlock_t       fs_dumpSessionsSpinLock;
#endif  // FS_HIST_WAIT


/*
 * Prototypes
 */

extern fs_Status fs_writeStartRecord(fs_Context *, fs_DumpContext_t *, uint8,
                                        uint8, fs_Time, fs_Time);
extern fs_Status fs_treeFullDump(fs_Context *, fs_DumpContext_t *);
extern fs_Status fs_dump(fs_Context *, fs_DumpContext_t *);
extern void fs_markInodeCreated(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                  char *, int, int);
extern int fs_isInodeCreated(fs_Context *, fs_DumpContext_t *, fs_Inode *);
extern char * fs_getLinkPath(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                int *);
extern void fs_markInodeRenamed(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                   int);
extern int fs_isInodeRenamed(fs_Context *, fs_DumpContext_t *, fs_Inode *);
extern int fs_isInodeTemp(fs_Context *, fs_DumpContext_t *, fs_Inode *);
extern int fs_isInodePathTemp(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                              char *);
extern fs_Status fs_incrementalDump(fs_Context *, fs_DumpContext_t *);
extern fs_Status fs_writeTape(fs_Context *, fs_DumpContext_t *);
extern fs_Status fs_writeDataTape(fs_Context *, fs_DumpContext_t *, char *);
extern fs_Status fs_dumpBlocks(fs_Context *context, fs_DumpContext_t *,
                               fs_Inode *, uint64, fs_DumpExtent_t *, uint32);
extern void fs_cleanDumpContext(fs_Context *, fs_DumpContext_t *, boolean);
extern int32 fs_fsstatThread(fs_Context *);
extern void fs_markLinkCreated(fs_Context *context,
                               fs_DumpContext_t *dumpContext,
                               fs_Inode *inode, char *path, int pathLen,
                               int isTmp);
extern int fs_isInodeLinked(fs_Context *context,
                            fs_DumpContext_t *dumpContext,
                            fs_Inode *inode, char *path);
extern void fs_fullDumpNodeFileHistoryThread(esm_handle_t smHandle,
                                             void *arg);
extern void fs_objectFullDumpThread(esm_handle_t handle, void *arg);
#endif  /* _FS_DUMP_H_ */
