// -*- tab-width: 8; c-basic-offset: 4; indent-tabs-mode: nil -*-
// ex: set tabstop=8 shiftwidth=4 expandtab:

/*
 * Name:  fs-mkfs.h
 *
 * RCS:  $Id: fs-mkfs.h,v 1.8 2002/06/06 21:02:51 bob Exp $
 *
 * Description:  Makefs low level routines exported for igrow, growfs
 *
 * Created By:  Byron Rakitzis <byron@agilestorage.com>
 *
 * Date Created:  2001-12-05
 */

#ifndef FS_MKFS_H
#define FS_MKFS_H

void fs_growInodeFile(fs_Context *context,
         fs_Parameters *oldParams, fs_Parameters *params);
void fs_mkfsZeroFile(
    fs_Context    *context,
    fs_WriteQueue *queue,
    fs_Inode      *inode,
    uint64	  startPage);


/*
 * Get the size of the log as a function of the total number of FS blocks
 * and the type of the log device.
 */
void
fs_mkfsGetLogSize(uint64 *logSize, uint64 *numBlocks, uint32 logDevType);

/**
 * Create the initial superblocks.
 */
void
fs_mkfsInitializeSuperblock(
    uint64          totalBlocks,
    fs_SuperBlock   *super,
    int32           blockSize);

/**
 * Write the superblocks to disk synchronously, non-transactionally.
 */
void
fs_writeSuperblocksSync(
    fs_Context      *context,
    fs_SuperBlock   *super);


/*
 * Write the superblocks to disk asynchronously and transactionally.
 */
void
fs_writeSuperblocksTrans(
    fs_Context      *context,
    fs_SuperBlock   *super);

#endif // FS_MKFS_H
