
#ifndef _FS_NDMP_API_H_
#define _FS_NDMP_API_H_


/*
 * Data type definations for various dump and restore related messages
 */

typedef struct fs_ndmp_DumpSetup {
    uint32              ndmpSessId;             // ndmp session id
    uint32              ndmp_ssc_src_port;
    int                 recursiveDescent;       // descent directories
    int                 level;                  // dump level 0 for full
    int                 fileHistory;            // file history to be provided
                                                // or not

    /*
     * Following fields are ignored for full backup
     */

    uint32              prevSnapTime;           // previous snapTime 
    uint32              cwdLen;                 // length of cwd
    uint32              labelLen;               // length of label
    uint32              prevSnapNameLen;        // length of snapshot name
    uint32              recSize;                // io size
    char                blob[1];                // name of label,
                                                // previous snapshot, etc
} fs_ndmp_DumpSetup_t;

typedef struct fs_ndmp_DumpNames {
    uint32              ndmpSessId;             // ndmp session id
    uint32              len;                    // len of msgBlock
    char                msgBlock[1];            // null terminated list of
                                                // path names
} fs_ndmp_DumpNames_t;

typedef struct fs_ndmp_RestoreNames {
    uint32              ndmpSessId;             // ndmp session id
    uint32              oldLen;                 // len of old name
    uint32              newLen;                 // len of new name
    char                msgBlock[1];            // null terminated list of
                                                // path names
} fs_ndmp_RestoreNames_t;

typedef struct fs_ndmp_Start {
    uint32              ndmpSessId;             // ndmp session id
} fs_ndmp_Start_t;

typedef struct fs_ndmp_Abort {
    uint32              ndmpSessId;             // ndmp session id
} fs_ndmp_Abort_t;

typedef struct fs_ndmp_RestoreSetup {
    uint32              ndmpSessId;             // ndmp session id
    uint32              ioSize;                 // data read from tape
    uint64              numBytes;                 // data read from tape
    int                 recursiveDescent;       // restore descents as well
    uint32              cwdLen;                 // length of cwd
    char                cwd[1];                 // current working dir
} fs_ndmp_RestoreSetup_t;

typedef struct fs_ndmp_IODone {
    uint32              ndmpSessId;             // ndmp cookie
    void                *fsCookie;              // filesystem cookie
    fs_Status           status;                 // io status
    uint32              dataLen;                // len of data if data returned
    uint32	            data_offset;            // offset from data ptr to get real data
	uint64				sequence;				// io sequence
    uint32	            spare;                	
    char                *data;                	// data
} fs_ndmp_IODone_t;

/*
 * Message definations
 */

typedef struct fs_MsgNdmpResp {
    int32	            requestType;        	/* Message Id */
    uint32              clientCookie;
    fs_Status           fsStatus;
} fs_MsgNdmpResp_t;


typedef struct fs_MsgSetupDumpReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_DUMP_SETUP
    evm_volId_t         volId;          
    fs_ndmp_DumpSetup_t dump;
} fs_MsgSetupDumpReq_t;

typedef fs_MsgSetupDumpReq_t    fs_MsgDumpSetupReq_t;

typedef fs_MsgNdmpResp_t         fs_MsgDumpSetupResp_t;

typedef struct fs_MsgDumpNamesReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_DUMP_NAMES
    evm_volId_t         volId;          
    fs_ndmp_DumpNames_t names;
} fs_MsgDumpNamesReq_t;

typedef fs_MsgNdmpResp_t         fs_MsgDumpNamesResp_t;

typedef struct fs_MsgStartDumpReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_DUMP_START
    evm_volId_t         volId;          
    fs_ndmp_Start_t     start;
} fs_MsgStartDumpReq_t;

typedef fs_MsgStartDumpReq_t    fs_MsgDumpStartReq_t;
typedef fs_MsgNdmpResp_t         fs_MsgDumpStartResp_t;

typedef struct fs_MsgAbortDumpReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_DUMP_ABORT
    evm_volId_t         volId;          
    fs_ndmp_Abort_t     abort;
} fs_MsgAbortDumpReq_t;
typedef fs_MsgAbortDumpReq_t    fs_MsgDumpAbortReq_t;
typedef fs_MsgNdmpResp_t         fs_MsgDumpAbortResp_t;

typedef struct fs_MsgSetupRestoreReq {
    fs_MsgReqHdr            hdr;        // Msg of type FS_MSG_REST_SETUP
    evm_volId_t             volId;          
    fs_ndmp_RestoreSetup_t restore;
} fs_MsgSetupRestoreReq_t;
typedef fs_MsgSetupRestoreReq_t    fs_MsgRestoreSetupReq_t;
typedef fs_MsgNdmpResp_t         fs_MsgRestoreSetupResp_t;

typedef struct fs_MsgRestoreNamesReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_REST_NAMES
    evm_volId_t             volId;          
    fs_ndmp_RestoreNames_t names;
} fs_MsgRestoreNamesReq_t;

typedef fs_MsgNdmpResp_t         fs_MsgRestoreNamesResp_t;

typedef struct fs_MsgStartRestoreReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_RESTORE_START
    evm_volId_t         volId;          
    fs_ndmp_Start_t     start;
} fs_MsgStartRestoreReq_t;
typedef fs_MsgStartRestoreReq_t    fs_MsgRestoreStartReq_t;
typedef fs_MsgNdmpResp_t            fs_MsgRestoreStartResp_t;


typedef struct fs_MsgAbortRestoreReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_RESTORE_ABORT
    evm_volId_t         volId;          
    fs_ndmp_Abort_t     abort;
} fs_MsgAbortRestoreReq_t;
typedef fs_MsgAbortRestoreReq_t    fs_MsgRestoreAbortReq_t;
typedef fs_MsgNdmpResp_t            fs_MsgRestoreAbortResp_t;

typedef struct fs_MsgDumpIODoneReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_DUMP_IODONE
    evm_volId_t         volId;          
    fs_ndmp_IODone_t    iodone;
} fs_MsgDumpIODoneReq_t;
typedef fs_MsgDumpIODoneReq_t    fs_MsgDumpIodoneReq_t;


typedef struct fs_MsgRestoreIODoneReq {
    fs_MsgReqHdr        hdr;            // Msg of type FS_MSG_RESTORE_IODONE
    evm_volId_t         volId;          
    fs_ndmp_IODone_t   iodone;
} fs_MsgRestoreIODoneReq_t;
typedef fs_MsgRestoreIODoneReq_t    fs_MsgRestoreIodoneReq_t;

#define FS_DUMP_MAX_SNAP_LEN    64

#ifdef FS_HIST_WAIT
#define FS_WILL_HIST_MESSAGE_FIT(dumpContext, size)                         \
        (((size) + 20 < (dumpContext)->session->charSpace) &&                 \
         ((((size) / 1024 + 1) * 2048) < (dumpContext)->session->mbufSpace))
#define FS_USE_HIST_MESSAGE(dumpContext, size)                              \
do {                                                                        \
        (dumpContext)->session->charSpace -= ((size + 20));                 \
        (dumpContext)->session->mbufSpace -= ((((size)/1024 + 1) * 2048));   \
} while (0)
#endif

#endif  /* _FS_NDM_API_H_*/
