/*
 * Name		: fs-restore.c
 * RCS		: $Id: fs-restore.c,v 1.89 2002/12/05 21:08:57 bob Exp $
 * Description	: Restore implementation
 * Created by	: Rahul Khona
 * Date Created	: 8/10/2001
 */
#if !defined(SSC)

#include "fs-includes.h"
#include "fs-restore.h"        // private declarations
#include "fs-ndmp-api.h"
#include <exports/ndmp-fs-api.h>

#ifndef FS_STANDALONE
#ifdef LINUX_TEST
#include <errno.h>
extern char *strstr(char *, char *);
#endif
#endif

#define D   printf
PRIVATE fs_Status fs_restoreCreateCwd(fs_Context *, fs_RestoreContext_t *);

PRIVATE fs_Status fs_restoreFullTree(fs_Context *, fs_RestoreContext_t *,
                                     fs_Inode **, char *, fs_DRAttr_t *);
PRIVATE fs_Status fs_restoreMkDir(fs_Context *, fs_RestoreContext_t*,
                                  fs_Inode **, char *, int, fs_Inode **);
PRIVATE fs_Status fs_restoreSetAttr(fs_Context *, fs_RestoreContext_t *,
                                    fs_Inode **, fs_DRAttr_t *);
PRIVATE fs_Status fs_restoreMkNode(fs_Context *, fs_RestoreContext_t*,
                                   fs_Inode **, char *, int, fs_DRAttr_t *,
                                   int , fs_Inode **);
PRIVATE fs_Status fs_restoreMkFset(fs_Context *, fs_RestoreContext_t *,
                                   fs_Inode **, char *, int, fs_Inode **);
PRIVATE fs_Status fs_restoreMkSyml(fs_Context *, fs_RestoreContext_t *,
                                   fs_Inode **, char *, int, fs_DRAttr_t *,
                                   fs_Inode            **);
PRIVATE fs_Status fs_restoreMkLink(fs_Context *, fs_RestoreContext_t *,
                                   fs_Inode **, char *, int, fs_DRAttr_t *,
                                   fs_Inode            **);
PRIVATE fs_Status fs_restoreMkFile(fs_Context *, fs_RestoreContext_t *,
                                   fs_Inode **, char *, int, fs_DRAttr_t *,
                                   fs_Inode            **);
PRIVATE fs_Status fs_restoreWrites(fs_Context *, fs_RestoreContext_t *,
                                   fs_Inode **, fs_Inode **);
PRIVATE fs_Status fs_drainDir(fs_Context *, fs_RestoreContext_t *, uint64);
PRIVATE fs_Inode * fs_parsePath(fs_Context *, fs_RestoreContext_t *, char *);
PRIVATE fs_Status fs_drainObject(fs_Context *, fs_RestoreContext_t *, uint64);

/*
 * Perform the full restore. This works only on level 0 dump. If the
 * dump is any other level, then an error should be returned
 */

/*
 * Perform the full restore. This works only on level 0 dump. If the
 * dump is any other level, then an error should be returned
 */

EXPORTED void
fs_yieldTemp(fs_Context	*context)
{
#if 0
    esm_event_t     event;
	int				rc;
    event.event  = FS_THREAD_EVENT_YIELD_WAKE;
    event.p2.u.p = NULL;
    rc = thread_event(context->smHandle, &event);
    if (rc != NFX_OK)
    {
        fs_error(context, FS_EEE_ERR, "fs_yieldTemp");
    }

    fs_yield(context, &event, FS_YIELD_TIME_ADMIN, FS_YIELD_TYPE_ADMIN);
    if (event.event != FS_THREAD_EVENT_YIELD_WAKE) {
        fs_error(context, FS_INTERNAL_ERR,
                 "fs_yieldTemp: event %d", event.event);
    }
#else
    fs_sleep(context, 0);
#endif
}

/*
 * Periodically yield the processor so that we don't hog the cpu.
 * If we have been consuming the CPU for more than a milli second, we will
 * give it back
 */

#define FS_DUMP_RESTORE_MAX_CPU 100
EXPORTED void
fs_yieldPeriodic(fs_Context *context)
{
	static boolean	first = TRUE;
	static eeeRTC_t	yieldTime;
    eeeRTC_t		runTime;

	if (first) {
		first = FALSE;
		eee_msToRTC(FS_DUMP_RESTORE_MAX_CPU, &yieldTime);
	}
	eee_rtcDuration(&context->tmpTime, &runTime);
    if (eee_rtcCompare(&runTime, &yieldTime) > 0) {
        fs_yieldTemp(context);
        eee_rtc(&context->tmpTime);
    }
    return;
}

/*
 * Check if log is not close to full
 * If yes, wake inode and buf daemons to empty it
 */  
EXPORTED inline void
fs_makeSureLogIsNotFull(fs_Context *context)
{
    fs_LogFsRoot *logRoot = context->fileSystem->logRoot;
    
    if (logRoot->currLen > 
        (FS_LOG_LOOP_BYTES(context->fileSystem->logSize) 
         * fs_incoreTune->logCommitHiWtrMk /100))
    {
        
        // Wakeup the inode cleaner threads.
        fs_inodeSignalCleaner(context, FS_INODE_CLEANER_LOG);

        // Wakeup the meta buffer writeback daemon.
        fs_bufSignalWriteback(context, FS_BUFLIST_META, FS_WB_WAKEUP_LOG);

        do {

            fs_sleep (context, 0);

        } while (logRoot->currLen > 
                 (FS_LOG_LOOP_BYTES(context->fileSystem->logSize) 
                  * fs_incoreTune->logCommitHiWtrMk /100));

    } 
}

/*
 * drop the modify lock, yield periodically to prevent from resource overrun
 * if grow is needed, sleep to let it happen
 */
EXPORTED inline void
fs_restoreSleepIfNeeded(fs_Context *context)
{
    //Drop the modify lock so that any thread wishing to get it exclusive gets
    //it
    fs_unlockShared(context, &context->fileSystem->accessLock);            
    context->flags &= ~FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
    fs_unlockShared(context, &context->fileSystem->modifyLock);            
    context->flags &= ~FS_CONTEXT_MODIFY_LOCK_ACQUIRED;

    if (fs_growIfNeeded(context)
     || !fs_haveMininodesAvail(context->fileSystem))
    {
        fs_sleep (context, 0);
    } else {
        fs_yieldPeriodic(context);
    }
    
    //Get it again
    fs_lockShared(context, &context->fileSystem->modifyLock, NULL);
    context->flags |= FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
    fs_lockShared(context, &context->fileSystem->accessLock, NULL);
    context->flags |= FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
}

        


EXPORTED fs_Status
fs_fullRestore(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext
)
{
    fs_Status               status;
    fs_DumpStartHeader_t    *start;
    fs_DumpOperation_t      *operation;
    fs_Inode                *root;
    fs_DRAttr_t             *attr;
    uint32                  len;
    char                    *cp, *np;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_fullrestore"));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Read tape in fs_fullrestore 1"));
    start = &restoreContext->record.u.start;

    /*
     * Writes log messages if required
     */


    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("checking start"));
    if (start->level != 0) {
        fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               FS_ERR_NONE, NULL, "Dump level not 0\n");
        return FS_ERR_INVAL;
    }

    /* TODO ????? REMOVE THIS ?????
     * For a full restore, root directory should be empty
     */

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("getting cwd root"));
    root = I_GET_EXCL(0, restoreContext->cwdInum);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("got cwd root"));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("reading tape again"));

    /* TODO ????? CORRECT THIS
     * Now that root is empty, we can start reading the tape restore.
     * We have to ensure that the first record on tape after start record
     * is Dir start for root, otherwise we cannot do a full a full restore
     */

    status = fs_readTape(context, restoreContext);
    if (status != FS_ERR_NONE) {
        fs_inodeReleaseExclusive(context, root);
        return status;
    }
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Read tape in fs_fullrestore 2"));

    operation = &restoreContext->record.u.operation;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("operation is %d",
             operation->header.op));
    attr = fs_ramAlloc(FS_RAM_RESTORE_ATTR, sizeof (fs_DRAttr_t));
    memcpy(attr, &operation->header.attr, sizeof (fs_DRAttr_t));

    len = operation->len;
    cp = &restoreContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    FS_ASSERT(strcmp(cp, "/") == 0);
    restoreContext->currentName[0] = '\0';
    np = restoreContext->currentName;
    //TODO ????? Revisit this. Really necessary ?
    if (strcmp(cp, "/") != 0) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("root not specified"));
        strcpy(restoreContext->currentName, cp);
        np = restoreContext->currentName + strlen(cp);
    }
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("calling fs_restoreFullTree 1"));
    status = fs_restoreFullTree(context, restoreContext, &root, np, attr);
    fs_ramDealloc(FS_RAM_RESTORE_ATTR, attr);
    fs_inodeReleaseExclusiveAll(context, root);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("returning from fullRestore"));
	//printf("Status after restore full tree %d\n", status);
    return status;
}


PRIVATE fs_Status
fs_restoreFullTree(
    fs_Context              *context,
    fs_RestoreContext_t     *restoreContext,
    fs_Inode                **sroot,
    char                    *sbuf,		//start root, buf & dirAttr
    fs_DRAttr_t             *sdirAttr
)
{
	DLL_HDR(fs_restoreFullTreeContext_t)	restoreFullTreeContexts;
	fs_restoreFullTreeContext_t	*rtCtx, *rtCtx2;
		
    fs_Status               status;
    fs_DumpOperation_t      *operation;
    uint32                  len;
    char                    *cp, *np;
    int                     end = 0;
    int                     ignore, objectIgnore;
    fs_Inumber              rootInum;
	int						nodeType;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreFullTree"));
	
	//Initialize the restore full tree contexts list
	DLL_INIT(&restoreFullTreeContexts);

	//Create initial context
	rtCtx = fs_ramAlloc(FS_RAM_RESTORE_DIRENTS, 
						sizeof(fs_restoreFullTreeContext_t));
	bzero(rtCtx, sizeof(fs_restoreFullTreeContext_t));
	rtCtx->root		= sroot;
	rtCtx->buf		= sbuf;
	rtCtx->dirAttr	= sdirAttr;
	
    do {
     	//If have been using the cpu since a long time, give others a chance
	    fs_yieldPeriodic(context);
	
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			goto goUp;
        }
        operation = &restoreContext->record.u.operation;
        memcpy(&rtCtx->attr, &operation->header.attr, sizeof (fs_DRAttr_t));
        cp = &restoreContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
        len = operation->len;

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("restorefull tree (i%llu) op is %d",
                 (*(rtCtx->root))->inum, operation->header.op));
        switch (operation->header.op) {

        case FS_DR_CREATE_DIR: case FS_DR_CREATE_FSET:

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Restoring dir/fset"));
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("cp len=%d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            ignore = fs_shouldIgnore(context, restoreContext,
                                     restoreContext->currentName, 
                                     np - restoreContext->currentName, NULL);
            if (ignore) {
                status = fs_drainDir(context, restoreContext,
                                      operation->header.headerId);
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Ignored directory"));
                break;
            }
            if (fs_shouldExtract(context, restoreContext,
                                 restoreContext->currentName,
                                 np - restoreContext->currentName) != 0) {
                if (operation->header.op == FS_DR_CREATE_DIR)   {
                    status = fs_restoreMkDir(context, restoreContext, 
                                       rtCtx->root, cp, len, &(rtCtx->inode));
                } else {
                    status = fs_restoreMkFset(context, restoreContext, 
                                       rtCtx->root, cp, len, &(rtCtx->inode));
                }
            } else {
                //There are files to be restored in this directory, but the
                //directory does not need to be restored, skip it
                rtCtx->inode = *rtCtx->root;
            }
            if (status != FS_ERR_NONE) {
                break;
            }
            
            FS_ASSERT(rtCtx->inode != NULL);
            FS_ASSERT(fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            rtCtx->rootInum = (*(rtCtx->root))->inum;
			rtCtx2 = fs_ramAlloc(FS_RAM_RESTORE_DIRENTS, 
								sizeof(fs_restoreFullTreeContext_t));
			bzero(rtCtx2, sizeof(fs_restoreFullTreeContext_t));
			rtCtx2->root	= &(rtCtx->inode);
    		rtCtx2->buf		= np;
	        if (rtCtx->inode != *rtCtx->root)    {
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
    					("releasing root %lld", rtCtx->rootInum));
                fs_inodeReleaseExclusiveAll(context, *(rtCtx->root));
    			rtCtx2->dirAttr = &rtCtx->attr;
            } else {
                rtCtx2->dirAttr = NULL;
            }
			DLL_INSERT_TAIL(&restoreFullTreeContexts, rtCtx, list);
			FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("going down..."));
			rtCtx = rtCtx2;		
			//We are now in the context of the new dir
			break;

        case FS_DR_DIR_END:
			if (rtCtx->dirAttr != NULL) {
	            status = fs_restoreSetAttr(context, restoreContext, 
											rtCtx->root, rtCtx->dirAttr);
			}
goUp:
			rtCtx2 = rtCtx;
			//Retrieve upper context
			DLL_REMOVE_TAIL(&restoreFullTreeContexts, rtCtx, list);
			if (rtCtx != NULL && rtCtx != DLL_NIL) {
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("going up..."));
				rootInum = rtCtx->rootInum;
				if (rootInum != (*(rtCtx2->root))->inum) 	{
					fs_inodeReleaseExclusiveAll(context, *(rtCtx2->root));
					*(rtCtx->root) = I_GET_EXCL(0, rootInum);
					FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
								("acquiring root %lld", rootInum));
				}					
			} else {
				end++;
			}
			//Free old context
			fs_ramDealloc(FS_RAM_RESTORE_DIRENTS, rtCtx2);
            break;

        case FS_DR_CREATE_FILE:

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Restoring file"));
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d and cp is %s", strlen(cp), cp));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           NULL);
            if (objectIgnore) {
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Draining %s", cp));
                status = fs_drainObject(context, restoreContext,
                                        operation->header.headerId);
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Drained %s", cp));
                break;
            }
            status = fs_restoreMkFile(context, restoreContext, rtCtx->root, cp,
                                      len, &rtCtx->attr, &(rtCtx->inode));
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            if (rtCtx->inode) {
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            }
            break;
			
        case FS_DR_CREATE_SYML:

            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           NULL);
            if (objectIgnore) {
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                break;
            }
            status = fs_restoreMkSyml(context, restoreContext, rtCtx->root,	cp,
                                        len, &rtCtx->attr, &(rtCtx->inode));
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            if (rtCtx->inode) {
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            }
            break;
			
        case FS_DR_CREATE_LINK:

            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d and cp is %s", strlen(cp), cp));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           NULL);
            if (objectIgnore) {
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Drain hardlink"));
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                break;
            }
            {
                char buf[FS_NAME_MAX+1];
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                        ("before mkLink length of cp is %d and cp is %s", 
                         strlen(cp), cp));
                strcpy(buf, cp);

                /*
                 * RestoreMkLink will read tape again. If cp points to
                 * the tape buffer contents then this will fail
                 */
                status = fs_restoreMkLink(context, restoreContext, rtCtx->root,
							   		buf, len, &rtCtx->attr, &(rtCtx->inode));
            }
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE || rtCtx->inode == NULL ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            if (rtCtx->inode) {
                fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            }
            break;

        case FS_DR_CREATE_FIFO:
			nodeType = FS_IFMT_FIFO;
			goto createNode;
        case FS_DR_CREATE_SOCK:
			nodeType = FS_IFMT_SOCK;
			goto createNode;
		case FS_DR_CREATE_BLOCK:
			nodeType = FS_IFMT_BLK;
			goto createNode;			
		case FS_DR_CREATE_CHAR:
			nodeType = FS_IFMT_CHR;
createNode:
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           NULL);
            if (objectIgnore) {
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                break;
            }
            status = fs_restoreMkNode(context, restoreContext, rtCtx->root, cp,
                                 len, &rtCtx->attr, nodeType, &(rtCtx->inode));
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            if (rtCtx->inode) {
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            }
            break;
            
        case FS_DR_TAPE_END:
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Reached end of the tape"));
			//Make sure there is no context remaining
			goto goUp;
			
        default:
			FS_ASSERT(0);
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Unknown op"));
                /*
                 * Send a log message
                 */
                fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               0, NULL, "Invalid tape - Unknown op.\n");

                status = FS_ERR_INVAL;
                break;
        }

        /*
         * Stop if there was an i/o err
         */

        if (status != FS_ERR_NONE && !end) {
			goto goUp;
        }

    } while (end == 0);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("Returning from restoreFullTree"));
    return status;
}

#if 0
PRIVATE fs_Status
fs_restoreFullTree(
    fs_Context              *context,
    fs_RestoreContext_t     *restoreContext,
    fs_Inode                **sroot,
    char                    *sbuf,		//start root, buf & dirAttr
    fs_DRAttr_t             *sdirAttr
)
{
	DLL_HDR(fs_restoreFullTreeContext_t)	restoreFullTreeContexts;
	fs_restoreFullTreeContext_t	*rtCtx, *rtCtx2;
		
    fs_Status               status/* = FS_ERR_NONE*/;
    fs_DumpOperation_t      *operation;
    uint32                  len;
    char                    *cp, *np;
    char                    *newPath/* = NULL*/;
    int                     end = 0;
    int                     ignore/* = 0*/, objectIgnore/* = 0*/;
    fs_Inumber              rootInum;
    fs_Inumber              alternateParentInum;
    fs_Inumber              alternateInum;
    char                    *newName;
    uint16                  newNameLen;
    fs_Inode                *alternateParent;
	int						nodeType;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreFullTree"));
	
	//Initialize the restore full tree contexts list
	DLL_INIT(&restoreFullTreeContexts);

	//Create initial context
	rtCtx = fs_ramAlloc(FS_RAM_RESTORE_DIRENTS, 
						sizeof(fs_restoreFullTreeContext_t));
	bzero(rtCtx, sizeof(fs_restoreFullTreeContext_t));
	rtCtx->root		= sroot;
	rtCtx->buf		= sbuf;
	rtCtx->dirAttr	= sdirAttr;
	
    do {
     	//If have been using the cpu since a long time, give others a chance
	    fs_yieldPeriodic(context);
	
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			goto goUp;
        }
        operation = &restoreContext->record.u.operation;
        memcpy(&rtCtx->attr, &operation->header.attr, sizeof (fs_DRAttr_t));
        cp = &restoreContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
        len = operation->len;

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("restorefull tree (i%llu) op is %d",
                 (*(rtCtx->root))->inum, operation->header.op));
        switch (operation->header.op) {

        case FS_DR_CREATE_DIR:

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Restoring create dir"));
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            ignore = fs_shouldIgnore(context, restoreContext,
                                     restoreContext->currentName, 
                                     np - restoreContext->currentName,
                                     &newPath);
            if (ignore) {
                status = fs_drainDir(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Ignored directory"));
                break;
            }

            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
									NDMP_LOG_MSG_TYP_FILE, FS_ERR_NOENT,
                       				FS_ERR_NOENT, newPath,
									"Path to destination does not exists\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkDir(context, restoreContext, 
									&alternateParent, newName, newNameLen,
                                    &(rtCtx->inode));
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Renamed mkdir status %d",
                         status));
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Alternate parent is %lld", alternateParent->inum));
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Acquiring root %llu",
													rootInum));
                *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                rtCtx->inode = I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                if (fs_shouldExtract(context, restoreContext,
                                      restoreContext->currentName,
                                      np - restoreContext->currentName) != 0) {
                    status = fs_restoreMkDir(context, restoreContext, 
											rtCtx->root, cp, len, 
                                            &(rtCtx->inode));
                } else {
	                status = fs_drainDir(context, restoreContext,
	                                      operation->header.headerId);
                    break;
                }
            }
            if (status != FS_ERR_NONE) {
                break;
            }
            FS_ASSERT(rtCtx->inode != NULL);
            FS_ASSERT(fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            rtCtx->rootInum = (*(rtCtx->root))->inum;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
						("Releasing root %lld", rtCtx->rootInum));
            fs_inodeReleaseExclusiveAll(context, *(rtCtx->root));

			rtCtx2 = fs_ramAlloc(FS_RAM_RESTORE_DIRENTS, 
								sizeof(fs_restoreFullTreeContext_t));
			bzero(rtCtx2, sizeof(fs_restoreFullTreeContext_t));
			rtCtx2->root	= &(rtCtx->inode);
			rtCtx2->buf		= np;
			rtCtx2->dirAttr = &rtCtx->attr;
			DLL_INSERT_TAIL(&restoreFullTreeContexts, rtCtx, list);
			FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("going down..."));
			rtCtx = rtCtx2;		
			//We are now in the context of the new dir
			break;

        case FS_DR_DIR_END:
			if (rtCtx->dirAttr != NULL) {
	            status = fs_restoreSetAttr(context, restoreContext, 
											rtCtx->root, rtCtx->dirAttr);
			}
goUp:
			rtCtx2 = rtCtx;
			//Retrieve upper context
			DLL_REMOVE_TAIL(&restoreFullTreeContexts, rtCtx, list);
			if (rtCtx != NULL && rtCtx != DLL_NIL) {
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("going up..."));
				rootInum = rtCtx->rootInum;
				if (rootInum != (*(rtCtx2->root))->inum) 	{
					fs_inodeReleaseExclusiveAll(context, *(rtCtx2->root));
					*(rtCtx->root) = I_GET_EXCL(0, rootInum);
					FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
								("acquiring root %lld", rootInum));
				}					
			} else {
				end++;
			}
			//Free old context
			fs_ramDealloc(FS_RAM_RESTORE_DIRENTS, rtCtx2);
            break;

        case FS_DR_CREATE_FILE:

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("Restoring create file"));
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d and cp is %s", strlen(cp), cp));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           &newPath);
            if (objectIgnore) {
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Draining %s", cp));
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Drained %s", cp));
                break;
            }
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                     ("create file newPath is %s", newPath));
            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_FILE, FS_ERR_NOENT,
                               FS_ERR_NOENT, newPath, 
							   "Path to destination does not exists\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkFile(context, restoreContext, 
                                         &alternateParent, newName, newNameLen,
                                         &rtCtx->attr, &(rtCtx->inode));
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                rtCtx->inode = I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                status = fs_restoreMkFile(context, restoreContext, rtCtx->root,
						   			    cp, len, &rtCtx->attr, &(rtCtx->inode));
            }
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            break;
			
        case FS_DR_CREATE_SYML:

            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           &newPath);
            if (objectIgnore) {
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                break;
            }
            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_FILE, FS_ERR_NOENT,
                               FS_ERR_NONE, newPath, 
							   "path to destination does not exists\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkSyml(context, restoreContext, 
                                         &alternateParent, newName, newNameLen, 
										 &rtCtx->attr, &rtCtx->inode);
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                *(rtCtx->root)	= I_GET_EXCL(0, rootInum); 
                rtCtx->inode	= I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                status = fs_restoreMkSyml(context, restoreContext, rtCtx->root,
							   			cp, len, &rtCtx->attr, &(rtCtx->inode));
            }
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            break;
			
        case FS_DR_CREATE_LINK:

            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d and cp is %s", strlen(cp), cp));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           &newPath);
//#ifndef FS_EXTRACT_LINK
#if 0          
            if (objectIgnore || restoreContext->extract) {
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Drain hardlink"));
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                break;
            }
            if (restoreContext->extract != 0) {

                // don't allow extraction of the links as we don't
                // know, if the main path is restored or not and at
                // this point, we cannot rewind tapes.

                /*
                 * SEND a message though
                 */

                FS_ASSERT(0);
                break;
            }
#else
            if (objectIgnore) {
				FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Drain hardlink"));
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                break;
            }
#endif
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                     ("create link newPath is %s", newPath));
            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_FILE, FS_ERR_NOENT,
                               FS_ERR_NOENT, newPath,
							   "Path to Destination does not exists\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkLink(context, restoreContext, 
                                         &alternateParent, newName, newNameLen, 
										 &rtCtx->attr, &(rtCtx->inode));
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                *(rtCtx->root)	= I_GET_EXCL(0, rootInum); 
                rtCtx->inode	= I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                char buf[256];
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                        ("before mkLink length of cp is %d and cp is %s", 
                         strlen(cp), cp));
                strcpy(buf, cp);

                /*
                 * RestoreMkLink will read tape again. If cp points to
                 * the tape buffer contents then this will fail
                 */
                status = fs_restoreMkLink(context, restoreContext, rtCtx->root,
							   		buf, len, &rtCtx->attr, &(rtCtx->inode));
            }
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE || rtCtx->inode == NULL ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            if (rtCtx->inode) {
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            }
            break;

        case FS_DR_CREATE_FIFO:
			nodeType = FS_IFMT_FIFO;
			goto createNode;
        case FS_DR_CREATE_SOCK:
			nodeType = FS_IFMT_SOCK;
			goto createNode;
		case FS_DR_CREATE_BLOCK:
			nodeType = FS_IFMT_BLK;
			goto createNode;			
		case FS_DR_CREATE_CHAR:
			nodeType = FS_IFMT_CHR;
createNode:
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            objectIgnore = fs_shouldIgnore(context, restoreContext,
                                           restoreContext->currentName,
                                           np - restoreContext->currentName,
                                           &newPath);
            if (objectIgnore) {
                status = fs_drainObject(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                break;
            }
            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_FILE, FS_ERR_NOENT,
                               FS_ERR_NOENT, newPath,
							   "Path to destination does not exist\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkNode(context, restoreContext, 
									&alternateParent, newName, newNameLen, 
									&rtCtx->attr, nodeType, &(rtCtx->inode));
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                rtCtx->inode = I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                status = fs_restoreMkNode(context, restoreContext, rtCtx->root,
				   			cp, len, &rtCtx->attr, nodeType, &(rtCtx->inode));
            }
            FS_ASSERT(status != FS_ERR_NONE ||  rtCtx->inode != NULL);
            FS_ASSERT(status != FS_ERR_NONE ||
                      fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
            break;

        case FS_DR_CREATE_FSET:
            np = rtCtx->buf;
            strcat(np, "/");
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("length of cp is %d", strlen(cp)));
            strcpy(np + 1, cp);
            FS_ASSERT(strlen(np) == len + 1);
            np += len + 1;
            FS_ASSERT(np - restoreContext->currentName == 
                      strlen(restoreContext->currentName));
            ignore = fs_shouldIgnore(context, restoreContext,
                                     restoreContext->currentName,
                                     np - restoreContext->currentName,
                                     &newPath);
            if (ignore) {
                status = fs_drainDir(context, restoreContext,
                                      operation->header.headerId);
                if (newPath) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                }
                break;
            }

            if (newPath != NULL) {
                rootInum = (*(rtCtx->root))->inum;
                fs_inodeReleaseExclusive(context, *(rtCtx->root));
                status = fs_decomposePath(context, restoreContext,
                                  newPath, &alternateParentInum,
                                  &alternateInum, &newName, &newNameLen, 0);
                if (status != FS_ERR_NONE) {
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }
                if (alternateInum != 0) {
					//The target directory already exists, it can not be 
					//changed to a fileset
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_FILE, FS_ERR_EXIST,
                               FS_ERR_EXIST, newPath,
							   "Destination object exists\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_EXIST;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                if (alternateParentInum == 0) {
                    fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_NOENT,
                               FS_ERR_NOENT, newPath,
							   "Path to destination does not exist\n");
                    *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                    status = FS_ERR_NOENT;
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                    if (newName) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                    }
                    break;
                }

                alternateParent = I_GET_EXCL(0, alternateParentInum);
                status = fs_restoreMkFset(context, restoreContext, 
								          &alternateParent, newName, 
                                          newNameLen, &(rtCtx->inode));
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Renamed mkdir status %d", status));
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Alternate parent is %lld", alternateParent->inum));
                fs_inodeReleaseExclusiveAll(context, alternateParent);
                if (rtCtx->inode) {
                    alternateInum = rtCtx->inode->inum;
                    fs_inodeReleaseExclusiveAll(context, rtCtx->inode);
                } else {
                    if (status == FS_ERR_NOSPC) {
                        *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                        if (newName) {
                            fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                        }
                        break;
                    }
                    FS_ASSERT(0);
                    break;
                }
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("Acquiring root %llu", rootInum));
                *(rtCtx->root) = I_GET_EXCL(0, rootInum); 
                rtCtx->inode = I_GET_EXCL(0, alternateInum);
                fs_ramDealloc(FS_RAM_RESTORE_NAME, newPath);
                if (newName) {
                    fs_ramDealloc(FS_RAM_RESTORE_NAME, newName);
                }
            } else {
                if (fs_shouldExtract(context, restoreContext,
                                      restoreContext->currentName,
                                      np - restoreContext->currentName) != 0) {
                    status = fs_restoreMkFset(context, restoreContext, 
											rtCtx->root, cp, len,
                                            &(rtCtx->inode));
                } else {
	                status = fs_drainDir(context, restoreContext,
	                                      operation->header.headerId);
                    break;
                }
            }
            if (status != FS_ERR_NONE) {
                break;
            }
            FS_ASSERT(rtCtx->inode != NULL);
            FS_ASSERT(fs_lockOwnedExclusive(context, &(rtCtx->inode->lock)));
            rtCtx->rootInum = (*(rtCtx->root))->inum;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
						("Releasing root %lld", rtCtx->rootInum));
            fs_inodeReleaseExclusiveAll(context, *(rtCtx->root));
			
			rtCtx2 = fs_ramAlloc(FS_RAM_RESTORE_DIRENTS, 
								sizeof(fs_restoreFullTreeContext_t));
			bzero(rtCtx2, sizeof(fs_restoreFullTreeContext_t));
			rtCtx2->root	= &(rtCtx->inode);
			rtCtx2->buf		= np;
			rtCtx2->dirAttr = &rtCtx->attr;
			DLL_INSERT_TAIL(&restoreFullTreeContexts, rtCtx, list);
			FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("going down..."));
			rtCtx = rtCtx2;		
			//We are now in the context of the new dir
            break;

        case FS_DR_TAPE_END:
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Reached end of the tape"));
			//Make sure there is no context remaining
			goto goUp;
			
        default:
			FS_ASSERT(0);
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Unknown op"));
                /*
                 * Send a log message
                 */
                fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               0, NULL, "Invalid tape - Unknown op.\n");

                status = FS_ERR_INVAL;
                break;
        }

        /*
         * Stop if there was an i/o err
         */

        if (status != FS_ERR_NONE && !end) {
			goto goUp;
        }

    } while (end == 0);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("Returning from restoreFullTree"));
    return status;
}
#endif

/*
 * Create a directory
 */

PRIVATE fs_Status
fs_restoreMkDir(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
	uint64				unstableId;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_restoreMkDir - adding %s", name));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));

    *rinode = NULL;

    inum = fs_dir_lookupent(context, *parent, name, nameLen);
    if (inum != 0) {
        *rinode = I_GET_EXCL(0, inum);
        if (!(FS_I_IS_DIR((*rinode)->dinode.attrs))) {
//            fs_inodeReleaseExclusive(context, *rinode);
            /*
             * Log an error message with ndmp
             */
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_EXIST, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_EXIST, name, "Object with same name exits\n");
            return FS_ERR_EXIST;
        }
        return FS_ERR_NONE;
    }

    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);

    /*
     * Don't bother about quota reservation. Quota Off has to be performed
     * and turned on again before restore
     */

    inode = fs_allocateDirectoryInode(context, *parent, 0777, 0, 0,
                                      (*parent)->dinode.fsetNum);
    if (inode == NULL) {
        return FS_ERR_IO;
    }

    inum = inode->inum;
    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }

    (*parent)->dinode.linkCnt += 1;
    (*parent)->dinode.mtime = fs_getTimeOfDay();
    (*parent)->dinode.atime = (*parent)->dinode.mtime;
    (*parent)->dinode.ctime = (*parent)->dinode.atime;
    (*parent)->dinode.version++;
    fs_inodeWrite(context, *parent, INO_WRITE_LOG);

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else
    unstableId = inode->inum;
    vl_addUnstableId(context, &unstableId);
    fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
    rc = NFX_OK;

    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    if (status == FS_ERR_NONE) {
        *parent = I_GET_EXCL(0, parentInum);
        *rinode = I_GET_EXCL(0, inum);
    }
    return status;
}

/*
 * Set attributes on the file
 */

PRIVATE fs_Status
fs_restoreSetAttr(
    fs_Context              *context,
    fs_RestoreContext_t     *restoreContext,
    fs_Inode                **root,
    fs_DRAttr_t             *attr
)
{
    fs_SecurityDescNfs      sdNew;
    fs_Inumber              inum = (*root)->inum;
    fs_Status               status = FS_ERR_NONE;
    int32                   rc;
	uint64				unstableId;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreSetAttr"));
    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);

    sdNew = (*root)->sd->u.nfs;
    sdNew.uid = attr->uid;
    sdNew.gid = attr->gid;

    (*root)->dinode.atime = attr->atime;
    (*root)->dinode.mtime = attr->mtime;
    (*root)->dinode.ctime = fs_getTimeOfDay();
    (*root)->dinode.version++;
    // BOB - Rahul, please visit this
    (*root)->dinode.userAttr = attr->simpleAttrs;
    //inode->dinode.simpleAttrs =  (inode->dinode.simpleAttrs & 
                                  //(FS_INODE_SATTR_NO_SNAP|
                                    //FS_INODE_SATTR_DIR_SPLIT)) |
                                    //attr->simpleAttrs;
    (*root)->dinode.attrs = (*root)->dinode.attrs & (FS_IFMT_MASK |
                                                     FS_IFLG_MASK);
    (*root)->dinode.attrs |= (attr->mode & FS_IATR_MASK);
    sdNew.mode = (uint16)(attr->mode & FS_IMODE_MASK);
    fs_securityDescSet(context, *(root), &sdNew.hdr);

    if (!FS_I_IS_DIR((*root)->dinode.attrs) &&
        !FS_I_IS_SYML((*root)->dinode.attrs)) {
        (*root)->dinode.size = attr->sizeDev.size;
    }
    fs_inodeWrite(context, *root, INO_WRITE_LOG);
#if 0
    rc = fs_logCommitEntry(context);
#else
	unstableId = (*root)->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;

    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *root = I_GET_EXCL(0, inum);
        return FS_ERR_IO;
    }
    *root = I_GET_EXCL(0, inum);
    return status;

}

PRIVATE fs_Status
fs_restoreMkNode(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_DRAttr_t         *attr,
    int                 mode,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode = NULL;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
	uint64				unstableId;
    fs_Time             now = fs_getTimeOfDay();
    fs_SecurityDescNfs  sdNew;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreMkNode"));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));

    *rinode = NULL;
    inum = fs_dir_lookupent(context, *parent, name, nameLen);
    if (inum != 0) {
        *rinode = I_GET_EXCL(0, inum);
        if (((*rinode)->dinode.attrs & FS_IFMT_MASK) != 
            (mode & FS_IFMT_MASK)) {
//            fs_inodeReleaseExclusive(context, *rinode);
            /*
             * Log an error message with ndmp
             */
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_EXIST, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_EXIST, name, "Object with same name exits\n");
            return FS_ERR_EXIST;
        }
    }

    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);

    if (inum != 0) {
        fs_inodeReleaseExclusive(context, *parent);
        goto set_attr;
    }
    /*
     * Don't bother about quota reservation. Quota Off has to be performed
     * and turned on again before restore
     */

    //inode = fs_allocateFileInode(context, *parent, mode | 777, attr->uid,
    inode = fs_allocateFileInode(context, *parent, mode, attr->uid,
                                 attr->gid, (*parent)->dinode.fsetNum);
    if (inode == NULL) {
        FS_ASSERT(fs_lockOwnedExclusive(context, &((*parent)->lock)));
        return FS_ERR_IO;
    }

    inum = inode->inum;
    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }
	/*
	 * Update parent's timestamps
	 */

    if (restoreContext->extract) {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.mtime = (*parent)->dinode.atime =
            (*parent)->dinode.ctime;
        (*parent)->dinode.version++;
    } else {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.version++;
    }

    fs_inodeWrite(context, *parent, INO_WRITE_LOG);

set_attr:
    inode->dinode.mtime = attr->mtime;
    inode->dinode.ctime = now;
    inode->dinode.version++;
    inode->dinode.atime = attr->atime;
    inode->dinode.size = 0;
    inode->dinode.attrs = inode->dinode.attrs & (FS_IFMT_MASK |
                                                 FS_IFLG_MASK);
    inode->dinode.attrs |= (attr->mode & FS_IATR_MASK);
    sdNew = inode->sd->u.nfs;
    sdNew.mode = (uint16)(attr->mode & FS_IMODE_MASK);
    fs_securityDescSet(context, inode, &sdNew.hdr);

    if (FS_I_IS_BLK(inode->dinode.attrs) ||
        FS_I_IS_CHR(inode->dinode.attrs)) {
        inode->dinode.u.devNum.major = attr->sizeDev.devNum.major;
        inode->dinode.u.devNum.minor = attr->sizeDev.devNum.minor;
    }

    if (restoreContext->extract) {
        inode->dinode.ctime = now;
        inode->dinode.version++;
        inode->dinode.mtime = inode->dinode.atime = inode->dinode.ctime;
    }
    fs_inodeWrite(context, inode, INO_WRITE_LOG);

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else

	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;

    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    *parent = I_GET_EXCL(0, parentInum);
    if (status == FS_ERR_NONE) {
        *rinode = I_GET_EXCL(0, inum);
    }
    return status;
}


/* TODO ???? CHECK THIS COMMENT. THINK IT IS WRONG ????
 * Todo also create fset structures besides the dir entry
 */

PRIVATE fs_Status
fs_restoreMkFset(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode, *fsetInfoInode;
    fs_FileSetIncoreRecord_t    *fset, *parentFset;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
	uint64				unstableId;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreMkfset"));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));

    parentFset = (*parent)->fset = fs_getFset(context, context->fileSystem,
                                              *parent);
    *rinode = fs_lookupKeepParentLocked(context, context->fileSystem, 0, 
                                        *parent, name);
    if (*rinode != NULL)    {
        if (!(FS_I_IS_DIR((*rinode)->dinode.attrs))
         || (*rinode)->fset == parentFset) {
            /*
             * Object exists and is not a fileset 
             * Log an error message with ndmp
             */
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_EXIST, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_EXIST, name, 
                                  "Object with same name exits\n");
            return FS_ERR_EXIST;
        }
        return FS_ERR_NONE;
    }

    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);

    /*
     * Don't bother about quota reservation. Quota Off has to be performed
     * and turned on again before restore
     */

    inode = fs_allocateDirectoryInode(context, *parent, 0777, 0, 0,
                                      (*parent)->dinode.fsetNum);
    if (inode == NULL) {
        FS_ASSERT(fs_lockOwnedExclusive(context, &((*parent)->lock)));
        return FS_ERR_IO;
    }

    inum = inode->inum;
    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }

    (*parent)->dinode.linkCnt += 1;
    (*parent)->dinode.ctime = fs_getTimeOfDay();
    (*parent)->dinode.version++;
    (*parent)->dinode.mtime = (*parent)->dinode.ctime;
    fs_inodeWrite(context, *parent, INO_WRITE_LOG);

    /*
     * Create the fileset
     */

    fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
    fs_lockExclusive(context, &parentFset->lock, NULL);
    fset = fs_makeFileSet(context, context->fileSystem, parentFset,
                          inode, fsetInfoInode);
    fs_unlockExclusive(context, &parentFset->lock);

    if (!fs_inodeWasModified(context, fsetInfoInode)) {
        fs_inodeReleaseExclusive(context, fsetInfoInode);
    }

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else

	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;
    
    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    *parent = I_GET_EXCL(0, parentInum);
    if (status == FS_ERR_NONE) {
        *rinode = I_GET_EXCL(0, inum);
    }
    return status;
}

PRIVATE fs_Status
fs_restoreMkSyml(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_DRAttr_t         *attr,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
    char                *cp, *buffer = NULL;
    fs_Buf              *bp;
    uint32              genNum;
	uint64				unstableId;
    fs_Time             now = fs_getTimeOfDay();
    fs_SecurityDescNfs  sdNew;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreMkSyml"));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));

    *rinode = NULL;
    inum = fs_dir_lookupent(context, *parent, name, nameLen);
    if (inum != 0) {
        *rinode = I_GET_EXCL(0, inum);
        if (!(FS_I_IS_SYML((*rinode)->dinode.attrs))) {
//            fs_inodeReleaseExclusive(context, *rinode);
            /*
             * Log an error message with ndmp
             */
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_EXIST, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_EXIST, name,
                                  "Object with same name exits\n");
            return FS_ERR_EXIST;
        }
    }


    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);
    if (inum != 0) {
        fs_inodeReleaseExclusive(context, *parent);
        if ((*rinode)->dinode.numblocks > 0) {
            fs_truncateFile(context, *rinode, 0, FALSE, NULL);

        } else {
            (*rinode)->dinode.size = 0;
        }
		inode = *rinode;
        goto set_attr;
    }

    /*
     * Don't bother about quota reservation. Quota Off has to be performed
     * and turned on again before restore
     */

    //inode = fs_allocateFileInode(context, *parent, FS_IFMT_SYML | 777,
    inode = fs_allocateFileInode(context, *parent, FS_IFMT_SYML,
                                 attr->uid, attr->gid,
                                 (*parent)->dinode.fsetNum);
    if (inode == NULL) {
        FS_ASSERT(fs_lockOwnedExclusive(context, &((*parent)->lock)));
        return FS_ERR_IO;
    }

    inum = inode->inum;
    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }

	/*
	 * Update parent's timestamps
	 */

    if (restoreContext->extract) {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.mtime = (*parent)->dinode.atime =
            (*parent)->dinode.ctime;
        (*parent)->dinode.version++;
    } else {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.version++;
    }
    fs_inodeWrite(context, *parent, INO_WRITE_LOG);
set_attr:
    inode->dinode.atime = attr->atime;
    inode->dinode.mtime = attr->mtime;
    inode->dinode.ctime = now;
    inode->dinode.version++;
    inode->dinode.attrs |= (attr->mode & FS_IATR_MASK);
    sdNew = inode->sd->u.nfs;
    sdNew.mode = (uint16)(attr->mode & FS_IMODE_MASK);
    fs_securityDescSet(context, inode, &sdNew.hdr);

    // BOB - Rahul, please visit this
    inode->dinode.userAttr = attr->simpleAttrs;
    //inode->dinode.simpleAttrs =  (inode->dinode.simpleAttrs & 
                                  //(FS_INODE_SATTR_NO_SNAP|
                                    //FS_INODE_SATTR_DIR_SPLIT)) |
                                    //attr->simpleAttrs;
    genNum = inode->dinode.genNum;
    inum = inode->inum;

    /*
     * Commit the transaction and release the locks when we read the tape
     */

#if 0
    rc = fs_logCommitEntry(context);
#else

	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;

    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        return FS_ERR_IO;
    }
    /*
     * Read in the link
     */

    status = fs_readTape(context, restoreContext);
    if (status != FS_ERR_NONE) {
        *rinode = inode;
        return status;
    }
    
    FS_ASSERT(attr->sizeDev.size <= FS_PATH_MAX);
    cp = restoreContext->record.u.dummy;
    if (attr->sizeDev.size > FS_DR_TBSIZE) {
        int32 remainSize = attr->sizeDev.size - FS_DR_TBSIZE;

        /*
         * Symbolic exceeds tape record size. Allocate memory to read it in
         */

        buffer = fs_ramAlloc(FS_RAM_RESTORE_DATA, attr->sizeDev.size); 
        memcpy(buffer, cp, FS_DR_TBSIZE);
        cp += FS_DR_TBSIZE;
more_symlink:
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
            *rinode = inode;
            return status;
        }
        memcpy(cp, restoreContext->record.u.dummy, 
               MIN(remainSize, FS_DR_TBSIZE));
        remainSize -= FS_DR_TBSIZE;
        if (remainSize > 0) {
            cp += FS_DR_TBSIZE;
            goto more_symlink;
        }
        cp = buffer;
    }
    
    *parent = I_GET_EXCL(0, parentInum);
    inode = I_GET_EXCL(0, inum);
    if (inode->dinode.genNum != genNum) {
        *rinode = inode;
        return FS_ERR_NONE;
    }

    fs_logBeginTransaction(context);


    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("link contents are %s", cp));
    
    inode->dinode.size = attr->sizeDev.size;
    
    if (inode->dinode.size < FS_INODE_INLINE_LEN) {
        memcpy(inode->dinode.u.data, cp, inode->dinode.size);
    } else {
        fs_emaproot_init(context, inode);
        bp = O_CREATE(inode, 0);
        memcpy(bp->u.data, cp, inode->dinode.size);
        FS_BUF_ZERO_ALL(bp);
        FS_BUF_WRITE(bp, 0, inode->dinode.size);
    }

    if (buffer) {
        fs_ramDealloc(FS_RAM_RESTORE_DATA, buffer);
    }

    if (restoreContext->extract) {
        inode->dinode.ctime = now;
        inode->dinode.version++;
        inode->dinode.mtime = inode->dinode.atime = inode->dinode.ctime;
    }
    fs_inodeWrite(context, inode, INO_WRITE_LOG);

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else

	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;
    
    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    *parent = I_GET_EXCL(0, parentInum);
    if (status == FS_ERR_NONE) {
        *rinode = I_GET_EXCL(0, inum);
    }
    return status;
}

/*
 * Read the pathname from the next tape record and then get the
 * inode and create link to it
 */

PRIVATE fs_Status
fs_restoreMkLink(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_DRAttr_t         *attr,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
    char                *cp;
	uint64				unstableId;
    fs_Time             now = fs_getTimeOfDay();

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreMkLink %s", name));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));

    *rinode = NULL;
    inum = fs_dir_lookupent(context, *parent, name, nameLen);
    if (inum != 0) {

        /*
         * XXX - Drain off the link name, watch out for handling longer
         * than 1k maxpath
         */
        fs_readTape(context, restoreContext);
        *rinode = I_GET_EXCL(0, inum);
        return FS_ERR_NONE;
    }

    /*
     * Read the link path
     */

    fs_readTape(context, restoreContext);

    /*
     * Initialize the transaction
     */


    cp = restoreContext->record.u.dummy;
    inode = fs_parsePath(context, restoreContext, cp);
//#ifdef FS_EXTRACT_LINKS
#if 1 
    if (inode == NULL) {
            char    buf[256];
            /*
             * We don't have the link target already extracted
             * Ask the clients to extract the target before
             * this link can be played
             */
            sprintf(buf, "Link target %s does not exist, please extract it"
                            " before you can extract %s", cp, name);
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_NOENT, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_NOENT, name, buf);
            //fs_inodeReleaseExclusive(context, (*parent));
            return FS_ERR_NOENT;
    }
#else
    FS_ASSERT(inode != NULL);
#endif
    fs_logBeginTransaction(context);
    inum = inode->inum;
//    fs_inodeReleaseExclusive(context, inode);
    inode = I_GET_EXCL(0, inum);
    
    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }
	/*
	 * Update parent's timestamps
	 */

    if (restoreContext->extract) {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.mtime = (*parent)->dinode.atime =
            (*parent)->dinode.ctime;
        (*parent)->dinode.version++;
        fs_inodeWrite(context, inode, INO_WRITE_LOG);
    } else {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.version++;
    }

    inode->dinode.linkCnt += 1;
    fs_inodeWrite(context, inode, INO_WRITE_LOG);

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else

	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;

    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    *parent = I_GET_EXCL(0, parentInum);
    if (status == FS_ERR_NONE) {
        *rinode = I_GET_EXCL(0, inum);
    }
    return status;
}

PRIVATE fs_Status
fs_restoreMkFile(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    fs_Inode            **parent,
    char                *name,
    int                 nameLen,
    fs_DRAttr_t         *attr,
    fs_Inode            **rinode
)
{
    fs_Inode            *inode;
    fs_Inumber          inum;
    fs_Inumber          parentInum = (*parent)->inum;
    int32               rc;
    fs_Status           status = FS_ERR_NONE;
	uint64				unstableId;
    fs_Time             now = fs_getTimeOfDay();
    fs_SecurityDescNfs  sdNew;

    if (!fs_haveMinfreeAvail(context->fileSystem)) {
        return FS_ERR_NOSPC;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("In restore Mkfile for %s", name));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_restoreMkFile for %s", name));
    FS_ASSERT(fs_lockOwnedExclusive(context, &(*parent)->lock));
    *rinode = NULL;
    inum = fs_dir_lookupent(context, *parent, name, nameLen);
    if (inum != 0) {
        *rinode = I_GET_EXCL(0, inum);
        if (!(FS_I_IS_REG((*rinode)->dinode.attrs))) {
//            fs_inodeReleaseExclusive(context, *rinode);
            /*
             * Log an error message with ndmp
             */
            fs_ndmpSendRestoreLog(context, restoreContext,
                                  FS_ERR_EXIST, NDMP_LOG_MSG_TYP_ERR,
                                  FS_ERR_EXIST, name, "object with same name exits\n");
            return FS_ERR_EXIST;
        }
    }

    /*
     * Initialize the transaction
     */

    fs_logBeginTransaction(context);
    if (inum != 0) {
        fs_inodeReleaseExclusive(context, *parent);
        fs_truncateFile(context, *rinode, 0, FALSE, NULL);
		inode = *rinode;
        goto set_attr;
    }

    /*
     * Don't bother about quota reservation. Quota Off has to be performed
     * and turned on again before restore
     */

    //inode = fs_allocateFileInode(context, *parent, FS_IFMT_REG | 777,
    inode = fs_allocateFileInode(context, *parent, FS_IFMT_REG,
                                 attr->uid, attr->gid,
                                 (*parent)->dinode.fsetNum);
    if (inode == NULL) {
        FS_ASSERT(fs_lockOwnedExclusive(context, &((*parent)->lock)));
        return FS_ERR_IO;
    }

    inum = inode->inum;

    rc = fs_dir_addent(context, *parent, name, nameLen, inum);
	if (rc != NFX_OK) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
	}
    
    status = fs_addLink(context, 0, inode, (*parent)->inum);
    if (status != FS_ERR_NONE) {
        FS_ASSERT(0);
        fs_inodeReleaseExclusive(context, inode);
        return FS_ERR_IO;
    }

	/*
	 * Update parent's timestamps
	 */

    if (restoreContext->extract) {
        (*parent)->dinode.ctime = now;
        (*parent)->dinode.mtime = (*parent)->dinode.atime =
            (*parent)->dinode.ctime;
        (*parent)->dinode.version++;
    }
    fs_inodeWrite(context, *parent, INO_WRITE_LOG);
set_attr:
    inode->dinode.atime = attr->atime;
    inode->dinode.mtime = attr->mtime;
    inode->dinode.ctime = now;
    inode->dinode.version++;
    inode->dinode.size = attr->sizeDev.size;
    inode->dinode.attrs |= (attr->mode & FS_IATR_MASK);
    sdNew = inode->sd->u.nfs;
    sdNew.mode = (uint16)(attr->mode & FS_IMODE_MASK);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_restoreMkFile for mode is %o %x", sdNew.mode, sdNew.mode));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_restoreMkFile for attr is %o %x", inode->dinode.attrs,
              inode->dinode.attrs));

    fs_securityDescSet(context, inode, &sdNew.hdr);

    // BOB - Rahul, please visit this
    inode->dinode.userAttr = attr->simpleAttrs;
    //inode->dinode.simpleAttrs =  (inode->dinode.simpleAttrs & 
                                  //(FS_INODE_SATTR_NO_SNAP|
                                    //FS_INODE_SATTR_DIR_SPLIT)) |
                                    //attr->simpleAttrs;
    /*
     * If this was a file extract, update time stamps of the file
     * as well
     */

    if (restoreContext->extract) {
        inode->dinode.ctime = now;
        inode->dinode.version++;
        inode->dinode.mtime = inode->dinode.atime = inode->dinode.ctime;
    }
    fs_inodeWrite(context, inode, INO_WRITE_LOG);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Size of the file is %lld",
             attr->sizeDev.size));

    /*
     * Commit the transaction
     */

#if 0
    rc = fs_logCommitEntry(context);
#else
	unstableId = inode->inum;
	vl_addUnstableId(context, &unstableId);
	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
	rc = NFX_OK;
    fs_makeSureLogIsNotFull(context);
#endif

    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        *parent = I_GET_EXCL(0, parentInum);
        return FS_ERR_IO;
    }

    if (status == FS_ERR_NONE) {
        *rinode = I_GET_EXCL(0, inum);
    } else {
        *parent = I_GET_EXCL(0, parentInum);
        return status;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("restoring writes for %s", name));

    /*
     * !!! Parent is not locked at this point, change the api to not take
     * parent argument as it is not required
     */

    /*
     * Mark the inode as being restored so that nfs code handles
     * cache coherancy
     */

    (*rinode)->flags |= INO_FLAGS_RESTORE_IN_PROGRESS;

    status = fs_restoreWrites(context, restoreContext, parent, rinode);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("restored writes for %s", name));

    /*
     * Flush the inode and invalidate buffers
     */

//    fs_bufSyncInodeBufs(context, inode);
//    fs_bufCacheFlush(context, inode);
    /*
     * Mark the inode as done doing restore, so that nfs can
     * take its fast path
     */

//    (*rinode)->flags &= ~INO_FLAGS_RESTORE_IN_PROGRESS;
    fs_inodeReleaseExclusive(context, *rinode);

    *parent = I_GET_EXCL(0, parentInum);
    *rinode = I_GET_EXCL(0, inum);

    return status;
}

/*
 * Write all the extents of the file
 */

PRIVATE fs_Status
fs_restoreWrites(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    fs_Inode            **parent,
    fs_Inode            **rinode
)
{
    fs_DumpOperation_t  *operation;
    fs_DumpDataExtension_t  *extension;
    int                 numExtents;
    fs_DumpExtent_t     *drExtent;
    uint64              i, j;
    char                *buf;
    uint64              headerId;
    fs_Status           status = FS_ERR_NONE;
    int                 objsize = 0;
    fs_Inumber          inum = (*rinode)->inum;
    uint32              genNum = (*rinode)->dinode.genNum;


    /*
     * If have been using the cpu since a long time, give others a chance
     */
    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_restoreWrites"));
    buf = fs_ramAlloc(FS_RAM_RESTORE_DATA, FS_DR_TBSIZE);
    FS_ASSERT(buf != NULL);
    /*
     * First Record contains the file creation operation and
     * might be overloaded with the extent information as well
     */

    operation = &restoreContext->record.u.operation;
    FS_ASSERT(operation->header.op == FS_DR_CREATE_FILE);
    numExtents = operation->header.dataExtents;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("numextents is %d", numExtents));

    headerId = operation->header.headerId;

    if (numExtents == 0) {
        goto more_extents;
    }

    objsize = sizeof (fs_DumpOperation_t) + operation->len;
    //objsize += (objsize & 7);
    objsize += 7;
    objsize &= ~7;   // align to 8 byte boundary
    drExtent = (fs_DumpExtent_t *)&restoreContext->record.u.dummy[objsize];

write_data:
    for (i = 0; i < numExtents; i++) {

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("dumping extent num %lld", i));

        /*
         * Process each extent
         */

        j = 0;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("extent offset is %lld len is %lld",
                 drExtent->offset, drExtent->len));
        while (j < drExtent->len) {

            /*
             * Drop the inode lock as we don't know how long will we take
             */

            fs_inodeReleaseExclusive(context, *rinode);
            status = fs_readDataTape(context, restoreContext, buf);
            if (status != FS_ERR_NONE) {
                *rinode = I_GET_EXCL(0, inum);
                fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                return status;
            }
            *rinode = I_GET_EXCL(0, inum);
			(*rinode)->flags |= INO_FLAGS_RESTORE_IN_PROGRESS;

            /*
             * If inode was removed when we dropped the lock, forget
             * about writing to it. Simply read the contents off the tape
             */

            if ((*rinode)->dinode.genNum == genNum) {
                status = fs_restoreWriteFile(context, restoreContext, rinode,
                                         buf,
                                        drExtent->offset + (j << FS_DR_TBSHIFT),
                                         FS_DR_TBSIZE);
            }
            if (status != FS_ERR_NONE) {
                fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                return status;
            }
            j++ ;
        }
        drExtent++;
    }
        
more_extents:

    /*
     * Read the tape till, we see DumpExtension records for this headerId
     */

    fs_readTape(context, restoreContext);
    extension = &restoreContext->record.u.extension;
    if (extension->header.headerId != headerId) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("op is %d", extension->header.op));
        fs_restorePushBackRead(context, restoreContext);
        fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
        return status;
    }
    
    numExtents = extension->header.dataExtents;
    drExtent = extension->extents;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("more data"));
    goto write_data;

    /* NEVER REACHED */
}

/*
 * Write 1 block worth of data to the file
 */

EXPORTED fs_Status
fs_restoreWriteFile(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    fs_Inode            **rinode,
    char                *cp,
    uint64              offset,
    uint64              bytes
)
{
    fs_Inumber          inum = (*rinode)->inum;
    fs_Buf              *buf;
    fs_Inode            *inode = *rinode;
    uint32              boff;
    int32               rc = NFX_OK;
    fs_Status           status = FS_ERR_NONE;
	int					tran = 0;
	uint64				unstableId;
    fs_Extent           extent;


    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restoreWriteFile for inum %lld", inum));
    boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("Writing at %lld %lld bytes boff = %d",
                                     offset, bytes, boff));
    buf = O_GET_EXCL(inode, offset >> FS_BLOCK_SHIFT);
    if (buf == NULL) {
        if (!fs_haveMinfreeAvail(context->fileSystem)) {
            return FS_ERR_NOSPC;
        }
		fs_logBeginTransaction(context);
		tran++;

        /*
         * Preallocate the extent rather than relying on buffer cache to
         * do it as this gives us better allocation
         */

        extent = fs_allocateExtent(context, inode, offset >> FS_BLOCK_SHIFT,
                                   1, FALSE);
        fs_emap_add(context, inode, offset >> FS_BLOCK_SHIFT, extent);
        buf = O_GET_EXCL(inode, offset >> FS_BLOCK_SHIFT);
        FS_ASSERT(buf != NULL);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Created buffer"));
        FS_BUF_ZERO_ALL(buf);
    }

    memcpy(&buf->u.data[boff], cp, bytes);
    FS_BUF_WRITE_NO_LOG(buf, 0, FS_BLOCK_SIZE);
	if (!fs_bufWasModified(context, buf)) {
			fs_bufReleaseExclusive(context, buf);
	}
    if (!fs_inodeWasModified(context, inode)) {
        fs_inodeReleaseExclusive(context, inode);
    }

	if (tran) {
#if 0
			rc = fs_logCommitEntry(context);
#else
    	unstableId = inode->inum;
    	vl_addUnstableId(context, &unstableId);
    	fs_logCommit(context, FS_LOG_WRITE_COMMIT_UNSTABLE, unstableId);
    	rc = NFX_OK;
        fs_makeSureLogIsNotFull(context);
#endif
	}
    
    fs_restoreSleepIfNeeded(context);
    
    if (rc != NFX_OK) {
        FS_ASSERT(0);
        return FS_ERR_IO;
    }
	/*
	 * Before initiating a log transaction, ensure that there is enough
	 * log space. If the log is more than 3/4 full, signal buffer cache write back
	 */

	if (context->fileSystem->logRoot->currLen >
		   (context->fileSystem->logSize * 3 / 4)) {
 		/*
		 * Signal the buffer cache
		 */
		fs_bufSyncWritebackLists(context);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("yielding to create log space"));
		fs_yieldTemp(context);
	}
    *rinode = I_GET_EXCL(0, inum);

    return status;
}

EXPORTED int
fs_isRestored(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,    
    char                *path
)
{
    fs_RestoreNameList_t    *entry;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("isRestored path %s", path));

    DLL_ITERATE(entry, &restoreContext->nameList, link) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                            ("entry path %s", entry->name));
            if (strcmp(entry->name, path) == 0 &&
                            entry->restored == 1) {
                    return 1;
            }
    }
    return 0;
}

EXPORTED int
fs_shouldIgnore(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    char                *buf,
    int                 bufLen,
    char                **newPath
)
{
    fs_RestoreNameList_t    *entry;
    int                     compare = 0;
    char                    *cp;
    int                     tried = 0;
    int                     parent = 0;
    int                     child = 0;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_ASSERT(strlen(buf) == bufLen);
 //   FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("in should ignore\n"));
    if (newPath)    {
        *newPath = NULL;
    }
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_shouldIgnore - %s", buf));
    DLL_ITERATE(entry, &restoreContext->nameList, link) {
        tried = 1;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("in should ignore comparing buf %s and entry %s\n",
                    buf, entry->name));
        compare = (bufLen != entry->nameLen || strcmp(buf, entry->name));

        /*
         * If this is an exact match, we cannot ignore it
         */

        if (compare == 0) {
            if (strcmp(entry->name, entry->newPath) != 0) {
                if (newPath != NULL)    {
                    if (*newPath) {
                        fs_ramDealloc(FS_RAM_RESTORE_NAME, *newPath);
                    }
                    *newPath = fs_ramAlloc(FS_RAM_RESTORE_NAME, 
                                           entry->newPathLen + 1);
                    strcpy(*newPath, entry->newPath);
                }
            }
            entry->restored = 1;
            return 0;
        }

        /*
         * If buf is parent of the object that needs to be restored
         * extract it.
         */

        cp = strstr(entry->name, buf);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("cp after name-buf %s\n", cp));
	    if (cp == entry->name &&
            ((cp[bufLen] == '/') || (buf[bufLen - 1] == '/'))) {
            parent = 1;

            /*
             * Search further to see if there is an exact match
             */
            continue;
        }

        /*
         * If this is descendent of a directory which is to be restored
         * restore it
         */

        cp = strstr(buf, entry->name);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("cp after buf-name %s\n", cp));
        if (cp == buf &&
            ((entry->name[entry->nameLen - 1] == '/' ||
             cp[entry->nameLen] == '/'))) {

            if (newPath != NULL)    {
                *newPath = fs_ramAlloc(FS_RAM_RESTORE_NAME,
                               bufLen + entry->newPathLen - entry->nameLen + 1);
                strcpy(*newPath, entry->newPath);
                strcat(*newPath, &buf[entry->nameLen]);
            }

            /*
             * Search further if there is an exact match
             */
            child = 1;
            continue;
        }

    }
    if (parent || child) {
        return 0;
    }
    if (tried) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Ignored %s", buf));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("returning ignore\n"));
        return 1;
    }
    return 0;
}

EXPORTED int
fs_shouldExtract(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    char                *buf,
    int                 bufLen
)
{
    fs_RestoreNameList_t    *entry;
    int                     compare = 0;
    char                    *cp;
    int                     tried = 0;
    int                     exactMatch = 0;
    int                     extractChild = 0;
    int                     extractParent = 0;
    int                     renameChild = 0;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("In fs_shouldExtract - %s", buf));
    DLL_ITERATE(entry, &restoreContext->nameList, link) {
        tried = 1;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Searching %s", entry->name));
        compare = (bufLen != entry->nameLen || strcmp(buf, entry->name));

        /*
         * If this is an exact match, we cannot ignore it
         */

        if (compare == 0) {
            exactMatch = 1;
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("restored being set for  %s", entry->name));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("shouldIgnore:restored being set for  %s", entry->name));
			entry->restored = 1;
            continue;
        }

        /*
         * If buf is parent of the object that needs to be restored, extract it.
         */

        cp = strstr(entry->name, buf);
        if (cp == entry->name &&
            ((entry->name[bufLen - 1] == '/' || cp[bufLen] == '/'))) {
            if (strcmp(entry->name, entry->newPath) == 0) {
                extractChild = 1;
            } else {
                //Check if component is part of newPath
                if (entry->newPathLen <= entry->nameLen
                 && bufLen + entry->newPathLen > entry->nameLen
                 && memcmp(entry->name + entry->nameLen - entry->newPathLen,
                            entry->newPath, entry->newPathLen) == 0)
                {
                    extractParent = 1;
                }                   
                renameChild = 1;
            }
            continue;
        }

        //If this is descendent of a directory which is to be restore,
        //restore it
        cp = strstr(buf, entry->name);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("cp after buf-name %s\n", cp));
        if (cp == buf &&
            ((entry->name[entry->nameLen - 1] == '/' ||
             cp[entry->nameLen] == '/'))) {
            // Search further if there is an exact match
            extractChild = 1;
            continue;
        }
    }

    /*
     * No pathnames are specified
     */
    if (!tried) {
        return 1;
    }
    if (extractChild || extractParent || exactMatch) {
        return 1;
    }
    return 0;
}

/*
 * I/O Routines
 */

EXPORTED fs_Status
fs_readDataTape(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext,
    char                *buf
)
{
    fs_Status           status;
    uint32              requestSize = FS_DR_TBSIZE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("In fs_readDataTape"));
    if (restoreContext->abort) {
        return FS_ERR_IO;
    }

    if ((restoreContext->iobufLen - restoreContext->curOffset) >=
        requestSize) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("readdata:curOffset %d request size %d",
                 restoreContext->curOffset, requestSize));
        memcpy(buf, &restoreContext->iobuf[restoreContext->curOffset],
               requestSize);
        restoreContext->curOffset += requestSize;
		if (restoreContext->curOffset == restoreContext->iobufLen/2 &&
			restoreContext->pendingIo < FS_MAX_TAPE_READ_PENDING) {
			fs_ndmpReadTape(context, restoreContext);
		}
        return FS_ERR_NONE;
    }
    status = fs_ndmpFetchTapeData(context, restoreContext);
    if (status != FS_ERR_NONE) {
        return status;
    }

#if 0
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("read data:curOffsed %lld iobufLen %lld",
             (uint64)restoreContext->curOffset,
             (uint64)restoreContext->iobufLen));
#ifndef FS_STANDALONE
    status = fs_ndmpReadTape(context, restoreContext);
    if (status != FS_ERR_NONE) {
        return status;
    }
#else
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Reading tape"));
    i = read(restoreContext->fd, restoreContext->iobuf,
             restoreContext->iobufLen);
    if (i == 0) {
        FS_ASSERT(0);
    }
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("read data:returned from read"));
#endif
#endif
    restoreContext->curOffset = 0;
    memcpy(buf, &restoreContext->iobuf[restoreContext->curOffset], requestSize);
    restoreContext->curOffset += requestSize;
    return FS_ERR_NONE;
}

EXPORTED fs_Status
fs_readTape(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext
)
{
    fs_Status           status;
    uint32              requestSize = FS_DR_TBSIZE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_readTape"));
    if (restoreContext->abort) {
        return FS_ERR_IO;
    }

    if ((restoreContext->iobufLen - restoreContext->curOffset) >=
        requestSize) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("curOffset %d request size %d",
                                         restoreContext->curOffset,
                                         requestSize));
        memcpy(&restoreContext->record.u.dummy[0],
              &restoreContext->iobuf[restoreContext->curOffset], requestSize);
        restoreContext->curOffset += requestSize;

		/*
		 * Read ahead tape - will get scheduled when we relinquish
		 * processor. We have a slack of 4 more than max pending tape
		 * requests
		 */
		if (restoreContext->curOffset == restoreContext->iobufLen/2 &&
			restoreContext->pendingIo < FS_MAX_TAPE_READ_PENDING) {
			fs_ndmpReadTape(context, restoreContext);
		}
        return FS_ERR_NONE;
    }
    status = fs_ndmpFetchTapeData(context, restoreContext);
    if (status != FS_ERR_NONE) {
        return status;
    }
#if 0

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("curOffsed %lld iobufLen %lld",
             (uint64)restoreContext->curOffset, (uint64)restoreContext->iobufLen));
#ifndef FS_STANDALONE
    status = fs_ndmpReadTape(context, restoreContext);
    if (status != FS_ERR_NONE) {
        return status;
    }
#else
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Reading tape"));
    i = read(restoreContext->fd, restoreContext->iobuf,
             restoreContext->iobufLen);
    if (i == 0) {
        FS_ASSERT(0);
    }
#endif
#endif
    restoreContext->curOffset = 0;
    memcpy(restoreContext->record.u.dummy,
          &restoreContext->iobuf[restoreContext->curOffset], requestSize);
    restoreContext->curOffset += requestSize;
    return FS_ERR_NONE;
}

EXPORTED fs_Status
fs_restorePushBackRead(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext
)
{

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_readPushBack"));
    restoreContext->curOffset -= FS_DR_TBSIZE;
    memcpy(&restoreContext->iobuf[restoreContext->curOffset],
          restoreContext->record.u.dummy, FS_DR_TBSIZE);
    return FS_ERR_NONE;
}

EXPORTED fs_Status
fs_restore(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext
)
{

    fs_Status               status;
    fs_DumpGenericHeader_t  *header;
    fs_DumpStartHeader_t    *start;
    int                     level;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    fs_restoreInvalidateTxrxCache(context);
    
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_restore"));

    //Create current working directory if doesn't exist
    if (restoreContext->cwd)    {
        status = fs_restoreCreateCwd(context, restoreContext);
        if (status != FS_ERR_NONE)  {
            return status;
        }
    } else {
        restoreContext->cwdInum = FS_ROOT_INODE;
    }
        
    /*
     * Read the tape to determine the format of data
     */

    status = fs_readTape(context, restoreContext);
    header = (fs_DumpGenericHeader_t *)&restoreContext->record;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("headerop is %x start is %x\n",
                  header->op, FS_DR_TAPE_START));
    if (header->op != FS_DR_TAPE_START) {
        /*
         * Send a message indicating invalid tape format
         */
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Not start record"));
        fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               FS_ERR_NONE, NULL,
                               "Invalid tape - Not start record.\n");

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Invalid tape - not start record\n"));
        return FS_ERR_INVAL;
    }
    start = (fs_DumpStartHeader_t *)header;
    if (start->magic != FS_DR_MAGIC) {
        /*
         * Send a message indicating invalid tape format
         */
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Invalid magic"));
        fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               FS_ERR_NONE, NULL,
                               "Invalid tape - Invalid magic.\n");

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("In invalid tape magic\n"));
        return FS_ERR_INVAL;
    } 
    if (start->header.version != FS_DR_VERSION) {
        /*
         * Send a message indicating invalid tape version
         */
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Invalid magic"));
        fs_ndmpSendRestoreLog(context, restoreContext,
                               NDMP_LOG_MSG_TYP_ERR, FS_ERR_INVAL,
                               FS_ERR_NONE, NULL,
                               "Unsupported version\n");

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Unsupported version\n"));
        return FS_ERR_INVAL;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Checking for level\n"));
    level = start->level;

    if (level > 0) {

        /*
         * Incremental dump
         */
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("Doing incremental restore\n"));
        status = fs_incrRestore(context, restoreContext);
    } else {
        /*
         * Full/Partial dump
         */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Doing full restore\n"));
        status = fs_fullRestore(context, restoreContext);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Status is %d", status));
    }

	/*
 	 * Ensure that all the oustanding  transactions are logged
 	 */
    
    //Drop the modify lock
    fs_unlockShared(context, &context->fileSystem->modifyLock);            
    context->flags &= ~FS_CONTEXT_MODIFY_LOCK_ACQUIRED;

	fs_freeze(context);
	fs_thaw(context);
    
    //Get it again
    fs_lockShared(context, &context->fileSystem->modifyLock, NULL);
    context->flags |= FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
    fs_restoreInvalidateTxrxCache(context);
    return status;
    
}

EXPORTED void
fs_cleanRestoreContext(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext,
    boolean             isException)
{
    fs_RestoreNameList_t    *nameEntry;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_cleanRestoreContext"));

    //Free name list entries
    DLL_REMOVE_HEAD(&restoreContext->nameList, nameEntry, link);
    while (nameEntry != NULL) {
        fs_ramDealloc(FS_RAM_RESTORE_NAME, nameEntry);
        DLL_REMOVE_HEAD(&restoreContext->nameList, nameEntry, link);
    }
}

PRIVATE fs_Status
fs_drainDir(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    uint64              id
)
{
    fs_Status           status = FS_ERR_NONE;
    fs_DumpGenericHeader_t  *header;
    fs_DumpOperation_t      *operation;
    fs_DumpDataExtension_t  *extension;
    uint64              i;
    fs_DumpExtent_t     *drExtent;
    uint32              numExtents;
    uint64              blocks;
	int					objsize;
    int                 nHeaderBlocks = 1;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

	FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In DrainDir for %lld\n", id));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_drainDir"));
more:
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Reading tape block"));
    status = fs_readTape(context, restoreContext);
    if (status != FS_ERR_NONE) {
        return status;
    }
    header = (fs_DumpGenericHeader_t *)&restoreContext->record;
    if (header->op == FS_DR_DIR_END) {
        if (header->headerId == id) {
            return status;
        }
		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("DIR header Id %lld headerId %lld\n", id, header->headerId));
    }
    switch (header->op) {

    case FS_DR_CREATE_DIR:
    case FS_DR_CREATE_CHAR:
    case FS_DR_CREATE_SOCK:
    case FS_DR_CREATE_BLOCK:
    case FS_DR_CREATE_FIFO:
    case FS_DR_CREATE_FSET:
		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Draining DIR,CHAR... %d header Id %lld\n", header->op,
			header->headerId));
        goto more;
    case FS_DR_CREATE_SYML:
    case FS_DR_CREATE_LINK:

		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Draining SYML,LNK... %d header Id %lld\n", header->op,
			header->headerId));
		/*
		 * There is assumption about 1k pathnames in this code. Address
		 * this when we start supporting larger than 1k pathnames
		 */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Draining symlink\n"));
        operation = (fs_DumpOperation_t *)header;
        nHeaderBlocks = (operation->len + sizeof (fs_DumpOperation_t) +
                         FS_DR_TBSIZE - 1) >> FS_DR_TBSHIFT;
        if (nHeaderBlocks > 1) {
            while (nHeaderBlocks > 1) {
                status = fs_readTape(context, restoreContext);
                if (status != FS_ERR_NONE) {
                    return status;
                }
                nHeaderBlocks--;
            }
            
        } else  if (header->dataExtents == 0) {
            return FS_ERR_NONE;
        }
        // Read the next block containing symlink contents
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
            return status;
        }
#if 0
        operation = (fs_DumpOperation_t *)header;
        if (header->dataExtents == 0) {
            goto more;
        }
        numExtents = 1;
		objsize = (operation->len + sizeof (fs_DumpOperation_t) + 7) & ~7;
        drExtent = (fs_DumpExtent_t *)(restoreContext->record.u.dummy + objsize);
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
            blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readTape(context, restoreContext);
            if (status != FS_ERR_NONE) {
                return status;
            }
        }
#endif
        goto more;
    case FS_DR_CREATE_FILE:
		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Draining FILE... %d header Id %lld\n", header->op,
			header->headerId));
        operation = (fs_DumpOperation_t *)header;
        if (header->dataExtents == 0) {
            goto more;
        }
        numExtents = header->dataExtents;
		objsize = (operation->len + sizeof (fs_DumpOperation_t) + 7) & ~7;
        drExtent = (fs_DumpExtent_t *)(restoreContext->record.u.dummy + objsize);
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
            blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readTape(context, restoreContext);
            if (status != FS_ERR_NONE) {
                return status;
            }
        }
        goto more;
    case FS_DR_DATA:
		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Draining DATA... %d header Id %lld\n", header->op,
			header->headerId));
        extension = (fs_DumpDataExtension_t *)header;
        if (header->dataExtents == 0) {
            goto more;
        }
        numExtents = header->dataExtents;
        drExtent =  extension->extents;
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
            blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readTape(context, restoreContext);
            if (status != FS_ERR_NONE) {
                return status;
            }
        
        }
        goto more;

    default:
		FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Draining Default... %d header Id %lld\n", header->op,
			header->headerId));
        goto more;
    }
}
PRIVATE fs_Status
fs_drainObject(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    uint64              id
)
{
    fs_Status           status = FS_ERR_NONE;
    fs_DumpGenericHeader_t  *header;
    fs_DumpOperation_t      *operation;
    fs_DumpDataExtension_t  *extension;
    uint64              i;
    fs_DumpExtent_t     *drExtent;
    uint32              numExtents;
    uint64              blocks;
    int                 nHeaderBlocks = 1;
	int					objsize;
	char				*buf;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_drainObject"));
	buf = fs_ramAlloc(FS_RAM_RESTORE_DATA, FS_DR_TBSIZE);
more:

    header = (fs_DumpGenericHeader_t *)&restoreContext->record;
    if (header->headerId != id) {
        status = fs_restorePushBackRead(context, restoreContext);
		fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
        return status;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Draining op %d id:%lld\n", header->op, header->headerId));
    switch (header->op) {

    case FS_DR_CREATE_SOCK:
    case FS_DR_CREATE_BLOCK:
    case FS_DR_CREATE_FIFO:
    case FS_DR_CREATE_FSET:
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Draining special objects\n"));
        operation = (fs_DumpOperation_t *)header;
        nHeaderBlocks = (operation->len + sizeof (fs_DumpOperation_t) +
                         FS_DR_TBSIZE - 1) >> FS_DR_TBSHIFT;
        if (nHeaderBlocks > 1) {
            while (nHeaderBlocks > 1) {
                status = fs_readDataTape(context, restoreContext, buf);
                if (status != FS_ERR_NONE) {
				    fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                    return status;
                }
                nHeaderBlocks--;
            }
            
        }

        // Read the next block
        status = fs_readTape(context, restoreContext);
	    fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
        return status;
    case FS_DR_CREATE_SYML:
    case FS_DR_CREATE_LINK:

		/*
		 * There is assumption about 1k pathnames in this code. Address
		 * this when we start supporting larger than 1k pathnames
		 */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Draining symlink\n"));
        operation = (fs_DumpOperation_t *)header;
        nHeaderBlocks = (operation->len + sizeof (fs_DumpOperation_t) +
                         FS_DR_TBSIZE - 1) >> FS_DR_TBSHIFT;
        if (nHeaderBlocks > 1) {
            while (nHeaderBlocks > 1) {
                status = fs_readDataTape(context, restoreContext, buf);
                if (status != FS_ERR_NONE) {
                    return status;
                }
                nHeaderBlocks--;
            }
            
        } else  if (header->dataExtents == 0) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return FS_ERR_NONE;
        }
        // Read the next block containing symlink contents
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return status;
        }

		// Read the next block
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return status;
        }
        goto more;
    case FS_DR_NAME_ONLY:
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Draining name\n"));
        operation = (fs_DumpOperation_t *)header;
        nHeaderBlocks = (operation->len + sizeof (fs_DumpOperation_t) +
                         FS_DR_TBSIZE - 1) >> FS_DR_TBSHIFT;
        if (nHeaderBlocks > 1) {
            while (nHeaderBlocks > 0) {
                status = fs_readTape(context, restoreContext);
                if (status != FS_ERR_NONE) {
					fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                    return status;
                }
                nHeaderBlocks--;
            }
            
        }
        numExtents = 1; // always 1 extent for symbolic link content
                        // on tape
		objsize = (operation->len + sizeof (fs_DumpOperation_t) + 7) & ~7;
        drExtent = (fs_DumpExtent_t *)(restoreContext->record.u.dummy +
                                       objsize);
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
            //blocks += (drExtent->len >> FS_DR_TBSHIFT);
            blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readDataTape(context, restoreContext, buf);
            if (status != FS_ERR_NONE) {
				fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                return status;
            }
        }
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return status;
        }
        goto more;
    case FS_DR_CREATE_FILE:
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                  ("Draining file\n"));
        operation = (fs_DumpOperation_t *)header;
        nHeaderBlocks = 
			 (((operation->len + sizeof (fs_DumpOperation_t) + 7) & ~7) +
                         FS_DR_TBSIZE - 1) >> FS_DR_TBSHIFT;
        if (nHeaderBlocks > 1) {
			FS_ASSERT(0);
            while (nHeaderBlocks > 1) {
                status = fs_readTape(context, restoreContext);
                if (status != FS_ERR_NONE) {
					fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                    return status;
                }
                nHeaderBlocks--;
            }
            goto more;
        }
        if (header->dataExtents == 0) {
				status = fs_readTape(context, restoreContext);
				if (status != FS_ERR_NONE) {
					fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
					return status;
				}
            goto more;
        }
        numExtents = header->dataExtents;
		objsize = (operation->len + sizeof (fs_DumpOperation_t) + 7) & ~7;
        drExtent = (fs_DumpExtent_t *)(restoreContext->record.u.dummy +
                                       objsize);
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
            //blocks += (drExtent->len >> FS_DR_TBSHIFT);
            blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readDataTape(context, restoreContext, buf);
            if (status != FS_ERR_NONE) {
				fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                return status;
            }
        }
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return status;
        }
        goto more;
    case FS_DR_DATA:
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Draining data\n"));
        extension = (fs_DumpDataExtension_t *)header;
        if (header->dataExtents == 0) {
            goto more;
        }
        numExtents = header->dataExtents;
        drExtent =  extension->extents;
        blocks = 0;
        for (i = 0; i < numExtents; i++) {
			blocks += drExtent->len;
			drExtent++;
        }
        for (i = 0; i < blocks; i++) {
            status = fs_readDataTape(context, restoreContext, buf);
            if (status != FS_ERR_NONE) {
				fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
                return status;
            }
        
        }
        status = fs_readTape(context, restoreContext);
        if (status != FS_ERR_NONE) {
			fs_ramDealloc(FS_RAM_RESTORE_DATA, buf);
            return status;
        }
        goto more;

    default:
        goto more;
    }
}

PRIVATE fs_Inode *
fs_parsePath(
    fs_Context              *context,
    fs_RestoreContext_t    *restoreContext,
    char                    *buf
)
{
    char        *p, *c;
    fs_Inode    *component = NULL;
    fs_Inode    *parentDir = NULL;
    fs_path_t   *path;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_parsePath"));
    path = fs_makePath(buf);
    //parentDir = I_GET_EXCL(0, FS_ROOT_INODE);
    parentDir = I_GET_EXCL(0, restoreContext->cwdInum);
	p = fs_getNextComponent(path);
	while (p != NULL) {

		/*
		 * Always look in live filesystem
		 */

		component = fs_lookup(context, context->fileSystem, 0, parentDir, p);
		if (component == NULL) {
			c = fs_getNextComponent(path);

			/*
			 * If it is not the end of the path, path is invalid
			 * otherwise we need to create component
			 */

			if (c != NULL) {
				fs_inodeReleaseExclusive(context, parentDir);
				fs_destroyPath(path);
				return NULL;
			}


			break;
		}

		/*
		 * Proceed with the next component in the path
		 */

		FS_ASSERT(fs_lockOwnedExclusive(context, &component->lock));
		p = fs_getNextComponent(path);
		if (p != NULL) {
			/*
			 * If the current parent is not a directory then
		 	 * the path is invalid
			 */

			if (!FS_I_IS_DIR(component->dinode.attrs)) {
				fs_inodeReleaseExclusive(context, component);
				fs_destroyPath(path);
				return NULL;
			}

			parentDir = component;
			component = NULL;
		}
	}

    //FS_ASSERT(component != NULL);
    fs_destroyPath(path);
    return component;
}

PRIVATE fs_Status 
fs_restoreCreateCwd(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext)
{
    char        *p;
    fs_Inode    *component = NULL;
    fs_Inode    *parentDir = NULL;
    fs_path_t   *path;
    fs_Status   status;
    fs_DRAttr_t attr;

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In restoreCreateCwd"));
    path = fs_makePath(restoreContext->cwd);
    component = parentDir = I_GET_EXCL(0, FS_ROOT_INODE);
	p = fs_getNextComponent(path);
	while (p != NULL) {
		component = fs_lookup(context, context->fileSystem, 0, parentDir, p);
		if (component == NULL) {
			//we need to create component
            status = fs_restoreMkDir(context, restoreContext, &parentDir,
                                     p, strlen(p), &component);
            if (status == FS_ERR_NONE)  {
                attr.uid = parentDir->sd->u.nfs.uid;
                attr.gid = parentDir->sd->u.nfs.gid;
                attr.atime = component->dinode.atime;
                attr.mtime = component->dinode.mtime;
                attr.ctime = component->dinode.ctime;
                attr.mode = (parentDir->sd->u.nfs.mode & FS_IMODE_MASK);
                attr.simpleAttrs = parentDir->dinode.userAttr;
                status = fs_restoreSetAttr(context, restoreContext, &component, 
                                           &attr);
                if (status != FS_ERR_NONE)  {
                    fs_inodeReleaseExclusive(context, component);
                } else {
                    fs_inodeReleaseExclusive(context, parentDir);
                }
            }
            if (status != FS_ERR_NONE)  {
				fs_inodeReleaseExclusive(context, parentDir);
				fs_destroyPath(path);
                return status;
            }
        } else {
			// If the current element is not a directory then cwd is invalid
			if (!FS_I_IS_DIR(component->dinode.attrs)) {
				fs_inodeReleaseExclusive(context, component);
				fs_destroyPath(path);
				return FS_ERR_NOTDIR;
			}
        }

		// Proceed with the next component in the path
		FS_ASSERT(fs_lockOwnedExclusive(context, &component->lock));
		p = fs_getNextComponent(path);
		if (p != NULL) {
			parentDir = component;
			component = NULL;
		}
	}

    FS_ASSERT(component != NULL);
    fs_destroyPath(path);
    restoreContext->cwdInum = component->inum;
	fs_inodeReleaseExclusive(context, component);
    return FS_ERR_NONE;
}

/**
 * flush the txrx attribute cache
 */
void
fs_restoreInvalidateTxrxCache(fs_Context *context)
{
    int32 rc;
    fs_MsgCacheInvReq msg;

    msg.hdr.requestType = FS_REQ_CACHE_INV;
    msg.hdr.len = sizeof(msg);

    msg.invType = FS_CACHE_INV_TYPE_ALL;

    rc = eee_sendMessage((void *) &msg, sizeof(msg),
                         PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                   NFX_NFP_TXRX_ANY,
                                           eee_getApplicationId("nfs_srvr"),
                                                   EEE_CONTROL,
                                                   NFX_COS_HIGH,
                                                   0),
                         fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
}

#endif  //SSC
