/*
 * Name:  fs-super.h
 *
 * RCS:  $Id: fs-super.h,v 1.39 2002/12/05 21:08:58 bob Exp $
 *
 * Description:  Filesystem superblock definition.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(FS_SUPER_H)
#define FS_SUPER_H


/**
 * The on-disk superblock.
 */
struct fs_SuperBlock
{
    uint32          magic;              // our magic number
#define FS_SUPER_MAGIC  0xef334979      // any magic to this?
    uint32          version;            // version for this filesystem
#define FS_SUPER_CURRENT_VERSION   20
    uint32          crc32;              // crc
    uint32          flags;              // recovery state flags, etc.
#define FS_SUPER_FLAG_CLEAN     0x0001  // filesystem is clean
#define FS_SUPER_FLAG_MKFS      0x0002  // filesystem is being mkfs'd
#define FS_SUPER_FLAG_MUST_EEK  0x0004  // filesystem is must be eek'd
    uint32          blkSize;            // num bytes for blks
    uint32          spareword;          // was numRsvd, never used
    uint64          numBlks;            // total num blks in fs
    uint64          blksUsed;           // blocks in use
    uint64          inodesUsed;         // inodes in use
    uint32          blockInodeRatio;    // ratio of blocks to inodes in fs
    uint32          nbrMount;
    uint32          nbrUmount;
    uint32          nbrDirtyMount;
    uint32          nbrMountFailedLogDev;
    uint32          nbrMountFailedReplay;
    uint32          nbrMountFailedEek;
    uint32          nbrMountFailedSnap;
    uint32          mkfsTime;           // time of FS creation
    uint32          logInitTimeSec;     // time (secs - nsec) log was last initialized
    uint32          logInitTimeNsec;

    uint32          numFset;            // Number of fsets in this filesystem
    uint32          logDevType;         // int volid, ext vol, nvram, etc
                                        // default: Internal vol (reserve vol space)
    uint32          logSize;            // Size of the log currently used for this volume.
    evm_volId_t     logVolId;           // Volume id for ext vol logging
                                        //   only valid for FS_LOG_DEVTYPE_VOLID
    uint64          snapshots;          // bitmap of snapshots in use
    fs_SnapState    snapState;          // in-progress snapshot state

    char            mountName[FS_NAME_MAX + 1]; // symbolic name fs mounted as
    uint32          logSpaceSize;       // Size of space reserved on this volume for
                                        // logging (may or may not be used for the log)

#define FS_MOUNT_NO_FAIL          0
#define FS_MOUNT_FAIL_LOGDEV      1
#define FS_MOUNT_FAIL_REPLAY      2
#define FS_MOUNT_FAIL_EEK         3
#define FS_MOUNT_FAIL_SNAP        4
    uint32          mountTime[FS_SUPER_NBR_MOUNTTIME];   // time of last N mount operations
    uint32          mountState[FS_SUPER_NBR_MOUNTTIME];  // status of last N mount operations
    uint32          mountFail[FS_SUPER_NBR_MOUNTTIME];   // reason why it failed
    uint32          umountTime[FS_SUPER_NBR_MOUNTTIME];  // time of last N controlled umount operations
#define FS_UMOUNT_NORMAL               0
#define FS_UMOUNT_FAIL_LOG_NOT_EMPTY   1
    uint32          umountState[FS_SUPER_NBR_MOUNTTIME]; // status of last N controlled umount operations

    uint64          logStartOffset;     // For FS_LOG_DEVTYPE_SELF, byte offset
                                        // where log device starts.

    uint32          growTime[FS_SUPER_NBR_MOUNTTIME]; // time of last N grows
    uint32          nbrGrow;            // Count of times fs grown.
    uint8           percentMinFree;     // Percentage of always avail blocks
    uint8           sharedReadDisallow; // Prevent NFS reads if file opened
                                        // with no-shared-read
    uint16          spare2;             // 2 byte spare
    uint64          inodeFileBlks;      // Blocks in use by inode file.
    uint64          inodeFileSize;      // Size of the inode file.
    uint32          codePage;           // placeholder for cifs support
#define FS_CODEPAGE_DEFAULT     1252
#define FS_CODEPAGE_ENGLISH_US  1252
    uint32          locale;             // placeholder for cifs support
#define FS_LOCALE_DEFAULT       0x409
#define FS_LOCALE_USA           0x409
    uint64          metaNodeExtStartBlk;// Reserved for future - starting block of metaNode extension
                                        // extent for support of > 64 snapshots.
    uint32          spare[48];          // spare data, initialized to 0. Allows to grow, with same CRC value
};

#endif /* FS_SUPER_H */












