/*
 * Name:  mcpu-sim.h
 *
 * RCS:  $Id: mcpu-sim.h,v 1.13 2002/03/21 03:20:35 bobonfrogger Exp $
 *
 * Description:  Definitions for providing a UNIX based Eagle simulatator for
 *               filesystem <-> mcpu testing.
 * 
 *
 * Date Created:  7/11/2001
 */

#if !defined(MCPU_SIM_H)
#define MCPU_SIM_H

#include "fs-includes.h"
#include "fs-sim.h"

#define MCPU_CONFIRM_COD     5000     // From MCPU to FS, to confirm that COD is valid
#define MCPU_LUC_WRITE_REQ   5001     // Rx form LUC, write into SAC
#define MCPU_LUC_READ_REQ    5002     // msg from FS, read from SAC and send to LUC
#define MCPU_SHOW_STAT_REQ   5003

typedef struct mcpu_luc_msg_req
{
	/********************
     ** BEGIN WARNING!!!!  - this must be kept in sync with fs-api.h
     ** and fssf-api.h and mcpu-sim.h message headers
	 ********************/
    uint32      requestType;            // Type of action being requested
    int32       len;
#if defined(FS_STANDALONE)
    int32       src;    // match fs_simMsgHdr
    int32       dest;   // match fs_simMsgHdr
    int32       nfsReqNum;
    int32       sockIndex;
#endif
	/********************
     ** END WARNING
	 ********************/
    nfx_cod_entry_t   cod[NFX_MAX_COD_MSG];  // Returned buffers
    uint32      nbrUsedCod;
    uint32      writeLength;
    char        data[0];
} mcpu_luc_msg_req_t;


#define MCPU_SHOW_SUMMARY    1
#define MCPU_SHOW_DETAIL     2
#define MCPU_SHOW_ALL        3

typedef struct mcpu_show_stat_req
{
	/********************
     ** BEGIN WARNING!!!!  - this must be kept in sync with fs-api.h
     ** and fssf-api.h and mcpu-sim.h message headers
	 ********************/
    uint32      requestType;            // Type of action being requested
    int32       len;
#if defined(FS_STANDALONE)
    int32       src;    // match fs_simMsgHdr
    int32       dest;   // match fs_simMsgHdr
    int32       nfsReqNum;
    int32       sockIndex;
#endif
	/********************
     ** END WARNING
	 ********************/
    nfx_cod_t   cod;
    int         level;
} mcpu_show_stat_req_t;



/*
 * mcpu_sim  API
 *
 */

/*
 * Return the total SAC size.
 */
uint64 mcpu_sim_totalSacSize();

/* 
 * init
 * returns NFX_OK if OK
 *         no error cases foreseen
 */

int32 mcpu_sim_sbm_init() ;

/*
 * frees completely the COD and all related structures
 * free the HMT entry, the list of DBL entries, and
 * all related records in the SAC
 * If no HMT index is available an error msg is printed
 * Returns NFX_OK if OK
 *         else negative
 */

int32 mcpu_sim_sbm_freeCOD (nfx_cod_t   COD) ;


/*
 * Simulates the data comming from the LUC,
 * Writes all data in the SAC, while allocating all
 * structures for the HMT and BDL lists.
 * The COD is the input reference.
 */

int mcpu_sim_rx_data_from_LUC (
    mcpu_luc_msg_req_t *mcpuReq
);

/*
 * Simulates the data read response form FS,
 * pickup the data form tre SAC with given COD,
 * and transmits to the LUC.
 * In the simulation, this is TX to the NFS-TX thread
 * The COD is the input reference.
 * The COD is not freed.
 */

int mcpu_sim_tx_data_to_LUC (
    fs_MsgReadResp  **fsReadResp
);


/* 
 * simulation function to write a cod (user data) into a file
 */
ssize_t
mcpu_sim_evm_write_data (
    nfx_cod_t   COD,
    int fd, 
    evm_volSize_t count);

/* 
 * simulation function to store to user data after reading 
 * from file into the SAC, and return the newly generated COD
 */
int 
mcpu_sim_evm_read_data (
    char *data, 
    evm_volSize_t length,
    nfx_cod_t *COD);


int32
mcpu_sim_split (
    nfx_cod_t   src_COD,
    uint32      src_offset,
    nfx_cod_t   dest_COD);

int32
mcpu_sim_merge (
    nfx_cod_t   src_COD,
    uint32      src_len,
    uint32      src_offset,
    nfx_cod_t   dst_COD,
    uint32      dst_offset);

int32
mcpu_sim_copy (
    nfx_cod_t   src_COD,
    uint32      src_len,
    uint32      src_offset,
    nfx_cod_t   dst_COD,
    uint32      dst_offset);

int32
mcpu_sim_cod_null (
    nfx_cod_t   COD,
    uint32      len,
    uint32      offset);


/*
 * The support function to show all data structures
 * (HMT, BDL) for given COD
 *
 */

int32 mcpu_sim_sbm_showCOD (nfx_cod_t COD, int32 level) ;

int32 mcpu_sim_sbm_show_all (int32 level) ;
     
int32 mcpu_sim_sbm_show_sum (void);


/*
 * end of API
 */





#endif /* MCPU_SIM_H */








