/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-tx.c
 *
 * RCS:  $Id: gmc-tx.c,v 1.19 2002/02/04 18:30:47 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Emulation routines for the GMC
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/24/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#ifndef SBM_REMOVE_GMC
#include <exports/cm-api.h>
#include "../sm-sbm/sbm-gmc.h"
#include "../sm-sbm/sbm.h"
#include "gmc.h"

#define GMC_TO_NEXT_BD(_p, _end, _start) {		\
		(uint32)(_p) += 8;						\
		(uint32)(_p) &= ~7;						\
		if ((_p) >= (_end))						\
			(_p) = (_start);					\
}

#define GMC_TO_NEXT_TXQ(_p, _end, _start) {		\
		(uint32)(_p) += 32;						\
		(uint32)(_p) &= ~31;					\
		if ((_p) >= (_end))						\
			(_p) = (_start);					\
}

/*-----------------------------------------------------------------
 * Name:	gmc_process_snim_txq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_process_snim_txq()
{
	sbm_txqQueueEntry_t		* txq, * next_to_last_txq;
	sbm_txq_bd_t			* bd;
	sbm_txqSendReadReq_t	* rr;
	sbm_txqSendDataToSNIM_t	* snim;
	enet_hdr_t				* enet_hdr;
	agile_hdr_t				* agile_hdr;
	gmc_app_hdr_t			* gmc_hdr;
	uchar8					* payload_ptr, * msg_ptr, *data_ptr;
	uchar8					* cont_msg, * buf;
	uchar8					* sac;
	uint32					control;
	uint32					tot_len, msg_len, enet_msg_len, data_len;
	uint32					cl_cnt, i;
	uint32					bd_cnt;
	int32					rc;

	txq = (sbm_txqQueueEntry_t *)(gmc->txq_snim_cntl.head);

	if ((uint32)txq & CACHELINE_MASK)
		printf("GMC : ERROR: GMC at misaligned TxQ entry address, 0x%x\n", txq);

	/*
	 * The control word is last word in 8-word entry
	 */
	control = txq->read_req.control;

	while (control & TXQ_GMC_OWN) {

		cl_cnt = (control & TXQ_CACHELINE_MASK) >> TXQ_CACHELINE_SHIFT;

		switch (control & TXQ_COMMAND_MASK)  {

			case TXQ_CMD_SEND_READ_REQ:
			{

				rr = (sbm_txqSendReadReq_t *)txq;
				cont_msg = (uchar8 *)rr;

				GMC_TO_NEXT_TXQ(cont_msg,
								(uchar8 *)gmc->txq_snim_cntl.end,
								(uchar8 *)gmc->txq_snim_cntl.start);

				msg_len = rr->message_len;
#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				tot_len = msg_len;
#else
				tot_len = msg_len + sizeof(gmc_app_hdr_t);
#endif
#else
				enet_msg_len = msg_len + sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t);
				tot_len = enet_msg_len + sizeof(enet_hdr_t);
#endif

				GMC_TRACE(("GMC : TXQ_CMD_SEND_READ_REQ: txq = 0x%x, msg_len = %d",
																rr, msg_len));

				buf = (uchar8 *)eee_ramAlloc(tot_len);
				if (buf == NULL) {
					printf("could not allocate memory for message\n");
					printf("leaving Read Req message in queue\n");
					return;
				}

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				payload_ptr = (uchar8 *)buf;
#else
				gmc_hdr = (gmc_app_hdr_t *)buf;
				payload_ptr = (uchar8 *)(buf + sizeof(gmc_app_hdr_t));
#endif
#else
				enet_hdr = (enet_hdr_t *)buf;
				agile_hdr = (agile_hdr_t *)(buf + sizeof(enet_hdr_t));
				gmc_hdr = (gmc_app_hdr_t *)(buf + sizeof(enet_hdr_t) +
														sizeof(agile_hdr_t));
				payload_ptr = (uchar8 *)(buf + sizeof(enet_hdr_t) +
								sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t));

				/*
				 * create enet hdr
				 */
				memcpy(enet_hdr->DA, rr->DA, 6);
				memcpy(enet_hdr->SA, gmc_regs->own_mac_addr, 6);
				enet_hdr->vlan = rr->vlan;
				enet_hdr->type = enet_msg_len;

				/*
				 * create agile hdr
				 */
				agile_hdr->pad = 0;
				agile_hdr->dest_port = rr->dest_port;
				agile_hdr->src_port = rr->source_port;
#endif

#ifndef NO_GMC_APP_NO_TX
				/*
				 * create GMC app hdr
				 */
				gmc_hdr->control = 0;
				gmc_hdr->id.tid = rr->TID;
#endif

				/*
				 * attach control message
				 */
#if 0
				memcpy(payload_ptr, cont_msg, msg_len);
#else
				if ((uint32)cont_msg + msg_len >=
									(uint32)gmc->txq_snim_cntl.end) {
					for (i=0; i<msg_len; i++) {
						*(payload_ptr+i) = *cont_msg;
						if (++cont_msg == gmc->txq_snim_cntl.end)
							cont_msg = gmc->txq_snim_cntl.start;
					}
				}
				else
					memcpy(payload_ptr, cont_msg, msg_len);
#endif

				/*
				 * Send message across "data plane"
				 */
GMC_TRACE(("GMC : sending %d bytes to 0x%x\n", tot_len, rr->dest_port));
#ifdef USE_DP_PROXY
				rc = dp_proxy_tx_flat_buffer(buf,
											tot_len,
											rr->dest_port,
											rr->source_port);
#else
				rc = eee_sendMessage(buf,
										tot_len,
										rr->dest_port,
										rr->source_port);
#endif
				if (rc != NFX_OK)
					printf("eee_sendMessage error, 0x%x\n", rc);

				eee_ramDealloc(buf);

				/*
				 * advance control pointers
				 */
				for (i=0; i<cl_cnt; i++) {
					bzero((uchar8 *)txq, 32);
					next_to_last_txq = txq;
					GMC_TO_NEXT_TXQ(txq,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}
				gmc->txq_snim_cntl.head = txq;

				/*
				 * update credit info for MCPU
				 */
				*gmc_regs->txq_snim_credit_addr = (uint32)next_to_last_txq;

				break;
			}

			case TXQ_CMD_SEND_DATA_TO_SNIM:
			{
				snim = (sbm_txqSendDataToSNIM_t *)txq;
				cont_msg = (uchar8 *)snim;

				GMC_TO_NEXT_TXQ(cont_msg,
								(uchar8 *)gmc->txq_snim_cntl.end,
								(uchar8 *)gmc->txq_snim_cntl.start);

				msg_len = snim->message_len;

				/*
				 * Calculate length of SAC data that will be appended later
				 */
				bd_cnt = control & TXQ_BUFCNT_MASK;
				bd = (sbm_txq_bd_t *)cont_msg;
				for (bd = (sbm_txq_bd_t *)cont_msg, i=0;
											i < msg_len;
													i+= sizeof(sbm_txq_bd_t)) {
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}

				for (data_len = 0, i=0; i<bd_cnt; i++) {
					data_len += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				tot_len = msg_len + data_len;
#else
				tot_len = msg_len + data_len + sizeof(gmc_app_hdr_t);
#endif
#else
				enet_msg_len = msg_len + data_len + sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t);
				tot_len = enet_msg_len + sizeof(enet_hdr_t);
#endif

				GMC_TRACE(("GMC : TXQ_CMD_SEND_DATA_TO_SNIM: total_len = %d\n",
																	tot_len));
				GMC_TRACE(("GMC : msg_len = %d, data_len = %d, hdr_len = %d\n",
														msg_len, data_len,
														sizeof(enet_hdr_t) +
														sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t)));

				buf = (uchar8 *)eee_ramAlloc(tot_len);
				if (buf == NULL) {
					printf("could not allocate memory for message\n");
					printf("leaving Read Req message in queue\n");
					return;
				}

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				msg_ptr = (uchar8 *)buf;
				data_ptr = msg_ptr + msg_len;
#else
				gmc_hdr = (gmc_app_hdr_t *)buf;
				msg_ptr = (uchar8 *)(buf + sizeof(gmc_app_hdr_t));
				data_ptr = msg_ptr + msg_len;
#endif
#else
				enet_hdr = (enet_hdr_t *)buf;
				agile_hdr = (agile_hdr_t *)(buf + sizeof(enet_hdr_t));
				gmc_hdr = (gmc_app_hdr_t *)(buf + sizeof(enet_hdr_t) +
														sizeof(agile_hdr_t));
				msg_ptr = (uchar8 *)(buf + sizeof(enet_hdr_t) +
								sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t));
				data_ptr = msg_ptr + msg_len;

				/*
				 * create enet hdr
				 */
				memcpy(enet_hdr->DA, snim->DA, 6);
				memcpy(enet_hdr->SA, gmc_regs->own_mac_addr, 6);
				enet_hdr->vlan = snim->vlan;
				enet_hdr->type = enet_msg_len;

				/*
				 * create agile hdr
				 */
				agile_hdr->pad = 0;
				agile_hdr->dest_port = snim->dest_port;
				agile_hdr->src_port = snim->source_port;
#endif

#ifndef NO_GMC_APP_NO_TX
				/*
				 * create GMC app hdr
				 */
				gmc_hdr->control = 0;
				gmc_hdr->id.tid = snim->TID;
#endif

				/*
				 * attach control message
				 */
#if 0
				memcpy(msg_ptr, cont_msg, msg_len);
#else
				if ((uint32)cont_msg + msg_len >=
									(uint32)gmc->txq_snim_cntl.end) {
					uchar8	* src_ptr = cont_msg;
					for (i=0; i<msg_len; i++) {
						*(msg_ptr+i) = *src_ptr;
						if (++src_ptr == gmc->txq_snim_cntl.end)
							src_ptr = gmc->txq_snim_cntl.start;
					}
				}
				else
					memcpy(msg_ptr, cont_msg, msg_len);
#endif

				/*
				 * add SAC data
				 */
				bd = (sbm_txq_bd_t *)cont_msg;
				for (bd = (sbm_txq_bd_t *)cont_msg, i=0;
											i < msg_len;
													i+= sizeof(sbm_txq_bd_t)) {
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}
				for (i=0; i<bd_cnt; i++) {
					sac = gmc->sac +
								(bd->sac_index * gmc_regs->gmc_sac_block_size);

GMC_TRACE(("GMC : copying %d bytes from index %d, sac 0x%x(+%d) to 0x%x\n",
			bd->length, bd->sac_index, sac, bd->offset, data_ptr));

					memcpy(data_ptr, sac+bd->offset, bd->length);
					data_ptr += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}

				/*
				 * Send message across "data plane"
				 */
GMC_TRACE(("GMC : sending %d bytes from %p to 0x%x\n",
		 tot_len, buf, snim->dest_port));
#ifdef USE_DP_PROXY
				rc = dp_proxy_tx_flat_buffer(buf,
											tot_len,
											snim->dest_port,
											snim->source_port);
#else
				rc = eee_sendMessage(buf,
									tot_len,
									snim->dest_port,
									snim->source_port);
#endif

				if (rc != NFX_OK)
					printf("eee_sendMessage error, 0x%x\n", rc);

				eee_ramDealloc(buf);

				/*
				 * advance control pointers
				 */
				for (i=0; i<cl_cnt; i++) {
					bzero((uchar8 *)txq, 32);
					next_to_last_txq = txq;
					GMC_TO_NEXT_TXQ(txq,
									gmc->txq_snim_cntl.end,
									gmc->txq_snim_cntl.start);
				}
				gmc->txq_snim_cntl.head = txq;

				/*
				 * update credit info for MCPU
				 */
				*gmc_regs->txq_snim_credit_addr = (uint32)next_to_last_txq;

				break;
			}

			default:
			{
				printf("GMC : ERROR: GMC read invalid command, 0x%x, at 0x%x\n",
															control, txq);
			}
		}

		control = txq->read_req.control; 

	}

}	/* gmc_process_snim_txq() */


/*-----------------------------------------------------------------
 * Name:	gmc_process_luc_txq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_process_luc_txq()
{
	sbm_txqQueueEntry_t		* txq, * next_to_last_txq;
	sbm_txq_bd_t			* bd;
	sbm_txqSendDataToLUC_t	* luc;
	enet_hdr_t				* enet_hdr;
	agile_hdr_t				* agile_hdr;
	gmc_app_hdr_t			* gmc_hdr;
	uchar8					* msg_ptr, *data_ptr;
	uchar8					* cont_msg, * buf;
	uchar8					* sac;
	uint32					control;
	uint32					tot_len, msg_len, enet_msg_len, data_len;
	uint32					cl_cnt, i;
	uint32					bd_cnt;
	uint32					test_app_id;
	uint32					dp, sp;
	int32					rc;

	txq = (sbm_txqQueueEntry_t *)(gmc->txq_luc_cntl.head);

	if ((uint32)txq & CACHELINE_MASK)
		printf("GMC : ERROR: GMC at misaligned TxQ entry address, 0x%x\n", txq);

	/*
	 * The control word is last word in 8-word entry
	 */
	control = txq->send_to_luc_req.control;

	while (control & TXQ_GMC_OWN) {

		cl_cnt = (control & TXQ_CACHELINE_MASK) >> TXQ_CACHELINE_SHIFT;

		switch (control & TXQ_COMMAND_MASK)  {

			case TXQ_CMD_SEND_DATA_TO_LUC:
			{
				luc = (sbm_txqSendDataToLUC_t *)txq;
				cont_msg = (uchar8 *)luc;

				GMC_TO_NEXT_TXQ(cont_msg,
								(uchar8 *)gmc->txq_luc_cntl.end,
								(uchar8 *)gmc->txq_luc_cntl.start);

				msg_len = luc->message_len;

				/*
				 * Calculate length of SAC data that will be appended later
				 */
				bd_cnt = control & TXQ_BUFCNT_MASK;
				bd = (sbm_txq_bd_t *)cont_msg;
				for (bd = (sbm_txq_bd_t *)cont_msg, i=0;
											i < msg_len;
													i+= sizeof(sbm_txq_bd_t)) {
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_luc_cntl.end,
									gmc->txq_luc_cntl.start);
				}

				for (data_len = 0, i=0; i<bd_cnt; i++) {
					data_len += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_luc_cntl.end,
									gmc->txq_luc_cntl.start);
				}

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				tot_len = msg_len + data_len;
#else
				tot_len = msg_len + data_len + sizeof(gmc_app_hdr_t);
#endif
#else
				enet_msg_len = msg_len + data_len + sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t);
				tot_len = enet_msg_len + sizeof(enet_hdr_t);
#endif

				GMC_TRACE(("GMC : TXQ_CMD_SEND_DATA_TO_LUC: total_len = %d\n",
																	tot_len));
				GMC_TRACE(("GMC : msg_len = %d, data_len = %d, hdr_len = %d\n",
														msg_len, data_len,
														sizeof(enet_hdr_t) +
														sizeof(agile_hdr_t) +
														sizeof(gmc_app_hdr_t)));

				buf = (uchar8 *)eee_ramAlloc(tot_len);
GMC_TRACE(("GMC: tot_len %d buf @ %p\n", tot_len, buf));
				if (buf == NULL) {
					printf("could not allocate memory for message\n");
					printf("leaving Read Req message in queue\n");
					return;
				}

#ifdef FIX_HDR_STUFF
#ifdef NO_GMC_APP_NO_TX
				msg_ptr = (uchar8 *)buf;
				data_ptr = msg_ptr + msg_len;
#else
				gmc_hdr = (gmc_app_hdr_t *)buf;
				msg_ptr = (uchar8 *)(buf + sizeof(gmc_app_hdr_t));
				data_ptr = msg_ptr + msg_len;
#endif
#else
				enet_hdr = (enet_hdr_t *)buf;
				agile_hdr = (agile_hdr_t *)(buf + sizeof(enet_hdr_t));
				gmc_hdr = (gmc_app_hdr_t *)(buf + sizeof(enet_hdr_t) +
														sizeof(agile_hdr_t));
				msg_ptr = (uchar8 *)(buf + sizeof(enet_hdr_t) +
								sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t));
				data_ptr = msg_ptr + msg_len;

				/*
				 * create enet hdr
				 */
				memcpy(enet_hdr->DA, luc->DA, 6);
				memcpy(enet_hdr->SA, gmc_regs->own_mac_addr, 6);
				enet_hdr->vlan = luc->vlan;
				enet_hdr->type = enet_msg_len;
#endif

				/*
				 * create agile hdr
				 */
				dp = luc->dest_port;

				sp = PACK_UNICAST_PORT_ADDRESS(
												nfxMySlotId,
												nfxMyCPUId,
												0,
												EEE_USER_DATA,
												NFX_COS_HIGH, 
												0);

#ifndef FIX_HDR_STUFF
				agile_hdr->pad = 0;
				agile_hdr->dest_port = dp;
				agile_hdr->src_port = sp;
#endif

#ifndef NO_GMC_APP_NO_TX
				/*
				 * create GMC app hdr
				 */
				gmc_hdr->control = 0;
				gmc_hdr->id.tid = luc->TID;
#endif

				/*
				 * attach control message
				 */
#if 0
				memcpy(msg_ptr, cont_msg, msg_len);
#else
				if ((uint32)cont_msg + msg_len >=
									(uint32)gmc->txq_luc_cntl.end) {
					uchar8	* src_ptr = cont_msg;
					for (i=0; i<msg_len; i++) {
						*(msg_ptr+i) = *src_ptr;
						if (++src_ptr == gmc->txq_luc_cntl.end)
							src_ptr = gmc->txq_luc_cntl.start;
					}
				}
				else
					memcpy(msg_ptr, cont_msg, msg_len);
#endif

				/*
				 * add SAC data
				 */
				bd = (sbm_txq_bd_t *)cont_msg;
				for (bd = (sbm_txq_bd_t *)cont_msg, i=0;
											i < msg_len;
													i+= sizeof(sbm_txq_bd_t)) {
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_luc_cntl.end,
									gmc->txq_luc_cntl.start);
				}
				for (i=0; i<bd_cnt; i++) {
					sac = gmc->sac +
								(bd->sac_index * gmc_regs->gmc_sac_block_size);
GMC_TRACE(("GMC : copying %d bytes from index %d, sac 0x%x to 0x%x\n", bd->length, bd->sac_index, sac, data_ptr));
					memcpy(data_ptr, sac+bd->offset, bd->length);
					data_ptr += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->txq_luc_cntl.end,
									gmc->txq_luc_cntl.start);
				}

				/*
				 * Send message across "data plane"
				 */
GMC_TRACE(("GMC : sending %d bytes to 0x%x\n", tot_len, dp));
				rc = eee_sendMessage(buf,
									tot_len,
									dp,
									sp);

				if (rc != NFX_OK)
					printf("eee_sendMessage error, 0x%x\n", rc);

				eee_ramDealloc(buf);

				/*
				 * advance control pointers
				 */
				for (i=0; i<cl_cnt; i++) {
					bzero((uchar8 *)txq, 32);
					next_to_last_txq = txq;
					GMC_TO_NEXT_TXQ(txq,
									gmc->txq_luc_cntl.end,
									gmc->txq_luc_cntl.start);
				}
				gmc->txq_luc_cntl.head = txq;

				/*
				 * update credit info for MCPU
				 */
				*gmc_regs->txq_luc_credit_addr = (uint32)next_to_last_txq;

				break;
			}

			default:
			{
				printf("GMC : ERROR: GMC read invalid command, 0x%x, at 0x%x\n",
															control, txq);
			}
		}

		control = txq->send_to_luc_req.control; 

	}

}	/* gmc_process_luc_txq() */

/*-----------------------------------------------------------------
 * Name:	gmc_process_copyq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/01/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_process_copyq()
{
	sbm_copyqQueueEntry_t	* txq, * next_to_last_txq;
	sbm_txqCopySACData_t	* copy;
	uint32					control;
	uint32					cl_cnt, i;
	sbm_gmcACKMsg_t			* ack;
	uchar8					* src, * dest;
	uint32					src_block, dest_block, src_offset, dest_offset, len;

	txq = (sbm_copyqQueueEntry_t *)(gmc->copyq_cntl.head);

	if ((uint32)txq & CACHELINE_MASK)
		printf("GMC : ERROR: GMC at misaligned TxQ entry address, 0x%x\n", txq);

	/*
	 * The control word is last word in 8-word entry
	 */
	control = txq->copyqe.control;

	while (control & TXQ_GMC_OWN) {

		cl_cnt = (control & TXQ_CACHELINE_MASK) >> TXQ_CACHELINE_SHIFT;

		switch (control & TXQ_COMMAND_MASK)  {

			case TXQ_CMD_COPY_SAC_DATA:
			{
				copy = (sbm_txqCopySACData_t *)txq;

				src_block = copy->src_SAC_block_index;
				src_offset = copy->src_offset;
				dest_block = copy->dest_SAC_block_index;
				dest_offset = copy->dest_offset;
				len = copy->num_bytes;

				if (src_offset + len > gmc_regs->gmc_sac_block_size)
					panic("GMC : offset + len > size (%d + %d > %d)\n",
								src_offset, len, gmc_regs->gmc_sac_block_size);
				src = gmc->sac + (src_block * gmc_regs->gmc_sac_block_size);
				dest = gmc->sac + (dest_block * gmc_regs->gmc_sac_block_size);

GMC_TRACE(("GMC COPY from sac 0x%x (off %d) to index 0x%x (off %d), bytes = %d\n",
									src, src_offset, dest, dest_offset, len));
				for (i=0; i<len; i++)
					*(dest + dest_offset + i) = *(src + src_offset + i);

				/*
				 * advance control pointers
				 */
				for (i=0; i<cl_cnt; i++) {
					bzero((uchar8 *)txq, 32);
					next_to_last_txq = txq;
					GMC_TO_NEXT_TXQ(txq,
									gmc->copyq_cntl.end,
									gmc->copyq_cntl.start);
				}
				gmc->copyq_cntl.head = txq;

				/*
				 * Send ACK message to given address
				 */
				if (copy->ACK_addr) {
					ack = (sbm_gmcACKMsg_t *)copy->ACK_addr;
					ack->TID = copy->TID;
					ack->param1 = 0;
					ack->param2 = 0;
					ack->control = ACK_CPU_OWN;
				}

				/*
				 * update credit info for MCPU
				 */
				*gmc_regs->copyq_credit_addr = (uint32)next_to_last_txq;

				break;
			}

			default:
			{
				printf("GMC : ERROR: GMC read invalid command, 0x%x, at 0x%x\n",
															control, txq);
			}
		}

		control = txq->copyqe.control;

	}

}	/* gmc_process_copyq() */

/*-----------------------------------------------------------------
 * Name:	gmc_process_metaq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/01/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_process_metaq()
{
	sbm_txqQueueEntry_t			* txq, * next_to_last_txq;
	sbm_txqRetrieveSACData_t	* meta;
	sbm_txq_bd_t				* bd;
	uchar8						* buf, * data_ptr;
	uchar8						* sac;
	uint32						control;
	uint32						tot_len;
	uint32						cl_cnt, i;
	uint32						bd_cnt;
	uint32						dp, sp;
	sbm_gmcACKMsg_t				* ack;

	txq = (sbm_txqQueueEntry_t *)(gmc->metaq_cntl.head);

	if ((uint32)txq & CACHELINE_MASK)
		printf("GMC : ERROR: GMC at misaligned TxQ entry address, 0x%x\n", txq);

	/*
	 * The control word is last word in 8-word entry
	 */
	control = txq->retrieve_req.control;

	while (control & TXQ_GMC_OWN) {

		cl_cnt = (control & TXQ_CACHELINE_MASK) >> TXQ_CACHELINE_SHIFT;

		switch (control & TXQ_COMMAND_MASK)  {

			case TXQ_CMD_RETRIEVE_SAC_DATA:
			{
				meta = (sbm_txqRetrieveSACData_t *)txq;

				bd = (sbm_txq_bd_t *)meta;
				GMC_TO_NEXT_TXQ((sbm_txqQueueEntry_t *)bd,
								gmc->metaq_cntl.end,
								gmc->metaq_cntl.start);

				bd_cnt = control & TXQ_BUFCNT_MASK;
				for (tot_len = 0, i=0; i<bd_cnt; i++) {
					tot_len += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->metaq_cntl.end,
									gmc->metaq_cntl.start);
				}

				buf = (uchar8 *)eee_ramAlloc(tot_len);
GMC_TRACE(("GMC: tot_len %d buf @ %p\n", tot_len, buf));
				if (buf == NULL) {
					printf("could not allocate memory for message\n");
					printf("leaving Read Req message in queue\n");
					return;
				}

				data_ptr = buf;

				/*
				 * add SAC data
				 */
				bd = (sbm_txq_bd_t *)meta;
				GMC_TO_NEXT_TXQ((sbm_txqQueueEntry_t *)bd,
								gmc->metaq_cntl.end,
								gmc->metaq_cntl.start);

				for (i=0; i<bd_cnt; i++) {
					sac = gmc->sac +
								(bd->sac_index * gmc_regs->gmc_sac_block_size);
GMC_TRACE(("GMC : copying %d bytes from index %d, sac 0x%x to 0x%x\n", bd->length, bd->sac_index, sac, data_ptr));
					memcpy(data_ptr, sac+bd->offset, bd->length);
					data_ptr += bd->length;
					GMC_TO_NEXT_BD((sbm_txqQueueEntry_t *)bd,
									gmc->metaq_cntl.end,
									gmc->metaq_cntl.start);
				}

GMC_TRACE(("GMC : copying data from 0x%x to 0x%x, %d bytes\n",
											buf, meta->PCI_mem_addr, tot_len));
				memcpy((uchar8 *)meta->PCI_mem_addr, (uchar8 *)buf, tot_len);

				eee_ramDealloc(buf);

				/*
				 * advance control pointers
				 */
				for (i=0; i<cl_cnt; i++) {
					bzero((uchar8 *)txq, 32);
					next_to_last_txq = txq;
					GMC_TO_NEXT_TXQ(txq,
									gmc->metaq_cntl.end,
									gmc->metaq_cntl.start);
				}
				gmc->metaq_cntl.head = txq;

				/*
				 * Send ACK message to given address
				 */
				if (meta->ACK_addr) {
					ack = (sbm_gmcACKMsg_t *)meta->ACK_addr;
					ack->TID = meta->TID;
					ack->param1 = 0;
					ack->param2 = 0;
					ack->control = ACK_CPU_OWN;
				}

				/*
				 * update credit info for MCPU
				 */
				*gmc_regs->metaq_credit_addr = (uint32)next_to_last_txq;

				break;
			}

			default:
			{
				printf("GMC : ERROR: GMC read invalid command, 0x%x, at 0x%x\n",
															control, txq);
			}
		}

		control = txq->retrieve_req.control;

	}

}	/* gmc_process_metaq() */

/*-----------------------------------------------------------------
 * Name:	gmc_process_internal_queues()
 *
 * Description:	Polling function for processing entries from TxQ's
 *				and CopyQ.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/24/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_process_internal_queues(
	uint32	max,
	uint32	tref
)
{
	/*
	 * Make sure outstanding copy operations are completed before
	 * doing any transmit operations.
	 */
#if 0
	gmc_process_copyq();
#endif

	gmc_process_snim_txq();
	gmc_process_luc_txq();
	gmc_process_metaq();

}	/* gmc_process_internal_queues() */

#endif	/* !SBM_REMOVE_GMC */
