/*
 *----------------------------------------------------------------------------
 *
 * Name:         gt64440mii.c
 *
 * Copyright (C) 2004, OnStor, Inc.
 *
 * Description : Contains Disco3 Ethernet Driver initialization routines.
 *               Ported from sm-gt64240-mii module. 
 *
 * Created by  : Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

#include <exports/hwMap.h>
#include "gt64440mii.h"

#ifndef SIBYTE_BC
#include <exports/mem-api.h>
#include <exports/malloc-api.h>
#include <exports/pci-api.h>
#include <exports/pci-reg.h>

gt64440mii_cfg_t *gt64440mii;
gt64440mii_desc_list_t txPktDesc[GT_MII_MAX_PORTS];


int32 gt64440mii_txUnicast(uint32, eee_desc_t *);
int gt64440mii_initPortRegs(uint32); 
int gt64440mii_fillRxQ(gt64440mii_rxdesc_t *, int); 
int gt64440mii_rxinitQ(gt64440mii_rxdesc_t *, int);
int gt64440mii_txinitQ(gt64440mii_txdesc_t *, int);
int gt64440mii_initDevice(void);
void gt64440mii_resetPort( uint32 port );

void gt64440mii_setMacAddr(uint32 port, uint32 macLo, uint32 macHi);

extern int gt_mii_diagLbPktReceive(eee_desc_t *);
extern int bp_getMyMacAddr(uint32, uint32, uint32, uint32, char *);


extern uchar8 *seepGetMyExtMacAddr(uint32);
extern address_t cached_to_uncached(address_t);

extern char eee_ssc_ext_mac[][6];
extern char eee_sp_ext_mac[][6];


/* 
 * PHY address table
 */
char phytab[] = { GT_MII_PHY0, GT_MII_PHY1, GT_MII_PHY2 };

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_initModule
 *
 * Description: 	Initializes the GT64440 MII device driver module.
 *
 * Returns: NFX_OK or NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
int
gt64440mii_initModule( void )
{
	int rc;
	
	gt64440mii = (gt64440mii_cfg_t *)eee_ramAlloc(sizeof(gt64440mii_cfg_t));
	if(!gt64440mii) 
	{
		printf("[%s] Error: Malloc Failed: gt64440mii_cfg_t.\n",__FUNCTION__);
		return(NFX_ERR);
	}
	memset(gt64440mii,0,sizeof(gt64440mii_cfg_t));

	/*
	 * Register with EEE and get app id
	 */
	rc = eee_allocateApplicationId("gt64440_mii");
	if( rc == NFX_ERR )
	{
		printf("[%s] Error: Allocate app id failed.",__FUNCTION__);
		return NFX_ERR;
	}
	gt64440mii->appId = rc;

#ifdef PROM
	gt64440mii->loopAddr = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, 
  					                                 nfxMyCPUId, 
					                                 gt64440mii->appId,
  					                                 EEE_USER_DATA,
					                                 NFX_COS_HIGH,
													 PROM_PORT_NET0);

	gt64440mii->loopAddrBP = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, 
  					                                 nfxMyCPUId, 
					                                 gt64440mii->appId,
  					                                 EEE_USER_DATA,
					                                 NFX_COS_HIGH,
													 PROM_PORT_MGMT);

	rc = eee_allocateApplicationId("lucRcv");
	if( rc == NFX_ERR )
	{
		printf("[%s] Error: Allocate IP app id failed.",__FUNCTION__);
		return NFX_ERR;
	}
	gt64440mii->ipAppPort = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, 
  													  nfxMyCPUId, 
													  rc,
  													  EEE_USER_DATA,
				  									  NFX_COS_HIGH,
													  PROM_PORT_NET0);

	rc = eee_registerReceiveFunc(gt64440mii->appId,  /* App Id */
								 gt64440mii_rxAppPkt,/* Ucast Rx Func */
								 NULL);	 	         /* Mcast Rx Func */
	if( rc == NFX_ERR )
	{
		printf("[%s] Error: registerReceiveFunc\n",__FUNCTION__);
		return NFX_ERR;
	}
	eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64440mii_rxPktPoll, 0 );
	eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64440mii_txCleanPoll, 0 );

	eee_registerTransmitFunc(PROM_PORT_MGMT, 
			                gt64440mii_txUnicast,
			                gt64440mii_txUnicast,
							1500);
	eee_registerTransmitFunc(PROM_PORT_NET0,
			                gt64440mii_txUnicast,
			                gt64440mii_txUnicast,
							1500);
#ifdef DIAG_R7000   
	eee_registerTransmitFunc( EEE_MGMT,
							 gt64440mii_txUnicast,
                             gt64440mii_txUnicast,
							 1500);
	eee_registerTransmitFunc( EEE_CONTROL,
							 gt64440mii_txUnicast,
                             gt64440mii_txUnicast,
							 1500);
#endif   
	eee_registerDefaultTransmitFunc(EEE_MGMT,
                                    gt64440mii_txUnicast, 9*1024);
	eee_registerDefaultTransmitFunc(EEE_CONTROL,
                                    gt64440mii_txUnicast, 1500);
	eee_registerDefaultTransmitFunc(EEE_USER_DATA,
                                    gt64440mii_txUnicast, 1500);

#elif defined(NFX_MOD_FCNIM) && defined(AGILE_HARDWARE)
 
	gt64440mii->loopAddr = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
                                                     nfxMyCPUId,
                                                     gt64440mii->appId,
                                                     EEE_CONTROL,
                                                     NFX_COS_HIGH,
                                                     FCNIM_PORT_BP_MGMT);     

	rc = eee_registerReceiveFunc(gt64440mii->appId,  /* App Id */
                                 gt64440mii_rxAppPkt,/* Ucast Rx Func */
                                 NULL);              /* Mcast Rx Func */
	if( rc == NFX_ERR )
	{
		printf("[%s] Error: registerReceiveFunc\n",__FUNCTION__);
		return NFX_ERR;
	}
	eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64440mii_rxPktPoll, 0 );
	eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64440mii_txCleanPoll, 0 );

	eee_registerDefaultTransmitFunc(EEE_CONTROL,
                                    gt64440mii_txUnicast,
									1500);

	eee_registerDefaultTransmitFunc(EEE_MGMT,
                                    gt64440mii_txUnicast,
									1500);
 
	eee_registerTransmitFunc( FCNIM_PORT_BP_CONTROL,
							  gt64440mii_txUnicast,
							  gt64440mii_txUnicast,         
							  1500);         
	eee_registerTransmitFunc( FCNIM_PORT_BP_MGMT,
							  gt64440mii_txUnicast,
							  gt64440mii_txUnicast,
							  1500);         
#endif // NFX_MOD_SSC

	if( gt64440mii_initDevice() == NFX_ERR ) 
	{
		printf("[%s] Error: Initialize device failed.",__FUNCTION__);
		return NFX_ERR;
	}

	gt_mii_portEna(0);		// Enable Port 1.
	return(NFX_OK);

} 


/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_initDevice
 *
 * Description: 	Initializes the GT64440 device.
 *
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
gt64440mii_initDevice( void )
{
	uint32 				i, port;
	uint32 				align_diff;
	uint32 				size_rx;
	uint32 				size_tx;
	uchar8 				*qstart;
	uint32				macLo=0;
	uint32				macHi=0;
	gt64440mii_txCtrl_t *txc;
	gt64440mii_rxCtrl_t *rxc;

	printf("[%s] Enter\n", __FUNCTION__);

	gt64440mii->baseAddr = GT64440MII_BASE_ADDR ;

	size_tx = sizeof(gt64440mii_txdesc_t) * GT_MII_TX_Q_LEN;
	size_rx = sizeof(gt64440mii_rxdesc_t) * GT_MII_RX_Q_LEN;

	for (port = 0; port < GT_MII_MAX_PORTS; port++)
	{
		txPktDesc[port].index = 0;
		txPktDesc[port].outdex = 0;
		for (i = 0; i < GT_MII_TX_Q_LEN; i++)
			txPktDesc[port].pktDesc[i] = NULL;

		/*
		 * Setup TX descriptor rings. 
		 */
		for( i=0; i < GT_MII_TX_Q_CNT; i++ )
		{
			txc = &(gt64440mii->port[port].txc[i]);

			/*
			 * Allocate contiguous memory for each ring.
			 */
			qstart = (uchar8 *)eee_ramAlloc(size_tx + 16);
			if(!qstart) 
			{
				printf("[%s] Error: Malloc Failed:Tx desc.\n", __FUNCTION__);
				return(NFX_ERR);
			}

			/* 
			 * Convert to un-cached memory address 
			 */
#ifdef FC_USE_NEW_MEM_MAP
			qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
			GT64440MII_2UNCACHED((uint32)qstart);
#endif
			/* 
			 * Align on 128 bit boundary.
			 */
			align_diff = (uint32)qstart & 0x0000000f;
			if( align_diff )
				align_diff = 16 - align_diff;
			qstart += align_diff;
			memset(qstart, 0, size_tx);

			/* 
			 * Setup tx desc. queue.
			 */
			txc->txqStart = (gt64440mii_txdesc_t *)qstart;
			txc->txCur    = txc->txqStart;
			txc->txCln    = txc->txqStart;
			txc->txqEnd   = txc->txqStart + GT_MII_TX_Q_LEN;
			txc->avail    = GT_MII_TX_Q_LEN;

			/* 
			 * Initialize queue.
			 */
			gt64440mii_txinitQ( txc->txqStart, GT_MII_TX_Q_LEN );
		}

		for( i=0; i < GT_MII_RX_Q_CNT; i++ )
		{
			rxc = &(gt64440mii->port[port].rxc[i]);

			/*
			 * Allocate contiguous memory for each ring.
			 */
			qstart = (uchar8 *)eee_ramAlloc(size_rx + 16);
			if(!qstart) 
			{
				printf("[%s] Error: Malloc Failed:Rx desc.\n", __FUNCTION__);
				return(NFX_ERR);
			}

			/* 
			 * Convert to un-cached memory address 
			 */
#ifdef FC_USE_NEW_MEM_MAP
			qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
			GT64440MII_2UNCACHED((uint32)qstart);
#endif
			/* 
			 * Align on 128 bit boundary.
			 */
			align_diff = (uint32)qstart & 0x0000000f;
			if( align_diff )
				align_diff = 16 - align_diff;
			qstart += align_diff;
			memset(qstart, 0, size_rx);

			/* 
			 * Setup Rx desc. queue.
			 */
			rxc->rxqStart = (gt64440mii_rxdesc_t *)qstart;
			rxc->rxCur    = rxc->rxqStart;
			rxc->rxqEnd   = rxc->rxqStart + GT_MII_RX_Q_LEN;
			rxc->avail    = GT_MII_RX_Q_LEN;
			gt64440mii->port[port].rxQ[i] = rxc->rxqStart;

			/* 
			 * Initialize queue.
			 */
			gt64440mii_rxinitQ( rxc->rxqStart, GT_MII_RX_Q_LEN );
			gt64440mii_fillRxQ( rxc->rxqStart, GT_MII_RX_Q_LEN );
		}

#ifndef SSC_MGMT
//-- Enter station address into hash table.
		if (isFC()) 
		{
#ifndef USE_TEMP_EXT_MAC_ADDR_SCHEME
			extern char eee_sp_mac_ext[4][6];
#endif

			if (port == 0) 
			{		//-- DEBUG PORT, Needs an external MAC address.
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
				uchar8 * macaddr;
				macaddr = (uchar8 *)seepGetMyExtMacAddr(nfxMyCPUId);


#if defined(R9000) && defined(EVAL_BOARD)
				// use momentum id: 00 03  cc 1d 00 9c
				macaddr[0]= 0x00;
				macaddr[1]= 0x03;
				macaddr[2]= 0xcc;
				macaddr[3]= 0x1d;
				macaddr[4]= 0x00;
				macaddr[5]= 0x9c;

				macLo = ((macaddr[4] << 8) |
						(macaddr[5]));
				macHi = ((macaddr[0] << 24) |
						(macaddr[1] << 16) |
						(macaddr[2] << 8) |
						(macaddr[3]));
				printf("isFc() 1 macLo = 0x%x, macHi = 0x%x\n", macLo, macHi);
#endif

				memcpy(gt64440mii->port[0].macAddr, macaddr, 6);
#else

				macLo = ((eee_sp_ext_mac[nfxMyCPUId][0] << 8) |
						(eee_sp_ext_mac[nfxMyCPUId][1]));
				macHi = ((eee_sp_ext_mac[nfxMyCPUId][2] << 24) |
						(eee_sp_ext_mac[nfxMyCPUId][3] << 16) |
						(eee_sp_ext_mac[nfxMyCPUId][4] << 8) |
						(eee_sp_ext_mac[nfxMyCPUId][5]));
			
				memcpy(gt64440mii->port[0].macAddr, 
				   		&eee_sp_ext_mac[nfxMyCPUId][0], 6);
#endif
			}
			else if (port == 1) 
			{		//-- MGMT PORT

	       	 	bp_getMyMacAddr( nfxMySlotId,
	                             nfxMyCPUId,
	                             1,
	                             EEE_MGMT,
	                             gt64440mii->port[port].macAddr);

				macLo = ((gt64440mii->port[port].macAddr[0] << 8) |
						(gt64440mii->port[port].macAddr[1]));
				macHi = ((gt64440mii->port[port].macAddr[2] << 24) |
						(gt64440mii->port[port].macAddr[3] << 16) |
						(gt64440mii->port[port].macAddr[4] << 8) |
						(gt64440mii->port[port].macAddr[5]));
			}
			else if (port == 2) 
			{		//-- CONTROL PORT
	       	 	bp_getMyMacAddr( nfxMySlotId,
	                             nfxMyCPUId,
	                             1,
	                             EEE_CONTROL,
	                             gt64440mii->port[port].macAddr);

				macLo = ((gt64440mii->port[port].macAddr[0] << 8) |
						(gt64440mii->port[port].macAddr[1]));
				macHi = ((gt64440mii->port[port].macAddr[2] << 24) |
						(gt64440mii->port[port].macAddr[3] << 16) |
						(gt64440mii->port[port].macAddr[4] << 8) |
						(gt64440mii->port[port].macAddr[5]));
			}
			else
				break;

		}

		if (isSSC()) 
		{
#ifndef USE_TEMP_EXT_MAC_ADDR_SCHEME
			extern char eee_ssc_ext_mac[2][6];
#endif
			uchar8 * macaddr;
			macaddr = (uchar8 *)seepGetMyExtMacAddr(nfxMyCPUId);

			if (port == 0) 
			{		//-- Backplane MGMT Port

		      	bp_getMyMacAddr(nfxMySlotId,
   		                       	nfxMyCPUId,
       		                   	1,
                          		EEE_MGMT,
                          		gt64440mii->port[port].macAddr);

				macLo = ((gt64440mii->port[port].macAddr[0] << 8) |
						(gt64440mii->port[port].macAddr[1]));
				macHi = ((gt64440mii->port[port].macAddr[2] << 24) |
						(gt64440mii->port[port].macAddr[3] << 16) |
						(gt64440mii->port[port].macAddr[4] << 8) |
						(gt64440mii->port[port].macAddr[5]));

#if defined(R9000) && defined(EVAL_BOARD)
				// use momentum id: 00 03  cc 1d 00 9c
				macaddr[0]= 0x00;
				macaddr[1]= 0x03;
				macaddr[2]= 0xcc;
				macaddr[3]= 0x1d;
				macaddr[4]= 0x00;
				macaddr[5]= 0x9c;

				macLo = ((macaddr[4] << 8) |
						(macaddr[5]));
				macHi = ((macaddr[0] << 24) |
						(macaddr[1] << 16) |
						(macaddr[2] << 8) |
						(macaddr[3]));
				memcpy(gt64440mii->port[0].macAddr, macaddr, 6);
				printf("isSSC()  macLo = 0x%x, macHi = 0x%x\n", macLo, macHi);
#endif

			}
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
			else if (port == 1) 
			{		//-- External Port 1
				uchar8 * macaddr;
				macaddr = (uchar8 *)seepGetMyExtMacAddr(0);
				macLo = ((macaddr[0] << 8) |
						(macaddr[1]));
				macHi = ((macaddr[2] << 24) |
						(macaddr[3] << 16) |
						(macaddr[4] << 8) |
						(macaddr[5]));
				//printf("macLo = 0x%x, macHi = 0x%x\n", macLo, macHi);
			}
			else if (port == 2) 
			{		//-- External Port 2
				uchar8 * macaddr;
				macaddr = (uchar8 *)seepGetMyExtMacAddr(1);
				macLo = ((macaddr[0] << 8) |
						(macaddr[1]));
				macHi = ((macaddr[2] << 24) |
						(macaddr[3] << 16) |
						(macaddr[4] << 8) |
						(macaddr[5]));
				//printf("macLo = 0x%x, macHi = 0x%x\n", macLo, macHi);
			}
#else
			else if (port == 1) 
			{		//-- External Port 1
				macLo = ((eee_ssc_ext_mac[0][0] << 8) |
						(eee_ssc_ext_mac[0][1]));
				macHi = ((eee_ssc_ext_mac[0][2] << 24) |
						(eee_ssc_ext_mac[0][3] << 16) |
						(eee_ssc_ext_mac[0][4] << 8) |
						(eee_ssc_ext_mac[0][5]));
			}
			else if (port == 2) 
			{		//-- External Port 2
				macLo = ((eee_ssc_ext_mac[1][0] << 8) |
						(eee_ssc_ext_mac[1][1]));
				macHi = ((eee_ssc_ext_mac[1][2] << 24) |
						(eee_ssc_ext_mac[1][3] << 16) |
						(eee_ssc_ext_mac[1][4] << 8) |
						(eee_ssc_ext_mac[1][5]));
			}
#endif
			else
				break;

		}
#endif	/* !SSC_MGMT */

		gt64440mii_resetPort(port); 
		gt64440mii_initPortRegs(port); 
		gt64440mii_clearStats(port);
		gt64440mii_setMacAddr(port, macLo, macHi);
	} /* for each port */

	printf("[%s] Leave\n", __FUNCTION__);
	return( NFX_OK ); 

} 

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_initPortRegs
 *
 * Description: 	Initializes the GT64440 MII registers.
 *
 * Returns: NFX_OK 
 *
 *------------------------------------------------------------------------------
 */
int
gt64440mii_initPortRegs(uint32 port) 
{
   uint32 val;

   //printf("[gt64440mii_initPortRegs] Enter\n");

	/* 
	 * Setup port config reg.
	 *   Default RXQ    - 0 (Receive IP & ARP Broadcasts)
	 *   Default RXQArp - 0
	 *   Disable TCP/UDP Capture.
	 */
	GT_WRITE(ETH_PORT_CONFIG_REG(port), PCR_DEFAULT_SETTING);

	/* 
	 * Setup port config extended reg.
	 *   Treat spanning tree as normal multicast. 
	 *   Disable partition.
	 */
	GT_WRITE(ETH_PORT_CONFIG_EXTEND_REG(port), PCXR_DEFAULT_SETTING);

	/* 
	 * Setup port serial control reg.
	 */
  	GT_WRITE(ETH_PORT_SERIAL_CONTROL_REG(port), PSCR_DEFAULT_SETTING);

	/*
	 * Setup SDMA config reg.
	 *   Rx Burst - 16 64-bit words
	 *   Tx Burst - 16 64-bit words
	 *   Rx Swap  - No
	 *   Tx Swap  - No
	 */
	GT_WRITE(ETH_SDMA_CONFIG_REG(port), SDCR_DEFAULT_SETTING | SDCR_BLMRT);

	/*
	 * Setup TX Q 0 Token Bucket count
	 * Setup TX Q 0 Token Bucket config
	 */
	GT_WRITE(ETH_TX_QUEUE_0_TOKEN_BUCKET_COUNT(port), 0x3fffffff);
	GT_WRITE(ETH_TX_QUEUE_0_TOKEN_BUCKET_CONFIG(port), 0x3ffffcff);

	/*
	 * Setup MTU reg
	 *   Set MTU to 0 - Disable port/queue bandwidth limitation.
	 */
	GT_WRITE(ETH_MAXIMUM_TRANSMIT_UNIT(port), 0);
                                                                                
                                                                                
#if defined(R9000) && defined(EVAL_BOARD)
	/* 
	 * reset PHY so that autoneg works out the initial state 
	 */
	gt_mii_writeSMI(GT_MII_GET_PHYID(port), MII_BMCR, BMCR_RESET, 0xffff);
#endif
                                                                                

   /*
    * Initialize Pointer for Rx Desc Queue
    */
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)(virt_to_phys((address_t)gt64440mii->port[port].rxc[0].rxqStart));
#else
   val = ((uint32)(gt64440mii->port[port].rxc[0].rxqStart) & ~V2PHYS_ADDR_MASK);
#endif

	GT_WRITE(ETH_RX_CURRENT_QUEUE_DESC_PTR_0(port), val);

   /*
    * Initialize Pointer for Tx Desc Queue
    */
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)(virt_to_phys((address_t)gt64440mii->port[port].txc[0].txqStart));
#else
   val = ((uint32)(gt64440mii->port[port].txc[0].txqStart) & ~V2PHYS_ADDR_MASK);
#endif

	GT_WRITE(ETH_TX_CURRENT_QUEUE_DESC_PTR_0(port), val);


   //printf("[gt64440mii_initPortRegs] Leave\n");
   return(NFX_OK);
} /* gt64440mii_initPortRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_rxinitQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *			descriptor memory into individual descriptors and 
 *			linking them together.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64440mii_rxinitQ(gt64440mii_rxdesc_t *qstart, int qLen)
{
   int i;
   uint32 val;
   gt64440mii_rxdesc_t *d;

   d  = qstart;
   /* Convert to physical address space */
   for(i=1; i < qLen; i++, d++) {
#ifdef FC_USE_NEW_MEM_MAP
      val = (uint32)((address_t)virt_to_phys((uint32)(d+1)));
#else
      val = (uint32)((uint32)(d+1) & ~V2PHYS_ADDR_MASK);
#endif
      d->link = (gt64440mii_rxdesc_t *)val;
   }
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)virt_to_phys((address_t)qstart);
#else
   val = (uint32)qstart & ~V2PHYS_ADDR_MASK;
#endif
   d->link = (gt64440mii_rxdesc_t *)val;
   return(NFX_OK);

} /* gt64440mii_rxinitQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_txinitQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *			descriptor memory into individual descriptors and 
 *			linking them together.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64440mii_txinitQ(gt64440mii_txdesc_t *qstart, int qLen)
{
   int i;
   uint32 val;
   gt64440mii_txdesc_t *d;

   d  = qstart;
   /* Convert to physical address space */
   for(i=1; i < qLen; i++, d++) {
#ifdef FC_USE_NEW_MEM_MAP
      val = (uint32)((address_t)virt_to_phys((uint32)(d+1)));
#else
      val = (uint32)((uint32)(d+1) & ~V2PHYS_ADDR_MASK);
#endif
      d->link = (gt64440mii_txdesc_t *)val;
   }
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)virt_to_phys((address_t)qstart);
#else
   val = (uint32)qstart & ~V2PHYS_ADDR_MASK;
#endif
   d->link = (gt64440mii_txdesc_t *)val;
   return(NFX_OK);

} /* gt64440mii_txinitQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_fillRxQ
 *
 * Description: 	Fills a receive queue with empty system data buffers.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64440mii_fillRxQ(gt64440mii_rxdesc_t *rxQ, int qLen) 
{
   int i;
   char *pBuf;
   gt64440mii_rxdesc_t *d;

   //printf("[gt64440mii_fillRxQ] Enter\n");
   d = rxQ;

   for(i=0; i < qLen; i++, d++)
   {
      pBuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
#ifdef FC_USE_NEW_MEM_MAP
      pBuf = (char *)virt_to_phys((address_t)pBuf);
#else
      (uint32)pBuf &= ~V2PHYS_ADDR_MASK; /* Convert to physical addr  */
#endif
      d->bufptr = (uchar8 *)pBuf;
      d->bufsize_bytecount = (GT_MII_RX_BUF_SIZE & ~7);
      d->cmdsts = (RX_O | RX_EI);
   }
   //printf("[gt64440mii_fillRxQ] Leave\n");
   return(NFX_OK);

} /* gt64440mii_fillRxQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_rxAppPkt
 *
 * Description: 	Receive packet sent to GT64440 MII mgmt layer.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
void
gt64440mii_rxAppPkt( eee_desc_t *edesc )
{
   gt_mii_diagLbPktReceive(edesc);
   return;
} /* gt64440mii_rxAppPkt */


#define ETH_DA_FILTER_UNICAST_TABLE_SIZE	16
#define ETH_DA_FILTER_SPECIAL_MULTICAST_TABLE_SIZE	256
#define ETH_DA_FILTER_OTHER_MULTICAST_TABLE_SIZE	256

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_resetPort
 *
 * Description: 	Reset a GT64440 ethernet port.
 *
 *------------------------------------------------------------------------------
 */
void
gt64440mii_resetPort( uint32 port )
{
	uint32 base_reg;
	int32  idx;
                                                                                
	/*
	 * Disable port.
	 */
	gt_mii_portDis(port);
                                                                                
	/*
	 * Clear unicast table.
	 */
	base_reg = ETH_DA_FILTER_UNICAST_TABLE_BASE(port);
	for (idx = 0; idx < ETH_DA_FILTER_UNICAST_TABLE_SIZE; idx += 4) 
	{
		GT_WRITE(base_reg + idx, 0);
	}

	/*
	 * Clear special mcast table
	 */
	base_reg = ETH_DA_FILTER_SPECIAL_MULTICAST_TABLE_BASE(port);
	for (idx = 0; idx < ETH_DA_FILTER_SPECIAL_MULTICAST_TABLE_SIZE; idx += 4) 
	{
		GT_WRITE(base_reg + idx, 0);
	}

	/*
	 * Clear other mcast table
	 */
	base_reg = ETH_DA_FILTER_OTHER_MULTICAST_TABLE_BASE(port);
	for (idx = 0; idx < ETH_DA_FILTER_OTHER_MULTICAST_TABLE_SIZE; idx += 4) 
	{
		GT_WRITE(base_reg + idx, 0);
	}

}



void 
gt_mii_writeSMI_common(uint32 unit, uint32 phy_id, uint32 reg, uint32 data){
        uint32 mask;
                                                                                
        mask = unit;
        gt_mii_writeSMI(phy_id, reg, data, mask);
}

/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_rxEna
 *
 * Description:     Enable port Rx Queue
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_rxEna(uint32 port)
{
	/*
	 * Enable Rx Q 0
	 */
	GT_WRITE(ETH_RECEIVE_QUEUE_COMMAND_REG(port), 0x01);
                                                                                
}
                                                                                
/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_txEna
 *
 * Description:     Enable port Tx Queue
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_txEna(uint32 port)
{
	/*
	 * Enable Tx Q 0
	 */
	GT_WRITE(ETH_TRANSMIT_QUEUE_COMMAND_REG(port), 0x01);
                                                                                
}
                                                                                
/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_rxDis
 *
 * Description:     Disable port Rx Queue
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_rxDis(uint32 port)
{
    uint32  status;
                                                                                
	/*
	 * Disable Receive DMAs by setting
	 * DISQ bits on RxQ command register.
	 */
	status = GT_READ(ETH_RECEIVE_QUEUE_COMMAND_REG(port)) << 8;
	if (status & 0xff00) 
	{
		GT_WRITE(ETH_RECEIVE_QUEUE_COMMAND_REG(port), status);
	}
                                                                                
	/*
	 * Wait for Rx SDMA to complete
	 */
	while (GT_READ(ETH_RECEIVE_QUEUE_COMMAND_REG(port)) & 0xff)
			;
}

/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_txDis
 *
 * Description:     Disable port Tx Queues
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_txDis(uint32 port)
{
    uint32  status;
                                                                                
	/*
	 * Disable Transmit DMAs by setting
	 * DISQ bits on TxQ command register.
	 */
	status = GT_READ(ETH_TRANSMIT_QUEUE_COMMAND_REG(port)) << 8;
	if (status & 0xff00) 
	{
		GT_WRITE(ETH_TRANSMIT_QUEUE_COMMAND_REG(port), status);
	}

	/*
	 * Wait for TX SDMA to complete
	 */
	while (GT_READ(ETH_TRANSMIT_QUEUE_COMMAND_REG(port)) & 0xff) 
			;
                                                                                
}
                                                                                
/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_portDis
 *
 * Description:     Clear port Enable Bit
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_portDis(uint32 port)
{
    uint32  status;
                                                                                
    /*
     * First, disable TX DMAs.
     */
	gt_mii_txDis(port);

    /*
     * Disable RX DMAs by setting
	 * DISQ bits on RxQ command register.
     */
	gt_mii_rxDis(port);

    /*
     * Read PSCR & clear PortEN bit. 
     */
    status  = GT_READ(ETH_PORT_SERIAL_CONTROL_REG(port));
    status &= ~PSCR_PORTEN;
    GT_WRITE(ETH_PORT_SERIAL_CONTROL_REG(port), status);
                                                                                
} /* gt_mii_portDis */

/*
 *------------------------------------------------------------------------------ * Function Name:   gt_mii_portEna
 *
 * Description:     Set port Enable Bit
 *
 *------------------------------------------------------------------------------ */
void
gt_mii_portEna(uint32 port)
{
    uint32  status;
                                                                                
    /*
     * Read PSCR & set PortEN bit. 
     */
    status  = GT_READ(ETH_PORT_SERIAL_CONTROL_REG(port));
    status |= PSCR_PORTEN;
    GT_WRITE(ETH_PORT_SERIAL_CONTROL_REG(port), status);

    /*
     * Enable TX DMA Queue 
     */
	gt_mii_txEna(port);

    /*
     * Enable RX DMA Queue 
     */
	gt_mii_rxEna(port);

} 

void
gt64440mii_setMacAddr( uint32 port, uint32 macLo, uint32 macHi )
{
	uint32	idx;
	uint32	val;

	/* 
	 * Set Mac address for port
	 */
	GT_WRITE(ETH_MAC_ADDR_LOW(port), macLo);
	GT_WRITE(ETH_MAC_ADDR_HIGH(port), macHi);

	/* 
	 * Add entry in DA filter unicast table.
	 * Bits MAC[3:2] index into table register.
	 * Bits MAC[1:0] index into byte in register.
	 * Accept frame to Queue 0.
	 */
	idx  = macLo & 0x0c;
	val  = GT_READ(ETH_DA_FILTER_UNICAST_TABLE_BASE(port) + idx);
	val |= 1 << ((macLo & 0x03) * 8);


	GT_WRITE(ETH_DA_FILTER_UNICAST_TABLE_BASE(port) + idx, val);
}

void
gt64440mii_clearStats(uint32 port)
{
    uint32 offset;
    uint32 value;

    for (offset  = ETH_MIB_GOOD_OCTETS_RCVD_LO;
         offset <= ETH_MIB_LATE_COLLISIONS;
         offset += sizeof(uint32))
    {
        value = GT_READ_MIB(port,offset);
    }

}

void
gt64440_initMem( void )
{
	uint32 regval;

        /* Map SDRAM */
        regval = GT_READ(CPU_BASE_ADDRESS_ENABLE) & 0x0f;
        GT_WRITE(ETH_BASE_ADDR_ENABLE_REG, regval);
                                                                                
        regval = GT_READ(SCS_0_BASE_ADDRESS) << 16;
        GT_WRITE(ETH_BAR_0, (regval & 0xffff0000) | 0x0e00);
        regval = GT_READ(SCS_0_SIZE) << 16;
        GT_WRITE(ETH_SIZE_REG_0, regval & 0xffff0000);
        GT_WRITE(ETH_HIGH_ADDR_REMAP_REG_0, 0);
                                                                                
        regval = GT_READ(SCS_1_BASE_ADDRESS) << 16;
        GT_WRITE(ETH_BAR_1, (regval & 0xffff0000) | 0x0d00);
        regval = GT_READ(SCS_1_SIZE) << 16;
        GT_WRITE(ETH_SIZE_REG_1, regval & 0xffff0000);
        GT_WRITE(ETH_HIGH_ADDR_REMAP_REG_1, 0);
                                                                                
        regval = GT_READ(SCS_2_BASE_ADDRESS) << 16;
        GT_WRITE(ETH_BAR_2, (regval & 0xffff0000) | 0x0b00);
        regval = GT_READ(SCS_2_SIZE) << 16;
        GT_WRITE(ETH_SIZE_REG_2, regval & 0xffff0000);
        GT_WRITE(ETH_HIGH_ADDR_REMAP_REG_2, 0);
                                                                                
        regval = GT_READ(SCS_3_BASE_ADDRESS) << 16;
        GT_WRITE(ETH_BAR_3, (regval & 0xffff0000) | 0x0700);
        regval = GT_READ(SCS_3_SIZE) << 16;
        GT_WRITE(ETH_SIZE_REG_3, regval & 0xffff0000);
        GT_WRITE(ETH_HIGH_ADDR_REMAP_REG_3, 0);
                                                                                
        /* Map internal SRAM */
        GT_WRITE(ETH_BAR_4, OC_SRAM_BASE | 0x02);
        GT_WRITE(ETH_SIZE_REG_4, (OC_SRAM_SIZE - 1) & 0xffff0000);
                                                                                
        /* R/W access for all ports */
        GT_WRITE(ETH_ACCESS_PROTECTION_REG(0), 0x0fff);
        GT_WRITE(ETH_ACCESS_PROTECTION_REG(2), 0x0fff);
        GT_WRITE(ETH_ACCESS_PROTECTION_REG(1), 0x0fff);
                                                                                
}

struct bartab {
        int     scslow, scssize;
        int     pci0size, pci1size;
        int     barh, barl;
} barlist[] = {
{ SCS_0_BASE_ADDRESS, SCS_0_SIZE,
  PCI_0SCS_0_BANK_SIZE, PCI_1SCS_0_BANK_SIZE,
  PCI_SCS_0_BASE_ADDRESS_HIGH, PCI_SCS_0_BASE_ADDRESS_LOW },
{ SCS_1_BASE_ADDRESS, SCS_1_SIZE,
  PCI_0SCS_1_BANK_SIZE, PCI_1SCS_1_BANK_SIZE,
  PCI_SCS_1_BASE_ADDRESS_HIGH, PCI_SCS_1_BASE_ADDRESS_LOW },
{ SCS_2_BASE_ADDRESS, SCS_2_SIZE,
  PCI_0SCS_2_BANK_SIZE, PCI_1SCS_2_BANK_SIZE,
  PCI_SCS_2_BASE_ADDRESS_HIGH, PCI_SCS_2_BASE_ADDRESS_LOW },
{ SCS_3_BASE_ADDRESS, SCS_3_SIZE,
  PCI_0SCS_3_BANK_SIZE, PCI_1SCS_3_BANK_SIZE,
  PCI_SCS_3_BASE_ADDRESS_HIGH, PCI_SCS_3_BASE_ADDRESS_LOW },
};
                                                                                
#define NBARS (sizeof(barlist) / sizeof(struct bartab))
                                                                                

void
gt64440mii_initPci( void )
{
	int32	rc, i;
	uint32	data;

    /* 
     * Adjust PCI drive strength 
     */
    GT_WRITE(PCI0_PADS_CALIBRATION, 0x80000000);
    GT_WRITE(PCI0_PADS_CALIBRATION, 0x00004444);
                                                                                
    GT_WRITE(PCI1_PADS_CALIBRATION, 0x80000000);
    GT_WRITE(PCI1_PADS_CALIBRATION, 0x00004444);

    /*
     *  Set up memory address decoders to map entire memory.
     *  But first move away bootrom map to high memory.
     */
printf("%s 1: BOOTCS_BASE_ADDRESS=%x\n", 
		__FUNCTION__, GT_READ(BOOTCS_BASE_ADDRESS));
printf("%s 2: BOOTCS_BASE_ADDRESS=%x\n", 
		__FUNCTION__, BOOT_BASE >> 16);
//    GT_WRITE(BOOTCS_BASE_ADDRESS, (BOOT_BASE <<16) >> 16);
//    GT_WRITE(BOOTCS_SIZE, (BOOT_SIZE - 1) >> 16);
                                                                                
printf("%s 3: BOOTCS_BASE_ADDRESS=%x\n", 
		__FUNCTION__, GT_READ(BOOTCS_BASE_ADDRESS));
    /*
     * Program bus numbers
     */
    GT_WRITE(PCI_0P2P_CONFIGURATION, 0x000000ff);
    GT_WRITE(PCI_1P2P_CONFIGURATION, 0x000100ff);
                                                                                
    /*
     *  Enable PCI 0 as master to do config cycles.
     */
    rc = pci_cfg_read(0, 0, 0, PCI_COMMAND_STATUS_REG, &data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_read failed\n", __FUNCTION__, __LINE__);
	}

    data |= PCI_COMMAND_MASTER_ENABLE | PCI_COMMAND_MEM_ENABLE;
    pci_cfg_write(0, 0, 0, PCI_COMMAND_STATUS_REG, data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_write failed\n", __FUNCTION__, __LINE__);
	}
                                                                                
    /*
     *  Enable PCI 1 as master to do config cycles.
     */
    rc = pci_cfg_read(1, 0, 0, PCI_COMMAND_STATUS_REG, &data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_read failed\n", __FUNCTION__, __LINE__);
	}
    data |= PCI_COMMAND_MASTER_ENABLE | PCI_COMMAND_MEM_ENABLE;
    pci_cfg_write(1, 0, 0, PCI_COMMAND_STATUS_REG, data);
	if (rc != NFX_OK)
	{
		printf("%s: line=%d pci_cfg_write failed\n", __FUNCTION__, __LINE__);
	}
                                                                                
    /*
     *  Set up CPU to PCI mappings. Use only one I/O and MEM each.
     */
    GT_WRITE(PCI_0I_O_BASE_ADDRESS, PCI0_IO_SPACE_BASE >> 16);
    GT_WRITE(PCI_0I_O_SIZE, (PCI0_IO_SPACE_SIZE - 1) >> 16);
    GT_WRITE(PCI_0MEMORY0_BASE_ADDRESS, PCI0_MEM_SPACE_BASE >> 16);
    GT_WRITE(PCI_0MEMORY0_SIZE, (PCI0_MEM_SPACE_SIZE - 1) >> 16);
                                                                                
    GT_WRITE(PCI_1I_O_BASE_ADDRESS, PCI1_IO_SPACE_BASE >> 16);
    GT_WRITE(PCI_1I_O_SIZE, (PCI1_IO_SPACE_SIZE - 1) >> 16);
    GT_WRITE(PCI_1MEMORY0_BASE_ADDRESS, PCI1_MEM_SPACE_BASE >> 16);
    GT_WRITE(PCI_1MEMORY0_SIZE, (PCI1_MEM_SPACE_SIZE - 1) >> 16);
                                                                                
    /*
     *  Set up mapping for PCI to localmem accesses.
     *  config regs to find mapping and size. BAR and
     *  size register should be set to match SDRAM SCS.
     *  We use PCI0 I/O and PCI0 MEM0. Same for PCI1.
     */
                                                                                
    data = GT_READ(CPU_BASE_ADDRESS_ENABLE);
    data |= (1<<11) | (1<<12) | (1<<13) | (1<<16) | (1<<17) | (1<<18);
    data &= ~((1<<9) | (1<<10) | (1<<14) | (1<<15));
    GT_WRITE(CPU_BASE_ADDRESS_ENABLE, data);
                                                                                
    for(i = 0; i < NBARS; i++) 
	{
		uint32 baselo, basesize, func;
                                                                                
        baselo = GT_READ(barlist[i].scslow) << 16;
        basesize = (GT_READ(barlist[i].scssize) + 1) << 16;
        basesize = -1;
                                                                                
        /* bus 0 */
        func = (barlist[i].barl & 0x700) >> 8;
        pci_cfg_read(0, 0, func, barlist[i].barl & 0xff, &data); 
        data &= 0xffff;
        data |= baselo & 0xfffff000;
        pci_cfg_write(0, 0, func, barlist[i].barl & 0xff, data);
        pci_cfg_write(0, 0, func, barlist[i].barh & 0xff, 0);
                                                                                
        /* bus 1 */
        func = (barlist[i].barl & 0x700) >> 8;
        pci_cfg_read(1, 0, func, barlist[i].barl & 0xff, &data); 
        data &= 0xffff;
        data |= baselo & 0xfffff000;
        pci_cfg_write(1, 0, func, barlist[i].barl & 0xff, data);
        pci_cfg_write(1, 0, func, barlist[i].barh & 0xff, 0);
                                                                                
        /* set size registers */
        GT_WRITE(barlist[i].pci0size, basesize & 0xfffff000);
        GT_WRITE(barlist[i].pci1size, basesize & 0xfffff000);
    }
                                                                                
    /* Configure the Internal Registers decode space */
    /* bus 0 */
    pci_cfg_read(0, 0, 0, 
        		PCI_INTERNAL_REGISTERS_MEMORY_MAPPED_BASE_ADDRESS, &data); 
	data &= 0xffff;
    data |= GT_BASE_ADDR & 0xfffff000;
    pci_cfg_write(0,0,0,
        PCI_INTERNAL_REGISTERS_MEMORY_MAPPED_BASE_ADDRESS, data);
                                                                                
    /* bus 1 */
    pci_cfg_read(1, 0, 0, 
        		PCI_INTERNAL_REGISTERS_MEMORY_MAPPED_BASE_ADDRESS, &data); 
	data &= 0xffff;
    data |= GT_BASE_ADDR & 0xfffff000;
    pci_cfg_write(1,0,0,
        PCI_INTERNAL_REGISTERS_MEMORY_MAPPED_BASE_ADDRESS, data);
                                                                                
    /* enable the PCI slave windows we want */
    data = GT_READ(PCI_0BASE_ADDRESS_REGISTERS_ENABLE);
    data |= GT_READ(CPU_BASE_ADDRESS_ENABLE) & 0x07ff;
    GT_WRITE(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, data);

    data = GT_READ(PCI_1BASE_ADDRESS_REGISTERS_ENABLE);
    data |= GT_READ(CPU_BASE_ADDRESS_ENABLE) & 0x07ff;
    GT_WRITE(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, data);
                                                                                
	gt64440_initMem( );

    /* configure internal arbiter for PCI0 */
    GT_WRITE(MPP_CONTROL0, 0x00001111);
    GT_WRITE(PCI_0_ARBITER, 0x80000030);
    GT_WRITE(MPP_CONTROL2, 0x00001111);
    GT_WRITE(PCI_1_ARBITER, 0x80000030);

}
#endif

/*
 *------------------------------------------------------------------------------
 * Function Name:   gt_mii_readSMI
 *
 * Description:     Read data from SMI device specified by phy & reg addresses.
 *
 * Return :         NFX_OK (value set with 16 bit data) or NFX_ERR 
 *------------------------------------------------------------------------------
 */
int32 
gt_mii_readSMI(uint32 phyAddr, uint32 regAddr, uint32 *value)
{
    uint32 data;
    int32 loop;

    /* 
	 * wait for device to become non-busy 
	 */
    while (GT_READ(ETH_SMI_REG) & SMI_BUSY)
        ;
                                                                                
    /* 
	 * Issue SMI Read command
	 */
    data  = SMI_DATA(0);
    data |= SMI_PHYAD(phyAddr);
    data |= SMI_REGAD(regAddr);
    data |= SMI_READ;
    GT_WRITE(ETH_SMI_REG, data);

    /* 
	 * wait for Data ready.
	 */
    for (loop=0; loop < 10000; loop++)
	{
	    /* 
		 * Read SMI data only after valid bit is set.
		 */
		if (GT_READ(ETH_SMI_REG) & SMI_READVALID)
		{
			*value = SMI_DATA( GT_READ(ETH_SMI_REG) );
			return (NFX_OK);
		}
	}
	printf("gt_mii_readSMI: read failed!\n");	
	return (NFX_ERR);
                                                                                
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   gt_mii_writeSMI
 *
 * Description:     write data to SMI device specified by phy & reg addresses.
 *
 *------------------------------------------------------------------------------
 */
int32 
gt_mii_writeSMI(uint32 phyAddr, uint32 regAddr, uint32 value, uint32 mask)
{
    uint32 data;
    uint32 tmp;
                                                                                
    /* 
	 * wait for device to become non-busy 
	 */
    while (GT_READ(ETH_SMI_REG) & SMI_BUSY)
        ;
                                                                                
    data  = SMI_DATA(value);
    data |= SMI_PHYAD(phyAddr);
    data |= SMI_REGAD(regAddr);
    data |= SMI_WRITE;
    GT_WRITE(ETH_SMI_REG, data);

	if ((gt_mii_readSMI(phyAddr, regAddr, &tmp) == NFX_OK))
	{
		if ((tmp & mask) == (data & mask))
			return( NFX_OK );
	}

    printf("%s: write verify failed! phyid %x reg %x wrote %x got %x\n",
            __FUNCTION__, phyAddr, regAddr, value, tmp);

	return NFX_ERR;
}

