/*
 *-----------------------------------------------------------------
 *
 * Name: timer.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.,
 *
 * Description: GT64440 timer routines.
 *
 * Created by:	Shaun Hui
 *
 * Date Created:	05/05/04
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>
#include <sys/gt64440r.h>
#include <exports/except-api.h>

/*-----------------------------------------------------------------
 * Name:	timer_start_us
 *
 * Description:	Starts GT timer. Parameter is in microseconds.
 *
 *-----------------------------------------------------------------
 */
void
timer_start_us(
	uint32	timer_num,
	uint32	us,
	void	(* int_func)(uint32, uint32, address_t)
)
{
	uint32  tc_control;
	uint32  tclk_cnt = get_busclock() * us;

	switch (timer_num) {
		case 3:
#define TIMER_INT_MASK_TC0		0x1
#define TIMER_INT_MASK_TC1		0x2
#define TIMER_INT_MASK_TC2		0x4
#define TIMER_INT_MASK_TC3		0x8

#define TIMER_INT_CAUSE_TC3		0x8

#define TIMER_CTRL_ENA_TC3		(1 << 24)
#define TIMER_CTRL_MODE_TC3		(1 << 25)

			/* 
			 * Setup TC3 timer counter value.
			 */
			GT_WRITE(TIMER_COUNTER3, tclk_cnt);

			/* 
			 * Enable TC3 timer interrupt
			 */
			GT_WRITE(TIMER_COUNTER_0_3_INTERRUPT_MASK, TIMER_INT_MASK_TC3);

			/* 
			 * Enable TC3 in timer mode
			 */
			tc_control = GT_READ(TIMER_COUNTER_0_3_CONTROL);
			tc_control &= ~0xFF000000;
			tc_control |= (TIMER_CTRL_ENA_TC3 | TIMER_CTRL_MODE_TC3);
			GT_WRITE(TIMER_COUNTER_0_3_CONTROL, tc_control);

			/* 
			 * Clear Timer interrupt cause for TC3
			 * Writing 0 clears bit, writing 1 has no effect.
			 */
			GT_WRITE(TIMER_COUNTER_0_3_INTERRUPT_CAUSE, ~TIMER_INT_CAUSE_TC3);
			break;

		default:
			panic("timer %d unsupported\n", timer_num);
	}

#if 0
	gtint_intr_registerHandler(2, int_func);
#endif
}

/*-----------------------------------------------------------------
 * Name:		timer_stop
 *
 * Description: Stops GT64440 timer.
 *
 *-----------------------------------------------------------------
 */
void
timer_stop(
	uint32	timer_num
)
{
	uint32 tc_imask;
	uint32 tc_control;

	switch (timer_num) {
		case 3:
			/* 
			 * Disable TC3 timer interrupt
			 */
			tc_imask = GT_READ(TIMER_COUNTER_0_3_INTERRUPT_MASK);
			tc_imask &= ~TIMER_INT_MASK_TC3;
			GT_WRITE(TIMER_COUNTER_0_3_INTERRUPT_MASK, tc_imask);

			/* 
			 * Disable TC3 
			 */
			tc_control = GT_READ(TIMER_COUNTER_0_3_CONTROL);
			tc_control &= ~TIMER_CTRL_ENA_TC3;
			GT_WRITE(TIMER_COUNTER_0_3_CONTROL, tc_control);

			break;

		default:
			panic("timer %d unsupported\n", timer_num);
	}

#if 0
	gtint_intr_unregisterHandler(2);
#endif
}

/*-----------------------------------------------------------------
 * Name:		timer_int_clear
 *
 * Description: Clears GT64440 timer interrupt.
 *
 *-----------------------------------------------------------------
 */
void
timer_int_clear(
	uint32	timer_num
)
{

	switch (timer_num) {
		case 3:
			/* 
			 * Clear Timer interrupt cause for TC3
			 * Writing 0 clears bit, writing 1 has no effect.
			 */
			GT_WRITE(TIMER_COUNTER_0_3_INTERRUPT_CAUSE, ~TIMER_INT_CAUSE_TC3);
			break;

		default:
			panic("timer %d unsupported\n", timer_num);
	}
}

