/*
 *-----------------------------------------------------------------
 *
 * Name:	i2c.c
 *
 * RCS:  $Id: i2c.c,v 1.10 2002/11/05 00:16:29 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Common I2C that branches of based on platform.
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created:	07/22/04	
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/seep.h>

#if defined (R7000) && !defined (R9000)	
int32 gt64240_i2c(uint32 op,uchar8 i2c_addr,uchar8 byte_addr,uint32 len,uchar8 *buf);
#elif defined (R9000)
int32 gt64440_i2c(uint32 op,uchar8 i2c_addr,uchar8 byte_addr,uint32 len,uchar8 *buf);
#elif defined (SIBYTE) && !defined (SIBYTE_BC)	
int32 sb1250_i2c(uint32 op,uchar8 bus,uchar8 i2c_addr,uchar8 byte_addr,uint32 len,uchar8 *buf);
#elif defined (SIBYTE_BC)	
int32 gt64440_i2c(uint32 op,uchar8 i2c_addr,uchar8 byte_addr,uint32 len,uchar8 *buf);
int32 sb1250_i2c(uint32 op,uchar8 bus,uchar8 i2c_addr,uchar8 byte_addr,uint32 len,uchar8 *buf);
#endif

/*-----------------------------------------------------------------
 * Name:	i2c()
 *
 * Description: Branch to the appropriate i2c application.
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created:	07/23/04
 *
 *-----------------------------------------------------------------
 */
int32
i2c(
	uint32	op,
	uchar8  bus,
	uchar8	i2c_addr,
	uchar8	byte_addr,
	uint32	len,
	uchar8	* buf
)
{

#if defined (R7000) && !defined (R9000)	
	return (gt64240_i2c(op, i2c_addr, byte_addr, len, buf));

#elif defined (SIBYTE) && !defined (SIBYTE_BC)	
	return (sb1250_i2c(op, bus, i2c_addr, byte_addr, len, buf));

#elif defined (R9000) || defined (SIBYTE_BC)	
#if defined (SIBYTE_BC)	
	/* Depending on the i2c_addr, call appropriate utlity */
	/* On Bobat TXRX, for the BOARD_SEEP, we access via GT64440 registers */
        if(i2c_addr == SSC_I2C_ADDR)
            return (gt64440_i2c(op, i2c_addr, byte_addr, len, buf));
        else    
	    return (sb1250_i2c(op, bus, i2c_addr, byte_addr, len, buf));

#elif defined (R9000)	
        return (gt64440_i2c(op, i2c_addr, byte_addr, len, buf));

#endif //SIBYTE_BC	
#endif	
	return NFX_ERR;
}
