/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipmd-stats.h
 *
 * RCS:  $Id: ipmd-stats.h,v 1.2 2002/04/26 00:06:23 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _IPMD_STATS_H
#define _IPMD_STATS_H

/*
 * memory structure for txrx stats collection
 */

typedef struct  
{
	void	*cp_ptr;			/* start of buffer to cpy*/
	int		cp_cnt;				/* num. of element to cpy */
} wi_t;							/* walk function input parameter */
	
typedef int	 (*cmp_func)(void*, void*); 
								/* key comparision func. */
								/* -1 less, 0 equal, 1 greater */
typedef void (*walk_func)(ipm_conn_t*, wi_t*);

/*  internal conn type */
typedef struct statconn {
  struct statconn	*next;	
#define IPM_NODE_INUSE	0x1
  uint32			flags;
  ipm_conn_t		conn;
} ipm_inconn_t;

/* card level stats ptrs */
typedef struct {
  ushort16		card;			/* card id */
  ushort16		ifcnt;			/* intf count associated with the card */
#define CARD_IFS_OFFSET			4
  ipm_ifstat_t	*ifstatp[IPM_LINK_MAX]; /* ptr to first ifstats  */
#define CARD_IPS_OFFSET			(CARD_IFS_OFFSET+sizeof(ipm_ipstat_t*)*IPM_LINK_MAX)
  ipm_ipstat_t	*ipstatp;		/* ptr to ip stats */
#define CARD_ICMPS_OFFSET		(CARD_IPS_OFFSET+sizeof(ipm_ipstat_t*))
  ipm_icpstat_t	*icmpstatp;		/* ptr to icmp stats */
#define CARD_TCPS_OFFSET		(CARD_ICMPS_OFFSET+sizeof(ipm_icpstat_t*))
  ipm_tcpstat_t	*tcpstatp;		/* ptr to tcp stats */
#define CARD_UDPS_OFFSET		(CARD_TCPS_OFFSET+sizeof(ipm_tcpstat_t*))
  ipm_udpstat_t	*udpstatp;		/* ptr to udp stats */
} ipm_card_t; 

/* root record */
typedef struct {
  ipm_card_t	*cardp;			/* ptr to card level stats ptr array */
  ipm_inconn_t  *freeconnp;		/* to free conn. table */
  ipm_inconn_t	*firstconnp;	/* first conn. allocated */
  ipm_inconn_t	*lastconnp;		/* last conn. allocated */
  uint32		freeconncnt;	/* free conn. available */
  uint32		tcpcnt;			/* tcp conn. count */
  ipm_inconn_t	*tcpconnp;		/* ptr to first tcp conn. */
  uint32		udpcnt;			/* udp conn. count  */
  ipm_inconn_t	*udpconnp;		/* ptr to first udp conn. */
} ipm_root_t;

extern ipm_root_t	*ipmd_stats_root;
	

#define IPM_ALLOC_CONN(_conn)											\
{																		\
		ipm_inconn_t	*_connaddr;										\
		_connaddr = ipmd_stats_root->freeconnp;							\
		if (!(_connaddr) || (_connaddr->flags & IPM_NODE_INUSE))		\
		   {															\
			 IPM_LOGERR(IPM_ERR_STATS);									\
			 _conn = 0;													\
		   }															\
		else															\
		  {																\
			_connaddr->flags = IPM_NODE_INUSE;							\
			_conn = ipmd_stats_root->freeconnp;							\
			ipmd_stats_root->freeconnp = _connaddr->next;				\
			_connaddr->next = 0;										\
			ipmd_stats_root->freeconncnt--;								\
		  }																\
}

#define IPM_FREE_CONN(_conn)											\
{																		\
		if (!_conn || !(_conn->flags & IPM_NODE_INUSE))			\
		  IPM_LOGERR(IPM_ERR_STATS);									\
		else															\
		  {																\
			_conn->next = ipmd_stats_root->freeconnp;				\
			_conn->flags = 0;										\
			ipmd_stats_root->freeconnp = _conn;							\
			ipmd_stats_root->freeconncnt++;								\
		  }																\
}

/* functions */		
int32 ipmd_InitStatsMem(void);
void  ipmd_PrintStatsMem(void);
int32 ipmd_ChkStatsMem(char*);
int32 ipmd_ProcConnMsg(uint32, uint32, int, ipm_mconn_t*);
int32 ipmd_ProcStatMsg(int, void*, uint32);
int32 ipmd_GetTxRxIfStats(char*, ipm_ifstat_t*, int, int*, int*);
int32 ipmd_GetTxRxIpStats(int32, void*, int, int*, int*);
int32 ipmd_GetTxRxIcmpStats(int32, void*, int, int*, int*);
int32 ipmd_GetTxRxTcpStats(int32, void*, int, int*, int*);
int32 ipmd_GetTxRxUdpStats(int32, void*, int, int*, int*);

int32 ipmd_GetTxRxTcpConnTable(ipm_conn_t*, int, int, int, int*, int*);
int32 ipmd_GetTxRxUdpConnTable(ipm_conn_t*, int, int, int, int*, int*);
int32 ipmd_GetTxRxTcpConn(ipm_conn_t*, ipm_conn_t*);
int32 ipmd_GetTxRxUdpConn(ipm_conn_t*, ipm_conn_t*);
int32 ipmd_GetTxRxTcpNxtConn(ipm_conn_t*, ipm_conn_t*);
int32 ipmd_GetTxRxUdpNxtConn(ipm_conn_t*, ipm_conn_t*);
#endif /* _IPMD_STATS_H */
