/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_link.h
 *
 * RCS:		$Id: ispfc_link.h,v 1.7 2002/11/14 00:52:21 nitinb Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver link initialization definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 5/31/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_LINK_H__
#define __ISPFC_LINK_H__

typedef struct {
	uint32	adap_init_state;
	uint32	adap_init_retry;
	uint32	adap_init_generic_counter;
	uint32	adap_init_timer_handle;
	uint32	adap_init_last_link_event;
	void (*adap_init_callback)(ispfc_cb_t *, void *, int32);
	void *	adap_init_cookie;
	char *  adap_diag_buf1;
	char *  adap_diag_buf2;
	uint32	adap_diag_transfer_length;
	char *	adap_diag_receive_buffer;
	char *	adap_diag_transmit_buffer;
	uint32	adap_diag_count;
	uint32	adap_diag_crc_count;
	uint32	adap_diag_disparity_count;
	uint32	adap_diag_frame_len_count;
	uint32	adap_diag_miscompares;
} ispfc_adap_init_state_t;

/*
 * Adapter initialization completion codes
 */
#define ADAP_INIT_STATUS_SUCCESS	NFX_OK
#define ADAP_INIT_STATUS_FAILED		NFX_ERR

typedef struct {
	uint32	conn_init_state;
	uint32	conn_init_retry;
	uint32	conn_init_timer_handle;
	void (*conn_init_callback)(ispfc_cb_t *, void *, int32);
	void *	conn_init_cookie;
} ispfc_conn_init_state_t;

/*
 * Connection initialization completion codes
 */
#define CONN_INIT_STATUS_SUCCESS	NFX_OK
#define CONN_INIT_STATUS_FAILED		NFX_ERR

typedef struct {
	uint32	link_init_state;
	void (*link_init_callback)(ispfc_cb_t *, void *, int32);
	void *	link_init_cookie;
} ispfc_link_init_state_t;

/*
 * Link initialization completion codes
 */
#define LINK_INIT_STATUS_SUCCESS	NFX_OK
#define LINK_INIT_STATUS_FAILED		NFX_ERR

int32	ispfc_link_management_handler(ispfc_cb_t *, uint32);

int          ispfc_fabric_map_init(ispfc_cb_t *);
int	     ispfc_fabric_map_allocate_id(ispfc_cb_t *);
int	     ispfc_fabric_map_deallocate_id(ispfc_cb_t *, int);
int	     ispfc_fabric_map_is_allocated_id(ispfc_cb_t *, int);
ispfc_dev_t *ispfc_fabric_map_find_lru(ispfc_cb_t *);

int          ispfc_device_handle_init(ispfc_cb_t *);
int32	     ispfc_device_handle_allocate(ispfc_cb_t *);
int	     ispfc_device_handle_deallocate(ispfc_cb_t *, int32);

void	     ispfc_queue_desc_on_ct_busy_queue(ispfc_cb_t *, ct_desc_t *);
void	     ispfc_queue_desc_on_ct_waiting_queue(ispfc_cb_t *, ct_desc_t *);
void	     ispfc_dequeue_desc_off_ct_busy_queue(ispfc_cb_t *, ct_desc_t *);
ct_desc_t *  ispfc_dequeue_desc_off_ct_waiting_queue(ispfc_cb_t *);
void         ispfc_purge_internal_desc_off_ct_waiting_queue(ispfc_cb_t *);

int	     ispfc_soft_reset_adapter(ispfc_cb_t *);
void         ispfc_send_fc_state_msg(int);

#endif /* __ISPFC_LINK_H__ */
