/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_scsi.h
 *
 * RCS:		$Id: ispfc_scsi.h,v 1.4 2001/08/03 00:38:55 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver SCSI command transport definitions
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/5/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_SCSI_H__
#define __ISPFC_SCSI_H__

int32 ispfc_scsi_send(scsi_desc_t *);
int32 ispfc_scsi_dispatch_desc(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *);
void  ispfc_dispose_of_scsi_desc(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *, int);

#if defined(ISPFC_SIM)
#define SCSI_DTYPE_MASK              0x1f
#define SCSI_DTYPE_DIRECT            0x00
#define SCSI_DTYPE_SEQUENTIAL        0x01
#define SCSI_DTYPE_PRINTER           0x02
#define SCSI_DTYPE_PROCESSOR         0x03
#define SCSI_DTYPE_WORM              0x04
#define SCSI_DTYPE_RODIRECT          0x05
#define SCSI_DTYPE_SCANNER           0x06
#define SCSI_DTYPE_OPTICAL           0x07
#define SCSI_DTYPE_CHANGER           0x08
#define SCSI_DTYPE_COMM              0x09
#define SCSI_DTYPE_BRIDGEROUTER      0x0C
#define SCSI_DTYPE_SES               0x0D
#define SCSI_DTYPE_UTILITY           0x0E

/* enable vital product data arg1 flag */
#define	SCSI_INQUIRY_EVPD		0x1

/* arg2 is page code */
#define	SCSI_INQUIRY_SERIALNO_PAGE_CODE	0x80

/*
 * get the serial number of device
 * byte 2 of CDB: page code 0x80
 */
typedef struct {
    byte	dev_type;	/* bit 7-5: qualifier, scsi_Inquiry.dev_type */
    byte	page_code;	/* should be 0x80 */
    byte	reserved;
    byte	page_len;
    byte	data[1];	/* of length page_len */
} scsi_inquiry_serialno_t;

typedef struct {
    uint32	last_block;
    uint32	block_length;
} scsi_read_capacity_t;

typedef struct {
	uint32	list_length;
	uint32	reserved;
	union lun_union {
		byte		lun1[8]; 
		ushort16	lun2[4]; 
		uint32		lun4[2]; 
		uint64		lun8;    
	} u[1]; /* actually list_length/sizeof(uint8) */
} scsi_report_luns_t;

#endif /* defined(ISPFC_SIM) */

#endif /* __ISPFC_SCSI_H__ */

