/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_scsi_tmo.c
 *
 * RCS:		$Id: ispfc_scsi_tmo.c,v 1.5 2002/10/25 22:24:36 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver SCSI task management operation
 *              code
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/5/01
 *
 *------------------------------------------------------------------------------
 */

#include "ispfc_private.h"
#include "ispfc_iocb.h"
#include "ispfc_scsi_tmo.h"
#include "ispfc_mbox.h"

int32 
ispfc_scsi_task_mgmt_send(scsi_task_mgmt_t *desc)
{
	/* XXX Setup priv_callback to move tmp off scsi_task_mgmt_busy */

	return(NFX_OK);
}

int32
ispfc_scsi_task_mgmt_dispatch(ispfc_cb_t *cb, ispfc_dev_t *dev, scsi_task_mgmt_t *tmo)
{
	VERIFY(dev->dev_handle == tmo->hal_handle);
	VERIFY(dev->scsi_task_mgmt_busy == NULL);

	if (ispfc_mbox_is_busy(cb))
		return(NFX_ERR);

	FC_DBG(0, ("%s: phys_dev_id 0x%x, handle %d, lun %d: "
		   "Dispatching TMO (%p) command %d\n", 
		   cb->debug_name, dev->phys_dev_id, dev->dev_handle, tmo->lun,
		   tmo, tmo->command));

	switch(tmo->command) {
	case SCSI_TMO_CLEAR_ACA:
	case SCSI_TMO_TARGET_RESET:
	case SCSI_TMO_CLEAR_TASKSET:
		return(NFX_ERR);

	case SCSI_TMO_ABORT_TASKSET:
		cb->mbox_in_valid_count = 3;
		cb->mbox_in[0] = ISPFC_ABORT_TASK_SET;
		cb->mbox_in[1] = dev->mapping_id << 8;
		cb->mbox_in[2] = tmo->lun;
		ispfc_mbox_start(cb, tmo->priv_callback, tmo);
		break;

	default:
		VERIFY(0);
	}

	dev->scsi_task_mgmt_busy = tmo;
	SET_DEV_STATE(dev, ISPFC_DS_STATE_TMO_PENDING);

	return(NFX_OK);
}
