/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag-util.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Bobcat platform related auto diag code.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 4/28/05
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)
                                                                                                                              
                                                                                                                              
#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
                                                                                                                              
#include <exports/i2c-api.h>
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#include <exports/seep.h>
#include <exports/gt64440mii.h>
#include <sys/bmfpga.h>
                                                                                                                              
#if defined(PROM_SIBYTE_BC)
#include <exports/bmc12500-eth.h>
#include <exports/lsi8104.h>
#include <sys/sb1250/sb1250_defs.h>
#endif
#include "load-bobcat.h"
#include "load-bobcat-diag.h"


#if defined(PROM_R9000)
// Not exported in ./sm-chassis/ so re-do her for now..
#define FAN_REG_OFFSET          2
#define FAN_MAX_CTRL            2
#define FAN_MAX_FAIL_COUNT      3
#define FAN_MAX_CHASSIS_TEMP    0x32
#define FAN_0_MAX_TACH_REG      2
#define FAN_1_MAX_TACH_REG      3
#define FAN_LOW_MIN_REV_COUNT   0x68
#define FAN_LOW_MAX_REV_COUNT   0x90
#define FAN_HIGH_MIN_REV_COUNT  0x98
#define FAN_HIGH_MAX_REV_COUNT  0xB0

#define FAN_TACH0_REG           0x0c

extern int32   fanCtrlTbl[FAN_MAX_CTRL];
extern int32   fanTachMax[FAN_MAX_CTRL];
/*******************************************************************
* Diags Monitor Fan Tach speeds.. 
*  derived from   "./sm-chassis/cm-fan.c"  Thanks
*
* Input  None.
* Output OK or Error
*******************************************************************/
int 
ad_fan_ctrl_tach_check()
{
    int32   i, j, fanError;
    int32   ctrl_addr;
    int32   tach_max;
    int32   tach_reg;
    int32   fan;
    uchar8  temp_val;
    uchar8  tach_val[FAN_MAX_CTRL][FAN_1_MAX_TACH_REG];
                                                                                                                              
    fan = 0;
    fanError = 0;
                                                                                                                              
    // Read tach register values from FAN Controllers 0 & 1
    for (i=0; i < FAN_MAX_CTRL; i++)
    {
        ctrl_addr = fanCtrlTbl[i];      // set ctrl I2C address
        tach_max  = fanTachMax[i];      // set # of tach reg to read
        tach_reg  = FAN_TACH0_REG;      // set base tach reg addr
                                                                                                                              
        for (j=0; j < tach_max; j++)
        {
            i2c(I2C_READ, 0, ctrl_addr, tach_reg, 1, &tach_val[i][j]);
            delay_us(10000);
           // we should be running a Slow speed.  Check range.
            // check if fan speed is within low range
            if (is_cmFanSpeedLow(tach_val[i][j]) != TRUE) {
                if( is_cmFanSpeedHigh(tach_val[i][j]) != TRUE) {
                    printf("   -- Error Fan %d  [ctrl=0x%0x tach=0x%x val=0x%0x]\n",
                                                fan, ctrl_addr, tach_reg, tach_val[i][j]);
                    fanError++;
                }
                else {
                    printf("   -- Error Fan %d at High Speed   [ctrl=0x%0x tach=0x%x val=0x%0x]\n",
                                                fan, ctrl_addr, tach_reg, tach_val[i][j]);
                    fanError++;
                }

            }
            // printf("  -- Debug Tach rang  - fan[%d] ctrl=0x%0x tach=0x%x val=0x%0x\n",
            //                                   fan, ctrl_addr, tach_reg, tach_val[i][j]);
            tach_reg += FAN_REG_OFFSET;
            fan++;
        }
    }
    // Check the temperature rang also
    // read temperature.
    i2c(I2C_READ, 0, I2C_ADDR_TEMP_SENSOR, 0xAA, 1, &temp_val);
                                                                                                                              
    // temperature too high = failure.
    if (temp_val >= FAN_MAX_CHASSIS_TEMP) {
        printf("   -- Error temperature too high  0x%02X  [Should be less than 0x%02X]\n", temp_val, FAN_MAX_CHASSIS_TEMP);
        fanError++;
    }
    // printf("   -- Debug Temp = 0x%02X Should be < than 0x%02X \n",temp_val,FAN_MAX_CHASSIS_TEMP);

    // Want to check all fans and temp before returning an Error
    if(fanError )
        return(NFX_ERR);
    else
        return NFX_OK;
}

#endif

#if defined(PROM_SIBYTE_BC)

/***********************************************************
* diag_get_model_info() 
*  get the model number 
*  that is programmed in SEEP.
*  Used in the diag routines to detect DRAM's (I2C tests) 
*   and to test /not test hardware dependant on model
* RETURNS: model number in decimal  ie 2220, 2240 , or 2260 /2280.
*   Error If no model info
***********************************************************/
int
diag_get_model_info()
{
    uchar8  * seep_str;
    char    sys_rev_str[8];
    char    model_str[20];
    int diagModelNum = 0;
    if(isTXRX() || isFP()) {
                                                                                                                                    
        /* Is this a system with 1, 2, or 4 drams?
        * To decide, look at model num in SEEP from TXRX
        */
        seepPCB_t   * seep = (seepPCB_t *)seep_struct[SEEP_PCB];
        bzero(sys_rev_str, 8);
        bzero(model_str, 20);

        seep_str = seep->revision;
        memcpy(sys_rev_str, seep_str, SEEP_CHASSIS_REVISION_LEN);
        printf("\tSystem Revision:       %s\n", sys_rev_str);

        seep_str = seep->model_num;
        memcpy(model_str,seep_str,15);
        printf("\tModel Number:          %s\n", model_str);
        /*
         * with new systems specified as system revision G4 or greater, 
         * we check configuration as a 2280 no matter what the Model number states.
         * This means all DIMS installed and jumper in FIFO mode.
         */
        printf(" -Sys Rev = %c%d\n",sys_rev_str[0],atoi(&sys_rev_str[1]));
        /*
         * Here we check the 1st character of the System revision. It must
         * be at either "g" or "G" and above, but not beyond (z,Z), then if its
         * a (G,g) then it will check the number to see if it's 4 or greater..
         */
        if (((sys_rev_str[0] >=  'G') && (sys_rev_str[0] < '[')) ||
            ((sys_rev_str[0] >=  'g') && (sys_rev_str[0] < '{'))) {
            if ((sys_rev_str[0] ==  'G') || (sys_rev_str[0] ==  'g')) {
                if (atoi(&sys_rev_str[1]) >= 4) {
                    diagModelNum = 2280;
                    return(diagModelNum);
                }
            } else {  /* must be something greater the G */
                diagModelNum = 2280;
                return(diagModelNum);
            }
        }
                                                                                                                                    
        /* 
         * SEEP_PCB_MODEL_LEN is defined as (9+1) which is not long enough to determin
         * Bobcat model.  So New define above BC_PCB_MODEL_LEN (12+1) 
         */
        if (!strncmp(model_str,"ONS-SYS-2220",BC_PCB_MODEL_LEN-1)) {
            diagModelNum = 2220;
        } else if (!strncmp(model_str,"ONS-SYS-2240",BC_PCB_MODEL_LEN-1)) {
            diagModelNum = 2240;
        } else if (!strncmp(model_str,"ONS-SYS-2260",BC_PCB_MODEL_LEN-1)) {
            diagModelNum = 2260;
        } else if (!strncmp(model_str,"ONS-SYS-2280",BC_PCB_MODEL_LEN-1)) {
            diagModelNum = 2280;
        } else {
            diagModelNum = 0;
        }
    }
    return(diagModelNum);
}

/**********************************************************
*  Support routines for detecting and setting 
*   loop-back mode for copper SFP's
**********************************************************/
/***************************************************
* auto detect SFP module by reading SFP SEEP
* Input = port 0 - 3
* Output = Fiber, Copper, or Error
***************************************************/
int
ad_detect_sfp( uint32 port)
{
 int rc = 0;
 unsigned char   buf[2];
 unsigned char   data;

    switch(port) {
        case 0: {
            // Set-up mux to point to port 0
            data = 4;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 0, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 1: {
            // Set-up mux to point to port 1
            data = 5;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 1, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 2: {
            // Set-up mux to point to port 2
            data = 6;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 2, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 3: {
            // Set-up mux to point to port 3
            data = 7;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 3, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }
        default: {
            printf("  -- Error auto diag internal Software error. File %s Function %s line %d \n",__FILE__,__FUNCTION__,__LINE__);
            return (AUTO_DIAG_TEST_ERROR);
            break;
        }

    }
    // reads internal seep of port X sfp
    rc = i2c(I2C_READ, 1, 0x50, 2, 1, buf);
    if (rc < 0) {
        printf("Error in I2C_READ SFP port %d Seep, rc = 0x%x\n",port,rc);
        return (AUTO_DIAG_TEST_ERROR);
    }

    // return type
    if (buf[0] == 0x07 ) // Fiber
        return (AD_FIBER);
    else if (buf[0] == 0 ) // Copper
        return (AD_COPPER);
    else 
        return (AUTO_DIAG_TEST_ERROR);

}

/**********************************************
* set port to loop back mode for copper SFP
* Input = port # 0-4
* Output = OK  or Error
**********************************************/
int 
ad_sfp_copper_lb_set( uint32 port)
{
 int rc = 0;
 unsigned char data;

    switch(port) {
        case 0: {
            // Set-up mux to point to port 0
            data = 4;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 0, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 1: {
            // Set-up mux to point to port 1
            data =  5;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 1, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 2: {
            // Set-up mux to point to port 2
            data =  6;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 2, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }

        case 3: {
            // Set-up mux to point to port 3
            data =  7;
            rc = i2c(I2C_WRITE, 1, 0x70, 0, 1, &data);
            if (rc < 0) {
                printf("Error in I2C_WRITE to MUX point to port 3, rc = 0x%x\n", rc);
                return (AUTO_DIAG_TEST_ERROR);
            }
            break;
        }
        default: {
            printf("  -- Error auto diag internal Software error. File %s Function %s line %d \n",__FILE__,__FUNCTION__,__LINE__);
            return (AUTO_DIAG_TEST_ERROR);
            break;
        }

    }
    // Looking at the right port  set it up for loop back (copper Only)
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x12, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x12, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x12, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x12, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x1b;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x09, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x09, 1, 0x1b], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x09, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x09, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x81;
    rc = i2c(I2C_WRITE, 1, 0x56, 0, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0, 1, 0x81], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x40;
    rc = i2c(I2C_WRITE, 1, 0x56, 0, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0, 1, 0x40], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 7;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 7], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 8;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 8], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 8;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 8], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x10;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 0x10], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x42;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 0x42], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 0], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x12;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1d, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1d, 1, 0x12], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x89;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 0x89], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    data = 0x01;
    rc = i2c(I2C_WRITE, 1, 0x56, 0x1e, 1, &data);
    if (rc < 0) {
        printf("Error in I2C_WRITE [1, 0x56, 0x1e, 1, 0x01], rc = 0x%x\n", rc);
        return (AUTO_DIAG_TEST_ERROR);
    }
    // All Ok return Ok
    return (NFX_OK);
}

/************************************************************
* Detect all SFP's  if Copper setup SFP for loop back
*                   if Fiber just print detected
*  Diags will call this before the GigE LB test is run 
*
* Input: Bobcat Model identifier; 
* Output: NFX_OK or AUTO_DIAG_TEST_ERROR
************************************************************/
int 
ad_detect_sfp_all( int diagModelNum)
{
    int port, portCnt, portType, rc, status;

    if (diagModelNum > 2220)
      portCnt = 4; // Means that it's a 2240 or greater model number
    else 
      portCnt = 2; // Means that it's a 2220

    for (port=0; port < portCnt; port++ ) {  // detect all SFP's .. if copper set it up for loop back
        portType = ad_detect_sfp( port); // get port Type  
        switch (portType) {
            case AD_COPPER:
            case AD_FIBER:
                printf("   -detected port %d type = %s\n",port, (portType == AD_FIBER)? "Fiber":"Copper");
                if (portType == AD_COPPER) { // go set it up for Loop Back
                    printf("    -setup port %d for copper loop back\n",port);
                    rc = ad_sfp_copper_lb_set( port);  
                    if (rc != NFX_OK) {
                        printf("    -Error SFP setup Copper LB mode\n");
                        return (status = ((( 1 << port) << 12) | AUTO_DIAG_TEST_ERROR));  // set port error bit in status
                    }
                }
                break;

           default:
                printf("    -Error SFP Auto Detect Port %d\n",port);
                return (status = (((1 << port) << 12) | AUTO_DIAG_TEST_ERROR));  // set port error bit in status
                break;

        } // end switch
    }
    return (NFX_OK); 
}

// Set up GigE Ports..   Auto detect, no packets run,  utility
int
ad_gige_setup()
{
 int testGroup = 0x00000000;
 int status = 0;
 int diagModelNum=0;
 uint32 i;

    if (isTXRX()) {
        printf(" -GigE autodetect port type and setup loopback\n");
        testGroup = DIAG_TEST_GROUP_DEV;  // Set group Number

        // get Model number
        diagModelNum = diag_get_model_info();

        // Clear Error Counters left over from SERDES internal loop-back test
        for (i = 0; i < 4; i++) {
            diag_lsi_clearStats( i);   // Clear Error Counters
        }

        // Clear LUC Stats  (mainly for looping test, power up is ok since LUC was reset)
        luc_clear_stats();

        // Go Auto detect SFP's and if it's copper set it up for loop back test
        status = ad_detect_sfp_all(diagModelNum); //  Model number tells how many to look 4
        if (status != NFX_OK) {
            printf(" -- Error Auto Detect SFP / SFP Loopback setup error. \n");
            // port bit for the error was set in the status returned from above
            return (status |= (testGroup | DIAG_TEST_GIGE_AD_SFP ));
        }

        delay_us(500000); // wait for copper setup yes.. without delay get pkt count errs
        // I added a sync  but that didn't help.. doubling the delay works (test looping)
        delay_us(500000); // wait for copper setup yes.. without delay get pkt count errs
    }                                                                                                                                
    return (status);
}



typedef struct {
        char  *str0;
        char  *str1;
        char  *str2;
        char  *str3;
        char  *str4;
} cmtest_set_t;

extern int32 do_ecc( int32   ac, char8 * av[]);

/*********************************************************
* utility to call ecc memnu item which ena / dis ables
* SBE (Single Bit Errors) exception processing
*
*  returns   NONE
**********************************************************/
int
ad_ena_sbe_exc( int ena)
{
    int rc = 0;

    cmtest_set_t mstr;

    mstr.str0 = "";
    mstr.str3 = "\n";
   
    mstr.str1 = "sbe";
    if (ena)
        mstr.str2 = "ena";
    else
        mstr.str2 = "dis";
  
    rc = do_ecc(3,(char **)&mstr);
 
    return (rc);

}
#endif  /* end PROM_SIBYTE_BC */

#endif  /* end (PROM_R9000) || defined(PROM_SIBYTE_BC) */
