/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Bobcat platform related auto diag code.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 4/04/05
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)
                                                                                                                                    
#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>

#include <exports/i2c-api.h>
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#include <exports/seep.h>
#include <exports/gt64440mii.h>
#include <sys/bmfpga.h>

#if defined(PROM_SIBYTE_BC)
#include <exports/bmc12500-eth.h>
#include <exports/lsi8104.h>
#include <sys/sb1250/sb1250_defs.h>
#endif
#include "load-bobcat.h"
#include "load-bobcat-diag.h"

#define AUTO_DIAG_TEST_ONE 0x00010000

#define MEG (1024*1024)
#define BC_PCB_MODEL_LEN (12+1)

extern int domt();  // prom-pmon/memtst.c
extern int dram_memtst();  // prom-pmon/memtst.c
extern int diag_pcmcia_socket_check();  // sm-pmondisk/pcmcia.c

#ifdef PROM_SIBYTE_BC

// Called by bc_run_mod_diags_nfp() in load-bobcat.c
// This will be the list of tests run fo Go No Go testing on Bobcat
unsigned int
runAutoDiags(int shortMem)
{
 int rc = 0, cacheMiss = 0;

   if( isTXRX() )
        printf(" Run TXRX Auto Diags \n");
   else
        printf(" Run FP Auto Diags \n");

   rc = ad_i2c_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_luc_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_lsi_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_gige_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_hypermem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_shmem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_dram_tst( shortMem, cacheMiss);
   if( rc != NFX_OK)
       return(rc);

   /*
    *  New memtest Parameters will be fixed for this sequence
    *  We will be calling it a numkber of times with different params.
    *  ad_dram_tst2( local, cached, gtmem)
    *       1st  call  local = 1 cached = 1 gtmem = 0  (admenu # 60, 64)
    *       2nd  call  local = 0 cached = 1 gtmem = 0  (admenu # 62, 66)
    */
   rc = ad_dram_tst2( 1, 1, 0);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_dram_tst2( 0, 1, 0);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
    return(rc);  // All done 
}

#endif

#ifdef PROM_R9000
// Called by bc_run_ssc_diags() in load-bobcat.c
// This will be the list of tests run fo Go No Go testing on Bobcat
// SSC ONLY !!!  The SSC will have some pre tests that will run before lettiing the TXRX and FP
//  processors run their diags.  SSC must 1st check out the Marval Internal Local SRAM.
//  This is where all communications (States) are sent and received.
unsigned int
runSSC_preDiags( int shortMem)
{
 int rc = 0;

   printf(" Run SSC Pre Diags \n");

   rc = ad_pci_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_pwr_fan_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_ssc_i2c_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_serdes_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_r9k_scratch_mem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_gt_scratch_mem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_ssc_txrx_image_mem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_ssc_fp_image_mem_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_gt_mem_tst( shortMem);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
    return(rc);  // All done 
}

/***************************************************************
** runSSC_AutoDiags runs after the Pre tests so
**  all modules can run at the same time *
**   Returns Status -- If status = 0 (Ok)  *
**    else return test number ored with test error
**      [xxxx.0001]  xxxx = test number  0001 = test error
***************************************************************/
unsigned int
runSSC_AutoDiags( int shortMem)
{
 int rc = 0;

   printf("\n Run SSC Auto Diags \n");

   rc = ad_pci2_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_ssc_i2c2_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_r9k_cache_mem_tst( shortMem);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_r9k_kseg0_mem_tst( shortMem);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_r9k_kseg1_mem_tst( shortMem);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_gt_uncached_mem_tst( shortMem);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   /*
    *  New memtest Parameters will be fixed for this sequence
    *  We will be calling it a numkber of times with different params.
    *  ad_dram_tst2( local, cached, gtmem)
    *       1st  call  local = 1 cached = 1 gtmem = 1  (admenu # 70)
    *       2nd  call  local = 1 cached = 1 gtmem = 0  (admenu # 72)
    */
   rc = ad_dram_tst2( 1, 1, 1);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_dram_tst2( 1, 1, 0);
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_bmfpga_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_rtc_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_cf_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
   rc = ad_eth_tst();
   if( rc != NFX_OK)
       return(rc);
                                                                                                                              
    return(rc);  // All done 
}

#endif

#endif  // PROM_R9000 || PROM_SIBYTE_BC
