/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Bobcat platform related autoload code.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 9/21/04
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)

#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
#if defined(PROM_SIBYTE_BC)
#include <sys/sb1250/sb1250_defs.h>
#endif
#include "load-bobcat.h"
#include "load-bobcat-diag.h"


#define LOAD_BSD_ADDRESS    (0x83000080)

load_bc_tbl_t  load_bc_tbl[] =
{
	{LOAD_BC_SSC_ADDR, 	LOAD_BC_SSC_IMAGE, LOAD_BC_SSC_NAME},
	{LOAD_BC_TXRX_ADDR, LOAD_BC_TXRX_IMAGE, LOAD_BC_TXRX_NAME},
	{LOAD_BC_FP_ADDR, 	LOAD_BC_FP_IMAGE, LOAD_BC_FP_NAME},
	{LOAD_BC_FC_ADDR, 	LOAD_BC_FC_IMAGE, LOAD_BC_FC_NAME},
};

load_cb_t volatile *load_cb_p; 

uchar8 *dummy_argv[1];

void do_cmd( char * );
int32 do_bsd_launch( int32   argc, uchar8  * argv[]);


/*-----------------------------------------------------------------
 * Name: load_bc_get_ssc_state
 *
 * Description: Return SSC auto load state of a given module.
 *
 *-----------------------------------------------------------------
 */
uint32
load_bc_get_ssc_state( int32 mod_id )
{
	return (load_cb_p->state_ssc[mod_id]);
}

/*-----------------------------------------------------------------
 * Name: load_bc_set_ssc_state
 *
 * Description: Set SSC auto load state of a given module.
 *
 *-----------------------------------------------------------------
 */
void
load_bc_set_ssc_state( int32 mod_id, int32 state )
{
	load_cb_p->state_ssc[mod_id] = state;
}


/*-----------------------------------------------------------------
 * Name: load_bc_get_mod_state
 *
 * Description: Return module auto load state of a given module.
 *
 *-----------------------------------------------------------------
 */
uint32
load_bc_get_mod_state( int32 mod_id )
{
	return (load_cb_p->state_mod[mod_id]);
}

/*-----------------------------------------------------------------
 * Name: load_bc_set_mod_state
 *
 * Description: Set module auto load state of a given module.
 *
 *-----------------------------------------------------------------
 */
void
load_bc_set_mod_state( int32 mod_id, int32 state )
{
	load_cb_p->state_mod[mod_id] = state;
}

/*-----------------------------------------------------------------
 * Name: load_bc_is_valid_mod_id
 *
 * Description: Check if given module id is in range.
 *
 *-----------------------------------------------------------------
 */
int32
load_bc_is_valid_mod_id( int32 mod_id )
{
	return (( mod_id >= LOAD_BC_MIN_ID) && ( mod_id < LOAD_BC_MAX_ID));
}

// -----------------------------------------------------------
// WTG  needed away to set STOPPED state for both SSC & SIByte
// Carefull 
//  The pointer is not set up on SiByte until later causing a
//   TLB exception  so this sets it up
// -----------------------------------------------------------
void 
diag_set_mod_state ( int32 mod_id, int32 state)
{
 // Set up the pointer to the struct dependent on defines 
#ifdef PROM_SIBYTE_BC
    load_cb_p = (load_cb_t *)(HW_OC_SRAM_BASE_FROM_TXRX + LOAD_BC_SRAM_OFFSET);
#endif

#ifdef PROM_R9000
    load_cb_p = (load_cb_t *)(OC_SRAM_BASE + LOAD_BC_SRAM_OFFSET);
#endif
  // Now go set the State
    load_bc_set_mod_state( mod_id, state);
}
/*-----------------------------------------------------------------
 * Code that run on Sibyte PROMs only (TXRX & FP)
 *-----------------------------------------------------------------
 */
#ifdef PROM_SIBYTE_BC
extern uint32 autoload_txrxfp_runtime;
void phb_init(void);

extern void ad_module_menu();
/*-----------------------------------------------------------------
 * Name: load_bc_load_mod_nfp
 *
 * Description: 
 *    Set our module autoload state to PROM_INIT.
 *    Wait for SSC to load TXRX/FP image by checking corresponding 
 *    SSC autoload state.  When image has been loaded, SSC will set 
 *    state to RT_LOADED.
 *    Delay 2 seconds (for BSD to bootup). 
 *    Re-initialize PCI window for runtime.
 *    Go to runtime.
 *-----------------------------------------------------------------
 */
void
load_bc_load_mod_nfp( void )
{
	int32	mod_id;

	load_cb_p = (load_cb_t *)(HW_OC_SRAM_BASE_FROM_TXRX + LOAD_BC_SRAM_OFFSET);

	mod_id = isTXRX() ? LOAD_BC_TXRX_ID : LOAD_BC_FP_ID;

	printf("Type ctrl-e to stop autoload.\n");
	printf("Waiting for %s image to be loaded...", 
			load_bc_tbl[mod_id].mod_name);

	load_bc_set_mod_state(mod_id, LOAD_BC_STATE_PROM_INIT);

	while (load_bc_get_ssc_state(mod_id) != LOAD_BC_STATE_RT_LOADED)
	{
		delay_us(100000);
		if (char_avail())
		{
			if (getchar() == CNTRL('e'))
			{
				printf("Autoload stopped.\n");
				return;
			}
		}
                // WTG add a way for the SSC to request a diag function on TXRX and FP
                if( load_bc_get_ssc_state(mod_id) == LOAD_BC_STATE_DIAG_INIT) {
                    printf(" Request from SSC -Diag Menu-\n");
                    ad_module_menu(); // Go to diag Menu mode wait for cmds from SSC
                    // need to return mod state back to PROM_INIT  so autoload will continue
                    load_bc_set_mod_state(mod_id, LOAD_BC_STATE_PROM_INIT);
                    printf(" Return from Diag Menu\n");
                }
	}
	printf("done\n");

	// re-initialize pci window
	autoload_txrxfp_runtime=0;
	phb_init();

	// Delay few seconds before we lauch runtime
	// to sync up with BSD.
	delay_us(2000000);


	do_cmd("g -e 0xffffffff83000000");
}

//int
//runAutoDiags()
//{
//   if( isTXRX() )
//	printf(" Run TXRX Auto Diags \n");
//   else
//	printf(" Run FP Auto Diags \n");
//	
//   return(0); //good status
//}

/*-----------------------------------------------------
 * WTG  Add diag states and report status
 *-----------------------------------------------------
 */
int
bc_run_mod_diags_nfp(int32 shortMem, int32 wait4all)
{
	int32	mod_id, status;

	load_cb_p = (load_cb_t *)(HW_OC_SRAM_BASE_FROM_TXRX + LOAD_BC_SRAM_OFFSET);

	mod_id = isTXRX() ? LOAD_BC_TXRX_ID : LOAD_BC_FP_ID;

	load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_INIT);

        if( wait4all) {
	    printf("Type ctrl-e to stop diags.\n");
	    printf("Waiting for SSC before Diags on %s can run...", 
			load_bc_tbl[mod_id].mod_name);


	    while (load_bc_get_ssc_state(mod_id) != LOAD_BC_STATE_DIAG_INIT)
	    {
		delay_us(100000);
		if (char_avail())
		{
			if (getchar() == CNTRL('e'))
			{
				printf(" Diags stopped.\n");
				load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
				return(1); // 1 = error since we stopped it
			}
		}
	    }
        }
	printf("\n\n Starting Diags\n");
	
	// Run auto diags and set correct state
	load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_RUNNING);
	load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_RUNNING);
	status = runAutoDiags(shortMem);
	if( status)   // Status xxxx.p001 =  [xxxx = test] [p] = port 001 = error
            // save the test number and send it back with msg 
	    load_bc_set_mod_state(mod_id, ((status & 0xFFFFFFF0)|LOAD_BC_STATE_DIAG_FAIL));
        else
	    load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_PASS);
		
        if( wait4all) {
	    printf("  - DB  Diags post status.\n");
	    while (load_bc_get_ssc_state(mod_id) == LOAD_BC_STATE_DIAG_RUNNING)
	    {
                ; // we wait for SSC to get out of running mode so status in mod state doesn't
                  // get over written by autoload PROM_INIT state.
            }
        }
	printf(" Diags Done.\n");
        return(status);  // 0 = Pass
}
#endif

/*-----------------------------------------------------------------
 * Code that run on R9000 PROM only (SSC) 
 *-----------------------------------------------------------------
 */
#ifdef PROM_R9000

int disk_load_file(char *fname, uint32 load_address);

/*-----------------------------------------------------------------
 * Name: load_bc_init
 *
 * Description: Initialize load control block & pointer.
 *
 *-----------------------------------------------------------------
 */
void
load_bc_init( void )
{
	load_cb_p = (load_cb_t *)(OC_SRAM_BASE + LOAD_BC_SRAM_OFFSET);
	memset((load_cb_t *)load_cb_p, 0, sizeof(load_cb_t));
}

/*-----------------------------------------------------------------
 * Name: load_bc_wait_mod_init
 *
 * Description: Wait for module autoload state to be set to INIT.
 *
 *-----------------------------------------------------------------
 */
int32
load_bc_wait_mod_init( int32 mod_id )
{
    uint32  timesUp = 0;

	printf("Type ctrl-e to stop autoload.\n");
	printf("Waiting for %s to enter autoload init state...", 
			load_bc_tbl[mod_id].mod_name);
        timesUp = 150;  /* 15 sec time out value based on delay value */
	do
	{
		delay_us(100000);
                timesUp--;
		if (char_avail())
		{
			if (getchar() == CNTRL('e'))
			{
				printf("Autoload stopped.\n");
				return NFX_ERR;
			}
		}
                if(!timesUp) { /* when timesUp = 0.. time out */
                    printf("  -Timeout- Waiting for Mod Init on %s...\n",load_bc_tbl[mod_id].mod_name);
                    load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_TIMEOUT);
                    return NFX_ERR;
                }

	} while (load_bc_get_mod_state(mod_id) != LOAD_BC_STATE_PROM_INIT);
	printf("done.\n");

	return NFX_OK;
}

void
seepGetTFTPServer(uint8 *ipaddr);

/*-----------------------------------------------------------------
 * Name: load_bc_load_mod_image
 *
 * Description: 
 *     Load image of a given module id from given device. Use
 *     load_bc_tbl[] to determine name of image & load address.
 *
 * Return: Result of diskload (NFX_OK or NFX_ERR).
 *-----------------------------------------------------------------
 */
int32
load_bc_load_mod_image(int32 mod_id, char *dev_str)
{
	char	load_str[128];
    uint8 load_src[ENV_MAX_STR_LEN];
    uint8 ipaddr[ENV_MAX_STR_LEN];

	if ( !load_bc_is_valid_mod_id(mod_id) )
	{
		printf("Invalid mod id %d\n", mod_id);
		return NFX_ERR;
	}

    if (seepenv_get(ENV_KEY_LOAD_SRC, load_src) != NFX_OK) {
        return NFX_ERR;
    }

    seepGetIpAddr(ipaddr);

	load_bc_set_ssc_state( mod_id, LOAD_BC_STATE_RT_LOADING);
	
    if (strcmp(load_src, "disk") == 0) {
        sprintf(load_str, "/dev/%s/%s", dev_str, load_bc_tbl[mod_id].load_name);

        return (disk_load_file(load_str, load_bc_tbl[mod_id].load_addr));
    } else if (strcmp(load_src, "network") == 0) {
        char tftp_server[16];
        seepGetTFTPServer(tftp_server);
        sprintf(load_str, "load -a %x %s %s/%s", 
                load_bc_tbl[mod_id].load_addr,
                tftp_server,
                ipaddr,
                strcmp("bsd", load_bc_tbl[mod_id].load_name) ?
                load_bc_tbl[mod_id].load_name : "vmlinux.bin");
        do_cmd(load_str);
        return NFX_OK;
    } else {
        return NFX_ERR;
    }
}

/*-----------------------------------------------------------------
 * Name: load_bc_load_images
 *
 * Description: Load all images to corresponding load addresses: 
 *              SSC, FP, TXRX & FP.
 *
 *-----------------------------------------------------------------
 */
void
load_bc_load_images( void )
{
	int32	mod_id;
	uchar8  dev_str[ENV_MAX_STR_LEN];
    uint8 load_src[ENV_MAX_STR_LEN];
    uint8 ipaddr[ENV_MAX_STR_LEN];

	// Get boot device string.
    if (seepenv_get(ENV_KEY_BOOT_DEV, dev_str) != NFX_OK)
	{
		printf("Error reading seep env.\n");
		printf("Autoload stopped.\n");
        return; 
	}

    if (seepenv_get(ENV_KEY_LOAD_SRC, load_src) != NFX_OK) {
        printf("Error reading seep env.\n");
        printf("Autoload stopped.\n");
        return; 
    }

    seepGetIpAddr(ipaddr);

    if ((strcmp(load_src, "network") == 0) &&
        (memcmp("10.2.", ipaddr, sizeof("10.2.") - 1) == 0)) {
        do_cmd("bsd rt add 10.0.0.0 255.255.0.0 10.2.0.1");
    }

    if ((strcmp(load_src, "network") == 0) &&
        (memcmp("10.1.", ipaddr, sizeof("10.1.") - 1) == 0)) {
        do_cmd("bsd rt add 10.0.0.0 255.255.0.0 10.1.1.1");
    }

	// Set up SSC state, needed by load_bc_wait_mod_init().
	load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_PROM_INIT);

	// Set up FC state since FC CPU is in reset until cpu1go().
	load_bc_set_mod_state(LOAD_BC_FC_ID, LOAD_BC_STATE_PROM_INIT);

	// Load images to memory.
	for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
	{
		// Stop if ctrl-e is entered.
		if (load_bc_wait_mod_init(mod_id) != NFX_OK) {
                    if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_TIMEOUT) {
                        extern int reboot_cmd();   /* in prom-pmon/commands.c */
                        reboot_cmd();
                    }
                    else {
			return;
                    }
                }
		// Stop if load err (disk/file not found)
		if (load_bc_load_mod_image(mod_id, dev_str) != NFX_OK)
			return;
	}

	// flush all loaded data from cache
	do_cmd("flush -d");

	// Unreset CPU1 (FC) and launch runtime
	do_cmd("cpu1go 13000000");

	// Signal modules that images are loaded.
	for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
	{
		load_bc_set_ssc_state( mod_id, LOAD_BC_STATE_RT_LOADED);
	}
  
	// launch BSD runtime. Will not return.
	//do_bsd_launch( 0, NULL);
	do_bsd_launch( 0, dummy_argv);
}

// =====================================================================
// WTG  Diag work here 
// =====================================================================
// bc_run_scc_diags()  calls 
//
//   runSSC_preDiags()
//     runs a short set of tests to see if other processors are visiable
//     to SSC.  Runs memory tests on shared memory. Sets up Shared memory i
//     to pass diag State info.
//     If any of these tests fail, we stop and jump to PORM Monitor
//
//   runSSC_AutoDiags()
//     After pre tests pass the SCC tells the TXRX and FP to start their
//     own tests. Then SSC continues with it's own tests.
//
//   SSC will then wait for other processors to finish.
//   SSC will print final status.
//
// for testing
// shortMem = 1 if diag env is set
/*---------------------------------------------------------
 * WTG  Add auto diag states, run tests, and report status
 *---------------------------------------------------------
 */
int
bc_run_ssc_diags(int32 shortMem, int32 wait4all)
{
	int32	mod_id, status;
        uint32 timeOut = 15;  // about a 15 second time out

        /* this will allow us to run some tests
	* before we allow TXRX and FP to run their tests */

	// run SSC pre auto diags 
        status = runSSC_preDiags( shortMem); 
        // save the test number and send it back with msg 
        if( status) { // Status xxxx.0001 =  [xxxx = test] 0001 = error
            load_bc_set_mod_state(LOAD_BC_SSC_ID, ((status & 0xFFFFFFF0)|LOAD_BC_STATE_DIAG_FAIL));
            return(status); // don't run the rest if error
        }

	// Set up SSC state in all mod Id's, so TXRX, and FP get started .
        for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
        {
	    //load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_DIAG_INIT);
	    load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_INIT);
        }
        // Set up FC state since FC CPU is in reset .
        load_bc_set_mod_state(LOAD_BC_FC_ID, LOAD_BC_STATE_DIAG_PASS);
        load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_DIAG_RUNNING);

        // Now that all are started set ssc_state to running so we all stay in sync
        for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
        {
	    // Wait for all to be in RUNNING State
            delay_us(100000);
            if (load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_RUNNING) {
                // Now set SSC state to Running also
	        load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_RUNNING);
            }
        }

	// run SSC  auto diags 
        status = runSSC_AutoDiags(shortMem);
	if( status)   // Status xxxx.0001 =  [xxxx = test] 0001 = error
            // save the test number and send it back with msg 
                load_bc_set_mod_state(LOAD_BC_SSC_ID, ((status & 0xFFFFFFF0)|LOAD_BC_STATE_DIAG_FAIL));
        else
                load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_DIAG_PASS);

        if( wait4all) {

	    // Now check for diag status on all  SSC, TXRX, FP, and FC
            printf(" Checking for Diags Done.\n");
	    printf("Type ctrl-e to stop Waiting for Diags done on Module.\n");

            for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
            {
                timeOut = 420;  // about a 7 minute time out
	        // Wait for all to be in NOT RUNNING State
                while ((load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_RUNNING) ||
		        (load_bc_get_mod_state(mod_id) == 0) ||   // The initial state (hasen't got that far yet)
	                (load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_INIT))
                {
                    delay_us(1000000);
                    timeOut--;
                    if (char_avail())
                    {
                        if (getchar() == CNTRL('e'))
                        {
                            printf(" Diags stopped on %s...\n",load_bc_tbl[mod_id].mod_name);
                            load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
                        }
                    }
                    if( timeOut <= 0) {
                        printf("  -Diag test Timeout on %s...\n",load_bc_tbl[mod_id].mod_name);
                        load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_TIMEOUT);
                        break;
                    }
                }
            }

    	    // Print Status of all modules diag test results <<<AND Release TXRX/FP from their status hold state>>>>	
            for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
            {
                printf(" Diags for %4s: ",load_bc_tbl[mod_id].mod_name);
                if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_PASS)
                    printf("  PASS");
                // Only an Error gets it's test number passed back with it's state
                //   Test number is upper 1/2 of State ie xxxx.psss  [xxxx] = test number [p] = port [sss] = state
                else if( (load_bc_get_mod_state(mod_id) & 0x0000000F) == LOAD_BC_STATE_DIAG_FAIL) {
                    printf("  FAILED");
                    // Call Error decode function 
                    autoDiagErrorPrint( load_bc_get_mod_state(mod_id)); // Send the 32 bits of status
                    status = 1;  // report back an error 
                }
                else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_STOPPED) {
                    printf("  STOPPED");
                    status = 1;  // report back an error 
                }
                else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_TIMEOUT) {
                    printf("  TIME-OUT");
                    status = 1;  // report back an error 
                }
                else {
                    printf("  UNKNOWN");
	            status = 1;  // report back an error
                }

                // RELEASE TXRX/FP  HOLD on STATUS POST .....
	        load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
            
                printf("\n");
            }
        }

        printf("\n Diags Done.\n");
        return(status);  // 0 = Pass
}



#endif

void
autoload_bc( void )
{
	watchdog_disable();

#ifdef PROM_SIBYTE_BC
	load_bc_load_mod_nfp();
#endif

#ifdef PROM_R9000
	load_bc_load_images();
#endif

	watchdog_restart();
}

//WTG  add auto diag calls
int
autodiag_bc( int32 shortMem, int32 wait4all )
{
	watchdog_disable();
#ifdef PROM_SIBYTE_BC
	return(bc_run_mod_diags_nfp(shortMem, wait4all));
#endif
#ifdef PROM_R9000
	return(bc_run_ssc_diags(shortMem, wait4all));
#endif
	watchdog_restart();
}

/***************************************************************
* Called from Prom menu. 
* Allow running of the Functional tests.
*  If doNotWait is set  the TXRX / FP will not wait
*    for the SSC to run Pre tests... Beware this could cause
*    some conflicts with memory tests.
* shortMem if set does a very short lenght of 40000 on memory
****************************************************************/
int
do_autodiag( int32  argc, uchar8  * argv[] )
{

  int rc = 0;
  int32 shortMem = 0;  // default to long (all memory)
  int32 wait4all = 1;  // default to wait for all uP to talk

    if (argc >= 2) {
        if (!strncmp(argv[1], "short", 5)) 
            shortMem = 1; // do very short memory tests
        else if (!strncmp(argv[1], "long", 4))
            shortMem = 0;
        else if (!strncmp(argv[1], "wait", 4))
            wait4all = 1;
        else if (!strncmp(argv[1], "nowait", 6))
            wait4all = 0;
        else
            return -1;
    }

    if( argc > 2) {
        if (!strncmp(argv[2], "nowait", 6)) 
            wait4all = 0;  // if not run from SSC just run tests
        else if (!strncmp(argv[2], "wait", 4))
            wait4all = 1;
        else
            return -1;
    }
   rc = autodiag_bc(shortMem, wait4all);
   return(rc);
}

#endif	// PROM_R9000 || PROM_SIBYTE_BC
