/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: load-flash.c,v 1.1 2002/09/25 23:16:17 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM
#include <nfx-incl.h>
#include <exports/prom-part-api.h>
#include "load.h"

typedef struct
{
    char *      load_p;
    uint32 *    size_p;
} LOAD_HANDLE_T;

LOAD_HANDLE_T handle;

/*-----------------------------------------------------------------
 * Name:	load_flash_sw
 *
 * Description: call back function to process the decompressed
 *              executable so that its ready to be invoked
 *
 *				Returns NFX_OK on successful load
 *
 * Created by:	John Hahn
 *
 * Date Created:	9/24/02
 *
 *-----------------------------------------------------------------
 */
static int
load_flash_sw(
    void *  uhdl,
    char *  buf_p,
    int     dlen)
{
    if (handle.load_p && (handle.load_p != buf_p)) {
        memcpy(handle.load_p, buf_p, dlen);
    }
    else
        handle.load_p = buf_p;

    if (handle.size_p) {
        *handle.size_p = dlen;
    }
    set_entry_addr((address_t)handle.load_p);
    printf(" %d (0x%x) bytes @ %p\n", dlen, dlen, handle.load_p);
    return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:	load_flash
 *
 * Description: Load sw from flash to the default load location or 
 *              user specified location
 *
 *				Returns NFX_OK on successful load
 *
 * Created by:	John Hahn
 *
 * Date Created:	9/24/02
 *
 *-----------------------------------------------------------------
 */
int32
load_flash(
	char8 *     fname,
	address_t	addr,
	uint32 *    nbytes
)
{
#ifdef AGILE_HARDWARE
    int rc;
    unsigned short prom_type;

    prom_part_desc_p desc_p;

    desc_p = prom_part_get_desc_from_name(fname);
    if (desc_p == NULL) {
        if (strncmp(fname, "fl", 2) == 0) {
#ifdef SIBYTE
            prom_type = PROM_PART_SIBYTE_DIAG;
#endif
#ifdef R7000
            prom_type = PROM_PART_R7K_DIAG;
#endif
        }
        else {
            printf("..  unknown fname\n");
            return NFX_ERR;
        }
    }
    else {
        prom_type = desc_p->part_type;
    }

    handle.load_p = (char *)addr;
    handle.size_p = nbytes;

    rc = prom_part_proc_data(
            prom_type,
            load_flash_sw,
            (void *)&handle);
    return(rc);
#else
    return(0);
#endif
}

#endif /* PROM */
