/*  
 *=============================================================================
 * File Name   :    lsi8104.c
 *
 *  Description:    LSI MAC routines.
 *
 *  Created    :    01-03-2002
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2002
 *=============================================================================
 */
#include <nfx-incl.h>
#ifdef SIBYTE

#include "lsi8104.h"

// WTG add Diag stuff
#include <exports/load-bobcat-diag.h>  // this has the SFP detect defines.
int ad_detect_sfp( uint32);
//

int lsi_gmac_write(int, uint32, uint16);
int lsi_gmac_read(int, uint32);
uint32 lsi_get_txPktCnt(int);
uint32 lsi_get_txByteCnt(int);
uint32 lsi_get_rxPktCnt(int);
uint32 lsi_get_rxByteCnt(int);
void diag_lsi_clearStats(int);
void lsi_get_stats(int, uint32 *);

extern void delay_us(uint32);
extern int get_cmd(char8 *);

long lsi_base_addr[LSI_MAX_NUM_DEVICES] = { 
											LSI_GMAC_BASE_ADDR0,
	 										LSI_GMAC_BASE_ADDR1, 
											LSI_GMAC_BASE_ADDR2, 
											LSI_GMAC_BASE_ADDR3, 
											LSI_GMAC_BASE_ADDR4, 
											LSI_GMAC_BASE_ADDR5};

/*
 *------------------------------------------------------------------------------
 * Function Name:	lsi_gmac_init
 *
 * Description: 	Initializes the LSI GMAC device.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	01-03-2002
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
lsi_gmac_init(int unit)
{
    uint32 portType = 0;  // New diag.. Check SFP Seep and set up lsi 

	lsi_gmac_conf1_t conf1;
	lsi_gmac_conf2_t conf2;
	lsi_gmac_conf3_t conf3;
	lsi_gmac_conf4_t conf4;
  
        // WTG go detect the SFP by reading Seep. Set LSI accordingly
        portType = ad_detect_sfp( unit); // get port Type


	lsi_gmac_write(unit, LSI_GMAC_CONFIG1, 0xf000); /* reset */
	delay_us(10000);

	/* all register settings provided by HW LUC spec */

	lsi_gmac_write(unit, LSI_GMAC_FILTER1, 0xffff); /* pass all */
	lsi_gmac_write(unit, LSI_GMAC_FILTER2, 0xffff); /* pass all */
	lsi_gmac_write(unit, LSI_GMAC_FILTER3, 0xffff); /* pass all */
	lsi_gmac_write(unit, LSI_GMAC_FILTER4, 0xffff); /* pass all */

	conf1.wd = 0;
	conf1.bits.rxeof = 1;
	conf1.bits.txcrc = 1;
	conf1.bits.txprmbl = 1;
	conf1.bits.ipg = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG1, conf1.wd);

	conf2.wd = 0;
	conf2.bits.discabort = 1;
	conf2.bits.disccode = 1;
	conf2.bits.discosize = 0;
	conf2.bits.discusize = 1;
	conf2.bits.disccrc = 1;
	conf2.bits.discoflow = 1;
#if defined(NFP_FP)
	conf2.bits.rxaccept = 1;
#else
	conf2.bits.rxaccept = 0;
#endif	
	lsi_gmac_write(unit, LSI_GMAC_CONFIG2, conf2.wd);

	conf3.wd = 0;
	conf3.bits.rxmax = 2;

        // WTG set up auto-negotiate depending on SFP type.
        if( portType == AD_FIBER)
	    conf3.bits.anenable = 1;
        else if (portType == AD_COPPER)
	    conf3.bits.anenable = 0;
        else 
	    conf3.bits.anenable = 1;

	conf3.bits.autoabort = 1;
	conf3.bits.autoclr = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG3, conf3.wd);
	
	conf4.wd = 0;	
	conf4.bits.cntmaxsize = 3;
	conf4.bits.endian = 1;
#ifdef NFP_FP
	if ((unit == 2) || (unit == 5)) // backplane control ports no longer used	
		conf4.bits.txdisable = 1;
#endif
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf4.wd);

	if (isTXRX()) {
		lsi_gmac_write(unit, LSI_GMAC_TX_THRESH, 0xf781);
		lsi_gmac_write(unit, LSI_GMAC_RX_THRESH, 0x4c4c);
		lsi_gmac_write(unit, LSI_GMAC_FLOWCTRL1, 0x8d00);
	}
	else {
		lsi_gmac_write(unit, LSI_GMAC_TX_THRESH, 0xd781);
		lsi_gmac_write(unit, LSI_GMAC_RX_THRESH, 0x4cfe);
		lsi_gmac_write(unit, LSI_GMAC_FLOWCTRL1, 0x8c00);
	}

	return(NFX_OK);

} /* lsi_gmac_init */


/*
 *------------------------------------------------------------------------------
 * Function Name:	lsi_gmac_wait
 *
 * Description: 	wait for lmux to become ready for timeout msecs
 *
 * Created by:		John Hahn
 *
 * Data Created:	10-09-2002
 * 					
 * Returns: 0 if timed out  
 *
 *------------------------------------------------------------------------------
 */
int
lsi_gmac_wait(uint32 timeout)
{
    uint16 reg_val = 0;

    while (reg_val != 0x1234 && timeout) {
        lsi_gmac_write(0, LSI_GMAC_MAC1, 0x1234);
        lsi_gmac_write(0, LSI_GMAC_MAC2, 0x5678);
        reg_val = lsi_gmac_read(0, LSI_GMAC_MAC1);
        delay_us(1000);
        --timeout;
    }
    if (timeout == 0)
        printf("lsi_gmac_wait: timeout waiting for lmux to become ready\n");

//printf("lsi  TO %d\n", timeout);
    return timeout;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:	lsi_init
 *
 * Description: 	Initializes all LSI GMAC devices.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	01-03-2002
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
lsi_init()
{
	uint32 unit;
	uint32 num_devices = 0;
    volatile uint32 *reset;

    if (isTXRX()) {
        num_devices = LSI_NUM_DEVICES_TXRX;
        reset = (volatile uint32 *)0xffffffffbf4000a8;
        *reset = 0; /* take network MACs out of reset */
        reset = (volatile uint32 *)0xffffffffbf4000a4;
        *reset = 0; /* take backplane MACs out of reset */
        delay_us(1000);
    }
    else
    if (isFP()) {
        num_devices = LSI_NUM_DEVICES_FP;
        // wait 10 secs max for lmux to ready itself 
        lsi_gmac_wait(10000);
    }

	if (num_devices > 0) {
		for (unit = 0; unit < num_devices; unit++)
			lsi_gmac_init(unit);
	}
	return NFX_OK;
}

int
lsi_gmac_write(int unit, uint32 reg, uint16 val)
{
	uint16 *long_addr;

	long_addr = (uint16 *)(lsi_base_addr[unit] + reg*2);
	*long_addr = val;
	return NFX_OK;
}

int
lsi_gmac_read(int unit, uint32 reg)
{
	uint16 *long_addr;
	uint16 val;

	long_addr = (uint16 *)(lsi_base_addr[unit] + reg*2);
	val = *long_addr;
/*
	printf("lsi_gmac_read: val at %lx = %d\n", long_addr, val);
*/
	return(val);
}

int
diag_lsi_uiGetInput( char *msg )
{
	int sel;
	char cmd[80];

	printf("%s: ", msg);
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	return sel;
} /* diag_lsi_uiGetInput */

int
diag_lsi_uiGetHex( char *msg )
{
	uint32 sel;
	char cmd[80];

	printf("%s: ", msg);
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%x", &sel);
	return sel;
} /* diag_lsi_uiGetHex */

void
diag_lsi_readReg(int unit)
{
	uint32 reg_addr;
	uint16 reg_val;

	reg_addr = diag_lsi_uiGetHex("Enter reg offset(hex): ");

	reg_val = lsi_gmac_read(unit, reg_addr);

	printf("Value at %0x: %0x\n", reg_addr, reg_val);
}

void
diag_lsi_writeReg(int unit)
{
	uint32 reg_addr;
	uint16 reg_val;

	reg_addr = diag_lsi_uiGetHex("Enter reg offset(hex): ");

	reg_val = diag_lsi_uiGetHex("Enter reg value(hex): ");

	lsi_gmac_write(unit, reg_addr, reg_val);
}

void 
diag_lsi_setLoop(int unit)
{
	uint32 enable;
	lsi_gmac_conf4_t conf;

	enable = diag_lsi_uiGetInput("Enter loopback mode (1=Enable, 0=Disable): ");
	if (enable > 1)
	{
   		printf("Invalid entry!\n");
   	 	return;
	}
	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	conf.bits.loopback = enable;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG4, conf.wd);
}

uint32
lsi_get_txPktCnt(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX1);
	return( data0 | (data1 << 16));
}

uint32
lsi_get_txByteCnt(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX1);
	return( data0 | (data1 << 16));
}

uint32
lsi_get_rxPktCnt(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS1);
	return( data0 | (data1 << 16));
}

uint32
lsi_get_rxByteCnt(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS1);
	return( data0 | (data1 << 16));
}

void
diag_lsi_viewStats(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS1);
	printf("Rx packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS1);
	printf("Rx bytes: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_BCASTPKTS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_BCASTPKTS1);
	printf("Rx broadcast packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_MCASTPKTS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_MCASTPKTS1);
	printf("Rx multicast packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_RX_CTRL0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_RX_CTRL1);
	printf("Rx ctrl packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX1);
	printf("Tx packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX1);
	printf("Tx bytes: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_BCAST_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_BCAST_TX1);
	printf("Tx broadcast packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_MCAST_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_MCAST_TX1);
	printf("Tx multicast packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_TX_CTRL0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_TX_CTRL1);
	printf("Tx ctrl packets: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_CRCALIGN0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_CRCALIGN1);
	printf("CRC align errors: %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_UNDERSIZE0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_UNDERSIZE1);
	printf("Undersized errors: %d\n", data0 | (data1 << 16));

    data0 = lsi_gmac_read(unit, LSI_GMAC_TX_DISC0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_TX_DISC1);
    printf("Tx underflows: %d\n", data0 | (data1 << 16));

    data0 = lsi_gmac_read(unit, LSI_GMAC_TX_BAD0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_TX_BAD1);
    printf("Tx errors: %d\n", data0 | (data1 << 16));

    data0 = lsi_gmac_read(unit, LSI_GMAC_RX_SYM0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_RX_SYM1);
    printf("Rx symbol errors: %d\n", data0 | (data1 << 16));

    data0 = lsi_gmac_read(unit, LSI_GMAC_RX_DROPS0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_RX_DROPS1);
    printf("Rx overflows: %d\n", data0 | (data1 << 16));
}

void
diag_lsi_viewStats_more(int unit)
{
	uint32 data0, data1;

	data0 = lsi_gmac_read(unit, LSI_GMAC_64_RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_64_RX1);
	printf("Rx packets (64 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_65_127RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_65_127RX1);
	printf("Rx packets (65-127 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_128_255RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_128_255RX1);
	printf("Rx packets (128-255 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_256_511RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_256_511RX1);
	printf("Rx packets (256-511 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_512_1023RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_512_1023RX1);
	printf("Rx packets (512-1023 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_1024_1518RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_1024_1518RX1);
	printf("Rx packets (1024-max bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_64_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_64_TX1);
	printf("Tx packets (64 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_65_127TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_65_127TX1);
	printf("Tx packets (65-127 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_128_255TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_128_255TX1);
	printf("Tx packets (128-255 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_256_511TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_256_511TX1);
	printf("Tx packets (256-511 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_512_1023TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_512_1023TX1);
	printf("Tx packets (512-1023 bytes): %d\n", data0 | (data1 << 16));

	data0 = lsi_gmac_read(unit, LSI_GMAC_1024_1518TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_1024_1518TX1);
	printf("Tx packets (1024-max bytes): %d\n", data0 | (data1 << 16));
}

/*
 *  This routine can be polled to gather stats for SNMP. All counters
 *  are cleared after each call. Tx/Rx byte counters can potentially 
 *  overflow if not read at least every 34 seconds. Counters returned 
 *  are sequential as listed below and match descriptions in LSI 8104 
 *  Technical Manual. Caller must pass pointer to buffer with space
 *  for 28 32-bit values for TxRx and 10 32-bit values for FP. 
 *
 *  Justification: Customer doesn't need to know counts of packets in 
 *  certain size ranges on backplane. Also don't need to know count of
 *  Tx/Rx pause frames on backplane.
 *
 *  Counters: 1,2,3,4,5,6,7,12,13,14,15,16,17,18,19,20,21,22,23,24,25,
 *  26,27,34,35,50,52,53
 *
 *  Note: Return all for TxRx, but only get subset for FP since some backplane
 *  stats are not really useful for customer to see. Need to avoid unnecessary
 *  register reads on FP since it needs all the CPU power it can get.
 *
 *  Counters for FP: 1,2,3,6,7,18,19,34,35,50
 */
void lsi_get_stats(int unit, uint32 *stat_buf)
{
	uint32 data0, data1;

    data0 = lsi_gmac_read(unit, LSI_GMAC_RX_DROPS0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_RX_DROPS1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS1);
	*stat_buf++ = data0 | (data1 << 16);

#ifdef NFP_TXRX
    data0 = lsi_gmac_read(unit, LSI_GMAC_BCASTPKTS0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_BCASTPKTS1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_MCASTPKTS0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_MCASTPKTS1);
	*stat_buf++ = data0 | (data1 << 16);
#endif

    data0 = lsi_gmac_read(unit, LSI_GMAC_CRCALIGN0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_CRCALIGN1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_UNDERSIZE0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_UNDERSIZE1);
	*stat_buf++ = data0 | (data1 << 16);

#ifdef NFP_TXRX
	data0 = lsi_gmac_read(unit, LSI_GMAC_64_RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_64_RX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_65_127RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_65_127RX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_128_255RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_128_255RX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_256_511RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_256_511RX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_512_1023RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_512_1023RX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_1024_1518RX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_1024_1518RX1);
	*stat_buf++ = data0 | (data1 << 16);
#endif

    data0 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_OCTETS_TX1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_PKTS_TX1);
	*stat_buf++ = data0 | (data1 << 16);

#ifdef NFP_TXRX
    data0 = lsi_gmac_read(unit, LSI_GMAC_BCAST_TX0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_BCAST_TX1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_MCAST_TX0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_MCAST_TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_64_TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_64_TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_65_127TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_65_127TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_128_255TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_128_255TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_256_511TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_256_511TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_512_1023TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_512_1023TX1);
	*stat_buf++ = data0 | (data1 << 16);

	data0 = lsi_gmac_read(unit, LSI_GMAC_1024_1518TX0);
	data1 = lsi_gmac_read(unit, LSI_GMAC_1024_1518TX1);
	*stat_buf++ = data0 | (data1 << 16);
#endif

    data0 = lsi_gmac_read(unit, LSI_GMAC_TX_DISC0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_TX_DISC1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_TX_BAD0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_TX_BAD1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_RX_SYM0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_RX_SYM1);
	*stat_buf++ = data0 | (data1 << 16);

#ifdef NFP_TXRX
    data0 = lsi_gmac_read(unit, LSI_GMAC_TX_CTRL0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_TX_CTRL1);
	*stat_buf++ = data0 | (data1 << 16);

    data0 = lsi_gmac_read(unit, LSI_GMAC_RX_CTRL0);
    data1 = lsi_gmac_read(unit, LSI_GMAC_RX_CTRL1);
	*stat_buf++ = data0 | (data1 << 16);
#endif
	diag_lsi_clearStats(unit);
}

void
diag_lsi_viewStatus(int unit)
{
	lsi_gmac_status_t status;
	lsi_gmac_conf4_t conf;

	status.wd = lsi_gmac_read(unit, LSI_GMAC_STATUS);
	printf("MAC status\n");
	printf("----------\n");
	printf("Synchronized: ");
	if (status.bits.rxsync)
		printf("Yes\n");
	else
		printf("No\n");
	printf("Signal Detect: ");
	if (status.bits.sigdetect)
		printf("High\n");
	else
		printf("Low\n");
	printf("Link: ");
	if (status.bits.link)
		printf("Pass\n");
	else
		printf("Fail\n");

	conf.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG4);
	printf("Loopback: ");
	if (conf.bits.loopback)
		printf("On\n");
	else
		printf("Off\n");
}

void
diag_lsi_clearStats(int unit)
{
	lsi_gmac_conf1_t conf1;

	conf1.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG1);
	conf1.bits.cntrst = 1;
	lsi_gmac_write(unit, LSI_GMAC_CONFIG1, conf1.wd);
}

void 
diag_lsi_reg_test()
{
	int i, regval;
	int num_devices = 0;

	if (isTXRX())
		num_devices = LSI_NUM_DEVICES_TXRX;
	else
	if (isFP())
		num_devices = LSI_NUM_DEVICES_FP;

	if (num_devices > 0) {

		for (i = 0; i < num_devices; i++) 
			lsi_gmac_write(i, 0, i+1);

		for (i = 0; i < num_devices; i++) {
			regval = lsi_gmac_read(i, 0);
			if (regval != i+1) 
				printf("Test failed on device #%d\n", i);
			else
				printf("Test passed on device #%d\n", i);
		}
		for (i = 0; i < num_devices; i++) 
			lsi_gmac_write(i, 0, 0);

		for (i = 0; i < num_devices; i++) {
			regval = lsi_gmac_read(i, 0);
			if (regval != 0)	
				printf("Test failed on device #%d\n", i);
			else
				printf("Test passed on device #%d\n", i);
		}
	}
	else
		printf("No devices found!\n");
}

void lsi_set_mac_addr(int unit, uchar8 *mac_addr)
{
	lsi_gmac_write(unit, LSI_GMAC_MAC1, (mac_addr[5] << 8) | mac_addr[4]);
    lsi_gmac_write(unit, LSI_GMAC_MAC2, (mac_addr[3] << 8) | mac_addr[2]);
    lsi_gmac_write(unit, LSI_GMAC_MAC3, (mac_addr[1] << 8) | mac_addr[0]);
}

int lsi_set_autoneg(uint16 unit, uint16 mode)
{
	lsi_gmac_conf1_t conf1;
	lsi_gmac_conf3_t conf3;

	if (!isTXRX()) {
		printf("lsi_set_autoneg: only available on TXRX!\n");
		return NFX_ERR;
	}
	if (unit > LSI_NUM_DEVICES_TXRX-1) {
		printf("lsi_set_autoneg: invalid port number = %d\n", unit);
		return NFX_ERR;
	}
	if (mode == 1) {
		conf3.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG3);
		conf3.bits.anenable = 1;
		lsi_gmac_write(unit, LSI_GMAC_CONFIG3, conf3.wd);
		conf1.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG1);
		conf1.bits.anrst = 1;
		lsi_gmac_write(unit, LSI_GMAC_CONFIG1, conf1.wd);
	}
	else 
	if (mode == 0) {
		conf3.wd = lsi_gmac_read(unit, LSI_GMAC_CONFIG3);
		conf3.bits.anenable = 0;
		lsi_gmac_write(unit, LSI_GMAC_CONFIG3, conf3.wd);
	}
	else {
		printf("lsi_set_autoneg: invalid mode = %d\n", mode);
		return NFX_ERR;
	}
	return NFX_OK;
}

int lsi_get_linkstatus(uint16 unit)
{
    lsi_gmac_status_t status;
    uint16 debounce1, debounce2;

    if (!isTXRX()) {
        printf("lsi_get_linkstatus: only available on TXRX!\n");
        return NFX_ERR;
    }
    if (unit > LSI_NUM_DEVICES_TXRX-1) {
        printf("lsi_get_linkstatus: invalid port number = %d\n", unit);
        return NFX_ERR;
    }
    /* status is not reliable when auto neg is off, so check a few times  */
    status.wd = lsi_gmac_read(unit, LSI_GMAC_STATUS);
    debounce1 = status.bits.link;
    status.wd = lsi_gmac_read(unit, LSI_GMAC_STATUS);
    debounce2 = status.bits.link;
    status.wd = lsi_gmac_read(unit, LSI_GMAC_STATUS);
    if (debounce1 && debounce2 && status.bits.link)
        return 1; /* link up */
    else
        return 0; /* link down */
}

int
lsi_gmac_uiShowDiagMenu(uint16 unit)
{
    int sel;
    char cmd[80];

    sel = 0;
    printf("LSI MAC Menu: %d\n", unit);
    printf("===============\n");
    printf("1 : Read reg\n");
    printf("2 : Write reg\n");
    printf("3 : Set loopback\n");
    printf("4 : View statistics\n");
    printf("5 : View status\n");
	printf("6 : Init LSI\n");
	printf("7 : Clear statistics\n");
	printf("8 : Run register test\n");
	printf("99: Quit\n");
	printf("Enter Selection: ");
    memset(cmd, 0, 80);
    get_cmd(cmd);
    sscanf(cmd, "%d", &sel);
    printf("\n");
    return sel;
}

int
lsi_gmac_uiDiag()
{
	int sel, unit;
	int num_devices = 0;

	unit = diag_lsi_uiGetInput("Enter unit number: ");
	if (isTXRX())
		num_devices = LSI_NUM_DEVICES_TXRX;
	else
	if (isFP())
		num_devices = LSI_NUM_DEVICES_FP;
	if (unit < 0 || unit >= num_devices) {
		printf("Invalid entry!\n");
		return(NFX_OK);
	}
	for (;;) {
		sel = lsi_gmac_uiShowDiagMenu(unit);
		switch (sel)
		{
			case 1: diag_lsi_readReg(unit);
					break;
			case 2: diag_lsi_writeReg(unit);
					break;
			case 3: diag_lsi_setLoop(unit);
					break;
			case 4: diag_lsi_viewStats(unit);
					printf("\n");
					diag_lsi_viewStats_more(unit);
					break;
			case 5: diag_lsi_viewStatus(unit);
					break;
			case 6: lsi_gmac_init(unit);
					break;
			case 7: diag_lsi_clearStats(unit);
					break;
			case 8: diag_lsi_reg_test();
					break;
			case 99:return(0);
			default:break;
		}
	}
	return(NFX_OK);
}
#endif /* SIBYTE */
