/* vi:ts=4 
 *-----------------------------------------------------------------
 *
 * Name: luc-rx.c
 *
 * RCS:  $Id: luc-rx.c,v 1.80 2002/11/01 00:15:15 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains LUC Receive related functions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 5/10/2001
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include <exports/cod-api.h>
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include <exports/tpl-api.h>
#ifndef PROM
#include <exports/sbm-api.h>
#endif
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/udp.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/ip_stats.h"
#include "../sm-bsd-netinet/udp_stats.h"
#include "../sm-bsd-net/ethernet.h"
#include "../sm-tpl/tpl.h"
#include "../sm-bsd-api/bsdrelay.h"

#include "luc-cam.h"
#include "luc-pkthdr.h"

void luc_dbgDescChain( pkt_desc_t *pktDesc);

pkt_desc_t*	luc_Reassem(pkt_desc_t  *pdesc, int link, void** pConn);
pkt_desc_t*	PktDescPtr;
int32		DataSize=0;

extern uint32	myIpaddr;
extern ushort16 nfsPort;

#define LUC_8K_COD		(8*1024)

//#define LUC_DBG_COD     1

static void 
luc_extractHdrInfoHw( pkt_desc_t		*pDesc, 
					  char				*pkt,  
					  luc_rxPktHdr_t	*pLucPktHdr)
					
{
	struct	ip			*ipHdr;
	ushort16			ipOffset;
	uchar8				classifier;
	uchar8				l3Offset, l4Offset, l5Offset;
	ushort16			index;
	ushort16			chkSum;

	/* l3 offset in short word */
	l3Offset = pLucPktHdr->lucrx_l3OffHW*2;
	chkSum = pLucPktHdr->lucrx_chkSum; /* network order */

	l4Offset = pLucPktHdr->lucrx_l4Offset;
	l5Offset = pLucPktHdr->lucrx_l5Offset;
	classifier= pLucPktHdr->lucrx_classifier;
	index    = ntohs(pLucPktHdr->lucrx_camIdx);


	/* setup pMac and pIp, since the pkts here must be arp or ip */
	pDesc->pMac = pkt;
	pDesc->pIpHdr = pkt + l3Offset;

	if (classifier == LUC_RXPKT_CLASS_ARP) {
		pDesc->pktSize = pDesc->hdrSize = 14 + 28; /* hard code for now */
		pDesc->protocol = htons(ETHERTYPE_ARP);
		return;
	}
	

    /*-------------------------------
	 * Extract IP Dst & Src Addresses.
     *-------------------------------
     */
	pDesc->protocol = htons(ETHERTYPE_IP);
	ipHdr = (struct ip *)pDesc->pIpHdr;
	pDesc->laddr  = (ipHdr->ip_dst.s_addr);
	pDesc->raddr  = (ipHdr->ip_src.s_addr);

	pDesc->pTplHdr = pkt + l4Offset;
	
	/* set pkt length for ip pkt */
	pDesc->pktSize = ntohs(ipHdr->ip_len) + l3Offset;

	
    /*-------------------------------
	 * Pkt is an IP fragment if More 
	 * frag bit or offset is set.
     *-------------------------------
     */
	ipOffset = ntohs(ipHdr->ip_off); 
	if (classifier == LUC_RXPKT_CLASS_FRAG) {
		pDesc->chksum = chkSum;
		pDesc->flags |= PKT_IP_FRAGMENT;
	}
	
	if (ipHdr->ip_p == TPL_PROT_UDP) {
		struct	udphdr	*udpHdr;

	    /*-------------------------------
		 * Not 1st IP fragment, app. hdr
		 * is after IP header.
	     *-------------------------------
	     */
		if ((pDesc->flags & PKT_IP_FRAGMENT) && 
			(ipOffset & IP_OFFMASK)) {
			
			/* update pHdr, hardware may not have l5 offset set */
			pDesc->pHdr = pDesc->pTplHdr;
            pDesc->lport = 0;
            pDesc->rport = 0;

		}
		else {

		    /*-------------------------------
			 * Extract UDP Dst & Src ports.
			 * Setup App. hdr ptr.
		     *-------------------------------
		     */
			udpHdr = (struct udphdr *)pDesc->pTplHdr;
			pDesc->lport = (udpHdr->uh_dport);
			pDesc->rport = (udpHdr->uh_sport);
			// pDesc->pHdr  = pkt + l5Offset;  /* l5 offset not availale yet */
			pDesc->pHdr = pDesc->pTplHdr + 8;

#if 0
			/* cam not supported */
			/* set up camindex if applicable */
			if (!(index & LUC_RXPKT_CAM_MISS)) {
				index  -= CAM_FLOW_BASE_IDX;
				pDesc->camIdx = index;
			}
#endif 

		}
		pDesc->tplProt= TPL_PROT_UDP;
	}
	else if (ipHdr->ip_p == TPL_PROT_TCP)	{
		
		struct	tcphdr	*tcpHdr;

	    /*-------------------------------
		 * Extract TCP Dst & Src ports.
		 * Setup App. hdr ptr.
	     *-------------------------------
	     */
		tcpHdr = (struct tcphdr *)pDesc->pTplHdr;
		pDesc->lport = (tcpHdr->th_dport);
		pDesc->rport = (tcpHdr->th_sport);
		pDesc->tplProt= TPL_PROT_TCP;
		pDesc->pHdr  = pkt + l5Offset;

#if 0
		/* set up camindex if applicable */
		if (!(index & LUC_RXPKT_CAM_MISS)) {
			index  -= CAM_FLOW_BASE_IDX;
			pDesc->camIdx = index;
		}
#endif 
	}
	else
	{
		pDesc->pHdr    = pDesc->pTplHdr;
		pDesc->tplProt = ipHdr->ip_p;
	}
	
    /*-------------------------------
     * Setup hdrSize for App. layer. 
     *-------------------------------
	*/
	pDesc->hdrSize = ntohs(ipHdr->ip_len) - (pDesc->pHdr - pDesc->pIpHdr);

	return;
}


static void 
luc_extractHdrInfo( pkt_desc_t *pDesc, char *pkt, uint32 pktLen)
{
	struct	ip			*ipHdr;
	pkt_ethHdr_t		*macHdr;
	ushort16			ipOffset;
	int					tcpOffset = 0;

#ifdef LINUX_TEST
    /*-------------------------------
	 * RAW Socket input. No MAC parsing
     *-------------------------------
     */
	pDesc->pMac     = pkt;
	pDesc->pIpHdr   = pkt;
	pDesc->protocol = htons(0x0800);

#else

	macHdr = (pkt_ethHdr_t *)pkt;
	pDesc->pMac     = pkt;
	pDesc->pIpHdr   = pkt + 14;
	pDesc->protocol = macHdr->type;

#endif

	if (ntohs(pDesc->protocol) == 0x0806)
		return;

    /*-------------------------------
	 * Extract IP Dst & Src Addresses.
     *-------------------------------
     */
	ipHdr = (struct ip *)pDesc->pIpHdr;
	pDesc->laddr  = (ipHdr->ip_dst.s_addr);
	pDesc->raddr  = (ipHdr->ip_src.s_addr);

	pDesc->pTplHdr = (char *)ipHdr + (ipHdr->ip_hl << 2);

	
    /*-------------------------------
	 * Pkt is an IP fragment if More 
	 * frag bit or offset is set.
     *-------------------------------
     */
    ipOffset = ntohs(ipHdr->ip_off); 
    if (ipOffset & (IP_MF | IP_OFFMASK | IP_RF))
		pDesc->flags |= PKT_IP_FRAGMENT;

	if (ipHdr->ip_p == TPL_PROT_UDP)
	{
		struct	udphdr	*udpHdr;

	    /*-------------------------------
		 * Not 1st IP fragment, app. hdr
		 * is after IP header.
	     *-------------------------------
	     */
		if ((pDesc->flags & PKT_IP_FRAGMENT) && 
			(ipOffset & IP_OFFMASK))
		{
			pDesc->pHdr = pDesc->pTplHdr;
            pDesc->lport = 0;
            pDesc->rport = 0;

		}
		else
		{
		    /*-------------------------------
			 * Extract UDP Dst & Src ports.
			 * Setup App. hdr ptr.
		     *-------------------------------
		     */
			udpHdr = (struct udphdr *)pDesc->pTplHdr;
			pDesc->lport = (udpHdr->uh_dport);
			pDesc->rport = (udpHdr->uh_sport);
			pDesc->pHdr  = pDesc->pTplHdr + sizeof(struct udphdr);
		}
		pDesc->tplProt= TPL_PROT_UDP;
	}
	else if (ipHdr->ip_p == TPL_PROT_TCP)
	{
		struct	tcphdr	*tcpHdr;

	    /*-------------------------------
		 * Extract TCP Dst & Src ports.
		 * Setup App. hdr ptr.
	     *-------------------------------
	     */
		tcpHdr = (struct tcphdr *)pDesc->pTplHdr;
		pDesc->lport = (tcpHdr->th_dport);
		pDesc->rport = (tcpHdr->th_sport);
		pDesc->pHdr  = pDesc->pTplHdr + (tcpHdr->th_off << 2);
		if (tcpHdr->th_flags & (TH_SYN |TH_RST)) 
			tcpOffset = (tcpHdr->th_off << 2) - PKT_TCP_HDR_SIZE;
		pDesc->tplProt= TPL_PROT_TCP;
	}
	else
	{
		pDesc->tplProt= ipHdr->ip_p;
	}
	
    /*-------------------------------
     * Setup hdrSize for App. layer. 
     *-------------------------------
	*/
		
	pDesc->hdrSize = ntohs(ipHdr->ip_len) - (pDesc->pHdr - pDesc->pIpHdr);

	/* Su, 09/12/02
	 * Possible that tcp hdr len > 20 and it is fast path
	 */
	if (ipHdr->ip_p == TPL_PROT_TCP)
		pDesc->hdrSize += tcpOffset;
}

/*-----------------------------------------------------------------
 * Name:        luc_mapPortToLink
 *
 * Description: Map EEE RX Port to LUC link ID.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 02/11/2002
 *-----------------------------------------------------------------
 */
int32 	
luc_mapPortToLink( int32 portId )
{

	switch (portId)
	{
#ifdef  PROM
	    /*-------------------------------
		 * Mgmt port is on BP.
	     *-------------------------------
	     */
		case	PROM_PORT_MGMT:
				return LUC_BP_LINK_ID;

#ifndef SIBYTE
		case	PROM_PORT_NET0:
				return 0;

		case	PROM_PORT_NET1:
				return 1;

#else
		case	PROM_PORT_TXRX0:
				return 0;

		case	PROM_PORT_TXRX1:
				return 1;
#endif
#else

		case	NFP_PORT_TXRX_ETH0:
				return 0;

		case	NFP_PORT_TXRX_ETH1:
				return 1;

		case	NFP_PORT_TXRX_ETH2:
				return 2;
#endif
		default:
				return -1;

	}
}

void
luc_rcvPkt( eee_desc_t   *eDescp)
{
	luc_stats_t		*pStats;
	pkt_desc_t		*pDesc;
	void			*pConn;
	byte			*pBuff;
	int32			camIdx;
	int32			link;
	uint32			bufCnt;
	uint32			pktLen;
	luc_rxPktHdr_t	*pLucPktHdr;
	uchar8			classifier;
	int32			err;
	eee_descBuf_t   *bd;
	void   			*pTime;
	int				lucHdrFlag = 0;
	ushort16		lport;
#if defined(PROM)
	byte			*pDstBuff;
#endif
	int32			actLink;
	
#ifdef PKT_USE_TIME_STAMP
    pTime = pkt_allocTimeStamp( );
    pkt_setTimeStamp(pTime, PKT_TIME_IDX_LUC_RX);
#endif

     /*-------------------------------
	  * Get the RX link ID.
      *-------------------------------
      */
	link = luc_mapPortToLink( EEE_GET_PORT_ID(eDescp->hdr.src_port) ); 

	if ((link < 0) || (link >= lucMaxLinkNum))
		goto dropRcvPkt;

	pStats   = &pLucCfg->linkCfg[link].rxStats;


	/* su, 04/14/02, process luc header is cheetah */
	if (isCheetah()) {
		/* add lucRxPkt header, to be removed later */

#if defined(PROM)
#if !defined(SIBYTE)
		goto process_pkt;
#else
		if (link == LUC_BP_LINK_ID)
			goto process_pkt;
#endif /* SIBYTE */
#endif /* PROM */

	
		lucHdrFlag = 1;
		
		
		bd = ((eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
			  &(eDescp->bd[0]) : &(eDescp->bd[1]));
		
		pLucPktHdr  = (luc_rxPktHdr_t*)bd->buf;

		/* convert lucrx_data to host order, 
		 * otherwise, bits fields will not read
		 * correctly
		 */
		pLucPktHdr->lucrx_data = ntohs(pLucPktHdr->lucrx_data);

		if (pLucCfg->rxVerify) {
			if (luc_Verify(eDescp) != NFX_OK) {
				pStats->verifyErr++;
				goto dropRcvPkt;
			}
		}
		
		/* get link/err/classifier */
		classifier= pLucPktHdr->lucrx_classifier;

		/* collect pkt stats classification */
		lucClassStats[classifier].pktCnt++;
		lucClassStats[classifier].byteCnt += bd->len - sizeof(luc_rxPktHdr_t);
		
		err = pLucPktHdr->lucrx_err;
		link = pLucPktHdr->lucrx_linkId;
		
		if ((link < 0) || (link >= lucMaxLinkNum)) {
			/* save link err stats */
			pStats   = &pLucCfg->linkCfg[0].rxStats;
			pStats->linkErr++;
			goto dropRcvPkt;
		}
		else
			pStats = &pLucCfg->linkCfg[link].rxStats;
		
		/* rcv pkt has error */
		if (err) {
			if (err & LUC_RXPKT_ERR_IPCSUM)
				pStats->hwBadIpCsum++;

			if (err & LUC_RXPKT_ERR_TPLCSUM)
				pStats->hwBadTplCsum++;

			if (err & LUC_RXPKT_ERR_IPHLEN)
				pStats->hwBadIpHlen++;

			if (err & LUC_RXPKT_ERR_TPLHLEN)
				pStats->hwBadTcpHlen++;

			goto dropRcvPkt;
		}

		
		/* only process arp and ip */
		if ((classifier == LUC_RXPKT_CLASS_MAC) ||
			(classifier == LUC_RXPKT_CLASS_NOPROT)) {
			pStats->hwBadClass++;
			goto  dropRcvPkt;
		}
		
		/* advang bd  */
		bd->buf += sizeof(luc_rxPktHdr_t);
		bd->len -= sizeof(luc_rxPktHdr_t);

		/* adjust pktsize */
		pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);	
		pktLen -= sizeof(luc_rxPktHdr_t);
		eDescp->hdr.control &= ~EDESC_TOTAL_LEN;		
		eDescp->hdr.control |= (pktLen & EDESC_TOTAL_LEN);
	}
		

#if defined(PROM)

process_pkt:	

#endif


    /*-------------------------------
	 * Extract pkt length & buf ptr.
     *-------------------------------
     */
    pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);
    bufCnt = (eDescp->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	pBuff  = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
         		(eDescp->bd[0].buf) : (eDescp->bd[1].buf);



#ifndef LINUX_TEST
	if (*pBuff & 0x01)
			pStats->mcastCnt++;
#endif

    /*-------------------------------
     * Allocate pkt desc. If fail
	 * free EEE desc & pkt.
     *-------------------------------
	 */
	if (!(pDesc = PKT_ALLOC_DESC()))
	{
		pStats->noPktDesc++;
		goto dropRcvPkt;
	}

#ifdef PROM
#if defined(R9000) && defined(EVAL_BOARD)
    /*-------------------------------
	 * GT64440 receives into short  
	 * boundary. 
     *-------------------------------
     */
    pBuff += 2;
#else
    /*-------------------------------
	 * GT64240 can only receive on 
	 * word boundary. With Ethernet
	 * header, IP address is at 16 bit.
	 *
	 * However BSD IP expects ip address 
	 * at 32 bit (u_int32_t s_addr).
	 * Copy packet into short offset 
	 * to work around.
     *-------------------------------
     */
    pDstBuff = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
    if (!pDstBuff)
    {
		pStats->noBuffErr++;
		goto dropRcvPkt;
    }

    pDstBuff += 2;
    memcpy(pDstBuff, pBuff, pktLen);

    eee_deallocateBuffer(pBuff);
    eDescp->bd[1].buf = pDstBuff;
	pBuff = pDstBuff; 
#endif
#endif

	pDesc->pktSize = pktLen;
	pDesc->hdrSize = pktLen;
	pDesc->dataSize= 0;
	pDesc->cod	   = 0;
	pDesc->eDescp  = eDescp;
	pDesc->pNext   = NULL;
	pDesc->camIdx  = -1;
	pDesc->pTime   = pTime;

	pDesc->link = link;
	
    /*-------------------------------
	 * Count total pkts & bytes rcvd.
     *-------------------------------
     */
	pStats->pktCnt++;
	pStats->byteCnt += pktLen;

    /*-------------------------------
	 * Extract various header info.
     *-------------------------------
     */
	if (lucHdrFlag) {
		luc_extractHdrInfoHw(pDesc, pBuff, pLucPktHdr);

		//	printf("%s: class %d, pktsize %d, hdrsize %d mac %x, ip %x, tpl %x, phdr %x\n",
		//	   __FUNCTION__, classifier, pDesc->pktSize, pDesc->hdrSize,
		//	   pDesc->pMac, pDesc->pIpHdr, pDesc->pTplHdr, pDesc->pHdr); 
#if  0
		if (!(*(pDesc->pMac) & 0x01))	{/* don't print broadcast */
			printf("%s: link %d, pLucHdr %x\n", __FUNCTION__, link, pLucPktHdr);
			printf("class %d, pktsize %d, hdrsize %d mac %x, ip %x, tpl %x, phdr %x\n",
				   classifier, pDesc->pktSize, pDesc->hdrSize,
				   pDesc->pMac, pDesc->pIpHdr, pDesc->pTplHdr, pDesc->pHdr);
			printf("proto %x, tplProto= %x, lport= %x, rport=%x, laddr=%x, raddr=%x\n",
				   pDesc->protocol, pDesc->tplProt,pDesc->lport,pDesc->rport,
				   pDesc->laddr, pDesc->raddr);
			bsd_ShowPkt(pLucPktHdr,(96));
		}
#endif /* 0 */
	}
	else
		luc_extractHdrInfo(pDesc, pBuff, pktLen);

	luc_trace("luc_simRcv", LUC_TRACE_PKT, pDesc, pDesc->pMac, pDesc->pktSize);

	lport = ntohs(pDesc->lport);
	if ((pDesc->tplProt == TPL_PROT_UDP) && BSD_BCAST_TO_SSC(lport))
	{
		/*
		 *Defect: 1698: Forward all UDP packets on port 111
		 *to slowpath so it will be relaying to SSC.  This
		 *will include all broadcasts on port 111
		 */
		pStats->slowPktCnt++;
		luc_ifSlowPath(link, pDesc);
		return;
	}
	
	switch (ntohs(pDesc->protocol))
	{
		case	0x0806:
				pStats->slowPktCnt++;
				luc_ifSlowPath(link, pDesc);
				return;

		case	0x0800:
			/* chk for slow path pkt only */
                /*-------------------------------
                 * Find matching local IP interface 
                 *-------------------------------
                 */

				/* Su, 10/26/02
				 * if link belongs to link aggregation vif,
				 * we need to pass active link to 
				 * luc_ipIfFindEntry
				 */
				actLink = luc_getActLALink(link);
				if (luc_ipIfFindEntry(actLink, pDesc->laddr) == NULL)
				{
					pStats->discards++;
			        PKT_FREE_PACKET(pDesc);
					return;
				}


#ifdef LINUX_TEST
                {
                    /*-------------------------------
                     * Only handle NFS/Mount over UDP for now.
                     *-------------------------------
                     */
                    int     i;
                    boolean portInLucUserTab = FALSE;
                    
                    for (i = 0; i < MAX_LUC_USER; i++)
                    {
                        if ((lucUserTab[i].inUse == TRUE) &&
                            (pDesc->lport == lucUserTab[i].port))
                        {
                            portInLucUserTab = TRUE;
                            break;
                        }                        
                    }// for
                    
                    if ((pDesc->lport != htons(nfsPort))   &&
                        (pDesc->lport != htons(nfsPort-1)) &&
                        (pDesc->lport != htons(nfsPort-2)) &&
                        (pDesc->lport != htons(nfsPort-3)) &&
                        (pDesc->lport != htons(nfsPort-4)) &&
                        (portInLucUserTab == FALSE)     
                        )
                    {
                        pStats->discards++;
                        PKT_FREE_PACKET(pDesc);
                        return;
                    }
                }
#endif

#if !defined(PROM)  
			    /*-------------------------------
			     * If IP fragment, go thru reassembly.
			     *-------------------------------
			     */
				if (pDesc->flags & PKT_IP_FRAGMENT) 
				{
					pDesc = luc_Reassem(pDesc, link, &pConn);
					

				    /*-------------------------------
					 * fast path reassembly completed
				     *-------------------------------
				     */
					if (pDesc)	
						goto rcvRsmPkt;

				    /*-------------------------------
					 * reassembly not finished yet 
				     *-------------------------------
				     */
					return;
				}
#else
			    /*-------------------------------
			     * If IP fragment, go thru slowpath.
			     *-------------------------------
			     */
				if (pDesc->flags & PKT_IP_FRAGMENT)
				{
					pStats->slowPktCnt++;
					luc_ifSlowPath(link, pDesc);
					return;
				}
#endif /* PROM */

			    /*-------------------------------
			     * Emulate a CAM lookup.
			     *-------------------------------
			     */
#if 0
				if (lucHdrFlag)
					camIdx = pDesc->camIdx;
				else
					camIdx = luc_simCamFind(pDesc->tplProt, 
											pDesc->laddr, 
											pDesc->raddr, 
											pDesc->lport, 
											pDesc->rport);
#endif /* 0 */
				camIdx = luc_simCamFind(pDesc->tplProt,
										pDesc->laddr,
										pDesc->raddr,
										pDesc->lport,
										pDesc->rport);
 
			    /*-------------------------------
			     * If lookup failed, go thru slowpath.
			     *-------------------------------
			     */
				if ( camIdx < 0)
				{

					pStats->slowPktCnt++;
					luc_ifSlowPath(link, pDesc);
					return;
				}

			    /*-------------------------------
			     * Get TPL Conn based on Cam Idx.
			     *-------------------------------
			     */
				pConn = luc_camGetUsrInfo( camIdx );

			    /*-------------------------------
				 * incr. ip cnt for non-fragmented pkts 
			     *-------------------------------
			     */
				ipstat.ips_total++; 

rcvRsmPkt:
			    /*-------------------------------
			     * Send pkt upto TPL.
			     *-------------------------------
			     */
				pDesc->flags |= PKT_FAST_PATH;

			    /*-------------------------------
				 * su, 05/06/02, update udp stats 
			     *-------------------------------
			     */
				if (((tpl_connCb_t*)pConn)->prot == TPL_PROT_UDP)
					udpstat.udps_ipackets++;
				
				if (tpl_rcvPkt( pConn, pDesc ) == TPL_FREE_PKT)
				{
					PKT_FREE_PACKET( pDesc );
				}
				pStats->fastPktCnt++;
				return;

		default:
				pStats->discards++;
				PKT_FREE_PACKET( pDesc );
				return;
	}

dropRcvPkt:
	eee_freePacket(eDescp);
#ifdef PKT_USE_TIME_STAMP
    pkt_freeTimeStamp( pTime );
#endif
	return;
}

#ifndef PROM

#ifdef NOTUSED
/*-----------------------------------------------------------------
 * Name:        luc_rcvGmcData
 *
 * Description: Receive Data from GMC and send it to network.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 05/10/2001
 *-----------------------------------------------------------------
 */
void
luc_rcvGmcData( eee_desc_t   *eDescp)
{
    int32           pktLen;
    uint32          ctrl;
	luc_stats_t		*pStats;
    luc_fragCb_t	*fcb;
    eee_descBuf_t   *bd;
    pkt_desc_t      *pDesc;
    pkt_desc_t      tmpDesc;
	uint32			dest_port;

    /*-------------------------------
	 * Exclude desc from pkt lengtn.
     *-------------------------------
     */
    ctrl    = eDescp->hdr.control;
    pktLen  = (ctrl & EDESC_TOTAL_LEN);
    pktLen -= (sizeof (pkt_desc_t **));	

	/* su, 06/05/02, pkt size excludes 
	 * luc_txHdr
	 */
	if (isCheetah()) 
		pktLen -= sizeof(luc_txPktHdr_t);
		
    ctrl    = (ctrl & ~EDESC_TOTAL_LEN) | pktLen;

    eDescp->hdr.control = ctrl;


    bd = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
	    &(eDescp->bd[0]) : &(eDescp->bd[1]);

    /*-------------------------------
     * Extract Pkt Desc from 1st word.
     *-------------------------------
     */
    pDesc  = *((pkt_desc_t **)(bd->buf));
	pStats = &pLucCfg->linkCfg[pDesc->link].txStats;

    bd->buf += (sizeof (pkt_desc_t **));
    bd->len -= (sizeof (pkt_desc_t **));

	/* su, 06/05/02, advance to mac frame */
	if (isCheetah()) {
		bd->buf += sizeof(luc_txPktHdr_t);
		bd->len -= sizeof(luc_txPktHdr_t);
	}
	
    /*-------------------------------
     * Save pDesc in EEE Desc attr field.
	 * Original EDesc has dest_port.
     *-------------------------------
     */
	eDescp->hdr.attr = (address_t) pDesc;

    /*-------------------------------
     * Save EEE desc. from GMC. Will be 
	 * released in luc tx complete.
     *-------------------------------
	 */
	pDesc->gmcDescp  = eDescp;

	pStats->byteCnt += pktLen;
	pStats->gmcPktCnt++;


	/* su, 05/06/02.
	 * incr. ip stats, tcp stats already
	 * done
	 */
	ipstat.ips_localout++;	


	/*-------------------------------
	 * Set up temp desc for IP/TCP/UDP
	 * header len & checksum.
	 *-------------------------------
	 */
	tmpDesc.eDescp  = eDescp;
	tmpDesc.flags   = pDesc->flags; /* su, 06/11/02, needed for chksum */
	tmpDesc.pMac    = bd->buf; 
	tmpDesc.pIpHdr  = bd->buf + (pDesc->pIpHdr  - pDesc->pMac);
	tmpDesc.pTplHdr = bd->buf + (pDesc->pTplHdr - pDesc->pMac);
	tmpDesc.pHdr    = bd->buf + (pDesc->pHdr    - pDesc->pMac);
	tmpDesc.hdrSize = bd->len - (pDesc->pHdr    - pDesc->pMac);
	
	/*-------------------------------
	 * Set IP len & checksum.
	 *-------------------------------
	 */
	tpl_setIpHdr(pDesc->pTplConn, &tmpDesc);
	
	/*-------------------------------
	 * Set TCP/UDP len & checksum.
	 *-------------------------------
	 */
	if (pDesc->tplProt == TPL_PROT_TCP)
		tpl_setTcpHdr(pDesc->pTplConn, &tmpDesc);
	else
		tpl_setUdpHdr(pDesc->pTplConn, &tmpDesc);
	
	
	if (isCheetah()) {
		/* add luc headers */
		if (luc_AddTxHdr(eDescp, pDesc) != NFX_OK) {
			pStats->verifyErr++;
			luc_simQueueXmtCmpl(eDescp, 0);
			return NFX_ERR;
		}
	}

    /*-------------------------------
     * Mark pkt as being sent.
     *-------------------------------
     */
    pDesc->flags |= PKT_USR_LUC_TX;

	/* Su, April 12/02. Set dest port as it has not been set */
	dest_port = pLucCfg->drvPort[pDesc->link];

#ifdef LINUX_TEST
	luc_simDllXmt(dest_port, eDescp);
#else
	if (isCheetah()) {				/* load balancing for cheeta */
        if (bmc12500Eth_txLink_edesc(dest_port, eDescp )
            != EEE_FORWARDED_PACKET) {
            pStats->discards++;
            luc_simQueueXmtCmpl( eDescp , 0);
        }
	}
	else
		bmc12500Eth_txUnicast(dest_port, eDescp );
#endif

}
#endif

char *luc_getEeeErrStr(int32 errCode)
{
	char *errStr;

	switch (errCode)
	{
		case 	EEE_ERR_NO_BUF:
				errStr = "EEE_ERR_NO_BUF";
				break;

		case 	EEE_ERR_NO_DESC:
				errStr = "EEE_ERR_NO_DESC";
				break;

		case 	EEE_ERR_PACKET_TOO_LARGE:
				errStr = "EEE_ERR_PACKET_TOO_LARGE";
				break;


		case 	EEE_FORWARDED_PACKET:
				errStr = "EEE_FORWARDED_PACKET";
				break;

		case EEE_DROP_PACKET :
				errStr = "EEE_DROP_PACKET";
				break;

		case EEE_TX_QUEUE_FULL :
				errStr = "EEE_TX_QUEUE_FULL";
				break;

		default:
				errStr = "Unkonw Err";
				break;
	}
	return (errStr);
}

#ifdef NOTUSED
/*-----------------------------------------------------------------
 * Name:        luc_sndCod
 *
 * Description: Sends Data received from Network to GMC with COD.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 02/05/2002 
 *-----------------------------------------------------------------
 */
static void
luc_sndCod( gmc_app_hdr_t 	*gmc_app_hdr,
			nfx_cod_entry_t *pCod,
			int32 			xferSize)
{
		luc_stats_t		*pStats;
		int32			rc;
		
        /*-------------------------------
         * Setup COD entry.
         *-------------------------------
         */
        pCod->cod    = COD_GET_NEXT( );
        pCod->offset = 0;
        pCod->length = xferSize;

        /*-------------------------------
         * Setup GMC header.
         *-------------------------------
         */
        gmc_app_hdr->control = pLucCfg->efsAppId;
        gmc_app_hdr->id.cod  = pCod->cod;
		gmc_app_hdr->cod_len = xferSize;

#ifdef USE_SAC_CHECKSUM
        /*-------------------------------
         * Checksum usr data before sending
		 * to GMC. Use 0 for CRC seed.
         *-------------------------------
         */
		gmc_app_hdr->chksum  = crc32_checksum( (void *)(gmc_app_hdr+1), 
														xferSize, 
														0 );
#endif

        /*-------------------------------
         * Send msg to SAC.
         *-------------------------------
         */
        if (rc = eee_sendMessage( gmc_app_hdr,
                            xferSize+sizeof(gmc_app_hdr_t),
                            pLucCfg->sacPort,
                            pLucCfg->efsPort ) != NFX_OK)
        {
			printf("%s: eee_sendMessage failed! rc=%s [%d] appId=%d sacId=%d\n",
					__FUNCTION__, luc_getEeeErrStr(rc), rc, 
					EEE_GET_APP_ID(pLucCfg->sacPort),
					eee_getApplicationId("sac"));
			
			luc_dbgDescChain( PktDescPtr);
			luc_showCodEntry(1, pCod);
            luc_panic();
        }

		pStats = &pLucCfg->linkCfg[0].rxStats;
	    pStats->gmcPktCnt++;

}
#endif

eee_desc_t *luc_getUltraBuff(uchar8 **pBuf )
{
    eee_desc_t  *eDescp;
    char        *eBuffp;
	
	eBuffp  = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, 0);
    if (eBuffp == NULL)
    {
        pPktCfg->stats.cntNoBuffErr++;
        return NULL;
    }
#ifdef EEE_ATTR
    eee_attrAssignBuf(eBuffp, EEE_ATTR_LUC_ALLOC);
#endif

	eDescp = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
    if (eDescp == NULL)
    {
        pPktCfg->stats.cntNoEeeDescErr++;
        eee_deallocateBuffer( eBuffp );
        return NULL; 
    }
#ifdef EEE_ATTR
    eee_attrAssignDesc(eDescp, EEE_ATTR_LUC_ALLOC);
#endif
    /*-------------------------------
     * Setup EEE descriptor & buffer.
     *-------------------------------
     */
    eDescp->bd[0].control = 0;
    eDescp->bd[1].buf     = eBuffp;
    eDescp->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_ULTRA;
    eDescp->hdr.dest_port = pLucCfg->sacPort;
    eDescp->hdr.src_port  = pLucCfg->efsPort;
    eDescp->hdr.control   = (( 1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

	*pBuf = eBuffp;

//	printf("%s: ultra eDescp=%x bufp=%x\n", __FUNCTION__, eDescp, eBuffp);
	return (eDescp);
	
}

#ifndef SBM_REV2
static void
luc_sndCodUltra( eee_desc_t	*eDescp,
			gmc_app_hdr_t 	*gmc_app_hdr,
			nfx_cod_entry_t *pCod,
			int32 			xferSize)
{
		int32			pktSize;
		int32			rc;

/*
		printf("%s: eDescp=%x xferSize=%d\n",
			__FUNCTION__, eDescp, xferSize);
		luc_showCodEntry(1, pCod);
*/
		pktSize = xferSize+sizeof(gmc_app_hdr_t);
	    eDescp->bd[1].len = pktSize;
	    eDescp->hdr.control = (pktSize & EDESC_TOTAL_LEN) |
                            (( 1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);


        /*-------------------------------
         * Setup COD entry.
         *-------------------------------
         */
        pCod->cod    = COD_GET_NEXT( );
        pCod->offset = 0;
        pCod->length = xferSize;

        /*-------------------------------
         * Setup GMC header.
         *-------------------------------
         */
        gmc_app_hdr->control = pLucCfg->efsAppId;
        gmc_app_hdr->id.cod  = pCod->cod;

#ifdef USE_SAC_CHECKSUM
        /*-------------------------------
         * Checksum usr data before sending
		 * to GMC. Use 0 for CRC seed.
         *-------------------------------
         */
		gmc_app_hdr->chksum  = crc32_checksum( (void *)(gmc_app_hdr+1), 
														xferSize, 
														0 );
#endif
        /*-------------------------------
         * Send msg to SAC.
         *-------------------------------
         */
		sbm_sac_rcv( eDescp );

}

/*-----------------------------------------------------------------
 * Name:        luc_sndGmcData
 *
 * Description: Sends Data received from Network to GMC with COD.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 05/10/2001
 *               01/30/2002 Added support for multi-cod.
 *               02/06/2002 Consolidated multi-pkt into an 8K COD.
 *-----------------------------------------------------------------
 */
int32
luc_sndGmcData( pkt_desc_t		*pktDesc,
                uchar8      	*pData,
                int32       	dataSize,
                nfx_cod_entry_t *codEntry)
{
	nfx_cod_entry_t	*pCod;
    gmc_app_hdr_t   *gmc_app_hdr;
	eee_descBuf_t	*bd;
	eee_desc_t  	*eDescp;
	eee_desc_t  	*sacDescp;
	pkt_desc_t  	*pDesc;
	luc_stats_t		*pStats;
    uchar8          *sacMsg;
    uchar8          *dstBuf;
    uchar8          *srcBuf;
    int32           copyLen;
    int32           bufCnt;
	int32			numOfCod;
	int32			codLength;
	int32			pktLen;
	int32			remain;
	int32			xferSize;


	luc_trace( "luc_sndGmcData", LUC_TRACE_PKT, 0, pData, dataSize);
	
	xferSize = 0;
	numOfCod = 0;
	pDesc    = pktDesc;
	pCod     = codEntry;
	remain   = dataSize;

	PktDescPtr    = pktDesc;
	DataSize      = dataSize;

    /*-------------------------------
     * Don't generate 0 length COD.
     *-------------------------------
     */
	if (dataSize == 0)
		return 0;

	if (dataSize < 0)
	{
		printf("%s: dataSize=%d! \n", __FUNCTION__, dataSize);
		luc_dbgDescChain( pktDesc);
		luc_panic();
	}

    /*-------------------------------
     * pHdr points to user data.
     *-------------------------------
     */
	pDesc->pHdr = pData;

    /*-------------------------------
     * Allocate an 8 K EEE message.
     *-------------------------------
     */
    sacDescp = luc_getUltraBuff( &sacMsg );

    if (sacDescp == NULL)
    {
        printf("%s: luc_getUltraBuff failed! \n", __FUNCTION__);
        luc_dbgDescChain( pktDesc);
        luc_panic();
    }

    /*-------------------------------
     * Setup dst buf info for copy.
     *-------------------------------
     */
   	dstBuf = sacMsg + sizeof(gmc_app_hdr_t);

	pStats = &pLucCfg->linkCfg[pDesc->link].rxStats;

    /*-------------------------------
     * Go thru IP reassembled pkts 
	 * which are chained using pNext.
	 *
	 * Assign 8KB/COD max.
     *-------------------------------
     */
	while(pDesc)
	{

		eDescp = pDesc->eDescp;
		pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);
		bufCnt = (eDescp->hdr.control & EDESC_BUFCNT_MASK) >>EDESC_BUFCNT_SHIFT;
		bd     = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
				 &(eDescp->bd[0]) : &(eDescp->bd[1]);

	    /*-------------------------------
	     * Exclude size of headers:
		 * MAC+IP+TCP+APP hdr for 1st buf.
		 * MAC+IP hdr for fragmenst.
	     *-------------------------------
	     */
		codLength = pktLen  - (pDesc->pHdr  - pDesc->pMac);
	    copyLen   = bd->len - (pDesc->pHdr  - pDesc->pMac);

	    /*-------------------------------
		 * Shaun 03/22/2002
		 * Fix for #214, #291
	     * cod/copy length can be negative
		 * due to "pullup" in rpc reassembly
		 * in tpl_rpcQueueRasm.  Skip desc.
	     *-------------------------------
	     */
		if ((codLength <=0) || (copyLen <= 0))
		{
			pkt_stats_t		*pStats;

			pStats = &pPktCfg->stats;

			if (copyLen <=0)
				pStats->cntBadCopyLen++;

			if (codLength <=0)
				pStats->cntBadCodLen++;
			goto skipPktDesc;
		}

#ifdef LUC_DBG_COD
		printf("%s pMac=0x%x pHdr=0x%x pktLen=%d ", 
 				__FUNCTION__, pDesc->pMac, pDesc->pHdr, pktLen);
		printf("codLen=%d bdLen=%d copyLen=%d\n", codLength, bd->len, copyLen);
#endif

	
	    /*-------------------------------
	     * Send cod if no room for this pkt.
	     *-------------------------------
	     */
		if ((codLength + xferSize) > LUC_8K_COD)
		{
//			luc_sndCod((gmc_app_hdr_t *)sacMsg, pCod, xferSize);
			luc_sndCodUltra(sacDescp, (gmc_app_hdr_t *)sacMsg, pCod, xferSize);
	    	pStats->gmcPktCnt++;

            /*-------------------------------
             * Allocate an 8 K EEE message.
             *-------------------------------
             */
            sacDescp = luc_getUltraBuff( &sacMsg );

            if (sacDescp == NULL)
            {
                printf("%s: luc_getUltraBuff failed! \n", __FUNCTION__);
                luc_dbgDescChain( pktDesc);
                luc_panic();
            }

		    /*-------------------------------
		     * Reset dst buf & xferSize
		     *-------------------------------
		     */
	    	dstBuf   = sacMsg + sizeof(gmc_app_hdr_t);
			xferSize = 0;

			pCod++;
			numOfCod++;
		}

	    /*-------------------------------
	     * Setup src buf info for copy.
	     *-------------------------------
	     */
	    srcBuf = pDesc->pHdr;		

	    /*-------------------------------
	     * Copy all buffers in EEE desc.
	     *-------------------------------
	     */
	    while (bufCnt--)
	    {

			if ((bd->len < 0) || (copyLen < 0))
			{
				printf("%s: bd->len=%d copylen=%d pHdr-pMac=%d! \n",
					__FUNCTION__, bd->len, copyLen, pDesc->pHdr-pDesc->pMac);
				luc_dbgDescChain( pktDesc);
				luc_panic();
			}


	        memcpy(dstBuf, srcBuf, copyLen);

		    /*-------------------------------
		     * Setup next EEE buffer for copy.
		     *-------------------------------
		     */
			bd++;
			dstBuf += copyLen;
			srcBuf  = bd->buf;
			copyLen = bd->len;
    	}

	    /*-------------------------------
	     * Datasize may be smaller than pkt 
		 * len due to padding. Copy data only.
	     *-------------------------------
	     */
		xferSize += (remain <= codLength) ? remain : codLength;
		remain   -= codLength;

	    /*-------------------------------
	     * No more data to copy,
	     * Send cod to GMC.
	     *-------------------------------
	     */
		if (remain <= 0)
		{
			luc_sndCodUltra( sacDescp, (gmc_app_hdr_t *)sacMsg, pCod, xferSize);
	    	pStats->gmcPktCnt++;

#ifdef NOTUSED
		    /*-------------------------------
		     * Reset dst buf & xferSize
		     *-------------------------------
		     */
	    	dstBuf   = sacMsg + sizeof(gmc_app_hdr_t);
			xferSize = 0;
#endif

			pCod++;
			numOfCod++;

		    /*-------------------------------
			 * Shaun 3/15/02: 
			 * Fix for bug #222
		     * No more data to copy, 
			 * break out of while( pDesc).
		     *-------------------------------
		     */
			break;
		}

skipPktDesc:
	    /*-------------------------------
	     * Advance to next pkt & cod entry.
	     *-------------------------------
	     */
		pDesc = pDesc->pNext;

	}

    /*-------------------------------
     * This should NOT happen unless
	 * pkt/bd len or dataSize is wrong.
     *-------------------------------
     */
	if (remain > 0)
	{
		pkt_stats_t	*pStats;

		printf("Bad pkt len\n");

		pStats = &pLucCfg->linkCfg[pDesc->link].rxStats;
		pStats->cntBadCodLen++;
		luc_panic();
	}


#ifdef LUC_DBG_COD
	luc_showCodEntry(numOfCod, codEntry);
#endif

	return (numOfCod);
}
#else

#ifdef LUC_DBG_COD
static void
luc_dbgCod( eee_desc_t *eDescp,
			nfx_cod_entry_t *pCod,
			int32 			xferSize)
{
	int32	pktLen;
	int32	bdLen;
	int32	bufCnt;
    eee_descBuf_t   *bd;


	printf("-------------------------------------\n");
	printf("%s: cod=0x%llx offset=%d len=%d\n", __FUNCTION__, 
			pCod->cod, pCod->offset, pCod->length);

	pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);
	bdLen = 0;

	printf("total pktlen=%d\n", pktLen);

	while(eDescp)
	{
        bufCnt = (eDescp->hdr.control & EDESC_BUFCNT_MASK) >>EDESC_BUFCNT_SHIFT;        
		bd     = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
                 &(eDescp->bd[0]) : &(eDescp->bd[1]);

		printf("eDescp=%p bufCnt=%d buf=%p bdLen=%d\n",
				eDescp, bufCnt, bd->buf, bd->len);

		bdLen += bd->len;

		eDescp = eDescp->hdr.next;
	}
	
	printf("total bdLen=%d\n", bdLen);
	printf("-------------------------------------\n");
}
#endif

static void
luc_sndCodNew( eee_desc_t	*eDescp,
			nfx_cod_entry_t *pCod,
			int32 			xferSize)
{

	if (xferSize <= 0)
	{
		pkt_stats_t	*pStats;

		pStats = &pPktCfg->stats;
		pStats->cntBadCodLen++;

		panic("%s: xferSize < 0\n", __FUNCTION__);
	}
                

	EEE_SET_PKT_LEN( eDescp, xferSize );

    /*-------------------------------
     * Setup COD entry.
     *-------------------------------
     */
    pCod->cod    = sbm_getCODFromDesc(eDescp, nfxMySlotId, nfxMyCPUId);
    pCod->offset = 0;
    pCod->length = xferSize;

#ifdef LUC_DBG_COD
	luc_dbgCod( eDescp, pCod, xferSize);
#endif

    /*-------------------------------
     * Send msg to SAC.
     *-------------------------------
     */
	sbm_sendCODToSac( pCod->cod );

}

eee_desc_t *
luc_copyToNewEbuf( uchar8 *srcBuf, int32 copyLen )
{
    eee_desc_t          *eDescp;
	char				*bufp;

	eDescp = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if (!eDescp)
	{
        pPktCfg->stats.cntNoEeeDescErr++;
		return (NULL);
	}

    bufp   = eee_allocateBuffer(EEE_SHARED_POOL,
								EEE_BUF_LARGE,
								NFX_COS_HIGH);
	if (!bufp)
	{
        pPktCfg->stats.cntNoBuffErr++;
		eee_deallocateDesc( eDescp );
		return (NULL);
	}

#ifdef EEE_ATTR
    eee_attrAssignDesc(eDescp, EEE_ATTR_LUC_ALLOC);
    eee_attrAssignBuf(bufp, EEE_ATTR_LUC_ALLOC);
#endif
    
    eDescp->bd[0].control = 0;
    eDescp->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
    eDescp->bd[1].len     = copyLen;
    eDescp->bd[1].buf     = bufp;

    eDescp->hdr.next      = NULL;
    eDescp->hdr.control   = 0;

    EEE_SET_NUM_BUFS( eDescp, 1 );

	memcpy(bufp, srcBuf, copyLen);

#ifdef LUC_DBG_COD
    printf("-------------------------\n");
    printf("%s:\n", __FUNCTION__);
	printf("eDescp=%p bufCnt=%d buf=%p bdLen=%d\n",
			eDescp, 1, bufp, copyLen);
    printf("-------------------------\n");
#endif

	return(eDescp);
}
			

int32
luc_sndGmcData( pkt_desc_t		*pktDesc,
                uchar8      	*pData,
                int32       	dataSize,
                int32       	dataOffset,
                int32       	padSize,
                nfx_cod_entry_t *codEntry)
{
	nfx_cod_entry_t	*pCod;
	eee_descBuf_t	*bd;
	eee_desc_t  	*eDescp;
	eee_desc_t  	*firstEDescp;
	eee_desc_t  	*prevEDescp;
	pkt_desc_t  	*pDesc;
	luc_stats_t		*pStats;
    uchar8          *srcBuf;
    int32           copyLen;
	int32			numOfCod;
	int32			codLength;
	int32			codBuffSize;
	int32			pktLen;
	int32			remain;
	int32			xferSize;


	luc_trace( "luc_sndGmcData", LUC_TRACE_PKT, 0, pData, dataSize);
	
	xferSize = 0;
	numOfCod = 0;
	pDesc    = pktDesc;
	pCod     = codEntry;
	remain   = dataSize;

	PktDescPtr    = pktDesc;
	DataSize      = dataSize;

    /*-------------------------------
     * Don't generate 0 length COD.
     *-------------------------------
     */
	if (dataSize == 0)
		return 0;

	if (dataSize < 0)
	{
		luc_dbgDescChain( pktDesc);
		panic("%s: dataSize=%d! \n", __FUNCTION__, dataSize);
	}

    /*-------------------------------
     * pHdr points to user data.
     *-------------------------------
     */
	/* update hdr size */
	pDesc->hdrSize -= ((address_t)pData - (address_t)pDesc->pHdr);

	pDesc->pHdr = pData;


	pStats = &pLucCfg->linkCfg[pDesc->link].rxStats;

    /*-------------------------------
     * Go thru IP reassembled pkts 
	 * which are chained using pNext.
	 *
	 * Assign 8KB/COD max.
     *-------------------------------
     */
	firstEDescp = NULL;
	prevEDescp = NULL;

	dataOffset &= (LUC_8K_COD - 1);
	codBuffSize = (LUC_8K_COD - dataOffset);

	while(pDesc)
	{

		eDescp = pDesc->eDescp;
		//	pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);
		/* get pktLen from pkt to avoid padding */
		pktLen = pDesc->hdrSize;
		
		bd     = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
				 &(eDescp->bd[0]) : &(eDescp->bd[1]);


	    /*-------------------------------
	     * Exclude size of headers:
		 * MAC+IP+TCP+APP hdr for 1st buf.
		 * MAC+IP hdr for fragmenst.
	     *-------------------------------
	     */
		//codLength = pktLen  - (pDesc->pHdr  - pDesc->pMac);
		codLength = pktLen;
		

	    //bd->len  -= (pDesc->pHdr  - pDesc->pMac);
		bd->len = pktLen;
		
		bd->buf   = pDesc->pHdr;

	    /*-------------------------------
		 * Shaun 09/12/02 Fix for #2180
	     * Exclude pad from last pkt
		 * before sending to SAC.
	     *-------------------------------
	     */
        if (pDesc->pNext == NULL)
        {
            codLength -= padSize;
            bd->len   -= padSize;
        }

	    /*-------------------------------
		 * Shaun 03/22/2002
		 * Fix for #214, #291
	     * cod/copy length can be negative
		 * due to "pullup" in rpc reassembly
		 * in tpl_rpcQueueRasm.  Skip desc.
	     *-------------------------------
	     */
		if (codLength <=0) 
		{
			pkt_stats_t		*pStats;

			pStats = &pPktCfg->stats;
			pStats->cntBadCodLen++;

			goto skipPktDesc;
		}

		if (firstEDescp == NULL)
			firstEDescp = eDescp;


#ifdef LUC_DBG_COD
		printf("%s pMac=0x%x pHdr=0x%x pktLen=%d ", 
 				__FUNCTION__, pDesc->pMac, pDesc->pHdr, pktLen);
		printf("codLen=%d bdLen=%d copyLen=%d\n", codLength, bd->len, copyLen);
#endif

	
	    /*-------------------------------
	     * Send cod if no room for this pkt.
	     *-------------------------------
	     */
		if ((codLength + xferSize) > codBuffSize)
		{
            eee_desc_t *newEDescp;

            if (prevEDescp)
			    prevEDescp->hdr.next = eDescp;

            eDescp->hdr.next     = NULL;

			copyLen = (codLength + xferSize) - codBuffSize;
			bd->len = codBuffSize - xferSize;

			srcBuf = bd->buf + bd->len;
			newEDescp = luc_copyToNewEbuf( srcBuf, copyLen );
			if (!newEDescp)
				return (0);

			xferSize += bd->len;
			remain   -= bd->len;
			codLength = copyLen;


			luc_sndCodNew( firstEDescp, pCod, xferSize);

			firstEDescp = newEDescp;
			eDescp      = newEDescp;
            bd          = &eDescp->bd[1];
			prevEDescp  = NULL;

	    	pStats->gmcPktCnt++;

			xferSize = 0;
			codBuffSize = LUC_8K_COD;
            dataOffset  = 0;

			pCod++;
			numOfCod++;
		}

        pDesc->eDescp = NULL;

        if (remain <= 0)
        {
            if (firstEDescp)
                eee_freePacket( firstEDescp );

            break;
        }

	    /*-------------------------------
	     * Datasize may be smaller than pkt 
		 * len due to padding. Copy data only.
	     *-------------------------------
	     */
		if (remain <= codLength) 
        {
            xferSize += remain;
            bd->len   = remain;
        }
        else
        {
		    xferSize += codLength;
        }

		remain   -= codLength;


		if (prevEDescp)
		{
			prevEDescp->hdr.next = eDescp;
			prevEDescp = eDescp;
		}
		else
			prevEDescp = eDescp;

	    /*-------------------------------
	     * No more data to copy,
	     * Send cod to GMC.
	     *-------------------------------
	     */
		if (remain <= 0) 
		{
			eDescp->hdr.next = NULL;

			luc_sndCodNew( firstEDescp, pCod, xferSize);
	    	pStats->gmcPktCnt++;

			firstEDescp = NULL;
			prevEDescp  = NULL;

			pCod++;
			numOfCod++;

		    /*-------------------------------
			 * Shaun 3/15/02: 
			 * Fix for bug #222
		     * No more data to copy, 
			 * break out of while( pDesc).
		     *-------------------------------
		     */
			break;
		}

skipPktDesc:
	    /*-------------------------------
	     * Advance to next pkt & cod entry.
	     *-------------------------------
	     */
		pDesc = pDesc->pNext;

	}

    /*-------------------------------
     * This should NOT happen unless
	 * pkt/bd len or dataSize is wrong.
     *-------------------------------
     */
	if (remain > 0)
	{
		pkt_stats_t	*pStats;

		pStats = &pPktCfg->stats;
		pStats->cntBadCodLen++;

		panic("%s: Bad pkt len\n", __FUNCTION__);
	}


#ifdef LUC_DBG_COD
	luc_showCodEntry(numOfCod, codEntry);
#endif

	return (numOfCod);
}


#endif	/* #ifndef SBM_REV2 */

#endif /* #ifndef PROM */


/*-----------------------------------------------------------------
 * Name:        luc_dbgDescChain
 *
 * Description: Debug code to display Pkt Desc Chain & buffer.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 03/15/2002 
 *-----------------------------------------------------------------
 */
void luc_dbgDescChain( pkt_desc_t *pktDesc)
{

	eee_descBuf_t	*bd;
	pkt_desc_t 		*pDesc;
	eee_desc_t 		*eDescp;
	int32			descCnt;
	int32			bufCnt;
	int32			pktLen;

	descCnt=0;
	pDesc = pktDesc;
	printf("DataSize = %d\n", DataSize);
	while(pDesc)
	{
		int32	i;

		descCnt++;
		eDescp = pDesc->eDescp;
    	pktLen = (eDescp->hdr.control & EDESC_TOTAL_LEN);
		bufCnt = (eDescp->hdr.control & EDESC_BUFCNT_MASK) >>
					EDESC_BUFCNT_SHIFT;
		bd     = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
					 &(eDescp->bd[0]) : &(eDescp->bd[1]);

		printf("pDesc=%p pHdr=%p pMac=%p edesc=%p bufCnt=%d pktlen=%d\n", 
				pDesc, pDesc->pHdr,pDesc->pMac, eDescp, bufCnt,pktLen);

		for (i=0; i <bufCnt; i++)
		{
			printf("cnt=%d bd=%p bd->buf=%p bd->len=%d\n",
					i, bd, bd->buf, bd->len);
			bd++;
		}	

		if (descCnt > 30)
		{
			printf("descCnt > 30\n");
			break;
		}

		pDesc = pDesc->pNext;
	}

}

