/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-sim.c
 *
 * RCS:  $Id: luc-sim.c,v 1.48 2002/08/28 18:19:20 bob Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: LUC Simulation.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 07/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>
#include <exports/cod-api.h>
#ifndef PROM
#include <exports/sbm-api.h>
#endif
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include <exports/tpl-api.h>
#include "../sm-tpl/tpl.h"
#include "luc-cam.h"

#ifdef LINUX_TEST
#include <exports/evm-api.h>
#include <exports/rpc.h>
#include <exports/fs-api.h>
#include <exports/mount.h>
#include <exports/nfs.h>
#include <exports/nfp-api.h>
#endif

#if	defined(PROM) || defined (EVAL_BOARD) || defined (AGILE_HARDWARE)
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/udp.h"
#endif


#ifdef LINUX_TEST
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#endif

uint32			myIpaddr=0;
ushort16		nfsPort =0;


int				mnt_udp_sock;
int				nfs_udp_sock;
int				tftp_sock;
int				loop_srvr_sock;
int				loop_clnt_sock;

byte 			raw_rbuff[10240]; /* Raw RX buff */
byte 			raw_tbuff[10240]; /* Raw TX buff */


void  luc_simDllRcv(void *sock, uint32 tref);
int32 luc_simDllXmt( uint32 dstPort, eee_desc_t  *eDescp);


int32 luc_simCamFind(	int32	 	proto,
						uint32 		laddr, 
						uint32 		raddr, 
						ushort16 	lport, 
						ushort16 	rport );

int32  luc_camAlloc(int32 type);

extern char print_buf[];
char *print_ipa( struct in_addr *ipa )
{
	sprintf(print_buf, "%d.%d.%d.%d", 
		(ipa->s_addr ) & 0xFF,
		(ipa->s_addr >>  8) & 0xFF,
		(ipa->s_addr >> 16) & 0xFF,
		(ipa->s_addr >> 24) & 0xFF);
	return (print_buf);
}

char *inet_ntoa( struct in_addr ipa )
{
    sprintf(print_buf, "%d.%d.%d.%d",
        (ipa.s_addr ) & 0xFF,
        (ipa.s_addr >>  8) & 0xFF,
        (ipa.s_addr >> 16) & 0xFF,
        (ipa.s_addr >> 24) & 0xFF);
    return (print_buf);
}

int ip_decode( struct ip *ip_hdr)
{
	
	printf("-----------------------------------------------------\n");
	printf("IP ");
	printf("\tver:%2x   hlen:%2x  tos :%2x   iplen :%4d\n", 
		ip_hdr->ip_v, 
		ip_hdr->ip_hl, 
		ntohs(ip_hdr->ip_tos), 
		ntohs(ip_hdr->ip_len));
	printf("\tId :%04x                    offset:%04x\n",
		ntohs(ip_hdr->ip_id), 
		ntohs(ip_hdr->ip_off));
	printf("\tttl:%02x            prot:%02x   csum  :%04x\n", 
		ip_hdr->ip_ttl, 
		ip_hdr->ip_p, 
		ntohs(ip_hdr->ip_sum ));
	printf("\tsrc ip: %s\n", print_ipa(&ip_hdr->ip_src));
	printf("\tdst ip: %s\n", print_ipa(&ip_hdr->ip_dst));

	return( ip_hdr->ip_hl << 2 );
}

int udp_decode( struct udphdr *udp_hdr)
{

#ifdef LINUX_TEST
	printf("-----------------------------------------------------\n");
	printf("UDP ");
	printf("\tsrc port:%5d   dst port:%5d\n", 
		ntohs(udp_hdr->source),
		ntohs(udp_hdr->dest));
	printf("\tlength  :%5d   checksum: %4x\n", 
		ntohs(udp_hdr->len),
		ntohs(udp_hdr->check));
#endif

	return (sizeof(struct udphdr));
}


void xdr_swap(byte *pkt, int32 hdrSize, int32 dir)
{
	int32			i, wsize;
	uint32			tmp;
	uint32			*wptr;

	wptr = (uint32 *)pkt;
	wsize = (hdrSize >> 2);
	for (i=0; i < wsize; i++)
	{
		tmp = (dir) ? htonl(*wptr) : ntohl(*wptr);		
		*wptr++ = tmp;
	}
}



#if 0
/*
 * RAL - moved to sm-libc/ipaddr.c
 */
int32 getIpAddr( char *ipAddrStr, uint32 *addr)
{
	byte	ipAddr[4];
	char	*ip;
	char	*p;
	char	c;
	int32	i;

	ip = ipAddrStr;
	p = ip;
	c = *p;
	i = 0;

	while ( i < 4)
	{
		if ((c == '.') || (c == '\0'))
		{
			*p = '\0';	
			ipAddr[i] = atoi(ip);
			ip = p+1;
			i++;

			if (i > 3)
				break;
		}
		p++;
		c = *p;
	}

	if (i < 4)
		return -1;

	*addr = (ipAddr[0] << 24) | (ipAddr[1] << 16) | 
			(ipAddr[2] <<  8) | (ipAddr[3] );
	
}
#endif

#ifdef LINUX_TEST
int luc_simInit( void  )
{
	int		raw_sock;
	int		opt;
	struct	sockaddr_in		svr_addr;

	/*-------------------------------
	 * Get NFS UDP socket. 
	 *-------------------------------
	 */
	nfs_udp_sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (nfs_udp_sock < 0)
	{
		printf ("can't open NFS UDP socket: %s\n", sys_errlist[errno]);
		exit(1);
	}

	/*-------------------------------
	 * Get Mount UDP socket. 
	 *-------------------------------
	 */
	mnt_udp_sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (mnt_udp_sock < 0)
	{
		printf ("can't open Mount UDP socket: %s\n", sys_errlist[errno]);
		exit(1);
	}

	/*-------------------------------
	 * Get RAW NFS socket for UDP.
	 *-------------------------------
	 */
	raw_sock = socket(PF_INET, SOCK_RAW, IPPROTO_UDP);
	if (raw_sock < 0)
	{
		printf ("can't open RAW socket: %s\n", sys_errlist[errno]);
		exit(1);
	}

	/*-------------------------------
	 * Make it non-blocking.
	 *-------------------------------
	 */
	if ((fcntl(raw_sock, F_SETFL, FNDELAY) < 0) ||
		(fcntl(nfs_udp_sock, F_SETFL, FNDELAY) < 0) 	||
		(fcntl(mnt_udp_sock, F_SETFL, FNDELAY) < 0))
	{
		printf ("can't fcntl socket: %s\n", sys_errlist[errno]);
		exit(1);
	}
	
	/*-------------------------------
	 * Bind the Mount socket.
	 *-------------------------------
	 */
	bzero( (char *) &svr_addr, sizeof(svr_addr) );
	svr_addr.sin_family 	 = AF_INET;
    svr_addr.sin_addr.s_addr = htonl(myIpaddr);
	svr_addr.sin_port 		 = htons(nfsPort-1);

	if (bind(mnt_udp_sock, (struct sockaddr *)&svr_addr, sizeof(svr_addr)) < 0)
	{
		printf ("can't bind UDP socket: %s\n", errno, sys_errlist[errno]);
		exit(1);
	}

	/*-------------------------------
	 * Bind the NFS socket.
	 *-------------------------------
	 */
	bzero( (char *) &svr_addr, sizeof(svr_addr) );
	svr_addr.sin_family 	 = AF_INET;
    svr_addr.sin_addr.s_addr = htonl(myIpaddr);
	svr_addr.sin_port 		 = htons(nfsPort);

	if (bind(nfs_udp_sock, (struct sockaddr *)&svr_addr, sizeof(svr_addr)) < 0)
	{
		printf ("can't bind NFS UDP socket: %s\n", errno, sys_errlist[errno]);
		exit(1);
	}

	if (bind(raw_sock, (struct sockaddr *)&svr_addr, sizeof(svr_addr)) < 0)
	{
		printf ("can't bind RAW NFS socket: %s\n", sys_errlist[errno]);
		exit(1);
	}

    printf(" bind : addr=%x port=%x \n",
			ntohl(svr_addr.sin_addr.s_addr), ntohs(svr_addr.sin_port));


    /*-------------------------------
     * Allocate Data Link Layer App Id.
	 * Register rx polling func.
	 *-------------------------------
	 */
	if (eee_registerPollingFunc(EEE_HIGH_PRIORITY, 
								luc_simDllRcv, 
								(void *)raw_sock) < 0)
		luc_panic( );

    /*-------------------------------
     * Register with EEE for Tx.
     *-------------------------------
     */
	if (eee_registerTransmitFunc(NFP_PORT_TXRX_ETH0,
								luc_simDllXmt,
								luc_simDllXmt) < 0)
		luc_panic();

	return (raw_sock);
}



luc_simQueueXmtCmpl( eee_desc_t *eDescp, uint32 status )
{
    luc_ring_t     		*ringPtr;
    luc_txCmplDesc_t	*startPtr;
    luc_txCmplDesc_t	*endPtr;
    luc_txCmplDesc_t	*rdPtr;
    luc_txCmplDesc_t	*wrPtr;

    ringPtr  = &pLucCfg->tcr;
    startPtr = ringPtr->startAddr;
    endPtr   = ringPtr->endAddr;
    rdPtr    = ringPtr->rdAddr;
    wrPtr    = ringPtr->wrAddr;

	if (((rdPtr == startPtr) && (wrPtr == endPtr)) ||
		(rdPtr == (wrPtr + 1)))
	{
		printf("TCR full\n");
		luc_panic();
	}	

	wrPtr->status = status;
	wrPtr->pDesc  = eDescp->hdr.attr;
	
	if (wrPtr != endPtr)
		wrPtr++;
	else
		wrPtr = startPtr;

	ringPtr->wrAddr = wrPtr;
}
#else

luc_simQueueXmtCmpl( eee_desc_t *eDescp, uint32 status )
{
	luc_xmtCmp( (pkt_desc_t *) eDescp->hdr.attr );
}

#endif

typedef struct	_luc_simCam_t
{
	int32					camIdx;
	int32					proto;
	uint32					laddr;
	uint32					raddr;
	ushort16				lport;
	ushort16				rport;
	void 					*pConn;
	struct _luc_simCam_t	*pNext;
	struct _luc_simCam_t	*pPrev;

}	luc_simCam_t;

#define LUC_SIM_CAM_TBL_SIZE	NFX_MAX_CONN
#define LUC_SIM_CAM_HASH_SIZE	(2 * LUC_SIM_CAM_TBL_SIZE)

luc_simCam_t	luc_lkupTbl[LUC_SIM_CAM_TBL_SIZE];
luc_simCam_t	*luc_hashTbl[LUC_SIM_CAM_HASH_SIZE];

luc_simCamInit( void )
{

	memset(luc_lkupTbl, 0, sizeof(luc_lkupTbl));
	memset(luc_hashTbl, 0, sizeof(luc_hashTbl));
}

int32
luc_simCamHash( int32		proto,
				uint32 		laddr, 
				uint32 		raddr, 
				ushort16 	lport, 
				ushort16 	rport)
{
	int32 idx;

    /*-------------------------------
     * Sum of rmt addr component * 256
     *-------------------------------
     */
	idx  = ((( raddr        & 0xff) + ((raddr >>  8) & 0xff) + 
			 ((raddr >> 16) & 0xff) + ((raddr >> 24) & 0xff)) << 8);

    /*-------------------------------
     * Sum of rmt port * 16
     *-------------------------------
     */
	idx += (((rport & 0xff) + ((rport >> 8) & 0xff)) << 4); 

    /*-------------------------------
     * Local addr, port * protocol
	 * decide lower bits of idx.
     *-------------------------------
     */
	idx += (( laddr        & 0xff) + ((laddr >>  8) & 0xff) + 
			((laddr >> 16) & 0xff) + ((laddr >> 24) & 0xff));
	idx += ((lport & 0xff) + ((lport >> 8) & 0xff) + (proto & 0xff));

	idx &= (LUC_SIM_CAM_HASH_SIZE-1); 
	
	return idx;
}

void
luc_simCamDel( int32 camIdx)
{
	luc_simCam_t	*hashEntry;
	luc_simCam_t	*camEntry;
	int32			hashIdx;
	int32	 		proto;
	uint32 			laddr; 
	uint32 			raddr; 
	ushort16 		lport; 
	ushort16 		rport;

	camEntry = &luc_lkupTbl[camIdx];
	proto = camEntry->proto;
	laddr = camEntry->laddr;
	raddr = camEntry->raddr;
	lport = camEntry->lport;
	rport = camEntry->rport;


//	printf("luc_simCamDel: : %d:%x:%x:%d:%d, index=%d\n",
//		   proto,laddr,raddr,ntohs(lport),ntohs(rport), camIdx);

    /*-------------------------------
     * Get hash index 
     *-------------------------------
     */
	hashIdx   = luc_simCamHash(proto, laddr, raddr, lport, rport);
	hashEntry = luc_hashTbl[hashIdx];

    /*-------------------------------
     * Remove entry from hash list.
     *-------------------------------
     */
	if (camEntry->pPrev)
	{
		camEntry->pPrev->pNext = camEntry->pNext;
	}
	if (camEntry->pNext)
	{
		camEntry->pNext->pPrev = camEntry->pPrev;
	}

    /*-------------------------------
     * Del entry from head of hash list.
     *-------------------------------
     */
	if (hashEntry == camEntry)
	{
		luc_hashTbl[hashIdx]   = camEntry->pNext;
	}

    /*-------------------------------
     * Clear user info (pConn).
     *-------------------------------
     */
	luc_camSetUsrInfo(camIdx, NULL);
	return;
}

int32
luc_simCamAdd(	int32	 	proto,
				uint32 		laddr, 
				uint32 		raddr, 
				ushort16 	lport, 
				ushort16 	rport )
{
	int32			camIdx;
	int32			hashIdx;
	luc_simCam_t	*camEntry;
	luc_simCam_t	*hashEntry;

//	printf("luc_simCamAdd: : %x:%x:%x:%x:%x\n",
//			proto,laddr,raddr,lport,rport);

    /*-------------------------------
     * Get hash index 
     *-------------------------------
     */
	hashIdx   = luc_simCamHash((proto&0xff), laddr, raddr, lport, rport);
	hashEntry = luc_hashTbl[hashIdx];
	
//	printf("%s: %d:0x%x:0x%x:%d:%d hash idx= %d\n",
//			__FUNCTION__, proto, laddr, raddr, lport, rport, hashIdx);

    /*-------------------------------
     * Allocate a "CAM" entry.
     *-------------------------------
     */
	camIdx = luc_camAlloc( CAM_FLOW_TBL_ID );
	if ((camIdx < 0) || (camIdx >= LUC_SIM_CAM_TBL_SIZE))
	{
		printf("%s: Cam Alloc Failed! idx=%d\n", __FUNCTION__, camIdx);
		return (LUC_CAM_ERR_NO_RSRC);
	}	

	camEntry = &luc_lkupTbl[camIdx];
	camEntry->camIdx = camIdx;
	camEntry->proto = proto&0xff;
	camEntry->laddr = laddr;
	camEntry->raddr = raddr;
	camEntry->lport = lport;
	camEntry->rport = rport;
	camEntry->pNext = NULL;
	camEntry->pPrev = NULL;

    /*-------------------------------
	 * No collision in hashing,
     * put new entry in hash tbl.
     *-------------------------------
     */
	if (!hashEntry)
	{
		luc_hashTbl[hashIdx] = camEntry;
//		printf("luc_simCamAdd: : %d:%x:%x:%d:%d, index=%d, hash=%d\n",
//			   proto,laddr,raddr,ntohs(lport),ntohs(rport), camIdx, hashIdx);
		return camIdx;
	}

    /*-------------------------------
	 * Hash into same bucket.
     * Chain new entry at head.
     *-------------------------------
     */
	/* hashEntry->pNext = camEntry;
	   camEntry->pPrev  = hashEntry; */

	/* Su, double linked list insert */
    camEntry->pPrev =  hashEntry;
	camEntry->pNext =  hashEntry->pNext;
	hashEntry->pNext = camEntry;
	if (camEntry->pNext)
		camEntry->pNext->pPrev = camEntry;
	

//	printf("luc_simCamAdd collision:  %d:%x:%x:%d:%d, index=%d, hash=%d\n",
//		   proto,laddr,raddr,ntohs(lport),ntohs(rport), camIdx, hashIdx);
	return camIdx;
}


int32 
luc_simCamFind(	int32	 	proto,
				uint32 		laddr, 
				uint32 		raddr, 
				ushort16 	lport, 
				ushort16 	rport )
{
	int32	 		hashIdx;
	luc_simCam_t	*hashEntry;

	hashIdx   = luc_simCamHash((proto&0xff), laddr, raddr, lport, rport);
	hashEntry = luc_hashTbl[hashIdx];

	while (hashEntry)
	{
		if ((hashEntry->laddr == laddr) &&
			(hashEntry->raddr == raddr) &&
			(hashEntry->lport == lport) &&
			(hashEntry->rport == rport)  &&
			 ((hashEntry->proto&0xff) == (proto&0xff)) )
		{
			if (luc_traceFlag)
			{
				printf("luc_simCamFind: Entry found for: %d:%x:%x:%d:%d, hash=%d\n",
						proto,laddr,raddr,ntohs(lport),ntohs(rport), hashIdx);
			}
			return( hashEntry->camIdx );
		}

		hashEntry = hashEntry->pNext;
	}

	if (luc_traceFlag)
	{
		printf("luc_simCamFind: No entry found for: %d:%x:%x:%d:%d, hash=%d\n",
				proto,laddr,raddr,ntohs(lport),ntohs(rport), hashIdx);
	}
	return -1;

}

/*-----------------------------------------------------------------
 * Name: luc_simStart
 *
 * Description: Start LUC Emulation.
 *
 *              luc start <ipaddr> <nfsport>
 * 
 * Created by : Shaun Hui
 * 
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int32 luc_simStart( int32 argc, char *argv[] )
{
	int32			port;

	if (argc != 4)
	{
		luc_cmdHelpCmd("start");
		return 0;
	}

	if (myIpaddr || nfsPort)
	{
		printf("LUC already started!\n");
		return 0;
	}

	if (luc_simStart_setIpAddr(argv[2]) < 0)
		return -1;

/*
	if (getIpAddr( argv[2], &myIpaddr ) < 0)
		return -1;
*/

#ifdef LINUX_TEST
	if ((port = atoi(argv[3])) < 0)
		return -1;

	nfsPort = (ushort16) port;
#else
	nfsPort = (ushort16) 2088;
	port    = (ushort16) 2088;
#endif

	return(luc_simStart_withArgs(port));
}

int32 luc_simStart_setIpAddr(uchar8 * str)
{
	nfsPort = (ushort16) 2088;
	return (getIpAddr(str, &myIpaddr));
}

int32 luc_simStart_withArgs(int32 port)
{
	uint32			ipMask;
	uchar8			ipm[20];

#ifdef PROM
	seepGetIpMask(ipm);
	getIpAddr(ipm, &ipMask);
#else
	ipMask = 0xFFFF0000;
#endif
	
	/* su, 05/10, don't panic in prom, because seep info may not be read */
	/* correctly between prom upgrades */
#ifdef PROM
	bsd_CreateIpIntf(htonl(myIpaddr),  htonl(ipMask), "fp0",   0, 0);
#else
	if (bsd_CreateIpIntf(htonl(myIpaddr),  htonl(ipMask), "fp0",   0, 0)
		!= NFX_OK)
		luc_panic();
#endif 

//	printf(" LUC Test Started\n");
	
	luc_simCamInit( );

#ifdef LINUX_TEST
	luc_simInit( );

	if (mnt_bindReq( myIpaddr ) != NFX_OK)
	{
		printf( "tpl_bind-MNT failed\n");
		return -1;
	}

	if (nfs_tplBindReq( myIpaddr ) != NFX_OK)
	{
		printf( "tpl_bind-NFS failed\n");
		return -1;
	}
#endif

	return 0;
}

#ifdef LINUX_TEST
/*-----------------------------------------------------------------
 * Name: 		luc_simDllRcv
 *
 * Description: Simulate packet reception from network driver using
 *				raw socket.
 * 
 * Created by : Shaun Hui
 * 
 * Date Created: 07/13/2001
 *-----------------------------------------------------------------
 */
void luc_simDllRcv(void *sock, uint32 tref)
{
    struct  sockaddr_in     from;
    eee_desc_t              *eDescp;
    eee_descBuf_t           *bd;
    char                    *srcBuf;
    char                    *dstBuf;
    int                     fromlen;
    int                     dataLen;
    int32                   copyLen;
    int32                   bufCnt;


    /*-------------------------------
     * Rx from raw sock. 
     *-------------------------------
     */
	dataLen = recvfrom((int)sock, raw_rbuff, sizeof(raw_rbuff), 
						0, &from, &fromlen);
	if ((dataLen < 0)  || (dataLen > sizeof(raw_rbuff)))
		return;

    /*-------------------------------
     * Findout how many EEE buff needed.
     * Ensure pkt fits within 1 eee desc.
     *-------------------------------
     */
    bufCnt = (dataLen+PKT_BUFF_INDENT)/EEE_BUFSIZE_LARGE + 1;
    if (bufCnt > EEE_MAX_BUFS)
        return;

	/*-------------------------------
	 * Get a EEE Descriptor.
	 *-------------------------------
	 */
	eDescp = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if (!eDescp)
		return;

    eee_attrAssignDesc(eDescp, EEE_ATTR_LUC_ALLOC);

	/*-------------------------------
	 * Setup EDesc info.
	 *-------------------------------
	 */
	memset( eDescp, 0, sizeof(eee_desc_t));
	eDescp->bd[0].control = 0;
	eDescp->hdr.dest_port = pLucCfg->rcvPort;
	eDescp->hdr.src_port  = pLucCfg->drvPort[0];
	eDescp->hdr.control   = (dataLen & EDESC_TOTAL_LEN) |
							((bufCnt<< EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
    
	/*-------------------------------
	 * Allocate EEE buffer.
	 *-------------------------------
	 */
	dstBuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
	if (!dstBuf)
		goto rxCleanUp;

    eee_attrAssignBuf(dstBuf, EEE_ATTR_LUC_ALLOC);

	/*-------------------------------
	 * Setup 1st dst & src buff & copy len.
	 * Use EEE bd[1] for 1st buff.
	 *-------------------------------
	 */
	bd     = &(eDescp->bd[1]); 
	dstBuf += PKT_BUFF_INDENT;
    srcBuf  = raw_rbuff;
	copyLen = (dataLen <= (EEE_BUFSIZE_LARGE - PKT_BUFF_INDENT)) ? 
			  dataLen : (EEE_BUFSIZE_LARGE - PKT_BUFF_INDENT) ;

	/*-------------------------------
	 * Copy from raw buf into EEE bufs.
	 *-------------------------------
	 */
    while( bufCnt-- )
    {
		/*-------------------------------
		 * Setup EEE BD & copy data.
		 *-------------------------------
		 */
        bd->buf     = dstBuf;
        bd->len     = copyLen;
        bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
        memcpy(dstBuf, srcBuf, copyLen);

		/*-------------------------------
		 * Setup src & len for next buff.
		 *-------------------------------
		 */
        srcBuf  += copyLen;
        dataLen -= copyLen;
		copyLen  = (dataLen <= (EEE_BUFSIZE_LARGE)) ? 
					dataLen : EEE_BUFSIZE_LARGE;
		bd++;

		if (!bufCnt)
			break;

		/*-------------------------------
		 * Allocate EEE buffers.
		 *-------------------------------
		 */
        dstBuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);

        eee_attrAssignBuf(dstBuf, EEE_ATTR_LUC_ALLOC);

		/*-------------------------------
		 * Clean up if no buffer.
		 *-------------------------------
		 */
        if (!dstBuf)
			goto rxCleanUp;
    }

	/*-------------------------------
	 * Hand pkt to upper layer
	 *-------------------------------
	 */
	eee_receivePacket( eDescp );
	return;

rxCleanUp:
	/*-------------------------------
	 * Free prev. allocated EEE buffers.
	 *-------------------------------
	 */
	bd     = &(eDescp->bd[1]); 
	while (bd->buf)
	{
		eee_deallocateBuffer( bd->buf );
		bd->buf     = 0;
		bd->len     = 0;
		bd->control = 0;
		bd++;
	}

	/*-------------------------------
	 * Free prev. allocated EEE desc.
	 *-------------------------------
	 */
	eee_deallocateDesc( eDescp );
	return;
	
}

/*-----------------------------------------------------------------
 * Name: 		luc_simDllXmt
 *
 * Description: Simulate packet transmission to network driver using
 *				raw socket.
 * 
 * Created by : Shaun Hui
 * 
 * Date Created: 07/13/2001
 *-----------------------------------------------------------------
 */
int32
luc_simDllXmt( uint32 dstPort, eee_desc_t  *eDescp)
{
	struct	sockaddr_in		cli_addr;
	pkt_desc_t				*pDesc;
	uint32					status;
    uint32                  dataLen;
    uint32                  copyLen;
    uint32                  bufCnt;
    uchar8                  *srcBuf;
    uchar8                  *dstBuf;
    eee_descBuf_t           *bd;
	int32					rc;
	int						sock;

    /*-------------------------------
     * Retrieve PKT Desc. from EEE Desc.
	 * Stored at the attr. field
     *-------------------------------
     */
	pDesc = (pkt_desc_t *)eDescp->hdr.attr;

    /*-------------------------------
     * Setup src, dst buf & copy len
     *-------------------------------
     */
    dstBuf = raw_tbuff;
    bufCnt = (eDescp->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	bd     = (eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
            &(eDescp->bd[0]) : &(eDescp->bd[1]);
    dataLen= 0;

    /*-------------------------------
     * Copy all EEE bufs to raw tx buff
     *-------------------------------
     */
    while( bufCnt--) 
    {
        copyLen = bd->len;
        srcBuf  = bd->buf;

        memcpy(dstBuf, srcBuf, copyLen);

        dstBuf  += copyLen;
        dataLen += copyLen;
		bd++;
    }

    /*-------------------------------
     * Pad NFS read data with '0' to
     * 4 byte word boundary.
     *-------------------------------
     */
    if ((uint32) dstBuf & 0x03)
    {
        uint32 pad;

        pad = sizeof(uint32) - ((uint32)dstBuf & 0x03);
        memset(dstBuf, 0, pad);
        dataLen += pad;
    }

    /*-------------------------------
     * Setup remote address & port.
     *-------------------------------
     */
	memset(&cli_addr, 0, sizeof(cli_addr));
	cli_addr.sin_family      = AF_INET;
	cli_addr.sin_port        = pDesc->rport; 
	cli_addr.sin_addr.s_addr = pDesc->raddr;

    /*-------------------------------
     * Check if NFS or Mount socket 
	 * should be use.
     *-------------------------------
     */
	if (pDesc->lport == htons(nfsPort) )
		sock = nfs_udp_sock ;
	else if (pDesc->lport == htons(nfsPort-1) )
		sock = mnt_udp_sock ;
	else if (pDesc->lport == htons(nfsPort-2) )
		sock = tftp_sock ;
	else if (pDesc->lport == htons(nfsPort-3) )
		sock = loop_srvr_sock ;
	else if (pDesc->lport == htons(nfsPort-4) )
		sock = loop_clnt_sock ;
    else
    {
        int     i;
        
        for (i = 0; i < MAX_LUC_USER; i++)
        {
            if ((lucUserTab[i].inUse == TRUE) &&
                (pDesc->lport == lucUserTab[i].port))
            {
                sock = lucUserTab[i].sock;
                printf("lport = %d\n", ntohs(pDesc->lport));
                printf("sock = %d\n", sock);
                break;
            }                        
        }// for
    }
   	
	rc = sendto(sock, raw_tbuff, dataLen, 0, &cli_addr, sizeof(cli_addr));
	if (rc < 0)
	{
		printf("Sendto failed %s\n", sys_errlist[errno]);
	}

    /*-------------------------------
     * Queue pDesc for Tx cmpl cleanup.
     *-------------------------------
	 */
	luc_simQueueXmtCmpl( eDescp , status);

	return EEE_FORWARDED_PACKET;
}
#endif

#ifdef LINUX_TEST

/* Functions to manipulate the array of LUC's users.
 * Use by data mover to add some flexibility to LUC.
 */

void
luc_initLucUserTable(void)
{
    int i;
    
    for (i = 0; i < MAX_LUC_USER; i++)
    {
        lucUserTab[i].inUse = FALSE;
        lucUserTab[i].port = 0;
        lucUserTab[i].sock = -1;
    } // for
}

int
luc_getFreeLucUserIndex(void)
{
    int i;

    for (i = 0; i < MAX_LUC_USER; i++)
    {
        if (lucUserTab[i].inUse == FALSE)
        {
            return(i);
        }
    } // for
    return(-1);
}

void
luc_freeLucUserIndex(int index)
{
    if (index < MAX_LUC_USER)
    {
        lucUserTab[index].inUse = FALSE;
        lucUserTab[index].port = 0;
        lucUserTab[index].sock = -1;
    }
    return;
}
#endif // LINUX_TEST
