/* vi:ts=4 
 *-----------------------------------------------------------------
 *
 * Name: luc-tx.c
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains LUC Hardware test functions.
 *
 * Created by:	Su Wang
 *
 * Date Created: 07/23/2002
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include <exports/cod-api.h>
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include <exports/tpl-api.h>

#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/udp.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/ip_stats.h"

#include "luc-cam.h"
#include "luc-pkthdr.h"

static char arp_buf[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
						  0x08, 0x00, 0x20, 0xae, 0xb2, 0x25, 
						  0x08, 0x06,
						  0x00, 0x01, 0x08, 0x00, 0x06, 0x04, 0x00, 0x01,
						  0x08, 0x00, 0x20, 0xae, 0xb2, 0x25, 0x0a, 0x03,
                          0x01, 0x11, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
						  0x0a, 0x03, 0x00, 0x15};


static char udp_buf[] ={ 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
						 0x08, 0x00, 0x20, 0xae, 0xb2, 0x25, 
						 0x08, 0x00,
						 0x45, 0x00, 0x00, 0x7c, 0xa7, 0x25, 0x00, 0x00, 
						 0x1e, 0x11, 0xdf, 0x0d, 0x0a, 0x01, 0x01, 0x09,
						 0x0a, 0x01, 0x01, 0x34, 0x03, 0x1f, 0x08, 0x01,
						 0x00, 0x68, 0x06, 0x06, 0x13, 0xa0, 0x74, 0x45,
						 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 
						 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
						 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
						 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x40, 0x3f,
						 0x00, 0x00, 0x00, 0x0b, 0x00, 0x00, 0xea, 0x61,
						 0x00, 0x00, 0xff, 0xfe, 0x00, 0x00, 0x02, 0x00,
						 0x00, 0x00, 0x20, 0x00, 0x80, 0x00, 0x0c, 0x01,
						 0x00, 0x00, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00,
						 0x00, 0x00, 0x09, 0x00, 0x3b, 0x73, 0x95, 0xa8,
						 0x00, 0x0a, 0xae, 0x7c, 0x3b, 0x4b, 0x44, 0xb9,
						 0x00, 0x03, 0x0d, 0x41, 0x3a, 0xae, 0x85, 0x4d,
						 0x00, 0x07, 0xa1, 0x20};


/* size 90, tcp sum = 10b5 */
static char tcp_buf [] ={0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
						 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 
						 0x08, 0x00,
						 0x45, 0x00, 0x00, 0x4c, 0x2e, 0xd6, 0x20, 0x00,
						 0x40, 0x06, 0x00, 0x00, 0x0a, 0x01, 0x01, 0x96,
                         0x0a, 0x01, 0x01, 0xc0, 0x0b, 0x99, 0x00, 0x17, 
						 0x1a, 0x59, 0x18, 0xb1, 0xd7, 0xc8, 0x2a, 0xd1,
                         0x80, 0x18, 0x7d, 0x78, 0x00, 0x00, 0x00, 0x00,
						 0x01, 0x01, 0x08, 0x0a, 0x10, 0xf8, 0x53, 0x3c,
                         0x00, 0xa0, 0x1e, 0xbe, 0xff, 0xfa, 0x22, 0x01,
						 0x0c, 0xff, 0xf0, 0xff, 0xfa, 0x22, 0x03, 0x04,
                         0x80, 0xff, 0xff, 0x0e, 0x80, 0xff, 0xff, 0xff,
						 0xf0, 0xff, 0xfd, 0x01};


char myMac[] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06};

#define LUC_TEST_PRINTF(a) 
#define LUC_TEST_ERR(a)		printf a

ushort16
luc_cksum(ushort16 *ptr, int len, uint32 value)
{
	int		nleft = len;
	uint32	sum = value;

  // debugPrintf(("chksum len %d\n", len));
  while (nleft > 1)
    {
      sum += *ptr++;
      nleft -= 2;
    }
    
  if (nleft == 1)
    sum += ((ushort16)*((char*)ptr)) & 0xff;

  // debugPrintf(("raw chksum val %lx\n", sum));
  sum = ((sum >> 16) & 0xffff) + (sum & 0xffff);
  sum += (sum >> 16) & 0xffff;
  
  //  debugPrintf(("chksum val %lx\n", ~sum & 0xffff));
  return (~sum & 0xffff);
}

int32 
luc_Verify(eee_desc_t   *eDescp)
{
	luc_rxPktHdr_t	*pLucPktHdr;
	uchar8			classifier;
	int32			link;
	int32			err;
	ushort16		index;
	uchar8			l3Offset, l4Offset, l5Offset;
	int				self = 0;
	char			*pLinkMac=0;
	char			*pBuff;
	
	
	
	pLucPktHdr  = (luc_rxPktHdr_t*)((eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
									(eDescp->bd[0].buf) : (eDescp->bd[1].buf));	


	/* pLucPktHdr 32 bits aligned */
	if ((address_t)pLucPktHdr & 0x03) {
		LUC_TEST_ERR(("bad lucpkt address %lx\n", (address_t)pLucPktHdr));
		return NFX_ERR;
	}

	/* pkt from my self */
  	if (memcmp(((char*)(pLucPktHdr + 1))+6, myMac, 6) == 0)
  		self = 1;
	l3Offset = pLucPktHdr->lucrx_l3OffHW * 2;

	l4Offset = pLucPktHdr->lucrx_l4Offset;
	l5Offset = pLucPktHdr->lucrx_l5Offset;
	classifier= pLucPktHdr->lucrx_classifier;
	index =    ntohs(pLucPktHdr->lucrx_camIdx);

	/* get info from luc header */
	link = pLucPktHdr->lucrx_linkId;
	err  = pLucPktHdr->lucrx_err;

#if  0 
	if ((classifier != LUC_RXPKT_CLASS_MAC) &&
		(classifier != LUC_RXPKT_CLASS_NOPROT)) {
		
		bsd_ShowPkt(pLucPktHdr, 14);
		printf("frame start  %lx\n", 
			   (address_t)(pLucPktHdr+1));
		printf("lucrx data   %x\n", pLucPktHdr->lucrx_data);
		printf("lucrx size %x\n",pLucPktHdr->lucrx_pktSize);
		printf("lucrx chksum %x\n", pLucPktHdr->lucrx_chkSum);
		printf("lucrx l5off  %x\n", l5Offset);
		printf("lucrx l4off  %x\n", l4Offset);
		printf("lucrx index  %x\n", index);
		printf("lucrx class  %x\n", classifier);
		printf("lucrx l3off  %x\n", l3Offset);
		printf("lucrx link   %x\n", link);
		printf("lucrx err    %x\n", err);
	}
#endif /* 0 */

	/* check for mac address matches */
	pLinkMac = luc_getMacAddr(link);
	pBuff = (char*)(pLucPktHdr + 1);
	if (!(*pBuff & 0x01) && (memcmp(pLinkMac, pBuff,6))) {
		LUC_TEST_ERR(("Luc_Verify: mac address doesnt match, link  %d, link mac %s\n",
					  link, bsdPrintMac(pLinkMac)));
		LUC_TEST_ERR(("pkt dest mac %s\n", bsdPrintMac(pLinkMac)));
		return NFX_ERR;
	}


	switch (classifier) 		{
	case LUC_RXPKT_CLASS_ARP:
		LUC_TEST_PRINTF(("arp rcvd: err %d, link %d, class %d, idx %d, offset %d/%d/%d\n",
						 err, link, classifier, index, l3Offset, l4Offset,l5Offset));
		break;
	
	case LUC_RXPKT_CLASS_MAC:
	case LUC_RXPKT_CLASS_NOPROT:
		LUC_TEST_PRINTF(("bad pkt rcv: err %d, link %d, class %d, idx %d, offset %d/%d/%d\n",
						 err, link, classifier, index, l3Offset, l4Offset,l5Offset));
		break;			/* let rx code handle */
	
	default:	
	{				/* must be an IP pkt */
		struct ip	*pIp;
		char		*tplHdr;
		int			ipHlen;
		uint32		sum;
		ushort16	tplLen;
		ushort16        rc;
		
		 
		pIp = (struct ip*)(((char*)(pLucPktHdr + 1)) + l3Offset);
		LUC_TEST_PRINTF(("ip pkt rcv: err %d, link %d, class %d, idx %x, proto %d offset %d/%d/%d\n",
						 err, link, classifier, index, pIp->ip_p, 
						 l3Offset, l4Offset,l5Offset));

		/* ip hdr cksum */
		ipHlen = l4Offset - l3Offset;
		if (luc_cksum((ushort16*)pIp, ipHlen, 0)) {
			LUC_TEST_ERR(("bad iphdr cksum: pIp %lx hlen %d\n",
						  (address_t)pIp, ipHlen));
			bsd_ShowPkt(pLucPktHdr, 48);
			break;
		}
		

		if ((pIp->ip_p == 6 || pIp->ip_p == 17)) { 

			/* tcp/udp chck sum */
			
			if (classifier != LUC_RXPKT_CLASS_FRAG) {
				sum = 0;
				sum += pIp->ip_src.s_addr & 0xffff;
				sum += (pIp->ip_src.s_addr >> 16) & 0xffff;
				sum += pIp->ip_dst.s_addr & 0xffff;
				sum += (pIp->ip_dst.s_addr >> 16) & 0xffff;
				sum += (pIp->ip_p & 0xff)<< 8; /* make it big edian */ 
				tplLen = ntohs(pIp->ip_len) - ipHlen;
				sum += htons(tplLen);
			
				tplHdr = ((char*)pIp) + ipHlen;

				/* udp chksum optional */
				if ((pIp->ip_p == 17) && 
					!(*(ushort16*)(tplHdr+6)))
					break;
				
				if ((rc = luc_cksum((ushort16*)tplHdr, tplLen, sum))) {
					LUC_TEST_ERR(("tcp/udp cksum err(%x): prot %d  tplen %d, ipHdr %lx, tplHdr %lx\n",
								  rc, pIp->ip_p, tplLen, (address_t)pIp, (address_t)tplHdr));
					bsd_ShowPkt(pLucPktHdr, tplLen+50);
				}
			}
			/* fragments */
			else {
				ushort16 chkSum = pLucPktHdr->lucrx_chkSum;
				ushort16 myChkSum;
			
				tplLen = ntohs(pIp->ip_len) - ipHlen;
				tplHdr = ((char*)pIp) + ipHlen;
				
				sum = 0;
				if (!(pIp->ip_off & htons(IP_OFFMASK))) {
					/* first fragment */
					sum += pIp->ip_src.s_addr & 0xffff;
					sum += (pIp->ip_src.s_addr >> 16) & 0xffff;
					sum += pIp->ip_dst.s_addr & 0xffff;
					sum += (pIp->ip_dst.s_addr >> 16) & 0xffff;
					sum += (pIp->ip_p & 0xff)<< 8; /* make it big edian */ 


					if (pIp->ip_p == 17)
						sum += *(ushort16*)(tplHdr + 4);
					else {
						/* don't know tcp seg length yet,
						 * unset tpl len
						 */
						uint32	tmp;
						tmp = chkSum + (~htons(tplLen) & 0xffff);
						tmp = (tmp & 0xffff) + (tmp>>16)&0xffff;
						tmp = (tmp & 0xffff) + (tmp>>16)&0xffff;
						chkSum = tmp;
					}
					
				}
			
				myChkSum = luc_cksum((ushort16*)tplHdr, tplLen, sum);
				
				/* uncomplement */
				myChkSum = ~myChkSum;
				
				if (chkSum && (myChkSum != chkSum)) {
					LUC_TEST_ERR(("tcp/udp frag cksum err(%x %x %x): prot %d  tplen %d, ipHdr %lx, tplHdr %lx\n",
								  myChkSum, chkSum, sum,
								  pIp->ip_p, tplLen, 
								  (address_t)pIp, (address_t)tplHdr));
					pLucPktHdr->lucrx_chkSum = myChkSum;
					bsd_ShowPkt(pLucPktHdr,tplLen+50);
				}
			}
		}
		break;
	} /* default */
		
	}
	
	/* free edesc, if self generated pkt externally looping bck */
	if (self)
		return NFX_ERR;
	else
		return NFX_OK;
}


#if !defined(PROM)
/* proto: ip protocol
 * frag: 0 unfraqmented, 1 fragmented pkts 
 */
extern uint32	luc_loopback_test;
void luc_sentTstPkt(uchar8 proto, int frag)
{
	luc_txPktHdr_t	lucPktHdr;
	eee_desc_t		*pDesc;
	uchar8			*bufPtr;
	int				pktLen;
	uchar8			*pktPtr;
	uint32			outPort;

	bzero(&lucPktHdr, sizeof(lucPktHdr));
	
	/* set up luc hdr for different pkts */
	switch(proto) 
	{
	case 6:
		pktLen = sizeof(tcp_buf);
		pktPtr = tcp_buf;
		lucPktHdr.luctx_linkId = 0;
		lucPktHdr.luctx_l3Offset = 14;
		lucPktHdr.luctx_l4Offset = 20 + 14;
		lucPktHdr.luctx_ipCsumOffset = 14 + 10;
		lucPktHdr.luctx_frag = 1;
		lucPktHdr.luctx_ctrl  |= (LUC_TXPKT_CTRL_IPCSUM | LUC_TXPKT_CTRL_TPLCSUM);

		if (!frag)	{				/* last fragment, fill in pseudo hdr */
			*(pktPtr + 21) = 0;
			lucPktHdr.luctx_tplLen = htons((pktLen - 34));
			memcpy(&(lucPktHdr.luctx_srcAddr),
				   (tcp_buf + 14 + 12), 4);
			memcpy(&(lucPktHdr.luctx_dstAddr),
				  (tcp_buf + 14 + 16), 4);
			lucPktHdr.luctx_proto = proto;
			lucPktHdr.luctx_tplCsumOffset = lucPktHdr.luctx_l4Offset + 16;
			lucPktHdr.luctx_lFrag = 1;
		}
		else
			*(pktPtr + 21) = 11;
		break;
		
	default:
		return;
	}
	
	pDesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
#ifdef EEE_ATTR
    eee_attrAssignDesc(pDesc, EEE_ATTR_LUC_ALLOC);
#endif
	bzero(pDesc, sizeof(eee_desc_t));
	bufPtr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
#ifdef EEE_ATTR
    eee_attrAssignBuf(bufPtr, EEE_ATTR_LUC_ALLOC);
#endif
	pDesc->bd[1].buf = bufPtr;

	/* bit fields to network order */
	lucPktHdr.luctx_data = htons(lucPktHdr.luctx_data);
	
	/* copy pkt to buff */
	memcpy(bufPtr, &lucPktHdr, sizeof(luc_txPktHdr_t));
	bufPtr += sizeof(luc_txPktHdr_t);
	memcpy(bufPtr, pktPtr, pktLen);
	pDesc->bd[1].len = 	pktLen + sizeof(luc_txPktHdr_t);
	pDesc->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
	pDesc->hdr.control = (1 << EDESC_BUFCNT_SHIFT) | pDesc->bd[1].len;

	/* set destination port */
	pDesc->hdr.src_port =  0xbbccddee;

#if defined(PROM)
	outPort = PROM_PORT_TXRX0;
#else
	outPort = NFP_PORT_TXRX_ETH0;
#endif /* PROM */
	pDesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
		nfxMySlotId,
		nfxMyCPUId,
		0,
		EEE_USER_DATA,
		NFX_COS_HIGH,
		outPort);

	bmc12500Eth_txLink_edesc( pDesc->hdr.dest_port,pDesc );
	
	return;
}
#endif /* PROM */


#if 0
void 
luc_Redirect(eee_desc_t	*eDescp)
{
	pkt_desc_t		pDesc;
	luc_txPktHdr_t	*pLucTxHdr;
	luc_rxPktHdr_t	lucRxHdr;
	pkt_ethHdr_t	*pMacHdr;
	struct	ip		*pIpHdr;
	char			*pBuff;
	eee_desc_t		*newDescp;


	pLucTxHdr =  (luc_txPktHdr_t*)((eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
								  (eDescp->bd[0].buf) : (eDescp->bd[1].buf));	
	LUC_TEST_PRINTF(("pkt %x in driver\n", pLucTxHdr));
	bsd_ShowPkt(pLucTxHdr, 64);

	lucRxHdr.lucrx_txSeq = 0;
	lucRxHdr.lucrx_txErrSeq = 0;
	lucRxHdr.lucrx_errLink = 0;	/* err code 0, link 0 */

	/* always go through slow path for now */
	lucRxHdr.lucrx_camIdx = htons(LUC_RXPKT_CAM_MISS);
	lucRxHdr.lucrx_l3Offset = pLucTxHdr->luctx_l3Offset;
	lucRxHdr.lucrx_l4Offset = pLucTxHdr->luctx_l4Offset;
	lucRxHdr.lucrx_l5Offset = 0;

	lucRxHdr.lucrx_pktSize = pLucTxHdr->luctx_pktSize;

	/* specify classifier */
	pMacHdr = (pkt_ethHdr_t*)(pLucTxHdr + 1);
	if (ntohs(pMacHdr->type) == 0x0806) /* arp */ {
		LUC_TEST_PRINTF(("arp redirect in driver\n"));
		lucRxHdr.lucrx_classifier = LUC_RXPKT_CLASS_ARP;
	}
	else if (ntohs(pMacHdr->type) == 0x0800) /* ip */ {
		pIpHdr = (struct ip*)(pMacHdr + 1);
		if (pIpHdr->ip_p == 6) {
			LUC_TEST_PRINTF(("tcp redirect in driver\n"));
			lucRxHdr.lucrx_classifier = LUC_RXPKT_CLASS_FULL;
		}
		else if (pIpHdr->ip_p == 17) {
			LUC_TEST_PRINTF(("udp redirect in driver\n"));
			lucRxHdr.lucrx_classifier = LUC_RXPKT_CLASS_FULL;
		}	
		else {
			LUC_TEST_PRINTF(("ip redirect in driver\n"));
			lucRxHdr.lucrx_classifier = LUC_RXPKT_CLASS_IPNOPROT;
		}
	}
	else {
		LUC_TEST_ERR(("%d: bad proto type %x\n",
						__FUNCTION__, ntohs(pMacHdr->type)));
		luc_simQueueXmtCmpl( eDescp , 0);
		return;
	}
	
	/* allocate new eDesc */
	pBuff = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
#ifdef EEE_ATTR
    eee_attrAssignBuf(pBuff, EEE_ATTR_LUC_ALLOC);
#endif
	newDescp = 	eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
#ifdef EEE_ATTR
    eee_attrAssignDesc(newDescp, EEE_ATTR_LUC_ALLOC);
#endif

	memset(newDescp, 0, sizeof(eee_desc_t));
	newDescp->bd[0].control = 0;
	newDescp->bd[1].buf     = pBuff;
	newDescp->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
	newDescp->hdr.dest_port = 0;
	newDescp->hdr.src_port  = 0;
	newDescp->hdr.control   = (( 1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

	/* copy */
	memcpy(newDescp->bd[1].buf, &lucRxHdr, sizeof(luc_rxPktHdr_t));
	memcpy((newDescp->bd[1].buf + sizeof(lucRxHdr)), 
		   pMacHdr, lucRxHdr.lucrx_pktSize);
	
	/* remove the old eDesc */
	luc_simQueueXmtCmpl( eDescp , 0);

	/* give to input */
	luc_rcvPkt(newDescp, 0);
	return;
}

void luc_WrapRx(eee_desc_t	*eDescp) 
{
	char			*pBuff;
	eee_descBuf_t	*bd;
	luc_rxPktHdr_t  *pLucRxPkt;
	pkt_ethHdr_t	*macHdr;
	struct	ip		*pIpHdr;

	/* copy to a new buffer with required offset */
	pBuff = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
#ifdef EEE_ATTR
    eee_attrAssignBuf(pBuff, EEE_ATTR_LUC_ALLOC);
#endif

	pBuff += PKT_BUFF_INDENT + 2; /* ether frame starts 16 bits aligned */
	bd = ((eDescp->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER) ?
		  &(eDescp->bd[0]) : &(eDescp->bd[1]));
	memcpy(pBuff, bd->buf, bd->len);
	
	/* dealloc old buffer */
	eee_deallocateBuffer(bd->buf);

	bd->buf = pBuff;
	macHdr = (pkt_ethHdr_t*)pBuff;
	

	/* emulate new luc headers */
	bd->buf -= sizeof(luc_rxPktHdr_t);
	bd->len += sizeof(luc_rxPktHdr_t);

	pLucRxPkt = (luc_rxPktHdr_t*)bd->buf; 
	pLucRxPkt->lucrx_txSeq = 0;
	pLucRxPkt->lucrx_txErrSeq = 0;
	pLucRxPkt->lucrx_errLink = 0; /* always from link 0 */
	
	/* always not found */
	pLucRxPkt->lucrx_camIdx = htons(LUC_RXPKT_CAM_MISS);

	
	pLucRxPkt->lucrx_pktSize = htons(bd->len - sizeof(luc_rxPktHdr_t));

	switch(ntohs(macHdr->type)) {
	case 0x800:					/* IP  */
		pIpHdr = (struct ip*)(macHdr + 1);
		pLucRxPkt->lucrx_l3Offset = 14;
		pLucRxPkt->lucrx_l4Offset = 14 + 20;
		
		if (htons(pIpHdr->ip_off) & (IP_MF | IP_OFFMASK | IP_RF))
			pLucRxPkt->lucrx_classifier = LUC_RXPKT_CLASS_FRAG;
		else if (pIpHdr->ip_p == 17) {
			pLucRxPkt->lucrx_l5Offset = pLucRxPkt->lucrx_l4Offset + 8;
			pLucRxPkt->lucrx_classifier = LUC_RXPKT_CLASS_FULL;
		}
		else if (pIpHdr->ip_p == 6) {
			pLucRxPkt->lucrx_l5Offset = pLucRxPkt->lucrx_l4Offset + 20;
			pLucRxPkt->lucrx_classifier = LUC_RXPKT_CLASS_FULL;
		}
		else
			pLucRxPkt->lucrx_classifier =  LUC_RXPKT_CLASS_IPNOPROT;
			
		break;
		
	case 0x806:				/* ARP */
		pLucRxPkt->lucrx_l3Offset = 14;
		pLucRxPkt->lucrx_classifier = LUC_RXPKT_CLASS_ARP;
		pLucRxPkt->lucrx_l4Offset = 0;
		pLucRxPkt->lucrx_l5Offset = 0;
		break;
	
	default:
		pLucRxPkt->lucrx_classifier = LUC_RXPKT_CLASS_NOPROT;
		pLucRxPkt->lucrx_l3Offset = 0;
		pLucRxPkt->lucrx_l4Offset = 0;
		pLucRxPkt->lucrx_l5Offset = 0;
		break;

	}
	
	return;
}

#endif /* 0 */

