/*
 *-----------------------------------------------------------------
 *
 * Name:	malloc-ui.c
 *
 * RCS:  $Id: malloc-ui.c,v 1.18 2002/11/05 00:24:10 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	UI functions for malloc module
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/4/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/malloc-api.h>
#include <exports/timer-api.h>
#include "malloc.h"

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showStats()
{
	uint32	i;

    if(mmBase.stackSize > 0)
    {
        printf("\n");
        printf("Malloc Stacks:\n");
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers           = %d\n",
                   mm_stack_buf_size[i], mm_stack_num_bufs[i]);
        }
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers avail     = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].avail);
        }
        for (i=0; i<MM_NUM_STACK_SIZES; i++) {
            printf("   num %4d byte buffers allocated = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].allocated);
            /*
             * freed = freed - num_bufs, since at init time we FREE all bufs to
             * add to stack
             */
            printf("   num %4d byte buffers freed     = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].freed - mm_stack_num_bufs[i]);
            printf("   num %4d byte buffers unavail   = %d\n",
                   mm_stack_buf_size[i], mm_stack[i].unavail);
        }
    }
    
	printf("\n");

	printf("Malloc Lists:\n");
	printf("   Number of %4d byte buffers = %d\n", MM_BUFSIZE, mm_num_bufs);
	printf("   Number of free buffers      = %d\n", mm_num_free);
	printf("   Largest contiguous buffer   = %d\n", mm_max_contig);
	printf("\n");

	printf("Malloc Errors:\n");
	printf("   malloc request too big  = %d\n", mm_stats.too_big);
	printf("   no contiguous buffer    = %d\n", mm_stats.no_contig);
	printf("   not enough free buffers = %d\n",
											mm_stats.not_enough_free_buffers);
	printf("\n");

#if defined(SSC_MGMT)
	printf("Free space avail is %d bytes (of %d total)\n",
		eee_ramTotalHeapAvail(), eee_ramTotalHeapSize());
#else
	printf("Free space avail is %lld bytes (of %lld total)\n",
		eee_ramTotalHeapAvail(), eee_ramTotalHeapSize());
#endif
	printf("\n");

}	/* malloc_showStats() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showFreeList()
{
	mm_free_node_t	* ptr;

	printf("free list head node at %p\n", &mm_free_list);
	printf("    Snext = %p\n", mm_free_list.Snext);
	printf("    Sprev = %p\n", mm_free_list.Sprev);
	printf("    Anext = %p\n", mm_free_list.Anext);
	printf("    Aprev = %p\n", mm_free_list.Aprev);

	printf("\nFREE SIZE LIST:\n");
	ptr = mm_free_list.Snext;
	while (ptr != &(mm_free_list)) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        Snext = %p\n", ptr->Snext);
		printf("        Sprev = %p\n", ptr->Sprev);
		ptr = ptr->Snext;
	}

	printf("\nFREE ADDR LIST:\n");
	ptr = mm_free_list.Anext;
	while (ptr != &(mm_free_list)) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        Anext = %p\n", ptr->Anext);
		printf("        Aprev = %p\n", ptr->Aprev);
		ptr = ptr->Anext;
	}

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocList()
{
	mm_alloc_node_t	* ptr;

	printf("alloc list head node at %p\n", &mm_alloc_list);
	printf("    next = %p\n", mm_alloc_list.next);

	printf("\nALLOC LIST:\n");
	ptr = mm_alloc_list.next;
	while (ptr) {
		printf("    Node: %p\n", ptr);
		printf("        size  = 0x%x\n", ptr->size);
		printf("        bdesc = %p\n", ptr->addr);
		printf("        next  = %p\n", ptr->next);
		ptr = ptr->next;
	}

	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showFreeNodeStack()
{
	uint32			i;
	mm_free_node_t	** ptr;

	printf("Num free nodes available = %d\n", mm_free_node_stack.avail);
	ptr = mm_free_node_stack.top;
	for (i=0; i<mm_free_node_stack.avail; i++, ptr++)
		printf("    %p\n", *ptr);
	printf("\n");
}

/*-----------------------------------------------------------------

 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocNodeStack()
{
	uint32			i;
	mm_alloc_node_t	** ptr;

	printf("Num alloc nodes available = %d\n", mm_alloc_node_stack.avail);
	ptr = mm_alloc_node_stack.top;
	for (i=0; i<mm_alloc_node_stack.avail; i++, ptr++)
		printf("    %p\n", *ptr);
	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
malloc_showAllocRanges()
{
	uint32	i, j, inc;

	i = 0;
	j = 0;

	for (inc = 0x20 ; j < 0x200; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x40 ; j < 0x400; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x80 ; j < 0x1000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x100 ; j < 0x2000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x100 ; j < 0x4000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x400 ; j < 0x8000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x800 ; j < 0x10000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	for (inc = 0x4000 ; j < 0x80000; j += inc, i++)
		if (mm_stats.malloc_size[i] != 0)
			printf("%6d - %-6d : %d\n", j, j+inc-1, mm_stats.malloc_size[i]);

	if (mm_stats.malloc_size[i] != 0)
		printf("       > 512K   : %d\n", mm_stats.malloc_size[i]);
}


void
malloc_clearHistogram()
{
#ifdef MEM_HISTOGRAM
	bzero(mem_histogram, sizeof(mem_histogram));
#endif  /* MEM_HISTOGRAM */
	bzero(mem_histogram_cont, sizeof(mem_histogram_cont));
}


/*
 * Print the histogram info
 *
 * with
 * mode:  boolean: 0: regular, sort by size, 1: sort by usage (see type)
 * output: boolean: 0: normal, 1: short notation
 * type:  int:     0: max usage, 1: curr usage, 2: total usage 
 * nbr:   if mode == 0 print only if value of given 'type' is bigger than 'nbr'
 * nbr:   if mode == 1 print only first 'nbr' of lines
 */

void
malloc_showHistogram (int mode, int output, int type, int nbr)
{
#define MAX_TOP_HISTOGRAM 100
	int32 loop, loop_1, loop_2 ;
	int32 linecnt = 0 ;
	int32 max_top_histogram [MAX_TOP_HISTOGRAM] ;
    mm_flex_desc_t *mmFlex = mmBase.flex;
    mm_flex_t      *flex;
    uint64          freeFlex = 0;
    uint64          usedFlex = 0;
    uint64          totalFlex = 0;

#ifdef MEM_HISTOGRAM_BUCKET
    int32 i, j;
    eeeGMT_t now;
#endif

    if (mmBase.flexSize > 0)
    {
        flex = mmFlex->start;

        if ((type < 0) || (type > 2))
        {
            type = MM_HIST_SORT;
        }

        if (mode)
        {
            /* sort by 'type' */
            /* find top nbr, min 5, max MAX_TOP_HISTOGRAM - 1 */
            if (nbr < 5)
                nbr = 5;
            if (nbr > (MAX_TOP_HISTOGRAM - 2))
                nbr = MAX_TOP_HISTOGRAM - 2;

            bzero(max_top_histogram, sizeof(max_top_histogram));

            for (loop = 0 ; loop < (MM_FLEX_MAX / MM_FLEX_MODULO); loop++)
            {
                freeFlex += flex[loop].free * (loop + 1) * MM_FLEX_MODULO;
                usedFlex += flex[loop].histogram[MM_HIST_CUR] * (loop + 1) * MM_FLEX_MODULO;
            
                if (flex[loop].histogram[type] != 0)
                {
                    loop_1 = nbr ;
                    while (loop_1 >= 0 && 
                           flex[max_top_histogram [loop_1]].histogram[type] < flex[loop].histogram[type])
                    {
                        loop_1-- ;
                    }
                    loop_1++;
                    for (loop_2 = nbr ; loop_2 > loop_1 ; loop_2--)
                    {
                        max_top_histogram [loop_2] = max_top_histogram [loop_2-1]; 
                    }
                    max_top_histogram [loop_1] = loop ;
                }
            }

            for (loop = 0 ; loop < nbr; loop++)
            {
                if (output)
                    printf ("%5d,  %8lld,  %8lld,  %8lld\n", 
                            max_top_histogram [loop], 
                            flex[max_top_histogram [loop]].histogram[MM_HIST_MAX],
                            flex[max_top_histogram [loop]].histogram[MM_HIST_CUR],
                            flex[max_top_histogram [loop]].histogram[MM_HIST_TOT]);
                else
                    printf ("Nbr of buf of %5d (+%3d) bytes : max: %8lld, current: %8lld, total: %lld\n", 
                            max_top_histogram [loop], MM_FLEX_MODULO-1, 
                            flex[max_top_histogram [loop]].histogram[MM_HIST_MAX],
                            flex[max_top_histogram [loop]].histogram[MM_HIST_CUR],
                            flex[max_top_histogram [loop]].histogram[MM_HIST_TOT]);
				
                ++linecnt ;
            }
        } 
        else 
        { 
            for (loop = 0 ; loop < (MM_FLEX_MAX / MM_FLEX_MODULO); loop++)
            {
                freeFlex += flex[loop].free * (loop + 1) * MM_FLEX_MODULO;
                usedFlex += flex[loop].histogram[MM_HIST_CUR] * (loop + 1) * MM_FLEX_MODULO;

                if (flex[loop].histogram[type] > nbr)
                {
                    if (output)
                    {
                        printf ("%5d,  %8lld,  %8lld,  %8lld\n", 
                                loop * MM_FLEX_MODULO, 
                                flex[loop].histogram[MM_HIST_MAX],
                                flex[loop].histogram[MM_HIST_CUR],
                                flex[loop].histogram[MM_HIST_TOT]);
                    } 
                    else 
                    {
                        if (MM_FLEX_MODULO == 1)
                            printf ("Nbr of buf of %5d bytes : max: %8lld, current: %8lld, total: %lld\n", 
                                    loop * MM_FLEX_MODULO,
                                    flex[loop].histogram[MM_HIST_MAX],
                                    flex[loop].histogram[MM_HIST_CUR],
                                    flex[loop].histogram[MM_HIST_TOT]);
                        else
                            printf ("Nbr of buf of %5d - %5d bytes : max: %8lld, current: %8lld, total: %lld\n", 
                                    loop * MM_FLEX_MODULO, (loop + 1) * MM_FLEX_MODULO - 1, 
                                    flex[loop].histogram[MM_HIST_MAX],
                                    flex[loop].histogram[MM_HIST_CUR],
                                    flex[loop].histogram[MM_HIST_TOT]);
                    }
                    ++linecnt ;
                }
            }
        }
        totalFlex = usedFlex + freeFlex + (((address_t)mmFlex->poolEnd - (address_t)mmFlex->poolStart));
    
        printf ("\n");
        printf ("Flexs buffers:      used: %10lld  free list: %10lld  free pool: %10lld\n", 
                usedFlex, freeFlex,
				(uint64)((uintptr_t)mmFlex->poolEnd - (uintptr_t)mmFlex->poolStart));
        printf ("No flex mem (using cont.) %10d\n", mm_stats.no_flex);
        printf ("\n");

    }
    
#ifdef MEM_HISTOGRAM
    if ((type < 0) || (type > 2))
    {
        type = MM_HIST_SORT;
    }

    if (mode)
    {
        /* sort by 'type' */
        /* find top nbr, min 5, max MAX_TOP_HISTOGRAM - 1 */
        if (nbr < 5)
            nbr = 5;
        if (nbr > (MAX_TOP_HISTOGRAM - 2))
            nbr = MAX_TOP_HISTOGRAM - 2;

        bzero(max_top_histogram, sizeof(max_top_histogram));

		for (loop = 0 ; loop < HISTOGRAM_MAX; loop++)
        {
            if (mem_histogram [type][loop] != 0)
            {
                loop_1 = nbr ;
				while (loop_1 >= 0 && 
                       mem_histogram [type][max_top_histogram [loop_1]] < mem_histogram [type][loop])
                {
					loop_1-- ;
                }
				loop_1++;
				for (loop_2 = nbr ; loop_2 > loop_1 ; loop_2--)
                {
					max_top_histogram [loop_2] = max_top_histogram [loop_2-1]; 
                }
				max_top_histogram [loop_1] = loop ;
            }
        }

		for (loop = 0 ; loop < nbr; loop++)
        {
			if (output)
                printf ("%5d,  %8d,  %8d,  %8d\n", 
                        max_top_histogram [loop], 
                        mem_histogram [MM_HIST_MAX][max_top_histogram [loop]],
                        mem_histogram [MM_HIST_CUR][max_top_histogram [loop]],
                        mem_histogram [MM_HIST_TOT][max_top_histogram [loop]]) ;
			else
                printf ("Nbr of buf of %5d (+%3d) bytes : max: %8d, current: %8d, total: %d\n", 
                        max_top_histogram [loop], HISTOGRAM_MODULO-1, 
                        mem_histogram [MM_HIST_MAX][max_top_histogram [loop]],
                        mem_histogram [MM_HIST_CUR][max_top_histogram [loop]],
                        mem_histogram [MM_HIST_TOT][max_top_histogram [loop]]) ;
				
			++linecnt ;
        }
    } 
	else 
    { 
		for (loop = 0 ; loop < HISTOGRAM_MAX; loop++)
        {
			if (mem_histogram [type][loop] > nbr)
            {
				if (output)
                {
					printf ("%5d,  %8d,  %8d,  %8d\n", 
                            loop * HISTOGRAM_MODULO, 
                            mem_histogram [MM_HIST_MAX][loop],
                            mem_histogram [MM_HIST_CUR][loop],
                            mem_histogram [MM_HIST_TOT][loop]) ;
                } 
                else 
                {
					if (HISTOGRAM_MODULO == 1)
                        printf ("Nbr of buf of %5d bytes : max: %8d, current: %8d, total: %d\n", 
                                loop * HISTOGRAM_MODULO,
                                mem_histogram [MM_HIST_MAX][loop],
                                mem_histogram [MM_HIST_CUR][loop],
                                mem_histogram [MM_HIST_TOT][loop]) ;
					else
                        printf ("Nbr of buf of %5d - %5d bytes : max: %8d, current: %8d, total: %d\n", 
                                loop * HISTOGRAM_MODULO, (loop + 1) * HISTOGRAM_MODULO - 1, 
                                mem_histogram [MM_HIST_MAX][loop],
                                mem_histogram [MM_HIST_CUR][loop],
                                mem_histogram [MM_HIST_TOT][loop]) ;
                                }
				++linecnt ;
            }
        }
    }

#endif  /* MEM_HISTOGRAM */

    if (output)
        printf ("cont     %8d,  %8d,  %8d\n", 
                mem_histogram_cont [MM_HIST_MAX],
                mem_histogram_cont [MM_HIST_CUR],
                mem_histogram_cont [MM_HIST_TOT]) ;
    else
        printf ("From continuous memory            : max: %8d, current: %8d, total: %d\n", 
                mem_histogram_cont [MM_HIST_MAX],
                mem_histogram_cont [MM_HIST_CUR],
                mem_histogram_cont [MM_HIST_TOT]) ;
    
    ++linecnt ;

    printf ("\n");
    printf ("Flexible pool buffers:   used %10lld - subtotal %10lld\n", 
            mm_stats.total_flex_used,
            mm_stats.total_flex_size);
    printf ("Fixed size buffers:      used %10lld - subtotal %10lld\n", 
            mm_stats.total_stack_used,
            mm_stats.total_stack_size);
    printf ("Continous mem:           used %10lld - subtotal %10lld\n", 
            mm_stats.total_cont_used,
            mm_stats.total_cont_size);

    printf ("\n");
#if defined(SSC_MGMT)
    printf ("Total memory:            used %10d - total    %10d  (%d%% in use)\n", 
#else
    printf ("Total memory:            used %10lld - total    %10lld  (%lld%% in use)\n", 
#endif
            eee_ramTotalHeapSize() - eee_ramTotalHeapAvail(),
            eee_ramTotalHeapSize(),
            (eee_ramTotalHeapSize() - eee_ramTotalHeapAvail()) * 100 / eee_ramTotalHeapSize());
    printf ("\n");

#ifdef MEM_HISTOGRAM_BUCKET
    eee_TimeGMT (&now);
    
    printf("Function      ");
    for (i=0; i < (MEM_HIST_NBR_BUCKETS - 1); i++) 
    {
        printf("> %6dus  ", memHistogramBucketsSize[i]);
    }
    printf("\n              ");
    for (i=1; i < (MEM_HIST_NBR_BUCKETS - 1); i++) 
    {
        printf("< %6dus  ", memHistogramBucketsSize[i]);
    }
    printf("\n");

    printf("FLEX ALLOC    ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_FLEX_ALLOC][j]);
    }
    printf("\n");

    printf("FLEX DEALLOC  ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_FLEX_DEALLOC][j]);
    }
    printf("\n");

    printf("STACK ALLOC   ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_STACK_ALLOC][j]);
    }
    printf("\n");

    printf("STACK DEALLOC ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_STACK_DEALLOC][j]);
    }
    printf("\n");

    printf("CONT  ALLOC   ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_CONT_ALLOC][j]);
    }
    printf("\n");

    printf("CONT  DEALLOC ");
    for (j=1; j<MEM_HIST_NBR_BUCKETS; j++) 
    {
        printf("%10lld  ", memHistogramBuckets[MEM_HIST_BUCKET_TYPE_CONT_DEALLOC][j]);
    }
    printf("\n");

    printf ("Boot time       %s (%d day, %d hour, %d min, %d sec\n", 
            eee_humandate(bootGMTTime.sec),
            ((now.sec - bootGMTTime.sec) / SEC_PER_DAY),
            (((now.sec - bootGMTTime.sec) % SEC_PER_DAY) / SEC_PER_HOUR),
            (((now.sec - bootGMTTime.sec) % SEC_PER_HOUR) / SEC_PER_MIN),
            ((now.sec - bootGMTTime.sec) % SEC_PER_MIN));
/*
    if ((eee.pfunc.clearTime.sec == 0) &&
        (eee.pfunc.clearTime.usec == 0))
    {
        eee.pfunc.clearTime.sec = bootGMTTime.sec ;
        eee.pfunc.clearTime.usec = bootGMTTime.usec ;
    }

    printf ("Clear counters  %s (%d day, %d hour, %d min, %d sec)\n", 
            eee_humandate(clearTime.sec),
            ((now.sec - clearTime.sec) / SEC_PER_DAY),
            (((now.sec - clearTime.sec) % SEC_PER_DAY) / SEC_PER_HOUR),
            (((now.sec - clearTime.sec) % SEC_PER_HOUR) / SEC_PER_MIN),
            ((now.sec - clearTime.sec) % SEC_PER_MIN));

    if (clear)
    {
        bzero(memHistogramBuckets, sizeof(memHistogramBuckets));
        // eee_TimeGMT (&clearTime);
    }
*/

#else
    // printf("Polling buckets not available\n");
#endif


}


typedef struct _malloc_owner {
	void	* ra;
	uint32	len;
} mm_owner_t;

#define MM_OWNER_MAX	0x2000
mm_owner_t	mm_owner[MM_OWNER_MAX];
uint32		mm_num_ra;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	returns index of RA in mm_owner[] or MM_OWNER_MAX if the RA
 *				is not presently in array
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int32
malloc_getOwnerIndex(
	void	* ra
)
{
	uint32	i;

	for (i=0; i< mm_num_ra; i++) {
		if (mm_owner[i].ra == ra)
			return i;
	}

	return MM_OWNER_MAX;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	returns index into mm_owner[] of new RA, or -1 if array is full
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int32
malloc_addNewOwner(
	void	* ra
)
{
	if (mm_num_ra == MM_OWNER_MAX)
		return -1;

	mm_owner[mm_num_ra].ra = ra;
	mm_owner[mm_num_ra].len = 0;

	mm_num_ra++;

	return (mm_num_ra-1);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	Increments total length allocated by the RA associated with
 *				the given index
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_incOwnerLen(
	uint32	index,
	uint32	len
)
{
	mm_owner[index].len += len;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_addOwner(
	void	* ra,
	uint32	len
)
{
	int32	oi;

	if ((oi = malloc_getOwnerIndex(ra)) == MM_OWNER_MAX) {
		if ((oi = malloc_addNewOwner(ra)) == -1) {
			printf("Could not add new malloc owner, ra = %p, len = 0x%x\n",
																ra, len);
			return;
		}
	}
	malloc_incOwnerLen(oi, len);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int
mm_owner_cmp(
	mm_owner_t	* p1,
	mm_owner_t	* p2
)
{
	if (p1->len < p2->len)
		return -1;
	if (p1->len == p2->len)
		return 0;
	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
malloc_showOwner()
{
	mm_alloc_node_t	* anp;
	uint32			i;

#ifndef MM_VALIDATE
	printf("owner information not available if compiled without MM_VALIDATE\n");
	return;
#endif

    if (mmBase.flexSize > 0)
    {
        mm_flex_desc_t  *mmFlex = mmBase.flex;
        mm_flex_t       *flex;
        mm_flex_head    *head;
#ifdef MM_USE_FLEX_TAIL
        mm_flex_tail    *tail;
#endif
        void		    *ra;
        uint32		     len;
        uint32           nbr = 0 ;

        flex = mmFlex->start;

        bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
        mm_num_ra = 0;

        NFX_SPIN_LOCK(&mmBase.flexSpin);

        for (i = 0 ; i < (MM_FLEX_MAX / MM_FLEX_MODULO); i++)
        {
            nbr = 0;
            LIST_FOREACH(head, &(flex[i].useList), entry)
                {
                    ASSERT(head->free_ra == 0);
                    ASSERT(head->malloc_ra);
#ifdef MM_USE_FLEX_TAIL
                    tail = (mm_flex_tail*)(((uchar8*)(head + 1)) + head->length);
                    ASSERT(tail->sig == MM_TAIL_SIG);
                    ASSERT(head->sig == MM_HEAD_SIG);
#endif
                    nbr++;

                    ra = head->malloc_ra;
                    len = head->size;       // the real num_bytes of malloc req
                    // len = head->length;     // the size aligned to the next 64 byte boundary
                    // printf("ra = 0x%x, len = %d\n", ra, len);
                    malloc_addOwner(ra, len);
                }
            ASSERT(nbr == flex[i].histogram[MM_HIST_CUR]);
        }
        
        NFX_SPIN_UNLOCK(&mmBase.flexSpin);

        // printf("calling qsort\n");
        qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
        // printf("back from qsort\n");

        printf("\nOwner list in flex memory\n\n");
        for (i=0; i<mm_num_ra; i++)
            printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
    }
    
    if(mmBase.stackSize > 0)
    {
        bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
        mm_num_ra = 0;

        NFX_SPIN_LOCK(&mmBase.stackSpin);

        //printf("Stacks:\n");
        for (i=0; i<MM_NUM_STACK_SIZES; i++) 
        {
            uchar8		* bdesc;
            mm_head_t	* head;
            void		* ra;
            uint32		len;

            //printf("    0x%x bytes:\n", mm_stack_buf_size[i]);
            for (bdesc = mm_stack_buf_base[i]; bdesc < mm_stack_buf_end[i];
                 bdesc += mm_stack_bdesc_size[i]) 
            {
                head = (mm_head_t *)bdesc;
                if ((head->free_ra == 0) && (head->malloc_ra)) 
                {
                    ra = head->malloc_ra;
                    len = head->length;
                    // printf("ra = 0x%x, len = %d\n", ra, len);
                    malloc_addOwner(ra, len);
                }
            }
        }

        NFX_SPIN_UNLOCK(&mmBase.stackSpin);

        // printf("calling qsort\n");
        qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
        // printf("back from qsort\n");

        printf("\nOwner list in fixed size stacks memory\n\n");
        for (i=0; i<mm_num_ra; i++)
            printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
	}

	bzero(mm_owner, MM_OWNER_MAX * sizeof(mm_owner_t));
    mm_num_ra = 0;

    NFX_SPIN_LOCK(&mmBase.contSpin);

	//printf("List:\n");
	anp = mm_alloc_list.next;
	while (anp) {
		mm_bdesc_t	* bdesc;
		void		* ra;
		uint32		len;

		bdesc = anp->addr;
		ra = bdesc->head.malloc_ra;
		len = bdesc->head.length;
		//printf("ra = 0x%x, len = %d\n", ra, len);
		malloc_addOwner(ra, len);
		anp = anp->next;
	}

    NFX_SPIN_UNLOCK(&mmBase.contSpin);

//  for (i=0; i<mm_num_ra; i++)
//		printf("RA : 0x%x, len = %d\n", mm_owner[i].ra, mm_owner[i].len);

//	printf("calling qsort\n");
	qsort(mm_owner, mm_num_ra, sizeof(mm_owner_t), mm_owner_cmp);
//	printf("back from qsort\n");

	printf("\nOwner list in continious memory\n\n");
	for (i=0; i<mm_num_ra; i++)
		printf("RA : %p, len = %d\n", mm_owner[i].ra, mm_owner[i].len);
}







