/*
 *-----------------------------------------------------------------
 *
 * Name:	mdk-init.c
 *
 * RCS:  $Id: mdk-init.c,v 1.8 2002/08/28 22:17:52 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <nfx-defs.h>
#include <exports/mdk-api.h>
#if defined (DIAG_R7000) || defined (DIAG_SIBYTE) 
#include <exports/eee-desc.h>

void mdk_rxAppPkt(eee_desc_t *);
extern int dp83820_initModule();
#endif


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
mdk_t *
mdk_init()
{

	int32 rc;

	/* Initialise app needed by diag. */
	mdk_appId = eee_allocateApplicationId("mdk");
	if( mdk_appId == NFX_ERR )
	{
		printf("[mdk_initModule] Error Allocate app id failed.");
		return NFX_ERR;
	}	
	rc = eee_registerReceiveFunc(	mdk_appId,
					mdk_rxAppPkt, /* Ucast Rx Func */
					NULL );

	if (rc == NFX_ERR)
	{
		printf("[mdk_initModule] Error: registerReceiverFunc\n");
		return NFX_ERR;
	}	
#ifdef DIAG_R7000
	/* This is only initialsed in runtime for FC */
	/*
	if(isFC())
		dp83820_initModule();
	*/
#endif	

	/* Make Banner */
	mdk.title = "Agile Storage Ultra Cool Diagnostic Kernel";
	mdk.date = __DATE__;
	mdk.time = __TIME__;
	mdk.version = "1.0";
	if(isSSC())
		mdk.card_type = "SSC";
	else if(isFC())
		mdk.card_type = "FC";
	else if(isTXRX())
		mdk.card_type = "TXRX";
	else if(isFP())
		mdk.card_type = "FP";

	mdk_package_list = &(mdk.pkg[0]);

	watchdog_disable();
	return &mdk;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
void
mdk_init_packages()
{
	extern mdk_package_t	diag_mem_pkg;
	extern mdk_package_t	diag_pci_pkg;
#if defined (DIAG_R7000)
	extern mdk_package_t	diag_gt_pkg;
	/* extern mdk_package_t	diag_rtc_pkg; */
	extern mdk_package_t	diag_bcm5600_pkg;
	extern mdk_package_t	diag_serdes_pkg;
#elif defined (DIAG_SIBYTE)
	extern mdk_package_t	diag_luc_pkg;
	extern mdk_package_t	diag_lsi8104_pkg;
	extern mdk_package_t	diag_sb_pkg;
#endif	
	extern mdk_package_t	diag_gmac_pkg;
	extern mdk_package_t	diag_serdes_pkg;
	extern mdk_package_t	diag_conflict_pkg;

	uint32	test_num;

	test_num = 0;
	mdkp->pkg[test_num++] = &diag_mem_pkg;
	mdkp->pkg[test_num++] = &diag_pci_pkg;
#if defined (DIAG_R7000)	
	mdkp->pkg[test_num++] = &diag_gt_pkg;
	/* mdkp->pkg[test_num++] = &diag_rtc_pkg; */
	mdkp->pkg[test_num++] = &diag_bcm5600_pkg;
#elif defined (DIAG_SIBYTE)	
	mdkp->pkg[test_num++] = &diag_luc_pkg;
	mdkp->pkg[test_num++] = &diag_lsi8104_pkg;
	mdkp->pkg[test_num++] = &diag_sb_pkg;
#endif	
	mdkp->pkg[test_num++] = &diag_gmac_pkg;
	mdkp->pkg[test_num++] = &diag_serdes_pkg;
	mdkp->pkg[test_num++] = &diag_conflict_pkg;

	mdkp->num_pkgs = test_num;

	pg_max = test_num/MDK_PACKAGES_PER_PAGE;
	mdkp->num_loops = 0;
}


#ifdef PROM
/*-----------------------------------------------------------------
 * Name: mdk_rxAppPkt
 *
 * Description: receives packets sent to MDK App.
 *
 * Created by:	lipika	
 *
 * Date Created:	2/25/02
 *
 *-----------------------------------------------------------------
 */
void
mdk_rxAppPkt(eee_desc_t *edesc )
{
	/* Need to analyse packets b4 calling correct rcv fn */
	eee_descBuf_t   bd;
	eee_descHdr_t hdr;
	uchar8	*buf;
	
	buf = edesc->bd[1].buf;
	hdr = edesc->hdr;
	/*
	printf("mdk_rxAppPkt edesc %08x buf %08x\n", edesc, buf);
	printf("%08x %08x %08x %08x\n",hdr.control, hdr.dest_port, hdr.src_port, hdr.attr);
	printf("%02x %02x %02x %02x\n",buf[0], buf[1], buf[2], buf[3]);
	*/
#if defined (DIAG_SIBYTE)	
	if(buf[0] == 'm' && buf[1] == 'd' && buf[2] == 'k')
	{		
		mdk_bmc12500Eth_diagLbPktReceive(edesc);
		return;
	}
	else 
#elif defined (DIAG_R7000)	
	if(buf[0] == 'b' && buf[1] == 'c' && buf[2] == 'm' )
	{		
		mdk_gt_mii_diagLbPktReceive(edesc);
		return;
	}
	else 
#endif	
	if(buf[1] == 'd' && buf[2] == 'p' && buf[3] == '8')
	{		
		mdk_dp83820_diagLbPktReceive(edesc);
		return;
	}
	else
	{
		//printf("ERROR: rxAppPkt: Pkt rcvd has unexpected data edesc %08x\n",edesc);	
		eee_freePacket(edesc);
	}	
	return;
}

#endif
