
 /*-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-conflict.c,v 1.7 2002/07/27 01:03:16 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#include <exports/eee-desc.h>
#include "mdk-dp83820.h"

/* Global Variables */
#if defined (DIAG_SIBYTE)
uint32 sb_conflict;
#elif defined (DIAG_R7000)
uint32 fc_conflict;
#endif

extern mdk_loopback_t mdk_testState[6];
extern uint32	time_handle_dp83820;
extern uint32	time_handle_port0;
extern uint32	time_handle_port1;
extern uint32	time_handle_port2;
extern uint32	time_handle_port3;
extern uint32	time_handle_port4;
extern uint32	time_handle_port5;

uint32 conflict_timeout;
uint32	time_handle_conflict;
#define SKIP_TEST 1

int32 mdk_conflict_timeout(int32 tmr_handle, void * usr_param0, void* usr_param1, void* usr_param2,void* usr_param3);


int32
conflict_pkg_init()
{
	
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
	if(tp->param[TP_LB_DURATION] == 0)
		tp->param[TP_LB_DURATION] = 5;
	if(!time_handle_dp83820)
		time_handle_dp83820 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#ifdef DIAG_SIBYTE
	if(!time_handle_port0)
		time_handle_port0 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#endif
	if(!time_handle_port1)
		time_handle_port1 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#ifdef DIAG_R7000
	if(!time_handle_port2)
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#endif
#ifdef DIAG_SIBYTE
	if(!time_handle_port2)
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port3)
		time_handle_port3 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port4)
		time_handle_port4 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port5)
		time_handle_port5 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#endif
}

/* MAIN TEST FUNCTIONS */

int32
sb_conflict_test(
	mdk_test_params_t       * tp
)
{
#ifdef DIAG_SIBYTE	
	uint32 rct;
	uint32 i, maxPort;
	uint32 dur;

	/* This test sets the external loopback at TXRX PHY, 
	 * and BC5208 PHY connected to National GMAC
	 */ 

	if(time_handle_conflict)
		eee_timerDestroy(time_handle_conflict);

	dur = tp->param[TP_LB_DURATION] * 1000;
	if(dur > 0)
		time_handle_conflict = eee_timerCreate(dur, EEE_TIMER_ONCE);
	if(time_handle_conflict == TIMER_ERR_NO_TIMER_AVAILABLE)
		printf("Conflict test: No timer available\n");

	conflict_timeout = 0;
	sb_conflict = 1;	

	if(isCheetah()) {
		if(isTXRX()) 
			maxPort = 4;
		else if(isFP())	
			maxPort = 6;
	}
	else 
		maxPort = BC1250_MAC_CNT;

	for(i=0;i<maxPort;i++) {
			sb_ext_lb_test_port(i, tp); 
	}

	rct = eee_timerStart(time_handle_conflict, mdk_conflict_timeout,0,0,0,0);

	if(rct != NFX_OK)
		printf("Conflict Test: Could not start timer. Error %08x \n",rct);

	if(isTXRX())
		gmac_extlb_test(tp);

	tp->resultCnt = maxPort;
	if(isTXRX())
		tp->resultCnt += 1; /* maxPort + GMAC */
	tp->testresult =  RET_SKIPPED;	
#endif	
	return(tp->testresult);
}

int32
fc_conflict_test(
	mdk_test_params_t       * tp
)
{
#ifdef DIAG_R7000
	uint32 rct;
	uint32 dur;

	/* This test sets the external loopback from all 
	 * RMII ports of the Galielo and the ports connected to National GMAC
	 */ 
	if(time_handle_conflict)
		eee_timerDestroy(time_handle_conflict);

	dur = tp->param[TP_LB_DURATION] * 1000;
	if(dur > 0)
		time_handle_conflict = eee_timerCreate(dur, EEE_TIMER_ONCE);

	if(time_handle_conflict == TIMER_ERR_NO_TIMER_AVAILABLE)
		printf("Conflict test: No timer available\n");

	conflict_timeout = 0;
	fc_conflict = 1;	
	gt_5228_ext_lb(tp); 
	gt_5600_ext_lb2(tp); 
	gt_gmac_extlb_test(tp);

	rct = eee_timerStart(time_handle_conflict, mdk_conflict_timeout,0,0,0,0);
	if(rct != NFX_OK)
		printf("Conflict Test: Could not start timer(%08x). Error %08x \n",time_handle_conflict,rct);

	tp->resultCnt = 3;
	tp->result[0].testresult = RET_SKIPPED;
	tp->result[1].testresult = RET_SKIPPED;
	tp->result[2].testresult = RET_SKIPPED;
	tp->testresult =  RET_SKIPPED;	
#endif	
	return(tp->testresult);
}


int32
conflict_pkg_err(
	mdk_log_entry_t	* err
)
{

	switch (err->testresult) {
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("%s: Conflict test TIMEOUT\n", err->testName);
			break;
		case ERR_PKTDROP:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		case ERR_LPBKDATA:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nData corruption.\n");
			break;
		default:
			printf("Unknown error, testresult = 0x%x\n", err->testresult);
			break;
	}

	return 0;
}


int32
mdk_conflict_timeout(int32 tmr_handle,
		 void * usr_param0,
		 void * usr_param1,
		 void * usr_param2,
		 void * usr_param3)
{

	uint32 i;

	if(tmr_handle == time_handle_conflict) 
	{	
		conflict_timeout = 1;

	}	
	return(NFX_OK);
}

