/* vi:ts=4
 * =============================================================================
 * File Name  :mdk_gt_util.c		
 *
 * Description:    
 *
 * Created    :	03-06-2002
 *
 * Author     : lipika	
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 * =============================================================================
 */
#ifdef DIAG_R7000
#include <exports/hwMap.h>
#include <exports/gt64240mii.h>
#include <exports/mdk-api.h>
#include <exports/bcm5600-ll.h>
#include "mdk-mem.h"


mdk_gt64240mii_test_t mdk_gt64240mii_testControl[3];
uint32 mdk_gt_mii_test_port = 0;
extern int32 gt64240mii_txUnicast(uint32, eee_desc_t *);

int32 gt_lpbk_timeout(int32 tmr_handle, void * usr_param0, void* usr_param1, void* usr_param2,void* usr_param3);
extern uint32 mac_lpbk, phy_5228_lpbk, phy_8353_lpbk;
extern uint32 set_phy_5228_lpbk;

char *mdk_gt_miiTestMode[] = {"INT","MAC","PHY","EXT",NULL};
extern uint32 fc_conflict;
extern uint32 conflict_timeout;
extern uint32 port0_added_hash;

extern mdk_loopback_t mdk_testState[6];

extern uint32	time_handle_dp83820;
extern uint32	time_handle_port0;
extern uint32	time_handle_port1;
extern uint32	time_handle_port2;

uint32 prvRxPktCnt[3];
uint32 consecutivePktLoss[3];

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagSetLbMode
 *
 * Description: 	Set loopback mode. Modified for diag.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-06-2001
 * 					
 * Returns: 		NFX_OK 
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagSetLbMode( uint32 port, uint32 mode )
{

   /*
    * Disable Rx and Tx
    */
   gt64240mii->port[port].pcr.wd = gt_mii_readPortReg(port, GT_MII_PCR);
   gt64240mii->port[port].pcr.bits.en = 0;
   gt_mii_writePortReg(port, GT_MII_PCR, gt64240mii->port[port].pcr.wd);
		
   /*
    * Set/Reset LB Mode
    */
   gt64240mii->port[port].pcr.bits.lpbk = mode;
   gt_mii_writePortReg(port, GT_MII_PCR, gt64240mii->port[port].pcr.wd);

   if (mode != GT_MII_LB_NONE)
      gt64240mii->port[port].rxc[0].loopback = 1;
   else
      gt64240mii->port[port].rxc[0].loopback = 0;
	
   /*
    * Enable Rx and Tx
    */
   gt_mii_diagPortEna(port); 

   return(NFX_OK);

} /* mdk_gt_mii_diagSetLbMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLBTest
 *
 * Description: 	Runs loopback test 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagLBTest(uint32 port, uint32 testMode)
{
   int32  i, rc;
   int32  pktCnt=0;
   int32  pktSize;
   uint32 destAddr=0;
   uchar8 *bufptr;
   uchar8 *tmp;
   uint32 rct;
   uint32 t_port=0;

	
	/* Run test for pkt number if not Conflict test */
	if (!fc_conflict)
		tp->param[TP_LB_DURATION] = 0; /* Only used for conflict test*/

   if(tp->param[TP_LBPKT_CNT]) {
		if(testMode == GT_LB_TEST_MAC)
			pktCnt = 1; /* Mac loopback floods pkts!! */
		else
			pktCnt = tp->param[TP_LBPKT_CNT];
   }		
   else
   		pktCnt = 1;

   pktSize = 64;

	if(mdk_gt64240mii_testControl[port].testState == TEST_INPROGRESS)
	{
		if( mdk_gt64240mii_testControl[port].testBuffer) {
			eee_ramDealloc(mdk_gt64240mii_testControl[port].testBuffer);
		}	
	}	

   memset(&mdk_gt64240mii_testControl[port],0,sizeof(mdk_gt64240mii_test_t));
   if((port == 0) || (port == 2)) {
		t_port = 0;
   }	
   else if(port == 1) {
		t_port = port;
	}	
	memset(&mdk_testState[t_port],0,sizeof(mdk_loopback_t));
   
	mdk_gt64240mii_testControl[port].testState   = TEST_INPROGRESS; 
	mdk_testState[t_port].state = TEST_INPROGRESS;
   	mdk_gt64240mii_testControl[port].testMode    = testMode;
   	mdk_gt64240mii_testControl[port].testPktCnt  = pktCnt;
   	mdk_gt64240mii_testControl[port].testPktSize = pktSize;

   /*
    * Allocate and initialize test data.
    */
	bufptr = (uchar8 *)eee_ramAlloc(pktSize);
   if( !bufptr ) {
      printf("Test data buffer allocation failed.\n");
      test_done = 1;
      return(NFX_OK);
   }
   mdk_gt64240mii_testControl[port].testBuffer = bufptr;
   mdk_gt64240mii_testControl[port].fVerify = 1;

   if(port == 0) {
			rct = eee_timerStart(time_handle_port0, gt_lpbk_timeout,0,0,0,0);
			if(rct != NFX_OK)
				printf("GT Loopback(test Start) Port %d : Could not start timer. Error %08x \n",port, rct);
			else
				mdk_gt64240mii_testControl[port].timer = time_handle_port0;
	}
   else if(port == 1) {
			rct = eee_timerStart(time_handle_port1, gt_lpbk_timeout,0,0,0,0);
			if(rct != NFX_OK)
				printf("GT Loopback(test Start) Port %d : Could not start timer %08x. Error %08x \n",port,time_handle_port1, rct);
			else 
				mdk_gt64240mii_testControl[port].timer = time_handle_port1;
   }		
   else if(port == 2) {
			rct = eee_timerStart(time_handle_port2, gt_lpbk_timeout,0,0,0,0);
			if(rct != NFX_OK)
				printf("GT Loopback(test Start) Port %d : Could not start timer %08x. Error %08x \n",port,time_handle_port2, rct);
			else 
				mdk_gt64240mii_testControl[port].timer = time_handle_port2;
   }		

   if(testMode == GT_LB_TEST_INT)
      mdk_gt_mii_diagSetLbMode(port, GT_MII_LB_INTERNAL);

   /* printf("bufptr: %x, pktSize: %d\n", bufptr, pktSize); */

   tmp = bufptr;
   /* Mark the packet as ours */
   *tmp++ = 'b';
   *tmp++ = 'c';
   *tmp++ = 'm';
   *tmp++ = '0' + port;
   *((uint32 *)tmp) = mdk_gt64240mii_testControl[port].txPktCnt;
   tmp+=4;
   for(i=0; i < (pktSize-8); i++ )
      *tmp++ = i & 0xFF;

   	gt_mii_diagPortEna(port); 

#ifdef NFX_MOD_SSC
   /*
    * Send first set of loopback packets.
    */
   destAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
				   						 nfxMyCPUId,
				 						 gt64240mii->appId,
				 						 EEE_USER_DATA,
				 						 NFX_COS_HIGH, 
										 SSC_PORT_BP_MGMT);
#endif // NFX_MOD_SSC
#ifdef PROM
   if(port == 1)
   	destAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
				   						 nfxMyCPUId,
										 mdk_appId,
				 						 EEE_MGMT,
				 						 NFX_COS_HIGH, 
										 PROM_PORT_MGMT);
   else if(port == 0) {
   		eee_registerTransmitFunc( EEE_USER_DATA,
								gt64240mii_txUnicast,
								gt64240mii_txUnicast,
								1500);
   		eee_registerDefaultTransmitFunc( EEE_USER_DATA,
        	                     gt64240mii_txUnicast,
							     1500);

   		destAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
					   						 nfxMyCPUId,
					 						 mdk_appId,
					 						 EEE_USER_DATA,
				 							 NFX_COS_HIGH, 
											 PROM_PORT_MGMT);
	}	
   else if(port == 2) 
	   	destAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
					   						 nfxMyCPUId,
					 						 mdk_appId,
					 						 EEE_CONTROL,
					 						 NFX_COS_HIGH, 
											 PROM_PORT_MGMT);
#endif   

   mdk_gt64240mii_testControl[port].testDestAddr = destAddr;

   for(i=0; i < 1; i++)
   {
      rc = eee_sendMessage(bufptr, pktSize, destAddr, destAddr);
      if (rc != EEE_FORWARDED_PACKET)
         printf("mdk_gt_mii_diagLBTest: error: eee_sendMessage, rc=%0x\n", rc);
   }
   mdk_gt64240mii_testControl[port].txPktCnt += 1;
   mdk_gt64240mii_testControl[port].txByteCnt += (1 * pktSize);
   return(NFX_OK);

} /* gt_mii_diagLBTest */

/*	
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagRemakeLbPkt
 *
 * Description: 	Receives Loop back packet 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagRemakeLbPkt(uint32 port)
{
   int32  pktSize = 64;
   uint32 destAddr=0;
   uchar8 *bufptr;
   uchar8 *tmp;
   uint32 rct, i;

	prvRxPktCnt[port] = mdk_gt64240mii_testControl[port].rxPktCnt;

   /*
    * Allocate and initialize test data.
    */
	if(mdk_gt64240mii_testControl[port].testState == TEST_INPROGRESS)
	{
		if( mdk_gt64240mii_testControl[port].testBuffer) {
			eee_ramDealloc(mdk_gt64240mii_testControl[port].testBuffer);
		}	
	}	
	bufptr = (uchar8 *)eee_ramAlloc(pktSize);
   	if( !bufptr ) {
      printf("Test data buffer allocation failed.\n");
      test_done = 1;
      return(NFX_OK);
   	}
   	if(port == 0) {
		rct = eee_timerStart(time_handle_port0, gt_lpbk_timeout,0,0,0,0);
		if(rct != NFX_OK)
			printf("GT Loopback(Remake test Start) Port %d : Could not start timer. Error %08x \n",port, rct);
		else
			mdk_gt64240mii_testControl[port].timer = time_handle_port0;
	}
   	else if(port == 1) {
		rct = eee_timerStart(time_handle_port1, gt_lpbk_timeout,0,0,0,0);
		if(rct != NFX_OK)
			printf("GT Loopback(Remake test Start) Port %d : Could not start timer %08x. 
							Error %08x \n",port,time_handle_port1, rct);
		else 
			mdk_gt64240mii_testControl[port].timer = time_handle_port1;
   	}		
   	else if(port == 2) {
		rct = eee_timerStart(time_handle_port2, gt_lpbk_timeout,0,0,0,0);
		if(rct != NFX_OK)
			printf("GT Loopback(Remake test Start) Port %d : Could not start timer %08x. 
							Error %08x \n",port,time_handle_port2, rct);
		else 
			mdk_gt64240mii_testControl[port].timer = time_handle_port2;
   	}		

   tmp = bufptr;
   /* Mark the packet as ours */
   *tmp++ = 'b';
   *tmp++ = 'c';
   *tmp++ = 'm';
   *tmp++ = '0' + port;
   *((uint32 *)tmp) = mdk_gt64240mii_testControl[port].txPktCnt;
   tmp+=4;
   for(i=0; i < (pktSize-8); i++ )
      *tmp++ = i & 0xFF;

   destAddr = mdk_gt64240mii_testControl[port].testDestAddr;
   mdk_gt64240mii_testControl[port].testBuffer = bufptr;

   for(i=0; i < 1; i++)
   {
      rct = eee_sendMessage(bufptr, pktSize, destAddr, destAddr);
      if (rct != EEE_FORWARDED_PACKET)
         printf("mdk_gt_mii_diagLBTest: error: eee_sendMessage, rc=%0x\n", rct);
   }
   mdk_gt64240mii_testControl[port].txPktCnt += 1;
   mdk_gt64240mii_testControl[port].txByteCnt += (1 * pktSize);
   return(NFX_OK);
}

/*	
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLbPktReceive
 *
 * Description: 	Receives Loop back packet 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagLbPktReceive( eee_desc_t *edesc )
{
   int32 rc;
   int32 pktLen;
   int32 bufCnt;
   mdk_gt64240mii_test_t *dptc;
	uchar8	*buf;
	uint32 port=0,t_port = 0;
	uint32 msg;
	uint32 seqNo;

	buf = edesc->bd[1].buf;
	seqNo = *((uint32 *)buf+1);
	/*
	printf("%02x %02x %02x %02x %02x %02x %02x %02x\n",buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6], buf[7]);
	*/

	if(buf[0] == 'b' && buf[1] == 'c' && buf[2] == 'm' )
	{		
		/* Pattern matches, check pkt is for which Port */
		if(buf[3] == '0') {
			t_port = port = 0;
		}
		else if(buf[3] == '1') {
			t_port = port = 1;
		}
		else if(buf[3] == '2') {
			/* mdk_testState[0/1] is for GT ports only */
			t_port = 0;
			port = 2;
		}
	}	
	else {
		if (tp->param[TP_VERBOSE] == 1) {
			printf("PKT destined for mdk_bmc12500 has wrong pattern!!\n");
			printf("%02x %02x %02x %02x \n",buf[0], buf[1], buf[2], buf[3]);
		}
		eee_freePacket(edesc);
		return;
	}

   dptc = &mdk_gt64240mii_testControl[port];

	/* If seqNo of pkt is < txPktCnt, it means that pkt is delayed,
	 * so we drop the pkt.
	 */ 
	if(seqNo < (dptc->txPktCnt - 1)) {
		//printf("Rcvd pkt %d but sent pkt %d\n",seqNo, dptc->txPktCnt);	
		eee_freePacket(edesc);
		return;
	}		


	/* Check if packet has come after timeout */
	if(mdk_testState[t_port].timeout)
	{
		if(!fc_conflict)	
			dptc->testState = TEST_END;
	}


   pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
   bufCnt = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);

   while(1) {
      switch( dptc->testState )
      {
			case TEST_INPROGRESS:
				dptc->rxPktCnt++; 
				dptc->rxByteCnt += pktLen;
				if( dptc->fVerify )
				   dptc->testState = TEST_VERIFY;
				else 
				{
					/* If running conflict test, run based on time
					 * NOT no. of pkts
					 */
					if(fc_conflict) {
							if(tp->param[TP_LB_DURATION] > 0) {
								if(!conflict_timeout)
									dptc->testState = TEST_CONTINUE;
								else {
									dptc->testState = TEST_END;
								}
							}
							else {
								   if( dptc->rxPktCnt >= dptc->testPktCnt)
									  dptc->testState = TEST_END;
								   else
									  dptc->testState = TEST_CONTINUE;
							}
					}
					else {
						   if( dptc->rxPktCnt >= dptc->testPktCnt)
							  dptc->testState = TEST_END;
						   else
							  dptc->testState = TEST_CONTINUE;
					}		   
				}
					break;

			case TEST_VERIFY:
				rc = mdk_gt_mii_diagLbVerifyData(edesc->bd[1].buf, 
									 edesc->bd[1].len, 
								 dptc->testBuffer,port);

				
				if(fc_conflict) {
						if(tp->param[TP_LB_DURATION] > 0) {
							if(!conflict_timeout)
								dptc->testState = rc;
							else {
								dptc->testState = TEST_END;
							}
						}
				}		
				else {	
						if( dptc->rxPktCnt >= dptc->testPktCnt)
						   dptc->testState = TEST_END;
						else
						   dptc->testState = rc;
				}		
				break;

			case TEST_CONTINUE:
				dptc->txPktCnt++; 
				dptc->txByteCnt += pktLen;
   				*((uint32 *)buf+1) = dptc->txPktCnt;
				rc = eee_timerStop(dptc->timer) ;
				if(rc != NFX_OK)
					printf("GT Loopback Port %d : Could not STOP timer. Error %08x \n",port, rc);
				edesc->hdr.dest_port = dptc->testDestAddr;
				rc = eee_forwardPacket(edesc);
				if(rc !=  EEE_FORWARDED_PACKET)
					printf("ERROR: Port %d Could not forward pkt.\n",port);	
				rc = eee_timerStart(dptc->timer, gt_lpbk_timeout,0,0,0,0);
				if(rc != NFX_OK)
					printf("GT Loopback Port %d : Could not START timer. Error %08x \n",port, rc);
				gt_mii_diagPortEna(port); 
				dptc->testState = TEST_INPROGRESS;
				if(fc_conflict)
					tp->result[t_port].testresult = RET_PASSED;
				return(NFX_OK);
						
			case TEST_END:
				if(dptc->testBuffer) {
					eee_ramDealloc(dptc->testBuffer);
				}   
				if(dptc->testMode == GT_LB_TEST_INT)
				   mdk_gt_mii_diagSetLbMode(mdk_gt_mii_test_port, GT_MII_LB_NONE);

				if (tp->param[TP_VERBOSE] == 1) 
					mdk_gt_mii_diagShowLbTestResults(dptc, port);
				dptc->testState = TEST_DONE;
				break;

			case TEST_DONE:
				eee_freePacket(edesc);
				if(tp->testresult != ERR_LPBKDATA) {
					//if(dptc->rxPktCnt < dptc->txPktCnt)
					//	tp->testresult = ERR_PKTDROP;
					//else 
					if(mdk_testState[t_port].timeout)
						tp->testresult = ERR_TIMEOUT;	
					else
						tp->testresult = RET_PASSED;
				}
				if(fc_conflict)
					tp->result[t_port].testresult = tp->testresult;
				mdk_testState[t_port].state = TEST_DONE;
				test_done = 1;
   				//if((phy_8353_lpbk) || (mac_lpbk))
				//	gt_reset_loopback(port); /* has to be reset manually for others*/
				eee_timerStop(dptc->timer) ;
				return(NFX_OK);
			  }		
   }
	mdk_testState[t_port].state = dptc->testState;
   return(tp->testresult = RET_PASSED);

} /* gt_mii_diagLbPktReceive */

/*
 *------------------------------------------------------------------------------
 * Function Name: 	gt_reset_loopback	
 *
 * Description: 	Turns off the loopback mode of the driver	
 *
 * Created by:		lipika	
 *
 * Data Created:	Mar 29 2002	
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
gt_reset_loopback(uint32 port)
{				
   /* Need to reset the loopback mode */
   if(phy_8353_lpbk == 1) {
	   uint32 phy_id=0;
	   uint32 tmp;
		if(isFC() && (nfxMyCPUId == 0)) { 
			if(port == 0)
				phy_id = 0xa;
			else if(port == 2)
				/* this does not work */
				/* phy_id = 0xb; */
				phy_id = 0xa;
			gt_mii_writeSMI(phy_id, 0x0, 0x1140);
			phy_8353_lpbk = 0;
   			gt_mii_readSMI(phy_id, 0x0, &tmp); 
   		}
   }
   else if(mac_lpbk == 1) {
	uint32 bcm_port_lb;
	if((isFC()) && (nfxMyCPUId == 0)) {
		if(port == 0)
			bcm_port_lb = 19;
		else if(port == 2)
			bcm_port_lb = 23;
		MAC_LOOPBACK_SET(&mac_fe_5600, 0, bcm_port_lb, 0);
	}		
	mac_lpbk = 0;
   }
   else if((phy_5228_lpbk)||(set_phy_5228_lpbk)) {
	uint32 phy_id=0;
	if((isFC()) && (nfxMyCPUId == 0)) {
			gt_mii_writeSMI(0x10, 0x0, 0x2100);
			delay_us(10000);
			gt_mii_writeSMI(0x11, 0x0, 0x2100);
			delay_us(10000);
			gt_mii_writeSMI(0x12, 0x0, 0x2100);
			delay_us(10000);
			gt_mii_writeSMI(0x13, 0x0, 0x2100);
			delay_us(10000);
	}	
	port = 1; /* for Management Plane */
	phy_5228_lpbk = 0;
   }
	return;	
}


/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagLbVerifyData
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagLbVerifyData( uchar8 *bufptr, int32 len, uchar8 *tbufptr ,uint32 port)
{
   int32  i;
   uchar8 *saveBfptr;

   if( !bufptr || !tbufptr )
      return(NFX_ERR);

#if 0
   uchar8 *tmp;
   tmp = (uchar8 *)bufptr;
   printf("Rx buffer at %0x: ",tmp);
   for (j=0; j < len; j++)
      printf("%0x ", *tmp++);
   printf("\n");
#endif

   //The first 8 bytes contain information. 
   bufptr+=8;
   tbufptr+=8;
   saveBfptr = bufptr;
   for(i=0; i < len-8; i++) {
      if( *bufptr != *tbufptr ) {
		if (tp->param[TP_VERBOSE] == 1) 
			 printf("GT %d: LB Data Error: TestVal=%02x, ActualVal=%02x, Addr=%x\n",
					   port,*tbufptr, *bufptr, bufptr);
			tp->testresult = ERR_LPBKDATA;
			//gt64240_printBuf((char *)saveBfptr, len, 0);
	 return(TEST_END);
      }
      bufptr++;
      tbufptr++;
   }
   return(TEST_CONTINUE);

} /* mdk_gt_mii_diagLbVerifyData */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_diagShowLbTestResults
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_gt_mii_diagShowLbTestResults(mdk_gt64240mii_test_t *dptc,uint32 port)
{
	printf("\nGT64240-MII LOOPBACK TEST RESULTS PORT %d\n", port);
	printf("=================================================\n\n");
   	printf("Test Mode         : %s\n", mdk_gt_miiTestMode[dptc->testMode - 1]);
	printf("Test Packet Size: ...........%d\n", dptc->testPktSize);
	printf("Test Packet Count: ..........%d\n", dptc->testPktCnt);
	printf("Packets Received: ...........%d\n", dptc->rxPktCnt);
	printf("Errored Packets Received: ...%d\n", dptc->rxErrPktCnt);
	printf("Bytes Received: .............%d\n", dptc->rxByteCnt);
	printf("Packets Transmitted: ........%d\n", dptc->txPktCnt);
	printf("Bytes Transmitted: ..........%d\n", dptc->txByteCnt);
	return(NFX_OK);
} /* gt_mii_diagShowLbTestResults */


int32
gt_lpbk_timeout(int32 tmr_handle,
		 void * usr_param0,
		 void * usr_param1,
		 void * usr_param2,
		 void * usr_param3)
{
	uint32 testresult;
	uint32 port=0, t_port=0;
    mdk_gt64240mii_test_t *dptc;

	if (tmr_handle == time_handle_port0) {
		t_port = port = 0;
	}
	else if(tmr_handle == time_handle_port1) {
		t_port = port = 1;
	}
	else if(tmr_handle == time_handle_port2) {
		t_port = 0; /* mapped to port 0 for mdk_testState*/
		port = 2;
	}		
    dptc = &mdk_gt64240mii_testControl[port];
	/* If a packet has not been received by this time
 	* we consider it dropped and format a new packet
 	* In case the old packet is received in betwn, we drop it
 	* at the receive function. Do this till we have tx
 	* testPktCnt. We declare test as failed if 20 consecutive
	* pkts are lost this way (More than this number takes a
	* long time for test to timeout).
 	*/ 
	//printf("gt %d timeout",port);
	if(!fc_conflict) {
		if(dptc->txPktCnt <= dptc->testPktCnt) {
			tp->testresult = ERR_TIMEOUT;
			if(prvRxPktCnt[port] == dptc->rxPktCnt) {
				consecutivePktLoss[port]++;	
			}	
			else
				consecutivePktLoss[port] = 0;	
			if(consecutivePktLoss[port] > 20)
			{
				/* If we drop 20 consecutive packets,
				 * declare test as failed
				 */ 	
			}		
			else {
				mdk_gt_mii_diagRemakeLbPkt(port);
				return(NFX_OK);
			}	
		}
	}	
	else if(!conflict_timeout) {
			tp->testresult = ERR_TIMEOUT;
			tp->result[t_port].testresult = tp->testresult;
			mdk_gt_mii_diagRemakeLbPkt(port);
			return(NFX_OK);
	}		

	/* if we did not receive ANY packets, test has failed */
	if(!dptc->rxPktCnt)
		tp->testresult = RET_FAILED;

	if(fc_conflict)
		tp->result[t_port].testresult = tp->testresult;
	mdk_testState[t_port].timeout = 1;
	//mdk_gt_mii_diagShowLbTestResults(dptc, port);
	if (tp->param[TP_VERBOSE] == 1) 
		mdk_gt_mii_diagShowLbTestResults(dptc, port);
   	//if((phy_8353_lpbk) || (mac_lpbk))
	//	gt_reset_loopback(port);
	if(!fc_conflict) {
		mdk_testState[t_port].state = TEST_DONE;
		test_done = 1;
	}
	else if(conflict_timeout) {
		mdk_testState[t_port].state = TEST_DONE;
		test_done = 1;
	}
	return(NFX_OK);
}

#if 0
int
mdk_gt_mii_diagViewTestStats()
{
   mdk_gt64240mii_test_t *t;
char *gt_miiTestStates[] = {"",
							"TEST_DONE",
			    "TEST_INPROGRESS",
			    "TEST_VERIFY",
			    "TEST_CONTINUE",
			    "TEST_END"
};

char *gt_miiTestMode[] = {"INTERNAL LOOPBACK","EXTERNAL LOOPBACK"};

   t = &mdk_gt64240mii_testControl;

   printf("GT64240-MII LOOPBACK TEST RESULTS\n");
   printf("=================================\n\n");
   printf("Test State        : %s\n", gt_miiTestStates[t->testState]);
   printf("Test Mode         : %s\n", gt_miiTestMode[t->testMode - 1]);
   printf("Test Packet Cnt   : %d\n", t->testPktCnt);
   printf("Test Packet Size  : %d\n", t->testPktSize);
   printf("Test Verify Flag  : %d\n", t->fVerify);
   printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
   printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
   printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
   printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
   printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
   printf("\n");
   return 0;

} /* gt64240mii_diagViewTestStats */
#endif
#endif

