/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:        mount.h
 *
 * RCS:  $Id: mount.h,v 1.47 2002/10/22 01:30:39 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: MOUNT protocol definitions.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
#ifndef _MNT_H
#define _MNT_H

/*
#include <exports/rpc.h>
*/

#define	MOUNTD_APP_NAME	"mountd"
#define	MNT_SRVR_APP_NAME		"mnt_srvr"	
#define	MNT_EXPINFO_MAX_LEN		1024	
#define MNT_MAX_PATHNAME_LEN    1024
#define	MNT_INFO_MAX		4

#define	MNT_MAX_FILESYS		255
#define	MAX_CHAR_LEN		256
#define	MAX_NAME_LEN		256
#define	MNT_MAX_OPTION_LEN	128	
#define	MNT_TAB_HASH_SIZE	511

#define	MNT_MSG_BUF_MAX		4096	
#define	MNT_MSG_RESP_BUF_MAX	1024	

#define	MNT_IP_ADDR_STRING_LEN	20
#define	MNT_MAX_HOSTNAME_LEN	255	

#define	MNT_LADDR_OPT_STR	"laddr="
#define	MNT_RO_OPT_STR		"ro"
#define	MNT_RW_OPT_STR		"rw"
#define	MNT_ROOT_SQUASH_STR	"root_squash"
#define	MNT_NO_ROOT_SQUASH_STR	"no_root_squash"
#define	MOUNTD_EXPORTS_FILE		"/usr/local/agile/etc/exports"
#define	MOUNTD_MAX_EXPORTS	1024	

#define MOUNTD_MAX_COOKIE	65535

/*
 *Export options
 *
 *no_root_squash: Allows root users on client computers to have root access.
 *root_squash:	Requests from root clients are mapped    to the nobody user and Group ID
 *				only have privilleges associated with each other.
 *ro: read only access
 *rw: read write access
 */
#define MOUNTD_NO_ROOT_SQUASH_OPT       0x00000001
#define MOUNTD_ROOT_SQUASH_OPT          0x00000002
#define MOUNTD_RO_OPT                   0x00000004
#define MOUNTD_RW_OPT                   0x00000008
#define	MOUNTD_LADDR_OPT				0x00000010
// Maximum filename length in bytes
#define FS_NAME_MAX 255					//FOR TESTING ONLY

/*
 *MOUNT protocol
 */
#define	NFS_PROG		100003
#define MNT_PROG      	100005
#define NLM_PROG        100021
#define NSM_PROG        100024
#define MNT_VER1        1
#define MNT_VER2        2
#define MNT_VER3        3
#define MNT_MIN_PROC    0
#define MNT_MAX_PROC    8
#define NFS_VER2		2
#define	NFS_VER3		3

#define NLM_VER1        1
#define NLM_VER2        2 
#define NLM_VER3        3
#define NLM_VER4        4

#define NSM_VER1        1

/*
 * Mountd error Status
 */
#define MOUNTD_ERR_VOL_NOT_FOUND				(MNT_ERROR_BASE)
#define MOUNTD_ERR_BAD_VOL_ID					(MNT_ERROR_BASE+1)
#define MOUNTD_ERR_PERMISSION_DENIED			(MNT_ERROR_BASE+2)
#define MOUNTD_ERR_INVALID_CLIENT_PATH			(MNT_ERROR_BASE+3)
#define MOUNTD_ERR_NO_HOST_FOUND				(MNT_ERROR_BASE+4)
#define MOUNTD_ERR_NO_EXPORT_FOUND				(MNT_ERROR_BASE+5)
#define MOUNTD_ERR_CONFLICT_EXPORT_OPT			(MNT_ERROR_BASE+6)
#define MOUNTD_ERR_INVALID_EXPORT_OPT			(MNT_ERROR_BASE+7)
#define MOUNTD_ERR_DUPLICATE_EXPORT				(MNT_ERROR_BASE+8)
#define MOUNTD_ERR_MORE_GENERIC_EXPORT_FOUND	(MNT_ERROR_BASE+9)
#define MOUNTD_ERR_MORE_SPECIFIC_EXPORT_FOUND	(MNT_ERROR_BASE+10)
#define MOUNTD_ERR_CONFLICT_EXPORT				(MNT_ERROR_BASE+11)
#define MOUNTD_ERR_MULTIPLE_INTERFACE			(MNT_ERROR_BASE+12)
#define MOUNTD_ERR_ENABLE_FAILURE				(MNT_ERROR_BASE+13)
#define MOUNTD_ERR_DISABLE_FAILURE				(MNT_ERROR_BASE+14)
#define MOUNTD_ERR_INTERNAL						(MNT_ERROR_BASE+15)
#define	MOUNTD_ERR_FILESYS_NOT_READY			(MNT_ERROR_BASE+16)
#define MOUNTD_ERR_EXPORT_INVALID_0_LOCIP		(MNT_ERROR_BASE+17)
#define MOUNTD_ERR_EXPORT_INVALID_LOCIP			(MNT_ERROR_BASE+18)
#define MOUNTD_ERR_EXPORT_INVALID_0_CLNT		(MNT_ERROR_BASE+19)
#define MOUNTD_ERR_EXPORT_INVALID_CLNT			(MNT_ERROR_BASE+20)
#define MOUNTD_ERR_EXPORT_INVALID_CLNT_LOCIP	(MNT_ERROR_BASE+21)
#define MOUNTD_ERR_PATHNAME_NOT_FOUND			(MNT_ERROR_BASE+22)
#define MOUNTD_ERR_MAX_NUM_OF_SHARES_REACHED 	(MNT_ERROR_BASE+23)

/*
 * TxRx Slot Id
 */
#define NFX_TXRX_SLOT   1

/*
 *MOUNT procedure numbers
 */
typedef enum mount_proc
{
	MNT_PROC_NULL           = 0,            /*Do nothing*/
	MNT_PROC_MNT            = 1,            /*Add MOUNT Entry*/
	MNT_PROC_DUMP           = 2,            /*Return MOUNT entries: like UNIX showmount*/
	MNT_PROC_UMNT           = 3,            /*Remove MOUNT entry*/
	MNT_PROC_UMNTALL        = 4,            /*Remove all MOUNT entries*/
	MNT_PROC_EXPORT         = 5,            /*Return Export list of export filesystems and*/
											/*their export information*/
	MNT_PROC_EXPORTALL      = 6,            /*Identical to previous procedure 5*/
											/*This procedure was eliminated in ver. 3*/
	MNT_PROC_PATHCONF       = 7             /*POSIX pathconf information*/
} mnt_proc_t;

typedef enum 
{
	MNT_INFO_REQ = MOUNTD_MSG_BASE,		//BASE+0:Get MOUNTD Info Req based 
										//on volume name
	MNT_INFO_RESP,						//BASE+1:Get MOUNTD Info Response
	MNT_INFO_REQ_BY_ID,					//BASE+2: Get MOUNTD Info Req based 
										//on volume ID
	MNT_INFO_RESP_BY_ID,				//BASE+3: Get MOUNTD info by ID response
	MNT_GET_VOLIDS,						//BASE+4:Get volume IDs for current
										//current exports. 
	MNT_EXPORT_REQ,						//BASE+5:export request
	MNT_UNEXPORT_REQ,					//BASE+6:unexport request
	MNT_EXPORT_UPDATE,					//BASE+7
	MNT_EXPORT_SHOW,					//BASE+8
	MNT_EXPORT_SHOWALL,					//BASE+9
	MNT_PORTMAP_REG_PORTS,				//BASE+10:Reg ports with PORTMAPPER
	MNT_IS_VOL_EXPORTED,				//BASE+11
	MNT_EXPORT_ENABLED,					//BASE+12
	MNT_EXPORT_ENABLED_ALL,				//BASE+13
	MNT_EXPORT_DISABLED,				//BASE+14
	MNT_EXPORT_DISABLED_ALL,			//BASE+15

	MNT_TRACE_ENABLE,					//BASE+16:Enable mountd trace
	MNT_TRACE_DISABLE,					//BASE+17:Disable mountd trace
	MNT_EXPRULE_DISPLAY,				//BASE+18:Display export rules and stats

	MNT_EXPORT_ENABLED_REQ,				//BASE+19:message sent from mountd to txrx
	MNT_EXPORT_ENABLED_ALL_REQ,			//BASE+20:message sent from mountd to txrx
	MNT_EXPORT_DISABLED_REQ,			//BASE+21:message sent from mountd to txrx
	MNT_EXPORT_DISABLED_ALL_REQ,		//BASE+22:message sent from mountd to txrx
	MNT_EXPORT_ADD_REQ,					//BASE+23:message sent from mountd to txrx
	MNT_EXPORT_DELETE_REQ,				//BASE+24:message sent from mountd to txrx
	MNT_EXPORT_DELETE_ALL_REQ,			//BASE+25:message sent from mountd to txrx
	MNT_UPD_FHANDLE_REQ,				//BASE+26:Send update Filehandle to MOUNTD SSC
	MNT_EXPORT_GET_FHANDLE_REQ,			//BASE+27:Get Filehandle Req
	MNT_GET_SHARES,						//Base+28
	MNT_GET_SHARES_RESP,				//Base+29
										//...
										//Please check Includes/nfx-defs.h
										//when adding message type to see if
										//we had gone beyond the range for MOUNTD 
	MNT_MSG_MAX							//BASE+30
} mnt_msgType_t;

typedef struct mnt_msgHdr
{
	mnt_msgType_t	msgType;		/*message type*/
	uint32			pad;
	uint64			private;
	int32			status;			/*message response status*/
	int32           reserved;   
} mnt_msgHdr_t;

NFX_ASSERT_STATIC(sizeof (mnt_msgHdr_t) % 8 == 0);


typedef struct mnt_infoEntry
{
	evm_volId_t	volId;
	uint64		inode;				/* inode number*/
	uint64		mntPnt;				/*inode number of the mount point*/
	uint32		localIpAddr;			
	int32		exportOptionMask;	
	uint32		hostAddr;
	uint32		exportInfoLen;		/*length of export info in bytes*/
	uint32		pathNameLen;		/*pathname length in bytes*/
	uint32		hostNameLen;		/*hostName length in bytes*/
	uint32		state;				
#define	MNT_SHARE_EXPORT_ENABLED	1
#define	MNT_SHARE_EXPORT_DISABLED	2
	uint32		pad;
	char		pathName[0];
	char		exportInfo[0];
	char		hostName[0];
} mnt_infoEntry_t;

// Advance a pointer to the next entry[]
// position.  Accounts for variable name length
// and 8-byte alignment requirements.

#define MNT_INFO_RESP_ADV_ENTRY_PTR(_ptr_)                                   \
    {                                                                        \
        mnt_infoEntry_t *infoEntryp = (mnt_infoEntry_t *) (_ptr_);           \
        uint32 entrySize = (sizeof(mnt_infoEntry_t) +                        \
                            infoEntryp->pathNameLen +                        \
                            infoEntryp->exportInfoLen +                      \
                            infoEntryp->hostNameLen + 3);                    \
        if (entrySize % sizeof(uint64))                                      \
            {entrySize += (sizeof(uint64) - (entrySize % sizeof(uint64)));}  \
        _ptr_ = (void *)(((char *) _ptr_) + entrySize);                      \
        _ptr_ = (void *)((address_t)(_ptr_) &                                \
                                ~(sizeof (uint64) - 1));                     \
    }

typedef struct mnt_infoReq
{
	mnt_msgHdr_t	msgHdr;			/*message header*/
	evm_volId_t		volId;
	uint64          inode;         	/* inode number*/
    uint64          mntPnt;        	/*inode number of the mount point*/
	uint32			ipAddr;			/*client IP address*/
	uint32			localIpAddr;	/*our local IP address*/
	uint32			uid;
	uint32			gid;
	uint32			respBufSize;	/*Max bytes of data in response*/
	uint32			cookie;			/*"Index" of the entry in mnt_InfoCB list*/
	uint32			nameLen;
	uint32			optionsLen;
	char			pathName[0];
	char			options[0];
} mnt_infoReq_t;

typedef struct mnt_infoResp
{
	mnt_msgHdr_t	msgHdr;
	uint32			cookie;			/*"Index" of the entry in mnt_InfoCB list*/
	uint32			numOfEntries;		/*number of entries*/	
	mnt_infoEntry_t	mtnInfoEntry[0];
} mnt_infoResp_t;

typedef struct mnt_infoAllReq
{
	mnt_msgHdr_t	msgHdr;			/*message header*/
	ushort16		nextIndex;		/*index record*/
} mnt_infoAllReq_t;

typedef struct mtn_infoAllResp
{
	mnt_msgHdr_t	msgHdr;				/*message header*/
	uint32			nextIndex;			/*next index record*/
	uint32			more_data;
	uint32			numOfEntries;		/*number of entries*/	
	mnt_infoEntry_t	mtnInfoEntry[0];
} mtn_infoAllResp_t;

typedef struct mnt_exportStateChangeReq
{
	mnt_msgHdr_t	msgHdr;				/*message header*/
	uint32			nameLen;			/*length of path name*/
	char			pathName[0];		/*path name of export*/
} mnt_exportStateChangeReq_t;

typedef struct mnt_exportAddReq
{
	mnt_msgHdr_t	msgHdr;				/*message header*/
	mnt_infoEntry_t	mntInfoEntry;
} mnt_exportAddReq_t;

typedef struct mnt_exportDeleteReq
{
	mnt_msgHdr_t	msgHdr;				/*message header*/
	uint32			localIpAddr;		/*local IP address*/
	uint32			clientIpAddr;		/*client IP address*/
	uint32			nameLen;			/*length of path name*/
	char			pathName[0];		/*path name of export*/
} mnt_exportDeleteReq_t;

typedef struct mnt_getFileHandleReq
{
	mnt_msgHdr_t	msgHdr;				/*message header*/
	evm_volId_t		volId;
	uint32			reserved;	
	uint32			nameLen;			/*length of path name*/
	char			pathName[0];		/*path name of export*/
} mnt_getFileHandleReq_t;


/*
 *Mount Info Control Block
 */
typedef struct mnt_infoCB
{
	struct mnt_infoCB	*prevp;			/*pointer to previous entry*/
	struct mnt_infoCB	*nextp;			/*pointer to next entry*/
	uint32				cookie;			/*unique to each existing mnt_InfoCB*/
	mnt_infoEntry_t		mntInfoEntry;
} mnt_infoCB_t;


typedef struct mountd_globalInfo
{
	mnt_infoCB_t	*mnt_infoHeadp;		/*head of the list*/
	uint32			numOfEntries;
	int32			mnt_fd;			
	int32			mountd_AppId;	
	uint32			cookie;
} mountd_globalInfo_t;



typedef struct mnt_statCounters
{
    uint64  mnt_ok;                 /*Mount successful*/
    uint64  mnt_errPerm;            /*Not owner.  Permission error*/
    uint64  mnt_errNoEnt;           /*No such file or directory*/
    uint64  mnt_errIO;              /*Some sort of hard error occurred when the
                                     *operation was in progress (ex: disk error)
                                     */
    uint64  mnt_errAccess;          /*Permission denied*/
    uint64  mnt_errNotDir;          /*Not a directory*/
    uint64  mnt_errInval;           /*Invalid argument*/
    uint64  mnt_errNameTooLong;     /*Name too long*/
    uint64  mnt_errNotSupp;         /*Operation not supported*/
    uint64  mnt_errSrvFault;        /*A failure on the server*/
	uint64	mnt_errFhandle;			/*stale fileHandle error*/
} mnt_statCounters_t;

typedef struct mnt_pktCounters
{
	uint64	mnt_null;				/*NULL*/
    uint64  mnt_addMnt;           	/*mount req*/
    uint64  mnt_dump;          		/*dump mount entries (showmount)*/
    uint64  mnt_umnt;          		/*remove mount entry req.*/
    uint64  mnt_umntAll;       		/*remove all mount entries req.*/
    uint64  mnt_export;        		/*export list*/
    uint64  mnt_exportAll;     		/*export list all*/
    uint64  mnt_pathConf;      		/*POSIX pathconf*/
} mnt_pktCounters_t;

typedef struct mnt_clientInfo
{
	int32				exportOptionMask;	/*client MOUNT export info*/
    mnt_statCounters_t  mnt_statCounters;   /*MOUNT procedure stat. counters*/
    mnt_pktCounters_t	mnt_rxPktCounters;  /*MOUNT receive packet stat. counters*/
    mnt_pktCounters_t	mnt_xmtPktCounters; /*MOUNT Xmit packet stat. counters*/

} mnt_clientInfo_t;

typedef struct mnt_debugCounters
{
	uint64				mnt_eeeAllocCnt1;
	uint64				mnt_eeeDeallocCnt1;
	uint64				mnt_eeeAllocCnt2;
	uint64				mnt_eeeDeallocCnt2;
	uint64				mnt_eeeAllocCnt3;
	uint64				mnt_eeeDeallocCnt3;
	uint64				mnt_eeeAllocCnt4;
	uint64				mnt_eeeDeallocCnt4;
    uint64              mnt_tplXmtPktSend;
    uint64              mnt_tplXmtPktCmpInd;
    uint64              mnt_tplXmtSendErr;
    uint64              mnt_tplDropRcvPkt;
    uint64              mnt_allocReqCB;
    uint64              mnt_freeReqCB;
	uint64				mnt_fsErrNoResource;
	uint64				mnt_totalNumOfReqXmtToFs;
	uint64				mnt_totalNumOfRespRcvdFromFs;
	uint64				mnt_maxReqsDropPktErr;
	uint64				mnt_rcvPktIndErr;
	uint64				mnt_xmtReqByIdToSsc;
	uint64				mnt_rxRespByIdFromSsc;
	uint64				mnt_xmtReqByNameToSsc;
	uint64				mnt_rxRespByNameFromSsc;
	uint64				mnt_eeeAllocGetFhandleReq;
	uint64				mnt_eeeFreeGetFhandleReq;
    uint64              mnt_allocAdminReqCB;
    uint64              mnt_freeAdminReqCB;
} mnt_debugCounters_t;

typedef struct mnt_globalCfg
{
	int32				mountdAppId;
	int32               eeeAppId;
	int32				evmAppId;			/*EVM EEE app ID*/
	int32				mnt_srvrAppId;		/*Mount AppId on TXRX*/
	int32				tmrId;
#define	MNT_GET_SHARES_TIME		30000		/*wait for 30 secs*/
	mnt_statCounters_t  mnt_statCounters;   /*MOUNT procedure statistic*/
    mnt_pktCounters_t	mnt_rxPktCounters;  /*MOUNT receive packet stat. counters*/
    mnt_pktCounters_t	mnt_xmtPktCounters; /*MOUNT Xmit packet stat. counters*/
	mnt_debugCounters_t	mnt_debugCounters;
} mnt_globalCfg_t;

typedef struct mnt_request
{
	rpcCallHdr_t	*rpcCallHdrp;			/*Pointer RPC Call hdr*/
	void			*clientCBp;				/*Client Control Block*/
	void        	*fileSysCBp;			/*File System Control*/
	void			*reqCBp;				/*request CB*/
	void			*pktDescp;				/*Tx pkt descriptor*/
	char			*procStart;				/*Start of rx MOUNT procedure*/
	int32			procLen;				/*MOUNT procedure length*/
	uchar8			namep;
	uint32			nameLen;
	fs_Handle		fileHandle;
	uchar8			*fileMsgp;
	uint32			fileMsgLen;
	uint32			reqState;
#define MNT_REQ_INPROG      1
#define MNT_REQ_REPLIED     2
#define MNT_REQ_DONE        3
#define MNT_REQ_GET_FHANDLE	4
	void			*tplConnp;				/*TPL conn ID*/
	void 			*nfsReqp;
	uint32			srcPort;				/*hold EEE source port from SSC app*/
	uint32			destPort;				/*hold EEE destination from SSC app*/
} mnt_request_t;

typedef struct mnt_tplVolInfo
{
    uint32      client_ipAddr;
    void        *mnt_pTplConn;
    void        *umnt_pTplConn;
    void        *nfs_pTplConn;  
}  mnt_tplVolInfo_t;

/*
 *Extern
 */
extern	mountd_globalInfo_t		mnt_info;
extern	mountd_globalInfo_t		*mnt_infop;
extern	mnt_globalCfg_t 		*mnt_globalCfgp;
extern	mountd_globalInfo_t		mountd_globalInfo;
extern	mountd_globalInfo_t		*mountd_globalInfop;

extern 	uint32 					trace_enable;


/*
 *MOUNT procedure prototypes
 */
#ifdef	NFP_TXRX

int32
mnt_procRxNullReq(mnt_request_t    *mntReqp         /*Mount request control block*/
);

int32
mnt_procRxMountReq (mnt_request_t    *mntReqp           /*Mount request control block*/
);

int32
mnt_procRxShowMountReq(mnt_request_t    *mntReqp       /*Mount request control block*/
);

int32
mnt_procRxUnmountReq (mnt_request_t    *mntReqp         /*Mount request control block*/
);

int32
mnt_procRxUnmountAllReq (mnt_request_t    *mntReqp      /*Mount request control block*/
);

int32
mnt_procExportListReq (mnt_request_t    *mntReqp       /*Mount request control block*/
);

int32
mnt_procExportAllReq (mnt_request_t    *mntReqp       /*Mount request control block*/
);

int32
mnt_procPathconfReq (mnt_request_t    *mntReqp       /*Mount request control block*/
);

int32 mnt_getVolId(char *name, mnt_request_t  *mntReqp);

/*
 *prototypes from sm-mount/mount.c
 */
int32
mnt_getMountdInfoReq(mnt_request_t *mntReqp, int32 msgType);

void mnt_msgRcv(eee_desc_t *eDescp );
int32   
mnt_storeAuthFlavor(char *msgp);
int32
mnt_getFileHandle(eee_desc_t *eDescp);
int32
mnt_sendAdminRespToMountd(
		int32 msgType,			//message type
		char *pathNamep, 		//pathName
		int32 pathNameLen,		//pathname length in bytes
		fs_Handle	*fhandlep,	//!=NULL, contains filehandle
		int32 status,			//NFX_OK or NFX_ERR		
		uint32 srcPort,
		uint32 destPort
);

void mnt_getNfsShares(uint32 cookie);

int32
mnt_getNfsShareHandler( uint32 tmr,
              void  *info1,
              void  *info2,
              void  *info3,
              void  *info4 );

#endif	//NFP_TXRX
/*
 *NFX MOUNTD prototypes
 *sm-mount/mountd.c prototypes
 *sm-mount/mountd-util.c prototypes
 *sm-mount/mountd-cmd.c prototypes
 */

/*************************
 * Prototypes in mountd.c
 *************************
 */

/*-----------------------------------------------------------------
 * Name:        mountd_findRulesInit
 *
 * Description:
 *      Initialize the find rules array
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/22/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_findRulesInit(void);

/*-----------------------------------------------------------------
 * Name:        mountd_displayRulesForExport
 *  
 * Description: 
 *      Display the rules.
 *
 * Created by:          Xiaobing Zhang
 *  
 * Date Created:        05/22/2002
 *  
 *-----------------------------------------------------------------
 */
void mountd_displayRulesForExport(void);

/*-----------------------------------------------------------------
 *  * Name:        mountd_isReqVolExported 
 *  
 * Description:
 *      Check if input volume (by Volume ID) is currently been 
 *      exported. 
 *  
 * Created by:          Joseph Vu
 *
 * Date Created:        03/20/02
 *
 *-----------------------------------------------------------------
 */
int mountd_isReqVolExported(evm_volId_t searchVolId);

/*-----------------------------------------------------------------
 * Name:        mountd_notifyCcm 
 * 
 * Description:
 *      Notify NCM/CCM if a volume (or directory) has been exported 
 *      or unexported. 
 *      
 * Created by:          Joseph Vu
 * 
 * Date Created:        03/20/02
 * 
 *-----------------------------------------------------------------
 */
void mountd_notifyCcm(mnt_infoEntry_t *mnt_infoEntryp, uint32 delflag);

/*-----------------------------------------------------------------
 * Name:        mountd_notifyCcmAll
 * 
 * Description:
 *      Notify NCM/CCM if a volume (or directory) has been exported.
 *  
 * Created by:          Joseph Vu
 * 
 * Date Created:        03/20/02
 *  
 *-----------------------------------------------------------------
 */
void mountd_notifyCcmAll(void);

/*-----------------------------------------------------------------
 * Name:       mountd_findInfoEntryById
 *
 * Description:
 *      Retrieve MOUNTD info control block based on input volume ID.
 *
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
//mnt_infoCB_t *mountd_findInfoEntryById(evm_volId_t inputVolId);
mnt_infoCB_t *mountd_findInfoEntryById(mnt_infoReq_t *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:        mountd_checkFindRulesForExport
 *
 * Description:
 *      Check whether the two export entries (represented by input)
 *      are duplicated, conflict, subset, or superset with each other.
 *  
 * Created by:          Xiaobing Zhang
 *  
 * Date Created:        05/18/2002
 *
 *-----------------------------------------------------------------
 */
int32 mountd_checkFindRulesForExport(int32 la, int32 ca, 
				                    int32 lb, int32 cb, int32 opt,
									int32 *reason);

/*-----------------------------------------------------------------
 * Name:        mountd_findInfoEntryForExport
 *
 * Description:
 *      Retrieve MOUNTD info control block based on input pathname,
 *      client and local IP addresses, and export option mask.
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/18/2002
 *
 *-----------------------------------------------------------------
 */
mnt_infoCB_t *mountd_findInfoEntryForExport(char    *pathNamep,
				                            uint32  localIpAddr,
											uint32  hostIpAddr,
											uint32  optionMask,
											uint32  *status);

/*-----------------------------------------------------------------
 * Name:        mountd_findInfoEntryForUnexport
 *  
 * Description:
 *      Retrieve MOUNTD info control block based on input pathname,
 *      client IP address, and local IP address.
 *  
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/18/2002
 *
 *-----------------------------------------------------------------
 */
mnt_infoCB_t *mountd_findInfoEntryForUnexport(char      *pathNamep,
				                              uint32    localIpAddr,
											  uint32    hostIpAddr,
											  int32		*status);

/*-----------------------------------------------------------------
 * Name:        mountd_findInfoEntryForMountReq
 *
 * Description:
 *      Retrieve MOUNTD info control block based on input pathname,
 *      client IP address, and local IP address from mount request.
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/18/2002
 *
 *-----------------------------------------------------------------
 */
mnt_infoCB_t *mountd_findInfoEntryForMountReq(char      *pathNamep,
				                              uint32    localIpAddr,
											  uint32    hostIpAddr);

/*-----------------------------------------------------------------
 * Name:        mountd_allocInfoCB
 *
 * Description:
 *      Allocate export entry for input export info record.
 *
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
mnt_infoCB_t *mountd_allocInfoCB (int32 pathNameLen, int32 exportInfoLen);

/*-----------------------------------------------------------------
 * Name:        mountd_delInfoAllCB
 *
 * Description:
 *      Delete all export entries from the export list.
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
int mountd_delInfoAllCB (uint32 notifyCcm);

/*-----------------------------------------------------------------
 * Name:        mountd_delInfoCBByLocalIpAddr
 *
 * Description:
 *      Delete the export entries given their local IP address.
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/20/2002
 *
 *-----------------------------------------------------------------
 */
int mountd_delInfoCBByLocalIpAddr (uint32 localIpAddr);

/*-----------------------------------------------------------------
 * Name:       mountd_delInfoCBByPointer 
 *
 * Description:
 *      Delete the export entrie.
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/18/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_delInfoCBByPointer (mnt_infoCB_t *mnt_infoCBp);

/*-----------------------------------------------------------------
 * Name:        mountd_addInfoCB
 *
 * Description:
 *      Add export info record to the export list.
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
int32 mountd_addInfoCB(mnt_infoCB_t *mnt_infoCBp,
						mnt_infoReq_t *mnt_infoReqp,
						char *pathNamep, int32 pathNameLen,
						char *exportInfop, int32 exportInfoLen,
						uint32 optionMask);

/*-----------------------------------------------------------------
 * Name:         mountd_getId
 *
 * Description:
 *      Get volume Id from EVM based for the input pathname.
 *
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
evm_volId_t mountd_getId(char *pathNamep, int32 pathNameLen);

/*-----------------------------------------------------------------
 * Name:            mountd_getvolIds
 *  
 * Description:
 *      Get volume Ids from EVM for all volumes specified in /exports file..
 *
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
int32 mountd_getvolIds(void);

/*-----------------------------------------------------------------
 * Name:    mountd_init 
 *
 * Description:
 *  Init. MOUNTD application runs on the SSC that got call at
 *  boot time to read the export information.  
 * Created by:          Joseph Vu
 *
 * Date Created:        07/10/01
 *
 *-----------------------------------------------------------------
 */
int32 mountd_init(void);

/*-----------------------------------------------------------------
 * Name:    mountd_setupRespMsg
 *
 * Description:
 *  Set up response message to send back to FP-NIM.
 * 
 * Created by:      Joseph Vu
 * 
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int32 mountd_setupRespMsg(
    mnt_infoResp_t  *mnt_infoResp,
    mnt_infoCB_t    *mnt_InfoCBp
);

/*-----------------------------------------------------------------
 * Name:    mountd_setupShowResp 
 *
 * Description:
 *  Set up response message to send to requester.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */ 
int32 mountd_setupShowResp(mnt_infoReq_t *mnt_infoReqp,
				           mnt_infoResp_t *mnt_infoResp,
						   uint32 maxSize);

/*-----------------------------------------------------------------
 * Name:    mountd_processExportOptionString
 *  
 * Description:
 *  Process an export option string from export request.
 *  
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/17/2002
 *  
 *-----------------------------------------------------------------
 */ 
int32 mountd_processExportOptionString(mnt_infoReq_t  *mnt_infoReqp,
				                       char            *optionp,
									   int32           *optionLenp,
									   int32           *optionMaskp);

/*-----------------------------------------------------------------
 * Name:    mountd_processExportEntry
 *
 * Description:
 *  Process an export entry.
 *  
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */ 
int mountd_processExportEntry(mnt_infoReq_t   *mnt_infoReqp,
        int exportUpdate);

/*-----------------------------------------------------------------
 * Name:    mountd_processUnexportEntry
 *
 * Description:
 *  Process an unexport entry.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_processUnexportEntry(mnt_infoReq_t   *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:  mountd_isFileSysReady     		 
 *
 * Description:
 *	Check if filesystem is ready to be used.
 *	The file system is ready to be used if the state
 *	is ONLINE.	
 *  
 * Created by:          Joseph Vu
 *
 * Date Created:        07/03/02
 *
 *-----------------------------------------------------------------
 */
int32	//TRUE or FALSE
mountd_isFileSysReady(evm_volId_t volId);

/*-----------------------------------------------------------------
 * Name:   mountd_updateFileHandle    		 
 *
 * Description:
 *	Update configure NFS share directory (mount point) with it
 *  corresponding filehandle from the file system.
 *
 * Created by:          Joseph Vu
 *
 * Date Created:        07/03/02
 *
 *-----------------------------------------------------------------
 */

int32
mountd_updateFileHandle(mnt_infoReq_t	*mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:      	mountd_findRulesInit
 *
 * Description:
 *     	Initialize the find rules array
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/22/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_findRulesInit();

/*-----------------------------------------------------------------
 * Name:      	mountd_displayRulesForExport
 *
 * Description:
 *     	Display the rules.
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        05/22/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_displayRulesForExport();



/******************************
 * Prototypes in mountd-util.c
 ******************************
 */

/*-----------------------------------------------------------------
 * Name:    mountd_isHostPresentInExpEntry
 *
 * Description:
 *  Check if client or host is allow to access current export
 *  entry.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int     //found = 0: not found
        //found = 1: found
mountd_isHostPresentInExpEntry(
    mnt_infoReq_t *mnt_infoReqp,
    mnt_infoCB_t *mnt_InfoCBp,
	int uiFlag
);

/*-----------------------------------------------------------------
 * Name:    mountd_getAddrFromName 
 *
 * Description:
 *  Returns IP address based on input client name string. 
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_getAddrFromName(char *namep);

/*-----------------------------------------------------------------
 * Name:   mountd_getIpAddr
 *
 * Description:
 *  Convert an IP address string into 32 bits integer.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_getIpAddr( char *ipAddrStr, uint32 *addr);

int mountd_extractIpaddr(char *inputStr, char *outputStr);

/*-----------------------------------------------------------------
 * Name:    mountd_extractHost
 *
 * Description:
 *  Extract Host from export Info String.  Host could be a hostname or
 *  an IP address.
 *  Example of export Info string:
 *      10.0.0.133(rw,no_root_squash) or
 *      hostName(rw,no_root_squash)
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_extractHost(char *hostStr, char *exportInfop, int32 exportInfoLen);

/*-----------------------------------------------------------------
 * Name:    mountd_getNextHost
 *
 * Description:
 *      Return length to next host in the export entry.
 *      example:
 *          hostName1(rw,no_root_squash) hostname2(ro) ....
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_getNextHost(char *exportInfop, int exportInfoLen);

/*-----------------------------------------------------------------
 * Name:    mountd_removeSpaces
 *
 * Description:
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
void mountd_removeSpaces(char *destp, char *sourcep, int length);

/*-----------------------------------------------------------------
 * Name:    mountd_extractOption
 *
 * Description:
 *  Extract option field in the current entry.
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_extractOption(char *exportp, char *optionp);


/*-----------------------------------------------------------------
 * Name:    mountd_checkOption
 *
 * Description:
 * Check export options. It must be one the following options:
 * no_root_squash:  Allows root users on client computers to
 *                  have root access.
 * root_squash:     Requests from root clients are mapped to
 *                  the nobody user and groupId only have privilleges
 *                  associated with each other.
 * ro:              read only access
 * rw:              read write access
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_checkOption(char *optionStr);

/*-----------------------------------------------------------------
 * Name:    mountd_processExportOptions
 *
 * Description:
 * Process export options.
 * options:
 * no_root_squash:  Allows root users on client computers to have root access.
 * root_squash:     Requests from root clients are mapped
 *                  to the nobody user and groupId
 *                  only have privilleges associated with each other.
 * ro:              read only access
 * rw:              read write access
 * laddr:           Check local interface address
 *     
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_processExportOptions(char *exportInfop, int length);

/*-----------------------------------------------------------------
 * Name:   mountd_isCharAlphaNumeric 
 *  
 * Description:
 *  Check if input character is is alpha-numeric.
 * Created by:      Joseph Vu
 *  
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_isCharAlphaNumeric(char *ch);

/*-----------------------------------------------------------------
 * Name:    mountd_isExportOptionPresent
 *
 * Description:
 *  Check any export options is specified.
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_isExportOptionPresent(char *exportp, int length);

/*-----------------------------------------------------------------
 * Name:    mountd_parseClientNameField
 *
 * Description:
 *  Parse input client name field in the format of
 *  client:/pathname into its own buffer.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_parseClientNameField(char *nameFieldp, int nameFieldLen,
    char *clientNamep, int *clientNameLen,
    char *pathNamep, int *pathNameLen);

/*-----------------------------------------------------------------
 * Name:        mountd_getVolumeName
 *
 * Description:
 * Extract volume name from the input pathname.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/28/01
 *
 *-----------------------------------------------------------------
 */
int mountd_getVolumeName(char *volNamep, int *volNameLen,
    char *pathNamep, int pathNameLen);

/*-----------------------------------------------------------------
 * Name:    mountd_portmap_register 
 * 
 * Description:
 *  Check any export options is specified.
 * Created by:      Joseph Vu
 * 
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
int32 mountd_portmap_register(void);

/*-----------------------------------------------------------------
 * Name:    mountd_getLocalAddr
 *
 * Description:
 *  Retrieves local address specified in export option field.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_getLocalAddr(char *exportp, int *localAddr);

/*-----------------------------------------------------------------
 * Name:    mountd_storeHostPart  
 *
 * Description:
 * Stores host part of the export info record
 * into new export buffer.  
 * Example:  IPAddr_Str(options)
 * where, IPAddr_Str is in standard dot notations.
 *  
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_storeHostPart(
    char *currExportp,
    int currExportLen,
    char *newExport,
    int *newLen
);

/*-----------------------------------------------------------------
 * Name:    mountd_storeHostPartByName
 *
 * Description:
 * Stores host part of the export info record
 * into new export buffer, where host field is
 * specified by name.  We need to consult NIS
 * server to resolve hostname.
 * Example:  clientName(options)
 *
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int mountd_storeHostPartByName(
    char *currExportp,
    int currExportLen,
    char *newExport,
    int *newLen,
    int *hostAddrp
);

/*-----------------------------------------------------------------
 * Name:    mountd_processHostPart
 *
 * Description:
 * Process host part of the export info record.
 * The host can be specified by IPAddr (in standard dot notation)
 * or hostname.  If host is specified by name we need to consult
 * NIS to revolve entry.  Return new data in input 
 * buffer (newExportp) and its update length (newExportLen).
 *  
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */ 
int mountd_processHostPart(
    char *currExportp,
    int currExportLen,
    char *newExportp,
    int *newExportLen,
    int *hostAddrp
);

/*-----------------------------------------------------------------
 * Name:    mountd_buildOptionString
 *
 * Description:
 * Given option mask, build the corresponding option string.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/21/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_buildOptionString(char *destStr, uint32 optionMask);

/*-----------------------------------------------------------------
 * Name:       	  mountd_pathNameCmp
 *
 * Description:
 *		Compare two export path names.
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        08/13/2002
 *
 *-----------------------------------------------------------------
 */
int mountd_pathNameCmp(char *name1, char *name2);

/*-----------------------------------------------------------------
 * Name:       	  mountd_ipAddrCmp
 *
 * Description:
 *		Compare two IP address strings.
 *
 * Created by:          Xiaobing Zhang
 *
 * Date Created:        08/13/2002
 *
 *-----------------------------------------------------------------
 */
int mountd_ipAddrCmp(unsigned int ipAddr1, unsigned int ipAddr2);

/*****************************
 * Prototypes in mountd-cmd.c
 *****************************
 */

/*-----------------------------------------------------------------
 * Name:    mountd_cmdDisplayInfoList
 *
 * Description:
 *  Display current export entries.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
void mountd_cmdDisplayInfoList(void);

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportAddReq 
 * 
 * Description:
 * Send message to NFS on TxRx for share add request.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/08/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_nfsExportAddReq(mnt_infoEntry_t *mntInfoEntryp);

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportDeleteReq 
 * 
 * Description:
 * Send message to NFS on TxRx for share delete request.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/08/2002
 *
 *-----------------------------------------------------------------
 */
void mountd_nfsExportDeleteReq(char *pathNamep,
							   uint32 localIpAddr,
							   uint32 clientIpAddr);

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportStateChangeReq 
 *  
 * Description:
 * Send message to NFS on TxRx for filesystem state change
 * from enabled to disabled or vice versa.
 *
 * Created by:      Xiaobing Zhang 
 *
 * Date Created:    05/01/2002
 *  
 *-----------------------------------------------------------------
 */ 
void mountd_nfsExportStateChangeReq(int32 msgType, mnt_infoReq_t *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:    mountd_exportStateChange
 *
 * Description:
 * Enable or disable input shares based on pathname.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/20/2002
 * 
 *-----------------------------------------------------------------
 */
int32 mountd_exportStateChange(mnt_infoReq_t *mnt_infoReqp, int32 state);

/*-----------------------------------------------------------------
 * Name:    mountd_enableAllExports
 * 
 * Description:
 * Enable all exports.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *  
 *-----------------------------------------------------------------
 */
void mountd_enableAllExports(void);

/*-----------------------------------------------------------------
 * Name:    mountd_enableExport 
 *
 * Description:
 * Enable an Input share.
 *
 * Created by:      Joseph Vu
 * Modified by:     Xiaobing Zhang
 *
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
int mountd_enableExport(mnt_infoReq_t   *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:    mountd_disableAllExports 
 * 
 * Description:
 * Disable all exports.
 * 
 * Created by:      Joseph Vu
 * 
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
void mountd_disableAllExports(void);

/*-----------------------------------------------------------------
 * Name:    mountd_disableExport 
 *
 * Description:
 * Disable an Input share.
 *
 * Created by:      Joseph Vu
 * Modified by:     Xiaobing Zhang
 *
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
int mountd_disableExport(mnt_infoReq_t   *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:   	mountd_getExportFileHandle 
 *
 * Description:
 * Send a request to TXRX to get the filehandle for 
 * the input share directory (mount point).
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    07/2/02
 * 
 *-----------------------------------------------------------------
 */
int
mountd_getExportFileHandle (
	char *pathNamep, 
	int	pathNameLen, 
	evm_volId_t volId);

void mountd_portmapRegMountPort(void);

int32 mountd_portmapRegNfsPort(void);
int32 mountd_portmapRegNlmPort(void);
int32 mountd_portmapRegNsmPort(void);


#endif  /*_MNT_H*/
