/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        mountd-cmd.c
 *
 * RCS:  $Id: mountd-cmd.c,v 1.14 2002/09/19 22:50:18 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain MOUNTD commands.
 * Created by: Joseph Vu
 *
 * Date Created:        9/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef  LINUX_TEST
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#endif	//LINUX_TEST

#include <nfx-incl.h>
#include <nfx-defs.h>
#include <exports/anpssc-api.h>

#include <exports/evm-api.h>
#include <exports/fs-api.h>
#include "rpc.h"
#include "mount.h"

#define MNT_DBG_PRINTF	if(mountd_trace_enable) printf

#define MOUNTD_ALLOC        malloc
#define MOUNTD_FREE         free

#define	MOUNTD_ADD_EXPORT	0x01	//Add export 
#define	MOUNTD_DEL_EXPORT	0x02	//Delete export 
#define	MOUNTD_MOD_EXPORT	0x04	//change export

#define	MOUNTD_OPT_RO				0x01	//Read only option
#define	MOUNTD_OPT_RW				0x02	//Read-write option
#define	MOUNTD_OPT_NO_ROOT_SQUASH	0x04	//no_root_squash option
#define	MOUNTD_OPT_ROOT_SQUASH		0x08	//root_squash option

int		//NFS_OK:  export OK
		//NFS_ERR: export error 
mountd_cmdExportFs(
	int action,				//MOUNTD_ADD_EXPORT or
							//MOUNTD_DEL_EXPORT or
							//MOUNTD_MOD_EXPORT
	int clientIPAddr,		//client IP address
	char *pathNamep, 		//pathname
	int	pathNameLen,		//pathname name length
	int exportMask			//bit mask of the export option
							//MOUNTD_OPT_RO: read-only
							//MOUNTD_OPT_RW: read-write
							//MOUNTD_OPT_NO_ROOT_SQUASH
							//MOUNTD_OPT_ROOT_SQUASH	
);							

extern uint32 mountd_trace_enable;

void mountd_nfsExportStateChangeReq(int32 msgType, mnt_infoReq_t *mnt_infoReqp);
void mountd_enableAllExports(void);
int mountd_enableExport(mnt_infoReq_t   *mnt_infoReqp);
void mountd_disableAllExports(void);
int mountd_disableExport(mnt_infoReq_t   *mnt_infoReqp);
extern mnt_infoCB_t *mountd_findEntryByName (mnt_infoReq_t   *mnt_infoReqp);

/*-----------------------------------------------------------------
 * Name:   	mountd_cmdDisplayInfoList 
 *
 * Description:
 *  Display current export entries.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
void mountd_cmdDisplayInfoList(void)
{
	char            *pathNamep;
	char            *exportInfop;
	mnt_infoCB_t    *currp;
	int             count;
	int             displayEntries;
	char            pathName[VOL_MAX_NAME];
	char            exportInfo[MAX_NAME_LEN];

	currp = mountd_globalInfop->mnt_infoHeadp;
	count = mountd_globalInfop->numOfEntries;
	displayEntries = 0;
	while ((currp != NULL) && (count > 0 ) )
	{
		memset(&pathName[0], 0, sizeof(pathName));
		memset(&exportInfo[0], 0, sizeof(exportInfo));
		pathNamep = currp->mntInfoEntry.pathName;
		exportInfop = pathNamep + currp->mntInfoEntry.pathNameLen;
		memcpy(&pathName[0], pathNamep, currp->mntInfoEntry.pathNameLen);
		memcpy(&exportInfo[0],
				exportInfop, currp->mntInfoEntry.exportInfoLen);
		if (currp->mntInfoEntry.volId.whole_thing != 0)
		{
			displayEntries++;
			printf("%s\n", &pathName[0]);
			if (currp->mntInfoEntry.exportInfoLen)
				printf("\t%s\n", &exportInfo[0]);
		}
		count--;
		currp = currp->nextp;
	}
	printf("Number of current export entries %d:\n", displayEntries);
}

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportAddReq 
 * 
 * Description:
 * Send message to NFS on TxRx for share add request.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/08/2002
 *
 *-----------------------------------------------------------------
 */
void
mountd_nfsExportAddReq(mnt_infoEntry_t *mntInfoEntryp)
{
	char	msgBuf[sizeof(mnt_exportAddReq_t)+ MAX_NAME_LEN];
	mnt_exportAddReq_t	*mnt_eaReqp;
	mnt_infoEntry_t		*mnt_infoEntryp;
	int					msgLen, stat;

	memset(msgBuf, 0, sizeof(msgBuf));

	mnt_eaReqp = (mnt_exportAddReq_t *)msgBuf;
	mnt_infoEntryp = &mnt_eaReqp->mntInfoEntry;

	mnt_eaReqp->msgHdr.msgType = MNT_EXPORT_ADD_REQ;
	mnt_eaReqp->msgHdr.private = 0;

	mnt_infoEntryp->localIpAddr      = mntInfoEntryp->localIpAddr;
	mnt_infoEntryp->hostAddr         = mntInfoEntryp->hostAddr;
	mnt_infoEntryp->exportInfoLen    = mntInfoEntryp->exportInfoLen;
	mnt_infoEntryp->exportOptionMask = mntInfoEntryp->exportOptionMask;
	mnt_infoEntryp->pathNameLen      = mntInfoEntryp->pathNameLen;
	memcpy(&mnt_infoEntryp->pathName[0], 
			&mntInfoEntryp->pathName[0], 
			mntInfoEntryp->pathNameLen);

	msgLen = sizeof(mnt_exportAddReq_t) + mntInfoEntryp->pathNameLen + 1;

	stat = sendAgileMsg (-1, mnt_eaReqp, msgLen,
				MNT_SRVR_APP_NAME, NFX_NFP_TXRX1, NFX_TXRX_SLOT, MGMTPLANE,
				0, NULL, 0);
	if  (stat < 0)     
	{
		printf("mountd_nfsExportAddReq: sendAgileMsg failed!\n");
	}   

	return;
}

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportDeleteReq 
 * 
 * Description:
 * Send message to NFS on TxRx for share delete request.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/08/2002
 *
 *-----------------------------------------------------------------
 */
void
mountd_nfsExportDeleteReq(char		*pathNamep,
						  uint32	localIpAddr,
						  uint32	clientIpAddr)
{
	char	msgBuf[sizeof(mnt_exportDeleteReq_t) + MAX_NAME_LEN];
	mnt_exportDeleteReq_t	*mnt_edReqp;
	int						msgLen, stat;

	MNT_DBG_PRINTF("mountd_nfsExportDeleteReq: "
					"pathNamep: %s, clientIpAddr: 0x%x\n", 
					pathNamep, clientIpAddr);
	mnt_edReqp = (mnt_exportDeleteReq_t *)msgBuf;

	if (pathNamep == NULL)
	{
		//"nfs share delete all" request
		mnt_edReqp->msgHdr.msgType = MNT_EXPORT_DELETE_ALL_REQ;
		mnt_edReqp->nameLen = 0;
		mnt_edReqp->pathName[0] = 0;
	}
	else
	{
		mnt_edReqp->msgHdr.msgType = MNT_EXPORT_DELETE_REQ;
		mnt_edReqp->nameLen = strlen(pathNamep);
		strcpy(&mnt_edReqp->pathName[0], pathNamep);
	}
	mnt_edReqp->localIpAddr = localIpAddr;
	mnt_edReqp->clientIpAddr = clientIpAddr;

	mnt_edReqp->msgHdr.private = 0;

	msgLen = sizeof(mnt_exportDeleteReq_t) + mnt_edReqp->nameLen + 1;

	stat = sendAgileMsg (-1, mnt_edReqp, msgLen,
				MNT_SRVR_APP_NAME, NFX_NFP_TXRX1, NFX_TXRX_SLOT, MGMTPLANE,
				0, NULL, 0);
	if  (stat < 0)     
	{
		printf("mountd_nfsExportDeleteReq: sendAgileMsg failed!\n");
	}   

	return;
}

/*-----------------------------------------------------------------
 * Name:    mountd_nfsExportStateChangeReq 
 * 
 * Description:
 * Send message to NFS on TxRx for filesystem state change
 * from enabled to disabled or vice versa.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/01/2002
 *
 *-----------------------------------------------------------------
 */
void
mountd_nfsExportStateChangeReq(int32 msgType, mnt_infoReq_t *mnt_infoReqp)
{
	char	msgBuf[sizeof(mnt_exportStateChangeReq_t) + MAX_NAME_LEN];
	mnt_exportStateChangeReq_t	*mnt_escReqp;
	int							msgLen, stat;

	mnt_escReqp = (mnt_exportStateChangeReq_t *)msgBuf;

	if ((msgType == MNT_EXPORT_ENABLED_REQ) ||
		(msgType == MNT_EXPORT_DISABLED_REQ))
	{
		if (!mnt_infoReqp)
			/*Error*/
			return;

		mnt_escReqp->nameLen = mnt_infoReqp->nameLen;
		strcpy(&mnt_escReqp->pathName[0], &mnt_infoReqp->pathName[0]);
	}
	else
	{
		mnt_escReqp->nameLen = 0;
		mnt_escReqp->pathName[0] = 0;
	}

	mnt_escReqp->msgHdr.private = 0;
	mnt_escReqp->msgHdr.msgType = msgType;

	msgLen = sizeof(mnt_exportStateChangeReq_t) + mnt_escReqp->nameLen + 1;

	stat = sendAgileMsg (-1, mnt_escReqp, msgLen,
				MNT_SRVR_APP_NAME, NFX_NFP_TXRX1, NFX_TXRX_SLOT, MGMTPLANE,
				0, NULL, 0);
	if  (stat < 0)     
	{
		printf("mountd_nfsExportStateChangeReq: sendAgileMsg failed!\n");
	}   

	return;
}

/*-----------------------------------------------------------------
 * Name:   	mountd_exportStateChange
 *
 * Description:
 * Enable or disable input shares based on pathname.
 *
 * Created by:      Xiaobing Zhang
 *
 * Date Created:    05/20/2002
 * 
 *-----------------------------------------------------------------
 */
int32	//Return number of entries changed
mountd_exportStateChange(mnt_infoReq_t *mnt_infoReqp, int32 state)
{
	mnt_infoCB_t    *currp;
	int32			count;
	char 			*inputPathNamep;
	int 			inputPathLen;
	int				foundEntryCnt = 0;

	if (mnt_infoReqp == NULL)
		return (foundEntryCnt);

	inputPathNamep = &mnt_infoReqp->pathName[0];
	inputPathLen = mnt_infoReqp->nameLen;
	if (*inputPathNamep == '/')
	{
		//eat leading '/'
		inputPathNamep++;
		inputPathLen--;
	}
	currp = mountd_globalInfop->mnt_infoHeadp;
	count = mountd_globalInfop->numOfEntries;
	
	MNT_DBG_PRINTF("mountd_exportStateChange: Entries: %d\n", count);
	MNT_DBG_PRINTF("mountd_exportStateChange: "
					"inputPathNamep: %s inputPathLen: %d\n", 
					inputPathNamep, inputPathLen);

	while ((currp != NULL) && (count > 0))
	{
		if ( (currp->mntInfoEntry.pathNameLen == inputPathLen) &&
			 (!(strncmp(currp->mntInfoEntry.pathName, inputPathNamep, inputPathLen))) )
		{
			currp->mntInfoEntry.state = state;
			foundEntryCnt++;
		}	
			
		count--;
		currp = currp->nextp;
	}

	return (foundEntryCnt);
}

/*-----------------------------------------------------------------
 * Name:    mountd_enableAllExports
 * 
 * Description:
 * Enable all exports.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
void mountd_enableAllExports(void)
{
	mnt_infoCB_t    *currp;
	int             count;
	evm_volId_t     volId;

	//printf("mountd_enableAllExports enter\n");
	count = mountd_globalInfop->numOfEntries;
	for (currp = mountd_globalInfop->mnt_infoHeadp;
		 ((currp != NULL) && (count > 0) );
		 count--, currp = currp->nextp)
	{
		currp->mntInfoEntry.state = MNT_SHARE_EXPORT_ENABLED;
        //Send message to TXRX to get update Filehandle
        volId = mountd_getId(&currp->mntInfoEntry.pathName[0],
							currp->mntInfoEntry.pathNameLen);
		if (volId.whole_thing <= 0)
		{
			/*
			MOUNTD_ELOG(class_1, err_s,
					mountd_globalInfop->mountd_AppId,
					0, 0, 0,
					(MOUNTD_BAD_VOLID_STR, MOUNTD_STR,
					&mnt_infoReqp->pathName[0]));
			*/
			continue;
        }
		//Get update filehandle for dir pathName 
		mountd_getExportFileHandle (&currp->mntInfoEntry.pathName[0],
					currp->mntInfoEntry.pathNameLen, volId);
	}

	//Send 'enable all' message to NFS on TxRx
	mountd_nfsExportStateChangeReq(MNT_EXPORT_ENABLED_ALL_REQ, NULL);
}

/*-----------------------------------------------------------------
 * Name:   	mountd_enableExport 
 *
 * Description:
 * Enable an Input share.
 *
 * Created by:      Joseph Vu
 * Modified by:		Xiaobing Zhang
 *
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
int mountd_enableExport(mnt_infoReq_t   *mnt_infoReqp)
{
	int				foundEntryCnt;
	int				status;
	evm_volId_t     volId;
	int		        pathLen;
	char			*pathNamep;

	//printf("mountd_enableExport: pathName: %s len: %d\n", 
	//			mnt_infoReqp->pathName, mnt_infoReqp->nameLen);

	foundEntryCnt = mountd_exportStateChange(mnt_infoReqp,
											MNT_SHARE_EXPORT_ENABLED);
	MNT_DBG_PRINTF("mountd_enableExport: foundEntryCnt: %d\n", foundEntryCnt);

	if (foundEntryCnt == 0)
		return (MOUNTD_ERR_NO_EXPORT_FOUND);

	//Send 'enable' message to NFS on TxRx
	mountd_nfsExportStateChangeReq(MNT_EXPORT_ENABLED_REQ, mnt_infoReqp);

	//check pathName
	pathNamep = &mnt_infoReqp->pathName[0];
	pathLen = mnt_infoReqp->nameLen;
    /*
     *Eat leading '/'
     */
    if (*pathNamep && *pathNamep == '/')
    {
        pathNamep += 1;
        pathLen -= 1;
	}

	//Send message to TXRX to get update Filehandle
	volId = mountd_getId(pathNamep, 
					pathLen);
	if (volId.whole_thing <= 0)
	{
		return (MOUNTD_ERR_VOL_NOT_FOUND);
	}
	status = mountd_getExportFileHandle (pathNamep,
				   	pathLen, volId);
	if (status != NFX_OK)
        MNT_DBG_PRINTF("mountd_enableExport: Failure to send update fileHandle message to TXRX!\n");
	
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	mountd_disableAllExports 
 * 
 * Description:
 * Disable all exports.
 * 
 * Created by:      Joseph Vu
 * 
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
void mountd_disableAllExports(void)
{
    mnt_infoCB_t    *currp;
    int             count;

	//printf("mountd_disableAllExports enter\n");
    count = mountd_globalInfop->numOfEntries;
    for (currp = mountd_globalInfop->mnt_infoHeadp;
         ((currp != NULL) && (count > 0) );
         count--, currp = currp->nextp)
    {
        currp->mntInfoEntry.state = MNT_SHARE_EXPORT_DISABLED;
	}

	//Send 'disable all' message to NFS on TxRx
	mountd_nfsExportStateChangeReq(MNT_EXPORT_DISABLED_ALL_REQ, NULL);
}

/*-----------------------------------------------------------------
 * Name:   	mountd_disableExport 
 *
 * Description:
 * Disable an Input share.
 *
 * Created by:      Joseph Vu
 * Modified by:		Xiaobing Zhang
 *
 * Date Created:    08/9/01
 * 
 *-----------------------------------------------------------------
 */
int mountd_disableExport(mnt_infoReq_t   *mnt_infoReqp)
{
	int				foundEntryCnt;

	//printf("mountd_disableExport: Enter\n");
	foundEntryCnt = mountd_exportStateChange(mnt_infoReqp,
											MNT_SHARE_EXPORT_DISABLED);
	MNT_DBG_PRINTF("mountd_disableExport: foundEntryCnt: %d\n", foundEntryCnt);

	if (foundEntryCnt == 0)
		return (MOUNTD_ERR_NO_EXPORT_FOUND);

	//Send 'disable' message to NFS on TxRx
	mountd_nfsExportStateChangeReq(MNT_EXPORT_DISABLED_REQ, mnt_infoReqp);

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	mountd_getExportFileHandle 
 *
 * Description:
 * Send a request to TXRX to get the filehandle for 
 * the input share directory (mount point).
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    07/2/02
 * 
 *-----------------------------------------------------------------
 */
int
mountd_getExportFileHandle (
	char *pathNamep, 
	int	pathNameLen, 
	evm_volId_t volId)
{
	char	msgBuf[sizeof(mnt_getFileHandleReq_t) + MAX_NAME_LEN + 1];
	mnt_getFileHandleReq_t	*mnt_getFHandleReqp;
	char					respbuf[1024];
	mnt_infoReq_t			*mnt_infoReqp;
	int						msgLen, stat;
	struct timeval      	timeout = { 20, 0};



	//Validate input pathname & pathname len
	if (!pathNamep)
		return (NFX_ERR);
	MNT_DBG_PRINTF("mountd_getExportFileHandle: pathName: %s len: %d\n", 
				pathNamep, pathNameLen);

	if  ( (pathNameLen  > MAX_NAME_LEN) ||
		(pathNameLen < 0) )
	{
		MNT_DBG_PRINTF("mountd_getExportFileHandle: Bad pathname length (%d)\n",
				pathNameLen);
		return (NFX_ERR);
	}
	//Init response buffer
	mnt_infoReqp = (mnt_infoReq_t	*)respbuf;
	//memset (respbuf, 0, sizeof(respbuf));
	bzero (respbuf, sizeof(respbuf));

	//set up message to send to NFS server on TXRX
	//memset(msgBuf, 0, sizeof(msgBuf));
	bzero(msgBuf, sizeof(msgBuf));
	mnt_getFHandleReqp = (mnt_getFileHandleReq_t *)msgBuf;

	mnt_getFHandleReqp->nameLen = pathNameLen;
	memcpy(&mnt_getFHandleReqp->pathName[0], 
			pathNamep, pathNameLen);

	//Set volume ID
	mnt_getFHandleReqp->volId = volId;
	mnt_getFHandleReqp->msgHdr.private = 0;
	mnt_getFHandleReqp->msgHdr.msgType = MNT_EXPORT_GET_FHANDLE_REQ;

	msgLen = sizeof(mnt_getFileHandleReq_t) + 
			pathNameLen + 1;

	stat = sendAgileMsg (-1, mnt_getFHandleReqp, msgLen,
				MNT_SRVR_APP_NAME, NFX_NFP_TXRX1, NFX_TXRX_SLOT, 
				MGMTPLANE,
				&timeout, respbuf, sizeof(respbuf));
	if  (stat < 0)     
	{
		printf("mountd_getExportFileHandle 1: Get update filehandle failed!\n");
		return (NFX_ERR);
	}
	if (mnt_infoReqp->msgHdr.status != NFX_OK)
	{
		printf("mountd_getExportFileHandle 2: Get update filehandle failed!\n");
		return (NFX_ERR);
	}
		
	MNT_DBG_PRINTF("mountd_getExportFileHandle 3-OK: volId: %llx inode: %llx mntPnt: %llx\n", 
				mnt_infoReqp->volId.whole_thing, 
				mnt_infoReqp->inode, mnt_infoReqp->mntPnt);

	//Update NFS share/mount point with update filehandle
	mountd_updateFileHandle(mnt_infoReqp);
	return (NFX_OK);
}


