/* 
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-data-func.c
 *
 * RCS:  $Id: ndmp-data-func.c,v 1.78 2002/12/10 18:40:23 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: 
 *
 * Created by: Anh Vu
 *
 * Date Created:        9/06/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include "ndmp.h"
#include "ndmp-data-sm.h"
#include "ndmp-data-v2-sm.h"
#include "ndmp-mover-sm.h"
#include <exports/fs-ndmp-api.h>
#include "ndmp-fs-api.h"
#include "ndmp-debug.h"

extern ndmp_NNIMCtrl_t ndmp_NNIMCtrl;
extern int ndmp_fp_msgStats[];

int fill_Data_get_state_info (Data_t *data, ndmp_data_get_state_rsp *gs_rsp, 
			esm_handle_t handle);
int ndmp_sndNotifHalted(NdmpNIMService_t *servp, int32 halt_reason);

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_data_init
 *
 * Description:     Device init state machine initialization function.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    09/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
void
ndmp_data_init(Data_t *data)
{

	// Data_t already zeroed out when session's created

	//memset(data, 0, sizeof(Data_t));

    data->oper = NDMP_DATA_OP_NOACTION;  /* current operation */
    data->start_ops_state = NDMP_OPS_STATE_NONE;  /* current ops state */
    data->state = NDMP_DATA_STATE_IDLE;  /* current state */
    data->addr_type = NDMP_ADDR_LOCAL; 	 /* address for connection to mover */
    data->connSock = -1;                 /* socket to remote mover, provided by Mover */
    data->listenSock = -1;               /* socket for listening for remote mover connections */
    data->haltReas = NDMP_DATA_HALT_NA;/* current reason */
	data->first_fs_rd_req = TRUE;		 /* first time do FS read request, for RESTORE */

} /* ndmp_data_init */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_data_create
 *
 * Description:     Device init state machine initialization function.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    09/17/01 
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int32
ndmp_Data_create(esm_handle_t   handle,     /* handle for this sm inst    */
                 int32          nextState,  /* next state from table      */
                 esm_event_t    thisEvent,  /* event driving this action  */
                 void           *userInfo)  /* ctrl blk linked to sm      */
{
    NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;
    Data_t                 *data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO,("Ndmp_data_create:   Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	ndmp_data_init(data);

    return(nextState);

} /* ndmp_Data_create */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_listen
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_listen(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXRsp_t          *rsp;
    ndmp_data_listen_rsp   *listen_rsp;
    ndmp_data_listen_req   *listen_req;
    ndmp_NFXReq_t          *req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t       *servp = (NdmpNIMService_t *) userInfo;
    Data_t                 *data = &servp->Data;
	int32				   rc, dst_port = servp->ssc_src_port, msglen;

    NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_listen:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

    /*
     * Create reject message and send it.
     */

	msglen = sizeof(ndmp_NFXRsp_t) + sizeof(ndmp_data_listen_rsp) - 1;
    NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp LISTEN");
    if (rsp == NULL)
	{
        ndmp_sendCmdRsp(req, dst_port, NDMP_UNDEFINED_ERR, 0);
        NDMP_FP_FREE(req, "msgp LISTEN"); /* msgp in ndmp_NNIMReceiveMsg */
        return esm_getCurrState(handle);
	}
	listen_rsp = (ndmp_data_listen_rsp *) rsp->content;

    rsp->hdr.sessId  = req->hdr.sessId;
    rsp->hdr.mid     = req->hdr.mid;
    rsp->ndmp_mid    = req->ndmp_mid;
	memcpy(&rsp->cl_hdr, &req->cl_hdr, sizeof(ndmp_Hdr));


    listen_req = (ndmp_data_listen_req *) req->content;

    listen_rsp->error = NDMP_NO_ERR;

    switch (listen_req->addr_type)
    {
        case NDMP_ADDR_LOCAL:
            listen_rsp->data_conn_addr.addr_type = NDMP_ADDR_LOCAL;
			data->listenSock = 0;
			data->dataConnAddr = 0;
			data->dataConnPort = 0;
            break;
        case NDMP_ADDR_TCP:
#if 0
            port = getFreePort();
            address = getlocalIPAddr();

			// Talked to Su on 4/9/02, if port specified is 0, he'll give
			// me an arbitrary one.

            ndmp_tpl_bindReq(port);
            listen_rsp->data_conn_addr.addr_type = NDMP_ADDR_TCP;
            listen_rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr = address;
            listen_rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port = port;
			data->dataConnAddr = address;
			data->dataConnPort = port;
#endif
            break;
    }
	data->connMode = NDMP_CONN_MODE_LISTEN;
    rc = eee_sendMessage((void *) rsp, msglen, dst_port, ndmp_NNIMCtrl.src_port);
    if (rc != NFX_OK)
    {
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_listen: eee_sendMessage failed! sessid %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_listen: eee_sendMessage failed! sessid %d\n", servp->sessId));
    } else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_DATA_LISTEN_RSP_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_DATA_LISTEN_RSP_SNT]++;
#endif
	}
   	data->addr_type = listen_req->addr_type;
    NDMP_FP_FREE(rsp, "nfxrsp LISTEN");
	NDMP_FP_FREE(req, "msgp LISTEN");		/* it was msgp in ndmp_NNIMReceiveMsg() */

    return nextState;

} /* ndmp_Data_listen */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_conn
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_conn(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_NFXReq_t   *req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    ndmp_data_conn_req   *conn_req = (ndmp_data_conn_req *) req->content;
    NdmpNIMService_t   *servp = (NdmpNIMService_t *) userInfo;
    Data_t         	*data = &servp->Data;
    int32           dst_port = servp->ssc_src_port, rc;
    esm_event_t     esmEvent;

    NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_conn:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

    switch (conn_req->addr.addr_type)
    {
        case NDMP_ADDR_LOCAL:
            /* Mover service has to be listening for Data to connect to, yeh? */
            if (esm_getCurrState(data->mover_smHdl) != NDMP_MOVER_STATE_LISTEN)
            {
                ndmp_sendCmdRsp(req, dst_port, NDMP_ILLEGAL_STATE_ERR, 0);
                NDMP_FP_FREE(req, "msgp DATA_CONN");
                return esm_getCurrState(handle);
            }

			/* Reply to client saying we're connected */
			ndmp_sendCmdRsp(req, dst_port, NDMP_NO_ERR, 0);

			/* send event to signal Mover to be in CONNECTED mode as well */
            bzero(&esmEvent, sizeof(esm_event_t));
            esmEvent.event = NDMP_MOVER_EVENT_CONNECTED;
#ifdef NFX_SMP
            rc = esm_deliverEvent(data->mover_smHdl, &esmEvent);
#else
            rc = esm_deliverEvent(servp->Mover.my_smHdl, &esmEvent);
#endif
     		if (rc != NFX_OK)
     		{
#ifdef NFX_SMP
         		NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_conn: ERROR, cannot post event %d to SM hdl %d",
                      	NDMP_MOVER_EVENT_CONNECTED, data->mover_smHdl));
         		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_conn: ERROR, cannot post event %d to SM hdl %d",
                      	NDMP_MOVER_EVENT_CONNECTED, data->mover_smHdl));
#else
         		NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_conn: ERROR, cannot post event %d to SM hdl %d",
                      	NDMP_MOVER_EVENT_CONNECTED, servp->Mover.my_smHdl));
         		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_conn: ERROR, cannot post event %d to SM hdl %d",
                      	NDMP_MOVER_EVENT_CONNECTED, servp->Mover.my_smHdl));
#endif
     		}

			/* We are now connected (locally) of course */
            NDMP_FP_FREE(req, "msgp DATA_CONN");
			return NDMP_DATA_STATE_CONNECTED;

        case NDMP_ADDR_TCP:
#if 0
    		rc = ndmp_tplBindReq(conn_req->addr.ndmp_addr_u.tcp_addr.ip_addr,
								conn_req->addr.ndmp_addr_u.tcp_addr.port, 
								TPL_BIND_CLIENT);
    		if (rc != NFX_OK)
    		{
                NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_conn:  ndmp_tplBindReq failed w/ %d\n", rc);
    			ndmp_sendCmdRsp(req, dst_port, NDMP_CONNECT_ERR, 0));
				return nextState;
    		}
        	/* Wait for addConnInd from TPL, will invoke CONNECTED event */

			data->dataConnAddr = conn_req->addr.ndmp_addr_u.tcp_addr.ip_addr;
			data->dataConnPort = conn_req->addr.ndmp_addr_u.tcp_addr.port;
			data->NFXReq = req;
#endif
            break;
        default:
            // shouldn't be here, already handled this err over SSC
    		NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_conn:  Invalid addr type %d, sessId %d\n", 
								conn_req->addr.addr_type, servp->sessId));
    		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_Data_conn:  Invalid addr type %d, sessId %d\n", 
								conn_req->addr.addr_type, servp->sessId));
    }

    NDMP_FP_FREE(req, "msgp DATA_CONN");
    return nextState;

} /* ndmp_Data_conn */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_connected
 *
 * Description:     TPL AddConnInd signal this event.  See what connection mode
 *                  we were in and do the right thing.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_connected(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
    Data_t                      *data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_connected:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

    if (data->connMode == NDMP_CONN_MODE_LISTEN)
    {
    } else
        // is NDMP_CONN_MODE_CONNECT
    {
#if 0
		// Remote backup setup
        /* TPL told us we've connected to the remote server,
         * reply NDMP client w/ his CONN request.
         */
        ndmp_sendCmdRsp(data->NFXReq, dst_port, NDMP_NO_ERR, 0);
        data->connMode = NDMP_CONN_MODE_CONNECT;
        data->addr_type = conn_req->addr.addr_type;
#endif
    }
    return nextState;

} /* ndmp_Data_connected */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_doBkupRecovErr
 *
 * Description:     Send
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_doBkupRecovErr(NdmpNIMService_t *servp, uint32 error)
{
	Data_t      *data = &servp->Data; 


    // Reply to NDMP client either indicating error
    // or saying that the backup has started

    ndmp_sendCmdRsp(data->ops_req, servp->ssc_src_port, error, 0);

	// release the original NDMP_START_xxx_REQ
    NDMP_FP_FREE(data->ops_req, "ops_req BKUP/RST");
	data->ops_req = NULL;

    if ((error != NDMP_NO_ERR))
    {
        ndmp_free_env(data);
        ndmp_free_flist(data);
    }                               

    return NFX_OK;

} /* ndmp_doBkupRecovErr */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_sndNamesReq
 *
 * Description:     We've sent a FS_REQ_REST_SETUP Req and now get its response.
 *                  Send a FS_REQ_REST_NAMES Req if things look ok, else reply
 *                  to NDMP client that a recovery cannot be started.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    12/20/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_sndNamesReq(NdmpNIMService_t *servp, int32 i)
{
    int32                       ret, msglen, old_nameLen, new_nameLen;
    Data_t                      *data = &servp->Data;
    fs_MsgRestoreNamesReq_t     *fs_rest_names_req;
 
    NDMP_DBG_PRINTF(D_INFO,("ndmp_sndNamesReq:  Begin\n"));                                   
	old_nameLen = strlen(data->fil_lst[i].orig_path);

	/* Legato will always fill in the dest_dir for file_list restore,
	 * whether you restore from same vol1 to vol1 or vol1 to vol2.
	 * BUT, if you restore the whole volume and do NOT specify
	 * the MOUNT point in the GUI, then it will leave the dest_dir
	 * EMPTY.  Jish!
	 */
    new_nameLen = strlen(data->fil_lst[i].dest_dir);

	msglen = sizeof(fs_MsgRestoreNamesReq_t) + 
					old_nameLen + new_nameLen + 1; /* really is (+2-1) */
    NDMP_FP_ALLOC(fs_rest_names_req, fs_MsgRestoreNamesReq_t, msglen, "fs_rest_names_req");
    if (fs_rest_names_req == NULL)
	{
   		NDMP_DBG_PRINTF(D_INFO,("ndmp_sndNamesReq:  Error w/ mallocing\n"));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
						("ndmp_sndNamesReq:  Error w/ mallocing\n"));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
	fs_rest_names_req->hdr.requestType = FS_REQ_REST_NAMES;
	fs_rest_names_req->hdr.len = msglen;	// len of whole msg
	fs_rest_names_req->volId.whole_thing = data->d_volId;
   	fs_rest_names_req->names.ndmpSessId = servp->sessId;
	fs_rest_names_req->names.oldLen = old_nameLen;
	fs_rest_names_req->names.newLen = new_nameLen;
		
    strcpy(fs_rest_names_req->names.msgBlock, data->fil_lst[i].orig_path);
	strcpy(&fs_rest_names_req->names.msgBlock[old_nameLen], data->fil_lst[i].dest_dir);	

	NDMP_DBG_PRINTF(D_INFO, 
			("ndmp_sndNamesReq:  msglen = %d, volId = %llx, oldname = %s,"
			" newname = %s\n"
			"ndmp_sndNamesReq:  fs_rest_names_req: \n----------------\nvolId: %llx\n"
			"old_name: %s\nold_name_len: %d\nnew_name: %s\nnew_name_len: %d\n", 
			msglen, data->d_volId, data->fil_lst[i].orig_path,
			data->fil_lst[i].dest_dir + 1, 
			fs_rest_names_req->volId.whole_thing, fs_rest_names_req->names.msgBlock, 
			fs_rest_names_req->names.oldLen, 
			&fs_rest_names_req->names.msgBlock[fs_rest_names_req->names.oldLen],
			fs_rest_names_req->names.newLen));

	/* send a RESTORE_NAMES msg to FS for each file to be restored. */
    ret = eee_sendMessage((void *) fs_rest_names_req, msglen, servp->fp_src_port, ndmp_NNIMCtrl.src_port);

    NDMP_FP_FREE(fs_rest_names_req, "fs_rest_names_req");

    if (ret != NFX_OK)
    {
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndNamesReq: eee_sendMessage failed! sessId %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sndNamesReq: eee_sendMessage failed! sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
    } 
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RST_NAMES_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_RST_NAMES_REQ_SNT]++;
#endif
	(data->flist_cur_indx)++;	

	return NFX_OK;
} /* ndmp_sndNamesReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_sndNameLst
 *
 * Description:     
 *
 * Created by:      Nitin Jain
 *
 * Data Created:    
 *
 *------------------------------------------------------------------------------
 */
int32
ndmp_Data_sndNameLst(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{

    NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;
	ndmp_NFXReq_t 	 *req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
	Data_t			 *data = &servp->Data;
	ndmp_NFXSendNamesReq_t *nameReq = (ndmp_NFXSendNamesReq_t *)req->content;
	int32			 sessId = req->hdr.sessId;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_sndNameLst:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));
    if (!req)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_sndNameLst: req NULL\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_send_fs_names_list: req NULL\n"));
        ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
        goto sndNameLst_done;
    }

	sessId = servp->sessId;
    servp->moreNames = req->userInfo;
    if (ndmp_save_flist(&nameReq->data[0], nameReq->num_files, data) == NFX_ERR)
    {
       NDMP_DBG_PRINTF(D_INFO, ("ndmp_send_fs_names_list: save_flist failed, sessId %d.\n", servp->sessId));
       E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
                        ("ndmp_send_fs_names_list: save_flist failed, sessId %d.\n", servp->sessId));
       ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
       goto sndNameLst_done;
    }
    data->flist_cur_indx = 0;

    ndmp_sndNamesReq(servp, data->flist_cur_indx);
    data->start_ops_state = NDMP_OPS_STATE_REST_NAMES_SENT;

sndNameLst_done:

	NDMP_FP_FREE(thisEvent.p1.u.p, "msgp sndNameLst");
	return nextState;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procRestSetupRsp
 *
 * Description:		We've sent a FS_REQ_REST_SETUP Req and now get its response.
 *					Send a FS_REQ_REST_NAMES Req if things look ok, else reply
 *					to NDMP client that a recovery cannot be started.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    12/20/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_procRestSetupRsp(NdmpNIMService_t *servp, fs_MsgRestoreSetupResp_t *fsrsp)
{
	Data_t  *data = &servp->Data;
    ndmp_NFXRsp_t rsp;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestSetupRsp:  Begin\n"));

	/* If FS indicates error, reply to client right away */
	if (data->start_ops_state != NDMP_OPS_STATE_REST_SETUP_SENT) 
	{ 
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestSetupRsp:  op_state not REST_SETUP_SENT\n"));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_procRestSetupRsp:  op_state not REST_SETUP_SENT\n"));
		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
	}
	if (fsrsp->fsStatus != FS_ERR_NONE)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestSetupRsp:  fsStatus error = %d\n", fsrsp->fsStatus));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_procRestSetupRsp:  fsStatus error = %d\n", fsrsp->fsStatus));
		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
	}

    /* Send message to SSC */
    memset(&rsp, 0, sizeof(rsp));
    rsp.hdr.sessId  = ((ndmp_NFXmsgHdr_t *)data->ops_req)->sessId;
    rsp.hdr.mid     = NDMP_MID_DATA_START_RECOVER_RSP;
    rsp.ndmp_mid    = NDMP_MID_DATA_START_RECOVER_RSP;

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestSetupRsp:  Send start_recov_rsp bk to SSC\n"));
    eee_sendMessage((uchar8 *)&rsp, sizeof(rsp), servp->ssc_src_port,
                    ndmp_NNIMCtrl.src_port);

    return NFX_OK;

} /* ndmp_procRestSetupRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procRestNameRsp
 *
 * Description:		FS has responded to our Names request msg.  If we have
 *					more Names request to send (based on the file list to
 *					be restored, we send a Names request for each file in 
 *					the file list), send it, otherwise, send a restore Start 
 *					message to FS to really begin this restore.  Once FS acked 
 *					the START msg, we'll send a reply to the NDMP client's 
 *					NDMP_DATA_START_RECOVER request.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int32
ndmp_procRestNameRsp(NdmpNIMService_t *servp, fs_MsgRestoreNamesResp_t *rsp)
{
	Data_t                      *data = &servp->Data;
    fs_MsgStartRestoreReq_t     fs_rest_start_req;
	int32						ret, msglen, error = NDMP_UNDEFINED_ERR;


	if (data->start_ops_state != NDMP_OPS_STATE_REST_NAMES_SENT) 
	{
    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestNamesRsp:  Rcvd rest_NAMES_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));
    	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
			("ndmp_procRestNamesRsp:  Rcvd rest_NAMES_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));

		return ndmp_doBkupRecovErr(servp, error);
	}

    /* If FS indicates error, reply to client right away, else send
	 * another NAMES msg if there is more in the restore list
	 * or send a START msg if there is none left.
	 */
    if  (rsp->fsStatus == FS_ERR_NONE)
	{
		/* If this is the last NAMES msg, send the START request
		 * else, send another NAMES request.
		 */
		if (data->flist_cur_indx < data->flist_len)	
		{
			ndmp_sndNamesReq(servp, data->flist_cur_indx);
		} else {
            if (!servp->moreNames)
            {
    		    /*-- Send FS Restore START command --*/
    		    msglen = sizeof(fs_MsgStartRestoreReq_t);
    		    memset(&fs_rest_start_req, 0, msglen);
    		    fs_rest_start_req.hdr.requestType = FS_REQ_REST_START;
    		    fs_rest_start_req.hdr.len = msglen;
    		    fs_rest_start_req.start.ndmpSessId = servp->sessId;
    		    fs_rest_start_req.volId.whole_thing = data->d_volId;
    		    ret = eee_sendMessage((void *) &fs_rest_start_req, msglen, servp->fp_src_port, ndmp_NNIMCtrl.src_port);
    		    if (ret != NFX_OK)
    		    {
        		    NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestNameRsp: eee_sendMessage failed! sessId %d\n", servp->sessId));
        		    E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								    ("ndmp_procRestNameRsp: eee_sendMessage failed! sessId %d\n", servp->sessId));
				    return ndmp_doBkupRecovErr(servp, error);
    		    }
#ifdef NFX_SMP
				ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RST_START_REQ_SNT], 1);
#else
			    ndmp_fp_msgStats[FS_RST_START_REQ_SNT]++;
#endif
			    data->start_ops_state = NDMP_OPS_STATE_REST_START_SENT;
            }
            else
            {
                ndmp_NFXSendNamesRsp_t rsp;
                rsp.mid = NDMP_MID_DATA_SEND_NAME_RSP;
                rsp.sessId = servp->sessId;
                eee_sendMessage((void *)&rsp, sizeof(rsp), servp->ssc_src_port, ndmp_NNIMCtrl.src_port);
            }
            ndmp_free_flist(data);
		}
	} else {
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestNamesRsp:  FS indicates error,"
				"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("ndmp_procRestNamesRsp:  FS indicates error,"
				"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));

    	// Reply to NDMP client indicating error
		return ndmp_doBkupRecovErr(servp, error);
	}

	return NFX_OK;
}   /* ndmp_procRestNameRsp */ 
/*  
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procRestStartRsp 
 *
 * Description:     Got a response for our restore Start message sent to FS earlier.  
 *					We can now send a reply to the client's NDMP_START_RECOV_REQ
 *					based on FS's status.
 *  
 * Created by:      Anh Vu
 *  
 * Data Created:    12/20/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */ 
int 
ndmp_procRestStartRsp(NdmpNIMService_t *servp, fs_MsgRestoreStartResp_t *rsp)
{   
	
    Data_t  *data = &servp->Data;
	int32	error;

	if (data->start_ops_state != NDMP_OPS_STATE_REST_START_SENT) 
	{
		error = NDMP_UNDEFINED_ERR;

    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestStartRsp:  Rcvd rest_START_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));
    	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
			("ndmp_procRestStartRsp:  Rcvd rest_START_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));
	} else {

    	/* If FS indicates error, reply to client right away */
		error = (rsp->fsStatus == FS_ERR_NONE)? 
			NDMP_NO_ERR:  NDMP_ILLEGAL_ARGS_ERR;	

		if (error == NDMP_NO_ERR)
		{
    		E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
				("ndmp_procRestStartRsp:  RESTORE begins, sessId %d\n", servp->sessId));
		} else {
    		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procRestStartRsp:  FS indicates error."
					"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));
    		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_procRestStartRsp:  FS indicates error."
					"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));
		}
	}
   	// Reply to NDMP client either indicating error
	return ndmp_doBkupRecovErr(servp, error);

}	/* ndmp_procRestStartRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_recov
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    11/16/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_recov(NdmpNIMService_t *servp, ndmp_NFXReq_t *req)
{
    ndmp_NFXRecovReq_t          *recov_req=(ndmp_NFXRecovReq_t *)req->content;
    Data_t                      *data = &servp->Data;
    Mover_t                     *mover = &servp->Mover;
    int32                       dst_port, error, msglen;
    fs_MsgSetupRestoreReq_t     *fs_rest_setup_req = NULL;
	int32						tot_env_data_len, 
								o_path_len = 0, 
                                cwdLen = 0; 
	char8						ret, *fs_var = NULL, 
								*o_path_ptr, 
								*o_ptr = NULL, *recursive, *bkupSiz;
                                // *d_path_ptr,*d_full_path=NULL,*d_ptr=NULL,
    char8                       cwd[1024], *d;
	int64						d_volId;

	/* For local recovery here */
	error = NDMP_NO_ERR;
	dst_port = servp->fp_src_port;
	data->ops_req = (void *) req;

	tot_env_data_len = recov_req->tot_env_data_len;
    bzero(cwd, sizeof(char)*1024);
    d = recov_req->data;
    cwdLen = *((int32 *)d);
    d += 4;
    memcpy(cwd, d, cwdLen);
	NDMP_INT_ALIGN(cwdLen);
    d += cwdLen;
	/* Save the environment variables sent to us by NDMP client */
	if (ndmp_save_env(d, recov_req->num_env, data) == NFX_ERR)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov:  save_env failed, sessId %d.\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_recov:  save_env failed, sessId %d.", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
    d += tot_env_data_len;

	/* Get the FILESYSTEM env. var's value */
	if ((fs_var = ndmp_getEnvVar("FILESYSTEM", data->env_lst, data->envLen)) == NULL)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov:  getenv FILESYSTEM failed, sessId %d.\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_recov:  getenv FILESYSTEM failed, sessId %d.\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
    } 

	/* Get the BKUPSIZ env. var's value */
	if ((bkupSiz = ndmp_getEnvVar(BKUP_SIZ_ENV_STR, data->env_lst, data->envLen)) == NULL)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov:  getenv BKUPSIZ failed, sessId %d.\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_recov:  getenv BKUPSIZ failed, sessId %d.\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
    } 
#ifdef NFX_SMP
	data->recov_bkupSiz = strtoq(bkupSiz, NULL, 0);
	NFX_SPIN_LOCK(&(mover->spin));
	mover->bytesLeftToRead = data->recov_bkupSiz;
	NFX_SPIN_UNLOCK(&(mover->spin));
#else
	data->recov_bkupSiz = mover->bytesLeftToRead = strtoq(bkupSiz, NULL, 0);
#endif

	msglen = sizeof(fs_MsgSetupRestoreReq_t) + cwdLen;

    NDMP_FP_ALLOC(fs_rest_setup_req, fs_MsgSetupRestoreReq_t, msglen, "fs_rest_setup_req");
    if (fs_rest_setup_req == NULL)
	{
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov:  Error w/ mallocing, sessId %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_recov:  Error w/ mallocing, sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}

	fs_rest_setup_req->hdr.requestType = FS_REQ_REST_SETUP;
	fs_rest_setup_req->restore.ndmpSessId  = servp->sessId;

	/* */
	o_ptr = fs_var+1;						// pass the 1st '/', ex: "/vol1/engr/users"
	o_path_ptr = strchr(o_ptr, '/');		// scan pass volname, strchr ret. NULL if find no '/'
											// else will point to '/'
	o_path_len =  (o_path_ptr != NULL)? strlen(o_path_ptr): 0;

	if ((recursive = ndmp_getEnvVar("RECURSIVE", data->env_lst, data->envLen)) != NULL)
	{ 
		fs_rest_setup_req->restore.recursiveDescent = 
				((*recursive == 'y') || (*recursive == 'Y') || 
				(*recursive == 't')|| (*recursive == 'T') )? TRUE: FALSE;
	} else  {
		fs_rest_setup_req->restore.recursiveDescent = TRUE;
	}	

	d_volId = recov_req->dst_fsvolId;
    fs_rest_setup_req->volId.whole_thing = d_volId;		// get volId of dest_dir

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov:  dest vol id 0x%llx\n", d_volId));
			
	/* We can set ioSize as 8K (max EEE can handle) later as enhancement */
#ifdef NFX_SMP
	fs_rest_setup_req->restore.ioSize = data->recSiz;
#else
	fs_rest_setup_req->restore.ioSize = servp->Mover.recSiz;
#endif
	fs_rest_setup_req->restore.numBytes = data->recov_bkupSiz;
	fs_rest_setup_req->restore.cwdLen = cwdLen;
    strcpy(fs_rest_setup_req->restore.cwd, cwd);
	
	/*-- Send FS Restore SETUP command. --*/
	fs_rest_setup_req->hdr.len = msglen;
	ret = eee_sendMessage((void *) fs_rest_setup_req, msglen, dst_port, ndmp_NNIMCtrl.src_port);

    NDMP_FP_FREE(fs_rest_setup_req, "fs_rest_setup_req");

    if (ret != NFX_OK)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_recov: eee_sendMessage failed!  sessId %d\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_recov: eee_sendMessage failed!  sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
    }
#ifdef NFX_SMP
        ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RST_SETUP_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_RST_SETUP_REQ_SNT]++;
#endif

	/* Save for sending FS_REST_NAMES_REQ later */
	data->o_path_ptr = o_path_ptr;	//o_path_ptr is from FILESYSTEM env
	data->o_path_len = o_path_len;
	data->d_volId = d_volId;

	data->start_ops_state = NDMP_OPS_STATE_REST_SETUP_SENT; 

	return NFX_OK;
} /* ndmp_recov */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_sndBkupStartReq
 *
 * Description:     Send a bkup Start message to FS to really begin this
 *                  bkup.  Once he acked this msg, we'll send a reply to
 *                  the client's NDMP_START_BACKUP_REQ.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int32
ndmp_sndBkupStartReq(NdmpNIMService_t *servp)
{
    fs_MsgStartDumpReq_t        fs_dmp_start_req;
    Data_t                      *data = &servp->Data;
    int32                       msglen, rc;
 
    /*-- Send FS Dump START command --*/
    msglen = sizeof(fs_MsgStartDumpReq_t);
    memset(&fs_dmp_start_req, 0, msglen);
    fs_dmp_start_req.hdr.requestType = FS_REQ_DUMP_START;
    fs_dmp_start_req.hdr.len = msglen;
    fs_dmp_start_req.start.ndmpSessId = servp->sessId;
    fs_dmp_start_req.volId.whole_thing = servp->volId.whole_thing;
    rc = eee_sendMessage((void *) &fs_dmp_start_req, msglen, servp->fp_src_port, ndmp_NNIMCtrl.src_port);
    if (rc != NFX_OK)
    {
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndBkupStartReq: eee_sendMessage failed!  sessId %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sndBkupStartReq: eee_sendMessage failed!  sessId %d\n", servp->sessId));
    } else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_DMP_START_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_DMP_START_REQ_SNT]++;
#endif
    	data->start_ops_state = NDMP_OPS_STATE_BKUP_START_SENT;
    	return NFX_ERR;
	}
 
    return NFX_OK;
 
} /* ndmp_sndBkupStartReq */             
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_procBkupSetupRsp
 *
 * Description:		We've sent a FS_REQ_DUMP_SETUP Req and now get its response.
 *					Send a FS_REQ_DUMP_NAMES Req if things look ok, else reply
 *					to NDMP client that a bkup cannot be started.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    12/20/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_procBkupSetupRsp(NdmpNIMService_t *servp, fs_MsgDumpSetupResp_t *rsp)
{
	int32						rc, msglen;
	Data_t                      *data = &servp->Data;
	fs_MsgDumpNamesReq_t		*fs_dmp_names_req;


    NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp:  Begin\n"));


	if (data->start_ops_state != NDMP_OPS_STATE_BKUP_SETUP_SENT) 
	{
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp:  Receive msg in wrong "
					"state %d, sessId %d\n", data->start_ops_state, servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_procBkupSetupRsp:  Receive msg in wrong "
					"state %d, sessId %d\n", data->start_ops_state, servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
	/* If FS indicates error, reply to client right away */
	if (rsp->fsStatus != FS_ERR_NONE)
	{
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp:  FS return error, "
					"sessId %d\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_procBkupSetupRsp:  FS return error, "
					"sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
	}

	/* For incremental backup, we don't need to send FS a
	 * NAMES message.
	 */
	if (data->dd.dump_level > 0 || data->partial_bkup)
	{
    	/*-- Send FS Dump START command --*/
		return ndmp_sndBkupStartReq(servp);
	}
		
    /*-- Send FS Dump NAME command --*/
    msglen = sizeof(fs_MsgDumpNamesReq_t) + data->path_len +1;
    NDMP_FP_ALLOC(fs_dmp_names_req, fs_MsgDumpNamesReq_t, msglen, "fs_dmp_names_req");
    if (fs_dmp_names_req == NULL)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp:  Error w/ mallocing, sessId %d\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_procBkupSetupRsp:  Error w/ mallocing, sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
    }
    fs_dmp_names_req->hdr.requestType = FS_REQ_DUMP_NAMES;
    fs_dmp_names_req->hdr.len = msglen; // len of whole msg
    fs_dmp_names_req->names.ndmpSessId = servp->sessId;
	fs_dmp_names_req->volId.whole_thing = servp->volId.whole_thing;
    fs_dmp_names_req->names.len = data->path_len+1;           // len of msgBlock + NULL
    if (data->path_ptr)
    {
        strcpy(fs_dmp_names_req->names.msgBlock, data->path_ptr);
    } else {
        // no path after filesystem name so either send a '/' indicating
        // full backup or according to Rahul, don't send him a names_req msg at all
        // he'll understand it as a default full bkup
        strcpy(fs_dmp_names_req->names.msgBlock, "/");
    }
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp:  Send FS_REQ_DUMP_NAMES\n"
	 	"msglen %d, volId.whole_thing 0x%llx, name len%d, name %s\n",
		msglen, servp->volId.whole_thing, fs_dmp_names_req->names.len,  
		fs_dmp_names_req->names.msgBlock));

    rc = eee_sendMessage((void *) fs_dmp_names_req, msglen, servp->fp_src_port, ndmp_NNIMCtrl.src_port);
    if (rc != NFX_OK)
    {
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupSetupRsp: eee_sendMessage failed!  sessId %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_procBkupSetupRsp: eee_sendMessage failed!  sessId %d\n", servp->sessId));
    } else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_DMP_NAMES_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_DMP_NAMES_REQ_SNT]++;
#endif
	}

    NDMP_FP_FREE(fs_dmp_names_req, "fs_dmp_names_req");
	data->start_ops_state = NDMP_OPS_STATE_BKUP_NAMES_SENT; 

    return NFX_OK;

} /* ndmp_procBkupSetupRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procBkupNameRsp
 *
 * Description:		Send a bkup Start message to FS to really begin this
 *					bkup.  Once he acked this msg, we'll send a reply to
 *					the client's NDMP_START_BACKUP_REQ.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int32
ndmp_procBkupNameRsp(NdmpNIMService_t *servp, fs_MsgDumpNamesResp_t *rsp)
{
	int32						rc;
    Data_t  					*data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupNameRsp:  Begin\n"));

   	/* If FS indicates error, reply to client right away, else send
 	 * a START msg.
 	 */
	if ((data->start_ops_state == NDMP_OPS_STATE_BKUP_NAMES_SENT) &&
    	(rsp->fsStatus == FS_ERR_NONE))
	{
    	/*-- Send FS Dump START command --*/
		rc = ndmp_sndBkupStartReq(servp);

	} else {
    	// Reply to NDMP client either indicating error
		
    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupNameRsp:  FS reject Names Req.  FS status = %d\n", rsp->fsStatus));
    	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_procBkupNameRsp:  FS reject Names Req.  FS status = %d\n", rsp->fsStatus));
		rc = ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
	return rc;

}   /* ndmp_procBkupNameRsp */ 
/*  
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procBkupStartRsp 
 *
 * Description:     Got a response to our bkup Start message to FS.  
 *					We can now send a reply to the client's NDMP_START_BACKUP_REQ
 *					based on FS's status.
 *  
 * Created by:      Anh Vu
 *  
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */ 
int 
ndmp_procBkupStartRsp(NdmpNIMService_t *servp, fs_MsgDumpStartResp_t *rsp)
{   
	
    Data_t  *data = &servp->Data;
	int32	error;

	if (data->start_ops_state != NDMP_OPS_STATE_BKUP_START_SENT)
	{
    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupStartRsp:  Rcvd dump_START_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));
    	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
			("ndmp_procBkupStartRsp:  Rcvd dump_START_Rsp "
			"in wrong state = %d, sessId %d\n", data->start_ops_state, servp->sessId));
		error = NDMP_UNDEFINED_ERR;

	} else {

    	/* If FS indicates error, reply to client right away */
		error = (rsp->fsStatus == FS_ERR_NONE)? 
				NDMP_NO_ERR:  NDMP_ILLEGAL_ARGS_ERR;	

		if (error == NDMP_NO_ERR)
		{
    		E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
				("ndmp_procBkupStartRsp:  BACKUP Begins, sessId %d\n", servp->sessId));
		} else {
    		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procBkupStartRsp:  FS indicates error."
					"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));
    		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_procBkupStartRsp:  FS indicates error."
					"  FS status = %d, sessId %d\n", rsp->fsStatus, servp->sessId));
		}
	}
   	/* Reply to NDMP client either indicating error */
	return  ndmp_doBkupRecovErr(servp, error);


}	/* ndmp_procBkupStartRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_opsRsp
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_opsRsp(esm_handle_t handle,
                    int32 nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void  *userInfo) /* ctrl blk linked to sm       */
{
	uint32                *mid = (uint32 *) thisEvent.p1.u.p;
    NdmpNIMService_t      *servp = (NdmpNIMService_t *) userInfo;
    Data_t                *data = &servp->Data;
	int32				  currState = esm_getCurrState(handle);
	int32				  state = currState, rc;


    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_opsRsp:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));
	switch(data->oper)
	{
	  case NDMP_DATA_OP_BACKUP:
		switch (*mid)
		{
    		case FS_REQ_DUMP_SETUP:
				ndmp_procBkupSetupRsp(servp, thisEvent.p1.u.p);        
				break;
			case FS_REQ_DUMP_NAMES:
				ndmp_procBkupNameRsp(servp, thisEvent.p1.u.p);        
				break;
			case FS_REQ_DUMP_START:
				ndmp_procBkupStartRsp(servp, thisEvent.p1.u.p);        
				state = nextState;
				break;
			default:
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_opsRsp:  Invalid FS Bkup response %d\n", *mid));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_Data_opsRsp:  Invalid FS Bkup response %d\n", *mid));
				rc = ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
		}
		break;
	  case NDMP_DATA_OP_RESTORE:
        switch (*mid)
        {
            case FS_REQ_REST_SETUP:
                ndmp_procRestSetupRsp(servp, thisEvent.p1.u.p);
                break;
            case FS_REQ_REST_NAMES:
                ndmp_procRestNameRsp(servp, thisEvent.p1.u.p);
                break;
            case FS_REQ_REST_START:
                ndmp_procRestStartRsp(servp, thisEvent.p1.u.p);
                state = nextState;
                break;
            default:
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_opsRsp:  Invalid FS Restore response %d\n", *mid));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_Data_opsRsp:  Invalid FS Restore response %d\n", *mid));
				rc = ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
        }
		break;
	  default:  // NDMP_DATA_OP_NOACTION

		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_opsRsp:  Invalid State (not in BKUP or REST operation) to get FS response %d, state %d\n", 
								*mid, data->oper));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_opsRsp:  Invalid State (not in BKUP or REST operation) to get FS response %d, state %d\n", 
						 *mid, data->oper));
	}
	
    NDMP_FP_FREE(thisEvent.p1.u.p, "msgp opsRsp");
	return state;

} /* ndmp_Data_opsRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_bkup
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_bkup (NdmpNIMService_t *servp, ndmp_NFXReq_t *req)
{
    dmpdate_t                   *ddate =  (dmpdate_t *) (req->content);
    ndmp_NFXBkupReq_t           *bkup_req =  
				(ndmp_NFXBkupReq_t *) (req->content + req->len1);
    Data_t                      *data = &servp->Data;
    int32                       dst_port, error, msglen;
    fs_MsgSetupDumpReq_t        *fs_dmp_setup_req = NULL;
    char8                       *fs_var, *ptr,
                                *fh, *recursive, *level_str, *path_ptr = NULL,
                                *fs_blob;
    int32                       rc, level, fs_name_len;

	/* For local backup here */
	error = NDMP_NO_ERR;
	data->ops_req = (void *) req;
	dst_port = servp->fp_src_port;
	data->path_ptr = NULL;
	data->path_len = 0;

	/* Save environment variables so that when bkup process is done, NDMP
	 * client will send a GET_ENV and a GET_STATE to us asking for all
	 * all this info.
	 */
	msglen = sizeof(fs_MsgSetupDumpReq_t);
	if (ndmp_save_env(bkup_req->env_data, bkup_req->num_env, data) == NFX_ERR)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_bkup:  Error in save_env, sessId %d\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_bkup:  Error in save_env, sessId %d\n", servp->sessId));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
	/* Figure out level first, see if need to alloc space for snapshot */
    level = 0;
	if ((level_str = ndmp_getEnvVar("LEVEL", data->env_lst, data->envLen)) != NULL)
	{
        if (strlen(ddate->snapshot) != 0)
		    level = atoi(level_str);
	}
    data->dd.dump_level = level;           // save the dump level for later use
	msglen += (level)? FS_DUMP_MAX_SNAP_LEN: 0;	

	if ((fs_var = ndmp_getEnvVar("FILESYSTEM", data->env_lst, data->envLen)) == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_bkup:  Error in getEnvVar FILESYSTEM, sessId %d\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_bkup:  Error in getEnvVar FILESYSTEM, sessId %d\n", servp->sessId));
        NDMP_FP_FREE(fs_dmp_setup_req, "fs_dmp_setup_req");

		return ndmp_doBkupRecovErr(servp, NDMP_ILLEGAL_ARGS_ERR);
	} 
	// Get path w/o the volname
	ptr = fs_var+1;		// pass the 1st '/', ex: "/0x10a9700/engr/users"
						// or "/0x10a9700"
	path_ptr = strchr(ptr, '/'); // scan pass volid, ret. NULL if find no '/'

	fs_name_len = (path_ptr == NULL)? strlen(ptr): (path_ptr - ptr);
	data->path_len =   (path_ptr == NULL)? 0: (strlen(ptr) - fs_name_len) ;
	data->path_ptr = path_ptr;	// save for later use in FS_REQ_DUMP_NAMES

    msglen += (data->path_len)?data->path_len:0;

    NDMP_FP_ALLOC(fs_dmp_setup_req, fs_MsgSetupDumpReq_t, msglen, "fs_dmp_setup_req");
    if (fs_dmp_setup_req == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_bkup:  Error in malloc\n"));
		return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
	}
    fs_blob = fs_dmp_setup_req->dump.blob;

	fs_dmp_setup_req->hdr.requestType = FS_REQ_DUMP_SETUP;
	fs_dmp_setup_req->dump.ndmpSessId = servp->sessId;
	fs_dmp_setup_req->dump.ndmp_ssc_src_port = servp->ssc_src_port;

	fs_dmp_setup_req->dump.cwdLen = 0;
    data->partial_bkup = 0;
    if (data->path_len)
    {
        if (level == 0) data->partial_bkup = 1;
        fs_dmp_setup_req->dump.cwdLen = data->path_len;
        memcpy(fs_blob, path_ptr, data->path_len);
        fs_blob += data->path_len;
    }

	servp->volId.whole_thing = bkup_req->fs_volId;
	fs_dmp_setup_req->volId.whole_thing = bkup_req->fs_volId;


	if ((fh = ndmp_getEnvVar("HIST", data->env_lst, data->envLen)) != NULL)
	{
		fs_dmp_setup_req->dump.fileHistory = data->do_file_hist = 
				((*fh == 'y') || (*fh == 'Y') || 
				(*fh == 'T') || (*fh == 't'))? TRUE: FALSE;
	} else 
	{
		fs_dmp_setup_req->dump.fileHistory = data->do_file_hist = FALSE;
	}

	if ((recursive = ndmp_getEnvVar("RECURSIVE", data->env_lst, data->envLen)) != NULL)
	{ 
		fs_dmp_setup_req->dump.recursiveDescent = 
				((*recursive == 'y') || (*recursive == 'Y') || 
				(*recursive == 't')|| (*recursive == 'T') )? TRUE: FALSE;
	} else 
		fs_dmp_setup_req->dump.recursiveDescent = TRUE;

	fs_dmp_setup_req->dump.labelLen = 0;		// unused right now
	if (level > 0)
	{
		NDMP_DBG_PRINTF(D_INFO, ("Increm. Bkup -  dumpdate info:\n"
								"------------------------------\n"
						"date: %d\nclient-ip: 0x%x\ndump_level: %d\nsnapshot: %s\n", 
						ddate->date, ddate->client_ip, ddate->dump_level, ddate->snapshot));

		fs_dmp_setup_req->dump.level = level;

		/* We have incremental backup, get the previous SnapShot info */
		fs_dmp_setup_req->dump.prevSnapTime = ddate->date;
		fs_dmp_setup_req->dump.prevSnapNameLen = strlen(ddate->snapshot);
		memcpy(fs_blob, ddate->snapshot, strlen(ddate->snapshot));
        fs_blob += fs_dmp_setup_req->dump.prevSnapNameLen;
	} else 
	{
		fs_dmp_setup_req->dump.level = 0;
		fs_dmp_setup_req->dump.prevSnapNameLen = 0;		// unused right now
	}
	fs_dmp_setup_req->dump.labelLen = 0;		// unused right now
#ifdef NFX_SMP
	fs_dmp_setup_req->dump.recSize = data->recSiz;  
#else
	fs_dmp_setup_req->dump.recSize = servp->Mover.recSiz;  
#endif
								// so FS can give us a buf this big in each i/o write req	

	/* Don't know if this is needed.
	 * For our system we'll always update dumpdate, talked to Rahul
 	 * about this already, has to do w/ us using snapshot.
	 */
	data->upd_dumpdate = TRUE;
	
	/*-- Send FS Dump SETUP command. --*/
	fs_dmp_setup_req->hdr.len = msglen;
   	rc = eee_sendMessage((void *) fs_dmp_setup_req, msglen, dst_port, ndmp_NNIMCtrl.src_port);
    if (rc != NFX_OK)
    {
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_bkup: eee_sendMessage failed!  sessId %d\n", servp->sessId));
       	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_bkup: eee_sendMessage failed!  sessId %d\n", servp->sessId));
    } else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_DMP_SETUP_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_DMP_SETUP_REQ_SNT]++;
#endif
	}
    NDMP_FP_FREE(fs_dmp_setup_req, "fs_dmp_setup_req");

	data->start_ops_state = NDMP_OPS_STATE_BKUP_SETUP_SENT; 

	return NFX_OK;
} /* ndmp_bkup */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_bkup_rcov
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_bkup_rcov(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_NFXReq_t   			*req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t   			*servp = (NdmpNIMService_t *) userInfo;
    Data_t         				*data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_bkup_rcov:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	if (req->hdr.mid == NDMP_MID_DATA_START_BACKUP)
	{
		if (ndmp_bkup(servp, req) == NFX_OK)
		{
    		data->oper = NDMP_DATA_OP_BACKUP;  /* current operation */
		} else {
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_bkup_rcov:  Error in BKUP, sessid = %d\n", servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_Data_bkup_rcov:  Error in BKUP, sessid = %d\n", servp->sessId));
		}	
	} else
	{
		if (ndmp_recov(servp, req) == NFX_OK)
		{
    		data->oper = NDMP_DATA_OP_RESTORE;  /* current operation */
		} else {
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_bkup_rcov:  Error in RESTORE, sessid = %d\n", servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_Data_bkup_rcov:  Error in RESTORE, sessid = %d\n", servp->sessId));
		}
	}

	/* req is saved for later use and will be freed in ndmp_data_rel_resource() func */
    return nextState;
	

} /* ndmp_Data_bkup_rcov */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_sndFsIODone 
 *
 * Description:		Send FS an IO Write or Read response.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/10/02
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_sndFsIODone(	NdmpNIMService_t *servp, 
					int32 reqTyp, 		// NDMP_MID_WRITE_DATA_REQ or NDMP_MID_READ_DATA_REQ
					ndmp_fs_info_t *fsInfo,
					//char *data_ptr,		// pertains to NDMP_MID_READ_REQ only
					eee_desc_t *ebuf_ptr,		// pertains to NDMP_MID_READ_REQ only
										// it's data to append to a READ request
					int32 dataLen)		// len of data returning to FS
{
    fs_MsgDumpIODoneReq_t       fs_dmp_iodone_req;
    fs_MsgRestoreIODoneReq_t    fs_rest_iodone_req;
	Data_t                      *data = &servp->Data;
	int32						rc, status;
	uint64                      sequence;

    //NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone:\n"));

	sequence = (fsInfo == NULL)? 0: fsInfo->ioCookie;
	status = (fsInfo == NULL)? FS_ERR_IO: fsInfo->fs_status;
	
    if (reqTyp == NDMP_MID_WRITE_DATA_REQ)
    {
        memset(&fs_dmp_iodone_req, 0, sizeof(fs_MsgDumpIODoneReq_t));
        fs_dmp_iodone_req.hdr.requestType = FS_REQ_DUMP_IODONE;
        fs_dmp_iodone_req.hdr.len = sizeof(fs_MsgDumpIODoneReq_t);
        fs_dmp_iodone_req.iodone.ndmpSessId = servp->sessId;
        fs_dmp_iodone_req.iodone.fsCookie= data->fsCookie;
        fs_dmp_iodone_req.volId.whole_thing = servp->volId.whole_thing;
        fs_dmp_iodone_req.iodone.sequence = sequence;
        fs_dmp_iodone_req.iodone.status = status;

        rc = eee_sendMessage((void *) &fs_dmp_iodone_req,
            sizeof(fs_MsgDumpIODoneReq_t), servp->fp_src_port, ndmp_NNIMCtrl.src_port);
    	if (rc != NFX_OK)
    	{
       		NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone: eee_sendMessage failed!  "
									"sessId %d\n", servp->sessId));
       		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_sndFsIODone: eee_sendMessage failed!  sessId %d\n", 
					 servp->sessId));
    	} else {
#ifdef NFX_SMP
			ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_WR_DATA_RSP_SNT], 1);
#else
			ndmp_fp_msgStats[FS_WR_DATA_RSP_SNT]++;
#endif
		}

    } else if (reqTyp == NDMP_MID_READ_DATA_REQ) {

#if 0	// whenever eee works w/ >8 k
		int bufCount, totalSiz, reqSiz;
		eee_desc_t 					*desc;
#endif

        memset(&fs_rest_iodone_req, 0, sizeof(fs_MsgRestoreIODoneReq_t));
		fs_rest_iodone_req.hdr.requestType = FS_REQ_REST_IODONE;
		fs_rest_iodone_req.hdr.len = sizeof(fs_MsgRestoreIODoneReq_t);
		fs_rest_iodone_req.iodone.ndmpSessId = servp->sessId;
		fs_rest_iodone_req.iodone.fsCookie= data->fsCookie;
		fs_rest_iodone_req.volId.whole_thing = data->d_volId;
        fs_rest_iodone_req.iodone.sequence = sequence;
        fs_rest_iodone_req.iodone.status = status;
		fs_rest_iodone_req.iodone.dataLen = dataLen;

		/* Large i/o fix:  Passing FS the pointer to the scsi msg
		 * and offset to get to the 60k data buffer - 04/04/02 
		 */
		if (ebuf_ptr)
		{
			fs_rest_iodone_req.iodone.data = (void *)ebuf_ptr;

#if 0  // EEE_BUFF_REST
			fs_rest_iodone_req->iodone.data_offset = sizeof(scsi_readRspMsg_t);
#endif            
		}
        rc = eee_sendMessage((void *) &fs_rest_iodone_req,
               sizeof(fs_MsgRestoreIODoneReq_t), servp->fp_src_port, ndmp_NNIMCtrl.src_port);
    	if (rc != NFX_OK)
    	{
       		NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone: eee_sendMessage failed!  sessId %d\n", servp->sessId));
       		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sndFsIODone: eee_sendMessage failed!  sessId %d\n", servp->sessId));
    	} else {
#ifdef NFX_SMP
			ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RD_DATA_RSP_SNT], 1);
#else
			ndmp_fp_msgStats[FS_RD_DATA_RSP_SNT]++;
#endif
		}
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone:  Sent FS Restore IODONE, rc = %d, ioCookie %lld\n", 
								rc, sequence));


#if 0 // for later when eee can handle > 8k. 04/04/02
		desc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
		if (desc == NULL)
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone:   eee_allocateDesc return NULL\n"));
        	return NFX_ERR;
		}
		bzero(desc, sizeof(eee_desc_t));

		/* Allocate buffer for request message which will go into buf 1 */
		fs_rest_iodone_req  = (fs_MsgRestoreIODoneReq_t *) eee_allocateBuffer(EEE_SHARED_POOL,
                                                      EDESC_BD_SMALL, 0);
       	fs_rest_iodone_req->hdr.requestType = FS_REQ_REST_IODONE;
       	fs_rest_iodone_req->hdr.len = msglen;
       	fs_rest_iodone_req->iodone.cookie = (void *)servp->sessId;
       	fs_rest_iodone_req->iodone.fsCookie= data->fsCookie;
       	fs_rest_iodone_req->volId.whole_thing = data->d_volId;
       	fs_rest_iodone_req->iodone.status = fs_status;
       	fs_rest_iodone_req->iodone.dataLen = dataLen;

		reqSiz = sizeof(fs_MsgRestoreIODoneReq_t);
		desc->bd[1].buf = (void *) fs_rest_iodone_req;
		desc->bd[1].len = reqSiz;
		desc->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_SMALL;

		bufCount = 1;

        if (dataLen)
        {
			/* Buf 2 have ptr to data */
			desc->bd[2].buf = data_ptr;
        	desc->bd[2].len = dataLen;
        	desc->bd[2].control = EDESC_BD_CMD_IN_USE | 
					EDESC_BD_CMD_NO_DEALLOC | EDESC_BD_LARGE;

            bufCount = 2;
		}
		totalSiz = reqSiz + dataLen;
	
		desc->hdr.control = (bufCount << EDESC_BUFCNT_SHIFT) | totalSiz;
       	desc->hdr.dest_port = servp->fp_src_port;
       	desc->hdr.src_port = ndmp_NNIMCtrl.src_port;

		// this will use our bd[1] as xmt buffer, no bcopy done.
		rc = eee_forwardPacket(desc);
       	if (rc != EEE_FORWARDED_PACKET) {
			if (rc != EEE_DROP_PACKET)
               	eee_freePacket(desc);
           	NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone: eee_forwardPacket failed.\n"));
           	return(NFX_ERR);
       	}
#endif

    } else {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndFsIODone:  Invalid request type %d, sessid %d\n", reqTyp, servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sndFsIODone:  Invalid request type %d, sessid %d\n", reqTyp, servp->sessId));
		return NFX_ERR;
	}

	return NFX_OK;

} /* ndmp_sndFsIODone */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_FsIOReq
 *
 * Description:		Process FS request to read/write data from/to Mover server
 *					(for local) or data connection (for remote).
 *
 * Created by:      Anh Vu
 *
 * Data Created:    11/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_FsIOReq(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
    Data_t                      *data = &servp->Data;
	char          				*msgp = (char *)thisEvent.p1.u.p;
	uint32						*msgTyp = (uint32 *) msgp, fs_buf_siz;
	ndmp_NFXReq_t           	*req;
	ndmp_mover_read_req			*rd_req;
	uint64						offset;
	int32						rc, msglen, currState, active;
    esm_event_t                 esmEvent;
	ndmp_fs_info_t				fsInfo, *rd_info;



	currState = esm_getCurrState(handle);
	active = (servp->version == NDMP_VER_2)? 
					NDMP_DATA_V2_STATE_ACTIVE : NDMP_DATA_STATE_ACTIVE;

	/* Backup process */
	if (*msgTyp == NDMP_MID_WRITE_DATA_REQ)
	{
       	ndmp_write_data_req_msg_t *wr_msgp;

		wr_msgp = (ndmp_write_data_req_msg_t *)msgp;

		/* If we're aborting OR we're in the non-ACTIVE state, 
		 * and FS just sent us an IO request, reply to it w/ errors right away 
		 * so it can stop its processing.
		 */
		if ((data->abortFlag == TRUE) || (currState != active))
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIOReq: Bkup - Aborting or not in "
							"ACTIVE state, ignore request w/ ioCookie %lld, sessId %d\n", 
							wr_msgp->ioCookie, servp->sessId));
          	NDMP_FP_FREE(msgp, "msgp WR_DATA_REQ");
			return nextState;
		}	

		if (data->addr_type == NDMP_ADDR_LOCAL)
		{
			if (ndmp_loop_bk)
			{
				/* Pretend that IO was written so FS can measure disk time */

#ifdef NFX_SMP
				NFX_SPIN_LOCK(&servp->Mover.spin);
				servp->Mover.dataWritten += data->recSiz;
				NFX_SPIN_UNLOCK(&servp->Mover.spin);
#else
				servp->Mover.dataWritten += servp->Mover.recSiz;
#endif

				fsInfo.fs_status = FS_ERR_NONE;
				fsInfo.ioCookie = wr_msgp->ioCookie;
				if (ndmp_sndFsIODone(servp, NDMP_MID_WRITE_DATA_REQ, 
					&fsInfo, NULL, 0) == NFX_ERR)
				{
					NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIOReq:  "
					"Err sending FS WRITE IO response\n"));
					return esm_getCurrState(handle);
				}
				return nextState;
			}

	        /* Send data to Mover so it can write/buffer this 1k of 
			 * buffer to be written to tape (LB).
			 */ 
        	servp->Data.fsCookie = wr_msgp->cl_cookie;

        	fs_buf_siz = NDMP_FS_WR_BUFSIZ;
            bzero(&esmEvent, sizeof(esm_event_t));
            esmEvent.event = NDMP_MOVER_EVENT_WRITE_DATA;
            esmEvent.p1.u.p = msgp;
#ifdef NFX_SMP
            if ((rc = esm_deliverEvent(data->mover_smHdl,
#else
            if ((rc = esm_deliverEvent(servp->Mover.my_smHdl,
#endif
                                        &esmEvent)) != NFX_OK)
     		{
         		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_procFsReq: ERROR, cannot post event NDMP_MOVER_EVENT_WRITE_DATA to SM hdl %d, sessId %d",
                      data->mover_smHdl, servp->sessId));
         		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					  ("ndmp_Data_procFsReq: ERROR, cannot post event NDMP_MOVER_EVENT_WRITE_DATA to SM hdl %d, sessId %d",
                      data->mover_smHdl, servp->sessId));
                NDMP_FP_FREE(msgp, "msgp WR_DATA_REQ");
     		}
		} else if (data->addr_type == NDMP_ADDR_TCP)  /* it's TCP */
		{
			/* Xmt this data to the TCP data connection */
		} else
		{
			return esm_getCurrState(handle);	
		}
	} else 		/* Restore process */
	{
       	ndmp_read_data_req_msg_t *rd_msgp;

		rd_msgp = (ndmp_read_data_req_msg_t *)msgp;
		
		/* If we're aborting, and FS just sent us an IO request, reply
		 * to it w/ errors right away so it can stop its processing.
		 */
		if ((data->abortFlag == TRUE) || (currState != active))
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIOReq: Restore - Aborting or not in "
						"ACTIVE state, ignore request w/ ioCookie %lld-%lld, sessId %d\n", 
						rd_msgp->ioCookie, (rd_msgp->ioCookie + rd_msgp->numIOs), servp->sessId));
          	NDMP_FP_FREE(msgp, "msgp RD_DATA_REQ");
		  	return nextState;
		}
		if (data->addr_type == NDMP_ADDR_LOCAL)
		{
			/* printf("data_fsIoReq:  bytesProcessed = %lld, recov_bkupSiz = %lld\n",
						data->bytesProcessed, data->recov_bkupSiz);
			 */
			if (ndmp_loop_bk)
			{
				/* Pretend that IO was written so FS can measure disk time */

#ifdef NFX_SMP
				NFX_SPIN_LOCK(&servp->Mover.spin);
				servp->Mover.bytesLeftToRead -= data->recSiz;
				NFX_SPIN_UNLOCK(&servp->Mover.spin);
#else
				servp->Mover.bytesLeftToRead -= servp->Mover.recSiz;
#endif
				fsInfo.fs_status = FS_ERR_NONE;
				fsInfo.ioCookie = rd_msgp->ioCookie;
				if (ndmp_sndFsIODone(servp, NDMP_MID_READ_DATA_REQ, 
					&fsInfo, NULL, 0) == NFX_ERR)
				{
					NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIOReq:  "
							"Err sending FS READ IO response\n"));
					return esm_getCurrState(handle);
				}
				return nextState;
			}
			if (data->first_fs_rd_req == TRUE)
			{
				offset = 0;
				data->first_fs_rd_req = FALSE;
			} else {
				offset = servp->Data.position;
			}
			/* Local, so we don't have to ask NDMP client to send
			 * Mover server the MOVER_READ, we do it ourselves.
			 */
			msglen = sizeof(ndmp_NFXReq_t) + sizeof (ndmp_mover_read_req);	
            NDMP_FP_ALLOC(req, ndmp_NFXReq_t, msglen, "nfxreq READ_DATA");
            req->hdr.mid = NDMP_MID_READ_DATA_REQ;
			rd_req = (ndmp_mover_read_req *) req->content;	

			/* Formulate MOVER_READ req */
			rd_req->offset.low = offset; // position relative to data stream
			rd_req->offset.high = offset >> 32;

#ifdef NFX_SMP
			rd_req->length.low = data->recSiz;  
#else
			rd_req->length.low = servp->Mover.recSiz;  
#endif
			rd_req->length.high = 0;

            bzero(&esmEvent, sizeof(esm_event_t));
            esmEvent.event = NDMP_MOVER_EVENT_READ_DATA;
            esmEvent.p1.u.p = req;
//New performace code
            NDMP_FP_ALLOC(rd_info, ndmp_fs_info_t, sizeof(ndmp_fs_info_t), "fs_info_t READ_DATA");
            rd_info->ioCookie = rd_msgp->ioCookie;
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIOReq:  rcvd req for ioCookie %lld rcvd\n", 
									rd_msgp->ioCookie));
            rd_info->numIOs = rd_msgp->numIOs;
            esmEvent.p2.u.p = rd_info;
            if ((rc = esm_deliverEvent(data->mover_smHdl, &esmEvent)) != NFX_OK)
            {
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_procFsReq: ERROR, cannot post event" 
					"NDMP_MOVER_EVENT_READ_DATA to SM hdl %d, sessId %d",
                      data->mover_smHdl, servp->sessId)); 
                E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_Data_procFsReq: ERROR, cannot post event NDMP_MOVER_EVENT_READ_DATA to SM hdl %d, sessId %d",
                      data->mover_smHdl, servp->sessId)); 
                NDMP_FP_FREE(req, "req READ_DATA_REQ");
            }
            NDMP_FP_FREE(msgp, "msgp WR/RD_DATA_REQ");

		} else if (data->addr_type == NDMP_ADDR_TCP)  /* it's TCP */
        { 
            /* Sends NOTIFY_DATA_READ so NDMP Client can send MOVER_READ 
			 * to the other server.
			 */
        } else
        {
            return esm_getCurrState(handle);
        }
	} 

    return nextState;
   

} /* ndmp_Data_FsIOReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_FsIORsp
 *
 * Description:		Send FS responses to its read/write request to 
 *					data from/to Mover server (for local) or data connection 
 *					(for remote).  We do this when the Mover Server tells
 *					us he's done writing/reading (for local backup/restore) 
 *					or when TPLxmtPkt or TPLrcvPkt signals us the data 
 *					has been xmitted or received over the data connection.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    11/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_FsIORsp(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
    Data_t                      *data = &servp->Data;
	ndmp_fs_info_t				*info = (ndmp_fs_info_t *) thisEvent.p2.u.p, fsInfo; 
	int32						active, dataLen = 0, fs_stat;
    int32       				dst_port = servp->fp_src_port, msglen;
	fs_MsgAbortRestoreReq_t     fs_rst_abort_req;
	fs_MsgAbortDumpReq_t     	fs_dmp_abort_req;

    //NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIORsp:  Begin, sessid %d, esm_hdl %d\n", 
                    //servp->sessId, handle));

	/* Backup process */
	active = (servp->version == NDMP_VER_2)?
			                    NDMP_DATA_V2_STATE_ACTIVE : NDMP_DATA_STATE_ACTIVE;
	if (data->oper == NDMP_DATA_OP_BACKUP)
	{
        /* We're here because either:
		 * 1)  Mover server signals us to notify FS to send more data 
		 * 	   for bkup OR
		 * 2) TPL xmtPktIndic tells us data is xmitted, we tell FS now 
		 * 	   so it can send us some more.
		 */ 
		/* If info is NULL, just send an ABORT to FS */
		if (info == NULL)
		{
			/* Send FS an abort */
			NDMP_DBG_PRINTF(D_INFO, ("fsiorsp:  info is NULL, send FS Abort, "
									"sessid %d\n", servp->sessId));
			msglen = sizeof(fs_MsgAbortDumpReq_t);
			fs_dmp_abort_req.hdr.requestType = FS_REQ_DUMP_ABORT;
			fs_dmp_abort_req.volId.whole_thing = servp->volId.whole_thing;
			fs_dmp_abort_req.abort.ndmpSessId = servp->sessId;

			eee_sendMessage((void *) &fs_dmp_abort_req, 
					sizeof(fs_MsgAbortDumpReq_t), dst_port, ndmp_NNIMCtrl.src_port);
#ifdef NFX_SMP
			ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_DMP_ABORT_REQ_SNT], 1);
#else
			ndmp_fp_msgStats[FS_DMP_ABORT_REQ_SNT]++;
#endif
            return nextState; 
		} else 
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIORsp:  Sessid %d.  BKUP, status sent to FS %s\n",
            	servp->sessId, (info->fs_status == NDMP_STAT_IO_OK)? "fs_err_none": "fs_err_io"));
			info->fs_status = (info->fs_status == NDMP_STAT_IO_OK)? FS_ERR_NONE: FS_ERR_IO;
		}
		if (ndmp_sndFsIODone(servp, NDMP_MID_WRITE_DATA_REQ, info, NULL, 0) == NFX_ERR)
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIORsp:  Err sending FS Write IO response, sessId %d\n", servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_FsIORsp:  Err sending FS Write IO response, sessId %d\n", 
						 servp->sessId));
			NDMP_FP_FREE(info, "info FSIORsp");
			return esm_getCurrState(handle);
		}
#ifdef NFX_SMP
		data->bytesProcessed += data->recSiz;
#else
		data->bytesProcessed += servp->Mover.recSiz;
#endif

	} else 		/* Restore process */
	{
		if (data->addr_type == NDMP_ADDR_LOCAL)
		{
#if 0  //EEE_BUF_RST
			scsi_readRspMsg_t	*scsi_rd_rsp = (scsi_readRspMsg_t *)thisEvent.p1.u.p;
#endif
			ndmp_NFXScsiRdRsp	*scsi_rd_rsp = (ndmp_NFXScsiRdRsp *)thisEvent.p1.u.p;

            if ((data->abortFlag == TRUE) || (esm_getCurrState(handle) != active))
            {
				NDMP_DBG_PRINTF(D_INFO, ("Fsiorsp:  abortFlag %d, current state %s, "
					"sessid %d\n", data->abortFlag, decode_data_state(esm_getCurrState(handle)), 
					servp->sessId));
				NDMP_FP_FREE(info, "fsInfo FSIORsp");	
                free_eee_buf_chain(scsi_rd_rsp->eBufptr);            //EEE_BUF_RST
				NDMP_FP_FREE(scsi_rd_rsp, "scsi_rd_rsp FSIORsp");	
				return esm_getCurrState(handle);
			}
			if (!info)
			{
				/* Send FS an abort */
				NDMP_DBG_PRINTF(D_INFO, ("fsiorsp:  info is NULL, send FS Abort, "
										"sessid %d\n", servp->sessId));
				msglen = sizeof(fs_MsgAbortRestoreReq_t);
				fs_rst_abort_req.hdr.requestType = FS_REQ_REST_ABORT;
				fs_rst_abort_req.volId.whole_thing = data->d_volId;
				fs_rst_abort_req.abort.ndmpSessId = servp->sessId;

				eee_sendMessage((void *) &fs_rst_abort_req, 
							sizeof(fs_MsgAbortRestoreReq_t), 
							dst_port, ndmp_NNIMCtrl.src_port);
	
#ifdef NFX_SMP
				ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RST_ABORT_REQ_SNT], 1);
#else
				ndmp_fp_msgStats[FS_RST_ABORT_REQ_SNT]++;
#endif
                if (scsi_rd_rsp)
                {
                    free_eee_buf_chain(scsi_rd_rsp->eBufptr);            //EEE_BUF_RST
				    NDMP_FP_FREE(scsi_rd_rsp, "scsi_rd_rsp FSIORsp");	
                }
                return nextState; 
			}
            if (!scsi_rd_rsp)
            {
				fsInfo.fs_status = FS_ERR_IO;
				fsInfo.ioCookie = info->ioCookie;
				NDMP_DBG_PRINTF(D_INFO, ("fsiorsp:  scsi_rd_rsp is NULL, ioCookie %lld\n", info->ioCookie));
			    if (ndmp_sndFsIODone(servp, NDMP_MID_READ_DATA_REQ, &fsInfo, NULL, 0) == NFX_ERR)
				{
					NDMP_DBG_PRINTF(D_INFO, 
						("ndmp_Data_FsIORsp:  Err sending FS Read IO response, ioCookie %lld, sessId %d\n", info->ioCookie, servp->sessId));
					E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_FsIORsp:  Err sending FS Read IO response, ioCookie %lld, sessId %d\n", info->ioCookie, servp->sessId));
					NDMP_FP_FREE(info, "info FSIORsp");
					return esm_getCurrState(handle);
				}
				NDMP_FP_FREE(info, "info FSIORsp");	
                return nextState; 
            }
#ifdef NFX_SMP
			NFX_SPIN_LOCK(&servp->Mover.spin);
			data->position = servp->Mover.curpos;	
			NFX_SPIN_UNLOCK(&servp->Mover.spin);
#else
			data->position = servp->Mover.curpos;	
#endif
			
			if (scsi_rd_rsp)
			{
				dataLen = scsi_rd_rsp->dataLen;
			}

			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_FsIORsp:  Sessid %d.  RESTORE, status sent to FS %s\n",
                servp->sessId, (info != NULL && info->fs_status == NDMP_STAT_IO_OK)? 
				"fs_err_none": "fs_err_io")); 

			if (info->fs_status == NDMP_STAT_IO_OK)
            {
                if (dataLen)
                    fs_stat = FS_ERR_NONE;
                else
                {
                    NDMP_DBG_PRINTF(D_INFO, ("SCSI Date len is 0\n"));
                    fs_stat = FS_ERR_IO;
                }
            }
            else {
                fs_stat = FS_ERR_IO;
			}
           	info->fs_status = fs_stat;

			if (ndmp_sndFsIODone(servp, NDMP_MID_READ_DATA_REQ, 	
								info, scsi_rd_rsp->eBufptr, dataLen) == NFX_ERR)
			{
				NDMP_DBG_PRINTF(D_INFO, 
						("ndmp_Data_FsIORsp:  Err sending FS Read IO response, sessId %d\n", servp->sessId));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_FsIORsp:  Err sending FS Read IO response, sessId %d\n", servp->sessId));

				NDMP_FP_FREE(info, "fsInfo FSIORsp");
                free_eee_buf_chain(scsi_rd_rsp->eBufptr);            //EEE_BUF_RST
				NDMP_FP_FREE(scsi_rd_rsp, "scsi_rd_rsp FSIORsp");	
				return esm_getCurrState(handle);
			}
			NDMP_FP_FREE(scsi_rd_rsp, "scsi_rd_rsp FSIORsp");
#ifdef NFX_SMP
			data->bytesProcessed += data->recSiz;
#else
			data->bytesProcessed += servp->Mover.recSiz;
#endif
			/* printf("ndmp_Data_FsIORsp:  data->bytesProcessed = %lld\n", 
					data->bytesProcessed); */
			
		} else if (data->addr_type == NDMP_ADDR_TCP)  /* it's TCP */
        { 
            /* Sends NOTIFY_DATA_READ so NDMP Client can send MOVER_READ 
			 * to the other server.
			 */
        } else
        {
            return esm_getCurrState(handle);
        }
	} 
	NDMP_FP_FREE(info, "fsInfo FSIORsp");

    return nextState;
   

} /* ndmp_Data_FsIORsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_restore
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_restore(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t               *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_restore:  Begin, sessid %d, esm_hdl %d\n", 
                            servp->sessId, handle));
    return nextState;

} /* ndmp_Data_restore */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_fwdDumpDateReq
 *
 * Description:     This is in response to FS send us a SetDumpDate msg.
 *                  We'll forward this request to SSC to write it into
 *                  ou /usr/local/agile/dumpdate file.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/28/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------ */
int
ndmp_saveDumpDate(NdmpNIMService_t *servp, ndmp_set_dumpdate_req_msg_t *fs_req)
{
    dmpdate_t       *ddrec;
    Data_t         *data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_saveDumpDate:  Begin, sessid %d, esm_hdl %d\n",
                            servp->sessId, servp->Data.my_smHdl));

    /* Don't do it if we're aborting */
    if (servp->Data.abortFlag == TRUE)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_saveDumpDate:  User requested abort, no DumpDate saved, sessId %d.\n", 
								servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_saveDumpDate:  User requested abort, no DumpDate saved, sessId %d.\n", 
						servp->sessId));
        return NFX_OK;
    }

	data->sendDumpDate = TRUE;	// means we need to do this later
	ddrec = &(data->dd);

    /* Fill in our dump date record */
    ddrec->fs_name.whole_thing = fs_req->file_sys_name.whole_thing;
    memcpy(ddrec->snapshot, fs_req->snapshot, fs_req->snapshot_len);
    ddrec->snapshot[fs_req->snapshot_len] = '\0';
    ddrec->date= fs_req->date;
    ddrec->dump_level = fs_req->dump_level;

    NDMP_DBG_PRINTF(D_DBG, ("ndmp_saveDumpDate\n"
   					"-------------------\n"
					"fsname: 0x%llx\n" "date  : %d\n" "sessid: 0x%x\n" "dump_level: %d\n" "snapshot: %s\n", 
					ddrec->fs_name.whole_thing, ddrec->date, servp->sessId, ddrec->dump_level, ddrec->snapshot));

    return NFX_OK;

} /* ndmp_saveDumpDate */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_sendDumpUpdate
 *
 * Description:     Send dump info to SSC.
 *
 * Created by:      Nitin Jain
 *
 * Data Created:    08/06/02
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
int ndmp_sendDumpUpdate(NdmpNIMService_t *servp)
{
    ndmp_NFXRsp_t   *NFXrsp;
    int             rc, msglen;
    Data_t         *data = &servp->Data;
    dmpdate_t       *ddrec, *dd = &(data->dd);


    /* Don't do it if we're aborting */
    if (servp->Data.abortFlag == TRUE)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendDumpUpdate:  User requested abort, no DumpDate written, sessId %d.\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sendDumpUpdate:  User requested abort, no DumpDate written, sessId %d.\n", servp->sessId));
        return NFX_OK;
    }

    msglen = sizeof(NFXrsp) + sizeof(dmpdate_t) -1;
    NDMP_FP_ALLOC(NFXrsp, ndmp_NFXRsp_t, msglen, "nfxrsp DumpUpdate");
    NFXrsp->hdr.sessId  = servp->sessId;
    NFXrsp->hdr.mid     = NDMP_MID_UPDATE_ENV_REQ;
    ddrec = (dmpdate_t *) NFXrsp->content;

    /* Fill in our dump date record */
    ddrec->fs_name.whole_thing = dd->fs_name.whole_thing;
    strcpy(ddrec->snapshot, dd->snapshot);
    // ddrec->sessId = servp->sessId;
    ddrec->date= dd->date;
    ddrec->dump_level = dd->dump_level;

    NDMP_DBG_PRINTF(D_DBG, ("ndmp_sendDumpUpdate\n"
    						"-------------------\n"
    						"fsname: 0x%llx\n" "date  : %d\n" "dump_level: %d\n" "snapshot: %s\n", 
							ddrec->fs_name.whole_thing, ddrec->date, ddrec->dump_level, ddrec->snapshot));

    /* send the msg */
    rc = eee_sendMessage((void *) NFXrsp, msglen, servp->ssc_src_port, ndmp_NNIMCtrl.src_port);
   	if (rc != NFX_OK)
   	{
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_sendDumpUpdate: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_sendDumpUpdate: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   	} else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_UPD_ENV_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_UPD_ENV_REQ_SNT]++;
#endif
	}		
    NDMP_FP_FREE(NFXrsp, "nfxrsp DumpUpdate");
    return NFX_OK;
} /* ndmp_sendDumpUpdate */


/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_fwdDumpDateReq
 *
 * Description:     This is in respond to FS sending us a SetDumpDate msg.
 *                  We'll forward this request to SSC to write it into
 *                  ou /usr/local/agile/dumpdate file.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/28/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------ */
int
ndmp_fwdDumpDateReq(NdmpNIMService_t *servp)
{
    ndmp_NFXRsp_t   *NFXrsp;
	int				rc, msglen;
    Data_t         *data = &servp->Data;
    dmpdate_t       *ddrec, *dd = &(data->dd);
	


    if (!dd)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_fwdDumpDateReq: Empty dumpdate entry in Data server, sessId %d\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_fwdDumpDateReq: Empty dumpdate entry in Data server, sessId %d\n", servp->sessId));
        return NFX_ERR;
    }

	/* Don't do it if we're aborting */
	if (servp->Data.abortFlag == TRUE)		
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_fwdDumpDateReq:  User requested abort, no DumpDate written, sessId %d.\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_fwdDumpDateReq:  User requested abort, no DumpDate written, sessId %d.\n", servp->sessId));
		return NFX_OK;
	}

	msglen = sizeof(NFXrsp) + sizeof(dmpdate_t) -1;
    NDMP_FP_ALLOC(NFXrsp, ndmp_NFXRsp_t, msglen, "nfxrsp DumpDate");
    NFXrsp->hdr.sessId  = servp->sessId;
    NFXrsp->hdr.mid     = NDMP_MID_SET_DUMPDATE_REQ;
	ddrec = (dmpdate_t *) NFXrsp->content;	

    /* Fill in our dump date record */
    ddrec->fs_name.whole_thing = dd->fs_name.whole_thing;
	strcpy(ddrec->snapshot, dd->snapshot);
    ddrec->date= dd->date;
    ddrec->dump_level = dd->dump_level;

    NDMP_DBG_PRINTF(D_DBG, ("ndmp_fwdDumpDateReq\n"
    				"-------------------\n"
    				"fsname: 0x%llx\n" "date  : %d\n" "dump_level: %d\n" "snapshot: %s\n", 
					ddrec->fs_name.whole_thing, ddrec->date,  ddrec->dump_level, ddrec->snapshot));

    /* send the msg */
    rc = eee_sendMessage((void *) NFXrsp, msglen, servp->ssc_src_port, ndmp_NNIMCtrl.src_port);
   	if (rc != NFX_OK)
   	{
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_fwdDumpDateReq: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_fwdDumpDateReq: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   	} else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_SET_DMPDATE_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_SET_DMPDATE_REQ_SNT]++;
#endif
	}
    NDMP_FP_FREE(NFXrsp, "nfxrsp DumpDate");
	return NFX_OK;

} /* ndmp_fwdDumpDateReq */
#if 0
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_data_rel_fh
 *                  - Clean fh structure accumulated before 
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_data_rel_fh(NdmpNIMService_t *servp)
{
	int i;

    // Clear our FH structure
    for (i = 0; i < servp->Fh.tot_dir_files; i++)
    {
        NDMP_FP_FREE(servp->Fh.fh_dir_lst[i], "fh_node_lst[]");
    }
    for (i = 0; i < servp->Fh.tot_node_files; i++)
    {
        NDMP_FP_FREE(servp->Fh.fh_node_lst[i], "fh_node_lst[]");
    }
    bzero(&servp->Fh, sizeof(Fh_t));

	return NFX_OK;
} /* ndmp_data_rel_fh */
#endif
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_snd_info
 *
 * Description:		It does the following:
 *					- Sends Logs 
 *						1) Messages (ERROR, WARNING, LOG, DBG, INFO) 
 *						2) File Log for recovery 
 *					- send File History during backup.
 *					- send Set-Dump-Date request issued by FS.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_snd_info(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	int32						*mid = (int32 *) thisEvent.p1.u.p,
   								rc, msglen;
	ndmp_send_log_req_msg_t		*log_req_from_fs = (ndmp_send_log_req_msg_t *) thisEvent.p1.u.p;
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
	uint32                  	dst_port = servp->ssc_src_port, msgType;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_snd_info:  Begin, sessid %d, esm_hdl %d\n", 
                            servp->sessId, handle));
	
	switch (*mid)
	{
		case NDMP_MID_SEND_LOG_REQ:
			msgType = log_req_from_fs->log_msgTyp;
			msglen = sizeof(ndmp_send_log_req_msg_t) + 
						log_req_from_fs->file_name_len +
						log_req_from_fs->log_txt_len - 1;

   			/* Now fwd LOG_MSG or LOG_FILE req to SSC to form a real log 
			 * request for us.  Hack here and change the msg id. 
			 */
			log_req_from_fs->hdr.mid = (msgType != NDMP_LOG_MSG_TYP_FILE)? 	
				NDMP_MID_LOG_MESSAGE: NDMP_MID_LOG_FILE;

   			rc = eee_sendMessage((void *)log_req_from_fs, msglen, 
									dst_port, ndmp_NNIMCtrl.src_port);
   			if (rc != NFX_OK)
   			{
   				NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_snd_info: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_snd_info: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   			} else {
#ifdef NFX_SMP
				ATOMIC_INC_INT(&ndmp_fp_msgStats[(msgType != NDMP_LOG_MSG_TYP_FILE)? SSC_LOG_MSG_REQ_SNT: SSC_LOG_FILE_REQ_SNT], 1);
#else
				ndmp_fp_msgStats[(msgType != NDMP_LOG_MSG_TYP_FILE)? SSC_LOG_MSG_REQ_SNT: SSC_LOG_FILE_REQ_SNT]++;
#endif
			}
			NDMP_DBG_PRINTF(D_INFO, ("filename = %s, Log msg = %s\n", log_req_from_fs->blob, 
						log_req_from_fs->blob+ log_req_from_fs->file_name_len));
			break;
		case NDMP_MID_SET_DUMPDATE_REQ:
			ndmp_saveDumpDate(servp, thisEvent.p1.u.p);	
            ndmp_sendDumpUpdate(servp);
			break;
		default:
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_snd_info:  Invalid msg type %d, sessId %d\n", *mid, servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_Data_snd_info:  Invalid msg type %d, sessId %d\n", *mid, servp->sessId));
	}

	/* free msg received */
    NDMP_FP_FREE(thisEvent.p1.u.p, "msgp SND_INFO");

	return nextState;

} /* ndmp_Data_snd_info */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_getenv
 *
 * Description:		We return to client the ENV variables that he sent us
 *					at time of BKUP or RECOVER begins.  We can also send back to him
 *					additioanal ENV vars or modified but must return all that
 *					he sent us, even the ones we don't support.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_getenv(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXReq_t   	*req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t 	*servp = (NdmpNIMService_t *) userInfo;
 	ndmp_NFXRsp_t       *rsp;
	ndmp_GetEnv_Rsp_t	*env_rsp;
    int32       		rc, dst_port = servp->ssc_src_port, msglen, 
						num_env_mod, name_len, val_len, offset_to_str;
	char8				val_buf[200];

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_getenv:  Begin, sessid %d, esm_hdl %d\n", 
							servp->sessId, servp->Data.my_smHdl));

	num_env_mod = 1;
	name_len = (strlen(BKUP_SIZ_ENV_STR));  // w/o null
#ifdef NFX_SMP
	NFX_SPIN_LOCK(&servp->Mover.spin);
	sprintf(val_buf, "%lld", servp->Mover.dataWritten);
	NFX_SPIN_UNLOCK(&servp->Mover.spin);
#else
	sprintf(val_buf, "%lld", servp->Mover.dataWritten);
#endif
	val_len = strlen(val_buf); // w/o null

    msglen = (sizeof(ndmp_NFXRsp_t) -1) + (sizeof(ndmp_GetEnv_Rsp_t)-1) 
			+ (num_env_mod * (sizeof(env_t))) + (name_len + val_len + 2);
    NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp DATA_GE");
    if (rsp == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_getenv:  Malloc error for rsp, sessId %d\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_Data_getenv:  Malloc error for rsp, sessId %d\n", servp->sessId));
        ndmp_sendCmdRsp(req, dst_port, NDMP_UNDEFINED_ERR, 0);
        NDMP_FP_FREE(req, "msgp DATA_GE");
        return esm_getCurrState(handle);
	}
	env_rsp = (ndmp_GetEnv_Rsp_t *) rsp->content;

    rsp->hdr.sessId  = req->hdr.sessId;
    rsp->hdr.mid     = req->hdr.mid;
    rsp->ndmp_mid    = req->ndmp_mid; 
	rsp->userInfo    = NDMP_NO_ERR;
	memcpy(&rsp->cl_hdr, &req->cl_hdr, sizeof(ndmp_Hdr));

	env_rsp->num_env_mod = 1;

	/* offset to where string chars for name/value pairs begin */
	offset_to_str = num_env_mod * sizeof(env_t);

	/* env_rsp->content will contain an array of env_t (num_env_mod items)
	* followed by the strings for name/value pairs themselves.
	*/
	memcpy(env_rsp->content, &name_len, sizeof(int32));
	memcpy(&env_rsp->content[sizeof(int32)], &val_len, sizeof(int32));
    strcpy(&env_rsp->content[offset_to_str], BKUP_SIZ_ENV_STR);

    env_rsp->content[offset_to_str + name_len] = '\0';

    strcpy(&env_rsp->content[offset_to_str + name_len + 1], val_buf);
    env_rsp->content[offset_to_str + name_len + 1 + val_len] = '\0';

	rc = eee_sendMessage((void *) rsp, msglen, dst_port, ndmp_NNIMCtrl.src_port);
	if (rc != NFX_OK)
   	{
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_snd_info: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_Data_snd_info: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   	} else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_DATA_GETENV_RSP_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_DATA_GETENV_RSP_SNT]++;
#endif
	}
    NDMP_FP_FREE(rsp, "nfxrsp DATA_GE");
    NDMP_FP_FREE(req, "msgp DATA_GE");
    return nextState;

} /* ndmp_Data_getenv */
/*
 *------------------------------------------------------------------------------
 * Function Name:   fill_get_state_info
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
fill_Data_get_state_info (Data_t *data, ndmp_data_get_state_rsp *gs_rsp, 
							esm_handle_t handle)	
{
	int32	cur_st;	


	gs_rsp->error   = NDMP_NO_ERR;

	gs_rsp->invalid   = (NDMP_DATA_STATE_EST_BYTES_REMAIN_INVALID |
						NDMP_DATA_STATE_EST_TIME_REMAIN_INVALID);		

    gs_rsp->op = data->oper;  /* current operation */
	cur_st = esm_getCurrState(handle);
    gs_rsp->state = (cur_st == NDMP_DATA_STATE_IDLE)? NDMP_DATA_ST_IDLE:
		(cur_st == NDMP_DATA_STATE_LISTEN)? NDMP_DATA_ST_LISTEN:  
		(cur_st == NDMP_DATA_STATE_CONNECTED)? NDMP_DATA_ST_CONNECTED:  
		(cur_st == NDMP_DATA_STATE_ACTIVE)? NDMP_DATA_ST_ACTIVE: NDMP_DATA_ST_HALTED;  
    gs_rsp->halt_reason = data->haltReas;/* current reason */
    gs_rsp->bytes_processed.low = data->bytesProcessed;        /* total bytes processed so far */
    gs_rsp->bytes_processed.high = data->bytesProcessed >> 32;        /* total bytes processed so far */
	gs_rsp->est_bytes_remain.low = data->bytesLeftToRead;
	gs_rsp->est_bytes_remain.high = data->bytesLeftToRead >> 32;

#if 0	//We don't support this for now.
	gs_rsp->est_time_remain = data->estTimeRemain;
#endif

    gs_rsp->read_offset.low = data->readOffset;      /* data read seek offset */
    gs_rsp->read_offset.high = data->readOffset >> 32;  
			/* data read seek offset */
    gs_rsp->read_length.low = data->readLen;         /* data read length */
    gs_rsp->read_length.high = data->readLen >> 32;  /* data read length */
	gs_rsp->data_conn_addr.addr_type = data->addr_type;
    if (data->addr_type == NDMP_ADDR_TCP)
    {
        gs_rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr = data->dataConnAddr;
        gs_rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port = data->dataConnPort;
    }

	NDMP_DBG_PRINTF(D_INFO, ("Data Get State:\n"
			"---------------\n"
			"State            = %s\n" 
			"Halt Reason  = %s\n" 
			"Bytes_processed  = %lld\n"
			"Est_bytes_remain = %lld\n"
			"Est_time_remain  = %lld\n"
			"Read_offset      = %lld\n"
			"Read_len         = %lld\n"
			"Addr type        = %s\n", 
			decode_data_state (cur_st),
			decode_data_halt_reas (data->haltReas),
			data->bytesProcessed, data->bytesLeftToRead, data->estTimeRemain, data->readOffset,
			data->readLen, (data->addr_type == NDMP_ADDR_LOCAL)? "local": "tcp"));

	return NFX_OK;
} /* fill_Data_get_state_info */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_get_state
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_get_state(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXReq_t   			*req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
 	ndmp_NFXRsp_t               *rsp;
	ndmp_data_get_state_rsp		*gs_rsp;
    NdmpNIMService_t 			*servp = (NdmpNIMService_t *) userInfo;
	Data_t                      *data = &servp->Data;
    int32       				rc, dst_port = servp->ssc_src_port, msglen;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_get_state:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	msglen = sizeof(ndmp_NFXRsp_t) + sizeof(ndmp_data_get_state_rsp) - 1;
    NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp DATA_GS");
    if (rsp == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_get_state:  Malloc error for rsp, sessId %d\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_Data_get_state:  Malloc error for rsp, sessId %d\n", servp->sessId));
        ndmp_sendCmdRsp(req, dst_port, NDMP_UNDEFINED_ERR, 0);
        NDMP_FP_FREE(req, "msgp DATA_GS");
        return esm_getCurrState(handle);
	}
	gs_rsp = (ndmp_data_get_state_rsp *) rsp->content;

    rsp->hdr.sessId  = req->hdr.sessId;
    rsp->hdr.mid     = req->hdr.mid;
    rsp->ndmp_mid    = req->ndmp_mid; 
	memcpy(&rsp->cl_hdr, &req->cl_hdr, sizeof(ndmp_Hdr));

	fill_Data_get_state_info (data, gs_rsp, handle);

	rc = eee_sendMessage((void *) rsp, msglen, dst_port, ndmp_NNIMCtrl.src_port);
	if (rc != NFX_OK)
   	{
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_get_state: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_Data_get_state: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   	} else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_DATA_GETSTATE_RSP_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_DATA_GETSTATE_RSP_SNT]++;
#endif
	}
    NDMP_FP_FREE(rsp, "nfxrsp DATA_GS");
    NDMP_FP_FREE(req, "msgp DATA_GS");
    return nextState;

} /* ndmp_Data_get_state */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_sndNotifHalted
 *
 * Description:     Send a HALT NOTIFICATION to NDMP client.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/29/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_sndNotifHalted(NdmpNIMService_t *servp, int32 halt_reason)
{
    ndmp_notify_data_halted_req *notify_msg;
    ndmp_NFXRsp_t       		*rsp;
    int                 		rc, msglen;


    msglen = sizeof (ndmp_NFXRsp_t) + sizeof (ndmp_notify_data_halted_req) -1;
    NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp NOTIF_HALTED");
    if (rsp == NULL)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_sndNotifHalted:  Malloc error for rsp, sessId %d\n", servp->sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_sndNotifHalted:  Malloc error for rsp, sessId %d\n", servp->sessId));
        return NFX_ERR;
    }
    rsp->hdr.sessId   = servp->sessId;
    rsp->hdr.mid      = NDMP_MID_NOTIFY_DATA_HALTED;
    rsp->ndmp_mid     = NDMP_NOTIFY_DATA_HALTED;
    rsp->userInfo     = 0;
    notify_msg = (ndmp_notify_data_halted_req *) rsp->content;
    notify_msg->reason = halt_reason;
 
    /* Now send NOTIFICATION to SSC to forward for us */
    rc = eee_sendMessage((void *)rsp, msglen, servp->ssc_src_port, ndmp_NNIMCtrl.src_port);
	if (rc != NFX_OK)
   	{
   		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_get_state: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_Data_get_state: eee_sendMessage failed!  sessId %d\n", servp->sessId));
   	} else {
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_NOTIFY_DATA_HALTED_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[SSC_NOTIFY_DATA_HALTED_REQ_SNT]++;
#endif

		E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("Sending NDMP Data Notify Halted:  reason = %s, sessId %d\n", 
				 decode_data_halt_reas(halt_reason), servp->sessId));
	}
    NDMP_FP_FREE(rsp, "nfxrsp NOTIF_HALTED");

	return NFX_OK;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_op_comp
 *
 * Description:		Signalled by FS backup/recovery operation completes.
 *					So Data server will send a HALT NOTIFICATION to NDMP
 *					client.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/29/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_op_comp(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_ops_cmplt_req_msg_t	*req = (ndmp_ops_cmplt_req_msg_t *) thisEvent.p1.u.p;
	NdmpNIMService_t 			*servp = (NdmpNIMService_t *) userInfo;
	Data_t		 				*data = (Data_t *) &servp->Data;
	Mover_t		 				*Mover = (Mover_t *) &servp->Mover;
	fs_Status					status;
    esm_event_t                 esmEvent;
	int32						rc, reason;

	
	if (data->addr_type == NDMP_ADDR_LOCAL)
	{

    	/* A connection closed on Mover Server, for a Backup process, will
		 * cause it to flush to tape whatever it has left in writeBuf 
		 */
        bzero(&esmEvent, sizeof(esm_event_t));
        esmEvent.event = NDMP_MOVER_EVENT_CONN_CLOSED;
#ifdef NFX_SMP
        rc = esm_deliverEvent(data->mover_smHdl, &esmEvent);
#else
        rc = esm_deliverEvent(servp->Mover.my_smHdl, &esmEvent);
#endif
       	if (rc != NFX_OK)
       	{
#ifdef NFX_SMP
       		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_op_comp: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
          				NDMP_MOVER_EVENT_CONN_CLOSED, data->mover_smHdl, servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
        				("ndmp_Data_op_comp: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, data->mover_smHdl, servp->sessId));
#else
       		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_op_comp: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
          				NDMP_MOVER_EVENT_CONN_CLOSED, servp->Mover.my_smHdl, servp->sessId));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
        				("ndmp_Data_op_comp: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, servp->Mover.my_smHdl, servp->sessId));
#endif
       	}
	
	} else {
		/* Use TPL to close the data connection */ 
	}
    /*
     * Create a NOTIFY_DATA_HALTED message and send it.
     */

	status = req->status;

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_op_comp:  FS status %s\n", 
							(status==FS_ERR_NONE)? "No Err": "i/o err"));
	E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("ndmp_Data_op_comp:  FS status %s\n", status==FS_ERR_NONE?"No Err": "i/o err"));

    reason = (status == FS_ERR_NONE)? NDMP_DATA_HALT_SUCCESSFUL: 
						NDMP_DATA_HALT_INTERNAL_ERROR ;

    if (reason == NDMP_DATA_HALT_INTERNAL_ERROR)
	{
        data->sendDumpDate = FALSE;
	} else {
		E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
   				("ndmp_Data_op_comp: %s completing, sessid %d.\n", 
				 (data->oper == NDMP_DATA_OP_BACKUP)? "Backup": "Restore", servp->sessId));
	}
    /* Now send NOTIFICATION to SSC to forward for us */
	ndmp_sndNotifHalted(servp, reason);
	data->haltReas = reason;

    NDMP_FP_FREE(req, "msgp OP_COMP");

	if (ndmp_perf_meas)
	{
#ifdef NFX_SMP
		NFX_SPIN_LOCK(&Mover->spin);
#endif
        printf("Delta time = %lld, wr_pend %lld\n", Mover->delta_time, Mover->wr_pend);
        printf("Avg time in microsec for WRITE i/os to SCSI:  %lld\n", Mover->delta_time/Mover->io_wr_done); 
        printf("Total # of Writes:  %lld\n\n", Mover->io_wr_done);

        printf("Avg Q depth: %lld\n", (Mover->wr_pend)/(Mover->io_wr_done));
#ifdef NFX_SMP
		NFX_SPIN_UNLOCK(&Mover->spin);
#endif
    }
    return nextState;

} /* ndmp_Data_op_comp */

/* *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_abort
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_abort(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_NFXReq_t               *req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
    Data_t                      *data = &servp->Data;
	fs_MsgAbortDumpReq_t		fs_dmp_abort_req;
	fs_MsgAbortRestoreReq_t		fs_rst_abort_req;
	int32						error = NDMP_NO_ERR;
    int32						currState = esm_getCurrState(handle); 
	//int32						mid;
    int32       				dst_port = servp->fp_src_port, msglen, rc = NFX_OK;
	esm_event_t     			esmEvent;
	int32						abort_typ = NDMP_AUTO_SESS_KILL; // used for UI
	int32						closing_state;



    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_abort:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));
	E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
   				("ndmp_Data_abort:  Received Abort from client, sessid %d.\n", servp->sessId));

	//new performance code - don't need this anymore, Rahul will 
	//discard all his i/o requests when we send him an abort.
	
	/* signal ABORT so next FS write or read request, we can notify FS. */	
	data->abortFlag = TRUE;		

	/* Send abort msg to FS */
	if (data->oper == NDMP_DATA_OP_BACKUP)
	{
		msglen = sizeof(fs_MsgAbortDumpReq_t);
		fs_dmp_abort_req.hdr.requestType = FS_REQ_DUMP_ABORT;
		fs_dmp_abort_req.volId.whole_thing = servp->volId.whole_thing;
		fs_dmp_abort_req.abort.ndmpSessId = servp->sessId;

		eee_sendMessage((void *) &fs_dmp_abort_req, 
				sizeof(fs_MsgAbortDumpReq_t), 
				dst_port, ndmp_NNIMCtrl.src_port);
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_DMP_ABORT_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_DMP_ABORT_REQ_SNT]++;
#endif

	} else if (data->oper == NDMP_DATA_OP_RESTORE) {
		msglen = sizeof(fs_MsgAbortRestoreReq_t);
		fs_rst_abort_req.hdr.requestType = FS_REQ_REST_ABORT;
		fs_rst_abort_req.volId.whole_thing = data->d_volId;
		fs_rst_abort_req.abort.ndmpSessId = servp->sessId;

		eee_sendMessage((void *) &fs_rst_abort_req, 
				sizeof(fs_MsgAbortRestoreReq_t), 
				dst_port, ndmp_NNIMCtrl.src_port);
	
#ifdef NFX_SMP
		ATOMIC_INC_INT(&ndmp_fp_msgStats[FS_RST_ABORT_REQ_SNT], 1);
#else
		ndmp_fp_msgStats[FS_RST_ABORT_REQ_SNT]++;
#endif
	}

   	NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_abort:  Send FS ABORT, sessid %d\n", servp->sessId));
	/* Close data connection if existed */
	closing_state = (servp->version == NDMP_VER_2)?
				((currState == NDMP_DATA_V2_STATE_ACTIVE)? 1: 0) :
				(((currState == NDMP_DATA_STATE_CONNECTED) ||
					(currState == NDMP_DATA_STATE_ACTIVE))? 1: 0);

	if (closing_state)
	{	
		/* If we're doing local backup */
		if (data->addr_type == NDMP_ADDR_LOCAL)
		{
			/* send event to Mover to move to CONN_CLOSE mode as well */
			/* later we'll send a message instead of events */
            bzero(&esmEvent, sizeof(esm_event_t));
        	esmEvent.event = NDMP_MOVER_EVENT_CONN_CLOSED;
        	esmEvent.p2.u.w = (data->abortFlag)? NOT_FLUSH_DATA_TO_TAPE: 0;
#ifdef NFX_SMP
        	rc = esm_deliverEvent(data->mover_smHdl, &esmEvent);
#else
        	rc = esm_deliverEvent(servp->Mover.my_smHdl, &esmEvent);
#endif
        	if (rc != NFX_OK)
        	{
#ifdef NFX_SMP
        		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_abort: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, data->mover_smHdl, servp->sessId));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
        				("ndmp_Data_abort: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, data->mover_smHdl, servp->sessId));
#else
        		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_abort: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, servp->Mover.my_smHdl, servp->sessId));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
        				("ndmp_Data_abort: ERROR, cannot post event %d to SM hdl %d, sessid %d\n",
           				NDMP_MOVER_EVENT_CONN_CLOSED, servp->Mover.my_smHdl, servp->sessId));
#endif
        	}
		} else {
			/* Remote... */
		}
	}

#ifndef NFX_SMP
	if (req->hdr.mid == NDMP_MID_CLOSE_SESSION)
	{
		ndmp_NFXReq_t rel_req;

		memset(&rel_req, 0, sizeof(rel_req));
		rel_req.hdr.mid = NDMP_MID_REL_RSRC;
		rel_req.hdr.sessId = req->hdr.sessId;
		rc = eee_sendMessage((uchar8 *)&rel_req, sizeof(rel_req), ndmp_NNIMCtrl.src_port, 
						ndmp_NNIMCtrl.src_port);
    	if (rc != NFX_OK)
    	{
       		NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_listen: eee_sendMessage failed! sessid %d\n", servp->sessId));
       		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_Data_listen: eee_sendMessage failed! sessid %d\n", servp->sessId));
    	} 
	} else {
		/* Send NDMP client the response to this request */
		ndmp_sendCmdRsp(req, servp->ssc_src_port, error, abort_typ);
	}

    NDMP_FP_FREE(req, "msgp DATA_ABORT")

    /* Now send NOTIFICATION to SSC to forward for us */
	ndmp_sndNotifHalted(servp, NDMP_DATA_HALT_ABORTED);

	data->haltReas = NDMP_DATA_HALT_ABORTED;

#else 
	if (req->hdr.mid != NDMP_MID_CLOSE_SESSION)
	{
		/* Send NDMP client the response to this request */
		ndmp_sendCmdRsp(req, servp->ssc_src_port, error, abort_typ);
	}

    /* Now send NOTIFICATION to SSC to forward for us */
	ndmp_sndNotifHalted(servp, NDMP_DATA_HALT_ABORTED);

	data->haltReas = NDMP_DATA_HALT_ABORTED;

	if (req->hdr.mid == NDMP_MID_CLOSE_SESSION)
	{
		ndmp_relSessRsrc(servp);
	}

    NDMP_FP_FREE(req, "msgp DATA_ABORT")
#endif

    return nextState;

} /* ndmp_Data_abort */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_data_rel_rsrc
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    3/21/02
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_data_rel_resource(NdmpNIMService_t *servp)
{
	Data_t *data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_data_rel_rsrc:  Begin, sessid %d\n", servp->sessId));

	/* Free the bkup or restore req that we save to be use later on
	 * while setting up w/ FS to start the process.
	 */
	if (data->ops_req)
	{
        NDMP_FP_FREE(data->ops_req, "ops_req BKUP/RST");
	}
	/* Free environment vars or file list for restore if any */
	ndmp_free_env(data);
    ndmp_free_flist(data);

#if 0
	/* Release file history accumulated from FS so far and have
	 * not sent to client.
	 */
   	NDMP_DBG_PRINTF(D_INFO, ("ndmp_data_rel_rsrc:  freeing fh\n"));
	ndmp_data_rel_fh(servp);
#endif
	return NFX_OK;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_stop
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_stop(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_NFXReq_t               *req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;
    Data_t                      *data = &servp->Data;


    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_stop:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	if ((data->sendDumpDate == TRUE) && (data->oper == NDMP_DATA_OP_BACKUP))
	{
		ndmp_fwdDumpDateReq(servp);	
		data->sendDumpDate = FALSE;
	}	

	/* Release all resources */
	ndmp_data_rel_resource(servp);

	/* Reset all data server state variables */
	ndmp_data_init(data);

	/* Reset all backup environment vars */

	/* Send NDMP client the response to this request */
	ndmp_sendCmdRsp(req, servp->ssc_src_port, NDMP_NO_ERR, 0);

    NDMP_FP_FREE(req, "msgp STOP");
    return nextState;

} /* ndmp_Data_stop */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_noop
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_noop(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo) /* ctrl blk linked to sm       */
{
	NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_noop: event %s (%d), cur_state %s, sessId %d, esm_hdl %d\n", 
					 decode_data_event(thisEvent.event), thisEvent.event, 
					 decode_data_state(esm_getCurrState(handle)), 
					 servp->sessId, handle));
    NDMP_FP_FREE(thisEvent.p1.u.p, "msgp");
    if (thisEvent.p2.u.p)
        NDMP_FP_FREE(thisEvent.p2.u.p, "p2 for thisInfo");

    return nextState;
} /* ndmp_Data_noop */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_err
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_err(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXReq_t *msg = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
	NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;
    int32       dst_port = servp->ssc_src_port;
    int32       errCode, currState;

    currState = esm_getCurrState(handle);

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_err:  event %s, cur_state %s, sessId %d, handle %d\n", 
							decode_data_event(thisEvent.event), decode_data_state(esm_getCurrState(handle)),
							servp->sessId, handle));
    
    errCode = NDMP_ILLEGAL_STATE_ERR;
    
    switch (thisEvent.event)
    {
        case NDMP_DATA_EVENT_LISTEN:
        case NDMP_DATA_EVENT_CONNECT:
            if (currState != NDMP_DATA_STATE_IDLE)        
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_CONNECTED:
            if (currState != NDMP_DATA_STATE_LISTEN)      
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_START_OPS:
            if (currState != NDMP_DATA_STATE_CONNECTED) 
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_GET_ENV:
            if ( (currState != NDMP_DATA_STATE_ACTIVE) && 
				 (currState != NDMP_DATA_STATE_HALTED) ) 
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_DATA_READY:
            if (currState == NDMP_DATA_STATE_CONNECTED)      
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_SND_INFO:
            // No need to send client any info. for this.
            // if (currState != NDMP_DATA_STATE_ACTIVE)      
            //     ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_ABORT:
            if (currState == NDMP_DATA_STATE_IDLE)        
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_STOP:
            if (currState != NDMP_DATA_STATE_HALTED)      
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_EVENT_INTERNAL_ERR:
            if ((currState == NDMP_DATA_STATE_CONNECTED) ||
				(currState == NDMP_DATA_STATE_ACTIVE) ||
				(currState == NDMP_DATA_STATE_HALTED)) 
			{
				/* .... */
			}
            break;
        case NDMP_DATA_EVENT_CONN_ERR:
            if ((currState == NDMP_DATA_STATE_CONNECTED) ||
				(currState == NDMP_DATA_STATE_ACTIVE)) 
			{
				/* .... */
			}
            break;
		default:
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_err:  Invalid event %d\n", thisEvent.event));
    }
    NDMP_FP_FREE(msg, "msgp");
    return nextState;

} /* ndmp_Data_err */
/*-----------------------------------------------------------------
 * Name:        ndmp_Data_bad_state
 *
 * Description: sm action function to handle bad state transitions
 *
 * created by : Anh Vu
 *
 * Date Created: 10/28/01
 *-----------------------------------------------------------------
 */

int32
ndmp_Data_bad_state(esm_handle_t handle,
                    int32       nextState,
                    esm_event_t thisEvent,
                    void        * userInfo)
{
	NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_bad_state:  event %s, cur_state %s, sessId %d, handle %d\n", 
							decode_data_event(thisEvent.event), decode_data_state(esm_getCurrState(handle)),
							servp->sessId, handle));

    return nextState;

} /* ndmp_Data_bad_state */

