/*
 *------------------------------------------------------------------------------
 *
 * Name:	ndmp-ssc.h
 *
 * RCS:		$Id: ndmp-ssc.h,v 1.29 2002/12/04 22:40:36 nitinj Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	NDMP definitions for SSC
 *
 * Created by:	Anh Vu
 *
 * Date Created: 02/18/02
 *
 *------------------------------------------------------------------------------
 */
#if !defined(_NDMP_SSC_H_)
#define _NDMP_SSC_H_

#include <stdio.h>
#include "ndmp-ui-api.h"

#define RCV_MSG 1
#define XMT_MSG 2

#define BKUP    1
#define RESTORE 2


/*------------- SCSI ---------------*/
typedef struct scsi
{
    int32           hdl;            /* scsi device handle */
    int32           datain_len;     /* For exec_cdb passthru cmds */
    int32           exec_cdb_dir_flags; /* For exec_cdb passthru cmds */
    char            devname[MAX_TAPE_ALIAS];    /* alias */
    char            phys_devname[MAX_TAPE_NAME];

#define NDMP_MAX_SENSE_DATA_LEN 32  /* This is what tape-driver return to us max, usually
                                     * 64 for SCSI, but we have FC.
                                     */

    uchar8          sense_data[NDMP_MAX_SENSE_DATA_LEN]; /* 32 bytes of specifying reason for error status */
} SCSI_t;

/*------------- Tape ---------------*/
typedef struct tape
{
    char8                   dev_opening;    /* used during tape open operation */
    char8                   req_owner;      /* who (which NDMP service issue this request */
    int32                   hdl;            /* tape device handle */
    uint64                  recCnt;         /* number of records written */
    ndmp_tape_open_mode     mode;           /* tape device open mode */
    int32                   status;
    int32                   media_err;      /* # of soft media errs since dev open */
    int32                   file_marks;     /* # of file marks from begin of tape to cur pos */
    int32                   tape_blks;      /* # of tape blocks from beg. of tape to cur pos  */
    int32                   tape_space;     /* in bytes */
    int32                   space_remain;   /* in bytes */
    int32                   last_op_media_err;  /* T/F if last tape operation result in media err */
    int32                   tw_data_out_len;    /* For tape write cmd - data out length */
    int32                   datain_len;     /* For exec_cdb passthru cmds */
    int32                   exec_cdb_dir_flags; /* For exec_cdb passthru cmds */
    char                    devname[MAX_TAPE_ALIAS];    /* alias */
    char                    phys_devname[MAX_TAPE_NAME];
                                /* wwn (16 bits LUN + 48 bits WWN = 16chars) +
                                 *  3 chars for Rewind +
                                 *  3 chars for Unload +
                                 * 32 chars for density
                                 */

} Tape_t;

/*------------- Mover  ---------------*/
typedef struct mover_ssc
{
    ndmp_mover_get_state_rsp    mover_state;
    int32                   mode;           /* current mode
                                             * (NOACTION, READ, WRITE)
                                             */
} Mover_SSC_t;

/*------------- Data  ---------------*/
typedef struct data_ssc
{
    int64               fs_volId;
    int64               fs_dstvolId;
    uint32              env_len;        /* # of env vars received from client */
    uint32              srvr_env_len;   /* final # of env vars from client + from us srvr */
    ndmp_pval           *env_lst;       /* environment data */
    dmpdate_t           dd;
    char                fs_volname[MAX_VOL_NAME];
    char                fs_dstvolname[MAX_VOL_NAME];
    ndmp_data_get_state_rsp data_state;
} Data_SSC_t;

/*------------- NDMP Services ---------------*/
typedef struct  ndmp_serv
{
    SCSI_t              SCSI;
    Tape_t              Tape;
    Data_SSC_t          Data;
    Mover_SSC_t         Mover;
    int32               sessId;
} NdmpService_t;

typedef struct VolEnt
{
	evm_volId_t		id;
	char			name[MAX_VOL_NAME];
} vol_entry_t;

/*------------- NDMP Session Data Structure ---------------*/
typedef struct ndmpSession
{
    int32               Agilesock;      /* socket used for agile msgs */
    int32               clientFd;
    struct sockaddr_in  host_ipaddr;    // address of NDMP client
    uint32              xmtSeq;         /* current xmt sequence */
    XDR                 xdr_cmd;        /* To specify encode, decode, or free cmd */
    boolean             authed;         /* Is this connection authorized yet? */
    boolean             auth_typ;       /* auth type requested */
    boolean             conn_closed;    /* connection closed? */
    boolean             tcp_disc;    	/* tcp connection broken/closed ? */
    boolean             conn_refused;   /* connection refused due to max sess
                                         * exceeded or ndmp disable? */
	boolean				conn_await_close;   /* Wait for some event before can close conn */
	boolean				wait_tape_open_rsp; /* Wait for tape open response to get hdl */ 
    CtrlMsg_t           msgRcvd;        /* last received request or reply message */
	void				*cl_msgent;
    char8               protoVer;
    int32               in_len;
    int32               sessId;
	char8				sess_idx;
    char8               challenge[NDMP_CHALLENGE_LEN];
    int32               nextFsIdx;      /* Next index of FS VolId to request FS Info,
                                         * used in ConfigGetFSInfo message processing.
                                         */
    int32               tot_numFs;      /* for CONFIG_GET_FS_INFO, we store total # of FS
                                         * to request FS info from FS.
                                         */
    int32               curr_numFs;     /* We send a FS info request at a time */
    fs_Stat             *fsInfoLst;     /* Contain list of FS status for all FS in system */
    vol_entry_t         *vol_lst;       /* Contain list of Volumes  in system */
    NdmpService_t       Serv;
    boolean             partial_bkup;  /* FALSE for full backup */
	int 				ndmp_ssc_chld_msgStats[MAX_SSC_CHLD_STAT];
    FILE                *logFp;        /* File pointer to log file */
	char				start_time[20];
    char                *nextblobPtr;  /* Pointer to next file to send */
    int32               fl_len;        /* Total len of saveList buffer */
    int32               fl_num;        /* Number of files */
    int32               fl_sent;       /* Number of files sent */
    char                *saveList;     /* Use to save messages for file list */
#ifdef NFX_SMP
	uint64				xmt2fp_seqId;  /* SeqId of msgs sent to NDMP on FP-NIM, 
										* so we can keep received msgs there in
										* order 
										*/
#endif
} ndmpSess_t;

/* Used to save info from a request msg to send a rsp back */
typedef struct ndmp_src
{
    int32   cpu;
    int32   slot;
    char    app[32];
} ndmp_src_t;

/*------ SEND LOG REQUEST -----------*/
/* Used by FS to tell NDMP server to send log to NDMP client 
 */
typedef struct {
    int32    mid;            /* Message Id */
    uint32   clientCookie;   /* NDMP handle/cookie (sessid) */
} ndmp_ssc_fsMsgHdr_t;

typedef struct
{
    ndmp_ssc_fsMsgHdr_t hdr;    /* NDMP_MID_SEND_LOG_REQ */
    int32           log_msgTyp; /* Type of log message defined above */
    fs_Status       status;     /* status of bkup/restore ops: 
                                 * FS_ERR_NONE      // successful
                                 * FS_ERR_NXIO      // tmp i/o error
                                 * FS_ERR_IO        // real i/o error
                                 * FS_ERR_NOENT     // entry not found
                                 * FS_ERR_NOSPACE   // no space to write a file
                                 * FS_ERR_NODIR     // no path found 
                                 */
                                /* The below is for NDMP_LOG_MSG_TYP_FILE: 
                                 */
    fs_Status       file_recov_stat;
                                /* May be these can be defined by FS-api.h as
                                 * FS_ERR_NONE  
                                 * FS_ERR_NOENT //file not found
                                 * FS_ERR_EXIST //dst path exists already
                                 * FS_ERR_INVAL //invalid arguments provided FS 
                                 * FS_ERR_IO        //catch all or io error 
                                 */
    int32       file_name_len;  /* Length of file */
    int32       log_txt_len;    /* Length of log msg */
    int32       dummy_pad1;
    void        *cl_cookie;     /* Client's (FS) handle/cookie */
    int32       dummy_pad2;
    char        blob[1];        /* Name of file not restored, only files w/name
                                 * that matches the name in recovery list.
                                 * Followed by log text.
                                 */
} ndmp_ssc_send_log_req_msg_t;

/*------------- NDMP Control Message Entry ---------------*/

/* Received on SSC side from NDMP client */
typedef struct ndmpMsgEntry
{
    ndmp_Msg	msgid;          /* which message is it? */
    boolean		authReqd;       /* authentication required? */
    bool_t		(*reqfunc)();
    bool_t		(*rspfunc)();   /* func to generate XDR reply */
    int			(*msgfunc_cl)(ndmpSess_t *sess); /* function to process message received */
    char8		*msg_name;      /* For debug msgs */
    void        (*printReq)(char *msg, ndmpSess_t *s); /* print func for req message */
    void        (*printRsp)(char *msg, ndmpSess_t *s); /* print func for rsp message */
	int			stat_idx_snt;	/* Index to update statistics on 
								   msg sent to FP or other processes on SSC,
								   based on the msg we rvcd from client. 
								 */
} ndmpMsgEnt_t;

/* Received on SSC side from other internal processes */
typedef struct ndmpAgileMsgEntry
{
    int         msgid;          /* which message is it? */
    int         (*msgfunc_int)(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
                            	/* function to process message received */
    char8       *msg_name;      /* For debug msgs */
	int			stat_idx_rcvd;	/* Index to update statistics on AGIL msg rcvd */
} ndmpAgileMsgEnt_t;




typedef struct ndmp_Req_info
{
    void    *msg;
	int32	msgId;
	ndmp_src_t src;
} ndmp_Req_info_t;

typedef struct ndmp_NFXSndRptReq
{
	uint32		mid;	/* NDMP_MID_CHILD_REPORT */
	int32		sessId;	/* child's sessId */
} ndmp_NFXSndRptReq_t;

typedef struct ndmp_NFXSndStatReq
{
	uint32		mid;	/* NDMP_MID_CHILD_SND_STATS */
	int32		sessId;	/* child's sessId */
	int			statistic[MAX_SSC_CHLD_STAT];
} ndmp_NFXSndStatsReq_t;


/* Received from UI process */
typedef struct ndmpParentUIMsgEntry
{
    int         msgid;          /* which message is it? */
    int         (*msgfunc_ui)(char8 *rcvbuf, ndmp_src_t *src);
                            /* function to process message received */
    char8       *msg_name;      /* For debug msgs */
	int			stat_idx;		/* Index to update statistics */
} ndmpParentUIMsgEnt_t;   

/*---------  Trace File entry ----------------- */
typedef struct _tfil_t
{
	struct tm time;
	struct _tfil_t *nxt;
} tfil_t;

/*--------------------------------------------------------------------
 *                  Function Prototypes
 *--------------------------------------------------------------------
 */
int ndmpXmtMsg( ndmpSess_t *sess, ndmp_HdrMsgTyp msgtyp,
            ndmp_Msg msgid, uint32 rspSeq, ndmp_Err err, void* data);
int32 ndmp_closeTapeDev(ndmpSess_t *sess);
int32 ndmp_closeScsiDev(ndmpSess_t *sess);
void getChallengStr(char8 *dst);
void ndmp_CloseSess(ndmpSess_t   *sess);
#ifdef NFX_SMP
int32 ndmp_procCloseSessRsp(ndmpSess_t *sess);
#endif
int ndmp_procConnCloseReq(ndmpSess_t *sess);
int ndmp_procConnOpenReq(ndmpSess_t *sess);
int ndmp_procConnCloseReq(ndmpSess_t *sess);
int ndmp_procConnClientAuthReq(ndmpSess_t *sess);
int ndmp_procConnSrvrAuthReq(ndmpSess_t *sess);
int32 ndmp_procConnSrvrAuthRsp(ndmpSess_t *sess, char8 *rspbuf, ndmp_src_t *src);

int ndmp_procCfgGetHostInfoReq(ndmpSess_t *sess);
int ndmp_procCfgGetSrvrInfoReq(ndmpSess_t *sess);
int ndmp_procCfgGetConnTypeReq(ndmpSess_t *sess);
int ndmp_procCfgGetAuthAttrReq(ndmpSess_t *sess);
int ndmp_procCfgGetButypeInfoReq(ndmpSess_t *sess);
int ndmp_procCfgGetFsInfoReq(ndmpSess_t *sess);
int ndmp_procCfgGetTapeInfoReq(ndmpSess_t *sess);
int ndmp_procCfgGetScsiInfoReq(ndmpSess_t *sess);

int ndmp_procDataGetStateReq(ndmpSess_t *sess);
int ndmp_procDataGetEnvReq(ndmpSess_t *sess);
int ndmp_procDataConnReq(ndmpSess_t *sess);
int ndmp_procDataListenReq(ndmpSess_t *sess);
int ndmp_procDataStartBkupReq(ndmpSess_t *sess);
int ndmp_procDataStartRecovReq(ndmpSess_t *sess);
int ndmp_procDataAbortReq(ndmpSess_t *sess);
int ndmp_procDataStopReq(ndmpSess_t *sess);

int ndmp_procMoverGetStateReq(ndmpSess_t *sess);
int ndmp_procMoverListenReq(ndmpSess_t *sess);
int ndmp_procMoverConnReq(ndmpSess_t *sess);
int ndmp_procMoverContReq(ndmpSess_t *sess);
int ndmp_procMoverReadReq(ndmpSess_t *sess);
int ndmp_procMoverAbortReq(ndmpSess_t *sess);
int ndmp_procMoverCloseReq(ndmpSess_t *sess);
int ndmp_procMoverStopReq(ndmpSess_t *sess);
int ndmp_procMoverSetWdwReq(ndmpSess_t *sess);
int ndmp_procMoverSetRecSizReq(ndmpSess_t *sess);
int ndmp_procFileHistMsg(ndmpSess_t *sess,  char8 *rcvbuf, ndmp_src_t *src);
int ndmp_procLogMsg(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_procLogFile(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_procNotifHaltedMsg(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);

int ndmp_procScsiOpenReq(ndmpSess_t *sess);
int ndmp_procScsiCloseReq(ndmpSess_t *sess);
int ndmp_procScsiGetStateReq(ndmpSess_t *sess);
int ndmp_procScsiExecCdbReq(ndmpSess_t *sess);
int ndmp_procScsiResetDevReq(ndmpSess_t *sess);
int ndmp_procScsiResetBusReq(ndmpSess_t *sess);
int ndmp_procScsiSetTargetReq(ndmpSess_t *sess);

int ndmp_procTapeOpenReq(ndmpSess_t *sess);
int ndmp_procTapeCloseReq(ndmpSess_t *sess);
int ndmp_procCloseCTagReq(uint32 sessId, char *dev);
int ndmp_procTapeGetStateReq(ndmpSess_t *sess);
int ndmp_procTapeWriteReq(ndmpSess_t *sess);
int ndmp_procTapeReadReq(ndmpSess_t *sess);
int ndmp_procTapeMtioReq(ndmpSess_t *sess);
int ndmp_procTapeExecCdbReq(ndmpSess_t *sess);

int32 ndmp_chkDevOpen(ndmpSess_t *sess, void *cl_rsp, int32 dev_typ);
int32 sndMsgtoTapeDrv(ndmpSess_t *sess, void *Req, int32 req_len, void *cl_rsp);

int32 ndmp_procTapeGetDevLstRsp(ndmpSess_t *sess, char8 *rcvbuf);
int32 ndmp_procTapeOpenRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeCloseRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeSpaceRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeWrFMRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeSetBlkSizRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeGetStateRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeLoadRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeReadRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapeWriteRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procTapePassThruRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);

int32 ndmp_procScsiGetDevLstRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procScsiOpenRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procScsiCloseRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_procScsiPassThruRsp(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);

int32 ndmp_sendTapeSpaceReq(ndmpSess_t *sess, char* rcvbuf, ndmp_src_t *src);
int32 ndmp_updateEnvReq(ndmpSess_t *sess, char* rcvbuf, ndmp_src_t *src);
int32 ndmp_addEnvReq(ndmpSess_t *sess, char* rcvbuf, ndmp_src_t *src);
int32 ndmp_procVolLstRsp(ndmpSess_t *sess, char8 *rcvbuf);
int32 ndmp_procFSInfoRsp(ndmpSess_t *sess, char8 *rcvbuf);

void ndmp_sendSnapRemoveIndication(const int64 volId, const char * const snapName);

void ndmp_deleteDumpRecord(const char *const fileName, const char *const snapname);
void ndmp_manageSnapShots(const char *const fileName, const int64 volId,
                          const dmpdate_t *const ddrec, const int32 tmp_all);
void ndmp_cleanDumpDate(char *evt);
int32 ndmp_setDumpDate(ndmpSess_t *sess, char8 *rcvbuf, ndmp_src_t *src);
int32 ndmp_sendCfgFSInfoRsp(ndmpSess_t *sess);
int32 ndmp_fwdMsgtoNNIM(ndmpSess_t *sess, uint32 ndmp_nfx_mid, int32 userInfo);
int32 ndmp_rspClient(ndmpSess_t *sess, void *cl_rsp);
int ndmp_sendFSInfoReq(ndmpSess_t *sess);
void ndmp_resetTapeDevState(ndmpSess_t *sess);
int32 ndmp_procDataGetStateReq(ndmpSess_t *sess);
int32 ndmp_ListenConnErrChk(ndmpSess_t *sess, int32 mode, int32 addr_type);
int32 ndmp_procDataGetEnvRsp(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int32 ndmp_procDM_Rsp(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int32 ndmp_procStartRecv_Rsp(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int32 ndmp_procNotifDM_Rsp(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);

int32 ndmp_sndLogMsg( ndmpSess_t *sess, int32 logTyp, char *msg, int32 msgid);
ndmpMsgEnt_t *ndmpGetMsgEnt(ndmpMsgEnt_t *entry, ndmp_Msg msgid);
int ndmpv2_procCfgGetHostInfoReq(ndmpSess_t *sess);
int ndmpv2_procCfgGetButypeAttrReq(ndmpSess_t *sess);
int ndmpv2_procDataStartBkupReq(ndmpSess_t *sess);
int ndmpv2_procDataStartRecovReq(ndmpSess_t *sess);
ndmpAgileMsgEnt_t *ndmpGetAgileMsgEnt(int msgid);

int32 ndmpGetFreeSessId(int32 *sessId);
void ndmpMarkFreeSessId(pid_t pid);
int32 ndmpFreeSessId(pid_t pid, int32 idx);
void ndmp_InitSessIdTbl(void);
void ndmp_readRegistryEntry(void);
void ndmpSetSessId(int idx, int32 sessId, int32 pid, 
		struct sockaddr_in *from);
ndmpSess_t *ndmp_CreateSess(int clientFd, int32 idx, int32 sessId,  
		struct sockaddr_in *from);
int free_env_on_SSC(Data_SSC_t *data);
char *decode_tape_ecode(int32 ecode);
char *decode_tape_mtio(int32 tape_op);
void ndmp_regSetOption(char *option, char *value);

/*** Func done by Parent ***/

ndmpParentUIMsgEnt_t *ndmpGetParentUIMsgEnt(int msgid); 
int ndmp_procChildReport(char *rcvbuf, ndmp_src_t *src);
int ndmp_procChildSndStats(char *rcvbuf, ndmp_src_t *src);
int ndmp_procEnaDisNdmpReq(char *rcvbuf, ndmp_src_t *src); 
int ndmp_procGetSessTblReq(char *rcvbuf, ndmp_src_t *src);
int ndmp_procSetDbgLevel(char *rcvbuf, ndmp_src_t *src);
int ndmp_procSetTraceFlag(char *rcvbuf, ndmp_src_t *src);
int ndmp_procSetTraceFiles(char *rcvbuf, ndmp_src_t *src);
int ndmp_procSetRestoreMode(char *rcvbuf, ndmp_src_t *src);
int ndmp_procSetVerReq(char *rcvbuf, ndmp_src_t *src);
int ndmp_procShowStateReq(char *rcvbuf, ndmp_src_t *src);
int ndmp_handleNfxEvent(char *rcvbuf, ndmp_src_t *src);
int ndmp_procShowStatisticReq(char *rcvbuf, ndmp_src_t *src);
int ndmp_procClearStatisticReq(char *rcvbuf, ndmp_src_t *src);

/*** Func done by Children ***/

int ndmp_ChildRpt2Parent(ndmpSess_t *sess);
int ndmp_ChildSndStats2Parent(ndmpSess_t *sess);
int ndmp_Child_ProcShowStatusReq(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_Child_ProcDelSessReq(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_Child_ProcSetDbgLevel(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_Child_ProcSetTraceFlag(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);
int ndmp_Child_ProcShowStatisticReq(ndmpSess_t *sess, char *rcvbuf, ndmp_src_t *src);

int ndmp_procUIDMGetStateRsp(ndmpSess_t *sess, char* rcvbuf, ndmp_src_t *src);

/*** Func to print messages ***/

void print_hex_dump(char *dmp, int32 len, char *t, FILE *fp);
void ndmp_print_hdr(ndmp_Hdr *hdr, char *msg_name, int32 rcv, ndmpSess_t *s);
void prt_xdr_ndmp_conn_open_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_conn_open_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_conn_client_auth_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_conn_client_auth_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_conn_srvr_auth_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_conn_srvr_auth_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_host_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_srvr_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_conn_type_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_auth_attr_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_auth_attr_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_butype_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_fs_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_tape_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_cfg_get_scsi_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_notify_data_halted_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_notify_conn_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_notify_mover_halted_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_notify_mover_paused_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_notify_data_read_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_log_file_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_log_msg_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_fh_add_file_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_fh_add_dir_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_fh_add_node_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_open_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_open_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_close_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_exec_cdb_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_exec_cdb_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_reset_dev_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_reset_bus_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_scsi_set_target_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_open_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_open_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_close_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_mtio_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_mtio_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_write_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_write_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_read_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_read_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_exec_cdb_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_tape_exec_cdb_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_start_bkup_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_start_bkup_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_start_recover_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_start_recover_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_abort_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_get_env_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_stop_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_listen_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_listen_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_conn_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_data_conn_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_listen_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_listen_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_cont_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_abort_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_stop_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_set_window_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_set_window_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_read_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_read_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_close_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_set_record_size_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_set_record_size_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_conn_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmp_mover_conn_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_cfg_get_host_info_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_cfg_get_butype_attr_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_cfg_get_butype_attr_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_log_log_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_log_dbg_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_log_file_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_fh_add_file_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_fh_add_dir_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_fh_add_node_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_tape_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_data_get_state_rsp(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_data_start_bkup_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_data_start_recover_req(char *msg, ndmpSess_t *s);
void prt_xdr_ndmpv2_mover_get_state_rsp(char *msg, ndmpSess_t *s);

#endif /* _NDMP_SSC_H_ */
