/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-main.c
 *
 * RCS:  $Id: ndmp-test.c,v 1.10 2002/01/16 22:50:29 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include "ndmp.h"
#include "ndmp-fs-api.h"
#include <exports/fs-ndmp-api.h>

//#ifdef TEST_FAKE_FS

#define LOG_BK_IN_PROGRESS	"Yep babe, a backup is in progress\n"
#define LOG_RST_IN_PROGRESS	"Yo dude, a restore is in progress\n"
#define FH_FILE_NAME		"/homes/anhv/me/you"
#define SNAPSHOT_STR		"SnapShotStr1"

ndmp_NNIMCtrl_t ndmp_FSCtrl;
int	sent_one_block = FALSE;
 
void ndmp_FSReceiveMsg(eee_desc_t *eDesc);

/*------------------------------------------------------------------------------ 
 * Function Name:   ndmp_fs_test_init
 *
 * Description:     Initializes the NDMP services on the N-NIM, ie. Data and Mover
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *
 *------------------------------------------------------------------------------ */
int32
ndmp_fs_test_init()
{
    int32       rc;
 
    /*
     *--------------------------------------
     * Register with EEE and get app id
     *--------------------------------------
     */
    NDMP_DBG_PRINTF("ndmp_fs_test_init:  Begin\n");
    memset ( &ndmp_FSCtrl, 0, sizeof (ndmp_NNIMCtrl_t));
    if ((rc = eee_allocateApplicationId("ndmp_fs")) == NFX_ERR)
    {
        NDMP_DBG_PRINTF("ndmp_fs_test_init:  eeeAllocate app id failed.\n");
        return NFX_ERR;
    }
    NDMP_DBG_PRINTF("ndmp_fs_test_init:  ndmp_fs_tst_appId = %d\n", rc);
    ndmp_FSCtrl.appId = rc;
    ndmp_FSCtrl.src_port = PACK_UNICAST_PORT_ADDRESS(
                                             nfxMySlotId,       /* src slot */
                                             nfxMyCPUId,        /* src cpu */
                                             ndmp_FSCtrl.appId,
                                             EEE_CONTROL,
                                             NFX_COS_HIGH,
											 0);
 
	NDMP_DBG_PRINTF("ndmp_fs_test_init:  src_port = %d\n", ndmp_FSCtrl.src_port);
    if (eee_registerReceiveFunc( ndmp_FSCtrl.appId,   /* App Id        */
                                  ndmp_FSReceiveMsg,  /* Ucast Rx Func */
                                  NULL) != NFX_OK)     /* Mcast Rx Func */
    {
        NDMP_DBG_PRINTF("ndmp_fs_test_init:  Register eee Rx func failed.");
        return NFX_ERR;
    }
    return NFX_OK;
 
}  /* ndmp_fs_test_init */     
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_FSsendWrReq
 *
 * Description:     Send a Write cmd to the NDMP Data server to write a 1k
 *					buffer of fake data.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 *------------------------------------------------------------------------------ */     
int
ndmp_FSsendWrReq(uint32 dst_port, void *cookie, int start_seq)
{                                           

	ndmp_write_data_req_msg_t	wr_req;	
	int	i, j;

	// then send a WRITE cmd to ndmp server
    wr_req.hdr.mid = NDMP_MID_WRITE_DATA_REQ;
    wr_req.hdr.ndmp_cookie = cookie;    // return ndmp cookie
    wr_req.cl_cookie = (void *)33; // fs cookie
 
    // fill up the buffer
    for (i = start_seq; i < (start_seq+8); i++)
	{
    	for (j = 0; j < 128; j++)
        	wr_req.buf[i+j] = j;
	} 
    NDMP_DBG_PRINTF("-------------------------------------------------------------------\n");
    NDMP_DBG_PRINTF("This is the data we're writing to tape:\n");
    for (i = 0; i < 1024; i++)
    {
       if ((i % 16) == 0)
           NDMP_DBG_PRINTF("\n[0x%04x]: ", i);
       NDMP_DBG_PRINTF("%02x ", wr_req.buf[i]);
    }
    NDMP_DBG_PRINTF("\n");
    NDMP_DBG_PRINTF("-------------------------------------------------------------------\n"); 
    // send it
    eee_sendMessage((void *) &wr_req, sizeof(wr_req), dst_port, ndmp_FSCtrl.src_port);                         
	return NFX_OK;
}
/*
 *------------------------------------------------------------------------------ 
 * Function Name:   ndmp_FSReceiveMsg
 *
 * Description:     Sends cmd response event to the Data and Mover
 *                  state machines.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 *------------------------------------------------------------------------------ */
void
ndmp_FSReceiveMsg(eee_desc_t *eDesc)
{
	fs_MsgReqHdr  				*fs_req_hdr;
    char          				*msgp;
	ndmp_read_data_req_msg_t	rd_req;	
	fs_MsgStatVolResp			fs_stat_rsp;
	ndmp_ops_cmplt_req_msg_t  	ops_done;
	ndmp_send_log_req_msg_t		*snd_log;	// send log
	ndmp_send_fh_req_msg_t  	*snd_fh;		// send file history
	ndmp_set_dumpdate_req_msg_t	*set_dd;		// set dumpdate
	int32						i, msglen, rc; 
	uint32						buffSize; 
	fs_MsgStartDumpReq_t		*fs_dmp_req_start;
	fs_MsgStartRestoreReq_t		*fs_rest_req_start;
	fs_MsgDumpIODoneReq_t		*fs_dmp_req_iodone;
	fs_MsgRestoreIODoneReq_t	*fs_rest_req_iodone;
 
    NDMP_DBG_PRINTF("ndmp_FSReceiveMsg:  Begin\n");
 
    if( !eDesc ) {
        NDMP_DBG_PRINTF("Null eDesc\n");
        return;
    }
 
    buffSize = eDesc->hdr.control & EDESC_TOTAL_LEN;
    /*
     * convert EEE packet to flat buffer
     */                         
	msgp = eee_ramAllocZero(buffSize);
    if (msgp == NULL)
    {
        NDMP_DBG_PRINTF("ndmp_FSReceiveMsg: no memory for eee Flat buffer");
        return;
    }
    if ((rc=eee_copyPacketToLocalBuffer(msgp, eDesc, buffSize, 0)) != buffSize)
    {
        NDMP_DBG_PRINTF("ndmp_FSReceiveMsg: error flattening EEE packet (expected %d got %d)",
                  buffSize, rc);
		eee_freePacket(eDesc);
        return;
    }

    /*
     * Point to request message portion of data buffer.
     */
	fs_req_hdr = (fs_MsgReqHdr *) msgp;
 
    switch( fs_req_hdr->requestType )
    {                           
        case FS_REQ_DUMP_START:    
			// we got a start request, first send a set dumpdate 
			// cmd to ndmp server
			fs_dmp_req_start = (fs_MsgStartDumpReq_t *)msgp;

			msglen = sizeof(ndmp_set_dumpdate_req_msg_t) + strlen(SNAPSHOT_STR) -1;
			set_dd = (ndmp_set_dumpdate_req_msg_t *)eee_ramAllocZero(msglen);
			set_dd->hdr.mid = NDMP_MID_SET_DUMPDATE_REQ;
			set_dd->hdr.ndmp_cookie = (void *)fs_dmp_req_start->start.cookie;	// return ndmp cookie
			set_dd->cl_cookie = (void *)33; // fs cookie
			set_dd->dump_level = 1;
			memcpy(&(set_dd->file_sys_name), &fs_dmp_req_start->volId , sizeof(evm_volId_t)); 
			set_dd->date = 1234567;
			set_dd->snapshot_len = strlen(SNAPSHOT_STR);
			memcpy(set_dd->snapshot, SNAPSHOT_STR, strlen(SNAPSHOT_STR));

			// send it
			eee_sendMessage((void *) set_dd, msglen, 
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		

			// then send a WRITE cmd to ndmp server
			ndmp_FSsendWrReq(eDesc->hdr.src_port, fs_dmp_req_start->start.cookie, 0);
			sent_one_block = TRUE;
			break;

		case FS_REQ_DUMP_IODONE:
			// send a log saying we're in progress
			fs_dmp_req_iodone = (fs_MsgDumpIODoneReq_t *)msgp;
			msglen = sizeof(ndmp_send_log_req_msg_t) + strlen(LOG_BK_IN_PROGRESS) -1;
			snd_log = (ndmp_send_log_req_msg_t *)eee_ramAllocZero(msglen);
			snd_log->hdr.mid = NDMP_MID_SEND_LOG_REQ;
			snd_log->hdr.ndmp_cookie = (void *)fs_dmp_req_iodone->iodone.cookie;	// return ndmp cookie
			snd_log->cl_cookie = (void *)33; // fs cookie
			snd_log->log_msgTyp = NDMP_LOG_MSG_TYP_INFO;
			snd_log->file_name_len = 0;
			snd_log->log_txt_len = strlen(LOG_BK_IN_PROGRESS);
			memcpy(snd_log->blob, LOG_BK_IN_PROGRESS, strlen(LOG_BK_IN_PROGRESS));
			// send it
			eee_sendMessage((void *) snd_log, msglen,
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		

			if (sent_one_block == TRUE)
			{
				// send a second WRITE cmd to ndmp server
				ndmp_FSsendWrReq(eDesc->hdr.src_port, fs_dmp_req_iodone->iodone.cookie, 2);
				eee_freePacket(eDesc);
				sent_one_block = FALSE;
				return;
			}
			// Send File History
			msglen = sizeof(ndmp_send_fh_req_msg_t) + strlen(FH_FILE_NAME) -1;
			snd_fh = (ndmp_send_fh_req_msg_t *)eee_ramAllocZero(msglen);
			snd_fh->hdr.mid = NDMP_MID_SEND_FH_REQ;
			snd_fh->hdr.ndmp_cookie = (void *)fs_dmp_req_iodone->iodone.cookie;	// return ndmp cookie
			snd_fh->cl_cookie = (void *)33; // fs cookie
			snd_fh->inode = 7;
			snd_fh->file_type = NDMP_F_REG;
			snd_fh->mtime = 4000;
			snd_fh->atime = 3000;
			snd_fh->ctime = 2000;
			snd_fh->group = 9;
			snd_fh->owner = 1;
			snd_fh->fattr = 22;
			snd_fh->size = 2450;
			snd_fh->links = 3;
			snd_fh->file_name_len = strlen(FH_FILE_NAME);
			memcpy(snd_fh->file_name, FH_FILE_NAME, strlen(FH_FILE_NAME));

			// send it
			eee_sendMessage((void *) snd_fh, msglen,
					eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		

			// ndmp server sends this when it's done writing to tape
			// we either send more or say dumping's done, say we're done
		
			ops_done.hdr.mid = NDMP_MID_OPS_COMPLETE_REQ;
			ops_done.hdr.ndmp_cookie = (void *)fs_dmp_req_iodone->iodone.cookie;	// return ndmp cookie
			ops_done.cl_cookie = (void *)33; // fs cookie
			ops_done.status = FS_ERR_NONE;
			// send it
			eee_sendMessage((void *) &ops_done, sizeof(ops_done), 
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		
			break;

        case FS_REQ_REST_START:    
			// we got a start request for RESTORE, first send a set dumpdate 
			// cmd to ndmp server
			fs_rest_req_start = (fs_MsgStartRestoreReq_t *)msgp;

			// send a READ cmd to ndmp server
			rd_req.hdr.mid = NDMP_MID_READ_DATA_REQ;
			rd_req.hdr.ndmp_cookie = (void *)fs_rest_req_start->start.cookie;	// return ndmp cookie
			rd_req.cl_cookie = (void *)33; // fs cookie

			// send it
			eee_sendMessage((void *) &rd_req, sizeof(rd_req), 
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		
			break;

		case FS_REQ_REST_IODONE:
			// send a log saying we're in progress
			fs_rest_req_iodone = (fs_MsgRestoreIODoneReq_t *)msgp;
			NDMP_DBG_PRINTF("-------------------------------------------------------------------\n");
			NDMP_DBG_PRINTF("This is the data we got from tape for the restore:\n");
			for (i = 0; i<fs_rest_req_iodone->iodone.dataLen; i++)
			{
				if ((i % 16) == 0)
                	NDMP_DBG_PRINTF("\n[0x%04x]: ", i);
           		NDMP_DBG_PRINTF("%02x ", fs_rest_req_iodone->iodone.data[i]);
			}
			NDMP_DBG_PRINTF("\n");
			NDMP_DBG_PRINTF("-------------------------------------------------------------------\n");
			msglen = sizeof(ndmp_send_log_req_msg_t) + strlen(LOG_RST_IN_PROGRESS) -1;
			snd_log = (ndmp_send_log_req_msg_t *)eee_ramAllocZero(msglen);
			snd_log->hdr.mid = NDMP_MID_SEND_LOG_REQ;
			snd_log->hdr.ndmp_cookie = (void *)fs_rest_req_iodone->iodone.cookie;	// return ndmp cookie
			snd_log->cl_cookie = (void *)33; // fs cookie
			snd_log->log_msgTyp = NDMP_LOG_MSG_TYP_INFO;
			snd_log->file_name_len = 0;
			snd_log->log_txt_len = strlen(LOG_RST_IN_PROGRESS);
			memcpy(snd_log->blob, LOG_RST_IN_PROGRESS, strlen(LOG_RST_IN_PROGRESS));
			// send it
			eee_sendMessage((void *) snd_log, msglen,
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		
			eee_ramDealloc(snd_log);

			// ndmp server sends this when it's done READING  from tape
			// we either ask for more or say restoring's done, say we're done
			
			ops_done.hdr.mid = NDMP_MID_OPS_COMPLETE_REQ;
			ops_done.hdr.ndmp_cookie = (void *)fs_rest_req_iodone->iodone.cookie;	// return ndmp cookie
			ops_done.cl_cookie = (void *)33; // fs cookie
			ops_done.status = FS_ERR_NONE;
			// send it
			eee_sendMessage((void *) &ops_done, sizeof(ops_done), 
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		
			break;

		case FS_REQ_FSSTAT_VOL:
			// make a bogus reply for our FS stat
			fs_stat_rsp.hdr.requestType = FS_REQ_FSSTAT_VOL;
			fs_stat_rsp.stat.fsBlockSize = 100;
			fs_stat_rsp.stat.numBlks = 3;
			fs_stat_rsp.stat.numBlksAvail = 20;
			fs_stat_rsp.stat.numBlksAvailUser = 1;
			fs_stat_rsp.stat.numInodes = 2500;
			fs_stat_rsp.stat.numInodesAvail = 2490;
			fs_stat_rsp.stat.numInodesAvailUser = 103;
			fs_stat_rsp.stat.readsPerSec = 20;
			fs_stat_rsp.stat.writesPerSec = 30;
			fs_stat_rsp.stat.avgReadSize = 2;
			fs_stat_rsp.stat.avgWriteSize = 1;

			// send it
			eee_sendMessage((void *) &fs_stat_rsp, sizeof(fs_stat_rsp), 
						eDesc->hdr.src_port, ndmp_FSCtrl.src_port);		
			break;
		default:	
			printf("ndmp_FSReceiveMsg:  Invalid msg received %d\n", fs_req_hdr->requestType);
	}
	eee_freePacket(eDesc);
	return;
}
//#endif /* TEST_FAKE_FS */
