
/*
 *-----------------------------------------------------------------
 *
 * Name:  ndmp-trace-msg.c
 *
 * RCS: 
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: Contain print funcs for NDMP message
 *
 * Created by: Nitin Jain
 *
 * Date Created: 08/20/02
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/anpssc-api.h>

#include "ndmp.h"
#include "ndmp-ssc.h"
#include "ndmp-debug.h"


void print_hex_dump(char *dmp, int32 len, char *t, FILE *fp)
{
    int32 i = 0, j, cnt;

    while (i < len)
    {
        fprintf(fp,"%s", t);
        cnt = ((len-i)>16)?16:(len-i);
        for (j = 0; j < cnt; j++)
            fprintf(fp,"%02x ", (uchar8)*(dmp+i+j));
        fprintf(fp,"\n");
        i += j;
    }

    return;
}

void ndmp_print_hdr(ndmp_Hdr *hdr, char *msg_name, int32 rcv, ndmpSess_t *s)
{
    struct timeval time;

    if (!s->logFp)
        return;

	if ((hdr->msgId != NDMP_MOVER_GET_STATE) && 
			(hdr->msgId != NDMP_FH_ADD_UNIX_DIR) &&
			(hdr->msgId != NDMP_FH_ADD_UNIX_NODE) &&
			(hdr->msgId != NDMP_FH_ADD_DIR) &&
			(hdr->msgId != NDMP_FH_ADD_UNIX_NODE) ) 
	{
    	gettimeofday(&time, 0);
    	if (rcv == RCV_MSG)
        	fprintf(s->logFp,"\nRcv (Sess:%d)(time:%ld)\n",
                                                   s->sessId, time.tv_sec);
    	else
        	fprintf(s->logFp,"\nXmt (Sess:%d)(time:%ld)\n",
                                                   s->sessId, time.tv_sec);
    	fprintf(s->logFp,"MID : 0x%x (%s)\n", hdr->msgId, msg_name);
    	fprintf(s->logFp,"XSeq : %ld\n", hdr->xmtSeq);
    	fprintf(s->logFp,"Tstamp : %ld\n", hdr->time_stamp);
    	fprintf(s->logFp,"RSeq : %ld\n", hdr->rspSeq);
    	fprintf(s->logFp,"Err : %d\n", hdr->error);
	}
    return;
}

void prt_xdr_ndmp_conn_open_req(char *msg, ndmpSess_t *s)
{
    ndmp_conn_open_req  *req = (ndmp_conn_open_req *) msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tVersion : %d\n", req->protoVer);

    return;
}

void prt_xdr_ndmp_conn_open_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_conn_open_rsp *rsp = (ndmp_conn_open_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_conn_client_auth_req(char *msg, ndmpSess_t *s)
{
    ndmp_conn_client_auth_req *req = (ndmp_conn_client_auth_req *)msg;
    int32 auth_type = req->auth_data.authTyp;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAuth Type : %d (%s)\n", auth_type,
                                 (auth_type == NDMP_AUTHNONE)?"None":
                                 (auth_type == NDMP_AUTHTXT)?"Text":
                                 (auth_type == NDMP_AUTHMD5)?"MD5":"Invalid");
    if (auth_type == NDMP_AUTHTXT)
    {
        fprintf(s->logFp,"\tId : %s\n",
                             req->auth_data.ndmp_AuthData_u.authTxt.authId);
        fprintf(s->logFp,"\tPswd : %s\n",
                             req->auth_data.ndmp_AuthData_u.authTxt.authPswd);
    }
    else if (auth_type == NDMP_AUTHMD5)
    {
        fprintf(s->logFp,"\tId : %s\n",
                           req->auth_data.ndmp_AuthData_u.authMd5.authId);
        fprintf(s->logFp,"\tDigest :");
        //                req->auth_data.ndmp_AuthData_u.authMd5.authDigest);
        print_hex_dump(req->auth_data.ndmp_AuthData_u.authMd5.authDigest,
                       16, " ", s->logFp);
    }

    return;
}

void prt_xdr_ndmp_conn_client_auth_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_conn_client_auth_rsp *rsp = (ndmp_conn_client_auth_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_conn_srvr_auth_req(char *msg, ndmpSess_t *s)
{
    ndmp_conn_srvr_auth_req *req = (ndmp_conn_srvr_auth_req *)msg;
    int32 reqAuthTyp = req->client_attr.authTyp;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAuth Type : %d (%s)\n", reqAuthTyp,
                                (reqAuthTyp == NDMP_AUTHNONE)?"None":
                                (reqAuthTyp == NDMP_AUTHTXT)?"Text":
                                (reqAuthTyp == NDMP_AUTHMD5)?"MD5":"Invalid");
    if (reqAuthTyp == NDMP_AUTHMD5)
        fprintf(s->logFp,"\tChallenge : %s\n",
                             req->client_attr.ndmp_AuthAttr_u.challenge);

    return;
}

void prt_xdr_ndmp_conn_srvr_auth_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_conn_srvr_auth_rsp *rsp = (ndmp_conn_srvr_auth_rsp *)msg;
    int32 auth_type = rsp->srvr_result.authTyp;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tAuth Type : %d (%s)\n", auth_type,
                                 (auth_type == NDMP_AUTHNONE)?"None":
                                 (auth_type == NDMP_AUTHTXT)?"Text":
                                 (auth_type == NDMP_AUTHMD5)?"MD5":"Invalid");
    if (auth_type == NDMP_AUTHTXT)
    {
        fprintf(s->logFp,"\tId : %s\n",
                             rsp->srvr_result.ndmp_AuthData_u.authTxt.authId);
        fprintf(s->logFp,"\tPswd : %s\n",
                           rsp->srvr_result.ndmp_AuthData_u.authTxt.authPswd);
    }
    else if (auth_type == NDMP_AUTHMD5)
    {
        fprintf(s->logFp,"\tId : %s\n",
                         rsp->srvr_result.ndmp_AuthData_u.authMd5.authId);
        fprintf(s->logFp,"\tDigest :");
        //               rsp->srvr_result.ndmp_AuthData_u.authMd5.authDigest);
        print_hex_dump(rsp->srvr_result.ndmp_AuthData_u.authMd5.authDigest,
                       16, " ", s->logFp);
    }

    return;
}

void prt_xdr_ndmp_cfg_get_host_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_host_info_rsp *rsp = (ndmp_cfg_get_host_info_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tHostname : %s\n", rsp->hostname);
    fprintf(s->logFp,"\tOS Type : %s\n", rsp->os_type);
    fprintf(s->logFp,"\tOS Vers : %s\n", rsp->os_vers);
    fprintf(s->logFp,"\tHostID : %s\n", rsp->hostid);

    return;
}

void prt_xdr_ndmp_cfg_get_srvr_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_srvr_info_rsp *rsp = (ndmp_cfg_get_srvr_info_rsp *)msg;
    int32 auth_type = *rsp->authTyp.authTyp_val;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tVendor Name : %s\n", rsp->vendorName);
    fprintf(s->logFp,"\tProd Name : %s\n", rsp->prodName);
    fprintf(s->logFp,"\tRev. Num : %s\n", rsp->revNum);
    fprintf(s->logFp,"\tAuth type len : %d\n", rsp->authTyp.authTyp_len);
    fprintf(s->logFp,"\tAuth type val : %d (%s)\n", auth_type,
                                 (auth_type == NDMP_AUTHNONE)?"None":
                                 (auth_type == NDMP_AUTHTXT)?"Text":
                                 (auth_type == NDMP_AUTHMD5)?"MD5":"Invalid");

    return;
}

void prt_xdr_ndmp_cfg_get_conn_type_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_conn_type_rsp *rsp = (ndmp_cfg_get_conn_type_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tAddr. type len: %d\n", rsp->addrTypes.addrTypes_len);
    fprintf(s->logFp,"\tAddr. type val: %d (%s)\n",
                             *rsp->addrTypes.addrTypes_val,
                             ((*rsp->addrTypes.addrTypes_val)?"TCP":"Local"));

    return;
}

void prt_xdr_ndmp_cfg_get_auth_attr_req(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_auth_attr_req *req = (ndmp_cfg_get_auth_attr_req *)msg;
    int32 auth_type = req->authTyp;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAuth type : %d (%s)\n", auth_type,
                                 (auth_type == NDMP_AUTHNONE)?"None":
                                 (auth_type == NDMP_AUTHTXT)?"Text":
                                 (auth_type == NDMP_AUTHMD5)?"MD5":"Invalid");

    return;
}

void prt_xdr_ndmp_cfg_get_auth_attr_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_auth_attr_rsp *rsp = (ndmp_cfg_get_auth_attr_rsp *)msg;
    int32 rspAuthType = rsp->srvrAttr.authTyp;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tAuth type : %d (%s)\n", rspAuthType,
                                (rspAuthType == NDMP_AUTHNONE)?"None":
                                (rspAuthType == NDMP_AUTHTXT)?"Text":
                                (rspAuthType == NDMP_AUTHMD5)?"MD5":"Invalid");
    if (rspAuthType == NDMP_AUTHMD5)
        fprintf(s->logFp,"\tChallenge : %s\n",
                             rsp->srvrAttr.ndmp_AuthAttr_u.challenge);
    

    return;
}

void prt_xdr_ndmp_cfg_get_butype_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_butype_info_rsp *rsp = (ndmp_cfg_get_butype_info_rsp *)msg;
    ndmp_butype_info  *bu_info;
    ndmp_pval *env_lst;
    int32 i, j;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tNum. Bkup info. : %d\n",
                                         rsp->butype_info.butype_info_len);
    for (i = 0; i < rsp->butype_info.butype_info_len; i++)
    {
        bu_info = &rsp->butype_info.butype_info_val[i];
        fprintf(s->logFp,"\tBkup type name[%i] : %s\n",i,bu_info->butype_name);
        fprintf(s->logFp,"\tNum. env : %d\n", bu_info->defEnv.defEnv_len);
        for (j = 0; j < bu_info->defEnv.defEnv_len; j++)
        {
            env_lst = &bu_info->defEnv.defEnv_val[j];
            fprintf(s->logFp,"\t\tName [Value] : %s [%s]\n",
                                     env_lst->name, env_lst->value);
        }
    }

    return;
}

void prt_xdr_ndmp_cfg_get_fs_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_fs_info_rsp *rsp = (ndmp_cfg_get_fs_info_rsp *)msg;
    ndmp_fs_info *fs_info;
    ndmp_pval *pval;
    int32 i, j;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tNum. FS : %d\n", rsp->fs_info.fs_info_len);

    for (i = 0; i < rsp->fs_info.fs_info_len; i++)
    {
        fs_info = &rsp->fs_info.fs_info_val[i];
        fprintf(s->logFp,"\tFS[%d]\n", i);
        fprintf(s->logFp,"\t\tinvalid : %ld\n", fs_info->invalid);
        fprintf(s->logFp,"\t\ttype : %s\n", fs_info->fs_type);
        fprintf(s->logFp,"\t\tLogical dev. : %s\n", fs_info->fs_logical_device);
        fprintf(s->logFp,"\t\tPhys. dev. : %s\n", fs_info->fs_physical_device);
        fprintf(s->logFp,"\t\tTot. size(H L) : %ld %ld\n",
                           fs_info->total_size.high, fs_info->total_size.low);
        fprintf(s->logFp,"\t\tUsed size(H L) : %ld %ld\n",
                             fs_info->used_size.high, fs_info->used_size.low);
        fprintf(s->logFp,"\t\tAvail. size(H L) : %ld %ld\n",
                           fs_info->avail_size.high, fs_info->avail_size.low);
        fprintf(s->logFp,"\t\tTot. Inodes (H L) : %ld %ld\n",
                       fs_info->total_inodes.high, fs_info->total_inodes.low);
        fprintf(s->logFp,"\t\tUsed Inodes (H L) : %ld %ld\n",
                         fs_info->used_inodes.high, fs_info->used_inodes.low);
        fprintf(s->logFp,"\t\tStatus : %s\n", fs_info->fs_status);

        fprintf(s->logFp,"\t\tNum. FS env. : %d\n", fs_info->fs_env.fs_env_len);

        for (j = 0; j < fs_info->fs_env.fs_env_len; j++)
        {
            pval = &fs_info->fs_env.fs_env_val[j];
            fprintf(s->logFp,"\t\t\tName [Value] : %s [%s]",
                                                 pval->name, pval->value);
        }
    }

    return;
}

void prt_xdr_ndmp_cfg_get_tape_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_tape_info_rsp *rsp = (ndmp_cfg_get_tape_info_rsp *)msg;
    ndmp_dev_info *tinfo;
    ndmp_dev_capab *clist;
    ndmp_pval *pval;
    int32 i, j, k;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tNum. tapes : %d\n", rsp->tape_info.tape_info_len);

    for (i = 0; i < rsp->tape_info.tape_info_len; i++)
    {
        tinfo = &rsp->tape_info.tape_info_val[i];
        fprintf(s->logFp,"\t\tModel [%d] : %s\n", i, tinfo->model);
        fprintf(s->logFp,"\t\tNum. capb. : %d\n", tinfo->caplist.caplist_len);
        for (j = 0; j < tinfo->caplist.caplist_len; j++)
        {
            clist = &tinfo->caplist.caplist_val[j];
            fprintf(s->logFp,"\t\t\tDev name : %s\n", clist->devname);
            fprintf(s->logFp,"\t\t\tDev attr : %ld\n", clist->attr);
            fprintf(s->logFp,"\t\t\tDev capb. num : %d\n",
                                                      clist->capab.capab_len);
            for (k = 0; k < clist->capab.capab_len; k++)
            {
                pval = &clist->capab.capab_val[k];
                fprintf(s->logFp,"\t\t\t\tName [Value] : %s [%s]",
                                                 pval->name, pval->value);
            }
        }
    }
    return;
}

void prt_xdr_ndmp_cfg_get_scsi_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_cfg_get_scsi_info_rsp *rsp = (ndmp_cfg_get_scsi_info_rsp *)msg;
    ndmp_dev_info *tinfo;
    ndmp_dev_capab *clist;
    ndmp_pval *pval;
    int32 i, j, k;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tNum. scsi dev : %d\n", rsp->scsi_info.scsi_info_len);

    for (i = 0; i < rsp->scsi_info.scsi_info_len; i++)
    {
        tinfo = &rsp->scsi_info.scsi_info_val[i];
        fprintf(s->logFp,"\t\tModel [%d] : %s\n", i, tinfo->model);
        fprintf(s->logFp,"\t\tNum. capb. : %d\n", tinfo->caplist.caplist_len);
        for (j = 0; j < tinfo->caplist.caplist_len; j++)
        {
            clist = &tinfo->caplist.caplist_val[j];
            fprintf(s->logFp,"\t\t\tDev name : %s\n", clist->devname);
            fprintf(s->logFp,"\t\t\tDev attr : %ld\n", clist->attr);
            fprintf(s->logFp,"\t\t\tDev capb. num : %d\n",
                                                      clist->capab.capab_len);
            for (k = 0; k < clist->capab.capab_len; k++)
            {
                pval = &clist->capab.capab_val[k];
                fprintf(s->logFp,"\t\t\t\tName [Value] : %s [%s]",
                                                 pval->name, pval->value);
            }
        }
    }

    return;
}

void prt_xdr_ndmp_notify_data_halted_req(char *msg, ndmpSess_t *s)
{
    ndmp_notify_data_halted_req *req = (ndmp_notify_data_halted_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tReason : %d (%s)\n", req->reason,
					decode_data_halt_reas(req->reason));
    if (req->text_reason)
        fprintf(s->logFp,"\tText : %s\n", req->text_reason);

    return;
}

void prt_xdr_ndmp_notify_conn_req(char *msg, ndmpSess_t *s)
{
    ndmp_notify_conn_req *req = (ndmp_notify_conn_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tReason : %d (%s)\n", req->reason,
                           (req->reason == NDMP_CONNECTED)?"CONNECTED":
                           (req->reason == NDMP_SHUTDOWN)?"SHUTDOWN":
                           (req->reason == NDMP_REFUSED)?"REFUSED":"UNKNOWN");

    fprintf(s->logFp,"\tProto Version : %d\n", req->protoVer);
    if (req->text_reason)
        fprintf(s->logFp,"\tText : %s\n", req->text_reason);

    return;
}

void prt_xdr_ndmp_notify_mover_halted_req(char *msg, ndmpSess_t *s)
{
    ndmp_notify_mover_halted_req *req = (ndmp_notify_mover_halted_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tReason : %d (%s)\n", req->reason, decode_mover_halt_reas(req->reason));
    if (req->text_reason)
        fprintf(s->logFp,"\tText : %s\n", req->text_reason);

    return;
}

void prt_xdr_ndmp_notify_mover_paused_req(char *msg, ndmpSess_t *s)
{
    ndmp_notify_mover_paused_req *req = (ndmp_notify_mover_paused_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tReason : %d (%s)\n", req->reason,
					decode_mover_pause_reas(req->reason));
    fprintf(s->logFp,"\tSeek pos. (H L) : %ld %ld\n",
                             req->seek_position.high, req->seek_position.low);

    return;
}

void prt_xdr_ndmp_notify_data_read_req(char *msg, ndmpSess_t *s)
{
    ndmp_notify_data_read_req *req = (ndmp_notify_data_read_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tOffset (H L) : %ld %ld\n",
                             req->offset.high, req->offset.low);
    fprintf(s->logFp,"\tLength (H L) : %ld %ld\n",
                             req->length.high, req->length.low);

    return;
}

void prt_xdr_ndmp_log_file_req(char *msg, ndmpSess_t *s)
{
    ndmp_log_file_req *req = (ndmp_log_file_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tName : %s\n", req->name);
    fprintf(s->logFp,"\tErr : %d\n", req->error);

    return;
}

void prt_xdr_ndmp_log_msg_req(char *msg, ndmpSess_t *s)
{
    ndmp_log_msg_req *req = (ndmp_log_msg_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tType : %d (%s)\n", req->log_type,
                     (req->log_type == NDMP_LOG_NORMAL)?"NORMAL":
                     (req->log_type == NDMP_LOG_DEBUG)?"DEBUG":
                     (req->log_type == NDMP_LOG_ERROR)?"ERROR":
                     (req->log_type == NDMP_LOG_WARNING)?"WARNING":"UNKNOWN");
    fprintf(s->logFp,"\tMsg ID : %ld\n", req->message_id);
    fprintf(s->logFp,"\tLog : %s\n", req->entry);

    return;
}

void prt_xdr_ndmp_fh_add_file_req(char *msg, ndmpSess_t *s)
{
    ndmp_fh_add_file_req *req = (ndmp_fh_add_file_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. files : %d\n", req->files.files_len);

    /* Print all file details if required */

    return;
}

void prt_xdr_ndmp_fh_add_dir_req(char *msg, ndmpSess_t *s)
{
    ndmp_fh_add_dir_req *req = (ndmp_fh_add_dir_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. Dirs : %d\n", req->dirs.dirs_len);

    /* Print all dir details if required */

    return;
}
void prt_xdr_ndmp_fh_add_node_req(char *msg, ndmpSess_t *s)
{
    ndmp_fh_add_node_req *req = (ndmp_fh_add_node_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. Nodes : %d\n", req->nodes.nodes_len);

    /* Print all node details if required */

    return;
}

void prt_xdr_ndmp_scsi_open_req(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_open_req *req = (ndmp_scsi_open_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tDevice : %s\n", req->device);

    return;
}

void prt_xdr_ndmp_scsi_open_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_open_rsp *rsp = (ndmp_scsi_open_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_scsi_close_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_close_rsp *rsp = (ndmp_scsi_close_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_scsi_exec_cdb_req(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_exec_cdb_req *req = (ndmp_scsi_exec_cdb_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tFlags : %ld\n", req->flags);
    fprintf(s->logFp,"\tTimeout : %ld\n", req->timeout);
    fprintf(s->logFp,"\tDatain Len : %ld\n", req->datain_len);
    fprintf(s->logFp,"\tCDB [len:%d] :\n", req->cdb.cdb_len);
    print_hex_dump(req->cdb.cdb_val, req->cdb.cdb_len, "\t", s->logFp);
    fprintf(s->logFp,"\tDataout [len:%d] :\n",req->dataout.dataout_len);
    print_hex_dump(req->dataout.dataout_val, req->dataout.dataout_len,
                   "\t", s->logFp);

    return;
}

void prt_xdr_ndmp_scsi_exec_cdb_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_exec_cdb_rsp *rsp = (ndmp_scsi_exec_cdb_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tStatus : %c\n", rsp->status);
    fprintf(s->logFp,"\tDataout Len : %ld\n", rsp->dataout_len);
    fprintf(s->logFp,"\tDatain [len:%d] :\n", rsp->datain.datain_len);
    print_hex_dump(rsp->datain.datain_val, rsp->datain.datain_len,
                   "\t", s->logFp);
    fprintf(s->logFp,"\tExtent [len:%d] :\n", rsp->ext_sense.ext_sense_len);
    print_hex_dump(rsp->ext_sense.ext_sense_val, rsp->ext_sense.ext_sense_len,
                   "\t", s->logFp);

    return;
}

void prt_xdr_ndmp_scsi_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_get_state_rsp *rsp = (ndmp_scsi_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tTarget Ctrl : %d\n", rsp->targ_crtller);
    fprintf(s->logFp,"\tTarget ID : %d\n", rsp->targ_id);
    fprintf(s->logFp,"\tTarget LUN : %d\n", rsp->targ_lun);

    return;
}

void prt_xdr_ndmp_scsi_reset_dev_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_reset_dev_rsp *rsp = (ndmp_scsi_reset_dev_rsp  *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_scsi_reset_bus_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_reset_bus_rsp *rsp = (ndmp_scsi_reset_bus_rsp  *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_scsi_set_target_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_scsi_set_target_rsp *rsp = (ndmp_scsi_set_target_rsp  *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_tape_open_req(char *msg, ndmpSess_t *s)
{
    ndmp_tape_open_req *req = (ndmp_tape_open_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tDevice : %s\n", req->devname);
    fprintf(s->logFp,"\tMode : %d (%s)\n", req->mode,
                              (req->mode == NDMP_TAPE_READ_MODE)?"Read Only":
                              (req->mode == NDMP_TAPE_RDWR_MODE)?"Read Write":
                                                                 "Invalid");

    return;
}

void prt_xdr_ndmp_tape_open_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_open_rsp *rsp = (ndmp_tape_open_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_tape_close_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_close_rsp *rsp = (ndmp_tape_close_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_tape_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_get_state_rsp *rsp = (ndmp_tape_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tInvalid : %ld\n", rsp->invalid);
    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tFlags : %ld\n", rsp->flags);
    fprintf(s->logFp,"\tFile Num : %ld\n", rsp->file_num);
    fprintf(s->logFp,"\tSoft Err : %ld\n", rsp->soft_errors);
    fprintf(s->logFp,"\tBlock Size : %ld\n", rsp->block_size);
    fprintf(s->logFp,"\tBlock # : %ld\n", rsp->blockno);
    fprintf(s->logFp,"\tTotal Space (H L): %ld %ld\n",
                               rsp->total_space.high, rsp->total_space.low);
    fprintf(s->logFp,"\tRemain Space (H L): %ld %ld\n",
                               rsp->space_remain.high, rsp->space_remain.low);
    fprintf(s->logFp,"\tPartition : %ld\n", rsp->partition);

    return;
}

void prt_xdr_ndmp_tape_mtio_req(char *msg, ndmpSess_t *s)
{
    ndmp_tape_mtio_req *req = (ndmp_tape_mtio_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tTape-IO : %d (%s)\n", req->tape_op,
                             (req->tape_op == NDMP_MTIO_FSF)?"FSF":
                             (req->tape_op == NDMP_MTIO_BSF)?"BSF":
                             (req->tape_op == NDMP_MTIO_FSR)?"FSR":
                             (req->tape_op == NDMP_MTIO_BSR)?"BSR":
                             (req->tape_op == NDMP_MTIO_REW)?"REW":
                             (req->tape_op == NDMP_MTIO_EOF)?"EOF":
                             (req->tape_op == NDMP_MTIO_OFF)?"OFF":"UNKNOWN");
    fprintf(s->logFp,"\tCount : %ld\n", req->count);

    return;
}

void prt_xdr_ndmp_tape_mtio_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_mtio_rsp *rsp = (ndmp_tape_mtio_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tresid cnt : %ld\n", rsp->resid_count);

    return;
}

void prt_xdr_ndmp_tape_write_req(char *msg, ndmpSess_t *s)
{
    ndmp_tape_write_req *req = (ndmp_tape_write_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tDataout len %d\n", req->data_out.data_out_len);
    fprintf(s->logFp,"\tDataout %dK <DATA>\n", req->data_out.data_out_len/1024);

    return;
}

void prt_xdr_ndmp_tape_write_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_write_rsp *rsp = (ndmp_tape_write_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tCount : %ld\n", rsp->count);

    return;
}

void prt_xdr_ndmp_tape_read_req(char *msg, ndmpSess_t *s)
{
    ndmp_tape_read_req *req = (ndmp_tape_read_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tCount : %ld\n", req->count);

    return;
}

void prt_xdr_ndmp_tape_read_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_read_rsp *rsp = (ndmp_tape_read_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tDatain len %d\n", rsp->data_in.data_in_len);
    fprintf(s->logFp,"\tDatain %dK <DATA>\n", rsp->data_in.data_in_len/1024);

    return;
}

void prt_xdr_ndmp_tape_exec_cdb_req(char *msg, ndmpSess_t *s)
{
    ndmp_tape_exec_cdb_req *req = (ndmp_tape_exec_cdb_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tFlags : %ld\n", req->flags);
    fprintf(s->logFp,"\tTimeout : %ld\n", req->timeout);
    fprintf(s->logFp,"\tDatain Len : %ld\n", req->datain_len);
    fprintf(s->logFp,"\tCDB [len:%d] : <DATA>\n", req->cdb.cdb_len);
    print_hex_dump(req->cdb.cdb_val, req->cdb.cdb_len, "\t", s->logFp);
    fprintf(s->logFp,"\tDataout [len:%d] : <DATA>\n", req->dataout.dataout_len);
    print_hex_dump(req->dataout.dataout_val, req->dataout.dataout_len,
                   "\t", s->logFp);

    return;
}

void prt_xdr_ndmp_tape_exec_cdb_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_tape_exec_cdb_rsp *rsp = (ndmp_tape_exec_cdb_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tStatus : %c\n", rsp->status);
    fprintf(s->logFp,"\tDataout Len : %ld\n", rsp->dataout_len);
    fprintf(s->logFp,"\tDatain [len:%d] : <DATA>\n", rsp->datain.datain_len);
    print_hex_dump(rsp->datain.datain_val, rsp->datain.datain_len,
                   "\t", s->logFp);
    fprintf(s->logFp,"\tExtent [len:%d] : <DATA>\n",
                                                rsp->ext_sense.ext_sense_len);
    print_hex_dump(rsp->ext_sense.ext_sense_val, rsp->ext_sense.ext_sense_len,
                   "\t", s->logFp);

    return;
}

void prt_xdr_ndmp_data_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_get_state_rsp *rsp = (ndmp_data_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tInvalid : %ld\n", rsp->invalid);
    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tOPS : %d (%s)\n", rsp->op,
                        (rsp->op == NDMP_DATA_OP_NOACTION)?"NO-ACTION":
                        (rsp->op == NDMP_DATA_OP_BACKUP)?"BACKUP":
                        (rsp->op == NDMP_DATA_OP_RESTORE)?"RESTORE":"INVALID");
    fprintf(s->logFp,"\tState : %d (%s)\n", rsp->state, 
                 (rsp->state == NDMP_DATA_ST_IDLE)?"IDLE":
                 (rsp->state == NDMP_DATA_ST_ACTIVE)?"ACTIVE":
                 (rsp->state == NDMP_DATA_ST_HALTED)?"HALTED":
                 (rsp->state == NDMP_DATA_ST_LISTEN)?"LISTEN":
                 (rsp->state == NDMP_DATA_ST_CONNECTED)?"CONNECTED":"UNKNOWN");
    fprintf(s->logFp,"\tHalt Reason : %d (%s)\n", rsp->halt_reason,
					decode_data_halt_reas(rsp->halt_reason));
    fprintf(s->logFp,"\tBytes Processed (H L) : %ld %ld\n",
                         rsp->bytes_processed.high, rsp->bytes_processed.low);
    fprintf(s->logFp,"\tBytes remina (H L) : %ld %ld\n",
                       rsp->est_bytes_remain.high, rsp->est_bytes_remain.low);
    fprintf(s->logFp,"\tTime Remain : %ld\n", rsp->est_time_remain);
    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n",
             rsp->data_conn_addr.addr_type,
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port);
    fprintf(s->logFp,"\tRead offset (H L) : %ld %ld\n",
                                 rsp->read_offset.high, rsp->read_offset.low);
    fprintf(s->logFp,"\tRead length (H L) : %ld %ld\n",
                                 rsp->read_length.high, rsp->read_length.low);

    return;
}

void prt_xdr_ndmp_data_start_bkup_req(char *msg, ndmpSess_t *s)
{
    ndmp_data_start_bkup_req *req = (ndmp_data_start_bkup_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tBkup type : %s\n", req->bu_type);
    fprintf(s->logFp,"\tNum Env. Var : %d\n", req->env.env_len);
    for (i = 0; i < req->env.env_len; i++)
        fprintf(s->logFp,"\t\tName (value) : %s (%s)\n",
                        req->env.env_val[i].name, req->env.env_val[i].value);

    return;
}

void prt_xdr_ndmp_data_start_bkup_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_start_bkup_rsp *rsp = (ndmp_data_start_bkup_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_data_start_recover_req(char *msg, ndmpSess_t *s)
{
    ndmp_data_start_recover_req *req = (ndmp_data_start_recover_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum Env. Var : %d\n", req->env.env_len);
    for (i = 0; i < req->env.env_len; i++)
        fprintf(s->logFp,"\t\tName (value) : %s (%s)\n",
                        req->env.env_val[i].name, req->env.env_val[i].value);

    fprintf(s->logFp,"\tNum. of names : %d\n", req->nlist.nlist_len);
    for (i = 0; i < req->nlist.nlist_len; i++)
    {
        fprintf(s->logFp,"\t\tOrig. path : %s \n",
                                            req->nlist.nlist_val[i].orig_path);
        fprintf(s->logFp,"\t\tDest. dir  : %s \n",
                                            req->nlist.nlist_val[i].dest_dir);
        fprintf(s->logFp,"\t\tNew name : %s \n",
                                            req->nlist.nlist_val[i].new_name);
        fprintf(s->logFp,"\t\tOther name : %s \n",
                                           req->nlist.nlist_val[i].other_name);
        fprintf(s->logFp,"\t\tNode (H L) : %ld %ld\n",
                                            req->nlist.nlist_val[i].node.high,
                                            req->nlist.nlist_val[i].node.low);
        fprintf(s->logFp,"\t\tFile History info (H L) : %ld %ld\n",
                                          req->nlist.nlist_val[i].fh_info.high,
                                          req->nlist.nlist_val[i].fh_info.low);
    }
    fprintf(s->logFp,"\tBkup type : %s\n", req->bu_type);

    return;
}

void prt_xdr_ndmp_data_start_recover_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_start_recover_rsp *rsp = (ndmp_data_start_recover_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_data_abort_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_abort_rsp *rsp = (ndmp_data_abort_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_data_get_env_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_get_env_rsp *rsp = (ndmp_data_get_env_rsp *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tNum Env. Var : %d\n", rsp->env.env_len);
    for (i = 0; i < rsp->env.env_len; i++)
        fprintf(s->logFp,"\t\tName (value) : %s (%s)\n",
                         rsp->env.env_val[i].name, rsp->env.env_val[i].value);

    return;
}

void prt_xdr_ndmp_data_stop_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_stop_rsp *rsp = (ndmp_data_stop_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_data_listen_req(char *msg, ndmpSess_t *s)
{
    ndmp_data_listen_req *req = (ndmp_data_listen_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAddr. type : %d (%s)\n", req->addr_type, 
                            (req->addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                            (req->addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");

    return;
}

void prt_xdr_ndmp_data_listen_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_listen_rsp *rsp = (ndmp_data_listen_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n",
             rsp->data_conn_addr.addr_type,
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port);

    return;
}

void prt_xdr_ndmp_data_conn_req(char *msg, ndmpSess_t *s)
{
    ndmp_data_conn_req *req = (ndmp_data_conn_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n", req->addr.addr_type,
                      (req->addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                      (req->addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                                     req->addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                                     req->addr.ndmp_addr_u.tcp_addr.port);

    return;
}

void prt_xdr_ndmp_data_conn_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_data_conn_rsp *rsp = (ndmp_data_conn_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_get_state_rsp *rsp = (ndmp_mover_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tState : %d (%s)\n", rsp->state,
                              (rsp->state == NDMP_MOVER_ST_IDLE)?"IDLE":
                              (rsp->state == NDMP_MOVER_ST_LISTEN)?"LISTEN":
                              (rsp->state == NDMP_MOVER_ST_ACTIVE)?"ACTIVE":
                              (rsp->state == NDMP_MOVER_ST_PAUSED)?"PAUSED":
                              (rsp->state == NDMP_MOVER_ST_HALTED)?"HALTED":
                                                                   "UNKNOWN");
    fprintf(s->logFp,"\tPause reason : %d (%s)\n", rsp->pause_reason,
					decode_mover_pause_reas(rsp->pause_reason));
    fprintf(s->logFp,"\tHalt reason : %d (%s)\n", rsp->halt_reason, 
					decode_mover_halt_reas(rsp->halt_reason));

#ifdef PRINT_COMPLETE_MESSAGES
    fprintf(s->logFp,"\tRecord Size : %ld\n", rsp->recsiz);
#endif
    fprintf(s->logFp,"\tRecord Num. : %ld\n", rsp->recnum);
    fprintf(s->logFp,"\tData written (H L) : %ld %ld\n",
                              rsp->data_written.high, rsp->data_written.low);
#ifdef PRINT_COMPLETE_MESSAGES
    fprintf(s->logFp,"\tSeek pos. (H L) : %ld %ld\n",
                                      rsp->seek_pos.high, rsp->seek_pos.low);
    fprintf(s->logFp,"\tBytes left to read (H L) : %ld %ld\n",
                  rsp->bytes_left_to_read.high, rsp->bytes_left_to_read.low);
    fprintf(s->logFp,"\tWindow offset (H L) : %ld %ld\n",
                            rsp->window_offset.high, rsp->window_offset.low);
    fprintf(s->logFp,"\tWindow len (H L) : %ld %ld\n",
                                  rsp->window_len.high, rsp->window_len.low);
    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n",
             rsp->data_conn_addr.addr_type,
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port);
#endif

    return;
}

void prt_xdr_ndmp_mover_listen_req(char *msg, ndmpSess_t *s)
{
    ndmp_mover_listen_req *req = (ndmp_mover_listen_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tMode : %d (%s)\n", req->mode,
                      (req->mode == NDMP_MOVER_MODE_READ)?"READ":
                      (req->mode == NDMP_MOVER_MODE_WRITE)?"WRITE":"UNKNOWN");
    fprintf(s->logFp,"\tAddr type : %d (%s)\n", req->addr_type,
                           (req->addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                           (req->addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");

    return;
}

void prt_xdr_ndmp_mover_listen_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_listen_rsp *rsp = (ndmp_mover_listen_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n",
             rsp->data_conn_addr.addr_type,
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port);

    return;
}
                                 
void prt_xdr_ndmp_mover_cont_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_cont_rsp *rsp = (ndmp_mover_cont_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_abort_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_abort_rsp *rsp = (ndmp_mover_abort_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_stop_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_stop_rsp *rsp = (ndmp_mover_stop_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_set_window_req(char *msg, ndmpSess_t *s)
{
    ndmp_mover_set_window_req *req = (ndmp_mover_set_window_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tOffset (H L) : %ld %ld\n",
                                        req->offset.high, req->offset.low);
    fprintf(s->logFp,"\tLength (H L) : %ld %ld\n",
                                        req->length.high, req->length.low);

    return;
}

void prt_xdr_ndmp_mover_set_window_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_set_window_rsp *rsp = (ndmp_mover_set_window_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_read_req(char *msg, ndmpSess_t *s)
{
    ndmp_mover_read_req *req = (ndmp_mover_read_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tOffset (H L) : %ld %ld\n",
                                        req->offset.high, req->offset.low);
    fprintf(s->logFp,"\tLength (H L) : %ld %ld\n",
                                        req->length.high, req->length.low);

    return;
}

void prt_xdr_ndmp_mover_read_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_read_rsp *rsp = (ndmp_mover_read_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_close_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_close_rsp *rsp = (ndmp_mover_close_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_set_record_size_req(char *msg, ndmpSess_t *s)
{
    ndmp_mover_set_record_size_req *req = (ndmp_mover_set_record_size_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tLength : %ld\n", req->len);

    return;
}

void prt_xdr_ndmp_mover_set_record_size_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_set_record_size_rsp *rsp = (ndmp_mover_set_record_size_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmp_mover_conn_req(char *msg, ndmpSess_t *s)
{
    ndmp_mover_conn_req *req = (ndmp_mover_conn_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tMode : %d (%s)\n", req->mode,
                            (req->mode == NDMP_MOVER_MODE_READ)?"READ":
                            (req->mode == NDMP_MOVER_MODE_WRITE)?"WRITE":
                                                                 "UNKNOWN");
    fprintf(s->logFp,"\tAddr :\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n", req->addr.addr_type,
                       (req->addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                       (req->addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                                  req->addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                                  req->addr.ndmp_addr_u.tcp_addr.port);

    return;
}

void prt_xdr_ndmp_mover_conn_rsp(char *msg, ndmpSess_t *s)
{
    ndmp_mover_conn_rsp *rsp = (ndmp_mover_conn_rsp *)msg;
    
    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);

    return;
}

void prt_xdr_ndmpv2_cfg_get_host_info_rsp(char *msg, ndmpSess_t *s)
{
    ndmpv2_cfg_get_host_info_rsp *rsp = (ndmpv2_cfg_get_host_info_rsp *)msg;
    int32 auth_type;

    if (!s->logFp)
        return;

    auth_type = *rsp->authTyp.authTyp_val;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tHost Name : %s\n", rsp->hostname);
    fprintf(s->logFp,"\tOS Type : %s\n", rsp->os_type);
    fprintf(s->logFp,"\tOS Vers : %s\n", rsp->os_vers);
    fprintf(s->logFp,"\tHost ID : %s\n", rsp->hostid);
    fprintf(s->logFp,"\tAuth type len : %d\n", rsp->authTyp.authTyp_len);
    fprintf(s->logFp,"\tAuth type val : %d (%s)\n", auth_type,
                                 (auth_type == NDMP_AUTHNONE)?"None":
                                 (auth_type == NDMP_AUTHTXT)?"Text":
                                 (auth_type == NDMP_AUTHMD5)?"MD5":"Invalid");

    return;
}

void prt_xdr_ndmpv2_cfg_get_butype_attr_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_cfg_get_butype_attr_req *req = (ndmpv2_cfg_get_butype_attr_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tBkup type : %s\n", req->butype_name);

    return;
}

void prt_xdr_ndmpv2_cfg_get_butype_attr_rsp(char *msg, ndmpSess_t *s)
{
    ndmpv2_cfg_get_butype_attr_rsp *rsp = (ndmpv2_cfg_get_butype_attr_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tAttr  : %ld\n", rsp->attrs);

    return;
}

void prt_xdr_ndmpv2_log_log_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_log_log_req *req = (ndmpv2_log_log_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tEntry : %s\n", req->entry);

    return;
}

void prt_xdr_ndmpv2_log_dbg_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_log_dbg_req *req = (ndmpv2_log_dbg_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tLevel : %d (%s)\n", req->level,
                      (req->level == NDMP_DBG_USER_INFO)?"USER INFO":
                      (req->level == NDMP_DBG_USER_SUMMARY)?"USER SUMMARY":
                      (req->level == NDMP_DBG_USER_DETAIL)?"USER DETAIL":
                      (req->level == NDMP_DBG_DIAG_INFO)?"DIAG INFO":
                      (req->level == NDMP_DBG_DIAG_SUMMARY)?"DIAG SUMMARY":
                      (req->level == NDMP_DBG_DIAG_DETAIL)?"DIAG DETAIL":
                      (req->level == NDMP_DBG_PROG_INFO)?"PROG INFO":
                      (req->level == NDMP_DBG_PROG_SUMMARY)?"PROG SUMMARY":
                      (req->level == NDMP_DBG_PROG_DETAIL)?"PROG DETAIL":
                                                           "UNKNOWN");
    fprintf(s->logFp,"\tMessage : %s\n", req->msg);

    return;
}

void prt_xdr_ndmpv2_log_file_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_log_file_req *req = (ndmpv2_log_file_req *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tName : %s\n", req->name);
    fprintf(s->logFp,"\tReserved : %d\n", req->reserved);
    fprintf(s->logFp,"\tErr : %d\n", req->error);

    return;
}

void prt_xdr_ndmpv2_fh_add_file_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_fh_add_file_req *req = (ndmpv2_fh_add_file_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. files : %d <file stats not printed>\n",
                                                      req->files.files_len);
    for (i = 0; i < req->files.files_len; i++)
        fprintf(s->logFp,"\t\tName : %s\n", req->files.files_val[i].name);

    return;
}

void prt_xdr_ndmpv2_fh_add_dir_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_fh_add_dir_req *req = (ndmpv2_fh_add_dir_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. dir : %d\n", req->dirs.dirs_len);
    for (i = 0; i < req->dirs.dirs_len; i++)
        fprintf(s->logFp,"\t\tName (node : %ld)(parent : %ld) : %s\n",
                           req->dirs.dirs_val[i].node,
                           req->dirs.dirs_val[i].parent,
                           req->dirs.dirs_val[i].name);
    return;
}

void prt_xdr_ndmpv2_fh_add_node_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_fh_add_node_req *req = (ndmpv2_fh_add_node_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tNum. nodes : %d <file stats not printed>\n",
                                                      req->nodes.nodes_len);
    for (i = 0; i < req->nodes.nodes_len; i++)
        fprintf(s->logFp,"\t\tnode : %ld\n", req->nodes.nodes_val[i].node);

    return;
}

void prt_xdr_ndmpv2_tape_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmpv2_tape_get_state_rsp *rsp = (ndmpv2_tape_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tFlags : %ld\n", rsp->flags);
    fprintf(s->logFp,"\tFile num : %ld\n", rsp->file_num);
    fprintf(s->logFp,"\tSoft err : %ld\n", rsp->soft_errors);
    fprintf(s->logFp,"\tBlock Size : %ld\n", rsp->block_size);
    fprintf(s->logFp,"\tBlock no : %ld\n", rsp->blockno);
    fprintf(s->logFp,"\tTotal space (H L): %ld %ld\n",
                             rsp->total_space.high, rsp->total_space.low);
    fprintf(s->logFp,"\tSpace remain (H L): %ld %ld\n",
                             rsp->space_remain.high, rsp->space_remain.low);

    return;
}

void prt_xdr_ndmpv2_data_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmpv2_data_get_state_rsp *rsp = (ndmpv2_data_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tOperation : %d (%s)\n", rsp->op,
                      (rsp->op == NDMP_DATA_OP_NOACTION)?"NO ACTION":
                      (rsp->op == NDMP_DATA_OP_BACKUP)?"BACKUP":
                      (rsp->op == NDMP_DATA_OP_RESTORE)?"RESTORE":"UNKNOWN");
    fprintf(s->logFp,"\tState : %d (%s)\n", rsp->state,
                 (rsp->state == NDMP_DATA_ST_IDLE)?"IDLE":
                 (rsp->state == NDMP_DATA_ST_ACTIVE)?"ACTIVE":
                 (rsp->state == NDMP_DATA_ST_HALTED)?"HALTED":
                 (rsp->state == NDMP_DATA_ST_LISTEN)?"LISTEN":
                 (rsp->state == NDMP_DATA_ST_CONNECTED)?"CONNECTED":"UNKNOWN");
    fprintf(s->logFp,"\tHalt reason : %d (%s)\n", rsp->halt_reason,
					decode_data_halt_reas(rsp->halt_reason));
    fprintf(s->logFp,"\tBytes proc. (H L) : %ld %ld\n",
                          rsp->bytes_processed.high, rsp->bytes_processed.low);
    fprintf(s->logFp,"\tEst. bytes remain (H L) : %ld %ld\n",
                     rsp->est_bytes_remain.high, rsp->est_bytes_remain.low);
    fprintf(s->logFp,"\tEst. time remain : %ld\n", rsp->est_time_remain);
    fprintf(s->logFp,"\tData Conn:\n");
    fprintf(s->logFp,"\t\tType : %d (%s)\n",
             rsp->data_conn_addr.addr_type,
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
             (rsp->data_conn_addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\t\tIP Addr : %ld\n",
                            rsp->data_conn_addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\t\tPort : %d\n",
                               rsp->data_conn_addr.ndmp_addr_u.tcp_addr.port);
    fprintf(s->logFp,"\tRead offset (H L) : %ld %ld\n",
                                 rsp->read_offset.high, rsp->read_offset.low);
    fprintf(s->logFp,"\tRead length (H L) : %ld %ld\n",
                                 rsp->read_length.high, rsp->read_length.low);

    return;
}

void prt_xdr_ndmpv2_data_start_bkup_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_data_start_bkup_req *req = (ndmpv2_data_start_bkup_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAddr type : %d (%s)\n", req->addr.addr_type,
                       (req->addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                       (req->addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\tIP Addr : %ld\n",
                                      req->addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\tPort : %d\n", req->addr.ndmp_addr_u.tcp_addr.port);
    fprintf(s->logFp,"\tBkup. type : %s\n", req->bu_type);
    fprintf(s->logFp,"\tNum. env : %d\n", req->env.env_len);
    for (i = 0; i < req->env.env_len; i++)
        fprintf(s->logFp,"\t\tName [Value] : %s [%s]\n",
                        req->env.env_val[i].name, req->env.env_val[i].value);

    return;
}

void prt_xdr_ndmpv2_data_start_recover_req(char *msg, ndmpSess_t *s)
{
    ndmpv2_data_start_recover_req *req = (ndmpv2_data_start_recover_req *)msg;
    int32 i;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tAddr type : %d (%s)\n", req->addr.addr_type,
                       (req->addr.addr_type == NDMP_ADDR_LOCAL)?"LOCAL":
                       (req->addr.addr_type == NDMP_ADDR_TCP)?"TCP":"UNKNOWN");
    fprintf(s->logFp,"\tIP Addr : %ld\n",
                                      req->addr.ndmp_addr_u.tcp_addr.ip_addr);
    fprintf(s->logFp,"\tPort : %d\n", req->addr.ndmp_addr_u.tcp_addr.port);
    fprintf(s->logFp,"\tNum Env. Var : %d\n", req->env.env_len);
    for (i = 0; i < req->env.env_len; i++)
        fprintf(s->logFp,"\t\tName (value) : %s (%s)\n",
                        req->env.env_val[i].name, req->env.env_val[i].value);

    fprintf(s->logFp,"\tNum. of names : %d\n", req->nlist.nlist_len);
    for (i = 0; i < req->nlist.nlist_len; i++)
    {
        fprintf(s->logFp,"\t\tOrig. path : %s \n",
                                            req->nlist.nlist_val[i].orig_path);
        fprintf(s->logFp,"\t\tDest. dir  : %s \n",
                                            req->nlist.nlist_val[i].dest_dir);
        fprintf(s->logFp,"\t\tReserved : %d\n", req->nlist.nlist_val[i].rsvd);
        fprintf(s->logFp,"\t\tFile History info (H L) : %ld %ld\n",
                                          req->nlist.nlist_val[i].fh_info.high,
                                          req->nlist.nlist_val[i].fh_info.low);
    }
    fprintf(s->logFp,"\tBkup type : %s\n", req->bu_type);

    return;
}

void prt_xdr_ndmpv2_mover_get_state_rsp(char *msg, ndmpSess_t *s)
{
    ndmpv2_mover_get_state_rsp *rsp = (ndmpv2_mover_get_state_rsp *)msg;

    if (!s->logFp)
        return;

    fprintf(s->logFp,"\tErr : %d\n", rsp->error);
    fprintf(s->logFp,"\tState : %d (%s)\n", rsp->state,
                              (rsp->state == NDMP_MOVER_ST_IDLE)?"IDLE":
                              (rsp->state == NDMP_MOVER_ST_LISTEN)?"LISTEN":
                              (rsp->state == NDMP_MOVER_ST_ACTIVE)?"ACTIVE":
                              (rsp->state == NDMP_MOVER_ST_PAUSED)?"PAUSED":
                              (rsp->state == NDMP_MOVER_ST_HALTED)?"HALTED":
                                                                   "UNKNOWN");
    fprintf(s->logFp,"\tPause reason : %d (%s)\n", rsp->pause_reason,
					decode_mover_pause_reas(rsp->pause_reason));
    fprintf(s->logFp,"\tHalt reason : %d (%s)\n", rsp->halt_reason, 
					decode_mover_halt_reas(rsp->halt_reason));

#ifdef PRINT_COMPLETE_MESSAGES
    fprintf(s->logFp,"\tRecord Size : %ld\n", rsp->recsiz);
#endif
    fprintf(s->logFp,"\tRecord Num. : %ld\n", rsp->recnum);
    fprintf(s->logFp,"\tData written (H L) : %ld %ld\n",
                              rsp->data_written.high, rsp->data_written.low);
#ifdef PRINT_COMPLETE_MESSAGES
    fprintf(s->logFp,"\tSeek pos. (H L) : %ld %ld\n",
                                      rsp->seek_pos.high, rsp->seek_pos.low);
    fprintf(s->logFp,"\tBytes left to read (H L) : %ld %ld\n",
                  rsp->bytes_left_to_read.high, rsp->bytes_left_to_read.low);
    fprintf(s->logFp,"\tWindow offset (H L) : %ld %ld\n",
                            rsp->window_offset.high, rsp->window_offset.low);
    fprintf(s->logFp,"\tWindow len (H L) : %ld %ld\n",
                                  rsp->window_len.high, rsp->window_len.low);
#endif

    return;
}
