/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp.h
 *
 * RCS:  $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Network Data Management Protocol (NDMP) definitions
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
#if !defined(NDMP_H)
#define NDMP_H

#ifdef MIN
#undef MIN
#endif
#ifdef MAX
#undef MAX
#endif

#include "ndmp_xdr.h"
#include <exports/esm-api.h>
#include <exports/evm-api.h>
#include <exports/elog-api.h>
#include <exports/fs-api.h>
#include <exports/event-api.h>
#include <exports/timer-api.h>
#include <exports/queue-api.h>
#include <exports/thread-api.h>

//#define NO_SCSIIO 1	/* If we want i/o between FP & FC to be 8k, then define */
//#define NDMP_V2


#ifdef NFP_FP
extern int ndmp_fp_malloc;
extern int ndmp_fp_free;

#define NDMP_FP_ALLOC(_ptr, _type, _size, _str)        \
{                                                      \
    _ptr = (_type *)eee_ramAllocZero(_size);           \
    if (_ptr != NULL)                                  \
        ndmp_fp_malloc++;                              \
    NDMP_DBG_PRINTF(D_MALLOC, ("\n\tMALLOC -> %s(line %d - %s): %p %d\n", __FILE__, __LINE__, _str, _ptr, ndmp_fp_malloc)); \
}

#define NDMP_FP_FREE(_ptr, _str)                       \
{                                                      \
    if (_ptr != NULL)                                  \
    {                                                  \
        ndmp_fp_free++;                                \
        NDMP_DBG_PRINTF(D_MALLOC, ("\n\tFREE -> %s(line %d - free %s): %p %d\n", __FILE__, __LINE__, _str, _ptr, ndmp_fp_free)); \
        eee_ramDealloc(_ptr);                          \
        _ptr = NULL;                                   \
    }                                                  \
}
#endif

#define NDMP_INT_ALIGN(_n)                             \
{                                                      \
    _n += 3;                                           \
    _n &= ~3;                                          \
}

#define BKUP_SIZ_ENV_STR "AGIL_BKUP_SIZ"
/*
 *------------------------------------
 * NDMP Internal Message Types
 *------------------------------------
 */
typedef enum {
    NDMP_MID_TAPE_SPACE_REQ = NDMP_MSG_BASE,	/* 9604 */
    NDMP_MID_TAPE_SPACE_RSP,
	NDMP_MID_CFG_GET_FS_INFO,
	NDMP_MID_NEW_SESSION,
	NDMP_MID_CLOSE_SESSION,
	NDMP_MID_REL_RSRC,
#ifdef  NFX_SMP
	NDMP_MID_NEW_SESSION_RSP,
	NDMP_MID_CLOSE_SESSION_RSP,
#endif
	NDMP_MID_DATA_LISTEN, 						/* 9612 */
	NDMP_MID_DATA_CONNECT,						
	NDMP_MID_DATA_START_BACKUP,
	NDMP_MID_DATA_START_RECOVER,
	NDMP_MID_DATA_GET_ENV,
	NDMP_MID_DATA_GET_STATE,
	NDMP_MID_DATA_STOP,
	NDMP_MID_DATA_ABORT,
	NDMP_MID_MOVER_LISTEN,
	NDMP_MID_MOVER_CONNECT,
	NDMP_MID_MOVER_CONTINUE,                    /* 9622 */
	NDMP_MID_MOVER_READ,
	NDMP_MID_MOVER_SET_RECORD_SIZE,
	NDMP_MID_MOVER_SET_WINDOW,
	NDMP_MID_MOVER_GET_STATE,
	NDMP_MID_MOVER_ABORT,
	NDMP_MID_MOVER_STOP,
	NDMP_MID_MOVER_CLOSE,
	NDMP_MID_LOG_MESSAGE,
	NDMP_MID_LOG_FILE,
	NDMP_MID_FH_ADD_FILE,                       /* 9632 */
	NDMP_MID_FH_ADD_DIR,
	NDMP_MID_FH_ADD_NODE,
	NDMP_MID_NOTIFY_DATA_READ,
	NDMP_MID_NOTIFY_DATA_HALTED,
	NDMP_MID_NOTIFY_MOVER_HALTED,
	NDMP_MID_NOTIFY_MOVER_PAUSED,
	NDMP_MID_UI_SET_DBG_LEVEL,
	NDMP_MID_UI_SET_TRACE_ON_OFF,
	NDMP_MID_UI_SET_TRACE_FILES,
	NDMP_MID_UI_DM_GET_STATE,                   /* 9642 */
    NDMP_MID_UPDATE_ENV_REQ,
	NDMP_MID_CHILD_REPORT,
	NDMP_MID_CHILD_SND_STATS,
	NDMP_MID_DATA_START_RECOVER_RSP,
	NDMP_MID_DATA_SEND_NAME_REQ,
	NDMP_MID_DATA_SEND_NAME_RSP,                /* 9648 */
	NDMP_MID_UI_SET_RESTORE_MODE
} ndmp_intern_msg_type_t;

#define NO_AUTH_REQ 0
#define AUTH_REQ    1

// have to be root to open this
#define dumpDateDir     NFX_DUMPDATE_DIR
#define ndmpLogDir      NFX_LOG_DIR

#define AGILE_VENDOR_NAME	"Agile Storage, Inc."
#define	AGILE_PRODUCT_NAME	"Eagle NFX"
#define AGILE_PROD_VERSION	"1.0"

#define ONE_K_SIZE  1024 
#define STDBUFSIZ  12 *(ONE_K_SIZE)
#define	NFX_MAX_CTRL_XMT_BYTE_LIMIT	1470
#define	MAX_VOL_NAME	256
#define NDMP_MAX_NUM_SESS	64

#define NDMP_CHALLENGE_LEN	64
#define NDMP_AGIL_MSG_MAX   2048        /* Max Agile messages */
#define NDMP_MAX_REQ_SIZ   	200         /* Max request size */
#define NDMP_MAX_MOVR_RECSIZ			8192
#define MAX_SESSID	0x7fffffff
#define NDMP_MAX_SESS		64			/* max number of NDMP sessions allowed */
#define NDMP_MOVER_MAX_WDWLEN		0xffffffffffffffff
#define NDMP_MAX_COUNT		0x7fffffff
#define NDMP_SVR_PORT		10000
#define NDMP_DATA_CONN_PORT		58000	/* data connection port we advertise */
#define NDMP_FH_MAX_NUM_FILES 100    /* num files to send in File History message 
									 * at a time.  Don't want this number to be
									 * too big since each fh msg from FS dump is
									 * > 1024 due to file path name. 
									 */
#define NDMP_MAX_SNAPSHOT_NAME_LEN  64

#define     NDMP_EFX_FS_TYPE_NAME   	"efs"
#define     NDMP_EFX_FS_TYPE_NAME_LEN   strlen(NDMP_EFX_FS_TYPE_NAME)+1
#define     NDMP_EFX_FS_TYPE_STATUS 	"ONLINE"
#define     NDMP_EFX_FS_LOG_DEV_LEN 	19
#define     NDMP_NUM_FS_ENV 			4
#define 	NDMP_CFG_BUTYP_NUM_ENV		10
//#define 	NDMP_EFX_BKUP_TYP_NAME 		"efs_d"
#define 	NDMP_EFX_BKUP_TYP_NAME 		"dump" //for SYNCSORT

/* NDMP state machine Id */
#define MOVER_SM	1
#define DATA_SM		2

/* NDMP server Id */
#define		NDMP_NO_ONE			0
#define		NDMP_CONNECT_SERVER	1
#define		NDMP_CONFIG_SERVER	2
#define		NDMP_TAPE_SERVER	3	
#define		NDMP_SCSI_SERVER	4
#define		NDMP_DATA_SERVER	5
#define		NDMP_MOVER_SERVER	6

/* NDMP Tape open mode */
#define TAPE_MODE_NONE			0
#define TAPE_MODE_RD_ONLY		1
#define TAPE_MODE_RD_WR			2

/* NDMP device type */
#define 	TAPE_DEV			1
#define 	SCSI_DEV			2


/* Bkup/Rest States while setting up operation w/ FS */
#define NDMP_OPS_STATE_NONE				0
#define NDMP_OPS_STATE_BKUP_SETUP_SENT	1
#define NDMP_OPS_STATE_BKUP_NAMES_SENT	2
#define NDMP_OPS_STATE_BKUP_START_SENT	3
#define NDMP_OPS_STATE_REST_SETUP_SENT	4
#define NDMP_OPS_STATE_REST_NAMES_SENT	5
#define NDMP_OPS_STATE_REST_START_SENT	6

/* NDMP data connection mode */
#define NDMP_CONN_MODE_LISTEN	1
#define NDMP_CONN_MODE_CONNECT	2


#define NDMP_STAT_IO_OK			1
#define NDMP_STAT_IO_ERR		2

/* Used to indicate whether ABORT can be carried
 * thru by client (when initiated by UI)
 * or by us.  IT depends which state DATA server
 * is in.
 */
#define NDMP_AUTO_SESS_KILL		1
#define NDMP_MANUAL_SESS_KILL	2


#define NOT_FLUSH_DATA_TO_TAPE      1

#define SSC_CARD	1
#define FP_CARD		2
#ifdef NFX_SMP
#define NO_LOCK		99
#define NDMP_THREAD_EVENT_GOT_MSG		1
#endif

/*------------- FP Stats ---------------*/
typedef enum {
	/* Convention:  The first few letters indicates who the msg was
	 * received from or who the msg is sent to.
	 * For ex:  CL_NEW_SESSION_REQ_RCVD means NEW_SESSION received from client side (SSC)
	 * 		    FS_DMP_SETUP_REQ_SNT means we sent DMP_SETUP msg to FS.
	 */

	/* Messages we can receive on NDMP FP side */

	SSC_NEW_SESSION_REQ_RCVD,				/* 0 */
	SSC_CLOSE_SESSION_REQ_RCVD,
	SSC_CLOSE_SESSION_RSP_SNT,
	SSC_UI_DM_GETSTATE_REQ_RCVD,	
	REL_RSRC_REQ_RCVD,
#ifndef NFX_SMP
	SSC_SEND_NAME_REQ,
#endif
#ifdef NO_SCSIIO
	SCSI_WR_RSP_RCVD,
	SCSI_RD_RSP_RCVD,
#endif
	FS_DMP_SETUP_RSP_RCVD,
	FS_DMP_NAMES_RSP_RCVD,		/*  */
	FS_DMP_START_RSP_RCVD,
	FS_DMP_ABORT_RSP_RCVD,		
	FS_RST_SETUP_RSP_RCVD,
	FS_RST_NAMES_RSP_RCVD,
	FS_RST_START_RSP_RCVD,		/*  */
	FS_RST_ABORT_RSP_RCVD,
	FS_SND_LOG_REQ_RCVD,		
	FS_SND_FH_DIR_REQ_RCVD,
	FS_SND_FH_NODE_REQ_RCVD,
	FS_SET_DMPDATE_REQ_RCVD,
	FS_RD_DATA_REQ_RCVD, 		/*  */
	FS_WR_DATA_REQ_RCVD,
	FS_OPS_CMPL_REQ_RCVD,		

	SSC_DATA_LISTEN_REQ_RCVD,
	SSC_DATA_CONN_REQ_RCVD,
	SSC_DATA_START_BKUP_REQ_RCVD,	/*  */
	SSC_DATA_START_RST_REQ_RCVD,
	SSC_DATA_RST_NAMES_REQ_RCVD,
	SSC_DATA_GETENV_REQ_RCVD,	
	SSC_DATA_GETSTATE_REQ_RCVD,
	SSC_DATA_STOP_REQ_RCVD,
	SSC_DATA_ABORT_REQ_RCVD,			/*  */

	SSC_MOVER_LISTEN_REQ_RCVD,
	SSC_MOVER_CONN_REQ_RCVD,		
	SSC_MOVER_CONT_REQ_RCVD,
	SSC_MOVER_RD_REQ_RCVD,
	SSC_MOVER_SET_RECSIZ_REQ_RCVD,	/*  */
	SSC_MOVER_SET_WDW_REQ_RCVD,
	SSC_MOVER_GET_STATE_REQ_RCVD,	
	SSC_MOVER_ABORT_REQ_RCVD,
	SSC_MOVER_STOP_REQ_RCVD,
	SSC_MOVER_CLOSE_REQ_RCVD,		/*  */

	/* Messages we sent from NDMP FP side */

	FS_DMP_SETUP_REQ_SNT,
	FS_DMP_NAMES_REQ_SNT,		/*  */
	FS_DMP_START_REQ_SNT,
	FS_DMP_ABORT_REQ_SNT,		
	FS_RST_SETUP_REQ_SNT,
	FS_RST_NAMES_REQ_SNT,
	FS_RST_START_REQ_SNT,		/*  */
	FS_RST_ABORT_REQ_SNT,
	FS_RD_DATA_RSP_SNT, 		/*  */
	FS_WR_DATA_RSP_SNT,

	SSC_SND_FH_DIR_REQ_SNT,
	SSC_SND_FH_NODE_REQ_SNT,
	SSC_SET_DMPDATE_REQ_SNT,

	SSC_NOTIFY_DATA_HALTED_REQ_SNT,
	SSC_NOTIFY_MOVER_HALTED_REQ_SNT,
	SSC_NOTIFY_DATA_RD_REQ_SNT,
	SSC_NOTIFY_MOVER_PAUSED_REQ_SNT,
	SSC_LOG_FILE_REQ_SNT,
	SSC_LOG_MSG_REQ_SNT,

	SCSI_RD_DATA_REQ_SNT, 		/*  */
    SCSI_RD_DATA_RSP_RCVD,
	SCSI_WR_DATA_REQ_SNT,
    SCSI_WR_DATA_RSP_RCVD,

	SSC_DATA_LISTEN_RSP_SNT,
	SSC_DATA_CONN_RSP_SNT,
	SSC_DATA_START_BKUP_RSP_SNT,	/*  */
	SSC_DATA_START_RST_RSP_SNT,
	SSC_DATA_GETENV_RSP_SNT,
	SSC_DATA_GETSTATE_RSP_SNT,
	SSC_DATA_STOP_RSP_SNT,
	SSC_DATA_ABORT_RSP_SNT,			/* */

	SSC_MOVER_LISTEN_RSP_SNT,
	SSC_MOVER_CONN_RSP_SNT,		
	SSC_MOVER_CONT_RSP_SNT,
	SSC_MOVER_RD_RSP_SNT,
	SSC_MOVER_SET_RECSIZ_RSP_SNT,	/* 30 */
	SSC_MOVER_SET_WDW_RSP_SNT,
	SSC_MOVER_GET_STATE_RSP_SNT,	
	SSC_MOVER_ABORT_RSP_SNT,
	SSC_MOVER_STOP_RSP_SNT,
	SSC_MOVER_CLOSE_RSP_SNT,		/* 35 */

	SSC_UPD_ENV_REQ_SNT,

	UNSUPPORTED_MSG_RCVD,
	UNSUPPORTED_MSG_SNT,

	MAX_FP_STAT								/* This will always be at the end */
} fp_stat_t;


/*---------  Env ----------------- */
typedef struct
{
	int32		name_len;			/* length of ENV name string w the NULL */
	int32		val_len;			/* length of VALUE string w the NULL */
} env_t;

/*------------- Dump Date ---------------*/
typedef struct
{
    evm_volId_t   	fs_name;
    uint32  		date;         // time from GMT time (will go up to year 2037)
    uint32  		client_ip;    // IP address of the client
	int32   		dump_level;   // increment level
	int32			curr_bkup_ind;      // 1 means backup in progress
	char      		snapshot[NDMP_MAX_SNAPSHOT_NAME_LEN+8];
} dmpdate_t;

/*------------ Info for read passed from Data Server ------------*/
typedef struct
{
	uint64			ioCookie;
    int32           numIOs;
	int32			fs_status;
} ndmp_fs_info_t;

/*------------ Info for i/o node ------------*/
typedef struct _ndmp_ioReqNode
{
    uint64                  ioCookie;  
    char                    *buf;       /* ptr of buf to write */
    eeeGMT_t                sent_time;
    eeeGMT_t                rcv_time;
    struct _ndmp_ioReqNode  *next;
} ndmp_ioReqNode_t;

typedef struct
{   
    uint32                  ioReqCnt;     /* Number of req. in pend queue */
    ndmp_ioReqNode_t        *head;    /* Head of the queue */
    ndmp_ioReqNode_t        *last;    /* Last member of the link list */
    ndmp_ioReqNode_t        *curr;    /* Last issued req to Scsi */
} ndmp_ioReqQ_t;

/*------------- Mover  ---------------*/
typedef struct mover
{
#ifdef NFX_SMP
	nfx_spinlock_t			spin;
#endif
    char8			        mode;           /* current mode (NOACTION, READ, WRITE) */
    char8			        awaiting_close; 
	char8					re_request_rd;	/* to reissue SCSI read after PAUSE->ACTIVE state */
	char8					re_request_wr;	/* to reissue SCSI write after PAUSE->ACTIVE state */
	uint64					re_request_rd_ioCookie;	/* to reissue SCSI read after PAUSE->ACTIVE state */
	uint64					re_request_wr_ioCookie;	/* to reissue SCSI read after PAUSE->ACTIVE state */
    ndmp_mover_pause_reason pauseReas;    	/* current pause reason */
    ndmp_mover_halt_reason  haltReas;     	/* current halt reason */
    uint64                  dataWritten;    /* total written to tape */
    uint64                  seekPos;   		/* position we want to be at */
    uint64                  wdwOffset;   	/* valid data window begin */
    uint64                 	wdwLen;   		/* valid data window length */
    uint64                  curpos;       	/* current tape pos */
    int32                  	recSiz;     	/* tape I/O record size */
    uint32                  recNum;      	/* current record num */
    int32                   listenSock;     /* data conn listen socket */
    int32                   connSock;       /* data conn socket, provided by remote Data service */
    int32                   listenConnId;   /* listen conn id, provided TPL */
    int32                   connConnId;     /* data conn id, provided by TPL */
    uint64                  rIndex;         /* buffer read  index */
    uint64                  wIndex;         /* buffer write index */
    uint64                  bytesLeftToRead;/* #bytes to read before end of window */
    uint32	                fs_rIndex;      /* index into FS buffer, used when FS buf's greater tapeRecSiz*/
	uint32					fs_bufSiz;
    ndmp_addr               dataAddr;  		/* address for connection to Data */
	ndmp_AddrTyp 			addr_type;		/* Local/TCP */
    char8                   *fsBuf;			/* FS data buffer */
    char8                   *writeBuf;		/* Tape RecSiz data buffer */
	int32		 			tape_hdl;	 	/* tape handle */
	esm_handle_t 			my_smHdl;		/* my State Machine handle */
	esm_handle_t 			data_smHdl;		/* Data's State Machine handle */
//new performance code
    int32                   wait_to_pause;  /* True when we hit a FM/EOM on a rd/wr request so we
                                             * can ignore subsequent read requests.  Also so
											 * we won't queue up more requests to SCSIIO as FS
											 * send more i/o reqs while we're waiting to pause.
                                             */
    uint32                  scsiioPendReq;  /* # of scsiio's pending at any time */
    uint64                  lastIoCookieRsp; /* Last OK or err response givein to FS */
    uint64                  nextIoCookieSend; /* Next ioCookie to send from */
    uint64                  maxIoCookieReq; /* More recent ioCookie request from FS */
	ndmp_ioReqQ_t           ioPendQ;    	/* Pending-req-to-Scsi queue */
    uint64                  delta_time;
    uint64                  io_wr_done;
    uint64                  wr_pend;
} Mover_t;

/*------------- Data  ---------------*/
typedef struct data
{
#ifdef NFX_SMP
	nfx_spinlock_t			spin;
#endif
    ndmp_data_ops         	oper;        /* current operation */
    ndmp_data_state         state;       /* current state */
    ndmp_data_halt_reason   haltReas; 	 /* current reason */
	ndmp_AddrTyp 			addr_type;	 /* LOCAL or TCP */
    boolean                 abortFlag;   /* abort operation flag */
    boolean                 ioReady;     /* mover sock read for I/O */
    ndmp_pval               *env_lst;    /* environment from backup or recover request */
    uint32                  envLen;      /* environment length */
    ndmp_name               *fil_lst;    /* recover file list */
    uint32                  flist_len;   /* recover file list length */
    uint32	                dataConnAddr;/* address for Data connection either in listen or connect mode*/
    uint16	                dataConnPort;/* Port for Data connection either in listen or connect mode*/
    ndmp_addr               moverAddr;   /* address for connection to Mover */
    int                     connSock;    /* socket to remote mover, provided by Mover */
    int                     listenSock;  /* socket listening on & provide to remote mover */
    int                     connMode; 	 /* data connection mode (LISTEN/CONNECT) */
	int						first_fs_rd_req;	/* First FS Read request for restore op */
	uint64					recov_bkupSiz; /* total bkup stream size used for restore */
    uint64                  readOffset;  /* data read seek offset */
    uint64                  readLen; 	 /* data read length */
    uint64                  bytesLeftToRead;/* #bytes to read til end of window */
    uint64                  bytesProcessed;/* total bytes processed so far */
    uint64                  estTimeRemain;/*  */
    uint64                  position;    /* current data stream pos */
    uint32                  discardLen;  /* bytes to discard */
	int						upd_dumpdate;/* T/F */
	esm_handle_t 			my_smHdl;	 /* State Machine handle */
	esm_handle_t 			mover_smHdl; /* Mover's State Machine handle */
	dmpdate_t				dd;			 /* dumpdate record saved */
	int32					sendDumpDate; /* T/F if FS gave us DumpDate */
#ifdef NFX_SMP
    int32                  	recSiz;     	/* tape I/O record size */
#endif
	void					*fsCookie;		
	int32					start_ops_state;	/* bkup/rst xmt/rcv req state */
	void					*ops_req;	 /* holds client's bkup/restore request */
	int32					path_len;	 /* len of file path to bkup (ex: '/', or '/etc') */
	char					*path_ptr;	 /* ptr to start of file path to bkup */
	char					*o_path_ptr; /* orig path ptr w/o volId, for restore */
	int32					o_path_len;	 /* orig path len w/o volId, for restore */
	int32					d_fs_name_len; /* dest volId (fs name) len, for restore */
	int32					flist_cur_indx;	/* cur idx of restore file list to send NAMES req to FS */
	int64					d_volId;
	int32					do_file_hist;/* TRUE/FALSE based on client request */
	int32					io_pending;	 /* TRUE if we have not respond to FS of its IO request */
	uint32					partial_bkup;
} Data_t;

#if 0
/*------------- File History  ---------------*/

typedef struct fh_dir_rec
{
    uint64          inode;
    uint64          p_inode;
	uint64			fname_offset;
    int32           fname_len;
    int32           pad;
} fh_dir_rec_t;

typedef struct fh_node_rec
{
    uint64          inode;
    int32		    file_type;
    uint32          links;          /* number of links */
    fs_Time         mtime;          /* time file last modified */
    fs_Time         atime;          /* time file last accessed */
    fs_Time         ctime;          /* time file data or file attributes was last modified */
    uint32          group;          /* file group id */
    uint32          owner;          /* file owner id */
    uint64          size;           /* file size in bytes */
    uint32          fattr;          /* system file attribute.  Unix file mode should be used for
                                     * Unix file system type. On UNIX file mode & type are
                                     * typically encoded together.  fattr must be set to just
                                     * the mode bits, excluding the type bits.
                                     */
    int32           pad;            /* align to 64 */
} fh_node_rec_t;

typedef struct fh
{
	int32					tot_dir_files;	/* num of files accumulated so far */
	int32					tot_node_files;	/* num of files accumulated so far */
	int32					tot_dir_len_in_bytes; /* we are bound by a STD_BUFSIZ bytes limitation
											   * on xmission.
                                               */
	int32					tot_node_len_in_bytes; /* we are bound by a STD_BUFSIZ bytes limitation
											   * on xmission.
											   */
	void					*fh_dir_lst[NDMP_FH_MAX_NUM_FILES];
	void					*fh_node_lst[NDMP_FH_MAX_NUM_FILES];
} Fh_t;
#endif

/*------------- NDMP Services ---------------*/
typedef struct ndmp_n_serv
{
    Data_t              Data;
    Mover_t             Mover;
	int32				sess_idx;
    int32               sessId;
    uint32              ssc_src_port;
    uint32              fp_src_port;
    evm_volId_t         volId;
	int32				version;
    int32               moreNames;
} NdmpNIMService_t;

/*------------- NDMP Control Message Structure ---------------*/
typedef struct CtrlMsg
{
    ndmp_Hdr         	hdr;			/* has xmtseq, timestamp, req/rsp, msgid, rspseq, err */
    char8				content[NDMP_MAX_REQ_SIZ];		/* content of msg specified by msgid */
} CtrlMsg_t;


/* Received on NNIM side */
typedef struct ndmpNNIMMsgEntry
{
    uint32       msgid;         /* which message is it? */
    int32        v2_event;    	/* V2 state machine event to queue */
    int32        v3_event;     	/* V3 state machine event to queue */
    int32		 sm;			/* which state machine MOVER/DATA */
	char8		 *msg_name;		/* For debug msgs */
	int32		 stat_idx_rcvd;		/* Index to update statistics of msg rcvd */
	int32		 stat_idx_snt;		/* Index to update statistics of msg sent */
	int32		 orig;			/* where did this msg come from, SSC or FP (for SMP sup) */
} ndmpNNIMMsgEnt_t; 

/*------------- Generic Internal NDMP Message Header Struc ---------------*/
/*------------- used among NDMP servers on different boards ---------------*/
typedef struct ndmp_NFXmsgHdr
{
	uint32 	mid;					/* NFX NDMP internal msg id */
#define NDMP_SESSID_ALL  0xFFFFFFFE
	int32 	sessId;
#ifdef NFX_SMP
	uint64	seqId;
	int32	idx;
#endif
	uint32	clientHdl;
} ndmp_NFXmsgHdr_t;

/*------------- Generic Internal NDMP Request Message Struc ---------------*/
typedef struct ndmp_NFXReq
{
	ndmp_NFXmsgHdr_t	hdr;
	ndmp_Hdr			cl_hdr;		/* hdr of client msg received */
	uint32				ndmp_mid;	/* actual NDMP mid, for */
	int32				userInfo;	/* could be anything */
	int32				len1;		
#ifdef NFX_SMP
	int32				spare;	// for later when we go to 64 bits	
#endif
	char8				content[1];	/* request content, usually msg forwarded */
} ndmp_NFXReq_t;

/*------------- Generic Internal NDMP Response Message Struc ---------------*/
typedef struct ndmp_NFXRsp
{
	ndmp_NFXmsgHdr_t	hdr;
	ndmp_Hdr			cl_hdr;		/* hdr of client msg received */
	uint32				ndmp_mid;	/* actual NDMP mid, for */
	int32				userInfo;	/* could be anything, usually error */
	int32				len;		/* len of content block */
#ifdef NFX_SMP
	int32				spare;		
#endif
	char8				content[1];	/* reply content, anything */
} ndmp_NFXRsp_t;

/*------------- Internal NDMP UI GetSEnv Rsp  ---------------*/

typedef struct 
{
	int32				num_env_mod;		/* Num of env need to be modified */
	char8				content[1];			/* this will be num_env_mod array of env_t,
											 * followed by char strings of name/value pairs.
											 */
} ndmp_GetEnv_Rsp_t;

/*------------- Internal NDMP UI GetState Request  ---------------*/

typedef struct ndmp_NFXUI_GetState_Rsp
{
    ndmp_NFXmsgHdr_t    hdr;
	int32				error;
    int32			    mover_mode; /* current Mover mode 
									 * (NOACTION, READ, WRITE) 
									 */
	ndmp_mover_get_state_rsp	mover_state;
	ndmp_data_get_state_rsp		data_state;
	
} ndmp_NFXUI_GetState_Rsp_t;

/*------------- Internal NDMP Tape Space Request from Mover ---------------*/
typedef struct 
{
	 ndmp_NFXmsgHdr_t    hdr;
  	 ndmp_tape_mtio_op	 ops;
	 int32				 count;
} ndmp_NFX_tape_space_req_t;

typedef struct 
{
	 ndmp_NFXmsgHdr_t    hdr;
  	 ndmp_tape_mtio_op	 ops;
	 ndmp_Err			 error;
} ndmp_EFX_tape_space_rsp_t;

#define BU_TYP_NAME_LEN_MAX		20

/*------------- Internal NDMP Bkup Request  ---------------*/
typedef struct
{
	int64   fs_volId;       		/* vol id of fs to be backed up*/ 
	int32	num_env;				/* number of environment vars pairs (name & value strings) */
	int32	tot_env_data_len;		/* len in bytes of all env.vars */
	int32	addr_typ;				/* Local/TCP (remote) */
	int32	ip_addr;				/* For TCP (remote) */
	int32	port;
	int32	spare;
	char8	env_data[1];
} ndmp_NFXBkupReq_t;

/*------------- Internal NDMP Restore Request  ---------------*/
typedef struct
{
	int64   org_fsvolId;
	int64   dst_fsvolId;
	int32	addr_typ;				/* Local/TCP (remote) */
	int32	ip_addr;				/* For TCP (remote) */
	int32	port;
	int32	spare;
	int32	num_env;				/* number of environment vars pairs (name & value strings) */
	int32	tot_env_data_len;		/* len in bytes of all env.vars */
	/*int32	num_files;*/				/* Num of set of files (oldname & newname) in 
									 * file list to restore 
									 */
	// int32	tot_file_name_len;		/* Total len of file names in bytes */
	char8	data[1];				/* namelen, name_data, value_len, value_data and so on */	
									/* then at data[tot_env_data_len] offset will be
									 * oldFileLen, oldFileName, newFileLen, newFileName */
} ndmp_NFXRecovReq_t;

typedef struct
{
    uint32 dataLen;
    uint32 eCode;
    eee_desc_t *eBufptr;
} ndmp_NFXScsiRdRsp;

typedef struct _ndmp_NFXSendNamesReq_t
{
    int32   num_files;
    int32   tot_file_name_len;
    char8   data[1];
} ndmp_NFXSendNamesReq_t;

typedef ndmp_NFXmsgHdr_t ndmp_NFXSendNamesRsp_t;

#ifdef NFX_SMP
typedef struct msgInfo
{
	int32 				sessid;
	int32				msgtyp;
	uint32				ssc_src_port;
	ndmpNNIMMsgEnt_t    *msgEnt;
	char				*msgp;
} msgCntxt_t;

typedef struct msg_q_q
{
	q_headNodePtr_t msg_q;
	uint64 nxt_seq_expected;
	int32 msgq_cnt;
} msg_q_t;
#endif

typedef struct sess_node
{
#ifdef NFX_SMP
	nfx_spinlock_t 		servp_lock;
	nfx_spinlock_t 		ssc_msgq_lock;
	nfx_spinlock_t 		fp_msgq_lock;
	msg_q_t             ssc_rcv_q;
	msg_q_t             fp_rcv_q;
#endif
    int32               sessid;
    NdmpNIMService_t    *servp;
#ifndef NFX_SMP
    struct sess_node    *nxt;
#endif
} sess_node_t;


/*------------- Control Msg ---------------*/
typedef struct ndmp_NNIMCtrl
{
    int32       appId;                          /* NDMP eee application id */
	int32		src_port;						/* store our source port for eee_sendmessage */
#ifdef NFX_SMP
	nfx_spinlock_t spin;
	sess_node_t sessLst[NDMP_MAX_NUM_SESS];	/* pts to a list of sessions */
	esm_handle_t msgThreadHdl;
#else
	sess_node_t *sessLst[NDMP_MAX_NUM_SESS];	/* pts to a list of sessions */
#endif 
} ndmp_NNIMCtrl_t;


/*--------------------------------------------------------------------
 * 					Function Prototypes
 *--------------------------------------------------------------------
 */
char8* ndmp_getEnvVar(char *env_name, ndmp_pval *env_lst, int env_lst_len);
int32 ndmp_getDumpDate(dmpdate_t *const *const ddrec, const int32 dump_level, 
                       const int64 volid, const char *const volname, 
                       const uint32 client_ip, char *const err);
//int32 ndmp_getDumpDate(dmpdate_t **ddrec, int32 dump_level, int64 volid, char *volname, uint32 client_ip, char *err);
void ndmp_data_init(Data_t *data);
int ndmp_sendCmdRsp( ndmp_NFXReq_t *req, int32 dst_port, uint32 error, int32 userInfo );
int ndmp_sendSCSIRdWrCmd(int type, uint64 ioCookie, NdmpNIMService_t *servp, char8 *buf);
void free_eee_buf_chain(eee_desc_t *ebuf);
void ndmp_NNIMProcessEvents(nfx_eventMsg_t *evt);
void ndmp_NNIMReceiveMsg(eee_desc_t *eDesc);

ndmpNNIMMsgEnt_t *ndmpGetNNIMMsgEnt(uint32 msgid);
int32 ndmp_close_conn(int32 sessId, Mover_t *mover, ndmp_mover_halt_reason halt_reas, int32 dst_port, char *reason);
int ndmp_save_env(char8 *env_data, int32 num_env, Data_t *data);
int ndmp_save_flist(char8 *flist_data, int32 num_fil, Data_t *data);
int ndmp_free_env(Data_t *data);
int ndmp_free_flist(Data_t *data);
void ndmp_mover_release_resource(Mover_t *mover);
char *ndmp_getAgileMsgName(uint32 msgtyp);
int ndmp_data_rel_resource(NdmpNIMService_t *servp);
int fill_Data_get_state_info(Data_t *data, ndmp_data_get_state_rsp *gs_rsp, esm_handle_t handle);
int fill_Mover_get_state_info(Mover_t *mover, ndmp_mover_get_state_rsp *gs_rsp, esm_handle_t handle);
sess_node_t *ndmp_find_sessid_entry(int32 sessId, int *idx);
int ndmp_del_sessid_entry(int32 idx, int32 sessId);
sess_node_t *ndmp_add_sessid_entry(int32 idx, int32 sessId);
void print_sessid_tbl();
int ndmp_snd_fh_2ssc(NdmpNIMService_t *servp, int32 mid);
ndmp_ioReqNode_t *ndmp_getNextFromQ(Mover_t *mover);
void ndmp_addReqToQueue(Mover_t *mover, uint64 ioCookie, char8 *buf);
int ndmp_delReqFromQueue(Mover_t *mover);
char *decode_data_halt_reas(int reas);
char *decode_data_state(int state);
char *decode_data_event(int event);
char *decode_mover_halt_reas(int reas);
char *decode_mover_pause_reas(int reas);
char *decode_mover_state(int state);
char *decode_mover_event(int event);
char *decode_scsi_ecode(int ecode);

int32 ndmp_sndNamesReq(NdmpNIMService_t *servp, int32 i);
int32 ndmp_doBkupRecovErr(NdmpNIMService_t *servp, uint32 error);

#ifdef NFX_SMP
int32 ndmp_InsertMsgInOrder(uint64 *seq_id, msgCntxt_t **msgI,  sess_node_t *node, int32 cardtyp);
int32 ndmp_send_evnt(sess_node_t *node);
int ndmp_relSessRsrc(NdmpNIMService_t *servp);
int cleanup_msg_q(nfx_spinlock_t *msgq_lock, msg_q_t *rcv_q);
#endif

#endif /* NDMP_H */



