/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-acl.c
 *
 * RCS:  $Id: nfs-acl.c,v 1.1 2002/08/29 22:30:52 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Contains NFS ACL Procedure.
 *
 * Created by: 	Joseph Vu 
 *
 * Date Created:  5/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/nfs-api.h>

/*-----------------------------------------------------------------
 * Name:		nfs_procRxNfsAcl 
 *
 * Description: Handle special packet NFS_ACL (program number=100027)
 *				send by the SUN Solaris client. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_procRxNfsAcl(nfs_request_t  *nfsReqp)
{
	uint32				txMsgLen;
	char				*txMsgp;
	rpcCallHdr_t        *rpcCallHdrp;
	nfp_clientCBPtr_t	clientCBp;
	pkt_desc_t 			*tplXmitPktDescp;
	uint32				*rpcReplyp;

	/*
	 *Increment NULL receive statistic counter
	 */
	rpcCallHdrp = nfsReqp->rpcCallHdrp;

	clientCBp = nfsReqp->clientCBp;

	/*
	 *Alloc TPL packet to send
	 */
	tplXmitPktDescp = (pkt_desc_t *)nfs_tplAllocPkt(clientCBp, 
										nfsReqp, 0);
	if (tplXmitPktDescp == NULL)
	{
        nfs_msgQueReplyMsg(clientCBp, NFS_TX_SUNACL_MSG_TYPE,
                        (void *)nfsReqp,0,0);
        return (NFX_ERR);
	}

	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;
	/*
	 *Start with RPC reply header
	 */
	rpcReplyp = (uint32 *)txMsgp;
	*rpcReplyp++ = rpcCallHdrp->rpc_xid;
	*rpcReplyp++ = RPC_REPLY;
	*rpcReplyp++ = RPC_REPLY_MSG_ACCEPTED;
	*rpcReplyp++ = NFS_RPC_AUTH_NONE;
	*rpcReplyp++ = 0;
	*rpcReplyp++ = RPC_REPLY_PROG_UNAVAIL;

    txMsgLen = 24;

	/*
	 *Set packet desc hdr size
	 */
	tplXmitPktDescp->hdrSize = txMsgLen;
	if (nfs_msgXmtPkt(tplXmitPktDescp, nfsReqp) != NFX_OK)
	{
		//Error:  set reqState to DONE so that
		//nfs_reqTimeoutHandler() will free nfsReqp
		nfsReqp->pktDescp = (void *)tplXmitPktDescp;
		nfsReqp->reqState = NFS_REQ_DONE;
	}

	return (NFX_OK);
}

#endif  //NFP_TXRX
