/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-attr.c
 *
 * RCS:  $Id: nfs-attr.c,v 1.29 2002/09/20 00:00:04 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 * 	Handle NFS attributes. 
 *
 * Created by: Joseph Vu
 *
 * Date Created:        7/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef	NFP_TXRX
#include <nfx-incl.h>
#include <exports/nfs-api.h>

#define NFS_DBG_PRINTF

/*-----------------------------------------------------------------
 * Name: fs_checkSetattrCtimeFlag
 *
 * Description:
 *  This function should be to handle request by a NFS version 3 client.
 * A client may request that the server check that the object is in an
 * expected state before performing the SETATTR operation.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_checkSetattrCtimeFlag(char *nfsMsgp,    /*Message buffer*/
            fs_Time *servCtimep,    /*Server ctime*/
            int32 *ctimeEqualFlag   /*compare ctime flag*/
)
{
//    uint32      guard;
    int32       length;
    fs_Time     *clientCtimep;
    char        *msgp;
   
    length = 0;
    msgp = nfsMsgp;

    *ctimeEqualFlag = TRUE;
    clientCtimep = (fs_Time *)msgp;
	//TODO
#ifdef	NOTUSED
    if ( !((clientCtimep->secs == servCtimep->secs) &&
        (clientCtimep->nsecs == servCtimep->nsecs) ) )
    {
        *ctimeEqualFlag = FALSE;
    }
#endif
    length += sizeof(fs_Time);

    return (length);
}

/*-----------------------------------------------------------------
 * Name: nfs_extractSetattr 
 * 
 * Description:
 *  Retrieves the Setattr (Set attributes) from message buffer
 *  received from the client and stores into attributes structure
 *  for sending to the file system.
 *  
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32   /*
         *length: length of the setattr message buffer
         *NFX_ERR: Error 
         */
nfs_extractSetattr(int32 nfs_version,       /*NFS Version*/
            char *nfsMsgp,      /*source message buffer*/
            fs_Attr *fsAttrp,   /*destination file system attributes structure*/
            uint32 *atimeType,  /*access time type setting*/
            uint32 *mtimeType   /*modification time setting*/
)
{
	int32   length;
	int32	totalLength;
	char	*nfs_msgp;

	nfs_msgp = nfsMsgp;
	totalLength = 0;

	/*
	 *Initializes file system attributes buffer
	 */
	fsAttrp->type = -1;
	fsAttrp->mode = -1;
	fsAttrp->numLinks = -1;
	fsAttrp->uid = -1;
	fsAttrp->gid = -1;
	fsAttrp->size = -1;
	fsAttrp->blockSize = -1;
	fsAttrp->devNum.major = -1;
	fsAttrp->devNum.minor = -1;
	fsAttrp->fsId = -1;
	fsAttrp->inode = -1;
	fsAttrp->ctime.secs = -1;
	fsAttrp->ctime.nsecs = -1;
	fsAttrp->rdev = -1;

	switch (nfs_version)
	{
		case NFS_VER2:
		{
			uint32			newValue;

			//extract mode field
			length = nfs_extractLongVal(nfs_msgp, 
											&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				//set mode field
				fsAttrp->mode = newValue;
			}
			//extract uid field
			length = nfs_extractLongVal(nfs_msgp, 
											&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				//Set uid field
				fsAttrp->uid = newValue;
			}
			//extract gid field
			length = nfs_extractLongVal(nfs_msgp, 
											&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				//Set gid field
				fsAttrp->gid = newValue;
			}
			//extract Size field
			length = nfs_extractLongVal(nfs_msgp, 
											&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				//Set the size field
				fsAttrp->size = newValue;
			}
			//extract atime field
			length = nfs_extractLongVal(nfs_msgp, 
										&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				fsAttrp->atime.secs = newValue;
                fsAttrp->atimeType = FS_ATTR_TIME_SET_CLIENT;
			}
            else
            {
				fsAttrp->atime.secs = -1;
                fsAttrp->atimeType = FS_ATTR_TIME_SET_DONT;
            }
			length = nfs_extractLongVal(nfs_msgp, 
										&newValue);
			nfs_msgp += length;
			if (newValue != -1)
			{
				fsAttrp->atime.nsecs = newValue * 1000;
			}
            else
            {
				fsAttrp->atime.nsecs = -1;
            }

			//extract mtime field
			length = nfs_extractLongVal(nfs_msgp, 
										&newValue);
			nfs_msgp += length;
            if (newValue != -1)
			{
				fsAttrp->mtime.secs = newValue;
                fsAttrp->mtimeType = FS_ATTR_TIME_SET_CLIENT;
			}
            else
            {
				fsAttrp->mtime.secs = -1;
                fsAttrp->mtimeType = FS_ATTR_TIME_SET_DONT;
            }
			length = nfs_extractLongVal(nfs_msgp, 
										&newValue);
			nfs_msgp += length;
			if (newValue != -1)
			{
				fsAttrp->mtime.nsecs = newValue * 1000;
			}
            else
            {
				fsAttrp->mtime.nsecs = -1;
            }
			return (sizeof (nfs_v2Sattr_t));
		}  
        case NFS_VER3:
        {
			uint32		valueSet;
			
			length = nfs_extractLongVal(nfs_msgp, &valueSet);
			nfs_msgp += length;
			totalLength += length;
			if (valueSet)
            {
				/*
				 *mode field is set
				 */
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->mode);
				nfs_msgp += length;
				totalLength += length;
			}
			length = nfs_extractLongVal(nfs_msgp, &valueSet);
			nfs_msgp += length;
			totalLength += length;
			if (valueSet)
            {
				/*
				 *uid field is set
				 */
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->uid);
				nfs_msgp += length;
				totalLength += length;
			}
			length = nfs_extractLongVal(nfs_msgp, &valueSet);
			nfs_msgp += length;
			totalLength += length;
			if (valueSet)
			{
				/*
				 *gid field is set
				 */
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->gid);
				nfs_msgp += length;
				totalLength += length;
			}
			length = nfs_extractLongVal(nfs_msgp, &valueSet);
			nfs_msgp += length;
			totalLength += length;
			if (valueSet)
			{
				/*
				 *size field is set
				 */
				length = nfs_extractLongLongVal(nfs_msgp, 
												&fsAttrp->size);
				nfs_msgp += length;
				totalLength += length;
			}
			length = nfs_extractLongVal(nfs_msgp, atimeType);
			nfs_msgp += length;
			totalLength += length;
			if (*atimeType == NFS_SATTR_CLIENT_TIME)
			{
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->atime.secs);
				nfs_msgp += length;
				totalLength += length;
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->atime.nsecs);
				nfs_msgp += length;
				totalLength += length;
                fsAttrp->atimeType = FS_ATTR_TIME_SET_CLIENT;
			}
            else if (*atimeType == NFS_SATTR_SERVER_TIME)
            {
                fsAttrp->atimeType = FS_ATTR_TIME_SET_SRVR;
	            fsAttrp->atime.secs = -1;
	            fsAttrp->atime.nsecs = -1;
            }
            else
            {
                fsAttrp->atimeType = FS_ATTR_TIME_SET_DONT;
	            fsAttrp->atime.secs = -1;
	            fsAttrp->atime.nsecs = -1;
            }
			length = nfs_extractLongVal(nfs_msgp, mtimeType);
			nfs_msgp += length;
			totalLength += length;
			if (*mtimeType == NFS_SATTR_CLIENT_TIME)
			{
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->mtime.secs);
				nfs_msgp += length;
				totalLength += length;
				length = nfs_extractLongVal(nfs_msgp, 
											&fsAttrp->mtime.nsecs);
				nfs_msgp += length;
				totalLength += length;
                fsAttrp->mtimeType = FS_ATTR_TIME_SET_CLIENT;
			}
            else if (*mtimeType == NFS_SATTR_SERVER_TIME)
            {
                fsAttrp->mtimeType = FS_ATTR_TIME_SET_SRVR;
	            fsAttrp->mtime.secs = -1;
	            fsAttrp->mtime.nsecs = -1;
            }
            else
            {
                fsAttrp->mtimeType = FS_ATTR_TIME_SET_DONT;
	            fsAttrp->mtime.secs = -1;
	            fsAttrp->mtime.nsecs = -1;
            }
			return (totalLength);
        }
	}

    return (NFX_ERR);
}

/*-----------------------------------------------------------------
 * Name: nfs_extractReaddirCookie 
 * 
 * Description:
 *      Extract READDIR Cookie field from message buffer.
 *      
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32   /*
         *return length field of cookie size or
         *NFX_ERR if error occurred
         */
nfs_extractReaddirCookie(uint32 nfs_version,    /*NFS version*/
        char *nfs_msgp,         /*source message buffer*/
        uint64 *cookie,         /*return cookie value*/
        uint64 *cookieVerf)     /*return cooker verifier*/
{
	int32	length;

    switch (nfs_version)
    {
        case NFS_VER2:
        {
			uint32		v2Cookie;

			nfs_extractLongVal(nfs_msgp, &v2Cookie);
			*cookie = v2Cookie;
            return (NFS_V2_COOKIE_SIZE);

        }
        case NFS_VER3:
        {
			char *msgp;

			msgp = nfs_msgp;
			length = nfs_extractLongLongVal(msgp, cookie);
			msgp += length;
			length = nfs_extractLongLongVal(msgp, cookieVerf);
            return (NFS_V3_COOKIE_SIZE+NFS_V3_COOKIE_VERF);

        }
    }
    return (NFX_ERR);
}


/*-----------------------------------------------------------------
 * Name: nfs_extractDevNum
 *
 * Description:
 *      Extract device number  from message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
		 *return length field in bytes of device number
		 *or NFX_ERR if error occurred
		 */
nfs_extractDevNum(char *nfs_msgp,       /*source message buffer*/
	fs_DevNum *devNump)             /*return device number*/
{
	uint32      value32;
	int32		length;
	char		*msgp;

	msgp = nfs_msgp;
	length = nfs_extractLongVal(msgp, &value32);
	msgp += length;
    devNump->major = value32;
	length = nfs_extractLongVal(msgp, &value32);
    devNump->minor = value32;

    return (sizeof(fs_DevNum));
}

/*-----------------------------------------------------------------
 * Name:nfs_storeDirEntry
 *
 * Description:
 *  Store Readdir directory entry into destination message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *length = number of bytes in readdir directory field
         */
nfs_storeDirEntry(uint32 nfs_version,           /*NFS version*/
	fs_DirEntry *fsDirEntryp,	/*Pointer to Readdir directory entry*/
	char *destMsgp			/*Pointer to destination message buffer*/
)
{
	char    *msgp;
	int32   length;
	int32   fieldLen;

	msgp = destMsgp;

	length = fieldLen =0;
	length = nfs_storeFieldPresentFLag(msgp, TRUE);
	msgp += length;
	fieldLen += length;
	/*
	 *store fileid (inode) number
	 */
	if (nfs_version == NFS_VER2)
		length = nfs_storeLongValToBuf(msgp, fsDirEntryp->inode);
	else
		length = nfs_storeLongLongValToBuf(msgp, fsDirEntryp->inode);

	msgp += length;
	fieldLen += length;

	/*
	 *store name of directory entry
	 */
	length = nfs_storeString(msgp, &fsDirEntryp->name[0], 
							fsDirEntryp->nameLen-1, FS_NAME_MAX);
	msgp += length;
	fieldLen += length;

	/*
	 *store the cookie (index) of the entry in the directory
	 */
	if (nfs_version == NFS_VER2)
		length = nfs_storeLongValToBuf(msgp, fsDirEntryp->cookie);
	else
		length = nfs_storeLongLongValToBuf(msgp, fsDirEntryp->cookie);
	fieldLen += length;

    //NFS_DBG_PRINTF("nfs_storeDirEntry 4: totalsize: %d\n", fieldLen);
	return (fieldLen);
}
/*-----------------------------------------------------------------
 * Name:    nfs_storeDirEntryPlus
 *
 * Description:
 *      Store Readdirplus directory entry into 
 * destination message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *length = number of bytes in readdirplus directory field
         */
nfs_storeDirEntryPlus(
	fs_DirEntryPlus *fsDirEntryPlusp,
	char *destMsgp  
)
{
    char        *msgp;
    int32       length;
    int32       fieldLen;
    fs_DirEntry *fsDirEntryp;

    msgp = destMsgp;

    length = fieldLen = 0;
    length = nfs_storeFieldPresentFLag(msgp, TRUE);
    msgp += length;
    fieldLen += length;

    fsDirEntryp = &fsDirEntryPlusp->dirEntry;
    /*
     *Store inode number
     */
    length = nfs_storeLongLongValToBuf(msgp, fsDirEntryp->inode);
    msgp += length;
    fieldLen += length;

    /*
     *store name of directory entry
     */
    length = nfs_storeString(msgp, &fsDirEntryp->name[0], 
							fsDirEntryp->nameLen-1, FS_NAME_MAX);
    msgp += length;
    fieldLen += length;

    /*
     *store the cookie (index) of the entry in the directory
     */
    length = nfs_storeLongLongValToBuf(msgp, fsDirEntryp->cookie);
    msgp += length;
    fieldLen += length;

    /*
     *Store the object post-op attributes
     */
    length = nfs_storeAttributes(NFS_READDIRPLUS, NFS_ATTR_AVAIL,
                &fsDirEntryPlusp->attr, msgp, NFS_VER3);
    msgp += length;
    fieldLen += length;

    length = nfs_storeFieldPresentFLag(msgp, TRUE);
    msgp += length;
    fieldLen += length;

    /*
     *Store object filehandle
     */
    length = nfs_storeFhandle(NFS_VER3, &fsDirEntryPlusp->handle, msgp);
    fieldLen += length;

    return (fieldLen);
}
/*-----------------------------------------------------------------
 * Name: nfs_storePreOpAttributes
 *
 * Description:
 * Store pre-operation file attributes information 
 * received from the file system
 * into a message buffer.
 * 
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *numofBytes: length of the pre-op attributes
         */
nfs_storePreOpAttributes(
	fs_Attr *fs_attrp,	/*file attributes from the file system*/
	char *destBuff,		/*destination mesage buffer*/
	uint32 attrAvail	/*is attribute available*/
)
{
    char                *msgp;
    nfs_preOpAttr_t     *preOpAttrp;

    msgp = destBuff;
#ifdef NOTUSED
    length = nfs_storeFieldPresentFLag(msgp, attrAvail);
    msgp += length;
    if ( (!attrAvail) || (fs_attrp == NULL) )
        return (length);
#endif

    *((uint32 *)msgp) = attrAvail;

    msgp += sizeof(uint32);
    if (!attrAvail)
        return (sizeof(uint32));


    preOpAttrp = (nfs_preOpAttr_t *)msgp;

//	nfs_storeLongLongValToBuf((char *)&preOpAttrp->faSize, 
//								fs_attrp->size);
//  memcpy (&preOpAttrp->faMtime, &fs_attrp->mtime, sizeof(fs_Time));
//  memcpy (&preOpAttrp->faCtime, &fs_attrp->ctime, sizeof(fs_Time));

	preOpAttrp->faSize[0] = (uint32) (fs_attrp->size >> 32);
	preOpAttrp->faSize[1] = (uint32) (fs_attrp->size & 0xFFFFFFFF);

	preOpAttrp->faMtime.secs  = fs_attrp->mtime.secs;
	preOpAttrp->faMtime.nsecs = fs_attrp->mtime.nsecs;

	preOpAttrp->faCtime.secs  = fs_attrp->ctime.secs;
	preOpAttrp->faCtime.nsecs = fs_attrp->ctime.nsecs;

    return (sizeof(uint32) + sizeof (nfs_preOpAttr_t));

}

/*-----------------------------------------------------------------
 * Name: nfs_msgStoreAttributes
 *
 * Description:
 *      Store file attributes information received from the file system
 *      into a message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */

int32   /*
         *length = number of bytes in the attributes field
         */
nfs_storeV3Attributes(uint32 	reqType,     	/*reqType*/
					  uchar8 	attrAvailable,	/*Is attribute available*/
					  fs_Attr 	*fs_attrp,		/*file attr from file system*/
					  char 		*destMsgp)		/*destination mesage buffer*/
{
    nfs_v3SpecificFattr_t   *v3Fattrp;
	int32					length;

	length = 0;

    if (reqType != NFS_GETATTR)
    {
       	*(uint32 *)destMsgp = attrAvailable;

        length    = sizeof(uint32);
        destMsgp += length;
    }

    if (!attrAvailable)
        return (length);


	v3Fattrp = (nfs_v3SpecificFattr_t *)destMsgp;

	v3Fattrp->ftype = fs_attrp->type;			
	v3Fattrp->mode  = (fs_attrp->mode & 0xFFF);	/*UNIX mode*/
	v3Fattrp->nlink = fs_attrp->numLinks;
	v3Fattrp->uid   = fs_attrp->uid;
	v3Fattrp->gid   = fs_attrp->gid;
			
	v3Fattrp->faSize[0] = (uint32) (fs_attrp->size >> 32);
	v3Fattrp->faSize[1] = (uint32) (fs_attrp->size & 0xFFFFFFFF);

	v3Fattrp->faUsed[0] = (uint32) (fs_attrp->size >> 32);
	v3Fattrp->faUsed[1] = (uint32) (fs_attrp->size & 0xFFFFFFFF);

	v3Fattrp->faRdev.major = fs_attrp->devNum.major;
	v3Fattrp->faRdev.minor = fs_attrp->devNum.minor;

	v3Fattrp->faFsid[0] = (uint32) (fs_attrp->fsId >> 32);
	v3Fattrp->faFsid[1] = (uint32) (fs_attrp->fsId & 0xFFFFFFFF);

	v3Fattrp->faFileid[0] = (uint32) (fs_attrp->inode >> 32);
	v3Fattrp->faFileid[1] = (uint32) (fs_attrp->inode & 0xFFFFFFFF);

	v3Fattrp->faAtime.secs  = fs_attrp->atime.secs;
	v3Fattrp->faAtime.nsecs = fs_attrp->atime.nsecs;

	v3Fattrp->faMtime.secs  = fs_attrp->mtime.secs;
	v3Fattrp->faMtime.nsecs = fs_attrp->mtime.nsecs;

	v3Fattrp->faCtime.secs  = fs_attrp->ctime.secs;
	v3Fattrp->faCtime.nsecs = fs_attrp->ctime.nsecs;

    return (length + sizeof (nfs_v3SpecificFattr_t));

}

int32   /*
         *length = number of bytes in the attributes field
         */
nfs_storeV2Attributes(uint32 	reqType,     	/*reqType*/
					  uchar8 	attrAvailable,	/*Is attribute available*/
					  fs_Attr 	*fs_attrp,		/*file attr from file system*/
					  char 		*destBuff)		/*destination mesage buffer*/
{
    nfs_v2SpecificFattr_t   *v2Fattrp;
	int32					length;

	length = 0;

    if (reqType == NFS_READDIR)
    {
		*(uint32 *)destBuff = attrAvailable;

        length    = sizeof(uint32);
        destBuff += length;
    }

    if (!attrAvailable)
        return (length);

    /*
     *file attributes
     */
	v2Fattrp = (nfs_v2SpecificFattr_t *)destBuff;

	v2Fattrp->ftype = fs_attrp->type;			
	v2Fattrp->mode  = fs_attrp->mode;	/*UNIX mode*/
	v2Fattrp->nlink = fs_attrp->numLinks;
	v2Fattrp->uid   = fs_attrp->uid;
	v2Fattrp->gid   = fs_attrp->gid;
			
	v2Fattrp->faSize        = (uint32)fs_attrp->size;
	v2Fattrp->faBlocksize   = fs_attrp->blockSize;
	v2Fattrp->faBlocks      = fs_attrp->blocks;
	v2Fattrp->faRdev        = fs_attrp->rdev;
	v2Fattrp->faFsid        = (uint32)fs_attrp->fsId;
	v2Fattrp->faFileid      = (uint32)fs_attrp->inode;
	v2Fattrp->faAtime.secs  = fs_attrp->atime.secs;
	v2Fattrp->faAtime.usecs = fs_attrp->atime.nsecs / 1000;
	v2Fattrp->faMtime.secs  = fs_attrp->mtime.secs;
	v2Fattrp->faMtime.usecs = fs_attrp->mtime.nsecs / 1000;
	v2Fattrp->faCtime.secs  = fs_attrp->ctime.secs;
	v2Fattrp->faCtime.usecs = fs_attrp->ctime.nsecs / 1000;

    return (length + sizeof(nfs_v2SpecificFattr_t));

}

int32   /*
         *length = number of bytes in the attributes field
         */
nfs_storeAttributes(uint32 reqType,     /*reqType*/
	uchar8 attrAvailable,	/*Is attribute available*/
	fs_Attr *fs_attrp,		/*file attributes from the file system*/
	char *destBuff,			/*destination mesage buffer*/
	int32 nfs_version)		/*NFS version*/
{

	if (nfs_version == NFS_VER3)
	{
		return(nfs_storeV3Attributes(reqType, attrAvailable, 
									fs_attrp, destBuff));
	}
	else
	{
		return(nfs_storeV2Attributes(reqType, attrAvailable, 
									fs_attrp, destBuff));
	}
}

#ifdef	NOTUSED
/*-----------------------------------------------------------------
 * Name:        nfs_msgStoreCacheAttributes
 *
 * Description:
 *      Store our cache file attributes information into a message buffer
 *      to be sent as a reply to a client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *return number of bytes for stroring the files attributes
         *in the destination message buffer
         */
nfs_storeCacheAttributes(
	nfs_fattr_t *nfs_attrp,    /*our cached file attributes*/
	char *destBuff,             /*destination mesage buffer*/
	int32 nfs_version,          /*NFS Version*/
	uint32 reqType
)  
{
	char                    *destMsgp;
	uint32                  *msg32p;
	int32                   length;
	nfs_v2SpecificFattr_t   *v2Fattrp;
	nfs_v3SpecificFattr_t   *v3Fattrp;

	length = 0;
	destMsgp = destBuff;
	if ( ( (nfs_version == NFS_VER3) && 
		(reqType && (reqType != NFS_GETATTR)) ) ||
		((nfs_version == NFS_VER2) && 
			(reqType && (reqType == NFS_READDIR)) ) )
	{
		length = nfs_storeFieldPresentFLag(destMsgp, 
										NFS_ATTR_AVAIL);
		destMsgp += length;
	}

	msg32p = (uint32 *)destMsgp;

	/*
	 *file attributes
	 */
	*msg32p++ = nfs_attrp->faType;    /*type of file*/
	if (nfs_version == NFS_VER3)
		*msg32p++ = (nfs_attrp->faMode & 0xFFF);
	else
		*msg32p++ = nfs_attrp->faMode;    /*UNIX mode*/
	*msg32p++ = nfs_attrp->faLink;    /*hard link count*/
	*msg32p++ = nfs_attrp->faUid;     /*file owner*/
	*msg32p++ = nfs_attrp->faGid;     /*file group*/

	length += 20;

	switch (nfs_version)
	{
		case NFS_VER2:
			v2Fattrp = &nfs_attrp->version.v2Fattr;
			memcpy ((char *)msg32p, v2Fattrp, 
					sizeof(nfs_v2SpecificFattr_t));
			length += sizeof (nfs_v2SpecificFattr_t);
			break;
		case NFS_VER3:
			v3Fattrp = &nfs_attrp->version.v3Fattr;
			memcpy ((char *)msg32p, v3Fattrp, 
					sizeof(nfs_v3SpecificFattr_t));
			length += sizeof (nfs_v3SpecificFattr_t);
			break;
		}
		return (length);
}

#endif	//NOTUSED

/*-----------------------------------------------------------------
 * Name:        nfs_storeFieldPresentFLag
 *
 * Description:
 *      Store value present flag into destination message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_storeFieldPresentFLag(char *msgp, uint32 presentFlag)
{
    uint32      *destMsgp;

    destMsgp = (uint32 *)msgp;

    if (presentFlag)
        *destMsgp = 1;
    else
        *destMsgp = 0;

    return (sizeof(uint32));
}

/*-----------------------------------------------------------------
 * Name:        nfs_storeStatInfo 
 * 
 * Description:
 *  Store Statfs (filesystem info) into destination message buffer.
 *  
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 * 
 *-----------------------------------------------------------------
 */
int32
nfs_storeStatInfo(char *destMsgp, fs_Stat *fsStatp, uint32 nfs_version)
{
	int32       length;
	char		*msgp;

	msgp = destMsgp;    
	switch (nfs_version)
	{
		case NFS_VER2:
		{

			//Optimum transfer size (tsize)
			length = nfs_storeLongValToBuf(msgp, 
								fsStatp->avgWriteSize);
			msgp += length;
			
			//Block size (bsize) 
            length = nfs_storeLongValToBuf(msgp,
								fsStatp->fsBlockSize);
			msgp += length;

			//Total number of bsize blocks
            length = nfs_storeLongValToBuf(msgp,
								fsStatp->numBlks);
			msgp += length;
 
			//Total number of free bsize blocks (bfree)
			length = nfs_storeLongValToBuf(msgp,
								fsStatp->numBlksAvail);
			msgp += length;

			//Total number of blocks available (bavail)
			length = nfs_storeLongValToBuf(msgp,
						fsStatp->numBlksAvailUser);
			msgp += length;

            length =  sizeof(nfs_v2Statfs_t);
            break; 
        }
        case NFS_VER3:
        {
			uint64		value64;

			//Total size of, in bytes of filesystem (tbytes)
			value64 = fsStatp->fsBlockSize * fsStatp->numBlks;
			length = nfs_storeLongLongValToBuf(msgp, value64);
			msgp += length;
			// *msgp64++ = value64;	
			// *(uint64 *)msgp = value64;
			// msgp += sizeof(value64);

			//Number of free spaces in bytes (fbytes)
			value64 = fsStatp->fsBlockSize * fsStatp->numBlksAvail;
			length = nfs_storeLongLongValToBuf(msgp, value64);
			msgp += length;

			//Available space  (abytes)
			value64 = fsStatp->fsBlockSize * fsStatp->numBlksAvailUser;
			length = nfs_storeLongLongValToBuf(msgp, value64);
			msgp += length;

			//Total number of file slots (tfiles)
			length = nfs_storeLongLongValToBuf(msgp, 
								fsStatp->numInodes);
			msgp += length;

			//Total number of file slots free (ffiles)
			length = nfs_storeLongLongValToBuf(msgp, 
								fsStatp->numInodesAvail);
			msgp += length;

			//Total number of file slots free 
			//to non-priviliged users (afiles)
			length = nfs_storeLongLongValToBuf(msgp, 
								fsStatp->numInodesAvailUser);
			msgp += length;

            //A measure of filesystem volatity (invarsec)
			length = nfs_storeLongValToBuf(msgp, 0);
			msgp += length;

            length = sizeof(nfs_v3Fsstat_t);
            break;
        }

    }

    return (length);
}

/*-----------------------------------------------------------------
 * Name:	nfs_storePathconf	
 *
 * Description:
 * Store PATHCONF-Retrieve POSIX information 
 * into destination message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_storePathconf(char *destMsgp, fs_Pathconf *pathconfp)
{
	uchar8  *msg8p;
	int32	length;

	length =0;
	msg8p = destMsgp;	
    //memcpy(destMsgp, pathconfp, sizeof(fs_Pathconf));

	*(uint32 *)msg8p = pathconfp->linkMax;
	msg8p += 4;
	length += 4;

	*(uint32 *)msg8p = pathconfp->nameMax;
	msg8p += 4;
	length += 4;

	*(uint32 *)msg8p = pathconfp->noTrunc;
	msg8p += 4;
	length += 4;
		
	*(uint32 *)msg8p = pathconfp->chownRestricted;
	msg8p += 4;
	length += 4;

	*(uint32 *)msg8p = pathconfp->caseInsensitive;
	msg8p += 4;
	length += 4;
	
	*(uint32 *)msg8p = pathconfp->casePreserving;
	msg8p += 4;
	length += 4;

    return (length);
}

/*-----------------------------------------------------------------
 * Name:        nfs_storeFsinfo
 *
 * Description:
 * Store FSINFO (Static filesystem information) 
 * into destination message buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_storeFsinfo(char *destMsgp, fs_Fsinfo *fsInfop)
{
    char		*msgp;
	int32		length;

	//Max size of Read request in bytes (rtmax)
    msgp = destMsgp;
	length = nfs_storeLongValToBuf(msgp,
						fsInfop->readMax);
	msgp += length;

    //Preferred size in bytes of Read req (rtpref)
    length = nfs_storeLongValToBuf(msgp,
						fsInfop->readPref);
	msgp += length;

	//Suggested multiple of Read req (rtmult)
	length = nfs_storeLongValToBuf(msgp,
							fsInfop->readMult);
	msgp += length;

	//Max size of Write req (wtmax)
    length = nfs_storeLongValToBuf(msgp,
								fsInfop->writeMax);
	msgp += length;

	//Prefered size of Write req (wtpref)
    length = nfs_storeLongValToBuf(msgp,
								fsInfop->writePref);
	msgp += length;

	//Suggested multiple size of Write req (wtmult)
    length = nfs_storeLongValToBuf(msgp,
								fsInfop->writeMult);
	msgp += length;

	//Preferred size of READDIR req (dtpref)
	length = nfs_storeLongValToBuf(msgp,
						fsInfop->readDirPref);
	msgp += length;

    //Max size of a file
    length = nfs_storeLongLongValToBuf(msgp, fsInfop->maxFileSize);
	msgp += length;

	//time delta
	length = nfs_storeLongValToBuf(msgp,
							fsInfop->timeDelta.secs);
	msgp += length;
	length = nfs_storeLongValToBuf(msgp,
							fsInfop->timeDelta.nsecs);
	msgp += length;

	//Properties
	length = nfs_storeLongValToBuf(msgp,
							fsInfop->fsProperties);

    return (sizeof(nfs_fsInfo_t));

}


/*-----------------------------------------------------------------
 * Name:       nfs_updateAttr
 *
 * Description:
 * Save the update attributes received from the 
 * file system to an existing File
 * Control  Block for future lookup.
 *
 * Created by:  Joseph Vu
 *
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
void nfs_updateAttr(nfp_fileCBPtr_t fileCBp, fs_Attr *fsAttrp, uint32 nfs_version)
{
    nfs_fileInfo_t          *nfs_fileInfop;


    if (fileCBp == NULL)
        return;
	
    nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
	memcpy(&nfs_fileInfop->fs_attr, fsAttrp, sizeof(fs_Attr));
    nfs_fileInfop->nfs_flag |= NFS_CACHE_FATTR;
	nfs_fileInfop->cacheTime = nfs_fileInfop->fs_attr.atime;
}

void nfs_updatefileSysAttr(nfp_fileSysCBPtr_t fileSysCBp, fs_Attr *fsAttrp, uint32 nfs_version)
{
    nfs_fileSysInfo_t		*fileSysInfop;

    if (fileSysCBp == NULL)
        return;

	fileSysInfop = &fileSysCBp->protocol.nfp_nfsFileSysInfo;

	memcpy(&fileSysInfop->fs_attr, fsAttrp, sizeof(fs_Attr));
    fileSysInfop->nfs_flag |= NFS_CACHE_FATTR;
}

/*-----------------------------------------------------------------
 * Name:        nfs_readAttrValue
 *
 * Description:
 *  Retrieve an attribute value from input File Control Block.
 *
 * Created by:  Joseph Vu
 *
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfs_readAttrValue (nfp_fileCBPtr_t fileCBp, uint32 attrType,
            void *inputValue, uint32 nfs_version)
{
    nfs_fileInfo_t          *nfs_fileInfop;
	fs_Attr					*fs_attrp;

    if (fileCBp == NULL)
        return (NFX_ERR);
    nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
	fs_attrp = &nfs_fileInfop->fs_attr;

    switch (attrType)
    {
        case NFS_ATTR_TYPE:
        {
            uint32      *type;

            type = (uint32 *)inputValue;
            *type = fs_attrp->attrType;
            return (NFX_OK);
        }
        case NFS_ATTR_MODE:
        {
            uint32      *mode;

            mode = (uint32 *)inputValue;
            *mode = fs_attrp->mode;
            return (NFX_OK);
        }
        case NFS_ATTR_NLINK:
        {
            uint32      *nlink;

            nlink = (uint32 *)inputValue;
            *nlink = fs_attrp->numLinks;
            return (NFX_OK);
        }
        case NFS_ATTR_UID:
        {
            uint32      *uid;

            uid = (uint32 *)inputValue;
            *uid = fs_attrp->uid;
            return (NFX_OK);
        }
        case NFS_ATTR_GID:
        {
            uint32      *gid;

            gid = (uint32 *)inputValue;
            *gid = fs_attrp->gid;
            return (NFX_OK);
        }
        case NFS_ATTR_SIZE:
        {
            uint32      *size32;

            if (nfs_version == NFS_VER2)
            {
                size32 = (uint32 *)inputValue;
                *size32 = (uint32)fs_attrp->size;
            }
            else
            {	
				nfs_storeLongLongValToBuf((char *)inputValue,	
								fs_attrp->size); 
            }
            return (NFX_OK);
        }
        case NFS_ATTR_BLOCKSIZE:
        {
            uint32      *blockSize;

            blockSize = (uint32 *)inputValue;
            *blockSize = fs_attrp->blockSize;
            return (NFX_OK);
        }
        case NFS_ATTR_USED:
        {
			nfs_storeLongLongValToBuf((char *)inputValue,	
							fs_attrp->size); 
            return (NFX_OK);
        }
        case NFS_ATTR_RDEV:
        {
            nfs_devNum_t    *devNum;
            uint32      *rdev;

            if (nfs_version == NFS_VER2)
            {
                rdev = (uint32 *)inputValue;
				*rdev = fs_attrp->rdev;
            }
            else
            {
                devNum = (nfs_devNum_t *)inputValue;
                devNum->major = fs_attrp->devNum.major;
                devNum->minor = fs_attrp->devNum.minor;
            }
            return (NFX_OK);
        }
        case NFS_ATTR_BLOCKS:
        {
            uint32      *blocks;

            blocks = (uint32 *)inputValue;
			*blocks = fs_attrp->blocks;			
            return (NFX_OK);
        }
        case NFS_ATTR_FSID:
        {

            uint32      *fsId32;

            if (nfs_version == NFS_VER2)
            {
                fsId32 = (uint32 *)inputValue;
                *fsId32 = (uint32)fs_attrp->fsId;
            }
            else
            {
				nfs_storeLongLongValToBuf((char *)inputValue,	
							fs_attrp->fsId); 
            }
            return (NFX_OK);
        }
        case NFS_ATTR_FILEID:
        {
            uint32      *fileId32;

            if (nfs_version == NFS_VER2)
            {
                fileId32 = (uint32 *)inputValue;
                *fileId32 = (uint32)fs_attrp->inode;
            }
            else
            {
				nfs_storeLongLongValToBuf((char *)inputValue,	
							fs_attrp->inode); 
            }
            return (NFX_OK);
        }
        case NFS_ATTR_ATIME:
        {
            fs_Time     *time;

            time = (fs_Time *)inputValue;
			time->secs = fs_attrp->atime.secs;
			time->nsecs = fs_attrp->atime.nsecs;
            return (NFX_OK);
        }
        case NFS_ATTR_CTIME:
        {
            fs_Time         *time;

            time = (fs_Time *)inputValue;
			time->secs = fs_attrp->ctime.secs;
			time->nsecs = fs_attrp->ctime.nsecs;
            return (NFX_OK);
        }
        case NFS_ATTR_MTIME:
        {
            fs_Time         *time;

            time = (fs_Time *)inputValue;
			time->secs = fs_attrp->mtime.secs;
			time->nsecs = fs_attrp->mtime.nsecs;
            return (NFX_OK);
        }

    }

    return (NFX_ERR);
}
#endif	//NFP_TXRX
