/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-util.c
 *                              
 * RCS:  $Id: nfs-util.c,v 1.35 2002/10/22 01:30:39 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  Contain NFS utility functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        7/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef  NFP_TXRX

#include <nfx-incl.h>
#include <exports/nfs-api.h>


uint32			nfs_callmsgFreeAll = 0;			//For debugging only
uint32			nfs_callmsgFreeAll2 = 0;		//For debugginng only

extern void tpl_checkConnActivity(uint32);

/*
 *------------------------------------------------------------------------------
 * Function Name:      	nfs_reqTimeoutHandler 
 *
 * Description:
 *		Periodic timeout function to delete oldest complete request entry from
 *		the Duplicate request cache.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/28/2000
 *
 *
 *------------------------------------------------------------------------------
 */
int32
nfs_reqTimeoutHandler(int32 timer_handle, void* param1, void* param2, 
				void* param3, void* param4)
{
	nfs_request_t 		*nfsReqp;	
	nfp_nodePtr_t		tailp;
	uint32              count;
	int32				iterations;
	int32				currTimeInSecs;
	int32				deltaSecs;
	nfp_list_t			*pReqCbList;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Enter\n", __FUNCTION__);
#endif

    count = 0;
	currTimeInSecs = nfs_utilGetTimeOfDayInSecs();

	pReqCbList = nfs_globalCfgp->pReqCbList;
	iterations = pReqCbList->nfp_reqCnt;

	for (tailp  = pReqCbList->nfp_tailp;
		 (tailp != NULL && iterations > 0);
		 tailp   = tailp->nfp_prevp, iterations--)
	{
		if (nfp_validateNode(tailp) != NFX_OK)
			return (NFX_ERR);

		nfsReqp = (nfs_request_t  *)tailp->nfp_entryCBp;
		if (nfsReqp->reqState == NFS_REQ_DONE)
		{
			if (nfsReqp->keepAliveCounter == 0)
			{
				nfs_callmsgFreeAll++;				//For debugging only
				nfs_msgFreeAll(nfsReqp, TRUE);
			}
			else
				nfsReqp->keepAliveCounter--;
		}
		else if (nfsReqp->reqState == NFS_REQ_SEND_TO_FS) 
		{
			//either timeout waiting from file system or
			//timeout wait TX compl from TPL
			deltaSecs = currTimeInSecs - nfsReqp->timeSendToFsInSecs;
			//Check for wrap
			if (deltaSecs < 0)
			{
				nfsReqp->timeSendToFsInSecs = currTimeInSecs;
				continue;
			}

			if (deltaSecs > NFS_MAX_WAIT_FS_RESP_TIME)
			{

				//Timeout no response from the file system

				nfsReqp->reqState = NFS_REQ_DONE;
				nfs_msgFreeAll(nfsReqp, 0);
				nfs_globalCfgp->nfs_debugPktCounters.nfs_fsNoRespRcvdErr++;
			}
		}
	}

#ifdef NOTUSED
		/* Shaun 09/27/02
		 *
		 * Delete clientCB with pending flag
		 * when no more nfs request.
		 */
	if ((clientCBp->nfp_flag & NFP_ENTRY_DEL_PENDING) &&
		(clientCBp->nfp_reqList.nfp_reqCnt <= 0))
	{
			
#ifdef NFS_DBG_PRINTF_ON
		printf("%s : clientCBp=%p deleted \n", __FUNCTION__, clientCBp);
#endif

		/* 
		 * Clear pending flag & delete ClientCB.
		 */
		clientCBp->nfp_flag &= ~NFP_ENTRY_DEL_PENDING;

		/* Shaun
		 * Disable clientCB delete for now.
		 * Caused nfsReqCB leakage.
		 *
		 * nfp_delClientCB (clientCBp);
		 */
	}
#endif


	//Increment activity time counter
	//for the File System Control Blocks (mount points)
	nfs_globalCfgp->nfs_activityTimeCnt++;
#ifdef	NOTUSED
	if (nfs_globalCfgp->nfs_activityTimeCnt >= NFS_MNT_PNT_ACT_TIME_CNT)
	{
		nfs_globalCfgp->nfs_activityTimeCnt = 0;
		nfs_checkMntPointActivity();
	}
#endif	//NOTUSED

	nfs_globalCfgp->nfs_BootUpTimeCnt++;

#ifdef NFS_DBG_PRINTF_ON
	printf("%s Exit\n", __FUNCTION__);
#endif

	return (NFX_OK);

}
/*
 *-----------------------------------------------------------------------
 * Function Name:   nfs_utilAllocReqPktType   	
 *
 * Description:
 *
 * Allocate a META packet descriptor or regular packet descriptor for 
 * transmission of response to the NFS client based on the request type.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/28/2000
 *
 *
 *----------------------------------------------------------------------
 */
pkt_desc_t *nfs_utilAllocReqPktType(
	nfs_request_t	*nfsReqp,	
	int32	dataLen)
{
	pkt_desc_t          *tplXmitPktDescp;

	if (nfsReqp->tplConnp == NULL)
		return (NULL);	

	/*
	 *Allocate META pkt descriptor for 
	 *for Readdir or Readdirplus
	 */
	if ( (nfsReqp->reqType == NFS_READDIR) || 
		 (nfsReqp->reqType == NFS_READDIRPLUS) )	
	{
		tplXmitPktDescp = TPL_ALLOC_PACKET_META(nfsReqp->tplConnp, 
												dataLen );	
	}
	else
		tplXmitPktDescp = TPL_ALLOC_PACKET(nfsReqp->tplConnp, dataLen);	

	return (tplXmitPktDescp);
}


/*
 *-----------------------------------------------------------------------
 * Function Name:      	nfs_tplAllocPkt 
 *
 * Description:
 * This function is called by NFS transmit functions to Allocate a packet
 * from the Transport Layer (TPL).  This function will try to delete
 * an oldest request entry from the duplicate request cache to service
 * new request if it cannot allocate packet from TPL.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/28/2000
 *
 *
 *----------------------------------------------------------------------
 */
pkt_desc_t *nfs_tplAllocPkt(nfp_clientCBPtr_t	clientCBp, 
							nfs_request_t 		*nfsReqp,
							int32				usrDataLen)
{
	pkt_desc_t          *tplXmitPktDescp;
	nfp_nodePtr_t		tailp;
	int32				done;
	int32				count;

	if ( (clientCBp == NULL) || (nfsReqp == NULL) )
		return (NULL);
	done = 0;
	count = clientCBp->nfp_reqList.nfp_reqCnt;
	tplXmitPktDescp = nfs_utilAllocReqPktType(nfsReqp, usrDataLen);
	if (tplXmitPktDescp == NULL)
	{
		/*
	 	 *Removes oldest entry in the duplicate reqs cache 
		 *to service new request.
	 	 */
		for (tailp = clientCBp->nfp_reqList.nfp_tailp; 
			(tailp != NULL && count > 0);
			tailp = tailp->nfp_prevp, count--)
		{
			nfsReqp = (nfs_request_t  *)tailp->nfp_entryCBp;
            if (nfsReqp->reqState == NFS_REQ_DONE)
            {
				nfs_callmsgFreeAll2++;
                nfs_msgFreeAll(nfsReqp, TRUE);
                break;
            }
		}		
	}
	else
	{	
		tplXmitPktDescp->dataSize = 0;
		nfs_globalCfgp->nfs_debugPktCounters.tpl_xmtPktAlloc++;
		return (tplXmitPktDescp);
	}

	/*
	 *Try again
	 */ 
	tplXmitPktDescp = nfs_utilAllocReqPktType(nfsReqp, usrDataLen);
	if (tplXmitPktDescp != NULL)
		tplXmitPktDescp->dataSize = 0;
	nfs_globalCfgp->nfs_debugPktCounters.tpl_xmtPktAlloc++;
	return (tplXmitPktDescp);
}

/*-----------------------------------------------------------------
 * Name:	nfs_cacheFindFileName 
 *
 * Description:
 *      Find a File Control Block with a correponding filename.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t nfs_cacheFindFileName(
        nfp_list_t *nodeListp,  /*list*/
        uchar8 *namep,          /*filename*/
        ushort16 nameLen)       /*filename length in bytes*/
{
    nfp_nodePtr_t       filep;
    nfp_fileCBPtr_t     fileCBp;
    nfs_fileInfo_t      *nfs_fileInfop;
    uint32              count;

    if ( (nodeListp->nfp_headp == NULL) || (namep == NULL) )
        return (NULL);

    count = nodeListp->nfp_reqCnt;
    for (filep = nodeListp->nfp_headp; ((count > 0) && (filep != NULL));
                filep = filep->nfp_nextp, count--)
    {
        fileCBp = (nfp_fileCBPtr_t)filep->nfp_entryCBp;
        if (fileCBp == NULL)
            continue;
        nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;

        if (!nfs_fileInfop->nfs_nameLen)
            continue;
        if ( (nfs_fileInfop->nfs_nameLen == nameLen  ) &&
                ( !(strncmp(nfs_fileInfop->nfs_namep, namep, nameLen)))  )

        {
            /*
             *found entry
             */
            return (fileCBp);
        }

    }

    return (NULL);
}

/*-----------------------------------------------------------------
 * Name:	nfs_findFileName 
 *
 * Description:
 *      Find a File Control Block with a correponding filename.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t nfs_findFileName(
        nfp_list_t *nodeListp,  /*list*/
        uchar8 *namep,          /*filename*/
        ushort16 nameLen)       /*filename length in bytes*/
{
    nfp_nodePtr_t       filep;
    nfp_fileCBPtr_t     fileCBp;
    nfs_fileInfo_t      *nfs_fileInfop;
    uint32              count;

    if ( (nodeListp->nfp_headp == NULL) || (namep == NULL) )
        return (NULL);

    count = nodeListp->nfp_reqCnt;
    for (filep = nodeListp->nfp_headp; ((count > 0) && (filep != NULL));
                filep = filep->nfp_nextp, count--)
    {
        fileCBp = (nfp_fileCBPtr_t)filep->nfp_entryCBp;
        if (fileCBp == NULL)
            continue;
        nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;

        if (!nfs_fileInfop->nfs_filenameLen)
            continue;
        if ( (nfs_fileInfop->nfs_filenameLen == nameLen  ) &&
                ( !(strncmp(nfs_fileInfop->nfs_fileNamep, namep, nameLen)))  )

        {
            /*
             *found entry
             */
            return (fileCBp);
        }

    }

    return (NULL);
}

/*-----------------------------------------------------------------
 * Name:	nfs_findReqCBByXid 
 * 
 * Description:
 * Find a Request Control Block with a correponding
 * transaction ID (XID). 
 * 
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
nfs_request_t *nfs_findReqCBByXid(uint32 inputXid)
{
	nfp_requestCBPtr_t	nfp_reqCBp;
	nfs_request_t       *nfsReqp;
	int32				count;
//	int32               i, iterations;


	count = 0;
    for (nfp_reqCBp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
        ((count < NFP_MAX_REQUEST) && (nfp_reqCBp != NULL)); count++, nfp_reqCBp++)
    {
		if (!(nfp_reqCBp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		nfsReqp = &nfp_reqCBp->protocol.nfs_req;
		if (nfp_validateReqCB(nfsReqp) != NFX_OK)
			continue;
		if ( (nfsReqp->rpcCallHdrp) &&
		 	(nfsReqp->rpcCallHdrp->rpc_xid ==  inputXid))
			return (nfsReqp);
	}
	return (NULL);
}
/*-----------------------------------------------------------------
 * Name: nfs_findReqCB
 *
 * Description:
 * Find a Request Control Block with a correponding 
 * transaction ID (XID) and
 * procedure number for a client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
nfs_request_t *nfs_findReqCB(
        nfp_clientCBPtr_t clientCBp,    /*Client control block*/
        uint32 xid,                     /*request transaction ID*/
        uint32 procNum)                 /*request procedure number*/
{
    nfs_request_t           *nfsReqp;
    nfp_nodePtr_t           reqp;
    int32                   reqCnt;
    nfp_list_t              *nodeListp;

    nodeListp = &clientCBp->nfp_reqList;
    reqCnt = nodeListp->nfp_reqCnt;
    for (reqp = nodeListp->nfp_headp; (reqp != NULL && reqCnt > 0);
        reqp = reqp->nfp_nextp, reqCnt--)
    {
		if (nfp_validateNode(reqp) != NFX_OK)
			continue;
        nfsReqp = (nfs_request_t *)reqp->nfp_entryCBp;
		if (nfp_validateNfsReqCB(nfsReqp) != NFX_OK)
			continue;
        if ( (nfsReqp->rpcCallHdrp) &&
            (nfsReqp->rpcCallHdrp->rpc_xid == xid) &&
                (nfsReqp->rpcCallHdrp->rpc_proc == procNum))
            return (nfsReqp);
    }

    return (NULL);
}
/*-----------------------------------------------------------------
 * Name: nfs_findReqCBByDesc
 *
 * Description:
 *  Find a Request Control Block with a correponding packet descriptor.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
nfs_request_t *nfs_findReqCBByDesc(
    nfp_clientCBPtr_t clientCBp,    /*Client control block*/
    pkt_desc_t *descp)              /*Pkt descriptor*/
{
//    nfp_nodePtr_t           filep;
    nfp_nodePtr_t           reqp;
//    nfp_fileCBPtr_t         fileCBp;
    nfs_request_t *         nfsReqp;
    int32                   reqCnt;
    pkt_desc_t              *reqPktDescp;
    nfp_list_t              *nodeListp;

    nodeListp = &clientCBp->nfp_reqList;
    reqCnt = nodeListp->nfp_reqCnt;

    for (reqp = clientCBp->nfp_reqList.nfp_headp; (reqp != NULL && reqCnt > 0);
        reqp = reqp->nfp_nextp, reqCnt--)
    {
        nfsReqp = (nfs_request_t *)reqp->nfp_entryCBp;
        reqPktDescp = (pkt_desc_t *)nfsReqp->pktDescp;
        if (reqPktDescp == descp)
        {
            /*
             *return found request entry
             */
            return (nfsReqp);
        }
    }

    return (NULL);
}

#endif  //NFP_TXRX
