/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfsv3-proc.h
 *
 * RCS:  $Id: nfsv3-proc.h,v 1.2 2002/10/04 01:29:25 shaun Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: Contains NFS V3 Procedures function prototypes.
 *
 * Created by: 	Shaun Hui
 *
 * Date Created: 10/01/02
 *
 * History:	8/27/02 - Shaun splitted nfs-proc.c into nfsv3-proc.c
 *					  nfsv2-proc.c
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFSV3_PROC_H
#define _NFSV3_PROC_H

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxNullReq
 *
 * Description: Handle NFS V3 NULL Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxNullReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxGetAttrReq
 *
 * Description: Handle NFS V3 Get Attribute Request.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxGetAttrReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxSetAttrReq
 *
 * Description: Handle NFS V3 Set Attribute Request.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxSetAttrReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxLookupReq
 *
 * Description: Handle NFS V3 Lookup requests
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxLookupReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxAccessReq
 *
 * Description: Handles NFS V3 Access Calls.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxAccessReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxReadLinkReq
 *
 * Description: Handle NFS V3 Readlink Calls.
 *              Procedure READLINK reads the pathname stored in the
 *              symbolic link. The pathname is an ASCII string that
 *              is opague to the server.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxReadLinkReq(    nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxReadReq
 *
 * Description: Handle NFS V3 Read Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxReadReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfs_procRxWriteReq
 *
 * Description: Handles NFS V3 Write Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxWriteReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxCreateReq
 *
 * Description: Handle NFS V3 Create Requests
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxCreateReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:    nfsv3_procRxMkdirReq
 *
 * Description: Handle NFS V3 Mkdir requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxMkdirReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxSymlinkReq
 *
 * Description: Handle NFS V3 Symlink Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxSymlinkReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv3_procRxMkNodReq
 *
 * Description: Handle NFS V3 Mknod requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxMkNodReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: 		nfsv3_procRxRemoveReq
 *
 * Description: Handles NFS V3 Remove Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxRemoveReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxRmdirReq
 *
 * Description: Handle NFS V3 Rmdir Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxRmdirReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxRenameReq
 *
 * Description: Handle NFS V3 Rename Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxRenameReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxLinkReq
 *
 * Description: Handle NFS V3 Link Requests
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxLinkReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxReaddirReq
 *
 * Description: Handle NFS V3 Readdir requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxReaddirReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:    nfs_procRxReaddirPlusReq
 *
 * Description: Handle NFS V3 ReadirPlus Requests
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxReaddirPlusReq( nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxStatfsReq
 *
 * Description: Handle NFS V3 Stats FS Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxStatfsReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxFsinfoReq
 *
 * Description: Handle NFS V3 FSInfo Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxFsinfoReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfs_procRxPathConfReq
 *
 * Description: Handles NFS V3 Pathconf Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxPathConfReq(    nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv3_procRxCommitReq
 *
 * Description: Handle NFS V3 Commit Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv3_procRxCommitReq(      nfs_request_t  *nfsReqp);


#endif /* _NFSV3_PROC_H */
