/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:		nlm-debug.c
 *
 * RCS:  $Id: nlm-debug.c,v 1.7 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: Contain NLM debugging and trace utility functions.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	07/20/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef	NFP_TXRX
#include <nfx-incl.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"

//#define NLM_DBG_PRINTF if(nlm_traceEnable) printf
//#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf

uint32 maxGetportRetries = 0;
uint32 maxGrantedRetries = 0;


static char *nlm_procName[] = {
                        "NULL",
                        "TEST",
                        "LOCK",
                        "CANCEL",
                        "UNLOCK",
                        "GRANTED",
                        "TEST_MSG",
                        "LOCK_MSG",
                        "CANCEL_MSG",
                        "UNLOCK_MSG",
                        "GRANTED_MSG",
                        "TEST_RES",
                        "LOCK_RES",
                        "CANCEL_RES",
                        "UNLOCK_RES",
                        "GRANTED_RES",
                        "NOT_ASSIGNED",
                        "NOT_ASSIGNED",
                        "NOT_ASSIGNED",
                        "NOT_ASSIGNED",
                        "NLM_SHARE",
                        "NLM_UNSHARE",
                        "NLM_NM_LOCK",
                        "NLM_FREE_ALL" };

static boolean nlm_isValidProc[NLM_MAX_PROC] =
{
        TRUE,                       /*0: NLM_NULL*/
        TRUE,                       /*1: NLM_TEST*/
        TRUE,                       /*2: NLM_LOCK*/
        TRUE,                       /*3: NLM_CANCEL*/
        TRUE,                       /*4: NLM_UNLOCK*/
        TRUE,                       /*5: NLM_GRANTED*/
        TRUE,                       /*6: NLM_TEST_MSG*/
        TRUE,                       /*7: NLM_LOCK_MSG*/
        TRUE,                       /*8: NLM_CANCEL_MSG*/
        TRUE,                       /*9: NLM_UNLOCK*/
        TRUE,                      	/*10:NLM_GRANTED_MSG*/
        TRUE,                      	/*11:  NLM_TEST_RES*/
        TRUE,                      	/*12:  NLM_LOCK_RES*/
        TRUE,                      	/*13:  NLM_CANCEL_RES*/
        TRUE,                      	/*14:  NLM_UNLOCK_RES*/
        TRUE,                      	/*15:  NLM_GRANTED_RES*/
        FALSE,                      /*16:  NOT ASSIGNED*/
        FALSE,                      /*17:  NOT ASSIGNED*/
        FALSE,                      /*18:  NOT ASSIGNED*/
        FALSE,                      /*19:  NOT ASSIGNED*/
        TRUE,                       /*20:  NLM_SHARE*/
        TRUE,                       /*21:  NLM_UNSHARE*/
        TRUE,                       /*22:  NLM_NM_LOCK*/
        TRUE,                       /*23:  NLM_FREE_ALL*/
};

//#define NLM_DEBUG

/*-----------------------------------------------------------------
 * Name:		nlm_showFileHandle
 *
 * Description: Display file handle.
 *              
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	07/20/2002
 *
 *-----------------------------------------------------------------
 */
void
nlm_showFileHandle(fs_Handle *fileHandlep, char *indentation)
{
	uint32	fileHandleLen, iteration, i;
	uint32	*bufp;

	if (fileHandlep == NULL)
		return;

	bufp = (uint32	*)fileHandlep;
	fileHandleLen = sizeof(fs_Handle);
	iteration = fileHandleLen >> 2;

	printf("%sfileHandle = ", indentation);
	for(i = 0; i < iteration; i++, bufp++)
	{
		printf("%x ", *bufp);
		if ((i % 4) == 3)
		{
			if (i != iteration -1)
				printf("\n%s             ", indentation);
			else
				printf("\n");
		}
	}
}

/*-----------------------------------------------------------------
 * Name:		nlm_showLock
 *
 * Description: Display lock infomation.
 *              
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	07/20/2002
 *
 *-----------------------------------------------------------------
 */
void
nlm_showLock(nfp_lockCB_t *lockCBp, char *indentation)
{
#ifdef NOT_USED
	nlm_lockRec_t 			*lockRecp;
	nlm_lockInfo_t			*lockInfop;
	int32					numOfLockRec = 0;

	if (lockCBp == NULL)
		return;

	lockInfop = &lockCBp->protocol.nfp_nlmLock;

	numOfLockRec = lockCBp->lockOwnerList.numOfEntries;
	printf("Number of locks in current lock list of LCB(%p): %d\n",
					lockCBp, numOfLockRec);

	DLL_ITERATE(lockRecp, &lockCBp->lockOwnerList.list,	
				lockList)
	{
		if ((lockRecp == NULL) || (lockRecp == DLL_NIL) )
			break;

		printf("%slockRecp = %p\n", indentation, lockRecp);
		printf("%slockRecp flags = %08x\n", indentation, lockRecp->flags);
		printf("%sExclusive = %s\n", indentation,
			(lockRecp->nlm_exclusive == TRUE) ? "TRUE": "FALSE");
		printf("%sCaller = %s\n", 
						indentation, lockInfop->nlm_callerName);
		nlm_showFileHandle(	&lockCBp->nfp_fileCbp->nfp_fileHandle, 
						indentation);
		printf("%sLock owner = %s\n", indentation, lockInfop->nlm_oh);
		printf("%sSvid = %d (process id)\n", indentation, lockInfop->nlm_svid);
		printf("%sOffset = %lld bytes\n", indentation, 
					lockRecp->nlm_l_offset);
		printf("%sLength = %lld bytes\n", indentation, 
					lockRecp->nlm_l_len);
		printf("%sState = %d\n", indentation, lockRecp->nlm_state);

		numOfLockRec--;
	}

	numOfLockRec = lockCBp->lockOwnerPendingList.numOfEntries;

	printf("Number of locks in pending lock list of LCB(%p): %d\n",
					lockCBp, numOfLockRec);

	DLL_ITERATE(lockRecp, &lockCBp->lockOwnerPendingList.list,
				lockList)
	{
		if ((lockRecp == NULL) || (lockRecp == DLL_NIL) )
			break;

		printf("%slockRecp = %p\n", indentation, lockRecp);
		printf("%slockRecp flags = %08x\n", indentation, lockRecp->flags);
		printf("%sExclusive = %s\n", indentation,
			(lockRecp->nlm_exclusive == TRUE) ? "TRUE": "FALSE");
		printf("%sCaller = %s\n", 
						indentation, lockInfop->nlm_callerName);
		nlm_showFileHandle(	&lockCBp->nfp_fileCbp->nfp_fileHandle, 
						indentation);
		printf("%sLock owner = %s\n", indentation, lockInfop->nlm_oh);
		printf("%sSvid = %d (process id)\n", indentation, lockInfop->nlm_svid);
		printf("%sOffset = %lld bytes\n", indentation, 
					lockRecp->nlm_l_offset);
		printf("%sLength = %lld bytes\n", indentation, 
					lockRecp->nlm_l_len);
		printf("%sState = %d\n", indentation, lockRecp->nlm_state);

		numOfLockRec--;
	}
#else
    printf ("Locks not stored with NLM\n");
    return;
#endif
}

/*-----------------------------------------------------------------
 * Name:		nlm_showLockList
 *
 * Description: Display locks in a lock list.
 *              
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	07/20/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef NOT_USED
void
nlm_showLockList(nfp_list_t *lockListp, char *indentation1, char *indentation2)
{
	nfp_lockCB_t	*lockCBp;
	nfp_node_t		*nodep;
	uint32			count = 0;

	nodep = lockListp->nfp_headp;
	while(nodep != NULL)
	{
		lockCBp = (nfp_lockCB_t *)nodep->nfp_entryCBp;
		if (indentation1)
			printf("%sLock %d:\n", indentation1, ++count);
		else
			printf("Lock %d:\n", ++count);
		nlm_showLock(lockCBp, indentation2);

		nodep = nodep->nfp_nextp;
	}
	printf("\n");

}
#endif

/*-----------------------------------------------------------------
 * Name:		nlm_showFileLocks
 *
 * Description: Display locks on a file.
 *              
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	07/20/2002
 *
 *-----------------------------------------------------------------
 */
void
nlm_showFileLocks(nfp_fileCB_t	*fileCBp)
{
#ifdef NOT_USED
	if (fileCBp == NULL)
		return;

	nlm_showLockList(&fileCBp->nfp_lockList, NULL, "\t");
#else
    printf ("Locks not stored with NLM\n");
    return;
#endif
}

/*-----------------------------------------------------------------
 * Name:		nlm_totalPktCnt
 * 
 * Description: Return total packets count on the NLM server.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	07/26/2002
 * 
 *-----------------------------------------------------------------
 */
uint64
nlm_totalPktCnt(uint64 *pktStatp)
{
	uint64	totalPktCnt = 0;

	totalPktCnt = pktStatp[NLM_NULL] +
				  pktStatp[NLM_TEST] +
				  pktStatp[NLM_LOCK] +
				  pktStatp[NLM_CANCEL] +
				  pktStatp[NLM_UNLOCK] +
				  pktStatp[NLM_GRANTED] +
				  pktStatp[NLM_TEST_MSG] +
				  pktStatp[NLM_LOCK_MSG] +
				  pktStatp[NLM_CANCEL_MSG] +
				  pktStatp[NLM_UNLOCK_MSG] +
				  pktStatp[NLM_GRANTED_MSG] +
				  pktStatp[NLM_TEST_RES] +
				  pktStatp[NLM_LOCK_RES] +
				  pktStatp[NLM_CANCEL_RES] +
				  pktStatp[NLM_UNLOCK_RES] +
				  pktStatp[NLM_GRANTED_RES] +
				  pktStatp[NLM_SHARE] +
				  pktStatp[NLM_UNSHARE] +
				  pktStatp[NLM_NM_LOCK] +
				  pktStatp[NLM_FREE_ALL];
	return (totalPktCnt);
}


/*-----------------------------------------------------------------
 * Name:		nlm_showErrorCounters
 * 
 * Description: Display NLM error counters.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	07/26/2002
 * Modified:		09/25/2002		JVu 
 *-----------------------------------------------------------------
 */
void
nlm_showErrorCounters(uint64 *errCntp)
{
	printf("NLM Granted:                     %lld\n", 
					errCntp[NLM_STAT_GRANTED]);
	printf("NLM Err Denied:                  %lld\n", 
					errCntp[NLM_ERR_DENIED]);
	printf("NLM Err Denied No Locks:         %lld\n", 
					errCntp[NLM_ERR_DENIED_NOLOCKS]);
	printf("NLM Err Blocked:                 %lld\n", 
					errCntp[NLM_ERR_BLOCKED]);
	printf("NLM Err Denied Grace Period:     %lld\n", 
					errCntp[NLM_ERR_DENIED_GRACE_PERIOD]);
	printf("NLM Err Deadlock:                %lld\n", 
					errCntp[NLM_ERR_DEADLCK]);
	printf("NLM Err Read-only:               %lld\n", 
					errCntp[NLM_ERR_ROFS]);
	printf("NLM Err Stale FH:                %lld\n", 
					errCntp[NLM_ERR_STALE_FH]);
	printf("NLM Err File Too Big:            %lld\n", 
					errCntp[NLM_ERR_FBIG]);
	printf("NLM Err Generic                  %lld\n", 
					errCntp[NLM_FAILED]);
}

/*-----------------------------------------------------------------
 * Name:		nlm_showDebugCounters
 * 
 * Description: Display NLM debug counters.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	07/26/2002
 * Modified:		09/25/2002		JVu 
 * 
 *-----------------------------------------------------------------
 */
void
nlm_showDebugCounters(nlm_debugCounters_t *dbgCntp)
{
	printf("TPL Xmt alloc pkts:              %lld\n", 
					dbgCntp->nlm_tplXmtPktAlloc);
	printf("TPL Xmt free pkts:               %lld\n", 
					dbgCntp->nlm_tplXmtPktFree);
	printf("TPL Xmt pkts:                    %lld\n", 
					dbgCntp->nlm_tplXmtPktSend);
	printf("TPL Xmt pkts completed:          %lld\n", 
					dbgCntp->nlm_tplXmtPktCmpInd);
	printf("TPL Drop rcvd pkts:              %lld\n", 
					dbgCntp->nlm_tplDropRcvPkt);
	printf("Alloc NLM req CBs:               %lld\n", 
					dbgCntp->nlm_allocReqCB);
	printf("Free NLM req CBs:                %lld\n", 
					dbgCntp->nlm_freeReqCB);
	printf("TPL Xmt Reply no pkt resources:  %lld\n", 
					dbgCntp->nlm_tplXmtNoPktResource);
	printf("Rx pkt indication error:         %lld\n", 
					dbgCntp->nlm_rcvPktIndErr);
	printf("Send unlock reply error:         %lld\n", 
					dbgCntp->nlm_sendUnlockReplyErr);
	printf("Rcv cancel error:                %lld\n", 
					dbgCntp->nlm_rcvCancelErr);
	printf("Rcv unlock error:                %lld\n", 
					dbgCntp->nlm_rcvUnlockErr);
	printf("Num of PKT Alloc Pkt errors:     %lld\n", 
					dbgCntp->nlm_xmtNoRsrcErr);
	printf("Num of alloc callerName bufs:    %lld\n", 
					dbgCntp->nlm_allocCallerName);
	printf("Num of free callerName bufs:     %lld\n", 
					dbgCntp->nlm_freeCallerName);
	printf("Num of alloc OH bufs:            %lld\n", 
					dbgCntp->nlm_allocOh);
	printf("Num of free Oh bufs:             %lld\n", 
					dbgCntp->nlm_freeOh);
	printf("Num of alloc Cookie bufs:        %lld\n", 
					dbgCntp->nlm_allocCookie);
	printf("Num of free Cookie bufs:         %lld\n", 
					dbgCntp->nlm_freeCookie);
	printf("Num of Getport Reqs:             %lld\n", 
					dbgCntp->xmtGetPortReqCnt);
	printf("Num of Getport Reply:            %lld\n", 
					dbgCntp->rxGetPortReplyCnt);
	printf("Num of Tx Getport Errs:          %lld\n", 
					dbgCntp->txGetportErr);
	printf("Num of Getport locate Req Errs:  %lld\n", 
					dbgCntp->rxGetPortLocateReqErr);
	printf("Num of Tx Getport Retries:       %lld\n", 
					dbgCntp->numTxGetportRetries);
	printf("Num of Granted Reqs:             %lld\n", 
					dbgCntp->xmtGrantedReqCnt);
	printf("Num of Granted Reply:            %lld\n", 
					dbgCntp->rxGrantedReplyCnt);
	printf("Num of Tx Granted Errs:          %lld\n", 
					dbgCntp->txGrantedErr);
	printf("Num of Tx Granted Retries:       %lld\n", 
					dbgCntp->numTxGrantedRetries);
	printf("Num of Granted locate Req Errs:  %lld\n", 
					dbgCntp->rxGrantedLocateReqErr);
	printf("Num of Granted Reply Errs:       %lld\n", 
					dbgCntp->rxGrantedErr);
	printf("Num of Rx Bad Cookie Errs:       %lld\n", 
					dbgCntp->numOfRxBadCookies);
	printf("Num of Rx Bad Alock Errs:        %lld\n", 
					dbgCntp->numOfRxBadAlockObj);
	printf("Num of Rx Grace Period Errs:     %lld\n", 
					dbgCntp->numOfRxGracePeriodErr);
	printf("Num of Add NLM ReqCb To List:    %lld\n", 
					dbgCntp->numOfAddNlmReqCbToList);
	printf("Num of Del NLM ReqCb From List:  %lld\n", 
					dbgCntp->numOfDelNlmReqCbFromList);
	printf("Num of Rx Granted Reply Errs:    %lld\n", 
					dbgCntp->nlm_grantedReplyErr);
	printf("Num of Add Pending Lock Errs:    %lld\n", 
					dbgCntp->numOfAddPendingLckErr);
	printf("Num of DeadLock Errs:            %lld\n", 
					dbgCntp->numOfDeadlockErr);
	printf("Num of Unlock Callback Errs:     %lld\n", 
					dbgCntp->numOfCallBackErr);
	printf("Num of Avail Lock Records:       %d\n", 
					nlm_globalCfgp->lockRecFreeCnt);
	printf("Num of Lock Records LWM:         %d\n", 
					nlm_globalCfgp->lockRecLwm );
	printf("Num of Alloc Lock Records:       %lld\n", 
					dbgCntp->numOfAllocLockRec);
	printf("Num of Free Lock Records:        %lld\n", 
					dbgCntp->numOfFreeLockRec);
	printf("Num of Alloc Lock Records Errors:%lld\n", 
					dbgCntp->numOfAllocLockRecErr);
	printf("Num of Lock Record Range Errors :%lld\n", 
					dbgCntp->numOfValidateLockRecRangeErr);
	printf("Num of Lock Record Inused Errors:%lld\n", 
					dbgCntp->numOfValidateLockRecInusedEerr);

#ifdef	NOTUSED
	//Currently NLM does not send any messages
	//to the file system
	printf("Alloc EEE buffers:               %lld\n", 
					dbgCntp->nlm_eeeAlloc);
	printf("Free EEE buffers:                %lld\n", 
					dbgCntp->nlm_eeeDealloc);
	printf("Num of requests sent to FS:      %lld\n", 
					dbgCntp->nlm_reqsXmtToFs);
	printf("Num of responses from FS:        %lld\n", 
					dbgCntp->nlm_rspsRcvFmFs);
	printf("Rx Invalid response from FS:     %lld\n", 
					dbgCntp->nlm_rcvInvalidFSRespErr);
	printf("No FS resource:                  %lld\n", 
					dbgCntp->nlm_fsResourceErr);
	printf("FS error:                        %lld\n", 
					dbgCntp->nlm_fsErr);

#endif	//NOTUSED
}

/*-----------------------------------------------------------------
 * Name:	nlm_showProcStats	
 * 
 * Description: Display NLM procedure statistics.
 *
 * Created by:	Joseph Vu	
 * 
 * Date Created:	07/20/2002
 * Modified:		09/25/2002		JVu 
 * 
 *-----------------------------------------------------------------
 */

void
nlm_showProcStats(void)
{
	int32		proc;
	uint64		total_nlmRxPkt, total_nlmTxPkt;

	printf("NLM PROC                        NLM-RX  NLM-TX\n");
	printf("-------------------------------------------------\n");

	for (proc =NLM_NULL; proc <= NLM_FREE_ALL; proc++)
	{
		if (nlm_isValidProc[proc] == FALSE)
			continue;

		printf("%12s:%21llu  ", nlm_procName[proc], 
				nlm_globalCfgp->nlm_rxPktCounters[proc]);
		printf(" %6llu ",nlm_globalCfgp->nlm_txPktCounters[proc]);

		printf("\n");
	}
	total_nlmRxPkt = nlm_totalPktCnt(nlm_globalCfgp->nlm_rxPktCounters);
	total_nlmTxPkt = nlm_totalPktCnt(nlm_globalCfgp->nlm_txPktCounters);
	printf("       Total:");
	printf("%21llu%9llu\n\n",total_nlmRxPkt, 
				total_nlmTxPkt);

}

/*-----------------------------------------------------------------
 * Name:		nlm_showStat 
 * 
 * Description: Display NLM statistics.
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	07/20/2002
 * Modified:		09/25/2002		JVu 
 * 
 *-----------------------------------------------------------------
 */
void
nlm_showStat()
{
	//show NLM procedure stats

	
	nlm_showProcStats();

	//show  NLM status counters
	printf("NLM Status counters\n");
	printf("------------------\n");
	nlm_showErrorCounters(&nlm_globalCfgp->nlm_statCounters[0]);

	printf("\nNLM debug counters\n");
	printf("------------------\n");
	nlm_showDebugCounters(&nlm_globalCfgp->nlm_debugCounters);


}

/*-----------------------------------------------------------------
 * Name:	nlm_clearStat	
 * 
 * Description: Clear NLM statistics 
 *
 * Created by:	Xiaobing Zhang
 * 
 * Date Created:	07/20/2002
 * 
 *-----------------------------------------------------------------
 */

void
nlm_clearStat(void)
{


	bzero (&nlm_globalCfgp->nlm_rxPktCounters, 
					sizeof(nlm_globalCfgp->nlm_rxPktCounters));
	bzero(&nlm_globalCfgp->nlm_txPktCounters, 
					sizeof(nlm_globalCfgp->nlm_txPktCounters));

	bzero(&nlm_globalCfgp->nlm_statCounters, 
					sizeof(nlm_globalCfgp->nlm_statCounters));

	bzero(&nlm_globalCfgp->nlm_debugCounters, 
					sizeof(nlm_globalCfgp->nlm_debugCounters));
 

}
#endif	//NFP_TXRX
