/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:			nsm-proc.c
 *
 * RCS:  $Id: nsm-proc.c,v 1.3 2002/10/17 01:38:56 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain functions that process NSM requests.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include "nsm.h"

//------------------
// NSM Rx procedures
//------------------
int32 (*nsm_procs[NSM_MAX_PROC]) (nsm_requestCB_t *nsmReqp) =
{
	nsm_procRxNullReq,			/*0: SM_NULL*/
	nsm_procRxStatReq,			/*1: SM_STAT*/
	nsm_procRxMonReq,			/*2: SM_MON*/
	nsm_procRxUnmonReq,			/*3: SM_UNMON*/
	nsm_procRxUnmonAllReq,		/*4: SM_UNMON_ALL*/
	nsm_procRxSimuCrashReq,		/*5: SM_SIMU_CRASH*/
	nsm_procRxNotifyReq,		/*6: SM_NOTIFY*/
};

int32
nsm_procRxMonReq(nsm_requestCB_t *reqCBp)
{
	nsm_globalCfgp->nsm_statCounters.nsm_errNotSupp++;
	return (NFX_ERR);
}

int32
nsm_procRxUnmonReq(nsm_requestCB_t *reqCBp)
{
	nsm_globalCfgp->nsm_statCounters.nsm_errNotSupp++;
	return (NFX_ERR);
}

int32
nsm_procRxUnmonAllReq(nsm_requestCB_t *reqCBp)
{
	nsm_globalCfgp->nsm_statCounters.nsm_errNotSupp++;
	return (NFX_ERR);
}

int32
nsm_procRxSimuCrashReq(nsm_requestCB_t *reqCBp)
{
	nsm_globalCfgp->nsm_statCounters.nsm_errNotSupp++;
	return (NFX_ERR);
}

/*-----------------------------------------------------------------
 * Name:		nsm_procRxNullReq
 *
 * Description:	Process received SM_NULL request from client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procRxNullReq(nsm_requestCB_t *reqCBp)
{
	int32				rc;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procRxNullReq: reqCBp = %p", reqCBp));

	// Send reply
	rc = nsm_procTxSendResp(reqCBp, 0);

	return (rc);
} // nsm_procRxNullReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procRxStatReq
 *
 * Description:	Process received SM_STAT request from client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procRxStatReq(nsm_requestCB_t *reqCBp)
{
	int32				rc;
	nsm_clientCB_t		*clientCBp;
	int32				status;

	clientCBp = reqCBp->clientCBp;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procRxStatReq: reqCBp = %p, clientCBp = %p",
							reqCBp, clientCBp));

	if (clientCBp->status == NSM_CLIENT_STATUS_MON)
		status = NSM_STAT_SUCC;
	else
		status = NSM_STAT_FAIL;

	// Send reply
	rc = nsm_procTxSendResp(reqCBp, status);

	return (rc);
} // nsm_procRxStatReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procRxNotifyReq
 *
 * Description:	Process received SM_NOTIFY request from client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procRxNotifyReq(nsm_requestCB_t *reqCBp)
{
	int32				rc = NFX_OK;
	nsm_clientCB_t		*clientCBp;
	evm_volId_t			dummyVolId;

	clientCBp = reqCBp->clientCBp;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procRxNotifyReq: reqCBp = %p, clientCBp = %p",
							reqCBp, clientCBp));

	if (clientCBp->status == NSM_CLIENT_STATUS_MON)
	{
		//----------------------------------------
		// Add a msgToThreadCB to the waiting list
		//----------------------------------------
		dummyVolId.whole_thing = 0;
		rc = nsm_addMsgToThreadCB(clientCBp->hostName,
								  0,
								  dummyVolId,
								  clientCBp,
								  NULL,
								  reqCBp,
								  NSM_MSG_NOTIFY);
		if (rc == NFX_ERR)
			return (rc);

		//------------------------------------
		// Send an event to the thread, 
		// if no message is sent to thread yet
		//------------------------------------
		if (nsm_firstMsgToThread == TRUE)
			rc = nsm_msgToThread();

		// CALL NLM to inform it the client is rebooting.
		nsm_globalCfgp->nlmCallbackFunc.clientDownInd(clientCBp->hostIpAddr);
	}
	else
	{
		// We are not monitoring the client that sent the SM_NOTIFY
		// request to us, but still send the reply so that the client
		// won't repeatedly send us the request.
		rc = nsm_procTxNotifyResp(reqCBp);
	}

	return (rc);
} // nsm_procRxNofityReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procMonReq
 *
 * Description:	Establish monitoring of a given host.
 *              Called by NLM module.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procMonReq(char				*hostName,
			   uint32			hostIpAddr,
			   evm_volId_t		volId,
			   nfp_clientCB_t	*nlm_clientCBp)		//NLM clientCB
{
	int32				rc;
	nsm_clientCB_t  	*clientCBp;
	nfp_node_t      	*nfpNodep;

	clientCBp = nsm_getClientCB(hostIpAddr);
	if (clientCBp != NULL)
	{
		//--------------------------------------
		// Check if volume is in the 
		//volume list of the clientCB
		//--------------------------------------
		nfpNodep = nsm_findVolumeCBNode(&clientCBp->volumeList, 
										volId);
		if (nfpNodep != NULL)
		{
			// The client has already been 
			// monitored on this volume
			return (NFX_OK);
		}

	}

	//----------------------------------------
	// Add a msgToThreadCB to the waiting list
	//----------------------------------------
	rc = nsm_addMsgToThreadCB(hostName,
							  hostIpAddr,
							  volId,
							  NULL,
							  nlm_clientCBp,
							  NULL,
							  NSM_MSG_MON);
	if (rc == NFX_ERR)
		return (rc);

	//------------------------------------
	// Send an event to the thread, 
	// if no message is sent to thread yet
	//------------------------------------
	if (nsm_firstMsgToThread == TRUE)
		rc = nsm_msgToThread();

	return (rc);
} // nsm_procMonReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procUnmonReq
 *
 * Description:	Stop monitoring a given host.
 *              Called by NLM module.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/16/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procUnmonReq(char			*hostName,
				 uint32			hostIpAddr,
				 evm_volId_t	volId,
				 nfp_clientCB_t	*nlm_clientCBp)		//NLM clientCB
{
	int32				rc;

	//----------------------------------------
	// Add a msgToThreadCB to the waiting list
	//----------------------------------------
	rc = nsm_addMsgToThreadCB(hostName,
							  hostIpAddr,
							  volId,
							  NULL,
							  nlm_clientCBp,
							  NULL,
							  NSM_MSG_UNMON);
	if (rc == NFX_ERR)
		return (rc);

	//------------------------------------
	// Send an event to the thread, 
	// if no message is sent to thread yet
	//------------------------------------
	if (nsm_firstMsgToThread == TRUE)
		rc = nsm_msgToThread();

	return (rc);
} // nsm_procUnmonReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procTxGetPortReq
 *
 * Description:	Sends a Getport command to client to get the NSM port.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procTxGetPortReq(nsm_requestCB_t *reqCBp)
{
	int32			rc;
	pkt_desc_t		*descp;
	char			*txMsgp;
	int32			txMsgLen;
	uint32			raddr;
	ushort16		rport;

	txMsgLen = 0;

	descp = PKT_ALLOC_PACKET(1);
	if (!descp)
	{
		nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtNoPktRsrcErr++;
		return (NFX_ERR);
	}

	//-----------------------------------
	// Prepare portmapper request message
	//-----------------------------------

	txMsgp = (char *)descp->pHdr;

	// Set up portmapper RPC header 
	txMsgLen =  nlm_msgRpcCallHdr(
							txMsgp,
							reqCBp->txCmdInfo.cmdXid,
							RPC_PORTMAP_PROG_NUM,
							RPC_PORTMAPPER_VER2,
							RPC_PORTMAPPER_GETPORT);	
	
	txMsgp += txMsgLen;

	// RPC NSM Program Num 
	*(uint32 *)txMsgp = RPC_NSM_PROG_NUM;
	txMsgp += 4;
	txMsgLen += 4;

	// RPC NSM Program Ver
	*(uint32 *)txMsgp = NSM_VER1;
	txMsgp += 4;
	txMsgLen += 4;

	// Transport protocol 
	*(uint32 *)txMsgp = TPL_PROT_UDP;
	txMsgp += 4;
	txMsgLen += 4;

	*(uint32 *)txMsgp = 0;			//port 0	
	txMsgp += 4;
	txMsgLen += 4;

	descp->hdrSize = txMsgLen;

	// Host to Net conversion
	xdr_swap(descp->pHdr, descp->hdrSize, 1);

	//-----------------------------------
	// Send Getport Req through slow path
	//-----------------------------------
	raddr = 0;
	rport = 0;
	if (reqCBp->txCmdInfo.pBsdSock == NULL)
	{
		// Use server socket
		raddr = reqCBp->txCmdInfo.raddr;
		rport = htons(reqCBp->txCmdInfo.rport);
		reqCBp->txCmdInfo.pBsdSock = nsm_globalCfgp->pmap_pBsdSock;
	}

	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxGetPortReq: "
					"nsm_pBsdSock: %p, raddr: %08x, port: %d, hdrSize: %d", 
					reqCBp->txCmdInfo.pBsdSock, reqCBp->txCmdInfo.raddr,
					reqCBp->txCmdInfo.rport, descp->hdrSize));

#if defined(NFX_SMP)
	rc = tpl_xmtPktSlow(raddr, rport, descp,
						reqCBp->txCmdInfo.pBsdSock, 0);
	
	/* pkt is freed on ncpu */
#else
	rc = bsd_SendSock(reqCBp->txCmdInfo.pBsdSock, 
					  raddr, 
					  rport, 
					  descp->pHdr, 
					  BSD_BUF_FLAT, 
					  descp->hdrSize, 
					  0);

	PKT_FREE_PACKET(descp);
#endif /* NFX_SMP */

	if (rc != NFX_OK)
		nsm_globalCfgp->nsm_debugCounters.nsm_txGetPortReqPktErr++;
	else
		nsm_globalCfgp->nsm_debugCounters.nsm_txGetPortReqPktCnt++;

	return (rc);
} // nsm_procTxGetPortReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procTxNotifyReq
 *
 * Description:	Sends a SM_NOTIFY to client after the NLM server reboots.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procTxNotifyReq(nsm_requestCB_t *reqCBp)
{
	int32				rc;
	nsm_clientCB_t		*clientCBp;
	pkt_desc_t			*descp;
	char				*txMsgp;
	int32				length, txMsgLen;
	uint32				raddr;
	ushort16			rport;

	clientCBp = reqCBp->clientCBp;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxNotifyReq: "
			"reqCBp = %p, clientCBp = %p, clientCBp->notifyReqp = %p",
			reqCBp, clientCBp, clientCBp->notifyReqp));

	descp = PKT_ALLOC_PACKET(1);
	if (!descp)
	{
		nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtNoPktRsrcErr++;
		return NFX_ERR;
	}

	//----------------------------------
	// Prepare SM_NOTIFY request message
	//----------------------------------

	txMsgp = (char *)descp->pHdr;

	// Set up NSM RPC header 
	txMsgLen =  nlm_msgRpcCallHdr(
							txMsgp,
							reqCBp->txCmdInfo.cmdXid,
							RPC_NSM_PROG_NUM,
							NSM_VER1,
							NSM_NOTIFY);
	txMsgp += txMsgLen;

	// Store NSM part
	
	length = nfs_storeString(txMsgp,
						 	clientCBp->hostName,
						 	clientCBp->nameLen,
						 	NSM_MAX_NAME);
	txMsgLen += length;
	txMsgp += length;

	// Store state
	length = nfs_storeLongValToBuf(txMsgp, nsm_globalCfgp->state);
	txMsgLen += length;
	txMsgp += length;

	descp->hdrSize = txMsgLen;

	// For debugging, display SM_NOTIFY request message
	if (nsm_traceEnable != 0)
		nsm_decodeReqPkt(descp, NSM_DECODE_TX);

	// Host to Net conversion
	xdr_swap(descp->pHdr, descp->hdrSize, 1);

	reqCBp->rpcCallHdrp = (rpcCallHdr_t *)descp->pHdr;
	reqCBp->txPktDescp = descp;

	//-----------------------------------
	// Send notify Req through slow path
	//-----------------------------------
	raddr = 0;
	rport = 0;
	if (reqCBp->txCmdInfo.pBsdSock == NULL)
	{
		// Use server socket
		raddr = reqCBp->txCmdInfo.raddr;
		rport = htons(reqCBp->txCmdInfo.rport);
		reqCBp->txCmdInfo.pBsdSock = nsm_globalCfgp->nsm_pBsdSock;
	}

	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxNotifyReq: "
					"nsm_pBsdSock: %p, raddr: %08x, port: %d, hdrSize: %d\n", 
					reqCBp->txCmdInfo.pBsdSock, reqCBp->txCmdInfo.raddr,
					reqCBp->txCmdInfo.rport, descp->hdrSize));
#if defined (NFX_SMP)
	rc = tpl_xmtPktSlow( raddr, rport, descp,
						 reqCBp->txCmdInfo.pBsdSock, 1);
#else
	rc = bsd_SendSock(reqCBp->txCmdInfo.pBsdSock, 
					  raddr,
					  rport,
					  descp->pHdr, 
					  BSD_BUF_FLAT, 
					  descp->hdrSize, 
					  0);
#endif /* NFX_SMP */

	if (rc != NFX_OK)
		nsm_globalCfgp->nsm_debugCounters.nsm_txNotifyReqPktErr++;
	else
		nsm_globalCfgp->nsm_debugCounters.nsm_txNotifyReqPktCnt++;

	//PKT_FREE_PACKET(descp);

	return (rc);
} // nsm_procTxNotifyReq()

/*-----------------------------------------------------------------
 * Name:		nsm_procTxNotifyResp
 *
 * Description:	Send SM_NOTIFY response message to client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		//NFX_OK or NFX_ERR
nsm_procTxNotifyResp(nsm_requestCB_t *reqCBp)
{
	int32	rc;

	// Send reply
	rc = nsm_procTxSendResp(reqCBp, 0);

	return (rc);
} // nsm_procTxNotifyResp()

/*-----------------------------------------------------------------
 * Name:		nsm_procTxPkt  
 *
 * Description: Transmit a NSM reply pkt to the client.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/22/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_procTxPkt(pkt_desc_t *tplXmitPktDescp, nsm_requestCB_t *reqCBp)
{
	int32		rc;

	if ((tplXmitPktDescp == NULL) || (reqCBp == NULL ))
	{
		return (NFX_ERR);
	}

	if (nsm_traceEnable != 0)
		nsm_decodeRspPkt(tplXmitPktDescp, reqCBp);

	if (reqCBp->reqState != NSM_REQ_DONE)
	{
		// Don't swap RPC/NFS headers for rexmit pkts 
		// since it had already been swapped.
		xdr_swap(tplXmitPktDescp->pHdr, tplXmitPktDescp->hdrSize, 1);
	}

	reqCBp->reqState = NSM_REQ_REPLIED;

	nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtPktSend++;

#ifdef NSM_UNIT_TEST
	printf("nsm_procTxPkt: Call tpl_xmtPkt to send packet out\n");
	rc = NFX_OK;
#else
	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxPkt: reqCBp->tplConnp = %p",
													reqCBp->tplConnp));
	rc = tpl_xmtPkt(reqCBp->tplConnp, tplXmitPktDescp);
	if (rc != NFX_OK)
	{
		nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtPktSendErr++;
	}
#endif //NSM_UNIT_TEST

	return (rc);
} // nsm_procTxPkt()

/*-------------------------------------------------------------------
 * Name:		nsm_procTxSendResp  
 *  
 * Description:	This function is called to send a response 
 *              to a client's request. 
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-------------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_procTxSendResp(nsm_requestCB_t *reqCBp, int32 status)
{
	rpcCallHdr_t		*rpcCallp;
	pkt_desc_t			*tplXmitPktDescp;
	char				*txMsgp;
	int32				txMsgLen;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxSendResp: "
					"reqCBp= %p, reqCBp->rpcCallHdrp = %p",
					reqCBp, reqCBp->rpcCallHdrp));

	if ((reqCBp == NULL) || (reqCBp->rpcCallHdrp == NULL))
		return (NFX_ERR);

	rpcCallp = reqCBp->rpcCallHdrp;

#ifdef NSM_UNIT_TEST
	{
		pkt_desc_t	nsmtestDesc;
		char		nsmtestMsg[2048];

		nsmtestDesc.pHdr = nsmtestMsg;
		tplXmitPktDescp = &nsmtestDesc;
	}
#else
	// Allocate TPL packet to send
	tplXmitPktDescp = TPL_ALLOC_PACKET(reqCBp->tplConnp, 0);
	if (tplXmitPktDescp == NULL)
	{
		nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtNoPktRsrcErr++;
		return (NFX_ERR);
	}
	nsm_globalCfgp->nsm_debugCounters.nsm_tplXmtPktAlloc++;
#endif //NSM_UNIT_TEST

	// Save NSM req in appInfo for callback complete Ind.
	tplXmitPktDescp->appInfo = reqCBp;
	reqCBp->txPktDescp = (void *)tplXmitPktDescp;

	NSM_PRINTF(LOG_DEBUG, ("nsm_procTxSendResp: reqCBp->txPktDescp = %p",
														reqCBp->txPktDescp));

	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;

	// Start with RPC reply header
	txMsgLen += nfs_msgRpcReply(txMsgp, rpcCallp->rpc_xid,
								RPC_REPLY_MSG_ACCEPTED, RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;

	// Special for SM_STAT request
	if (rpcCallp->rpc_proc == NSM_STAT)
	{
		// Set return status
		*(uint32 *)txMsgp = status;

		txMsgp += 4;
		txMsgLen += 4;

		// Set status monitor state
		*(uint32 *)txMsgp = nsm_globalCfgp->state;

		txMsgp += 4;
		txMsgLen += 4;
	}

	//Set packet desc hdr size
	tplXmitPktDescp->hdrSize = txMsgLen;

	if (nsm_procTxPkt(tplXmitPktDescp, reqCBp) != NFX_OK)
	{
#ifdef NOTUSED
		// Set reqState to DONE so that 
		// nsm_reqTimeoutHandler() will free reqCB
		reqCBp->reqState = NSM_REQ_DONE;
#endif //NOTUSED
		return (NFX_ERR);
	}

	nsm_globalCfgp->nsm_debugCounters.nsm_txRspPktCnt[rpcCallp->rpc_proc]++;

	return (NFX_OK);
} // nsm_procTxSendResp()

/*-------------------------------------------------------------------
 * Name:		nsm_procTxSendErrResp  
 *  
 * Description:	This function is called to send an error response 
 *              to a client's request. 
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-------------------------------------------------------------------
 */
int32
nsm_procTxSendErrResp(nsm_requestCB_t	*reqCBp,		//Ptr to NSM request
					  int32				status)			//Error status
{
	NSM_PRINTF(LOG_INFO, ("nsm_procTxSendErrResp: status: %d", status));

	return (nsm_procTxSendResp(reqCBp, status));
} // nsm_procTxSendErrResp()

#endif	//NFP_TXRX
