/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:			nsm-test.c
 *
 * RCS:  $Id: nsm-test.c,v 1.1.1.1 2002/09/13 01:05:53 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain NSM unit test functions.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	08/26/2002
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include "nsm.h"

char nsm_TxtDumpMem[]="         0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F  0123456789ABCDEF\n";

void nsm_memDump(void *address, uint32 size)
{
int i, j;
uint32 s, m;

  s = 0;
  printf(nsm_TxtDumpMem);
  while (s < size)
  {
    if (s + 16 < size) j = 16;
    else j = size - s;
    printf("%p ", address);
    for (i = 0; i < j; i++)
      printf("%02x ", *(((uchar8*)address)+i));
    if (j < 16) for (i = j; i < 16; i++) printf("   ");
    for (i = 0; i < j; i++) {
      m = *(((uchar8*)address)+i);
      if (m >= 0x20 && m < 0x7f) printf("%c", m);
      else printf(".");
    }
    printf("\n");
    address += j;
    s += j;
  }
}


#ifdef NSM_UNIT_TEST

evm_volId_t		nsmtest_volId1, nsmtest_volId2;
char			nsmtest_volName1[VOL_MAX_NAME + 1];
char			nsmtest_volName2[VOL_MAX_NAME + 1];

nfp_clientCB_t	*nsmtest_nlmClientCBp1, *nsmtest_nlmClientCBp2;
nfp_clientCB_t	*nsmtest_nlmClientCBp3, *nsmtest_nlmClientCBp4;

uint32			nsmtest_hostIpAddr1, nsmtest_hostIpAddr2;
uint32			nsmtest_hostIpAddr3, nsmtest_hostIpAddr4;

char			nsmtest_hostName1[NSM_MAX_NAME + 1];
char			nsmtest_hostName2[NSM_MAX_NAME + 1];
char			nsmtest_hostName3[NSM_MAX_NAME + 1];
char			nsmtest_hostName4[NSM_MAX_NAME + 1];

pkt_desc_t		nsmtest_pkt;
char			nsmtest_msgBuf[2048];

int32
nsmtest_monCmpInd(evm_volId_t        volId,
				  nfp_clientCB_t     *nlm_clientCBp,
				  int32              status)
{
	printf("nsmtest_monCmpInd: Finished processing monitor request\n");
	printf("nsmtest_monCmpInd: volId.whole_thing = 0x%llx, nlm_clientCBp = %p, status = %d\n", volId.whole_thing, nlm_clientCBp, status);
	return NFX_OK;
}

int32
nsmtest_unmonCmpInd(evm_volId_t        volId,
				    nfp_clientCB_t     *nlm_clientCBp,
					int32              status)
{
	printf("nsmtest_unmonCmpInd: Finished processing unmonitor request\n");
	printf("nsmtest_unmonCmpInd: volId.whole_thing = 0x%llx, nlm_clientCBp = %p, status = %d\n", volId.whole_thing, nlm_clientCBp, status);
	return NFX_OK;
}

int32
nsmtest_clientDownInd(uint32	hostIpAddr)
{
	printf("nsmtest_clientDownInd: Finished client Notify request\n");
	printf("nsmtest_clientDownInd: hostIpAddr = 0x%x\n", hostIpAddr);
	return NFX_OK;
}

void
nsmtest_init()
{
	int32					rc;
	nsm_nlmCallbackCB_t		callbackCB;
	

	//------------------
	// Register callback
	//------------------
	callbackCB.monCmpInd = nsmtest_monCmpInd;
	callbackCB.unmonCmpInd = nsmtest_unmonCmpInd;
	callbackCB.clientDownInd = nsmtest_clientDownInd;	
	nsm_registerCallback(&callbackCB);

	//-------------------
	// Init 3 nlm clients
	//-------------------

	// Client 1
	nsmtest_nlmClientCBp1 = nfp_allocClientCB();
	if (nsmtest_nlmClientCBp1 == NULL)
	{
		printf("nsmtest_init: nfp_allocClientCB() failed!\n");
		return;
	}
	//nsmtest_hostIpAddr1 = 0x4e02020a;			//10.2.2.78
	nsmtest_hostIpAddr1 = 0x4e02030a;			//10.3.2.78
	strcpy(nsmtest_hostName1, "client1@agilestorage.lab");

	// Client 2
	nsmtest_nlmClientCBp2 = nfp_allocClientCB();
	if (nsmtest_nlmClientCBp2 == NULL)
	{
		printf("nsmtest_init: nfp_allocClientCB() failed!\n");
		return;
	}
	nsmtest_hostIpAddr2 = 0xb0a020a;			//10.2.10.11
	strcpy(nsmtest_hostName2, "client2@agilestorage.lab");

	// Client 3
	nsmtest_nlmClientCBp3 = nfp_allocClientCB();
	if (nsmtest_nlmClientCBp3 == NULL)
	{
		printf("nsmtest_init: nfp_allocClientCB() failed!\n");
		return;
	}
	nsmtest_hostIpAddr3 = 0x10b020a;			//10.2.11.1
	strcpy(nsmtest_hostName3, "client3@agilestorage.lab");

	//---------------------------------
	// Monitor the 3 hosts on 2 volumes
	//---------------------------------

	rc = nsm_procMonReq(nsmtest_hostName1,
						nsmtest_hostIpAddr1,
						nsmtest_volId1,
						nsmtest_nlmClientCBp1);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName1, nsmtest_hostIpAddr1, nsmtest_volName1);

	rc = nsm_procMonReq(nsmtest_hostName1,
						nsmtest_hostIpAddr1,
						nsmtest_volId2,
						nsmtest_nlmClientCBp1);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName1, nsmtest_hostIpAddr1, nsmtest_volName2);

	/*
	rc = nsm_procMonReq(nsmtest_hostName2,
						nsmtest_hostIpAddr2,
						nsmtest_volId1,
						nsmtest_nlmClientCBp2);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName2, nsmtest_hostIpAddr2, nsmtest_volName1);

	rc = nsm_procMonReq(nsmtest_hostName2,
						nsmtest_hostIpAddr2,
						nsmtest_volId2,
						nsmtest_nlmClientCBp2);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName2, nsmtest_hostIpAddr2, nsmtest_volName2);

	rc = nsm_procMonReq(nsmtest_hostName3,
						nsmtest_hostIpAddr3,
						nsmtest_volId1,
						nsmtest_nlmClientCBp3);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName3, nsmtest_hostIpAddr3, nsmtest_volName1);

	rc = nsm_procMonReq(nsmtest_hostName3,
						nsmtest_hostIpAddr3,
						nsmtest_volId2,
						nsmtest_nlmClientCBp3);
	printf("nsmtest_init: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName3, nsmtest_hostIpAddr3, nsmtest_volName2);
	*/

	return;
} // nsmtest_init()

void
nsmtest_mon()
{
	int32		rc;

	// Client 4
	nsmtest_nlmClientCBp4 = nfp_allocClientCB();
	if (nsmtest_nlmClientCBp4 == NULL)
	{
		printf("nsmtest_mon: nfp_allocClientCB() failed!\n");
		return;
	}
	nsmtest_hostIpAddr4 = 0x20f020a;			//10.2.15.2
	strcpy(nsmtest_hostName4, "client4@agilestorage.lab");

	// Monitor request
	rc = nsm_procMonReq(nsmtest_hostName4,
						nsmtest_hostIpAddr4,
						nsmtest_volId1,
						nsmtest_nlmClientCBp4);
	printf("nsmtest_mon: rc = %d. Monitoring %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName4, nsmtest_hostIpAddr4, nsmtest_volName1);

	return;
} // nsmtest_mon()

void
nsmtest_unmon()
{
	int32		rc;

	// Un-monitor client 2 on volId1
	rc = nsm_procUnmonReq(nsmtest_hostName2,
						  nsmtest_hostIpAddr2,
						  nsmtest_volId1,
						  nsmtest_nlmClientCBp2);
	printf("nsmtest_mon: rc = %d. Unmonitor %s (0x%x) on volume %s\n",
				rc, nsmtest_hostName2, nsmtest_hostIpAddr2, nsmtest_volName1);

	return;
} // nsmtest_unmon()

char nsmtest_notifyReqMsg[] = { //client1@agilestorage.lab
0x3d, 0x2d, 0x94, 0xff, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x02, 0x00, 0x01, 0x86, 0xb8,
0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x06,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,

0x00, 0x00, 0x00, 0x18, 0x63, 0x6c, 0x69, 0x65,
0x6e, 0x74, 0x31, 0x40, 0x61, 0x67, 0x69, 0x6c,
0x65, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x67, 0x65,
0x2e, 0x6c, 0x61, 0x62, 0x00, 0x00, 0x00, 0x23
};

void
nsmtest_simMsg(char *msg, ushort16 msgLen)
{
	nsmtest_pkt.pHdr = nsmtest_msgBuf;
	nsmtest_pkt.hdrSize = msgLen;
	memcpy(nsmtest_msgBuf, msg, msgLen);

	nsmtest_pkt.flags |= PKT_DESC_INUSE;
	return;
};

void
nsmtest_notify()
{
	tpl_bindCb_t	*pBind = 0x1;
	void			*connIdp = 0x1;
	void			*appHdlpp;
	uint32			raddr = nsmtest_hostIpAddr1;
	ushort16		rport;

	printf("nsmtest_notify: Simulating SM_NOTIFY\n");

	nsmtest_simMsg(nsmtest_notifyReqMsg, sizeof(nsmtest_notifyReqMsg));

	nsm_tplAddConnInd(pBind, connIdp, &appHdlpp, raddr, rport);

	nsm_tplRcvPktInd(appHdlpp, &nsmtest_pkt);
	
	return;
}

char nsmtest_notifyRespMsg[] = {
0x00, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x01,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06,
};

void
nsmtest_notifyResp()
{
	tpl_bindCb_t	*pBind = 0x1;
	tpl_connCb_t	connCB;
	void			*connIdp;
	void			*appHdlpp;
	uint32			raddr = 0x4e02030a;           //10.3.2.78
	ushort16		rport = ntohs(32768);

	nsmtest_simMsg(nsmtest_notifyRespMsg, sizeof(nsmtest_notifyRespMsg));

	connIdp = &connCB;
	nsm_tplAddConnInd(pBind, connIdp, &appHdlpp, raddr, rport);

	nsm_tplRcvPktInd(appHdlpp, &nsmtest_pkt);
	
	return;
}

#endif //NSM_UNIT_TEST
#endif	//NFP_TXRX
