/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : opt.h
 *
 * Description  : Main header file of the option module.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 03-SEP-2001
 *
 *------------------------------------------------------------------------------
 */
#ifndef _OPT_H_
#define _OPT_H_

/*------------------------------------------------------------------------------
 * Include files
 *------------------------------------------------------------------------------
 */

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()

#include "nfx-incl.h"
#include "opt-api.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"

/*------------------------------------------------------------------------------
 * Constants and macros.
 *------------------------------------------------------------------------------
 */

// Debuggage et. al.

#undef REGM_DEBUG_MSGS
#undef REGM_FCT_MSGS
#define REGM_ERR_MSGS

#if defined (REGM_DEBUG_MSGS)
#define OPT_DBG_PRINT(x) E_LOG (class_1, debug_s, 69, 0, 0, 0, x)
#else
#define OPT_DBG_PRINT(x)
#endif

#if defined (REGM_FCT_MSGS)
#define OPT_FCT_PRINT(x) E_LOG (class_1, info_s, 69, 0, 0, 0, x)
#else 
#define OPT_FCT_PRINT(x)
#endif

#if defined (REGM_ERR_MSGS)
#define OPT_ERR_PRINT(x) E_LOG (class_1, err_s, 69, 0, 0, 0, x)
#else
#define OPT_ERR_PRINT(x)
#endif

// Wait for EVM to respond (10s)
//static struct timeval waitRegmTo = {10, 0};

/*------------------------------------------------------------------------------
 * Constants and macros
 *------------------------------------------------------------------------------
 */

#define MAX_ENTRY           512 // to be optimized

/*------------------------------------------------------------------------------
 * Type definitions.
 *------------------------------------------------------------------------------
 */

/* Info needed to reply to a sender.
 */
typedef struct
{
	char * name;
	void (* cmd) (char * option, char * value);
} opt_cmd_t;

/* Info needed to reply to a sender.
 */
typedef struct
{
    char    name[256];
    uint32  cpu;
    uint32  slot;
    uint32  plane;
} opt_senderInfo_t;

/* Basic message structure.
 */
typedef struct
{
    char    buf[MAX_ENTRY * (OPT_MAX_OPTION_LEN + MAX_VALUE_LEN)];
    uint32  len;
} opt_msg_t;

/*----------------------------------------------------------------------------*/

#endif // _OPT_H_
