#ifndef __DISKIO_H
#define __DISKIO_H

typedef int off_t;
typedef unsigned int u_int;
typedef unsigned int daddr_t;
typedef int32 int32_t;
typedef uint32 u_int32_t;
typedef uint64 u_int64_t;
typedef int64 int64_t;
typedef uint16 u_int16_t;
typedef int16 int16_t;
typedef unsigned int size_t;
typedef	u_int32_t	ino_t;		/* inode number */
typedef int time_t;
typedef char8 int8_t;
typedef uchar8 u_int8_t;
typedef unsigned char u_char;
typedef unsigned long u_long;
typedef int dev_t;
typedef int mode_t;
typedef int nlink_t;
typedef int uid_t;
typedef int gid_t;
typedef int ssize_t;

#define __P(x) x

#define MAXFRAG 8
#define MAXPATHLEN 256
#define MAXBSIZE (64*1024)
#define MAXSYMLINKS 32

/* where values for lseek(2) */
#define	SEEK_SET	0	/* set file offset to offset */
#define	SEEK_CUR	1	/* set file offset to current plus offset */
#define	SEEK_END	2	/* set file offset to EOF plus offset */

struct stat {
	dev_t	  st_dev;		/* inode's device */
	ino_t	  st_ino;		/* inode's number */
	mode_t	  st_mode;		/* inode protection mode */
	nlink_t	  st_nlink;		/* number of hard links */
	uid_t	  st_uid;		/* user ID of the file's owner */
	gid_t	  st_gid;		/* group ID of the file's group */
	dev_t	  st_rdev;		/* device type */
//#ifndef _POSIX_SOURCE
#if 0	
	struct	timespec st_atimespec;	/* time of last access */
	struct	timespec st_mtimespec;	/* time of last data modification */
	struct	timespec st_ctimespec;	/* time of last file status change */
#else
	time_t	  st_atime;		/* time of last access */
	long	  st_atimensec;		/* nsec of last access */
	time_t	  st_mtime;		/* time of last data modification */
	long	  st_mtimensec;		/* nsec of last data modification */
	time_t	  st_ctime;		/* time of last file status change */
	long	  st_ctimensec;		/* nsec of last file status change */
#endif
	off_t	  st_size;		/* file size, in bytes */
	int64_t	  st_blocks;		/* blocks allocated for file */
	u_int32_t st_blksize;		/* optimal blocksize for I/O */
	u_int32_t st_flags;		/* user defined flags for file */
	u_int32_t st_gen;		/* file generation number */
	int32_t	  st_lspare;
	int64_t	  st_qspare[2];
};


struct open_file {
	int		f_flags;	/* see F_* below */
	struct devsw	*f_dev;		/* pointer to device operations */
	void		*f_devdata;	/* device specific data */
	struct fs_ops	*f_ops;		/* pointer to file system operations */
	void		*f_fsdata;	/* file system specific data */
	off_t		f_offset;	/* current file offset (F_RAW) */
};

/* Device switch */
struct devsw {
	char	*dv_name;
	int	(*dv_strategy) __P((void *devdata, int rw,
				    daddr_t blk, size_t size,
				    void *buf, size_t *rsize));
	int	(*dv_open) __P((struct open_file *f, ...));
	int	(*dv_close) __P((struct open_file *f));
	int	(*dv_ioctl) __P((struct open_file *f, u_long cmd, void *data));
};


#define	SOPEN_MAX	4
extern struct open_file files[];

/* f_flags values */
#define	F_READ		0x0001	/* file opened for reading */
#define	F_WRITE		0x0002	/* file opened for writing */
#define	F_RAW		0x0004	/* raw device open - no file system */
#define F_NODEV		0x0008	/* network open - no device */

/*
 * This structure is used to define file system operations in a file system
 * independent way.
 */
struct fs_ops {
	int	(*open) __P((char *path, struct open_file *f));
	int	(*close) __P((struct open_file *f));
	int	(*read) __P((struct open_file *f, void *buf,
			     size_t size, size_t *resid));
	int	(*write) __P((struct open_file *f, void *buf,
			     size_t size, size_t *resid));
	off_t	(*seek) __P((struct open_file *f, off_t offset, int where));
	int	(*stat) __P((struct open_file *f, struct stat *sb));
	int	(*readdir) __P((struct open_file *f, char *));
};

extern struct fs_ops file_system[];
extern int nfsys;


int diostrategy(void *devdata, int rv, daddr_t bn, unsigned reqcnt,
				char *addr, unsigned *cnt);
int dioopen(struct open_file *f, ...);
int dioclose(struct open_file *f);
int pmon_lseek (int, off_t, int);
int pmon_read (int, char *, int);
int pmon_open (char *, int, ...);
int devopen(struct open_file *f, const char *fname, char **file);


#define	ENXIO		6			/* Device not configured */
#define EIO 		5			/* Input/output error */
#define	EFBIG		27			/* File too large */
#define	ENOENT		2			/* No such file or directory */
#define	EINVAL		22		/* Invalid argument */
#define	ENOTDIR		20		/* Not a directory */
#define	EROFS		30		/* Read-only file system */
#define	EMFILE		24		/* Too many open files */
#define	EBADF		9		/* Bad file descriptor */


int ufs_open(char *path, struct open_file *f);

/* bytes to disk blocks */
#define DEV_BSHIFT 9

#define	btodb(x)	((x) >> DEV_BSHIFT)
#define dbtob(x)	((x) << DEV_BSHIFT)



#endif
