/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ppm-api.c
 *
 * RCS:  $Id: ppm-api.c,v 1.5 2001/12/18 20:43:38 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: this module take care of control plane messaging
 *				between txrx and ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "exports/eee-api.h"
#include "exports/cm-api.h"
#include "exports/ppm-api.h"

extern ppmc_ctrl_t		ppm_cb[];
extern int				ppm_appid;
extern uint32			ppmeee_src;

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_RegRcv
 *
 * Description: register with txrx control. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_RegRcv(int		gid,	/* group id */
		   void*		usrdata,
		   ppm_func		func	/* callbck when msg received */
		   )
{
  if ((gid >= PPM_CLIENT_MAX)	||
	  (func == NULL)		||
	  ppm_cb[gid].ppmc_gid		||
	  ppm_cb[gid].ppmc_data		||
	  ppm_cb[gid].ppmc_call )
	return NFX_ERR;

  ppm_cb[gid].ppmc_gid = gid;
  ppm_cb[gid].ppmc_data = usrdata;
  ppm_cb[gid].ppmc_call = func;

  return NFX_OK;
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_UnregRcv
 *
 * Description: unregister with txrx control
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_UnregRcv(int gid)		/* group id */
{
  if ((gid >= PPM_CLIENT_MAX))
	 return NFX_ERR;
  
  bzero(&ppm_cb[gid], sizeof(ppmc_ctrl_t));
  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_SendMsg(int    gid,			/* group id */
			int    action,		/* action on msg */
			int	   slot,
			int    cpu,
			void	*msg,		/* ptr to real msg */
			uint32	size,		/* size of msg */
			int    bcast
			)
{
  ppm_mhdr_t	*ppmh;
  uint32		saddr;
  uint32		daddr;

  /* msg must be allocated with PPM_MSG_ALLOC, */
  /* there are some space at begining of buff for ppm hdr */
  ppmh = (ppm_mhdr_t*)(((char*)msg) - sizeof(ppm_mhdr_t));

  ppmh->ppmh_cpu = nfxMyCPUId;
  ppmh->ppmh_slot = nfxMySlotId;
  ppmh->ppmh_gid = gid;
  ppmh->ppmh_action = action;
  ppmh->ppmh_size = size + sizeof(ppm_mhdr_t);

  if (ppmh->ppmh_size > PPM_MSG_MAX_SIZE)
	return NFX_ERR;
  
  /* check for bcast */
  if (bcast)
	{
	  /* no support to bcast yet, send to ssc */
	  bcast = 0;
	  slot = 0;
	  cpu  = NFX_SSC_UNIX;
	}


  daddr = PACK_UNICAST_PORT_ADDRESS(
								slot,
								cpu,
								ppm_appid,
								EEE_MGMT,
								NFX_COS_HIGH,
								0 );
  saddr = ppmeee_src;

 // printf("msg send to slot %d, cpu %d, app %d \n", slot, cpu, ppm_appid);
 // bsd_ShowPkt((char*)ppmh, size+sizeof(*ppmh));	
  return  eee_sendMessage((char*)ppmh,
						  size+sizeof(*ppmh),
						  daddr, saddr);
}
