/*
 *-----------------------------------------------------------------
 *
 * Name:    mcount.S
 *
 * RCS:  $Id: mcount.S,v 1.6 2002/09/12 20:02:49 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:   mcount trampoline code
 *
 * Created by:  Byron Rakitzis
 *
 * Date Created:    11/7/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-target.h>

#if !defined(LINUX_TEST)

#include <nfx-incl.h>
#include <sys/mips.h>

.set noreorder
.set noat

.globl _mcount
.ent _mcount
	
#if defined(__long64)

_mcount:
	daddiu sp,sp,-64
	sd a0,48(sp)
	sd a1,56(sp)
	sd a2,64(sp)
	sd a3,72(sp)
	sd $at,32(sp)
	sd ra,40(sp)
	move a1,ra			/* selfpc */
	jal ___mcount
	move a0,$at			/* frompc */
	ld a0,48(sp)
	ld a1,56(sp)
	ld a2,64(sp)
	ld a3,72(sp)
	ld ra,40(sp)
	ld $at,32(sp)
	daddiu sp,sp,80
	j ra
	move ra,$at			/* restore frompc */
.end _mcount

#else

_mcount:
	sw a0,8(sp)
	sw a1,12(sp)
	sw a2,16(sp)
	sw a3,20(sp)
	sw $at,0(sp)
	sw ra,4(sp)
	move a1,ra			/* selfpc */
	jal ___mcount
	move a0,$at			/* frompc */

	lw a0,8(sp)
	lw a1,12(sp)
	lw a2,16(sp)
	lw a3,20(sp)
	lw ra,4(sp)
	lw $at,0(sp)
	addu sp,sp,8
	j ra
	move ra,$at			/* restore frompc */
.end _mcount
		
#endif		

#else /* LINUX_TEST */

.text
        .align 4
.globl mcount
        .type    mcount,@function
mcount:
        pushl   %ebp
        movl    %esp, %ebp
        subl    $8, %esp

        movl 4(%ebp),%eax
        movl (%ebp),%edx;movl 4(%edx),%edx

        subl    $8, %esp
        pushl   %eax
        pushl   %edx
        call    ___mcount
        leave
        ret

#endif /* LINUX_TEST */

/*
 * Somewhat-tested MP-safe mcount using load-linked/store-conditional
 */
#if 0
// void mcount(frompc, selfpc)
// {
// 
//       int hash = (frompc ^ selfpc) % TABLE_SIZE;
//       int i = hash;
// 
//       do {
//               volatile Arc *arc = &table[i];
// 
//               if (arc->frompc == 0) {
//                       int old = loadLinked(&arc->frompc);
//                       if (old == 0
//                           && storeConditional(&arc->frompc, frompc)) {
//                               arc->count = 1;
//                               arc->selfpc = selfpc;
//                               return;
//                       }
//                       assert(arc->frompc != 0);
//               }
// 
//               if (arc->frompc == frompc && arc->selfpc == selfpc) {
//                       while (TRUE) {
//                               int count = loadLinked(&arc->count);
//                               if (storeConditional
//                                   (&arc->count, count + 1)) return;
//                       }
//               }
// 
//               i = (i + 1) % TABLE_SIZE;
//       } while (i != hash);
// 
//       // table was full
// }

#include <nfx-incl.h>
#include <sys/mips.h>

.abicalls

.set noreorder
.set noat

.text
.align 4
.globl _mcount
.type    _mcount,@function
_mcount:
        xor t0,$at,ra            # hash = frompc ^ selfpc

	####################
	#
	# Hand-crafted code to compute a table size of 12*65536.
	#
	# This is slightly bogus, should be keyed of a CPP constant.

	srl t0,2
        andi t0,0xffff		# hash is multiple of 4 (instructions
	sll t0,2		# on mips are 4-byte aligned)

	addu t1,t0,t0		# hash *= 3
	addu t0,t0,t1

	li t5,12*0xffff

	#
	#
	####################


        move t1,t0              # i = hash
        la t2,mcount_table
        addu t3,t2,t1           # arc = &table[i]

loop:

#
#       Possible optimization: try a regular load before an ll assuming
#       you will hit an arc.
#
        lw t4,0(t3)             # if arc->frompc != 0
        bnez t4,check_arc
	nop

        ll t4,0(t3)             # if arc->frompc != 0
        bnez t4,check_arc
	nop

        move t4,$at             # or if sc(&arc->frompc, frompc) == 0
        sc t4,0(t3)
        beqz t4,check_arc
        lw t4,0(t3)             # hoisted from check_arc

        sw ra,4(t3)             # arc->selfpc = selfpc
        b incr_count
        nop

check_arc:
        # lw t4,0(t3)           # frompc == arc->frompc
        bne t4,$at,next_iter
                                # next instruction ok to mispredict

        lw t4,4(t3)             # selfpc == arc->selfpc
        bne t4,ra,next_iter
	nop

incr_count:
        ll t4,8(t3)
        addiu t4,1
        sc t4,8(t3)
        beqz t4,incr_count
        nop

	addu sp,sp,8
	j ra
	move ra,$at

next_iter:
        addiu t1,12              # i = (i + 1) % TABLE_SIZE;
	beq t1,t5,no_wrap
	nop
	move t1,zero

no_wrap:
        bne t0,t1,loop          # while (i != hash);
        addu t3,t2,t1           # hoisted from top of loop

	addu sp,sp,8
	j ra
	move ra,$at
#endif
