/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: from-ui.c,v 1.12 2002/10/16 02:57:52 johnh Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/26/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/malloc-api.h>

#include <exports/from-api.h>
#include <exports/prom-part-api.h>

#define DEBUG_FROM  PROM

#ifdef DEBUG_FROM
static int32
verif_sector(int32 select, int32 offset, uchar8 * src_p, uchar8 * verif_p)
{
    int32 rv = NFX_OK;
    int swapped;
    int i;

    if (select != from_current) {
        from_swap_base();
        swapped = 1;
    }
    else
        swapped = 0;

    rv = from_programbytes(
            offset,
            (TYPE_LEN *) src_p,
            FROM_SECTORSIZE);
    if (rv != NFX_OK) {
        printf("Error: verif_sector(), from_programbytes() returned %d\n", rv);
        goto exit;
    }
    
    // clear verif buffer
    memset(verif_p, 0x00, FROM_SECTORSIZE);
    rv = from_readbytes(
            offset,
            (TYPE_LEN *) verif_p,
            FROM_SECTORSIZE);
    if (rv != NFX_OK) {
        printf("Error: verif_sector(), from_readbytes() returned %d\n", rv);
        goto exit;
    }

    for(i=0; i < FROM_SECTORSIZE; i++) {
        if (src_p[i] != verif_p[i]) {
            printf("Error: prom stress @[0x%08x] expect 0x%02x, got0x%02x\n",
                    offset+i, src_p[i], verif_p[i]);
            rv = NFX_ERR;
            goto exit;
        }
    }

exit:
    if (swapped)
        from_swap_base();
    return rv;
}

static uchar8 test_buf[FROM_SECTORSIZE];
static uchar8 verif_buf[FROM_SECTORSIZE];

static void
stress(int32 select, int32 offset, int32 len, int count)
{
    int32 rv;
    int j;
    int n;
    int offs;
    int res;
    int16 * sp;

    if ((offset & FROM_SECTORMASK) ||
        (len    & FROM_SECTORMASK))
    {
        printf("Error: offset (0x%x) and len (0x%x) must be sector (0x%x) aligned\n",
                offset, len, FROM_SECTORSIZE);
        return;
    }

    n = 0;
    while (n < count) {
        offs = offset;
        res = len;
        while (res > 0) {
            printf("\t... test 0s\n");
            memset(test_buf, 0x00, FROM_SECTORSIZE);
            rv = verif_sector(select, offs, test_buf, verif_buf); 
            if (rv != NFX_OK)
                return;
            printf("\t... test 1s\n");
            memset(test_buf, 0xff, FROM_SECTORSIZE);
            rv = verif_sector(select, offs, test_buf, verif_buf); 
            if (rv != NFX_OK)
                return;
            printf("\t... test 01s\n");
            memset(test_buf, 0x55, FROM_SECTORSIZE);
            rv = verif_sector(select, offs, test_buf, verif_buf); 
            if (rv != NFX_OK)
                return;
            printf("\t... test 10s\n");
            memset(test_buf, 0xAA, FROM_SECTORSIZE);
            rv = verif_sector(select, offs, test_buf, verif_buf); 
            if (rv != NFX_OK)
                return;
            for (j=0, sp=(int16 *)test_buf; j < (FROM_SECTORSIZE/sizeof(int16)); j++) {
                *sp++ = (int16) (0xFFFF & j);
            }
            printf("\t... test address\n");
            rv = verif_sector(select, offs, test_buf, verif_buf); 
            if (rv != NFX_OK)
                return;

            offs += FROM_SECTORSIZE;
            res  -= FROM_SECTORSIZE;

        }
        n++;
        printf("\tDone iteration %d out of %d\n", n, count);
    }
}
#endif  /* DEBUG_FROM */

/*-----------------------------------------------------------------
 * Name:	copy_programbytes()
 *
 * Description: Copies from the currently booting flash-rom to 
 *              specified memory.
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created:	05/26/04
 *
 *-----------------------------------------------------------------
 */
int
copy_programbytes(int32 offset, TYPE_LEN * dst_buf, uint32 len)
{
	int32 rc=0;

	rc = from_readbytes(offset, dst_buf, len);
	return (rc);	
	
}		

/*-----------------------------------------------------------------
 * Name:	do_prom_cmd()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/8/01
 *
			printf("prom id\n");
			printf("prom read  <offset> <length>\n");
			printf("prom erase [0|1] <sector>\n");
			printf("prom fill  [0|1] <offset> <src_addr> <length>\n");
			printf("prom upgrade <addr> [<length>]\n");
			printf("prom upgrade <ipaddr> <fname>\n");
			printf("prom stress [0|1] <offset> <len> <repeat-count>\n");
			printf("prom view [partition [<part-name> | [<address> <len>]]]\n");
			printf("prom flag set|reset <part-name> <val>\n");             
			printf("prom autoload dis|ena  <part-name>\n");             
 *-----------------------------------------------------------------
 */
int
do_prom_cmd(
	int32	ac,
	uchar8	* av[]
)
{
	int32	ret = 0;
#ifdef PROM
	int32	swapped = 0;
#endif

	if (ac < 2)
		return -1;

	if (!NFX_SPIN_TRYLOCK(&prom_lock)) {
		printf("prom access currently locked\n");
		return 0;
	}

	if (strncmp(av[1], "id", 2) == 0) {
		TYPE_LEN	man, dev;
		ret = from_readId(&man, &dev);
		printf("Manufacturer = 0x%x, Device Id = 0x%x\n", man, dev);
		goto cmd_exit;
	}
	else if (strncmp(av[1], "autoload", 4) == 0) {
        int set;

        if (ac != 4) {
	        printf("prom flag set|reset <partition-name> <val>\n");             
			ret = -1;
			goto cmd_exit;
        }

        if (strncmp(av[2], "dis", 3) == 0)
            set = 1;
        else if (strncmp(av[2], "ena", 3) == 0)
            set = 0;
        else {
	        printf("prom autoload dis|ena <partition-name>\n");             
			ret = -1;
			goto cmd_exit;
        }

        ret = prom_part_update_flag(
                av[3],
                (unsigned short) (0x0004),
                set);
    }
	else if (strncmp(av[1], "flag", 4) == 0) {
        int set;

        if (ac != 5) {
	        printf("prom flag set|reset <partition-name> <val>\n");             
			ret = -1;
			goto cmd_exit;
        }

        if (strncmp(av[2], "set", 3) == 0)
            set = 1;
        else if (strncmp(av[2], "reset", 3) == 0)
            set = 0;
        else {
	        printf("prom flag set|reset <partition-name> <val>\n");             
			ret = -1;
			goto cmd_exit;
        }

        ret = prom_part_update_flag(
                av[3],
                (unsigned short) (0xFFFF & atox(av[4])),
                set);
    }
	else if (strncmp(av[1], "view", 2) == 0) {
        if (ac >= 3 && strncmp(av[2], "partition", 2) == 0) {
            prom_part_desc_p desc_p;
            address_t addr;
            uint32  len;
            int from_sav;

            if (ac == 5) {
                // view partition control block at a specific location 
                addr = atolx(av[3]);
                len = atox(av[4]);
            }
            else if (ac == 4) {
                desc_p = prom_part_get_desc_from_name(av[3]);
                if (desc_p == NULL) {
                    printf("prom partition %s not found\n", av[3]);
                    goto cmd_exit;
                }

                if (desc_p->part_type == PROM_PART_R7K_RPROM ||
                    desc_p->part_type == PROM_PART_SIBYTE_RPROM ||
		    desc_p->part_type == PROM_PART_R9K_RPROM)
                    from_sav = from_select_recov();
                else
                    from_sav = from_select_std();

                addr = FROM_ADDR(address_t, desc_p->part_offs);
                len = desc_p->part_size;

                from_set_current(from_sav);
            }
            else {
                // view all partition control blocks
                addr = 0;
                len = 0;
            }
            prom_part_print_info((void *)addr, len);
        }
        else {
            from_print_info();
        }
	}
#ifdef PROM
	else if (strncmp(av[1], "erase", 2) == 0) {
		uint32	sec;
		uint32	sel;

        sel = from_current;
		if (ac ==  3) {
            sec = atox(av[2]);
        }
        else if (ac == 4) {
            sel = atox(av[2]);
            sec = atox(av[3]);
        }
        else {
			printf("prom erase [0|1] <sector>\n");
			goto cmd_exit;
        }
        if (sel != from_current) {
            from_swap_base();
            swapped = 1;
        }
		printf("erasing sector %d\n", sec);
		ret = from_erasesector(sec);
        if (swapped)
            from_swap_base();
	}
	else if (strncmp(av[1], "copy", 2) == 0) {
		uint32		offset, len;
		address_t	addr;

		if (ac == 5) {
            offset = atox(av[3]);
            addr = atolx(av[4]);
            len = atox(av[5]);
        }
        else {
			printf("prom copy <offset> <dstn_addr> <length>\n");
			goto cmd_exit;
        }
		ret = copy_programbytes(offset, (TYPE_LEN *)addr, len);
	}		
	else if (strncmp(av[1], "fill", 2) == 0) {
		uint32		offset, len, sel;
		address_t	addr;

        sel = from_current;
		if (ac == 5) {
            offset = atox(av[2]);
            addr = atolx(av[3]);
            len = atox(av[4]);
        }
        else if (ac == 6) {
            sel = atox(av[2]);
            offset = atox(av[3]);
            addr = atolx(av[4]);
            len = atox(av[5]);
        }
        else {
			printf("prom fill [0|1] <offset> <src_addr> <length>\n");
			goto cmd_exit;
        }

        if (sel != from_current) {
            from_swap_base();
            swapped = 1;
        }
		ret = from_programbytes(offset, (TYPE_LEN *)addr, len);
        if (swapped)
            from_swap_base();
	}
#endif
#if defined(PROM) || defined(SSC_MGMT)
	else if (strncmp(av[1], "upgrade", 2) == 0) {
		char			* fname;
		unsigned int	ip;
		int				rc;
		address_t		addr;
        int32           len;

        if (ac > 2) {
			if (strcmp(av[2], "fp") != 0) {
                printf("\nPROM ugprade in progress, please wait...\n");
				if (isFP() || isSSCmgmt()) {
					addr = atolx(av[2]);
                    if (ac == 4)
                        len = atox(av[3]);
                    else
                        len = 0x80000;

					rc = prom_part_upgrade((char *)addr, len);
					if (rc != NFX_OK)
						printf("Error Upgrading PROM image from mem\n");
					else
						goto cmd_exit;
				}
#if defined(PROM)
				else {
					if (ac == 3) {
						fname = av[2];
						printf("%s\n", fname);
					}
					else if (ac == 4) {
                        extern int32 isIpAddr(uchar8 * str);

						fname = av[3];
						if (isIpAddr(av[2])) {
							if (getIpAddr( av[2], &ip ) < 0) {
								ret = -1;
								goto cmd_exit;
							}
						}
						else {
							ret = -1;
							goto cmd_exit;
						}
//						printf("ip = 0x%x, %s\n", ip, fname);

						from_upgrade(ip, fname, 0);
					}
				}
#endif
			}
		}
		else
			ret = -1;
	}
#endif
#ifdef DEBUG_FROM
	else if (strncmp(av[1], "stress", 4) == 0) {
		uint32	select, offset, len, repeat;

		if (ac < 6) {
			printf("prom stress 0|1 <offset> <length> <repeat>\n");
			goto cmd_exit;
		}
		select = atox(av[2]);
		offset = atox(av[3]);
		len = atox(av[4]);
		repeat = atox(av[5]);

        stress(select, offset, len, repeat);
    }
#endif  /*DEBUG_FROM*/
	else
		ret = -1;

cmd_exit:
	NFX_SPIN_UNLOCK(&prom_lock);
	return ret;

}

