/*
 *-----------------------------------------------------------------
 *
 * Name:	from-upgrade.c
 *
 * RCS:  $Id: from-upgrade.c,v 1.17 2002/11/06 22:46:06 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	 Upgrade routines
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */


#include <nfx-incl.h>
#ifdef PROM
#include <exports/tftp-api.h>
#endif
#include "from-api.h"
#include "prom-part-api.h"

#if defined(__OpenBSD__) && defined(_KERNEL)
#include <sys/types.h>
#include <sys/systm.h>
#endif

#if defined(PROM_SIBYTE)
uchar8  * prom_upgrade_area = (uchar8 *)(0x40000000);
#elif defined (PROM_R7000) || defined (PROM_R9000)
uchar8  * prom_upgrade_area = (uchar8 *)(0x83000000);
#else   /* all others */
uchar8    prom_upgrade_area[PROM_UPGRADE_MAX_SIZE];
#endif  /* SIBYTE & PROM_R7000 PROM */

uchar8	* from_base;
uchar8	* from_ptr;
uint32	  from_len;

#undef DEBUG_PRINT

#ifdef DEBUG_PRINT
#define DPRINT(_x)  printf _x
#else
#define DPRINT(_x)
#endif

/******************************************************************************/

#ifdef SIBYTE
/* 4MB sibyte prom */
prom_part_desc_t prom_part_rprom_layout[]   = PART_DESC_SIBYTE_RPROM_LAYOUT;
#endif
#ifdef PROM_R7000
/* 2MB r7k prom */
prom_part_desc_t prom_part_rprom_layout[]   = PART_DESC_R7K_RPROM_LAYOUT;
#endif
#ifdef PROM_R9000
/* 2MB r7k prom */
prom_part_desc_t prom_part_rprom_layout[]   = PART_DESC_R9K_RPROM_LAYOUT;
#endif
#if defined(SSC_MGMT) || defined(PROM_SSC_MGMT)
/* there is no prom_ssc_mgmt (toshiba) rprom */
prom_part_desc_t prom_part_rprom_layout[]   = {PART_DESC_NULL};
#endif

#ifdef SIBYTE
/* 4MB sibyte prom */
prom_part_desc_t prom_part_rt_layout[]      = PART_DESC_SIBYTE_LAYOUT;
prom_part_desc_t prom_part_rt_fp_layout[]   = PART_DESC_SIBYTE_FP_LAYOUT;
#endif
#ifdef PROM_R7000
/* 2MB r7k prom */
prom_part_desc_t prom_part_rt_layout[]      = PART_DESC_R7K_LAYOUT;
#endif
#ifdef PROM_R9000
/* 2MB r7k prom */
prom_part_desc_t prom_part_rt_layout[]      = PART_DESC_R9K_LAYOUT;
#endif
#if defined(SSC_MGMT) || defined(PROM_SSC_MGMT)
/* 0.5 MB ssc_mgmt (toshiba) prom */
prom_part_desc_t prom_part_rt_layout[]      = PART_DESC_MGMT_LAYOUT;
#endif

prom_part_desc_p prom_part_layout;

struct prom_part_desc *prom_layouts[2];

void
prom_part_init(void)
{
    if (primary_is_rprom) {
		prom_layouts[0] = prom_part_rprom_layout;
#if defined(SIBYTE)
		if (isFP()) 
			prom_layouts[1] = prom_part_rt_fp_layout;
		else
			prom_layouts[1] = prom_part_rt_layout;
#else
		prom_layouts[1] = prom_part_rt_layout;
#endif
        prom_part_layout = prom_part_rprom_layout;
    }
	else {
#if defined(SIBYTE)
		if (isFP()) {
			prom_part_layout = prom_part_rt_fp_layout;
			prom_layouts[0] = prom_part_rt_fp_layout;
		}
		else {
			prom_layouts[0] = prom_part_rt_layout;
		}
#else
		prom_layouts[0] = prom_part_rt_layout;
#endif
		prom_layouts[1] = prom_part_rprom_layout;
		prom_part_layout = prom_part_rt_layout;
	}
}

/******************************************************************************/

#ifdef PROM

int32	fetch_fp_image;
int32	upgrade_busy;

extern uchar8	* eee_sm_prom_upgrade;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
prom_cmdDataInd(
	char8	* pData,
	int32	size
)
{
	memcpy(from_ptr, pData, size);

	from_len += size;
	from_ptr += size;

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
prom_cmdEventInd(
	void	* tftpCb,
	int32	event
)
{
    int rv;

	switch (event) {
		case TFTP_EVT_TIMEOUT:
            printf("TFTP Timeout.\n");
            break;

		case TFTP_EVT_ERROR:
            printf("TFTP Error.\n");
            break;

		case TFTP_EVT_EOF:
            printf("TFTP transfer completed.\n");
            if (!fetch_fp_image) {
                printf("\nPROM ugprade in progress, please wait...\n");
                rv = prom_part_upgrade(from_base, from_len);
                if (rv != NFX_OK)
                    printf("Error Upgrading PROM image from net\n");
            }
            else {
                printf("Got FP bootrom image, addr = %p\n",
                                                    eee_sm_prom_upgrade);
            }
            break;

		default:
            printf("Unknown TFTP Event.\n");
            break;
	}

	upgrade_busy = 0;

	tftp_close(tftpCb);
	return 0;
}

/*-----------------------------------------------------------------
 * Name:	from_upgrade()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
from_upgrade(
	uint32	ip,
	uchar8	* fname,
	uint32	fetch_fp
)
{
	void	* fd;

	if (fetch_fp)
		fetch_fp_image = 1;
	else
		fetch_fp_image = 0;

	if (fetch_fp_image)
		from_ptr = from_base = eee_sm_prom_upgrade;
	else
		from_ptr = from_base = prom_upgrade_area;

	from_len = 0;

	fd = tftp_open(fname, ip, prom_cmdDataInd, prom_cmdEventInd);
	if (!fd) {
		printf("Error opening remote file, %s\n", fname);
		return -1;
	}

	upgrade_busy = 1;

	while (1) {
		eee_poll(1);
		if (!upgrade_busy)
			break;
	}

	return 0;
}
#endif  /* PROM */


/*
 * print old prom compress record
 */
void
from_old_prom_print(
        uchar8 *    buf_p)
{
    comp_rec_t * cr_p = (comp_rec_t *) &buf_p[OLD_PROM_COMP_REC_OFFSET];

    printf("  !!! OLD PROM FORMAT compress rec @%p, Base %p\n",cr_p, buf_p);
    /* check for uninitialized info_str in flash ie. all 1's */ 
    printf("\tsignature           : 0x%x\n", cr_p->signature);
    printf("\tcompressed_start    : 0x%x\n", cr_p->compressed_start);
    printf("\tcompressed_size     : 0x%x\n", cr_p->compressed_size);
    printf("\tprom_type           : 0x%x\n", cr_p->prom_type);
}

/*
 * get the prom type and valid length for the partition(s)
 */
int
from_old_prom_type(
        uchar8 *    buf_p,
        int32       prom_len)
{
    /* check for the classic compress record
     * bootroms, for 512KB Am29LV040B and prom_rec at 0x6ffec
     */
    if ((prom_len == AMD_29LV040_SIZE) || (prom_len == ST_M29W040_SIZE)) {
        comp_rec_t * cr_p = (comp_rec_t *) &buf_p[OLD_PROM_COMP_REC_OFFSET];

        if (cr_p->signature == COMPRESS_SIGNATURE) {
            if (cr_p->prom_type == PROM_PART_R7K ||
		cr_p->prom_type == PROM_PART_R9K ||
                cr_p->prom_type == PROM_PART_SIBYTE ||
                cr_p->prom_type == PROM_PART_MGMT) {

                return cr_p->prom_type;
            }
        }
    }
    return PROM_PART_UNKNOWN;
}


/*-----------------------------------------------------------------
 * Name :	from_check_prom_type()
 *
 * Description: 
 *
 * Created by:	John Hahn
 *
 * Date Created:	08/09/02
 *
 *-----------------------------------------------------------------
 */
int
from_check_old_prom_type(uchar8 *buf, int len)
{
#if defined(SIBYTE)
	return from_old_prom_type(buf, len) == PROM_PART_SIBYTE;
#elif defined(PROM_R7000)
	return from_old_prom_type(buf, len) == PROM_PART_R7K;
#elif defined(PROM_R9000)
	return from_old_prom_type(buf, len) == PROM_PART_R9K;
#elif defined(SSC_MGMT)
	return from_old_prom_type(buf, len) == PROM_PART_MGMT;
#else
#error "Invalid platform"
#endif
}

static int
prom_part_check_full_size(int len)
{
    return (len == AMD_29F040_SIZE ||
            len == AMD_29LV040_SIZE ||
            len == AMD_29LV017_SIZE ||
            len == AMD_29LV033_SIZE ||
	    len == ST_M29W040_SIZE);
}

/*
 * check if the partition prom_type is RPROM
 * or if the len is rprom full size
 */
static int
prom_part_check_rprom(prom_part_cb_p cb_p, int len)
{
    switch (cb_p->prom_type) {
#if defined(SIBYTE)
        case PROM_PART_SIBYTE_RPROM:
#endif
#if defined(PROM_R7000)
        case PROM_PART_R7K_RPROM:
#endif
#if defined(PROM_R9000)
        case PROM_PART_R9K_RPROM:
#endif
            return 1;
            break;

        default:
            if ((len == AMD_29LV040_SIZE) || (len == ST_M29W040_SIZE))
                return 1;
            break;
    }
    return 0;
}

/*
 * get the prom type and valid length for the partition(s)
 */
int
from_get_prom_info(
        char *              buf_p,
        int                 data_len,
        prom_part_cb_p *    cb_pp,
        prom_part_desc_p *  desc_pp)
{
    char *              base_p;
    prom_part_cb_p      cb_p;
    prom_part_desc_p    desc_p;
    int                 full_size;

    desc_p = NULL;
    base_p = buf_p;
    cb_p = prom_part_get_cb(buf_p, data_len);

    full_size = prom_part_check_full_size(data_len);

    if (!full_size) {
        if (prom_part_verify(cb_p, base_p, 1 /*XXX verbose*/) != NFX_OK)
            return NFX_ERR;
    }
    else {
        /* full prom in buf, go thru valid partitions in prom buf */
        if (!prom_part_check_rprom(cb_p, data_len)) {
#if defined(SIBYTE)
            if (isFP())
                desc_p = prom_part_rt_fp_layout;
            else
#endif
                desc_p = prom_part_rt_layout;
        }
        else
            desc_p = prom_part_rprom_layout;

        while (desc_p->part_name != NULL) {
            if (desc_p->part_type == PROM_PART_SYS)
                goto get_prom_info_exit;

            base_p = buf_p + desc_p->part_offs;
            cb_p = prom_part_get_cb(base_p, desc_p->part_size);
            if (prom_part_verify(cb_p, base_p, 1 /*XXX verbose*/) != NFX_OK)
                return NFX_ERR;

            if (prom_part_check_rprom(cb_p, 0))
                goto get_prom_info_exit;
            desc_p++;
        }
    }

    desc_p = prom_part_get_desc_from_type(cb_p->prom_type);
    if (desc_p == NULL)
        return NFX_ERR;

    switch (cb_p->prom_type) {
#if defined(SIBYTE)
        case PROM_PART_SIBYTE:
        case PROM_PART_SIBYTE_DIAG:
        case PROM_PART_SIBYTE_RPROM:
        case PROM_PART_LMUX_HEX:
        case PROM_PART_LUC_HEX:
#endif
#if defined(PROM_R7000)
        case PROM_PART_R7K:
        case PROM_PART_R7K_DIAG:
        case PROM_PART_R7K_RPROM:
#endif
#if defined(PROM_R9000)
        case PROM_PART_R9K:
        case PROM_PART_R9K_DIAG:
        case PROM_PART_R9K_RPROM:
#endif
#if defined(SSC_MGMT) || defined(PROM_SSC_MGMT)
        case PROM_PART_MGMT:
#endif
            if ((data_len + PROM_PART_CB_SIZE) > desc_p->part_size)
                return NFX_ERR;
            break;

        default:
            return NFX_ERR;
            break;
    }
get_prom_info_exit:
    *cb_pp   = cb_p;
    *desc_pp = desc_p;
    return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:            prom_part_upgrade
 *
 * Description:     install the prom binary image in buf_p     
 *
 * Created by:	    John Hahn
 *
 * Date Created:    7/25/2002
 *
 *-----------------------------------------------------------------
 */
int
prom_part_upgrade(
        char *      buf_p,
        int         data_len)
{
    int                 rv;
    int                 from_sav;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;

    DPRINT(("prom_part_upgrade: buf_p %p data_len 0x%x\n", buf_p, data_len));

    if (data_len < PROM_PART_CB_SIZE)
        return NFX_ERR;

    /*
     * check for the old type of bootrom.bin and take care of it
     */
    if (from_check_old_prom_type(buf_p, data_len)) {

        from_sav = from_select_recov();
        rv = from_programbytes(0, buf_p, data_len);
        from_set_current(from_sav);
        return rv;
    }

    rv = from_get_prom_info(buf_p, data_len, &cb_p, &desc_p);
    if (rv != NFX_OK)
        return rv;

#ifdef DEBUG_PRINT
    DPRINT(("partition info: (rprom=%d)\n",prom_part_check_rprom(cb_p, data_len)));
    PART_DESC_PRINT(desc_p);
    PART_CB_PRINT(cb_p, buf_p);
#endif

    if (prom_part_check_rprom(cb_p, data_len)) {
        // for RPRM the last sector may be shared w/ crashdump and fake seepenv
        if ((cb_p->compressed_start + cb_p->compressed_size) > PROM_RPRM_CODE_SIZE) { 
            printf("Upgrade Error RPROM code size 0x%x + 0x%x > 0x%x\n",
                cb_p->compressed_start,
                cb_p->compressed_size,
                PROM_RPRM_CODE_SIZE);
            return NFX_ERR;
        }


        from_sav = from_select_recov();
    }
    else
        from_sav = from_select_std();

    if (from_sav == FROM_SEL_INVALID)
        return NFX_ERR;

    /*
     * full prom upgrade
     */
    if (prom_part_check_full_size(data_len)){
        rv = from_programbytes(0, buf_p, data_len);
        goto prom_part_upgrade_exit;
    }

    rv = from_programbytes(
            desc_p->part_offs,
            buf_p,
            data_len - PROM_PART_CB_SIZE);

    if (rv != NFX_OK)
        goto prom_part_upgrade_exit;

    rv = from_programbytes(
            desc_p->part_offs + desc_p->part_size - PROM_PART_CB_SIZE,
            (void *)cb_p,
            PROM_PART_CB_SIZE);

    /*
     * find the prom_part_cb at the end of the buffer
     * verify the crc32 over header and data
     * if full prom check the data_len, cb's  and program it
     * if single prom image, check the cb program the data then the cb
     * at the end of the partition for the image
     * 
     */
prom_part_upgrade_exit:
    from_set_current(from_sav);
    return rv;
}

/*
 * return prom partition descriptor of partition name
 * NOTE: the desc may not be the one for the from_current
 * and you need know wether to switch froms !!!
 */
prom_part_desc_p
prom_part_get_desc_from_name(
        char * name)
{
    prom_part_desc_p desc_p = prom_part_layout;

    while (desc_p->part_name != NULL) {
        if (strcmp(name, desc_p->part_name) == 0)
            return desc_p;
        desc_p++;
    }

    if (prom_part_layout == prom_part_rprom_layout) {
#if defined(SIBYTE)
        if (isFP())
            desc_p = prom_part_rt_fp_layout;
        else
#endif
            desc_p = prom_part_rt_layout;
    }
    else
        desc_p = prom_part_rprom_layout;

    while (desc_p->part_name != NULL) {
        if (strcmp(name, desc_p->part_name) == 0)
            return desc_p;
        desc_p++;
    }

    return NULL;
}

/*
 * return prom partition descriptor of partition prom_type
 */
prom_part_desc_p
prom_part_get_desc_from_type(
        unsigned short  prom_type)
{
    prom_part_desc_p desc_p = prom_part_layout;

    if (prom_type == PROM_PART_R7K_RPROM ||
	prom_type == PROM_PART_R9K_RPROM ||	    
        prom_type == PROM_PART_SIBYTE_RPROM)
        desc_p = prom_part_rprom_layout;
    else {
#if defined(SIBYTE)
        if (isFP())
            desc_p = prom_part_rt_fp_layout;
        else
#endif
            desc_p = prom_part_rt_layout;
    }

    while (desc_p->part_name != NULL) {
        if (prom_type == desc_p->part_type)
            return desc_p;
        desc_p++;
    }
    return NULL;
}


struct prom_part_desc *
prom_get_layout(int unit)
{
	return prom_layouts[unit];
}


/*
 * return partition control block at the end of the given buffer of len
 */
prom_part_cb_p
prom_part_get_cb(
        char *  buf_p,
        int     len)
{
    return (prom_part_cb_p) (buf_p + len - PROM_PART_CB_SIZE);
}

/*
 * following are verified on the partition:
 * in the control block: signature, header crc32, partition data crc32
 */                
int
prom_part_verify(
        prom_part_cb_p  cb_p,
        char *          base_p,
        int             verbose)
{
    uint32 crc32;

    if (cb_p->signature != PROM_PART_SIGNATURE) {
        if (verbose)
            printf("Error partition cb@%p bad signature 0x%x, expect 0x%x\n",
                    cb_p, cb_p->signature, PROM_PART_SIGNATURE);
        return NFX_ERR;
    }

    crc32 = crc32_zlib(0, cb_p, PROM_PART_CB_CRC32_SIZE);
    if (crc32 != cb_p->header_crc32) {
        if (verbose)
            printf("Error partition cb@%p header corrupt crc32 0x%x hdr 0x%x\n",
                    cb_p, crc32, cb_p->header_crc32);
        return NFX_ERR;
    }
    switch (cb_p->prom_type) {
#if defined(SIBYTE)
        case PROM_PART_SIBYTE:
        case PROM_PART_SIBYTE_DIAG:
        case PROM_PART_SIBYTE_RPROM:
        case PROM_PART_LMUX_HEX:
        case PROM_PART_LUC_HEX:
#endif
#if defined(PROM_R7000)
        case PROM_PART_R7K:
        case PROM_PART_R7K_DIAG:
        case PROM_PART_R7K_RPROM:
#endif
#if defined(PROM_R9000)
        case PROM_PART_R9K:
        case PROM_PART_R9K_DIAG:
        case PROM_PART_R9K_RPROM:
#endif
#if defined(SSC_MGMT) || defined(PROM_SSC_MGMT)
        case PROM_PART_MGMT:
#endif
            // partition is allowed for this platform, proceed ...
            break;

        case PROM_PART_SYS:
            return NFX_OK;
            break;

        default:
            // if we got here then the partition doesnt belong
            if (verbose)
                printf("Error prom partition type %d doesnt belong here\n", cb_p->prom_type);
            return NFX_ERR;
    }
    if (cb_p->flags | PROM_PART_CRC32_FLAG) {
        if (cb_p->compressed_start + cb_p->compressed_size) {
            crc32 = crc32_zlib(0, base_p,
                    cb_p->compressed_start + cb_p->compressed_size);
            if (crc32 == cb_p->partition_crc32) {
                return NFX_OK;
            }
            else {
                if (verbose)
                    printf("Error, partition data corrupt crc32 0x%x data 0x%x\n", crc32, cb_p->partition_crc32);
            }
        }
        else {
            if (verbose)
                printf("Error, no partition data\n");
        }
    }
    return NFX_ERR;
}

static void
prom_part_print(
    prom_part_desc_p    desc_p,
    prom_part_cb_p      cb_p,
    char *              base_p)
{

    if (desc_p != NULL)
        PART_DESC_PRINT(desc_p);
    
    PART_CB_PRINT(cb_p, base_p);
}


int
prom_part_update_flag(
    char *          part_name,
    unsigned short  flags,
    int             set)
{
    int rv;
    int from_sav;
    char *              base_p;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;
    prom_part_cb_t      control_block;

    desc_p = prom_part_get_desc_from_name(part_name);
    if (desc_p == NULL ||
        (desc_p->part_type != PROM_PART_LUC_HEX &&
         desc_p->part_type != PROM_PART_LMUX_HEX))
    {
        printf("prom_part_update_flag: invalid partition %s\n", part_name);
        return NFX_ERR;
    }

    from_sav = from_select_std();
    if (from_sav == FROM_SEL_INVALID)
        return NFX_ERR;

    base_p = FROM_ADDR(char *, desc_p->part_offs);
    cb_p = prom_part_get_cb(base_p, desc_p->part_size);

    control_block = *cb_p;

    if (set) {
        control_block.flags |= flags;
    }
    else {  /* set the bits */
        control_block.flags &= ~flags;
    }
    if (control_block.flags == cb_p->flags) {
        /* nothing to change */
        rv = NFX_OK;
        goto update_flag_exit;
    }

    printf("prom_part_update_flag: old=%04x new=%04x\n",
            cb_p->flags, control_block.flags);

    /* recalculate the header crc32 */
    control_block.header_crc32 = crc32_zlib(
            0, &control_block, PROM_PART_CB_CRC32_SIZE);
    rv = from_programbytes(
            desc_p->part_offs + desc_p->part_size - PROM_PART_CB_SIZE,
            (void *)&control_block,
            PROM_PART_CB_SIZE);

update_flag_exit:
    from_set_current(from_sav);
    return rv;
}

/*
 * return prom partition control block flags 
 */
int
prom_part_get_cb_flags(
    char *          part_name,
    unsigned short  * flags)
{
    int from_sav;
    char *              base_p;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;

    desc_p = prom_part_get_desc_from_name(part_name);
    if (desc_p == NULL ||
        (desc_p->part_type != PROM_PART_LUC_HEX &&
         desc_p->part_type != PROM_PART_LMUX_HEX))
    {
        printf("prom_part_update_flag: invalid partition %s\n", part_name);
        return NFX_ERR;
    }

    from_sav = from_select_std();
    if (from_sav == FROM_SEL_INVALID)
        return NFX_ERR;

    base_p = FROM_ADDR(char *, desc_p->part_offs);
    cb_p = prom_part_get_cb(base_p, desc_p->part_size);
    *flags = cb_p->flags;

    from_set_current(from_sav);
    return NFX_OK;
}

void
prom_part_print_info(
        void *  addr,
        int     len)
{
    char *              base_p;
    char *              part_p;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;
    int                 full_size;

    full_size = prom_part_check_full_size(len);

    if (!full_size && addr) {
        base_p = (char *) addr;

        if (from_check_old_prom_type(base_p, len)) {
            from_old_prom_print(base_p);
        }
        else {
            cb_p   = prom_part_get_cb(base_p, len);
            desc_p = prom_part_get_desc_from_type(cb_p->prom_type);
            prom_part_verify(cb_p, base_p, 1 /*verbose*/);
            prom_part_print(desc_p, cb_p, base_p);
        }
    }
    else {
        if (addr) {
            base_p = (char *)addr;
        }
        else {
            printf("\nPrimary flash memory\n");
            base_p = FROM_ADDR(char *, 0);
        }

        if (from_check_old_prom_type(base_p, AMD_29LV040_SIZE)) {
            from_old_prom_print(base_p);
        }
	else if (from_check_old_prom_type(base_p, ST_M29W040_SIZE)) {
            from_old_prom_print(base_p);
        }
        else {

            if (addr && len) {
                if (!prom_part_check_rprom((void *)base_p, len)) {
#if defined(SIBYTE)
                    if (isFP())
                        desc_p = prom_part_rt_fp_layout;
                    else
#endif
                        desc_p = prom_part_rt_layout;
                }
                else
                    desc_p = prom_part_rprom_layout;
            }
            else
                desc_p = prom_part_layout;

            while (desc_p->part_name != NULL) {
                part_p = base_p + desc_p->part_offs;

                cb_p = prom_part_get_cb(part_p, desc_p->part_size);
                prom_part_verify(cb_p, part_p, 1 /*verbose*/);
                prom_part_print(desc_p, cb_p, part_p);
                printf("\n");
                desc_p++;
            }
        }

        // user  requested the default print out all flash partition info
        // check if we should print out the alternate flash
        if (!addr && (from_probed_proms > 1)) {
            int from_sav = from_swap_base();

            printf("\nAlternate flash memory\n");
            if (from_check_old_prom_type(FROM_ADDR(char *,0), FROM_SIZE)) {
                from_old_prom_print(FROM_ADDR(char *,0));
            }
            else {
                // set to the other layout
                if (prom_part_layout == prom_part_rprom_layout) {
#if defined(SIBYTE)
                    if (isFP())
                        desc_p = prom_part_rt_fp_layout;
                    else
#endif
                        desc_p = prom_part_rt_layout;
                }
                else
                    desc_p = prom_part_rprom_layout;

                while (desc_p->part_name != NULL) {
                    part_p = FROM_ADDR(char *, desc_p->part_offs);
                    cb_p = prom_part_get_cb(part_p, desc_p->part_size);
                    prom_part_verify(cb_p, part_p, 1 /*verbose*/);
                    prom_part_print(desc_p, cb_p, part_p);
                    printf("\n");
                    desc_p++;
                }
            }
            from_set_current(from_sav);
        }
    }
}

#if defined(PROM) && defined(AGILE_HARDWARE)
int32
prom_part_proc_data(
    unsigned short prom_type,
    prom_part_proc_user_f user_callback_func,
    void * user_handle)
{
    char *              base_p;
    char *              buf_p;
    unsigned long       uncomp_len;
    int                 len;
    int                 buf_size;
    int                 res;
    int                 rv;
    int                 from_sav;
    prom_part_desc_p    desc_p;
    prom_part_cb_p      cb_p;

    buf_size = PROM_UPGRADE_MAX_SIZE;

    from_sav = from_select_std();
    if (from_sav == FROM_SEL_INVALID)
        return NFX_ERR;

    desc_p = prom_part_get_desc_from_type(prom_type);
    if (desc_p == NULL) {
        printf("prom_part_proc_data: invalid prom_type %d\n", prom_type);
        rv = NFX_ERR;
        goto proc_data_exit;
    }
            
    base_p = FROM_ADDR(char *, desc_p->part_offs);
    cb_p = prom_part_get_cb(base_p, desc_p->part_size);
    if (cb_p->flags & PROM_PART_DISABLED) {
        rv = NFX_ERR;
        goto proc_data_exit;
    }
#ifdef DEBUG_PRINT
    PART_DESC_PRINT(desc_p);
    PART_CB_PRINT(cb_p, base_p);
#endif
    rv = prom_part_verify(cb_p, base_p, 1 /*verbose*/);
    if (rv != NFX_OK)
        goto proc_data_exit;

    buf_p = prom_upgrade_area;
    res = buf_size;
    len = cb_p->compressed_start;
    if ((cb_p->compressed_start + cb_p->decompressed_size) > res) {
        printf("error partition data 0x%x + 0x%x > output buffer 0x%x\n",
            cb_p->compressed_start, cb_p->decompressed_size, res);
        rv = NFX_ERR;
        goto proc_data_exit;
    }
    if (len) {
        memcpy(buf_p, base_p, len);
        res -= len;
    }
    if (cb_p->flags & PROM_PART_COMPRESSION_FLAG) {
        uncomp_len = res;
        rv = uncompress(
                &buf_p[len],
                &uncomp_len,
                &base_p[len],
                cb_p->compressed_size); 
        if (rv) {
            DPRINT(("prom_part_proc_data: uncompress error %d\n", rv));
            DPRINT((" &buf_p[len=%08x]=0x%p, &uncomp_len=%p *%08lx\n",
                    len, &buf_p[len], &uncomp_len, uncomp_len));
            DPRINT((" &base_p[len]=%p cb_p->compressed_size=%08x\n",
                    &base_p[len], cb_p->compressed_size)); 
            goto proc_data_exit;
        }
    }
    else
        uncomp_len = 0;
    buf_size = len + uncomp_len;

    DPRINT(("data uncompressed to %p raw=%d uncompressed=%ld total=%d bytes\n",
                buf_p, len, uncomp_len, buf_size));
    if (buf_size <= 0) {
        rv = NFX_ERR;
        goto proc_data_exit;
    }

    rv = (*user_callback_func)(user_handle, buf_p, buf_size);
proc_data_exit:
    from_set_current(from_sav);
    return rv;
}
#endif

