/*
 *-----------------------------------------------------------------
 *
 * Name:	queue.c
 *
 * RCS:  $Id: queue.c,v 1.7 2002/10/22 01:30:40 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Internal queue functions, display functions, debugging
 *				functions, etc
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/23/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/queue-api.h>
#include "queue.h"

int q_isInited = 0;

/*-----------------------------------------------------------------
 * Name:	q_initQueue
 *
 * Description:	initialize the queue structures
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/23/01
 *
 *-----------------------------------------------------------------
 */
void
q_initQueue()
{
	uint32	i;

    if (q_isInited)
        panic("duplicate call to q_initQueue");

    q_isInited = 1;

	bzero(&queues, sizeof(queues_t));
	queues.num_queues_avail = Q_MAX_QUEUES;
	queues.num_nodes_avail = Q_NUM_NODES;

	for (i=0; i<Q_MAX_QUEUES; i++)
		queues.head_node[i].index = i;

	for (i=0; i<Q_NUM_NODES-1; i++) {
		queues.nodes[i].next = &(queues.nodes[i+1]);
	}
	queues.nodes[Q_NUM_NODES-1].next = 0;

	queues.next_free_node = &(queues.nodes[0]);
	queues.next_free_queue = 0;

}	/* q_initQueue() */

/*-----------------------------------------------------------------
 * Name:	q_getFreeNode()
 *
 * Description:	Get a free node from the pool
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/24/01
 *
 *-----------------------------------------------------------------
 */
q_nodePtr_t						/* returns pointer to new node or NULL */
q_getFreeNode()
{
	q_nodePtr_t temp;

	temp = queues.next_free_node;

	if (temp) {
		queues.next_free_node = queues.next_free_node->next;
		queues.num_nodes_avail--;
	}

	return (temp);

}	/* q_getFreeNode() */

/*-----------------------------------------------------------------
 * Name:	q_displayQueue()
 *
 * Description:	Display a queue.  Not very practical except for testing
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/23/01
 *
 *-----------------------------------------------------------------
 */
void
q_displayQueue(
	q_headNodePtr_t	head_ptr		/* queue head pointer */
)
{
	q_nodePtr_t	q_ptr;
	uint32		index, i;

	index = head_ptr->index;

	printf("Queue %d : head_node = %p, read_pointer = %p, owner = %p\n",
            index, head_ptr, head_ptr->read_ptr, head_ptr->owner);
	for (i=0, q_ptr = head_ptr->next; i<head_ptr->size;
												i++, q_ptr = q_ptr->next) {
		printf("           Node: %p (0x%llx, 0x%llx)\n",
								q_ptr, q_ptr->item1, q_ptr->item2);
	}
	printf("\n");

}	/* q_displayQueue() */

/*-----------------------------------------------------------------
 * Name:	q_displayStats()
 *
 * Description:	Display top level queue stats.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/23/01
 *
 *-----------------------------------------------------------------
 */
void
q_displayStats()
{
	uint32	i;

	printf("Total number of queues = %d\n", Q_MAX_QUEUES);
	printf("Number of active queues = %d\n",
									Q_MAX_QUEUES-queues.num_queues_avail);
	printf("Total number of queue nodes = %d\n", Q_NUM_NODES);
	printf("Number of free queue nodes = %d\n", queues.num_nodes_avail);

	for (i=0; i<Q_MAX_QUEUES; i++) {
		if (queues.head_node[i].in_use)
			q_displayQueue(&(queues.head_node[i]));
	}

}	/* q_displayStats() */

