/*
 *------------------------------------------------------------------------------
 *
 * Name:	raid-api.h
 *
 * RCS:		$Id: raid-api.h,v 1.24 2002/10/23 18:37:03 nitinb Exp $
 *
 * Copyright (C) 2001, 2002 Agile Storage Inc.
 *
 * Description:	RAID config manager API definitions.
 *
 * Created by:	Tiffany Chiang
 *
 * Date Created: 4/9/02
 *
 *------------------------------------------------------------------------------
 */
#ifndef __RAID_API_H__
#define __RAID_API_H__

#define RAID_MB			(1024 * 1024)
#define RAID_SECTORS_PER_MB	(1024 * 2)

#define RAID_MAX_LUNS		32

/*
 *-----------------------------------------------------------
 * RAID Result Codes
 *-----------------------------------------------------------
 */
#define RAID_RCODE_SUCCESS			0
#define RAID_RCODE_NOT_OPEN			1
#define RAID_RCODE_NO_DEVICE			2
#define RAID_RCODE_BUSY				3
#define RAID_RCODE_SDM_ERROR			4
#define RAID_RCODE_IO_ERROR			5
#define RAID_RCODE_INVALID_PARAMETERS		6
#define RAID_RCODE_INVALID_PHYSICAL_DEVICE	7
#define RAID_RCODE_INVALID_LUN			8
#define RAID_RCODE_INSUFFICIENT_SPACE		9
#define RAID_RCODE_TOO_MANY_LUNS		10
#define RAID_RCODE_UNABLE_TO_DELETE_LUN		11
#define RAID_RCODE_LUN_IS_BUSY			12
#define RAID_RCODE_CANNOT_OPEN_FILE		13
#define RAID_RCODE_PARSE_ERROR			14
#define RAID_RCODE_PHYSDEV_UNCONFIGURED		15
#define RAID_RCODE_DISK_UPGRADE_NOTSUPPORTED	16
#define RAID_RCODE_NOT_IMPLEMENTED		20

/*
 *-----------------------------------------------------------
 * RAID Controller Message Types
 *-----------------------------------------------------------
 */
typedef enum {
	/* RAID config manager miscellaneous messages */
	RAID_MID_GET_LIST_REQ = RAID_MSG_BASE,		// 10112
	RAID_MID_GET_LIST_RSP,				// 10113
	RAID_MID_GET_INFO_REQ,				// 10114
	RAID_MID_GET_INFO_RSP,				// 10115
	RAID_MID_OPEN_REQ,				// 10116
	RAID_MID_OPEN_RSP,				// 10117
	RAID_MID_CLOSE_REQ,				// 10118
	RAID_MID_CLOSE_RSP,				// 10119

	RAID_MID_SET_DEBUG_LEVEL_REQ, 			// 10120
	RAID_MID_SET_DEBUG_LEVEL_RSP, 			// 10121

	/* RAID Controller Messages */
	RAID_MID_CTRL_INFO_REQ,				// 10122
	RAID_MID_CTRL_INFO_RSP,				// 10123
	RAID_MID_CTRL_GETPARAM_REQ, 			// 10124
	RAID_MID_CTRL_GETPARAM_RSP, 			// 10125
	RAID_MID_CTRL_SETPARAM_REQ, 			// 10126
	RAID_MID_CTRL_SETPARAM_RSP, 			// 10127
	RAID_MID_CTRL_GETSTATE_REQ, 			// 10128
	RAID_MID_CTRL_GETSTATE_RSP, 			// 10129
	RAID_MID_CTRL_SETSTATE_REQ,			// 10130
	RAID_MID_CTRL_SETSTATE_RSP,			// 10131
	RAID_MID_CTRL_CLRCFG_REQ, 			// 10132
	RAID_MID_CTRL_CLRCFG_RSP,			// 10133
	RAID_MID_CTRL_RESET_REQ, 			// 10134
	RAID_MID_CTRL_RESET_RSP, 			// 10135
	RAID_MID_CTRL_CFGDATA_REQ, 			// 10136
	RAID_MID_CTRL_CFGDATA_RSP, 			// 10137
	RAID_MID_CTRL_FWUPGD_REQ,			// 10138
	RAID_MID_CTRL_FWUPGD_RSP,			// 10139
	RAID_MID_CTRL_SETCLOCK_REQ,			// 10140
	RAID_MID_CTRL_SETCLOCK_RSP,			// 10141
	RAID_MID_CTRL_FLUCACHE_REQ, 			// 10142
	RAID_MID_CTRL_FLUCACHE_RSP, 			// 10143
	/* Physical Device Messages */
	RAID_MID_PHYDEV_LIST_REQ, 			// 10144
	RAID_MID_PHYDEV_LIST_RSP, 			// 10145
	RAID_MID_PHYDEV_INFO_REQ, 			// 10146
	RAID_MID_PHYDEV_INFO_RSP, 			// 10147
	RAID_MID_PHYDEV_STATS_REQ, 			// 10148
	RAID_MID_PHYDEV_STATS_RSP, 			// 10149
	RAID_MID_PHYDEV_STATE_REQ,			// 10150
	RAID_MID_PHYDEV_STATE_RSP,			// 10151
	RAID_MID_PHYDEV_FWUPGD_REQ,			// 10152
	RAID_MID_PHYDEV_FWUPGD_RSP,			// 10153
	RAID_MID_PHYDEV_ID_REQ,				// 10154
	RAID_MID_PHYDEV_ID_RSP,				// 10155
	RAID_MID_PHY_PASSTHRU_REQ,			// 10156
	RAID_MID_PHY_PASSTHRU_RSP,			// 10157
	/* Logical Device Messages */	
	RAID_MID_LOGDEV_LIST_REQ,			// 10158
	RAID_MID_LOGDEV_LIST_RSP, 			// 10159
	RAID_MID_LOGDEV_INFO_REQ, 			// 10160
	RAID_MID_LOGDEV_INFO_RSP, 			// 10161
	RAID_MID_LOGDEV_PHYLIST_REQ, 			// 10162
	RAID_MID_LOGDEV_PHYLIST_RSP, 			// 10163
	RAID_MID_LOGDEV_STATS_REQ,			// 10164
	RAID_MID_LOGDEV_STATS_RSP,			// 10165
	RAID_MID_LOGDEV_ID_REQ,				// 10166
	RAID_MID_LOGDEV_ID_RSP,				// 10167
	/* Logical Device/LUN Config. Messages */
	RAID_MID_LUN_CREATE_REQ,			// 10168
	RAID_MID_LUN_CREATE_RSP, 			// 10169
	RAID_MID_LUN_CREATE_SIMPLE_REQ,			// 10170
	RAID_MID_LUN_CREATE_SIMPLE_RSP,			// 10171
	RAID_MID_LUN_DELETE_REQ, 			// 10172
	RAID_MID_LUN_DELETE_RSP,			// 10173
	RAID_MID_LUN_INIT_REQ,				// 10174
	RAID_MID_LUN_INIT_RSP,				// 10175
	RAID_MID_LUN_CHECK_REQ,				// 10176
	RAID_MID_LUN_CHECK_RSP,				// 10177
	RAID_MID_LUN_CACHE_REQ,				// 10178
	RAID_MID_LUN_CACHE_RSP,				// 10179
	/* LUN Masking Messages */
	RAID_MID_LUNMASK_INFO_REQ,			// 10180
	RAID_MID_LUNMASK_INFO_RSP,			// 10181
	RAID_MID_LUNMASK_ADD_REQ,			// 10182
	RAID_MID_LUNMASK_ADD_RSP,			// 10183
	RAID_MID_LUNMASK_DEL_REQ,			// 10184
	RAID_MID_LUNMASK_DEL_RSP,			// 10185
	/* Misc. Messages */
	RAID_MID_LUN_CANCELBP_REQ,			// 10186
	RAID_MID_LUN_CANCELBP_RSP,			// 10187
	RAID_MID_LUN_BPLIST_REQ, 			// 10188
	RAID_MID_LUN_BPLIST_RSP, 			// 10189
	RAID_MID_ENCLOSE_LIST_REQ,			// 10190
	RAID_MID_ENCLOSE_LIST_RSP,			// 10191
	RAID_MID_ENCLOSE_SCAN_REQ,			// 10192
	RAID_MID_ENCLOSE_SCAN_RSP,			// 10193
	RAID_MID_UPDATE_DRV_FW_REQ,			// 10194
	RAID_MID_UPDATE_DRV_FW_RSP,			// 10195
	/* Wildcard error response */
	RAID_MID_ERROR_RSP,				// 10196
	RAID_MSG_BASE_END				// 10197
} raid_msgTypes_t;


/*
 *-----------------------------------------------------------
 * RAID Controller Id
 *-----------------------------------------------------------
 */
#define RAID_CTRL_ID_LEN		64

typedef char raid_ctrl_id_t[RAID_CTRL_ID_LEN+1];

/*
 *-----------------------------------------------------------
 * RAID Message Header
 *-----------------------------------------------------------
 */
typedef struct _raid_msgHdr
{
	raid_msgTypes_t	mid;				// message ID
	uint64		cookie;				// Client transaction handle/cookie
	raid_ctrl_id_t	id;				// controller ID
} raid_msgHdr_t;

/* 
 ******************************************************************
 *
 * Messages for RAID controller discovery
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Get RAID List Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getListReq_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_GET_LIST_REQ
} raid_getListReqMsg_t;

typedef struct _raid_getListRsp_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_GET_LIST_RSP
	uint32			rCode;			// return code 
	uint32			numEntries;		// Number of entries in idList
	raid_ctrl_id_t		idList[1];		// Variable len list == numEntries
} raid_getListRspMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_GET_INFO_REQ
} raid_getInfoReqMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_GET_INFO_RSP
	uint32			rCode;			// return code 
	scsi_id_t		scsi_id;		// 64-bit SCSI ID of device 
	uint64			wwn;			// WWN of device
	uint32			lun;			// LUN of device 
#define RAID_DEV_STATE_CLOSED      0
#define RAID_DEV_STATE_OPEN_IDLE   1
#define RAID_DEV_STATE_OPEN_BUSY   2
#define RAID_DEV_STATE_CLOSING     3
	uint32			state;			// state of device
} raid_getInfoRspMsg_t;

/*
 *-----------------------------------------------------------
 * Connection Establishment/Teardown Messages
 *-----------------------------------------------------------
 */
typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_OPEN_REQ 
} raid_openReqMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_OPEN_RSP
	uint32			rCode;
	scsi_id_t		scsi_id;		// 64-bit SCSI ID of open device 
	uint64			wwn;			// WWN of open device 
	uint32			lun;			// LUN of open device
} raid_openRspMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_CLOSE_REQ
} raid_closeReqMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_CLOSE_RSP
	uint32			rCode;			// return code 
} raid_closeRspMsg_t;

/*
 *-----------------------------------------------
 * Driver admin messages
 *-----------------------------------------------
 */
typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_SET_DEBUG_LEVEL_REQ
	uint32			debug_level;
} raid_setDebugLevelReqMsg_t;

typedef struct {
	raid_msgHdr_t		header;			// RAID_MID_SET_DEBUG_LEVEL_RSP
	uint32			rCode;			// return code 
} raid_setDebugLevelRspMsg_t;

/*
 *-----------------------------------------------
 * Generic response message
 *-----------------------------------------------
 */
typedef struct
{
	raid_msgHdr_t		header;		
	uint32			rCode;		
} raid_genericRspMsg_t;

/* 
 ******************************************************************
 *
 * Messages for getting RAID controller Info./State/Parameters
 *
 ******************************************************************
 */

#define RAID_CTRL_NAME_LEN	16
#define RAID_CTRL_TYPE_LEN	16
#define RAID_CTRL_SERIAL_LEN	16
#define RAID_CTRL_VENDOR_LEN	16
#define RAID_CTRL_MODEL_LEN	20
#define RAID_CTRL_FWVER_LEN	20
#define RAID_CTRL_HWVER_LEN	20

/*
 *-----------------------------------------------------------
 * Get RAID Controller Info. Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getCtrlInfoReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_INFO_REQ
} raid_ctrlInfoReqMsg_t;

typedef struct _raid_getCtrlInfoRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_INFO_RSP
	uint32			rCode;			// return code 
	char			ctrlType[RAID_CTRL_TYPE_LEN+1];			
	char			ctrlName[RAID_CTRL_NAME_LEN+1];			
	char			serialNumber[RAID_CTRL_SERIAL_LEN+1];	
	char			vendorName[RAID_CTRL_VENDOR_LEN+1];		
	char			hardwareVer[RAID_CTRL_HWVER_LEN+1];
	char			firmwareVer[RAID_CTRL_FWVER_LEN+1];
	uint32			cacheInMB;
	uint32 			memoryInMB;
	uchar8			bbuPresent;
	uint32			numDriveChannels;
	uint32			maxDrivesPerChannel;
	uint64			ctrlWwn;
	uint64			dualCtrlWwn;
	uint32			numLogicalDevices;
	uint32			numPhysicalDevices;
} raid_ctrlInfoRspMsg_t;


/*
 *-----------------------------------------------------------
 * RAID Controller Parameter Structure
 *-----------------------------------------------------------
 */

typedef struct _raid_ctrlParam
{
	uchar8			readAhead;		// Is read ahead enabled
	uchar8			reasRestrict;		// Is reassign restricted to 1 block enabled
	uchar8			trueVerify;		// Is true verification of data enabled
	uchar8			writeVerify;		// Is write through verification enabled
	uchar8			bgInit;			// Is background init enabled
	uchar8			superReadAhead;		// Is super read ahead enabled
	uchar8			coalOptimal;		// Is coalescing optimisation enabled
	uchar8			autoRebuild;		// Is auto rebuild enabled
	uchar8			operFault;		// Is operation fault manage enabled
	short16			rebuildRate;		// Default rebuild rate (0 - 50)
	short16			initRate;		// Default init rate (0 - 50)
	short16			consistencyCheckRate;   // Default Consistency Check rate (0 - 50)
	short16			startupMode;		// Disk startup mode
#define RAID_STARTUP_AUTO	0			// automatic
#define RAID_STARTUP_POWER	1			// on power
#define RAID_STARTUP_CMD	2			// on command
	short16			startupNumDev;		// Startup number of devices (1 - 8)
	short16			startupDelay;		// Startup delay (0 - 255)
	short16			scsiStartupDelay;	// SCSI startup delay (0 - 255)
	uchar8			vendorTUR;		// Is vendor unique TUR enabled
	uchar8			disCCInvLun;		// Is disable CC for invalid LUN enabled
	uchar8			noPause;		// Is no pause on controller nor ready enabled
	uchar8			queueFullBusy;		// Is on queue full give busy enabled
	uchar8			disBusyFailback;	// Is disable busy on Failback enabled
	uchar8			nodeNameRet;		// Is node name retention enabled
	uchar8			safteofups;		// Is SAF-TE use of UPS enabled
	short16			topology;		// Topology
#define RAID_TOPO_INACTPORT	0			// Inactive Port
#define RAID_TOPO_MULTIPORT	1			// Multiport
#define RAID_TOPO_CLUSTER	2			// Clustering
#define RAID_TOPO_MULTITID	3			// MultiTID
#define RAID_TOPO_MASSLAV	4			// Master Slave
	uchar8			resetProp;		// Is reset propagation enabled
	short16			rs232PortType;		// RS232 port type
#define RAID_PORT_DEBUG		0			// debug
#define RAID_PORT_SLPVT100	3			// SLP/VT 100
	short16			rs232BaudRate;		// RS232 baud rate (2400, 4800, 9600, or 19200)
	uchar8			smartLrgTrans;		// Is smart large transfer enabled
	short16			frameSize;		// 2048, 1024, or 512
	short16			pciLatCtrl;		// PCI latency control
#define RAID_PLC_LONG		0			// long
#define RAID_PLC_MEDIUM		1			// medium
#define RAID_PLC_SHORT		2			// short
	uchar8			autoFailback;		// Is auto failback enabled
	uchar8			forceSimplex;		// Is force simplex enabled
	uchar8			conserveCache;		// Is conservative cache mode enabled
	uchar8			noRSTCOM;		// Is simplex no RSTCOM enabled
	short16			queueLimit;		// Queue limit (1 - 230)
	uchar8			ctrl0Port0;		// Is C0P0 enabled
	uchar8			ctrl0Port1;		// Is C0P1 enabled
	uchar8			ctrl1Port0;		// Is C1P0 enabled
	uchar8			ctrl1Port1;		// Is C1P1 enabled
	uchar8			ctrl0Port0LoopId;
	uchar8			ctrl0Port1LoopId;
	uchar8			ctrl1Port0LoopId;
	uchar8			ctrl1Port1LoopId;
	uchar8			debugDump;		// Is debug dump enabled
	short16			rofInterval;		// ROF rearm interval
#define RAID_ROFI_NEVER		0			// never
#define RAID_ROFI_3min		1			// 3 minutes
#define RAID_ROFI_5min		2			// 5 minutes
#define RAID_ROFI_15min		3			// 15 minutes
#define RAID_ROFI_30min		4			// 30 minutes
#define RAID_ROFI_60min		5			// 60 minutes
#define RAID_ROFI_90min		6			// 90 minutes
#define RAID_ROFI_2hour		7			// 2 hours
#define RAID_ROFI_4hour		8			// 4 hours
#define RAID_ROFI_8hour		9			// 8 hours
#define RAID_ROFI_12hour	10			// 12 hours
#define RAID_ROFI_16hour	11			// 16 hours
#define RAID_ROFI_24hour	12			// 24 hours
#define RAID_ROFI_2day		13			// 2 days
#define RAID_ROFI_4day		14			// 4 days
#define RAID_ROFI_7day		15			// 7 days
	short16			rofReboot;		// ROF reboot count (1 - 15, 0 means disable)
} raid_ctrlParam_t;


/*
 *-----------------------------------------------------------
 * Get RAID Controller Parameters Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getCtrlParamReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_GETPARAM_REQ
} raid_getCtrlParamReqMsg_t;

typedef struct _raid_getCtrlParamRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_GETPARAM_RSP
	uint32			rCode;			// return code 
	raid_ctrlParam_t	params;
} raid_getCtrlParamRspMsg_t;


/*
 *-----------------------------------------------------------
 * Set RAID Controller Parameters Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_setCtrlParamReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_SETPARAM_REQ
	raid_ctrlParam_t	params;
} raid_setCtrlParamReqMsg_t;

typedef struct _raid_setCtrlParamRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_SETPARAM_RSP
	uint32			rCode;			// return code 
} raid_setCtrlParamRspMsg_t;


/*
 *-----------------------------------------------------------
 * Get RAID Controller State Messages
 *-----------------------------------------------------------
 */

#define RAID_CTRL_STATE_LEN	256

typedef struct _raid_getCtrlStateReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_GETSTATE_REQ
} raid_getCtrlStateReqMsg_t;

typedef struct _raid_getCtrlStateRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_GETSTATE_RSP
	uint32			rCode;			// return code 
	char			masterSlaveState[RAID_CTRL_STATE_LEN+1];
	char			masterSlaveAdditionInfo[RAID_CTRL_STATE_LEN+1];
	char			parterState[RAID_CTRL_STATE_LEN+1];
} raid_getCtrlStateRspMsg_t;



/* 
 ******************************************************************
 *
 * Messages for getting physical device Info.
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Physical Device Id
 *-----------------------------------------------------------
 */
typedef struct _raid_phydev_id
{
	short16			channelNumber;
	short16			targetId;
} raid_phydev_id_t;


/*
 *-----------------------------------------------------------
 * Get Physical Device List Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getPhyDevListReq_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_LIST_REQ
} raid_phyDevListReqMsg_t;

typedef struct _raid_getPhyDevListRsp_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_LIST_RSP
	uint32			rCode;			// return code 
	uint32			numEntries;		// Number of entries in idList
	raid_phydev_id_t	idList[1];		// Variable len list == numEntries
} raid_phyDevListRspMsg_t;


/*
 *-----------------------------------------------------------
 * Get Physical Device Info. Messages
 *-----------------------------------------------------------
 */

#define RAID_PHYDEV_SERIAL_LEN	16
#define RAID_PHYDEV_VENDOR_LEN	16
#define RAID_PHYDEV_MODEL_LEN	20
#define RAID_PHYDEV_MCVER_LEN	16

typedef struct _raid_getPhyDevInfoReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_INFO_REQ
	raid_phydev_id_t	id;
} raid_phyDevInfoReqMsg_t;

typedef struct _raid_getPhyDevInfoRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_INFO_RSP
	uint32			rCode;			// return code 
	raid_phydev_id_t	id;
	char			vendorName[RAID_PHYDEV_VENDOR_LEN+1];		
	char			modelName[RAID_PHYDEV_MODEL_LEN+1];		
	char			serialNumber[RAID_PHYDEV_SERIAL_LEN+1];
	char			microcodeVer[RAID_PHYDEV_MCVER_LEN+1];
	uint64			wwn;
	short16			enclosure;
	short16			row;
	short16			column;
	ushort16		rdn;
	uint32 			physicalSizeInMB;
	uint32 			usableSizeInMB;
	uint32 			freeSizeInMB;
	uint32 			sectorSizeInByte;
	short16			deviceState;
#define RAID_PHYDEVSTATE_UNCONFIGURED   0x00
#define RAID_PHYDEVSTATE_OPTIMAL	0x01
#define RAID_PHYDEVSTATE_REBUILD	0x03
#define RAID_PHYDEVSTATE_FAILED		0x08
#define RAID_PHYDEVSTATE_CRITICAL	0x09
#define RAID_PHYDEVSTATE_REBUILDING     0x0b
#define RAID_PHYDEVSTATE_GLOBHOTSPARE   0x21
#define RAID_PHYDEVSTATE_PERMHOTSPARE   0x61
	short16			portInUse;
} raid_phyDevInfoRspMsg_t;


/*
 *-----------------------------------------------------------
 * Get Physical Device Stats Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getPhyDevStatsReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_STATS_REQ
	raid_phydev_id_t	id;
	int32			reset_on_read;		// Reset counters
} raid_phyDevStatsReqMsg_t;

typedef struct _raid_getPhyDevStatsRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_STATS_RSP
	uint32			rCode;			// return code 
	raid_phydev_id_t	id;
	uint32			numMsecSinceStart;
	/* Total Read/Write Performance Data including Cache */
	uint32			numReads;
	uint32			numWrites;
	uint32			dataReadsInKB;
	uint32			dataWritesInKB;
	/* Error Data */
	uint32			numSoftErrors;
	uint32			numHardErrors;
	uint32			numParityErrors;
	uint32			numMiscErrors;
} raid_phyDevStatsRspMsg_t;



/* 
 ******************************************************************
 *
 * Messages for configuring physical device. 
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Modify a Physical Device State Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_setPhyDevStateReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_STATE_REQ
	raid_phydev_id_t	id;			// channelNumber -1 indicate all devices
	short16			state;			// New State
#define RAID_STATE_ONLINE	0			// Online or Optimal
#define RAID_STATE_OFFLINE	1			// Offline
#define RAID_STATE_SPARE	2			// Hotspare or Spare
} raid_setPhyDevStateReqMsg_t;

typedef struct _raid_setPhyDevStateRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_STATE_RSP
	uint32			rCode;			// return code 
} raid_setPhyDevStateRspMsg_t;


/*
 *-----------------------------------------------------------
 * Upgrade a Physical Device Firmware Messages
 *-----------------------------------------------------------
 */

#define RAID_FW_FILENAME_LEN	128

typedef struct _raid_phyDevFwUpgdReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_FWUPGD_REQ
	raid_phydev_id_t	id;
	char			fileName[RAID_FW_FILENAME_LEN+1];
} raid_phyDevFwUpgdReqMsg_t;

typedef struct _raid_phyDevFwUpgdRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_FWUPGD_RSP
	uint32			rCode;			// return code 
} raid_phyDevFwUpgdRspMsg_t;



/* 
 ******************************************************************
 *
 * Messages for getting logical device Info.
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Logical Device Id
 *-----------------------------------------------------------
 */
typedef short16 raid_logdev_id_t;


/*
 *-----------------------------------------------------------
 * Get Logical Device List Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getLogDevListReq_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_LIST_REQ
} raid_logDevListReqMsg_t;

typedef struct _raid_getLogDevListRsp_msg 
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_LIST_RSP
	uint32			rCode;			// return code 
	uint32			numEntries;		// Number of entries in idList
	raid_logdev_id_t	idList[1];		// Variable len list == numEntries
} raid_logDevListRspMsg_t;


/*
 *-----------------------------------------------------------
 * Get Logical Device Info. Messages
 *-----------------------------------------------------------
 */
/*
 *-----------------------------------------------------------
 * RAID Level Codes
 *-----------------------------------------------------------
 */
#define RAID_LEVEL_RAID_0				0x00
#define RAID_LEVEL_RAID_1				0x01
#define RAID_LEVEL_RAID_3_RIGHT_ASYMETRIC_PARITY	0x03
#define RAID_LEVEL_RAID_5_RIGHT_ASYMETRIC_PARITY	0x05
#define RAID_LEVEL_RAID_0_PLUS_1			0x06
#define RAID_LEVEL_RAID_JBOD				0x07
#define RAID_LEVEL_RAID_3_FIXED_PARITY			0x09
#define RAID_LEVEL_RAID_3_LEFT_ASYMETRIC_PARITY		0x0b
#define RAID_LEVEL_RAID_5_LEFT_ASYMETRIC_PARITY		0x0d

typedef struct _raid_getLogDevInfoReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_INFO_REQ
	raid_logdev_id_t	id;
} raid_logDevInfoReqMsg_t;

typedef struct _raid_getLogDevInfoRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_INFO_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	uint32			raidLevel;
	uint32 			physicalSizeInMB;
	uint32 			actualSizeInMB;
	uint32 			stripeSizeInKB;
	uint32			cacheLineSizeInKB;
	short16			deviceState;
#define RAID_LOGDEVSTATE_UNCONFIGURED   0x00
#define RAID_LOGDEVSTATE_OPTIMAL	0x01
#define RAID_LOGDEVSTATE_FAILED		0x08
#define RAID_LOGDEVSTATE_CRITICAL	0x09
#define RAID_LOGDEVSTATE_REBUILDING     0x0b
	uchar8			readCacheEnabled;	// Is read cache enabled
	uchar8			writeCacheEnabled;	// Is write cache enabled
	uchar8			isInitialized;		// Is initialized
	uchar8			hasBgTask;		// Has any background tasks
} raid_logDevInfoRspMsg_t;


/*
 *-----------------------------------------------------------
 * Logical Device Physical Drive Info. Structure
 *-----------------------------------------------------------
 */

typedef struct _raid_logDevPhyDrv
{
	ushort16		rdn;
	short16			channelNumber;
	short16			targetId;
	uint32			startBlock;
	uint32			usedBlocks;
	uint32			totalBlocks;
} raid_logDevPhyDrv_t;


/*
 *-----------------------------------------------------------
 * Get Logical Device Physical Drive Info. Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getLogDevPhyListReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_PHYLIST_REQ
	raid_logdev_id_t	id;
} raid_logDevPhyListReqMsg_t;

typedef struct _raid_getLogDevPhyListRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_PHYLIST_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	short16			numDevices;		// Number of entries in driveList
	raid_logDevPhyDrv_t	driveList[1];		// Variable len list == numDevices
} raid_logDevPhyListRspMsg_t;


/*
 *-----------------------------------------------------------
 * Get Logical Device Stats Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getLogDevStatsReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_STATS_REQ
	raid_logdev_id_t	id;
} raid_logDevStatsReqMsg_t;

typedef struct _raid_getLogDevStatsRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_STATS_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	uint32			numMsecSinceStart;
	/* Total Read/Write Performance Data including Cache */
	uint32			numReads;
	uint32			numWrites;
	uint32			dataReadsInKB;
	uint32			dataWritesInKB;
	/* Error Data */
	uint32			numSoftErrors;
	uint32			numHardErrors;
	uint32			numParityErrors;
	uint32			numMiscErrors;
} raid_logDevStatsRspMsg_t;



/* 
 ******************************************************************
 *
 * Messages for create/remove/initialize Logical Device (LUN)
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Logical Device Definition Structure
 *-----------------------------------------------------------
 */
typedef struct _raid_logDev_def
{
	uint32			raidLevel;
	uint32 			sizeInMB;
	uchar8			cacheEnabled;		// Is write cache enabled
	uchar8			autoInit;		// Is auto init enabled
	uint32			stripeSizeInKB;
	short16			numOfPhyDevices;	// Number of entries in phyDevList
	raid_phydev_id_t	phyDevList[1];		// Variable len list == numOfPhyDevices
} raid_logDev_def_t;


/*
 *-----------------------------------------------------------
 * Create LUN Messages
 *	Create a new LUN using the given Logical Device 
 *	Definition. LUN is created w/o LUN Masking.
 *-----------------------------------------------------------
 */

typedef struct _raid_createLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CREATE_REQ
	raid_logDev_def_t	lundef;
} raid_createLunReqMsg_t;

typedef struct _raid_createLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CREATE_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
} raid_createLunRspMsg_t;


/*
 *-----------------------------------------------------------
 * Create Simple LUN Messages
 *	Create n LUNs of the same RAID type.
 *	LUNs are created w/o LUN Masking.
 *-----------------------------------------------------------
 */

typedef struct _raid_createSimpleLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CREATE_SIMPLE_REQ
	uint32			lunCount;		// Number of LUNs to create
	uint32			raidLevel;
	uint32 			sizeInMB;
	uint32			stripeSizeInKB;
} raid_createSimpleLunReqMsg_t;

typedef struct _raid_createSimpleLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CREATE_SIMPLE_RSP
	uint32			rCode;			// return code 
} raid_createSimpleLunRspMsg_t;


/*
 *-----------------------------------------------------------
 * Delete LUN Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_deleteLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_DELETE_REQ
	raid_logdev_id_t	id;
} raid_deleteLunReqMsg_t;

typedef struct _raid_deleteLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_DELETE_RSP
	uint32			rCode;			// return code 
} raid_deleteLunRspMsg_t;


/*
 *-----------------------------------------------------------
 * Init LUN Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_initLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_INIT_REQ
	short16			type;			// Init Type
#define RAID_INITTYPE_ONE	0			// Init one LUN, defined in id
#define RAID_INITTYPE_ALL	1			// Init all LUNs
#define RAID_INITTYPE_ALLNOT	2			// Init all uninitialized LUNs
	raid_logdev_id_t	id;
} raid_initLunReqMsg_t;

typedef struct _raid_initLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_INIT_RSP
	uint32			rCode;			// return code 
	uint32			init_started_count;	
} raid_initLunRspMsg_t;


/*
 *-----------------------------------------------------------
 * Consistency check a Logical Device (LUN) Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_checkLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CHECK_REQ
	short16			type;			// Check Type
#define RAID_CHKTYPE_CHECK	0			// Check only
#define RAID_CHKTYPE_CHKRES	1			// Check and Restore
	raid_logdev_id_t	id;
} raid_checkLunReqMsg_t;

typedef struct _raid_checkLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CHECK_RSP
	uint32			rCode;			// return code 
} raid_checkLunRspMsg_t;

/*
 *-----------------------------------------------------------
 * Set cache parameter for a Logical Device (LUN) Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_cacheLunReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CACHE_REQ
	raid_logdev_id_t	id;
	uint32			rc_enabled;		// Read Cache enabled, 0 or 1
	uint32			wc_enabled;		// Write Cache enabled, 0 or 1
} raid_cacheLunReqMsg_t;

typedef struct _raid_cacheLunRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CACHE_RSP
	uint32			rCode;			// return code 
} raid_cacheLunRspMsg_t;

/* 
 ******************************************************************
 *
 * Messages for add/delete LUN Masking. 
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Get LUN Masking Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getLunMaskReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_INFO_REQ
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
} raid_getLunMaskReqMsg_t;

typedef struct _raid_getLunMaskRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_INFO_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
	short16			numOfHostWwns;		// Number of entries in hostWwnList
	uint64			hostWwnList[1];		// Variable len list == numOfHostWwns
} raid_getLunMaskRspMsg_t;


/*
 *-----------------------------------------------------------
 * Add LUN Masking Messages
 *	Add the given host wwns to LUN's host wwn table
 *-----------------------------------------------------------
 */

typedef struct _raid_addLunMaskReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_ADD_REQ
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
	short16			numOfHostWwns;		// Number of entries in hostWwnList
	uint64			hostWwnList[1];		// Variable len list == numOfHostWwns
} raid_addLunMaskReqMsg_t;

typedef struct _raid_addLunMaskRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_ADD_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
} raid_addLunMaskRspMsg_t;


/*
 *-----------------------------------------------------------
 * Delete LUN Masking Messages
 *	Delete the given host wwns from LUN's host wwn table
 *-----------------------------------------------------------
 */

typedef struct _raid_delLunMaskReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_DEL_REQ
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
	short16			numOfHostWwns;		// Number of entries in hostWwnList
	uint64			hostWwnList[1];		// Variable len list == numOfHostWwns
} raid_delLunMaskReqMsg_t;

typedef struct _raid_delLunMaskRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUNMASK_DEL_RSP
	uint32			rCode;			// return code 
	raid_logdev_id_t	id;
	short16			ctrlNumber;		// Controller number
	short16			portNumber;
} raid_delLunMaskRspMsg_t;



/* 
 ******************************************************************
 *
 * Messages for configuring RAID controller
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * RAID Controller Clear Configuration Messages
 *	All the LUN/LUN Masking info. is erased.
 *-----------------------------------------------------------
 */

typedef struct _raid_ctrlClrCfgReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_CLRCFG_REQ
} raid_ctrlClrCfgReqMsg_t;

typedef struct _raid_ctrlClrCfgRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_CLRCFG_RSP
	uint32			rCode;			// return code 
} raid_ctrlClrCfgRspMsg_t;


/*
 *-----------------------------------------------------------
 * Set Controller State Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_ctrlPartStateReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_SETSTATE_REQ
	short16			type;			// controller type
#define RAID_CTRLTYPE_PRIMARY	0
#define RAID_CTRLTYPE_SECONDARY	1
	short16			state;			// state
#define RAID_STATE_ONLINE	0
#define RAID_STATE_OFFLINE	1
} raid_ctrlPartStateReqMsg_t;

typedef struct _raid_ctrlPartStateRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_SETSTATE_RSP
	uint32			rCode;			// return code 
} raid_ctrlPartStateRspMsg_t;


/*
 *-----------------------------------------------------------
 * Reset Controller Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_ctrlResetReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_RESET_REQ
} raid_ctrlResetReqMsg_t;

typedef struct _raid_ctrlResetRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_RESET_RSP
	uint32			rCode;			// return code 
} raid_ctrlResetRspMsg_t;


/*
 *-----------------------------------------------------------
 * Save/Load RAID Controller Configuration Messages
 *-----------------------------------------------------------
 */

#define RAID_CFG_FILENAME_LEN	128

typedef struct _raid_ctrlConfigReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_CFGDATA_REQ
	short16			type;			// message type
#define RAID_CONFIG_LOAD	0
#define RAID_CONFIG_SAVE	1
	char			fileName[RAID_CFG_FILENAME_LEN+1];
} raid_ctrlConfigReqMsg_t;

typedef struct _raid_ctrlConfigRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_CFGDATA_RSP
	uint32			rCode;			// return code 
} raid_ctrlConfigRspMsg_t;


/*
 *-----------------------------------------------------------
 * Upgrade RAID Controller Firmware Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_ctrlFwUpgdReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_FWUPGD_REQ
	char			fileName[RAID_FW_FILENAME_LEN+1];
} raid_ctrlFwUpgdReqMsg_t;

typedef struct _raid_ctrlFwUpgdRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_FWUPGD_RSP
	uint32			rCode;			// return code 
} raid_ctrlFwUpgdRspMsg_t;



/* 
 ******************************************************************
 *
 * Misc. Messages. 
 *
 ******************************************************************
 */

/*
 *-----------------------------------------------------------
 * Identify a Physical Device Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_idPhyDevReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_ID_REQ
	raid_phydev_id_t	id;
} raid_idPhyDevReqMsg_t;

typedef struct _raid_idPhyDevRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_PHYDEV_ID_RSP
	uint32			rCode;			// return code 
} raid_idPhyDevRspMsg_t;


/*
 *-----------------------------------------------------------
 * Identify a Logical Device Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_idLogDevReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_ID_REQ
	raid_logdev_id_t	id;
} raid_idLogDevReqMsg_t;

typedef struct _raid_idLogDevRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LOGDEV_ID_RSP
	uint32			rCode;			// return code 
} raid_idLogDevRspMsg_t;


/*
 *-----------------------------------------------------------
 * Cancel Background Process Messages
 *	Cancel all background tasks of the given process type 
 *	on ALL LUNs.
 *-----------------------------------------------------------
 */

typedef struct _raid_cancelBpReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CANCELBP_REQ
	short16			type;			// Process Type
#define RAID_BPTYPE_CHECK	0			// Consistency check
#define RAID_BPTYUE_CHECHRES	1			// Check and restore
#define RAID_BPTYPE_INIT	2			// INIT
#define RAID_BPTYPE_BGINIT	3			// BGINIT - background init
#define RAID_BPTYPE_REBUILD	4			// REBUILD 
#define RAID_BPTYPE_ALL		5			// ALL - all background process
} raid_cancelBpReqMsg_t;

typedef struct _raid_cancelBpRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_CANCELBP_RSP
	uint32			rCode;			// return code 
} raid_cancelBpRspMsg_t;


/*
 *-----------------------------------------------------------
 * Background Process Structure
 *-----------------------------------------------------------
 */
typedef struct _raid_backProc
{
	raid_logdev_id_t	id;
	short16			checkProgress;		// complete percentage, or -1 if not active
	short16			initProgress;		// complete percentage, or -1 if not active
	short16			rebuildProgress;	// complete percentage, or -1 if not active
	short16			expandProgress;		// complete percentage, or -1 if not active
} raid_backProc_t;


/*
 *-----------------------------------------------------------
 * Display Background Process Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getBpListReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_BPLIST_REQ
} raid_bpListReqMsg_t;

typedef struct _raid_getBpListRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_LUN_BPLIST_RSP
	uint32			rCode;			// return code 
	short16			numEntries;		// Number of entries in bpList
	raid_backProc_t		bpList[1];		// Variable len list == numEntries
} raid_bpListRspMsg_t;


/*
 *-----------------------------------------------------------
 * Component Status Structure 
 *-----------------------------------------------------------
 */

typedef struct _raid_compStatus
{
	short16			status;
	int32			data1;			// component specific data
	int32			data2;			// component specific data
	int32			data3;			// component specific data
	int32			data4;			// component specific data
} raid_compStatus_t;


/*
 *-----------------------------------------------------------
 * Enclosure (SES) Structure
 *-----------------------------------------------------------
 */
#define RAID_ENC_VENDOR_LEN	16
#define RAID_ENC_PRODUCT_LEN	16
#define RAID_ENC_PRODUCTVER_LEN	16
#define RAID_ENC_PRIPATH_LEN	16
#define RAID_ENC_SECPATH_LEN	16

typedef struct _raid_scsiEnc
{
	short16			encNumber;
	char			vendorID[RAID_ENC_VENDOR_LEN+1];		
	char			productID[RAID_ENC_PRODUCT_LEN+1];	
	char			ProductRev[RAID_ENC_PRODUCTVER_LEN+1];
	uint64			wwn;
	short16			numOfBays;
	short16			status;
	short16			sesAvailable;
	char			primaryPath[RAID_ENC_PRIPATH_LEN+1];
	char			secondaryPath[RAID_ENC_SECPATH_LEN+1];
	short16			numFans;		// Number of entries in fanList, fan
	raid_compStatus_t	fanList[1];		// Variable len list == numFans
	short16			numPSs;			// Number of entries in psList, power supply 
	raid_compStatus_t	psList[1];		// Variable len list == numPSs
	short16			numTemps;		// Number of entries in tempList, temperature
	raid_compStatus_t	tempList[1];		// Variable len list == numTemps
	short16			numAlarms;		// Number of entries in alarmList, alarm
	raid_compStatus_t	alarmList[1];		// Variable len list == numAlarms
	short16			numUPSs;		// Number of entries in upsList, UPS
	raid_compStatus_t	upsList[1];		// Variable len list == numUPSs
} raid_scsiEnc_t;


/*
 *-----------------------------------------------------------
 * Display Enclosure Info Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_getEncloseListReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_ENCLOSE_LIST_REQ
} raid_encloseListReqMsg_t;

typedef struct _raid_getEncloseListRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_ENCLOSE_LIST_RSP
	uint32			rCode;			// return code 
	short16			numOfEnc;		// Number of entries in encList
	raid_scsiEnc_t		encList[1];		// Variable len list == numOfEnc
} raid_encloseListRspMsg_t;


/*
 *-----------------------------------------------------------
 * Scan for new Enclosures and Disks Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_scanEncDiskReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_ENCLOSE_SCAN_REQ
} raid_scanEncDiskReqMsg_t;


/*
 *-----------------------------------------------------------
 * Set real-time clock in RAID controller Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_setCtrlClockReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_SETCLOCK_REQ
} raid_setCtrlClockReqMsg_t;


/*
 *-----------------------------------------------------------
 * Flush write cache in RAID controller Messages
 *-----------------------------------------------------------
 */

typedef struct _raid_flushCtrlCacheReq_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_FLUCACHE_REQ
} raid_flushCtrlCacheReqMsg_t;

typedef struct _raid_flushCtrlCacheRsp_msg
{
	raid_msgHdr_t		header;			// RAID_MID_CTRL_FLUCACHE_RSP
	uint32			rCode;			// return code 
} raid_flushCtrlCacheRspMsg_t;


/*
 *------------------------------------------------------------
 * Physical Device Pass Through Messages
 *------------------------------------------------------------
 */

#define RAID_PASSTHRU_PAGELIST			1
#define RAID_PASSTHRU_BUFFER_RUN_PAGE		2
#define RAID_PASSTHRU_WRITE_ERROR_PAGE		3
#define RAID_PASSTHRU_READ_ERROR_PAGE		4
#define RAID_PASSTHRU_REV_READ_ERROR_PAGE	5
#define RAID_PASSTHRU_VERIFY_ERROR_PAGE		6
#define RAID_PASSTHRU_NON_MEDIUM_ERROR_PAGE	7
#define RAID_PASSTHRU_LAST_N_ERROR_PAGE		8
#define RAID_PASSTHRU_DEFERRED_ERROR_PAGE	9
#define RAID_PASSTHRU_TEMP_PAGE			10
#define RAID_PASSTHRU_CYCLE_COUNTER_PAGE	11
#define RAID_PASSTHRU_APP_CLIENT_PAGE		12
#define RAID_PASSTHRU_TEST_RESULTS_PAGE		13
#define RAID_PASSTHRU_READ_LONG			14
#define RAID_PASSTHRU_WRITE_LONG		15
#define RAID_PASSTHRU_MODE_SENSE		16
#define RAID_PASSTHRU_UPGRADE_DRV_FW		17

#define RAID_PASSTHRU_DEFAULT_RD_WR_SIZE	549

typedef struct _raid_phyPassThruReq_msg
{
	raid_msgHdr_t	header;			// RAID_MID_PHY_PASSTHRU_REQ
	uint32		channelNumber;		// channel number
	uint32		targetId;		// target id
	uint16		cmd;			// pass thro. cmd to execute
	uint32		lba;			// logical block offset
	uint16		dataOut;		// 0 for data in and 1 for data out
	uint16		dataLen;		// size of data to be read/written
	char		data[1];		// data to be sent with command
} raid_phyPassThruReqMsg_t;

typedef struct _raid_phyPassThruRsp_msg
{
	raid_msgHdr_t	header;			// RAID_MID_PHY_PASSTHRU_RSP
	uint32		rCode;			// return code
	uint32		dataLen;		// size of data returned
	char		data[1];	
} raid_phyPassThruRspMsg_t;

/*
 *---------------------------------------
 * SCSI DEVICE PAGE CODES
 *---------------------------------------
 */
#define SCSI_LOGPAGE_CODE_SUPPORTED_PAGES		0x00
#define SCSI_LOGPAGE_CODE_BUFFER_OVER_UNDER_RUN		0x01
#define SCSI_LOGPAGE_CODE_ERROR_COUNTER_WRITE		0x02
#define SCSI_LOGPAGE_CODE_ERROR_COUNTER_READ		0x03
#define SCSI_LOGPAGE_CODE_ERROR_COUNTER_READ_REV	0x04
#define SCSI_LOGPAGE_CODE_ERROR_COUNTER_VERIFY		0x05
#define SCSI_LOGPAGE_CODE_NON_MEDIUM_ERROR		0x06
#define SCSI_LOGPAGE_CODE_LAST_N_ERRORS			0x07
#define SCSI_LOGPAGE_CODE_LAST_N_DEFERRED_ERRORS	0x0B
#define SCSI_LOGPAGE_CODE_TEMPERATURE			0x0D
#define SCSI_LOGPAGE_CODE_START_STOP_CYCLE_COUNTER	0x0E
#define SCSI_LOGPAGE_CODE_APPLICATION_CLIENT		0x0F
#define SCSI_LOGPAGE_CODE_SELF_TEST_RESULTS		0x10

#define SCSI_ERROR_LOGPAGE_NO_DELAY_ERRORS		0x0
#define SCSI_ERROR_LOGPAGE_DELAY_ERRORS			0x1
#define SCSI_ERROR_LOGPAGE_RETRIED_OPS			0x2
#define SCSI_ERROR_LOGPAGE_CORRECTED_ERRORS		0x3
#define SCSI_ERROR_LOGPAGE_NUM_ECC_PROCESSED		0x4
#define SCSI_ERROR_LOGPAGE_BYTES_PROCESSED		0x5
#define SCSI_ERROR_LOGPAGE_UNCORRECTED_ERRORS		0x6
#define SCSI_ERROR_LOGPAGE_FIRST_RESERVED		0x7
#define SCSI_ERROR_LOGPAGE_LAST_RESERVED		0x7FFF

#define ALL_PAGE_CODES	-1

#define RAID_PASSTHRU_CHUNK_SIZE		0x4000
#define DISK_VENDOR_SEAGATE			0x1
#define DISK_VENDOR_HITACHI			0x2
#define DISK_VENDOR_IBM				0x3
#define DISK_VENDOR_UNKNOWN			0xFF

typedef struct _raid_updateDrvFwReq_msg
{
	raid_msgHdr_t	header;			// RAID_MID_UPDATE_DRV_FW_REQ
	uint32		channelNumber;		// channel number
	uint32		targetId;		// target id
	char		filePath[256];		// firmware file path
} raid_updateDrvFwReqMsg_t;

typedef struct _raid_updateDrvFwRsp_msg
{
	raid_msgHdr_t	header;			// RAID_MID_UPDATE_DRV_FW_RSP
	uint32		rCode;			// return code
} raid_updateDrvFwRspMsg_t;

#endif /* __RAID_API_H__ */
