/*
 *-----------------------------------------------------------------
 *
 * Name:	rtc-api.c
 *
 * RCS:  $Id: rtc-api.c,v 1.3 2001/11/28 18:47:35 ericc Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "rtc-api.h"

#ifdef LINUX_TEST
#include <sys/time.h>
#endif
ds1501_t	rtc_info;

uchar8  day_string[7][6] = {
	"Sun", "Mon", "Tue", "Wed", "Thurs", "Fri", "Sat"
};

uchar8	month_string[12][4] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

/* note: for eval builds, rtc_ptr should point to the 
	global defined rtc registers but for now point it to a global var*/
ds1501_t *rtc_ptr = &rtc_info;

/*-----------------------------------------------------------------
 * Name:	rtcGetTimeDay()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */
uint32
rtcGetTime()
{
	uint32 *timeday_ptr = (uint32*)(rtc_ptr->seconds); 
	return *(timeday_ptr);
}

void rtcDisplayTime()
{
	uchar8 hour, minute, second, temp;

	temp = rtc_ptr->hours & RTC_HOUR_MASK >> 4;  /* get tens of hours */
	hour   = rtc_ptr->hours & 0xF + temp * 10;

	temp = rtc_ptr->minutes & RTC_MIN_MASK >> 4;  /* get tens of minutes */
	minute = rtc_ptr->minutes & 0xF + temp * 10;
	
	temp = rtc_ptr->seconds & RTC_SEC_MASK >> 4;  /* get tens of seconds */
	second = rtc_ptr->seconds & 0xF + temp * 10;
	
	printf("%d:%d:%d", hour, minute, second);
	
}

void rtcDisplayDateTime()
{
	uchar8 day, month, date;
	uint32 year, century;

	day = rtc_ptr->day;

	printf("%s ", day_string[rtc_ptr->day]);

	month = (rtc_ptr->month & 0xF) + ((rtc_ptr->month & 0x10) >> 4) * 10;
	printf("%s ", month_string[month]);

	date = ((rtc_ptr->date & 0xF0) >> 4) * 10 + (rtc_ptr->date & 0xF);
	printf("%d ", date);

	rtcDisplayTime();

	year 	= rtc_ptr->year;
	year 	= ((year & 0xF0) >> 4) * 10 + (year & 0xF);
	
	century = rtc_ptr->century;
	century	= ((century & 0xF0) >> 4) * 10 + (century & 0xF);

	printf("%d\n",year + century);

}

/*-----------------------------------------------------------------
 * Name:	rtcGetDate()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */
uint32
rtcGetDate()
{
	uint32 *date_ptr = (uint32*)rtc_ptr->year;
	return *(date_ptr);
}
/*-----------------------------------------------------------------
 * Name:	rtcUIGetTime()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */
void
rtcUIGetDateTime()
{
#ifdef LINUX_TEST
	time_t ttt;
	struct tm *ptr;

	ttt = time(NULL);

	ptr = localtime(&ttt);
	printf("%s",ctime(ptr));
#else
	rtcGetDate();
	rtcDisplayDateTime();
	
#endif
}

/*-----------------------------------------------------------------
 * Name:	()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */
void
rtcSetTime(uint32 value)
{

	*(uint32 *)(rtc_ptr) = value;
}

void
rtcSetAlarmBits(uint32 condition)
{

	switch (condition)
	{
		case RTC_PER_SEC:
			rtc_ptr->alarm_seconds 	|= 0x80;
			rtc_ptr->alarm_minutes 	|= 0x80;
			rtc_ptr->alarm_hours 	|= 0x80;
			rtc_ptr->alarm_day_date |= 0x80;
			break;
		case RTC_MATCH_SEC:
			rtc_ptr->alarm_seconds 	&= ~0x80;
			rtc_ptr->alarm_minutes 	|= 0x80;
			rtc_ptr->alarm_hours 	|= 0x80;
			rtc_ptr->alarm_day_date |= 0x80;
			break;
		case RTC_MATCH_MIN_SEC:
			rtc_ptr->alarm_seconds 	&= ~0x80;
			rtc_ptr->alarm_minutes 	&= ~0x80;
			rtc_ptr->alarm_hours 	|= 0x80;
			rtc_ptr->alarm_day_date |= 0x80;
			break;
		case RTC_MATCH_HOUR_MIN_SEC:
			rtc_ptr->alarm_seconds 	&= ~0x80;
			rtc_ptr->alarm_minutes 	&= ~0x80;
			rtc_ptr->alarm_hours 	&= ~0x80;
			rtc_ptr->alarm_day_date |= 0x80;
			break;
		case RTC_MATCH_DATE_HOUR_MIN_SEC:
			rtc_ptr->alarm_seconds 	&= ~0x80;
			rtc_ptr->alarm_minutes 	&= ~0x80;
			rtc_ptr->alarm_hours 	&= ~0x80;
			rtc_ptr->alarm_day_date &= ~0xC0;
			break;
		case RTC_MATCH_DAY_HOUR_MIN_SEC:
			rtc_ptr->alarm_seconds 	&= ~0x80;
			rtc_ptr->alarm_minutes 	&= ~0x80;
			rtc_ptr->alarm_hours 	&= ~0x80;
			rtc_ptr->alarm_day_date &= ~0x80;
			rtc_ptr->alarm_day_date |= 0x40;
			break;
	}
}

void
rtcSetTimeItem(uint32 item, uchar8 item_value)
{
	uchar8	value = ((item_value / 10) << 4) | (item_value % 10);

	/* first clear out the TE bit so updates don't occur */
	*(uchar8*)(rtc_ptr->control_b) &= RTC_TE_DISABLE;

	if ((item >= RTC_ALARM_SECONDS) && (item <= RTC_ALARM_DAY))
	{
		value &= ~0x80;

		if (item == RTC_ALARM_DAY)
			value &= ~0x40;

		value |= (*(uchar8*)(rtc_ptr + item) & 0xC0);

	}
	else if (item == RTC_MONTH)
	{
		value &= 0x1F;
		value |= (*(uchar8*)(rtc_ptr + item) & 0xE0);
	}
	*(uchar8*)(rtc_ptr + item) = value;

#if 0
	switch (item)
	{
		case RTC_SECONDS:
		rtc_ptr->seconds = value;
		break;
		case RTC_MINUTES:
		rtc_ptr->minutes = value;
		break;
		case RTC_HOURS:
		rtc_ptr->hours = value;
		break;
		case RTC_DAY:
		rtc_ptr->day = item_value;
		break;
		case RTC_DATE:
		rtc_ptr->date = value;
		break;
		case RTC_MONTH:
		rtc_ptr->month = (((item_value / 10) << 4) & 0x1) | 
				(item_value % 10);
		break;
		case RTC_YEAR:
		rtc_ptr->year = value;
		break;
		case RTC_CENTURY:
		rtc_ptr->century = value;
		break;

	}
#endif

	/* reenable the TE bit so updates occur */
	*(uchar8*)rtc_ptr->control_b |= RTC_TE_ENABLE;
}


/*-----------------------------------------------------------------
 * Name:	()
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */
void
rtcRamByteWrite(uchar8 address, uchar8 value)
{

	if (rtcCheckBattery() == RTC_BATTERY_LOW)
		printf("Warning! rtc battery voltage low.\n");

	rtc_ptr->extended_ram_addr = address;
	rtc_ptr->extended_ram_data = value;
}

/*-----------------------------------------------------------------
 * Name:	rtcRamBurstWrite(uchar8 address, uchar8 *value, 
 *				uchar8 burstsize)
 *
 * Description:	Write in Burst mode to the rtc's extended ram.
 *
 * Created by:	ericc
 *
 * Date Created:	9/18/01
 *
 *-----------------------------------------------------------------
 */
void
rtcRamBurstWrite(uchar8 address, uchar8 *src, uchar8 burstsize)
{

	if (rtcCheckBattery() == RTC_BATTERY_LOW)
		printf("Warning! rtc battery voltage low.\n");

	/* set BME bit for burst mode transactions*/
	rtc_ptr->control_b |= 0x20;

	rtc_ptr->extended_ram_addr = address;
	
	while (burstsize)
	{
		rtc_ptr->extended_ram_data = *src;
		src++;
		burstsize--;
	}

	/* unset BME bit */
	rtc_ptr->control_b &= ~0x20;
}

/*-----------------------------------------------------------------
 * Name:	rtcRamBurstRead(uchar8 address, uchar8 *value, 
 *				uchar8 burstsize)
 *
 * Description:	Read in Burst mode from the rtc's extended ram.
 *
 * Created by:	ericc
 *
 * Date Created:	9/18/01
 *
 *-----------------------------------------------------------------
 */
void
rtcRamBurstRead(uchar8 address, uchar8 *dest, uchar8 burstsize)
{
	/* set BME bit for burst mode transactions*/
	rtc_ptr->control_b |= 0x20;

	rtc_ptr->extended_ram_addr = address;
	
	while (burstsize)
	{
		*dest = rtc_ptr->extended_ram_data;
		dest++;
		burstsize--;
	}

	/* unset BME bit */
	rtc_ptr->control_b &= ~0x20;
}

uchar8
rtcRamByteRead(uchar8 address)
{
	rtc_ptr->extended_ram_addr = address;
	return rtc_ptr->extended_ram_data;
}

int32
rtcCheckBattery()
{
	if (rtc_ptr->control_a & 0xC0)
		return -1;	/*low voltage*/
	else return 1;
}


/*-----------------------------------------------------------------
 * Name:	rtcRamBurstRead(uchar8 address, uchar8 *value, 
 *				uchar8 burstsize)
 *
 * Description:	Read in Burst mode from the rtc's extended ram.
 *
 * Created by:	ericc
 *
 * Date Created:	9/18/01
 *
 *-----------------------------------------------------------------
 */
void
rtcSetWatchDog(uchar8 num_seconds, uchar8 num_milli_seconds)
{
	num_milli_seconds *= 10;  /* convert to hundredths of seconds */
	rtc_ptr->watchdog1 = (num_milli_seconds / 10) || (num_milli_seconds % 10);
	rtc_ptr->watchdog2 = (num_seconds / 10) || (num_seconds % 10);
}

void 
rtcDisableWatchDog()
{
	rtc_ptr->watchdog1 = 0;
	rtc_ptr->watchdog2 = 0;
}
