/*  
 *=============================================================================
 * File Name   :    sb1250-dm-mov.c
 *
 *  Description:    
 *
 *  Created    :    09 February 2002
 *
 *  Author     :    Frank Salzmann
 *
 *  RCS        :    $Id: sb1250-dm-mov.c,v 1.4 2002/09/17 22:28:47 rick Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2002
 *=============================================================================
 */

#include "sb1250-dm.h"

int sb1250dm_trace = 0;

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_movData
 *
 * Description: 	DMA's data from source to destination. 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 23, 2001
 * 					
 * Returns     : 	NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_movData_edesc( uint32 dest, 
						eee_desc_t *pktDesc, 
						char *destBuf,
						uint32 chnl_id )
{
	int i;
	int bufCnt;
	int pktLen;
	uint32 offset;
	sb1250dmCfg_t            *cfg;
	sb1250dm_qCtrl_t         *dmqc;
	volatile sb1250dm_desc_t *qCur;
	volatile sb1250dm_regs_t *regs;

	printf("sb1250dm_movData_edesc: enter, dest: %x\n", dest);
	
	cfg = sb1250dmCfg[chnl_id-1];
	if( cfg == 0 ) {
		printf("Invalid port addr: %d\n", EEE_GET_PORT_ID(dest));
		return( EEE_DROP_PACKET );
	}

	regs = &(cfg->regs);
	dmqc = &(cfg->dmqc);

	qCur   = dmqc->qtxCur;
	pktLen = pktDesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = (pktDesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;

	/* 
	 * Check for room on the transmit queue.
	 */
	if( bufCnt > (SB1250_DM_Q_LEN - dmqc->qd)  ) {
		//-- No room on the dma queue.
		printf("DMA Q Full: bufCnt: %d, desc qd: %d, ", bufCnt, dmqc->qd);
		printf("dm_desc_cnt: %llx\n", *(regs->dm_desc_count));	
		dmqc->qFullCnt++;
		return( EEE_DROP_PACKET );
	}

	if( bufCnt == 1 ) {


	}
	else {
		uint32 byteCnt;
		uint32 j = bufCnt-1;

		if( pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
			i=0;
		else
			i=1;

		//-----------------------------------------------------
		//-- Setup descriptor for first buffer in the packet.
		//-----------------------------------------------------

		while(j--) {

		}

		*(regs->dm_desc_count) = bufCnt;	// Give descs to data mover
	}

	return(NFX_OK);
} // sb1250dm_movData_edesc

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_movData_buf
 *
 * Description: 	Transmits a buffer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 09, 2002
 * 					
 * Returns: 		SB1250DM_OK
 *					SB1250DM_ERR
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_movData_buf( uint32 chnl_id, 
					  uint32 dataLen, 
					  char *srcBuf, 
					  char *destBuf,
                      void * cookie)
					  
{
	sb1250dmCfg_t            *cfg;
	sb1250dm_qCtrl_t         *dmqc;
	volatile sb1250dm_desc_t *qCur;
	volatile sb1250dm_regs_t *regs;
    uint32                   index;

	//printf("sb1250dm_movData_buf: enter\n");

	cfg = sb1250dmCfg[chnl_id-1];
	if( cfg == 0 ) {
		printf("Invalid chnl_id: %d\n",chnl_id); 
		return( SB1250DM_ERR );
	}

	if( cfg->inUse == 0 ) {
		printf("SB1250DM: Channel %d not open.\n", cfg->chnlId+1);
		return( SB1250DM_ERR );
	}


	regs = &(cfg->regs);
	dmqc = &(cfg->dmqc);
	qCur = dmqc->qtxCur;

	/* 
	 * Check for room on the transmit queue.
	 */
	if( (SB1250_DM_Q_LEN - dmqc->qd) < 2 ) {
		//-- No room on the tx queue.
		printf("DMQ Full: bufCnt: %d, desc qd: %d\n", 1, dmqc->qd);
		dmqc->qFullCnt++;
		return( SB1250DM_ERR );
	}

	if( sb1250dm_trace ) {
		printf("SB1250dm: Chnl: %d, Desc Cnt: %d\n", chnl_id, *regs->dm_desc_count);
		printf("srcBuf: %llx, destBuf: %llx\n", srcBuf, destBuf);
	}

//printf("datamover, v_dest = %p, v_src = %p, len = 0x%x\n", destBuf, srcBuf, dataLen);
//printf("           p_dest = %p, p_src = %p\n",VIRT_TO_PHYS(destBuf), VIRT_TO_PHYS(srcBuf));

	//--  Setup DMA descriptor.
    if (chnl_id == SB1250DM_CHNL_SAC_TO_LUC) {
    	SB1250DM_DESC_SET_DST_ADDR_LOCAL(qCur, (uint64)destBuf);
    }
    else {
    	SB1250DM_DESC_SET_DST_ADDR(qCur, (uint64)destBuf);
    }
	SB1250DM_DESC_SET_SRC_ADDR(qCur, (uint64)srcBuf);
	SB1250DM_DESC_SET_LEN(qCur, dataLen);

    index = qCur - dmqc->qStart;
//    printf("movData_buf, index = %d, cookie = 0x%llx\n", index, cookie);
    cfg->cb_cookie[index] = cookie;

	dmqc->qd++;
	if( ++qCur >= dmqc->qEnd )
		qCur = dmqc->qStart;

	*(regs->dm_desc_count) = 1;	// Give desc to data mover.

	dmqc->qtxCur  = qCur;

	dmqc->bufCnt++;
	dmqc->byteCnt += dataLen;

	return(SB1250DM_OK);
} // sb1250dm_movData_buf

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_movCleanup
 *
 * Description: 	Cleans up descriptors on DMA mover Q.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 26, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_movCleanup(sb1250dmCfg_t *cfg)
{
	char                     *buf, *dstbuf;
	uint32					 len;
	uint32				     clean_cnt;
	uint32					 chip_descCnt;
	sb1250dm_regs_t          *regs;
	sb1250dm_qCtrl_t		 *dmqc;
	volatile sb1250dm_desc_t *ccur;
	volatile sb1250dm_desc_t *tcur;
    uint32                   index;
    uint32                      i;

	regs = &(cfg->regs);
	dmqc = &(cfg->dmqc);

	ccur = dmqc->qclnCur;
	tcur = dmqc->qtxCur;

	chip_descCnt = *regs->dm_desc_count;

	//-- Sanity check counters.
	if( dmqc->qd < chip_descCnt ) {
		//printf("Err: Tx queue depth:%d  < chip desc cnt:%d\n", 
		//txc->qd, chip_descCnt);
		return 0;
	}

	clean_cnt = dmqc->qd - chip_descCnt;
	while( ccur != tcur && (clean_cnt--) ) {
		
		if( cfg->callback ) {
			SB1250DM_DESC_GET_SRC_ADDR( ccur, buf );
			len = SB1250DM_DESC_GET_LEN(ccur);

            index = ccur - dmqc->qStart;
//          printf("movCleanup, index = %d, cookie = 0x%llx\n",
//                                              index, cfg->cb_cookie[index]);
			cfg->callback(buf, len, cfg->cb_cookie[index]);
            cfg->cb_cookie[index] = 0;
		}

		//-- Decrement number of descriptors on dma queue.
		dmqc->qd--;
		ccur->dw1      = 0;
		ccur->dw2      = 0;

		if( ++ccur >= dmqc->qEnd )
			ccur = dmqc->qStart;
	}
	dmqc->qclnCur = ccur;
	return(SB1250DM_OK);
} // sb1250dm_movCleanup 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_movCompletePoll
 *
 * Description: 	Polling function registered with the EEE layer and 
 * 					called to cleanup the transmit queues.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002 
 *------------------------------------------------------------------------------
 */
void
sb1250dm_movCompletePoll(void *cb, uint32 tlimit)
{
	int            i;
	sb1250dmCfg_t *cfg;

#if defined(NFX_SMP) && defined(NFP_TXRX)
	if (!CM_IS_TXRX_NCPU)
		return;
#endif /* NFX_SMP */

	for( i=0; i < SB1250DM_CHNL_CNT; i++ ) {
		cfg = sb1250dmCfg[i];
		if( cfg->inUse == 1 )	
			sb1250dm_movCleanup( cfg );
	}
	return;
} // sb1250dm_movCompletePoll 
