/*  
 *=============================================================================
 * File Name	: sb1250-dm.c
 *
 *  Description	:    
 *
 *  Created		: 09 February 2002
 *
 *  Author		: Frank Salzmann
 *
 *  RCS			: $Id: sb1250-dm.c,v 1.7 2002/09/23 20:57:34 rick Exp $ 
 *
 *  Copyright(c)  Agile Storage, Inc. 2002
 *=============================================================================
 */
#include "sb1250-dm.h"

//
//-- Global Variables
//
sb1250dmCfg_t *sb1250dmCfg[SB1250DM_CHNL_CNT];

//
//-- Function Prototypes
//
int sb1250dm_movData();
void sb1250dm_movCompletePoll();
int dp_proxy_cb(char *, int);

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_initModule
 *
 * Description: 	Initializes the SB1250 Mover Device Driver module.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 19, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_initModule()
{
#if defined(SIBYTE)
	int i, rc, appId, chnl;
	sb1250dmCfg_t *cfg;
	
	printf("sb1250dm_initModule: [Enter]\n");

	for(i=0; i < SB1250DM_CHNL_CNT; i++ ) {

		cfg = (sb1250dmCfg_t *)eee_ramAlloc(sizeof(sb1250dmCfg_t));
		sb1250dmCfg[i] = cfg;

		if( !cfg ) {
			printf("[sb1250dm_initModule] Err: Malloc Failed: sb1250dmCfg.\n");
			return(NFX_ERR);
		}
		memset(cfg,0,sizeof(sb1250dmCfg_t));

		sb1250dm_setRegAddrs(cfg, i);

		//-- Mark Chnl Id: 
		cfg->chnlId = i;

	}

#ifndef PROM
	//-- Always open chnl1 to reserve its use by the data proxy code.
	chnl = sb1250dm_openChnl( SB1250DM_CHNL1, dp_proxy_cb );
	printf("SB1250DM: Opened data proxy mover channel: %d\n", chnl);
#endif

	//-- Register DMA ring cleanup polling function.
	rc = eee_registerPollingFunc(EEE_HIGH_PRIORITY,
								 sb1250dm_movCompletePoll,
								 cfg);

	if( rc == EEE_ERR_POLLING_TABLE_FULL )
		printf("SB1250: Err Cannot reg movCompletePoll func. Tbl full\n");
	else
	if( rc == EEE_ERR_BAD_PARAM )
		printf("SB1250: Err Cannot reg movCompletePoll func. Invalid Pri\n");
	else
		printf("SB1250: movCompletePoll function registered. \n");

	printf("sb1250dm_initModule: [Leave]\n");

#endif // SIBYTE

	return(NFX_OK);
} // sb1250dm_initModule 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_setRegAddrs
 *
 * Description: 	Initializes the sb1250 mover register addresses.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 20, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_setRegAddrs( sb1250dmCfg_t *pCfg, uint32 chnl )
{
	switch( chnl ) {

		case 0:
			pCfg->regs.dm_desc_base     = (uint64 *)PHYS_TO_K1(0x10020B00);
			pCfg->regs.dm_desc_count    = (uint64 *)PHYS_TO_K1(0x10020B08);
			pCfg->regs.dm_cur_desc_addr = (uint64 *)PHYS_TO_K1(0x10020B10);
			pCfg->regs.dm_dbg_desc_base = (uint64 *)PHYS_TO_K1(0x10020B18);
			break;

		case 1:
			pCfg->regs.dm_desc_base     = (uint64 *)PHYS_TO_K1(0x10020B20);
			pCfg->regs.dm_desc_count    = (uint64 *)PHYS_TO_K1(0x10020B28);
			pCfg->regs.dm_cur_desc_addr = (uint64 *)PHYS_TO_K1(0x10020B30);
			pCfg->regs.dm_dbg_desc_base = (uint64 *)PHYS_TO_K1(0x10020B38);
			break;

		case 2:
			pCfg->regs.dm_desc_base     = (uint64 *)PHYS_TO_K1(0x10020B40);
			pCfg->regs.dm_desc_count    = (uint64 *)PHYS_TO_K1(0x10020B48);
			pCfg->regs.dm_cur_desc_addr = (uint64 *)PHYS_TO_K1(0x10020B50);
			pCfg->regs.dm_dbg_desc_base = (uint64 *)PHYS_TO_K1(0x10020B58);
			break;

		case 3:
			pCfg->regs.dm_desc_base     = (uint64 *)PHYS_TO_K1(0x10020B60);
			pCfg->regs.dm_desc_count    = (uint64 *)PHYS_TO_K1(0x10020B68);
			pCfg->regs.dm_cur_desc_addr = (uint64 *)PHYS_TO_K1(0x10020B70);
			pCfg->regs.dm_dbg_desc_base = (uint64 *)PHYS_TO_K1(0x10020B78);
			break;
	}

	return NFX_OK;
} // sb1250dm_setRegAddrs

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_openChnl
 *
 * Description: 	Opens a sibyte data mover channel.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 21, 2001
 * 					
 * Returns: 		Success: Open Channel Id: 1-4
 *					Error  : 0
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_openChnl(int reqChnl,  int (*callback)() )
{
	uint32 i;
	uint32 chnlId = 0;

	if( reqChnl == SB1250DM_CHNL_ANY ) {

		//-- Open next available channel.
		for(i=0; i < SB1250DM_CHNL_CNT; i++) {
			if( sb1250dmCfg[i]->inUse == 0 ) {
				//-- Initialize channel.
				sb1250dmCfg[i]->inUse = 1;
				sb1250dmCfg[i]->callback = callback;
				sb1250dm_deviceInit(sb1250dmCfg[i]);
				chnlId = i+1;
				return chnlId;
			}
		}
	}
	else //-- Try to open requested channel
	if((reqChnl >= SB1250DM_CHNL1) && (reqChnl <= SB1250DM_CHNL4)) {
		chnlId = reqChnl-1;
		if( sb1250dmCfg[chnlId]->inUse == 0 ) {
				//-- Initialize channel.
				sb1250dmCfg[chnlId]->inUse = 1;
				sb1250dmCfg[chnlId]->callback = callback;
				sb1250dm_deviceInit(sb1250dmCfg[chnlId]);
				return reqChnl;
		}
	}
	return SB1250DM_CHNL_NOT_AVAILABLE;
}	// sb1250dm_openChnl

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_closeChnl
 *
 * Description: 	Opens a sibyte data mover channel.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 21, 2001
 * 					
 * Returns: 		NFX_OK		
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_closeChnl( int chnlId )
{
	int i = chnlId - 1;
	sb1250dm_regs_t   *regs;
	sb1250dm_descBar_t bar;


	//-- Close channel.
	if( sb1250dmCfg[i]->inUse == 1 ) {

		//-- First stop the data mover.
		regs = &(sb1250dmCfg[i]->regs);
		bar.wd         = 0;
		bar.bits.enbl  = 0;	
		bar.bits.abort = 1;
		sb1250dm_writeReg(regs->dm_desc_base, bar.wd);

		//-- Deallocate the dma ring.
		eee_ramDealloc( sb1250dmCfg[i]->dmqc.qStart );

		//-- Mark channel available
		sb1250dmCfg[i]->inUse    = 0;
		sb1250dmCfg[i]->callback = 0;
		memset(&(sb1250dmCfg[i]->dmqc), 0, sizeof( sb1250dm_qCtrl_t));
	}
	return SB1250DM_OK;
}	// sb1250dm_closeChnl

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_deviceInit
 *
 * Description: 	This function initializes SB1250 Data Mover.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 20, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_deviceInit(sb1250dmCfg_t *cfg) 
{
	int i, rc;
	uint64 val;
	uint64 chip_cnt, zero_cnt;
	uint64 *reg;
	sb1250dm_regs_t *regs;
	sb1250dm_descBar_t bar;

	//printf("[sb1250dm_deviceInit] Enter\n");

	regs = &(cfg->regs);

	sb1250dm_allocRing(cfg);

	//printf("regs: %x, desc_base: %x\n", regs, regs->dm_desc_base);

	//-- Clear the DMA desc count register.
	chip_cnt = sb1250dm_readReg(regs->dm_desc_count);
	if( chip_cnt > 0 ) {
		zero_cnt = (chip_cnt ^ 0xFFFFFFFFFFFFFFFF) + 1;
	//	printf("chip tx desc cnt: %x\n", chip_cnt);
		//zero_cnt &= 0x000000000000FFFF;
	//	printf("chip tx desc cnt: %x\n", zero_cnt);
	//	printf("new chip cnt%x\n", zero_cnt + chip_cnt);
		sb1250dm_writeReg(regs->dm_desc_count, 0);
	}

	bar.wd = 0;
	bar.bits.base        = VIRT_TO_PHYS((uint64)(cfg->dmqc.qStart));
	bar.bits.ring_size   = SB1250_DM_Q_LEN;
	bar.bits.enbl        = 1;
	bar.bits.error_reset = 1;
	sb1250dm_writeReg(regs->dm_desc_base, bar.wd);
	
	//printf("[sb1250dm_deviceInit] Leave\n");
	return NFX_OK;
} // sb1250dm_deviceInit

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_allocRing
 *
 * Description: 	This function allocates and initializes memory for 
 *					the data mover DMA ring.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 20, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_allocRing(sb1250dmCfg_t *cfg) 
{
	uint32           qsize;
	uint32           align_diff;
	sb1250dm_qCtrl_t *dmqc;

	sb1250dm_desc_t  *qstart;

	//printf("[sb1250dm_allocRing] Enter\n");

	dmqc = &cfg->dmqc;
 
	qsize = SB1250_DM_Q_LEN * sizeof(sb1250dm_desc_t);
	qstart = (sb1250dm_desc_t *)eee_ramAlloc(qsize + 64);
#ifndef PROM
	if (spans_dram_blocks(qstart, qsize+64))
		panic("Invalid allocRing, VtoP(start) = %p, VtoP(end) = %p\n",
									VIRT_TO_PHYS(qstart),
									VIRT_TO_PHYS((address_t)qstart+(qsize+64)));
#endif

    cfg->cb_cookie = (void **)eee_ramAlloc(SB1250_DM_Q_LEN * sizeof(void *));
 
	//printf("dmq start: %x: %d bytes allocated\n", qstart, qsize+64);
 
	align_diff = (uint32)qstart & 0x00000007;
 
	if( align_diff )
		align_diff = 8 - align_diff;
	(address_t)qstart += align_diff;
 
	memset(qstart, 0, qsize);

	dmqc->qStart = (sb1250dm_desc_t *)qstart;
	dmqc->qtxCur = dmqc->qclnCur = dmqc->qStart;
	dmqc->qEnd = dmqc->qStart + SB1250_DM_Q_LEN;

	//printf("[sb1250dm_allocRing] Leave\n");
	return NFX_OK;
} // sb1250dm_allocRing

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_resetChannel
 *
 * Description: 	Resets an individual data mover channel.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 21, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_resetChannel(sb1250dmCfg_t *cfg) 
{

} /* sb1250dm_resetChannel */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_rxAppPkt
 *
 * Description: 	Receive packet sent to BMC12500 MAC mgmt layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 12, 2001
 *------------------------------------------------------------------------------
 */
void
sb1250dm_rxAppPkt( eee_desc_t *edesc )
{
	//sb1250dm_diagLbPktReceive(edesc);
	return;
} /* sb1250dm_rxAppPkt */


/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_writeReg
 *
 * Description: 	Writes 64-bit value to a device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 *------------------------------------------------------------------------------
 */
int
sb1250dm_writeReg(volatile uint64 *reg, uint64 val) 
{
	*reg = val;
	return NFX_OK;
} /* sb1250dm_writeReg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_readReg
 *
 * Description: 	Returns 64-bit value read from device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 *------------------------------------------------------------------------------
 */
uint64
sb1250dm_readReg(volatile uint64 *reg) 
{
	uint64 val;

	val = *reg;
	return(val);
} /* sb1250dm_readReg */
