/* AME $Id: amerom.h,v 1.12 2001/08/10 23:15:48 mtv Exp $ */

/*
 * Copyright 2000 SiByte, Inc.
 * THIS IS UNPUBLISHED, PROPRIETARY SOURCE CODE OF SiByte, Inc.
 */

/*
 * Public interfaces to the 'rom-ish' routines provided by the
 * Absolutely Minimal Environment.
 */

#ifndef _AMEROM_H_
#define _AMEROM_H_

/*
 * Console infrastructure.
 */
#ifndef __ASSEMBLER__
struct amecons {
	void	(*init)(void * /*pvtdata*/, int /*serbaud*/);
	int	(*getchar)(void * /*pvtdata*/);
	void	(*putchar)(void * /*pvtdata*/, int /*c*/);
	void	*pvtdata;
};

#define	DEFAULT_CONSOLE			_cons_sb1250scn_chan0
#define	DEFAULT_CONSOLE_SPEED		57600

void	_consinit(void);			/* private */

extern const struct amecons _cons_sb1250scn_chan0;
extern const struct amecons _cons_sb1250scn_chan1;
extern const struct amecons _cons_ucmem;
extern const struct amecons _cons_cfecons;

/*
 * If these are defined by programs to have non-zero values, those values
 * will be used.  Otherwise, the defaults will be used.
 */
extern const struct amecons *_console;
extern int _console_speed;

#endif

/*
 * Trap type numbers, passed to trap functions as first argument.
 */
#define	TRAP_PANIC		0	/* panic() call */
#define	TRAP_TLB_REFILL		1
#define	TRAP_XTLB_REFILL	2
#define	TRAP_CACHE_ERROR	3
#define	TRAP_GENERAL		4
#define	TRAP_INTERRUPT		5

#define	DEFAULT_TRAP_FUNC	_trap

#ifndef __ASSEMBLER__
extern const char *_trapnames[];
extern const int _ntrapnames;

/*
 * Trap handler function definition.
 *
 * The 'gprs' argument is a pointer to 32 unsigned long values (in order;
 * yes, we could have skipped $0, but we didn't), the integer register
 * file.  They can be changed by the trap handler, and upon its return
 * the new values will be placed in the registers.
 *
 * If the program's trap handler wishes to modify any CP0 registers (or
 * other machine state), it has to do so directly (i.e. using "mtc0"
 * instructions).
 */
/* Definitions for saving and restoring registers */
#if defined(__mips64)
typedef unsigned long long regtype;
#else
typedef unsigned int regtype;
#endif
typedef void _ame_trapfunc(unsigned long type, regtype *gprs);

/*
 * The default trap handling function.  Simply prints out information
 * about the trap and spin-loops.
 *
 * This is not directly callable from C.  If necessary, it should
 * be used by declaring a global function pointer variable like:
 *
 *     static _ame_trapfunc *real_trap = &_trap;
 *
 * then calling through it.
 */
_ame_trapfunc _trap;

/*
 * If this function pointer is defined by the program to have a non-zero
 * values, that value will be used.  Otherwise, the default will be used.
 * It may be modified at run-time to change the trap handling function
 * in use, if desired.
 */
extern _ame_trapfunc *_trap_func;
#endif

/*
 * default sizes for memory, stack, and (unused) red-zone below stack.
 * memory is assumed to start at physical address 0.
 */
#define	DEFAULT_MEM_SIZE	(32 * 1024 * 1024)
#define	DEFAULT_STACK_SIZE	(128 * 1024)
#define	DEFAULT_REDZONE_SIZE	(32 * 1024)

#ifndef __ASSEMBLER__
/*
 * If these are defined by programs to have non-zero values, those values
 * will be used.  Otherwise, the defaults will be used.
 */
extern unsigned long _mem_size, _stack_size, _redzone_size;
#endif


/*
 * default maximum number of CPUs.  Controls placement of start-up stacks
 * for multiple CPUs.
 */
#define	DEFAULT_MAX_CPU_COUNT	8

#ifndef __ASSEMBLER__
/*
 * If these are defined by programs to have non-zero values, those values
 * will be used.  Otherwise, the defaults will be used.
 */
extern unsigned long _max_cpu_count;
#endif


/*
 * Miscellaneous bits
 */

#ifndef __ASSEMBLER__

/*
 * Function pointer whose function is called by secondary CPUs when
 * they start up.  Argument is CPU number.  If NULL, they spin waiting
 * for it to become non-NULL.  If it returns, the secondary CPU stack
 * pointer is reinitialized and the loop-or-call process begins again.
 * (To completely disable a secondary CPU, put it back into reset.)
 */
extern void (*_seccpu_start_func)(unsigned long);

/*
 * Function to use to cause the simulator to exit.  Defaults
 * to _sim_exit() if not overridden.
 */

/* CFE */
void    _cfe_exit(int)
	    __attribute__((__noreturn__));

/* GDB/standalone functional simulator */
void	_sim_exit(int)
	    __attribute__((__noreturn__));

/* diag mode in fn sim or verilog/emulation */
void	_diag_exit(int)
	    __attribute__((__noreturn__));

/* hardware (tight loop) */
void	_hw_exit(int)
	    __attribute__((__noreturn__));


extern void (*_exit_func)(int) __attribute__((__noreturn__));


/*
 * These are not directly callable from C.  If necessary, they should
 * be used by declaring a global function pointer variable like:
 *
 *     static void (*real_panic)(int) __attribute__((__noreturn__)) = &_panic;
 *
 * then calling through it.
 */
void	_panic(void)
	    __attribute__((__noreturn__));
#endif

#define	DEFAULT_EXIT_FUNC	_sim_exit

#endif
