/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id$
 *
 * Copyright (C) 2003, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "sbprof.h"
#include <exports/malloc-api.h>
#include <exports/timer-api.h>
#include <exports/esm-api.h>
#include <exports/file-api.h>
#include "sbprof-thread.h"

uchar8		sbprof_fname[80];
address_t	sbprof_addr;
uint32		sbprof_len;

char * strdup(char * str);

struct sbprof_dump_cb {
	uchar8		* filename;
	address_t	addr;
	uint32		len;
}; 

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
void
sbprof_dump(
	uchar8	* fname,
	address_t	addr,
	uint32	len
)
{
	esm_event_t		event;
	esm_handle_t	esmHandle;
	struct sbprof_dump_cb	* cb;

	cb = eee_ramAlloc(sizeof(*cb));

	cb->filename = strdup(fname);
	cb->addr = addr;
	cb->len = len;

	if (sbprof_thread_sm_create(&esmHandle, cb) != NFX_OK) {
		printf("sbprof_dump: error creating write thread\n");
		return;
	}

	event.event = SBPROF_THREAD_EVENT_START;
	if (esm_deliverEvent(esmHandle, &event) != NFX_OK) {
		printf("sbprof_dump: error starting writer thread\n");
		return;
	}
}

int32	sbprof_timer;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
sbprof_timeout(
	uint32	handle,
	void	* p1,
	void	* p2,
	void	* p3,
	void	* p4
)
{
	if ((address_t)p1 == (address_t)nfxMyCoreId) {
		printf("sbprof timer expired\n");
		printf("stop sb1 profiler\n");

		sbprof_stop();

		eee_timerStop(sbprof_timer);
		eee_timerDestroy(sbprof_timer);
	}
	else {
		printf("sbprof expired on wrong core, restarting\n");
		eee_timerStart(handle, sbprof_timeout, p1, p2, p3, p4);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
sbprof_cmd(
	int32	argc,
	uchar8	* argv[]
)
{
	uint64	p = (uint64)nfxMyCoreId;

	if (!strcmp(argv[1], "start")) {
		if (argc == 2)
			printf("start sb1 profiler\n");
		else if (argc == 3) {
			uint32	t;
			t = atoi(argv[2]);
			printf("start sb1 profiler, running for %d seconds\n", t);
			sbprof_timer = eee_timerCreate(1000*t, EEE_TIMER_ONCE);
			eee_timerStart(sbprof_timer, sbprof_timeout, (void *)p, 0, 0, 0);
		}
		sbprof_start();
	}
	else if (!strcmp(argv[1], "stop")) {
		printf("stop sb1 profiler\n");
		sbprof_stop();
	}
	else if (!strcmp(argv[1], "dump")) {
		printf("dump sb1 profiler data: ");
		if (argc != 3) {
			printf("sbprof dump <filename>\n");
			return 0;
		}
		strcpy(sbprof_fname, argv[2]);
//		sbprof_addr = atolx(argv[3]);
//		sbprof_len = atox(argv[4]);
		printf("fname = %s, addr = 0x%lx, len = 0x%x\n",
										sbprof_fname, sbprof_addr, sbprof_len);
		sbprof_dump(sbprof_fname, sbprof_addr, sbprof_len);
	}
	else
		printf("sbprof <start|stop>\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
void
do_sbprof_dump(struct sbprof_dump_cb *cb)
{
	int fd, rc;

	printf("Writing SB1 profiler data to %s, this may take some time.\n",
			cb->filename);

	fd = file_create(cb->filename, 0644);
	if (fd < 0)
	{
		printf("do_sbprof_dump: couldn't create %s\n", cb->filename);
		return;
	}

	rc = file_write(fd, (void *) cb->addr, cb->len);
	if (rc != cb->len)
	{
		printf("do_sbprof_dump: short write samples(0x%x/0x%x)\n", cb->len, rc);
		file_close(fd);
		return;
	}

	file_close(fd);

	printf("SB1 profiler data created at %s.\n", cb->filename);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *		Thread create action function.  We must have this, but
 *		we needn't do anything here.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
sbprof_thread_create(
    esm_handle_t        handle,
    int32               nextState,
    esm_event_t         event,
    void                *smCookie)
{
    return nextState;

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *		Another must have for the thread.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
sbprof_thread_bad_state(
    esm_handle_t        handle,
    int32               nextState,
    esm_event_t         thisEvent,
    void                *userInfo)
{
    panic("sbprof_thread bad state\n");
    return nextState;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *		Start action function for our dump thread.
 *		Do the work of writing out the file.  We need a
 *		thread to interface with the file library.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
sbprof_thread_start(
    esm_handle_t        handle,
    int32               nextState,
    esm_event_t         event,
    void                *smCookie)
{
#if defined(_PROFILING_)
    do_sbprof_dump(smCookie);
#endif
    return nextState;
}



