/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mem-api.c,v 1.10 2002/11/21 00:27:31 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/17/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mem-api.h>
#ifdef LARGE_MEMORY
#include <exports/pte-api.h>
#endif
#include <exports/smp-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/23/02
 *
 *-----------------------------------------------------------------
 */
void
mem_map_init()
{
	extern void mem_map_tlb_init(uint64);

	mem_map_tlb_init(0);
}

/*-----------------------------------------------------------------
 * Name:	is_valid_addr(adr)
 *
 * Description: returns 0 or 1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/17/02
 *
 *-----------------------------------------------------------------
 */
int32
is_valid_addr(
	address_t	adr
)
{

#ifdef LARGE_MEMORY
	uint32	i;

	if ((adr >= K0BASE) && (adr < K2BASE)) 
		return 1;

	/* PCI space */
	if ((adr >= 0x50000000) && (adr < 0x52000000))
		return 1;

	/* wka */
	if ((adr >= 0x73000000) && (adr < 0x73200000))
		return 1;

	for (i=1; i<MMU_NUM_TABLES+1; i++) {
		if ((adr >= mmu_virt_start[i]) && (adr < mmu_virt_end[i]))
			return 1;
	}

#else

	return 1;

#endif

#ifdef NFX_SMP
	if ((adr >= 0x60000000) && (adr < 0x60000000 + SMP_CLIENTPRIVATE_TLB_SIZE*2))
		return 1;
#endif

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/20/02
 *
 *-----------------------------------------------------------------
 */
address_t
virt_to_phys(
	address_t	virt
)
{
	address_t	phys;

#ifdef LARGE_MEMORY

	ushort16	i;
	address_t	phys_offset, save_base;

#ifdef NFP_FP

	if ((virt >= mmu_virt_start[MMU_SM_FP_REGION]) &&
									(virt < mmu_virt_end[MMU_SM_FP_REGION])) {
		phys_offset = virt - mmu_virt_start[MMU_SM_FP_REGION] +
									mmu_phys_offset_start[MMU_SM_FP_REGION];

		phys_offset &= 0x0000000FFFFFFFFF;

		if (phys_offset >= 0x40000000)
			phys = 0x100000000 + phys_offset - 0x40000000;
		else if (phys_offset >= 0x30000000)
			phys = 0xC0000000 + phys_offset - 0x30000000;
		else if (phys_offset >= 0x10000000)
			phys = 0x80000000 + phys_offset - 0x10000000;
		else
			phys = phys_offset;

		goto converted_to_phys;
	}
	else if ((virt < mmu_virt_end[MMU_MALLOC_REGION]) &&
								(virt >= mmu_virt_start[MMU_MALLOC_REGION])) {
		phys_offset = virt - mmu_virt_start[MMU_MALLOC_REGION] +
									mmu_phys_offset_start[MMU_MALLOC_REGION];

		phys_offset &= 0x0000000FFFFFFFFF;

		if (phys_offset >= 0x40000000)
			phys = 0x100000000 + phys_offset - 0x40000000;
		else if (phys_offset >= 0x30000000)
			phys = 0xC0000000 + phys_offset - 0x30000000;
		else if (phys_offset >= 0x10000000)
			phys = 0x80000000 + phys_offset - 0x10000000;
		else
			phys = phys_offset;

		goto converted_to_phys;
	}
#endif	/* NFP_FP */
	for (i=1; i<=MMU_NUM_TABLES; i++) {

		if ((virt < mmu_virt_end[i]) && (virt >= mmu_virt_start[i])) {
			phys_offset = virt - mmu_virt_start[i] + mmu_phys_offset_start[i];

			save_base = phys_offset & 0xF000000000;
			phys_offset &= 0x0000000FFFFFFFFF;

			if (phys_offset >= 0x40000000)
				phys = 0x100000000 + phys_offset - 0x40000000;
			else if (phys_offset >= 0x30000000)
				phys = 0xC0000000 + phys_offset - 0x30000000;
			else if (phys_offset >= 0x10000000)
				phys = 0x80000000 + phys_offset - 0x10000000;
			else
				phys = phys_offset;

			phys += save_base;
			goto converted_to_phys;
		}
	}
#endif /* LARGE_MEMORY */

	if ((virt >= K0BASE) && (virt < K1BASE)) {
		phys = KVA_TO_PA(virt);
	}
	else {
		address_t	ra;

		ra = get_ra();
//		panic("Invalid virt, %p, ra = %p\n", virt, ra);
		*(volatile uint32 *)0 = virt;
		phys = 0;
	}

#ifdef LARGE_MEMORY
converted_to_phys:
#endif
//	printf("virt to phys: VIRT = %p, PHYS = %p\n", virt, phys);

	return phys;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/20/02
 *
 *-----------------------------------------------------------------
 */
address_t
phys_to_virt(
	address_t	phys
)
{
	address_t	virt, save_base;
    address_t	ra;
#ifdef LARGE_MEMORY
	ushort16	i;
	address_t	phys_offset;
#endif

	save_base = phys & 0xF000000000;
	phys &= 0x0000000FFFFFFFFF;

#ifdef LARGE_MEMORY
	if (phys >= 0x100000000)
		phys_offset = phys - 0x100000000 + 0x40000000;
	else if (phys >= 0xC0000000)
		phys_offset = phys - 0xC0000000 + 0x30000000;
	else if (phys >= 0x80000000)
		phys_offset = phys - 0x80000000 + 0x10000000;
	else
		phys_offset = phys;

	phys_offset += save_base;

#ifdef NFP_FP
	if ((phys_offset >= mmu_phys_offset_start[MMU_SM_FP_REGION]) &&
					(phys_offset < mmu_phys_offset_end[MMU_SM_FP_REGION])) {
		virt = phys_offset - mmu_phys_offset_start[MMU_SM_FP_REGION] +
											mmu_virt_start[MMU_SM_FP_REGION];
		goto converted_to_virt;
	}
	else if ((phys_offset < mmu_phys_offset_end[MMU_MALLOC_REGION]) &&
					(phys_offset >= mmu_phys_offset_start[MMU_MALLOC_REGION])) {
		virt = phys_offset - mmu_phys_offset_start[MMU_MALLOC_REGION] +
											mmu_virt_start[MMU_MALLOC_REGION];
		goto converted_to_virt;
	}
#endif	/* NFP_FP */
	for (i=1; i<=MMU_NUM_TABLES; i++) {
		if ((phys_offset < mmu_phys_offset_end[i]) &&
								(phys_offset >= mmu_phys_offset_start[i])) {
			virt = phys_offset - mmu_phys_offset_start[i] + mmu_virt_start[i];
			goto converted_to_virt;
		}
	}
#else  /* !LARGE_MEMORY */

    // assume small mem config < 512 MB phys address space
	if (phys < 0x20000000) {
		virt = PA_TO_KVA0(phys);
        goto converted_to_virt;
	}
#endif /* LARGE_MEMORY */

    ra = get_ra();
//    panic("Invalid phys, %p, offset = %p, ra = %p\n", phys, phys_offset, ra);
	*(volatile uint32 *)0 = phys;
	virt = 0;

converted_to_virt:
//	printf("phys to virt: PHYS = %p, VIRT = %p\n", phys, virt);

	return virt;
}

#ifndef PROM
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/23/02
 *
 *-----------------------------------------------------------------
 */
address_t
uncached_to_cached(
	address_t	uncached
)
{
	address_t	ra;

#ifdef LARGE_MEMORY

#ifdef NFP_FP
    if ((uncached >= mmu_virt_start[MMU_SM_TXRX_REGION]) &&
                            (uncached < mmu_virt_end[MMU_SM_TXRX_REGION]))
        return (uncached - 0x2000000000 + 0x6000000000);
#endif

#ifdef NFP_TXRX
    if ((uncached >= mmu_virt_start[MMU_SM_FP_REGION]) &&
                            (uncached < mmu_virt_end[MMU_SM_FP_REGION]))
        return (uncached - 0x4000000000 + 0x7000000000);
#endif

#else
    if ((uncached >= K1BASE) && (uncached < K2BASE))
        return K1_TO_K0(uncached);
#endif

	ra = (address_t)get_ra();
	panic("Invalid uncached addr, %p, ra = %p\n", uncached, ra);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/23/02
 *
 *-----------------------------------------------------------------
 */
address_t
cached_to_uncached(
	address_t	cached
)
{
	address_t	ra;

#ifdef LARGE_MEMORY

#ifdef NFP_FP
    if ((cached >= mmu_virt_start[MMU_SM_TXRX_CACHED_REGION]) &&
                            (cached < mmu_virt_end[MMU_SM_TXRX_CACHED_REGION]))
        return (cached - 0x6000000000 + 0x2000000000);
#endif

#ifdef NFP_TXRX
    if ((cached >= mmu_virt_start[MMU_SM_FP_CACHED_REGION]) &&
                            (cached < mmu_virt_end[MMU_SM_FP_CACHED_REGION]))
        return (cached - 0x7000000000 + 0x4000000000);
#endif

#else
    if ((cached >= K0BASE) && (cached < K1BASE))
        return K0_TO_K1(cached);
#endif

	ra = (address_t)get_ra();
	panic("Invalid cached addr, %p, ra = %p\n", cached, ra);

	return 0;
}

#else
/***************************************************************
*  Not sure why this was in membench.c  sm-sysdvt
*  Moved to here cuz membench got removed to make space in RPROM
*
****************************************************************
*/
address_t
uncached_to_cached(
        address_t       uncached
)
{
        address_t       addr = 0;

        if (isTXRX()) {
                addr = uncached & ~0xF0000000;
                addr |= 0x70000000;
        }
        else if (isFP()) {
                addr = uncached & ~0xF0000000;
                addr |= 0x60000000;
        }

        return addr;
}

address_t
cached_to_uncached(
        address_t       cached
)
{
        address_t       addr = 0;

        if (isTXRX()) {
                addr = cached & ~0xF0000000;
                addr |= 0x40000000;
        }
        else if (isFP()) {
                addr = cached & ~0xF0000000;
                addr |= 0x20000000;
        }

        return addr;
}
#endif /* !PROM */
