/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: perf_count.c,v 1.4 2002/10/25 01:14:37 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/bsp-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
void
perf_init_count(
	int32	counter_index,
	int32	event_id
)
{
	uint32	cntrl, read = 0;

	printf("init_count, %d, %d\n", counter_index, event_id);
	cntrl = (event_id << 5) | 0xF;

	printf("    writing 0x%x\n", cntrl);
	if (counter_index == 0) {
		__asm__ ("mtc0	%0,$25,0" : : "r" (cntrl));
		__asm__ ("mfc0	%0,$25,0" : "=r" (read) : );
	}
	else if (counter_index == 1) {
		__asm__ ("mtc0	%0,$25,2" : : "r" (cntrl));
		__asm__ ("mfc0	%0,$25,2" : "=r" (read) : );
	}
	else if (counter_index == 2) {
		__asm__ ("mtc0	%0,$25,4" : : "r" (cntrl));
		__asm__ ("mfc0	%0,$25,4" : "=r" (read) : );
	}
	else if (counter_index == 3) {
		__asm__ ("mtc0	%0,$25,6" : : "r" (cntrl));
		__asm__ ("mfc0	%0,$25,6" : "=r" (read) : );
	}
	printf("    read back 0x%x\n", read);
	
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
void
perf_start_counters()
{
	uint32	cntrl;
	uint64	counter_ov;

	printf("start_counters\n");

	__asm__ ("mfc0	%0,$25,0" : "=r" (cntrl) : );
	printf("read cntrl from sel0, cntrl = 0x%x\n", cntrl);
	cntrl |= (1<<30);
	__asm__ ("mtc0	%0,$25,0" : : "r" (cntrl));
	counter_ov = ((uint64)1<<40);
	__asm__ ("dmtc0	%0,$25,1" : : "r" (counter_ov));

	__asm__ ("dmtc0	$0,$25,3" : : );
	__asm__ ("dmtc0	$0,$25,5" : : );
	__asm__ ("dmtc0	$0,$25,7" : : );
	__asm__ ("dmtc0	$0,$25,1" : : );
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
void
perf_stop_counters()
{
	__asm__ ("mtc0	$0,$25,0" : : );
	__asm__ ("mtc0	$0,$25,2" : : );
	__asm__ ("mtc0	$0,$25,4" : : );
	__asm__ ("mtc0	$0,$25,6" : : );

	printf("stop_counters\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
uint64
perf_read_count(
	int32	counter_index
)
{
	uint64	count = 0;

	if (counter_index == 0)
		__asm__ ("dmfc0	%0,$25,1" : "=r" (count) : );
	else if (counter_index == 1)
		__asm__ ("dmfc0	%0,$25,3" : "=r" (count) : );
	else if (counter_index == 2)
		__asm__ ("dmfc0	%0,$25,5" : "=r" (count) : );
	else if (counter_index == 3)
		__asm__ ("dmfc0	%0,$25,7" : "=r" (count) : );

	return count;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
perf_name(
	int32	event_id
)
{
	if (event_id == 0)
		return "INSTRUCTION COUNT";
	else if (event_id == 15)
		return "ICACHE MISS";
	else if (event_id == 16)
		return "CYCLES";
	else if (event_id == 40)
		return "ISSUE TO L0";

	return "Unknown";
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/24/02
 *
 *-----------------------------------------------------------------
 */
int32
do_perf_count(
	int32	argc,
	char8	*argv[]
)
{
	if (strcmp(argv[1], "start") == 0) {
		perf_stop_counters();
		perf_init_count(0, SB1250_PERF_CYCLES);
		perf_init_count(1, SB1250_PERF_INST_EXE);
		perf_init_count(2, SB1250_PERF_ICACHE_MISS);
		perf_start_counters();
	}
	else if (strcmp(argv[1], "stop") == 0) {
		perf_stop_counters();
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_CYCLES),
															perf_read_count(0));
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_INST_EXE),
															perf_read_count(1));
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_ICACHE_MISS),
															perf_read_count(2));
	}
	else if (strcmp(argv[1], "view") == 0) {
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_CYCLES),
															perf_read_count(0));
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_INST_EXE),
															perf_read_count(1));
		printf("perf_count %s = 0x%llx\n", perf_name(SB1250_PERF_ICACHE_MISS),
															perf_read_count(2));
	}
	else
		return -1;

	return 0;
}

