/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: smbus.c,v 1.3 2002/11/07 01:17:17 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/13/02
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

//#include <sys/sb1250/sbmips.h>
#include <sys/mips.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_mc.h>
#include <sys/sb1250/sb1250_smbus.h>
#include <sys/sb1250/sb1250_scd.h>
#include <sys/sb1250/sb1250_defs.h>


typedef long sbport_t;

#define CFG_DRAM_SMBUS_CHANNEL  0

#define WRITECSR(csr,val) *((volatile uint64 *) (csr)) = (val)
#define READCSR(csr) (*((volatile uint64 *) (csr)))


#if 0
/*  *********************************************************************
    *  SB1250_SMBUS_INIT()
    *  
    *  Initialize SMBUS channel 
    *  
    *  Input parameters: 
    *  	   none
    *  	   
    *  Return value:
    *  	   smbus_base - KSEG1 address of SMBus channel
    *  
    *  Registers used:
    *  	   tmp0
    ********************************************************************* */

static sbport_t sb1250_smbus_init(void)
{
    sbport_t base;

    base = PHYS_TO_K1(A_SMB_BASE(CFG_DRAM_SMBUS_CHANNEL));

    WRITECSR(base+R_SMB_FREQ,K_SMB_FREQ_100KHZ);
    WRITECSR(base+R_SMB_CONTROL,0);

    return base;
}
#endif
	

/*  *********************************************************************
    *  SB1250_SMBUS_WAITREADY()
    *  
    *  Wait for SMBUS channel to be ready.
    *  
    *  Input parameters: 
    *  	   smbus_base - SMBus channel base (K1seg addr)
    *  	   
    *  Return value:
    *  	   ret0 - 0 if no error occured, else -1
    *  
    *  Registers used:
    *  	   tmp0,tmp1
    ********************************************************************* */

int sb1250_smbus_waitready(sbport_t base)
{
    uint64 status;

    /*	
     * Wait for busy bit to clear
     */

    for (;;) {
	status = READCSR(base+R_SMB_STATUS);
	if (!(status & M_SMB_BUSY)) break;
	}

    /*
     * Isolate error bit and clear error status
     */

    status &= M_SMB_ERROR;
    WRITECSR(base+R_SMB_STATUS,status);

    /*
     * Return status
     */
    
    return (status) ? -1 : 0;
}



/*  *********************************************************************
    *  SB1250_SMBUS_READBYTE()
    *  
    *  Read a byte from a serial ROM attached to an SMBus channel
    *  
    *  Input parameters: 
    *      base - SMBus channel base address (K1seg addr)
    *  	   dev - address of device on SMBUS
    *      offset - address of byte within device on SMBUS
    *  	   
    *  Return value:
    *  	   byte from device (-1 indicates an error)
    ********************************************************************* */


int sb1250_smbus_readbyte(sbport_t base,unsigned int dev,unsigned int offset)
{
    int res;

    /*
     * Wait for channel to be ready
     */

    res = sb1250_smbus_waitready(base);

    if (res < 0)
		return res;
    /*
     * Set up a READ BYTE command.  This command has no associated
     * data field, the command code is the data 
     */

    WRITECSR(base+R_SMB_CMD,offset);
    WRITECSR(base+R_SMB_START,dev | V_SMB_TT(K_SMB_TT_CMD_RD1BYTE));

    /* 
     * Wait for the command to complete
     */

    res = sb1250_smbus_waitready(base);
    if (res < 0)
		return res;
    /*
     * Return the data byte
     */

    return (int) ((READCSR(base+R_SMB_DATA)) & 0xFF);
}


/*  *********************************************************************
    *  SB1250_SMBUS_WRITEBYTE()
    *  
    *  Write a byte to a device attached to the SMBus channel
    *  
    *  Input parameters: 
    *      base - SMBus channel base address (K1seg addr)
    *  	   dev - address of device on SMBUS
    *      offset - address of byte within device on SMBUS
    *  	   data - data to be put in dev 
    *  Return value:
    *  	   byte from device (-1 indicates an error)
    ********************************************************************* */
int sb1250_smbus_writebyte(sbport_t base,unsigned int dev,unsigned int offset,char data)
{
    int res;

    /*
     * Wait for channel to be ready
     */

    res = sb1250_smbus_waitready(base);
    if (res < 0)
		return res;

    /*
     * Set up a WRITE BYTE command.
     */

    WRITECSR(base+R_SMB_CMD,offset);
    WRITECSR(base+R_SMB_DATA,data);
    WRITECSR(base+R_SMB_START,dev | V_SMB_TT_WR2BYTE);

    /* 
     * Wait for the command to complete
     */

    res = sb1250_smbus_waitready(base);

    if (res < 0)
		return res;

	return (0);
}

