/*
 *-----------------------------------------------------------------
 *
 * Name:	wd.c
 *
 * RCS:  $Id: wd.c,v 1.1 2002/04/10 16:57:35 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Watchdog implementation
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_int.h>
#include <exports/cm-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
wd_enable(
	uint32	nsec
)
{
	volatile sb1250_wdtimer_t *	wd = (sb1250_wdtimer_t *)
							PHYS_TO_K1(A_SCD_WDOG_BASE((uint64)nfxMyCoreId));

	if (nsec > 8) {
		printf("Cannot set watchdog expiry > 8, setting to 8 sec\n");
		nsec = 8;
	}

	wd->config = 0;
	wd->init_cnt = nsec * 1000000;
	wd->config = 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
wd_disable()
{
	volatile sb1250_wdtimer_t *	wd = (sb1250_wdtimer_t *)
							PHYS_TO_K1(A_SCD_WDOG_BASE((uint64)nfxMyCoreId));

	wd->config = 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
wd_kick()
{
	extern uint32	watchdog_state;
	volatile sb1250_wdtimer_t *	wd = (sb1250_wdtimer_t *)
							PHYS_TO_K1(A_SCD_WDOG_BASE((uint64)nfxMyCoreId));

	if (watchdog_state)
		wd->config = 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
wd_init()
{
	volatile uint64	* imask;
	volatile uint64	* imap;

	imask = (volatile uint64 *)
		PHYS_TO_K1(A_IMR_REGISTER((uint64)nfxMyCoreId,R_IMR_INTERRUPT_MASK));
	imap = (volatile uint64 *)
	PHYS_TO_K1(A_IMR_REGISTER((uint64)nfxMyCoreId,R_IMR_INTERRUPT_MAP_BASE));

	switch (nfxMyCoreId) {
		case 0:
			*imask &= ~M_INT_WATCHDOG_TIMER_0;
			*(imap+K_INT_WATCHDOG_TIMER_0) = 6;			/* generate NMI */
			break;
		case 1:
			*imask &= ~M_INT_WATCHDOG_TIMER_1;
			*(imap+K_INT_WATCHDOG_TIMER_1) = 6;			/* generate NMI */
			break;
		default:
			panic("invalid coreId %d in wd_init\n", nfxMyCoreId);
	}
}

