#ifdef LINUX_TEST
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include <exports/esm-api.h>
#include <exports/scsi-api.h>
#include "scsi-test.h"


#define TEST_MAX_DB_SIZE	2048		/* Maximum data buffer size */

void test_rcvSdmMsg(void *, uint32);

#ifdef LINUX_TEST
int unixMsgQ_rx;
int unixMsgQ_tx;
#endif /* LINUX_TEST */

int
scsi_testMenu()
{
	int sel;
	char cmd[80];
	uint32 path;

	printf("Starting scsiTest\n");


#ifdef NOTDEF
	printf("Getting SDM ipc q id.....\n");
	unixMsgQ_rx = msgget((key_t)SDM_TX_MSGQ_KEY,0666);
    if( unixMsgQ_rx < 0 ) {
        printf("Error: Could not get RX IPC message queue id!\n");
        exit(1);
    }
	unixMsgQ_tx = msgget((key_t)SDM_RX_MSGQ_KEY,0666 | IPC_CREAT);
    if( unixMsgQ_tx < 0 ) {
        printf("Error: Could not get TX IPC message queue id!\n");
        exit(1);
    }
	
	printf("Register UNIX Msg Receive Polling function.\n");
	eee_registerPollingFunc(EEE_HIGH_PRIORITY, test_rcvSdmMsg, unixMsgQ_rx);
#endif /* LINUX_TEST */


	/* Run the main test menu */
	for(;;) {

		sel = test_printMenu();

		switch(sel) 
		{
			case 1:
				scsi_showDeviceList();
				break;
			case 2:
				scsi_showCtrl();
				break;
			case 3:
				printf("Enter Device Path: ");
#ifdef LINUX_TEST
				fflush(0);
#endif /* LINUX_TEST */
				get_cmd(cmd);
				sscanf(cmd, "%x", &path);
				scsi_showDevReqQ((scsi_path_t)path);
				break;
			case 4:
				scsi_setTracing();
				break;
			case 99:
				printf("scsiTest program terminated.....\n\n");
				return(0);
			default:
				printf("Try Again\n");
		}
	}
} /* scsi_test_main */

int
test_printMenu()
{
	int sel;
	char cmd[80];

	printf("\n");
	printf("1).  Show Device List\n");
	printf("2).  Show SCSI Control Block\n");
	printf("3).  View Device Req Q.\n");
	printf("4).  Enable/Disable Tracing\n");
	//printf("5).  View SCSI Descriptor\n");
	printf("99). Quit\n");
	printf("\nEnter Selection: ");
#ifdef LINUX_TEST
	fflush(0);
#endif /* LINUX_TEST */
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
}

void
test_rcvSdmMsg( void *cb, uint32 tlimit )
{
#ifdef LINUX_TEST
	int q_id, rc;
	sdmTestMsg_t msg; 

	q_id = (int)cb;
	rc = msgrcv(q_id, &msg, sizeof(sdmTestMsg_t), 0, MSG_NOERROR | IPC_NOWAIT);
	if( rc > 0 ) {
		/* test_procSdmMsg(&msg, rc); */
		dp83820_sim_rx(&msg.str, rc);
	}
#endif /* LINUX_TEST */
	return;
} /* test_rcvSdmMsg */

int
test_sendSdmMsg( char *msg, int len )
{
#ifdef LINUX_TEST
	int rc;
	sdmTestMsg_t unixMsg;

	memset(&unixMsg, 0, sizeof(sdmTestMsg_t));
	memcpy(unixMsg.str, msg, len);
	unixMsg.type = 1;
	rc = msgsnd(unixMsgQ_tx, &unixMsg, (len + sizeof(int)), 0); 
#endif /* LINUX_TEST */
	return 0;
} /* sdm_sendSdmMsg */

int
test_procSdmMsg( sdmTestMsg_t *msg, int len )
{
#ifdef LINUX_TEST
	int  i;
	eee_desc_t *edesc;
	char *db, *dataBuf, *data;

	printf("[test_procSdmMsg] Enter\n");

	edesc = (eee_desc_t *)eee_ramAlloc(sizeof(eee_desc_t));
	if( !edesc ) {
		printf("---- EDESC eee_ramAlloc failed.\n");
		return 0;
	}
		
	db = dataBuf = (char *)eee_ramAlloc(len);
	if( !db ) {
		printf("---- Data Buffer eee_ramAlloc failed.\n");
		eee_ramDealloc(edesc);
		return 0;
	}

	/* File data into the buffer */
	data = msg->str;
	for(i=0; i < len; i++) 
		*db++ = *data++;

	/* Setup edesc */
	edesc->bd[1].buf = dataBuf;
	edesc->bd[1].len = len;

	edesc->hdr.control = (len & EDESC_TOTAL_LEN) |
		((1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

	scsi_receiveMsg(edesc);
	printf("[test_procSdmMsg] Leave\n");

#endif /* LINUX_TEST */
	return 0;	
} /* test_procSdmMsg */

int
readHalDataFile( char *dataFile )
{
#ifdef LINUX_TEST
    int len;
    char *buf, *data;
    hal_newDev_t *newDev;

    len = test_readFile2Buf(dataFile, &buf);
    if( len == 0 ) {
        printf("---- Test Failed.\n");
        return -1;
    }
    newDev = (hal_newDev_t *)buf;
    while(len > 0) {
        scsi_llDeviceNotify(newDev->event,
                            newDev->tType,
                            newDev->llHandle,
                            newDev->devWWN,
                            newDev->chnWWN,
                            &(newDev->chName[0]),
                            newDev->lun);
        newDev++;
        len -= sizeof(hal_newDev_t);
    }
    eee_ramDealloc(buf);
#endif /* LINUX_TEST */
    return 0;
} /* readHalDataFile */

int
readDscDataFile( char *dataFile )
{
#ifdef LINUX_TEST
	eee_desc_t *edesc;
	edesc = (eee_desc_t *)test_readFile2Edesc(dataFile);
	if( !edesc ) {
		printf("---- Test Failed.\n");
		return -1;
	}

	scsi_receiveMsg(edesc); 

#endif /* LINUX_TEST */
	return 0;
} /* readDscDataFile */
int
readGMCDataFile( char *dataFile )
{
#ifdef LINUX_TEST
	eee_desc_t *edesc;
	edesc = (eee_desc_t *)test_readFile2Edesc(dataFile);
	if( !edesc ) {
		printf("---- Test Failed.\n");
		return -1;
	}

	scsi_receiveMsg( edesc );

#endif /* LINUX_TEST */
	return 0;
} /* runGMCDataTest */

int
test_readFile2Buf(char *filename, char **dataBuf)
{
#ifdef LINUX_TEST
	int  len;
	char ch;
	char *db;
	int fdesc;

	len = 0;

	fdesc = open(filename, O_RDONLY);
	if( fdesc == -1 ) {
		printf("---- Cannot find file: %s\n", filename);
		return 0;
	}

	db = *dataBuf = (char *)eee_ramAlloc(TEST_MAX_DB_SIZE);
	if( !db ) {
		printf("---- Data Buffer eee_ramAlloc failed.\n");
		return 0;
	}

	/* File data into the buffer */
	while( read(fdesc, db, 1) != 0 ) {
		db++;
		len++;
	}
	close(fdesc);

	printf("**** Data Bytes Read: %d\n", len);

	return len;
#else
	return 0;
#endif /* LINUX_TEST */
} /* test_readFile2Buf */

eee_desc_t *
test_readFile2Edesc(char *filename)
{
#ifdef LINUX_TEST
	int  len;
	char ch;
	eee_desc_t *edesc;
	char *db, *dataBuf;
	int fdesc;

	fdesc = open(filename, O_RDONLY);
	if( fdesc == -1 ) {
		printf("---- Cannot find file: %s\n", filename);
		return 0;
	}

	edesc = (eee_desc_t *)eee_ramAlloc(sizeof(eee_desc_t));
	if( !edesc ) {
		printf("---- EDESC eee_ramAlloc failed.\n");
		return 0;
	}
		
	db = dataBuf = (char *)eee_ramAlloc(TEST_MAX_DB_SIZE);
	if( !db ) {
		printf("---- Data Buffer eee_ramAlloc failed.\n");
		eee_ramDealloc(edesc);
		return 0;
	}

	/* File data into the buffer */
	len=0;
	while( read(fdesc, db, 1) != 0 ) {
		db++;
		len++;
	}
	close(fdesc);
#if 0
	while( (ch = fgetc(filep)) != EOF ) {
		*db++ = ch;
		len++;
	}
#endif

	printf("**** Data Bytes Read: %d\n", len);

	/* Setup edesc */
	edesc->bd[1].buf = dataBuf;
	edesc->bd[1].len = len;

	edesc->hdr.control = (len & EDESC_TOTAL_LEN) |
		((1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

	return edesc;
#else
	return 0;
#endif /* LINUX_TEST */
} /* test_readFile2Edesc */

int
test_devRsp(scsi_desc_t *sd, uchar8 *data, int32 len)
{
	int  i;
	eee_desc_t *edesc;
	char *db, *dataBuf;

	printf("[test_devRsp] Enter\n");

	edesc = (eee_desc_t *)eee_ramAlloc(sizeof(eee_desc_t));
	if( !edesc ) {
		printf("---- EDESC eee_ramAlloc failed.\n");
		return 0;
	}
		
	db = dataBuf = (char *)eee_ramAlloc(len);
	if( !db ) {
		printf("---- Data Buffer eee_ramAlloc failed.\n");
		eee_ramDealloc(edesc);
		return 0;
	}

	/* File data into the buffer */
	for(i=0; i < len; i++) 
		*db++ = *data++;

	/* Setup edesc */
	edesc->bd[1].buf = dataBuf;
	edesc->bd[1].len = len;

	edesc->hdr.control = (len & EDESC_TOTAL_LEN) |
		((1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

	sd->eDesc = edesc;
	scsi_llReceive( sd );
	printf("[test_devRsp] Leave\n");
	return 0;	
} /* test_devRsp */

uchar8 inquiryData[] = {
 0x00,0x80,0x02,0x02,0x26,0x00,0x00,0x18,0x48,0x50,0x20,0x20,0x20,0x20,0x20,0x20,
 0x48,0x50,0x33,0x35,0x34,0x38,0x30,0x41,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
 0x41,0x20,0x20,0x20,0x30,0x30,0x30,0x2f,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

int
test_inquiryResponse(scsi_desc_t *sd)
{
	uint32 len;

	printf("[test_inquiryResponse] Enter\n");

	len = sizeof(inquiryData);
	test_devRsp(sd, inquiryData, len);

	printf("[test_inquiryResponse] Leave\n");
	return 0;	
} /* test_inquiryResponse */

uchar8 rcapData[] = { 0x00,0x80,0x02,0x02,0x26,0x00,0x00,0x18 };

int
test_rcapResponse(scsi_desc_t *sd)
{
	uint32 len;

	printf("[test_rcapResponse] Enter\n");

	len = sizeof(rcapData);
	test_devRsp(sd, rcapData, len);

	printf("[test_rcapResponse] Leave\n");
	return 0;	
} /* test_rcapResponse */

uchar8 rlunData[] = { 0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00, 
 0x03
};

int
test_rlunResponse(scsi_desc_t *sd)
{
	uint32 len;

	printf("[test_rlunResponse] Enter\n");

	len = sizeof(rlunData);
	test_devRsp(sd, rlunData, len);

	printf("[test_rlunResponse] Leave\n");
	return 0;	
} /* test_rlunResponse */

int32 
test_devRsp_tmrh( uint32 tmr, void *p1, void *p2, void *p3, void *p4 )
{
	scsi_desc_t *sd;

	eee_timerStop(tmr);
	eee_timerDestroy(tmr);

	sd = (scsi_desc_t *)p1;

	switch( sd->control & SD_OPCODE_MASK )
	{
		case SD_OPCODE_INQUIRY:
			test_inquiryResponse(sd);
			break;
		case SD_OPCODE_RLUN:
			test_rlunResponse(sd); 
			break;
		case SD_OPCODE_RCAP:
			test_rcapResponse(sd);
			break;
		default:
			printf("[test_devRsp_tmrh] Unknown device response: %d\n", 
				   sd->control & SD_OPCODE_MASK);
	}
	return 0;
} /* test_devRsp_tmrh */

int
test_sendInquiryRsp( scsi_desc_t *sd)
{
	int32 tmr;

	tmr = eee_timerCreate(100, EEE_TIMER_ONCE);

	eee_timerStart( tmr,
					test_devRsp_tmrh,
					sd,0, 0, 0);

	return 0;
} /* test_sendInquiryRsp */

int
test_sendRcapRsp( scsi_desc_t *sd)
{
	int32 tmr;

	tmr = eee_timerCreate(100, EEE_TIMER_ONCE);

	eee_timerStart( tmr,
					test_devRsp_tmrh,
					sd,0, 0, 0);

	return 0;
} /* test_sendRcapRsp */

int
test_sendRlunRsp( scsi_desc_t *sd)
{
	int32 tmr;

	tmr = eee_timerCreate(100, EEE_TIMER_ONCE);

	eee_timerStart( tmr,
					test_devRsp_tmrh,
					sd,0, 0, 0);

	return 0;
} /* test_sendRlunRsp */

