/*
 *------------------------------------------------------------------------------
 *
 * Name:	scsi-queue.h
 *
 * RCS:		$Id: scsi-queue.h,v 1.2 2001/08/29 01:06:51 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Definitions for SCSI descriptor queues.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 8/22/2001
 *
 *------------------------------------------------------------------------------
 */

#ifndef __SCSI_QUEUE_H__
#define __SCSI_QUEUE_H__

#define Q_USE_LOCALITY	1

typedef struct {
	scsi_desc_t *		q_head;
	ushort16		q_len;
#ifdef Q_USE_LOCALITY
	scsi_desc_t *		last_insertion_point;
#endif

	/* Stats */
	uint32			q_traversals;
	uint32			q_enqueues;
	uint32			q_dequeues;
} scsi_q_simple_t;

#define Q_TYPE_SORTED	1
#define Q_TYPE_LILO	2

typedef struct {
	uint32			q_type;
	int32			q_timer_duration;
	int32 (*q_timer_callback)(void *);
	void *			q_cookie;

	/* For sorted Q */
	scsi_q_simple_t	*	sorted_q_a;
	scsi_q_simple_t *	sorted_q_b;

	/* For LILO Q */
	scsi_q_simple_t *	lilo_q;

	int32			q_timer_handle;
	int32			q_timer_running;
} scsi_q_t;

scsi_q_t *	scsi_queue_create(uint32, int32 timer_duration, int32 (*callback)(void *), void *cookie);
void		scsi_queue_destroy(scsi_q_t *);

void		scsi_enqueue(scsi_q_t *, scsi_desc_t *);
scsi_desc_t *	scsi_dequeue(scsi_q_t *);
scsi_desc_t *	scsi_queue_head(scsi_q_t *);
int32		scsi_queue_len(scsi_q_t *);
void		scsi_queue_timer_restart(scsi_q_t *);
void		scsi_queue_timer_stop(scsi_q_t *);

void		scsi_queue_dump(scsi_q_t *);

void		scsi_enqueue_simple_lilo(scsi_q_simple_t *, scsi_desc_t *);
void		scsi_enqueue_simple_sorted(scsi_q_simple_t *, scsi_desc_t *);
scsi_desc_t *	scsi_dequeue_simple(scsi_q_simple_t *);

#endif /* __SCSI_QUEUE_H__ */
