/*  
 *------------------------------------------------------------------------------
 *  FileName   : scsit-ui.c
 *
 *  Description: SCSI Test application embedded user-interface functions.
 *
 *	Created    : 09 August 2002
 *
 *  Author     : Frank Salzmann
 *
 *  RCS        : $Id: scsit-ui.c,v 1.3 2002/09/25 20:16:04 frank Exp $
 *
 *  Copyright (c)  Agile Storage, Inc. 2001, 2002
 *------------------------------------------------------------------------------
 */			 
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include <exports/esm-api.h>
#include "../sm-scsi/scsi-api.h"
#include "../sm-sdm/sdm-api.h"
#include "scsit.h"


char scsitest_msgBuf[10000];
int64 scsit_getElapsedTime_usec( int32 end_sec, int32 end_usec, int32 start_sec, int32 start_usec );

#define LEAF	0
#define BRANCH	1

typedef struct _scsitest_ui_node {
	char *name;
	int   type;				// UI_BRANCH_NODE | UI_LEAF_NODE
	int   arg_cnt;			// Min argument count for leaf node
	struct _scsitest_ui_node *branch;
	int (*cmd_func)();
	char *usage;
} scsit_ui_node_t;

// User-Interface Command Function Prototypes
int scsitest_ui_usage( char *prg_name, scsit_ui_node_t *nd );
int scsitest_ui_parse(char *prg_name, scsit_ui_node_t *nd, int argc, char *argv[] );
int scsitest_ui_viewStats();
int scsitest_ui_clearStats();
int scsitest_ui_read();
int scsitest_ui_readu();
int scsitest_ui_readx();
int scsitest_ui_write();
int scsitest_ui_writeu();
int scsitest_ui_stop();
int scsitest_ui_restart();

//-- View Commands
scsit_ui_node_t scsit_ui_view[] = {
  {"stats" , LEAF,  0,  0, scsitest_ui_viewStats, "view stats"},
  {0, 0, 0, 0, 0, 0}
};

//-- Clear Commands
scsit_ui_node_t scsit_ui_clear[] = {
  {"stats" , LEAF,  0,  0, scsitest_ui_clearStats   , "clear stats"},
  {0, 0, 0, 0, 0, 0}
};

//-- Top Level Command Table
scsit_ui_node_t scsit_ui_root[] = {
  {"clear",   	  BRANCH,2,  scsit_ui_clear, 0, 0},
  {"read",        LEAF,  4,  0, scsitest_ui_read,    "read <handle> <lbastart> <numBlks> <mode>"  },
  {"readu",       LEAF,  4,  0, scsitest_ui_readu,   "readu <handle> <lbastart> <numBlks> <mode>"},
  {"restart",     LEAF,  1,  0, scsitest_ui_restart, "restart <handle>"                        },
  {"stop",        LEAF,  1,  0, scsitest_ui_stop,    "stop <handle>"                              },
  {"write",       LEAF,  4,  0, scsitest_ui_write,   "write <handle> <lbastart> <numBlks> <mode>"},
  {"writeu",      LEAF,  4,  0, scsitest_ui_writeu,  "writeu <handle> <lbastart> <numBlks> <mode>"},
  {"view",   	  BRANCH,2,  scsit_ui_view, 0, 0},
  {0, 0, 0, 0, 0, 0}
};

scsitest_uiAdmin( int32 argc, char **argv )
{
	if( argc == 1 ) {
		printf("\n");
		scsitest_ui_usage(argv[0], &scsit_ui_root[0]);
		printf("\n");
		return(0);
	}
	scsitest_ui_parse(argv[0], &scsit_ui_root[0], --argc, &argv[1]);
	return 0;

} // scsitest_uiAdmin

int
scsitest_ui_usage( char *prg_name, scsit_ui_node_t *nd )
{
	scsit_ui_node_t *node;

	node = nd;
	while(node->name) {
		if( node->type == LEAF )
			printf("Usage: %s %s\n", prg_name, node->usage);
		else
			scsitest_ui_usage(prg_name, node->branch);
		node++;
	} // while
	return 0;
} // scsitest_ui_usage

int
scsitest_ui_parse(char *prg_name, scsit_ui_node_t *nd, int argc, char *argv[] )
{
	int  rc;
	char *tok;
	scsit_ui_node_t *node;
	
	node = nd;
	tok = argv[0];

	if( tok ) {
		while( node->name ) {
			if( !(strcmp(tok, node->name)) ) {
				if( node->type == LEAF ) {
					if( --argc >= node->arg_cnt ) {
						rc = node->cmd_func(argc, &argv[1]);
						if( rc < 0 )
							printf("Usage: %s %s\n", prg_name, node->usage);
					}
					else
						printf("Usage: %s %s\n", prg_name, node->usage);
				}
				else {
					if( --argc > 0 )
						scsitest_ui_parse(prg_name, node->branch, argc, &argv[1]);
					else
						scsitest_ui_usage(prg_name, node->branch);
				}
				return 0;
			}
			else {
				node++;
			}
		} // while
	}
	printf("Command Not Found.\n");
	return 0;
} // scsitest_ui_parse

/*-----------------------------------------------------------------
 * Name: scsitest_ui_read
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: August 05, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_ui_read(int argc, char *argv[])
{
	scsi_test_t *trec;
	scsi_readReqMsg_t *readReq;
	scsi_path_t path;
	uint32 i;
	uint32 scsiAppAddr;
	uint32 handle;
	uint32 lbStart;
	uint32 blkCnt;
	uint32 mode;
	uint32 len;

	if( argc != 4 )
		return -1;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);
	lbStart = strtoq(argv[1],NULL,0);
	blkCnt  = strtoq(argv[2],NULL,0);
	mode    = strtoq(argv[3],NULL,0);

	printf("READ META TEST: Handle: %x, lbStart: %d, blkCnt: %d, I/O's: %d, mode: %d\n", 
		   handle, lbStart, blkCnt, blkCnt/SCSITEST_READ_BLKCNT, mode);

	//-- Find next available test record.
	for(i=0; i < SCSITEST_MAX_LUNS; i++) {
		if( scsitest[i].control == SCSITEST_CONTROL_AVAIL ) {
			trec = &scsitest[i];
			trec->control  = SCSITEST_CONTROL_INUSE;
			trec->testType = SCSITEST_TYPE_READ;
			trec->handle   = handle;
			trec->lbStart  = lbStart;
			trec->lbEnd    = lbStart + blkCnt;
			trec->numBlks  = blkCnt;
			trec->testMode = mode;
			trec->chandle  = i;
			trec->readIoCnt = 0;
			break;
		}
	}
	if( !trec ) {
		printf("SCSITEST: No test record available.\n");
		return(NFX_ERR); 
	}

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Create SCSI read request message and send it on down.
	readReq = (scsi_readReqMsg_t *)&scsitest_msgBuf[0];
	readReq->hdr.mid     = SCSI_MID_READ_REQ;
	readReq->hdr.cHandle = trec->chandle;
	readReq->hdr.path    = path;
	readReq->transSeqId  = 0;
	readReq->blkOffset   = lbStart;
	readReq->blkCnt      = SCSITEST_READ_BLKCNT;
	readReq->pad[0]      = 0;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			readReq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			readReq->attributes  = 0;
	readReq->dataDest    = 0;
	
	//-- Send down the initial seed requests to the SCSI application.
	trec->blkCur = SCSITEST_MAX_PKT;
	len = sizeof(scsi_readReqMsg_t);
	for(i=0; i < SCSITEST_MAX_PKT; i++) {
		readReq->hdr.cHandle = (trec->chandle | ((i << 16) & 0xFFFF0000));
		eee_sendMessage((uchar8 *)readReq, len, scsiAppAddr, scsitest_addr);
	}
	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_ui_read

/*-----------------------------------------------------------------
 * Name: scsitest_ui_write
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: August 05, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_ui_write(int argc, char *argv[])
{
	scsi_test_t *trec;
	scsi_writeReqMsg_t *wreq;
	scsi_path_t path;
	uint32      i;
	uint32      scsiAppAddr;
	uint32      handle;
	uint32      lbStart;
	uint32      blkCnt;
	uint32      mode;
	uint32      len;
	eee_desc_t *edesc;
	char       *buf;

	if( argc != 4 )
		return -1;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);
	lbStart = strtoq(argv[1],NULL,0);
	blkCnt  = strtoq(argv[2],NULL,0);
	mode    = strtoq(argv[3],NULL,0);

	printf("WRITE META TEST: Handle: %x, lbStart: %d, blkCnt: %d, I/O's: %d, mode: %d\n", 
		   handle, lbStart, blkCnt, blkCnt/SCSITEST_READ_BLKCNT, mode);

	//-- Find next available test record.
	for(i=0; i < SCSITEST_MAX_LUNS; i++) {
		if( scsitest[i].control == SCSITEST_CONTROL_AVAIL ) {
			trec = &scsitest[i];
			trec->control  = SCSITEST_CONTROL_INUSE;
			trec->testType = SCSITEST_TYPE_WRITE;
			trec->handle   = handle;
			trec->lbStart  = lbStart;
			trec->lbEnd    = lbStart + blkCnt;
			trec->numBlks  = blkCnt;
			trec->testMode = mode;
			trec->chandle  = i;
			trec->readIoCnt = 0;
			break;
		}
	}
	if( !trec ) {
		printf("SCSITEST: No test record available.\n");
		return(NFX_ERR); 
	}

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Send down the initial seed requests to the SCSI application.
	trec->blkCur = 64;
	for(i=0; i < 64; i++) {

		//-- Allocate Ultra Buffer
		buf =  eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA,0);
		if( !buf ) {
			return(NFX_ERR);  
		}

		//-- Allocate edesc
		edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
		if( !edesc ) {
			eee_deallocateBuffer(buf);
			return(NFX_ERR);
		}

		//-- Create SCSI write request message 
		buf+=128;
		wreq = (scsi_writeReqMsg_t *)buf;
		wreq->hdr.mid     = SCSI_MID_WRITE_REQ;
		wreq->hdr.cHandle = trec->chandle;
		wreq->hdr.path    = path;
		wreq->transSeqId  = 0;
		wreq->blkOffset   = lbStart;
		wreq->blkCnt      = SCSITEST_WRITE_BLKCNT;
		if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			wreq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			wreq->attributes  = 0;
		wreq->checkSum    = 0;
	
		//-- Setup edesc and send pkt on down to SCSI.
		len = sizeof(scsi_writeReqMsg_t) + (SCSITEST_WRITE_BLKCNT * 512);
		edesc->bd[1].buf = buf;
		edesc->bd[1].len = len;
		edesc->hdr.src_port  = scsitest_addr;
		edesc->hdr.dest_port = scsiAppAddr;
		edesc->hdr.control = ((len & EDESC_TOTAL_LEN) | 
						  ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));

		eee_forwardPacket(edesc);
	}

	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_ui_write

//-----------------------------------------------------------------
// Name: scsitest_ui_stop
// 
// Description: Display SCSI message statistics.
//
// Created by:  Frank Salzmann
//
// Date Created: August 05, 2002
//-----------------------------------------------------------------
int
scsitest_ui_stop(int argc, char *argv[])
{
	uint32 i;
	uint32 handle;
	scsi_test_t *trec;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);

	for(i=0; i <SCSITEST_MAX_LUNS; i++) {
		trec = &scsitest[i];
		if(trec->handle == handle ) {
			trec->testMode = SCSITEST_MODE_STOP; 
			return(NFX_OK);
		}
	} 
	printf("SCSITEST: Handle: %x not found.\n", handle);
	return 0;
} // scsitest_ui_stop


//-----------------------------------------------------------------
// Name: scsitest_ui_viewStats
// 
// Description: Display SCSI message statistics.
//
// Created by:  Frank Salzmann
//
// Date Created: April 01, 2002
//-----------------------------------------------------------------
int
scsitest_ui_viewStats(int argc, char *argv[])
{
	uint32 i;
	scsi_test_t *trec;
	eeeGMT_t stop_t;
	int64 usecs;
	uint32 secs;

	eee_TimeGMT( &stop_t );

	
	printf("SCSITEST Stats\n");
	printf("==============\n");
	printf("Errors: %d\n", scsitest_errCnt);
	printf("            Rd IO     Avg Rd      Rd        Rd       Wr IO     Avg Wr      Wr         Wr   \n");
	printf(" Handle      Cnt      (usec)    IO/sec   byte/sec     Cnt      (usec)    IO/sec   byte/sec\n");
	printf("---------  --------  --------  --------  ---------  --------  --------  --------  ---------\n");
	for(i=0; i < SCSITEST_MAX_LUNS; i++ ) {
		trec = &scsitest[i];
		if( trec->control == SCSITEST_CONTROL_INUSE ) {
			usecs = scsit_getElapsedTime_usec( stop_t.sec, stop_t.usec, 
											   trec->start_t.sec, trec->start_t.usec);
			secs = usecs / 1000000;

			if(secs == 0)
				secs = 1;

			printf("0x%x  %8lld  %8lld  %8lld  %9lld  %8lld  %8lld  %8lld  %9lld\n", 
				   trec->handle, 
				   trec->readIoCnt,
			       trec->readIoCnt ? usecs / trec->readIoCnt:0,
				   trec->readIoCnt/secs,
				   (trec->readIoCnt*512*SCSITEST_READ_BLKCNT) / secs,
				   trec->writeIoCnt, 
			       trec->writeIoCnt ? usecs / trec->writeIoCnt:0,
				   trec->writeIoCnt/secs,
				   (trec->writeIoCnt*512*SCSITEST_WRITE_BLKCNT) / secs);
		} 
	}
	

	return 0;
} // scsitest_ui_viewStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsitest_ui_clearStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	August 05, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsitest_ui_clearStats()
{
	
	uint32 i;
	scsi_test_t *trec;

	for(i=0; i < SCSITEST_MAX_LUNS; i++ ) {
		trec = &scsitest[i];
		trec->readIoCnt     = 0;
		trec->writeIoCnt    = 0;
		trec->tTime_read    = 0;
		trec->tTime_write   = 0;
		eee_TimeGMT(&(trec->start_t));
	}
	printf("SCSI test stats cleared!\n");
	return NFX_OK;
} // scsitest_ui_clearStats

/*-----------------------------------------------------------------
 * Name: scsitest_ui_readu
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: September 21, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_ui_readu(int argc, char *argv[])
{
	scsi_test_t *trec;
	scsi_readReqMsg_t *readReq;
	scsi_path_t path;
	uint32 i;
	uint32 scsiAppAddr;
	uint32 handle;
	uint32 lbStart;
	uint32 blkCnt;
	uint32 mode;
	uint32 len;

	if( argc != 4 )
		return -1;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);
	lbStart = strtoq(argv[1],NULL,0);
	blkCnt  = strtoq(argv[2],NULL,0);
	mode    = strtoq(argv[3],NULL,0);

	printf("READ USER TEST: Handle: %x, lbStart: %d, blkCnt: %d, I/O's: %d, mode: %d\n", 
		   handle, lbStart, blkCnt, blkCnt/SCSITEST_READ_BLKCNT, mode);

	//-- Find next available test record.
	for(i=0; i < SCSITEST_MAX_LUNS; i++) {
		if( scsitest[i].control == SCSITEST_CONTROL_AVAIL ) {
			trec = &scsitest[i];
			trec->control  = SCSITEST_CONTROL_INUSE;
			trec->testType = SCSITEST_TYPE_UREAD;
			trec->handle   = handle;
			trec->lbStart  = lbStart;
			trec->lbEnd    = lbStart + blkCnt;
			trec->numBlks  = blkCnt;
			trec->testMode = mode;
			trec->chandle  = i;
			trec->readIoCnt = 0;
			break;
		}
	}
	if( !trec ) {
		printf("SCSITEST: No test record available.\n");
		return(NFX_ERR); 
	}

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Create SCSI read request message and send it on down.
	readReq = (scsi_readReqMsg_t *)&scsitest_msgBuf[0];
	readReq->hdr.mid     = SCSI_MID_READ_REQ;
	readReq->hdr.cHandle = trec->chandle;
	readReq->hdr.path    = path;
	readReq->transSeqId  = 0;
	readReq->blkOffset   = lbStart;
	readReq->blkCnt      = SCSITEST_READ_BLKCNT;
	readReq->pad[0]      = 0;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			readReq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			readReq->attributes  = 0;
	readReq->dataDest    = 0;
	
	//-- Send down the initial seed requests to the SCSI application.
	trec->blkCur = SCSITEST_MAX_PKT;
	len = sizeof(scsi_readReqMsg_t);
	for(i=0; i < SCSITEST_MAX_PKT; i++) {
		readReq->hdr.cHandle = (trec->chandle | ((i << 16) & 0xFFFF0000));
		eee_sendMessage((uchar8 *)readReq, len, scsiAppAddr, scsitest_sac_addr);
	}
	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_ui_readu

/*-----------------------------------------------------------------
 * Name: scsitest_ui_writeu
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: August 05, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_ui_writeu(int argc, char *argv[])
{
	scsi_test_t *trec;
	scsi_readReqMsg_t *readReq;
	scsi_path_t path;
	uint32      i;
	uint32      scsiAppAddr;
	uint32      handle;
	uint32      lbStart;
	uint32      blkCnt;
	uint32      mode;
	uint32      len;

	if( argc != 4 )
		return -1;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);
	lbStart = strtoq(argv[1],NULL,0);
	blkCnt  = strtoq(argv[2],NULL,0);
	mode    = strtoq(argv[3],NULL,0);

	printf("WRITE USER TEST: Handle: %x, lbStart: %d, blkCnt: %d, I/O's: %d, mode: %d\n", 
		   handle, lbStart, blkCnt, blkCnt/SCSITEST_READ_BLKCNT, mode);

	//-- Find next available test record.
	for(i=0; i < SCSITEST_MAX_LUNS; i++) {
		if( scsitest[i].control == SCSITEST_CONTROL_AVAIL ) {
			trec = &scsitest[i];
			trec->control   = SCSITEST_CONTROL_INUSE;
			trec->testType  = SCSITEST_TYPE_UWRITE;
			trec->handle    = handle;
			trec->lbStart   = lbStart;
			trec->lbEnd     = lbStart + blkCnt;
			trec->numBlks   = blkCnt;
			trec->testMode  = mode;
			trec->chandle   = i;
			trec->readIoCnt = 0;
			trec->blkCur    = 0;
			break;
		}
	}
	if( !trec ) {
		printf("SCSITEST: No test record available.\n");
		return(NFX_ERR); 
	}

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}


	//-- We need to seed the SAC with a COD that can then be written over and 
	//-- over to the disk.  So..
	//-- Create SCSI read request message 
	//-- Create SCSI read request message and send it on down.
	readReq = (scsi_readReqMsg_t *)&scsitest_msgBuf[0];
	readReq->hdr.mid     = SCSI_MID_READ_REQ;
	readReq->hdr.cHandle = trec->chandle;
	readReq->hdr.path    = path;
	readReq->transSeqId  = 0;
	readReq->blkOffset   = lbStart;
	readReq->blkCnt      = SCSITEST_WRITE_BLKCNT;
	readReq->pad[0]      = 0;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			readReq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			readReq->attributes  = 0;
	readReq->dataDest    = 0;
	
	//-- Send down the request to the SCSI application.
	len = sizeof(scsi_readReqMsg_t);
	readReq->hdr.cHandle = (trec->chandle | ((i << 16) & 0xFFFF0000));
	eee_sendMessage((uchar8 *)readReq, len, scsiAppAddr, scsitest_sac_addr);

	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_ui_writeu

/*-----------------------------------------------------------------
 * Name: scsitest_ui_restart
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: September 06, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_ui_restart(int argc, char *argv[])
{
	scsi_test_t *trec;
	scsi_readReqMsg_t *readReq;
	scsi_path_t path;
	uint32 i;
	uint32 scsiAppAddr;
	uint32 handle;
	uint32 lbStart;
	uint32 blkCnt;
	uint32 mode;
	uint32 len;

	if( argc != 4 )
		return -1;

	//-- Gather test parameters.
	handle  = strtoq(argv[0],NULL,0);

	printf("Restart: Handle: %x\n", handle);

	//-- Find next available test record.
	for(i=0; i < SCSITEST_MAX_LUNS; i++) {
		if( scsitest[i].control == SCSITEST_CONTROL_AVAIL ) {
			trec = &scsitest[i];
			trec->control  = SCSITEST_CONTROL_INUSE;
			trec->testType = SCSITEST_TYPE_READ;
			trec->handle   = handle;
			trec->lbStart  = lbStart;
			trec->lbEnd    = lbStart + blkCnt;
			trec->numBlks  = blkCnt;
			trec->testMode = mode;
			trec->chandle  = i;
			trec->readIoCnt = 0;
			break;
		}
	}
	if( !trec ) {
		printf("SCSITEST: No test record available.\n");
		return(NFX_ERR); 
	}

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Create SCSI read request message and send it on down.
	readReq = (scsi_readReqMsg_t *)&scsitest_msgBuf[0];
	readReq->hdr.mid     = SCSI_MID_READ_REQ;
	readReq->hdr.cHandle = trec->chandle;
	readReq->hdr.path    = path;
	readReq->transSeqId  = 0;
	readReq->blkOffset   = lbStart;
	readReq->blkCnt      = SCSITEST_READ_BLKCNT;
	readReq->pad[0]      = 0;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			readReq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			readReq->attributes  = 0;
	readReq->dataDest    = 0;
	
	//-- Send down the initial seed requests to the SCSI application.
	trec->blkCur = SCSITEST_MAX_PKT;
	len = sizeof(scsi_readReqMsg_t);
	for(i=0; i < SCSITEST_MAX_PKT; i++) {
		readReq->hdr.cHandle = (trec->chandle | ((i << 16) & 0xFFFF0000));
		eee_sendMessage((uchar8 *)readReq, len, scsiAppAddr, scsitest_addr);
	}

	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_ui_restart
