/*
 * ----------------------------------------------------------------------------
 *  FileName: scsit.c
 *
 *  Description:  
 *
 *  Created:    05 August 2002
 *
 *  Author:     Frank Salzmann
 *
 *  Copyright (c)  Agile Storage, Inc. 2001, 2002
 *
 *  CVS Version:  $Id: scsit.c,v 1.3 2002/09/25 20:16:04 frank Exp $ 
 * ----------------------------------------------------------------------------
 */


#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include <exports/esm-api.h>
#include <exports/sbm-api.h>
#include "../sm-scsi/scsi-api.h"
#include "../sm-sdm/sdm-api.h"
#include "scsit.h"

// 
// SCSITEST Global Variables
//
uint32	        scsitest_appId;
uint32          scsitest_addr;
uint32          scsitest_sac_addr;
uint32          scsitest_sbm_addr;
uint32          scsitest_errCnt = 0;
scsi_test_t     scsitest[SCSITEST_MAX_LUNS];

int
sdma_procHandleReqMsg(eee_desc_t *edesc, sdm_sdmaHandleReqMsg_t *i_msg);
int64
scsit_getElapsedTime_usec( int32 end_sec, int32 end_usec, int32 start_sec, int32 start_usec );

/*
 *------------------------------------------------------------------------------
 * Function Name:   scsitest_init
 *
 * Description:     Initializes the SDM agent.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    August 20, 2001
 *
 *------------------------------------------------------------------------------
 */
int
scsitest_init()
{
	int    rc;
	uint32 scsitest_sac_appId;
	uint32 scsitest_sbm_appId;

	// Initialize device handle table.
	// Table will be indexed by slot/cpu/handle.
	memset(scsitest, 0, sizeof(scsitest));

	// Setup scsitest messaging.
	scsitest_appId = eee_getApplicationId("scsitest");
    if( scsitest_appId == NFX_ERR )
    {
		printf("SCSITEST: Get scsitest app id failed.");
        return NFX_ERR;
    }

    scsitest_addr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId,
											   nfxMyCPUId,
											   scsitest_appId,
											   EEE_USER_DATA,
											   NFX_COS_HIGH,
											   0);


	scsitest_sac_appId = eee_getApplicationId("sac_scsit");

    if( scsitest_sac_appId == NFX_ERR )
    {
		printf("SCSITEST: Get sac app id failed.");
        return NFX_ERR;
    }
	scsitest_sac_addr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId,
												   NFX_NFP_TXRX_ANY,
												   scsitest_sac_appId,
												   EEE_SAC,
												   NFX_COS_HIGH,
												   PORT_BEST_FIT);

	rc = eee_registerReceiveFunc( scsitest_appId,   // App Id
                                  scsitest_rcvMsg,  // Ucast Rx Func
                                  NULL);            // Mcast Rx Func
    if( rc == NFX_ERR )
    {
        printf("SCSITEST: Register application rx func failed.");
        return NFX_ERR;
    }


	scsitest_sbm_appId = eee_getApplicationId("sbm");

    if( scsitest_sbm_appId == NFX_ERR )
    {
		printf("SCSITEST: Get sbm app id failed.");
        return NFX_ERR;
    }
	scsitest_sbm_addr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId,
												   NFX_NFP_TXRX_ANY,
												   scsitest_sbm_appId,
												   EEE_CONTROL,
												   NFX_COS_HIGH,
												   0);
	return(NFX_OK);

} // scsitest_init

/*
 *------------------------------------------------------------------------------
 * Function Name:   scsitest_rcvMsg
 *
 * Description:     Receives messages sent to scsitest app.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    August 5, 2002
 *
 *------------------------------------------------------------------------------
 */
void
scsitest_rcvMsg( eee_desc_t *edesc )
{
    uint32        mid;
    scsi_msgHdr_t *msg;
    eee_descBuf_t *bd;
 
    // printf("[scsitest_rcvMsg] Enter");
 
    /*
     * Point to request message portion of data buffer.
     */
    if( !edesc ) {
        printf("Null eDesc");
        return;
    }
 
    bd = &edesc->bd[1];
    msg = (scsi_msgHdr_t *)(bd->buf);
    mid = msg->mid;

	switch( mid )
    {
        case SCSI_MID_READ_RSP:
            scsitest_procReadRsp( edesc, msg );
            break;
 
        case SCSI_MID_WRITE_RSP:
            scsitest_procWriteRsp( edesc, msg );
            break;

		case SCSI_MID_ERR_RSP:
			scsitest_errCnt++;
			eee_freePacket(edesc);
			break;
        default: // Invalid Message
            printf("[scsitest_rcvMsg] Invalid Message: %x\n", mid);
			eee_freePacket(edesc);
    }

    // printf("[scsitest_rcvMsg] Enter");
    return;

} // scsitest_rcvMsg

int
scsitest_procReadRsp( eee_desc_t *edesc, scsi_readRspMsg_t *i_msg ) 
{
	scsi_test_t *trec;
	uint32 chandle;
	uint32 pkti;
	int32 deltat;


	chandle = i_msg->hdr.cHandle;
	trec = &scsitest[chandle & 0x0000FFFF];
	pkti = ((chandle & 0xFFFF0000) >> 16) & 0x0000FFFF;

	trec->readIoCnt++;
	trec->blkCur+=SCSITEST_READ_BLKCNT;

	trec->lbCur+=SCSITEST_READ_BLKCNT;

	//printf("scsitest: procReadRsp: testtype: %d\n", trec->testType);

	if( trec->testMode == SCSITEST_MODE_STOP ) {
		if( trec->control == SCSITEST_CONTROL_INUSE ) {
			printf("Read Test Stopped For: Handle: %x, I/O Cnt: %lld\n", trec->handle, trec->readIoCnt);
			trec->control = SCSITEST_CONTROL_AVAIL;
		}
		eee_freePacket(edesc);
		return(NFX_OK);
	}

	if( trec->testType == SCSITEST_TYPE_UWRITE ) {
		//-- Get COD from the read rsp message.
		trec->cod = ((scsi_readRspMsg_t *)(edesc->bd[1].buf))->codHandle;
		printf("SAC has seed COD: %llx\n", trec->cod);
		scsitest_writeu_start(trec);
		eee_freePacket(edesc);
		return(NFX_OK);
	}

	if( trec->testMode == SCSITEST_MODE_NORMAL ) {

		if( trec->blkCur >= trec->numBlks ) {
			printf("Read Test Completed: Handle: %x, Blocks Req: %d, I/O's req: %d, I/O's rcvd: %lld\n", 
				   trec->handle, trec->numBlks, trec->numBlks/SCSITEST_READ_BLKCNT, trec->readIoCnt);
			trec->control = SCSITEST_CONTROL_AVAIL;
			trec->testMode = SCSITEST_MODE_STOP;
			eee_freePacket(edesc);
		}
		else {
			if( trec->testType == SCSITEST_TYPE_UREAD ) 
				scsitest_sendUReadReq( trec, edesc, pkti );
			else
				scsitest_sendReadReq( trec, edesc, pkti );
		}
	}
	else
	if( trec->testMode == SCSITEST_MODE_LOOP ) {
		if( trec->blkCur >= trec->numBlks ) {
			trec->blkCur = 0;
			trec->lbCur = trec->lbStart;
		}
		if( trec->testType == SCSITEST_TYPE_UREAD )
			scsitest_sendUReadReq( trec, edesc, pkti );
		else
			scsitest_sendReadReq( trec, edesc, pkti );
	}
	else
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI ) {
		if( trec->blkCur >= trec->numBlks ) {
			trec->blkCur = 0;
			trec->lbCur = trec->lbStart;
		}
		if( trec->testType == SCSITEST_TYPE_UREAD )
			scsitest_sendUReadReq( trec, edesc, pkti );
		else
			scsitest_sendReadReq( trec, edesc, pkti );
	}
	else {
		printf("SCSITEST: Invalid testmode: %d. Terminating test. \n", trec->testMode);
		eee_freePacket(edesc);
	}

	return(NFX_OK);
} // scsitest_procReadRsp

uint32 scsi_seq = 0;

scsitest_sendReadReq( scsi_test_t *trec, eee_desc_t *edesc, uint32 pkti)
{
	scsi_readReqMsg_t *o_msg;
	uint32 path;
	uint32 scsiAppAddr;
	uint32 len;
	
	//printf("scsitest_sendReadReq\n");

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(trec->handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", trec->handle); 
		return(NFX_ERR);
	}
	o_msg = (scsi_readReqMsg_t *)edesc->bd[1].buf;
	o_msg->hdr.mid     = SCSI_MID_READ_REQ;
	o_msg->hdr.cHandle = (trec->chandle | ((pkti << 16) & 0xFFFF0000));
	o_msg->hdr.path    = path;
	o_msg->transSeqId  = 0;
	o_msg->blkOffset   = trec->lbCur;
	o_msg->blkCnt      = SCSITEST_READ_BLKCNT;
	o_msg->pad[0]      = scsi_seq++;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
		o_msg->attributes  = SCSI_MSG_ATTR_LOOPBACK;
	else
		o_msg->attributes  = 0;
	o_msg->dataDest    = 0;
	
	//-- Loop request back down to scsi application.
	len = sizeof(scsi_readReqMsg_t);
	edesc->hdr.dest_port = scsiAppAddr;
	edesc->hdr.src_port  = scsitest_addr;

	edesc->hdr.control = ((len & EDESC_TOTAL_LEN) | 
						  ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));

	eee_forwardPacket(edesc);

	return(NFX_OK);
} // scsitest_sendReadReq

int
scsitest_sendUReadReq( scsi_test_t *trec, eee_desc_t *ed_in, uint32 pkti)
{
	scsi_readRspMsg_t *i_msg;
	scsi_readReqMsg_t *o_msg;
	uint32 scsiAppAddr;
	eee_desc_t *ed_out;
	uint32 path;
	uint32 len;
	char *buf;
	nfx_cod_t cod;

	//printf("scsitest_sendUReadReq\n");


	//-- Allocate Ultra Buffer
	buf =  eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA,0);
	if( !buf ) {
		return(NFX_ERR);  
	}

	//-- Allocate edesc
	ed_out = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if( !ed_out ) {
		eee_deallocateBuffer(buf);
		return(NFX_ERR);
	}


	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(trec->handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", trec->handle); 
		return(NFX_ERR);
	}
	o_msg = (scsi_readReqMsg_t *)buf;
	o_msg->hdr.mid     = SCSI_MID_READ_REQ;
	o_msg->hdr.cHandle = (trec->chandle | ((pkti << 16) & 0xFFFF0000));
	o_msg->hdr.path    = path;
	o_msg->transSeqId  = 0;
	o_msg->blkOffset   = trec->lbCur;
	o_msg->blkCnt      = SCSITEST_READ_BLKCNT;
	o_msg->pad[0]      = scsi_seq++;
	o_msg->dataDest    = 0;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
		o_msg->attributes  = SCSI_MSG_ATTR_LOOPBACK;
	else
		o_msg->attributes  = 0;
	

	//-- Loop request back down to scsi application.
	len = sizeof(scsi_readReqMsg_t);
	ed_out->bd[1].buf     = buf;
	ed_out->hdr.dest_port = scsiAppAddr;
	ed_out->hdr.src_port  = scsitest_sac_addr;
	ed_out->hdr.control   = ((len & EDESC_TOTAL_LEN) | 
							 ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));

	eee_forwardPacket(ed_out);

	//-- Get COD from the read rsp message.
	i_msg = (scsi_readRspMsg_t *)ed_in->bd[1].buf;
	cod = i_msg->codHandle;

	//printf("release cod: %llx\n", cod);

	//-- Send message to the SAC to release this COD.
	scsit_free_cod(cod);
	eee_freePacket(ed_in);

	return(NFX_OK);
} // scsitest_sendUReadReq

int
scsitest_procWriteRsp( eee_desc_t *edesc, scsi_writeRspMsg_t *i_msg ) 
{
	scsi_test_t *trec;
	uint32 chandle;

	// printf("scsitest: procWriteRsp\n");

	chandle = i_msg->hdr.cHandle;
	trec = &scsitest[chandle];

	trec->writeIoCnt++;
	trec->blkCur++;
	trec->lbCur+=SCSITEST_WRITE_BLKCNT;

	if( trec->testMode == SCSITEST_MODE_STOP ) {
		if( trec->control == SCSITEST_CONTROL_INUSE ) {
			printf("Write Test Stopped For: Handle: %x, I/O Cnt: %lld\n", trec->handle, trec->writeIoCnt);
			trec->control = SCSITEST_CONTROL_AVAIL;
		}
		eee_freePacket(edesc);
		return(NFX_OK);
	}

	if( trec->testMode == SCSITEST_MODE_NORMAL ) {

		if( trec->blkCur >= trec->numBlks ) {
			printf("Write Test Completed For: Handle: %x, I/O Cnt: %lld\n", trec->handle, trec->writeIoCnt);
			trec->control = SCSITEST_CONTROL_AVAIL;
			trec->testMode = SCSITEST_MODE_STOP;
			eee_freePacket(edesc);
		}
		else {
			if( trec->testType == SCSITEST_TYPE_UWRITE ) {
				scsitest_sendUWriteReq( trec );
				eee_freePacket(edesc);
			}
			else
				scsitest_sendWriteReq( trec, edesc );
		}
	}
	else
	if( trec->testMode == SCSITEST_MODE_LOOP ) {
		if( trec->blkCur >= trec->numBlks ) {
			trec->blkCur = 0;
			trec->lbCur = trec->lbStart;
		}
		if( trec->testType == SCSITEST_TYPE_UWRITE ) {
			scsitest_sendUWriteReq( trec );
			eee_freePacket(edesc);
		}
		else
			scsitest_sendWriteReq( trec, edesc );
	}
	else
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI ) {
		if( trec->blkCur >= trec->numBlks ) {
			trec->blkCur = 0;
			trec->lbCur = trec->lbStart;
		}
		if( trec->testType == SCSITEST_TYPE_UWRITE ) {
			scsitest_sendUWriteReq( trec );
			eee_freePacket(edesc);
		}
		else
			scsitest_sendWriteReq( trec, edesc );
	}
	else {
		printf("SCSITEST: Invalid testmode: %d. Terminating test. \n", trec->testMode);
		eee_freePacket(edesc);
	}


	return(NFX_OK);
} // scsitest_procWriteRsp

int
scsitest_sendWriteReq( scsi_test_t *trec, eee_desc_t *edesc )
{
	scsi_writeReqMsg_t *o_msg;
	uint32 path;
	uint32 scsiAppAddr;
	uint32 len;

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(trec->handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", trec->handle); 
		return(NFX_ERR);
	}
	o_msg = (scsi_writeReqMsg_t *)&(edesc->bd[1].buf[64]);
	o_msg->hdr.mid     = SCSI_MID_WRITE_REQ;
	o_msg->hdr.cHandle = trec->chandle;
	o_msg->hdr.path    = path;
	o_msg->transSeqId  = 0;
	o_msg->blkOffset   = trec->lbCur;
	o_msg->blkCnt      = SCSITEST_WRITE_BLKCNT;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
		o_msg->attributes  = SCSI_MSG_ATTR_LOOPBACK;
	else
		o_msg->attributes  = 0;
	o_msg->checkSum    = 0;
	
	//-- Loop request back down to scsi application.
	len = sizeof(scsi_writeReqMsg_t) + (SCSITEST_WRITE_BLKCNT * 512);
	edesc->bd[1].buf = (char *)o_msg;
	edesc->bd[1].len = len;
	edesc->hdr.dest_port = scsiAppAddr;
	edesc->hdr.src_port  = scsitest_addr;
	edesc->hdr.control = ((len & EDESC_TOTAL_LEN) | 
						  ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));
	eee_forwardPacket(edesc);

	return(NFX_OK);
} // scsitest_sendWriteReq

//------------------------------------------------------------------------------
// Function Name:   scsitest_printBuf
// 
// Description:     
//
// Data Created:    May 09, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
scsitest_printBuf(char *buf, int len)
{
    int i, j;
    char ch;
    char *data;
 
    data = buf;
    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");
    return 0;
} // scsitest_printBuf


int64
scsit_getElapsedTime_usec( int32 end_sec, int32 end_usec, int32 start_sec, int32 start_usec )
{
	return ((((int64)end_sec - (int64)start_sec) * 1000000) +
             (int64)end_usec) -
             (int64)start_usec;

} // scsit_getElapsedTime_usec

int
scsit_free_cod( nfx_cod_t cod )
{
  int32 rc ;
  sbm_freeCOD_t freemsg;
 
  freemsg.cmd = CMD_TO_MCPU_MANIPULATE_FREE;
  freemsg.COD = cod;

  // send msg to SAC.
  rc = eee_sendMessage( &freemsg,
                        sizeof(freemsg),
						scsitest_sbm_addr,
                        scsitest_addr);
  if (rc != NFX_OK)
  {
	  printf("scsit_free_cod failed %#llx\n", cod);
  }
} // scsit_free_cod
/*-----------------------------------------------------------------
 * Name: scsitest_writeu_start
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: August 05, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_writeu_start(scsi_test_t *trec)
{
	scsi_writeReqMsg_t *wreq;
	scsi_path_t path;
	uint32      i;
	uint32      scsiAppAddr;
	uint32      handle;
	uint32      lbStart;
	uint32      blkCnt;
	uint32      mode;
	uint32      len;
	eee_desc_t *edesc;
	char       *buf;
	sbm_sendControlAndSACData_t *   sbmMsg=NULL;    // SBM WRITE msg  

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(trec->handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Send down the initial seed requests to the SCSI application.
	for(i=0; i < SCSITEST_MAX_PKT; i++) {

		//-- Allocate Ultra Buffer
		buf =  eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA,0);
		if( !buf ) {
			return(NFX_ERR);  
		}

		//-- Allocate edesc
		edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
		if( !edesc ) {
			eee_deallocateBuffer(buf);
			return(NFX_ERR);
		}
		edesc->hdr.attr = 0;
		edesc->hdr.next = NULL;
		edesc->hdr.offset = 0;

		//-- Fill in the SBM message dest_port filled in below
		sbmMsg = (sbm_sendControlAndSACData_t *)buf;
		sbmMsg->cmd         = CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM;
		sbmMsg->source_port = scsitest_addr;
		sbmMsg->dest_port   = scsiAppAddr;
		sbmMsg->msg_len     = sizeof(scsi_writeReqMsg_t);
		sbmMsg->COD         = trec->cod;
		sbmMsg->length      = SCSITEST_WRITE_BLKCNT * 512;
		sbmMsg->offset      = 0; // TODO: need offset in cod

		//-- Create SCSI write request message 
		wreq = (scsi_writeReqMsg_t *) &sbmMsg->msg;
		wreq->hdr.mid     = SCSI_MID_WRITE_REQ;
		wreq->hdr.cHandle = trec->chandle;
		wreq->hdr.path    = path;
		wreq->transSeqId  = 0;
		wreq->blkOffset   = lbStart;
		wreq->blkCnt      = SCSITEST_WRITE_BLKCNT;
		if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
			wreq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
		else
			wreq->attributes  = 0;
		wreq->checkSum    = 0;
	
		//-- Setup edesc and send pkt on down to SCSI.
		len = sizeof(scsi_writeReqMsg_t) + sizeof(sbm_sendControlAndSACData_t);
		edesc->bd[1].buf = buf;
		edesc->bd[1].len = len;
		edesc->hdr.src_port  = scsitest_addr;
		edesc->hdr.dest_port = scsitest_sbm_addr;
		edesc->hdr.control = ((len & EDESC_TOTAL_LEN) | 
						  ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));

		eee_forwardPacket(edesc);
	}

	eee_TimeGMT(&(trec->start_t));
	return NFX_OK;
} // scsitest_writeu_start

/*-----------------------------------------------------------------
 * Name: scsitest_sendUWriteReq
 *
 * Description: 
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: August 05, 2002
 *
 *-----------------------------------------------------------------
 */
int
scsitest_sendUWriteReq(scsi_test_t *trec)
{
	scsi_writeReqMsg_t *wreq;
	scsi_path_t path;
	uint32      scsiAppAddr;
	uint32      handle;
	uint32      lbStart;
	uint32      len;
	eee_desc_t *edesc;
	char       *buf;
	sbm_sendControlAndSACData_t *   sbmMsg=NULL;    // SBM WRITE msg  

	//-- Map handle to device path and scsi_port addr.
	if( sdma_handleToPath(trec->handle, &path, &scsiAppAddr) != NFX_OK ) {
		printf("SCSITEST: Invalid device handle: (It may not be open.) %x\n", handle); 
		return(NFX_ERR);
	}

	//-- Allocate Ultra Buffer
	buf =  eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA,0);
	if( !buf ) {
		return(NFX_ERR);  
	}

	//-- Allocate edesc
	edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if( !edesc ) {
		eee_deallocateBuffer(buf);
		return(NFX_ERR);
	}
	edesc->hdr.attr = 0;
	edesc->hdr.next = NULL;
	edesc->hdr.offset = 0;

	//-- Fill in the SBM message dest_port filled in below
	sbmMsg = (sbm_sendControlAndSACData_t *)buf;
	sbmMsg->cmd         = CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM;
	sbmMsg->source_port = scsitest_addr;
	sbmMsg->dest_port   = scsiAppAddr;
	sbmMsg->msg_len     = sizeof(scsi_writeReqMsg_t);
	sbmMsg->COD         = trec->cod;
	sbmMsg->length      = SCSITEST_WRITE_BLKCNT * 512;
	sbmMsg->offset      = 0; // TODO: need offset in cod
	
	//-- Create SCSI write request message 
	wreq = (scsi_writeReqMsg_t *) &sbmMsg->msg;
	wreq->hdr.mid     = SCSI_MID_WRITE_REQ;
	wreq->hdr.cHandle = trec->chandle;
	wreq->hdr.path    = path;
	wreq->transSeqId  = 0;
	wreq->blkOffset   = trec->lbCur;
	wreq->blkCnt      = SCSITEST_WRITE_BLKCNT;
	if( trec->testMode == SCSITEST_MODE_LOOP_SCSI )
		wreq->attributes  = SCSI_MSG_ATTR_LOOPBACK;
	else
		wreq->attributes  = 0;
	wreq->checkSum    = 0;
	
	//-- Setup edesc and send pkt on down to SCSI.
	len = sizeof(scsi_writeReqMsg_t) + sizeof(sbm_sendControlAndSACData_t);
	edesc->bd[1].buf = buf;
	edesc->bd[1].len = len;
	edesc->hdr.src_port  = scsitest_addr;
	edesc->hdr.dest_port = scsitest_sbm_addr;
	edesc->hdr.control = ((len & EDESC_TOTAL_LEN) | 
						  ((1<<EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK));

	eee_forwardPacket(edesc);

	return NFX_OK;
} // scsitest_sendUWriteReq
