/*  File Name  :    sdm-api.h
 *
 *  RCS        :    $Id: sdm-api.h,v 1.61 2002/11/19 20:38:56 nitinb Exp $
 *
 *  Description:    Contains message and api definitions for use of the storage
 *					device manager.
 *
 *  Created    :    05 July 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#ifndef _SDM_API_H_
#define _SDM_API_H_

#define SDM_DEV_NAME_LEN			128
#define SDM_MODEL_NAME_LEN			16
#define SDM_VENDOR_NAME_LEN	 		8

#define SDM_MAX_PATHS               16
#define SDM_MAX_HANDLES_PER_DEV     4

//------------------------------------
// SDM Result Codes 
//------------------------------------
#define SDM_OK		 		 0
#define SDM_FAIL			-1
#define SDM_TO_MANY_OPENS	-2
#define SDM_NOT_OPEN    	-3
#define SDM_INVALID_PARM	-4

//------------------------------------
// SDM Device States
//------------------------------------
#define SDM_DEV_STATE_CLOSED	0
#define SDM_DEV_STATE_OPENED	1
#define SDM_DEV_STATE_CLOSING	2
#define SDM_DEV_STATE_OPENING	3
#define SDM_DEV_STATE_INIT		4
#define SDM_DEV_STATE_INIT_OPENING	5
#define SDM_DEV_STATE_INIT_OPENED	6

//------------------------------------
// SDM Handle Defines
//------------------------------------
#define SDM_OPEN_HANDLE_MAX 4096        // Maximum open device handles
#define SDM_SLOT_MAX        4           // Maximum storage nim slots
#define SDM_PORT_MAX        4           // Maximum ports on a storage nim

#define SDM_HANDLE_SET_FP_SLOT(_slot_, _handle_) \
                   (_handle_) |= (((_slot_) << 24) & 0x0F000000)

#define SDM_HANDLE_SET_SLOT(_slot_, _handle_) \
                   (_handle_) |= (((_slot_) << 20) & 0x00F00000)

#define SDM_HANDLE_SET_PORT(_port_, _handle_)       \
                   (_handle_) |= (((_port_) << 16) & 0x000F0000)

#define SDM_HANDLE_SET_INDEX(_index_, _handle_)       \
                   (_handle_) |= (((_index_)) & 0x0000FFFF)

#define SDM_HANDLE_GET_FP_SLOT(_handle_) \
                   (((_handle_) & 0x0F000000) >> 24)

#define SDM_HANDLE_GET_SLOT(_handle_) \
                   (((_handle_) & 0x00F00000) >> 20)

#define SDM_HANDLE_GET_PORT(_handle_) \
                   (((_handle_) & 0x000F0000) >> 16)

#define SDM_HANDLE_GET_INDEX(_handle_) \
                   ((_handle_) & 0x0000FFFF)

//----------------------------------------------------------------------------
//-- SDM Path Type
//----------------------------------------------------------------------------
typedef struct _sdm_path_info
{
	uint32		state;
#define SDM_PATH_STATE_DOWN			0
#define SDM_PATH_STATE_ERRORED		1
#define SDM_PATH_STATE_AVAILABLE	2	
#define SDM_PATH_STATE_ACTIVE		3
	scsi_path_t	path;
	uint64      wwn;				//-- WWN and LUN associated with path.
	uint32      lun;
	uint32		llHandle;			//-- Transport layer device handle.
	uint32      handleSlot;		    //-- FP slot to open handle on. If this is zero
	                                //-- then the handle is created only on the SSC.
} sdm_path_t;

typedef struct _sdm_dev_strings
{
	char        name[SDM_DEV_NAME_LEN];
    char        vendor[SDM_VENDOR_NAME_LEN+1];
    char        model[SDM_MODEL_NAME_LEN+1];
    char        productId[SDM_MODEL_NAME_LEN+1];
	char        sn[32];
	char		revision[5];
} sdm_dev_strings_t;

/*
 *------------------------------------
 * SDM Message Types
 *------------------------------------
 */
typedef enum {
	SDM_MID_DEV_LIST_REQ = SDM_MSG_BASE,
	SDM_MID_DEV_LIST_RSP,	
	SDM_MID_DEV_INFO_REQ,
	SDM_MID_DEV_INFO_RSP,
	SDM_MID_DEV_OPEN_REQ,
	SDM_MID_DEV_OPEN_RSP,
	SDM_MID_DEV_CLOSE_REQ,
	SDM_MID_DEV_CLOSE_RSP,
	SDM_MID_DEV_ADD,
	SDM_MID_DEV_REMOVE,
	SDM_MID_PATH_ERR,
	SDM_MID_REBALANCE,
	SDM_MID_WRITE_REQ,
	SDM_MID_WRITE_RSP,
	SDM_MID_READ_REQ,
	SDM_MID_READ_RSP,
	SDM_MID_SDMA_SET_PATH_REQ,
	SDM_MID_SDMA_REM_PATH_REQ,
	SDM_MID_DEV_STATUS_REQ,
	SDM_MID_DEV_STATUS_RSP,
	SDM_MID_RELAY_CDB_REQ,
	SDM_MID_RELAY_CDB_RSP,
	SDM_MID_SAC_WRITE_REQ,
	SDM_MID_SAC_WRITE_RSP,
	SDM_MID_DEV_DEL_REQ,
	SDM_MID_DEL_ALL_REQ,
	SDM_MID_CLOSE_ID_REQ,
	SDM_MID_DEV_RESET_REQ,
	SDM_MID_DEV_RESET_RSP,
	SDM_MID_IOCTL_REQ,
	SDM_MID_IOCTL_RSP,
	SDM_MID_ERR_RSP,
	SDM_MID_GET_SDMHANDLE_REQ,
	SDM_MID_GET_SDMHANDLE_RSP,
	SDM_MID_GET_SCSIID_REQ,
	SDM_MID_GET_SCSIID_RSP,
	SDM_MID_GET_STATS_REQ,
	SDM_MID_GET_STATS_RSP,
	SDM_MID_CLOSE_ALL_DISK_REQ, 
	SDM_MID_NEW_DEV,
	SDM_MID_RAID_INIT_REQ,
	SDM_MID_RAID_INIT_RSP,
	SDM_MID_REDISCOVER_REQ,
	SDM_MID_DEV_RESERVE_REQ,
	SDM_MID_DEV_RESERVE_RSP,
	SDM_MID_SDMA_HANDLE_REQ,
	SDM_MID_SDMA_HANDLE_RSP,
	SDM_MID_DEV_CONTROL_REQ,
	SDM_MID_DEV_CONTROL_RSP,
	SDM_MID_DEV_MASKLIST_REQ,
	SDM_MID_DEV_MASKLIST_RSP,
	SDM_MID_CLR_STATS_REQ,
	SDM_MID_NULL_MSG
} sdm_msgTypes_t;

/*
 *SP------------------------------------
 * SDM Message Header
 *------------------------------------
 */
typedef struct _sdm_msgHdr
{
	uint32 mid;
	uint32 chandle;
} sdm_msgHdr_t;

/*
 *------------------------------------
 * Get Device List Messages
 *
 * APP ----> SDM
 *
 * Used by an application to get the 
 * current list of devices in the SDM
 * device database.
 *------------------------------------
 */
typedef struct _sdm_getDevListReq_msg 
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_LIST_REQ
	uint32       type;					// Device type
#define SDM_DEVLIST_ALL			0xFF			 
#define SDM_DEVLIST_DISK		0		// == SCSI_DEVTYPE_DISK, (exclude init state)
#define SDM_DEVLIST_TAPE		1		// == SCSI_DEVTYPE_TAPE
#define SDM_DEVLIST_STACKER		8		// == SCSI_DEVTYPE_MEDIA_CHANGER
#define SDM_DEVLIST_ALL_DISK	0xFE	// == All disk types no matter the state.
    uint32       port;                  // Port number.
#define SDM_DEVLIST_PORTALL     0xFF    // We return everything
    char         vendorName[SDM_VENDOR_NAME_LEN+1]; // Device vendor string.
                                                    // If empty, get all vendors
} sdm_devListReqMsg_t;

typedef struct _sdm_getDevListRsp_msg 
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_LIST_RSP
	uint32       numEntries;			// Number of entries in idList
	scsi_id_t	 idList[1];				// Variable len list == numEntries
} sdm_devListRspMsg_t;

/*
 *----------------------------------------
 * Get SCSI Device Info Messages
 *
 * APP ----> SDM
 *
 * Used by an application to get detailed
 * information about a particular device.
 *----------------------------------------
 */
typedef struct _sdm_getDevInfoReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_INFO_REQ
	scsi_id_t	 id;					// 64 bit device wwn/lun.
} sdm_devInfoReqMsg_t;

typedef struct _sdm_getDevInfoRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_INFO_RSP
	scsi_id_t	 id;					// 64 bit device wwn/lun.
	uint64       wwn;					// Device Raw FC WWN
	uint32       lun;					// Device LUN ID.
	uint32		 rc;					// Return Code: SDM_OK | SDM_FAIL
	uint32		 state;					// Device state
	uint32       openHandle;			// This is the handle if the device is in an open state.
	uint32		 devType;				// SCSI device type
	uint32		 raidLevel;			
	uint32		 blockCnt;				// Logical block count.
	uint32       blockSize;				// Logical block size.
	sdm_dev_strings_t st;					// Device strings
} sdm_devInfoRspMsg_t;

/*
 *------------------------------------------
 * Device Delete Request Message
 *
 * SDM ----> EVM_CFGD
 *
 * Sent by the SDM to request the EVM_CFGD
 * close and delete a device.
 *------------------------------------------
 */
typedef struct _sdm_devDelReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_DEL_REQ
	scsi_id_t	 id;					// 64 bit device wwn/lun.
} sdm_devDelReqMsg_t;

/*
 *----------------------------------------------
 * Delete All Request Message
 *
 * SCSI ----> SDM
 *
 * Sent to the SDM by the SCSI application to
 * delete all devices on a particular slot/port.
 *----------------------------------------------
 */
typedef struct _sdm_delAllReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEL_ALL_REQ
	uint32		 slot;
	uint32		 port;					// Same as CPU Id
} sdm_delAllReqMsg_t;

/*
 *------------------------------------
 * Open Device Messages
 *
 * APP ----> SDM
 * 
 * Sent to the SDM by an application to 
 * open a device for reading/writing or
 * some other opertion.
 *------------------------------------
 */

#define		SDM_OPEN_USE_PORT		0x0001
#define		SDM_OPEN_USE_PATH		0x0002

typedef struct _sdm_openDevReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_OPEN_REQ
	scsi_id_t	 id;					// 64 bit device wwn/lun.
	uint32       slot;					// Slot to generate handle on. 1:FP or 0:SSC.
	uint32		 port;					// FC slice to use for active path
	scsi_path_t	 path;					// scsi path to use as active path
	uint32		 flags;					// open flags, see above
} sdm_openReqMsg_t;

typedef struct _sdm_openDevRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_OPEN_RSP
	int32        rCode;					// SDM_OK | SDM_FAIL | SDM_TO_MANY_OPENS
	int32		 handle;				// Open device handle
} sdm_openRspMsg_t;

/*
 *------------------------------------
 * Close Device Messages
 *
 * APP ----> SDM
 *
 * Sent to the SDM by an application 
 * once the application no longer needs
 * to send SCSI operations to the  
 * device.
 *------------------------------------
 */
typedef struct _sdm_closeDevReq_msg		// APP ----> SDM
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_CLOSE_REQ
	int32        handle;				// Open device handle
} sdm_closeReqMsg_t;

typedef struct _sdm_closeDevRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_CLOSE_RSP
	int32		 rCode;					// SDM_OK | SDM_FAIL
} sdm_closeRspMsg_t;

typedef struct _sdm_closeIdReq_msg		// STORAGE CMD ----> SDM
{										// Special case msg.
	sdm_msgHdr_t hdr;					// SDM_MID_CLOSE_ID_REQ
	scsi_id_t	 id;
} sdm_closeIdReqMsg_t;

/*
 *------------------------------------
 * Device Control Messages
 *
 * APP ----> SDM
 * 
 * Sent to the SDM by an application to 
 * change operation parameters of an 
 * open device.
 *------------------------------------
 */
#define SDM_DEV_CONTROL_ENABLE_MASK_SORTING				0x00000001
#define SDM_DEV_CONTROL_ENABLE_MASK_COALLESCING			0x00000002
#define SDM_DEV_CONTROL_ENABLE_MASK_CS_VERIFY			0x00000004

#define SDM_DEV_CONTROL_ENABLE_SORTING					0x00000001
#define SDM_DEV_CONTROL_DISABLE_SORTING					0x00000000
#define SDM_DEV_CONTROL_ENABLE_COALLESCING				0x00000002
#define SDM_DEV_CONTROL_DISABLE_COALLESCING				0x00000000
#define SDM_DEV_CONTROL_ENABLE_CS_VERIFY				0x00000004
#define SDM_DEV_CONTROL_DISABLE_CS_VERIFY				0x00000000

typedef struct _sdm_controlDevReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_CONTROL_REQ
	scsi_id_t	 handle;
	uint32		 control_enable_mask;	// Gate
	uint32		 control_mask;			// Enable/Disable
} sdm_controlReqMsg_t;

typedef struct _sdm_controlDevRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_CONTROL_RSP
	int32        eCode;					// SDM_OK | SDM_FAIL
} sdm_controlRspMsg_t;

/*
 *------------------------------------------
 * Add Device Path Message
 *
 * SCSI ----> SDM
 *
 * Used by SCSI to inform the SDM that a
 * new device/path has been discovered.
 *------------------------------------------
 */
typedef struct _sdm_addDev_msg {
    sdm_msgHdr_t hdr;					// SDM_MID_DEV_ADD
    scsi_id_t   id;
	uint64      wwn;					// Device WWN
	uint64		chn_wwn;				// Egress switch port WWN
    scsi_path_t path;
    uint32      type;
    uint32      lbCnt;
    uint32      lbSize;
	uint32		state;					// Current Dev State
	uint32      llHandle;				// Transport layer device handle
	uint32      raidLevel;
	sdm_dev_strings_t	st;				// Device strings
} sdm_addDevMsg_t;

/*
 *---------------------------------------------
 * Remove Device Path Message
 *
 * SCSI ----> SDM 
 *
 * Used by SCSI to inform SDM that a device
 * path is no longer present. 
 *---------------------------------------------
 */
typedef struct _sdm_remDev_msg {
    sdm_msgHdr_t hdr;					// SDM_MID_DEV_REMOVE
    scsi_id_t   id;
    scsi_path_t path;
} sdm_remDevMsg_t;

/*
 *-----------------------------------
 * Device Path Error Message 
 *-----------------------------------
 */
typedef struct _sdm_pathErr_msg {
	sdm_msgHdr_t hdr;					// SDM_MID_PATH_ERR
	scsi_id_t 	id;
	scsi_path_t	path;
	uint32		eCode;
} sdm_pathErrMsg_t;

/*
 *---------------------------------
 * Rebalance Message
 *---------------------------------
 */
typedef struct _sdm_rebalance_msg {
	sdm_msgHdr_t hdr;					// SDM_MID_REBALANCE
	scsi_path_t	path;
} sdm_rebalanceMsg_t;

/*
 *---------------------------------
 * READ/WRITE request attributes
 *---------------------------------
 */
#define SDM_MSG_ATTR_PRI_MASK			0x0000000FL
#define SDM_MSG_ATTR_NO_REORDER			0x00000010L
#define SDM_MSG_ATTR_PAD_WRITE			0x00000020L
#define SDM_MSG_ATTR_PHY_OP				0x00000040L
#define SDM_MSG_ATTR_KICK_WRITE_GATE	0x00000100L
#define SDM_MSG_ATTR_IGNORE_WRITE_GATE	0x00000200L

/*
 *---------------------------------
 * Write Data Messages
 *---------------------------------
 */
typedef struct _sdm_writeDataReqMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_WRITE_REQ
	uint32	     handle;
	uint32		 blkOffset;
	uint32		 blkCnt;
	uint32		 blkSize;
	uint32		 attributes;			// See SDM_MSG_ATTR_* 
	char         data[1];			    // Variable == (blkSize * blkCnt)
} sdm_writeDataReqMsg_t;

typedef struct _sdm_writeDataRspMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_WRITE_RSP
	int32		eCode;					// SDM_FAIL | SDM_OK | SCSI_MSG_ECODE_*
} sdm_writeDataRspMsg_t;

/*
 *---------------------------------
 * Read Data Messages
 *---------------------------------
 */
typedef struct _sdm_readDataReqMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_READ_REQ
	uint32	     handle;
	uint32		 blkOffset;
	uint32		 blkCnt;
	uint32		 blkSize;  
	uint32		 attributes;			// See SDM_MSG_ATTR_* 
	uint32		 dataDest;				// Set this up if you want the data
										// to be sent to GMC.
} sdm_readDataReqMsg_t;

typedef struct _sdm_readDataRspMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_READ_RSP
	uint32		 eCode;					// SDM_FAIL | SDM_OK | SCSI_MSG_ECODE_*
	uint32		 dataLen;				// Number of data bytes read from dev
	uint64		 codHandle;				// Only Valid if data sent to SAC
	uint32		 data[0];
} sdm_readDataRspMsg_t;

/*
 *----------------------------------------------------------------
 * Set Path Message
 * 
 * SDM ---->SDM_AGENT
 *
 * Tells the SDM_AGENT to associate a handle with a physical
 * scsi path.
 *----------------------------------------------------------------
 */
typedef struct _sdm_setPathReqMsg {
	sdm_msgHdr_t hdr;
	scsi_id_t    id;
	uint32       handle;   				// SDM_MID_SDMA_SET_PATH_REQ
	scsi_path_t	 path;
} sdm_setPathReqMsg_t;

/*
 *-----------------------------------------------
 * Rem Path Message
 * 
 * SDM ---->SDM_AGENT
 *
 * Tells the SDM_AGENT to remove a handle/path.
 *-----------------------------------------------
 */
typedef struct _sdm_remPathReqMsg {
	sdm_msgHdr_t hdr;
	uint32       handle;   				// SDM_MID_SDMA_REM_PATH_REQ
} sdm_remPathReqMsg_t;

//-----------------------------------------------
// Handle Req Message
// 
// SDM ---->SDM_AGENT
//
// FOR INTERNAL SDM USE ONLY. Not for general use.
//
// Tells SDMA to generate a sdm handle for a 
// scsi_id/path.
//-----------------------------------------------
typedef struct _sdm_sdmaHandleReqMsg {
	sdm_msgHdr_t hdr;			// SDM_MID_SDMA_HANDLE_REQ
	scsi_id_t    id;   	
	scsi_path_t	 path;
} sdm_sdmaHandleReqMsg_t;

//-----------------------------------------------
// Handle Rsp Message
// 
// SDM_AGENT----->SDM
//
// FOR INTERNAL SDM USE ONLY. Not for general use.
//
// Returns current sdma handle/path for scsi_id
//-----------------------------------------------
typedef struct _sdm_sdmaHandleRspMsg {
	sdm_msgHdr_t hdr;			// SDM_MID_SDMA_HANDLE_RSP
	uint32       rCode;			// SDM_OK | SDM_FAIL
	uint32       handle;   	
	uint32       path;
} sdm_sdmaHandleRspMsg_t;

/*
 *---------------------------------
 * Device Status Messages
 *---------------------------------
 */
typedef struct _sdm_devStatusReqMsg {
    sdm_msgHdr_t hdr;                   			// SDM_MID_DEV_STATUS_REQ
	scsi_id_t	 id;
} sdm_devStatusReqMsg_t;

typedef struct _sdm_devStatusRspMsg {
    sdm_msgHdr_t hdr;                   			// SDM_MID_DEV_STATUS_RSP
	uint64		 wwn;								// Device FC WWN
	uint32       lun;								// Device LUN Id.
    uint32       eCode;                 			// SDM_OK | SDM_FAIL
	uint32		 state;								// Device State, OPEN | CLOSED
	uint32		 devType;							// SCSI device type
	uint32		 raidLevel;
	scsi_path_t  activePath	;            			// Path currently being used
	uint32       pathCnt;							// Number of paths to device
	sdm_path_t   paths[SDM_MAX_PATHS];				// Path list
	uint32       handleCnt;							// Number of open handles.
	uint32       handles[SDM_MAX_HANDLES_PER_DEV];	// Handle list
	uint32		 blockCnt;							// Logical block count -- DISK ONLY
	uint32       blockSize;							// Logical block size -- DISK ONLY
	uint32		 tape_position_eom;					// Tape position at EOM -- TAPE ONLY
	uint32		 tape_position_bom;					// Tape position at BOM -- TAPE ONLY
	int32		 tape_position_blockno;				// Tape position block number -- TAPE ONLY
	int32		 tape_position_fileno;				// Tape position file number -- TAPE ONLY
	sdm_dev_strings_t	st;							// Device strings
} sdm_devStatusRspMsg_t;

/*
 *---------------------------------
 * Relay CDB Messages
 *---------------------------------
 */
typedef struct _sdm_relayCdbReqMsg {
    sdm_msgHdr_t hdr;       	// SDM_MID_RELAY_CDB_REQ
	scsi_id_t	 handle;
	char         cdb[16];		// Data block containing 16 bytes SCSI CDB
	uint32      retryCnt;       // Number of times retry attempted  
	uint32      timeOutCnt;     // Number of seconds before timeout 
	uint32      maxTimeToLive;  // Num secs desc can live   120sec 
	int32       dataInLen;		// Specifies len of data in
	int			 dataLen;		// Length of data out
	char		 data[1];		// Variable length CDB data. 
} sdm_relayCdbReqMsg_t;

typedef struct _sdm_relayCdbRspMsg {
    sdm_msgHdr_t hdr;                   // SDM_MID_RELAY_CDB_RSP
    uint32       eCode;                 // SDM_OK | SDM_FAIL
	uint32 		 status;           		// SCSI status byte ret by device. 
    uchar8 		 senseData[32];
    int32  		 dataLen;
    uchar8 		 data[1];          		// Variable length data 
} sdm_relayCdbRspMsg_t;

/*
 *---------------------------------
 * SAC Data Write Messages
 *---------------------------------
 */
typedef struct _sdm_sacWriteReqMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_SAC_WRITE_REQ
	uint32	     handle;
	uint32		 blkOffset;
	uint32		 blkCnt;
	uint64		 cod;
} sdm_sacWriteReqMsg_t;

typedef struct _sdm_sacWriteRspMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_SAC_WRITE_RSP
	uint32		eCode;					// SDM_OK | SDM_FAIL
} sdm_sacWriteRspMsg_t;

/*
 *---------------------------------
 * Device Reset messages
 *---------------------------------
 */
typedef struct _sdm_devResetReqMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_RESET_REQ
	uint32		 handle;
} sdm_devResetReqMsg_t;

typedef struct _sdm_devResetRspMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_RESET_RSP
	uint32		 eCode;					// SDM_OK | SDM_FAIL
} sdm_devResetRspMsg_t;

/*
 *---------------------------------
 * IOCTL messages
 *---------------------------------
 */
#define SDM_IOCTL_MAX_ARGS		16
typedef struct _sdm_ioctlReqMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_IOCTL_REQ
	uint32		 code;
	uint32		 slot;
	uint32		 port;
	uint32		 arg_count;
	uint64		 args[SDM_IOCTL_MAX_ARGS];
} sdm_ioctlReqMsg_t;

typedef struct _sdm_ioctlRspMsg {
	sdm_msgHdr_t hdr;					// SDM_MID_IOCTL_RSP
	uint32		 eCode;					// SDM_OK | SDM_FAIL
    int32  		 dataLen;
    uint64 		 data[1];          		// Variable length data 
} sdm_ioctlRspMsg_t;

/*
 *------------------------
 * SDM Error Rsp Msg
 *------------------------
 */
typedef struct _sdm_errRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_ERR_RSP
	uint32		 eCode;
} sdm_errRspMsg_t;

//------------------------------------
// Get SDM_HANDLE Messages
//
// APP ----> SDM
//
// Sent to the SDM by an application 
// to get the open SDM_HANDLE for a 
// WWN/Lun.
//------------------------------------

typedef struct _sdm_getSdmHandleReq_msg		// APP ----> SDM
{
	sdm_msgHdr_t hdr;						// SDM_MID_GET_SDMHANDLE_REQ
	uint64		 wwn;
	uint32		 lun;
	char		 devname[SDM_DEV_NAME_LEN];	// device name
} sdm_getSdmHandleReqMsg_t;

typedef struct _sdm_getSdmHandleRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_GET_SDMHANDLE_RSP
	int32		 rCode;					// SDM_OK | SDM_FAIL
	uint32       sdmHandle;				// Open device handle
} sdm_getSdmHandleRspMsg_t;

//------------------------------------
// Get SCSI_ID Messages
//
// APP ----> SDM
//
// Sent to the SDM by an application 
// to translate a WWN/LUN pair into a
// SCSI Id. OR you can have the device
// name string translated into a SCSI Id.
//------------------------------------
typedef struct _sdm_getSCSIIdReq_msg		// APP ----> SDM
{
	sdm_msgHdr_t hdr;						// SDM_MID_GET_SCSIID_REQ
	uint64		 wwn;						// wwn/lun instead of name.	
	uint32		 lun;
	char         name[64];					// Device name. Instead of wwn/lun.
} sdm_getSCSIIdReqMsg_t;

typedef struct _sdm_getSCSIIdRsp_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_GET_SCSIID_RSP
	scsi_id_t	 id;					// 64 bit SCSI ID
	int32		 rCode;					// SDM_OK | SDM_FAIL
} sdm_getSCSIIdRspMsg_t;

//---------------------------------
//-- Device Stats Messages
//---------------------------------

#define SDM_STATS_TYPE_DEV		0x0001
#define SDM_STATS_TYPE_DEVALL	0x0002
#define SDM_STATS_TYPE_SCSI		0x0004

#define SDM_STATS_SUBTYPE_USE_WWN	0x0001
#define SDM_STATS_SUBTYPE_USE_NAME	0x0002
#define SDM_STATS_SUBTYPE_USE_ID	0x0004
#define SDM_STATS_SUBTYPE_USE_PORT	0x0008

typedef struct _sdm_getStatsReqMsg {
    sdm_msgHdr_t hdr;                   			// SDM_MID_GET_STATS_REQ
	ushort16	 type;								// see above
	ushort16	 subtype;							// valid only for SDM_STATS_TYPE_DEV
	uint64       wwn;								// valid only if SDM_STATS_SUBTYPE_USE_WWN is set
	uint32       lun;								// valid only if SDM_STATS_SUBTYPE_USE_WWN is set
	scsi_id_t	 id;								// valid only if SDM_STATS_SUBTYPE_USE_ID is set
	uint32		 port;								// valid only if SDM_STATS_SUBTYPE_USE_PORT is set
													//		   or if type == SDM_STATS_TYPE_SCSI
	char		 devname[SDM_DEV_NAME_LEN];			// valid only if SDM_STATS_SUBTYPE_USE_NAME is set
} sdm_getStatsReqMsg_t;


typedef struct _sdm_getStatsRspMsg {
    sdm_msgHdr_t hdr;                   			// SDM_MID_GET_STATS_RSP
    int32        eCode;                 			// SDM_OK | SDM_FAIL
	union  _scsi_dev_stats stats;
} sdm_getStatsRspMsg_t;


typedef struct _sdm_clearStatsReq_msg {

	sdm_msgHdr_t	hdr;
	uint32			type;

	struct _dev_info {
		uint64	wwn;
		uint32	lun;
		char	devname[128+1];
	};
	struct _scsi_info {
		uint32	slot;
		uint32	port;
	};
	union _info {
		struct _dev_info dev_info;
		struct _scsi_info scsi_info;
	} info;
} sdm_clearStatsReqMsg_t;


/*
 *----------------------------------------------
 * Close All Disks Request Message
 *
 * APP ----> SDM
 *
 * Sent to the SDM by an SSC application to
 * close all disk devices. Used by the EVM.
 *----------------------------------------------
 */
typedef struct _sdm_closeAllDiskReq_msg
{
	sdm_msgHdr_t hdr;					// SDM_MID_CLOSE_ALL_DISK_REQ
} sdm_closeAllDiskReqMsg_t;

//----------------------------------------------
// New Device Message
//
// SCSI ----> SDM
//
// Sent to the SDM by the SCSI application to
// when a new device has been detected.
//----------------------------------------------
typedef struct _sdm_newDevice_msg
{
    sdm_msgHdr_t hdr;                   // SDM_MID_NEW_DEV
    scsi_id_t    id;
    uint64       wwn;
    uint64       chn_wwn;
	uint32       llHandle;
    scsi_path_t  path;
    char         inqRsp[256];           // Data from a SCSI INQ
} sdm_newDevMsg_t;


//-------------------------------------------------
// Raid Init Messages
//
// SDM ----> RCM
//
// Sent to the raid config manager by the SDM
// when a raid device lun 0 is discovered.
//-------------------------------------------------
typedef struct _sdm_raidInitReqMsg {
    sdm_msgHdr_t hdr;       		// SDM_MID_RAID_INIT_REQ 
	uint64       wwn;
	uint32       path;
	char         inqRsp[256];
} sdm_raidInitReqMsg_t;

typedef struct _sdm_raidInitRspMsg {
    sdm_msgHdr_t hdr;            	// SDM_MID_RAID_INIT_RSP
    uint32       eCode;          	// SDM_OK | SDM_FAIL
} sdm_raidInitRspMsg_t;


//-------------------------------------------------
// Rediscover Message
//
// SPM ----> SDM
//
// Sent to the SDM by the SPM to initiate a 
// lun rediscovery operation on a particular WWN.
//-------------------------------------------------
typedef struct _sdm_rediscoverReqMsg {
    sdm_msgHdr_t hdr;       		// SDM_MID_REDISCOVER_REQ 
	uint64       wwn;
} sdm_rediscReqMsg_t;


//------------------------------------------
// Reserve Device Messages
//
// EVM/SPM ----> SDM
//
// Sent to the SDM by the evm or spm
// to issue persistent reserve out command to
// a scsi lun.
//------------------------------------------
typedef struct _sdm_reserveDevReq_msg	
{
	sdm_msgHdr_t hdr;					// SDM_MID_DEV_RESERVE_REQ
	int32        handle;				// Open device handle
	int32		 action;
#define SDM_DEV_RSV_ACTION_REGISTER	0
#define SDM_DEV_RSV_ACTION_RESERVE	1
#define SDM_DEV_RSV_ACTION_RELEASE	2
#define SDM_DEV_RSV_ACTION_CLEAR	3
#define SDM_DEV_RSV_ACTION_PREEMPT	4
	uint64		key;
#define SDM_DEV_RSV_KEY_DEFAULT		0x1122334455667788
} sdm_reserveReqMsg_t;

typedef struct _sdm_reserveDevRsp_msg
{
	sdm_msgHdr_t hdr;			// SDM_MID_DEV_RESERVE_RSP
	int32		 rCode;			// SDM_OK | SDM_FAIL
	int32		 cause;			// Reason for SDM_FAIL return code.
#define SDM_DEV_RSV_CONFLICT		1		// Reservation Conflict 
#define SDM_DEV_RSV_DEV_ERROR		2		// Device returned error
#define SDM_DEV_RSV_BAD_HANDLE		3		// Invalid SDM device handle
} sdm_reserveRspMsg_t;


//---------------------------------------------------
// Show LUN Mask message
// 
// Sent to SDM to list current lun masks
// --------------------------------------------------
typedef struct _sdm_conf_attr {
	uint32 action:4;
	uint32 slot  :12;
	uint32 port  :16;
} sdm_conf_attr_t;

typedef struct _sdm_conf
{
	uint64			wwn;
	uint32          lun;
	sdm_conf_attr_t attr;
	char			devname[128+1];
} sdm_conf_t;

typedef struct _sdm_getMaskListReq_msg
{
	sdm_msgHdr_t hdr;			// SDM_MID_DEV_MASKLIST_REQ
} sdm_getMaskListReqMsg_t;

typedef struct _sdm_getMaskListRsp_msg
{
	sdm_msgHdr_t hdr;			// SDM_MID_DEV_MASKLIST_RSP
	uint32		 count;			// number of entries
	sdm_conf_t	 list[1];		// first entry
} sdm_getMaskListRspMsg_t;


#endif // _SDM_API_H_
