//------------------------------------------------------------------------------
//  FileName   : sdm-init-changer.c
//
//  Description: Contains state tables and initialization functions for
//               changer devices. 
//
//	Created    : 28 March 2002
//
//  Author     : Frank Salzmann
//
//  RCS        : $Id: sdm-init-changer.c,v 1.3 2002/11/09 00:43:01 nitinb Exp $
//
//  Copyright (c)  Agile Storage, Inc. 2002
//------------------------------------------------------------------------------

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#undef NULL

#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/elog-api.h>

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"


//-----------------------------------------------------------------------
//-- CHANGERS
//-----------------------------------------------------------------------
int sdm_initChangerGeneric(sdm_dir_t *, char *);

sdm_dist_t sdm_icGenericST[] = {
	{SDM_DIS_INIT, SDM_DIE_INIT_DEV, SDM_DIE_RLUN_REQ, sdm_initChangerGeneric}, 
	{SDM_DIS_INIT, SDM_DIE_INIT_LUN, SDM_DIE_SN_REQ  , sdm_initChangerGeneric}, 
	{SDM_DIS_INIT, SDM_DIE_RLUN_REQ, SDM_DIE_RLUN_RSP, sdm_sendRLunCmd   },
	{SDM_DIS_INIT, SDM_DIE_RLUN_RSP, SDM_DIE_SN_REQ  , sdm_procRLun      }, 
	{SDM_DIS_INIT, SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq     },
	{SDM_DIS_INIT, SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procSnRsp     },
	{SDM_DIS_INIT, SDM_DIE_FIN     , NULL            , sdm_initFinish    },
	{SDM_DIS_INIT, SDM_DIE_TO      , NULL            , sdm_initTimeout   },
	{0,0,0}
};

//-- List of supported tape changer devices.
sdm_devSupport_t sdm_changerList[] = {
	{"STK" , "L40"            , "*", sdm_icGenericST },
	{"STK" , "*"              , "*", sdm_icGenericST },
	{"*"   , "*"              , "*", sdm_icGenericST },  // Changer Catch All
	{0,0,"",0}
};


//------------------------------------------------------------------------------
// Function Name:   sdm_initChangerGeneric
// 
// Description:     
//
// Data Created:    March 16, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initChangerGeneric(sdm_dir_t *dir, char *inqRsp)
{
	//printf("sdm_initChangerGeneric\n");

	//-- Create scsi id
	dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initChangerGeneric
