/*  FileName: sdm-main.c
 *
 *  RCS: $Id: sdm-main.c,v 1.50 2002/11/19 20:38:56 nitinb Exp $
 *
 *  Description: 
 *
 *  Created:    10 July 2001
 *
 *  Author:     Frank Salzmann
 *
 *  Copyright (c)  Agile Storage, Inc. 2001
 */




#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#undef NULL

#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/elog-api.h>
#include "../sm-scsi-test/scsi-test.h"

#define DEVICE_DB_FILE "sdm.db"
#define SDM_CONF_FILE  "/usr/local/agile/etc/sdm.conf"

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"
#include "sdm-conf.h"
#include "sdm-msg.h"

sdm_tmr_head_t	sdmTmrHd;
eDB_t         	*sdmDevDB; 
sdm_msgDesc_t 	sdmDispatchedMsgList[SDM_TRANSACTION_MAX];
sdm_handle_t  	devHandles[SDM_SLOT_MAX][SDM_SLOT_MAX][SDM_PORT_MAX][SDM_OPEN_HANDLE_MAX];
sdm_mp_desc_t	smps[SDM_MAX_MPS];

int				sdm_idCompare(scsi_id_t, scsi_id_t);
scsi_path_t		sdm_getActivePath( sdm_devDBRec_t * );

static void     sdm_sigTerm(int sig);
static void     sdm_sigHup(int sig);
static int		sdm_timerInit( void );
static int		sdm_allocDevHandle( sdm_devDBRec_t *, scsi_path_t );

int				sdm_sendSyncTimeout( int, int );

extern char *devType_str[];

#define SDM_DAEMON	1

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_main
 *
 * Description:     Initializes the SDM deamon.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *
 *------------------------------------------------------------------------------
 */
int
main()
{
	int port;
	sigset_t set;  


#if SDM_DAEMON
	//-- Setup the signal handling.
	sigfillset(&set);
    // sigdelset(&set, SIGALRM);
    sigdelset(&set, SIGTERM);
    sigdelset(&set, SIGHUP);
    sigdelset(&set, SIGCHLD);
    sigdelset(&set, SIGSEGV);
    sigdelset(&set, SIGBUS);
    sigprocmask(SIG_SETMASK, &set, NULL);

	//-- Set up our termination handler
    signal (SIGTERM, sdm_sigTerm);
    signal (SIGHUP, sdm_sigHup); 
#endif // SDM_DAEMON

	//-- Init some data structures
	// memset(sdmDispatchedMsgList, 0, sizeof(sdmDispatchedMsgList));
	memset(devHandles, 0, sizeof(devHandles));
	sdm_timerInit();

	sdm_msgInit();
	sdm_slotPortMsgInit(NFX_SLOTTYPE_SSC, NFX_SSC_UNIX); 
	for(port=0; port < NFX_FCNIM_NUM_CPUS; port++)
		sdm_slotPortMsgInit(NFX_SLOTTYPE_FC, port);

	sdm_confRead(SDM_CONF_FILE);
	sdmDevDB = (eDB_t *)sdm_dbInit(DEVICE_DB_FILE, "SDM DEVICE DATABASE");
	
	//-- Tell all SCSI application to send up their device lists
	for(port = 0; port < NFX_FCNIM_NUM_CPUS; port++)
		sdm_sendSyncReq(NFX_SLOTTYPE_FC, port);

#if SDM_DAEMON
	kill (getppid(), SIGUSR1);
	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Agile Storage Device Manager (c)2002: Started"));
	while(1)
		sdm_rcvMsg();
#else
	printf("\nAGILE STORAGE SCSI DEVICE MANAGER (c)2002\n");
	printf("==================================================\n\n");
	sdm_testMenu();
	sdm_dbClose(sdmDevDB);
#endif

} /* main */

//------------------------------------------------------------------------------
// Function Name:   sdm_allocDevHandle
//
// Description:     Allocates a device handle in response to an open request.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 23, 2001
//------------------------------------------------------------------------------
int
sdm_allocDevHandle( sdm_devDBRec_t *devRec, scsi_path_t path )
{
	uint32 i;
	uint32 slot, port, handle;
	sdm_handle_t *sdm_handle;
	
	slot = SCSI_PATH_GET_SLOT(path);
	port = SCSI_PATH_GET_PORT(path);
	
	handle = 0;
	for(i=0; i < SDM_OPEN_HANDLE_MAX; i++ ) { 
		// Look for empy space.
		sdm_handle = &devHandles[0][slot][port][i];
		if( sdm_handle->dev == 0 ) {
			handle = i;
			SDM_HANDLE_SET_SLOT(slot,handle);
			SDM_HANDLE_SET_PORT(port,handle);
			sdm_handle->dev   = devRec;
			sdm_handle->path = path;
			devRec->handleCnt++;
			devRec->openHandles[0] = handle;
			break;
		}
	}
	return(handle);
} /* sdm_allocDevHandle */

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_getHandleEntry
 *
 * Description:     
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 23, 2001
 *
 *------------------------------------------------------------------------------
 */
sdm_handle_t *
sdm_getHandleEntry( uint32 handle )
{
	uint32 fp_slot, sp_slot, port, index;
	
	fp_slot = SDM_HANDLE_GET_FP_SLOT(handle);
	sp_slot = SDM_HANDLE_GET_SLOT(handle);
	port = SDM_HANDLE_GET_PORT(handle);
	index = handle & 0x0000FFFF;

	//-- I removed the check for a valid "dev" entry. The check needs
	//-- to be done by the function calling sdm_getHandleEntry. The
	//-- dev field in the sdm_handle structure can become null if the
	//-- device is removed.
	if( (fp_slot > NFX_SLOT_MAX) || 
		(sp_slot > NFX_SLOT_MAX) || 
		(port >= NFX_FCNIM_NUM_CPUS))
		return 0;
	else	
		return &devHandles[fp_slot][sp_slot][port][index];

} /* sdm_getHandleEntry */

//------------------------------------------------------------------------------
// Function Name:   sdm_saveHandleEntry
//
// Description:     
//
// Created by:      Frank Salzmann
//
// Data Created:    May 10, 2000
//------------------------------------------------------------------------------
int
sdm_saveHandleEntry( sdm_devDBRec_t *devRec, scsi_path_t path, uint32 handle )
{
	uint32 fp_slot, sp_slot, port, i;
	sdm_handle_t *h_entry;

	i    = handle & 0x0000FFFF;
	port = SDM_HANDLE_GET_PORT(handle);
	sp_slot = SDM_HANDLE_GET_SLOT(handle);
	fp_slot = SDM_HANDLE_GET_FP_SLOT(handle);
	h_entry = &devHandles[fp_slot][sp_slot][port][i];

	if((fp_slot > NFX_SLOT_MAX)||(sp_slot > NFX_SLOT_MAX) || 
	   (port >= NFX_FCNIM_NUM_CPUS)) 
		return SDM_FAIL;
	else {
		h_entry->dev  = devRec;
		h_entry->path = path;
		devRec->handleCnt++;
		devRec->openHandles[0] = handle;
	}
	return SDM_OK;
} // sdm_saveHandleEntry 

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_devHandle2Path
 *
 * Description:     Takes a device handle and returns the scsi_path_t associated
 *					with it.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 23, 2001
 *
 *------------------------------------------------------------------------------
 */
scsi_path_t
sdm_devHandle2Path( uint32 handle )
{
	uint32 fp_slot, sp_slot, port, index;
	
	fp_slot = SDM_HANDLE_GET_FP_SLOT(handle);
	sp_slot = SDM_HANDLE_GET_SLOT(handle);
	port    = SDM_HANDLE_GET_PORT(handle);
	index   = handle & 0x0000FFFF;
	
	return( devHandles[fp_slot][sp_slot][port][index].path );
} /* sdm_devHandle2Path */

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_releaseDevHandle
 *
 * Description:     
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 23, 2001
 *
 *------------------------------------------------------------------------------
 */
int
sdm_releaseDevHandle( uint32 handle )
{
	uint32 fp_slot, sp_slot, port, index;
	
	//-- Tell the SDM_AGENT on the FP to release
	//-- the handle.
	if( SDM_HANDLE_GET_FP_SLOT(handle) == 1 )
		sdm_sendRemPathMsg(handle);

	fp_slot = SDM_HANDLE_GET_FP_SLOT(handle);
	sp_slot = SDM_HANDLE_GET_SLOT(handle);
	port = SDM_HANDLE_GET_PORT(handle);
	index = handle & 0x0000FFFF;
	devHandles[fp_slot][sp_slot][port][index].path = 0;

	if( devHandles[fp_slot][sp_slot][port][index].dev )
		devHandles[fp_slot][sp_slot][port][index].dev->handleCnt--;
	devHandles[fp_slot][sp_slot][port][index].dev = 0;

	return 0; 
} /* sdm_releaseDevHandle */

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_wwnToHandle
 *
 * Description:     Takes a WWN/LUN and if the device is open will return
 *                  the sdm_handle.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    February 1, 2002
 *
 *------------------------------------------------------------------------------
 */
uint32
sdm_wwnToHandle( uint64 wwn, uint32 lun )
{
	uint32 handle = 0;
	sdm_devDBRec_t *devRec;
	//-- Step one is to find the sdm device record for this 
	//-- wwn/lun.
	devRec = sdm_dbFindWwnLun( sdmDevDB, wwn, lun );
	if( devRec ) {
		if( devRec->handleCnt > 0 )
			handle = devRec->openHandles[0];
	}
	
	return handle;
} // sdm_wwnToHandle

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_idToHandle
 *
 * Description:     Takes a scsi_id_t and if the device is open will return
 *                  the sdm_handle.
 *
 * Created by:      Nitin Bahadur
 *
 * Data Created:    Sept. 19, 2002
 *
 *------------------------------------------------------------------------------
 */
uint32
sdm_idToHandle( scsi_id_t id )
{
	uint32 handle = 0;
	sdm_devDBRec_t *devRec;

	devRec = sdm_dbFindRec(sdmDevDB, id);
	if( devRec && devRec->id == id && devRec->handleCnt > 0) 
		handle = devRec->openHandles[0];

	return handle;
} // sdm_idToHandle

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_wwnToID
 *
 * Description:     Takes a WWN/LUN and returns the valid SCSI ID.
 *
 * Created by:      Radek Aster
 *
 * Data Created:    March 19, 2002
 *
 *------------------------------------------------------------------------------
 */
scsi_id_t
sdm_wwnToID( uint64 wwn, uint32 lun )
{
	sdm_devDBRec_t *devRec;

	devRec = sdm_dbFindWwnLun( sdmDevDB, wwn, lun );
	if (devRec)
		return(devRec->id);
	else
		return(0);
} // sdm_wwnToID

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_nameToID
 *
 * Description:     Takes a device name string and returns the valid SCSI ID.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    April 05, 2002
 *
 *------------------------------------------------------------------------------
 */
scsi_id_t
sdm_nameToID(char *name)
{
	uint32 i, len, devCnt;
	scsi_id_t       id;
	sdm_devDBRec_t *devRec;

	len = SDM_DEV_NAME_LEN;
	devCnt = sdm_dbGetNumEntries( sdmDevDB, SDM_DEVLIST_ALL, SDM_DEVLIST_PORTALL, NULL );
	id = 0;
	for( i=0; i < devCnt; i++ ) {
		devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
		if( devRec ) {
			id = devRec->id;
			if( !strncmp(devRec->st.name, name, len) )
				return(id);
		}
	}
	return((scsi_id_t)NULL);
} // sdm_nameToID

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_nameToHandle
 *
 * Description:     Takes a device name string and returns an open handle.
 *
 * Created by:      Nitin Bahadur
 *
 * Data Created:    Sept 23, 2002
 *
 *------------------------------------------------------------------------------
 */
uint32
sdm_nameToHandle(char *name)
{
	uint32 i, len, devCnt;
	scsi_id_t       id;
	sdm_devDBRec_t *devRec;

	len = SDM_DEV_NAME_LEN;
	devCnt = sdm_dbGetNumEntries( sdmDevDB, SDM_DEVLIST_ALL, SDM_DEVLIST_PORTALL, NULL );
	id = 0;
	for( i=0; i < devCnt; i++ ) {
		devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
		if( devRec ) {
			id = devRec->id;
			if( !strncmp(devRec->st.name, name, len) ) {
				if(devRec->handleCnt > 0)
					return (devRec->openHandles[0]);
				return(0);
			}
		}
	}
	return(0);
}


/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_openPath
 *
 * Description:     Selects an active path from the device database record and
 *					opens it if necessary.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 23, 2001
 *
 *------------------------------------------------------------------------------
 */
scsi_path_t
sdm_openPath(sdm_devDBRec_t *devRec, uint32 handleSlot, uint32 port, uint32 path, uint32 flags)
{
	int        i;

	// Look for a path to the device that is currently active.
	for( i=0; i < devRec->pathCnt; i++ ) {
		if( devRec->pathList[i].state == SDM_PATH_STATE_ACTIVE )
			return( devRec->pathList[i].path );
	}

	// No path is active so select the first available one 
	// and return it.  NOTE: Eventually this algorithim will evolve to
	// consider path loading and costing.
	for( i=0; i < devRec->pathCnt; i++ ) {
		if( devRec->pathList[i].state == SDM_PATH_STATE_AVAILABLE ) {
			if( !flags || ((flags & SDM_OPEN_USE_PATH) && devRec->pathList[i].path == path) ||
			   ((flags & SDM_OPEN_USE_PORT) && (SCSI_PATH_GET_PORT(devRec->pathList[i].path)) == port)) {
					sdm_sendScsiOpenReq(devRec->pathList[i].path, devRec);
					devRec->pathList[i].state      = SDM_PATH_STATE_ACTIVE;
					devRec->pathList[i].handleSlot = handleSlot;
					return( devRec->pathList[i].path );
			} // end of inner if
		} // end of outer if
	}
	
	// Nothing is available.
	return(0);
} /* sdm_openPath */

//------------------------------------------------------------------------------
// Function Name:   sdm_removePath
//
// Description:     Removes device path information and from the device 
//					database. If the last path to a device is removed then
//					the device entry itself will get removed.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 26, 2001
//
//------------------------------------------------------------------------------
int
sdm_removePath( sdm_devDBRec_t *devRec, scsi_path_t path )
{
	uint32 i, j;

	//-- Remove path from path list.
	for(i=0; i < devRec->pathCnt; i++) {
		if( devRec->pathList[i].path == path ) {
			//-- Wipe out path entry information.
			memset(&devRec->pathList[i], 0, sizeof(sdm_path_t));
			
			//-- Consolidate path list.
			for(j=i+1; j < devRec->pathCnt; i++, j++) {
				memcpy(&devRec->pathList[i], &devRec->pathList[j], sizeof(sdm_path_t));
				memset(&devRec->pathList[j], 0, sizeof(sdm_path_t));
			}
			devRec->pathCnt--;
			break;
		}
	}
	return SDM_OK;
} // sdm_removePath 

//------------------------------------------------------------------------------
// Function Name:   sdm_getActivePath
//
// Description:     Returns the path currently being used to send request to
// 					a device.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 25, 2001
//------------------------------------------------------------------------------
scsi_path_t
sdm_getActivePath( sdm_devDBRec_t *devRec )
{
	int        i;

	//-- Look for a path to the device that is currently active.
	for( i=0; i < devRec->pathCnt; i++ ) {
		if( devRec->pathList[i].state == SDM_PATH_STATE_ACTIVE )
			return( devRec->pathList[i].path );
	}

	//-- Active path no found
	return(0);
} // sdm_getActivePath

//------------------------------------------------------------------------------
// Function Name:   sdm_getActivePathEntry
//
// Description:     Returns the sdm_path entry for the path currently being 
//                  used to send request to	a device.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 25, 2001
//------------------------------------------------------------------------------
sdm_path_t *
sdm_getActivePathEntry( sdm_devDBRec_t *devRec )
{
	int i;

	if( devRec == NULL )
		return 0;

	//-- Look for a path to the device that is currently active.
	for( i=0; i < devRec->pathCnt; i++ ) 
		if( devRec->pathList[i].state == SDM_PATH_STATE_ACTIVE )
			return( &devRec->pathList[i] );

	//-- Active path not found
	return(0);
} // sdm_getActivePathEntry

//------------------------------------------------------------------------------
// Function Name:   sdm_setPathState
//
// Description:     
//
// Created by:      Frank Salzmann
//
// Data Created:    July 26, 2001
//------------------------------------------------------------------------------
int
sdm_setPathState( sdm_devDBRec_t *devRec, scsi_path_t path, uint32 state )
{
	int        i;

	for( i=0; i < devRec->pathCnt; i++ ) 
		if( devRec->pathList[i].path == path )
			devRec->pathList[i].state = state;

	return(0);
} // sdm_setPathState

int
sdm_devOpenSM( sdm_devDBRec_t *devRec, uint32 event, void *msg )
{
	uint32			  f_sendRsp = 0;
	uint32            handle;
	sdm_path_t        *p_entry;
	scsi_path_t       path;
	sdm_openRspMsg_t  sdm_o_msg;
	scsi_openRspMsg_t *scsi_i_msg;
	

	switch(event)
	{
		case SDM_MID_DEV_OPEN_REQ:
		{
			sdm_openReqMsg_t  *sdm_i_msg;

			sdm_i_msg = (sdm_openReqMsg_t *)msg; 
			switch( devRec->state ) {

				case SDM_DEV_STATE_OPENED:
				case SDM_DEV_STATE_INIT_OPENED:
					path = sdm_getActivePath( devRec );
					handle = sdm_allocDevHandle( devRec, path );
					sdm_o_msg.rCode  = SDM_OK;
					sdm_o_msg.handle = handle;
					f_sendRsp        = 1;
					sdm_sendSetPathMsg(devRec->id, handle, path);

					E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
						   ("SDM: Open:  0x%016llx/%d", 
							devRec->wwn,devRec->lun));
					break;

				case SDM_DEV_STATE_CLOSED:
					path = sdm_openPath(devRec, sdm_i_msg->slot, sdm_i_msg->port, sdm_i_msg->path, sdm_i_msg->flags);
					if( path == 0 ) 
						return(SDM_FAIL);
					devRec->state = SDM_DEV_STATE_OPENING;
					break;

				case SDM_DEV_STATE_INIT:
					path = sdm_openPath(devRec, sdm_i_msg->slot, sdm_i_msg->port, sdm_i_msg->path, sdm_i_msg->flags);
					if( path == 0 )
						return(SDM_FAIL);
					devRec->state = SDM_DEV_STATE_INIT_OPENING;
					break;

				case SDM_DEV_STATE_OPENING:
				case SDM_DEV_STATE_INIT_OPENING:
				case SDM_DEV_STATE_CLOSING:
					// Device is busy
					E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
						   ("SDM: Open: Device Busy: 0x%016llx:%d", 
							devRec->wwn, devRec->lun));
					sdm_o_msg.rCode  = SDM_FAIL;
					f_sendRsp        = 1;

					break;
			}
			break;
		}
		case SCSI_MID_OPEN_RSP:	

			scsi_i_msg = (scsi_openRspMsg_t *)msg;
			
			switch( scsi_i_msg->eCode ) {

				case SCSI_MSG_ECODE_OK:

					p_entry = sdm_getActivePathEntry(devRec);
					if( p_entry == NULL )
						return SDM_OK;

					path = p_entry->path;

					switch( devRec->state ) {
						case SDM_DEV_STATE_OPENED:
						case SDM_DEV_STATE_INIT_OPENED:
							handle = devRec->openHandles[0];
							sdm_sendSetPathMsg(devRec->id, handle, path);
							E_LOG (class_1, notice_s, 
								   eee_getApplicationId("sdm"), 0, 0, 0,
								   ("SDM: New Path: 0x%016llx:%d", 
									devRec->wwn, devRec->lun));
							break;

						case SDM_DEV_STATE_OPENING:
						case SDM_DEV_STATE_INIT_OPENING:
						default:
							//-- See if this handle only exists on the SSC
							if( p_entry->handleSlot == 0 ) {
								handle = sdm_allocDevHandle( devRec, path );
								sdm_o_msg.rCode  = SDM_OK;
								sdm_o_msg.handle = handle;
								f_sendRsp        = 1;

								if(devRec->state == SDM_DEV_STATE_INIT_OPENING)
									devRec->state = SDM_DEV_STATE_INIT_OPENED;
								else
									devRec->state    = SDM_DEV_STATE_OPENED;
								
								//sdm_sendSetPathMsg(devRec->id, handle, path);
								E_LOG (class_1, notice_s, 
									   eee_getApplicationId("sdm"), 0, 0, 0,
									   ("SDM: Open:  0x%016llx:%d", 
										devRec->wwn, devRec->lun));
							}
							else { //-- Open handle on the FP_NIM
								sdm_sendSdmaHandleReq(devRec,
													  devRec->id,
													  path,
													  p_entry->handleSlot);
							}
							break;
					}
					break;

				default:
					sdm_o_msg.rCode  = SDM_FAIL;
					f_sendRsp        = 1;
					switch(devRec->state) {
						case SDM_DEV_STATE_INIT_OPENING:
								devRec->state = SDM_DEV_STATE_INIT;
								break;
						default:
								devRec->state    = SDM_DEV_STATE_CLOSED;
								break;
					}
					path = sdm_getActivePath( devRec );
					sdm_setPathState(devRec, path, SDM_PATH_STATE_AVAILABLE);
					E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
						("SDM: Open: Failed: 0x%016llx:%d:", 
						 devRec->wwn, devRec->lun));
			}
			break;

		case SDM_MID_SDMA_HANDLE_RSP:
		{
			sdm_sdmaHandleRspMsg_t  *sdm_i_msg;

			sdm_i_msg = (sdm_sdmaHandleRspMsg_t *)msg;

			//printf("sdma handle req event: handle: %x, path: %x\n", 
			  // sdm_i_msg->handle, sdm_i_msg->path);

			sdm_saveHandleEntry(devRec, sdm_i_msg->path, sdm_i_msg->handle);
			switch(devRec->state) {
				case SDM_DEV_STATE_INIT_OPENING:
						devRec->state = SDM_DEV_STATE_INIT_OPENED;
						break;
				default:
						devRec->state = SDM_DEV_STATE_OPENED;
						break;
			}

			//-- Setup Open Response Message
			sdm_o_msg.rCode  = SDM_OK;
			sdm_o_msg.handle = sdm_i_msg->handle;
			f_sendRsp        = 1;

			E_LOG (class_1, notice_s, 
				   eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM: Open:  0x%016llx:%d", 
					devRec->wwn, devRec->lun));
			break;
		}

		case SDM_SDMA_HANDLE_TIMEOUT_EVENT:
		case SDM_TIMEOUT_EVENT:	

			sdm_o_msg.rCode  = SDM_FAIL;
			f_sendRsp        = 1;
			switch(devRec->state) {
				case SDM_DEV_STATE_INIT_OPENING:
					devRec->state = SDM_DEV_STATE_INIT;
					break;
				default:
					devRec->state = SDM_DEV_STATE_CLOSED;
					break;
			}
			path = sdm_getActivePath( devRec );
			sdm_setPathState(devRec, path, SDM_PATH_STATE_AVAILABLE);
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Open: Timed Out: 0x%016llx:%d", 
				 devRec->wwn, devRec->lun));
			break;
	} /* event */

	if( f_sendRsp ) {
		sdm_o_msg.hdr.mid     = SDM_MID_DEV_OPEN_RSP;
		sdm_o_msg.hdr.chandle = devRec->rspInfo.chandle;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_openRspMsg_t), 
					 devRec->rspInfo.srcApp, 
					 devRec->rspInfo.srcCpu, 
					 devRec->rspInfo.srcSlot, 0,0,0);
	}

	return SDM_OK;
} /* sdm_devOpenSM */

int
sdm_timerInit(void)
{
	memset(&sdmTmrHd, 0, sizeof(sdm_tmr_head_t));
	return SDM_OK;
} // sdm_timerInit

void
sdm_timerProc(void)
{
	sdm_tmr_t *tmr;
	sdm_tmr_t *expired_tmr;
    struct timeval tv;

	// Look at the timer list for things to do.
	if( sdmTmrHd.active_timers ) {
        bzero(&tv, sizeof(tv));
        if (gettimeofday(&tv, NULL))
        {
            // printf("SDM: Err: Timer tick failed to get time of day.\n");
            return;
        }
		tmr = sdmTmrHd.list_start;
		while(tmr) {
			// If timer has expired then execute the callback function and
			// unlink the timer.
			// if( --(tmr->time_remain) == 0 ) {
			if( tmr->time_expire <= tv.tv_sec ) {
				tmr->callback(tmr->param1, tmr->param2);
				expired_tmr = tmr;
				tmr = tmr->next;
				sdm_timerStop( expired_tmr );
			}
			else {
				tmr = tmr->next;
			}
		}
	}

	// Re-start 1 sec system timer if there are any SDM timers active.
	// if( sdmTmrHd.active_timers ) {
		// if( signal(SIGALRM, sdm_timerProc) == SIG_ERR ) 
			//printf("SDM Timer Restart Failed.\n");
			// ;
		// else
			// alarm(1);		// Re-Start 1 second timer.
	// }
	return;
} // sdm_timer

sdm_tmr_t *
sdm_timerStart( int secs, int (*callBack)(), int param1, int param2)
{
	sdm_tmr_t *tmr;
    struct timeval tv;

	tmr = (sdm_tmr_t *)malloc(sizeof(sdm_tmr_t));
	if( !tmr ) {
		//printf("SDM: Err: Could not allocate timer block.\n");
		return(0);
	}

    bzero(&tv, sizeof(tv));
    if (gettimeofday(&tv, NULL))
    {
        //printf("SDM: Err: Timer start failed to get time of day.\n");
        return NULL;
    }

	// Setup Timer block.
	// tmr->time_len = tmr->time_remain = secs;
    tmr->time_expire = tv.tv_sec + secs;
	tmr->callback = callBack;
	tmr->param1   = param1;
	tmr->param2   = param2;

	// Link to timer chain.
	if( sdmTmrHd.active_timers == 0 ) {
        sdmTmrHd.list_start = tmr;
    }
    else  {
        sdmTmrHd.list_end->next = tmr;
    }

	sdmTmrHd.list_end       = tmr;
	tmr->next                = 0;

	++(sdmTmrHd.active_timers);
	// if( ++(sdmTmrHd.active_timers) == 1 ) {
		// if( signal(SIGALRM, sdm_timerProc) == SIG_ERR ) 
			//printf("SDM Timer Init Failed.\n");
			// ;
		// else
			// alarm(1);		// Start a 1 second timer.
	// }

	return(tmr);
} // sdm_timerStart

int 
sdm_timerStop( sdm_tmr_t *stopped_tmr )
{
	sdm_tmr_t *tmr;
	sdm_tmr_t *prev;

	if( !stopped_tmr )
		return(SDM_OK);

	// Is this the first timer on the list?
	if( stopped_tmr == sdmTmrHd.list_start ) {
		sdmTmrHd.list_start = stopped_tmr->next;
		sdmTmrHd.active_timers--;
		free(stopped_tmr);
		return( SDM_OK );
	}
	
	prev = sdmTmrHd.list_start;
	tmr  = prev->next;
	while( tmr ) {
		if( tmr == stopped_tmr ) {
			prev->next = tmr->next;
			
			if( tmr == sdmTmrHd.list_end )
				sdmTmrHd.list_end = prev;

			sdmTmrHd.active_timers--;
			free(stopped_tmr);
			return(SDM_OK);
		}
		prev = tmr; 
		tmr  = tmr->next;
	}

	return(SDM_OK);
} // sdm_timerStop

int
sdm_openTimeout(int param1, int param2)
{
	uint32          tid = (uint32) param2;
	sdm_devDBRec_t *devRec  = (sdm_devDBRec_t *)param1;
	
	sdm_deallocTransaction(tid);
	sdm_devOpenSM( devRec, SDM_TIMEOUT_EVENT, 0 );

	return( SDM_OK );
} // sdm_openTimeout

int
sdm_sdmaHandleTimeout(int param1, int param2)
{
	uint32          tid = (uint32) param2;
	sdm_devDBRec_t *devRec  = (sdm_devDBRec_t *)param1;
	
	sdm_deallocTransaction(tid);
	sdm_devOpenSM( devRec, SDM_SDMA_HANDLE_TIMEOUT_EVENT, 0 );

	return( SDM_OK );
} // sdm_sdmaHandleTimeout

static void     
sdm_sigTerm(int sig)
{
	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Agile Storage Device Manager (c)2002: Terminated"));
	exit(0);
} // sdm_sigTerm

static void     
sdm_sigHup(int sig)
{
	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Agile Storage Device Manager (c)2002: Re-Started"));
	return;
} // sdm_sigHup


int
sdm_sendSyncTimeout(int slot, int port)
{

	sdm_sendSyncReq(slot, port);

	port++;
	if(port < 4)
		sdm_timerStart(3, sdm_sendSyncTimeout, slot, port);

	return 0;
} // sdm_sendSyncTimeout
