#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <unistd.h>

#undef NULL

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/scsi-api.h>
#include <exports/sbm-api.h>
#include <exports/elog-api.h>
#include <exports/event-api.h>
#include "../sm-scsi-test/scsi-test.h"
#include "../sm-scsi/scsi-dev-cmds.h"
#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"
#include "sdm-conf.h"
#include "sdm-msg.h"

#include <exports/anpssc-api.h>
#include <exports/libc.h>
#include <exports/spm-api.h>
#include <exports/ncm-api.h>

int sdm_rxif[NFX_FCNIM_NUM_CPUS+1];
#define SDM_GP_SOCK		NFX_FCNIM_NUM_CPUS	// sdm general purpose socket index

extern eDB_t        *sdmDevDB;

#define sdm_htonl(_x_) (_x_)
#define sdm_htons(_x_) (_x_)
#define sdm_ntohl(_x_) (_x_)
#define sdm_ntohs(_x_) (_x_)

#define SDM_DEBUG_RX_MSG 0
#define SDM_DEBUG_TX_MSG 0

char *devType_str[] = {"disk",		"tape",
					   "printer",	"processor",
					   "worm",		"cdrom",
					   "scanner",	"optical_disk",
					   "changer",	"comm_dev",
					   "printer1",	"printer2",
					   "raid",		"ses",
					   "unknown",	"unknown1"
};

/*
 * Don't change value of MAX_PACKET_DATA unless you know exactly what you're doing.
 * (1) It must be a integral multiple of 1024 = (2 * standard
 *     block size). This is important in that for 520 BPS
 *     disks, checksums are added to this data chunk before
 *     being passed to the HAL layer.
 * (2) Must be less that 10/100BaseT MTU, i.e. < 1500.  
 * (3) For related discussion, see ispfc_edesc_allocate() in sm-ispfc/ispfc_misc.c
 */
#define MAX_PACKET_DATA 1024

//
//-- Function Prototypes
//
int sdm_procRcvMsg( char *, int, uchar8 *, uint32, uint32);
int sdm_procDevListReq( sdm_msgDesc_t * );
int sdm_procDevInfoReq( sdm_msgDesc_t * );
int sdm_procOpenReq( sdm_msgDesc_t * );
int sdm_procCloseReq( sdm_msgDesc_t * );
int sdm_procWriteReq( sdm_msgDesc_t * );
int sdm_procReadReq( sdm_msgDesc_t * );
int sdm_procDevStatusReq( sdm_msgDesc_t * );
int sdm_procRelayReq( sdm_msgDesc_t * );
int sdm_procScsiReadRsp( sdm_msgDesc_t * );
int sdm_procScsiWriteRsp( sdm_msgDesc_t * );
int sdm_procSacWriteReq( sdm_msgDesc_t * );
int sdm_procAddDevice( sdm_msgDesc_t * );
int sdm_procRemoveDevice( sdm_msgDesc_t * );
int sdm_procDevReset( sdm_msgDesc_t * );
int sdm_procIoctl( sdm_msgDesc_t * );
int sdm_procScsiOpenRsp( sdm_msgDesc_t * );
int sdm_procScsiErrRsp( sdm_msgDesc_t * );
int sdm_procScsiRelayRsp( sdm_msgDesc_t * );
int sdm_procSCSIDevResetRsp( sdm_msgDesc_t * );
int sdm_procSCSIDevStatusRsp( sdm_msgDesc_t * );
int sdm_procSCSIIoctlRsp( sdm_msgDesc_t * );
int sdm_sendScsiCloseReq(scsi_path_t, uint32);
int sdm_procCloseAllDiskReq( sdm_msgDesc_t * );
int sdm_delAll(uint32, uint32);
int sdm_procStatsReq( sdm_msgDesc_t *mdesc );
int sdm_procDelAllReq( sdm_msgDesc_t *mdesc );
int sdm_procScsiCloseRsp( sdm_msgDesc_t *mdesc );
int sdm_procScsiStatsRsp( sdm_msgDesc_t *mdesc );
int sdm_procGetSdmHandleReq(sdm_msgDesc_t *mdesc);
int sdm_procGetSCSIIDReq(sdm_msgDesc_t *mdesc);
int sdm_sendScsiStatsReq( scsi_id_t id, uint32 destSlot, uint32 destCpu, int type, uint32 tid );
int sdm_sendScsiDelMsg( scsi_path_t path );
int sdm_procReserveRsp( uint32, scsi_relayRspMsg_t * ); 
int sdm_procDevReserveReq( sdm_msgDesc_t *mdesc );
int sdm_procSdmaHandleRsp( sdm_msgDesc_t *mdesc );
int sdm_procDevControlReq( sdm_msgDesc_t *mdesc );
int sdm_procDevControlRsp( sdm_msgDesc_t *mdesc );
int sdm_procNfxEvents( sdm_msgDesc_t *mdesc );
int sdm_procGetMaskListReq( sdm_msgDesc_t *mdesc );
int sdm_procClrStatsReq( sdm_msgDesc_t *mdesc );
int sdm_procNullMsg( sdm_msgDesc_t *mdesc );


//-- SDM Message Processing Table
sdm_msgTab_t sdm_msgTab[] = {
    { SDM_MID_DEV_LIST_REQ      , sdm_procDevListReq       },
    { SDM_MID_DEV_INFO_REQ      , sdm_procDevInfoReq       },
    { SDM_MID_DEV_OPEN_REQ      , sdm_procOpenReq          },
    { SDM_MID_DEV_CLOSE_REQ     , sdm_procCloseReq         },
    { SDM_MID_CLOSE_ID_REQ      , sdm_procCloseReq         },
    { SDM_MID_WRITE_REQ         , sdm_procWriteReq         },
    { SDM_MID_READ_REQ          , sdm_procReadReq          },
    { SDM_MID_DEV_STATUS_REQ    , sdm_procDevStatusReq     },
    { SDM_MID_RELAY_CDB_REQ     , sdm_procRelayReq         },
	{ SDM_MID_GET_STATS_REQ     , sdm_procStatsReq         },
    { SCSI_MID_READ_RSP         , sdm_procScsiReadRsp      },
    { SCSI_MID_WRITE_RSP        , sdm_procScsiWriteRsp     },
    { SDM_MID_SAC_WRITE_REQ     , sdm_procSacWriteReq      },
    { SDM_MID_DEV_ADD           , sdm_procAddDevice        },
    { SDM_MID_DEV_REMOVE        , sdm_procRemoveDevice     },
    { SDM_MID_DEL_ALL_REQ       , sdm_procDelAllReq        },
    { SDM_MID_DEV_RESET_REQ     , sdm_procDevReset         },
    { SDM_MID_IOCTL_REQ         , sdm_procIoctl            },
    { SCSI_MID_OPEN_RSP         , sdm_procScsiOpenRsp      },
    { SCSI_MID_CLOSE_RSP        , sdm_procScsiCloseRsp     },
    { SCSI_MID_ERR_RSP          , sdm_procScsiErrRsp       },
    { SCSI_MID_RELAY_RSP        , sdm_procScsiRelayRsp     },
    { SCSI_MID_DEV_RESET_RSP    , sdm_procSCSIDevResetRsp  },
    { SCSI_MID_DEV_STATUS_RSP   , sdm_procSCSIDevStatusRsp },
    { SCSI_MID_IOCTL_RSP        , sdm_procSCSIIoctlRsp     },
    { SDM_MID_GET_SDMHANDLE_REQ , sdm_procGetSdmHandleReq  },
    { SDM_MID_GET_SCSIID_REQ    , sdm_procGetSCSIIDReq     },
    { SCSI_MID_GET_STATS_RSP    , sdm_procScsiStatsRsp     },
    { SDM_MID_CLOSE_ALL_DISK_REQ, sdm_procCloseAllDiskReq  },
    { SDM_MID_NEW_DEV           , sdm_procNewDevMsg        },
    { SCSI_MID_ADD_RSP          , sdm_procScsiAddRsp       },
    { SDM_MID_REDISCOVER_REQ    , sdm_procRediscReq        },
    { SDM_MID_DEV_RESERVE_REQ   , sdm_procDevReserveReq    },
    { SDM_MID_SDMA_HANDLE_RSP   , sdm_procSdmaHandleRsp    },
	{ SDM_MID_DEV_CONTROL_REQ   , sdm_procDevControlReq    },
	{ SCSI_MID_DEV_CONTROL_RSP  , sdm_procDevControlRsp    },
	{ NFX_EVENT_POST	        , sdm_procNfxEvents        },
	{ SDM_MID_DEV_MASKLIST_REQ	, sdm_procGetMaskListReq   },
	{ SDM_MID_CLR_STATS_REQ		, sdm_procClrStatsReq	   },
	{ SDM_MID_NULL_MSG			, sdm_procNullMsg		   },
    {0,0}
};


int
sdm_msgInit()
{
	memset(sdmDispatchedMsgList, 0, sizeof(sdmDispatchedMsgList));
	memset(smps, 0, sizeof(smps));
	memset((void *)&sdm_rxif, 0, sizeof(sdm_rxif));

	return 0;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_slotPortMsgInit
 *
 * Description:     Register to receive SDM messages.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *
 *------------------------------------------------------------------------------
 */
int
sdm_slotPortMsgInit(int slot, int port)
{
	char app_name[10];
	int index;

	if(slot == NFX_SLOTTYPE_FC) {
		sprintf(app_name, "sdmfc%d", port);
		index = port;
	}
	else {
		strcpy(app_name, "sdm");
		index = SDM_GP_SOCK;
	}

	sdm_rxif[index] = regAgileApp(app_name);
	if( sdm_rxif[index] < 0 ) {
		perror("SDM error registering for messages");
		exit(1);
	}

	{
	    int rc;
	    int rcvbuf = 0, rcvbuf_len = sizeof(rcvbuf);
	    int sndbuf = 0, sndbuf_len = sizeof(sndbuf);

	    rc = getsockopt(sdm_rxif[index], SOL_SOCKET, SO_RCVBUF, &rcvbuf, &rcvbuf_len);
	    if (rc < 0)
			perror("sdm_slotPortMsgInit: getsockopt failed");

	    rc = getsockopt(sdm_rxif[index], SOL_SOCKET, SO_SNDBUF, &sndbuf, &sndbuf_len);
	    if (rc < 0)
			perror("sdm_slotPortMsgInit: getsockopt failed");

		if(slot == NFX_SLOTTYPE_FC) {
			rc = eee_connect(sdm_rxif[index], slot, port, eee_getApplicationId("scsi"));
			if(rc) {
				printf("sdm_slotPortMsgInit: eee_connect for slot %d port %d failed\n", slot, port);
				exit(1);
			}
		}
	}

	return 0;
} /* sdm_slotPortMsgInit */

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_sendMsg
 *
 * Description:     Send SDM messages.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendMsg( char *msg, 
			 int len, 
			 char *app, 
			 uint32 to_cpu, 
			 uint32 to_slot,
			 struct timeval *to,
			 char *rsp_buf,
			 uint32 rsp_len)
{
	int rc;
	int index;

#if SDM_DEBUG_TX_MSG
	printf("SDM tx msg: %d bytes to app: %s, cpu: %d, slot: %d\n", 
		   len, app, to_cpu, to_slot);
#endif
	
	index = (to_slot == NFX_SLOTTYPE_FC) ? to_cpu : SDM_GP_SOCK;

	rc = sendAgileMsg(	sdm_rxif[index], 
					  	msg, 
					  	len, 
						app, 
						to_cpu, 
						to_slot, 
					  	0, to, rsp_buf, rsp_len);
	if( rc < 0 )
		printf("sdm send msg error\n");

	return 0;
} /* sdm_sendMsg */

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_rcvMsg
 *
 * Description:     Receive SDM messages.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *
 *------------------------------------------------------------------------------
 */
int
sdm_rcvMsg()
{
	int            bytes;
	uint32         srcCpu, srcSlot, srcPlane ;
	char           srcApp[32];
	static char	   msg[65536*4];
    struct timeval rcv_to = {1, 0};
	fd_set		   rcvfds;
	int			   i, rc;
	int			   max_fd = 0;

	for(i=0; i < sizeof(sdm_rxif)/sizeof(int); i++) {
		if(sdm_rxif[i]) {
			FD_SET(sdm_rxif[i], &rcvfds);
			max_fd = (sdm_rxif[i] > max_fd) ? sdm_rxif[i] : max_fd;
		}
	}

	rc = select(max_fd + 1, &rcvfds, 0, 0, &rcv_to);
	if(rc <= 0) return 0;		// ignore errors and timeouts

	for(i=0; i < sizeof(sdm_rxif)/sizeof(int) && rc; i++) {
		if(sdm_rxif[i] && FD_ISSET(sdm_rxif[i], &rcvfds)) {
			rcv_to.tv_sec = 1;
			rcv_to.tv_usec = 0;
			bytes = recvAgileMsg(sdm_rxif[i], 
								 &msg, 
								 sizeof(msg), 
								 &rcv_to, 
								 srcApp, &srcCpu, &srcSlot, &srcPlane);
			if(bytes > 0) {
				#if SDM_DEBUG_RX_MSG
					printf("SDM rx msg: %d bytes from app: %s, cpu: %d, slot: %d\n", 
						   bytes, srcApp, srcCpu, srcSlot);
				#endif
	
		
				sdm_procRcvMsg(msg, bytes, srcApp, srcCpu, srcSlot);
			}
			rc--;
		} // end of outer if
	} // end of for loop

    /* After processing the incoming request. Always process the timer */
    sdm_timerProc();
	return 0;
} /* sdm_rcvMsg */

/*
 *-----------------------------------------------------------------------------
 * Function Name:   sdm_allocTransaction
 *
 * Description:     Finds an open slot in the sdmDispatchedMsgList table 
 *					and returns	it's index == tid.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 25, 2001
 *
 *-----------------------------------------------------------------------------
 */
int
sdm_allocTransaction( sdm_devDBRec_t *devRec, sdm_msgDesc_t *mdesc ) 
{
	int i;

	for( i=0; i < SDM_TRANSACTION_MAX; i++ ) {
		if( sdmDispatchedMsgList[i].inuse       == 0 ) {
			memset(&sdmDispatchedMsgList[i], 0, sizeof(sdm_msgDesc_t));
			sdmDispatchedMsgList[i].inuse       = 1;
			sdmDispatchedMsgList[i].dev         = devRec;
			sdmDispatchedMsgList[i].callBackFunc = NULL;
			sdmDispatchedMsgList[i].callBackData = NULL;
			if( mdesc ) {
				sdmDispatchedMsgList[i].fsendRsp  = 1;
				sdmDispatchedMsgList[i].srcHandle = mdesc->srcHandle;
				sdmDispatchedMsgList[i].srcCpu = mdesc->srcCpu;
				sdmDispatchedMsgList[i].srcSlot = mdesc->srcSlot;
				memcpy(sdmDispatchedMsgList[i].srcApp, mdesc->srcApp, 32);
			}
			return(i);
		}
	}
	return(-1);
}
//-----------------------------------------------------------------------------
// Function Name:   sdm_allocRelayTransaction
//
// Description:     Finds an open slot in the sdmDispatchedMsgList table 
//					and returns	it's index == tid.
//
// Created by:      Frank Salzmann
//
// Data Created:    April 19, 2002
//
//-----------------------------------------------------------------------------
int
sdm_allocRelayTransaction( sdm_devDBRec_t *devRec, 
						   sdm_msgDesc_t *mdesc,
						   int (*callBackFunc)(),
						   int callBackData) 
{
	int i;

	for( i=0; i < SDM_TRANSACTION_MAX; i++ ) {
		if( sdmDispatchedMsgList[i].inuse        == 0 ) {
			memset(&sdmDispatchedMsgList[i], 0, sizeof(sdm_msgDesc_t));
			sdmDispatchedMsgList[i].inuse        = 1;
			sdmDispatchedMsgList[i].dev          = devRec;
			sdmDispatchedMsgList[i].callBackFunc = callBackFunc;
			sdmDispatchedMsgList[i].callBackData = callBackData;

			if( mdesc ) {
				sdmDispatchedMsgList[i].fsendRsp  = 1;
				sdmDispatchedMsgList[i].srcHandle = mdesc->srcHandle;
				sdmDispatchedMsgList[i].srcCpu    = mdesc->srcCpu;
				sdmDispatchedMsgList[i].srcSlot   = mdesc->srcSlot;
				memcpy(sdmDispatchedMsgList[i].srcApp, mdesc->srcApp, 32);
			}
			return(i | 0x10000000);
		}
	}
	return(-1);
} // sdm_allocRelayTransaction

//-----------------------------------------------------------------------------
//Function Name:   sdm_deallocTransaction
//
// Description:     Frees up entry in sdmDispatchedMsgList array.  
//
// Created by:      Frank Salzmann
//
// Data Created:    August 10, 2001
//
//-----------------------------------------------------------------------------
int
sdm_deallocTransaction( int tid ) 
{
	sdmDispatchedMsgList[tid].dev = 0;
	sdmDispatchedMsgList[tid].inuse = 0;

	return(0);
}
/*
 *-----------------------------------------------------------------------------
 * Function Name:   sdm_procRcvMsg
 *
 * Description:     Process messages sent to SDM.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *
 *-----------------------------------------------------------------------------
 */
int
sdm_procRcvMsg( char *msg, 
				int mlen, 
				uchar8 *srcApp, 
				uint32 srcCpu, 
				uint32 srcSlot)
{
	uint32 i, mid;
	scsi_msgHdr_t *hdr;
	sdm_msgDesc_t mdesc;
	int (*procFunc)() = NULL;

	hdr = (scsi_msgHdr_t *)msg;

	mdesc.msg     = (void *)msg;
	mdesc.len     = mlen;
	mdesc.srcCpu  = srcCpu;
	mdesc.srcSlot = srcSlot;
	memcpy(mdesc.srcApp, srcApp, 32);

	mid = hdr->mid;

    //-- Look for message id and associated processing function in the
    //-- SDM message table.
    for(i=0; sdm_msgTab[i].mid; i++) {
        if( sdm_msgTab[i].mid == mid ) {
            procFunc = sdm_msgTab[i].procFunc;
			procFunc(&mdesc);
            return(SDM_OK);
        }
    }
	printf("Unknown Message Received: ID: %x\n", mid);
	return(SDM_OK);

} /* sdm_procRcvMsg */ 

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_procAddDevice
 *
 * Description:     Adds device WWN and path information to device database.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 10, 2001
 *------------------------------------------------------------------------------
 */
int
sdm_procAddDevice( sdm_msgDesc_t *mdesc )
{
	uint32         i, fnewdev = 0;
	uint32         pathfound;
	sdm_devDBRec_t *devRec;	
	scsi_sdmAckMsg_t o_msg;
	sdm_addDevMsg_t  *i_msg;
	scsi_id_t      id;
	scsi_path_t	   path;

	i_msg = (sdm_addDevMsg_t *)mdesc->msg;

	//sdm_printDevAddMsg(i_msg); 

	id   = i_msg->id;
	path = i_msg->path;

	//-- Make sure device/path is allowed
	if( sdm_confVerify(i_msg->wwn, SCSI_PATH_GET_LUN(path), path, i_msg->st.name) < 0 ) {
		//printf("SDM_MSG: Device 0x%016llx masked on slot %d, port %d.\n", 
		//	   id, SCSI_PATH_GET_SLOT(path), SCSI_PATH_GET_PORT(path));
		return 0;
	}

	/*
	 * Look for device in database.
	 */
	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, id);
	 
	/*
	 * If found then just update the entry with the new path 
	 * and channel information.
	 */
	if( devRec ) {
		//printf("Device already in database\n");
		// See if the path has already been recorded.
		pathfound = 0;
		for(i=0; i < devRec->pathCnt; i++) {
			if( devRec->pathList[i].path == path ) {
				pathfound = 1;

				// Check for inconsistency between SDM and SCSI
				// Could happen if the SP_NIM was restarted and the 
				// SDM was not.
				if( i_msg->state == SCSI_DEV_STATE_CLOSED ) {
					if( devRec->state == SDM_DEV_STATE_OPENED || devRec->state == SDM_DEV_STATE_INIT_OPENED) {
						if( sdm_getActivePath(devRec) == path ) {
							
						}
					}
				}
				break; //-- Send Ack back to SCSI.
			}
		}

		if( !pathfound ) {
			devRec->pathList[devRec->pathCnt].path  = path;
			devRec->pathList[devRec->pathCnt].state = SDM_PATH_STATE_AVAILABLE;
			devRec->pathList[devRec->pathCnt].wwn   = i_msg->wwn;
			devRec->pathList[devRec->pathCnt].lun   = 
				                                SCSI_PATH_GET_LUN(i_msg->path);
			devRec->pathList[devRec->pathCnt].llHandle =  i_msg->llHandle;
			devRec->pathCnt++;
		}
	}
	else {
		/*
		 * If not found then add a new record to the database.
	  	 */
		devRec = (sdm_devDBRec_t *)malloc(sizeof(sdm_devDBRec_t));
		if( !devRec ) {
			//printf("Error: Could not allocated device database record. ");
			//printf("Device not added to database.\n");
			return(-1);
		}
		else  {
			memset(devRec, 0, sizeof(sdm_devDBRec_t));
			devRec->id = id;
			devRec->wwn = i_msg->wwn; 
			devRec->lun = SCSI_PATH_GET_LUN(i_msg->path);

			devRec->devType = sdm_ntohl(i_msg->type);
			if( devRec->devType == SCSI_DEVTYPE_DISK ) {
				devRec->lbCnt = i_msg->lbCnt;
				devRec->lbSize = i_msg->lbSize;
				devRec->raidLevel = i_msg->raidLevel;

#ifdef TURN_OFF_BLOCK_CHECKSUMMING
				/* XXX Radek For now assume all disks with 512BPS support block checksumming */
				if (devRec->lbSize == 512)
					devRec->lbCnt = devRec->lbCnt/(BCSUM_BLOCK_SPAN+1)*BCSUM_BLOCK_SPAN - SCSI_MAPPING_OFFSET_BLOCKS;
#endif
			}
			else { // TAPE
				devRec->lbCnt = 0;
				devRec->lbSize = 0;
			}

			devRec->pathList[devRec->pathCnt].path  = path;
			devRec->pathList[devRec->pathCnt].state = SDM_PATH_STATE_AVAILABLE;
			devRec->pathList[devRec->pathCnt].wwn   = i_msg->wwn;
			devRec->pathList[devRec->pathCnt].lun   = 
				                                SCSI_PATH_GET_LUN(i_msg->path);
			devRec->pathList[devRec->pathCnt].llHandle =  i_msg->llHandle;
			devRec->pathCnt++;
			memcpy((void *)&devRec->st, &i_msg->st, sizeof(sdm_dev_strings_t));

   			sdm_dbAddRec(sdmDevDB, devRec->id, devRec, sizeof(sdm_devDBRec_t),1);

			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Add:   0x%016llx:%d", 
				 devRec->wwn, devRec->lun));

			//printf("SDM: Add:   0x%016llx:%d", 
			// devRec->wwn, devRec->lun);

			// See if the SCSI application has the device 
			// in the init mode. If not the this notify is
			// probably a response to a resync message
			// Which means we don't need to ack or send evm 
			// a notify.
#if 0
			if( i_msg->state != SCSI_DEV_STATE_INIT ) {
				return SDM_OK;
			}
#endif
			
			fnewdev = 1;
		}
	}

	//-- Send SDM_ACK back to the SCSI driver.
	o_msg.hdr.mid     = SCSI_MID_SDM_ACK;
	o_msg.hdr.cHandle = i_msg->hdr.chandle;
	o_msg.hdr.path    = i_msg->path;
	sdm_sendMsg( (char *)&o_msg, sizeof(o_msg), 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	//-- Send new device notification msg to the evm.
	if( fnewdev )
    {
	    if( devRec->devType == SCSI_DEVTYPE_DISK )
		    sdm_sendDevAddNotify("evm", devRec);
        else
            sdm_sendDevAddNotifySpm(devRec);
    }

	return SDM_OK;
} // sdm_procAddDevice 

/*
 *------------------------------------------------------------------------------
 * Function Name:   sdm_procRemoveDevice
 *
 * Description:     Removes device path information and from the device 
 *					database. If the last path to a device is removed then
 *					the device entry itself will get removed.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    July 26, 2001
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procRemoveDevice( sdm_msgDesc_t *mdesc )
{
	int              isActivePath;
	sdm_devDBRec_t  *devRec;	
	sdm_remDevMsg_t *i_msg;
	scsi_id_t      id;
	scsi_path_t	   path;
	sdm_path_t     *sdm_p_entry;
	uint32         handleSlot;
	sdm_handle_t   *sdm_h_entry;

	//printf("sdm_procRemoveDevice\n");
	i_msg = (sdm_remDevMsg_t *)mdesc->msg;

	id   = i_msg->id;
	path = i_msg->path;

	/*
	 * Look for device in database.
	 */
	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, id);
	 
	/*
	 * If found then just update the entry with the new path 
	 * and channel information.
	 */
	if( devRec ) {

		if( path == sdm_getActivePath(devRec) ) {
			isActivePath = 1;
			sdm_p_entry = sdm_getActivePathEntry(devRec);
			handleSlot = (uint32)sdm_p_entry->handleSlot;
		}
		else
			isActivePath = 0;

		//-- Remove this path from the device database.
		sdm_removePath( devRec, path); 

		//-- If all paths to a device have been removed then remove the
		//-- device from the database..
		if( devRec->pathCnt <= 0 ) {

			if( devRec->handleCnt > 0 ) {    
				sdm_h_entry = (sdm_handle_t *)sdm_getHandleEntry( devRec->openHandles[0] );
				if( sdm_h_entry) {
					sdm_h_entry->dev = 0;
				}
			}
			
			sdm_sendDevDelNotify("evm", devRec);	
			
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM: Remove (RemoveDevice): 0x%016llx:%d", 
					devRec->wwn, devRec->lun));
			
			sdm_dbRemRec( sdmDevDB, id );
		}
		else {	//-- If removed path was active then use alternate path.
			if(isActivePath)
				sdm_openPath(devRec, handleSlot, 0/*port*/, 0/*path*/, 0/*flags*/);
		}
	}
	return 0;
} // sdm_procRemoveDevice

int
sdm_procDevReset(sdm_msgDesc_t *mdesc)
{
	sdm_devResetReqMsg_t *	sdm_i_msg;
	sdm_devResetRspMsg_t	sdm_o_msg;
	scsi_devResetReqMsg_t	scsi_o_msg;
	uint32					destCpu, destSlot, mlen;
	scsi_path_t				path;
	sdm_handle_t *			h_entry;
	sdm_devDBRec_t *		devRec;

	sdm_i_msg = (sdm_devResetReqMsg_t *)mdesc->msg;

	h_entry = (sdm_handle_t *)sdm_getHandleEntry(sdm_i_msg->handle);
	if ( !h_entry || !(h_entry->dev) ) {
		sdm_o_msg.hdr.mid = SDM_MID_DEV_RESET_RSP;
		sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;
		sdm_o_msg.eCode = SDM_FAIL;
		sdm_sendMsg((char *)&sdm_o_msg, sizeof(sdm_devResetRspMsg_t), 
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return(0);
	}

	path = h_entry->path;
	devRec = h_entry->dev;

	mlen = sizeof(scsi_devResetReqMsg_t); 
	bzero(&scsi_o_msg, mlen);
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;

	scsi_o_msg.hdr.mid = SCSI_MID_DEV_RESET_REQ;
	scsi_o_msg.hdr.cHandle = sdm_allocTransaction(devRec, mdesc);
	scsi_o_msg.hdr.path = path;

	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
	sdm_sendMsg((char *)&scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0, 0);

	return SDM_OK;
} // sdm_procDevResetReq 

int 
sdm_procIoctl( sdm_msgDesc_t *mdesc )
{
	sdm_ioctlReqMsg_t *		sdm_i_msg;
	int						i;
	scsi_ioctlReqMsg_t		scsi_o_msg;
	uint32					destCpu, destSlot, mlen;

	sdm_i_msg = (sdm_ioctlReqMsg_t *)mdesc->msg;

	mlen = sizeof(scsi_ioctlReqMsg_t); 
	bzero(&scsi_o_msg, mlen);
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;

	scsi_o_msg.hdr.mid = SCSI_MID_IOCTL_REQ;
	scsi_o_msg.hdr.cHandle = sdm_allocTransaction(NULL, mdesc);
	scsi_o_msg.hdr.path = 
		((sdm_i_msg->slot << SCSI_PATH_SLOT_SHIFT) & SCSI_PATH_SLOT_MASK) |
		((sdm_i_msg->port << SCSI_PATH_PORT_SHIFT) & SCSI_PATH_PORT_MASK);
	scsi_o_msg.code = sdm_i_msg->code;
	scsi_o_msg.arg_count = MIN(sdm_i_msg->arg_count, SCSI_IOCTL_MAX_ARGS);
	for (i = 0; i < scsi_o_msg.arg_count; ++i)
		scsi_o_msg.args[i] = sdm_i_msg->args[i];

	destCpu = sdm_i_msg->port;
	destSlot = sdm_i_msg->slot;
	sdm_sendMsg((char *)&scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0, 0);

	return SDM_OK;
} /* sdm_procIoctl */



/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procDevListReq
 *
 * Description: 	Sends list of all device ids contained in the SDM device DB.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 11, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procDevListReq( sdm_msgDesc_t *mdesc )
{
	uint32				rspSize;
	uint32              dbEntries;
	uint32              type, port;
    char                *vendor;
	sdm_devListRspMsg_t *o_msg;
	sdm_devListReqMsg_t *i_msg; 

#if SDM_DEBUG_RX_MSG
	printf("sdm_procDevListReq: Enter\n");
#endif

	i_msg = (sdm_devListReqMsg_t *)mdesc->msg;
	type  = i_msg->type;
    port  = i_msg->port;
    vendor = (strlen(i_msg->vendorName))?i_msg->vendorName:NULL;

	dbEntries = sdm_dbGetNumEntries(sdmDevDB, type, port, vendor);

	if( dbEntries > 0 ) {
		rspSize = sizeof(sdm_devListRspMsg_t) + (sizeof(scsi_id_t) * dbEntries);
		o_msg = (sdm_devListRspMsg_t *)malloc(rspSize);
		memset(o_msg, 0, rspSize);

		if(o_msg) 
			sdm_dbGetDevList(sdmDevDB, &o_msg->idList[0], type, port, vendor); 
		else
			return NFX_ERR;
	}
	else {
		rspSize = sizeof(sdm_devListRspMsg_t);
		o_msg = (sdm_devListRspMsg_t *)malloc( rspSize );
		if( !o_msg )
			return NFX_ERR;
	}
	o_msg->hdr.mid     = SDM_MID_DEV_LIST_RSP;
	o_msg->numEntries  = dbEntries;	
	o_msg->hdr.chandle = i_msg->hdr.chandle;

	sdm_sendMsg( (char *)o_msg, rspSize, 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	free(o_msg);
	return NFX_OK;
} /* sdm_procDevListReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procDevInfoReq
 *
 * Description: 	Returns message containing detailed information about
 *					a particular device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 20, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procDevInfoReq( sdm_msgDesc_t *mdesc ) 
{
	scsi_id_t			id;
	sdm_devDBRec_t      *devRec;
	sdm_devInfoRspMsg_t o_msg;
	sdm_devInfoReqMsg_t *i_msg; 

#if SDM_DEBUG_RX_MSG
	printf("sdm_procDevInfoReq: Enter\n");
#endif

	i_msg = (sdm_devInfoReqMsg_t *)mdesc->msg;
	id    = i_msg->id;
	
	memset(&o_msg, 0, sizeof(sdm_devInfoRspMsg_t));
	o_msg.hdr.mid = SDM_MID_DEV_INFO_RSP;
	o_msg.hdr.chandle = i_msg->hdr.chandle;

	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, id);
	if( !devRec ) {
		//printf("Device Record Not Found: %llx\n", id);
		o_msg.rc = SDM_FAIL;
	}
	else {
		o_msg.rc        = SDM_OK;
		o_msg.id        = id;
		o_msg.wwn       = devRec->wwn;
		o_msg.lun       = devRec->lun;

		o_msg.state     = devRec->state;
		if( devRec->state == SDM_DEV_STATE_OPENED ||
			devRec->state == SDM_DEV_STATE_INIT_OPENED )
			o_msg.openHandle = devRec->openHandles[0];
		else
			o_msg.openHandle = 0;

		o_msg.raidLevel = devRec->raidLevel;
		o_msg.devType   = devRec->devType;
		o_msg.blockCnt  = devRec->lbCnt;
		o_msg.blockSize = devRec->lbSize;
		memcpy((void *)&o_msg.st, &devRec->st, sizeof(o_msg.st));
	}

	sdm_sendMsg( (char *)&o_msg, sizeof(sdm_devInfoRspMsg_t), 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
	return NFX_OK;
} /* sdm_procDevInfoReq */
/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procDevStatusReq
 *
 * Description: 	Returns message containing status information about
 *					a particular device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	August 21, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procDevStatusReq( sdm_msgDesc_t *mdesc ) 
{
	uint32						i;
	scsi_id_t					id;
	sdm_devDBRec_t *			devRec;
	sdm_devStatusRspMsg_t		o_msg;
	sdm_devStatusReqMsg_t *		i_msg; 
	scsi_devStatusReqMsg_t		scsi_o_msg;
	scsi_path_t					path;
	uint32						destCpu, destSlot;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procDevStatusReq: Enter\n");
#endif

	i_msg = (sdm_devStatusReqMsg_t *)mdesc->msg;
	id    = i_msg->id;
	
	memset(&o_msg, 0, sizeof(sdm_devStatusRspMsg_t));
	o_msg.hdr.mid = SDM_MID_DEV_STATUS_RSP;
	o_msg.hdr.chandle = i_msg->hdr.chandle;

	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, id);
	if( !devRec ) {
		o_msg.eCode = SDM_FAIL;
		sdm_sendMsg( (char *)&o_msg, sizeof(sdm_devStatusRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
	}
	else {
		/*
		 * For non-tape devices, we can return immediately. For tapes,
		 * we need to query the SCSI layer for the current tape
		 * position.  
		 */
		if (devRec->devType != SCSI_DEVTYPE_TAPE) {
			o_msg.eCode      = SDM_OK;
			o_msg.wwn        = devRec->wwn;
			o_msg.lun        = devRec->lun;
			o_msg.state		 = devRec->state;
			o_msg.raidLevel  = devRec->raidLevel;
			o_msg.activePath = sdm_getActivePath(devRec);
			o_msg.pathCnt    = devRec->pathCnt;
			o_msg.handleCnt  = devRec->handleCnt;
			o_msg.devType    = devRec->devType;
			o_msg.blockCnt   = devRec->lbCnt;
			o_msg.blockSize  = devRec->lbSize;
			memcpy((void *)&o_msg.st, &devRec->st, sizeof(o_msg.st));
			for(i=0; i < SDM_MAX_PATHS; i++) {
				o_msg.paths[i].path  = devRec->pathList[i].path;
				o_msg.paths[i].state = devRec->pathList[i].state;
				o_msg.paths[i].wwn   = devRec->pathList[i].wwn;
				o_msg.paths[i].lun   = devRec->pathList[i].lun;
			}

			for(i=0; i < SDM_MAX_HANDLES_PER_DEV; i++)
				o_msg.handles[i] = devRec->openHandles[i];
	
			sdm_sendMsg( (char *)&o_msg, sizeof(sdm_devStatusRspMsg_t), 
						 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		}
		else {
			/* Get active path if device open, else first available path */
			path = sdm_getActivePath(devRec);
			if (path == 0)
				path = devRec->pathList[0].path;
			destCpu = SCSI_PATH_GET_PORT(path);
			destSlot = SCSI_PATH_GET_SLOT(path);

			bzero(&scsi_o_msg, sizeof(scsi_devStatusReqMsg_t));
			mdesc->srcHandle = i_msg->hdr.chandle;

			scsi_o_msg.hdr.mid = SCSI_MID_DEV_STATUS_REQ;
			scsi_o_msg.hdr.cHandle = sdm_allocTransaction(devRec, mdesc);
			scsi_o_msg.hdr.path = path;

			sdm_sendMsg((char *)&scsi_o_msg, sizeof(scsi_devStatusReqMsg_t), "scsi", destCpu, destSlot, 0, 0, 0);
		}
	}
	return NFX_OK;
} /* sdm_procDevStatusReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procOpenReq
 *
 * Description: 	Opens a device for reading/writing and returns a handle.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 21, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procOpenReq(sdm_msgDesc_t *mdesc)
{
	scsi_id_t		 id;
	sdm_devDBRec_t   *devRec;
	sdm_openReqMsg_t *sdm_i_msg; 
	sdm_openRspMsg_t  sdm_o_msg;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procOpenReq: Enter\n");
#endif

	memset(&sdm_o_msg, 0, sizeof(sdm_openRspMsg_t));
	sdm_i_msg = (sdm_openReqMsg_t *)mdesc->msg;
	id        = sdm_i_msg->id;

	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, id);
	if( !devRec ) {

		// Send response back to the application that made the open request.
		sdm_o_msg.hdr.mid     = SDM_MID_DEV_OPEN_RSP;
		sdm_o_msg.rCode       = SDM_FAIL;
		sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;

		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_openRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

		//printf("Device Record Not Found: %llx\n", id);

		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
			   ("SDM: Open: Device Not Found"));
	}
	else {
		//-- Only allow device to be opened once.
		if( devRec->state != SDM_DEV_STATE_CLOSED && devRec->state != SDM_DEV_STATE_INIT) { 
			
			sdm_o_msg.hdr.mid     = SDM_MID_DEV_OPEN_RSP;
			sdm_o_msg.rCode       = SDM_TO_MANY_OPENS;
			sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;

			sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_openRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Open:  0x%016llx:%d: Device Already Open", 
				 devRec->wwn, devRec->lun));
		}
		else {
			devRec->rspInfo.chandle = sdm_i_msg->hdr.chandle;
			devRec->rspInfo.srcCpu = mdesc->srcCpu;
			devRec->rspInfo.srcSlot = mdesc->srcSlot;
			memcpy(devRec->rspInfo.srcApp, mdesc->srcApp, 32);
			if(sdm_devOpenSM( devRec, SDM_MID_DEV_OPEN_REQ, sdm_i_msg ) != SDM_OK) {

				sdm_o_msg.hdr.mid     = SDM_MID_DEV_OPEN_RSP;
				sdm_o_msg.rCode       = SDM_INVALID_PARM;
				sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;
				sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_openRspMsg_t),
							mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
				E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
						("SDM: Open: No path found for device 0x%llx:%d, specified path %d port %d\n",
						 devRec->wwn, devRec->lun, sdm_i_msg->path, sdm_i_msg->port));
			}
		}
	}
	return NFX_OK;
} /* sdm_procOpenReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendScsiOpenReq
 *
 * Description: 	Send request to SCSI application to open a path to a device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 25, 2001
 *
 * Returns:        	tid 
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendScsiOpenReq( scsi_path_t path, sdm_devDBRec_t *devRec )
{
	int    tid;
	uint32 destCpu, destSlot;
	scsi_openReqMsg_t scsi_o_msg;
	sdm_tmr_t         *tmr;

	memset(&scsi_o_msg, 0, sizeof(scsi_openReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_OPEN_REQ;
	scsi_o_msg.hdr.path    = sdm_htonl(path);

	// A reply from the SCSI application is expected so setup the
	// cHandle and stash away info from the original open request so
	// that a reply can be associated with the proper device record.
	tid = sdm_allocTransaction( devRec, 0 );
	scsi_o_msg.hdr.cHandle = tid;
				
	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
				
#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiOpenReq: destCpu: %d, destSlot: %d, tid: %d, path: %x\n", 
		   destCpu, destSlot, tid, path);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_openReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	// Start a 3 second timer.
	tmr = sdm_timerStart(3, sdm_openTimeout, (int)devRec, (int)tid);
	sdmDispatchedMsgList[tid].tmr = tmr;; 
	return NFX_OK;
} /* sdm_sendScsiOpenReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procScsiOpenRsp
 *
 * Description: 	Handles device open response message sent to the SDM by
 *					a scsi application instance.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 23, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procScsiOpenRsp(sdm_msgDesc_t *mdesc)
{
	uint32			 tid;
	sdm_devDBRec_t   *devRec;
	scsi_openRspMsg_t *scsi_i_msg;

	scsi_i_msg = (scsi_openRspMsg_t *)mdesc->msg;
	
	tid = sdm_ntohl(scsi_i_msg->hdr.cHandle);
	devRec  = sdmDispatchedMsgList[tid].dev; 
	sdm_deallocTransaction(tid);
	if( !devRec ) {
		//printf("[OPEN_RSP] tid: %x...no device rec found\n", tid);
		return(0);
	}

	sdm_timerStop(sdmDispatchedMsgList[tid].tmr); 
	sdm_devOpenSM( devRec, SCSI_MID_OPEN_RSP, scsi_i_msg );
	return SDM_OK;
} /* sdm_procScsiOpenRsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendSetPathMsg
 *
 * Description: 	Sends a message to the SDM Agent on the FP NIM to 
 *					associate an open device handle with a scsi_path_t.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	August 21, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendSetPathMsg(uint64 id, uint32 handle, scsi_path_t path)
{
	sdm_setPathReqMsg_t sdm_o_msg;

	memset(&sdm_o_msg, 0, sizeof(sdm_setPathReqMsg_t));
	sdm_o_msg.hdr.mid = SDM_MID_SDMA_SET_PATH_REQ;
	sdm_o_msg.handle  = handle;
	sdm_o_msg.path    = path;
	sdm_o_msg.id      = id;

	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_setPathReqMsg_t), 
				 "sdm_agent", NFX_NFP_FP2, 1 , 0, 0, 0);

	return SDM_OK;
} // sdm_sendSetPathMsg

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendRemPathMsg
 *
 * Description: 	Sends a message to the SDM Agent on the FP NIM to 
 *					associate an open device handle with a scsi_path_t.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	August 21, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendRemPathMsg( uint32 handle )
{
	sdm_remPathReqMsg_t sdm_o_msg;

	memset(&sdm_o_msg, 0, sizeof(sdm_remPathReqMsg_t));
	sdm_o_msg.hdr.mid = SDM_MID_SDMA_REM_PATH_REQ;
	sdm_o_msg.handle  = handle;

	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_remPathReqMsg_t), 
				 "sdm_agent", NFX_NFP_FP2, 1 , 0, 0, 0);

	return SDM_OK;
} // sdm_sendRemPathMsg

//------------------------------------------------------------------------------
// Function Name:	sdm_sendSdmaHandleReq
//
// Description: 	Sends a message to the SDM Agent on the FP NIM to 
//					create a sdm_handle for a scsi_id/path and return it.
//
// Created by:		Frank Salzmann
//
// Data Created:	May 07, 2002
//
// Returns:         NFX_OK
//
//------------------------------------------------------------------------------
int
sdm_sendSdmaHandleReq( sdm_devDBRec_t *devRec, 
					   scsi_id_t       id, 
					   scsi_path_t     path, 
					   uint32          sdma_slot )
{
	int                     tid;
	sdm_tmr_t              *tmr;
	sdm_sdmaHandleReqMsg_t  sdm_o_msg;


	sdma_slot = 1;
	tid = sdm_allocTransaction( devRec, 0 );

	memset(&sdm_o_msg, 0, sizeof(sdm_sdmaHandleReqMsg_t));
	sdm_o_msg.hdr.mid     = SDM_MID_SDMA_HANDLE_REQ;
	sdm_o_msg.hdr.chandle = tid;
	sdm_o_msg.id          = id;
	sdm_o_msg.path        = path;

#if SDM_DEBUG_TX_MSG
    printf("sdm_sendSdmaHandleReq: id: %llx, path: %llx, sdma_slot: %d\n",
		   id, path, sdma_slot);
	sdm_printBuf((char *)&sdm_o_msg, sizeof(sdm_sdmaHandleReqMsg_t));
#endif

	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_sdmaHandleReqMsg_t), 
				 "sdm_agent", NFX_NFP_FP2, sdma_slot , 0, 0, 0);

	// Start a 3 second timer.
	tmr = sdm_timerStart(3, sdm_sdmaHandleTimeout, (int)devRec, (int)tid);
	sdmDispatchedMsgList[tid].tmr = tmr;

	return SDM_OK;
} // sdm_sendSdmaHandleReq

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procCloseReq
 *
 * Description: 	Handles device close request message sent to the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 26, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procCloseReq( sdm_msgDesc_t *mdesc )
{
	uint32			  chandle;
	uint32			  devHandle;
	scsi_path_t		  path;
	sdm_handle_t	  *h_entry;
	sdm_devDBRec_t    *devRec;
	sdm_closeReqMsg_t *sdm_i_msg; 
	sdm_closeRspMsg_t  sdm_o_msg;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procCloseReq: Enter\n");
#endif

	memset(&sdm_o_msg, 0, sizeof(sdm_closeRspMsg_t));

	if( ((sdm_msgHdr_t *)(mdesc->msg))->mid == SDM_MID_CLOSE_ID_REQ ) {
		scsi_id_t id;

		id = ((sdm_closeIdReqMsg_t *)(mdesc->msg))->id;
		
		devHandle = sdm_idToHandle(id);
		if( devHandle )
			h_entry = (sdm_handle_t *)sdm_getHandleEntry(devHandle);
		else
			h_entry = 0;
	}
	else {
		sdm_i_msg = (sdm_closeReqMsg_t *)mdesc->msg;
		chandle = sdm_i_msg->hdr.chandle;
		devHandle = sdm_i_msg->handle;
		h_entry   = (sdm_handle_t *)sdm_getHandleEntry( devHandle );
	}

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
		   ("SDM: (CloseReq): handle: %x, h_entry: %x", devHandle, h_entry));

	if( !h_entry ) {
		devRec = sdm_dbFindId(sdmDevDB, ((sdm_closeIdReqMsg_t *)(mdesc->msg))->id);
		if(devRec && devRec->state == SDM_DEV_STATE_CLOSED)
			sdm_o_msg.rCode = SDM_NOT_OPEN;
		else
			sdm_o_msg.rCode = SDM_FAIL;
	}
	else {
		path   = h_entry->path;
		devRec = h_entry->dev;

		//-- At this stage of the game the device record may have been removed. This will be
		//-- the case if the closeReq message was sent by the EVM in response to our message
		//-- telling it that the device has been removed.
		if( devRec ) {

			sdm_setPathState( devRec, path, SDM_PATH_STATE_AVAILABLE );

			switch(devRec->state) {
				case SDM_DEV_STATE_INIT_OPENING:
				case SDM_DEV_STATE_INIT_OPENED:
					devRec->state = SDM_DEV_STATE_INIT;
					break;
				default:
					devRec->state = SDM_DEV_STATE_CLOSED;
					break;
			}

			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM: Send Close Rsp (CloseReq): 0x%016llx:%d", 
					devRec->wwn, devRec->lun));
		}
		else {
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM: Close Handle, Device Removed (CloseReq): %x", devHandle));
		}

		sdm_releaseDevHandle(devHandle);
		sdm_o_msg.rCode = SDM_OK;
	}

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid  = SDM_MID_DEV_CLOSE_RSP;
	sdm_o_msg.hdr.chandle = chandle;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_closeRspMsg_t), 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	return SDM_OK;
} /* sdm_procCloseReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendScsiCloseReq
 *
 * Description: 	Send request to SCSI application to close a path to a device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 25, 2001
 *
 * Returns:        	tid 
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendScsiCloseReq( scsi_path_t path, uint32 tid )
{
	uint32 destCpu, destSlot;
	scsi_closeReqMsg_t scsi_o_msg;

	memset(&scsi_o_msg, 0, sizeof(scsi_closeReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_CLOSE_REQ;
	scsi_o_msg.hdr.path    = path;
	scsi_o_msg.hdr.cHandle = tid;
				
	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
				
#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiCloseReq: destCpu: %d, destSlot: %d, path: %x\n", 
		   destCpu, destSlot, path);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_closeReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	return NFX_OK;
} /* sdm_sendScsiCloseReq */
/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procScsiCloseRsp
 *
 * Description: 	Handles device close response message sent to the SDM by
 *					a scsi application instance.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 22, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procScsiCloseRsp(sdm_msgDesc_t *mdesc)
{
	uint32			 tid;
	sdm_msgDesc_t    *rspInfo;
	sdm_devDBRec_t   *devRec;
	sdm_closeRspMsg_t   sdm_o_msg;
	scsi_closeRspMsg_t *scsi_i_msg;

	scsi_i_msg = (scsi_closeRspMsg_t *)mdesc->msg;
	
	tid = scsi_i_msg->hdr.cHandle;
	if( tid < 0 || tid > SDM_TRANSACTION_MAX )
		return 0;		// Bogus

	rspInfo = &sdmDispatchedMsgList[tid]; 
	devRec  = rspInfo->dev;
	if( !devRec ) {
		//printf("[SDM_CLOSE_RSP] tid: %x...no device rec found\n", 
		//	   tid);
		return(0);
	}
	// sdm_timerStop(sdmDispatchedMsgList[tid].tmr); 
	devRec->state = SDM_DEV_STATE_CLOSED;

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid     = SDM_MID_DEV_CLOSE_RSP;
	sdm_o_msg.hdr.chandle = rspInfo->srcHandle;
	sdm_o_msg.rCode       = SDM_OK;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_closeRspMsg_t), 
				 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);

	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Close: 0x%016llx:%d", 
				 devRec->wwn, devRec->lun));

	sdm_deallocTransaction(tid);
	return SDM_OK;
} /* sdm_procScsiCloseRsp */
/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procCloseAllDiskReq
 *
 * Description: 	Handles request to close all disk devices. Usually 
 * 					sent to the SDM by the EVM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	March 06, 2002
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procCloseAllDiskReq( sdm_msgDesc_t *mdesc )
{
	uint32			  		 i, devCnt;
	uint32			  		 devHandle;
	uint64            		 id = 0;
	scsi_path_t		  		 path;
	sdm_handle_t	  		 *h_entry;
	sdm_devDBRec_t    		 *devRec;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procCloseAllDiskReq: Enter\n");
#endif

	devCnt = sdm_dbGetNumEntries( sdmDevDB, SDM_DEVLIST_ALL, SDM_DEVLIST_PORTALL, NULL );

	for( i=0; i < devCnt; i++ ) {

		devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
		if( devRec ) {
			id = devRec->id;
			if( devRec->state == SDM_DEV_STATE_OPENED || devRec->state == SDM_DEV_STATE_INIT_OPENED ) {
				devHandle = devRec->openHandles[0];
				h_entry   = (sdm_handle_t *)sdm_getHandleEntry( devHandle );

				path = h_entry->path;
				sdm_releaseDevHandle( devHandle );

				sdm_setPathState( devRec, path, SDM_PATH_STATE_AVAILABLE );
				devRec->state = SDM_DEV_STATE_CLOSED;

				E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				("SDM: Close: 0x%016llx:%d", 
				 devRec->wwn, devRec->lun));
			}
		}
	}

	return SDM_OK;
} // sdm_procCloseAllDiskReq 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procScsiErrRsp
 *
 * Description: 	Handles device open response message sent to the SDM by
 *					a scsi application instance.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 23, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procScsiErrRsp( sdm_msgDesc_t *mdesc )
{
	scsi_errRspMsg_t *	scsi_i_msg;
	sdm_errRspMsg_t		sdm_o_msg; 
	uint32				tid;
	sdm_msgDesc_t *		req_context;
	sdm_devDBRec_t *	devRec;

	scsi_i_msg = (scsi_errRspMsg_t *)mdesc->msg;

	//printf("sdm_procScsiErrRsp: cHandle 0x%x\n", scsi_i_msg->hdr.cHandle);

	tid = scsi_i_msg->hdr.cHandle;

    printf("sdm_procScsiErrRsp: TID : %d\n", tid);

	//-- Must be an internal relay request made by the 
	//-- SDM device initialization code. 
	if( tid > SDM_TRANSACTION_MAX ) {
		sdm_initDevProcMsg((sdm_dir_t *)tid, scsi_i_msg);
		return SDM_OK;
	}

	req_context = &sdmDispatchedMsgList[tid];

	devRec = req_context->dev;
	bzero(&sdm_o_msg, sizeof(sdm_errRspMsg_t));
	if (!devRec)
		sdm_o_msg.eCode = SDM_FAIL;
	else
		sdm_o_msg.eCode = (scsi_i_msg->eCode == SCSI_MSG_ECODE_OK) ? SDM_OK : scsi_i_msg->eCode;

	sdm_o_msg.hdr.mid = SDM_MID_ERR_RSP;
	sdm_o_msg.hdr.chandle = req_context->srcHandle;
	sdm_sendMsg((char *)&sdm_o_msg, sizeof(sdm_errRspMsg_t), 
				req_context->srcApp, req_context->srcCpu, req_context->srcSlot, 0, 0, 0);
	sdm_deallocTransaction(tid);
	return SDM_OK;
} /* sdm_procScsiErrRsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendDevAddNotify
 *
 * Description: 	Sends a new device notification message.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 25, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendDevAddNotify( char *destApp, sdm_devDBRec_t *devRec )
{
	sdm_devInfoRspMsg_t sdm_o_msg;

	memset(&sdm_o_msg, 0, sizeof(sdm_devInfoRspMsg_t));
	sdm_o_msg.hdr.mid	= SDM_MID_DEV_INFO_RSP;
	sdm_o_msg.rc        = SDM_OK;
	sdm_o_msg.id        = devRec->id;
	sdm_o_msg.wwn		= devRec->wwn;
    sdm_o_msg.lun       = devRec->lun;
	sdm_o_msg.state     = devRec->state;
    sdm_o_msg.devType   = devRec->devType;
	sdm_o_msg.blockCnt  = devRec->lbCnt;
	sdm_o_msg.blockSize = devRec->lbSize;
    sdm_o_msg.raidLevel = devRec->raidLevel;
	memcpy((void *)&sdm_o_msg.st, &devRec->st, sizeof(sdm_o_msg.st));

	//printf("Vendor: %s, Model: %s\n", sdm_o_msg.vendorName, sdm_o_msg.modelName);

	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_devInfoRspMsg_t), 
				 destApp, NFX_SSC_UNIX, SSC_SLOT, 0, 0, 0);
	return 0;
} /* sdm_sendDevAddNotify */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendDevAddNotifySpm
 *
 * Description: 	Sends a new device notification message.
 *
 * Created by:		Nitin Jain
 *
 * Data Created:	Sept 26, 2002
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendDevAddNotifySpm( sdm_devDBRec_t *devRec )
{
    ssize_t            ncmMsgSize;
    ncmMsg_t           *ncmMsg = NULL;
    spm_MsgAddLunReq_t *spmReq = NULL;

    ncmMsgSize = sizeof(ncmMsg_t) + sizeof(spm_MsgAddLunReq_t);
    ncmMsg = (ncmMsg_t *)malloc(ncmMsgSize);
    if (!ncmMsg)
        return SDM_FAIL;

	memset(ncmMsg, 0, ncmMsgSize);
    ncmMsg->msgType = NCM_SEND;
    strcpy(ncmMsg->appName, "spm");
    strcpy(ncmMsg->nodeName, "PCC");
    ncmMsg->rqResp = 0;    // no response expected
    spmReq = (spm_MsgAddLunReq_t *) ncmMsg->appMsg;

    spmReq->hdr.msgType = SPM_ADD_LUN_REQ;
    gethostname(spmReq->hdr.node, EVM_MAX_NODE_NAME);
    spmReq->lunAttr.scsiId = devRec->id;
    spmReq->lunAttr.wwn = devRec->wwn;
    spmReq->lunAttr.lun = devRec->lun;
    strcpy(spmReq->lunAttr.name, devRec->st.name);
    strcpy(spmReq->lunAttr.vendor, devRec->st.vendor);
    strcpy(spmReq->lunAttr.model, devRec->st.model);
    spmReq->lunAttr.raidLevel = devRec->raidLevel;
    spmReq->lunAttr.size = devRec->lbCnt;
    spmReq->lunAttr.devType = devRec->devType;
 
	//printf("Vendor: %s, Model: %s\n", sdm_o_msg.vendorName, sdm_o_msg.modelName);

	sdm_sendMsg((char *)ncmMsg, ncmMsgSize, "ncm", NFX_SSC_UNIX, SSC_SLOT, 0, 0, 0);
	return 0;
} /* sdm_sendDevAddNotifySpm */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendDevDelNotify
 *
 * Description: 	Sends a device delete request message.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 30, 2002
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendDevDelNotify( char *destApp, sdm_devDBRec_t *devRec )
{
	sdm_devDelReqMsg_t sdm_o_msg;

	sdm_o_msg.hdr.mid	= SDM_MID_DEV_DEL_REQ;
	sdm_o_msg.id        = devRec->id;

	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_devDelReqMsg_t), 
				 destApp, NFX_SSC_UNIX, SSC_SLOT, 0, 0, 0);
	return 0;
} /* sdm_sendDevDelNotify */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procWriteReq
 *
 * Description: 	Handles a write data request message being sent to a 
 *					SCSI device via the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 26, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procWriteReq(sdm_msgDesc_t *mdesc )
{
	uint32                 tid, destCpu, destSlot, mlen, dataLen, totalDataLen;
	char                  *msgData;
	scsi_path_t            path;
	sdm_handle_t          *h_entry;
	sdm_devDBRec_t        *devRec;
	scsi_writeReqMsg_t    *scsi_o_msg;
	sdm_writeDataReqMsg_t *sdm_i_msg;
	char				  *in_p;
	uint32				   seqId, seqMax, transId;

	sdm_i_msg = (sdm_writeDataReqMsg_t *)mdesc->msg;

	// Translate handle into a device path.
	h_entry  = (sdm_handle_t *)sdm_getHandleEntry( sdm_i_msg->handle );
	if( !h_entry || !(h_entry->dev) ) {
		sdm_writeDataRspMsg_t sdm_o_msg;
		sdm_o_msg.hdr.mid	  = SDM_MID_WRITE_RSP;
		sdm_o_msg.hdr.chandle = mdesc->srcHandle;
		sdm_o_msg.eCode       = SDM_FAIL;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_writeDataRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return 0;
	}

	devRec = h_entry->dev;
	path   = h_entry->path;

	// A reply from the SCSI application is expected so setup the
	// cHandle and stash away info from the original open request so
	// that a reply can be associated with the proper device record.
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;
	tid = sdm_allocTransaction(devRec, mdesc);

	// Setup SCSI write msg
	totalDataLen = (sdm_i_msg->blkCnt * (sdm_i_msg->blkSize ? sdm_i_msg->blkSize : 1));
	in_p = sdm_i_msg->data;
	transId = sdm_getTransactionId();
	seqMax = (totalDataLen / MAX_PACKET_DATA) + ((totalDataLen % MAX_PACKET_DATA) ? 1 : 0);
	seqId = 1;

	scsi_o_msg = (scsi_writeReqMsg_t *)malloc(sizeof(scsi_writeReqMsg_t) + MAX_PACKET_DATA);
	if( !scsi_o_msg ) {
		//printf("Error Cannot allocate buffer for scsi_writeMsgReq\n");
		return SDM_FAIL;
	}
	memset(scsi_o_msg, 0, sizeof(scsi_writeReqMsg_t));
	scsi_o_msg->hdr.mid     = SCSI_MID_WRITE_REQ;
	scsi_o_msg->hdr.cHandle = tid;
	scsi_o_msg->hdr.path    = path;
	scsi_o_msg->blkOffset   = sdm_i_msg->blkOffset;
	scsi_o_msg->blkCnt      = sdm_i_msg->blkCnt;
	scsi_o_msg->attributes  = 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_NO_REORDER) ? SCSI_MSG_ATTR_NO_REORDER : 0) | 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_PAD_WRITE) ? SCSI_MSG_ATTR_PAD_WRITE : 0) |
		((sdm_i_msg->attributes & SDM_MSG_ATTR_PHY_OP) ? SCSI_MSG_ATTR_PHY_OP : 0) | 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_IGNORE_WRITE_GATE) ? SCSI_MSG_ATTR_IGNORE_WRITE_GATE : 0) | 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_KICK_WRITE_GATE) ? SCSI_MSG_ATTR_KICK_WRITE_GATE : 0) |
		SCSI_MSG_ATTR_NO_GUAR_DELIVERY;

	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);

	while (totalDataLen) {
	    dataLen = MIN(totalDataLen, MAX_PACKET_DATA);
		mlen = sizeof(scsi_writeReqMsg_t) + dataLen; 
		scsi_o_msg->transSeqId  = SCSI_MSG_TRANSSEQID_PACK(transId, seqMax, seqId);

		msgData = ((char *)scsi_o_msg) + sizeof(scsi_writeReqMsg_t);
		memcpy(msgData, in_p, dataLen); 
        if (devRec->devType == SCSI_DEVTYPE_DISK)
		    scsi_o_msg->checkSum = uint32_checksum(msgData, dataLen, 0);

		/* LIODEBUG
		printf("SCSI_MID_WRITE_REQ: cHandle 0x%x, path 0x%x, transSeqId 0x%x\n",
		       scsi_o_msg->hdr.cHandle,
		       scsi_o_msg->hdr.path,
		       scsi_o_msg->transSeqId);
		*/
		// Send write command and data to the SCSI application.
		sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
#if SDM_DEBUG_TX_MSG
		printf("sdm_procWriteReq: tid: %x, path: %x, s_blk: %d, blk_cnt: %d\n", 
			   tid, path, scsi_o_msg->blkOffset, scsi_o_msg->blkCnt);
#endif

		in_p += dataLen;
		totalDataLen -= dataLen;
		++seqId;
	}
	free( scsi_o_msg );
	return SDM_OK;
} /* sdm_procWriteReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procScsiWriteRsp
 *
 * Description: 	Handles a write data response message sent back by the
 *					SCSI application
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 28, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procScsiWriteRsp(sdm_msgDesc_t *mdesc )
{
	uint32				  tid;
	sdm_msgDesc_t		  *rspInfo;
	sdm_devDBRec_t        *devRec;
	sdm_writeDataRspMsg_t  sdm_o_msg; 
	scsi_writeRspMsg_t    *scsi_i_msg;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procScsiWriteRsp\n");
#endif

	memset(&sdm_o_msg, 0, sizeof(sdm_writeDataRspMsg_t));

	scsi_i_msg = (scsi_writeRspMsg_t *)mdesc->msg;
	/* LIODEBUG
	printf("SCSI_MID_WRITE_RSP: cHandle 0x%x, path 0x%x, transSeqId 0x%x, "
	       "eCode 0x%x\n", 
	       scsi_i_msg->hdr.cHandle,
	       scsi_i_msg->hdr.path,
	       scsi_i_msg->transSeqId,
	       scsi_i_msg->eCode);
	*/
	tid = scsi_i_msg->hdr.cHandle;
	rspInfo = &sdmDispatchedMsgList[tid];

	devRec = rspInfo->dev;
	if( !devRec ) {
		sdm_o_msg.eCode       = SDM_FAIL;
	}
	else {
		sdm_o_msg.eCode       = (scsi_i_msg->eCode == SCSI_MSG_ECODE_OK) ? SDM_OK : scsi_i_msg->eCode;
	}

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid     = SDM_MID_WRITE_RSP;
	sdm_o_msg.hdr.chandle = rspInfo->srcHandle;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_writeDataRspMsg_t), 
				 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);

	sdm_deallocTransaction(tid);
	return SDM_OK;
} /* sdm_procScsiWriteRsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procReadReq
 *
 * Description: 	Handles a read data request message being sent to a 
 *					SCSI device via the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 26, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procReadReq(sdm_msgDesc_t *mdesc )
{
	uint32                tid, destCpu, destSlot, mlen;
	scsi_path_t           path;
	sdm_handle_t         *h_entry;
	sdm_devDBRec_t       *devRec;
	scsi_readReqMsg_t    scsi_o_msg;
	sdm_readDataReqMsg_t *sdm_i_msg;

	sdm_i_msg = (sdm_readDataReqMsg_t *)mdesc->msg;

	// Translate handle into a device path.
	h_entry  = (sdm_handle_t *)sdm_getHandleEntry( sdm_i_msg->handle );
	if( !h_entry || !(h_entry->dev) ) {
		sdm_readDataRspMsg_t sdm_o_msg;
		sdm_o_msg.hdr.mid	  = SDM_MID_READ_RSP;
		sdm_o_msg.hdr.chandle = mdesc->srcHandle;
		sdm_o_msg.eCode       = SDM_FAIL;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_readDataRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return 0;
	}

	path   = h_entry->path;
	devRec = h_entry->dev;

	// A reply from the SCSI application is expected so setup the
	// cHandle and stash away info from the original open request so
	// that a reply can be associated with the proper device record.
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;
	tid = sdm_allocTransaction(devRec, mdesc);
				
	// Setup SCSI write msg
	mlen = sizeof(scsi_readReqMsg_t); 

	memset(&scsi_o_msg, 0, mlen);
	scsi_o_msg.hdr.mid     = SCSI_MID_READ_REQ;
	scsi_o_msg.hdr.cHandle = tid;
	scsi_o_msg.hdr.path    = path;
	scsi_o_msg.blkOffset   = sdm_i_msg->blkOffset;
	scsi_o_msg.blkCnt      = sdm_i_msg->blkCnt;
	scsi_o_msg.attributes  = 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_NO_REORDER) ? SCSI_MSG_ATTR_NO_REORDER : 0) | 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_PHY_OP) ? SCSI_MSG_ATTR_PHY_OP : 0) | 
		((sdm_i_msg->attributes & SDM_MSG_ATTR_KICK_WRITE_GATE) ? SCSI_MSG_ATTR_KICK_WRITE_GATE : 0) |
		SCSI_MSG_ATTR_NO_GUAR_DELIVERY;
	scsi_o_msg.dataDest    = sdm_i_msg->dataDest;

	/* LIODEBUG
	printf("SCSI_MID_READ_REQ: cHandle 0x%x, path 0x%x, transSeqId 0x%x\n",
	       scsi_o_msg->hdr.cHandle,
	       scsi_o_msg->hdr.path,
	       scsi_o_msg->transSeqId);
	*/

	// Send read command and data to the SCSI application.
	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
	sdm_sendMsg( (char *)&scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);

#if SDM_DEBUG_TX_MSG
	printf("sdm_procReadReq: tid: %x, path: %x, s_blk: %d, blk_cnt: %d\n", 
		   tid, path, scsi_o_msg.blkOffset, scsi_o_msg.blkCnt);
#endif

	return SDM_OK;
} /* sdm_procReadReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procScsiReadRsp
 *
 * Description: 	Handles a read data response message sent back by the
 *					SCSI application
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 28, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procScsiReadRsp(sdm_msgDesc_t *mdesc )
{
	int					 i, dlen, rspLen;
	char                 *tmp;
	uint32				 tid;
	uint64				 cod;
	sdm_msgDesc_t		 *rspInfo;
	sdm_devDBRec_t       *devRec;
	scsi_readRspMsg_t    *scsi_i_msg;
	sdm_readDataRspMsg_t *sdm_o_msg; 
	uint32				 scsi_eCode;
	sdm_mp_desc_t		 *smpd;
	uint32				 transSeqId;
	char				 *freeBuf = NULL;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procScsiReadRsp\n");
#endif

	scsi_i_msg = (scsi_readRspMsg_t *)mdesc->msg;
	/* LIODEBUG
	printf("SCSI_MID_READ_RSP: cHandle 0x%x, path 0x%x, transSeqId 0x%x, "
		   "eCode 0x%x, codHandle 0x%llx, dataLen %d\n", 
		   scsi_i_msg->hdr.cHandle,
		   scsi_i_msg->hdr.path,
		   scsi_i_msg->transSeqId,
		   scsi_i_msg->eCode,
		   scsi_i_msg->codHandle,
		   scsi_i_msg->dataLen);
	*/
	tid = scsi_i_msg->hdr.cHandle;
	rspInfo = &sdmDispatchedMsgList[tid];
	scsi_eCode = scsi_i_msg->eCode;

	if( (transSeqId = scsi_i_msg->transSeqId) ) { 
			if (SCSI_SEQUENCE_MAX(transSeqId) == 0) {
				/* Invalid SEQUENCE MAX, reject request */
				//printf("sdm_procScsiReadRsp: Invalid Sequence Max for READ response; rejecting.\n");
				return NFX_OK;
			}
			if (SCSI_SEQUENCE_ID(transSeqId) == 0) {
				/* Invalid SEQUENCE ID, reject request */
				//printf("sdm_procScsiReadRsp: Invalid Sequence Id for READ response; rejecting.\n");
				return NFX_OK;
			}
			
			/* First packet in multi-packet sequence ? */
			if (SCSI_SEQUENCE_ID(transSeqId) == 1) {
				/* Find unallocated SMP entry */
				for (i = 0; i < SDM_MAX_MPS; ++i)
					if (smps[i].transactionId == 0)
						break;
				if (i == SDM_MAX_MPS) {
					/* Cannot find unused SMP entry */
					//printf("sdm_procScsiReadRsp: Failed to find unallocated SMP entry; rejecting.\n");
					return NFX_OK; /* XXX need to finish up */
				}
				smpd = &smps[i];
				smpd->transactionId = SCSI_TRANSACTION_ID(transSeqId);
				smpd->maxSequenceId = SCSI_SEQUENCE_MAX(transSeqId);
				smpd->lastSequenceId = SCSI_SEQUENCE_ID(transSeqId);
				smpd->firstCODHandle = scsi_i_msg->codHandle;
				/* Allocate "estimated" full buffer size; include space for header */
				smpd->firstBufP = (byte *)malloc(SCSI_SEQUENCE_MAX(transSeqId) * (EEE_MAX_BUFS-1) * EEE_BUFSIZE_LARGE);
				if(smpd->firstBufP == NULL) {
					sdm_o_msg = (sdm_readDataRspMsg_t *)malloc(sizeof(sdm_readDataRspMsg_t));
					memset((void *)sdm_o_msg, 0, sizeof(sdm_readDataRspMsg_t));
					sdm_o_msg->hdr.mid = SDM_MID_READ_RSP;
					sdm_o_msg->eCode = SDM_FAIL;
					sdm_o_msg->hdr.chandle = rspInfo->srcHandle;
					rspLen = sizeof(sdm_readDataRspMsg_t); 
					smpd->transactionId = 0;
					goto cleanup;
				}
				smpd->firstBufferOffset = 64;
				smpd->firstBufP += smpd->firstBufferOffset;
				smpd->lastBufP = smpd->firstBufP;
				smpd->totalDataLen = 0;
			}
			else {
				/* Lookup existing SMP entry */
				for (i = 0; i < SDM_MAX_MPS; ++i)
					if (smps[i].transactionId == SCSI_TRANSACTION_ID(transSeqId))
						break;
				if (i == SDM_MAX_MPS) { 
				    //printf("sdm_procScsiReadRsp: Cannot find SMP entry; First packet dropped?; aborting.\n");
				    return NFX_OK; /* XXX need to finish up abort */
				}
				smpd = &smps[i];				
				smpd->lastSequenceId = SCSI_SEQUENCE_ID(transSeqId);
			}

			/* Copy data into local buffer */
			tmp = (char *)scsi_i_msg;
			tmp += sizeof(scsi_readRspMsg_t);
			memcpy(smpd->lastBufP, tmp, scsi_i_msg->dataLen);
			smpd->lastBufP += scsi_i_msg->dataLen;
			smpd->totalDataLen += scsi_i_msg->dataLen;

			/* Last packet in multi-packet sequence ? */
			if (smpd->lastSequenceId < smpd->maxSequenceId) {
				return NFX_OK;
			}
			else {
				dlen = smpd->totalDataLen;
				cod = smpd->firstCODHandle;

				/*
				 * The alloced data buffer has already been allocated
				 * large enough to accomodate the message header.  
				 */
				freeBuf = smpd->firstBufP - smpd->firstBufferOffset;
				tmp = smpd->firstBufP;
				sdm_o_msg = (sdm_readDataRspMsg_t *)(tmp-(sizeof(sdm_readDataRspMsg_t)));

				/* Deallocate SMP entry */
				smpd->transactionId = 0;
			}
	} /* if (transSeqId = scsi_i_msg->transSeqId) */
	else {
		dlen    = scsi_i_msg->dataLen;
		cod     = scsi_i_msg->codHandle;

		/*
		 * To avoid copying the data bytes contained in the response
		 * message. The sdm_readDataRspMsg header will overwrite the
		 * scsi_readRspMsg header. This is possible because the scsi
		 * rsp header is larger than the sdm header.  
		 */
		memset(scsi_i_msg, 0, sizeof(scsi_readRspMsg_t));
		tmp       = (char *)scsi_i_msg;
		tmp       += sizeof(scsi_readRspMsg_t);
		sdm_o_msg =  (sdm_readDataRspMsg_t *)(tmp-(sizeof(sdm_readDataRspMsg_t)));
	}

	rspLen = sizeof(sdm_readDataRspMsg_t) + dlen; 
	
	devRec = rspInfo->dev;
	if( !devRec ) {
		sdm_o_msg->eCode       = SDM_FAIL;
	}
	else {
		sdm_o_msg->eCode       = (scsi_eCode == SCSI_MSG_ECODE_OK) ? SDM_OK : scsi_eCode;
	}

	// Send response back to the application that made the read request.
	sdm_o_msg->hdr.mid     = SDM_MID_READ_RSP;
	sdm_o_msg->hdr.chandle = rspInfo->srcHandle;
	sdm_o_msg->dataLen     = dlen; 
	sdm_o_msg->codHandle   = cod;

cleanup:
	sdm_sendMsg( (char *)sdm_o_msg, rspLen, 
				 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);

	sdm_deallocTransaction(tid);

	if (freeBuf)
		free(freeBuf);

	return SDM_OK;
} /* sdm_procScsiReadRsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procRelayReq
 *
 * Description: 	Handles a read data request message being sent to a 
 *					SCSI device via the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	July 26, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procRelayReq(sdm_msgDesc_t *mdesc )
{
	uint32                tid, destCpu, destSlot, mlen, dataLen, totalDataLen;
	char                 *msgData;
	scsi_path_t           path;
	sdm_handle_t         *h_entry;
	sdm_devDBRec_t       *devRec;
	scsi_relayReqMsg_t   *scsi_o_msg;
	sdm_relayCdbReqMsg_t *sdm_i_msg;
	char				 *in_p;
	uint32				  seqId, seqMax, transId;

	sdm_i_msg = (sdm_relayCdbReqMsg_t *)mdesc->msg;

	// Translate handle into a device path.

	h_entry  = (sdm_handle_t *)sdm_getHandleEntry( sdm_i_msg->handle );
	if( !h_entry || !(h_entry->dev) ) {
		sdm_relayCdbRspMsg_t sdm_o_msg;
		bzero(&sdm_o_msg, sizeof(sdm_relayCdbRspMsg_t));
		sdm_o_msg.hdr.mid	  = SDM_MID_RELAY_CDB_RSP;
		sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;
		sdm_o_msg.eCode       = SDM_FAIL;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_relayCdbRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return 0;
	}

	path   = h_entry->path;
	devRec = h_entry->dev;

	// A reply from the SCSI application is expected so setup the
	// cHandle and stash away info from the original open request so
	// that a reply can be associated with the proper device record.
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;
	tid = sdm_allocTransaction(devRec, mdesc);

	if (sdm_i_msg->dataLen) {
		/* Data OUT request */
		totalDataLen = sdm_i_msg->dataLen;
		in_p = sdm_i_msg->data;
		transId = sdm_getTransactionId();
		seqMax = (totalDataLen / MAX_PACKET_DATA) + ((totalDataLen % MAX_PACKET_DATA) ? 1 : 0);
		seqId = 1;

		scsi_o_msg = (scsi_relayReqMsg_t *)malloc(sizeof(scsi_relayReqMsg_t) - 1 + MAX_PACKET_DATA);
		if (!scsi_o_msg)
			return SDM_FAIL;
		
		memset(scsi_o_msg, 0, sizeof(scsi_relayReqMsg_t));
		scsi_o_msg->hdr.mid       = SCSI_MID_RELAY_REQ;
		scsi_o_msg->hdr.cHandle   = tid;
		scsi_o_msg->hdr.path      = path;
		scsi_o_msg->lun           = SCSI_PATH_GET_LUN(path);
		scsi_o_msg->retryCnt      = sdm_i_msg->retryCnt;
		scsi_o_msg->timeOutCnt    = sdm_i_msg->timeOutCnt;
		scsi_o_msg->maxTimeToLive = sdm_i_msg->maxTimeToLive;
		scsi_o_msg->dataInLen     = 0;
		memcpy(scsi_o_msg->cdb, sdm_i_msg->cdb, 16);

		destCpu = SCSI_PATH_GET_PORT(path);
		destSlot = SCSI_PATH_GET_SLOT(path);
		
		while (totalDataLen) {
			dataLen = MIN(totalDataLen, MAX_PACKET_DATA);
			mlen = sizeof(scsi_relayReqMsg_t) - 1 + dataLen;
			scsi_o_msg->transSeqId  = SCSI_MSG_TRANSSEQID_PACK(transId, seqMax, seqId);

			msgData = ((char *)scsi_o_msg) + sizeof(scsi_relayReqMsg_t) - 1;
			memcpy(msgData, in_p, dataLen); 
			scsi_o_msg->dataLen = dataLen;

			/* LIODEBUG
			printf("SCSI_MID_RELAY_REQ: cHandle 0x%x, path 0x%x, transSeqId 0x%x\n",
				   scsi_o_msg->hdr.cHandle,
				   scsi_o_msg->hdr.path,
				   scsi_o_msg->transSeqId);
			*/

			// Send write command and data to the SCSI application.
			sdm_sendMsg((char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0, 0);
			
			in_p += dataLen;
			totalDataLen -= dataLen;
			++seqId;
		}
	}
	else {
		/* Data IN request */
		mlen = sizeof(scsi_relayReqMsg_t);
		scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);

		memset(scsi_o_msg, 0, mlen);
		scsi_o_msg->hdr.mid       = SCSI_MID_RELAY_REQ;
		scsi_o_msg->hdr.cHandle   = tid;
		scsi_o_msg->hdr.path      = path;
		scsi_o_msg->lun           = SCSI_PATH_GET_LUN(path);
		scsi_o_msg->retryCnt      = sdm_i_msg->retryCnt;
		scsi_o_msg->timeOutCnt    = sdm_i_msg->timeOutCnt;
		scsi_o_msg->maxTimeToLive = sdm_i_msg->maxTimeToLive;
		scsi_o_msg->dataInLen     = sdm_i_msg->dataInLen;
		memcpy(scsi_o_msg->cdb, sdm_i_msg->cdb, 16);

		/* LIODEBUG
		printf("SCSI_MID_RELAY_REQ: cHandle 0x%x, path 0x%x, transSeqId 0x%x\n",
			   scsi_o_msg->hdr.cHandle,
			   scsi_o_msg->hdr.path,
			   scsi_o_msg->transSeqId);
		*/

		// Send read command and data to the SCSI application.
		destCpu = SCSI_PATH_GET_PORT(path);
		destSlot = SCSI_PATH_GET_SLOT(path);
		sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
	}
				
	free(scsi_o_msg);

#if SDM_DEBUG_TX_MSG
	printf("sdm_procRelayReq: tid: %x, path: %x\n", 
		   tid, path);
#endif
	return SDM_OK;
} // sdm_procRelayReq 

//------------------------------------------------------------------------------
// Function Name:	sdm_procScsiRelayRsp
//
// Description: 	Handles a read data response message sent back by the
//					SCSI application
//
// Created by:		Frank Salzmann
//
// Data Created:	July 28, 2001
//
// Returns:         SDM_OK
//
//------------------------------------------------------------------------------
int
sdm_procScsiRelayRsp(sdm_msgDesc_t *mdesc )
{
	int					  i;
	uint32				  tid;
	sdm_msgDesc_t *		  rspInfo;
	sdm_devDBRec_t *	  devRec;
	scsi_relayRspMsg_t *  scsi_i_msg;
	scsi_relayRspMsg_t *  i_scsi_i_msg;
	uint32				  i_scsi_i_hdr_len;
	sdm_relayCdbRspMsg_t *sdm_o_msg; 
	int32				  sdm_o_hdr_len;
	int32				  sdm_o_msg_len;
	sdm_mp_desc_t *		  smpd;
	uint32				  transSeqId;
	int					  reassembled_msg_len;
	int					  reassembled_msg_offset;
	char *				  reassembled_msg_p;
	char *				  reassembled_msg_base = NULL;
	

#if SDM_DEBUG_RX_MSG
	printf("sdm_procScsiRelayRsp\n");
#endif

	scsi_i_msg = (scsi_relayRspMsg_t *)mdesc->msg;

#if 0 //-- LIODEBUG
	 printf("SCSI_MID_RELAY_RSP: cHandle 0x%x, path 0x%x, transSeqId 0x%x, "
			"status 0x%x, dataLen %d\n", 
			scsi_i_msg->hdr.cHandle,
			scsi_i_msg->hdr.path,
			scsi_i_msg->transSeqId,
			scsi_i_msg->status,
			scsi_i_msg->dataLen);
#endif //-- LIODEBUG

	if( (transSeqId = scsi_i_msg->transSeqId) ) { 
		if (SCSI_SEQUENCE_MAX(transSeqId) == 0) {
			/* Invalid SEQUENCE MAX, reject request */
			// printf("sdm_procScsiRelayRsp: Invalid Sequence Max for READ response; rejecting.\n");
			return NFX_OK;
		}
		if (SCSI_SEQUENCE_ID(transSeqId) == 0) {
			/* Invalid SEQUENCE ID, reject request */
			// printf("sdm_procScsiRelayRsp: Invalid Sequence Id for READ response; rejecting.\n");
			return NFX_OK;
		}
			
		/* First packet in multi-packet sequence ? */
		if (SCSI_SEQUENCE_ID(transSeqId) == 1) {
			/* Find unallocated SMP entry */
			for (i = 0; i < SDM_MAX_MPS; ++i)
				if (smps[i].transactionId == 0)
					break;
			if (i == SDM_MAX_MPS) {
				/* Cannot find unused SMP entry */
				// printf("sdm_procScsiRelayRsp: Failed to find unallocated SMP entry; rejecting.\n");
				return NFX_OK; /* XXX need to finish up */
			}
			smpd = &smps[i];
			smpd->transactionId = SCSI_TRANSACTION_ID(transSeqId);
			smpd->maxSequenceId = SCSI_SEQUENCE_MAX(transSeqId);
			smpd->lastSequenceId = SCSI_SEQUENCE_ID(transSeqId);
			/* Allocate "estimated" full buffer size; include space for header */
			smpd->firstBufP = (byte *)malloc(SCSI_SEQUENCE_MAX(transSeqId) * (EEE_MAX_BUFS-1) * EEE_BUFSIZE_LARGE);
			if(smpd->firstBufP == NULL) {
				sdm_o_msg = (sdm_relayCdbRspMsg_t *)malloc(sizeof(sdm_relayCdbRspMsg_t));
				memset((void *)sdm_o_msg, 0, sizeof(sdm_relayCdbRspMsg_t));
				sdm_o_msg->hdr.mid = SDM_MID_RELAY_CDB_RSP;
				sdm_o_msg->eCode = SDM_FAIL;
				sdm_o_msg_len = sizeof(sdm_relayCdbRspMsg_t);
				smpd->transactionId = 0;
				goto cleanup;
			}
			smpd->firstBufferOffset = 128;
			smpd->firstBufP += smpd->firstBufferOffset;
			smpd->lastBufP = smpd->firstBufP;
			smpd->totalDataLen = 0;
		}
		else {
			/* Lookup existing SMP entry */
			for (i = 0; i < SDM_MAX_MPS; ++i)
				if (smps[i].transactionId == SCSI_TRANSACTION_ID(transSeqId))
					break;
			if (i == SDM_MAX_MPS) { 
				// printf("sdm_procScsiRelayRsp: Cannot find SMP entry; First packet dropped?; aborting.\n");
				return NFX_OK; /* XXX need to finish up abort */
			}
			smpd = &smps[i];				
			smpd->lastSequenceId = SCSI_SEQUENCE_ID(transSeqId);
		}
			
		/* Copy data into local buffer */
		memcpy(smpd->lastBufP, scsi_i_msg->data, scsi_i_msg->dataLen);
		smpd->lastBufP += scsi_i_msg->dataLen;
		smpd->totalDataLen += scsi_i_msg->dataLen;

		/* Last packet in multi-packet sequence ? */
		if (smpd->lastSequenceId < smpd->maxSequenceId) {
			return NFX_OK;
		}
		else {
			reassembled_msg_len = smpd->totalDataLen;
			reassembled_msg_base = smpd->firstBufP - smpd->firstBufferOffset;
			reassembled_msg_p = smpd->firstBufP;
			reassembled_msg_offset = smpd->firstBufferOffset;
				
			/* Deallocate SMP entry */
			smpd->transactionId = 0;
		}
	} /* if (transSeqId = scsi_i_msg->transSeqId) */
	else {
		reassembled_msg_len = scsi_i_msg->dataLen;
		reassembled_msg_offset = 128;
		reassembled_msg_base = malloc(reassembled_msg_len + reassembled_msg_offset);
		reassembled_msg_p = reassembled_msg_base + reassembled_msg_offset;
		memset(reassembled_msg_base, 0, reassembled_msg_offset);
		memcpy(reassembled_msg_p, scsi_i_msg->data, scsi_i_msg->dataLen);
	}
	
	tid = scsi_i_msg->hdr.cHandle;
	
	/* 
	 * Is this in internal relay request made by the SDM device
	 * initialization code ? 
	 */
	if (tid > SDM_TRANSACTION_MAX ) {
		tid &= ~0x10000000;
		rspInfo = &sdmDispatchedMsgList[tid];
		
		/* 
		 * Construct an amulgumated SCSI_MID_RELAY_RSP consisting
		 * essentially of the header of the last fragment (they're all
		 * the same, except for the transactionID). The data length
		 * with be the reassembled length.  
		 */

		NFX_ASSERT(sizeof(scsi_relayRspMsg_t) <= reassembled_msg_offset);

		/* hack hack hack -- compute actual length of header, since struct is not packed */
		i_scsi_i_msg = NULL;
		i_scsi_i_hdr_len = (int32)(&(i_scsi_i_msg->data));

		i_scsi_i_msg = (scsi_relayRspMsg_t *)(reassembled_msg_p - i_scsi_i_hdr_len);
		memcpy(i_scsi_i_msg, scsi_i_msg, i_scsi_i_hdr_len);
		i_scsi_i_msg->dataLen = reassembled_msg_len;
		
		/* 
		 * Jump to the callback function that will process this
		 * response.  
		 */
		rspInfo->callBackFunc(rspInfo->callBackData, i_scsi_i_msg);
	}
	else {

		rspInfo = &sdmDispatchedMsgList[tid];
		devRec = rspInfo->dev;

		NFX_ASSERT(sizeof(sdm_relayCdbRspMsg_t) <= reassembled_msg_offset);
		sdm_o_msg_len = sizeof(sdm_relayCdbRspMsg_t) + reassembled_msg_len;
	
		/* hack hack hack -- compute actual length of header, since struct is not packed */
		sdm_o_msg = NULL;
		sdm_o_hdr_len = (int32)(&(sdm_o_msg->data));

		sdm_o_msg = (sdm_relayCdbRspMsg_t *)(reassembled_msg_p - sdm_o_hdr_len); 

		/* 
		 * Send response back to the application that made the relay
		 * request. 
		 */
		sdm_o_msg->eCode = (devRec) ? SDM_OK : SDM_FAIL;
		sdm_o_msg->hdr.mid = SDM_MID_RELAY_CDB_RSP;
		sdm_o_msg->hdr.chandle = rspInfo->srcHandle;
		sdm_o_msg->status = scsi_i_msg->status;
		memcpy(sdm_o_msg->senseData, scsi_i_msg->senseData, 32);
		sdm_o_msg->dataLen = reassembled_msg_len;
cleanup:
		sdm_sendMsg( (char *)sdm_o_msg, sdm_o_msg_len,
					 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);
	}

	sdm_deallocTransaction(tid);

	if (reassembled_msg_base)
		free(reassembled_msg_base);

	return SDM_OK;
} /* sdm_procScsiRelayRsp */

int 
sdm_procSCSIDevResetRsp(sdm_msgDesc_t *mdesc)
{
	scsi_devResetRspMsg_t *		scsi_i_msg;
	sdm_devResetRspMsg_t *		sdm_o_msg; 
	uint32						tid;
	sdm_msgDesc_t *				req_context;

	scsi_i_msg = (scsi_devResetRspMsg_t *)mdesc->msg;
	tid = sdm_ntohl(scsi_i_msg->hdr.cHandle);
	req_context = &sdmDispatchedMsgList[tid];
	
	sdm_o_msg = (sdm_devResetRspMsg_t *)malloc(sizeof(sdm_devResetRspMsg_t));
	bzero(sdm_o_msg, sizeof(sdm_devResetRspMsg_t));

	sdm_o_msg->hdr.mid = SDM_MID_DEV_RESET_RSP;
	sdm_o_msg->hdr.chandle = req_context->srcHandle;
	sdm_o_msg->eCode = sdm_ntohl(scsi_i_msg->eCode);
	sdm_sendMsg((char *)sdm_o_msg, sizeof(sdm_devResetRspMsg_t),
				req_context->srcApp, req_context->srcCpu, req_context->srcSlot, 
				0, 0, 0);
	free(sdm_o_msg);
	sdm_deallocTransaction(tid);
	return SDM_OK;
}

int 
sdm_procSCSIDevStatusRsp(sdm_msgDesc_t *mdesc)
{
	scsi_devStatusRspMsg_t *	scsi_i_msg;
	uint32						tid;
	sdm_msgDesc_t *				req_context;
	sdm_devDBRec_t *            devRec;
	sdm_devStatusRspMsg_t 		sdm_o_msg; 
	uint32						i;

	scsi_i_msg = (scsi_devStatusRspMsg_t *)mdesc->msg;
	tid = sdm_ntohl(scsi_i_msg->hdr.cHandle);
	req_context = &sdmDispatchedMsgList[tid];

	memset(&sdm_o_msg, 0, sizeof(sdm_devStatusRspMsg_t));
	sdm_o_msg.hdr.mid = SDM_MID_DEV_STATUS_RSP;
    sdm_o_msg.hdr.chandle = req_context->srcHandle;
    devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, scsi_i_msg->scsi_id);
    if (!devRec) {
        sdm_o_msg.eCode = SDM_FAIL;
        sdm_sendMsg((char *)&sdm_o_msg, sizeof(sdm_devStatusRspMsg_t),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
    }
	else {
		sdm_o_msg.hdr.mid    = SDM_MID_DEV_STATUS_RSP;
		sdm_o_msg.hdr.chandle = req_context->srcHandle;
		sdm_o_msg.eCode      = SDM_OK;
		sdm_o_msg.wwn        = devRec->wwn;
		sdm_o_msg.state      = devRec->state;
		sdm_o_msg.activePath = sdm_getActivePath(devRec);
		sdm_o_msg.pathCnt    = devRec->pathCnt;
		sdm_o_msg.handleCnt  = devRec->handleCnt;
		sdm_o_msg.devType    = devRec->devType;
		sdm_o_msg.blockCnt   = devRec->lbCnt;
		sdm_o_msg.blockSize  = devRec->lbSize;
		memcpy((void *)&sdm_o_msg.st, &devRec->st, sizeof(sdm_o_msg.st));
		for (i = 0; i < SDM_MAX_PATHS; i++) {
			sdm_o_msg.paths[i].path  = devRec->pathList[i].path;
			sdm_o_msg.paths[i].state = devRec->pathList[i].state;
			sdm_o_msg.paths[i].wwn   = devRec->pathList[i].wwn;
			sdm_o_msg.paths[i].lun   = devRec->pathList[i].lun;
		}

		for (i = 0; i < SDM_MAX_HANDLES_PER_DEV; i++)
			sdm_o_msg.handles[i] = devRec->openHandles[i];

		if (scsi_i_msg->eCode == SCSI_MSG_ECODE_OK) {
			sdm_o_msg.tape_position_eom = scsi_i_msg->tape_position_eom;
			sdm_o_msg.tape_position_bom = scsi_i_msg->tape_position_bom;
			sdm_o_msg.tape_position_blockno = scsi_i_msg->tape_position_blockno;
			sdm_o_msg.tape_position_fileno = scsi_i_msg->tape_position_fileno;
		}
		else {
			sdm_o_msg.tape_position_eom = 0;
			sdm_o_msg.tape_position_bom = 0;
			sdm_o_msg.tape_position_blockno = -1;
			sdm_o_msg.tape_position_fileno = -1;
		}

		sdm_sendMsg((char *)&sdm_o_msg, sizeof(sdm_devStatusRspMsg_t),
					req_context->srcApp, req_context->srcCpu, req_context->srcSlot, 
					0, 0, 0);
	}
	sdm_deallocTransaction(tid);
	return SDM_OK;
}

int 
sdm_procSCSIIoctlRsp(sdm_msgDesc_t *mdesc)
{
	scsi_ioctlRspMsg_t *	scsi_i_msg;
	sdm_ioctlRspMsg_t *		sdm_o_msg; 
	uint32					tid;
	sdm_msgDesc_t *			req_context;
	uint32					mlen;

	scsi_i_msg = (scsi_ioctlRspMsg_t *)mdesc->msg;
	tid = sdm_ntohl(scsi_i_msg->hdr.cHandle);
	req_context = &sdmDispatchedMsgList[tid];
	
	mlen = sizeof(sdm_ioctlRspMsg_t) + scsi_i_msg->dataLen;
	sdm_o_msg = (sdm_ioctlRspMsg_t *)malloc(mlen);
	bzero(sdm_o_msg, sizeof(sdm_ioctlRspMsg_t));

	sdm_o_msg->hdr.mid = SDM_MID_IOCTL_RSP;
	sdm_o_msg->hdr.chandle = req_context->srcHandle;
	sdm_o_msg->eCode = scsi_i_msg->eCode;
	sdm_o_msg->dataLen = scsi_i_msg->dataLen;
	bcopy(scsi_i_msg->data, sdm_o_msg->data, scsi_i_msg->dataLen);
	sdm_sendMsg((char *)sdm_o_msg, sizeof(sdm_ioctlRspMsg_t) + sdm_o_msg->dataLen,
				req_context->srcApp, req_context->srcCpu, req_context->srcSlot, 
				0, 0, 0);
	free(sdm_o_msg);
	sdm_deallocTransaction(tid);
	return SDM_OK;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procSacWriteReq
 *
 * Description: 	Handles a SAC data write request message being sent to a 
 *					SCSI device via the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	September 12, 2001
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procSacWriteReq(sdm_msgDesc_t *mdesc )
{
	uint32                 		tid;
	uint32						mlen, scsi_port, sdmAddr;
	scsi_path_t            		path;
	sdm_handle_t          		*h_entry;
	sdm_devDBRec_t        		*devRec;
	sdm_sacWriteReqMsg_t  		*sdm_i_msg;
	sbm_sendControlAndSACData_t *sac_writeCmd;
	scsi_writeReqMsg_t          scsi_msg;

	sdm_i_msg = (sdm_sacWriteReqMsg_t *)mdesc->msg;

	// Translate handle into a device path.
	h_entry  = (sdm_handle_t *)sdm_getHandleEntry( sdm_i_msg->handle );
	if( !h_entry || !(h_entry->dev) ) {
		sdm_writeDataRspMsg_t sdm_o_msg;
		sdm_o_msg.hdr.mid	  = SDM_MID_WRITE_RSP;
		sdm_o_msg.hdr.chandle = mdesc->srcHandle;
		sdm_o_msg.eCode       = SDM_FAIL;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_writeDataRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return 0;
	}

	devRec = h_entry->dev;
	path   = h_entry->path;

	// A reply from the SCSI application is expected so setup the
	// tid and stash away info from the original open request so
	// that a reply can be associated with the proper device record.
	mdesc->srcHandle = sdm_i_msg->hdr.chandle;
	tid = sdm_allocTransaction(devRec, mdesc);
				
	// Setup SAC write msg
	mlen = sizeof(scsi_writeReqMsg_t) + sizeof(sbm_sendControlAndSACData_t);;
	sac_writeCmd = (sbm_sendControlAndSACData_t *)malloc( mlen ); 
	memset(sac_writeCmd, 0, mlen);

	// Setup the MCPU write command msg.
	sac_writeCmd->cmd = CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM;
	sac_writeCmd->TID = 0x65656565;
	sac_writeCmd->COD = sdm_i_msg->cod;

	scsi_port = PACK_UNICAST_PORT_ADDRESS( 2,
										   3,
										   7,
										   EEE_USER_DATA,
										   NFX_COS_HIGH,
										   0);
	
	sdmAddr = PACK_UNICAST_PORT_ADDRESS(0,
										0,
										65,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0);

	sac_writeCmd->dest_port   = scsi_port;
	sac_writeCmd->source_port = sdmAddr;
	sac_writeCmd->length = devRec->lbSize * sdm_i_msg->blkCnt;
	sac_writeCmd->offset = 0;
	sac_writeCmd->msg_len = sizeof(scsi_writeReqMsg_t);

	// Setup the SCSI write Cmd.
	scsi_msg.hdr.mid     = SCSI_MID_WRITE_REQ;
	scsi_msg.hdr.cHandle = sdm_i_msg->hdr.chandle;
	scsi_msg.hdr.path    = path;
	scsi_msg.blkOffset   = sdm_i_msg->blkOffset;
	scsi_msg.blkCnt      = sdm_i_msg->blkCnt;
	scsi_msg.transSeqId  = 0;
	memcpy(&(sac_writeCmd->msg), (char *)&scsi_msg, sizeof(scsi_writeReqMsg_t));

	// Send SAC write command to the MCPU
	sdm_sendMsg( (char *)sac_writeCmd, mlen, "sbm", NFX_NFP_TXRX1, 1, 0, 0,0);

#if SDM_DEBUG_TX_MSG
	printf("sdm_procWriteReq: tid: %x, path: %x, s_blk: %d, blk_cnt: %d\n", 
		   tid, path, sdm_i_msg->blkOffset, sdm_i_msg->blkCnt);
#endif
	
	free(sac_writeCmd);
	return SDM_OK;
} /* sdm_procSacWriteReq */

uint64
sdm_swap64(uint64 val)
{
#if 0
	int i, j;
	sdm_64_t in;
	sdm_64_t out;

#if NFX_SOCKET_BACKPLANE
	in.w = val;
	for(i=0, j=7; i < 8; i++, j--)
		out.b[i] = in.b[j];
	//printf("before: %llx, after: %llx\n", val, out.w);
	return(out.w);
#else
	return(val);
#endif
#endif
	return(val);

} // sdm_swap64

/*
 * sdm_getTransactionId - 
 *
 * The transaction ID should be unique across all SCSI command
 * originators. For now, since the only geneator of "large" I/Os will
 * be the SDM, we can limit the generation of the transaction ID
 * appropriately.  
 */
ushort16
sdm_getTransactionId(void)
{
	static ushort16 transactionId = 1;

	if (++transactionId == 0)
		transactionId = 1;
	return(transactionId);
}

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendSyncReq
 *
 * Description: 	Send device synchronization request to SCSI application.
 *					Which will result in the SCSI application sending 
 *					sdm_addDevMsg_t to the SDM for each device in the 
 *					SCSI application device list. The purpose of this message 
 *					is to allow the SDM to resychronize with the SCSI 
 *					application following a shutdown of the SDM.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 18, 2002
 *
 * Returns:        	NFX_OK 
 *
 *------------------------------------------------------------------------------
 */
int
sdm_sendSyncReq( uint32 destSlot, uint32 destCpu )
{
	scsi_syncReqMsg_t scsi_o_msg;

	memset(&scsi_o_msg, 0, sizeof(scsi_syncReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_SYNC_REQ;
	scsi_o_msg.hdr.path    = 0;
	scsi_o_msg.hdr.cHandle = 0;
				
#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiSyncReq: destCpu: %d, destSlot: %d\n", 
		   destCpu, destSlot);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_syncReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	return NFX_OK;
} // sdm_sendSyncReq 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procDelAllReqRsp
 *
 * Description: 	Handles request made by SCSI to remove all device
 *					paths for a SP_NIM slot/port.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 30, 2002
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procDelAllReq(sdm_msgDesc_t *mdesc )
{
	uint32				  slot, port;
	sdm_delAllReqMsg_t   *sdm_i_msg;

	sdm_i_msg = (sdm_delAllReqMsg_t *)mdesc->msg;
	
	E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
		   ("SDM: SCSI Reset: Delete All: Slot: %d, Port: %d", sdm_i_msg->slot, 
			sdm_i_msg->port));
	
	slot = sdm_i_msg->slot;
	port = sdm_i_msg->port;

	sdm_delAll( slot, port );
	return SDM_OK;
} /* sdm_procDelAllReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_delAll
 *
 * Description: 	Handles request made by SCSI to remove all device
 *					paths for a SP_NIM slot/port.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	January 30, 2002
 *
 * Returns:         SDM_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_delAll(uint32 slot, uint32 port)
{
	uint32				  i, j, devCnt, pathCnt;
	uint64				  prevId = 0;
	uint64                id     = 0;
	scsi_path_t			  path;
	sdm_handle_t         *sdm_h_entry;
	sdm_devDBRec_t       *devRec;
	
	devCnt = sdm_dbGetNumEntries( sdmDevDB, SDM_DEVLIST_ALL, SDM_DEVLIST_PORTALL, NULL );

	for( i=0; i < devCnt; i++ ) {

		devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
		if( devRec ) {
			id = devRec->id;
			pathCnt = devRec->pathCnt;	
			for(j=0; j < pathCnt; j++) {
				path = devRec->pathList[j].path;
				if( SCSI_PATH_GET_SLOT(path) == slot &&
					SCSI_PATH_GET_PORT(path) == port ) {
					sdm_removePath( devRec, path );
					
					//-- If this was the only path to the device, 
					//-- the it is time to delete the device record.
					//-- But first we need to check if there are any
					//-- open handles. If so then we send a dev del
					//-- message to the evm and wait for it to send
					//-- a close request message back to us. Upon receiving
					//-- the close request we will delete the dev 
					//-- record. If there are no open handles then we
					//-- remove the device record right here.
					if( devRec->pathCnt <= 0 ) {

						if( devRec->handleCnt > 0 ) {    
							sdm_h_entry = (sdm_handle_t *)sdm_getHandleEntry( devRec->openHandles[0] );
							if( sdm_h_entry) {
								sdm_h_entry->dev = 0;
							}
						}

						sdm_sendDevDelNotify("evm", devRec);	

						E_LOG(class_1, notice_s, eee_getApplicationId("sdm"),
							  0, 0, 0,
							  ("SDM: Remove (delAll): 0x%016llx:%d", 
							   devRec->wwn, devRec->lun));

						sdm_dbRemRec( sdmDevDB, devRec->id );
						id = prevId; //-- Dev rec was destroyed so use prev
						             //-- Id to get next devRec.
					}
					break; //-- Only one path per slot/port so if we found it
					       //-- get out of here.
				}
			}
		}
		else 			//-- Should never happen. If it does we have a hole 
			break;      //-- in the dev list.

		prevId = id;
		// devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
	}
	return SDM_OK;
} // sdm_delAll

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procGetSdmHandleReq
 *
 * Description: 	Handles request to obtain the SDM_HANDLE for an open
 *					WWN/Lun.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 05, 2001
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procGetSdmHandleReq(sdm_msgDesc_t *mdesc)
{
	int32					 rc;
	uint32			  		 sdmHandle;
	sdm_getSdmHandleRspMsg_t  sdm_o_msg;
	sdm_getSdmHandleReqMsg_t *sdm_i_msg;

	sdm_i_msg = (sdm_getSdmHandleReqMsg_t *)mdesc->msg;

	if(sdm_i_msg->wwn)	
		sdmHandle = sdm_wwnToHandle( sdm_i_msg->wwn, sdm_i_msg->lun );
	else
		sdmHandle = sdm_nameToHandle (sdm_i_msg->devname);
	if( sdmHandle == 0 )
		rc = SDM_FAIL;

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid     = SDM_MID_GET_SDMHANDLE_RSP;
	sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;
	sdm_o_msg.rCode       = rc;
	sdm_o_msg.sdmHandle   = sdmHandle;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_getSdmHandleRspMsg_t), 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	return SDM_OK;
} // sdm_procGetSdmHandleReq 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procGetSCSIIDReq
 *
 * Description: 	Handles request to translate a WWN/LUN pair into a SCSI ID
 *
 * Created by:		Radek Aster
 *
 * Data Created:	March 19, 2002
 *
 * Returns:         NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sdm_procGetSCSIIDReq(sdm_msgDesc_t *mdesc)
{
	int32					 rc;
	scsi_id_t				 id;
	sdm_getSCSIIdRspMsg_t    sdm_o_msg;
	sdm_getSCSIIdReqMsg_t *  sdm_i_msg;

	sdm_i_msg = (sdm_getSCSIIdReqMsg_t *)mdesc->msg;

	//-- See if the WWN is the search key
	if( sdm_i_msg->wwn > 0 ) 
		id = sdm_wwnToID( sdm_i_msg->wwn, sdm_i_msg->lun );
	else  //-- WWN is null so use the device name.
		id = sdm_nameToID(sdm_i_msg->name);

	if (id == 0)
		rc = SDM_FAIL;
	else 
		rc = SDM_OK;

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid     = SDM_MID_GET_SCSIID_RSP;
	sdm_o_msg.hdr.chandle = sdm_i_msg->hdr.chandle;
	sdm_o_msg.rCode       = rc;
	sdm_o_msg.id          = id;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_getSCSIIdRspMsg_t), 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	return SDM_OK;
} // sdm_procGetSCSIIDReq 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_procStatsReq
 *
 * Description: 	Handles a get storage dev statistics request from an app.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 06, 2002
 *
 * Returns:         NFX_OK
 *------------------------------------------------------------------------------
 */
int
sdm_procStatsReq( sdm_msgDesc_t *mdesc )
{
	int32			  rc = SDM_FAIL;
	uint32			  tid;
	sdm_devDBRec_t    *devRec = NULL;
	sdm_getStatsReqMsg_t *sdm_i_msg; 
	sdm_getStatsRspMsg_t  sdm_o_msg;
	uint32			  port;
	scsi_id_t		  id = 0;
	int				  i;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procStatsReq: Enter\n");
#endif

	sdm_i_msg = (sdm_getStatsReqMsg_t *)mdesc->msg;

	// stats for a device
	if(sdm_i_msg->type == SDM_STATS_TYPE_DEV) {

		// get device
		if(sdm_i_msg->subtype & SDM_STATS_SUBTYPE_USE_WWN)
			devRec    = (sdm_devDBRec_t *)sdm_dbFindWwnLun(sdmDevDB, sdm_i_msg->wwn, sdm_i_msg->lun);
		else if(sdm_i_msg->subtype & SDM_STATS_SUBTYPE_USE_NAME)
			devRec	  = (sdm_devDBRec_t *)sdm_dbFindDevname(sdmDevDB, sdm_i_msg->devname);
		else if(sdm_i_msg->subtype & SDM_STATS_SUBTYPE_USE_ID)
			devRec	  = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, sdm_i_msg->id);

		// device not found !
		if( ! devRec ) goto error_reply;

		// if port is specified then use that port, or simply use open path
		if(sdm_i_msg->subtype & SDM_STATS_SUBTYPE_USE_PORT) {
			for(i=0; i < devRec->pathCnt; i++) {
				port = SCSI_PATH_GET_PORT(devRec->pathList[i].path);
				if(sdm_i_msg->port == port)
					break;
			}
			if(i == devRec->pathCnt) goto error_reply;
		}

		// port is not specified, so use the port on which device is open
		else {
			if(devRec->handleCnt <= 0) {
				rc = SDM_NOT_OPEN;
				goto error_reply;
			}
			port = SDM_HANDLE_GET_PORT(devRec->openHandles[0]);
		}

		id = devRec->id;
	}

	else if(sdm_i_msg->type == SDM_STATS_TYPE_SCSI) {
		port = sdm_i_msg->port;
	}

	else 
		goto error_reply;

	tid  = sdm_allocTransaction( devRec, mdesc );
	sdm_sendScsiStatsReq(id, 2 /* FC */, port, sdm_i_msg->type, tid);

	return SDM_OK;

error_reply:
	memset(&sdm_o_msg, 0, sizeof(sdm_getStatsRspMsg_t));

	// Send response back to the application that made the close request.
	sdm_o_msg.hdr.mid     = SDM_MID_GET_STATS_RSP;
	sdm_o_msg.eCode       = rc;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_getStatsRspMsg_t), 
				mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	return SDM_OK;
} // sdm_procStatsReq

/*
 *------------------------------------------------------------------------------
 * Function Name:	sdm_sendScsiStatsReq
 *
 * Description: 	Send request to SCSI application to get device statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 06, 2002
 *
 * Returns:        	tid 
 *------------------------------------------------------------------------------
 */
int
sdm_sendScsiStatsReq( scsi_id_t id, uint32 destSlot, uint32 destCpu, int type, uint32 tid )
{
	scsi_getStatsReqMsg_t scsi_o_msg;

	memset(&scsi_o_msg, 0, sizeof(scsi_getStatsReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_GET_STATS_REQ;
	scsi_o_msg.hdr.cHandle = tid;
	scsi_o_msg.id		   = id;
	if(type == SDM_STATS_TYPE_DEV)
		scsi_o_msg.type = SCSI_STATS_TYPE_DEV;
	else if(type == SDM_STATS_TYPE_SCSI) 
		scsi_o_msg.type = SCSI_STATS_TYPE_SCSI;	

#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiStatsReq: destCpu: %d, destSlot: %d\n", 
		   destCpu, destSlot);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_getStatsReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	return NFX_OK;
} // sdm_sendScsiStatsReq 

//------------------------------------------------------------------------------
// Function Name:	sdm_procScsiStatsRsp
//
// Description: 	Handles scsi response to a get device stats request.
//
// Created by:		Frank Salzmann
//
// Data Created:	February 06, 2002
//
// Returns:         NFX_OK
//
//------------------------------------------------------------------------------
int
sdm_procScsiStatsRsp(sdm_msgDesc_t *mdesc)
{
	uint32			      tid;
	sdm_msgDesc_t        *rspInfo;
	sdm_devDBRec_t       *devRec;
	sdm_getStatsRspMsg_t  sdm_o_msg;
	scsi_getStatsRspMsg_t *scsi_i_msg;

	scsi_i_msg = (scsi_getStatsRspMsg_t *)mdesc->msg;
	memset((void *)&sdm_o_msg, 0, sizeof(sdm_o_msg));
	
	tid     = scsi_i_msg->hdr.cHandle;
	if( tid < 0 || tid > SDM_TRANSACTION_MAX )
		return 0;		// Bogus

	rspInfo = &sdmDispatchedMsgList[tid]; 
	devRec  = rspInfo->dev;
	if( scsi_i_msg->type == SCSI_STATS_TYPE_DEV && !devRec ) {
		sdm_o_msg.eCode = SDM_FAIL;
	}
	else {
		sdm_o_msg.eCode = (scsi_i_msg->eCode == SCSI_MSG_ECODE_OK) ? SDM_OK : SDM_FAIL;
		if(scsi_i_msg->type == SCSI_STATS_TYPE_SCSI) 
			memcpy((void *)&sdm_o_msg.stats.scsi_stats, &scsi_i_msg->stats.scsi_stats, sizeof(scsi_stats_t));
		else 
			memcpy((void *)&sdm_o_msg.stats.dev_stats, &scsi_i_msg->stats.dev_stats, sizeof(dev_stats_t));
	}

	// Send response back to the application that made the stats request.
	sdm_o_msg.hdr.mid     = SDM_MID_GET_STATS_RSP;
	sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_getStatsRspMsg_t), 
				 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);

	sdm_deallocTransaction(tid);
	return SDM_OK;
} // sdm_procScsiStatsRsp 

//------------------------------------------------------------------------------
// Function Name:	sdm_sendScsiDelMsg
//
// Description: 	Send request to SCSI application to delete device.
//
// Created by:		Frank Salzmann
//
// Data Created:	March 20, 2002
//
// Returns:        	tid 
//------------------------------------------------------------------------------
int
sdm_sendScsiDelMsg( scsi_path_t path )
{
	uint32 destCpu, destSlot;
	scsi_deleteReqMsg_t scsi_o_msg;

	memset(&scsi_o_msg, 0, sizeof(scsi_deleteReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_DELETE_REQ;
	scsi_o_msg.hdr.path    = path;
	scsi_o_msg.hdr.cHandle = 0;
				
	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
				
#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiDelMsg: destCpu: %d, destSlot: %d, path: %x\n", 
		   destCpu, destSlot, path);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_deleteReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	return NFX_OK;
} // sdm_sendScsiDelMsg 

//------------------------------------------------------------------------------
// Function Name:	sdm_sendScsiAddMsg
//
// Description: 	Send request to SCSI application to add a device.
//
// Created by:		Frank Salzmann
//
// Data Created:	March 20, 2002
//
// Returns:        	NFX_OK 
//------------------------------------------------------------------------------
int
sdm_sendScsiAddMsg( sdm_dir_t *dir )
{
	uint32 destCpu, destSlot;
	scsi_addReqMsg_t scsi_o_msg;

	memset(&scsi_o_msg, 0, sizeof(scsi_addReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_ADD_REQ;
	scsi_o_msg.hdr.path    = 0;
	scsi_o_msg.hdr.cHandle = (uint32)dir;

	scsi_o_msg.id          = dir->id;
	scsi_o_msg.wwn         = dir->wwn;
	scsi_o_msg.chn_wwn     = dir->chn_wwn;
	scsi_o_msg.lun         = dir->lun;
	scsi_o_msg.llHandle    = dir->llHandle;
	scsi_o_msg.devType     = dir->devType;
	scsi_o_msg.blockCnt    = dir->physicalBlockCnt;
	scsi_o_msg.blockSize   = dir->physicalBlockSize;
	scsi_o_msg.raidLevel   = dir->raidLevel;
	
	strncpy(scsi_o_msg.sn, dir->st.sn, 32);
	strncpy(scsi_o_msg.vendor, dir->st.vendor, 9);
	strncpy(scsi_o_msg.model, dir->st.model, 17);
	strncpy(scsi_o_msg.productId, dir->st.productId, 17);
	strncpy(scsi_o_msg.name, dir->st.name, SDM_DEV_NAME_LEN);

#ifdef TURN_OFF_BLOCK_CHECKSUMMING
	/* XXX Radek For now assume all disks with 512BPS support block checksumming */
	if (dir->devType == SCSI_DEVTYPE_DISK && dir->physicalBlockSize == 512) {
		scsi_o_msg.map.flags = SCSI_MAPPING_FLAGS_BCS_ENABLED;
		scsi_o_msg.map.offset = SCSI_MAPPING_OFFSET_BLOCKS;
	}
#endif
	if(dir->control & SDM_DIE_CTRL_INIT_ERROR) // device is in init state
		scsi_o_msg.sdmState = SDM_DEV_STATE_INIT;
	
	destCpu = SCSI_PATH_GET_PORT(dir->path);
	destSlot = SCSI_PATH_GET_SLOT(dir->path);
				
#if SDM_DEBUG_TX_MSG
	printf("sdm_sendScsiAddMsg: destCpu: %d, destSlot: %d, path: %x\n", 
		   destCpu, destSlot, dir->path);
#endif

	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_addReqMsg_t),
				 "scsi", destCpu, destSlot, 0, 0,0);
				 
	return NFX_OK;
} // sdm_sendScsiAddMsg 

//------------------------------------------------------------------------------
// Function Name:	sdm_procReserveRsp
//
// Description: 	Handles scsi device reservation response.
//
// Created by:		Frank Salzmann
//
// Data Created:	April 19, 2002
//
// Returns:         NFX_OK
//------------------------------------------------------------------------------
int
sdm_procReserveRsp( uint32 notUsed, scsi_relayRspMsg_t *scsi_i_msg ) 
{ 
	uint32 rc, tid, cause, scsi_status;
	sdm_msgDesc_t *rspInfo;

	//printf("sdm_procReserveRsp: \n");

	tid = scsi_i_msg->hdr.cHandle & ~0x10000000;
	rspInfo = &sdmDispatchedMsgList[tid];

	if (scsi_i_msg->status & SD_DRV_STATUS_MASK) {
		printf("\nreserve cmd failed: non-zero driver status: 0x%x\n\n", 
			   (scsi_i_msg->status & SD_DRV_STATUS_MASK) >> SD_DRV_STATUS_SHIFT);
		rc = SDM_FAIL;
		cause = SDM_DEV_RSV_DEV_ERROR;
	}
	else {
		scsi_status = (scsi_i_msg->status & SD_DEV_STATUS_MASK) >> SD_DEV_STATUS_SHIFT;

		switch( scsi_status ) {
			case SCSI_DEV_STATUS_GOOD: 
				rc = SDM_OK;
				//printf("\nPersistent Reserve Out:  Action completed!\n");
				break;
			case SCSI_DEV_STATUS_CHECK:
				//printf("\nDevice returned: CHECK\n");
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_DEV_ERROR;
				break;

			case SCSI_DEV_STATUS_BUSY:
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_DEV_ERROR;
				//printf("\nDevice returned: BUSY\n");
				break;
			case SCSI_DEV_STATUS_RESERVE_CONFLICT:
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_CONFLICT;
				//printf("\nDevice returned: RESERVE CONFLICT\n");
				break;
			case SCSI_DEV_STATUS_CMD_TERMINATED:
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_DEV_ERROR;
				//printf("\nDevice returned: CMD TERMINATED\n");
				break;
			case SCSI_DEV_STATUS_TASK_SET_FULL:
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_DEV_ERROR;
				//printf("\nDevice returned: TASK SET FULL\n");
				break;
			default:
				//printf("\nDevice returned: Other Error status\n");
				rc = SDM_FAIL;
				cause = SDM_DEV_RSV_DEV_ERROR;
		}
	}

	//-- Send Response back to originating application. mdesc will
	//-- be NULL if the request was initiated by the SDM internally.
	if(rspInfo->fsendRsp) {
		sdm_reserveRspMsg_t  sdm_o_msg;
		memset(&sdm_o_msg, 0, sizeof(sdm_reserveRspMsg_t));
		sdm_o_msg.rCode       = rc;
		sdm_o_msg.cause       = cause;
		sdm_o_msg.hdr.mid     = SDM_MID_DEV_RESERVE_RSP;
		sdm_o_msg.hdr.chandle = rspInfo->srcHandle;
		//-- Send failure response back to the application that made the request.
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_reserveRspMsg_t), 
					 rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);
	}

	return SDM_OK;
} // sdm_procReserveRsp

//------------------------------------------------------------------------------
// Function Name:	sdm_procDevReserveReq
//
// Description: 	Handles scsi device reservation request.
//
// Created by:		Frank Salzmann
//
// Data Created:	April 19, 2002
//
// Returns:         NFX_OK
//------------------------------------------------------------------------------
int
sdm_procDevReserveReq( sdm_msgDesc_t *mdesc )
{
	uint32			  	tid, mlen, destCpu, destSlot, allocLen;
	uint32			  	chandle;
	uint32			  	devHandle;
	uint64				reserveKey, serviceKey;
	scsi_path_t		  	path;
	sdm_handle_t	  	*h_entry;
	sdm_devDBRec_t    	*devRec;
	sdm_reserveReqMsg_t *sdm_i_msg; 
	scsi_relayReqMsg_t  *scsi_o_msg;

#if SDM_DEBUG_RX_MSG
	printf("sdm_procDevReserveReq: Enter\n");
#endif

	sdm_i_msg = (sdm_reserveReqMsg_t *)mdesc->msg;
	chandle   = sdm_i_msg->hdr.chandle;
	devHandle = sdm_i_msg->handle;
	h_entry   = (sdm_handle_t *)sdm_getHandleEntry( devHandle );

	if( !h_entry || !(devRec = h_entry->dev) ) {
		sdm_reserveRspMsg_t  sdm_o_msg;
		memset(&sdm_o_msg, 0, sizeof(sdm_reserveRspMsg_t));
		sdm_o_msg.rCode       = SDM_FAIL;
		sdm_o_msg.cause       = SDM_DEV_RSV_BAD_HANDLE;
		sdm_o_msg.hdr.mid     = SDM_MID_DEV_RESERVE_RSP;
		sdm_o_msg.hdr.chandle = chandle;
		//-- Send failure response back to the application that made the request.
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(sdm_reserveRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
	}
	else {
		path             = h_entry->path;
		mdesc->srcHandle = chandle;
		tid = sdm_allocRelayTransaction( devRec, 
										 mdesc,
										 sdm_procReserveRsp,
										 NULL);

		allocLen = 24;
		mlen = sizeof(scsi_relayReqMsg_t) - 1 + allocLen;
		scsi_o_msg = (scsi_relayReqMsg_t *)malloc(mlen);
		memset(scsi_o_msg, 0, mlen);

		scsi_o_msg->hdr.mid       = SCSI_MID_RELAY_REQ;
		scsi_o_msg->hdr.cHandle   = tid;
		scsi_o_msg->hdr.path      = path;
		scsi_o_msg->lun           = SCSI_PATH_GET_LUN(path);
		scsi_o_msg->retryCnt      = 0;
		scsi_o_msg->timeOutCnt    = 0;
		scsi_o_msg->maxTimeToLive = 0;
		scsi_o_msg->dataInLen     = 0;
		scsi_o_msg->dataLen       = allocLen;		//-- Data Out Length
		
		//-- Setup the reserve request CDB 
		scsi_o_msg->cdb[0]    = 0x5F;  			// SCSI PERSISTENT RESERVE OUT command
		scsi_o_msg->cdb[1]    = sdm_i_msg->action; 	
		scsi_o_msg->cdb[7]    = (allocLen & 0xFF00) >> 8;
		scsi_o_msg->cdb[8]    = allocLen & 0xFF;

		if( sdm_i_msg->action == SDM_DEV_RSV_ACTION_REGISTER ) {
			reserveKey = 0;
			scsi_o_msg->cdb[2] = 0;				
			serviceKey = sdm_i_msg->key;
		}
		else {
			scsi_o_msg->cdb[2]    = 1;				// Scope/Type  : LU/Write Exclusive
			serviceKey = 0;
			reserveKey = sdm_i_msg->key;
		}

		//-- Reserve Out Parameter List
		scsi_o_msg->data[0]  = (reserveKey & 0xFF00000000000000) >> 56;
		scsi_o_msg->data[1]  = (reserveKey & 0x00FF000000000000) >> 48;
		scsi_o_msg->data[2]  = (reserveKey & 0x0000FF0000000000) >> 40;
		scsi_o_msg->data[3]  = (reserveKey & 0x000000FF00000000) >> 32;
		scsi_o_msg->data[4]  = (reserveKey & 0x00000000FF000000) >> 24;
		scsi_o_msg->data[5]  = (reserveKey & 0x0000000000FF0000) >> 16;
		scsi_o_msg->data[6]  = (reserveKey & 0x000000000000FF00) >> 8;
		scsi_o_msg->data[7]  = (reserveKey & 0x00000000000000FF);

		scsi_o_msg->data[8]  = (serviceKey & 0xFF00000000000000) >> 56;
		scsi_o_msg->data[9]  = (serviceKey & 0x00FF000000000000) >> 48;
		scsi_o_msg->data[10] = (serviceKey & 0x0000FF0000000000) >> 40;
		scsi_o_msg->data[11] = (serviceKey & 0x000000FF00000000) >> 32;
		scsi_o_msg->data[12] = (serviceKey & 0x00000000FF000000) >> 24;
		scsi_o_msg->data[13] = (serviceKey & 0x0000000000FF0000) >> 16;
		scsi_o_msg->data[14] = (serviceKey & 0x000000000000FF00) >> 8;
		scsi_o_msg->data[15] = (serviceKey & 0x00000000000000FF);

		// Send read command and data to the SCSI application.
		destCpu = SCSI_PATH_GET_PORT(path);
		destSlot = SCSI_PATH_GET_SLOT(path);
		sdm_sendMsg( (char *)scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);
		
		return SDM_OK; // Send response after SCSI close ack
	}
	
	return SDM_OK;
} // sdm_procDevReserveReq

//------------------------------------------------------------------------------
// Function Name:	sdm_procSdmaHandleRsp
//
// Description: 	Handles device open response message sent to the SDM by
//					a scsi application instance.
//
// Created by:		Frank Salzmann
//
// Data Created:	July 23, 2001
//
// Returns:         NFX_OK
//------------------------------------------------------------------------------
int
sdm_procSdmaHandleRsp(sdm_msgDesc_t *mdesc)
{
	uint32			 tid;
	sdm_devDBRec_t   *devRec;
	sdm_sdmaHandleRspMsg_t *sdm_i_msg;

	sdm_i_msg = (sdm_sdmaHandleRspMsg_t *)mdesc->msg;
	
	tid = sdm_i_msg->hdr.chandle;
	devRec = sdmDispatchedMsgList[tid].dev; 
	sdm_deallocTransaction(tid);
	if( !devRec ) {
		//printf("[OPEN_RSP] tid: %x...no device rec found\n", tid);
		return(0);
	}
	sdm_timerStop(sdmDispatchedMsgList[tid].tmr); 

	//-- Send event to the dev open state machine.
	sdm_devOpenSM( devRec, SDM_MID_SDMA_HANDLE_RSP, sdm_i_msg );

	return SDM_OK;
} // sdm_procSdmaHandleRsp 

int 
sdm_procDevControlReq(sdm_msgDesc_t *mdesc)
{
	uint32                tid, destCpu, destSlot, mlen;
	scsi_path_t           path;
	sdm_handle_t         *h_entry;
	sdm_devDBRec_t       *devRec;
	scsi_controlReqMsg_t  scsi_o_msg;
	sdm_controlReqMsg_t  *sdm_i_msg;

	sdm_i_msg = (sdm_controlReqMsg_t *)mdesc->msg;

	// Translate handle into a device path.
	h_entry  = (sdm_handle_t *)sdm_getHandleEntry( sdm_i_msg->handle );
	if( !h_entry || !(h_entry->dev) ) {
		sdm_readDataRspMsg_t sdm_o_msg;
		sdm_o_msg.hdr.mid	  = SDM_MID_DEV_CONTROL_RSP;
		sdm_o_msg.hdr.chandle = mdesc->srcHandle;
		sdm_o_msg.eCode       = SDM_FAIL;
		sdm_sendMsg( (char *)&sdm_o_msg, sizeof(scsi_controlRspMsg_t), 
					 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
		return 0;
	}

	path   = h_entry->path;
	devRec = h_entry->dev;

	mdesc->srcHandle = sdm_i_msg->hdr.chandle;
	tid = sdm_allocTransaction(devRec, mdesc);
				
	mlen = sizeof(scsi_controlReqMsg_t); 

	memset(&scsi_o_msg, 0, mlen);
	scsi_o_msg.hdr.mid     = SCSI_MID_DEV_CONTROL_REQ;
	scsi_o_msg.hdr.cHandle = tid;
	scsi_o_msg.hdr.path    = path;
	scsi_o_msg.control_enable_mask = sdm_i_msg->control_enable_mask;
	scsi_o_msg.control_mask = sdm_i_msg->control_mask;

	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
	sdm_sendMsg( (char *)&scsi_o_msg, mlen, "scsi", destCpu, destSlot, 0, 0,0);

	return SDM_OK;
} /* sdm_procDevControlReq */

int
sdm_procDevControlRsp(sdm_msgDesc_t *mdesc)
{
	uint32				  tid;
	sdm_msgDesc_t		 *rspInfo;
	sdm_devDBRec_t       *devRec;
	scsi_controlRspMsg_t *scsi_i_msg;
	sdm_controlRspMsg_t   sdm_o_msg;

	scsi_i_msg = (scsi_controlRspMsg_t *)mdesc->msg;

	tid = scsi_i_msg->hdr.cHandle;

	rspInfo = &sdmDispatchedMsgList[tid];
	devRec = rspInfo->dev;
	if (!devRec || scsi_i_msg->status != SCSI_MSG_ECODE_OK)
		sdm_o_msg.eCode = SDM_FAIL;
	else
		sdm_o_msg.eCode = SDM_OK;

	sdm_o_msg.hdr.mid     = SDM_MID_DEV_CONTROL_RSP;
	sdm_o_msg.hdr.chandle = rspInfo->srcHandle;

	sdm_sendMsg((char *)&sdm_o_msg, sizeof(sdm_controlRspMsg_t),
				rspInfo->srcApp, rspInfo->srcCpu, rspInfo->srcSlot, 0, 0, 0);

	sdm_deallocTransaction(tid);

	return SDM_OK;
} /* sdm_procDevControlRsp */



//------------------------------------------------------------------------------
// Function Name:   sdm_procNfxEvents
//
// Description:     Handle system events generated by the Event manager.
//
// Created by:      Frank Salzmann
//
// Data Created:    August 15, 2002
//------------------------------------------------------------------------------
int
sdm_procNfxEvents( sdm_msgDesc_t *mdesc )
{
	nfx_eventMsg_t *evt;
	nfx_eventType_t evtType;

	evt = (nfx_eventMsg_t *)mdesc->msg;
	evtType = evt->eventType;
	
	if(evtType == NFX_EVENT_CPU ) {

		uint32 slot, port, state;

		slot  = evt->param.cpuEvent.slot;
		port  = evt->param.cpuEvent.cpu;
		state = evt->param.cpuEvent.state;

		//-- Only care about sp_nim cpu down events.
		if( slot == 2 ) {
			if( state == EVENT_STATE_DOWN ) {
				//-- Remove all paths on this port.
				sdm_delAll(slot, port);
			} 
		}
	}
	return SDM_OK;
} // sdm_procNfxEvents


//-------------------------------------------------------------------------------
// Function Name:	sdm_sendDelDevReq
//
// Description:		Sends a msg. to scsi to remove a device from its list
//
// Date Created:	Sept. 25, 2002
//
// Returns:			SDM_OK
// ------------------------------------------------------------------------------
int
sdm_sendDelDevReq (scsi_id_t id, scsi_path_t path)
{
	scsi_delDevReqMsg_t scsi_o_msg;
	uint32              port;
	uint32              slot;
 
	memset(&scsi_o_msg, 0, sizeof(scsi_delDevReqMsg_t));
	scsi_o_msg.hdr.mid     = SCSI_MID_DEL_DEV_REQ;
	scsi_o_msg.id = id;
 
	slot = SCSI_PATH_GET_SLOT(path);
	port = SCSI_PATH_GET_PORT(path);

	//printf("sdm_sendDelDevReq: destCpu: %d, destSlot: %d\n",
	//		port, slot);
	sdm_sendMsg( (char *)&scsi_o_msg, sizeof(scsi_delDevReqMsg_t),
					"scsi", port, slot, 0, 0, 0);

    return SDM_OK;
} // sdm_sendDelDevReq


//------------------------------------------------------------------------------
// Function Name:   sdm_procGetMaskListReq
//
// Description:     Gets list of LUN masks
//
// Created by:      Nitin Bahadur
//
// Date Created:    Sept 30, 2002
//------------------------------------------------------------------------------
int
sdm_procGetMaskListReq( sdm_msgDesc_t *mdesc )
{
	sdm_getMaskListRspMsg_t *sdm_o_msg;
	int count, i;
	int size;
	sdm_devDBRec_t *devRec;

	// get number of entries in mask table
	count = sdm_confGetMaskCount();

	// allocate space for the response message
	if(count) 
		size = sizeof(sdm_getMaskListRspMsg_t) + (count-1) * sizeof(sdm_conf_t);
	else
		size = sizeof(sdm_getMaskListRspMsg_t);

	sdm_o_msg = (sdm_getMaskListRspMsg_t *)malloc(size);
	if(!sdm_o_msg) return NFX_ERR;
	
	// set fields in response and get the mask list
	memset((void *)sdm_o_msg, 0, size);
	sdm_o_msg->hdr.mid = SDM_MID_DEV_MASKLIST_RSP;
	sdm_o_msg->count = count;

	if(count) 
		sdm_confGetMasks((sdm_conf_t *)sdm_o_msg->list);
	for(i=0; i < count; i++) {
		devRec = sdm_dbFindWwnLun(sdmDevDB, sdm_o_msg->list[i].wwn, sdm_o_msg->list[i].lun);
		if(devRec) 
			strncpy(sdm_o_msg->list[i].devname, devRec->st.name, sizeof(sdm_o_msg->list[i].devname));
		else if(strlen(sdm_o_msg->list[i].devname) == 0)
			strcpy(sdm_o_msg->list[i].devname, "N/A");
	}

	// send the response
	sdm_sendMsg( (char *)sdm_o_msg, size, 
				 mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);
	free(sdm_o_msg);

	return SDM_OK;
} // sdm_procGetMaskListReq


//--------------------------------------------------------------------------------
// Function Name:	sdm_procClrStatsReq
//
// Description:		Clears scsi or device statistics (as requested)
//
// Created By:		Nitin Bahadur
//
// Date Created:	Oct 11, 2002
// ------------------------------------------------------------------------------
int
sdm_procClrStatsReq( sdm_msgDesc_t *mdesc )
{
	sdm_clearStatsReqMsg_t	*req = (sdm_clearStatsReqMsg_t *)mdesc->msg;
	scsi_clearStatsReqMsg_t scsi_req;
	sdm_devDBRec_t			*devRec;
	int						destCpu;
	int						destSlot;
	int						i;

	memset((void *)&scsi_req, 0, sizeof(scsi_req));
	scsi_req.hdr.mid = SCSI_MID_CLR_STATS_REQ;

	switch(req->type) {
		case SDM_STATS_TYPE_DEV:

			if(req->info.dev_info.wwn)
				devRec    = (sdm_devDBRec_t *)sdm_dbFindWwnLun(sdmDevDB, req->info.dev_info.wwn, req->info.dev_info.lun);
			else
				devRec	  = (sdm_devDBRec_t *)sdm_dbFindDevname(sdmDevDB, req->info.dev_info.devname);

			if(!devRec) return SDM_OK;

			scsi_req.type = SCSI_STATS_TYPE_DEV;
			scsi_req.id = devRec->id;

			for(i=0; i < devRec->pathCnt; i++) {
				destSlot = SCSI_PATH_GET_SLOT(devRec->pathList[i].path);
				destCpu = SCSI_PATH_GET_PORT(devRec->pathList[i].path);
				sdm_sendMsg( (char *)&scsi_req, sizeof(scsi_req),
							 "scsi", destCpu, destSlot, 0, 0,0);
			}
			break;

		case SDM_STATS_TYPE_DEVALL:
			scsi_req.type = SCSI_STATS_TYPE_DEVALL;
			destSlot = 2;
			for(destCpu = 0; destCpu < 4; destCpu++)
				sdm_sendMsg( (char *)&scsi_req, sizeof(scsi_req),
							 "scsi", destCpu, destSlot, 0, 0,0);
			break;

		case SDM_STATS_TYPE_SCSI:
			destSlot = req->info.scsi_info.slot;
			destCpu = req->info.scsi_info.port;
			scsi_req.type = SCSI_STATS_TYPE_SCSI;
			sdm_sendMsg( (char *)&scsi_req, sizeof(scsi_req),
						 "scsi", destCpu, destSlot, 0, 0,0);
			break;
		default:
			return SDM_OK;
	}

	return SDM_OK;
} // sdm_procClrStatsReq


//--------------------------------------------------------------------------------
// Function Name:	sdm_procNullMsg
//
// Description:		Null msg to sdm. returns back an ack to sending process.
//
// Created By:		Nitin Bahadur
//
// Date Created:	Nov 15, 2002
// ------------------------------------------------------------------------------
int sdm_procNullMsg( sdm_msgDesc_t *mdesc )
{
	scsi_msgHdr_t	msg;

	memset((void *)&msg, 0, sizeof(msg));
	msg.mid = SCSI_MID_NULL_RSP;

	sdm_sendMsg((char *)&msg, sizeof(msg), mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot, 0, 0, 0);

	return SDM_OK;
} // sdm_procNullMsg
