/*
 * RCS: $Id: sdm-test-ui.c,v 1.9 2002/11/09 00:37:35 nitinb Exp $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>

#undef NULL

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/scsi-api.h>
#include "../sm-scsi-test/scsi-test.h"

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-conf.h"
#include "sdm-init.h"
#include "sdm-msg.h"

int32 nfxMySlotId = 0;
int32 nfxMyCPUId = 1;

fd_set rfds;

// Function Prototypes
static int	sdm_printMenu( void );
static int	sdm_getCmd(char *);
static int	sdm_uiOpenDev( void );
static int	sdm_uiCloseDev( void );
static int	sdm_showDevList( eDB_t * ); 
static int	sdm_showDBSummary( eDB_t * );
static int	sdm_showDevRec( eDB_t * ); 
static int	sdm_showHandleTable( void ); 
static int	sdm_print_eCode( uint32 );

int 
sdm_testMenu(void)
{
	int sel;
	while( (sel = sdm_printMenu() ))  {
		switch(sel) {

			case 1:	
				sdm_uiOpenDev();
				break;

			case 2: 
				sdm_uiCloseDev();
				break;

			case 3: 
				sdm_showDBSummary(sdmDevDB); 
				break;

			case 4: 
				sdm_showDevList(sdmDevDB); 
				break;

			case 5: 
				sdm_showDevRec(sdmDevDB); 
				break;

			case 6: 
				sdm_confViewRecs(); 
				break;

			case 7: 
				sdm_showHandleTable(); 
				break;

			case 99:
				return 0;
			default:
		}
	}

	return 0;
} /* sdm_testMenu */

int
sdm_printMenu(void)
{
	char buf[80];
	int sel;

	printf("\nAGILE STORAGE SDM TEST MENU\n");
	printf("===========================\n");
	printf("1. Open Device.\n");
	printf("2. Close Device.\n");
	printf("3. View Device DB Summary.\n");
	printf("4. View Device List.\n");
	printf("5. View Device Record.\n");
	printf("6. View Dev Mask Config.\n");
	printf("7. View Dev Handle Table.\n");
	printf("99. Quit.\n");
	printf("Enter Selection: ");
	fflush(0);
	sdm_getCmd(buf);
	sscanf(buf, "%d", &sel);
	return(sel);
}

int
sdm_getCmd(char *str)
{
    struct timeval  tv;
    int             rc;
    int             c;
    int             col=0;
    uint32          counter=0;

    bzero(str, 80);

    while (1) {
        /*
         * check out read descriptor for tty
         */
        FD_ZERO(&rfds);
        FD_SET(0, &rfds);

        /* return immediately */
        tv.tv_sec = 0;
        tv.tv_usec = 0;

        if ((rc = select(1, &rfds, NULL, NULL, &tv)) > 0) {
            if (!(FD_ISSET(0, &rfds)))
                printf("ERROR: c = 0x%x, but ISSET = 0\n", c);
            while ((c = getchar()) != '\n') {
                str[col++] = c;
            }
            str[col] = '\0';
            return 0;
        }
        else {
            if (counter++ == 2) {
                counter = 0;
                sdm_rcvMsg();
            }
            if (FD_ISSET(0, &rfds))
                printf("ERROR: c = 0, but ISSET != 0\n");
        }
    }

	return 0;
}
/******************* COMMAND HANDLER FUNCTIONS  START *******************/

unsigned long long g_tmp1;
unsigned long long g_wwn;
unsigned long long g_tmp2;

int
sdm_uiOpenDev(void)
{
	uint32 path;
	uint32 destCpu, destSlot;
	sdm_devDBRec_t    *devRec;
	scsi_openReqMsg_t o_msg;

	memset(&o_msg, 0, sizeof(scsi_openReqMsg_t));
	
	printf("Enter Device WWN: ");
	scanf("%llx", &g_wwn);

	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, g_wwn);
	if( !devRec ) {
		printf("Record Not Found.\n");
		return 0;
	}

	path = devRec->pathList[0].path;
	/* sdmMsgList[sdm_cHandle] = devRec; */

	o_msg.hdr.mid     = SCSI_MID_OPEN_REQ;
	o_msg.hdr.cHandle = 0;
	o_msg.hdr.path    = path;
	
	destCpu = SCSI_PATH_GET_PORT(path);
	destSlot = SCSI_PATH_GET_SLOT(path);
	printf("sdm open dev: slot: %d, cpu: %d\n", destSlot, destCpu); /** mydebug **/
	sdm_sendMsg( (char *)&o_msg, sizeof(scsi_openReqMsg_t), 
				 "scsi", destCpu, destSlot, 0, 0, 0);
	return 0;
} /* sdm_uiOpenDev */

int
sdm_uiCloseDev(void)
{
#if 0
	uint32 dest_port;
	unsigned int path;
	scsi_closeDevMsg_t c_msg;

	memset(&c_msg, 0, sizeof(scsi_closeDevMsg_t));
	
	printf("Enter Device Path: ");
	scanf("%x", &path);
	c_msg.mid     = SCSI_MSG_ID_CLOSE_DEV;
	c_msg.cHandle = 0;
	c_msg.path    = path;

	dest_port = PACK_PORT_ADDRESS(0,0,7,EEE_CONTROL,0, 1);
	sdm_sendMsg( (char *)&c_msg, sizeof(scsi_closeDevMsg_t), dest_port);
#endif
	return 0;
} /* sdm_uiCloseDev */


/******************* MESSAGE DISPLAY FUNCTIONS START *******************/
int
sdm_printDevAddMsg(sdm_addDevMsg_t *msg)
{
	printf("mid       : %016x : %s\n", msg->hdr.mid, "DEVICE ADD");
	printf("cHandle   : %016x : \n", msg->hdr.chandle);
	printf("path      : %016x : \n", msg->path);
	printf("id        : %016llx : \n", msg->id);
	printf("type      : %016x : \n", msg->type);
	printf("blocks    : %016x : \n", msg->lbCnt);
	printf("block size: %016x : \n", msg->lbSize);
	printf("vendor    : %16s : \n", msg->st.vendor);
	printf("model     : %16s : \n", msg->st.model);
	printf("\n\n");
	return 0;
} /* sdm_printDevAddMsg */

int
sdm_printDevOpenRsp(scsi_openRspMsg_t *msg)
{
	printf("mid       : %08x : %s\n", msg->hdr.mid, "OPEN RESPONSE");
	printf("cHandle   : %08x : \n", msg->hdr.cHandle);
	sdm_print_eCode(msg->eCode);
	printf("\n\n");

	return 0;
}

int
sdm_printDevCloseRsp(scsi_closeRspMsg_t *msg)
{
	printf("mid       : %08x : %s\n", msg->hdr.mid, "CLOSE RESPONSE");
	printf("cHandle   : %08x : \n", msg->hdr.cHandle);
	sdm_print_eCode(msg->eCode);
	printf("\n\n");

	return 0;
}

int
sdm_print_eCode(uint32 eCode)
{
	printf("eCode     : %08x : ", eCode);
	switch(eCode) {
		case SCSI_MSG_ECODE_OK:
			printf("%s\n", "OK");
			break;
		case SCSI_MSG_ECODE_NOSUCH_DEV:
			printf("%s\n", "NOSUCH DEVICE");
			break;
		case SCSI_MSG_ECODE_DEV_CLOSED:
			printf("%s\n", "DEVICE CLOSED");
			break;
		case SCSI_MSG_ECODE_INVALID_MSG:
			printf("%s\n", "INVALID MESSAGE ID");
			break;
		case SCSI_MSG_ECODE_NO_RESOURCE:
			printf("%s\n", "NO RESOURCE");
			break;
		case SCSI_MSG_ECODE_NO_MEDIA:
			printf("%s\n", "MEDIA NOT INSTALLED");
			break;
		case SCSI_MSG_ECODE_READ_ONLY:
			printf("%s\n", "DEVICE READ ONLY");
			break;
		case SCSI_MSG_ECODE_OO_RANGE:
			printf("%s\n", "OUT OF RANGE");
			break;
		default:
			printf("%s\n", "Status Unknown");
	}

	return 0;
} /* print_statusCode */

int
sdm_showDevList( eDB_t *db ) 
{
	int i;
	eDB_drec_t      *drec;
	sdm_devDBRec_t *dev;

	printf("INDEX           WWN        STATE    HANDLE   PATHCNT\n");
	printf("-----    ----------------  ------  --------  -------\n");

	drec = (eDB_drec_t *)db->drec_head;
	for( i=0; (i < db->mrec->recCount) && (drec); i++ ) {
		dev = (sdm_devDBRec_t *)drec->dobject;
		printf("%5d    %016llx  %6s  %08x  %7d\n", i, dev->id, 
			   dev->state == SDM_DEV_STATE_OPENED ? "OPEN":"CLOSED",
			   dev->openHandles[0], 
			   dev->pathCnt);
		drec = drec->next;
	}
	return 0;
} /* sdm_showDevList */

int
sdm_showDBSummary(eDB_t *db) 
{
	printf("\nDatabase Summary:\n");
	printf("=================\n");
	printf("Name         : %s\n", db->mrec->dbName);
	printf("Record Count : %d\n", db->mrec->recCount);
	printf("\n");
	return 0;
} /* sdm_showDevDBSummary */

int
sdm_showDevRec(eDB_t *db) 
{
	uint64 key;
	sdm_devDBRec_t *devRec;


	printf("Enter Device WWN: ");
	scanf("%llx", &key);

	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(db, key);
	if( !devRec ) {
		printf("Record Not Found.\n");
		return 0;
	}
	
	printf("\nDevice Record:\n");
	printf("==============\n");
	printf("Device WWN   : %llx\n", devRec->id);
	printf("Device Name  : %s\n", devRec->st.name);
	printf("Vendor Name  : %s\n", devRec->st.vendor);
	printf("Model Name   : %s\n", devRec->st.model);
	printf("Block Cnt    : %d\n", devRec->lbCnt);
	printf("Block Size   : %d\n", devRec->lbSize);
	printf("Device State : %s\n",
				   	devRec->state == SDM_DEV_STATE_OPENED ? "OPEN":"CLOSED");
	printf("Path         : %08x\n", devRec->pathList[0].path);
	printf("Path State   : %08x\n", devRec->pathList[0].state);
	printf("\n");
	return 0;
}
/**************** MESSAGE DISPLAY FUNCTIONS END *******************/

int
sdm_showHandleTable(void) 
{
	int handle,fp_slot, sp_slot, sp_port, i, j;
	sdm_handle_t *h_entry;

	j=0;
	printf("\n");
	printf("SDM HANDLE TABLE ENTRIES\n");
	printf("------------------------\n");
	for( fp_slot=0; fp_slot < SDM_SLOT_MAX; fp_slot++ ) {
		for( sp_slot=0; sp_slot < SDM_SLOT_MAX; sp_slot++ ) {
			for( sp_port=0; sp_port < SDM_PORT_MAX; sp_port++ ) {
				for( i=0; i < SDM_OPEN_HANDLE_MAX; i++ ) {
					h_entry = &devHandles[fp_slot][sp_slot][sp_port][i];
					if( h_entry->dev ) {
						handle = 0;
						SDM_HANDLE_SET_FP_SLOT(fp_slot, handle);
						SDM_HANDLE_SET_SLOT(sp_slot, handle);
						SDM_HANDLE_SET_PORT(sp_port, handle);
						SDM_HANDLE_SET_INDEX(i, handle);
						printf("%03d). Handle: %x ... Path: %x ... Id: %016llx\n", 
							   j, handle, h_entry->path, h_entry->dev->id);
						j++;
					}
				}
			}
		}
	}
	printf("\n");
	return 0;
} // sdm_showHandleTable 
