//------------------------------------------------------------------------------
//  FileName   : sdm-verify.c
//
//  Description: 
//
//	Created    : 06 March 2002
//
//  Author     : Frank Salzmann
//
//  RCS        : $Id: sdm-verify.c,v 1.9 2002/11/09 00:37:35 nitinb Exp $ 
//
//  Copyright (c)  Agile Storage, Inc. 2002
//------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#undef NULL

#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/elog-api.h>
//#include "../sm-scsi-test/scsi-test.h"

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"
#include "sdm-msg.h"
#include "../sm-scsi/scsi-dev-cmds.h"

//
//-- Function Prototypes
//
static int	sdm_verifyLun( sdm_devDBRec_t *, uint64, uint32, uint32, uint32 );
static int	sdm_verifyRemPath( sdm_dir_t * );
static int	sdm_rediscDev( sdm_dir_t *, uchar8 * );

//------------------------------------------------------------------------------
// Function Name:	sdm_procRediscReq
//
// Description: 	Handles lun rediscover request message.
//
// Created by:		Frank Salzmann
//
// Data Created:	April 05, 2002
//
// Returns:        	NFX_OK 
//------------------------------------------------------------------------------
int
sdm_procRediscReq(sdm_msgDesc_t *mdesc)
{
	uint32				i,j,devCnt;
	uint64				id = 0;
	uint64				wwn;
	sdm_devDBRec_t 		*devRec;
	sdm_rediscReqMsg_t 	*i_msg;

	i_msg = (sdm_rediscReqMsg_t *)mdesc->msg;

	//-- First see if all the luns we currently have in our
	//-- device database are still present.
	devCnt = sdm_dbGetNumEntries( sdmDevDB, SDM_DEVLIST_ALL, SDM_DEVLIST_PORTALL, NULL );
	wwn = i_msg->wwn;
	for( i=0; i < devCnt; i++ ) {

		devRec = (sdm_devDBRec_t *)sdm_dbGetNextRec( sdmDevDB, id );
		if( devRec ) {
			id = devRec->id;

			for(j=0; j < devRec->pathCnt; j++) {
				if(devRec->pathList[j].wwn == wwn) {

					sdm_verifyLun( devRec,
								   wwn,
								   devRec->pathList[j].lun,
								   devRec->pathList[j].path,
								   devRec->pathList[j].llHandle);

					E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
						   0, 0, 0, ("SDM: Verify:     0x%016llx:%d", 
									 devRec->wwn, devRec->lun));
				}
			}
		}
	}
	//-- Now look for additional luns.

	return NFX_OK;
} // sdm_procRediscReq


//------------------------------------------------------------------------------
// Function Name:   sdm_verifyLun
// 
// Description:     
//
// Data Created:    April 09, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_verifyLun(sdm_devDBRec_t *devRec,
			  uint64 		  wwn, 
			  uint32 		  lun, 
			  uint32 		  path,
			  uint32    	  llHandle)
{
	sdm_dir_t *dir;

	dir = (sdm_dir_t *)malloc(sizeof(sdm_dir_t));
	if( !dir )
		return SDM_FAIL;

	memset(dir, 0, sizeof(sdm_dir_t));
	dir->id       = devRec->id;
	dir->path     = path;
	dir->wwn      = wwn;
	dir->lun      = lun;
	dir->llHandle = llHandle;
	dir->state    = SDM_DIS_VERIFY;
	dir->nextEvt  = SDM_DIE_INQ_RSP;
	dir->devType  = devRec->devType;
	memcpy((void *)&dir->st, &devRec->st, sizeof(dir->st));

	if( sdm_setInitST( dir ) == SDM_FAIL )
		return SDM_FAIL;
	
	//-- Send SCSI inquiry command to the lun. Because at this
	//-- point we don't know what it is.
	sdm_sendInqCmd(dir);

	return SDM_OK;
} // sdm_verifyLun

//------------------------------------------------------------------------------
// Function Name:   sdm_verifyInqRsp
// 
// Description:     
//
// Data Created:    April 09, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_verifyInqRsp(sdm_dir_t *dir, scsi_relayRspMsg_t *i_msg )
{
	scsi_sense_t   *sense;
	
	//-- Make sure that this is not an error response
	//-- message coming back from the SCSI layer.
	if( i_msg->hdr.mid != SCSI_MID_RELAY_RSP ) {
		E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
			   0, 0, 0, ("SDM: Verify Failed: SCSI Path Error: 0x%016llx:%d", 
						 dir->wwn, dir->lun));

		//-- Get rid of this SCSI device path.
		sdm_verifyRemPath(dir);
		return(SDM_OK);
	}

	switch(i_msg->status & SD_DEV_STATUS_MASK)  
	{
		case SCSI_DEV_STATUS_CHECK:
		case SCSI_DEV_STATUS_CMD_TERMINATED:
			if( i_msg->status & SD_DEV_SENSE_VALID ) {

				sense = (scsi_sense_t *)i_msg->senseData;

				/* Look at sense data */
				switch( sense->senseKey )
				{
					case SCSI_SENSE_KEY_NOT_READY:
					case SCSI_SENSE_KEY_HARDWARE_ERROR:
					case SCSI_SENSE_KEY_ILLEGAL_REQUEST:
						//-- Fatal Error unable to access device 
						//-- Assume that the dev/lun is no longer
						//-- with us. So remove it from the device
						//-- database.
						E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
							   0, 0, 0, 
							   ("SDM: Verify Failed: Device Error: 0x%016llx:%d", 
								dir->wwn, dir->lun));
						break;

					case SCSI_SENSE_KEY_NO_SENSE:
					case SCSI_SENSE_KEY_RECOVERED_ERROR:
					default:
						//-- Device responded ok so leave it alone.
						E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
							   0, 0, 0, ("SDM: Verify: Check No Sense Data: 0x%016llx:%d", 
										 dir->wwn, dir->lun));
						break;
				}
			}
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
				   0, 0, 0, 
				   ("SDM: Verify Failed: Remove Path: 0x%016llx:%d", 
					dir->wwn, dir->lun));
			
			sdm_verifyRemPath(dir);
			break;

		case SCSI_DEV_STATUS_GOOD:
		case SCSI_DEV_STATUS_CONDITION_MET:
		default:
			//-- Take a look at the periphiral qualifier field to see if an
			//-- invalid device type is returned.
			if( dir->devType == SCSI_DEVTYPE_DISK && i_msg->data[0] != SCSI_DEVTYPE_DISK ) {
				E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
					   0, 0, 0, ("SDM: Verify Failed: Lun Not Detected: 0x%016llx:%d", 
								 dir->wwn, dir->lun));

				//-- Get rid of this SCSI device path.
				sdm_verifyRemPath(dir);
			}
			else {
				//-- Device responded ok so leave it alone.
				E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 
					   0, 0, 0, ("SDM: Verify Ok: 0x%016llx:%d", 
								 dir->wwn, dir->lun));

				//-- When Lun Zero responds that it is still there
				//-- initiate phase 2 of the rediscovery operation
				//-- which is to see if there are any  additional luns 
				if( dir->lun == 0 ) {
					sdm_rediscDev( dir, i_msg->data );
					return SDM_OK;				//-- Return here. Don't free dir.
				}
			}
			break;
	}
	free(dir);
	return SDM_OK;
} // sdm_verifyInqRsp

//------------------------------------------------------------------------------
// Function Name:   sdm_verifyRemPath
// 
// Description:     
//
// Data Created:    April 09, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_verifyRemPath( sdm_dir_t *dir )
{
	sdm_devDBRec_t *devRec;
	sdm_path_t     *sdm_p_entry;
	uint32         handleSlot;
	sdm_handle_t   *sdm_h_entry;
	int              isActivePath;

	// Look for device in database.
	devRec = (sdm_devDBRec_t *)sdm_dbFindRec(sdmDevDB, 
											 dir->id);
	if( devRec ) {
		
		if( dir->path == sdm_getActivePath(devRec) ) {
			isActivePath = 1;
			sdm_p_entry = sdm_getActivePathEntry(devRec);
			handleSlot = sdm_p_entry->handleSlot;
		}
		else
			isActivePath = 0;

		//-- Remove this path from the device database.
		sdm_removePath( devRec, dir->path); 


		//-- If all paths to a device have been removed 
		//-- then remove the device from the database..
		if( devRec->pathCnt <= 0 ) {

			if( devRec->handleCnt > 0 ) {    
				sdm_h_entry = (sdm_handle_t *)sdm_getHandleEntry( devRec->openHandles[0] );
				if( sdm_h_entry) {
					sdm_h_entry->dev = 0;
				}
			}
			
			sdm_sendDevDelNotify("evm", devRec);	

			// ask scsi to remove dev from its db
			sdm_sendDelDevReq (dir->id, dir->path);
			
			E_LOG (class_1, notice_s, eee_getApplicationId("sdm"), 0, 0, 0,
				   ("SDM: Verify (RemoveDevice): 0x%016llx:%d", 
					devRec->wwn, devRec->lun));
			
			sdm_dbRemRec( sdmDevDB, dir->id );
		}
		else {	//-- If removed path was active then use alternate path.
			if(isActivePath)
				sdm_openPath(devRec, handleSlot, 0/*port*/, 0/*path*/, 0/*flags*/);
		}
	}

	return SDM_OK;
} // sdm_verifyRemPath

//------------------------------------------------------------------------------
// Function Name:   sdm_rediscDev
// 
// Description:     
//
// Data Created:    April 09, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_rediscDev( sdm_dir_t *dir, uchar8 *inqRsp )
{
	dir->state   = SDM_DIS_INIT;
	dir->nextEvt = SDM_DIE_INIT_DEV;

	//sdm_printBuf(inqRsp, 100);

	sdm_procInqRsp(dir, inqRsp);

	return SDM_OK;
} // sdm_rediscDev


//------------------------------------------------------------------------------
// Function Name:   sdm_printBuf
// 
// Description:     
//
// Data Created:    April 10, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_printBuf(char *buf, int len)
{
    int i, j;
    char ch;
    char *data;
 
    data = buf;
    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");
    return 0;
} // sdm_printBuf

