/*
 *-----------------------------------------------------------------
 *
 * Name:	seep.h
 *
 * RCS:  $Id: seep.h,v 1.8 2002/11/23 00:55:33 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _SEEP_H
#define _SEEP_H

/* NOTE: This file is included from the run-time tree.
 * Don't include any files outside of the following ifdef or you will
 * break the build. If there is a need to add a #include here for
 * run-time, put it in nfx-tree/code/sm-seep/seep.h.
 */
#ifdef PROM
#include <exports/seep-api.h>
#endif

#define I2C_START       0
#define I2C_STOP        1
#define I2C_READ        2
#define I2C_WRITE       3
#define I2C_DUMP        4
#define I2C_READ_LAST   5


#define	SSC_I2C_SEEP_ADDR	0xA0
#define	SSC_I2C_SEEP_BUS	0

#ifdef PROM
//@@@ remove if (when) not used by PROM
#define	SSC_I2C_ADDR SSC_I2C_SEEP_ADDR
#endif

#define	FC_I2C_BOARD_ADDR		0xA6
#define	FC_I2C_PCB_ADDR		    0xA0
#define	FC_I2C_ADDR		        FC_I2C_PCB_ADDR

#define	FP_I2C_ADDR		        0x50
#define FP_I2C_MUX              0x70
#define FP_I2C_SFP              0x50

#define	SEEP_NAME		1
#define	SEEP_PART		2
#define	SEEP_REVISION	3
#define	SEEP_DEVIATION	4
#define	SEEP_POWER		5
#define	SEEP_SERIAL		6
#define	SEEP_MISC		7
#define	SEEP_IP			8

#define SEEP_TOTAL_LEN		256

#define SEEP_MAX_STRING_LEN		80

#define SEEP_SIG_PCB			"CB01"
#define SEEP_SIG_CHASSIS		"C313"
#define SEEP_SIG_POWER			"C777"

#define SEEP_SIG_LEN				(4+1)
#define	SEEP_PCB_MODEL_LEN			(9+1)
#define	SEEP_PCB_ASSEMBLY_LEN		(13+1)
#define	SEEP_PCB_SERIAL_LEN			(10+1)
#define	SEEP_PCB_REVISION_LEN		(4+1)
#define	SEEP_PCB_DEVIATION_LEN		(11+1)
#define	SEEP_PCB_IP_ADDR_LEN		(15+1)
#define	SEEP_PCB_IP_MASK_LEN		(15+1)
#define	SEEP_PCB_DIAG_TEST_LEN		(1+1)
#define	SEEP_PCB_RESULT_LEN		(1+1)
#define	SEEP_PCB_DATE_LEN		(6+1)
#define	SEEP_PCB_MISC_INFO_LEN		(8+1)
#define	SEEP_PCB_UPTIME_LEN		(3+1)
#define	SEEP_PCB_REBOOTNUM_LEN		(1+1)

#define SEEP_CHASSIS_MODEL_LEN		(15+1)
#define	SEEP_CHASSIS_SERIAL_LEN		(10+1)
#define	SEEP_CHASSIS_REVISION_LEN	(5+1)
#define	SEEP_CHASSIS_DEVIATION_LEN	(5+1)
#define SEEP_CHASSIS_MAC_ADDR_LEN	(12+1)
#define	SEEP_CHASSIS_UPTIME_LEN		(4+1)
#define	SEEP_CHASSIS_REBOOTNUM_LEN	(1+1)

#define SEEP_CHASSIS_SYSTEM_MODEL_OFFSET    0x39
#define SEEP_CHASSIS_CUMSYSUPTIME_OFFSET    0x49
#define SEEP_CHASSIS_FPREBOOT_OFFSET        0x53

#define SEEP_ENV_START		147 //32 bytes
#define SEEP_NVRAM_START	179
#define SEEP_VER_STRING_LEN_SIZE	(SEEP_TOTAL_LEN -  SEEP_NVRAM_START - 1)

#define SEEP_PCB_CUM_UPTIME	0x86
#define SEEP_PCB_UPTIME_SINCE_REBOOT	0x8A

typedef struct {
	uchar8	signature[SEEP_SIG_LEN]; // 5
	uchar8	model_num[SEEP_CHASSIS_MODEL_LEN]; //16
	uchar8	serial_num[SEEP_CHASSIS_SERIAL_LEN]; // 11
	uchar8	revision[SEEP_CHASSIS_REVISION_LEN]; // 6
	uchar8	board_serial_num[SEEP_PCB_SERIAL_LEN]; // 11
	uchar8	board_revision[SEEP_PCB_REVISION_LEN]; // 5
	uchar8	deviation[SEEP_PCB_DEVIATION_LEN]; // 12 
	uchar8	result_ict[SEEP_PCB_RESULT_LEN]; // 2
	uchar8	date_ict[SEEP_PCB_DATE_LEN]; // 7
	uchar8	result_ft[SEEP_PCB_RESULT_LEN]; // 2
	uchar8	ip_addr[SEEP_PCB_IP_ADDR_LEN]; // 16
	uchar8	ip_mask[SEEP_PCB_IP_MASK_LEN]; // 16
	uchar8	no_of_reboots[SEEP_PCB_REBOOTNUM_LEN]; // 2 
    uchar8  dummy_spare[10]; // 10
	uchar8	mac_addr[SEEP_CHASSIS_MAC_ADDR_LEN]; //13
	uchar8	uptime_since_reboot[SEEP_PCB_UPTIME_LEN]; // 4 
	uchar8	cumulative_uptime[SEEP_PCB_UPTIME_LEN]; // 4 
	uchar8	cluster_uptime[SEEP_CHASSIS_UPTIME_LEN]; // 5 
} seepPCB_t;

#ifndef SSC
extern uchar8	seep_struct[NUM_SEEPS][SEEP_TOTAL_LEN];
#endif

#ifdef NFX_SMP
extern nfx_global_spinlock_t wait_seep_lock;
#endif

#endif	/* _SEEP_H */

