#ifndef __GT64240_UART_H
#define __GT64240_UART_H

#include "gt64240.h"

#define GT_MPSC_UNITS 2

#define TTY_DEFAULT_RATE 57600

struct gtuart {
#if defined(__OpenBSD__)
	struct device dev;
	struct tty *tty;
	int initialize;				/* If non-zero, device is being initialized */
#endif
	int unit;
	struct gtsdma dma;

#if !defined(GT_MPSC_GOOD_REGS)
	uint32 regs[GT_MPSC_ESR + 1];
#endif	
};

struct gtuart *gtu_sc(int unit);
#if !defined(GT_MPSC_GOOD_REGS)
extern uint32 gtu_mainregs[3];
#endif

static inline int
gtu_valid_main_reg(uint32 reg)
{
	return reg == GT_MPSC_MRR || reg == GT_MPSC_RCRR ||
		reg == GT_MPSC_TCRR;
}

static inline uint32
gtu_read_mainreg(uint32 reg)
{
	assert(gtu_valid_main_reg(reg));
#if defined(GT_MPSC_GOOD_REGS)
	return GT_SWAP(GT64W(reg));
#else
	return gtu_mainregs[(reg - GT_MPSC_MRR) / 4];
#endif	
}


static inline void
gtu_write_mainreg(uint32 reg, uint32 val)
{
	assert(gtu_valid_main_reg(reg));
	GT_REG_WRITE(reg, val);
#if !defined(GT_MPSC_GOOD_REGS)
	gtu_mainregs[(reg - GT_MPSC_MRR) / 4] = val;
#endif
}


static inline int
gtu_valid_unit(uint32 unit)
{
	return unit < GT_MPSC_UNITS;
}


static inline uint32
gtu_unit2base(uint32 unit)
{
	assert(gtu_valid_unit(unit));
	return GT_MPSC_0 + 0x1000 * unit;
}


static inline int
gtu_valid_reg(uint32 reg)
{
	return GT_MPSC_MMCRL <= reg && reg <= GT_MPSC_ESR;
}

static inline uint32
gtu_read_reg(uint32 unit, uint32 reg)
{
#if defined(GT_MPSC_GOOD_REGS)
	assert(gtu_valid_unit(unit) && gtu_valid_reg(reg));
	return GT_SWAP(GT64W(gtu_unit2base(unit) + reg * sizeof(uint32)));
#else
	return gtu_sc(unit)->regs[reg];
#endif
}


static inline void
gtu_write_reg(uint32 unit, uint32 reg, uint32 val)
{
	assert(gtu_valid_unit(unit) && gtu_valid_reg(reg));
	GT_REG_WRITE(gtu_unit2base(unit) + reg * sizeof(uint32), val);
#if !defined(GT_MPSC_GOOD_REGS)
	gtu_sc(unit)->regs[reg] = val;
#endif
}


static inline int
gtu_valid_intreg(uint32 reg)
{
	return reg == GT_MPSC_CAUSE || reg == GT_MPSC_MASK;
}


static inline uint32
gtu_unit2intoffset(uint32 unit)
{
	return unit == 0 ? 0x0 : 0x8;
}

static inline uint32
gtu_read_intreg(uint32 unit, uint32 reg)
{
	assert(gtu_valid_unit(unit) && gtu_valid_intreg(reg));
	return GT_SWAP(GT64W(gtu_unit2intoffset(unit) + reg));
}


static inline void
gtu_write_intreg(uint32 unit, uint32 reg, uint32 val)
{
	assert(gtu_valid_unit(unit) && gtu_valid_intreg(reg));
	GT_REG_WRITE(gtu_unit2intoffset(unit) + reg, GT_SWAP(val));
}


int gtbrg_set(uint32 unit, int baud);
int gtu_baud(struct gtuart *sc, int baud);
int gtu_txrdy(struct gtuart *sc);
int gtu_rxrdy(struct gtuart *sc);
int gtu_txchar(struct gtuart *sc, int data);
int gtu_rxchar(struct gtuart *sc);
int32 gt64240_console(uint32 op, uint32 chan, int32 data);
void gtu_loopback_mode(struct gtuart *sc);
void gtu_normal_mode(struct gtuart *sc);
void gtu_softintr(void *);
void gtuart_print_status(void);

#if defined(__OpenBSD__)

struct tty;
struct termios;

struct consdev;
void	gtuartcnprobe	(struct consdev *);
void	gtuartcninit	(struct consdev *);
int		gtuartcngetc	(dev_t);
void	gtuartcnputc	(dev_t, int);
void	gtuartcnpollc	(dev_t, int);


#endif

#endif
