#ifndef __GT64240_BRG_H
#define __GT64240_BRG_H

#include "gt64240.h"


#define GT_BRG_UNITS 3

#define BCLK_IN_RATE 3686400

#define GT_BRG_0 0xb200			/* BRG0 base */
#define GT_BRG_1 0xb208			/* BRG1 base */
#define GT_BRG_2 0xb210			/* BRG2 base */


#define GT_BRG_CAUSE 0xb834		/* Interrupt Cause Register */
#define GT_BRG_MASK 0xb8b4		/* Interrupt Mask Register */

#define GT_BRG_BCR 0x0			/* Configuration Register */
#define GT_BRG_BTR 0x4			/* Tuning Register */


/* BRG Configuration Register bits */
#define GT_BRG_BCR_EN (1<<16)	/* Enable */
#define GT_BRG_BCR_RST (1<<17)	/* Reset */
#define GT_BRG_BCR_CLKS_OFFSET (18) 
#define GT_BRG_BCR_BCLK (0x0 << GT_BRG_BCR_CLKS_OFFSET)
#define GT_BRG_BCR_SCLK0 (0x2 << GT_BRG_BCR_CLKS_OFFSET)
#define GT_BRG_BCR_TSCLK0 (0x3 << GT_BRG_BCR_CLKS_OFFSET)
#define GT_BRG_BCR_SCLK1 (0x6 << GT_BRG_BCR_CLKS_OFFSET)
#define GT_BRG_BCR_TSCLK1 (0x7 << GT_BRG_BCR_CLKS_OFFSET)
#define GT_BRG_BCR_TCLK (0x8 << GT_BRG_BCR_CLKS_OFFSET)

/* BRG Interrupt Register bits */
#define GT_BRG_BTR0 (1<<0)
#define GT_BRG_BTR1 (1<<1)
#define GT_BRG_BTR2 (1<<2)


static inline int
gtbrg_unit_valid(uint32 unit)
{
	return unit < GT_BRG_UNITS;
}


static inline int
gtbrg_reg_valid(uint32 reg)
{
	return reg == GT_BRG_BCR || reg == GT_BRG_BTR;
}


static inline int
gtbrg_unit2offset(uint32 unit)
{
	return unit * 8 + GT_BRG_0;
}


static inline uint32
gtbrg_read_reg(uint32 unit, uint32 reg)
{
	assert(gtbrg_unit_valid(unit) && gtbrg_reg_valid(reg));
	return GT_SWAP(GT64W(gtbrg_unit2offset(unit) + reg));
}


static inline void
gtbrg_write_reg(uint32 unit, uint32 reg, uint32 val)
{
	assert(gtbrg_unit_valid(unit) && gtbrg_reg_valid(reg));
#if RAL_XXX
	GT_REG_WRITE(gtbrg_unit2offset(unit), GT_SWAP(val));
#else
	GT_REG_WRITE(gtbrg_unit2offset(unit) + reg, GT_SWAP(val));
#endif
}


extern int gtbrg_setcdv(uint32 unit, int baud);
extern void gtbrg_init(uint32 unit);

#endif
