#if !defined(LINUX_TEST) && !defined(EVAL_BOARD) && !defined(SSC_MGMT) && !defined(SIBYTE)

#if defined(__OpenBSD__)
#include <sys/types.h>
#include <sys/tty.h>
#include <sys/param.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <machine/autoconf.h>
#include <machine/cpu.h>
#include <mips64/dev/clockvar.h>
#include <mips64/archtype.h>
#include <sys/conf.h>
#include <pmonmips/gtuartvar.h>
#endif


#include <nfx-incl.h>
#include <nfx-defs.h>
#include <exports/hwMap.h>
#include <exports/eee-error.h>
#include "gt64240-sdma.h"
#include "gt64240-uart.h"
#include "gt64240-brg.h"
#include <exports/serio-api.h>


#if defined(DEBUG_UART)
#define debug(x) printf x
#else
#define debug(x)
#endif


#if !defined(GT_MPSC_GOOD_REGS)
uint32 gtu_mainregs[3] = { 0x3fffffff, 0x0, 0x0 };
#endif

static int32 gtu_init(struct gtuart *sc, uint32 unit);

#if defined(__OpenBSD__)

cdev_decl(gtuart);

static int gtuart_match(struct device *, void *, void *);
static void gtuart_attach(struct device *, struct device *, void *);

struct cfattach gtuart_ca = {
	sizeof(struct gtuart), gtuart_match, gtuart_attach
};

struct cfdriver gtuart_cd = {
	0, "gtuart", DV_TTY, 0, 0
};


static int
gtuart_match(struct device *parent, void *cfdata, void *aux)
{
	struct confargs *ca = (struct confargs *)aux;

	return (strcmp(ca->ca_name, gtuart_cd.cd_name) == 0);
}


static void
gtuart_attach(struct device *parent, struct device *self, void *aux)
{
	struct gtuart *sc = (struct gtuart *)self;
	intrmask_t s;

	s = splimp();
	gtu_init(sc, sc->dev.dv_unit);
	splx(s);

	printf("\n");
}

struct gtuart *
gtu_sc(int unit)
{
	return gtuart_cd.cd_devs[unit];
}

#else
struct gtuart uarts[GT_MPSC_UNITS];

struct gtuart *
gtu_sc(int unit)
{
	return &uarts[unit];
}

#endif



void
gtbrg_init(uint32 unit)
{
	gtbrg_write_reg(unit, GT_BRG_BCR, GT_BRG_BCR_EN | GT_BRG_BCR_BCLK);
	gtbrg_write_reg(unit, GT_BRG_BTR, 0);
}


int
gtbrg_setcdv(uint32 unit, int baud)
{
	uint32 cdv;

#if RAL_XXX
	cdv = BCLK_IN_RATE / (16 * baud) - 1;
	gtbrg_write_reg(unit, GT_BRG_BCR, cdv | GT_BRG_BCR_EN | GT_BRG_BCR_BCLK);
#else
	extern uint32 get_busclock(void);

	cdv = ((get_busclock() * 1000000)/(2*baud*16))-1;
	gtbrg_write_reg(unit, GT_BRG_BCR, cdv | GT_BRG_BCR_EN | GT_BRG_BCR_TCLK);
#endif

	return NFX_OK;
}



static int
gtu_unit2mrrshift(uint32 unit)
{
	return unit * GT_MPSC_MRR_MR1_OFFSET;
}


static int
gtu_unit2crrshift(uint32 unit)
{
	return unit * GT_MPSC_RCRR_CRR1_OFFSET;
}


static void
gtu_setbrg(uint32 unit, uint32 reg, uint32 brg)
{
	uint32 v;
	int shift;
	
	assert(reg == GT_MPSC_RCRR || reg == GT_MPSC_TCRR);

	v = gtu_read_mainreg(reg);
	shift = gtu_unit2crrshift(unit);
	v &= ~(0xf << shift);
	v |= (brg << shift);
	gtu_write_mainreg(reg, v);
}


static void
gtu_abort_receive(struct gtuart *sc)
{
	int i;
	
	/* Write abort receive command and wait for the bit to clear */
	gtu_write_reg(sc->unit, GT_MPSC_CR, GT_MPSC_CR_RA);
#if defined(GT_MPSC_GOOD_REGS)
	while (gtu_read_reg(sc->unit, GT_MPSC_CR) & GT_MPSC_CR_RA);
#else	
	for (i = 0; i < 1000; ++i)
		gtu_read_reg(sc->unit, GT_MPSC_CR);
#endif	
}


static void
gtu_connect(struct gtuart *sc)
{
	uint32 v;

	debug(("gtu_connect:\n"));

	v = gtu_read_mainreg(GT_MPSC_MRR);
	v &= ~(0x7 << gtu_unit2mrrshift(sc->unit));
	v |= (GT_MPSC_MRR_MR_SERx << gtu_unit2mrrshift(sc->unit));
	gtu_write_mainreg(GT_MPSC_MRR, v);
}


#if 0
static void
gtu_set_databits(struct gtuart *sc, int bits)
{
	uint32 v;

	debug(("gtu_set_databits\n"));
	v = gtu_read_reg(sc->unit, GT_MPSC_PCR);
	v &= GT_MPSC_PCR_CL_MASK << GT_MPSC_PCR_CL_OFFSET;
	v |= bits << GT_MPSC_PCR_CL_OFFSET;
	gtu_write_reg(sc->unit, GT_MPSC_MPCR, v);
}

#endif


static int32
gtu_init(struct gtuart *sc, uint32 unit)
{
	assert(gtu_valid_unit(unit));

	sc->unit = unit;

#ifndef PROM_R9000
	/* Setup port multiplexor register */
	GT_REG_WRITE(0xf010, 0x1102);
#endif

	/* Set the port into UART mode, enable receiver and transmitter */
	gtu_write_reg(unit, GT_MPSC_MMCRL,
				  GT_MPSC_MMCRL_MODE_UART | GT_MPSC_MMCRL_ER
				  | GT_MPSC_MMCRL_ET | GT_MPSC_MMCRL_NLM);

	/* Connect this UART port to BRG<unit> */
	gtu_setbrg(unit, GT_MPSC_RCRR, GT_MPSC_TCRR_BRG0);
	gtu_setbrg(unit, GT_MPSC_TCRR, GT_MPSC_TCRR_BRG0);

	/* Initialize BRG0 */
	gtbrg_init(0);

	/* Stop any pending rx */
	gtu_abort_receive(sc);

	/* initialize SDMA */
	gtsdma_init(&sc->dma, unit);

	/* Set default baud rate */
	gtu_baud(sc, TTY_DEFAULT_RATE);

	/* Connect to serial port */
	gtu_connect(sc);

	/* allow byte-aligned buffers, 16 Tx clock divider, 16 rx clock divider,
	 16 byte preamble, 0xf preamble pattern */
	gtu_write_reg(unit, GT_MPSC_MMCRH,
				  GT_MPSC_MMCRH_RDW | (2 << GT_MPSC_MMCRH_TCDV_OFFSET) |
				  (2 << GT_MPSC_MMCRH_RCDV_OFFSET) |
				  (6 << GT_MPSC_MMCRH_TPL_OFFSET) |
				  (0xf << GT_MPSC_MMCRH_TPPT_OFFSET));

	gtu_write_reg(unit, GT_MPSC_UBSR, 0x003000ab);
	gtu_write_reg(unit, GT_MPSC_MIR, 0);

	gtu_write_reg(unit, GT_MPSC_CFR, 0);
	gtu_write_reg(unit, GT_MPSC_CHR5, 0);
	gtu_write_reg(unit, GT_MPSC_CHR6, 0);
	gtu_write_reg(unit, GT_MPSC_CHR7, 0);
	gtu_write_reg(unit, GT_MPSC_CHR8, 0);

#if RAL_XXX
	/* Set 7 data bits, 1 stop bit */
	gtu_write_reg(unit, GT_MPSC_PCR, GT_MPSC_PCR_CL_7);
#else
	gtu_write_reg(unit, GT_MPSC_PCR, GT_MPSC_PCR_CL_8);
#endif

	gtu_write_reg(unit, GT_MPSC_CR, GT_MPSC_CR_EH);

	/* Enable interrupts */
	gtu_write_intreg(unit, GT_MPSC_MASK, 0xffffffff);

	debug(("gtu_init: done: esr: %x\n", gtu_read_reg(unit, GT_MPSC_ESR)));

	return NFX_OK;
}


#if !defined(__OpenBSD__)

int32 gt64240_console(uint32 op, uint32 chan, int32 data)
{
	int rc = -1;
	struct gtuart *sc;
	

	if (gtu_valid_unit(chan) && (sc = &uarts[chan])) {
		switch (op) {
		case OP_INIT:
#if !defined(PROM) && !defined(EXCEPTIONS_IN_KSEG0)
			if (chan == 0)
				rc = gtu_init(sc, chan);
			else {
				sc->unit = chan;
				gtu_connect(sc);
			}
#else
			rc = gtu_init(sc, chan);
#endif
			break;
		case OP_BAUD:
			rc = gtu_baud(sc, data);
			break;
		case OP_TXRDY:
			rc = gtu_txrdy(sc);
			break;
		case OP_RXRDY:
			rc = gtu_rxrdy(sc);
			break;
		case OP_TX:
			rc = gtu_txchar(sc, data);
			break;
		case OP_RX:
			rc = gtu_rxchar(sc);
			break;
		}
	}

	return rc;
}

#ifdef R9000
/*
 * Redirects uart driver to gt SDMA uart.
 * Can only be done after GT & EEE are initialized.
 */
void
gt_console_init(void)
{
extern int32   (*console_driver)(uint32 op, uint32 chan, int32 data);
extern int32 tty1_printf (const char *fmt, ...);  //sm-libc printf.c

	gt64240_console(OP_INIT, 0, 0);
	gt64240_console(OP_INIT, 1, 0);
	console_driver = gt64240_console;
//4AndyS
// tty1_printf(" Hi Andy! This one is for you. Main Uart re-directed to SDMA\n");
tty1_printf(" \n\n -PROM 16552 Port 0 Reset \n\n\n\n");
 
}
#endif

#endif

int
gtu_baud(struct gtuart *sc, int baud)
{
	return gtbrg_setcdv(0, baud);
}


int
gtu_txrdy(struct gtuart *sc)
{
gtsdma_intr(sc);
	return sc->dma.tx.free != 0;
}


int
gtu_rxrdy(struct gtuart *sc)
{
	gtsdma_intr(&sc->dma);
	return sc->dma.rx.last != sc->dma.rx.first;
}


int
gtu_txchar(struct gtuart *sc, int data)
{
	char c = data;

	debug(("gtu_txchar: %c\n", c));

	return gtsdma_tx(&sc->dma, &c, 1);
}


int
gtu_rxchar(struct gtuart *sc)
{
	char c;

	gtsdma_rx(&sc->dma, &c, 1);
	return c;
}


#if defined(__OpenBSD__)

void
gtu_loopback_mode(struct gtuart *sc)
{
	gtu_write_reg(sc->unit, GT_MPSC_MMCRL,
				  gtu_read_reg(sc->unit, GT_MPSC_MMCRL) | GT_MPSC_MMCRL_LPBK);
}


void
gtu_normal_mode(struct gtuart *sc)
{
	gtu_write_reg(sc->unit, GT_MPSC_MMCRL,
				  gtu_read_reg(sc->unit, GT_MPSC_MMCRL) & ~GT_MPSC_MMCRL_LPBK);
}


int
gtuartopen(dev_t dev, int flag, int mode, struct proc *p)
{
	int rc = ENXIO;
	
	debug(("gtuartopen: %d\n", minor(dev)));
	
	if (minor(dev) < gtuart_cd.cd_ndevs) {
		
		rc = 0;
	}

	return rc;
}


int
gtuartclose(dev_t dev, int flag, int mode, struct proc *p)
{
	debug(("gtuartclose: %d\n", minor(dev)));
	
	return 0;
}


int
gtuartread(dev_t dev, struct uio *uio, int flag)
{
	return ENXIO;
}


int
gtuartwrite(dev_t dev, struct uio *uio, int flag)
{
	return ENXIO;
}


static int
do_ioctl(struct gtuart *sc, u_long cmd, caddr_t data, int flag, struct proc *p)
{
	int rc;
	
	debug(("do_ioctl: cmd=%x d=%x\n", cmd, data));

	switch (cmd) {
	case GTUART_LOOPBACK_MODE :
		gtu_loopback_mode(sc);
		rc = 0;
		break;
	case GTUART_NORMAL_MODE:
		gtu_normal_mode(sc);
		rc = 0;
		break;
	case GTUART_RXRDY:
		*data = gtu_rxrdy(sc);
		rc = 0;
		break;
	case GTUART_TXRDY:
		*data = gtu_txrdy(sc);
		rc = 0;
		break;
	case GTUART_WRITE:
		gtu_txchar(sc, *data);
		rc = 0;
		break;
	case GTUART_READ:
		*data = gtu_rxchar(sc);
		rc = 0;
		break;
	default:
		rc = EINVAL;
	}

	return rc;
}


int
gtuartioctl(dev_t dev, u_long cmd, caddr_t data, int flag,
			struct proc *p)
{
	int m = minor(dev);
	int rc = ENXIO;
	struct gtuart *sc;

	if (m < gtuart_cd.cd_ndevs && (sc = gtuart_cd.cd_devs[m]))
		rc = do_ioctl(sc, cmd, data, flag, p);

	return rc;
}


int
gtuartstop(struct tty *tp, int flag)
{
	return 0;
}


struct tty *
gtuarttty(dev_t dev)
{
	return 0;
}


#endif

#endif	/* !LINUX_TEST && !EVAL_BOARD && !SSC_MGMT && !SIBYTE */
