/*
 *-----------------------------------------------------------------
 *
 * Name:	ns16550.c
 *
 * RCS:  $Id: ns16550.c,v 1.6 2001/12/02 02:38:04 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */

#if (defined(EVAL_BOARD) && !defined(SIBYTE) && defined(PROM)) || defined(R9000)

#include "nfx-incl.h"

#include "../sm-serio/serio-api.h"
#include "../sm-hw/hwMap.h"
#include "sys/ns16550.h"
#include "sys/gt64440r.h"


extern void    delay_us(uint32);
extern int32   (*dbg_console_driver)(int op, int chan, int data);

static int nsfifo;

static int
nsinit (volatile ns16550dev *dp, int chan)
{

    /* force access to id reg */
    dp->cfcr = 0;
    dp->iir = 0;
    if ((dp->iir & 0x38) != 0) {
	return 1;
    }

    /* in case it really is a 16550, enable the fifos */
    dp->fifo = FIFO_ENABLE;

    delay_us(100);

    dp->fifo = FIFO_ENABLE | FIFO_RCV_RST | FIFO_XMT_RST | FIFO_TRIGGER_1;

    delay_us(100);

    if ((dp->iir & IIR_FIFO_MASK) == IIR_FIFO_MASK)
	nsfifo = 1;
    else {
	dp->fifo = 0;
    }

    return 0;
}


extern int	init_done;

static int
nsprogram (volatile ns16550dev *dp, int chan, int baudrate)
{
    unsigned int brtc;
    int timeout;

    /* wait for Tx fifo to completely drain */
    timeout = 10000;
    while (!(dp->lsr & LSR_TSRE))
	if (--timeout == 0)
	    break;

    if (baudrate == 0)
      return 1;

    brtc = BRTC (baudrate);

    dp->cfcr = CFCR_DLAB;
    dp->data = brtc & 0xff;
    dp->ier = brtc >> 8;
    dp->cfcr = CFCR_8BITS;
    dp->mcr = MCR_DTR | MCR_RTS | MCR_IENABLE;
    dp->ier = 0;

    return 0;

}

int
ns16550 (int op, int chan, int data)
{
    volatile ns16550dev *dp;

    dp = NS16550_GET_CHAN_BASE( chan );

    if (chan > 1)
	return 1;

    switch (op) {
    case OP_INIT:
	return nsinit (dp, chan);

    case OP_BAUD:
	return nsprogram (dp, chan, data);

    case OP_TXRDY:
	return (dp->lsr & LSR_TXRDY);

    case OP_TX:
	dp->data = data;  
	break;

    case OP_RXRDY:
	return (dp->lsr & LSR_RXRDY);

    case OP_RX:
	return dp->data & 0xff;

    case OP_RXSTOP:
	if (data)
	    dp->mcr &= ~MCR_RTS;	    /* disable Rx */
	else
	    dp->mcr |= MCR_RTS;	    	    /* enable Rx */
	break;
    }
    return 0;
}

// This is in run time ...
// put in for Andy  #3 4Andy
void
dbg_console_init(int chan)
{
                                                                                
        ns16550(OP_INIT, chan, 0);
        ns16550(OP_BAUD, chan, 57600);
        dbg_console_driver = ns16550;
}

#endif	/* EVAL_BOARD && !SIBYTE && PROM */

