/*
 *-----------------------------------------------------------------
 *
 * Name:	spinlock-macros.S
 *
 * RCS:  $Id: spinlock-macros.S,v 1.4 2002/12/09 23:54:18 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/19/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#if 0
	.globl	SPINLOCK_LOCK
	.globl	SPINLOCK_UNLOCK
	.globl	SPINLOCK_TRYLOCK
#endif

	.globl	ATOMIC_INC_INT
	.globl	ATOMIC_INC_DWORD
	.globl	ATOMIC_DEC_INT
	.globl	ATOMIC_DEC_DWORD

	.globl	ATOMIC_OR_INT
	.globl	ATOMIC_AND_INT
	.globl	ATOMIC_OR_DWORD
	.globl	ATOMIC_AND_DWORD

	.set	noreorder

#ifdef SIBYTE

#if 0
/*-----------------------------------------------------------------
 * Name:	SPINLOCK_LOCK(_lockp)
 *
 * Description:	Acquire a spinlock.
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	SPINLOCK_LOCK

SPINLOCK_LOCK:
	daddiu	sp,sp,-8
	sync
	ll	t0,0(a0)
	bne	t0,zero,.-4
	li	t0,1
	sc	t0,0(a0)
	beq	t0,zero,.-16
	nop
	j	ra
	daddiu	sp,sp,8

	.end	SPINLOCK_LOCK

/*-----------------------------------------------------------------
 * Name:	SPINLOCK_UNLOCK(_lockp)
 *
 * Description: Release a spinlock.
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	SPINLOCK_UNLOCK

SPINLOCK_UNLOCK:
	daddiu	sp,sp,-8
	sync
	sw	zero,0(a0)
	j	ra
	daddiu	sp,sp,8

	.end	SPINLOCK_UNLOCK

/*-----------------------------------------------------------------
 * Name: SPINLOCK_TRYLOCK(_lockp)
 *
 * Description: Try to acquire a spinlock.  If spinlock successfully acquired,
 *				return 1.  If fails to get spinlock, immediately
 *				return 0 (don't spin, waiting for lock).
 *
 *				_lockp is of nfx_spinlock_t type.
 *
 *				Returns 1 is spinlock is acquired, 0 if not acquired.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	SPINLOCK_TRYLOCK

SPINLOCK_TRYLOCK:
	daddiu	sp,sp,-8
	sync
	ll	v0,0(a0)
	bne	v0,zero,.+16
	move	v0,zero
	li	v0,1
	sc	v0,0(a0)
	j	ra
	daddiu	sp,sp,8

	.end	SPINLOCK_TRYLOCK
#endif

/*-----------------------------------------------------------------
 * Name: ATOMIC_INC_INT(_iptr, _step)
 *
 * Description:	Atomically increment the 32-bit contents of an address.
 *
 *				Treats the value as an unsigned integer (counter).
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_INC_INT

ATOMIC_INC_INT:
	daddiu	sp,sp,-8
	sync
	ll	t0,0(a0)
	addu	t0,t0,a1
	sc	t0,0(a0)
	beq	t0,zero,.-12
	nop
	j	ra
	daddiu	sp,sp,8

	.end	ATOMIC_INC_INT

/*-----------------------------------------------------------------
 * Name: ATOMIC_INC_DWORD(_iptr, _step)
 *
 * Description:	Atomically increment the 64-bit contents of an address.
 *
 *				Treats the value as an unsigned dword (counter).
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_INC_DWORD

ATOMIC_INC_DWORD:
	daddiu	sp,sp,-8
	sync
	lld	t0,0(a0)
	daddu	t0,t0,a1
	scd	t0,0(a0)
	beq	t0,zero,.-12
	nop
	j	ra
	daddiu	sp,sp,8

	.end	ATOMIC_INC_DWORD

/*-----------------------------------------------------------------
 * Name: ATOMIC_DEC_INT(_iptr, _step)
 *
 * Description:	Atomically decrement the 32-bit contents of an address.
 *
 *				Treats the value as an unsigned integer (counter).
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_DEC_INT

ATOMIC_DEC_INT:
	daddiu	sp,sp,-8
	sync
	ll	t0,0(a0)
	subu	t0,t0,a1
	sc	t0,0(a0)
	beq	t0,zero,.-12
	nop
	j	ra
	daddiu	sp,sp,8

	.end	ATOMIC_DEC_INT

/*-----------------------------------------------------------------
 * Name: ATOMIC_DEC_DWORD(_dwptr, _step)
 *
 * Description:	Atomically decrement the 64-bit contents of an address.
 *
 *				Treats the value as an unsigned dword (counter).
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_DEC_DWORD
ATOMIC_DEC_DWORD:
	daddiu	sp,sp,-8
	sync
	lld	t0,0(a0)
	dsubu	t0,t0,a1
	scd	t0,0(a0)
	beq	t0,zero,.-12
	nop
	j	ra
	daddiu	sp,sp,8

	.end	ATOMIC_DEC_DWORD

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/5/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_OR_INT
ATOMIC_OR_INT:
	daddiu	sp,sp,-8
	sync
	ll		v0,0(a0)
	or		t0,v0,a1
	sc		t0,0(a0)
	beq		t0,zero,.-12
	nop
	j		ra
	daddiu	sp,sp,8

	.end	ATOMIC_OR_INT

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/5/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_AND_INT
ATOMIC_AND_INT:
	daddiu	sp,sp,-8
	sync
	ll		v0,0(a0)
	and		t0,v0,a1
	sc		t0,0(a0)
	beq		t0,zero,.-12
	nop
	j		ra
	daddiu	sp,sp,8

	.end	ATOMIC_AND_INT

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/5/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_OR_DWORD
ATOMIC_OR_DWORD:
	daddiu	sp,sp,-8
	sync
	lld		v0,0(a0)
	or		t0,v0,a1
	scd		t0,0(a0)
	beq		t0,zero,.-12
	nop
	j		ra
	daddiu	sp,sp,8

	.end	ATOMIC_OR_DWORD

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/5/02
 *
 *-----------------------------------------------------------------
 */
	.ent	ATOMIC_AND_DWORD
ATOMIC_AND_DWORD:
	daddiu	sp,sp,-8
	sync
	lld		v0,0(a0)
	and		t0,v0,a1
	scd		t0,0(a0)
	beq		t0,zero,.-12
	nop
	j		ra
	daddiu	sp,sp,8

	.end	ATOMIC_AND_DWORD

#endif	/* SIBYTE */

