/*------------------------------------------------------------------------------
 * RCS:
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Name         : spm-srm.c
 *
 * Description  : This file contains interface to SRM
 *
 * Created by   : Nitin Jain
 * Date Created : 13-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

#include "spm.h"

/*
 * Global variables.
 */
extern char spmNodeName[SPM_MAX_NODE_NAME_LEN];
extern char currCluster[SPM_MAX_CLUSTER_NAME];

/*
 * Local function prototypes.
 */

/*
 * Global functions.
 */

/*
 *------------------------------------------------------------------------------
 * Name         : spm_sendSPMInitInd
 *
 * Description  : Send Init indication to SRM
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 13-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

int32 spm_sendSPMInitInd()
{
    spmSrm_MsgSpmInitInd_t req;
    int32 rc = 0;

    SPM_FCT_PRINT(("spm_sendSPMInitInd..."));

    bzero(&req, sizeof(spmSrm_MsgSpmInitInd_t));
    req.hdr.msgType = SPM_SRM_INIT_IND;
    req.hdr.clientPrivate  = getpid();
    strcpy(req.hdr.node, spmNodeName);
    strcpy(req.cluster, currCluster);

    rc = sendAgileMsg(-1, &req, sizeof(spmSrm_MsgSpmInitInd_t),
                      "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                      NULL, NULL, 0);

    SPM_FCT_PRINT(("...spm_sendSPMInitInd"));

    return rc;
}

/*
 *------------------------------------------------------------------------------
 * Name         : spm_sendNewLunInd
 *
 * Description  : Send new LUN indication
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 13-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

int32 spm_sendNewLunInd(spm_lunAttr_t *attr, char *devNode)
{
    spmSrm_MsgNewLunInd_t req;
    int32 rc = 0;

    SPM_FCT_PRINT(("spm_sendNewLunInd..."));

    bzero(&req, sizeof(spmSrm_MsgNewLunInd_t));
    req.hdr.msgType = SPM_SRM_NEW_LUN_IND;
    req.hdr.clientPrivate  = getpid();
    strcpy(req.hdr.node, spmNodeName);
    strcpy(req.node, devNode);
    strcpy(req.cluster, currCluster);
    req.attr.scsiId = attr->scsiId;
    req.attr.wwn    = attr->wwn;
    req.attr.lun    = attr->lun;
    req.attr.raidLevel = attr->raidLevel;
    strcpy(req.attr.name, attr->name);
    strcpy(req.attr.vendor, attr->vendor);
    strcpy(req.attr.model, attr->model);
    req.attr.size  = attr->size;
    switch(attr->state)
    {
        case EVM_FREE_LUN :
            req.attr.state = SPM_LUN_FREE;
            break;
        case EVM_USED_LUN :
            req.attr.state = SPM_LUN_USED;
            break;
        case EVM_OC_FREE_LUN :
        case EVM_OC_USED_LUN :
            req.attr.state = SPM_LUN_OC;
            break;
        case EVM_FOREIGN_LUN :
            req.attr.state = SPM_LUN_FOREIGN;
            break;
    }

    rc = sendAgileMsg(-1, &req, sizeof(spmSrm_MsgNewLunInd_t),
                      "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                      NULL, NULL, 0);

    SPM_FCT_PRINT(("...spm_sendNewLunInd"));

    return rc;
}

/*
 *------------------------------------------------------------------------------
 * Name         : spm_sendDelLunInd
 *
 * Description  : Send delete LUN indication
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 13-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

int32 spm_sendDelLunInd(char *devName, char *devNode)
{
    spmSrm_MsgDelLunInd_t req;
    int32 rc = 0;

    SPM_FCT_PRINT(("spm_sendDelLunInd..."));

    bzero(&req, sizeof(spmSrm_MsgDelLunInd_t));
    req.hdr.msgType = SPM_SRM_DEL_LUN_IND;
    req.hdr.clientPrivate  = getpid();
    strcpy(req.hdr.node, spmNodeName);
    strcpy(req.node, devNode);
    strcpy(req.cluster, currCluster);
    strcpy(req.devName, devName);

    rc = sendAgileMsg(-1, &req, sizeof(spmSrm_MsgDelLunInd_t),
                      "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                      NULL, NULL, 0);

    SPM_FCT_PRINT(("...spm_sendDelLunInd"));

    return rc;
}

/*
 *------------------------------------------------------------------------------
 * Name         : spm_sendLunChangeStateInd
 *
 * Description  : Send LUN change state indication
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 13-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

int32 spm_sendLunChangeStateInd(char *dev, spm_lunState_t frmState,
                                           spm_lunState_t toState)
{
    spm_MsgLunChangeStateInd_t *req;
    int32 reqSize = 0;
    int32 rc = 0;

    SPM_FCT_PRINT(("spm_sendLunChangeStateInd..."));

    reqSize = sizeof(spm_MsgLunChangeStateInd_t);
    SPM_ALLOC_ZERO(req, spm_MsgLunChangeStateInd_t, reqSize);
    req->hdr.msgType = SPM_SRM_LUN_STATE_CHANGE_IND;
    req->hdr.clientPrivate  = getpid();
    strcpy(req->hdr.node, spmNodeName);
    strcpy(req->cluster, currCluster);
    req->devCnt = 1;
    req->dev[0].frmState = frmState;
    req->dev[0].toState  = toState;
    strcpy(req->dev[0].name, dev);

    rc = sendAgileMsg(-1, req, reqSize,
                      "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                      NULL, NULL, 0);

    SPM_FREE(req);
    SPM_FCT_PRINT(("...spm_sendLunChangeStateInd"));

    return rc;
}
