//------------------------------------------------------------------------------
// RCS:  $Id: stats-fc.c,v 1.4 2002/11/15 22:16:06 syen Exp $
// Copyright (C) 2002, Agile Storage, Inc.
//
// Name         : stats-fc.c
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/2002
//
//------------------------------------------------------------------------------


#if defined(FCNIM)

#include "stats.h"
#include "stats-dbg.h"

//------------------------------------------------------------------------------
// Global variables
//------------------------------------------------------------------------------

stats_fcDp83820Stats_t	fcDp83820Stats;
stats_fcGt64240Stats_t	fcGt64240Stats;

typedef struct _dp83820Stats_t
{
	uint32 rxErrors;
	uint32 rxFcsErrs;
	uint32 rxOverruns;
	uint32 rxAlignErrs;
	uint32 rxSymErrs;
	uint32 rxLenErrs;
	uint32 rxPkts;
	uint32 rxBytes;
	uint32 rxAborts;
	uint32 rxNoBufErrs;
	uint32 rxNoSysDesc;
	uint32 rxRuntPkts;
	uint32 rxCrcErrs;
	uint32 txPkts;
	uint32 txBytes;
	uint32 txQueueFull;
	uint32 txAborts;
	uint32 txUnderruns;
} dp83820Stats_t;

typedef struct _gt64240miiStats_t
{
	uint32 rxBytes;
	uint32 txBytes;
	uint32 rxPkts;
	uint32 txPkts;
	uint32 rxCrcErrs;
	uint32 rxFragments;
	uint32 rxJabbers;
	uint32 rxCollisions;
	uint32 rxLateCollis;
	uint32 rxErrors;
	uint32 rxUndersized;
	uint32 rxNoSysDesc;
	uint32 rxNoBufErrs;
	uint32 txUnderruns;
	uint32 txQueueFull;
} gt64240miiStats_t;

static unsigned int statsStep = 0;


// Local functions prototype.
static void stats_GetDp83820Stats(void);
static void stats_GetGt64240Stats(void);

// external functions prototype
void dp83820_get_stats(uint32 *bufptr);
void gt64240mii_get_stats(uint32 *bufptr);


//------------------------------------------------------------------------------
// Name         : init_statsCounters()
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

void
init_statsCounters()
{
	memset(&fcDp83820Stats, 0, sizeof(stats_fcDp83820Stats_t));
	memset(&fcGt64240Stats, 0, sizeof(stats_fcGt64240Stats_t));
}


//------------------------------------------------------------------------------
// Name         : stats_GetDp83820Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

static void
stats_GetDp83820Stats()
{
dp83820Stats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetDp83820Stats()\n");

	memset(&stats, 0, sizeof(dp83820Stats_t));

	dp83820_get_stats((uint32 *) &stats);

	fcDp83820Stats.rxBytes		+= (uint64) stats.rxBytes;
	fcDp83820Stats.rxPkts		+= (uint64) stats.rxPkts;
	fcDp83820Stats.rxErrors		+= (uint64) stats.rxErrors;
	fcDp83820Stats.rxFcsErrs	+= (uint64) stats.rxFcsErrs;
	fcDp83820Stats.rxOverruns	+= (uint64) stats.rxOverruns;
	fcDp83820Stats.rxAlignErrs	+= (uint64) stats.rxAlignErrs;
	fcDp83820Stats.rxSymbolErrs	+= (uint64) stats.rxSymErrs;
	fcDp83820Stats.rxLenErrs	+= (uint64) stats.rxLenErrs;
	fcDp83820Stats.rxAborts		+= (uint64) stats.rxAborts;
	fcDp83820Stats.rxNoBufErrs	+= (uint64) stats.rxNoBufErrs;
	fcDp83820Stats.rxNoSysDescs	+= (uint64) stats.rxNoSysDesc;
	fcDp83820Stats.rxRuntPkts	+= (uint64) stats.rxRuntPkts;
	fcDp83820Stats.rxCrcErrs	+= (uint64) stats.rxCrcErrs;
	fcDp83820Stats.txBytes		+= (uint64) stats.txBytes;
	fcDp83820Stats.txPkts		+= (uint64) stats.txPkts;
	fcDp83820Stats.txQFullDrops	+= (uint64) stats.txQueueFull;
	fcDp83820Stats.txAborts		+= (uint64) stats.txAborts;
	fcDp83820Stats.txUnderruns	+= (uint64) stats.txUnderruns;

	stats_trace(LOG_DEBUG, "< stats_GetDp83820Stats()\n");
}


//------------------------------------------------------------------------------
// Name         : stats_GetGt64240Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

static void
stats_GetGt64240Stats()
{
gt64240miiStats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetGt64240Stats()\n");

	memset(&stats, 0, sizeof(gt64240miiStats_t));
	gt64240mii_get_stats((uint32 *) &stats);

	fcGt64240Stats.rxBytes		+= (uint64) stats.rxBytes;
	fcGt64240Stats.rxPkts		+= (uint64) stats.rxPkts;
	fcGt64240Stats.rxCrcErrs	+= (uint64) stats.rxCrcErrs;
	fcGt64240Stats.rxFragments	+= (uint64) stats.rxFragments;
	fcGt64240Stats.rxJabbers	+= (uint64) stats.rxJabbers;
	fcGt64240Stats.rxCollisions	+= (uint64) stats.rxCollisions;
	fcGt64240Stats.rxLateCollis	+= (uint64) stats.rxLateCollis;
	fcGt64240Stats.rxErrors		+= (uint64) stats.rxErrors;
	fcGt64240Stats.rxUndersized	+= (uint64) stats.rxUndersized;
	fcGt64240Stats.rxNoSysDescs	+= (uint64) stats.rxNoSysDesc;
	fcGt64240Stats.rxNoBufErrs	+= (uint64) stats.rxNoBufErrs;

	fcGt64240Stats.txBytes		+= (uint64) stats.txBytes;
	fcGt64240Stats.txPkts		+= (uint64) stats.txPkts;
	fcGt64240Stats.txQFullDrops	+= (uint64) stats.txQueueFull;
	fcGt64240Stats.txUnderruns	+= (uint64) stats.txUnderruns;

	stats_trace(LOG_DEBUG, "< stats_GetGt64240Stats()\n");
	return;
}


//------------------------------------------------------------------------------
// Name         : statsTimerProc
// 
// Description  : Callback function when timer expires.
//                Timer wake up every 5 seconds for FP stats collector.
//                This is defined in stats-fp.h with STATS_TICKS.
//                For each wake up, get the specified stats below. After
//                all 4 stats are finished, we cycle it again...
//                0. dp83820
//                1. gt64240mii
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

int32
statsTimerProc(int32 tmr_handle,
	void *usr_param1, void *usr_param2, void *usr_param3, void *usr_param4)
{
	stats_trace(LOG_DEBUG, "> statsTimerProc() - step %d\n", statsStep);

#if 1
// this code should be removed for formal release
	if (!collect_data)
		return NFX_OK;
#endif

	if (statsStep & 0x00000001)
		stats_GetDp83820Stats();
	else
		stats_GetGt64240Stats();

	statsStep++;

	stats_trace(LOG_DEBUG, "< statsTimerProc()\n");

	return NFX_OK;
}


//------------------------------------------------------------------------------
// Name         : stats_processGetStats
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

int32
stats_getStats(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_GetFCStatsReq_t	*req = (stats_GetFCStatsReq_t *) msgBuf;
	stats_GetFCStatsRsp_t	*rsp;
	uint32					rspSize, dataSize = 0;
	char					*ptr = NULL;

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");

	NFX_ASSERT(msgLen >= sizeof(stats_GetFCStatsReq_t));

	switch (req->type)
	{
	case STATS_FC_DP83820:
		dataSize = sizeof(stats_fcDp83820Stats_t);
		ptr = (char *) &fcDp83820Stats;
		break;
	case STATS_FC_GT64240:
		dataSize = sizeof(stats_fcGt64240Stats_t);
		ptr = (char *) &fcGt64240Stats;
		break;
	}

	rspSize = sizeof(stats_GetFCStatsRsp_t) + dataSize;
	rsp = (stats_GetFCStatsRsp_t *) eee_ramAllocZero(rspSize);
	NFX_ASSERT(rsp != NULL);

	rsp->hdr.msgType = req->hdr.msgType;
	rsp->hdr.clientPrivate = req->hdr.clientPrivate;
	rsp->type = req->type;

	if (ptr)
	{
		rsp->status = NFX_OK;
		NFX_SPIN_LOCK(&stats_lock);
		memcpy(rsp->data, ptr, dataSize);
		NFX_SPIN_UNLOCK(&stats_lock);
	}
	else
		rsp->status = NFX_ERR;

	eee_sendMessage(rsp, rspSize, srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");
	return NFX_OK;
}

//------------------------------------------------------------------------------
// Name         : stats_resetCounters
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

void
stats_resetCounters(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_resetCountersReq_t	*req = (stats_resetCountersReq_t *) msgBuf;
	stats_resetCountersRsp_t	rsp;

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");

	NFX_ASSERT(msgLen >= sizeof(stats_resetCountersReq_t));

	rsp.hdr.msgType = req->hdr.msgType;
	rsp.hdr.clientPrivate = req->hdr.clientPrivate;
	rsp.status = NFX_OK;
	rsp.type = req->type;

	NFX_SPIN_LOCK(&stats_lock);
	switch (req->type)
	{
	case STATS_FC_DP83820:
		memset(&fcDp83820Stats, 0, sizeof(stats_fcDp83820Stats_t));
		break;
	case STATS_FC_GT64240:
		memset(&fcGt64240Stats, 0, sizeof(stats_fcGt64240Stats_t));
		break;
	default:
		rsp.status = NFX_ERR;
		break;
	}
	NFX_SPIN_UNLOCK(&stats_lock);

	eee_sendMessage(&rsp, sizeof(stats_resetCountersRsp_t), srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");
	return;
}

#endif		// defined(FCNIM)
