//------------------------------------------------------------------------------
// RCS:  $Id: stats-sscmgmt.c,v 1.4 2002/11/22 23:22:00 syen Exp $
// Copyright (C) 2002, Agile Storage, Inc.
//
// Name         : stats-sscmgmt.c
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/2002
//
//------------------------------------------------------------------------------


#if defined(SSC_MGMT)

#include "stats.h"
#include "stats-dbg.h"

//------------------------------------------------------------------------------
// Global variables
//------------------------------------------------------------------------------

stats_sscmgmtBcm5600Stats_t	sscmgmtBcm5600Stats[STATS_SSCMGMT_TOTAL_PORTS];
static int sscmgmtPort[STATS_SSCMGMT_TOTAL_PORTS] =
{
	STATS_SSCMGMT_PORT_FC0,
	STATS_SSCMGMT_PORT_FC1,
	STATS_SSCMGMT_PORT_FC2,
	STATS_SSCMGMT_PORT_FC3,
	STATS_SSCMGMT_PORT_TXRX,
	STATS_SSCMGMT_PORT_SSC
};

typedef struct _bcm5600Stats_t
{
	uint32	rxBytes;
	uint32	rxPkts;
	uint32	rxSymbolErrs;
	uint32	rxAlignErrs;
	uint32	rxDiscards;
	uint32	rxFalseCarrs;
	uint32	rxFcsErrs;
	uint32	rxUndersized;
	uint32	rxJabbers;
	uint32	rxFragments;
	uint32	txPkts;
	uint32	txBytes;
	uint32	txFcsErrs;
	uint32	txAborts;
	uint32	txFragments;
	uint32	txJabbers;
} bcm5600Stats_t;


// Local functions prototype.
static void stats_GetBcm5600Stats(int port);

// external functions prototype
void bcm5600_get_stats(int port, uint32 *bufptr);
int cm_checkFanFail(void);
int cm_checkPowerFail(void);


//------------------------------------------------------------------------------
// Name         : init_statsCounters()
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

void
init_statsCounters()
{
	memset(sscmgmtBcm5600Stats, 0,
			sizeof(stats_sscmgmtBcm5600Stats_t)*STATS_SSCMGMT_TOTAL_PORTS);
}


//------------------------------------------------------------------------------
// Name         : stats_GetBcm5600Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

static void
stats_GetBcm5600Stats(int port)
{
bcm5600Stats_t stats;
int index = 0;

	stats_trace(LOG_DEBUG, "> stats_GetBcm5600Stats()\n");

	memset(&stats, 0, sizeof(bcm5600Stats_t));
	bcm5600_get_stats(port, (uint32 *) &stats);

	switch (port)
	{
	case STATS_SSCMGMT_PORT_FC0:
	case STATS_SSCMGMT_PORT_FC1:
	case STATS_SSCMGMT_PORT_FC2:
	case STATS_SSCMGMT_PORT_FC3:
	case STATS_SSCMGMT_PORT_TXRX:
		index = port;
		break;
	case STATS_SSCMGMT_PORT_SSC:
		index = 5;
		break;
	}
	sscmgmtBcm5600Stats[index].rxBytes		+= (uint64) stats.rxBytes;
	sscmgmtBcm5600Stats[index].rxPkts		+= (uint64) stats.rxPkts;
	sscmgmtBcm5600Stats[index].rxFcsErrs	+= (uint64) stats.rxFcsErrs;
	sscmgmtBcm5600Stats[index].rxAlignErrs	+= (uint64) stats.rxAlignErrs;
	sscmgmtBcm5600Stats[index].rxSymbolErrs	+= (uint64) stats.rxSymbolErrs;
	sscmgmtBcm5600Stats[index].rxFalseCarrs	+= (uint64) stats.rxFalseCarrs;
	sscmgmtBcm5600Stats[index].rxUndersized	+= (uint64) stats.rxUndersized;
	sscmgmtBcm5600Stats[index].rxFragments	+= (uint64) stats.rxFragments;
	sscmgmtBcm5600Stats[index].rxJabbers	+= (uint64) stats.rxJabbers;
	sscmgmtBcm5600Stats[index].rxDiscards	+= (uint64) stats.rxDiscards;

	sscmgmtBcm5600Stats[index].txBytes		+= (uint64) stats.txBytes;
	sscmgmtBcm5600Stats[index].txPkts		+= (uint64) stats.txPkts;
	sscmgmtBcm5600Stats[index].txFcsErrs	+= (uint64) stats.txFcsErrs;
	sscmgmtBcm5600Stats[index].txAborts		+= (uint64) stats.txAborts;
	sscmgmtBcm5600Stats[index].txFragments	+= (uint64) stats.txFragments;
	sscmgmtBcm5600Stats[index].txJabbers	+= (uint64) stats.txJabbers;

	stats_trace(LOG_DEBUG, "< stats_GetBcm5600Stats()\n");
	return;
}


//------------------------------------------------------------------------------
// Name         : statsTimerProc
// 
// Description  : Callback function when timer expires.
//                Timer wake up every .75 seconds for SSC_MGMT stats collector.
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

int32
statsTimerProc(int32 tmr_handle,
	void *usr_param1, void *usr_param2, void *usr_param3, void *usr_param4)
{
int i;

	stats_trace(LOG_DEBUG, "> statsTimerProc()\n");

#if 1
// this code should be removed for formal release
	if (!collect_data)
		return NFX_OK;
#endif

	for (i = 0; i < STATS_SSCMGMT_TOTAL_PORTS; i++)
		stats_GetBcm5600Stats(sscmgmtPort[i]);

#if 0
// for traps...
static int fanFailNotified = 0;
static int powerFailNotified = 0;

	if (cm_checkFanFail())
	{
		if (!fanFailNotified)
		{
			// notify code here...
			fanFailNotified = 1;
		}
	}
	else
		fanFailNotified = 0;

	if (cm_checkPowerFail())
	{
		if (!powerFailNotified)
		{
			// notify code here...
			powerFailNotified = 1;
		}
	}
	else
		powerFailNotified = 0;

#endif

	stats_trace(LOG_DEBUG, "< statsTimerProc()\n");

	return NFX_OK;
}


//------------------------------------------------------------------------------
// Name         : stats_processGetStats
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

int32
stats_getStats(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_GetSscMgmtStatsReq_t	*req = (stats_GetSscMgmtStatsReq_t *) msgBuf;
	stats_GetSscMgmtStatsRsp_t	*rsp;
	uint32						rspSize, dataSize = 0;
	char						*ptr = NULL;

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");

	NFX_ASSERT(msgLen >= sizeof(stats_GetSscMgmtStatsReq_t));

	switch (req->type)
	{
	case STATS_SSCMGMT_BCM5600:
		switch (req->port)
		{
		case STATS_SSCMGMT_PORT_FC0:
		case STATS_SSCMGMT_PORT_FC1:
		case STATS_SSCMGMT_PORT_FC2:
		case STATS_SSCMGMT_PORT_FC3:
		case STATS_SSCMGMT_PORT_TXRX:
			dataSize = sizeof(stats_sscmgmtBcm5600Stats_t);
			ptr = (char *) &sscmgmtBcm5600Stats[req->port];
			break;
		case STATS_SSCMGMT_PORT_SSC:
			dataSize = sizeof(stats_sscmgmtBcm5600Stats_t);
			ptr = (char *) &sscmgmtBcm5600Stats[5];
			break;
		}
		break;
	}

	rspSize = sizeof(stats_GetSscMgmtStatsRsp_t) + dataSize;
	rsp = (stats_GetSscMgmtStatsRsp_t *) eee_ramAllocZero(rspSize);
	NFX_ASSERT(rsp != NULL);

	rsp->hdr.msgType = req->hdr.msgType;
	rsp->hdr.clientPrivate = req->hdr.clientPrivate;
	rsp->type = req->type;
	rsp->port = req->port;

	if (ptr)
	{
		rsp->status = NFX_OK;
		NFX_SPIN_LOCK(&stats_lock);
		memcpy(rsp->data, ptr, dataSize);
		NFX_SPIN_UNLOCK(&stats_lock);
	}
	else
		rsp->status = NFX_ERR;

	eee_sendMessage(rsp, rspSize, srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");
	return NFX_OK;
}

//------------------------------------------------------------------------------
// Name         : stats_resetCounters
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

void
stats_resetCounters(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_resetCountersReq_t	*req = (stats_resetCountersReq_t *) msgBuf;
	stats_resetCountersRsp_t	rsp;

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");

	NFX_ASSERT(msgLen >= sizeof(stats_resetCountersReq_t));

	rsp.hdr.msgType = req->hdr.msgType;
	rsp.hdr.clientPrivate = req->hdr.clientPrivate;
	rsp.status = NFX_OK;
	rsp.type = req->type;

	NFX_SPIN_LOCK(&stats_lock);
	switch (req->type)
	{
	case STATS_SSCMGMT_BCM5600:
		switch (req->port)
		{
		case STATS_SSCMGMT_PORT_FC0:
			memset(&sscmgmtBcm5600Stats[0], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		case STATS_SSCMGMT_PORT_FC1:
			memset(&sscmgmtBcm5600Stats[1], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		case STATS_SSCMGMT_PORT_FC2:
			memset(&sscmgmtBcm5600Stats[2], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		case STATS_SSCMGMT_PORT_FC3:
			memset(&sscmgmtBcm5600Stats[3], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		case STATS_SSCMGMT_PORT_TXRX:
			memset(&sscmgmtBcm5600Stats[4], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		case STATS_SSCMGMT_PORT_SSC:
			memset(&sscmgmtBcm5600Stats[5], 0,
					sizeof(stats_sscmgmtBcm5600Stats_t));
			break;
		default:
			rsp.status = NFX_ERR;
			break;
		}
		break;
	default:
		rsp.status = NFX_ERR;
		break;
	}
	NFX_SPIN_UNLOCK(&stats_lock);

	eee_sendMessage(&rsp, sizeof(stats_resetCountersRsp_t), srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");
	return;
}

#endif		// defined(SSC_MGMT)
