/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-common.h
 *
 * RCS:		$Id: tape-common.h,v 1.13 2002/11/07 23:20:57 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver common definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 9/29/01
 *
 *------------------------------------------------------------------------------
 */
#ifndef __TAPE_COMMON_H__
#define __TAPE_COMMON_H__

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/time.h>

#include <nfx-target.h>
#include <nfx-error.h>
#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/debug.h>
#include <exports/sdm-api.h>
#include <exports/anpssc-api.h>

#define TAPE_VENDOR_NAME_LEN	(SDM_VENDOR_NAME_LEN+1)
#define TAPE_MODEL_NAME_LEN	(SDM_MODEL_NAME_LEN+1)
#define TAPE_DENSITY_NAME_LEN	32
#define TAPE_DENSITY_COUNT	4

#define TAPE_MAX_PHYSICAL	64

#include "tape-api.h"
#include "tape-cap.h"
#include "tape-sm.h"
#include "tape-debug.h"

void panic(char *, ...);

tape_dev_t *tape_device_new(scsi_id_t, uint64 wwn, uint32 lun, char *, char *);
tape_dev_t *tape_device_lookup_by_id(scsi_id_t);
tape_dev_t *tape_device_lookup_by_client_tag(uint32, char *);
tape_dev_t *tape_device_lookup_by_alias(int32);
tape_dev_t *tape_device_lookup_by_handle(uint32);
tape_dev_t *tape_device_lookup_by_index(ushort16);
void        tape_device_destroy(tape_dev_t *);

extern tape_dev_t * tape_devs;

#define SWAP4(_p_) \
	{ \
		  *((uint32 *)(_p_)) = ((*((uint32 *)(_p_)) >> 24) & 0x000000FFL | \
				        (*((uint32 *)(_p_)) >>  8) & 0x0000FF00L | \
				        (*((uint32 *)(_p_)) <<  8) & 0x00FF0000L | \
				        (*((uint32 *)(_p_)) << 24) & 0xFF000000L); \
	}

#endif /* __TAPE_COMMON_H__ */
