/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-sm.c
 *
 * RCS:		$Id: tape-sm.c,v 1.39 2002/11/12 19:28:22 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver device state machines.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 10/1/01
 *
 *------------------------------------------------------------------------------
 */

#include "tape-common.h"
#include "tape-device.h"
#include "tape-msg.h"
#include "../sm-scsi/scsi-dev-cmds.h"

void tape_do_open_reply_busy(tape_dev_t *, void *);
void tape_do_open_reply_success(tape_dev_t *, void *);
void tape_do_open_reply_aborted(tape_dev_t *, void *);
void tape_do_open_reply_io_error(tape_dev_t *, void *);
void tape_do_close_reply_not_open(tape_dev_t *, void *);
void tape_do_reject_serf_close(tape_dev_t *, void *);
void tape_do_close_reply_success(tape_dev_t *, void *);
void tape_save_close_and_do_sdm_close(tape_dev_t *, void *);
void tape_do_hold_close(tape_dev_t *, void *);

void tape_do_drop_relay_and_scream(tape_dev_t *, void *);

void tape_do_sdm_open_failed(tape_dev_t *, void *);

void tape_do_sdm_open(tape_dev_t *, void *);
void tape_do_sdm_info(tape_dev_t *, void *);
void tape_do_sdm_close(tape_dev_t *, void *);
void tape_do_tur_after_sdm_open(tape_dev_t *, void *);
void tape_do_tur_after_sdm_info(tape_dev_t *, void *); 
void tape_do_tur_retry(tape_dev_t *, void *); 
void tape_do_mode_sense_compression(tape_dev_t *, void *);
void tape_do_mode_select_compression(tape_dev_t *, void *);
void tape_do_mode_select_dlteeprom(tape_dev_t *, void *);
void tape_do_load(tape_dev_t *, void *);
void tape_do_unload(tape_dev_t *, void *);
void tape_do_rewind(tape_dev_t *, void *);
void tape_do_read_position(tape_dev_t *, void *);

void tape_do_get_state_reply_imm(tape_dev_t *, void *);
void tape_do_get_state_reply_imm_invalid(tape_dev_t *, void *);
void tape_do_get_state_reply_success(tape_dev_t *, void *);
void tape_do_get_state_reply_success_and_rewind(tape_dev_t *, void *);
void tape_do_get_state_reply_failed_and_rewind(tape_dev_t *, void *);
void tape_do_get_state_reply_failed(tape_dev_t *, void *);
void tape_do_get_state_reply_not_open(tape_dev_t *, void *);
void tape_do_get_state_reply_failed_and_reply_close_success(tape_dev_t *, void *);

void tape_do_message_req(tape_dev_t *, void *);
void tape_do_message_req_reply_success(tape_dev_t *, void *);
void tape_do_message_req_reply_io_error(tape_dev_t *, void *);
void tape_do_message_req_reply_busy(tape_dev_t *, void *);
void tape_do_message_req_reply_not_open(tape_dev_t *, void *);
void tape_do_message_req_reply_success_and_rewind(tape_dev_t *, void *);
void tape_do_message_req_reply_failed_and_rewind(tape_dev_t *, void *);
void tape_do_message_req_reply_success_and_reply_close_success(tape_dev_t *, void *);
void tape_do_message_req_reply_failed_and_reply_close_success(tape_dev_t *, void *);

void mc_do_open_reply_success_after_sdm_open(tape_dev_t *, void *);
void mc_do_open_reply_success_after_sdm_info(tape_dev_t *, void *);
void mc_save_close_and_do_sdm_close(tape_dev_t *, void *);
void mc_do_message_req_reply_success_and_sdm_close(tape_dev_t *, void *);
void mc_do_message_req_reply_failed_and_sdm_close(tape_dev_t *, void *);
void mc_do_sdm_close(tape_dev_t *, void *);

tape_sm_t tape_dev_sm[TAPE_STATE_COUNT] = {
{
	TAPE_STATE_CLOSED,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_SDM_OPEN_PENDING,			tape_do_sdm_open },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_CLOSED,				tape_do_drop_relay_and_scream },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_CLOSED,				tape_do_drop_relay_and_scream },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_CLOSED,				tape_do_drop_relay_and_scream },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_CLOSED,				tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_CLOSED,				tape_do_close_reply_not_open },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_CLOSED,				tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_TERMINAL,				NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_CLOSED,				tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_CLOSED,				tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_CLOSED,				tape_do_close_reply_not_open },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_SDM_OPEN_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_SDM_OPEN_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_TUR_PENDING,				tape_do_tur_after_sdm_open },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_CLOSED,				tape_do_sdm_open_failed },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_SDM_OPEN_PENDING,			tape_do_sdm_info },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_TUR_PENDING,				tape_do_tur_after_sdm_info },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_CLOSED,				tape_do_sdm_open_failed },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_OPEN_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_SDM_OPEN_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_SDM_OPEN_PENDING,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_OPEN_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_OPEN,				mc_do_open_reply_success_after_sdm_open },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_CLOSED,				tape_do_sdm_open_failed },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_SDM_OPEN_PENDING,			tape_do_sdm_info },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_OPEN,				mc_do_open_reply_success_after_sdm_info },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_CLOSED,				tape_do_sdm_open_failed },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_TUR_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_TUR_PENDING,				tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_mode_sense_compression }, 
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_LOAD_PENDING,			tape_do_load },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_TUR_PENDING,				tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_TUR_PENDING,				tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_TUR_PENDING,				tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_TUR_PENDING,				tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_LOAD_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_LOAD_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_mode_sense_compression }, 
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_LOAD_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_LOAD_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_LOAD_PENDING,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_LOAD_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_mode_select_compression },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_mode_select_dlteeprom },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_OPEN,				tape_do_open_reply_success },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_CLOSED,				tape_do_open_reply_io_error },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_CLOSED,				tape_do_open_reply_io_error },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	} 
},
{
	TAPE_STATE_OPEN,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN,				tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_REWIND_PENDING,			tape_do_rewind },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN,				tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ACTIVE,				tape_do_message_req },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_READ_POSITION_PENDING,		tape_do_read_position },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING,			mc_do_sdm_close },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN,				tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_OPEN_ACTIVE,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN_ACTIVE,				tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_OPEN,				tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_OPEN,				tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_OPEN,				tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_OPEN,				tape_do_message_req_reply_success },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_HOLDING_CLOSE,			tape_do_hold_close },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ACTIVE,				tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ACTIVE,				tape_do_message_req_reply_busy },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING,			NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_OPEN_ACTIVE,				tape_do_get_state_reply_imm },
		{ MC_EVENT_CLOSE_REQ,			MC_STATE_HOLDING_CLOSE,				tape_do_hold_close },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ACTIVE,				tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_HOLDING_CLOSE, 
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_HOLDING_CLOSE,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_failed_and_rewind },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_failed_and_rewind },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_REWIND_PENDING,                      tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_REWIND_PENDING,                      tape_do_message_req_reply_failed_and_rewind },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_REWIND_PENDING,                      tape_do_message_req_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_HOLDING_CLOSE,			tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_HOLDING_CLOSE,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_HOLDING_CLOSE,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_HOLDING_CLOSE,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_HOLDING_CLOSE,			tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_HOLDING_CLOSE,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_READ_POSITION_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_READ_POSITION_PENDING,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_OPEN,				tape_do_get_state_reply_success },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_OPEN,				tape_do_get_state_reply_failed },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_hold_close },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_READ_POSITION_PENDING,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_READ_POSITION_PENDING,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_READ_POSITION_PENDING,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_hold_close },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_READ_POSITION_PENDING,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_REWIND_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_REWIND_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_UNLOAD_PENDING,			tape_do_unload },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_UNLOAD_PENDING,			tape_do_unload }, 
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_UNLOAD_PENDING,			tape_do_unload }, 
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_REWIND_PENDING,			tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_REWIND_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_REWIND_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_CLOSE_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_REWIND_PENDING,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_REWIND_PENDING,			tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_REWIND_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_UNLOAD_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_UNLOAD_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_UNLOAD_PENDING,			tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_UNLOAD_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_UNLOAD_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_CLOSE_PENDING,		NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_UNLOAD_PENDING,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_UNLOAD_PENDING,			tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_UNLOAD_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_SDM_CLOSE_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_CLOSED,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_CLOSED,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_CLOSED,				tape_do_close_reply_success },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_CLOSED,				tape_do_close_reply_success },
	}
},
{
	TAPE_STATE_DESTROY_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_PENDING,			tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_io_error },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_PENDING,			tape_do_close_reply_success },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_PENDING,			tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_PENDING,			tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING,			NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_PENDING,			tape_do_get_state_reply_failed },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_PENDING,			tape_do_close_reply_success },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_PENDING,			tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_get_state_reply_failed },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_get_state_reply_failed },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_save_close_and_do_sdm_close },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_message_req_reply_busy },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_get_state_reply_imm_invalid },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		mc_save_close_and_do_sdm_close },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{ 
	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
	}
},
{
	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,
	{
		{ TAPE_EVENT_OPEN_REQ,          	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,     	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,     	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_failed_and_reply_close_success },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_READ_DONE,     	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_failed_and_reply_close_success },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_failed_and_reply_close_success },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_message_req_reply_success_and_reply_close_success },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,         	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,       	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	NULL },
		{ TAPE_EVENT_GET_STATE_REQ,     	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,           	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,      	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,       	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,     	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,       	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,     	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,      	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_DESTROY_OPEN_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_TERMINAL,				tape_do_sdm_open_failed },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_OPEN_PENDING,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_TERMINAL,				tape_do_sdm_open_failed },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL }, 
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,	NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_open_reply_aborted },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_open_reply_aborted },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{ 
	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_TERMINAL,				tape_do_open_reply_io_error },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_TERMINAL,				tape_do_open_reply_io_error },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },	
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		tape_do_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	} 
},
{
	TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_REWIND_PENDING,			tape_do_get_state_reply_success_and_rewind },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_REWIND_PENDING,			tape_do_get_state_reply_failed_and_rewind },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_TERMINAL,				tape_do_get_state_reply_failed_and_reply_close_success },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_TERMINAL,				tape_do_get_state_reply_failed_and_reply_close_success },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	NULL },	
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,	tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	TAPE_STATE_DESTROY_CLOSE_PENDING,
	{
		{ TAPE_EVENT_OPEN_REQ,			TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_TERMINAL,				tape_do_close_reply_success },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_reject_serf_close },
		{ TAPE_EVENT_MESSAGE_REQ,		TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_DESTROY_CLOSE_PENDING,		NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		TAPE_STATE_DESTROY_CLOSE_PENDING,		tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
},
{
	MC_STATE_HOLDING_CLOSE,
	{
		{ TAPE_EVENT_OPEN_REQ,			MC_STATE_HOLDING_CLOSE,				tape_do_open_reply_busy },
		{ TAPE_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_RELAY_DONE,		TAPE_STATE_SDM_CLOSE_PENDING,			mc_do_message_req_reply_success_and_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_FAILED,		TAPE_STATE_SDM_CLOSE_PENDING,			mc_do_message_req_reply_failed_and_sdm_close },
		{ TAPE_EVENT_SDM_RELAY_SCSI_ERROR,	TAPE_STATE_SDM_CLOSE_PENDING,			mc_do_message_req_reply_success_and_sdm_close },
		{ TAPE_EVENT_SDM_READ_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_READ_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_WRITE_SCSI_ERROR,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_DONE,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_SDM_DEV_STATUS_FAILED,	TAPE_STATE_INVALID,				NULL },
		{ TAPE_EVENT_CLOSE_REQ,			MC_STATE_HOLDING_CLOSE,				tape_do_close_reply_not_open },
		{ TAPE_EVENT_SERF_CLOSE_REQ,		MC_STATE_HOLDING_CLOSE,				tape_do_close_reply_not_open },
		{ TAPE_EVENT_MESSAGE_REQ,		MC_STATE_HOLDING_CLOSE,				tape_do_message_req_reply_not_open },
		{ TAPE_EVENT_DEVICE_REMOVED,		TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,		NULL },
		{ TAPE_EVENT_GET_STATE_REQ,		MC_STATE_HOLDING_CLOSE,				tape_do_get_state_reply_not_open },
		{ MC_EVENT_CLOSE_REQ,			MC_STATE_HOLDING_CLOSE,				tape_do_close_reply_not_open },
		{ MC_EVENT_SERF_CLOSE_REQ,		MC_STATE_HOLDING_CLOSE,				tape_do_reject_serf_close },
		{ MC_EVENT_SDM_OPEN_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_OPEN_ALREADY_OPEN,	TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_INFO_FAILED,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_DONE,		TAPE_STATE_INVALID,				NULL },
		{ MC_EVENT_SDM_CLOSE_FAILED,		TAPE_STATE_INVALID,				NULL },
	}
}};

void tape_do_open_reply_busy(tape_dev_t *dev, void *cookie)
{
	tape_open_rsp_msg_t	tape_open_rsp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_open_reply_busy: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is tape open message context. */
	VERIFY(cookie);
	context = cookie;

	/* Reply to current tape open message -- device is busy */
	tape_open_rsp.hdr.mid = (dev->mc ? MC_MID_OPEN_RSP : TAPE_MID_OPEN_RSP);
	tape_open_rsp.hdr.cookie = context->req_msg->cookie;
	tape_open_rsp.ecode = TAPE_ECODE_DEVICE_BUSY;
	tape_send_msg(&tape_open_rsp, sizeof(tape_open_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_open_reply_success(tape_dev_t *dev, void *cookie) 
{
	tape_open_rsp_msg_t	tape_open_rsp;
	int			rewind = dev->open_attributes & TAPE_ATTRIB_REWIND_ON_CLOSE;
	int			unload = dev->open_attributes & TAPE_ATTRIB_UNLOAD_ON_CLOSE;
	int			density = (dev->open_attributes & TAPE_ATTRIB_DENSITY_MASK) >> TAPE_ATTRIB_DENSITY_SHIFT;

	TP_DBG(2, ("tape_do_open_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the DLT EEPROM mode select response if valid, or NULL is mode select skipped. */
	if (cookie) {
		/* Free the pending context */
		VERIFY(dev->pending_req_context);
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}

	/* Reply to tape open message */
	VERIFY(dev->open_req_context);
	tape_open_rsp.hdr.mid = TAPE_MID_OPEN_RSP;
	tape_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	tape_open_rsp.ecode = TAPE_ECODE_SUCCESS;
	tape_open_rsp.handle = dev->open_handle;
	tape_open_rsp.scsi_id = dev->id;
	tape_open_rsp.wwn = dev->wwn;
	tape_open_rsp.lun = dev->lun;
	if (dev->alias < 0) {
		sprintf(tape_open_rsp.devicename, "%016llx_%s_%s_%s",
			dev->id, 
			rewind ? "R" : "NR",
			unload ? "U" : "NU",
			dev->cap->densities[density].dens_name);
	}
	else {
		sprintf(tape_open_rsp.devicename, "%s%s%d%s",
			rewind ? "R" : "NR",
			unload ? "U" : "NU",
			dev->alias,
			dev->cap->densities[density].alias_dens_name);
	}
	sprintf(tape_open_rsp.physical_devicename, "%016llx_%s_%s_%s",
		dev->id, 
		rewind ? "R" : "NR",
		unload ? "U" : "NU",
		dev->cap->densities[density].dens_name);
	tape_open_rsp.client_tag = dev->open_client_tag;
	tape_send_msg(&tape_open_rsp, sizeof(tape_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the tape open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void tape_do_open_reply_aborted(tape_dev_t *dev, void *cookie)
{
	tape_open_rsp_msg_t	tape_open_rsp;
	
	TP_DBG(2, ("tape_do_open_reply_aborted: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is close context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (tape_msg_context_t *)cookie;

	/* Reply to tape open message */
	VERIFY(dev->open_req_context);
	tape_open_rsp.hdr.mid = TAPE_MID_OPEN_RSP;
	tape_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	tape_open_rsp.ecode = TAPE_ECODE_IO_ERROR;
	tape_send_msg(&tape_open_rsp, sizeof(tape_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the tape open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void tape_do_open_reply_io_error(tape_dev_t *dev, void *cookie)
{
	tape_open_rsp_msg_t	tape_open_rsp;
	
	TP_DBG(2, ("tape_do_sdm_open_reply_io_error: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM relay response */
	VERIFY(cookie);
	dev->open_handle = -1;

	/* Reply to tape open message -- I/O error */
	VERIFY(dev->open_req_context);
	tape_open_rsp.hdr.mid = TAPE_MID_OPEN_RSP;
	tape_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	tape_open_rsp.ecode = TAPE_ECODE_IO_ERROR;
	tape_send_msg(&tape_open_rsp, sizeof(tape_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the tape open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void tape_do_close_reply_not_open(tape_dev_t *dev, void *cookie) 
{
	tape_close_rsp_msg_t	resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_close_not_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the close context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(tape_close_rsp_msg_t));
	resp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP: TAPE_MID_CLOSE_RSP);
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
	tape_send_msg(&resp, sizeof(tape_close_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_reject_serf_close(tape_dev_t *dev, void *cookie)
{
	tape_close_rsp_msg_t	resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_reject_serf_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the close context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(tape_close_rsp_msg_t));
	resp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP: TAPE_MID_CLOSE_RSP);
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN_BY_CLIENT;
	tape_send_msg(&resp, sizeof(tape_close_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_close_reply_success(tape_dev_t *dev, void *cookie) 
{
	tape_close_rsp_msg_t	tape_close_rsp;
	
	TP_DBG(2, ("tape_do_close_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM close response */
	VERIFY(cookie);
	dev->open_handle = -1;

	/* Reply to tape close message */
	VERIFY(dev->close_req_context);
	tape_close_rsp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP : TAPE_MID_CLOSE_RSP);
	tape_close_rsp.hdr.cookie = dev->close_req_context->req_msg->cookie;
	tape_close_rsp.ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(&tape_close_rsp, sizeof(tape_close_rsp_msg_t), 
		      dev->close_req_context->req_src_app,
		      dev->close_req_context->req_src_cpu,
		      dev->close_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the tape close context */
	VERIFY(dev->close_req_context);
	tape_free_context(dev->close_req_context);
	dev->close_req_context = NULL;
}

void tape_save_close_and_do_sdm_close(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("tape_save_close_and_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the TAPE_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (tape_msg_context_t *)cookie;

	tape_do_sdm_close(dev, NULL);
}

void tape_do_hold_close(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("tape_do_hold_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the TAPE_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (tape_msg_context_t *)cookie;
}

void 
tape_do_drop_relay_and_scream(tape_dev_t *dev, void *cookie)
{
	TP_DBG(0, ("tape_do_drop_relay_and_scream: dev 0x%p, cookie 0x%p\n", dev, cookie));
	TP_DBG(0, ("Received relay response while in CLOSED state -- dropping.\n"));
}

void tape_do_sdm_open_failed(tape_dev_t *dev, void *cookie)
{
	tape_open_rsp_msg_t	tape_open_rsp;
	
	TP_DBG(2, ("tape_do_sdm_open_failed: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM open or info response */
	VERIFY(cookie);

	/* Reply to tape open message -- no such device */
	VERIFY(dev->open_req_context);
	tape_open_rsp.hdr.mid = (dev->mc ? MC_MID_OPEN_RSP: TAPE_MID_OPEN_RSP);
	tape_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	tape_open_rsp.ecode = TAPE_ECODE_NO_DEVICE;
	tape_send_msg(&tape_open_rsp, sizeof(tape_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the tape open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void tape_do_sdm_open(tape_dev_t *dev, void *cookie) 
{
	sdm_openReqMsg_t *	sdm_open_req;

	TP_DBG(2, ("tape_do_sdm_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the TAPE_OPEN context */
	VERIFY(cookie);
	VERIFY(dev->open_req_context == NULL);
	dev->open_req_context = (tape_msg_context_t *)cookie;

	/* Send SDM open message */
	sdm_open_req = (sdm_openReqMsg_t *)malloc(sizeof(sdm_openReqMsg_t));

	sdm_open_req->hdr.mid = SDM_MID_DEV_OPEN_REQ;
	sdm_open_req->hdr.chandle = tape_device_to_chandle(dev);
	sdm_open_req->id = dev->id;
	sdm_open_req->slot = 1;	/* need to open on FP */
	sdm_open_req->flags = 0;

	tape_send_msg(sdm_open_req, sizeof(sdm_openReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_open_req;
	
	/* Save client/owner tag */
	if (dev->open_req_context->req_msg->mid == TAPE_MID_OPEN_REQ)
		dev->open_client_tag = ((tape_open_req_msg_t *)(dev->open_req_context->req_msg))->client_tag;
	else
		dev->open_client_tag = ((mc_open_req_msg_t *)(dev->open_req_context->req_msg))->client_tag;
}

void tape_do_sdm_info(tape_dev_t *dev, void *cookie)
{
	sdm_devInfoReqMsg_t *	sdm_info_req;

	TP_DBG(2, ("tape_do_sdm_info: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* The cookie is the SDM OPEN reponse */
	VERIFY(cookie);
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Send SDM info message */
	sdm_info_req = (sdm_devInfoReqMsg_t *)malloc(sizeof(sdm_devInfoReqMsg_t));

	sdm_info_req->hdr.mid = SDM_MID_DEV_INFO_REQ;
	sdm_info_req->hdr.chandle = tape_device_to_chandle(dev);
	sdm_info_req->id = dev->id;

	tape_send_msg(sdm_info_req, sizeof(sdm_devInfoReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_info_req;
}

void tape_do_sdm_close(tape_dev_t *dev, void *cookie)
{
	sdm_closeReqMsg_t *	sdm_close_req;

	TP_DBG(2, ("tape_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	if (cookie) {
		VERIFY(dev->pending_req_context);
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}

	/* Send SDM close message */
	sdm_close_req = (sdm_closeReqMsg_t *)malloc(sizeof(sdm_closeReqMsg_t));

	sdm_close_req->hdr.mid = SDM_MID_DEV_CLOSE_REQ;
	sdm_close_req->hdr.chandle = tape_device_to_chandle(dev);
	sdm_close_req->handle = dev->open_handle;

	tape_send_msg(sdm_close_req, sizeof(sdm_closeReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_close_req;
}

void tape_do_tur_after_sdm_open(tape_dev_t *dev, void *cookie) 
{
	sdm_relayCdbReqMsg_t *	scsi_msg;
	sdm_openRspMsg_t *	sdm_open_rsp;

	TP_DBG(2, ("tape_do_tur_after_sdm_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM open response */
	VERIFY(cookie);
	sdm_open_rsp = cookie;
	VERIFY(sdm_open_rsp->hdr.mid == SDM_MID_DEV_OPEN_RSP);
	dev->open_handle = sdm_open_rsp->handle;
	dev->open_attributes = dev->open_req_context->private;

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	scsi_msg = tape_build_scsi_tur_msg(dev);
	VERIFY(scsi_msg);
	
	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_tur_after_sdm_info(tape_dev_t *dev, void *cookie)
{
	sdm_relayCdbReqMsg_t *	scsi_msg;
	sdm_devInfoRspMsg_t *	sdm_info_rsp;

	TP_DBG(2, ("tape_do_tur_after_sdm_info: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM info response */
	VERIFY(cookie);
	sdm_info_rsp = cookie;
	VERIFY(sdm_info_rsp->hdr.mid == SDM_MID_DEV_INFO_RSP);
	VERIFY(dev->state == TAPE_STATE_TUR_PENDING);
	VERIFY(sdm_info_rsp->openHandle > 0);
	dev->open_handle = sdm_info_rsp->openHandle;
	dev->open_attributes = dev->open_req_context->private;

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	scsi_msg = tape_build_scsi_tur_msg(dev);
	VERIFY(scsi_msg);
	
	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_tur_retry(tape_dev_t *dev, void *cookie)
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_tur_retry: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM relay response */
	VERIFY(cookie);
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	scsi_msg = tape_build_scsi_tur_msg(dev);
	VERIFY(scsi_msg);
	
	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_mode_sense_compression(tape_dev_t *dev, void *cookie) 
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_mode_sense_compression: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the TUR response */
	VERIFY(cookie);

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	scsi_msg = tape_build_scsi_mode_sense_compression_msg(dev);
	VERIFY(scsi_msg);

	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_mode_select_compression(tape_dev_t *dev, void *cookie) 
{
	sdm_relayCdbRspMsg_t *	scsi_resp = cookie;
	sdm_relayCdbReqMsg_t *	scsi_msg;
	scsi_mode_header_t *	mode_hdr;
	scsi_mode_block_desc_t *mode_block_desc;
	scsi_mode_page_desc_t * mode_page;

	TP_DBG(2, ("tape_do_mode_select_compression: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the mode sense response */
	VERIFY(cookie);
	scsi_resp = cookie;

	/* We're expecting data */
	VERIFY(scsi_resp->status & SD_DEV_DATA_VALID);
	VERIFY(scsi_resp->dataLen >= sizeof(scsi_mode_header_t) + sizeof(scsi_mode_compress_mode_t) + 2);

#if 0
	printf("MODE SENSE DATA: \n");
	{
	    int i;

	    for (i = 0; i < scsi_resp->dataLen; ++i) {
		if (i > 0 && (i % 16) == 0)
		    printf("\n");
		printf("%02x ", scsi_resp->data[i]); 
	    }	
	}
	printf("\n");
#endif

	mode_hdr = (scsi_mode_header_t *)&scsi_resp->data[0];
	mode_block_desc = (scsi_mode_block_desc_t *)(mode_hdr + 1);
	mode_page = (scsi_mode_page_desc_t *)(mode_block_desc + 1);

	VERIFY((mode_page->code & 0x3F) == SCSI_MPAGE_COMPRESSION);

	/* Pull out Write protect bit */
	dev->write_protected = (mode_hdr->flag & 0x80) ? 1 : 0;

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	scsi_msg = tape_build_scsi_mode_select_compression_msg(dev, scsi_resp);
	VERIFY(scsi_msg);

#if 0
	printf("MODE SELECT DATA: \n");
	{
	    int i;

	    for (i = 0; i < scsi_msg->dataLen; ++i) {
		if (i > 0 && (i % 16) == 0)
		    printf("\n");
		printf("%02x ", scsi_msg->data[i]); 
	    }	
	}
	printf("\n");
#endif

	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t) + scsi_msg->dataLen, "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void 
tape_do_mode_select_dlteeprom(tape_dev_t *dev, void *cookie)
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_mode_select_dlteeprom: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the mode select response */
	VERIFY(cookie);

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Do we need to do the DLT specific mode select request ? */
	if (tape_is_DLT(dev->cap)) {
		scsi_msg = tape_build_scsi_mode_select_dlteeprom_msg(dev);
		VERIFY(scsi_msg);

#if 0
		printf("MODE SELECT DATA: \n");
		{
			int i;

			for (i = 0; i < scsi_msg->dataLen; ++i) {
				if (i > 0 && (i % 16) == 0)
					printf("\n");
				printf("%02x ", scsi_msg->data[i]); 
			}	
		}
		printf("\n");
#endif

		tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t) + scsi_msg->dataLen, "sdm", 0, 0);

		/* Stash the message */
		VERIFY(dev->pending_req_context == NULL);
		dev->pending_req_context = scsi_msg;
	}
	else {
		/* Skip mode select -- fake out a successfull mode select completion */
		tape_sm_run(dev, NULL, TAPE_EVENT_SDM_RELAY_DONE);
	}
}

void tape_do_load(tape_dev_t *dev, void *cookie) 
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_load: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the TUR response */
	VERIFY(cookie);

	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Fire off an load */
	scsi_msg = tape_build_scsi_load_msg(dev);
	VERIFY(scsi_msg);
	
	tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_unload(tape_dev_t *dev, void *cookie)
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_unload: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is rewind response, or NULL if rewind was skipped */
	if (cookie) {
		VERIFY(dev->pending_req_context);
		free(dev->pending_req_context);
		dev->pending_req_context = NULL;
	}

	/* Do we *need* to unload on a close? */
	if (dev->open_attributes & TAPE_ATTRIB_UNLOAD_ON_CLOSE) {
		/* Fire off an unload */
		scsi_msg = tape_build_scsi_unload_msg(dev);
		VERIFY(scsi_msg);
	
		tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

		/* Stash the message */
		VERIFY(dev->pending_req_context == NULL);
		dev->pending_req_context = scsi_msg;
	}
	else {
		/* Skip unload -- fake out a successfull unload completion */
		tape_sm_run(dev, NULL, TAPE_EVENT_SDM_RELAY_DONE);
	}
}

void tape_do_rewind(tape_dev_t *dev, void *cookie)
{
	sdm_relayCdbReqMsg_t *	scsi_msg;

	TP_DBG(2, ("tape_do_rewind: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the TAPE_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (tape_msg_context_t *)cookie;

	/* Do we *need* to rewind on a close? */
	if (dev->open_attributes & TAPE_ATTRIB_REWIND_ON_CLOSE) {
		/* Fire off a rewind */
		scsi_msg = tape_build_scsi_rewind_msg(dev);
		VERIFY(scsi_msg);
	
		tape_send_msg(scsi_msg, sizeof(sdm_relayCdbReqMsg_t), "sdm", 0, 0);

		/* Stash the message */
		VERIFY(dev->pending_req_context == NULL);
		dev->pending_req_context = scsi_msg;
	}
	else {
		/* Skip rewind -- fake out a successfull rewind completion */
		tape_sm_run(dev, NULL, TAPE_EVENT_SDM_RELAY_DONE);
	}
}

void 
tape_do_read_position(tape_dev_t *dev, void *cookie)
{
	sdm_devStatusReqMsg_t * scsi_msg;
	
	TP_DBG(2, ("tape_do_read_position: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the get_state context */
	VERIFY(cookie);
	VERIFY(dev->get_state_req_context == NULL);
	dev->get_state_req_context = cookie;

	scsi_msg = (sdm_devStatusReqMsg_t *)malloc(sizeof(sdm_devStatusReqMsg_t));
	VERIFY(scsi_msg);
	bzero(scsi_msg, sizeof(sdm_devStatusReqMsg_t));
	scsi_msg->hdr.mid = SDM_MID_DEV_STATUS_REQ;
	scsi_msg->hdr.chandle = tape_device_to_chandle(dev);
	scsi_msg->id = dev->id;
	tape_send_msg(scsi_msg, sizeof(sdm_devStatusReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = scsi_msg;
}

void tape_do_get_state_reply_imm(tape_dev_t *dev, void *cookie)
{
	tape_get_state_rsp_msg_t	resp;
	tape_msg_context_t *		context;
	int				density;

	TP_DBG(2, ("tape_do_get_state_reply_imm: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the get_state context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(tape_get_state_rsp_msg_t));

	/* fill in immediate device state */
	density = (dev->open_attributes & TAPE_ATTRIB_DENSITY_MASK) >> TAPE_ATTRIB_DENSITY_SHIFT;
	resp.flags = 
		((dev->open_attributes & TAPE_ATTRIB_REWIND_ON_CLOSE) ? TAPE_STATE_FLAGS_REWIND_ON_CLOSE : 0) |
		((dev->open_attributes & TAPE_ATTRIB_UNLOAD_ON_CLOSE) ? TAPE_STATE_FLAGS_UNLOAD_ON_CLOSE : 0) |
		(density << TAPE_STATE_FLAGS_DENSITY_SHIFT) |
		(dev->write_protected ? TAPE_STATE_FLAGS_WRT_PROTECT : 0) |
		(dev->bom ? TAPE_STATE_FLAGS_BOM : 0) |
		(dev->eom ? TAPE_STATE_FLAGS_EOM : 0);

	if (dev->fileno >= 0) {
		resp.fileno = dev->fileno;
		resp.flags |= TAPE_STATE_FLAGS_FILENO_VALID;
	}

	if (dev->blockno >= 0) { 
		resp.blockno = dev->blockno;
		resp.flags |= TAPE_STATE_FLAGS_BLOCKNO_VALID;
	}

	resp.blocksize = dev->fixed_blocksize;
	resp.flags |= TAPE_STATE_FLAGS_BLOCKSIZE_VALID;

	if (dev->cap->densities[density].total_capacity != UNKNOWN_CAPACITY) {
		resp.total_space = dev->cap->densities[density].total_capacity;
		resp.flags |= TAPE_STATE_FLAGS_TOTAL_SPACE_VALID;
	}

	/* Reply to tape get_state message */
	resp.hdr.mid = TAPE_MID_GET_STATE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(&resp, sizeof(tape_get_state_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_get_state_reply_imm_invalid(tape_dev_t *dev, void *cookie)
{
	tape_get_state_rsp_msg_t	resp;
	tape_msg_context_t *		context;
	int				density;

	TP_DBG(2, ("tape_do_get_state_reply_imm_invalid: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the get_state context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(tape_get_state_rsp_msg_t));

	/* fill in immediate device state */
	density = (dev->open_attributes & TAPE_ATTRIB_DENSITY_MASK) >> TAPE_ATTRIB_DENSITY_SHIFT;
	resp.flags = 
		((dev->open_attributes & TAPE_ATTRIB_REWIND_ON_CLOSE) ? TAPE_STATE_FLAGS_REWIND_ON_CLOSE : 0) |
		((dev->open_attributes & TAPE_ATTRIB_UNLOAD_ON_CLOSE) ? TAPE_STATE_FLAGS_UNLOAD_ON_CLOSE : 0) |
		(density << TAPE_STATE_FLAGS_DENSITY_SHIFT);

	/* Reply to tape get_state message */
	resp.hdr.mid = TAPE_MID_GET_STATE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(&resp, sizeof(tape_get_state_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_get_state_reply_success(tape_dev_t *dev, void *cookie)
{
	sdm_devStatusRspMsg_t *	scsi_resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_get_state_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is dev status response */
	VERIFY(cookie);
	scsi_resp = cookie;
	
	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	dev->bom = 0;
	dev->eom = 0;
	dev->blockno = -1;
	dev->fileno = -1;

	dev->fileno = scsi_resp->tape_position_fileno;
	dev->blockno = scsi_resp->tape_position_blockno;
	dev->bom = scsi_resp->tape_position_bom;
	dev->eom = scsi_resp->tape_position_eom;

	/* Pull get state context and pass it along .... */
	VERIFY(dev->get_state_req_context);
	context = dev->get_state_req_context;
	dev->get_state_req_context = NULL;
	tape_do_get_state_reply_imm(dev, context);
}

void tape_do_get_state_reply_success_and_rewind(tape_dev_t *dev, void *cookie)
{
	sdm_devStatusRspMsg_t *	scsi_resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_get_state_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is dev status response */
	VERIFY(cookie);
	scsi_resp = cookie;
	
	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	dev->bom = 0;
	dev->eom = 0;
	dev->blockno = -1;
	dev->fileno = -1;

	dev->fileno = scsi_resp->tape_position_fileno;
	dev->blockno = scsi_resp->tape_position_blockno;
	dev->bom = scsi_resp->tape_position_bom;
	dev->eom = scsi_resp->tape_position_eom;

	/* Pull get state context and pass it along .... */
	VERIFY(dev->get_state_req_context);
	context = dev->get_state_req_context;
	dev->get_state_req_context = NULL;
	tape_do_get_state_reply_imm(dev, context);

        /* Start close state machine */
        VERIFY(dev->close_req_context);
        cookie = dev->close_req_context;
        dev->close_req_context = NULL;
        tape_do_rewind(dev, cookie);
}

void tape_do_get_state_reply_failed_and_rewind(tape_dev_t *dev, void *cookie)
{
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_get_state_reply_failed_and_rewind: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	dev->bom = 0;
	dev->eom = 0;
	dev->blockno = -1;
	dev->fileno = -1;

	/* Pull get state context and pass it along .... */
	VERIFY(dev->get_state_req_context);
	context = dev->get_state_req_context;
	dev->get_state_req_context = NULL;
	tape_do_get_state_reply_imm(dev, context);

    /* Start close state machine */
    VERIFY(dev->close_req_context);
    cookie = dev->close_req_context;
    dev->close_req_context = NULL;
    tape_do_rewind(dev, cookie);
}

void tape_do_get_state_reply_failed(tape_dev_t *dev, void *cookie)
{
	tape_msg_context_t *		context;
	
	TP_DBG(2, ("tape_do_get_state_reply_failed: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is read position relay response */
	VERIFY(cookie);

	/* Position is unknown */
	dev->bom = 0;
	dev->eom = 0;
	dev->blockno = -1;
	dev->fileno = -1;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Pull get state context and pass it along .... */
	VERIFY(dev->get_state_req_context);
	context = dev->get_state_req_context;
	dev->get_state_req_context = NULL;
	tape_do_get_state_reply_imm(dev, context);
}

void tape_do_get_state_reply_failed_and_reply_close_success(tape_dev_t *dev, void *cookie)
{
	tape_msg_context_t *		context;
	tape_close_rsp_msg_t		tape_close_rsp;
	
	TP_DBG(2, ("tape_do_get_state_reply_failed_and_reply_close_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is read position relay response */
	VERIFY(cookie);

	/* Position is unknown */
	dev->bom = 0;
	dev->eom = 0;
	dev->blockno = -1;
	dev->fileno = -1;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Pull get state context and pass it along .... */
	VERIFY(dev->get_state_req_context);
	context = dev->get_state_req_context;
	dev->get_state_req_context = NULL;
	tape_do_get_state_reply_imm(dev, context);

        tape_close_rsp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP : TAPE_MID_CLOSE_RSP);
        tape_close_rsp.hdr.cookie = dev->close_req_context->req_msg->cookie;
        tape_close_rsp.ecode = TAPE_ECODE_SUCCESS;
        tape_send_msg(&tape_close_rsp, sizeof(tape_close_rsp_msg_t),
                      dev->close_req_context->req_src_app,
                      dev->close_req_context->req_src_cpu,
                      dev->close_req_context->req_src_slot);

        /* Free the tape close context */
        VERIFY(dev->close_req_context);
        tape_free_context(dev->close_req_context);
        dev->close_req_context = NULL;
}

void tape_do_get_state_reply_not_open(tape_dev_t *dev, void *cookie)
{
	tape_get_state_rsp_msg_t	resp;
	tape_msg_context_t *		context;

	TP_DBG(2, ("tape_do_get_state_reply_not_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the get_state context */
	VERIFY(cookie);
	context = cookie;

	bzero(&resp, sizeof(tape_get_state_rsp_msg_t));
	resp.hdr.mid = TAPE_MID_GET_STATE_RSP;
	resp.hdr.cookie = context->req_msg->cookie;
	resp.ecode = TAPE_ECODE_DEVICE_NOT_OPEN;
	tape_send_msg(&resp, sizeof(tape_get_state_rsp_msg_t), 
		      context->req_src_app,
		      context->req_src_cpu,
		      context->req_src_slot);
	tape_free_context(context);
}

void tape_do_message_req(tape_dev_t *dev, void *cookie)
{
	void *  scsi_msg = NULL;
	int	scsi_msg_len = 0;

	TP_DBG(2, ("tape_do_message_req: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is message context */
        VERIFY(cookie);
        VERIFY(dev->message_req_context == NULL);
        dev->message_req_context = cookie;
	
	switch(dev->message_req_context->req_msg->mid) {
	case TAPE_MID_SPACE_REQ:
		scsi_msg = tape_build_scsi_space_msg(dev, (tape_space_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = sizeof(sdm_relayCdbReqMsg_t);
		break;

	case TAPE_MID_WRITE_FM_REQ:
		scsi_msg = tape_build_scsi_write_fms_msg(dev, (tape_write_fm_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = sizeof(sdm_relayCdbReqMsg_t);
		break;

	case TAPE_MID_LOAD_REQ:
		if (((tape_load_req_msg_t *)dev->message_req_context->req_msg)->load_or_unload)
			scsi_msg = tape_build_scsi_load_msg(dev);
		else
			scsi_msg = tape_build_scsi_unload_msg(dev);
		scsi_msg_len = sizeof(sdm_relayCdbReqMsg_t);
		break;

	case TAPE_MID_READ_REQ:
		scsi_msg = tape_build_scsi_read_msg(dev, (tape_read_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = sizeof(sdm_readDataReqMsg_t);
		break;

	case TAPE_MID_WRITE_REQ:
		scsi_msg = tape_build_scsi_write_msg(dev, (tape_write_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = (int)(&(((sdm_writeDataReqMsg_t *)NULL)->data));
		scsi_msg_len += ((sdm_writeDataReqMsg_t *)scsi_msg)->blkCnt;
		break;

	case TAPE_MID_PASSTHRU_REQ:
		scsi_msg = tape_build_scsi_passthru_msg(dev, (tape_passthru_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = (int)(&(((sdm_relayCdbReqMsg_t *)NULL)->data));
		scsi_msg_len += ((sdm_relayCdbReqMsg_t *)scsi_msg)->dataLen;		
		break;

	case MC_MID_PASSTHRU_REQ:
		scsi_msg = mc_build_scsi_passthru_msg(dev, (mc_passthru_req_msg_t *)dev->message_req_context->req_msg);
		scsi_msg_len = (int)(&(((sdm_relayCdbReqMsg_t *)NULL)->data));
		scsi_msg_len += ((sdm_relayCdbReqMsg_t *)scsi_msg)->dataLen;		
		break;

	default:
		VERIFY(0);
	}
        VERIFY(scsi_msg);

        tape_send_msg(scsi_msg, scsi_msg_len, "sdm", 0, 0);

        /* Stash the message */
        VERIFY(dev->pending_req_context == NULL);
        dev->pending_req_context = scsi_msg;
}

void 
tape_do_message_req_reply_success(tape_dev_t *dev, void *cookie)
{
	void *			scsi_resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_message_req_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is SDM relay/read/write response */
	VERIFY(cookie);
	scsi_resp = cookie;

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;
	VERIFY(context->callback_success);
	(*context->callback_success)(context, scsi_resp);

	/* Free the tape message context */
	tape_free_context(dev->message_req_context);
	dev->message_req_context = NULL;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;
}

void tape_do_message_req_reply_io_error(tape_dev_t *dev, void *cookie) 
{
	void *			scsi_resp;
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_message_req_reply_io_error: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is relay/read/write response */
	VERIFY(cookie);
	scsi_resp = cookie;

	VERIFY(dev->message_req_context);
	context = dev->message_req_context;
	VERIFY(context->callback_failed);
	(*context->callback_failed)(context, TAPE_ECODE_IO_ERROR);

	/* Free the tape message context */
	tape_free_context(dev->message_req_context);
	dev->message_req_context = NULL;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;
}

void tape_do_message_req_reply_busy(tape_dev_t *dev, void *cookie)
{
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_message_req_reply_success: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is tape message context */
	VERIFY(cookie);
	context = cookie;
	VERIFY(context->callback_failed);
	(*context->callback_failed)(context, TAPE_ECODE_DEVICE_BUSY);

	/* Free the tape message context */
	tape_free_context(context);
}

void tape_do_message_req_reply_not_open(tape_dev_t *dev, void *cookie)
{
	tape_msg_context_t *	context;

	TP_DBG(2, ("tape_do_message_req_reply_not_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is tape message context */
	VERIFY(cookie);
	context = cookie;
	VERIFY(context->callback_failed);
	(*context->callback_failed)(context, TAPE_ECODE_DEVICE_NOT_OPEN);

	/* Free the tape message context */
	tape_free_context(context);
}


void 
tape_do_message_req_reply_success_and_rewind(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("tape_do_message_req_reply_success_and_rewind: dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_success(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;
	dev->close_req_context = NULL;
	tape_do_rewind(dev, cookie);
}

void 
tape_do_message_req_reply_failed_and_rewind(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("tape_do_message_req_reply_failed_and_rewind: dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_io_error(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;
	dev->close_req_context = NULL;
	tape_do_rewind(dev, cookie);
}

void 
tape_do_message_req_reply_success_and_reply_close_success(tape_dev_t *dev, void *cookie)
{
	tape_close_rsp_msg_t	tape_close_rsp;

	TP_DBG(2, ("tape_do_message_req_reply_success_and_reply_close_success: dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_success(dev, cookie);

	tape_close_rsp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP : TAPE_MID_CLOSE_RSP);
	tape_close_rsp.hdr.cookie = dev->close_req_context->req_msg->cookie;
	tape_close_rsp.ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(&tape_close_rsp, sizeof(tape_close_rsp_msg_t), 
		      dev->close_req_context->req_src_app,
		      dev->close_req_context->req_src_cpu,
		      dev->close_req_context->req_src_slot);

	/* Free the tape close context */
	VERIFY(dev->close_req_context);
	tape_free_context(dev->close_req_context);
	dev->close_req_context = NULL;
}

void 
tape_do_message_req_reply_failed_and_reply_close_success(tape_dev_t *dev, void *cookie)
{
	tape_close_rsp_msg_t	tape_close_rsp;

	TP_DBG(2, ("tape_do_message_req_reply_failed_and_reply_close_success dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_io_error(dev, cookie);

	tape_close_rsp.hdr.mid = (dev->mc ? MC_MID_CLOSE_RSP : TAPE_MID_CLOSE_RSP);
	tape_close_rsp.hdr.cookie = dev->close_req_context->req_msg->cookie;
	tape_close_rsp.ecode = TAPE_ECODE_SUCCESS;
	tape_send_msg(&tape_close_rsp, sizeof(tape_close_rsp_msg_t), 
		      dev->close_req_context->req_src_app,
		      dev->close_req_context->req_src_cpu,
		      dev->close_req_context->req_src_slot);

	/* Free the tape close context */
	VERIFY(dev->close_req_context);
	tape_free_context(dev->close_req_context);
	dev->close_req_context = NULL;
}

void mc_do_open_reply_success_after_sdm_open(tape_dev_t *dev, void *cookie)
{
	mc_open_rsp_msg_t	mc_open_rsp;
	sdm_openRspMsg_t *	sdm_open_rsp;

	TP_DBG(2, ("mc_do_open_reply_success_after_sdm_open: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM open response */
	VERIFY(cookie);
	sdm_open_rsp = cookie;
	VERIFY(sdm_open_rsp->hdr.mid == SDM_MID_DEV_OPEN_RSP);
	dev->open_handle = sdm_open_rsp->handle;
	dev->open_attributes = dev->open_req_context->private;

	/* Reply to mc open message */
	VERIFY(dev->open_req_context);
	mc_open_rsp.hdr.mid = MC_MID_OPEN_RSP;
	mc_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	mc_open_rsp.ecode = TAPE_ECODE_SUCCESS;
	mc_open_rsp.handle = dev->open_handle;
	mc_open_rsp.scsi_id = dev->id;
	mc_open_rsp.wwn = dev->wwn;
	mc_open_rsp.lun = dev->lun;
	if (dev->alias < 0)
		sprintf(mc_open_rsp.devicename, "%016llx", dev->id);
	else
		sprintf(mc_open_rsp.devicename, "mc%d", dev->alias);
	sprintf(mc_open_rsp.physical_devicename, "%016llx", dev->id);
	tape_send_msg(&mc_open_rsp, sizeof(mc_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Free the mc open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void mc_do_open_reply_success_after_sdm_info(tape_dev_t *dev, void *cookie)
{
	mc_open_rsp_msg_t	mc_open_rsp;
	sdm_devInfoRspMsg_t *	sdm_info_rsp;

	TP_DBG(2, ("mc_do_open_reply_success_after_sdm_info: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Cookie is the SDM info response */
	VERIFY(cookie);
	sdm_info_rsp = cookie;
	VERIFY(sdm_info_rsp->hdr.mid == SDM_MID_DEV_INFO_RSP);
	VERIFY(sdm_info_rsp->state == SDM_DEV_STATE_OPENED);
	VERIFY(sdm_info_rsp->openHandle > 0);
	dev->open_handle = sdm_info_rsp->openHandle;
	dev->open_attributes = dev->open_req_context->private;

	/* Free the pending context */
	VERIFY(dev->pending_req_context);
	free(dev->pending_req_context);
	dev->pending_req_context = NULL;

	/* Reply to mc open message */
	VERIFY(dev->open_req_context);
	mc_open_rsp.hdr.mid = MC_MID_OPEN_RSP;
	mc_open_rsp.hdr.cookie = dev->open_req_context->req_msg->cookie;
	mc_open_rsp.ecode = TAPE_ECODE_SUCCESS;
	mc_open_rsp.handle = dev->open_handle;
	mc_open_rsp.scsi_id = dev->id;
	mc_open_rsp.wwn = dev->wwn;
	mc_open_rsp.lun = dev->lun;
	if (dev->alias < 0)
		sprintf(mc_open_rsp.devicename, "%016llx", dev->id);
	else
		sprintf(mc_open_rsp.devicename, "mc%d", dev->alias);
	sprintf(mc_open_rsp.physical_devicename, "%016llx", dev->id);
	tape_send_msg(&mc_open_rsp, sizeof(mc_open_rsp_msg_t), 
		      dev->open_req_context->req_src_app,
		      dev->open_req_context->req_src_cpu,
		      dev->open_req_context->req_src_slot);

	/* Free the mc open context */
	VERIFY(dev->open_req_context);
	tape_free_context(dev->open_req_context);
	dev->open_req_context = NULL;
}

void mc_save_close_and_do_sdm_close(tape_dev_t *dev, void *cookie)
{
#if 0
	VERIFY(0);
	/* xxx */
#else
        TP_DBG(2, ("mc_save_close_and_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));
        mc_do_sdm_close(dev, cookie);
#endif
}

void mc_do_message_req_reply_success_and_sdm_close(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("mc_do_message_req_reply_success_and_sdm_close: dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_success(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;
	dev->close_req_context = NULL;
	mc_do_sdm_close(dev, cookie);
}

void mc_do_message_req_reply_failed_and_sdm_close(tape_dev_t *dev, void *cookie)
{
	TP_DBG(2, ("mc_do_message_req_reply_failed_and_sdm_close: dev 0x%p, cookie 0x%p\n", 
		   dev, cookie));

	/* 
	 * Cookie is the relay/write/read reply; pass it along.
	 * Send message reply 
	 */
	tape_do_message_req_reply_io_error(dev, cookie);

	/* Start close state machine */
	VERIFY(dev->close_req_context);
	cookie = dev->close_req_context;
	dev->close_req_context = NULL;
	mc_do_sdm_close(dev, cookie);
}

void mc_do_sdm_close(tape_dev_t *dev, void *cookie)
{
	sdm_closeReqMsg_t *	sdm_close_req;

	TP_DBG(2, ("mc_do_sdm_close: dev 0x%p, cookie 0x%p\n", dev, cookie));

	/* Stash the cookie -- it's the MC_CLOSE context */
	VERIFY(cookie);
	VERIFY(dev->close_req_context == NULL);
	dev->close_req_context = (tape_msg_context_t *)cookie;

	/* Send SDM close message */
	sdm_close_req = (sdm_closeReqMsg_t *)malloc(sizeof(sdm_closeReqMsg_t));

	sdm_close_req->hdr.mid = SDM_MID_DEV_CLOSE_REQ;
	sdm_close_req->hdr.chandle = tape_device_to_chandle(dev);
	sdm_close_req->handle = dev->open_handle;

	tape_send_msg(sdm_close_req, sizeof(sdm_closeReqMsg_t), "sdm", 0, 0);

	/* Stash the message */
	VERIFY(dev->pending_req_context == NULL);
	dev->pending_req_context = sdm_close_req;
}

void tape_sm_create(tape_dev_t *dev) 
{
	TP_DBG(2, ("tape_sm_create: dev 0x%p\n", dev));

	dev->dev_sm = tape_dev_sm;
	dev->state = TAPE_STATE_INITIAL;
}

void tape_sm_destroy(tape_dev_t *dev) 
{
	TP_DBG(2, ("tape_sm_destroy: dev 0x%p\n", dev));

	VERIFY(dev->open_req_context == NULL);
	VERIFY(dev->close_req_context == NULL);
	VERIFY(dev->pending_req_context == NULL);
	VERIFY(dev->message_req_context == NULL);
	tape_device_destroy(dev);
}

void tape_sm_run(tape_dev_t *dev, void *cookie, tape_dev_event_t event) 
{
	tape_sm_t *	ts;

	TP_DBG(2, ("tape_sm_run: dev 0x%p, cookie 0x%p, current_state %d, event %d\n", 
		   dev, cookie, dev->state, event));	

	ts = &dev->dev_sm[dev->state];
	VERIFY(ts->current_state == dev->state);
	VERIFY(ts->events[event].event == event);

	/* Move to next state */
	if (ts->events[event].next_state == TAPE_STATE_INVALID)
		panic("Invalid transition in tape_sm_run: current_state %d, dev 0x%p, cookie 0x%p, event %d\n",
		      dev->state, dev, cookie, event);
	else
	if (ts->events[event].next_state == TAPE_STATE_TERMINAL) {
		TP_DBG(2, ("tape_sm_run: Reached terminal state: current_state %d, dev 0x%p, cookie 0x%p, event %d\n", 
			   dev->state, dev, cookie, event)); 
		/* Execute transition callout before destroying device */
		if (ts->events[event].callout)
			(*ts->events[event].callout)(dev, cookie);
		tape_sm_destroy(dev);
	}
	else {
		TP_DBG(2, ("tape_sm_run: Next state %d.\n", ts->events[event].next_state));
		dev->state = ts->events[event].next_state;
		/* Execute transition callout function */
		if (ts->events[event].callout)
			(*ts->events[event].callout)(dev, cookie);
	}
}

tape_msg_context_t *    
tape_save_context(tape_msg_hdr_t *msg, char *app, uint32 cpu, uint32 slot)
{
	tape_msg_context_t *context = NULL;

	context = (tape_msg_context_t *)malloc(sizeof(tape_msg_context_t));
	bzero(context, sizeof(tape_msg_context_t));
	strcpy(context->req_src_app, app);
	context->req_src_cpu = cpu;
	context->req_src_slot = slot;

	switch(msg->mid) {
	case TAPE_MID_GET_DEVLIST_REQ:
		break;

	case TAPE_MID_OPEN_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_open_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_open_req_msg_t));
		break;

	case TAPE_MID_CLOSE_REQ:
    case TAPE_MID_ID_CLOSE_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_close_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_close_req_msg_t));
		break;

	case TAPE_MID_GET_STATE_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_get_state_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_get_state_req_msg_t));
		break;

	case TAPE_MID_SPACE_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_space_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_space_req_msg_t));
		break;

	case TAPE_MID_WRITE_FM_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_write_fm_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_write_fm_req_msg_t));
		break;

	case TAPE_MID_LOAD_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_load_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_load_req_msg_t));
		break;

	case TAPE_MID_READ_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(tape_read_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(tape_read_req_msg_t));
		break;

	case TAPE_MID_WRITE_REQ: 
		{ 
			int	count = sizeof(tape_write_req_msg_t)-1 + ((tape_write_req_msg_t *)msg)->bytecount;

			context->req_msg = (tape_msg_hdr_t *)malloc(count);
			bcopy(msg, context->req_msg, count);
		}
		break;

	case TAPE_MID_PASSTHRU_REQ:
		{ 
			int	count = sizeof(tape_passthru_req_msg_t);

			if (((tape_passthru_req_msg_t *)msg)->flags & TAPE_PASSTHRU_FLAGS_DATA_OUT)
				count += ((tape_passthru_req_msg_t *)msg)->data_len;
			context->req_msg = (tape_msg_hdr_t *)malloc(count);
			bcopy(msg, context->req_msg, count);
		}
		break;

	case TAPE_MID_SET_DEBUG_LEVEL_REQ:
		break;

	case MC_MID_GET_DEVLIST_REQ:
		break;

	case MC_MID_OPEN_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(mc_open_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(mc_open_req_msg_t));
		break;

	case MC_MID_CLOSE_REQ:
		context->req_msg = (tape_msg_hdr_t *)malloc(sizeof(mc_close_req_msg_t));
		bcopy(msg, context->req_msg, sizeof(mc_close_req_msg_t));
		break;

	case MC_MID_PASSTHRU_REQ:
		{ 
			int	count = sizeof(mc_passthru_req_msg_t);

			if (((mc_passthru_req_msg_t *)msg)->flags & MC_PASSTHRU_FLAGS_DATA_OUT)
				count += ((mc_passthru_req_msg_t *)msg)->data_len;
			context->req_msg = (tape_msg_hdr_t *)malloc(count);
			bcopy(msg, context->req_msg, count);
		}
		break;

	default:
		;
	}
	return(context);
}

void
tape_free_context(tape_msg_context_t *context)
{
	if (context->req_msg)
		free(context->req_msg);
	free(context);
}

void 
tape_close_all_devices(void)
{
	tape_dev_t *    dev;
	tape_dev_t *    next;

	if ((dev = tape_devs)) {
		do {
			next = dev->next;
			if (dev->state != TAPE_STATE_CLOSED) {
				TP_DBG(2, ("tape_close_all_devices: Closing %s dev 0x%p\n", 
					   (dev->mc ? "Media-changer" : "Tape"), dev));
				if (dev->pending_req_context) {
					free(dev->pending_req_context);
					dev->pending_req_context = NULL;
				}
				tape_do_sdm_close(dev, NULL);
			}
			else
				TP_DBG(2, ("tape_close_all_devices: No need to close %s dev 0x%p\n", 
					   (dev->mc ? "Media-changer" : "Tape"), dev)); 
			dev = next;
		} while (dev != tape_devs);
	}
}
