/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tftp-sim.c
 *
 * RCS:  $Id: tftp-sim.c,v 1.3 2001/11/20 19:13:46 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Used in Simulation environment to open TFTP socket.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 09/27/2001
 *
 *-----------------------------------------------------------------
 */
#ifdef LINUX_TEST

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#include <exports/tftp-api.h>
#include "tftp.h"

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

char tftp_rbuff[1024];
extern int  tftp_sock;


int 
tftp_simOpen( tftp_cb_t *tftpCb )
{
	struct sockaddr_in 	svr_addr;

    /*-------------------------------
     * Get TFTP UDP socket.
     *-------------------------------
     */
    tftp_sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (tftp_sock < 0)
    {
        printf ("can't open TFTP UDP socket: %s\n", sys_errlist[errno]);
        return -1;
    }

    /*-------------------------------
     * Make it non-blocking.
     *-------------------------------
     */
    if (fcntl(tftp_sock, F_SETFL, FNDELAY) < 0)
    {
        printf ("can't fcntl socket: %s\n", sys_errlist[errno]);
        return -1;
    }

    /*-------------------------------
     * Bind the TFTP socket.
     *-------------------------------
     */
    bzero( (char *) &svr_addr, sizeof(svr_addr) );
    svr_addr.sin_family      = AF_INET;
    svr_addr.sin_addr.s_addr = htonl(tftpCb->laddr);
    svr_addr.sin_port        = htons(tftpCb->lport);

    if (bind(tftp_sock, (struct sockaddr *)&svr_addr, sizeof(svr_addr)) < 0)
    {
        printf ("can't bind TFTP socket: %s\n", errno, sys_errlist[errno]);
        return -1;
    }

	return (tftp_sock);
}

tftp_simXmt( tftp_cb_t *tftpCb, pkt_desc_t *pDesc )
{
	struct sockaddr_in 	cli_addr;

    cli_addr.sin_family      = AF_INET;
    cli_addr.sin_port        = htons(tftpCb->rport);
    cli_addr.sin_addr.s_addr = htonl(tftpCb->raddr);
   
    if(sendto(tftp_sock, pDesc->pHdr, pDesc->hdrSize,
            0, &cli_addr, sizeof(cli_addr)) < 0)
    {   
        printf("Sendto failed %s\n", sys_errlist[errno]);
    }

	PKT_FREE_PACKET(pDesc);
}

tftp_simClose( int32 sock )
{
	close(sock);
}
#endif
