/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tftp-tpl.c
 *
 * RCS:  $Id: tftp-tpl.c,v 1.4 2002/02/13 18:24:13 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains TFTP to TPL interface routines.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 09/25/2001
 *
 *-----------------------------------------------------------------
 */
#include    <nfx-incl.h>

#include    <exports/pkt-api.h>
#include    <exports/luc-api.h>
#include    <exports/tpl-api.h>
#include    "../sm-tpl/tpl.h"
#include    <exports/tftp-api.h>
#include    "tftp.h"


/*-----------------------------------------------------------------
 * Name:        tftp_tplDelConnInd
 *
 * Description: Called by TPL when a connection is deleted.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/25/01
 *-----------------------------------------------------------------
 */
static int32	
tftp_tplDelConnInd( void *pHdl, void *pConn)
{
	tftp_cb_t	*tftpCb;

	printf("tftp_tplDelConnInd\n");

	tftpCb = pHdl;
	tftpCb->pTplConn = NULL;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:        tftp_tplAddConnInd
 *
 * Description: Called back from TPL when a new conn is added.
 *				This happens when TFTP server sends data on 
 *				different UDP port (instead of port 69).
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/25/01
 *-----------------------------------------------------------------
 */
static int32	
tftp_tplAddConnInd( tpl_bindCb_t	*pBind,
					void 			*pConnHdl, 
					void 			**pAppHdl, 
					uint32 			raddr, 
					ushort16 		rport )
{
	tpl_connCb_t *pConn;

	printf("tftp_tplAddConnInd\n");

	pConn = pConnHdl;

	TftpCbPtr->pTplConn = pConn;
	TftpCbPtr->raddr = raddr;
	TftpCbPtr->rport = rport;

	*pAppHdl = TftpCbPtr;

	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name:        tftp_tplXmtCmpInd
 *
 * Description: Called by TPL when a packet is sent.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/25/01
 *-----------------------------------------------------------------
 */
static int32	
tftp_tplXmtCmpInd( void *pHdl, pkt_desc_t *pDesc)
{
	/*-------------------------------
	 * Just free it back to TPL.
	 *-------------------------------
	 */
	return TPL_FREE_PKT;
}

/*-----------------------------------------------------------------
 * Name:        tftp_tplBindReq
 *
 * Description: Binds with TPL
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/25/01
 *-----------------------------------------------------------------
 */
int32
tftp_tplBindReq( tftp_cb_t *tftpCb )
{
	tpl_bindCb_t	tftpBindReq;
	tpl_bindCb_t	*pBindRsp;
	int32			rc;

	memset( &tftpBindReq, 0, sizeof(tpl_bindCb_t));

	tftpBindReq.laddr   	  = 0;
	tftpBindReq.lport   	  = ntohs(tftpCb->lport);
	tftpBindReq.tplProt 	  = TPL_PROT_UDP;
	tftpBindReq.ul_addConnInd = tftp_tplAddConnInd;
	tftpBindReq.ul_delConnInd = tftp_tplDelConnInd;
	tftpBindReq.ul_rcvPktInd  = tftp_rcvInd ;
	tftpBindReq.ul_xmtCmpInd  = tftp_tplXmtCmpInd ;

	rc = tpl_bindReq( &tftpBindReq, &pBindRsp, TPL_BIND_SERVER );
	if (rc != NFX_OK)
	{
		TFTP_LOGERR(TFTP_ERR_BIND_FAIL, &tftpBindReq );
		return (TFTP_ERR_BIND_FAIL);
	}

	tftpCb->pTplBind = pBindRsp;
	tftpCb->pBsdSock = pBindRsp->pBsdSock;
	
	return NFX_OK;
}
