/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tftp.h
 *
 * RCS:  $Id: tftp.h,v 1.1.1.1 2001/10/12 03:52:49 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains TFTP protocol internal definitions & prototypes.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 09/15/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _TFTP_H
#define _TFTP_H

#define TFTP_LOGERR( code, info )	tftp_logErr( __FILE__, __LINE__, code,info)
#define	TFTP_PANIC()				{ *(char *)0 = 1; }

/*-------------------------------
 * TFTP Opcodes
 *-------------------------------
 */
#define TFTP_OP_RRQ					1		
#define TFTP_OP_WRQ					2
#define TFTP_OP_DATA				3
#define TFTP_OP_ACK					4
#define TFTP_OP_ERROR				5

#define TFTP_SRVR_PORT				69		/* tftp server port */

#define TFTP_SEG_SIZE				512		/* per RFC 783 */
#define TFTP_ACK_SIZE				4		/* 4 bytes ack pkt */

#define TFTP_MAX_FILENAME			128		/* Filename + NULL char */
#define TFTP_MAX_RETRY				5		
#define TFTP_RETRY_TIME				5000	/* 5 secs betwee retries */


/*-------------------------------
 * TFTP Statistics
 *-------------------------------
 */
typedef struct 
{
	uint32		rcvRrqCnt;		/* RRQ received */
	uint32		rcvWrqCnt;		/* WRQ received */
	uint32		rcvDataCnt;		/* Data received */
	uint32		rcvAckCnt;		/* Ack received */
	uint32		rcvErrCnt;		/* Err received */
	uint32		xmtRrqCnt;		/* RRQ sent */
	uint32		xmtWrqCnt;		/* WRQ sent */
	uint32		xmtDataCnt;		/* Data sent */
	uint32		xmtAckCnt;		/* Ack sent */
	uint32		xmtErrCnt;		/* Err sent */
	uint32		xmtRetryCnt;	/* # of send retries */
	uint32		xmtNoRsrcCnt;	/* # of no resource err */
} tftp_stats_t;

/*-------------------------------
 * TFTP Control Block 
 *-------------------------------
 */
typedef struct _tftpCb_t 
{
	void			*pTplBind;	/* TPL Bind CB ptr */
	void			*pTplConn;	/* TPL Conn CB ptr */
	void			*pBsdSock;	/* BSD socket ptr */
	uint32			laddr;		/* Local IP */
	uint32			raddr;		/* Remote IP */
	ushort16		lport;		/* Local port */
	ushort16		rport;		/* Remote port */
    ushort16		block;		/* data block Id received */
#define TFTP_FLAG_ERR				0x4000
#define TFTP_FLAG_EOF				0x8000
    ushort16		flags;
    short16			mode;		/* Ascii or binary */
    short16			retry;		/* Tx retry count */
	int32			tmr_id;		/* Timer ID */
	int32			(*dataInd)( char *pData, int32 size);/* App data upcall*/
	int32			(*eventInd)(void *tftpCb,int32 event ); /* event upcall*/
	char			filename[TFTP_MAX_FILENAME];		/* xfer file name */
	tftp_stats_t	stats;								/* tftp stats */
} tftp_cb_t;

/*-------------------------------
 * TFTP packet header
 *-------------------------------
 */
typedef struct  _tftp_hdr_t
{
	short16			th_opcode;				/* packet type */
	union 
	{
		ushort16	tu_block;       		/* block # */
		short16		tu_code;                /* error code */
		char		tu_stuff[1];			/* request packet stuff */
	} __attribute__ ((__packed__)) th_u;
	char			th_data[1];				/* data or error string */
} __attribute__ ((__packed__)) tftp_hdr_t;

#define th_block        th_u.tu_block
#define th_code         th_u.tu_code
#define th_stuff        th_u.tu_stuff
#define th_msg          th_data

int32 tftp_rcvInd(void *pHdl, pkt_desc_t *pDesc);

void tftp_sendRrq( tftp_cb_t *tftpCb );
void tftp_sendAck( tftp_cb_t *tftpCb );

int32 tftp_tplBindReq( tftp_cb_t *tftpCb );


extern	tftp_cb_t	*TftpCbPtr;
#endif /* _TFTP_H */
