/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: cifsauth-api.h
 *
 * RCS:  $Id: cifsauth-api.h,v 1.6 2002/02/04 23:03:32 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: This interface is used by the embedded CIFS protocol SW
 *              to perform pass-through authentication with NT domain
 *              controllers. NT domain/file system IP address relationship
 *              is maintained externally to this interface.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 7 Dec 2001
 *
 *-----------------------------------------------------------------
 */
#include "nfx-incl.h"

#define CIFSAUTH_REQ (CIFS_MSG_BASE + 101)
#define CIFSAUTH_INFO (CIFS_MSG_BASE + 102)

typedef struct {
	uint32		msgType;				// The unique message id
	uint64		clientPrivate;			// opaque cookie to identify response
} cifsauthHdr_t;

typedef char nt_user_t[1024];			// NULL terminated string
typedef char nt_domain_t[256];			// NULL terminated string

typedef struct {
		int len;						// Encrypted passwords always 24 chars
		char blob[24];
} pass_t;

// Send this message to the "cifs-auth" application on the SSC

#define IN_ADDR_T uint32_t	// Would have used <sys/types.h> except for this 
							// needs to compile for the TX/RX.

typedef struct {
	cifsauthHdr_t	hdr;				// CIFSAUTH_REQ
	IN_ADDR_T		fromAddr;			// On this IP address
	nt_user_t		userName;			// This user
	nt_domain_t		domName;			// Saying this domain
	char			chal[8];			// We gave them this challenge
	pass_t			lm_resp;			// presented these encrypted passwords
	pass_t			nt_resp;
} cifsauthReqMsg_t;

// and get this in response

typedef struct {
	uint32      g_rid;
	uint32      attr;
} ca_gid_t;

typedef struct {						// Important bits from NET_USER_INFO_3
	uint32			user_rid;
	uint32			group_rid;
	uint32			user_flgs;

	uint32			num_gids;			// num_groups2
	ca_gid_t		gids[10];

	uint32			num_auths;			// dom_sid
	uint8			sid_rev_num;		// sid
	uint8			id_auth[6];
	uint32			sub_auths[15];
} cifsauthCred_t;

typedef enum {
		stat_ok,						// All is well
		failed_logon,					// couldn't log on to domain
		no_pdc,							// couldn't find PDC
		no_trust_pw						// couldn't get trust acct password
} cifsauthStat_t;

typedef struct {
	cifsauthHdr_t	hdr;				// CIFSAUTH_INFO
	cifsauthStat_t	status;				// Good(0) or bad (and possibly why)
	uint32			ntstatus;			// if we got such a beast
	cifsauthCred_t	userCred;			// If good, the user's credentials
} cifsauthInfoMsg_t;
