// vi:ts=4
#include <fcntl.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "exports/anpssc-api.h"
#include "cifsauth-api.h"

static void
printCred (cifsauthInfoMsg_t *r)
{
	cifsauthCred_t *c = &(r->userCred);
	int i;

	if (r->status != NFX_OK) {
		printf("Request failed\n");
		return;
	}
	printf ("urid: %d, grid: %d, uflgs: 0x%x\n", c->user_rid, c->group_rid, 
	        c->user_flgs);

	printf ("numGids: %d\n", c->num_gids);
	printf ("g_rid/attr: ");
	for (i = 0; i < 10; i++) {
		printf ("%d/0x%x ", c->gids[i].g_rid, c->gids[i].attr);
	}
	printf ("\n");

	printf ("num_auths: %d, sid_rev: %d\n", c->num_auths, c->sid_rev_num);
	printf ("id_auth: ");
	for (i = 0; i < 6; i++) {
		printf ("%x ", c->id_auth[i]);
	}
	printf ("\n");

	printf ("sub_auths: ");
	for (i = 0; i < 15; i++) {
		printf ("%x ", c->sub_auths[i]);
	}
	printf ("\n");
}

extern int
main (int argc, char *argv[])
{
	int fd;
	int sz;
	char buf[1024];
	cifsauthReqMsg_t rq;
	cifsauthInfoMsg_t rsp;
	int rc;

	fd = open (argv[1], O_RDONLY, 0);
	if (fd < 0) {
		perror ("Open");
		return 1;
	}

	while (read (fd, &sz, 4) == 4) {
		memset (&rq, 0, sizeof(rq));
		rq.hdr.msgType = CIFSAUTH_REQ;
		rq.hdr.clientPrivate = getpid();

		if (read (fd, buf, sz) != sz) {
			perror ("short user name");
			return 1;
		}
		memcpy (rq.userName, buf, sizeof (rq.userName));

		if (read (fd, &sz, 4) != 4) {
			perror ("short domain size");
			return 1;
		}
		if (read (fd, buf, sz) != sz) {
			perror ("short domain name");
			return 1;
		}
		memcpy (rq.domName, buf, sizeof (rq.domName));

		if (read (fd, &sz, 4) != 4) {
			perror ("short lm_resp size");
			return 1;
		}
		if (read (fd, buf, sz) != sz) {
			perror ("short lm_resp name");
			return 1;
		}
		rq.lm_resp.len = sz;
		memcpy (rq.lm_resp.blob, buf, sizeof (rq.lm_resp.blob));

		if (read (fd, &sz, 4) != 4) {
			perror ("short nt_resp size");
			return 1;
		}
		if (read (fd, buf, sz) != sz) {
			perror ("short nt_resp name");
			return 1;
		}
		rq.nt_resp.len = sz;
		memcpy (rq.nt_resp.blob, buf, sizeof (rq.nt_resp.blob));

		if (read (fd, &sz, 4) != 4) {
			perror ("short chal size");
			return 1;
		}
		if (read (fd, buf, sz) != sz) {
			perror ("short chal name");
			return 1;
		}
		memcpy (rq.chal, buf, sizeof(rq.chal));

		rq.fromAddr = inet_addr("10.0.0.127");

		printf ("Sending: %s\n", rq.userName);

		rc = sendAgileMsg (-1, &rq, sizeof(rq), "cifsauth", 0, 0, 0, NULL,
		                   &rsp, sizeof(rsp));

		printf ("rc: %d, msgType: %d, cp: %lld\n", rc, 
		         rsp.hdr.msgType, rsp.hdr.clientPrivate);
		printCred (&rsp);
		printf ("\n");
	}
	return 0;
}
