/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ccm-api.h
 *
 * RCS:  $Id: ccm-api.h,v 1.14 2002/10/04 02:17:05 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: How to talk to the CCM
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 14 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#if !defined (__ccm_api_h_)
#define __ccm_api_h_

#include "exports/opt-api.h"	// Need the option string lengths

typedef enum {
	ccmLclCfgRq = CCM_MSG_BASE	// Give me your local config info
	, ccmLclCfg					// Sync local config
	, ccmShutDown				// System is going down
	, ccmTakeOver				// Take over a node's services
    , ccmStatRsp				// General status respose
	, ccmCfgSync				// PCC to SCC config sync
	, ccmRqPCCName				// Get this, return real name
	, ccmPCCName				// and here it is
} ccmMsg_t;

#define MXNDNAMLEN 256
#define NMMGMTPORTS 2

// All messages have this header

typedef struct {
	ccmMsg_t		msgType;			// What message it is
	uint64			clientPrivate;		// Something you may use to id repsonse
} ccmHdr_t;

typedef struct {
	ccmHdr_t		hdr;				// ccmStatRsp
	uint32			status;				// True or false wrt Rq complete
} ccmRsp_Msg_t;

// By operator command on the PCC, transfer the services assigned to old_node
// to new_node. Gets you ccmStatRsp when take over complete.

typedef struct {
	ccmHdr_t        hdr;				// ccmTakeOver
	char			new_node[MXNDNAMLEN];// Make this node takeover
	char			old_node[MXNDNAMLEN];// this nodes services
} ccmTakeOverRq_t;

// CCM on the PCC uses this to request a slave node to sync with it

typedef struct {
	ccmHdr_t		hdr;				// ccmLclCfgRq PCC wants our local cfg
} ccmLclCfgRq_t;

// Slave nodes send many of these to PCC when it asks via ccmLclCfgRq

typedef struct {
	ccmHdr_t		hdr;				// ccmLclCfg
	char			name[MXNDNAMLEN];	// Its name
	char			cfg_pair[1];		// Really two \0 terminated strings
} ccmLclCfg_t;

// PCC sends these to SCC to synchronize cluster config

typedef struct {
	ccmHdr_t		hdr;				// ccmCfgSync
	char			cfg_strings[1];		// Really two \0 terminated strings
} ccmCfgSync_t;

typedef struct {
	ccmHdr_t		hdr;				// ccmShutDown
	char 			fromNode[MXNDNAMLEN];// Its name
	int				done;				// In the response set true if all OK
} ccmShutdown_Msg_t;

typedef struct {
	ccmHdr_t		hdr;				// ccmRqPCCName
} ccmRqPCCName_t;

typedef struct {
	ccmHdr_t		hdr;				// ccmPCCName
	char			PCCName[MXNDNAMLEN];// This be it
} ccmPCCName_t;

#endif
