/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ccm-route.c
 *
 * RCS:  $Id: ccm-route.c,v 1.4 2002/06/14 15:18:49 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Contains the things that CCM uses to deal with routes
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 2 May 2002
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>

#include "nfx-defs.h"
#include "nfx-types.h"
#include "exports/evm-api.h"
#include "exports/elog-api.h"
#include "exports/ipmd-lib.h"
#include "exports/event-api.h"

#include "ccm-api.h"
#include "ncm-api.h"
#include "ccm-loc.h"

#define ROUTE_OBJS    "cluster.route.*"
#define ROUTE_OBJ_FMT "cluster.route.%08x.%08x"

// Record the route change event

extern void
rte_chg (nfx_eventMsg_t *msg)
{

	char **rte_obj;
	char *rte_key;

	E_LOG (class_1, debug_s, myappid, 0, 0, 0,
	       ("ccm got route %s event for %08x/%08x gw %08x", 
	        msg->param.ipRteEvent.state == EVENT_STATE_REMOVE ? 
	        "delete" : "add", msg->param.ipRteEvent.destNet, 
	        msg->param.ipRteEvent.destMask,
	        msg->param.ipRteEvent.gateWay));

	if (msg->param.ipRteEvent.state == EVENT_STATE_REMOVE) {
		asprintf (&rte_key, "cluster.route.%08x.*", 
		          msg->param.ipRteEvent.destNet); 
		rte_obj = get_cfg_val (rte_key);
		if (rte_obj) {
			del_cfg_obj (rte_obj);
			free (rte_obj[1]);
			rte_obj[1] = NULL;
			sync_SCC (rte_obj);
		}
		else {
			E_LOG (class_1, debug_s, myappid, 0, 0, 0,
			       ("Got delete event but route not in cfg %s", rte_key));
		}
		free (rte_key);
	}
	else {
		rte_obj = malloc (3 * sizeof (char *));
		asprintf (&(rte_obj[0]), ROUTE_OBJ_FMT, 
		          msg->param.ipRteEvent.destNet, 
		          msg->param.ipRteEvent.destMask);
		asprintf (&(rte_obj[1]), "%08x", msg->param.ipRteEvent.gateWay);
		rte_obj[2] = NULL;

		put_cfg_val (rte_obj);
		sync_SCC (rte_obj);
	}

	free_cfg_val (rte_obj);
}

// Given this IP address & mask, restore the routers it reaches

extern void
restore_rte (uint32 if_addr, uint32 mask, char *node)
{
	char **route_lst;
	char **rte;
	uint32 gw_addr;
	uint32 net_addr;
	uint32 net_mask;
	uint32 net_part;
	char *it;

	net_part = if_addr & mask;

	route_lst = get_cfg_val (ROUTE_OBJS);
	if (route_lst) {

		for (rte = route_lst; *rte; rte += 2) {
			gw_addr = strtol (rte[1], NULL, 16);

			if ((gw_addr & mask) == net_part) {

				it = rte[0] + strlen (ROUTE_OBJS) - 1; // fuget da '*'
				net_addr = strtol (it, NULL, 16);
				net_mask = strtol (&(it[9]), NULL, 16);

				if (ipmd_AddRtCluster (net_addr, net_mask, gw_addr, 
				    NULL, -1, node) == NFX_OK) {
					E_LOG (class_1, debug_s, myappid, 0, 0, 0,
					       ("route restored %x/%x to %x", net_addr, 
					        net_mask, gw_addr));
				}
				else {
					E_LOG (class_1, err_s, myappid, 0, 0, 0,
					       ("FAILED route restore %x/%x to %x", net_addr, 
					        net_mask, gw_addr));
				}
			}
		}

		free_cfg_val (route_lst);
	}
}
