/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ncm-api.h
 *
 * RCS:  $Id: ncm-api.h,v 1.9 2002/10/08 00:40:35 syen Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: How you send node-directed application messages
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 5 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#if !defined(__ncm_api_h)
#define __ncm_api_h

#include <sys/types.h>				// We use the BSD types

#include "nfx-defs.h"

typedef enum {
	NCM_SEND = NCM_MSG_BASE,		// Send it to node/app
	NCM_RESP,						// Response from node/app
	NCM_RQ_NODE_LIST,				// Someone wants connected node list
	NCM_NODE_LIST,					// These are the nodes that are connected
	NCM_NODE_EVENT,					// Node arrived or departed (spm)
	NCM_REJECT,						// NCM_SEND Rejected
	NCM_SEND_EMB					// Send this to an embedded CPU
} ncmMsg_type_t;

#define MAXNAME 256

// This encapsulates node-directed application messages. The applications
// WERE on the SSC only.
// The NCM_SEND_EMB message was created to allow sending from SSC on
// one node to an embedded application on another node. Why didn't I
// do this in the first place? Didn't think we'd need it.
// You'll want to either malloc() a buffer for this or cast it to a
// long long buffer on the stack to ensure proper address alignment.

typedef struct {
	ncmMsg_type_t	msgType;			// NCM_SEND, NCM_SEND_EMB or NCM_RESP
	char			appName[MAXNAME];	// To/From what Application
	char			nodeName[MAXNAME];	// on what node
	int				rqResp;				// I expect a response
	int				slot;				// Which slot - NCM_SEND_EMB only
	int				cpu;				// Which CPU - NCM_SEND_EMB only
	long long		appMsg[1];			// App msg starts here (8 byte aligned)
} ncmMsg_t;
#define NCM_MSGHDR_LEN	(sizeof(ncmMsg_t) - sizeof(long long))

typedef enum {
	INVALID_DEST_NODE				// Destination node doesn't exist
} ncmRej_t;

typedef struct {
	ncmMsg_type_t	msgType;			// NCM_REJECT
	ncmRej_t		reason;				// Why request was rejected
} ncmReject_t;

typedef struct {
	ncmMsg_type_t	msgType;			// NCM_RQ_NODE_LIST
} ncmRqNodeList_t;

typedef struct {
	ncmMsg_type_t	msgType;			// NCM_NODE_LIST
	int				numConnNodes;		// How many nodes are connected
	char			nodeNames[1][MAXNAME];// Array of MAXNAME arrays
} ncmNodeList_t;

typedef struct {
	ncmMsg_type_t	msgType;			// NCM_NODE_EVENT
	int				new;				// True (!0) = new node
	char			nodeName[MAXNAME];	// Called themselves this
} ncmnodeEvent_t;
#endif
