// vi:ts=4
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>


#include "nfx-defs.h"
#include "exports/anpssc-api.h"
#include "exports/evm-api.h"
#include "exports/event-api.h"
#include "ccm-api.h"

#define LVID 0xbeefbabedead

#define EXPVOL1 "/exportVol"
#define EXPADDR1 "10.0.0.185"
#define EXPVOL2 "/exportAgain"
#define EXPADDR2 "10.1.0.127"
#define P1 "fp1.0"
#define P2 "fp1.1"

static char node[80];

static void
mkvis(char *node, uint64 id)
{
	nfx_eventMsg_t vis;
	int rc;

	vis.msgType = NFX_EVENT_POST;
	strcpy (vis.nodeName, node);
	vis.eventType = NFX_EVENT_VOLUME;
	vis.param.volEvent.event = EVENT_VOL_CREATE;
	vis.param.volEvent.volId.whole_thing = id;
	vis.param.volEvent.state = EVENT_STATE_UP;

	rc = sendAgileMsg (-1, &vis, sizeof(vis),
	                   "ccm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

static void
lvonline(char *name, uint64 id)
{
	evm_MsgLVolEvt_t ol;
	int rc;

	ol.hdr.msgType = EVM_LVOL_EVT;
	ol.hdr.clientPrivate = getpid();
	strcpy (ol.node, name);
	ol.lvid.whole_thing = id;
	ol.event = EVM_LVOL_ONLINE_E;

	rc = sendAgileMsg (-1, &ol, sizeof(ol),
	                   "ccm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

static void
lvoffline(char *node, uint64 id)
{
	evm_MsgLVolEvt_t ol;
	int rc;

	ol.hdr.msgType = EVM_LVOL_EVT;
	ol.hdr.clientPrivate = getpid();
	strcpy (ol.node, node);
	ol.lvid.whole_thing = id;
	ol.event = EVM_LVOL_OFFLINE_E;

	rc = sendAgileMsg (-1, &ol, sizeof(ol),
	                   "ccm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

static void
lvdestroy(char *node, uint64 id)
{
	evm_MsgLVolEvt_t ol;
	int rc;

	ol.hdr.msgType = EVM_LVOL_EVT;
	ol.hdr.clientPrivate = getpid();
	strcpy (ol.node, node);
	ol.lvid.whole_thing = id;
	ol.event = EVM_LVOL_DESTROY_E;

	rc = sendAgileMsg (-1, &ol, sizeof(ol),
	                   "ccm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

static void
lvexport(char *nd, char *expvol, char *expaddr, int un, uint64 id)
{
	nfx_eventMsg_t exp_evt;
	int rc;

	memset (&exp_evt, 0, sizeof(exp_evt));

	exp_evt.msgType = NFX_EVENT_POST;
	strcpy (exp_evt.nodeName, nd);
	exp_evt.eventScope = NFX_EVENT_LOCAL;
	exp_evt.eventType = NFX_EVENT_NFS;

	exp_evt.param.exportEvent.volId.whole_thing = id;
	exp_evt.param.exportEvent.expAddr = inet_addr(expaddr);
	strcpy (exp_evt.param.exportEvent.expPath, expvol);
	strcpy (exp_evt.param.exportEvent.expOptions, 
	        "fred(rw) joe(ro) tim(rw,no_root_squash)");
	exp_evt.param.exportEvent.state = (un) ? EVENT_STATE_DOWN : EVENT_STATE_UP;

	rc = sendAgileMsg (-1, &exp_evt, sizeof(exp_evt),
	                   "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

static void
cfgGbE (char *nd, char *port, in_addr_t ip, int mask, int del)
{
	nfx_eventMsg_t msg;
	int rc;

	memset (&msg, 0, sizeof(msg));

	msg.msgType = NFX_EVENT_POST;
	msg.eventType = NFX_EVENT_IP_IF;
	strcpy (msg.param.ipIfEvent.portName, nd);
	strcat (msg.param.ipIfEvent.portName, ":");
	strcat (msg.param.ipIfEvent.portName, port);
	msg.param.ipIfEvent.ipa = ip;
	msg.param.ipIfEvent.mask = mask;
	msg.param.ipIfEvent.state = del ? EVENT_STATE_REMOVE : EVENT_STATE_UP;

	rc = sendAgileMsg (-1, &msg, sizeof(msg),
	                   "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
	                   NULL, 0);
	if (rc) {
		printf ("Should not be %d\n", rc);
	}
}

extern int
main()
{
	char line[80];
	uint64 lv_id = LVID;

	gethostname (node, sizeof(node));

	while (1) {
		printf ("Currently selected target node: %s\n", node);
		printf ("n\t- Set target node name\n");
		printf ("v\t- Make %llx visible\n", lv_id);
		printf ("i\t- Set LV id\n");
		printf ("d\t- destroy\n");
		printf ("on\t- Online\n");
		printf ("off\t- Offline\n");
		printf ("e1\t- Export as %s:%s\n", EXPVOL1, EXPADDR1);
		printf ("e2\t- Export as %s:%s\n", EXPVOL2, EXPADDR2);
		printf ("u1\t- Unexport as %s:%s\n", EXPVOL1, EXPADDR1);
		printf ("u2\t- Unexport as %s:%s\n", EXPVOL2, EXPADDR2);
		printf ("p1\t- Configure port %s as %s\n", P1, EXPADDR1);
		printf ("p2\t- Configure port %s as %s\n", P2, EXPADDR2);
		printf ("x1\t- Unconfigure port %s as %s\n", P1, EXPADDR1);
		printf ("x2\t- Unconfigure port %s as %s\n", P2, EXPADDR2);

		printf ("-> ");
		fgets (line, sizeof(line), stdin);

		if (feof(stdin)) {
			printf("\n\n");
			exit(0);
		}

		if (!strncmp(line, "v", 1)) {
			mkvis(node, lv_id);
		}
		else if (!strncmp(line, "n", 1)) {
			printf ("Set new target node to ->");
			fgets (node, sizeof(node), stdin);
			*strchr(node, '\n') = '\0';
		}
		else if (!strncmp(line, "i", 1)) {
			printf ("Set new LV id to ->");
			fgets (line, sizeof(line), stdin);
			*strchr(line, '\n') = '\0';
			lv_id = strtoq (line, NULL, 16);
		}
		else if (!strncmp(line, "d", 1)) {
			lvdestroy(node, lv_id);
		}
		else if (!strncmp(line, "on", 2)) {
			lvonline(node, lv_id);
		}
		else if (!strncmp(line, "off", 3)) {
			lvoffline(node, lv_id);
		}
		else if (!strncmp(line, "e1", 2)) {
			printf ("exporting as %s:%s\n", EXPVOL1, EXPADDR1);
			lvexport(node, EXPVOL1, EXPADDR1, 0, lv_id);
		}
		else if (!strncmp(line, "e2", 2)) {
			printf ("exporting as %s:%s\n", EXPVOL2, EXPADDR2);
			lvexport(node, EXPVOL2, EXPADDR2, 0, lv_id);
		}
		else if (!strncmp(line, "u1", 2)) {
			printf ("unexporting as %s:%s\n", EXPVOL1, EXPADDR1);
			lvexport(node, EXPVOL1, EXPADDR1, 1, lv_id);
		}
		else if (!strncmp(line, "u2", 2)) {
			printf ("unexporting as %s:%s\n", EXPVOL2, EXPADDR2);
			lvexport(node, EXPVOL2, EXPADDR2, 1, lv_id);
		}
		else if (!strncmp(line, "p1", 2)) {
			cfgGbE (node, P1, inet_addr(EXPADDR1), 0xffffff, 0);
		}
		else if (!strncmp(line, "p2", 2)) {
			cfgGbE (node, P2, inet_addr(EXPADDR2), 0xffffff, 0);
		}
		else if (!strncmp(line, "x1", 2)) {
			cfgGbE (node, P1, inet_addr(EXPADDR1), 0xffffff, 1);
		}
		else if (!strncmp(line, "x2", 2)) {
			cfgGbE (node, P2, inet_addr(EXPADDR2), 0xffffff, 1);
		}
		else {
			printf ("Pbbthtt 8-P\n");
		}
	}

}
