/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: declic
 *
 * RCS:  $Id: declic.c,v 1.6 2002/08/28 22:10:16 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Decode license key to see if it's good &c.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 04 Jan 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#if defined (__ADDER) || defined (__DELER)
#include <fcntl.h>
#include <db.h>
#include <time.h>

#include "licman-api.h"
#endif

#include "licrec.h"

extern int main (int argc, char *argv[])
{
	key_rec_t lic_rec;
	u_int32_t top_part;
	u_int32_t mid_part;
	u_int64_t bot_part;
	u_int32_t chassis_id;
	u_int32_t prod_id;
	u_int32_t sw_prod_id;
	u_int32_t hw_prod_id;
	u_int32_t sw_rev;
	u_int32_t sernum;
	time_t expdate;
#if defined (__DECODER)
	char *expstr;
#endif
	char *nxt_field;
	u_int8_t sum, *keyptr;
	int i;
#if defined (__ADDER) || defined (__DELER)
	DB *licDb;
	DBT keyThang;
	DBT dataThang;
#endif

	if (argc != 2) {
		fprintf (stderr, "Usage: %s key\n", argv[0]);
		return 1;
	}

	nxt_field = argv[1];
	top_part = strtol (strsep (&nxt_field, "-"), NULL, 16);
	if (!nxt_field) {
		fprintf (stdout, "Invalid key\n");
		return 1;
	}
	mid_part = strtol (strsep (&nxt_field, "-"), NULL, 16);
	if (!nxt_field) {
		fprintf (stdout, "Invalid key\n");
		return 1;
	}
	bot_part = strtoq (nxt_field, NULL, 16);

	lic_rec.lic_key = bot_part | ((u_int64_t)mid_part << 40) |
	            ((u_int64_t)top_part << 56);

	keyptr = (u_int8_t *)&lic_rec.lic_key;
	for (sum = 0, i = 0; i < sizeof (lic_rec.lic_key); i++) {
		sum ^= keyptr[i];
	}

	if (sum != 0xff) {
		fprintf (stderr, "Invalid key\n");
		return 1;
	}

	prod_id = mid_part;
	sw_prod_id = prod_id & 0xff;
	hw_prod_id = (prod_id >> 8) & 0xff;
	sernum = (u_int32_t)(bot_part & 0xfffff);
	expdate = (time_t)(sernum & 0xfff) << 20;
	sw_rev = sernum >> 12;
	chassis_id = (u_int32_t)(bot_part >> 20);
	chassis_id ^= prod_id ^ (~sernum & 0xfffff);

#if defined (__DECODER)
	fprintf (stdout, "SW Prod Id: %d\n", sw_prod_id);
	fprintf (stdout, "HW Prod Id: %d\n", hw_prod_id);
	fprintf (stdout, "SW Rev: %d\n", sw_rev);
	fprintf (stdout, "Chassis Id: %d\n", chassis_id);
	if (expdate == 0xfff00000) {
		expstr = "Perpetual";
	}
	else {
		asprintf (&expstr, "Trial expires %s", asctime (gmtime(&expdate)));
	}
	fprintf (stdout, "Expires: %s\n", expstr);
#endif

#if defined (__ADDER) || defined (__DELER)
	if ((sw_prod_id <= 0) || (sw_prod_id >= LIC_INVALID)) {
		fprintf (stderr, "Invalid key\n");
		return 1;
	}

	licDb = dbopen (LICDBFILE, O_CREAT | O_RDWR,
	                0644, DB_BTREE, NULL);

	if (!licDb) {
		perror ("Unable to open license database");
		return 1;
	}

	keyThang.data = &sw_prod_id;
	keyThang.size = sizeof (sw_prod_id);
	dataThang.data = &lic_rec;
	dataThang.size = sizeof (lic_rec);

#if defined (__ADDER)
	lic_rec.install_time = time(NULL);
	if (licDb->put (licDb, &keyThang, &dataThang, 0)) {
		fprintf (stderr, "Error writing to license database\n");
	}
#else
	if (licDb->del (licDb, &keyThang, 0)) {
		fprintf (stderr, "Error deleting key from database\n");
	}
#endif
	licDb->close (licDb);
#endif
	return 0;
}
