/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: cmd_disk.c
 *
 * RCS:  $Id: cmd_disk.c,v 1.3 2002/04/12 18:18:46 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Frank Salzmann 
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

#undef USE_DANGEROUS_CMDS	// Define this if you want the write and copy cmds
/*-----------------------------------------------------------------
 * Name: disk_read
 *
 * Description: Raw disk block read.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (disk_read_blk,
         disk_read_blk_cmd,
         "disk read WWN LUN LBA LBC [-s]",
         "disk Command\n"
         "Read disk block command.\n"
         "Device world-wide name\n"
         "Device lun id\n"
         "Logical block address\n"
         "Logical block count. Must be even.\n"
         "Optional superblock read\n")
{
	execute_command ("/usr/local/agile/bin/read", argc, 
    	             argv[0], argv[1], argv[2], argv[3], argv[4], NULL);
    return CMD_SUCCESS;
} // disk_read  cmd

#if defined(USE_DANGEROUS_CMDS)
/*-----------------------------------------------------------------
 * Name: disk_write
 *
 * Description: Raw disk block write.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (disk_write,
         disk_write_cmd,
         "disk write WWN LUN LBA LBC",
         "disk Command\n"
         "Write disk block command.\n"
         "Device world-wide name\n"
         "Device lun id\n"
         "Logical block address\n"
         "Logical block count. Must be even.\n")
{
	printf("Command Not Implimented.\n");
    return CMD_SUCCESS;
} // disk_write  cmd

/*-----------------------------------------------------------------
 * Name: disk_copy
 *
 * Description: Send mode select command to scsi storage device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (disk_copy,
         disk_copy_cmd,
         "disk copy WWN LUN WWN LUN",
         "disk Command\n"
         "Copy disk command.\n"
         "device world-wide name\n"
         "device lun id\n"
         "device world-wide name\n"
         "device lun id\n")
{
	//printf("Command Not Implimented.\n");
    return CMD_SUCCESS;
} // disk_copy  cmd
#endif

/*-----------------------------------------------------------------
 * Name: disk_erase
 *
 * Description: Erase disk contents.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (disk_erase_blk,
         disk_erase_blk_cmd,
         "disk erase WWN LUN LBA LBC",
         "disk Command\n"
         "Erase disk block command.\n"
         "Device world-wide name\n"
         "Device lun id\n"
         "Logical block address\n"
         "Logical block count. Must be even. 256 Max.\n")
{
	execute_command ("/usr/local/agile/bin/write", argc, 
    	             argv[0], argv[1], argv[2], argv[3], NULL);
    return CMD_SUCCESS;
} // disk_erase  cmd

void
install_disk (priv_t myPriv, enum node_type node)
{
	switch (myPriv)
    {
        case diagPriv:
			install_element (node, &disk_read_blk_cmd);	
#if defined(USE_DANGEROUS_CMDS)
			install_element (node, &disk_write_cmd);	
			install_element (node, &disk_copy_cmd);	
#endif
			install_element (node, &disk_erase_blk_cmd);	

        case fullPriv:
        case changePriv:
        case addPriv:
        case viewPriv:
    }
    return;
}
