/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: cmd_license
 *
 * RCS:  $Id: cmd_license.c,v 1.3 2002/04/16 17:24:38 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 25 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

#define LIC_STR "Manage Software Licenses\n"

DEFUN	(lic_show,
		show_cmd,
		"license show",
		LIC_STR
		"Show installed software licenses\n")
{
	execute_command("/usr/local/agile/bin/dumplic", 0, NULL, NULL);
	return CMD_SUCCESS;
}

DEFUN	(lic_add,
		add_cmd,
		"license add LICENSE-KEY",
		LIC_STR
		"Add key to license database\n")
{
	become_root();
	execute_command("/usr/local/agile/bin/addlic", 1, argv[0], NULL);
	unbecome_root();
	return CMD_SUCCESS;
}

DEFUN	(lic_del,
		del_cmd,
		"license del LICENSE-KEY",
		LIC_STR
		"Remove key from license database\n")
{
	become_root();
	execute_command("/usr/local/agile/bin/dellic", 1, argv[0], NULL);
	unbecome_root();
	return CMD_SUCCESS;
}

void
install_license (priv_t myPriv, enum node_type node)
{
/*
	Organizing this switch statment such that the higher priority commands
	are first allows us to fall through and pick up the lower priority
	ones at no extra cost.
 */
	switch (myPriv) {
		case diagPriv:
		case fullPriv:
			install_element (node, &del_cmd);
		case changePriv:
		case addPriv:
			install_element (node, &add_cmd);
		case viewPriv:
/*
	These commands are available in all modes
 */
			install_element (node, &show_cmd);
	}
}
