/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: cmd_tape.c
 *
 * RCS:  $Id: cmd_tape.c,v 1.2 2002/04/12 02:15:59 raster Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Frank Salzmann 
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

DEFUN   (tape_devlist,
         tape_devlist_cmd,
         "tape devlist",
         "Tape Command\n"
         "Get list of available tape devices.\n")
{
    execute_command ("tape-ui", 1, "devlist");
    return CMD_SUCCESS;
} 

DEFUN   (tape_mc_devlist,
         tape_mc_devlist_cmd,
         "tape mc devlist",
         "Tape Command\n"
		 "Media Changer command\n"
         "Get list of available media changer devices.\n")
{
    execute_command ("mc-ui", 1, "devlist");
    return CMD_SUCCESS;
} 

DEFUN   (tape_open,
         tape_open_cmd,
         "tape open DEVNAME",
         "Tape Command\n"
         "Open a tape device.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "open", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_close,
         tape_close_cmd,
         "tape close DEVNAME",
         "Tape Command\n"
         "Close a tape device.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "close", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_devstate,
         tape_devstate_cmd,
         "tape devstate DEVNAME",
         "Tape Command\n"
         "Show state of a tape device.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "devstate", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_sniffer,
         tape_sniffer_cmd,
         "tape sniffer DEVNAME",
         "Tape Command\n"
         "Diagnostic tape media scan utility.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-sniffer", 1, argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_fsr,
         tape_fsr_cmd,
         "tape fsr DEVNAME RECCOUNT",
         "Tape Command\n"
         "Forward space records.\n"
		 "Physical or Logical tape device name.\n"
		 "Record count.\n")
{
    execute_command ("tape-ui", 3, "fsr", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_bsr,
         tape_bsr_cmd,
         "tape bsr DEVNAME RECCOUNT",
         "Tape Command\n"
         "Backward space records.\n"
		 "Physical or Logical tape device name.\n"
		 "Record count.\n")
{
    execute_command ("tape-ui", 3, "bsr", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_fsf,
         tape_fsf_cmd,
         "tape fsf DEVNAME RECCOUNT",
         "Tape Command\n"
         "Forward space records filemarks.\n"
		 "Physical or Logical tape device name.\n"
		 "Filemark count.\n")
{
    execute_command ("tape-ui", 3, "fsf", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_bsf,
         tape_bsf_cmd,
         "tape bsf DEVNAME RECCOUNT",
         "Tape Command\n"
         "Backward space filemarks.\n"
		 "Physical or Logical tape device name.\n"
		 "Filemark count.\n")
{
    execute_command ("tape-ui", 3, "bsf", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_eod,
         tape_eod_cmd,
         "tape eod DEVNAME",
         "Tape Command\n"
         "Forward space to end-of-data.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "eod", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_rewind,
         tape_rewind_cmd,
         "tape rewind DEVNAME",
         "Tape Command\n"
         "Rewind to beginning-of-tape.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "rewind", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_wfm,
         tape_wfm_cmd,
         "tape wfm DEVNAME FMCOUNT",
         "Tape Command\n"
         "Write filemarks.\n"
		 "Physical or Logical tape device name.\n"
		 "Filemark count.\n")
{
    execute_command ("tape-ui", 3, "wfm", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_load,
         tape_load_cmd,
         "tape load DEVNAME",
         "Tape Command\n"
         "Load tape cartridge.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "load", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_unload,
         tape_unload_cmd,
         "tape unload DEVNAME",
         "Tape Command\n"
         "Unload tape cartridge.\n"
		 "Physical or Logical tape device name.\n")
{
    execute_command ("tape-ui", 2, "load", argv[0]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_read,
         tape_read_cmd,
         "tape read DEVNAME DATALEN",
         "Tape Command\n"
         "Read a tape block.\n"
		 "Physical or Logical tape device name.\n"
		 "Expected data length.\n")
{
    execute_command ("tape-ui", 3, "read", argv[0], argv[1]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_write,
         tape_write_cmd,
         "tape write DEVNAME DATALEN PATTERN",
         "Tape Command\n"
         "write a tape block.\n"
		 "Physical or Logical tape device name.\n"
		 "Block data length.\n"
		 "One-byte pattern to write.\n")
{
    execute_command ("tape-ui", 4, "write", argv[0], argv[1], argv[3]);
    return CMD_SUCCESS;
} 

DEFUN   (tape_debug,
         tape_debug_cmd,
         "tape debug LEVEL",
         "Tape Command\n"
         "Set/Reset tape driver diagnostic level (0=silent, - 10=highest verbosity).\n"
		 "Debug level.\n")
{
    execute_command ("tape-ui", 3, "dbg_level", argv[0]);
    return CMD_SUCCESS;
} 

void
install_tape (priv_t myPriv, enum node_type node)
{
	switch (myPriv)
    {
        case diagPriv:
			install_element (node, &tape_sniffer_cmd);	
			install_element (node, &tape_read_cmd);
			install_element (node, &tape_write_cmd);
			install_element (node, &tape_debug_cmd);
        case fullPriv:
			install_element (node, &tape_devlist_cmd);	
			install_element (node, &tape_mc_devlist_cmd);	
			install_element (node, &tape_open_cmd);	
			install_element (node, &tape_close_cmd);
			install_element (node, &tape_devstate_cmd);
			install_element (node, &tape_fsr_cmd);
			install_element (node, &tape_bsr_cmd);
			install_element (node, &tape_fsf_cmd);
			install_element (node, &tape_bsf_cmd);
			install_element (node, &tape_eod_cmd);
			install_element (node, &tape_rewind_cmd);
			install_element (node, &tape_wfm_cmd);
			install_element (node, &tape_load_cmd);
			install_element (node, &tape_unload_cmd);
        case changePriv:
        case addPriv:
        case viewPriv:
    }
    return;
}
