/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: config_rules.c
 *
 * RCS:  $Id: config_rules.c,v 1.3 2002/10/22 01:30:44 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains the rules concerning configuration within a cluster
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 27 Sep 2002
 *
 *-----------------------------------------------------------------
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "nfx-defs.h"
#include "exports/opt-api.h"
#include "exports/ncm-api.h"
#include "exports/anpssc-api.h"
#include "exports/evm-api.h"
#include "cfgrules-api.h"

#define FIRST_LIST_SIZE 4096	// Actually half first buffer size

int
ok_to_online (evm_volId_t lv, char *on_node)
{
	char **admin_state;
	char *key;
	char *asg_node;
	ncmRqNodeList_t rq_node_list;
	ncmNodeList_t *conn_node_list;
	int list_size;
	struct timeval waitThis = {2, 0};
	int rc;
	int i;
	ncmMsg_t *viaNcm;
	evm_MsgVolGetAttrib_t *rqVolAttr;
	long long rspBuf [(sizeof(ncmMsg_t) + sizeof(evm_MsgVolCreate_t)) /
	                  sizeof(long long)];
	ncmMsg_t *ncmRsp = (ncmMsg_t *)rspBuf;
	evm_MsgVolCreate_t *volAttr = (evm_MsgVolCreate_t *)ncmRsp->appMsg;
/*
 * Check to see if that LV is assigned to the node
 */
	asprintf (&key, "cluster.lv.%016llx", lv.whole_thing);
	admin_state = get_cfg_val (key);
	free (key);
	if (admin_state) {	// Assigned somewhere
		asg_node = strdup(strchr (admin_state[1], ':') + 1);
		rc = strcasecmp (on_node, asg_node);	// Assigned to the named node?
		free_cfg_val (admin_state);
		if (!rc) {
			return 1;	// Yup, assigned to on_node
		}
	}
	else {
		return 1;	// Not assigned so OK to online anywhere
	}
/*
 * Let's see if its assigned node is connected into the cluster
 */
	list_size = FIRST_LIST_SIZE;
	rq_node_list.msgType = NCM_RQ_NODE_LIST;
	do {
		list_size *= 2;
		conn_node_list = malloc (list_size);
		rc = sendAgileMsg (-1, &rq_node_list, sizeof (rq_node_list),
		                   "ncm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						   &waitThis,
						   conn_node_list, list_size);
	} while (rc == list_size);

	if ((rc <= 0) || (conn_node_list->msgType != NCM_NODE_LIST)) {
		free (conn_node_list);
#if defined (DEBUG)
		fprintf (stderr, "Cannot get node list from ncm\n");
#endif
		return 0;
	}

	rc = conn_node_list->numConnNodes;
	for (i = 0; i < rc; i++) {
		if (!strcasecmp (conn_node_list->nodeNames[i], asg_node)) {
			break;
		}
	}
	free (conn_node_list);

	if (i == rc) {	// Assigned node not connected to cluster. Probably dead
		return 1;
	}
/*
 * Check to see if the LV is online on its assigned node. Ask EVM over there.
 */
	i = sizeof(ncmMsg_t) + sizeof (evm_MsgVolGetAttrib_t);
	viaNcm = malloc (i);
	viaNcm->msgType = NCM_SEND;
	strcpy (viaNcm->appName, "evm");
	strcpy (viaNcm->nodeName, asg_node);
	free (asg_node);
	viaNcm->rqResp = 1;

	rqVolAttr = (evm_MsgVolGetAttrib_t *)viaNcm->appMsg;
	rqVolAttr->hdr.msgType = EVM_VOL_GET_ATTRIB;
	rqVolAttr->id = lv;

	waitThis.tv_sec *= 2;	// Twice as patient when going off-node
	rc = sendAgileMsg (-1, viaNcm, i, 
	                   "ncm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
	                   &waitThis,
	                   rspBuf, sizeof(rspBuf));
	free (viaNcm);

	if ((rc <= 0) || 
	    (ncmRsp->msgType != NCM_RESP) ||
	    (volAttr->hdr.msgType != EVM_VOL_CONFIG)) {
#if defined (DEBUG)
		fprintf (stderr, "Unable to retrieve LV config\n");
#endif
		return 0;
	}
/*
 * If it's not online on its assigned node, its OK to be online elsewhere
 */
	return !(volAttr->config.onLine);
}
