/* vi:ts=4
 */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <neteee/eee.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>

#include "exports/anpssc-api.h"
#include "exports/eee-error.h"		// Error code definitions
#include "exports/rcon-api.h"

extern int
main(int argc, char *argv[])
{
	struct sockaddr_eee us, them;
	socklen_t them_len = sizeof (them);
	int sock;
	int rc;
	char buf[8192];
	rconCntlMsg_t *ctl = (rconCntlMsg_t *)buf;
	rconDataMsg_t *dat = (rconDataMsg_t *)buf;
	rconStatMsg_t stat;
	fd_set selset;

	if (argc != 3) {
		fprintf (stderr, "Run it as: %s slot cpu\n", argv[0]);
		exit (1);
	}
	
	memset (&us, 0, sizeof(us));
	us.seee_family = AF_EEE;
	us.seee_bplane = EEE_CONTROLBPLANE;
	us.seee_app = eee_getApplicationId("rcon");
	if (us.seee_app == EEE_ERR_INVALID_APP_NAME) {
		fprintf (stderr, "Unknown application id rcon\n");
		exit (1);
	}
	us.seee_slot = strtol (argv[1], NULL, 10);
	us.seee_cpu = strtol (argv[2], NULL, 10);

	if ((sock = socket(PF_EEE, SOCK_DGRAM, 0)) >= 0) {
		if (bind(sock, (struct sockaddr *)&us, sizeof(us)) < 0) {
			perror ("Error binding socket");
			return 1;
		}
	}
	else {
		perror ("Error creating socket");
		return 1;
	}

	fprintf (stderr, "OK to run rcon %d %d\n", us.seee_cpu, us.seee_slot);

	rc = recvfrom (sock, buf, sizeof(buf), 0, (struct sockaddr *)&them, &them_len);
	if (rc <= 0) {
		perror ("recvfrom error");
		exit (1);
	}

	if ((ctl->hdr.msgType != RCON_CNTL) || (ctl->action != INIT_CONNECTION)) {
		fprintf (stderr, "First message not control (init) message\n");
		exit (1);
	}

	memset (&stat, 0, sizeof(stat));
	stat.hdr.msgType = RCON_STAT;
	stat.hdr.clientPrivate = ctl->hdr.clientPrivate;
	stat.stat = NFX_OK;

	rc = sendto (sock, &stat, sizeof(stat), 0, (struct sockaddr *)&them, 
	             them_len);
	if (rc <= 0) {
		perror ("sendto error");
		exit (1);
	}

	memset (buf, 0, sizeof(buf));
	dat->hdr.msgType = RCON_DATA;

	while (1) {
		FD_ZERO (&selset);
		FD_SET (sock, &selset);
		FD_SET (0, &selset);

		rc = select (sock + 1, &selset, NULL, NULL, NULL);

		if (FD_ISSET (0, &selset)) {
			fgets (dat->buf, 8000, stdin);
			if (feof(stdin)) {
				rconCntlMsg_t bye;
				bye.hdr.msgType = RCON_CNTL;
				bye.action = DESTROY_CONNECTION;
				sendto (sock, &bye, sizeof(bye), 0, 
				        (struct sockaddr *)&them, them_len);
				fprintf (stderr, "Sent destroy message\n");
				break;
			}
			dat->len = strlen (dat->buf) + 1;
			sendto (sock, dat, sizeof (rconDataMsg_t) + strlen (dat->buf), 0,
			        (struct sockaddr *)&them, them_len);
		}

		if (FD_ISSET (sock, &selset)) {
			recvfrom (sock, buf, sizeof(buf), 0, 
			         (struct sockaddr *)&them, &them_len);
			if (dat->hdr.msgType == RCON_CNTL) {
				fprintf (stderr, "Got control message. Ciao\n");
				break;
			}
			fputs (dat->buf, stdout);
		}
	}

	return 0;
}
