/*
 * Name:  fs-shell.c
 *
 * RCS:  $Id: fs-shell.c,v 1.254 2002/12/10 18:42:46 bob Exp $
 *
 * Description:  Command line interface functions for sm-fs
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */
 
#include "fs-includes.h"
#include <ctype.h>

#if defined(SSC)
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <exports/anpssc-api.h>
#include <time.h>
#include <limits.h>
#include <exports/ncm-api.h>
#include <exports/lnm-api.h>
#include <exports/fs-opt-api.h>
#endif

#include <exports/elog-api.h>
#include <exports/scsi-api.h>

/* Declare globals used by eee_rtc calls.  The actual values will be returned
 * as part of the responses.
 */
uint64 count_per_usec = 1;

#define BYTES_IN_GB     (1024*1024*1024)
#define BYTES_IN_MB     (1024*1024)

#define FS_SSC_ARG_MAX   30
#define USAGE_STR_SIZE   1024
#define MAX_APP_ID       64

#if !defined(FS_STANDALONE)

#define NCM_REQUEST(_type_) \
            ((_type_) >= NCM_MSG_BASE)

#define FIELD_OFFSET( _Type_, _Field_) \
                            (&(((_Type_*)(0))->_Field_))

#define CONTAINING_RECORD( _Addr_, _Type_, _Field_)                         \
                                                    ((_Type_ *)     (       \
                                                    (char*)   (_Addr_) -    \
                                                    (char *)(&((_Type_*)0)->_Field_)))
                                                    
#define FS_SHELL_MEM_ALLOC(_p_, _type_, _size_)  {                              \
        (_p_) = (_type_*) malloc((_size_))                                     ;\
        FS_ASSERT((_p_) != NULL)                                               ;\
        memset ((_p_), 0, (_size_))                                            ;\
        }

#define FS_SHELL_MEM_FREE(_p_)    free((_p_))

#define FS_SHELL_RESOURCE_ERROR      (NFX_ERR + 1)
#define FS_SHELL_SEND_ERROR          (NFX_ERR + 2)
#define FS_SHELL_SEND_TIMEOUT        (NFX_ERR + 3)

#define FS_SET_SEND_ERROR(err) \
        (((err) <= 0) ? (((err) < 0) ? FS_SHELL_SEND_ERROR : FS_SHELL_SEND_TIMEOUT) : \
                                                                                NFX_OK)

struct timeval  def_clus_timeout = {300,0};   /* 5 minutes max for command to execute over 
                                                         cluster */



// TODO - this is duplicated in fs-shell.c - FIX THIS!
//  Previous msec values:
//  { 1,5,10,15,20,25,30,40,50,100,500,1000,5000,10000};
//  Now in usecs:
static int32 fs_nfsStatsUsecs[FS_NFS_TIME_MAX] =
    { 10,50,100,200,500,1000,1500,2000,3000,5000,10000,20000,50000,100000};
#endif
#if defined(FS_STANDALONE)
extern uint64 fs_volexer_stats_usec[FS_TIME_STATS_BUCKETS_MAX];
#else
uint64 fs_volexer_stats_usec[FS_TIME_STATS_BUCKETS_MAX] = { 10000, 50000, 100000, 200000, 300000, 400000, 500000, 1000000 };
#endif

typedef struct 
{
    char *name;
    uint32 flag;
} tab_t;

static int parse(char *argv[], char *input);

static boolean fs_shell_get_volid (char *id_str, evm_volId_t *volId, char *nodeName);
static boolean fs_shell_is_vol_online(evm_volId_t  volId, char *nodeName);
static int shell_parse_parameter(tab_t *tab, char *name, char *list, int *flag);
fs_MsgReqHdr * fs_shell_parsecmd(char *input, fs_MsgReqHdr * (**resp)(int stat, fs_MsgRespHdr *));
static fs_MsgReqHdr * fscmd_parse_opts(int argc, char **argv);

static fs_MsgReqHdr *cmd_mkfs(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_viewfs(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_mount(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_romount(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_unmount(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_emapdebug(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_eek(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_zap(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_growfs(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_sync(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_show(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_flush(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_help(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_exit(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_mkfset(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_rmfset(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_quotaon(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_quotaoff(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_get_fset_usage(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_set_def_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_set_ug_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_set_fset_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_get_fset_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_get_ug_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_report_ug_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_report_fset_quota(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_list_fsets(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_elog(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_freeze(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_thaw(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_revert(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_show(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_sched(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_admin(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_state(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_pin(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snap_rename(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_localmirror(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_cachestats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_memstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_reqstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_logstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_logdata(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_logtest(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_crash(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_shell(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_setup_dump(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_start_dump(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_set_dump_path(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_setup_restore(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_start_restore(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_add_restore_path(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_snapdiag(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_nfsstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_yieldstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_errstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_lockstats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_mcl(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_fssfLookup(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_fssfOpen(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_fssfClose(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_fssfGetattr(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_abort(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_volexer(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_allocStats(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_incoreTune(int argc, char *argv[]);
static fs_MsgReqHdr *cmd_dnlcstats(int argc, char *argv[]);

static fs_MsgReqHdr *def_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *viewfs_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *mkfset_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *rmfset_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *quotaon_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *quotaoff_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *get_fset_usage_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *report_fset_usage_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *set_def_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *set_ug_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *set_fset_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *get_ug_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *get_ug_usage_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *get_fset_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *report_ug_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *report_ug_usage_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *report_fset_quota_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *list_fsets_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *cachestats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *memstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *reqstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *logstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *context_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *logdata_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *snapdiag_stat_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *snap_show_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *eek_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *nfsstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *yieldstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *errstats_response(int status, fs_MsgRespHdr *hdr);
static fs_MsgReqHdr *lockstats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *mcl_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *fssfLookup_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *fssfOpen_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *fssfClose_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *fssfGetattr_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *volexer_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *allocStats_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *incoreTune_response(int status, fs_MsgRespHdr *);
static fs_MsgReqHdr *dnlcstats_response(int status, fs_MsgRespHdr *);

/*
 * Temporary Function for Disabled function
 */

static fs_MsgReqHdr *
cmd_unsupp(int argc, char *argv[])
{
        printf("Feature not supported in this release\n");
        return NULL;
}

#if defined(SSC) || defined(FS_STANDALONE)
// global ptr to cmd response handler
fs_MsgReqHdr *    (*cmdRespHandler)(int status, fs_MsgRespHdr *);
#endif

struct {
    char *name;
    fs_MsgReqHdr *(*cmd)(int argc, char *argv[]);
    fs_MsgReqHdr *(*resp)(int status, fs_MsgRespHdr *hdr);
    int min_args, max_args;
    char *usage;
} cmds[] = {
    {"mkfs",    cmd_mkfs,	def_response,	1, 3, "mkfs <volname> [<numBlks>] [ext-log]"},
    {"viewfs",    cmd_viewfs,	viewfs_response,	0, 1, "viewfs [-v]"},
    {"mount",   cmd_mount,	def_response,	1, 4, "mount <volname> [DoNotEek] [DoNotReplay] [log-device=NVRAM|NULL|<volname>]"},
    {"mount_ro",   cmd_romount,	def_response,	1, 1, "mount <volname>"},
    {"umount",  cmd_unmount,	def_response,	1, 1, "umount <volname>"},
    {"unmount", cmd_unmount,	def_response,	1, 1, "unmount <volname>"},
    {"emapdebug", cmd_emapdebug,	def_response,	1, 1, "emapdebug <volname>"},
    {"eek",	cmd_eek,	eek_response,	1, 6, "eek <volname> [-y | -n]"},
    {"zap",	cmd_zap,	def_response,	2, 4, "zap <volname> <action> <inum> <data>"},
    {"growfs",	cmd_growfs,	def_response,	2, 2, "growfs <volname> <numBlks>"},
    {"sync",    cmd_sync, 	def_response,	1, 1, "sync <volname>"},
    {"show",    cmd_show,	def_response,	2, 2, "show locked <volname>"},
    {"flush",   cmd_flush,	def_response,	1, 1, "flush <volname>"},
    {"help",    cmd_help,	def_response,	0, 0, "help"},
#if !defined(SSC)
    {"shell",   cmd_shell,	def_response,	0, 99, "shell"},
#endif
    {"?",       cmd_help,	def_response,	0, 0, "?"},
    {"exit",    cmd_exit,	def_response,	0, 0, "exit"},
    {"quit",    cmd_exit,	def_response,	0, 0, "quit"},
    {"elog",    cmd_elog,	def_response,	0, 10, "elog <cmd> <parameters>"},
    {"freeze",  cmd_freeze, def_response,   1, 1, "freeze <volname>" },
    {"thaw",    cmd_thaw,   def_response,   1, 1, "thaw <volnaem>" },
	{"mkfset",	cmd_mkfset,	mkfset_response,	2, 2, "mkfset <volname> <path>"},
	{"rmfset",	cmd_rmfset,	rmfset_response,	2, 2, "rmfset <volname> <path>"},
	{"quotaon",	cmd_quotaon,	quotaon_response,	2, 2,
												"quotaon <volname> <path>"},
	{"quotaoff",	cmd_quotaoff,	quotaoff_response,	2, 2,
												"quotaoff <volname> <path>"},
    {"snap",    cmd_snap,   def_response,   3, 3,
     "snap [create | remove] <volname> name" }, 

    {"snap_state",    cmd_snap_state,   def_response,   2, 2,
     "snap_state <volname> [enable | disable]" }, 

    {"snap_revert",    cmd_snap_revert,   def_response,   1, 3,
     "snap_revert <volname> [-n name | -i snapnum]" }, 

    {"snap_show",    cmd_snap_show,  snap_show_response, 1, 2,
     "snap_show [sched | usage] <volname>"},

    {"snap_sched",    cmd_snap_sched,  def_response, 3, 9,
     "<volname> [-w max-weekly] [-d max-daily] [-h max-hourly] [-l 1,2,...24]"},

    {"snap_admin",    cmd_snap_admin,  def_response, 0, 3,
     "snap_admin [<volname> <internalSnaps> <internalInterval>]"},

    {"snap_pin",    cmd_snap_pin,  def_response, 3, 3,
     "snap_pin <volname> <name> [pin | unpin]"},

    {"snap_rename", cmd_snap_rename,  def_response, 3, 3,
     "snap_rename <volname> <oldname> <newname>"},

    {"localmirror", cmd_localmirror, def_response, 3, 4, 
     "localmirror <from-volId> <to-volId> <new-name> [<old-name>]" },    

	{"set_def_quota",	cmd_set_def_quota,	set_def_quota_response,	9, 9,
	 "set_def_quota <volname> <path> aggr <file soft>\n"
	 "  <file hard> <file time> <size soft> <size hard> <size time>"},

	{"set_ug_quota",	cmd_unsupp,	set_ug_quota_response,	5, 10,
	 "set_ug_quota <volname> <path> <user|group> <uid|gid> -d | <file soft> \n"
	 "  <file hard> <file time> <size soft> <size hard> <size time>"},

	{"set_fset_quota",	cmd_set_fset_quota,	set_fset_quota_response,	3, 8,
	 "set_fset_quota <volname> <path> -d | <file soft> <file hard> <file time>"
	 "\n  <size soft> <size hard> <size time>"},

	{"get_fset_quota",	cmd_get_fset_quota,	get_fset_quota_response,	2, 3,
	 "get_fset_quota <volname> <path>"},

	{"get_ug_quota", cmd_unsupp,	get_ug_quota_response,	4, 4,
	 "get_ug_quota <volname> <path> <user|group> <uid | gid>"},

	{"get_ug_usage", cmd_unsupp,	get_ug_usage_response,	4, 4,
	 "get_ug_usage <volname> <path> <user|group> <uid | gid>"},

	{"report_fset_quota",	cmd_report_fset_quota,	report_fset_quota_response,
																1, 3,
	 "report_fset_quota <volname> [<index>] [<number return>]"},
	{"report_ug_quota",	cmd_unsupp,	report_ug_quota_response,
																3, 5,
	 "report_ug_quota <volname> <path> <user|group> <index> <number return>"},
	{"report_ug_usage",	cmd_unsupp,	report_ug_usage_response,
																3, 5,
	 "report_ug_usage <volname> <path> <user|group> <index> <number return>"},
	{"list_fsets",	cmd_list_fsets,	list_fsets_response, 1, 3,
	 "list_fsets <volname> [<index>] [<number return>]"},
	{"get_fset_usage",	cmd_get_fset_usage,	get_fset_usage_response, 2, 2,
	 "get_fset_usage <volname> <path>"},
	{"report_fset_usage", cmd_report_fset_quota, report_fset_usage_response, 
                            1, 3,
      "report_fset_usage <volname> [<index>] [<number return>]"},

    {"cachestats", cmd_cachestats,	cachestats_response,	0, 3, "cachestats [-z] [-v] <volname>"},
    {"memstats", cmd_memstats,  memstats_response,	0, 0, "memstats"},
    {"reqstats", cmd_reqstats,  reqstats_response,	0, 2, "reqstats [-z] [-v] <volname>"},
    {"logstats", cmd_logstats,  logstats_response,      1, 4, "logstats [-z] [-v] <volname>"},
    {"logdata",  cmd_logdata,   logdata_response,	1, 5, "logdata <volname> [<nbrEntries>] [<offset>] [<criteria> <value>]"},
    {"logtest",  cmd_logtest,   def_response,	    1, 4, "logtest <volname> <crash | replay | crash-now> <n> <crashPhase>"},
    {"crash",    cmd_crash,     def_response,	    1, 3, "crash <volname> [<type> [<count>]]"},

    {"setup_dump",    cmd_setup_dump,     def_response,	    1, 7, "setup_dump <volname> level recursive filehistory prev snap time label prevsnap name"},
    {"start_dump",    cmd_start_dump,     def_response,	    1, 1, "start_dump <volname>"},
    {"set_dump_path", cmd_set_dump_path, def_response, 1, 2, "set_dump_path <volname> <pathname>"},
    {"setup_restore", cmd_setup_restore, def_response, 1, 4, "setup_restore <volname> iosize recursive "},
    {"start_restore", cmd_start_restore, def_response,1, 1, "restore_dump <volname>"},
    {"add_restore_path", cmd_add_restore_path, def_response, 3, 4, "add_restore_path <volname> <pathname> [newpath]"},
    {"snapdiag", cmd_snapdiag, snapdiag_stat_response, 1, 4, "snapdiag (refbytes|snapbits|stat|crash) <volId | crashPhase> [snapname]"},
    {"nfsstats", cmd_nfsstats,  nfsstats_response,	0, 3, "nfsstats [-v|-z] [<volname>]"},
    {"yieldstats", cmd_yieldstats,  yieldstats_response,	0, 6, "yieldstats [-a|-z|-i|-r|-d] <volname>"},
    {"errstats", cmd_errstats,  errstats_response,	0, 2, "errstats [-z] [<volname>]"},
    {"lockstats", cmd_lockstats,  lockstats_response,	1, 8, "lockstats [-a|-z] [-d|-r|-b] <volname>|<reqType>|<daemon>"},
    {"mcl", cmd_mcl, mcl_response, 2, 2, "mcl <volume> <objname>"},
    
    // FSSF commands
    {"fssf_lookup", cmd_fssfLookup, fssfLookup_response, 2, 2, "fssf_lookup <volume> <filename>"},
    {"fssf_open", cmd_fssfOpen, fssfOpen_response, 2, 2, "fssf_open <volume> <filename>"},
    {"fssf_close", cmd_fssfClose, fssfClose_response,  1, 1, "fssf_close <stateid>"},
    {"fssf_getattr", cmd_fssfGetattr, fssfGetattr_response,  1, 1, "fssf_getattr <stateid>"},

    {"abort", cmd_abort,  def_response,	1, 1, "abort <volname>"},
    {"volexer", cmd_volexer,  volexer_response, 1, 12, "volexer <start|stop|stat> <volname> [user|meta rand|seq|both rw|ro|wo lp|nl nr|re <load> <seed> <numblks>] [<numBlksSequential> <sequentialIoSize>]}"},
    {"allocstats", cmd_allocStats,  allocStats_response,  1, 5, "allocstats <volname> {all|<inum>|</path+filename>} [<snapnum>] [-v]"},
    {"incoretune", cmd_incoreTune,  incoreTune_response,  0, 4, "incoretune [set name value]"},
    {"dnlcstats", cmd_dnlcstats,	dnlcstats_response,	0, 2, "dnlcstats [-z|-f] <volname>"},
    {0, 0, 0, 0, 0}
};


tab_t elogtab[] = {
//  {"enable",           ELOG_CONFIGURE_ENABLE},
    {"severity",         ELOG_CONFIGURE_SEVERITY},
    {"test",             ELOG_CONFIGURE_TEST},
    {"trace",            ELOG_CONFIGURE_APP_SET_BITMAP},
    {"untrace",          ELOG_CONFIGURE_APP_CLEAR_BITMAP},

    {"show-date",        ELOG_CONFIGURE_SHOW_DATE},
    {"format",           ELOG_CONFIGURE_FORMAT},
    {"show-line-file",   ELOG_CONFIGURE_SHOW_LINE},
    {"show-seq",         ELOG_CONFIGURE_SHOW_SEQ},
    {"show-name",        ELOG_CONFIGURE_SHOW_APP_NAME},
    {"show-severity",    ELOG_CONFIGURE_SHOW_SEVERITY},
    {"show-local",       ELOG_CONFIGURE_SHOW_LOCAL},

    {0, 0}
};

tab_t severitytab[] = {
    {"emerg",     LOG_EMERG},
    {"alert",     LOG_ALERT},
    {"crit",      LOG_CRIT},
    {"err",       LOG_ERR},
    {"warning",   LOG_WARNING},
    {"notice",    LOG_NOTICE},
    {"info",      LOG_INFO},
    {"debug",     LOG_DEBUG},
    {0, 0}
};

tab_t onofftab[] = {
    {"on",        1},
    {"off",       0},
    {0, 0}
};

tab_t showlocaltab[] = {
    {"off",          ELOG_DISABLED},
    {"stdout",       ELOG_LOCAL_STDOUT},
    {"stderr",       ELOG_LOCAL_STDERR},
    {0, 0}
};

tab_t datetab[] = {
    {"on",        ELOG_SHOW_DATE},
    {"off",       ELOG_DISABLED},
    {"msec",      ELOG_SHOW_DATE_MSEC},
    {"usec",      ELOG_SHOW_DATE_USEC},
    {0, 0}
};

tab_t formattab[] = {
    {"one",       ELOG_ONE_LINE},
    {"multi",     ELOG_MULTIPLE_LINE},
    {"min",       ELOG_FORMAT_MIN_INFO},
    {"max",       ELOG_FORMAT_MAX_INFO},
    {0, 0}
};

tab_t wbwakeuptab[] = {
    {"invalid",		FS_WB_WAKEUP_INVALID},
    {"pageThresh",	FS_WB_WAKEUP_PAGE_THRESH},
    {"all",		FS_WB_WAKEUP_ALL},
    {"pageSync",	FS_WB_WAKEUP_PAGE_SYNC},
    {"sync",		FS_WB_WAKEUP_SYNC},
    {"inoSync",		FS_WB_WAKEUP_INO_SYNC},
    {"replay",		FS_WB_WAKEUP_REPLAY},
    {"log",		FS_WB_WAKEUP_LOG},
    {"waiter",		FS_WB_WAKEUP_WAITER},
    {"periodic",	FS_WB_WAKEUP_PERIODIC},
    {"resched",	        FS_WB_WAKEUP_RESCHED},
    {"pad",	        FS_WB_WAKEUP_PAD},
    {0, 0}
};

tab_t inocleanertab[] = {
    {"invalid",		FS_INODE_CLEANER_INVALID},
    {"log",		FS_INODE_CLEANER_LOG},
    {"all",		FS_INODE_CLEANER_ALL},
    {"start",		FS_INODE_CLEANER_START},
    {"refdecr",		FS_INODE_CLEANER_REFDECR},
    {"pad",		FS_INODE_CLEANER_PAD},
    {0, 0}
};


tab_t incoreTunetab[] = {
    {"inodePctAvailLoWtrMk",          FS_TUNE_inodePctAvailLoWtrMk},
    {"inodePctAvailHiWtrMk",          FS_TUNE_inodePctAvailHiWtrMk},
    {"logPctUseLoWtrMk",              FS_TUNE_logPctUseLoWtrMk},
    {"logPctUseHiWtrMk",              FS_TUNE_logPctUseHiWtrMk},
    {"bufPctAvailLoWtrMk",            FS_TUNE_bufPctAvailLoWtrMk},
    {"bufPctAvailHiWtrMk",            FS_TUNE_bufPctAvailHiWtrMk},
    {"memPctAvailMin",                FS_TUNE_memPctAvailMin},
    {"bufWritesPerEvent",             FS_TUNE_bufWritesPerEvent},
    {"writesPendingMax",              FS_TUNE_writesPendingMax},
    {"perFsIoPendingMax",             FS_TUNE_perFsIoPendingMax},
    {"inodeCleanPerPass",             FS_TUNE_inodeCleanPerPass},
    {"activeTransToWait",             FS_TUNE_activeTransToWait},
    {"maxCpuUsageUsec",               FS_TUNE_maxCpuUsageUsec},
    {"dummy",                         -1},
    {"inodePctMem",                   FS_TUNE_inodePctMem},
    {"wbSleepMsecs",                  FS_TUNE_wbSleepMsecs},
    {"readdirMaxSpace",               FS_TUNE_readdirMaxSpace},
    {"userDirtyThreshold",            FS_TUNE_userDirtyThreshold},
    {"metaDirtyThreshold",            FS_TUNE_metaDirtyThreshold},
    {"enableReadAhead",               FS_TUNE_enableReadAhead},
    {"readAheadMaxSize",              FS_TUNE_readAheadMaxSize},
    {"readAheadDefaultSize",          FS_TUNE_readAheadDefaultSize},
    {"enableAtimeUpdate",             FS_TUNE_enableAtimeUpdate},
    {"vollogIoMax",                   FS_TUNE_vollogIoMax},
    {"vollogLoopBack",                FS_TUNE_vollogLoopBack},
    {"sacMinAvailable",               FS_TUNE_sacMinAvailable},
    {"sacFreeCount",                  FS_TUNE_sacFreeCount},
    {"ultraMinAvailable",             FS_TUNE_ultraMinAvailable},
    {"memMinAvailable",               FS_TUNE_memMinAvailable},
    {"bufFreeCount",                  FS_TUNE_bufFreeCount},
    {"inodeFreeCount",                FS_TUNE_inodeFreeCount},
    {"userBufMinInUse",               FS_TUNE_userBufMinInUse},
    {"logCommitHiWtrMk",              FS_TUNE_logCommitHiWtrMk},
    {"bufUserPctMem",                 FS_TUNE_bufUserPctMem},
    {"bufMetaNonDirPctMem",           FS_TUNE_bufMetaNonDirPctMem},
    {"bufMetaDirPctMem",              FS_TUNE_bufMetaDirPctMem},
    {"bufMetaCompNonDirPctMem",       FS_TUNE_bufMetaCompNonDirPctMem},
    {"bufMetaCompDirPctMem",          FS_TUNE_bufMetaCompDirPctMem},
    {"metaFreeCount",                 FS_TUNE_metaFreeCount},
    {"metaDirMinInUse",               FS_TUNE_metaDirMinInUse},
    {"metaNonDirMinInUse",            FS_TUNE_metaNonDirMinInUse},
    {"metaCompDirMinInUse",           FS_TUNE_metaCompDirMinInUse},
    {"metaCompNonDirMinInUse",        FS_TUNE_metaCompNonDirMinInUse},

    {"resourceAbsLOG",                FS_TUNE_ABS_LOG},
    {"resourceRelLOG",                FS_TUNE_REL_LOG},
    {"resourceLoadLOG",               FS_TUNE_LOAD_LOG},

    {"resourceAbsUBC",                FS_TUNE_ABS_UBC},
    {"resourceRelUBC",                FS_TUNE_REL_UBC},
    {"resourceLoadUBC",               FS_TUNE_LOAD_UBC},

    {"resourceAbsMBCNondir",          FS_TUNE_ABS_MBC_ND},
    {"resourceRelMBCNondir",          FS_TUNE_REL_MBC_ND},
    {"resourceLoadMBCNondir",         FS_TUNE_LOAD_MBC_ND},

    {"resourceAbsMBCCompNondir",      FS_TUNE_ABS_MBC_CND},
    {"resourceRelMBCCompNondir",      FS_TUNE_REL_MBC_CND},
    {"resourceLoadMBCCompNondir",     FS_TUNE_LOAD_MBC_CND},

    {"resourceAbsMBCDir",             FS_TUNE_ABS_MBC_D},
    {"resourceRelMBCDir",             FS_TUNE_REL_MBC_D},
    {"resourceLoadMBCDir",            FS_TUNE_LOAD_MBC_D},

    {"resourceAbsMBCCompDir",         FS_TUNE_ABS_MBC_CD},
    {"resourceRelMBCCompDir",         FS_TUNE_REL_MBC_CD},
    {"resourceLoadMBCCompDir",        FS_TUNE_LOAD_MBC_CD},

    {"resourceAbsINC",                FS_TUNE_ABS_INC},
    {"resourceRelINC",                FS_TUNE_REL_INC},
    {"resourceLoadINC",               FS_TUNE_LOAD_INC},

    {"resourceAbsMEM",                FS_TUNE_ABS_MEM},
    {"resourceRelMEM",                FS_TUNE_REL_MEM},
    {"resourceLoadMEM",               FS_TUNE_LOAD_MEM},

    {"resourceAbsSAC",                FS_TUNE_ABS_SAC},
    {"resourceRelSAC",                FS_TUNE_REL_SAC},
    {"resourceLoadSAC",               FS_TUNE_LOAD_SAC},

    {"resourceAbsULT",                FS_TUNE_ABS_ULT},
    {"resourceRelULT",                FS_TUNE_REL_ULT},
    {"resourceLoadULT",               FS_TUNE_LOAD_ULT},

    {"resourceAbsVOL",                FS_TUNE_ABS_VOL},
    {"resourceRelVOL",                FS_TUNE_REL_VOL},
    {"resourceLoadVOL",               FS_TUNE_LOAD_VOL},

    {0, 0}
};

tab_t fs_requestsTab[] = {
    {"null",        FS_REQ_NULL},
    {"getattr",     FS_REQ_GET_ATTR},
    {"setattr",     FS_REQ_SET_ATTR},
    {"lookup",      FS_REQ_LOOKUP},
    {"access",      FS_REQ_ACCESS},
    {"readlink",    FS_REQ_READLINK},
    {"read",        FS_REQ_READ},
    {"write",       FS_REQ_WRITE},
    {"create",      FS_REQ_CREATE},
    {"mkdir",       FS_REQ_MKDIR},
    {"symlink",     FS_REQ_SYMLINK},
    {"mknod",       FS_REQ_MKNOD},
    {"remove",      FS_REQ_REMOVE},
    {"rmdir",       FS_REQ_RMDIR},
    {"rename",      FS_REQ_RENAME},
    {"hardlink",    FS_REQ_HARDLINK},
    {"readdir",     FS_REQ_READDIR},
    {"readdirplus", FS_REQ_READDIRPLUS},
    {"stat",        FS_REQ_STAT},
    {"fsinfo",      FS_REQ_FSINFO},
    {"pathconf",    FS_REQ_PATHCONF},
    {"commit",      FS_REQ_COMMIT},
    {0, 0}
};

tab_t fs_daemonTypeTab[] = {
    {"invalid",     FS_DAEMON_TYPE_INVALID},
    {"wblog",       FS_DAEMON_TYPE_VOLLOG},
    {"wbmeta",      FS_DAEMON_TYPE_WB_META},
    {"wbuser",      FS_DAEMON_TYPE_WB_USER},
    {"inoclnr",     FS_DAEMON_TYPE_INO_CLNR},
    {"inogrow",     FS_DAEMON_TYPE_INO_GROW},
    {"inocnt",      FS_DAEMON_TYPE_INO_CNT},
    {"refcnt",      FS_DAEMON_TYPE_REF_CNT},
    {"snap",        FS_DAEMON_TYPE_SNAP},
    {"freezer",     FS_DAEMON_TYPE_FREEZER},
    {"alloc",       FS_DAEMON_TYPE_ALLOC},
    {"buddy",       FS_DAEMON_TYPE_BUDDY},
    {"trash",       FS_DAEMON_TYPE_TRASH},
    {"dump",        FS_DAEMON_TYPE_DUMP},
    {"max daemon",  FS_DAEMON_TYPE_MAX},
    {0, 0}
};

tab_t fs_lockTypeTab[] = {
    {"invalid",     FS_LOCK_TYPE_INVALID},
    {"buf",         FS_LOCK_TYPE_BUF},
    {"inode",       FS_LOCK_TYPE_INODE},
    {"quota",       FS_LOCK_TYPE_QUOTA},
    {"fileset",     FS_LOCK_TYPE_FILESET},
    {"fileset_trav",FS_LOCK_TYPE_FILESET_TRAVERSAL},
    {"boot_alloc",  FS_LOCK_TYPE_BOOT},
    {"access",      FS_LOCK_TYPE_ACCESS},
    {"modify",      FS_LOCK_TYPE_MODIFY},
    {"freezer",     FS_LOCK_TYPE_FREEZE},
    {"btree",       FS_LOCK_TYPE_BTREE},
    {"tape lock",   FS_LOCK_TYPE_TAPE},
    {"eek lock",    FS_LOCK_TYPE_EEK},
    {"max lock",    FS_LOCK_TYPE_MAX},
    {0, 0}
};
/* 
 * Some very application specific stuff
 * Needs cleanup, and some redesign (*.h files)
 */

tab_t apptab[MAX_APP_ID];

// to store known applications
// int fs_applicationId;
#ifdef SSC
int fs_applicationId = 18 ;
int nvram_appl_id = 24 ;
#else
extern int nvram_appl_id;
#endif

tab_t fs_tracetab[] = {
    {"all",     ~0},
    {"req",     1<<FS_TRACE_REQ},
    {"buf",     1<<FS_TRACE_BUF},
    {"ino",     1<<FS_TRACE_INO},
    {"lck",     1<<FS_TRACE_LCK},
    {"log",     1<<FS_TRACE_LOG},
    {"ior",     1<<FS_TRACE_IO},
    {"space",   1<<FS_TRACE_SPACE},
    {"cod",     1<<FS_TRACE_COD},
    {"snap",    1<<FS_TRACE_SNAP},
    {"verbose", 1<<FS_TRACE_VERBOSE},
    {"vollog",  1<<FS_TRACE_VOLLOG},
    {"readdir", 1<<FS_TRACE_READDIR},
    {"dumpres", 1<<FS_TRACE_DUMP_RESTORE},
    {0, 0}
};


typedef enum nv_traceLvlNum
{
    NV_TRACE_MNGT,
    NV_TRACE_WRITE,
    NV_TRACE_READ,
    NV_TRACE_LVL_MAX
} nv_TraceLvlNum;

tab_t nv_tracetab[] = {
    {"all",    ~0},
    {"mngt",   1<<NV_TRACE_MNGT},
    {"write",  1<<NV_TRACE_WRITE},
    {"read",   1<<NV_TRACE_READ},
    {0, 0}
};




#if defined(SSC) || defined(FS_STANDALONE)

/*-----------------------------------------------------------------------------
 * Help for tracking mem allocs/deallocs.
 *	// Do not add to this list without updating
 *	// fs_RamAllocType;
 */
char *fs_ramTypeName[FS_RAM_TYPE_MAX] =
{
    "reqMsg",
    "respMsg",
    "nfsRespArg",
    "codData",
    "context",
    "bufPtr",
    "bufShadowData",
    "bufPageHdr",
    "inode",
    "logSumm",
    "logDep",
    "logSummArr",
    "logBufArr",
    "logInoArr",
    "logRecord",
    "rdwr",
    "lockWait",
    "bufData",
    "readData",
    "writeData",
    "mkfsData",
    "simMsgQentry",
    "simMisc",
    "logReadBuf",
    "logWriteBuf",
    "coreMisc",
    "cod",
    "mcpu",
  	"super",
  	"arena",
  	"quota",
  	"fileset",
  	"fsetContext",
  	"quotaContext",
  	"quotaContextList",
  	"makepath",
  	"external",
	"skipheap",
    "dump_dirents",
    "dump_renamlink",
    "dump_name",
    "dump_nameentry",
    "dump_context",
    "restore_attr",
    "restore_data",
    "restore_name",
    "restore_dirents",
    "restore_context",
    "bufClone",
    "bufCloneData",
    "AgileMirrorSourceSession",
    "AgileMirrorTargetSession",
    "Snapshot name",
    "last snapshot name",
    "ExtentVector",
    "RegionSummary",
    "LogBuffer",
    "GetFileSetPath",
    "LogUndoSumm",
    "LogBufDep",
    "DirLeaf",
    "RestoreIODone",
    "Waiter",
    "PendingReadRequests",
    "benchmark",
    "contextSharedLocks",
    "restore_io_done_q",
    "fork_context",
    "full_fh_context",
    "file_history_queue",
    "readahead",
    "sdesc",
    "iobuffer for dump",
    "compact leaf",
    "readdir entries",
    "readdir cacheent",
    "dnlc entry",
    "user_zero_fill",
    "volume_log_cache",
    "dirhash tp",
    "dirhash table",
#ifdef FS_HIST_WAIT
    "dump session"
#endif
	// Do not add to this list without updating fs_RamAllocType;
};

char *
fs_lockStatsTypeName[FS_LOCKSTATS_MAX] =
{
    "modify",
    "access",
    "freeze",
    "dumps",
    "restores",
    NULL
};

/*-----------------------------------------------------------------------------
 * Help for tracking request counters.
 *
 * NB: these requests have to match the order of requests in requests.tab!!!!
 *     AND fs_ReqType (fs-api.h)
 */

char *fs_reqTypeName[FS_REQ_MAX] =
{
    "null",
    "getAttr",
    "setAttr",
    "lookup",
    "access",
    "readLink",
    "read",
    "write",
    "create",
    "mkdir",
    "symLink",
    "mknod",
    "remove",
    "rmdir",
    "rename",
    "hardLink",
    "readDir",
    "readDirPlus",
    "stat",
    "fsinfo",
    "pathconf",
    "commit",
    "txcompl",
    "sync",
    "mount",
    "unmount",
    "snapCreate",
    "snapRemove",
    "makeFs",
    "checkFs",
    "getRoot",
    "trace",
    "showLocked",
    "emapdebug",
    "flush",
    "createfset",
    "removefset",
    "reportquota",
    "setquota",
    "getquota",
    "quotaon",
    "quotaoff",
    "listfset",
    "freeze",
    "thaw",
    "cachestats",
    "memstats",
    "reqstats",
    "context",
    "logdata",
    "crash",
    "dumpSetup",
    "dumpNames",
    "dumpStart",
    "dumpAbort",
    "dumpIodone",
    "restoreSetup",
    "restoreNames",
    "restoreStart",
    "restoreAbort",
    "restoreIodone",
    "fsstatVol",
    "eek",
    "growfs",
    "snapdiag",
    "amDeltaSetup",
    "amDeltaGetExtent",
    "amDeltaComplete",
    "amDeltaInstallSetup",
    "amDeltaInstallExtent",
    "amDeltaInstallCommit",
    "amLastSnapshot",
    "snapShow",
    "snapSched",
    "snapAdmin",
    "snapRevert",
    "eekCont",
    "nfsstats",
    "yieldstats",
    "lockstats",
	"logtest",
	"zap",
	"getfilesetpath",
    "logstats",
    "viewFs",
    "abort",
    "volExer",
    "allocStats",
    "setMinFree",
    "getMinFree",
    "mcl",
    "errstats",
    "benchMark",
    "incoreTune",
    "dnlcstats",
} ;
/*
 * NB: these requests have to match the order of requests in requests.tab!!!!
 *     AND fs_ReqType (fs-api.h)
 */

#endif // SSC || FS_STANDALONE


#if defined(SSC)

#define fs_ramAlloc(_what_, _size_)  malloc(_size_)
#define fs_ramDealloc(_what_, _ptr_) free(_ptr_)
#define fs_abort()  exit(2)
#define EVM_RESP_MAX 1024
static char evmRespBuf[EVM_RESP_MAX];

static int
send_evm_message(void *msg, int msgLen)
{
    struct timeval      timeout = { 10, 0};
    int                 rc;

    rc = sendAgileMsg(  -1,     // system assigned tmp socket
                        msg,
                        msgLen,
                        "evm",
                        NFX_SSC_UNIX,
                        0,  // TODO -replace with correct cm api call for SSC slot
                        MGMTPLANE,
                        &timeout,
                        evmRespBuf,
                        EVM_RESP_MAX);
    if (rc <= 0) {
        perror("sendAgileMessage");
        fprintf(stderr, "Failed to communicate with EVM\n");
        return NFX_ERR;
    }
    return NFX_OK;
}
#endif // SSC

#define GET_VOLID_FLAG_LVOL_ONLINE    0x1
#define GET_VOLID_FLAG_VOL_ONLINE     0x2
#ifndef FS_STANDALONE
/*
 * This routine returns the number of blocks in the logical volume.
 *     - Query the pcc to obtain a list of nodes in the cluster.
 *     - Query the evm on each node in the cluster to obtain the lvol attribute. 
 *     - reutrn NFX_OK with the numblocks iff the lvol is online on one of the nodes
 *             in the cluster.
*/
static int32
fs_get_clus_vol_mkfs_numblks (evm_volId_t volId, uint64 *numBlks)
{

    evm_MsgVolGetAttrib_t   *evmReq;
    evm_MsgVolCreate_t      *evmResp;
    ncmRqNodeList_t         ncmRqNL;
    ncmNodeList_t           *ncmList = NULL;
    ncmMsg_t                *ncmReq = NULL;
    ncmMsg_t                *ncmResp = NULL;
    uint32                  ncmReqSize, ncmRespSize;
    uint32                  i;
    int32                   rc = NFX_ERR;

    ncmReqSize = sizeof (ncmMsg_type_t) + sizeof (int32) + 
                 MAX_NODE_PER_CLUSTER * LNM_MAX_NODE_NAME;
    
    FS_SHELL_MEM_ALLOC(ncmList, ncmNodeList_t, ncmReqSize);

    ncmRqNL.msgType = NCM_RQ_NODE_LIST;

    rc = sendAgileMsg (-1, &ncmRqNL, sizeof (ncmRqNL), "ncm", 
                       NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &def_clus_timeout,  
                       ncmList, ncmReqSize);

    if (rc >= sizeof (ncmRqNodeList_t) && (ncmList->msgType == NCM_NODE_LIST) &&
                                           ncmList->numConnNodes >= 1) {

        /*
         * We need to send a message to the evm on each node in the cluster in
         * turn, in order to find where the lvol is actually online.
        */

        ncmReqSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolGetAttrib_t);
        FS_SHELL_MEM_ALLOC(ncmReq, ncmMsg_t, ncmReqSize);

        /*
         * Initialize the request message.
        */
        ncmReq->msgType = NCM_SEND;
        ncmReq->rqResp = 1;
        strcpy (ncmReq->appName, "evm");

        evmReq = (evm_MsgVolGetAttrib_t *)ncmReq->appMsg; 
        evmReq->hdr.msgType = EVM_VOL_GET_ATTRIB; 
        evmReq->id = volId;

        /*
         * and the response message.
        */

        ncmRespSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolCreate_t) - sizeof (uint64);
        FS_SHELL_MEM_ALLOC(ncmResp, ncmMsg_t, ncmRespSize);

        for (i = 0; i < ncmList->numConnNodes; i++) {
 
            strcpy (ncmReq->nodeName, ncmList->nodeNames[i]);

            rc = sendAgileMsg (-1, ncmReq, ncmReqSize, "ncm", NFX_SSC_UNIX, SSC_SLOT,
                               MGMTPLANE, &def_clus_timeout, ncmResp, ncmRespSize);

            if (rc == ncmRespSize && ncmResp->msgType == NCM_RESP) {

                evmResp = (evm_MsgVolCreate_t *)ncmResp->appMsg;

                if (evmResp->config.onLine) {

                    *numBlks = evmResp->config.currSize;
                    *numBlks = ((*numBlks) * EVM_LBLK_SIZE) / FS_BLOCK_SIZE;
                    rc = NFX_OK;
                    break;

                }

            } 

        }

    } else {

        rc = FS_SET_SEND_ERROR(rc);

    }
  
out:
    if (ncmList != NULL) {

        FS_SHELL_MEM_FREE(ncmList);

    }

    if (ncmReq != NULL) {

        FS_SHELL_MEM_FREE(ncmReq);

    }

    if (ncmResp != NULL) {

        FS_SHELL_MEM_FREE(ncmResp);

    }

    return rc;

}
#endif

/*
 *  After meeting with Laurent and Allen, it was decided to make as minimal changes to the
 *  fs-shell.c source module as possible to minimize changes to nfxshell and documentation and
 *  testing so close to the scheduled release date.  
 *  This means that for clusterizing the commands in this modules:
 *
 *      - fs_shell_get_volid () was enhanced to obtain the volid from the cluster conf file.
 *      - fs_sell_is_vol_online () was enhanced to obtain the status of the logical volume 
 *                              and the node name in the cluster on which it is online.
 *      - get_mkfs_numblks () was enhanced to obtain the number of blocks from the evm, on
 *                            appropriate node in the cluster.
 *      - alloc_msg () was enhanced to always allocate a ncm message which may or may not be
 *                            used by the main() steering routine.
 *
 *  It would have been nice if there was enough time to change all the commands to take
 *  options in the popular -optletter "option string" format, which is more flexible for extending the
 *  commands or making changes to the options.   Furthermore since the above functions and some
 *  others were not planned for enhancement, the code flow in each routine could have been made
 *  much better as well as the overall program.  Because of the lack of time, and the effort involved,
 *  it has been added to the big list of TODO items.
 *  -Shyam
*/

boolean
fs_shell_get_volid (char *id_str, evm_volId_t *volId, char *nodeName)
{

#if defined(FS_STANDALONE)

    uint32 id = strtoull (id_str, NULL, 10);

    volId->whole_thing = 0;
    volId->details.u.parts.clusterUnique = id;
    volId->details.u.parts.reserved = 0;

    if (id <= 0 || id > FS_SIM_VOL_MAX) {
        fprintf(stderr, "volId %d out of range\n", id);
        return FALSE;
    }

#elif defined(SSC)

    int32 rc;

    rc = fs_opt_get_clus_vol_id (id_str, volId);

    if (rc != NFX_OK) {

        fprintf (stderr, "volume %s not found\n", id_str);
        return FALSE;

    }

    if (nodeName != NULL) {

        rc = fs_opt_get_clus_vol_online_node (*volId, nodeName);

        if (rc != NFX_OK) {

            fprintf (stderr, "volume %s not online\n", id_str);
            return FALSE;

        }

    }

#endif

    return TRUE;

}

struct timeval waitEvmTo = {10, 0};

boolean
fs_shell_is_vol_online(evm_volId_t  volId, char *nodeName)
{

#if defined(FS_STANDALONE)
    return TRUE;
#elif defined(SSC)
    int32  rc;

    rc = fs_opt_get_clus_lvol_online_node (volId, nodeName);

    if (rc != NFX_OK) {
        fprintf (stderr, "logical volume 0x%llx not online\n",
                volId.whole_thing);
        return FALSE;
    }
    return TRUE;
#endif

}

/*
 * caller must provide the 
 * - tab: table with name and flags (terminated with {0,0}.
 * - name: the name to look for
 * - list: an char array, big enough to contain all data, as help string
 * return 
 * - NFX_OK  : then the flag is valid
 * - NFX_ERR : then the list is valid
 */
  
int
shell_parse_parameter(tab_t *tab, char *name, char *list, int *flag)
{
    int loop;
    
    for (loop = 0; tab->name != NULL; loop++, tab++) 
    {
        if (strcmp(name, tab->name) == 0) 
        {
            *flag = tab->flag;
            return (NFX_OK);
        }
        list += sprintf(list,"%s|", tab->name);
    }
    list--;
    *list = 0;
    
    return(NFX_ERR);
}

fs_MsgReqHdr *
fs_shell_parsecmd(char *input, fs_MsgReqHdr * (**resp)(int stat, fs_MsgRespHdr *))
{
    char *argv[FS_SSC_ARG_MAX];
    int argc;
    int i;

    argc = parse(argv, input);
    if (argc <= 0)
        return NULL;

    for (i = 0; cmds[i].name != NULL; i++)
        if (strcmp(cmds[i].name, argv[0]) == 0) {
            if (argc < cmds[i].min_args+1 || argc > cmds[i].max_args+1) {
                printf("usage: %s\n", cmds[i].usage);
                return NULL;
            }
			*resp = cmds[i].resp;
            return cmds[i].cmd(argc, argv);
        }

    printf("%s: command not found\n", argv[0]);
    return NULL;
}

static int
parse(char *argv[], char *input) {
    char *delim = " \t\n";
    int argc = 0;
    char *token;

    token = strtok(input, delim);

    while (token != NULL) {
        argv[argc++] = token;
        if (argc == FS_SSC_ARG_MAX) {
            printf("arg list too long\n");
            return -1;
        }
        token = strtok(NULL, delim);
    }

    return argc;
}

/*
 * Please see comments on the header of fs_shell_get_volid () for details about changes made
 * to this routine to handle ncm messages.
 * We allocate extra memory for ncm request to fit in and initialize it.  It is up to the 
 * main () routine to use it or not.    We return the pointer to the fs_MsgReqHdr, so the
 * main routine has to back up the pointer to access the entire ncm message.
*/

static void *
alloc_msg(int type, uint32 size, char *nodeName)
{
    fs_MsgReqHdr *hdr;
    void         *msg;
#ifndef FS_STANDALONE
    uint32       size_ncm;
    ncmMsg_t     *ncmReq;

    msg = NULL;
    ncmReq = NULL;

    size_ncm = size + sizeof (ncmMsg_t);

    ncmReq = (ncmMsg_t *) fs_ramAlloc(FS_RAM_SIM_MISC, size_ncm);

    if (ncmReq != NULL) {

        bzero(ncmReq, size_ncm);
    
        strcpy (ncmReq->appName, "efs");
        if (nodeName != NULL) {

            strcpy (ncmReq->nodeName, nodeName);
            ncmReq->msgType = NCM_SEND_EMB;

        }
        ncmReq->rqResp = 1;
        ncmReq->slot = NFX_SLOTTYPE_NFP;
        ncmReq->cpu = NFX_NFP_FP2;

        /*
         * Initialize the ncm header.  This may or maynot be actually routed to ncm.
        */

        msg = (void *)ncmReq->appMsg;
        hdr = msg;
        hdr->requestType = type;
        hdr->len = size;

    }
#else
    msg = fs_ramAlloc(FS_RAM_SIM_MISC, size);
    bzero(msg, size);

    hdr = msg;
    hdr->requestType = type;
    hdr->len = size;
#endif

    return msg;
}

static void
dealloc_msg(void *msgp)
{
#ifndef FS_STANDALONE
    ncmMsg_t *ncmReq;
    ncmReq = (ncmMsg_t *) CONTAINING_RECORD(msgp, ncmMsg_t, appMsg);
    fs_ramDealloc(FS_RAM_SIM_MISC, ncmReq);
#else
    fs_ramDealloc(FS_RAM_SIM_MISC, msgp);
#endif
}

#define MKFS_LO         FS_MIN_BLOCKS
#define MKFS_HI         FS_MAX_BLOCKS_SUPPORTED

#define MKFS_DEFAULT     5000ull      // ~40 MB with 8K blocks
// define MKFS_DEFAULT_LOG  125ull      // ~1 MB with 8K blocks
#define MKFS_DEFAULT_LOG   30ull      // ~0.24 MB with 8K blocks

NFX_ASSERT_STATIC(MKFS_LO <= MKFS_DEFAULT);
NFX_ASSERT_STATIC(MKFS_DEFAULT <= MKFS_HI);

static uint64
get_mkfs_numblks(evm_volId_t volId)
{
    uint64  size = 0;

#if defined(FS_STANDALONE)

    size =  FS_MAX_BLOCKS;

#elif defined(SSC)
    int32  rc;

    rc = fs_get_clus_vol_mkfs_numblks (volId, &size);

#endif

    return size;
}    

static fs_MsgReqHdr *
cmd_mkfs(int argc, char *argv[])
{
    fs_MsgMakeFsReq *mkfs;
    evm_volId_t volId;
    uint64 numBlks, numVolBlks;
    uint32 ext_log = FALSE;
#ifndef FS_STANDALONE
    ncmMsg_t   *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif
    
    FS_ASSERT(argc >= 2 && argc <= 4);
    if (!fs_shell_get_volid(argv[1], &volId, NULL))
        return NULL;

    if (!fs_shell_is_vol_online (volId, nodeName)) {

        return NULL;

    }

    numVolBlks = get_mkfs_numblks(volId);

#if defined(FS_STANDALONE)
    numBlks = MKFS_DEFAULT;
#else
    numBlks = numVolBlks;
#endif

    if (argc == 3 || argc == 4)
    {
        if (strcmp ("ext-log", argv[2]) == 0)
        {
            ext_log = TRUE;
        }
        else
        {
            if (strcasecmp (argv[2], "0x") == 0) {

                numBlks = strtoull(argv[2], NULL, 16);

            } else {

                numBlks = strtoull(argv[2], NULL, 10);

            }
            if (numBlks > numVolBlks) {
                printf("numBlks %lld greater than volume %lld\n",
                    numBlks, numVolBlks);
                return NULL;
            }
        }
    }

    if (numBlks < MKFS_LO || numBlks > MKFS_HI)
    {
        printf("numBlks %lld out of range (%lld .. %lld)\n",
               numBlks, MKFS_LO, MKFS_HI);
        return NULL;
    }

    if (argc == 4) 
    {
        if (strcmp ("ext-log", argv[3]) == 0)
        {
            ext_log = TRUE;
        }
        else
        {
            printf ("Wrong option, use default (log device == SELF)\n");
        }
    }
    
    mkfs = alloc_msg(FS_REQ_MAKEFS, sizeof *mkfs, nodeName);
    mkfs->volId = volId;
    mkfs->numBlks = numBlks;
    mkfs->blkSize = FS_BLOCK_SIZE;
    mkfs->ext_log = ext_log;

    printf("mkfs on vol %llx of %lld 8K blocks\n", volId.whole_thing, numBlks);

    return (fs_MsgReqHdr *) mkfs;
}

static fs_MsgReqHdr *
cmd_viewfs(int argc, char *argv[])
{
    fs_MsgViewFsReq *msg;

    msg = alloc_msg(FS_REQ_VIEWFS, sizeof *msg, NULL);

    msg->clientprivate = FALSE;
    if (argc == 2)
    {
        if (!strcmp("-v", argv[1]))
        {
            msg->clientprivate = TRUE;   // verbose
        }
    }
    msg->type = FS_VIEW_LIST_OF_VOLUMES;
    msg->volId.whole_thing = 0;    // not used

    
    return (fs_MsgReqHdr *)msg;
}

static void
viewfs_info(
    fs_FSInfo         *fsInfo,
    boolean           verbose)
{
    uint32            loop_1, loop_2 ;
    
    if (verbose)
    {
        printf("Name      : %s\n", fsInfo->mountName);
        printf("State     : ");
    }
    else
    {
        if (fsInfo->nameLength > 25)
        {
            fsInfo->mountName[25] = 0;
            printf("%25s... ", fsInfo->mountName);
        }
        else
        {
            printf("%25s    ", fsInfo->mountName);
        }
    }
        
    switch (fsInfo->state)
    {
    case FS_VOL_STATE_ONLINE:
        printf ("online         ");
        break;
    case FS_VOL_STATE_ONLINE_RDONLY:
        printf ("online RO      ");
        break;
    case FS_VOL_STATE_OFFLINE:
        printf ("offline        ");
        break;
    case FS_VOL_STATE_PAUSED:
        printf ("paused         ");
        break;
    case FS_VOL_STATE_MOUNTING:
        printf ("mounting       ");
        break;
    case FS_VOL_STATE_UNMOUNTING:
        printf ("unmounting     ");
        break;
    case FS_VOL_STATE_FAILING:
        printf ("failing        ");
        break;
    case FS_VOL_STATE_BACKUP_RESTORE:
        printf ("backup/restore ");
        break;
    default:
        printf ("unknown   %3d  ", fsInfo->state);
        break;
    }
        
    if (verbose)
    {
        printf("\n");
        printf("VolId     : 0x%016llx\n", fsInfo->volId.whole_thing);
        printf("Size      : %lld MByte (Nbr blocks %lld)\n", 
               fsInfo->stat.numBlks * fsInfo->stat.fsBlockSize / 1024 / 1024, 
               fsInfo->stat.numBlks);
        printf("Available : %lld MByte (Nbr blocks %lld)\n", 
               fsInfo->stat.numBlksAvail * fsInfo->stat.fsBlockSize / 1024 / 1024, 
               fsInfo->stat.numBlksAvail);
        printf("Inodes    : %lld  (available %lld)\n", 
               fsInfo->stat.numInodes,
               fsInfo->stat.numInodesAvail);
        printf("grow HWM  : %d\n", fsInfo->percentMinFree);
        printf("Mkfs      : %s", ctime((time_t *)&fsInfo->mkfsTime));
        printf("Number of mounts: %d (dirty: %d )\n", 
               fsInfo->nbrMount, 
               fsInfo->nbrDirtyMount);
        printf("Number of umounts: %d\n", 
               fsInfo->nbrUmount);
        printf("Number of mount failures: %d (log) %d (replay) %d (eek) %d (snap)\n", 
               fsInfo->nbrMountFailedLogDev,
               fsInfo->nbrMountFailedReplay,
               fsInfo->nbrMountFailedEek,
               fsInfo->nbrMountFailedSnap);

        for (loop_1 = 0 ; loop_1 < FS_SUPER_NBR_MOUNTTIME; loop_1++)
        {
            if (fsInfo->mountTime[loop_1] == 0)
            {
                break;
            }
        }
        printf("Result of last %d mount operation(s)\n", loop_1);
        for (loop_2 = 0 ; loop_2 < loop_1; loop_2++)
        {
            if (fsInfo->mountFail[loop_2] == FS_MOUNT_NO_FAIL)
            {
                switch (fsInfo->mountState[loop_2])
                {
                case FS_VOL_STATE_ONLINE:
                    printf ("  online         ");
                    break;
                case FS_VOL_STATE_ONLINE_RDONLY:
                    printf ("  online RO      ");
                    break;
                case FS_VOL_STATE_OFFLINE:
                    printf ("  offline        ");
                    break;
                default:
                    printf ("  unknown state  ");
                    break;
                }
            }
            else
            {
                switch (fsInfo->mountFail[loop_2])
                {
                case FS_MOUNT_FAIL_LOGDEV:
                    printf ("  Failed due to wrong log device  ");
                    break;
                case FS_MOUNT_FAIL_REPLAY:
                    printf ("  Failed during replay  ");
                    break;
                case FS_MOUNT_FAIL_EEK:
                    printf ("  Failed during eek  ");
                    break;
                case FS_MOUNT_FAIL_SNAP:
                    printf ("  Failed during snap resume  ");
                    break;
                default:
                    printf ("  Failed for unknown reason %d  ", fsInfo->mountFail[loop_2]);
                    break;
                }
            }
            printf ("  Mount time %s", ctime((time_t *)&fsInfo->mountTime[loop_2]));
            // printf ("\n");  // ctime adds the \n
        }

        for (loop_1 = 0 ; loop_1 < FS_SUPER_NBR_MOUNTTIME; loop_1++)
        {
            if (fsInfo->umountTime[loop_1] == 0)
            {
                break;
            }
        }
        printf("Result of last %d umount operation(s)\n", loop_1);
        for (loop_2 = 0 ; loop_2 < loop_1; loop_2++)
        {
            switch (fsInfo->umountState[loop_2])
            {
            case FS_UMOUNT_NORMAL:
                break;
            default:
                printf ("  unknown state");
                break;
            }
            printf ("  umount time %s", ctime((time_t *)&fsInfo->umountTime[loop_2]));
            // printf ("\n");  // ctime adds the \n
        }
        printf ("\n");
    }
    else
    {
        printf("0x%016llx  ", fsInfo->volId.whole_thing);
        printf("%lld MByte\n", fsInfo->stat.numBlks * fsInfo->stat.fsBlockSize / 1024 / 1024);
    }
    
}

#define SHELL_MAX_VOLID_TO_STORE   187   // max msg = 1500 / volid size = 8
static evm_volId_t volIdList[SHELL_MAX_VOLID_TO_STORE];
static uint32 volIdListEntry;
static uint32 volIdListMAX;


static fs_MsgReqHdr *
viewfs_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgViewFsResp  *resp = (fs_MsgViewFsResp *) hdr;
    fs_FSInfo         *fsInfo = &(resp->u.fsInfo[0]);
    evm_volId_t       *volId = &(resp->u.volId[0]);
    boolean           verbose = resp->clientprivate;
    uint32            loop;
    fs_MsgViewFsReq   *req;

    if (resp->type == FS_VIEW_LIST_OF_VOLUMES)
    {
        FS_ASSERT(resp->nbrFs < SHELL_MAX_VOLID_TO_STORE);
        
        if (resp->nbrFs == 0)
        {
            printf("No file systems\n");
            return NULL;
        }

        volIdListMAX = resp->nbrFs;
        volIdListEntry = 0;
        
        for (loop = 0 ; loop < resp->nbrFs ; loop++)
        {
            volIdList[loop].whole_thing = volId->whole_thing;
            volId++;
        }

        if (! verbose)
        {
            printf("Name                         state      volId               Size \n");
            printf("---------------------------------------------------------------------\n");
        }

        if (resp->totalNbrFs != resp->nbrFs)
        {
            printf("Total known files systems: %d (show only the first %d)\n", 
                   resp->totalNbrFs, resp->nbrFs);
        }
            
    }
    else
    {
        FS_ASSERT(resp->type == FS_VIEW_ONE_VOLUME);

        if (resp->nbrFs == 0)
        {
            printf ("Error retrieving info for volId 0x%016llx\n",
                    volIdList[volIdListEntry].whole_thing);
        }
        else
        {
            FS_ASSERT(resp->nbrFs == 1);

            viewfs_info (fsInfo, verbose);
        }

        volIdListEntry++;
        
        if (volIdListEntry >= volIdListMAX)
        {
            // end of list
            return NULL;
        }
        
    }
    
    req = alloc_msg(FS_REQ_VIEWFS, sizeof *req, NULL);

    req->clientprivate = resp->clientprivate;

    req->type = FS_VIEW_ONE_VOLUME;

    req->volId.whole_thing = volIdList[volIdListEntry].whole_thing;

    return (fs_MsgReqHdr *)req;
}





static fs_MsgReqHdr *
cmd_mount(int argc, char *argv[])
{
    fs_MsgMountReq *mount;
    evm_volId_t volId;
    evm_volId_t logVolId;
    uint32 logDevType, logSize, numBlks;
    int32 nameLen = strlen(argv[1]);
    int32 skipEek, skipReplay;
    int32 loop;
    char *namep = argv[1];
#ifndef FS_STANDALONE
    ncmMsg_t *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName = NULL;
#endif

    FS_ASSERT(argc >= 2 && argc <= 5);
    if (!fs_shell_get_volid(argv[1], &volId, NULL))
        return NULL;

    if (!fs_shell_is_vol_online(volId, nodeName))
        return NULL;
    
    logDevType = FS_LOG_DEVTYPE_DEFAULT;
    logSize = 0 ;
    
    skipReplay = FALSE;
    skipEek = FALSE;

    for (loop = 2; loop < argc ; loop++)
    {
        if (strcmp ("DoNotReplay", argv[loop]) == 0)
        {
            skipReplay = TRUE;
            continue;
        }
        if (strcmp ("DoNotEek", argv[loop]) == 0)
        {
            skipEek = TRUE;
            continue;
        }
        if (strcmp ("NVRAM", argv[loop]) == 0)
        {
            logDevType = FS_LOG_DEVTYPE_NVRAM;
            logSize = 2 * 1024 * 1024;
            continue;
        }
        if (strcmp ("NULL", argv[loop]) == 0)
        {
            logDevType = FS_LOG_DEVTYPE_NULL;
            logSize = 16 * 1024 * 1024;
            continue;
        }
        // if we are here, check if this parameter is a valid volume
        if (!fs_shell_get_volid(argv[loop], &logVolId, NULL))
        {
            printf("Wrong volume id for log device \n");
            return NULL;
        }

#if defined(FS_STANDALONE)
        numBlks = MKFS_DEFAULT_LOG;
#else
        numBlks = get_mkfs_numblks(logVolId);
#endif
        numBlks--;   // to be save, not to use to much. Todo, exact match 
        numBlks--;        
        if (numBlks <= 0)
        {
            printf("Wrong num blocks for log device\n");
            return NULL;
        }
        logDevType = FS_LOG_DEVTYPE_VOLID;
        logSize = numBlks * FS_BLOCK_SIZE;
    } 

    mount = alloc_msg(FS_REQ_MOUNT, sizeof *mount + nameLen, nodeName);
    mount->volId = volId;
    mount->logDevType = logDevType;
    mount->logVolId = logVolId;
    mount->logSize = logSize;
    mount->skipReplay = skipReplay;
    mount->skipEek = skipEek;
    mount->fsNameLen = nameLen;
    strncpy(mount->fsName, namep, nameLen);

    return (fs_MsgReqHdr *)mount;
}
static fs_MsgReqHdr *
cmd_romount(int argc, char *argv[])
{
    fs_MsgMountReq *mount;
    evm_volId_t volId;
    evm_volId_t logVolId;
    uint32 logDevType, logSize;
    int32 nameLen = strlen(argv[1]);
    char *namep = argv[1];
#ifndef FS_STANDALONE
    ncmMsg_t *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName = NULL;
#endif

    FS_ASSERT(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, NULL))
        return NULL;

    if (!fs_shell_is_vol_online(volId, nodeName))
        return NULL;

    logDevType = FS_LOG_DEVTYPE_DEFAULT;
    logSize = 0 ;

    mount = alloc_msg(FS_REQ_MOUNT, sizeof *mount + nameLen, nodeName);
    mount->volId = volId;
    mount->logDevType = logDevType;
    mount->logVolId = logVolId;
    mount->logSize = logSize;
    mount->fsNameLen = nameLen;
    mount->rdonly = 1;
    strncpy(mount->fsName, namep, nameLen);

    return (fs_MsgReqHdr *)mount;
}

static fs_MsgReqHdr *
cmd_unmount(int argc, char *argv[])
{
    fs_MsgUnmountReq *unmount;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    ncmMsg_t *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName = NULL;
#endif

    FS_ASSERT(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    unmount = alloc_msg(FS_REQ_UNMOUNT, sizeof *unmount, nodeName);
    unmount->volId = volId;

    return (fs_MsgReqHdr *)unmount;
}

static fs_MsgReqHdr *
cmd_emapdebug(int argc, char *argv[])
{
    fs_MsgEmapdebugReq *emap;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    ncmMsg_t *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName = NULL;
#endif

    FS_ASSERT(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    emap = alloc_msg(FS_REQ_EMAPDEBUG, sizeof *emap, nodeName);
    emap->volId = volId;

    return (fs_MsgReqHdr *)emap;
}

static evm_volId_t eekVolId;

static fs_MsgReqHdr *
cmd_eek(int argc, char *argv[])
{
#ifndef FS_STANDALONE
    int stat;
    struct timeval evm_to = {60, 0};
    evm_MsgVolGetAttrib_t attrrq;
    evm_MsgVolCreate_t cfg;
    char nodeName[LNM_MAX_NODE_NAME];
    ncmMsg_t  *ncmReq;
#else
    char *nodeName;
#endif

    fs_MsgEekReq *eek;
    fs_MsgMountReq *mount;
    evm_volId_t volId;

    FS_ASSERT(argc >= 2);
    if (!fs_shell_get_volid(argv[1], &volId, NULL))
        return NULL;

    eekVolId = volId;


#ifndef FS_STANDALONE
    /*
     * PR #1746: make sure the vol is online before eeking
     */
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    nodeName = NULL;
#endif
    if (!fs_shell_is_vol_online(volId, nodeName)) {
        printf("eek: vol needs to be online\n");
        return NULL;
    }

    eek = alloc_msg(FS_REQ_EEK,
                    sizeof(fs_MsgEekReq) + strlen(argv[1]) + 1, // fsName
                    nodeName);
    eek->volId = volId;
    eek->default_reply = FS_EEK_QUERY_NO;

    if (argc > 2) {
        extern int optind;
        int ch;
#ifdef FS_STANDALONE
	extern char *optplace = "";
#endif

	/*
	 * Reset getopt to start at 2nd arg.
	 */
        optind = 2;

        while ((ch = getopt(argc, argv, "yp:t:")) >= 0) {
            switch (ch) {
            case 'y':
                printf("running eek -y\n");
                eek->default_reply = FS_EEK_QUERY_YES;
                break;
            case 'p':
                // eek->phase = atoi(optarg);
		// printf("eek phase %d\n", eek->phase);
                break;
            case 't':
                // eek->nthreads = atoi(optarg);
		// printf("eek starting %d threads\n", eek->nthreads);
                break;
            case '?':
            default:
                printf("usage: eek <volname> [-y] [-p phase] [-t nthreads]\n");
                fs_ramDealloc(FS_RAM_SIM_MISC, eek);
                return NULL;
            }
        }
    }

    /*
     * Dummy up a mount request inside eek to allow an unmounted
     * FS to be mounted.
     */
    mount = &eek->mountreq;
    mount->volId = volId;
    mount->logDevType = FS_LOG_DEVTYPE_DEFAULT;
    // mount->logVolId = 0xdeadbeef;
    mount->logSize = 0xfeedface;
    mount->eek = TRUE;
    mount->fsNameLen = strlen(argv[1]);
    strncpy(mount->fsName, argv[1], mount->fsNameLen);
    mount->fsName[mount->fsNameLen + 1] = 0;
    //printf("fsName %s\n", mount->fsName);

    return (fs_MsgReqHdr *)eek;
}


static fs_MsgReqHdr *
cmd_zap(int argc, char *argv[])
{
    fs_MsgZapReq *zap;
    evm_volId_t volId;

    FS_ASSERT(argc >= 3);
    if (!fs_shell_get_volid(argv[1], &volId, NULL))
        return NULL;

    switch (*argv[2]) {
    default:
	printf("zap action: one of s[how], b[tree], i[node], d[irent], r[aw-io], l[inkCnt]\n");
    printf("zap [data]: overwrite block with this data\n");
	return NULL;
    case 'b':
    case 'd':
    case 'i':
    case 'l':
    case 'r':
    case 's':
	break;
    }

    zap = alloc_msg(FS_REQ_ZAP, sizeof *zap, NULL);
    zap->volId = volId;
    zap->action = *argv[2];
    zap->inum = (argc > 3) ? strtoull(argv[3], NULL, 0) : 0;
    zap->data = (argc > 4) ? strtoull(argv[4], NULL, 0) : 0;

    return (fs_MsgReqHdr *)zap;
}

static fs_MsgReqHdr *
cmd_growfs(int argc, char *argv[])
{
    fs_MsgGrowfsReq *growfs;
    evm_volId_t volId;
    uint64 numBlks;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc == 3);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    growfs = alloc_msg(FS_REQ_GROWFS, sizeof *growfs, nodeName);
    growfs->volId = volId;

    numBlks = strtoull(argv[2], NULL, 10);
    if (numBlks < MKFS_LO || numBlks > MKFS_HI)
    {
        printf("numBlks %lld out of range (%lld .. %lld)\n",
               numBlks, MKFS_LO, MKFS_HI);
        return NULL;
    }
        
    growfs->newNumBlks = numBlks;

    return (fs_MsgReqHdr *)growfs;
}

static fs_MsgReqHdr *
cmd_sync(int argc, char *argv[])
{
    fs_MsgSyncReq *sync;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    sync = alloc_msg(FS_REQ_SYNC, sizeof *sync, nodeName);
    sync->volId = volId;

    return (fs_MsgReqHdr *)sync;
}

static fs_MsgReqHdr *
cmd_flush(int argc, char *argv[])
{
    fs_MsgFlushReq *flush;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    flush = alloc_msg(FS_REQ_FLUSH, sizeof *flush, nodeName);
    flush->volId = volId;

    return (fs_MsgReqHdr *)flush;
}

static fs_MsgReqHdr *
cmd_freeze(int argc, char *argv[])
{
    fs_MsgFreezeReq *freeze;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    assert(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    freeze = alloc_msg(FS_REQ_FREEZE, sizeof *freeze, nodeName);
    freeze->volId = volId;

    return &freeze->hdr;
}

static fs_MsgReqHdr *
cmd_thaw(int argc, char *argv[])
{
    fs_MsgThawReq *thaw;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    assert(argc == 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    thaw = alloc_msg(FS_REQ_THAW, sizeof *thaw, nodeName);
    thaw->volId = volId;

    return &thaw->hdr;
}

/*
 * XXX: TODO: make this a generic "show" command with "environment
 * variables".
 */
static fs_MsgReqHdr *
cmd_show(int argc, char *argv[])
{
    fs_MsgShowLockedReq *shlck;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc == 3);
    if (!fs_shell_get_volid(argv[2], &volId, nodeName))
        return NULL;

    shlck = alloc_msg(FS_REQ_SHOW_LOCKED, sizeof *shlck, nodeName);
    shlck->volId = volId;

    return (fs_MsgReqHdr *)shlck;
}

static fs_MsgReqHdr *
cmd_help(int argc, char *argv[])
{
    int i;
    for (i = 0; cmds[i].name != NULL; i++)
        printf("%s\n", cmds[i].usage);
    return NULL;
}

#if !defined(SSC)
#include <sys/wait.h>

static int fs_sim_system(char *command)
{
    int pid, status;
    extern char **environ;

    if (command == 0)
        return 1;
    pid = fork();
    if (pid == -1)
        return -1;
    if (pid == 0) {
        char *argv[4];
        char *shell = getenv("SHELL");
        if (shell == NULL)
            shell = "/bin/sh";
        argv[0] = "sh";
        argv[1] = "-c";
        argv[2] = command;
        argv[3] = 0;
        execve(shell, argv, environ);
        exit(127);
    }
    do {
        if (waitpid(pid, &status, 0) == -1) {
            if (errno != EINTR)
                return -1;
        } else
            return status;
    } while(1);
}

static fs_MsgReqHdr *cmd_shell(int argc, char *argv[])
{
    int i;
    char buf[1024];
    char *s = buf;
    for (i = 1; i < argc; i++) {
        int len = strlen(argv[i]);
        if (i > 1)
            *s++ = ' ';
        memcpy(s, argv[i], len);
        s += len;
        *s = '\0';
    }
    fs_sim_system(buf);
    return NULL;
}
#endif


#if defined(SSC)

// see elog-shell for the good implementation
// (eee_getAppIdFromName and eee_getNameFromAppId are unkown)
// the values of fs_applicationId and nvram_appl_id are hardcoded

#endif

char efs_name[] = "efs";
char nvram_name[] = "nvram";

static int
get_applications (void)
{
    int app_tab = 0 ;
    
    apptab[app_tab].name = efs_name ;
    apptab[app_tab].flag = fs_applicationId ;

    app_tab++;
    
    apptab[app_tab].name = nvram_name ;
    apptab[app_tab].flag = nvram_appl_id ;

    app_tab++;
    
    apptab[app_tab].name = 0 ;
    apptab[app_tab].flag = 0 ;

    return (NFX_OK);
}


char tablist[USAGE_STR_SIZE];
char usage_cmd[USAGE_STR_SIZE];
char usage_name[] = "elog" ;
char null_name[] = "" ;

static fs_MsgReqHdr *
cmd_elog(int argc, char *argv[])
{
    e_log_configure_msg_t msg;
    int command = 0 ;
    int value_1 = 0;
    int value_2 = 0;
    int rc = NFX_ERR ;
    char *cmd_name = NULL ;
    char *val_1_name = NULL ;
    char *val_2_name = NULL ;
    int tmp, val_argc = 0 ;

    if (argc >= 2)
        cmd_name = argv[1] ;
    else
        cmd_name = null_name ;

    if (argc >= 3)
        val_1_name = argv[2] ;
    else
        val_1_name = null_name ;
    
    if (argc >= 4)
    {
        val_2_name = argv[3] ;
        val_argc = 3 ;
    }
    else
        val_2_name = null_name ;
    
    // FS_ASSERT(argc == 3);

#if !defined SSC
    get_applications();
#endif

    if ((rc = shell_parse_parameter (elogtab, cmd_name, tablist, &command)) != NFX_OK)
    {
        printf("Usage: %s <%s>\n", usage_name, tablist);
        return NULL;
    }
    
    switch(command)
    {
    case ELOG_CONFIGURE_SEVERITY:
    case ELOG_CONFIGURE_TEST:
    case ELOG_CONFIGURE_APP_DEFAULT_SEVERITY:
        rc = shell_parse_parameter (severitytab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_ENABLE:
    case ELOG_CONFIGURE_SHOW_LINE:
    case ELOG_CONFIGURE_SHOW_SEQ:
    case ELOG_CONFIGURE_SHOW_SEVERITY:
    case ELOG_CONFIGURE_SHOW_APP_NAME:
        rc = shell_parse_parameter (onofftab, val_1_name, tablist, &value_1);
        break;
        
    case ELOG_CONFIGURE_SHOW_LOCAL:
        rc = shell_parse_parameter (showlocaltab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_APP_DEFAULT_BITMAP:
        if (sscanf (val_1_name, "0x%x", &value_1) == 1)
        {
            rc = NFX_OK;
        }
         else
        {
            sprintf(tablist, "0xVAL");
            rc = NFX_ERR;
        }
        break;

    case ELOG_CONFIGURE_APP_SEVERITY:
        if ((rc = shell_parse_parameter (apptab, val_1_name, tablist, &value_1)) == NFX_OK)
        {
            rc = shell_parse_parameter (severitytab, val_2_name, tablist, &value_2);
        }
        if (rc != NFX_OK)
        {
            sprintf(usage_cmd, "app-sev <app> <severity>, choice:  ");
            cmd_name = usage_cmd;
        }
        break;
        
    case ELOG_CONFIGURE_APP_SET_BITMAP:
    case ELOG_CONFIGURE_APP_CLEAR_BITMAP:
             if ((rc = shell_parse_parameter (apptab, val_1_name, tablist, &value_1)) == NFX_OK)
        {
            if (value_1 == fs_applicationId)
            {
                while ((rc == NFX_OK) && (val_argc < argc))
                {
                    rc = shell_parse_parameter (fs_tracetab, argv[val_argc], tablist, &tmp);
                    value_2 |= tmp;
                    val_argc++;
                }
            }
            else
            if (value_1 == nvram_appl_id)
            {
                while ((rc == NFX_OK) && (val_argc < argc))
                {
                    rc = shell_parse_parameter (nv_tracetab, argv[val_argc], tablist, &tmp);
                    value_2 |= tmp;
                    val_argc++;
                }
            }
            else
            {
                sprintf(tablist, "No specific data for this application");
                rc = NFX_ERR;
            }
        }
        if (rc != NFX_OK)
        {
            sprintf(usage_cmd, "[un]trace <app> <type>, choice:  ");
            cmd_name = usage_cmd;
        }
        break;

    case ELOG_CONFIGURE_SHOW_DATE:
        rc = shell_parse_parameter (datetab, val_1_name, tablist, &value_1);
        break;

    case ELOG_CONFIGURE_FORMAT:
        rc = shell_parse_parameter (formattab, val_1_name, tablist, &value_1);
        break;

    default:
        rc = NFX_ERR;
        break;        
    }

    if (rc == NFX_OK)
    {
#if !defined(SSC)
        switch(command)
        {
        case ELOG_CONFIGURE_SEVERITY:
            elog_severity = value_1;
            break;
        }
#endif
    }
    else
    {
        printf("Usage: %s %s <%s>\n", usage_name, cmd_name, tablist);
        return NULL;
    }
    
    if (rc == NFX_OK)
    {
        //msg = (e_log_configure_msg_t *)alloc_msg(command,
        //                                         sizeof (e_log_configure_msg_t));
        msg.command = command;
        msg.len = sizeof(e_log_configure_msg_t);
        msg.value_1 = value_1;
        msg.value_2 = value_2;
        
#if defined(FS_STANDALONE)
        e_log_configure (&msg, sizeof(e_log_configure_msg_t));
        // fs_ramDealloc(FS_RAM_SIM_MISC, msg);
        return NULL;
#else
        // Todo: send msg to elog !!!!
        // return (fs_MsgReqHdr *)msg;
        return NULL;
#endif
    }

    return NULL;
}

static fs_MsgReqHdr *
cmd_exit(int argc, char *argv[])
{
    //extern uint32 parentPid;
    //kill(parentPid, SIGTERM);
    exit(0);
    /*NOTREACHED*/
    return NULL;
}



static fs_MsgReqHdr *
cmd_cachestats(int argc, char *argv[])
{
    fs_MsgCachestatsReq *msg;
    evm_volId_t volId;
    boolean isClear = FALSE;
    boolean isVerbose = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;
    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (strcmp("-v", argv[carg]) == 0)
        {
            isVerbose = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argumunt, nor volid
            return NULL;
        }
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif

    msg = alloc_msg(FS_REQ_CACHESTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isClear = isClear;
    msg->isVerbose = isVerbose;

    return (fs_MsgReqHdr *)msg;
}


char *fs_inode_name[FS_MAX_META_INODE] = {
    "bad inode                 ",
    "inode inode               ",
    "root inode                ",
    "ref count inode           ",
    "buddy inode               ",
    "unused inode              ",
    "full block inode          ",
    "dir in inode              ",
    "fileset info inode        ",
    "fileset user quota inode  ",
    "fileset group quota inode ",
    "null inode                ",
    "trashcan inode            ",
    "snapshot dir inode        ",
    "snapshot but inode        ",
    "link inode                ",
    "tempdir inode             ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
    "unused                    ",
};

static fs_MsgReqHdr *
cachestats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgCachestatsResp   *msg = (fs_MsgCachestatsResp *) hdr;
    int loop, itype;

    if (msg->volId.whole_thing == 0)
        printf("\nCachestats for all volumes:\n");
    else
        printf("\nCachestats for 0x%llx:\n", msg->volId.whole_thing);

    printf("\nType            Max        Used       Dirty        Hits      Misses 1/00        COW        LWM\n");
    printf("------------------------------------------------------------------------------------------------\n");

    itype = FS_BUF_CACHE_ITYPE_NONDIR;
    printf("8knondir %10d  %10d  %10d  %10d  %10d %4d %10d %10d\n",
           msg->bufCacheStats[FS_BUFLIST_META][itype].size,
           msg->bufCacheStats[FS_BUFLIST_META][itype].used,
           msg->bufCacheStats[FS_BUFLIST_META][itype].dirty,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits,
           msg->bufCacheStats[FS_BUFLIST_META][itype].misses,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits ? 
           (uint32)((uint64)msg->bufCacheStats[FS_BUFLIST_META][itype].misses * 1000 / 
                    msg->bufCacheStats[FS_BUFLIST_META][itype].hits) : 0,
           msg->bufCacheStats[FS_BUFLIST_META][itype].cows,
#ifdef FS_DYN_CACHE_SIZE
           msg->bufCacheStats[FS_BUFLIST_META][itype].LWM
#else
           0
#endif
        );

    itype = FS_BUF_CACHE_ITYPE_COMP_NONDIR;
    printf("1knondir %10d  %10d  %10d  %10d  %10d %4d %10d %10d\n",
           msg->bufCacheStats[FS_BUFLIST_META][itype].size,
           msg->bufCacheStats[FS_BUFLIST_META][itype].used,
           msg->bufCacheStats[FS_BUFLIST_META][itype].dirty,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits,
           msg->bufCacheStats[FS_BUFLIST_META][itype].misses,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits ? 
           (uint32)((uint64)msg->bufCacheStats[FS_BUFLIST_META][itype].misses * 1000 / 
                    msg->bufCacheStats[FS_BUFLIST_META][itype].hits) : 0,
           msg->bufCacheStats[FS_BUFLIST_META][itype].cows,
#ifdef FS_DYN_CACHE_SIZE
           msg->bufCacheStats[FS_BUFLIST_META][itype].LWM
#else
           0
#endif
        );

    itype = FS_BUF_CACHE_ITYPE_DIR;
    printf("8kdir    %10d  %10d  %10d  %10d  %10d %4d %10d %10d\n",
           msg->bufCacheStats[FS_BUFLIST_META][itype].size,
           msg->bufCacheStats[FS_BUFLIST_META][itype].used,
           msg->bufCacheStats[FS_BUFLIST_META][itype].dirty,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits,
           msg->bufCacheStats[FS_BUFLIST_META][itype].misses,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits ? 
           (uint32)((uint64)msg->bufCacheStats[FS_BUFLIST_META][itype].misses * 1000 / 
                    msg->bufCacheStats[FS_BUFLIST_META][itype].hits) : 0,
           msg->bufCacheStats[FS_BUFLIST_META][itype].cows,
#ifdef FS_DYN_CACHE_SIZE
           msg->bufCacheStats[FS_BUFLIST_META][itype].LWM
#else
           0
#endif
        );

    itype = FS_BUF_CACHE_ITYPE_COMP_DIR;
    printf("1kdir    %10d  %10d  %10d  %10d  %10d %4d %10d %10d\n",
           msg->bufCacheStats[FS_BUFLIST_META][itype].size,
           msg->bufCacheStats[FS_BUFLIST_META][itype].used,
           msg->bufCacheStats[FS_BUFLIST_META][itype].dirty,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits,
           msg->bufCacheStats[FS_BUFLIST_META][itype].misses,
           msg->bufCacheStats[FS_BUFLIST_META][itype].hits ? 
           (uint32)((uint64)msg->bufCacheStats[FS_BUFLIST_META][itype].misses * 1000 / 
                    msg->bufCacheStats[FS_BUFLIST_META][itype].hits) : 0,
           msg->bufCacheStats[FS_BUFLIST_META][itype].cows,
#ifdef FS_DYN_CACHE_SIZE
           msg->bufCacheStats[FS_BUFLIST_META][itype].LWM
#else
           0
#endif
        );

    itype = FS_BUF_CACHE_ITYPE_NONDIR;
    printf("user     %10d  %10d  %10d  %10d  %10d %4d %10d %10d\n",
           msg->bufCacheStats[FS_BUFLIST_USER][itype].size,
           msg->bufCacheStats[FS_BUFLIST_USER][itype].used,
           msg->bufCacheStats[FS_BUFLIST_USER][itype].dirty,
           msg->bufCacheStats[FS_BUFLIST_USER][itype].hits,
           msg->bufCacheStats[FS_BUFLIST_USER][itype].misses,
           msg->bufCacheStats[FS_BUFLIST_USER][itype].hits ?
           (uint32)((uint64)msg->bufCacheStats[FS_BUFLIST_USER][itype].misses * 1000 / 
                    msg->bufCacheStats[FS_BUFLIST_USER][itype].hits) : 0, 
           msg->bufCacheStats[FS_BUFLIST_USER][itype].cows,
#ifdef FS_DYN_CACHE_SIZE
           msg->bufCacheStats[FS_BUFLIST_USER][itype].LWM
#else
           0
#endif
        );

    printf("inodes   %10d  %10d  %10d  %10d  %10d %4d %10s %10d\n",
           msg->inodeCacheStats[0].size,
           msg->inodeCacheStats[0].used,
           msg->inodeCacheStats[0].dirty,
           msg->inodeCacheStats[0].hits,
           msg->inodeCacheStats[0].misses,
           msg->inodeCacheStats[0].hits ?
           (uint32)((uint64)msg->inodeCacheStats[0].misses * 1000 / 
                    msg->inodeCacheStats[0].hits) : 0,
           "",
#ifdef FS_DYN_CACHE_SIZE
           msg->inodeCacheStats[0].LWM
#else
           0
#endif
        );

    printf("\nI/O         Curr        Max           Meta        User         Log\n");
    printf("Type        Pending     Pending       Total       Total       Total\n");
    printf("-------------------------------------------------------------------\n");
    printf("read     %10d  %10d  %10d  %10d  %10d\n",
           msg->currReadsOutstanding,
           msg->maxReadsOutstanding,
           msg->totalMetaReads,
           msg->totalUserReads,
           msg->totalLogReads);
    printf("write    %10d  %10d  %10d  %10d  %10d\n",
           msg->currWritesOutstanding,
           msg->maxWritesOutstanding,
           msg->totalMetaWrites,
           msg->totalUserWrites,
           msg->totalLogWrites);
    printf("read-ahead                       %10d  %10d\n",
           msg->totalMetaReadAheads,
           msg->totalUserReadAheads);


    if ( ! msg->isVerbose)
    {
        return NULL;
    }
    
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
    printf("\n");
    for (loop = 0 ; loop < FS_MAX_META_INODE; loop++)
    {
        if (msg->missCntType[loop])
        {
            printf("Miss in inode $:  %s  count: %d\n", fs_inode_name[loop], msg->missCntType[loop]);
        }
    }
#endif
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
    printf("\n");
    for (loop = 0 ; loop < FS_MAX_META_INODE; loop++)
    {
        if (msg->bufMissCntType[FS_BUFLIST_META][loop])
        {
            printf("Miss in meta buf $:  %s  count: %d\n", 
                   fs_inode_name[loop], msg->bufMissCntType[FS_BUFLIST_META][loop]);
        }
    }
    printf("\n");
    for (loop = 0 ; loop < FS_MAX_META_INODE; loop++)
    {
        if (msg->bufUseCntType[FS_BUFLIST_META][loop])
        {
            printf("Use  in meta buf $:  %s  count: %d\n", 
                   fs_inode_name[loop], msg->bufUseCntType[FS_BUFLIST_META][loop]);
        }
    }
#endif

    printf("\n");
    for (loop = 0 ; loop < FS_MAX_META_INODE; loop++)
    {
        if (msg->diskWritebackType[loop])
        {
            printf("Writeback buf for inode:  %s  disk_writes: %d  trans_writes: %d\n",
		fs_inode_name[loop], msg->diskWritebackType[loop],
		msg->transWriteType[loop]);
        }
    }

    printf("wb wakeup counts:\n");
    printf("eventType                meta          user\n");
    for (loop = 0; loop < FS_WB_WAKEUP_MAX; loop++)
    {
        FS_ASSERT(wbwakeuptab[loop].flag == loop);

        if ((msg->bufWbCount[FS_BUFLIST_META][loop] + msg->bufWbCount[FS_BUFLIST_USER][loop]) > 0)
        {
            printf("%-12s     %12d  %12d\n", 
                   wbwakeuptab[loop].name,
                   msg->bufWbCount[FS_BUFLIST_META][loop],
                   msg->bufWbCount[FS_BUFLIST_USER][loop]);
        }
    }
    
    printf("Max per Loop     %12d  %12d\n", 
           msg->metaBufWbMaxPerLoop, 
           msg->userBufWbMaxPerLoop);

    printf("\ninode cleaner counts:\n");
    for (loop = 0; loop < FS_INODE_CLEANER_MAX; loop++)
    {
        FS_ASSERT(inocleanertab[loop].flag == loop);

        if (msg->inodeCleanerCount[loop] > 0)
        {
            printf("%-12s     %12d\n", 
                   inocleanertab[loop].name,
                   msg->inodeCleanerCount[loop]);
        }
    }
    printf("Max per Loop     %12d\n", 
           msg->inodeCleanerMaxPerLoop);

#ifdef FS_INODE_CHECK_HASH_DEPTH
    printf("\n");    
    printf("Inode cache hash size              %d\n", msg->inodeHashMax);
    printf("Inode cache hash used buckets      %d\n", msg->inodeHashUsedEntries);
    printf("Inode cache hash total entries     %d\n", msg->inodeHashTotalEntries);
    if (msg->inodeHashUsedEntries > 0)
        printf("Inode cache hash avg depth         %d\n", msg->inodeHashTotalEntries / msg->inodeHashUsedEntries);
    printf("Inode cache hash max depth         %d\n", msg->inodeHashMaxDepth);
    printf("Inode cache lookups                %d\n", msg->nbrLookups);
    printf("Inode cache buckets scanned        %d\n", msg->nbrScanned);
    if (msg->nbrLookups > 0)
        printf("Inode cache avg depth lookup       %d\n", msg->nbrScanned / msg->nbrLookups);
#endif

#ifdef FS_BUF_BLK_CHECK_HASH_DEPTH
    printf("\n");    
    printf("User Buf blk cache hash size            %d\n", msg->UserBufBlkHashMax);
    printf("User Buf blk cache hash used buckets    %d\n", msg->UserBufBlkHashUsedEntries);
    printf("User Buf blk cache hash total entries   %d\n", msg->UserBufBlkHashTotalEntries);
    if (msg->UserBufBlkHashUsedEntries > 0)
        printf("User Buf blk cache hash avg depth       %d\n", msg->UserBufBlkHashTotalEntries / msg->UserBufBlkHashUsedEntries);
    printf("User Buf blk cache hash max depth       %d\n", msg->UserBufBlkHashMaxDepth);
    printf("\n");    
    printf("Meta Buf blk cache hash size            %d\n", msg->MetaBufBlkHashMax);
    printf("Meta Buf blk cache hash used buckets    %d\n", msg->MetaBufBlkHashUsedEntries);
    printf("Meta Buf blk cache hash total entries   %d\n", msg->MetaBufBlkHashTotalEntries);
    if (msg->MetaBufBlkHashUsedEntries > 0)
        printf("Meta Buf blk cache hash avg depth       %d\n", msg->MetaBufBlkHashTotalEntries / msg->MetaBufBlkHashUsedEntries);
    printf("Meta Buf blk cache hash max depth       %d\n", msg->MetaBufBlkHashMaxDepth);
#endif

#ifdef FS_BUF_PAGE_CHECK_HASH_DEPTH
    printf("\n");    
    printf("User Buf page cache hash size           %d\n", msg->UserBufPageHashMax);
    printf("User Buf page cache hash used buckets   %d\n", msg->UserBufPageHashUsedEntries);
    printf("User Buf page cache hash total entries  %d\n", msg->UserBufPageHashTotalEntries);
    if (msg->UserBufPageHashUsedEntries > 0)
        printf("User Buf page cache hash avg depth      %d\n", msg->UserBufPageHashTotalEntries / msg->UserBufPageHashUsedEntries);
    printf("User Buf page cache hash max depth      %d\n", msg->UserBufPageHashMaxDepth);
    printf("\n");    
    printf("Meta Buf page cache hash size           %d\n", msg->MetaBufPageHashMax);
    printf("Meta Buf page cache hash used buckets   %d\n", msg->MetaBufPageHashUsedEntries);
    printf("Meta Buf page cache hash total entries  %d\n", msg->MetaBufPageHashTotalEntries);
    if (msg->MetaBufPageHashUsedEntries > 0)
        printf("Meta Buf page cache hash avg depth      %d\n", msg->MetaBufPageHashTotalEntries / msg->MetaBufPageHashUsedEntries);
    printf("Meta Buf page cache hash max depth      %d\n", msg->MetaBufPageHashMaxDepth);
#endif

#ifdef FS_BUF_INUM_CHECK_HASH_DEPTH
    printf("\n");    
    printf("User Buf inum cache hash size           %d\n", msg->UserBufInumHashMax);
    printf("User Buf inum cache hash used buckets   %d\n", msg->UserBufInumHashUsedEntries);
    printf("User Buf inum cache hash total entries  %d\n", msg->UserBufInumHashTotalEntries);
    if (msg->UserBufInumHashUsedEntries > 0)
        printf("User Buf inum cache hash avg depth      %d\n", msg->UserBufInumHashTotalEntries / msg->UserBufInumHashUsedEntries);
    printf("User Buf inum cache hash max depth      %d\n", msg->UserBufInumHashMaxDepth);
    printf("\n");    
    printf("Meta Buf inum cache hash size           %d\n", msg->MetaBufInumHashMax);
    printf("Meta Buf inum cache hash used buckets   %d\n", msg->MetaBufInumHashUsedEntries);
    printf("Meta Buf inum cache hash total entries  %d\n", msg->MetaBufInumHashTotalEntries);
    if (msg->MetaBufInumHashUsedEntries > 0)
        printf("Meta Buf inum cache hash avg depth      %d\n", msg->MetaBufInumHashTotalEntries / msg->MetaBufInumHashUsedEntries);
    printf("Meta Buf inum cache hash max depth      %d\n", msg->MetaBufInumHashMaxDepth);
#endif

    return NULL;
}

static fs_MsgReqHdr *
cmd_memstats(int argc, char *argv[])
{
    fs_MsgMemstatsReq *msg;
    //evm_volId_t volId;

    FS_ASSERT(argc == 1);
    //if (!fs_shell_get_volid(argv[1], &volId))
    //    return NULL;

    msg = alloc_msg(FS_REQ_MEMSTATS, sizeof *msg, NULL);
    msg->volId.whole_thing = 0;    // not used

    return (fs_MsgReqHdr *)msg;
}
static fs_MsgReqHdr *
memstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgMemstatsResp  *msg = (fs_MsgMemstatsResp *) hdr;
    int32               i;

    printf("\nType                            Alloc       Free       Diff\n");
    printf("-----------------------------------------------------------\n");
    for (i = 0; i < FS_RAM_TYPE_MAX; i++)
    {
        printf("%-26s %10d %10d %10d\n", &fs_ramTypeName[i][0],
               msg->ramStats[FS_RAM_STATS_ALLOC][i],
               msg->ramStats[FS_RAM_STATS_DEALLOC][i],
               msg->ramStats[FS_RAM_STATS_ALLOC][i] -
               msg->ramStats[FS_RAM_STATS_DEALLOC][i]);
    } 
    printf("\n");

#ifndef FS_STANDALONE
    printf("Total Flex  Used %12lld   subtotal %12lld\n", msg->total_flex_used, msg->total_flex_size);
    if (msg->total_stack_size)
        printf("Total Stack Used %12lld   subtotal %12lld\n", msg->total_stack_used, msg->total_stack_size);
    printf("Total Cont  Used %12lld   subtotal %12lld\n", msg->total_cont_used, msg->total_cont_size);
    printf("Total Mem   Used %12lld   total    %12lld (%2lld%% used)\n", 
           msg->total_flex_used + msg->total_stack_used + msg->total_cont_used,
           msg->total_flex_size + msg->total_stack_size + msg->total_cont_size,
           ((msg->total_flex_used + msg->total_stack_used + msg->total_cont_used) * 100 / 
            (msg->total_flex_size + msg->total_stack_size + msg->total_cont_size)));
    
    printf("\n");

    printf("FP   SMALL EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_SMALL, 
           msg->eeeBufs[EEE_BUF_SMALL] - msg->eeeAvailableBufs[EEE_BUF_SMALL],
           msg->eeeBufs[EEE_BUF_SMALL] - msg->eeeLWMBufs[EEE_BUF_SMALL],
           msg->eeeBufs[EEE_BUF_SMALL]);
    printf("FP   LARGE EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_LARGE,
           msg->eeeBufs[EEE_BUF_LARGE] - msg->eeeAvailableBufs[EEE_BUF_LARGE], 
           msg->eeeBufs[EEE_BUF_LARGE] - msg->eeeLWMBufs[EEE_BUF_LARGE],
           msg->eeeBufs[EEE_BUF_LARGE]);
    printf("FP   ULTRA EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_ULTRA,
           msg->eeeBufs[EEE_BUF_ULTRA] - msg->eeeAvailableBufs[EEE_BUF_ULTRA], 
           msg->eeeBufs[EEE_BUF_ULTRA] - msg->eeeLWMBufs[EEE_BUF_ULTRA],
           msg->eeeBufs[EEE_BUF_ULTRA]);
#ifdef LARGE_MEMORY
    if (msg->eeeBufs[EEE_BUF_MGMT])
    {
        printf("FP   MGMT  EEE BUF         used %8d (max %8d)   total %8d\n", 
               msg->eeeBufs[EEE_BUF_MGMT] - msg->eeeAvailableBufs[EEE_BUF_MGMT], 
               msg->eeeBufs[EEE_BUF_MGMT] - msg->eeeLWMBufs[EEE_BUF_MGMT],
               msg->eeeBufs[EEE_BUF_MGMT]);
    }
#endif
    
    printf("\n");

    printf("TXRX SMALL EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_SMALL, 
           msg->eeeTxrxBufs[EEE_BUF_SMALL] - msg->eeeTxrxAvailableBufs[EEE_BUF_SMALL],
           msg->eeeTxrxBufs[EEE_BUF_SMALL] - msg->eeeTxrxLWMBufs[EEE_BUF_SMALL],
           msg->eeeTxrxBufs[EEE_BUF_SMALL]);
    printf("TXRX LARGE EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_LARGE,
           msg->eeeTxrxBufs[EEE_BUF_LARGE] - msg->eeeTxrxAvailableBufs[EEE_BUF_LARGE], 
           msg->eeeTxrxBufs[EEE_BUF_LARGE] - msg->eeeTxrxLWMBufs[EEE_BUF_LARGE],
           msg->eeeTxrxBufs[EEE_BUF_LARGE]);
    printf("TXRX ULTRA EEE BUF (%4d)  used %8d (max %8d)   total %8d\n", 
           EEE_BUFSIZE_ULTRA,
           msg->eeeTxrxBufs[EEE_BUF_ULTRA] - msg->eeeTxrxAvailableBufs[EEE_BUF_ULTRA], 
           msg->eeeTxrxBufs[EEE_BUF_ULTRA] - msg->eeeTxrxLWMBufs[EEE_BUF_ULTRA],
           msg->eeeTxrxBufs[EEE_BUF_ULTRA]);
#ifdef LARGE_MEMORY
    if (msg->eeeTxrxBufs[EEE_BUF_MGMT])
    {
        printf("TXRX MGMT  EEE BUF         used %8d (max %8d)   total %8d\n", 
               msg->eeeTxrxBufs[EEE_BUF_MGMT] - msg->eeeTxrxAvailableBufs[EEE_BUF_MGMT], 
               msg->eeeTxrxBufs[EEE_BUF_MGMT] - msg->eeeTxrxLWMBufs[EEE_BUF_MGMT],
               msg->eeeTxrxBufs[EEE_BUF_MGMT]);
    }
#endif    

#endif


    return NULL;
}

static fs_MsgReqHdr *
cmd_reqstats(int argc, char *argv[])
{
    fs_MsgReqstatsReq *msg;
    evm_volId_t volId;
    boolean isVerbose = FALSE;
    boolean isClear = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;

    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-v", argv[carg]) == 0)
        {
            isVerbose = TRUE;
        }
        else if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argumunt, nor volid
            return NULL;
        }
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) 
    {
        strcpy (nodeName, "local");
    }
#endif

    msg = alloc_msg(FS_REQ_REQSTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isVerbose = isVerbose;
    msg->isClear = isClear;    

    return (fs_MsgReqHdr *)msg;
}

static fs_MsgReqHdr *
reqstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgReqstatsResp  *msg = (fs_MsgReqstatsResp *) hdr;
    uint64              total = 0, totalOK = 0, totalError = 0;
    int32               i; 

    if (msg->volId.whole_thing == 0)
    {
        printf("global request stats\n");
    }
    else
    {
        printf("request stats on volume  0x%llx\n",msg->volId.whole_thing);
    }
    
    printf("\n%-22s %12s  %12s  %12s\n", "Type", "Rcvd", "OK", "Error");
    printf("-------------------------------------------------\n");
    for (i = 0; i < FS_REQ_MAX; i++)
    {
        printf("%-22s %12lld  %12lld  %12lld\n", fs_reqTypeName[i], 
               msg->rcvd.num[i], msg->reply.ok[i], msg->reply.err[i]);
        total += msg->rcvd.num[i];
        totalOK += msg->reply.ok[i];
        totalError += msg->reply.err[i];
    }
    printf("\n%-22s %12lld  %12lld  %12lld\n\n", "total", total, totalOK, totalError);

    return NULL;
}

static fs_MsgReqHdr *
cmd_logstats(int argc, char *argv[])
{
    fs_MsgLogstatsReq *msg;
    evm_volId_t volId;
    boolean isVerbose = FALSE;
    boolean isClear = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;

    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-v", argv[carg]) == 0)
        {
            isVerbose = TRUE;
        }
        else if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argumunt, nor volid
            return NULL;
        }
    }

    msg = alloc_msg (FS_REQ_LOGSTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isVerbose = isVerbose;
    msg->isClear = isClear;

    return (fs_MsgReqHdr *)msg;
}

static fs_MsgReqHdr *
logstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgLogstatsResp  *resp = (fs_MsgLogstatsResp *) hdr;
    float               divisor, dividend;

    switch (resp->logDevType)
    {
    case FS_LOG_DEVTYPE_NVRAM:
        printf ("Log Device: NVRAM\n");
        break;
    case FS_LOG_DEVTYPE_SELF:
        printf ("Log Device: SELF\n");
        break;
    case FS_LOG_DEVTYPE_VOLID:
        printf ("Log Device: VOLID\n");
         break;
    case FS_LOG_DEVTYPE_NULL:
        printf ("Log Device: NULL\n");
        break;
    default:
        printf ("Log Device: unknown %d\n", resp->logDevType);
        return NULL;
        break;
    }
    printf ("Log size:  %d\n", resp->logSize);
    printf ("Current head ptr:  %d\n", resp->head);
    printf ("Current length  :  %d\n", resp->currLen);
    printf ("Current tail ptr:  %d\n", resp->tail);
    
    switch (resp->logDevType) 
    {
    case FS_LOG_DEVTYPE_SELF:
    case FS_LOG_DEVTYPE_VOLID:
        printf ("Current number of volume log buffers in use %d (max %d)\n",
                resp->u.vollog.currNbrBuf, resp->u.vollog.maxNbrBuf);
        printf ("Current number of volume log buffers in i/o %d (max %d)\n",
                resp->u.vollog.currNbrIO, resp->u.vollog.maxNbrIO);
		printf ("Total number of volume log buffers written  %d\n",
                resp->u.vollog.totalNbrIO);
        break;
    default:
        break;
    }

    if (resp->isVerbose)
    {
        int nbrEntries = 0;
    
        //         fs_logPrintStats(&resp->logStats, resp->currLen);

        printf ("\n");
        printf ("Nbr of Meta 1 entries          %12d\n", resp->logStats.nbrMetaOne);
        printf ("Nbr of Meta 2 entries          %12d\n", resp->logStats.nbrMetaTwo);
        printf ("Nbr of User   entries          %12d\n", resp->logStats.nbrUser);
        printf ("Nbr of Alloc  entries          %12d\n", resp->logStats.nbrAlloc);
        printf ("Nbr of Zero U entries          %12d\n", resp->logStats.nbrZeroUser);
        printf ("Nbr of Zero M entries          %12d\n", resp->logStats.nbrZeroMeta);
        printf ("Nbr of Remove entries          %12d\n", resp->logStats.nbrUserRm);
        printf ("Nbr of Bitmap entries          %12d\n", resp->logStats.nbrBitmap);
        printf ("Nbr of Inode  entries          %12d\n", resp->logStats.nbrInode);
        printf ("Nbr of InodeFlush  entries     %12d\n", resp->logStats.nbrInodeFlush);
        printf ("Nbr of BufFlush  entries       %12d\n", resp->logStats.nbrBufFlush);
        printf ("Nbr of Commit entries          %12d\n", resp->logStats.nbrCommit);
        printf ("Nbr of Done entries            %12d\n", resp->logStats.nbrMarkDone);
        printf ("Nbr of EOL entries             %12d\n", resp->logStats.nbrEOL);
        printf ("Nbr of extra bitmap creations  %12d\n", 
                                                resp->logStats.nbrExtraBitmapsCreate);
#ifdef LOG_TUNE_STATISTICS
        printf ("\nNbr of fixed inode dep         %12d\n", resp->logStats.nbrFixedInodeDep);
        printf ("Nbr of variable inode dep      %12d\n", 
                                                      resp->logStats.nbrVariableInodeDep);
        printf ("max Nbr of inode dep           %12d\n", resp->logStats.maxInodeDep);
        printf ("min Nbr of inode dep           %12d\n", resp->logStats.minInodeDep);
        printf ("\nNbr of fixed buf dep           %12d\n", resp->logStats.nbrFixedBufDep);
        printf ("Nbr of variable buf dep        %12d\n", resp->logStats.nbrVariableBufDep);
        printf ("max Nbr of buf dep             %12d\n", resp->logStats.maxBufDep);
        printf ("min Nbr of buf dep             %12d\n", resp->logStats.minBufDep);
        printf ("\nNbr of fixed ioDone used       %12d\n", 
                                                        resp->logStats.nbrFixedioDoneUsed);
        printf ("Nbr of variable ioDone used    %12d\n", 
                                                      resp->logStats.nbrVariableioDoneUsed);
        printf ("max Nbr of ioDone used         %12d\n", resp->logStats.ioDoneMax);
        printf ("min Nbr of ioDone used         %12d\n", resp->logStats.ioDoneMin);
        divisor = resp->logStats.ioDoneUsed;
        dividend = resp->logStats.bufUsed;
        if (dividend != 0) {

            printf ("avg Nbr of ioDone used                 %.2f\n", divisor / dividend);

        }
        printf ("\n");
#endif

        nbrEntries += resp->logStats.nbrMetaOne;
        nbrEntries += resp->logStats.nbrMetaTwo;
        nbrEntries += resp->logStats.nbrUser;
        nbrEntries += resp->logStats.nbrAlloc;
        nbrEntries += resp->logStats.nbrZeroUser;
        nbrEntries += resp->logStats.nbrZeroMeta;
        nbrEntries += resp->logStats.nbrUserRm;
        nbrEntries += resp->logStats.nbrBitmap;
        nbrEntries += resp->logStats.nbrInode;
        nbrEntries += resp->logStats.nbrInodeFlush;
        nbrEntries += resp->logStats.nbrBufFlush;
        nbrEntries += resp->logStats.nbrCommit;
        nbrEntries += resp->logStats.nbrMarkDone;
        nbrEntries += resp->logStats.nbrEOL;

        printf ("Total bytes in log             %12lld\n", resp->logStats.logTotalBytes);
        printf ("Total nbr of records           %12d\n", nbrEntries);
        printf ("Total nbr of transactions      %12d\n", resp->logStats.nbrCommit);
        printf ("\n");

        if (resp->logStats.nbrCommit)
        {
            divisor = nbrEntries;
            dividend = resp->logStats.nbrCommit;
            printf ("AVG nbr of entries per transaction   %.2f\n", divisor / dividend);

            divisor = resp->logStats.logTotalBytes;
            dividend = resp->logStats.nbrCommit;
            printf ("AVG nbr of bytes per transaction     %.2f\n", divisor / dividend);
        }
        
        if (nbrEntries) 
        {
            divisor = resp->logStats.logTotalBytes;
            dividend = nbrEntries;
            printf ("AVG nbr of bytes per record          %.2f\n", divisor / dividend);
        }

        printf ("Non full buf log bytes written       %lld\n",  
                resp->logStats.ioPartialBytes);
        printf ("Nbr of non full buf log writes       %d\n",  
                resp->logStats.ioPartialLogWrites);
#ifdef FS_VOLLOG_WAIT_FOR_ONGOING_TRANS
        printf ("Nbr of postponed non full writes     %d\n",  
                resp->logStats.postponedPartialLogWrites);
        printf ("Nbr of forced writes                 %d\n",  
                resp->logStats.forceWriteDone);
#endif
        if (resp->logStats.ioPartialLogWrites) 
        {
            dividend = resp->logStats.ioPartialLogWrites;
            divisor = resp->logStats.ioPartialBytes;
            printf ("Average non full buf log bytes       %.2f\n", divisor/dividend);
        }

        printf ("Full log buf bytes written           %lld\n",  
                resp->logStats.ioTotalBytes);
        printf ("Nbr of full log buf writes           %d\n",  
                resp->logStats.ioTotalLogWrites);

        if (resp->logStats.ioTotalLogWrites) 
        {
            dividend = resp->logStats.ioTotalLogWrites;
            divisor = resp->logStats.ioTotalBytes;
            printf ("Average full log buf bytes           %.2f\n", divisor/dividend);
        }
    }

    return NULL;
}


static fs_MsgReqHdr *
cmd_nfsstats(int argc, char *argv[])
{
    fs_MsgNfsstatsReq *msg;
    evm_volId_t volId;
    boolean isVerbose = FALSE;
    boolean isClear = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;

    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-v", argv[carg]) == 0)
        {
            isVerbose = TRUE;
        }
        else if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argumunt, nor volid
            return NULL;
        }
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif
    msg = alloc_msg(FS_REQ_NFSSTATS, sizeof *msg, nodeName);
    msg->volId = volId; 
    msg->isVerbose = isVerbose;
    msg->isClear = isClear;

    return (fs_MsgReqHdr *)msg;
}


static fs_MsgReqHdr *
nfsstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgNfsstatsResp  *msg = (fs_MsgNfsstatsResp *) hdr;
    uint64              total[FS_REQ_NFS_MAX];
    uint64              average[FS_REQ_NFS_MAX];
    uint64              pctOps[FS_REQ_NFS_MAX];
    uint64              pctTime[FS_REQ_NFS_MAX];
    uint64              totalTime[FS_REQ_NFS_MAX];
    uint64              totalTimeAvg[FS_REQ_NFS_MAX];
    uint64              totalAll, totalTimeAll, totalCpuAll, overall, numTypesDone = 0;
    int32               i, j; 

#define CEILING_DIV_64(_num, _den)  ((_num + _den - 1) / _den)

    bzero(total, sizeof total);
    bzero(average, sizeof average);
    bzero(totalTime, sizeof totalTime);
    bzero(totalTimeAvg, sizeof totalTimeAvg);
    bzero(pctOps, sizeof pctOps);
    bzero(pctTime, sizeof pctTime);
    totalAll = 0;
    totalTimeAll = 0;
    totalCpuAll = 0 ;

	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = msg->count_per_usec;

	// Calculate totals.
    for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        total[i] = msg->recvNfsReqStats.num[i];
        totalAll += total[i];
		totalTime[i] = eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TOTAL]));
        totalTimeAll += totalTime[i];
        totalCpuAll += eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ON_CPU]));
        if (total[i])
        {
            average[i] = totalTime[i] / total[i];
        }
    }

    // Calculate percent over count
    for (i = 0; totalAll && i < FS_REQ_NFS_MAX; i++)
    {
        pctOps[i] = total[i] * 100 / totalAll;
    }

    // Calculate percent of time.
    for (i = 0; totalTimeAll && i < FS_REQ_NFS_MAX; i++)
    {
        pctTime[i] = totalTime[i] * 100 / totalTimeAll;
    }

    // Calculate overall response time.
    for (i = 0, overall = 0; i < FS_REQ_NFS_MAX; i++)
    {
        overall += average[i];
        if (total[i])
            numTypesDone += 1;
    }
    if (numTypesDone)
    {
        overall = overall / numTypesDone;
    }


#ifdef FS_DEBUG_NFS_DELTA_BUCKET
    // Calculate average times in usecs.
    for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        int32 cnt;
        cnt = 0;
        for (j = 0; j < FS_NFS_TIME_MAX; j++)
        {
            // average out the inflation
            int32 aveUsec;
            if (j == 0)
                aveUsec = fs_nfsStatsUsecs[j];
            else
                aveUsec = (fs_nfsStatsUsecs[j-1] + fs_nfsStatsUsecs[j]) / 2;
            cnt += msg->nfsStats.num[i][j];
            totalTimeAvg[i] += msg->nfsStats.num[i][j] * aveUsec;
        }
        if (cnt)
            totalTimeAvg[i] /= cnt;
    }
#endif  // FS_DEBUG_NFS_DELTA_BUCKET

    printf("Type              Count       %%Ops      %%Time    AvgUsec    Errors\n");
    printf("------------------------------------------------------------------\n");
    for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        printf("%-12s %10lld %10lld %10lld %10lld %8d\n",
               fs_reqTypeName[i],
               total[i],
               pctOps[i],
               pctTime[i],
               average[i],
               msg->nfsStats.errs[i]);
    }

    printf("\n");
    printf("Total ops:                   %lld\n", totalAll);
    // printf("Overall response time: %lld usecs\n", overall);
    if (totalAll)
    {
        printf("Average response time:       %lld usecs\n", totalTimeAll / totalAll);
        printf("Average cpu usage:           %lld usecs\n", totalCpuAll / totalAll);
    }
     
    if (msg->isVerbose)
    {
        printf("Current active requests: %d\n", msg->currentActiveReqs);

#ifdef FS_DEBUG_NFS_DELTA_BUCKET

// see also FS_NFS_TIME_MAX
#define FS_DUMP_NFS_STATS_PER_LINE     7
        printf("\n");
        printf("Type         ");
        for (j = 0 ; j < FS_DUMP_NFS_STATS_PER_LINE ; j++)
        {
            printf(" %10dus", fs_nfsStatsUsecs[j]);
        }
        printf("\n");
        for (i = 0; i < FS_REQ_NFS_MAX; i++)
        {
            printf("%-12s ", fs_reqTypeName[i]);
            for (j = 0 ; j < FS_DUMP_NFS_STATS_PER_LINE ; j++)
            {
                printf(" %12lld", msg->nfsStats.num[i][j]);
            }
            printf("\n");
        }
        printf("\n");
        printf("Type         ");
        for (j = FS_DUMP_NFS_STATS_PER_LINE ; j < FS_NFS_TIME_MAX ; j++)
        {
            printf(" %10dus", fs_nfsStatsUsecs[j]);
        }
        printf("\n");
        for (i = 0; i < FS_REQ_NFS_MAX; i++)
        {
            printf("%-12s ", fs_reqTypeName[i]);
            for (j = FS_DUMP_NFS_STATS_PER_LINE ; j < FS_NFS_TIME_MAX ; j++)
            {
                printf(" %12lld", msg->nfsStats.num[i][j]);
            }
            printf("\n");
        }
#endif
    }
    printf("\n");

    return NULL;
}

static fs_MsgReqHdr *
cmd_errstats(int argc, char *argv[])
{
    fs_MsgErrstatsReq *msg;
    boolean isClear = FALSE;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;
    FS_ASSERT(argc == 1 || argc == 2 || argc == 3);
    if (argc == 2)
    {
        if (!strcmp("-z", argv[1]))
            isClear = TRUE;
        else if (!fs_shell_get_volid(argv[1], &volId, nodeName))
            return NULL;    
    }
    else if (argc == 3)
    {
        if (!strcmp("-z", argv[1]))
            isClear = TRUE;
        if (!fs_shell_get_volid(argv[2], &volId, nodeName))
            return NULL;    
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif

    msg = alloc_msg(FS_REQ_ERRSTATS, sizeof *msg, nodeName);
    msg->volId = volId; 
    msg->isClear = isClear;

    return (fs_MsgReqHdr *)msg;
}

static fs_MsgReqHdr *
errstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgErrstatsResp  *msg = (fs_MsgErrstatsResp *) hdr;
    int32 i, total = 0;

    printf("Type                             Count\n");
    printf("--------------------------------------\n");
    printf("No log resource               %8d\n", msg->errStats.resource[FS_RESOURCE_LOG]);
    printf("No user buffer resource       %8d\n", msg->errStats.resource[FS_RESOURCE_USERBUFCACHE]);
    printf("No meta 8knondir resource     %8d\n", msg->errStats.resource[FS_RESOURCE_METABUFCACHE_NONDIR]);
    printf("No meta 1knondir resource     %8d\n", msg->errStats.resource[FS_RESOURCE_METABUFCACHE_COMP_NONDIR]);
    printf("No meta 8kdir resource        %8d\n", msg->errStats.resource[FS_RESOURCE_METABUFCACHE_DIR]);
    printf("No meta 1kdir resource        %8d\n", msg->errStats.resource[FS_RESOURCE_METABUFCACHE_COMP_DIR]);
    printf("No inode cache resource       %8d\n", msg->errStats.resource[FS_RESOURCE_INODECACHE]);
    printf("No memory resource            %8d\n", msg->errStats.resource[FS_RESOURCE_MEM]);
    printf("No sac buffer resource        %8d\n", msg->errStats.resource[FS_RESOURCE_SAC]);
    printf("No ultra buffer resource      %8d\n", msg->errStats.resource[FS_RESOURCE_ULTRABUF]);
    printf("No vol log Buffer resource    %8d\n", msg->errStats.resource[FS_RESOURCE_VOL_LOG]);
    printf("No esm stack                  %8d\n", msg->errStats.resource[FS_RESOURCE_NO_ESM]);
    printf("No IO resource                %8d\n", msg->errStats.resource[FS_RESOURCE_IO]);
    for (i = 0; i < FS_RESOURCE_TYPE_MAX; i++)
    {
        total += msg->errStats.resource[i];
    }
    printf("\nTotal: %d\n", total);

    return NULL;
}

static fs_MsgReqHdr *
cmd_yieldstats(int argc, char *argv[])
{
    fs_MsgYieldstatsReq *msg;
    evm_volId_t volId;
    boolean isClear = FALSE;
    boolean isAverage = TRUE;
    boolean showIO = FALSE;
    boolean showRestore = FALSE;
    boolean showDaemon = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;

    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-t", argv[carg]) == 0)
        {
            isAverage = FALSE;
        }
        else if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (strcmp("-i", argv[carg]) == 0)
        {
            showIO = TRUE;
        }
        else if (strcmp("-d", argv[carg]) == 0)
        {
            showDaemon = TRUE;
        }
        else if (strcmp("-r", argv[carg]) == 0)
        {
            showRestore = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argument, nor volid
            return NULL;
        }
    }
#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif

    msg = alloc_msg(FS_REQ_YIELDSTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isClear = isClear;
    msg->isAverage = isAverage;
    msg->showIO = showIO;
    msg->showDaemon = showDaemon;
    msg->showRestore = showRestore;
    
    return (fs_MsgReqHdr *)msg;
}

static fs_MsgReqHdr *
yieldstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgYieldstatsResp    *msg = (fs_MsgYieldstatsResp *) hdr;
    int32                   i, j;
    uint64                  total = 0;
	eeeRTC_t				totalTimeAll, totalCpuAll, sum;

	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = msg->count_per_usec;

	totalTimeAll = *(eee_rtcZero());
	totalCpuAll = *(eee_rtcZero());
	for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        total += msg->recvNfsReqStats.num[i];
		(void)eee_rtcIncr(&totalTimeAll,
						  &(msg->yieldStats.num[i][FS_YIELD_TYPE_TOTAL]));
		(void)eee_rtcIncr(&totalCpuAll,
						  &(msg->yieldStats.num[i][FS_YIELD_TYPE_ON_CPU]));
    }

    if (msg->isAverage) 
    {
        for (i = 0; i < FS_REQ_NFS_MAX; i++)
        {
            if (msg->recvNfsReqStats.num[i])
            {
                for (j = 0; j < FS_YIELD_TYPE_MAX; j++)
                {
					eee_rtcDiv(&(msg->yieldStats.num[i][j]),
							   msg->recvNfsReqStats.num[i]);
                }
            }
        }
    }

    printf("ReqType           count    cpuUsec     ioUsec   lockUsec    admUsec    elapsed\n");
    printf("------------------------------------------------------------------------------\n");
    for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        printf("%-12s %10lld %10lld %10lld %10lld %10lld %10lld\n",
               fs_reqTypeName[i],
               msg->recvNfsReqStats.num[i],
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ON_CPU])),
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UR])) +
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UW])) +
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MR])) +
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MW])) +
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_COMMIT])) +
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_LOG])),
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_LOCK])),
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ADMIN])),
               eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TOTAL])));
    }

    printf("\n");

    if (msg->showIO)
    {
        printf("ReqType           count  UserRead UserWrite  MetaRead MetaWrite       Log\n");
        printf("-----------------------------------------------------------------------------------\n");
        for (i = 0; i < FS_REQ_NFS_MAX; i++)
        {
            printf("%-12s %10lld %9lld %9lld %9lld %9lld %9lld\n",
                   fs_reqTypeName[i],
                   msg->recvNfsReqStats.num[i],
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UR])),
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UW])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_COMMIT])),
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MR])),
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MW])),
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_LOG])));
        }
        printf("\n");

    }
    
    for (i = 0; i < FS_REQ_NFS_MAX; i++)
    {
        if (eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TXCOMPL])))
        {
            printf("%-12s %10lld    Txcomplete:  %10lld Usec\n",
                   fs_reqTypeName[i],
                   msg->recvNfsReqStats.num[i],
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TXCOMPL])));
            printf("\n");
        }
        if (eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_COD])))
        {
            printf("%-12s %10lld    Rx COD:      %10lld Usec\n",
                   fs_reqTypeName[i],
                   msg->recvNfsReqStats.num[i],
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_COD])));
            printf("\n");
        }
        if (eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])))
        {
            printf("%-12s %10lld    Sleep:       %10lld Usec\n",
                   fs_reqTypeName[i],
                   msg->recvNfsReqStats.num[i],
                   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])));
            printf("\n");
        }
    }


    printf("Total ops:                   %lld\n", total);
    if (total)
    {
        printf("Average response time:       %lld usecs\n",
			   eee_rtcToUsec(&totalTimeAll) / total);
        printf("Average cpu usage:           %lld usecs\n",
			   eee_rtcToUsec(&totalCpuAll) / total);
    }
    printf("Cur/Max active requests:     %d/%d\n",
            msg->currentActiveReqs, msg->maxActiveReqs);
#ifdef FS_VOLLOG_WAIT_FOR_ONGOING_TRANS
    if (msg->volId.whole_thing != 0) 
    {
        printf("Cur/Max log transactions:    %d/%d\n",
               msg->nbrOngoingTrans, msg->maxOngoingTrans);
    }
#endif
    printf("Max contexts:                %d\n",
            msg->maxNbrContexts);
    printf("Total bytes Tx over hyper channel to TXRX: %lld\n",
		   msg->ht_bytes_tx);
	printf("Total bytes Rx over hyper channel from TXRX: %lld\n",
		   msg->ht_bytes_rx);

    if (msg->showRestore)
    {
        printf("\n");

        printf("Dump/Restore Statistics (total times in usec since last reset)\n");
        for (i = FS_REQ_NFS_MAX; i < FS_REQ_NFS_MAX + 2; i++)
        {
            if (eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])))
            {
                printf("%-12s    Sleep:       %10lld Usec\n",
                       i == FS_REQ_NFS_MAX ? "Dump": "Restore",
                       eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])));
                printf("\n");
            }
        }

        printf("\n");
//              123456789012 123456789012 123456789012 123456789012 123456789012 123456789012 123456789012 123456789012 123456789012
        printf("ReqType           cpuUsec       ioUsec     lockUsec      admUsec    tape read   tape write      elapsed\n");
        printf("-------------------------------------------------------------------------------------------------------\n");
        for (i = FS_REQ_NFS_MAX; i < FS_REQ_NFS_MAX + 2; i++) {
            printf("%-12s %12lld %12lld %12lld %12lld %12lld %12lld %12lld\n",
                   i == FS_REQ_NFS_MAX ? "Dump": "Restore",
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ON_CPU])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UR])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UW])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MR])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MW])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_COMMIT])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_LOG])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_LOCK])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ADMIN])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TR])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TW])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TOTAL])));
        }
    }
    
    if (msg->showDaemon)
    {
        printf("\n");
        printf("Thread Statistics (total times in usec since last reset)\n");
        printf("ReqType           cpuUsec       ioUsec     lockUsec      admUsec    tape read   tape write      elapsed\n");
        printf("-------------------------------------------------------------------------------------------------------\n");
        for (i = FS_REQ_NFS_MAX + 2; i < FS_REQ_NFS_MAX + 2 + FS_DAEMON_TYPE_MAX; i++) 
        {
            printf("%-12s %12llu %12lld %12lld %12lld %12lld %12lld %12lld\n",
                   fs_daemonTypeTab[i - 2 - FS_REQ_NFS_MAX].name,
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ON_CPU])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UR])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UW])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MR])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MW])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_COMMIT])) +
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_LOG])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_LOCK])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_ADMIN])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TR])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TW])),
				   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_TOTAL])));
        }
        for (i = FS_REQ_NFS_MAX + 2; i < FS_REQ_NFS_MAX + 2 + FS_DAEMON_TYPE_MAX; i++)
        {
            if (eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])))
            {
                printf("%-12s    Sleep:       %12lld Usec\n",
                       fs_daemonTypeTab[i - 2 - FS_REQ_NFS_MAX].name,
                       eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_SLEEP])));
            }
        }
        if (msg->showIO)
        {
            printf("\n");
            printf("ReqType          UserRead    UserWrite     MetaRead    MetaWrite     TapeRead    TapeWrite          Log\n");
            printf("--------------------------------------------------------------------------------------------------------------------\n");
            for (i = FS_REQ_NFS_MAX + 2; i < FS_REQ_NFS_MAX + 2 +
                     FS_DAEMON_TYPE_MAX; i++)
            {
                printf("%-12s %12lld %12lld %12lld %12lld %12lld %12lld %12lld\n",
                       fs_daemonTypeTab[i - 2 - FS_REQ_NFS_MAX].name,
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UR])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_UW])) +
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_COMMIT])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MR])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_MW])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TR])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_TW])),
					   eee_rtcToUsec(&(msg->yieldStats.num[i][FS_YIELD_TYPE_IO_LOG])));
            }
        }
    }
    
    return NULL;
}

static fs_MsgReqHdr *
cmd_lockstats(int argc, char *argv[])
{
    fs_MsgLockstatsReq *msg;
    evm_volId_t volId;
    boolean isClear = FALSE;
    boolean isAverage = FALSE;
    int32 rc, carg, val, lockReqType;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif
    volId.whole_thing = 0;
    lockReqType = 0;

    for (carg = 1; carg < argc; carg++)
    {
        if (!strcmp("-z", argv[carg]))
        {
            isClear = TRUE;
        }
        else if (!strcmp("-a", argv[carg]))
        {
            isAverage = TRUE;
        }
        else if (!strcmp("-b", argv[carg]))
        {
            lockReqType = FS_LOCK_OWNER_MAX + 1;
        }
        else if (!strcmp("-r", argv[carg]))
        {
            if (argc <= carg + 1)
            {
                // Force parse_parameter to issue usage.
                carg -= 1;
            }

            rc = shell_parse_parameter(fs_requestsTab, argv[carg + 1],
                                       tablist, &val);
            if (rc != NFX_OK)
            {
                printf("Usage: lockstats -r <%s>\n", tablist);
                return NULL;
            }
            lockReqType = val;
            carg += 1;
        }
        else if (!strcmp("-d", argv[carg]))
        {
            if (argc <= carg + 1)
            {
                // Force parse_parameter to issue usage.
                carg -= 1;
            }

            rc = shell_parse_parameter(fs_daemonTypeTab, argv[carg + 1],
                                       tablist, &val);
            if (rc != NFX_OK)
            {
                printf("Usage: lockstats -d <%s>\n", tablist);
                return NULL;
            }
            lockReqType = val + FS_LOCK_DAEMON_OFFSET;
            carg += 1;
        }
        else
        {
            if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
                return NULL;
        }
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif

    msg = alloc_msg(FS_REQ_LOCKSTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isClear = isClear;
    msg->isAverage = isAverage;
    msg->lockReqType = lockReqType;

    return (fs_MsgReqHdr *)msg;
}

char lockOwnerUnknown[] = "Unkown";
char lockOwnerShared[]  = "Shared";


static char *
fs_lockstats_get_req_type_name(int32 type)
{
    if (type == FS_LOCK_OWNER_SHARED)
    {
        return lockOwnerShared;
    }
    else
    if (type == FS_LOCK_OWNER_UNK)
    {
        return lockOwnerUnknown;
    }
    else
    if (type >= FS_LOCK_DAEMON_OFFSET)
    {
        FS_ASSERT(type - FS_LOCK_DAEMON_OFFSET == 
            fs_daemonTypeTab[type - FS_LOCK_DAEMON_OFFSET].flag);  
        return fs_daemonTypeTab[type - FS_LOCK_DAEMON_OFFSET].name;
    }
    else 
    {
        FS_ASSERT(type == fs_requestsTab[type].flag);  
        return fs_requestsTab[type].name;
    }
}

static fs_MsgReqHdr *
lockstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgLockstatsResp     *msg = (void *) hdr;
    int32                   i, j;
    
	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = msg->count_per_usec;

    printf("\n");
    if (msg->volId.whole_thing != 0)
    {
        printf("LockType         usecs\n");
        printf("----------------------\n");
        for (i = 0; i < FS_LOCKSTATS_MAX; i++)
        {
            if (fs_lockStatsTypeName[i] != NULL)
            {
                printf("%-12s %10lld\n",
                    fs_lockStatsTypeName[i],
                    eee_rtcToUsec(&(msg->u.waitTime[i])));
            }
        }
    }
    else if (msg->lockReqType < FS_LOCK_OWNER_MAX)
    {
        int32 lockType, ownerType;
        char *reqTypeName;

        reqTypeName = fs_lockstats_get_req_type_name(msg->lockReqType);

        printf("Lock wait stats for %s requests:\n", reqTypeName);

        for (lockType = 0; lockType < FS_LOCK_TYPE_MAX; lockType++)
        {
            for (ownerType = 0; ownerType < FS_LOCK_OWNER_MAX; ownerType++)
            {
                if (msg->u.waitStats[lockType][ownerType])
                {
                    printf("  %-12s   %-12s  %12d\n",
                           fs_lockTypeTab[lockType].name,
                           fs_lockstats_get_req_type_name(ownerType),
                           msg->u.waitStats[lockType][ownerType]);
                }
            }
        }
    }
    else
    {
        int32 inum;

        printf("Lock wait stats for meta bufs page:\n");
        for (inum = 0; inum < FS_MAX_META_INODE; inum++)
        {
            if (msg->u.lockStatBuf.metaBufInumPage[inum] != 0)
            {
                printf("  inum %2d     %12d\n", 
                       inum,
                       msg->u.lockStatBuf.metaBufInumPage[inum]);
            }
        }
        printf("Lock wait stats for meta bufs other:\n");
        for (inum = 0; inum < FS_MAX_META_INODE; inum++)
        {
            if (msg->u.lockStatBuf.metaBufInumOther[inum] != 0)
            {
                printf("  inum %2d     %12d\n", 
                       inum,
                       msg->u.lockStatBuf.metaBufInumOther[inum]);
            }
        }
        if (msg->u.lockStatBuf.userBufInumPage > 0)
        {
            printf("\n");
            printf("Lock wait stats for user bufs page:   %d\n",
                msg->u.lockStatBuf.userBufInumPage);
        }
        if (msg->u.lockStatBuf.userBufInumOther > 0)
        {
            printf("\n");
            printf("Lock wait stats for user bufs other:   %d\n",
                msg->u.lockStatBuf.userBufInumOther);
        }
    }
    printf("\n");
    printf("Max shared lock list size:             %d\n", msg->maxSharedLockListSize);
    printf("\n");

    return NULL;
}



static fs_MsgReqHdr *
cmd_logdata(int argc, char *argv[])
{
    fs_MsgLogdataReq *msg;
    evm_volId_t volId;
    int32 nbrEntries = 1;
    int32 offset = 0;
	int32 criteria = 0;
	uint64 value = 0;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc >= 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;
    if (argc > 2)
        nbrEntries = atoi(argv[2]);
    if (argc > 3)
        offset = atoi(argv[3]);
	if (argc > 4)
	{
		criteria = atoi(argv[4]);
    }
	if (argc > 5)
	{
		value = atol(argv[5]);
	}

    msg = alloc_msg(FS_REQ_LOGDATA, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->nbrEntries = nbrEntries;
    msg->offset = offset;
    msg->criteria = criteria;
    msg->value = value;

    return (fs_MsgReqHdr *)msg;
}
static fs_MsgReqHdr *
logdata_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgLogdataResp  *msg = (fs_MsgLogdataResp *) hdr;

#ifndef FS_STANDALONE
    printf("%s\n", msg->data);
#endif
    return NULL;
}


static fs_MsgReqHdr *
cmd_logtest(int argc, char *argv[])
{
    char *usage = "logtest: <volname> <crash | replay | crash-now> <n> <crashPhase>";
    fs_MsgLogtestReq *msg;
    evm_volId_t volId;
    int32       type;
    int32       crashPhase = 0;
    int32       n = 0;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc < 2)
    {
        printf("%s\n", usage);
        return NULL;
    }
    if (strcmp(argv[2], "crash") == 0)
        type = FS_LOG_TEST_FORCE_CRASH;
    else if (strcmp(argv[2], "replay") == 0)
        type = FS_LOG_TEST_REPLAY;
    else if (strcmp(argv[2], "crash-now") == 0)
        type = FS_LOG_TEST_CRASH_NOW;
    else {
        printf("%s\n", usage);
        return NULL;
    }

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    if (type != FS_LOG_TEST_CRASH_NOW)
    {
        n = atoi(argv[3]);
        crashPhase = atoi(argv[4]);
    }
    msg = alloc_msg(FS_REQ_LOG_TEST, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->type = type;
    msg->crashPhase = crashPhase;
    msg->n = n;

    printf("logtest: run %d tests with crashPhase %d\n",
            n, crashPhase);

    return (fs_MsgReqHdr *) msg;
}


static fs_MsgReqHdr *
cmd_crash(int argc, char *argv[])
{
    fs_MsgCrashReq *msg;
    evm_volId_t volId;
    int32 type = 1;
    int32 count = 1;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    FS_ASSERT(argc >= 2);
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;
    if (argc > 2)
        type = atoi(argv[2]);
    if (argc > 3)
        count = atoi(argv[3]);

    msg = alloc_msg(FS_REQ_CRASH, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->type = type;
    msg->count = count;

    return (fs_MsgReqHdr *)msg;
}


/**
 * Cause the specified volume to be aborted.
 */
static fs_MsgReqHdr *
cmd_abort(int argc, char *argv[])
{
    fs_MsgAbortReq *msg;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    msg = alloc_msg(FS_REQ_ABORT, sizeof *msg, nodeName);
    msg->volId = volId;

    return (fs_MsgReqHdr *)msg;
}


static fs_MsgReqHdr *
cmd_volexer(int argc, char *argv[])
{
    fs_MsgVolExerReq *msg;
    evm_volId_t volId;
    uint64 numVolBlks, numBlks, numBlksSequential = 2048;
    uint32 load = 100;
    uint32 seed = 0;
    uint32 what, sequentialIoSize = 8; // this many FS_BLOCKs
    uint32 loadType = FS_VOL_EXER_TYPE_BOTH;
    uint32 ioType = FS_VOL_EXER_IOTYPE_RW;
    uint64 attrib = 0;
    boolean isUser = FALSE;
#ifndef FS_STANDALONE
    ncmMsg_t  *ncmReq;
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc < 3)
    {
        printf(
    "Usage: volexer <start|stop|stat> <volname> [user|meta rand|seq|both rw|ro|wo lp|nl nr|re <load> <seed>\n"
    "               <numblks>] [<numBlksSequential> <sequentialIoSize>]\n");
        return NULL;
    }

    if (!strcmp(argv[1], "start"))
        what = FS_VOL_EXER_START;
    else if (!strcmp(argv[1], "stop"))
        what = FS_VOL_EXER_STOP;
    else if (!strcmp(argv[1], "stat"))
        what = FS_VOL_EXER_STAT;
    else {
        printf("unknown exer req %s\n", argv[1]);
        return NULL;
    }

    if (!fs_shell_get_volid(argv[2], &volId, NULL))
        return NULL;
    if (!fs_shell_is_vol_online(volId, nodeName))
        return NULL;

    numVolBlks = get_mkfs_numblks(volId);
#if defined(FS_STANDALONE)
    numBlks = MKFS_DEFAULT;
#else
    numBlks = numVolBlks;
#endif

    if (argc > 3)
    {
        if (!strcmp(argv[3], "user"))
            isUser = TRUE;
        else if (!strcmp(argv[3], "meta"))
            isUser = FALSE;
        else {
            printf("unknown data type %s\n", argv[3]);
            return NULL;
        }
    }

    if (argc > 4)
    {
        if (!strcmp(argv[4], "rand"))
            loadType = FS_VOL_EXER_TYPE_RANDOM;
        else if (!strcmp(argv[4], "seq"))
            loadType = FS_VOL_EXER_TYPE_SEQUENTIAL;
        else if (!strcmp(argv[4], "both"))
            loadType = FS_VOL_EXER_TYPE_BOTH;
        else {
            printf("unknown loadType %s\n", argv[4]);
            return NULL;
        }
    }

    if (argc > 5)
    {
        if (!strcmp(argv[5], "rw"))
            ioType = FS_VOL_EXER_IOTYPE_RW;
        else if (!strcmp(argv[5], "ro"))
            ioType = FS_VOL_EXER_IOTYPE_RO;
        else if (!strcmp(argv[5], "wo"))
            ioType = FS_VOL_EXER_IOTYPE_WO;
        else {
            printf("unknown ioType %s\n", argv[5]);
            return NULL;
        }
    }

    if (argc > 6) {

            if (!strcmp(argv[6], "lp")) {

                attrib |= SCSI_MSG_ATTR_LOOPBACK;

            }

    }
    if (argc > 7) {

        if (!strcmp(argv[7], "nr")) {

            attrib |= SCSI_MSG_ATTR_NO_REORDER;

        }

    }

    if (argc > 8)
        load = atoi(argv[8]);

    if (argc > 9)
        seed = atoi(argv[9]);

    if (argc > 10) {
        numBlks = strtoull(argv[10], NULL, 10);
        if (numBlks > numVolBlks) {
            printf("numBlks %lld greater than volume %lld\n",
                numBlks, numVolBlks);
            return NULL;
        }
    }

    if (loadType == FS_VOL_EXER_TYPE_SEQUENTIAL)
        numBlksSequential = numBlks;
    if (argc > 11) {
        numBlksSequential = strtoull(argv[11], NULL, 10);
    }

    if (argc > 12) {
        sequentialIoSize = strtol(argv[12], NULL, 10);
        if (sequentialIoSize > 64)
        {
            printf("max sequentialIoSize is %d blks\n",
                64);
            return NULL;
        }
    }


    if (numBlks < MKFS_LO || numBlks > MKFS_HI)
    {
        printf("numBlks %lld out of range (%lld .. %lld)\n",
               numBlks, MKFS_LO, MKFS_HI);
        return NULL;
    }

    if (numBlks < numBlksSequential)
    {
        printf("numBlks %lld less than numBlksSequential %lld\n",
               numBlks, numBlksSequential);
        return NULL;
    }

    if ((loadType == FS_VOL_EXER_TYPE_BOTH ||
         loadType == FS_VOL_EXER_TYPE_RANDOM) &&
        numBlks <= numBlksSequential)
    {
        printf("numBlks %lld numBlksSequential %lld, no space for RANDOM\n",
               numBlks, numBlksSequential);
        return NULL;
    }

    msg = alloc_msg(FS_REQ_VOLEXER, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->numBlks = numBlks;
    msg->what = what;
    msg->load = load;
    msg->seed = seed;
    msg->loadType = loadType;
    msg->ioType = ioType;
    msg->numBlksSequential = numBlksSequential;
    msg->sequentialIoSize = sequentialIoSize;
    msg->attrib = attrib;
    msg->isUser = isUser;
#ifndef FS_STANDALONE
    ncmReq = (ncmMsg_t *)CONTAINING_RECORD(msg, ncmMsg_t, appMsg);
    ncmReq->msgType = NCM_SEND_EMB;
#endif

    return (fs_MsgReqHdr *)msg;
}


static char *fs_volexerLoadTypeStr[] = { "unknown", "rand", "seq", "both" };
static char *fs_volexerIoTypeStr[] = { "unknown", "rw", "ro", "wo" };

static fs_MsgReqHdr *
volexer_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgVolExerResp *resp = (void *) hdr;
    uint64 bytesTransferred;
    int32 i;

	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = resp->count_per_usec;

    switch(resp->what)
    {
    case FS_VOL_EXER_START:
        if (hdr->fsStatus == NFX_OK)
        {
            printf("volume exerciser started on 0x%llx\n",
                   resp->volId.whole_thing);
        }
        else
        {
            printf("volume exerciser failed to start on 0x%llx\n",
                   resp->volId.whole_thing);
        }
        break;
    case FS_VOL_EXER_STOP:
        if (hdr->fsStatus == NFX_OK)
        {
            printf("volume exerciser signaled to stop on 0x%llx\n",
                   resp->volId.whole_thing);
        }
        else
        {
            printf("volume exerciser failed to stop on 0x%llx\n",
                   resp->volId.whole_thing);
        }
        // FALLTHROUGH
    case FS_VOL_EXER_STAT:
        if (hdr->fsStatus == NFX_OK)
        {
            printf("\nVolume Exerciser Stats on vol 0x%llx:\n\n",
                   resp->volId.whole_thing);
            printf("   dataType:  %s\n", resp->isUser ? "user" : "meta");
            printf("       load:  %d\n", resp->load);
            printf("       seed:  %d\n", resp->seed);
            printf("   loadType:  %s\n", fs_volexerLoadTypeStr[resp->loadType]);
            printf("     ioType:  %s\n", fs_volexerIoTypeStr[resp->ioType]);
            printf("    numBlks:  %lld\n", resp->numBlks);
            printf(" numBlksSeq:  %lld\n", resp->numBlksSequential);
            printf("  seqIoSize:  %d\n", resp->sequentialIoSize);
            printf("   readPend:  %d\n", resp->readPending);
            printf("  writePend:  %d\n", resp->writePending);
            printf("   readDone:  %d\n", resp->readDone);
            printf("  writeDone:  %d\n", resp->writeDone);
            printf(" genNumErrs:  %d\n", resp->genNumErrors);
            printf("   dataErrs:  %d\n", resp->dataErrors);
#ifndef FS_STANDALONE
            /*
             * timing values not useful on standalone.
            */
            printf("\n");
            printf("    ***** evm Read io response time distribution *****\n");

            bytesTransferred = 0;
            for (i = 0; i < FS_TIME_STATS_BUCKETS_MAX; i++) {

                printf ("      Less than %lld us, %d responses\n",
                                fs_volexer_stats_usec[i], resp->respTimeRead[i]);
                bytesTransferred += resp->respTimeRead[i];

            }
            bytesTransferred *= 8192;

            if ((eee_rtcCompare(&resp->runTime, eee_rtcZero()) != 0) &&
                (eee_rtcCompare(&resp->totalRespTimeRead, eee_rtcZero()) != 0))
            {
                /*
                 *  Bytes/usec <=> MB/Sec.
                 */
                printf("\n    total io performed = %lld bytes\n",
                    bytesTransferred);
                printf("    total response time = %lld usecs\n",
                    eee_rtcToUsec(&resp->totalRespTimeRead));
                printf("    average throughput = %lld MB/Sec\n",
                    bytesTransferred / eee_rtcToUsec(&resp->runTime));
                printf("    average latency = %lld usecs\n",
                    eee_rtcToUsec(&resp->totalRespTimeRead) / (bytesTransferred / 8192));
            }

            printf("\n");
            printf("            Max response time:  %lld usec\n",
				   eee_rtcToUsec(&resp->respTimeMaxRead));
            printf("            Min response time:  %lld usec\n",
				   eee_rtcToUsec(&resp->respTimeMinRead));
            printf("            Max no ios pending: %d\n", resp->ioQDepthMaxRead);
            printf("            Min no ios pending: %d\n", resp->ioQDepthMinRead);
            printf("    Max completion time in esm: %lld usec\n",
				   eee_rtcToUsec(&resp->esmTimeMaxRead));
            printf("    Min completion time in esm: %lld usec\n",
				   eee_rtcToUsec(&resp->esmTimeMinRead));

            printf("\n");
            printf("    ***** evm Write io response time distribution *****\n");

            bytesTransferred = 0;
            for (i = 0; i < FS_TIME_STATS_BUCKETS_MAX; i++) {

                printf ("      Less than %lld us, %d responses\n",
                                fs_volexer_stats_usec[i], resp->respTimeWrite[i]);
                bytesTransferred += resp->respTimeWrite[i];

            }
            bytesTransferred *= 8192;

			if ((eee_rtcCompare(&resp->runTime, eee_rtcZero()) != 0) &&
                (eee_rtcCompare(&resp->totalRespTimeWrite, eee_rtcZero()) != 0))
            {
                /*
                 *  Bytes/usec <=> MB/Sec.
                 */
                printf("\n    total io performed = %lld bytes\n",
                    bytesTransferred);
                printf("    total response time = %lld usecs\n",
                    eee_rtcToUsec(&resp->totalRespTimeWrite));
                printf("    average throughput = %lld MB/Sec\n",
                    bytesTransferred / eee_rtcToUsec(&resp->runTime));
                printf("    average latency = %lld usecs\n",
                    eee_rtcToUsec(&resp->totalRespTimeWrite) / (bytesTransferred / 8192));
            }

            printf("\n");
            printf("            Max response time:  %lld usec\n",
				   eee_rtcToUsec(&resp->respTimeMaxWrite));
            printf("            Min response time:  %lld usec\n",
				   eee_rtcToUsec(&resp->respTimeMinWrite));
            printf("            Max no ios pending: %d\n", resp->ioQDepthMaxWrite);
            printf("            Min no ios pending: %d\n", resp->ioQDepthMinWrite);
            printf("    Max completion time in esm: %lld usec\n",
				   eee_rtcToUsec(&resp->esmTimeMaxWrite));
            printf("    Min completion time in esm: %lld usec\n",
				   eee_rtcToUsec(&resp->esmTimeMinWrite));
#endif
        }
        else
        {
            printf("volume exerciser failed to find stats on 0x%llx\n",
                   resp->volId.whole_thing);
        }
        break;
    default:
        printf("unknown response type %d\n", resp->what);
        break;
    }

    printf("\n");

	return NULL;
}

/*------------------------------------------------------------------------------
 * Fragmentation analysis tools.
 *------------------------------------------------------------------------------
 */

typedef struct
{
    evm_volId_t volId;
    char        objName[FS_PATH_MAX];
    int32       snapnum;
    boolean     print;
#ifndef FS_STANDALONE
    char        nodeName[LNM_MAX_NODE_NAME];
#else
    char        *nodeName;
#endif
} fs_cb_t;

/* Allocation statistics.
 */

static fs_MsgReqHdr *
cmd_allocStats(int argc, char *argv[])
{
    fs_cb_t *           cb = NULL;
    uint32              reqSize;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    NFX_ASSERT((3 <= argc) && (argc <= 5));
    
    /* Initalize the control block;
     */
    cb = (fs_cb_t *) fs_ramAlloc(FS_RAM_REQ_MSG, sizeof(fs_cb_t));
    bzero((void *) cb, sizeof(fs_cb_t));
    NFX_ASSERT(cb != NULL);
    strcpy(cb->objName, argv[2]);
    cb->print = FALSE;
    cb->snapnum = 0; // live
    if (argc == 4)
    {
        if (!strcmp(argv[3], "-v"))
            cb->print = TRUE;
        else
            cb->snapnum = atoi(argv[3]);
    }
    if (argc == 5)
    {
        if (!strcmp(argv[4], "-v"))
            cb->print = TRUE;
        else
            cb->snapnum = atoi(argv[3]);
    }

    /* Get the volume id.
     */
    if (fs_shell_get_volid(argv[1], &(cb->volId), nodeName) == FALSE)
    {
        free(cb);
        return(NULL);
    }

#ifndef FS_STANDALONE
        strcpy (cb->nodeName, nodeName);
#else
        cb->nodeName = (char *) NULL;
#endif

    // Get stats on all file system
    if (!strcmp(argv[2], "all"))
    {
        fs_MsgAllocStatsReq *   req = NULL;

        req = alloc_msg(FS_REQ_ALLOCSTATS, sizeof *req, nodeName);
        req->volId = cb->volId;
        req->inum = 0;
        req->print = cb->print;
        req->snapnum = cb->snapnum;
        return((fs_MsgReqHdr *) req);
    }
    // Get stats of one file specified by its inum
    else if (argv[2][0] != '/')
    {
        fs_MsgAllocStatsReq *   req = NULL;

        req = alloc_msg(FS_REQ_ALLOCSTATS, sizeof *req, nodeName);
        req->volId = cb->volId;
        req->inum = atoi(argv[2]);
        req->print = cb->print;
        req->snapnum = cb->snapnum;
        return((fs_MsgReqHdr *) req);
    }
    // Get stats of one file specified by its name
    else
    {
        fs_MsgGetRootReq *  req = NULL;
        
        /* Get the root handle for a mount point in a filesystem.
         */
        reqSize = sizeof(fs_MsgGetRootReq) + strlen("/") + 1;
        req = (fs_MsgGetRootReq *) alloc_msg(FS_RAM_REQ_MSG, reqSize, nodeName);
        req->hdr.requestType = FS_REQ_GET_ROOT;
        req->hdr.attrType = 0;
        req->hdr.uid = 0;   // root
        req->hdr.gid = 0;   // root
        req->hdr.proto = FS_PROTO_NFS_V3;
        req->hdr.numGids = 0;
        req->hdr.clientPrivate = (uint64) (address_t) cb;
        req->volId = cb->volId;
        req->pathLen = strlen("/");
        strcpy(req->pathName, "/");
        return((fs_MsgReqHdr *) req);
    }
    NFX_ASSERT(FALSE); // should never come here!!!
}

static fs_MsgReqHdr *
allocStats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_Handle   dirHandle;
    fs_Handle   fileHandle;
    uint32 *    msgType = NULL;

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    switch(*msgType)
    {
    case FS_REQ_GET_ROOT:
    {
        fs_MsgGetRootResp * rsp = (fs_MsgGetRootResp *) hdr;
        fs_cb_t *           cb = (fs_cb_t *) (address_t) rsp->hdr.clientPrivate;
        uint32              reqSize;
        fs_MsgMclReq *      req = NULL;
        
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }

        /* Now that we have the root handle, let's lookup for the file.
         */
        reqSize = sizeof(fs_MsgMclReq) + strlen(cb->objName) + 1;
        req = (fs_MsgMclReq *) alloc_msg(FS_RAM_REQ_MSG, reqSize, cb->nodeName);
        req->hdr.requestType = FS_REQ_MCL;
        req->hdr.len = reqSize;
        req->hdr.attrType = FS_POST_ATTRS;
        req->hdr.uid = 0;   // root
        req->hdr.gid = 0;   // root
        req->hdr.proto = FS_PROTO_NFS_V3;
        req->hdr.numGids = 0;
        req->hdr.clientPrivate = (uint64) (address_t) cb;
        memcpy((void *) &(req->parentHandle), (const void *) &(rsp->dirHandle),
               sizeof(fs_Handle));
        req->objLen = strlen(cb->objName);
        strcpy(req->objName, cb->objName);
        return((fs_MsgReqHdr *) req);
    } // case FS_REQ_GET_ROOT

    case FS_REQ_MCL:
    {
        fs_MsgMclResp * rsp = (fs_MsgMclResp *) hdr;
        fs_cb_t *       cb = (fs_cb_t *) (address_t) rsp->hdr.clientPrivate;
        fs_MsgAllocStatsReq *   msg = NULL;

        if (rsp->hdr.fsStatus != NFX_OK)
        {
            printf("File %s not found.\n", cb->objName);
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }
        NFX_ASSERT(rsp->numAttrs == 1);
        NFX_ASSERT(rsp->attr[0].attrType == FS_POST_ATTRS);
            
        msg = alloc_msg(FS_REQ_ALLOCSTATS, sizeof *msg, cb->nodeName);
        msg->volId = cb->volId;
        msg->inum = rsp->handle.inum;
        msg->print = cb->print;
        msg->snapnum = cb->snapnum;
        
        fs_ramDealloc(FS_RAM_REQ_MSG, cb);
        
        return (fs_MsgReqHdr *)msg;
    } // case FS_REQ_MCL

    case FS_REQ_ALLOCSTATS:
    {
        fs_MsgAllocStatsResp *  rsp = (fs_MsgAllocStatsResp *) hdr;

        if (rsp->inum == 0)
        {
            fs_fsysStat_t * fsysStat = &(rsp->fsysStat);
            fs_fileStat_t * fileStat = &(rsp->fileStat);
            //uint32          frag = 0;
            float           frag = 0;
            
            printf("\nFree space statistics:\n");
            printf("\tfree space size           : %10lld 8k blocks\n",
                   fileStat->totalExtentSize);
            printf("\tnb extents                : %10lld\n",
                   fileStat->nbExtent);
            printf("\tavg extent size           : %10lld\n",
                   (fileStat->nbExtent == 0)? 0 :
                   fileStat->totalExtentSize / fileStat->nbExtent);
            printf("\tmin extent size           : %10lld\n",
                   fileStat->minExtentSize);
            printf("\tmax extent size           : %10lld\n",
                   fileStat->maxExtentSize);
            if (fileStat->nbExtent == 1)
            {
                frag = 0;
            }
            else if (fileStat->nbExtent == fileStat->totalExtentSize)
            {
                frag = 100;
            }
            else
            {
                frag = ((float)(fileStat->nbExtent - 1) /
                        (float)(fileStat->totalExtentSize - 1)
                        * 100);
            }
            printf("\tfree space fragmentation  : %10.2f %%\n", frag);

            printf("\nFile system statistics:\n");
            printf("\tnb files                  : %10lld\n", fsysStat->nbFile);
            if (fsysStat->nbFile != 0)
            {
                printf("\tavg file size             : %10lld bytes\n",
                       fsysStat->totalFileSize / fsysStat->nbFile);
#ifdef ALLOC_FILE_SIZE_MEDIANE
                printf("\tmedian file size          : %10lld bytes\n",
                       fsysStat->medianeFileSize);
#endif // ALLOC_FILE_SIZE_MEDIANE
                printf("\tnb extents                : %10lld\n",
                       fsysStat->nbExtent);
                printf("\tavg nb extents per file   : %10.2f\n",
                       (float)fsysStat->nbExtent / (float)fsysStat->nbFile);
                printf("\tmin nb extents per file   : %10lld\n",
                       fsysStat->minExtentPerFile);
                printf("\tmax nb extents per file   : %10lld (inum %lld)\n",
                       fsysStat->maxExtentPerFile, fsysStat->inumWithMaxExtent);
                printf("\tmin extent size           : %10lld\n",
                       fsysStat->minExtentSize);
                printf("\tmax extent size           : %10lld\n",
                       fsysStat->maxExtentSize);
                printf("\ttotal nb fragmented files : %10lld\n",
                       fsysStat->nbFragmentedFile);
                if (fsysStat->nbExtent == fsysStat->nbFile)
                {
                    frag = 0;
                }
                //else if (fsysStat->totalExtentSize == fsysStat->nbFile)
                //{
                //    frag = 100;
                //}
                else
                {
                    frag = ((float)(fsysStat->nbExtent - fsysStat->nbFile) /
                            (float)(fsysStat->totalExtentSize - fsysStat->nbFile)
                            * 100);
                }
                printf("\tfragmentation             : %10.2f %%\n", frag);
                printf("\tpercent files fragmented  : %10.2f %%\n",
                       (float)fsysStat->nbFragmentedFile * 100 /
                       (float)fsysStat->nbFile);
            }
        }
        else
        {
            fs_fileStat_t * fileStat = &(rsp->fileStat);
            
            printf("\nFile statistics:\n");
            printf("\tfile size                 : %10lld bytes\n",
                   fileStat->fileSize);
            printf("\tnb extents                : %10lld\n",
                   fileStat->nbExtent);
            printf("\tavg extent size           : %10lld\n",
                   (fileStat->nbExtent == 0)? 0 :
                   fileStat->totalExtentSize / fileStat->nbExtent);
            printf("\tmin extent size           : %10lld\n",
                   fileStat->minExtentSize);
            printf("\tmax extent size           : %10lld\n",
                   fileStat->maxExtentSize);
        }
        printf("Done.\n");
        break;
    } // case FS_REQ_ALLOCSTATS

    default:
    {
        break;
    }    
    } // switch

    return(NULL);    
}

/*
 * Incore Tune
 *
 */

static fs_MsgReqHdr *
cmd_incoreTune(int argc, char *argv[])
{
    uint32               reqSize;
    fs_MsgIncoreTuneReq  *req = NULL;
    int32               fieldId = -1;  // default: don't set 
    int64               value = 0;

    if (argc > 3)
    {
        if (strcmp(argv[1], "set") == 0)
        {
            if (shell_parse_parameter (incoreTunetab, argv[2], tablist, &fieldId) != NFX_OK)
            {
                printf("Usage: incoretune set <%s> <value>\n", tablist);
                return NULL;
            }
            value = atol(argv[3]);
            printf("Set %s (%d) to %lld\n", argv[2], fieldId, value);
         }
        else
        {
            printf("Usage: incoretune set <name> <value>\n");
            return NULL;
        }
    }

    req = alloc_msg(FS_REQ_INCORE_TUNE, sizeof *req, NULL);
    req->fieldId = fieldId;
    req->value = value;

    return((fs_MsgReqHdr *) req);
}

static fs_MsgReqHdr *
incoreTune_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgIncoreTuneResp    *resp = (fs_MsgIncoreTuneResp*)hdr;
    int32                   res;    

	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = resp->count_per_usec;

    printf ("inodePctAvailLoWtrMk          %d\n", resp->incore.inodePctAvailLoWtrMk);
    printf ("inodePctAvailHiWtrMk          %d\n", resp->incore.inodePctAvailHiWtrMk);
    printf ("logPctUseLoWtrMk              %d\n", resp->incore.logPctUseLoWtrMk);
    printf ("logPctUseHiWtrMk              %d\n", resp->incore.logPctUseHiWtrMk);
    printf ("bufPctAvailLoWtrMk            %d\n", resp->incore.bufPctAvailLoWtrMk);
    printf ("bufPctAvailHiWtrMk            %d\n", resp->incore.bufPctAvailHiWtrMk);
    printf ("memPctAvailMin                %d\n", resp->incore.memPctAvailMin);
    printf ("bufWritesPerEvent             %d\n", resp->incore.bufWritesPerEvent);
    printf ("writesPendingMax              %d\n", resp->incore.writesPendingMax);
    printf ("perFsIoPendingMax             %d\n", resp->incore.perFsIoPendingMax);
    printf ("inodeCleanPerPass             %d\n", resp->incore.inodeCleanPerPass);
    printf ("activeTransToWait             %d\n", resp->incore.activeTransToWait);;
    printf ("maxCpuUsageUsec               %lld\n",
			eee_rtcToUsec(&resp->incore.maxCpuUsageRTC));
    printf ("inodePctMem                   %d\n", resp->incore.inodePctMem);
    printf ("wbSleepMsecs                  %d\n", resp->incore.wbSleepMsecs);
    printf ("readdirMaxSpace               %d\n", resp->incore.readdirMaxSpace);
    printf ("userDirtyThreshold            %d\n", resp->incore.userDirtyThreshold);
    printf ("metaDirtyThreshold            %d\n", resp->incore.metaDirtyThreshold);
    printf ("enableReadAhead               %d\n", resp->incore.enableReadAhead);
    printf ("readAheadMaxSize              %d\n", resp->incore.readAheadMaxSize);
    printf ("readAheadDefaultSize          %d\n", resp->incore.readAheadDefaultSize);
    printf ("enableAtimeUpdate             %d\n", resp->incore.enableAtimeUpdate);
    printf ("vollogIoMax                   %d\n", resp->incore.vollogIoMax);
    printf ("vollogLoopBack                %d\n", resp->incore.vollogLoopBack);
    printf ("sacMinAvailable               %d\n", resp->incore.sacMinAvailable);
    printf ("sacFreeCount                  %d\n", resp->incore.sacFreeCount);
    printf ("ultraMinAvailable             %d\n", resp->incore.ultraMinAvailable);
    printf ("memMinAvailable               %d\n", resp->incore.memMinAvailable);
    printf ("bufFreeCount                  %d\n", resp->incore.bufFreeCount);
    printf ("inodeFreeCount                %d\n", resp->incore.inodeFreeCount);
    printf ("userBufMinInUse               %d\n", resp->incore.userBufMinInUse);
    printf ("logCommitHiWtrMk              %d\n", resp->incore.logCommitHiWtrMk);
    printf ("bufUserPctMem                 %d\n", resp->incore.bufUserPctMem);
    printf ("bufMetaNonDirPctMem           %d\n", resp->incore.bufMetaNonDirPctMem);
    printf ("bufMetaDirPctMem              %d\n", resp->incore.bufMetaDirPctMem);
    printf ("bufMetaCompNonDirPctMem       %d\n", resp->incore.bufMetaCompNonDirPctMem);
    printf ("bufMetaCompDirPctMem          %d\n", resp->incore.bufMetaCompDirPctMem);
    printf ("metaFreeCount                 %d\n", resp->incore.metaFreeCount);
    printf ("metaDirMinInUse               %d\n", resp->incore.metaDirMinInUse);
    printf ("metaNonDirMinInUse            %d\n", resp->incore.metaNonDirMinInUse);
    printf ("metaCompDirMinInUse           %d\n", resp->incore.metaCompDirMinInUse);
    printf ("metaCompNonDirMinInUse        %d\n", resp->incore.metaCompNonDirMinInUse);

    // -2: no settings for FS_RESOURCE_NO_ESM & FS_RESOURCE_IO
    for (res = 0 ; res < FS_RESOURCE_TYPE_MAX - 2; res ++)
    {
        printf ("%30s Abs  %d\n", 
                incoreTunetab[3 * res + 0 +  FS_TUNE_RESOURCE].name, 
                resp->incore.resource[res].absMin);
        printf ("%30s Rel  %d\n", 
                incoreTunetab[3 * res + 1 +  FS_TUNE_RESOURCE].name, 
                resp->incore.resource[res].relMin);
        printf ("%30s Load %d\n", 
                incoreTunetab[3 * res + 2 +  FS_TUNE_RESOURCE].name, 
                resp->incore.resource[res].loadFactor);
    }
    
    return(NULL);    
}


static fs_MsgReqHdr *
cmd_dnlcstats(int argc, char *argv[])
{
    fs_MsgDnlcstatsReq *msg;
    evm_volId_t volId;
    boolean isClear = FALSE;
    boolean isFlush = FALSE;
    int carg;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    volId.whole_thing = 0;
    for (carg = 1 ; carg < argc ; carg++)
    {
        if (strcmp("-z", argv[carg]) == 0)
        {
            isClear = TRUE;
        }
        else if (strcmp("-f", argv[carg]) == 0)
        {
            isFlush = TRUE;
        }
        else if (!fs_shell_get_volid(argv[carg], &volId, nodeName))
        {
            // not a valid argumunt, nor volid
            return NULL;
        }
    }

#ifndef FS_STANDALONE
    if (volId.whole_thing == 0) {

        strcpy (nodeName, "local");

    }
#endif

    if (volId.whole_thing == 0 && isFlush)
    {
        printf("dnlc flush must specify volume\n");
        return NULL;
    }

    msg = alloc_msg(FS_REQ_DNLCSTATS, sizeof *msg, nodeName);
    msg->volId = volId;
    msg->isClear = isClear;
    msg->isFlush = isFlush;

    return (fs_MsgReqHdr *)msg;
}


static fs_MsgReqHdr *
dnlcstats_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgDnlcstatsResp   *msg = (fs_MsgDnlcstatsResp *) hdr;
    int loop;

    if (msg->volId.whole_thing == 0)
    {
        printf("\ndnlc stats for all volumes:\n\n");
        printf("max:           %d\n", msg->stats.usedLimit);
        printf("used:          %d\n", msg->stats.used);
        printf("maxDepth:      %d\n", msg->maxDepth);
        printf("avgDepth:      %d\n", msg->usedBuckets ? 
                                         msg->stats.used / msg->usedBuckets : 0);
        printf("usedBuckets:   %d\n", msg->usedBuckets);
    }
    else
    {
        printf("\ndnlc stats for 0x%llx:\n\n", msg->volId.whole_thing);
    }

    printf("hits:          %d\n", msg->stats.hits);
    printf("hitsNegative:  %d\n", msg->stats.hitsNegative);
    printf("missNotFound:  %d\n", msg->stats.missNotFound);
    printf("missTooBig:    %d\n", msg->stats.missTooBig);
    printf("missInode:     %d\n", msg->stats.missInode);
    printf("inserts:       %d\n", msg->stats.inserts);
    printf("removes:       %d\n", msg->stats.removes);
    printf("setNegatives:  %d\n", msg->stats.setNegatives);
    printf("setPositives:  %d\n", msg->stats.setPositives);
    printf("\n");

    return NULL;
}


/*
 *
 */


static fs_MsgReqHdr *
def_response(int status, fs_MsgRespHdr *hdr)
{
	return NULL;
}


/* --------------------------------------------------------------------------*
 *                      Fileset / Quota UI                                   *
 * --------------------------------------------------------------------------*/

#define	TEST_PATH_LEN(len)  if (len > FS_PATH_MAX) {                    \
                                printf("path too long\n");              \
                                return NULL;                            \
                            }

//TODO PUT THIS BEFORE FUNCTION BODIES
static evm_volId_t	curVolId;
static char         *tmpResp = NULL;

static int          cmd_max_entries = 0;
static int          cmd_cur_index;
static char			cmd_path[FS_PATH_MAX];
#ifndef FS_STANDALONE
static              char gNodeName[LNM_MAX_NODE_NAME] = "";
#else
static              char *gNodeName = NULL;
#endif


static fs_MsgGetFilesetPathReq_t *
fs_makeGetFilesetPathReq(uint64 inum, uint16 fsetId)
{
    fs_MsgGetFilesetPathReq_t   *preq;
    
    preq = (fs_MsgGetFilesetPathReq_t *)alloc_msg(FS_REQ_GET_FILESET_PATH,
                                 sizeof (fs_MsgGetFilesetPathReq_t), gNodeName);
    preq->volId     = curVolId;
    preq->snapnum   = 0;
    preq->inum      = inum;
    preq->fsetId    = fsetId; 
    
    return preq;
}

static void fs_PrintQuotaInfo(fs_QuotaBlk_t *qblk, boolean printUsage)
{
    char str[128];

    if (printUsage) {
    	if (qblk->flag & FS_QUOTA_ON) {
#ifdef FS_DEBUG_QUOTA
            sprintf(str, "Current number of files:%lld", qblk->inodesConsumed);
            printf("    %-40sSize (%lld blocks):%lld MB\n", str, 
                   qblk->blocksConsumed,
                   qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
#else            
            sprintf(str, "Current number of files:%llu", qblk->inodesConsumed);
            printf("    %-40sCurrent size           :%llu MB\n", str,
                   qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
#endif                   
       	} else {
            printf("    Quota for this fileset is %s, usage not available\n",
                 (qblk->flag & FS_QUOTA_TURN_ON) ? "being turned on" : "off");
    	}
    }
    if (qblk->inodeSoftLimit != 0) 
        sprintf(str, "Files soft limit       :%llu", qblk->inodeSoftLimit);
    else
        sprintf(str, "Files soft limit       :unlimited");
    printf("    %-40s", str);
    if (qblk->blockSoftLimit != 0) 
        printf("Size soft limit        :%llu MB\n", 
               qblk->blockSoftLimit / (BYTES_IN_MB / FS_BLOCK_SIZE));
    else
        printf("Size soft limit        :unlimited\n");
    if (qblk->inodeHardLimit != 0) 
        sprintf(str, "Files hard limit       :%llu ", qblk->inodeHardLimit);
    else
        sprintf(str, "Files hard limit       :unlimited");
    printf("    %-40s", str);
    if (qblk->blockHardLimit != 0) 
        printf("Size hard limit        :%llu MB\n", 
               qblk->blockHardLimit / (BYTES_IN_MB / FS_BLOCK_SIZE));
    else
        printf("Size hard limit        :unlimited\n");

    if (printUsage) {
    	if (!(qblk->flag & FS_QUOTA_ON)) {
            qblk->inodeSoftLimitTime = 0;
            qblk->blockSoftLimitTime = 0;
        } else {
            //Check whether we are under the soft limit, with time counter still
            //running (no reservation has been done since under soft limit).
            //Reset the counter for display
            if (qblk->inodeSoftLimitTime != 0
             && qblk->inodesConsumed < qblk->inodeSoftLimit)    {
                 qblk->inodeSoftLimitTime = 0;
            }
            if (qblk->blockSoftLimitTime != 0
//           && qblk->blocksConsumed < qblk->blockSoftLimit + 8) //safety margin
             && qblk->blocksConsumed < qblk->blockSoftLimit)
            {
                qblk->blockSoftLimitTime = 0;
            }
        }
        sprintf(str, "Soft limit exceeded for:%u hours", 
                qblk->inodeSoftLimitTime / 3600);
        printf("    %-40sSoft limit exceeded for:%u hours\n", str,
                qblk->blockSoftLimitTime / 3600);
    }
    if (qblk->inodeTimeLimit != 0)
        sprintf(str, "Soft time limit allowed:%u hours ",
                qblk->inodeTimeLimit / 3600);
    else
        sprintf(str, "Soft time limit allowed:unlimited");
    printf("    %-40s", str);
    if (qblk->blockTimeLimit != 0)
        printf("Soft time limit allowed:%u hours\n",
               qblk->blockTimeLimit / 3600);
    else
        printf("Soft time limit allowed:unlimited\n");
}




static fs_MsgReqHdr *
cmd_mkfset(int argc, char *argv[])
{
	fs_MsgCreateFileSetReq_t	*req;
	evm_volId_t					volId;
	char						*path;
	int							len;

	FS_ASSERT(argc == 3);

    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	req = (fs_MsgCreateFileSetReq_t *)alloc_msg(FS_REQ_CREATE_FILESET,
								 sizeof (fs_MsgCreateFileSetReq_t), gNodeName);

	strcpy(req->path, path);
	req->fsid = volId;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
mkfset_response(int status, fs_MsgRespHdr *hdr)
{
    return NULL;
}

static fs_MsgReqHdr *
cmd_rmfset(int argc, char *argv[])
{
	fs_MsgRemoveFileSetReq_t	*req;
	evm_volId_t					volId;
	char						*path;
	int							len;

	FS_ASSERT(argc == 3);

    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	req = (fs_MsgRemoveFileSetReq_t *)alloc_msg(FS_REQ_REMOVE_FILESET,
								 sizeof (fs_MsgRemoveFileSetReq_t), gNodeName);

	req->fsid = volId;
	strcpy(req->path, path);
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
rmfset_response(int status, fs_MsgRespHdr *hdr)
{
    return NULL;
}

static fs_MsgReqHdr *
cmd_quotaon(int argc, char *argv[])
{
	fs_MsgQuotaOnReq_t	*req;
	evm_volId_t					volId;
	char						*path;
	int							len;

	FS_ASSERT(argc == 3);

    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	req = (fs_MsgQuotaOnReq_t *)alloc_msg(FS_REQ_QUOTA_ON,
									 sizeof (fs_MsgQuotaOnReq_t), gNodeName);

	req->fsid = volId;
	strcpy(req->path, path);
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
quotaon_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgQuotaOnResp_t *resp;

    resp = (fs_MsgQuotaOnResp_t*)hdr;
    
    switch (resp->status)   {
        case FS_QUOTA_TURN_ON:  printf("Quota is being turned on\n");
                                break;
        case FS_QUOTA_ON:       printf("Quota is on\n");
                                break;
        default:                printf("Quota is in undefined state\n");
    }
	return NULL;
}

static fs_MsgReqHdr *
cmd_quotaoff(int argc, char *argv[])
{
	fs_MsgQuotaOffReq_t	*req;
	evm_volId_t					volId;
	char						*path;
	int							len;

	FS_ASSERT(argc == 3);

    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	req = (fs_MsgQuotaOffReq_t *)alloc_msg(FS_REQ_QUOTA_OFF,
									 sizeof (fs_MsgQuotaOffReq_t), gNodeName);

	req->fsid = volId;
	strcpy(req->path, path);
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
quotaoff_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgQuotaOffResp_t *resp;

    resp = (fs_MsgQuotaOffResp_t*)hdr;
    
    switch (resp->status)   {
        case 0:
            printf("Quota is off\n");
            break;
        case FS_QUOTA_TURN_ON:  
            printf("Quota is being turned on, please try later\n");
            break;
        default:
            printf("Quota is in undefined state\n");
    }
	return NULL;
}

static boolean fs_checkQuotaLimits( long isoft,
                                    long ihard,
                                    long itime,
                                    long bsoft,
                                    long bhard,
                                    long btime)
{
    if (isoft > ihard)  {
        printf(
          "Warning: number of files soft limit is bigger than hard limit\n");
    }
    if (bsoft > bhard)  {
        printf("Warning: size soft limit is bigger than hard limit\n");
    }        
    if (isoft == LONG_MAX || isoft < 0)  {
        printf("Error: number of files soft limit is not valid\n");
        return FALSE;
    }
    if (ihard == LONG_MAX || ihard < 0)  {
        printf("Error: number of files hard limit is not valid\n");
        return FALSE;
    }
    if (itime == LONG_MAX || itime < 0)  {
        printf("Error: number of files time limit is not valid\n");
        return FALSE;
    }
    if (bsoft == LONG_MAX || bsoft < 0)  {
        printf("Error: size soft limit is not valid\n");
        return FALSE;
    }
    if (bhard == LONG_MAX || bhard < 0)  {
        printf("Error: size hard limit is not valid\n");
        return FALSE;
    }
    if (btime == LONG_MAX || btime < 0)  {
        printf("Error: size time limit is not valid\n");
        return FALSE;
    }
    
    return TRUE;
}
                                    
static fs_MsgReqHdr *
cmd_set_def_quota(int argc, char *argv[])
{
	fs_MsgSetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	char				*type;
	long				isoft;
	long				ihard;
	long				itime;
	long				bsoft;
	long				bhard;
	long				btime;
	char				*endp;
	fs_QuotaBlk_t		*qblk;
	
	FS_ASSERT(argc == 10);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	type = argv[3];

	FS_ASSERT(strcmp(type, "aggr") == 0);

	isoft = strtol(argv[4], &endp, 0);
	ihard = strtol(argv[5], &endp, 0);
	itime = strtol(argv[6], &endp, 0);
	bsoft = strtol(argv[7], &endp, 0);
	bhard = strtol(argv[8], &endp, 0);
	btime = strtol(argv[9], &endp, 0);

    if (!fs_checkQuotaLimits(isoft, ihard, itime, bsoft, bhard, btime)) {
        return NULL;
    }
    
	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgSetQuotaReq_t *)alloc_msg(FS_REQ_SET_QUOTA,
									   sizeof (fs_MsgSetQuotaReq_t), gNodeName);
	
	req->fsid = volId;
	req->numberLimits = 1;
	strcpy(req->path, path);
	qblk = &req->limits[0];
	qblk->id = 0;
	qblk->flag |= FS_QUOTA_DEFAULT;
    qblk->flag |= FS_QUOTA_AGGREGATE;

	qblk->inodeSoftLimit = isoft;
	qblk->inodeHardLimit = ihard;
	qblk->inodeTimeLimit = itime * 3600;
	qblk->blockSoftLimit = bsoft * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockHardLimit = bhard * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockTimeLimit = btime * 3600;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
set_def_quota_response(int status, fs_MsgRespHdr *hdr)
{
	return NULL;
}

static fs_MsgReqHdr *
cmd_set_ug_quota(int argc, char *argv[])
{
	fs_MsgSetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	char				*type;
	long				id;
	long				isoft;
	long				ihard;
	long				itime;
	long				bsoft;
	long				bhard;
	long				btime;
	char				*endp;
	fs_QuotaBlk_t		*qblk;
    uint16              flag = 0;
	
	FS_ASSERT(argc == 6 || argc == 11);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	type = argv[3];

	FS_ASSERT((strcmp(type, "user") == 0) ||
			  (strcmp(type, "group") == 0));

	id = strtol(argv[4], &endp, 0);

    if (argc == 6)  {
        //Use default quota
        if (strcmp(argv[5], "-d"))  {
            printf("invalid parameter\n");
            return NULL;
        }
        flag |= FS_QUOTA_USE_DEFAULTS;
        isoft = ihard = itime = bsoft = bhard = btime = 0;
    } else {
    	isoft = strtol(argv[5], &endp, 0);
    	ihard = strtol(argv[6], &endp, 0);
    	itime = strtol(argv[7], &endp, 0);
    	bsoft = strtol(argv[8], &endp, 0);
    	bhard = strtol(argv[9], &endp, 0);
    	btime = strtol(argv[10], &endp, 0);
        if (!fs_checkQuotaLimits(isoft, ihard, itime, bsoft, bhard, btime)) {
            return NULL;
        }
    }
                    
	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgSetQuotaReq_t *)alloc_msg(FS_REQ_SET_QUOTA,
									   sizeof (fs_MsgSetQuotaReq_t), gNodeName);
	
	req->fsid = volId;
	req->numberLimits = 1;
	strcpy(req->path, path);
	qblk = &req->limits[0];
	qblk->id = id;
	qblk->flag = flag;
	if (strcmp(type, "user") == 0) {
		qblk->flag |= FS_QUOTA_USER;
	} else {
		qblk->flag |= FS_QUOTA_GROUP;
	}

	qblk->inodeSoftLimit = isoft;
	qblk->inodeHardLimit = ihard;
	qblk->inodeTimeLimit = itime * 3600;
	qblk->blockSoftLimit = bsoft * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockHardLimit = bhard * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockTimeLimit = btime * 3600;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
set_ug_quota_response(int status, fs_MsgRespHdr *hdr)
{
	return NULL;
}


static fs_MsgReqHdr *
cmd_set_fset_quota(int argc, char *argv[])
{
	fs_MsgSetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	long				isoft;
	long				ihard;
	long				itime;
	long				bsoft;
	long				bhard;
	long				btime;
	char				*endp;
	fs_QuotaBlk_t		*qblk;
    uint16              flag = FS_QUOTA_AGGREGATE;
	
	FS_ASSERT(argc == 4 || argc == 9);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);

    if (argc == 4)  {
        //Use default quota
        if (strcmp(argv[3], "-d"))  {
            printf("invalid parameter\n");
            return NULL;
        }
        flag |= FS_QUOTA_USE_DEFAULTS;
        isoft = ihard = itime = bsoft = bhard = btime = 0;
    } else {
    	isoft = strtol(argv[3], &endp, 0);
    	ihard = strtol(argv[4], &endp, 0);
    	itime = strtol(argv[5], &endp, 0);
    	bsoft = strtol(argv[6], &endp, 0);
    	bhard = strtol(argv[7], &endp, 0);
    	btime = strtol(argv[8], &endp, 0);
        if (!fs_checkQuotaLimits(isoft, ihard, itime, bsoft, bhard, btime)) {
            return NULL;
        }
    }
                    
	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgSetQuotaReq_t *)alloc_msg(FS_REQ_SET_QUOTA,
									   sizeof (fs_MsgSetQuotaReq_t), gNodeName);
	
	req->fsid = volId;
	req->numberLimits = 1;
	strcpy(req->path, path);
	qblk = &req->limits[0];
	qblk->id = 0;
	qblk->flag = flag;

	qblk->inodeSoftLimit = isoft;
	qblk->inodeHardLimit = ihard;
	qblk->inodeTimeLimit = itime * 3600;
	qblk->blockSoftLimit = bsoft * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockHardLimit = bhard * (BYTES_IN_MB / FS_BLOCK_SIZE);
	qblk->blockTimeLimit = btime * 3600;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
set_fset_quota_response(int status, fs_MsgRespHdr *hdr)
{
	return NULL;
}

static fs_MsgReqHdr *
cmd_get_fset_usage(int argc, char *argv[])
{
	fs_MsgGetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	fs_GetQuota_t		*qblk;
	
	FS_ASSERT(argc == 3);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);
	TEST_PATH_LEN(len);

	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */
	req = (fs_MsgGetQuotaReq_t *)alloc_msg(FS_REQ_GET_QUOTA,
									   sizeof (fs_MsgGetQuotaReq_t), gNodeName);
	req->fsid = volId;
	req->numberEntities = 1;
	strcpy(req->path, path);
	qblk = &req->entities[0];
	qblk->id = 0;
	qblk->flag = FS_QUOTA_AGGREGATE;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
get_fset_usage_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgGetQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	char			*header;

	resp = (fs_MsgGetQuotaResp_t *)hdr;
	qblk = &resp->responses[0];

	FS_ASSERT(resp != NULL);
	if (status != NFX_OK || resp->numberResponse < 1) {
		fprintf(stderr, "get fset usage failed (status %d)\n", status);
        fprintf(stderr, "resonses = %d\n", resp->numberResponse);
		return NULL;
	}

	if (qblk->flag & FS_QUOTA_ON) {
        printf("Number of files:%llu\n"
               "Size           :%llu MB\n",
               qblk->inodesConsumed, 
               qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
   	} else {
        printf("Quota for this fileset is %s, usage not available\n",
                (qblk->flag & FS_QUOTA_TURN_ON) ? "being turned on" : "off");
	}
	
	return NULL;
}

static fs_MsgReqHdr *
cmd_get_fset_quota(int argc, char *argv[])
{
	fs_MsgGetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len, i;
	char				*path;
	char				*type;
	fs_GetQuota_t		*qblk;
    static uint8 getAllArray[4] = { FS_QUOTA_AGGREGATE, 
                                    FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE };

	FS_ASSERT(argc >= 3);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);
	TEST_PATH_LEN(len);

	req = (fs_MsgGetQuotaReq_t *)alloc_msg(FS_REQ_GET_QUOTA,
									   sizeof (fs_MsgGetQuotaReq_t), gNodeName);
	
	req->fsid = volId;
	strcpy(req->path, path);
   	qblk = &req->entities[0];

    if (argc == 4)  {
    	req->numberEntities = 1;
    	type = argv[3];

    	FS_ASSERT((strcmp(type, "aggr") == 0) ||
    			  (strcmp(type, "def_aggr") == 0));

    	qblk->id = 0;
    	qblk->flag = 0;
    	if (strcmp(type, "def_aggr") == 0) {
    		qblk->flag |= FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE;
    	} else {
    		qblk->flag |= FS_QUOTA_AGGREGATE;
    	}
    } else {    //get everything
    	req->numberEntities = 2;
        for (i = 0; i < req->numberEntities; i++)  {
            qblk->id = 0;
            qblk->flag = getAllArray[i];            
            qblk++;
        }            
    }
    
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
get_fset_quota_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgGetQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	char			*header;
	int				i;

	resp = (fs_MsgGetQuotaResp_t *)hdr;
	qblk = &resp->responses[0];

	FS_ASSERT(resp != NULL);
	if (status != NFX_OK || resp->numberResponse < 1) {
		fprintf(stderr, "get fset quota failed (status %d)\n", status);
        fprintf(stderr, "resonses = %d\n", resp->numberResponse);
		return NULL;
	}

    for (i = 0; i < resp->numberResponse; i++)  {
    	if (qblk->flag & FS_QUOTA_DEFAULT) {
    		if (qblk->flag & FS_QUOTA_USER) {
    			header ="Default User quota";
    		} else if (qblk->flag & FS_QUOTA_GROUP) {
    			header ="Default Group quota";
    		} else {
    			header ="Default quota for Filesets created in this fileset";
    		}
    	} else {
            if (qblk->flag & FS_QUOTA_USE_DEFAULTS) {
                header ="Fileset quota (uses default quota from its parent)";
            } else {
                header ="Fileset quota";
            }
        }
        printf("- %s\n", header);
        fs_PrintQuotaInfo(qblk, !(qblk->flag & FS_QUOTA_DEFAULT));

        qblk++;
    }
	
	return NULL;
}

static fs_MsgReqHdr *
cmd_get_ug_quota(int argc, char *argv[])
{
	fs_MsgGetQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	char				*type;
	fs_GetQuota_t			*qblk;
	
	FS_ASSERT(argc == 5);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);
	type = argv[3];

	FS_ASSERT((strcmp(type, "user") == 0) ||
			  (strcmp(type, "group") == 0));

	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgGetQuotaReq_t *)alloc_msg(FS_REQ_GET_QUOTA,
									   sizeof (fs_MsgGetQuotaReq_t), gNodeName);
	
	qblk = &req->entities[0];
	req->numberEntities = 1;
	req->fsid = volId;
	strcpy(req->path, path);
	qblk->id = atoi(argv[4]);
	qblk->flag = 0;
	if (strcmp(type, "user") == 0) {
		qblk->flag |= FS_QUOTA_USER;
	} else if (strcmp(type, "group") == 0) {
		qblk->flag |= FS_QUOTA_GROUP;
	}
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
get_ug_quota_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgGetQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;

	resp = (fs_MsgGetQuotaResp_t *)hdr;
	qblk = &resp->responses[0];

	FS_ASSERT(resp != NULL);
	FS_ASSERT((qblk->flag & (FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE)) == 0);
	if (status != NFX_OK || resp->numberResponse < 1) {
		printf("get user/group quota failed");
		return NULL;
	}

    if (qblk->flag & FS_QUOTA_USER) {
    	printf("Quota for user %-llu%s\n", (unsigned long long)qblk->id,
               (qblk->flag & FS_QUOTA_USE_DEFAULTS) ? 
               "  (uses fileset default user quota)" : "");
    } else {
    	printf("Quota for group %-llu%s\n", (unsigned long long)qblk->id,
               (qblk->flag & FS_QUOTA_USE_DEFAULTS) ? 
               "  (uses fileset default group quota)" : "");
    }
    fs_PrintQuotaInfo(qblk, TRUE);
    
	return NULL;
}

static fs_MsgReqHdr *
get_ug_usage_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgGetQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;

	resp = (fs_MsgGetQuotaResp_t *)hdr;
	qblk = &resp->responses[0];

	FS_ASSERT(resp != NULL);
	FS_ASSERT((qblk->flag & (FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE)) == 0);
	if (status != NFX_OK || resp->numberResponse < 1) {
		printf("get user/group usage failed");
		return NULL;
	}

	if (qblk->flag & FS_QUOTA_ON) {
        printf("Number of files:%llu\n"
               "Size           :%llu MB\n",
               qblk->inodesConsumed, 
               qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
   	} else {
        printf("Quota for this fileset is %s, usage not available\n",
                (qblk->flag & FS_QUOTA_TURN_ON) ? "being turned on" : "off");
	}
	return NULL;
}

static fs_MsgReqHdr *
cmd_report_fset_quota(int argc, char *argv[])
{
	fs_MsgReportQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	int					index;
	int					count;
	
	FS_ASSERT(argc >= 2);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
    if (argc >= 3)
    	index = atoi(argv[2]);
    else
        index = 0;
    
    cmd_max_entries = (FS_SHELL_RESP_MAX - sizeof(fs_MsgReportQuotaResp_t))
                           / sizeof(fs_QuotaBlk_t);
    if (argc == 4)
    	count = atoi(argv[3]);
    else
        count = cmd_max_entries;
    
	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
	req->fsid = curVolId = volId;
    req->path[0] = '\0';
	req->flag = FS_QUOTA_AGGREGATE;
	req->startIndex = index;
	req->numberEntries = count;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
report_fset_usage_list_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	int				i;

	resp = (fs_MsgReportQuotaResp_t *)hdr;
	FS_ASSERT(resp != NULL);
	if (status != NFX_OK || (resp->numberResponses < 1 && tmpResp == NULL)) {
		fprintf(stderr, "report fset usage failed\n");
		fprintf(stderr, "status = %d, responses = %d\n",
                status, resp->numberResponses);
		return NULL;
	}
    if (resp->numberResponses < 1)
        return NULL;
        
    if (tmpResp == NULL)
        tmpResp = malloc(FS_SHELL_RESP_MAX);
    memset (tmpResp, 0, FS_SHELL_RESP_MAX);
    memcpy(tmpResp, resp, hdr->len);
            
    qblk = &resp->responses[0];
    cmd_cur_index = 0;

    return (fs_MsgReqHdr *)
           (fs_makeGetFilesetPathReq(qblk->rootInode, qblk->id));
}

static fs_MsgReqHdr *
report_fset_usage_path_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp = (fs_MsgReportQuotaResp_t*)tmpResp;
    fs_MsgGetFilesetPathResp_t  *presp;
	fs_QuotaBlk_t	*qblk;

    presp = (fs_MsgGetFilesetPathResp_t *)hdr;
	FS_ASSERT(resp != NULL);

	if (status != FS_ERR_NONE) {
		fprintf(stderr, "getting fileset path failed\n");
		return NULL;
	}

    qblk = &resp->responses[cmd_cur_index];
    
    printf("Fileset %02llu  %s\n", qblk->id, presp->path);
	if (qblk->flag & FS_QUOTA_ON) {
        printf("    Number of files:%-10llu Size:%llu MB\n",
               qblk->inodesConsumed, 
               qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
   	} else {
        printf("    Quota for this fileset is %s, usage not available\n",
                (qblk->flag & FS_QUOTA_TURN_ON) ? "being turned on" : "off");
	}
    
    if (++cmd_cur_index < resp->numberResponses) {
        //Get next fileset path
        qblk++;
        return (fs_MsgReqHdr *)
               (fs_makeGetFilesetPathReq(qblk->rootInode, qblk->id));
    } else {
        //Get next part of filesets usage 
        if (resp->numberResponses == cmd_max_entries)
        {
        	fs_MsgReportQuotaReq_t	*req;
            
        	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
        	req->fsid = curVolId;
            req->path[0] = '\0';
        	req->flag = FS_QUOTA_AGGREGATE;
        	req->startIndex = qblk->id + 1;
        	req->numberEntries = MIN(cmd_max_entries, 
                                     FS_FILESET_MAX_ID - req->startIndex);
            return (fs_MsgReqHdr *)req;
        }
    }

	return NULL;
}

static fs_MsgReqHdr *
report_fset_usage_response(int status, fs_MsgRespHdr *hdr)
{
    switch(hdr->requestType)
    {
        case FS_REQ_REPORT_QUOTA:
            return report_fset_usage_list_response(status, hdr);
        case FS_REQ_GET_FILESET_PATH:
            return report_fset_usage_path_response(status, hdr);
    }
    return NULL;
}


static fs_MsgReqHdr *
report_fset_quota_list_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	int				i;

	resp = (fs_MsgReportQuotaResp_t *)hdr;
	FS_ASSERT(resp != NULL);
	if (status != NFX_OK || (resp->numberResponses < 1 && tmpResp == NULL)) {
		fprintf(stderr, "report fset quota failed\n");
		fprintf(stderr, "status = %d, responses = %d\n",
                status, resp->numberResponses);
		return NULL;
	}
    if (resp->numberResponses < 1)
        return NULL;
        
    if (tmpResp == NULL)
        tmpResp = malloc(FS_SHELL_RESP_MAX);
    memcpy(tmpResp, resp, hdr->len);
            
    qblk = &resp->responses[0];
    cmd_cur_index = 0;

    return (fs_MsgReqHdr *)
           (fs_makeGetFilesetPathReq(qblk->rootInode, qblk->id));
}

static fs_MsgReqHdr *
report_fset_quota_path_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp = (fs_MsgReportQuotaResp_t*)tmpResp;
    fs_MsgGetFilesetPathResp_t  *presp;
	fs_QuotaBlk_t	*qblk;

    presp = (fs_MsgGetFilesetPathResp_t *)hdr;
	FS_ASSERT(resp != NULL);

	if (status != FS_ERR_NONE) {
		fprintf(stderr, "getting fileset path failed\n");
		return NULL;
	}

    qblk = &resp->responses[cmd_cur_index];
    
    printf("Fileset %02llu  %s  %s\n", qblk->id, presp->path,
            (qblk->flag & FS_QUOTA_USE_DEFAULTS) ? 
            "\n    Uses default quota from its parent" : "" );

    fs_PrintQuotaInfo(qblk, TRUE);
    
    if (++cmd_cur_index < resp->numberResponses) {
        //Get next fileset path
        qblk++;
        return (fs_MsgReqHdr *)
               (fs_makeGetFilesetPathReq(qblk->rootInode, qblk->id));
    } else {
        //Get next part of filesets usage 
        if (resp->numberResponses == cmd_max_entries)
        {
        	fs_MsgReportQuotaReq_t	*req;

        	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
        	req->fsid = curVolId;
            req->path[0] = '\0';
        	req->flag = FS_QUOTA_AGGREGATE;
        	req->startIndex = qblk->id + 1;
        	req->numberEntries = MIN(cmd_max_entries, 
                                     FS_FILESET_MAX_ID - req->startIndex);
            return (fs_MsgReqHdr *)req;
        }
    }

	return NULL;
}

static fs_MsgReqHdr *
report_fset_quota_response(int status, fs_MsgRespHdr *hdr)
{
    switch(hdr->requestType)
    {
        case FS_REQ_REPORT_QUOTA:
            return report_fset_quota_list_response(status, hdr);
        case FS_REQ_GET_FILESET_PATH:
            return report_fset_quota_path_response(status, hdr);
    }
    return NULL;
}


static fs_MsgReqHdr *
cmd_report_ug_quota(int argc, char *argv[])
{
	fs_MsgReportQuotaReq_t	*req;
	evm_volId_t			volId;
	int					len;
	char				*path;
	char				*type;
	int					index;
	int					count;
	
	FS_ASSERT(argc >= 4);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
	path = argv[2];
	len = strlen(path);

	TEST_PATH_LEN(len);

	type = argv[3];

	FS_ASSERT((strcmp(type, "user") == 0) || (strcmp(type, "group") == 0));
    if (argc >= 5)
    	index = atoi(argv[4]);
    else
        index = 0;

    cmd_max_entries = (FS_SHELL_RESP_MAX - sizeof(fs_MsgReportQuotaResp_t))
                           / sizeof(fs_QuotaBlk_t);
    if (argc >= 6)  
    	count = atoi(argv[5]);
    else
        count = cmd_max_entries;

	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
	req->fsid = curVolId = volId;
	strcpy(req->path, path);
    strcpy(cmd_path, path);
	req->startIndex = index;
	req->numberEntries = count;
	if (strcmp(type, "user") == 0) {
		req->flag = FS_QUOTA_USER;
	} else {
		req->flag = FS_QUOTA_GROUP;
	}
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
report_ug_quota_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	int				i;
    static boolean  emptyResponse = TRUE;

	resp = (fs_MsgReportQuotaResp_t *)hdr;
	FS_ASSERT(resp != NULL);
	if (status != NFX_OK) {
		fprintf(stderr, "report user/group quota failed (status %d)\n", status);
        fprintf(stderr, "responses = %d\n", resp->numberResponses);
		return NULL;
	}
    if (resp->numberResponses < 1 && emptyResponse) {
        printf("No consumption record found\n"
               "Quota for this fileset may not be on\n");
        return NULL;
    }

    emptyResponse = FALSE;
    
	qblk = &resp->responses[0];
	for (i = 0; i < resp->numberResponses; i++) {
        if (qblk->flag & FS_QUOTA_USER) {
        	printf("Quota for user %-llu%s\n", (unsigned long long)qblk->id,
                   (qblk->flag & FS_QUOTA_USE_DEFAULTS) ? 
                   "  (uses fileset default user quota)" : "");
        } else {
        	printf("Quota for group %-llu%s\n", (unsigned long long)qblk->id,
                   (qblk->flag & FS_QUOTA_USE_DEFAULTS) ? 
                   "  (uses fileset default group quota)" : "");
        }
        fs_PrintQuotaInfo(qblk, TRUE);
        qblk++;        
	}

    if (resp->numberResponses == cmd_max_entries)
    {
        //Get next part of user/group quota 
      	fs_MsgReportQuotaReq_t	*req;

       	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
       	req->fsid = curVolId;
        strcpy(req->path, cmd_path);
       	req->flag = (--qblk)->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP);
       	req->startIndex = qblk->id + 1;
       	req->numberEntries = cmd_max_entries;

        return (fs_MsgReqHdr *)req;
    }
    
#if defined(FS_STANDALONE)
    emptyResponse = TRUE;   
#endif    
	return NULL;
}

static fs_MsgReqHdr *
report_ug_usage_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgReportQuotaResp_t	*resp;
	fs_QuotaBlk_t	*qblk;
	int				i;
    static boolean  emptyResponse = TRUE;

	resp = (fs_MsgReportQuotaResp_t *)hdr;
	FS_ASSERT(resp != NULL);
	if (status != NFX_OK) {
		fprintf(stderr, "report user/group usage failed (status %d)\n", status);
        fprintf(stderr, "responses = %d\n", resp->numberResponses);
		return NULL;
	}
    if (resp->numberResponses < 1 && emptyResponse) {
        printf("No consumption record found\n"
               "Quota for this fileset may not be on\n");
        return NULL;
    }

    emptyResponse = FALSE;
    
	qblk = &resp->responses[0];
	for (i = 0; i < resp->numberResponses; i++) {

		printf("%s %-5llu   Number of files:%-10llu Size:%llu MB\n",
    			(qblk->flag & FS_QUOTA_USER) ? "User" : "Group",
				(unsigned long long)qblk->id,
                qblk->inodesConsumed, 
                qblk->blocksConsumed / (BYTES_IN_MB / FS_BLOCK_SIZE));
        qblk++;        
	}

    if (resp->numberResponses == cmd_max_entries)
    {
        //Get next part of user/group quota 
      	fs_MsgReportQuotaReq_t	*req;

       	req = (fs_MsgReportQuotaReq_t *)alloc_msg(FS_REQ_REPORT_QUOTA,
								   sizeof (fs_MsgReportQuotaReq_t), gNodeName);
       	req->fsid = curVolId;
        strcpy(req->path, cmd_path);
       	req->flag = (--qblk)->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP);
       	req->startIndex = qblk->id + 1;
       	req->numberEntries = cmd_max_entries;

        return (fs_MsgReqHdr *)req;
    }
    
#if defined(FS_STANDALONE)
    emptyResponse = TRUE;   
#endif    
	return NULL;
}

static fs_MsgReqHdr *
cmd_list_fsets(int argc, char *argv[])
{
	fs_MsgListFileSetsReq_t	*req;
	evm_volId_t			volId;
	int					index;
	int					count;
	
	FS_ASSERT(argc >= 2);
    if (!fs_shell_get_volid(argv[1], &volId, gNodeName))
        return NULL;
    if (argc >= 3)
    	index = atoi(argv[2]);
    else
        index = 0;
    
    cmd_max_entries = (FS_SHELL_RESP_MAX - sizeof(fs_MsgListFileSetsResp_t))
                       / sizeof(fs_FileSetList_t);
    if (argc == 4)
    	count = atoi(argv[3]);
    else
        count = cmd_max_entries;

	/*
	 * Now that we have validated all the arguments construct the
	 * message
	 */

	req = (fs_MsgListFileSetsReq_t *)alloc_msg(FS_REQ_LIST_FILESETS,
							   sizeof (fs_MsgListFileSetsReq_t), gNodeName);
	
	req->fsid = curVolId = volId;
	req->startIndex = index;
	req->numberEntries = count;
	return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
list_fsets_list_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgListFileSetsResp_t	*resp;
    fs_MsgGetFilesetPathReq_t   *preq;
	fs_FileSetList_t	*list;
	int				i;
    char            *name;
                
	resp = (fs_MsgListFileSetsResp_t *)hdr;
	FS_ASSERT(resp != NULL);
	if (status != NFX_OK || (resp->numberEntries < 1 && tmpResp == NULL)) {
		fprintf(stderr, "list fsets failed\n");
		fprintf(stderr, "status = %d, responses = %d\n",
                status, resp->numberEntries);
		return NULL;
	}
    if (resp->numberEntries < 1)
        return NULL;
    
    if (tmpResp == NULL)
        tmpResp = malloc(FS_SHELL_RESP_MAX);
    memcpy(tmpResp, resp, hdr->len);

	list = &resp->filesets[0];
    preq = fs_makeGetFilesetPathReq(list->rootInode, list->id);
    
    cmd_cur_index = 0;

	return (fs_MsgReqHdr *)preq;
}

static fs_MsgReqHdr *
list_fsets_path_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgListFileSetsResp_t	*resp = (fs_MsgListFileSetsResp_t*)tmpResp;
    fs_MsgGetFilesetPathResp_t  *presp;
	fs_FileSetList_t	*list;

    presp = (fs_MsgGetFilesetPathResp_t *)hdr;
	FS_ASSERT(resp != NULL);

	if (status != FS_ERR_NONE) {
		fprintf(stderr, "getting fileset %d path failed\n", 
                        cmd_cur_index);
		return NULL;
	}

	list = &resp->filesets[cmd_cur_index];
    if (list->parentId == FS_FILESET_INVAL_ID)
        printf("%-2d root      path:%s\n", list->id, presp->path);
    else
        printf("%-2d parent:%-2d path:%s\n", list->id, list->parentId, 
                                            presp->path);

    if (++cmd_cur_index < resp->numberEntries) {
        //Get next fileset path
        list++;
        return (fs_MsgReqHdr *)
                fs_makeGetFilesetPathReq(list->rootInode, list->id);
    } else {
        //Get next part of fileset list
        if (resp->numberEntries == cmd_max_entries)
        {
        	fs_MsgListFileSetsReq_t	*req;

        	req = (fs_MsgListFileSetsReq_t *)alloc_msg(FS_REQ_LIST_FILESETS,
							   sizeof (fs_MsgListFileSetsReq_t), gNodeName);
        	req->fsid = curVolId;
        	req->startIndex = list->id + 1;
        	req->numberEntries = MIN(cmd_max_entries, 
                                     FS_FILESET_MAX_ID - req->startIndex);
            return (fs_MsgReqHdr *)req;
        }
    }

	return NULL;
}


static fs_MsgReqHdr *
list_fsets_response(int status, fs_MsgRespHdr *hdr)
{
    switch(hdr->requestType)
    {
        case FS_REQ_LIST_FILESETS:
            return list_fsets_list_response(status, hdr);
        case FS_REQ_GET_FILESET_PATH:
            return list_fsets_path_response(status, hdr);
    }
    return NULL;
}


static fs_MsgReqHdr *cmd_snap(int argc, char **argv)
{
    static char *usage = "usage: snap (create|remove) <volId> <name>";
	evm_volId_t volId;
    char *name;
    int len;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc != 4) {
        printf("%s\n", usage);
        return NULL;
    }
    name = argv[3];
    len = strlen(name);

    if (!fs_shell_get_volid(argv[2], &volId, nodeName))
        return NULL;

    if (strcmp(argv[1], "create") == 0) {
        fs_MsgSnapCreateReq *req
            = alloc_msg(FS_REQ_SNAP_CREATE,
                        offsetof(fs_MsgSnapCreateReq, snapDirName[len + 1]), nodeName);
        req->volId = volId;
        req->nameLen = len;
        strcpy(req->snapDirName, name);
        return &req->hdr;
    }
    else if (strcmp(argv[1], "remove") == 0) {
        fs_MsgSnapRemoveReq *req
            = alloc_msg(FS_REQ_SNAP_REMOVE,
                        offsetof(fs_MsgSnapRemoveReq, snapDirName[len + 1]), nodeName);
        req->volId = volId;
        req->nameLen = len;
        strcpy(req->snapDirName, name);
        return &req->hdr;
    }
    else {
        printf("%s\n", usage);
        return NULL;
    }
}

static fs_MsgReqHdr *cmd_snap_state(int argc, char **argv)
{
    static char *usage = "usage: snap_state <volId> [enable | disable]";
	evm_volId_t volId;
    char *state;
    int len;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc != 3) {
        printf("%s\n", usage);
        return NULL;
    }
    state = argv[2];
    len = strlen(state);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName)) {
        return NULL;
    }

    if (strcmp(state, "enable") == 0) {
        fs_MsgSnapStateReq *req
            = alloc_msg(FS_REQ_SNAP_STATE, sizeof(fs_MsgSnapStateReq), nodeName);
        req->volId = volId;
        req->enable = TRUE;
        return &req->hdr;
    }
    else if (strcmp(state, "disable") == 0) {
        fs_MsgSnapStateReq *req
            = alloc_msg(FS_REQ_SNAP_STATE, sizeof(fs_MsgSnapStateReq), nodeName);
        req->volId = volId;
        req->enable = FALSE;
        return &req->hdr;
    }
    else {
        printf("%s\n", usage);
        return NULL;
    }
}


static fs_MsgReqHdr *cmd_snap_pin(int argc, char **argv)
{
    static char *usage = "usage: VOL SNAPNAME [pin | unpin]";
	evm_volId_t volId;
    char *state;
    char *name;
    int len;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc != 4) {
        printf("%s\n", usage);
        return NULL;
    }
    if (!fs_shell_get_volid(argv[1], &volId, nodeName)) {
        return NULL;
    }
    name = argv[2];
    len = strlen(name);
    state = argv[3];
    if (strcmp(state, "pin") == 0) {
        fs_MsgSnapPinReq *req
            = alloc_msg(FS_REQ_SNAP_PIN,
                        offsetof(fs_MsgSnapPinReq, snapname[len + 1]), nodeName);
        req->volId = volId;
        req->pin = TRUE;
        strcpy(req->snapname, name);
        req->nameLen = len;
        return &req->hdr;
    }
    else if (strcmp(state, "unpin") == 0) {
        fs_MsgSnapPinReq *req
            = alloc_msg(FS_REQ_SNAP_PIN,
                        offsetof(fs_MsgSnapPinReq, snapname[len + 1]), nodeName);
        req->volId = volId;
        req->pin = FALSE;
        strcpy(req->snapname, name);
        req->nameLen = len;
        return &req->hdr;
    }
    else {
        printf("%s\n", usage);
        return NULL;
    }
}


static fs_MsgReqHdr *cmd_snap_rename(int argc, char **argv)
{
    static char *usage = "usage: VOL OLDNAME NEWNAME";
    fs_MsgSnapRenameReq *req;
	evm_volId_t volId;
    char *newname, *oldname;
    int newlen, oldlen;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc != 4) {
        printf("%s\n", usage);
        return NULL;
    }
    if (!fs_shell_get_volid(argv[1], &volId, nodeName)) {
        return NULL;
    }
    oldname = argv[2];
    oldlen = strlen(oldname);
    newname = argv[3];
    newlen = strlen(newname);
    if ((newlen >= FS_NAME_MAX) || (oldlen >= FS_NAME_MAX)) {
        printf("snapshot rename: name must be < %d\n", FS_NAME_MAX);
        return NULL;
    }
    req = alloc_msg(FS_REQ_SNAP_RENAME, sizeof(fs_MsgSnapRenameReq), nodeName);
    req->volId = volId;
    strcpy(req->newname, newname);
    req->newname[newlen] = 0;
    strcpy(req->oldname, oldname);
    req->oldname[oldlen] = 0;
    return &req->hdr;
}


static fs_MsgReqHdr *cmd_snap_revert(int argc, char **argv)
{
    static char *usage = "usage: snap_revert <volname> [-n name | -i snapnum]";
    fs_MsgSnapRevertReq *req;
	evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if ((argc != 2) && (argc != 4)) {
        printf("%s\n", usage);
        return NULL;
    }
    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;
    if (argc == 4) {
        if (!strcmp(argv[2], "-n")) {
            int len;

            len = strlen(argv[3]);
            req = (fs_MsgSnapRevertReq *) alloc_msg(FS_REQ_SNAP_REVERT,
                        offsetof(fs_MsgSnapRevertReq, snapname[len + 1]), nodeName);
            strcpy(req->snapname, argv[3]);
            req->nameLen = len;
            req->type = FS_SNAP_REVERT_NAME;
            //printf("snap revert: name %s\n", req->snapname);
        }
        if (!strcmp(argv[2], "-i")) {
            int32 snapnum = atoi(argv[3]);

            if ((snapnum <= 0) || (snapnum > FS_SNAP_MAX_INTERNAL)) {
                printf("snap revert: invalid snapnum %d (1-%d)\n", snapnum, FS_SNAP_MAX_INTERNAL);
                return NULL;
            }
            req = (fs_MsgSnapRevertReq *) alloc_msg(FS_REQ_SNAP_REVERT, 
                                         sizeof(fs_MsgSnapRevertReq), nodeName);
            req->snapnum = snapnum;
            req->type = FS_SNAP_REVERT_ID;
            //printf("snap revert: id %d\n", req->snapnum);
        }
    } else {
        req = (fs_MsgSnapRevertReq *) alloc_msg(FS_REQ_SNAP_REVERT, sizeof(fs_MsgSnapRevertReq), nodeName);
        req->type = FS_SNAP_REVERT_NEWEST;
        //printf("snap revert: newest\n");
    }
    req->volId = volId;
    return &req->hdr;
}

/*
 * snap sched
 */

static int has_digits(char *str)
{
    int cnt;

    for (cnt=0; cnt < strlen(str); cnt++) {
        if (!isdigit(str[cnt])) {
            return 0;
        }
    }
    return 1;
}

static fs_MsgReqHdr *cmd_snap_sched(int argc, char **argv)
{
    static char *usage = "options: [-w MAX_WEEKLY] [-d MAX_DAILY] [-h MAX_HOURLY] [-l HOURS]";
    fs_MsgSnapSchedReq *req = NULL;
    evm_volId_t volId;
    char c, *s, h[32];
    int i, nHours = 0;
    extern int optind;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
    extern char *optplace = "";
#endif

    if (!fs_shell_get_volid(argv[1], &volId, nodeName)) {
        printf("cannot get volume %s\n", argv[1]);
        return NULL;
    }

    req = alloc_msg(FS_REQ_SNAP_SCHED, sizeof(fs_MsgSnapSchedReq), nodeName);
    req->volId = volId;

    /*
     * Reset getopt to start at 2nd arg.
     */
    optind = 2;

    while ((c = getopt(argc, argv, "w:d:h:l:")) != -1) {
        switch (c) {
            case ':':
                printf("missing parameter for option\n");
                printf("%s\n", usage);
                goto cmd_snap_sched_err;
            case '?':
                printf("unknown option\n");
                printf("%s\n", usage);
                goto cmd_snap_sched_err;
            case 'w':
                if (req->sched.maxWeeklySet) {
                    printf("-w option already specified\n");
                    goto cmd_snap_sched_err;
                }
                if (has_digits(optarg)) {
                    req->sched.maxWeeklySet = TRUE;
                    req->sched.maxWeekly = atoi(optarg);
                } else {
                    printf("-w option requires a number\n");
                    goto cmd_snap_sched_err;
                }
                break;
            case 'd':
                if (req->sched.maxDailySet) {
                    printf("-d option already specified\n");
                    goto cmd_snap_sched_err;
                }
                if (has_digits(optarg)) { 
                    req->sched.maxDailySet = TRUE;
                    req->sched.maxDaily = atoi(optarg);
                } else {
                    printf("-d option requires a number\n");
                    goto cmd_snap_sched_err;
                }
                break;
            case 'h':
                if (req->sched.maxHourlySet) {
                    printf("-h option already specified\n");
                    goto cmd_snap_sched_err;
                }
                if (has_digits(optarg)) {
                    req->sched.maxHourlySet = TRUE;
                    req->sched.maxHourly = atoi(optarg);
                } else {
                    printf("-h option requires a number\n");
                    goto cmd_snap_sched_err;
                }
                break;
            case 'l':
                if (req->sched.hoursSet) {
                    printf("-l option already specified\n");
                    goto cmd_snap_sched_err;
                }
                s = strtok(optarg, ",");
                if (!has_digits(s))
                {
                    printf("-l option requires a number greater than 0\n");
                    goto cmd_snap_sched_err;
                }
                req->sched.hoursSet = TRUE;
                req->sched.hours[nHours] = atoi(s);
                if ((req->sched.hours[nHours] < 1) || (req->sched.hours[nHours] > 24)) {
                    printf("invalid hour %d (specify 1-24)\n", req->sched.hours[nHours]);
                    goto cmd_snap_sched_err;
                }
                nHours++;
                while (s) {
                    s = strtok(NULL, ",");
                    if (s && has_digits(s)) {
                        req->sched.hours[nHours] = atoi(s);
                        if ((req->sched.hours[nHours] < 1) || (req->sched.hours[nHours] > 24)) {
                            printf("invalid hour %d (specify 1-24)\n", req->sched.hours[nHours]);
                            goto cmd_snap_sched_err;
                        }
                        nHours++;
                    }
                }
                break;
        } // switch
    }

    if (!req->sched.maxWeeklySet && !req->sched.maxDailySet && !req->sched.maxHourlySet) {
        printf("no options given\n");
        printf("%s\n", usage);
        goto cmd_snap_sched_err;
    }
    if (!req->sched.maxHourlySet && req->sched.hoursSet) {
        printf("MAX_HOURLY must be specified with the -l option\n");
        printf("%s\n", usage);
        goto cmd_snap_sched_err;
    }

    // sanity checks max values

    if (req->sched.maxWeekly+req->sched.maxDaily+req->sched.maxHourly > FS_MAX_USER_SNAPSHOTS) {
        printf("maximum of %d snapshots available\n",
                FS_MAX_USER_SNAPSHOTS);
        goto cmd_snap_sched_err;
    }

    //printf("snapshot schedule: -w %d -d %d -h %d\n",
    //      req->sched.maxWeekly, req->sched.maxDaily, req->sched.maxHourly);
    //if (req->sched.hoursSet) {
    //  for (i = 0; i < nHours; i++) {
    //      printf("%d ", req->sched.hours[i]);
    //  }
    //  printf("\n");
    //}
 
    return &req->hdr;

cmd_snap_sched_err:

    dealloc_msg(req);
    return NULL;
}

/*
 * snap admin
 */
static fs_MsgReqHdr *cmd_snap_admin(int argc, char **argv)
{
    static char *usage = "usage: snap_admin [<volname> <internalSnaps> <internalInterval>]";
    evm_volId_t volId;
    fs_MsgSnapAdminReq *req;
    struct timeval now;
    struct tm *tm_now;

#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (argc == 1)
    {
        req = alloc_msg(FS_REQ_SNAP_ADMIN, sizeof(fs_MsgSnapAdminReq), NULL);
        req->type = FS_SNAP_ADMIN_SCHED;
        gettimeofday(&now, NULL);
        tm_now = localtime((const time_t *) &now.tv_sec);
        req->data.time.hour = tm_now->tm_hour;
        if (req->data.time.hour == 0) {
            req->data.time.hour = FS_SNAP_MIDNIGHT;
        }
        req->data.time.wday = tm_now->tm_wday;
        if (req->data.time.wday == 0) {
            req->data.time.wday = FS_SNAP_SUNDAY;
        }
        req->data.time.min = tm_now->tm_min;
        if (req->data.time.min == 0) {
            req->data.time.min = FS_SNAP_ON_THE_HOUR;
        }
        printf("snap_admin: snapshot scheduled at hour %d min %d\n",
                req->data.time.hour, req->data.time.min);
    }
    else if (argc == 4)
    {
        uint32 maxInternalSnaps;

        if (!fs_shell_get_volid(argv[1], &volId, nodeName)) {
            printf("snapshot admin: cannot get volume %s\n", argv[1]);
            return NULL;
        }
        maxInternalSnaps = atoi(argv[2]);
        if (maxInternalSnaps > FS_SNAP_MAX_INTERNAL)
        {
            printf("snapshot admin: max internal snapshots is %d\n",
                FS_SNAP_MAX_INTERNAL);
            return NULL;
        }
        req = alloc_msg(FS_REQ_SNAP_ADMIN, sizeof(fs_MsgSnapAdminReq), nodeName);
        req->type = FS_SNAP_ADMIN_SET_INTERNAL;
        req->data.snapInternals.volId = volId;
        req->data.snapInternals.maxInternalSnaps = maxInternalSnaps;
        req->data.snapInternals.internalTimeout = atoi(argv[3]);

        if (req->data.snapInternals.maxInternalSnaps)
        {
            printf("snap_admin: internal snapshot scheduled every %d minutes with max %d for vol %lld\n",
                req->data.snapInternals.internalTimeout,
                req->data.snapInternals.maxInternalSnaps,
                volId.whole_thing);
        }
        else
        {
            printf("snap_admin: internal snapshot disabled for vol %lld\n",
                    volId.whole_thing);
        }
    }
    else
    {
        printf("%s\n", usage);
        return NULL;
    }

    return &req->hdr;
}

/*
 * snap show
 */
static fs_MsgReqHdr *cmd_snap_show(int argc, char **argv)
{
    static char *usage = "usage: snap_show [sched | usage] <volname>\n";
    fs_MsgSnapShowReq *req;
    evm_volId_t volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    // volId required for both show commands

    if (argc != 3) {
        printf(usage);
        return NULL;
    }
    if (!fs_shell_get_volid(argv[2], &volId, nodeName)) {
        printf("snap_show: cannot get volume %s\n", argv[2]);
        return NULL;
    }
    req = alloc_msg(FS_REQ_SNAP_SHOW, sizeof(fs_MsgSnapShowReq), nodeName);
    req->volId = volId;

    if (strcmp(argv[1], "sched") == 0) {
        req->type = FS_SNAP_SHOW_SCHED;
        return &req->hdr;
    }
    else if (strcmp(argv[1], "usage") == 0) {
        // show names first, the response will actually get the usage
        req->type = FS_SNAP_SHOW_USAGE;
        return &req->hdr;
    } else {
        printf(usage);
        return NULL;
    }
}

/*
 * snap show qsort compare routine
 */
static int
snap_show_cmp(const void *e1, const void *e2)
{
    fs_SnapUsage *u1 = (fs_SnapUsage *) e1, *u2 = (fs_SnapUsage *) e2;

    if (u1->ctime.secs > u2->ctime.secs)
        return -1;
    if (u1->ctime.secs == u2->ctime.secs)
        return 0;
    if (u1->ctime.secs < u2->ctime.secs)
        return 1;
    return 0;
}

/*
 * snap show response
 */
static fs_MsgReqHdr *
snap_show_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgSnapShowResp *resp = (fs_MsgSnapShowResp *) hdr;
    fs_MsgSnapShowReq req;
    fs_SnapSched *sched;
    fs_SnapUsage *usage, usages[FS_MAX_USER_SNAPSHOTS];
    uint32  rc, i, n=0, percent, *hours, pin = 0;
    char *time;
    struct timeval timeout = {60, 0};

    if (resp->type == FS_SNAP_SHOW_SCHED) {
        sched = &resp->sched;
        if (sched->enable == FALSE) {
            printf("  ** snapshot scheduling currently disabled for this volume **\n");
        }
        hours = &sched->hours[0];
        printf("  schedule\n");
        printf("  ========\n");
        if (sched->maxHourly && *hours) {
            printf("  %2d %2d %2d @",
                    sched->maxWeekly, sched->maxDaily, sched->maxHourly);
            while (*hours) {
                printf("%d ", *hours++);
            }
            printf("\n");
        } else {
            printf("  %2d %2d %2d\n",
                    sched->maxWeekly, sched->maxDaily, sched->maxHourly);
        }
    }
    else if (resp->type == FS_SNAP_SHOW_USAGE) {
        if (!resp->nUsers && !resp->nInternals) return NULL;
        if (resp->nUsers) {
            printf("   snapshot                    %%        blks          create time\n");
            printf("   ========                   ==============          ===========\n");
            usage = &resp->user[0];
            // sort by ctime
            qsort((void *) usage, resp->nUsers, sizeof(fs_SnapUsage), snap_show_cmp);
            for (i = 0; i < resp->nUsers; usage++, i++) {
                // time string contains a nl but ok if data is last print
                time = ctime((time_t *) &usage->ctime.secs);
                percent = (usage->size * 100) / (usage->size + resp->totalBlks);
                if (usage->pin) {
                    printf("** %-24s  %2d%%  %10lld          %-10s",
                            usage->name, percent, usage->size, time);
                    pin++;
                } else {
                    printf("   %-24s  %2d%%  %10lld          %-10s",
                            usage->name, percent, usage->size, time);
                }
            }
        }
        if (pin) {
            printf("\n** pinned snapshots\n");
        }
        if (resp->nInternals) {
            char name[32];

            printf("\n");
            printf("   internal snapshot ids       %%        blks          create time\n");
            printf("   =====================      ==============          ===========\n");
            usage = &resp->internal[0];
            // sort by ctime
            qsort((void *) usage, resp->nInternals, sizeof(fs_SnapUsage), snap_show_cmp);
            for (i = 0; i < resp->nInternals; usage++, i++) {
                // time string contains a nl but ok if data is last print
                time = ctime((time_t *) &usage->ctime.secs);
                percent = (usage->size * 100) / (usage->size + resp->totalBlks);
                sprintf(name, "%d", usage->snapnum);
                printf("   %-24s  %2d%%  %10lld          %-10s",
                        name, percent, usage->size, time);
            }
        }
        printf("\nTotal blocks ...... %lld\n", resp->totalBlks);
        printf("Blocks used ....... %lld\n", resp->blksUsed);
        printf("Available ......... %lld\n", resp->totalBlks - resp->blksUsed);
        printf("\n\nNOTES:\n\n");
        printf("    1. blks is the number of blocks used only by this snapshot\n");
        printf("       (does not include the blocks that are also used by other snapshots\n");
        printf("        or by the live file system\n\n");
        printf("    2. percentage is equal to\n\n");
        printf("        (this snapshot * 100%) / (this snapshot + file system space)\n\n");
        printf("    3. pinned snapshots cannot be deleted until they are unpinned by\n");
        printf("       the snapshot unpin command\n\n");
    }
    else {
        printf("snap_show: invalid response from FP (%d)\n", resp->type);
    }
    return NULL;
}

/*
 * eek response; possibly drive a loop
 */
static fs_MsgReqHdr *
eek_response(int status, fs_MsgRespHdr *hdr)
{
	fs_MsgEekResp *resp = (fs_MsgEekResp *) hdr;
	fs_MsgEekReq *eek;
	char buf[256];
	char *cp;
	int reply;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    if (resp->has_status) {
	    printf("%s\n", resp->msg);
        if (resp->quit) {
            return NULL;
        }
    }
    else {
	    if (!resp->has_query)
		    return NULL;

	    do {
		    printf("%s ", resp->msg);
		    fgets(buf, (sizeof buf) - 1, stdin);
		    buf[(sizeof buf) - 1] = '\0';
		    cp = buf;

		    while (*cp != '\0' && isspace(*cp))
			    cp++;
		    reply = -1;
		    switch (tolower(*cp)) {
		    case 'n': reply = FS_EEK_QUERY_NO; break;
		    case 'y': reply = FS_EEK_QUERY_YES; break;
		    // not yet supported
            // case 'q': reply = FS_EEK_QUERY_QUIT; break;
		    }
		    if (reply < 0) {
			    printf("please answer \"yes\", or \"no\"\n");
		    }
	    } while (reply < 0);
    }

#ifndef FS_STANDALONE
    // call just to get name
    // ignore return value since state is paused, ie not online
    fs_shell_is_vol_online (eekVolId, nodeName);
#endif

	eek = alloc_msg(FS_REQ_EEK, sizeof *eek, nodeName);
	eek->query_reply = reply;
	eek->private = resp->private;
	((fs_MsgReqHdr *)eek)->requestType = FS_REQ_EEKCONT; // XXX: HACK 

	return (fs_MsgReqHdr *)eek;
}

/*
 * snap diagnostics
 */
static fs_MsgReqHdr *cmd_snapdiag(int argc, char **argv)
{
    fs_MsgSnapdiagReq *req;
    static char *usage = "usage: snapdiag (refbytes|snapbits|stat|crash) <volId | crashPhase> [snapname]";
    evm_volId_t volId;
    int len=0;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    // stat and crash require no vol id

    if (strcmp(argv[1], "stat") == 0) {
        if (argc != 2) {
            printf("usage: snapdiag stat\n");
            return NULL;
        }
        req = alloc_msg(FS_REQ_SNAP_DIAG,
                        offsetof(fs_MsgSnapdiagReq, snapDirName[len + 1]), NULL);
        req->cmd = FS_SNAP_DIAG_GET_STATS;
        return &req->hdr;
    }
    else if (strcmp(argv[1], "crash") == 0) {
        if (argc != 3) {
            printf("usage: snapdiag crash crashPhase (0-13)\n");
            return NULL;
        }
        req = alloc_msg(FS_REQ_SNAP_DIAG,
                        offsetof(fs_MsgSnapdiagReq, snapDirName[len + 1]), NULL);
        req->cmd = FS_SNAP_DIAG_FORCE_CRASH;
        req->phase = atoi(argv[2]);
        printf("snapdiag: force a crash during phase %d\n", req->phase);
        return &req->hdr;
    }

    // refbytes and snapbits require volId

    if (!fs_shell_get_volid(argv[2], &volId, nodeName))
    {
        printf("snapdiag: cannot get volId\n");
        return NULL;
    }

    if (strcmp(argv[1], "refbytes") == 0) {
        if (argc != 3) {
            printf("usage: snapdiag refbytes <volId>\n");
            return NULL;
        }
        req = alloc_msg(FS_REQ_SNAP_DIAG,
                        offsetof(fs_MsgSnapdiagReq, snapDirName[len + 1]), nodeName);
        req->cmd = FS_SNAP_DIAG_GET_REFBYTES;
        req->volId = volId;
        return &req->hdr;
    }

    // snapbits require vol id and snapname

    if (strcmp(argv[1], "snapbits") == 0) {
        if (argc != 4) {
            printf("usage: snapdiag snapbits <volId> <snapname>\n");
            return NULL;
        }
        len = strlen(argv[3]);
        req = alloc_msg(FS_REQ_SNAP_DIAG,
                        offsetof(fs_MsgSnapdiagReq, snapDirName[len + 1]), nodeName);
        req->volId = volId;
        req->cmd = FS_SNAP_DIAG_GET_SNAPBITS;
        req->nameLen = len;
        strcpy(req->snapDirName, argv[3]);
        return &req->hdr;
    }
    else {
        printf("%s\n", usage);
        return NULL;
    }
}
static fs_MsgReqHdr *
snapdiag_stat_response(int status, fs_MsgRespHdr *hdr)
{
    fs_MsgSnapdiagResp   *msg = (fs_MsgSnapdiagResp *) hdr;

    if (msg->cmd != FS_SNAP_DIAG_GET_STATS)
        return NULL;

    printf("Number of snap creates ................. %10d\n", msg->stats.snapCreates);
    printf("Number of snap removes ................. %10d\n", msg->stats.snapRemoves);
    printf("Number of snap reverts ................. %10d\n", msg->stats.snapReverts);

    printf("\ncreate and remove:\n"); 
    printf("  Reference snap counter increments .... %10d\n", msg->stats.refCntIncs);
    printf("  Reference snap counter decrements .... %10d\n", msg->stats.refCntDecs);
    printf("  Reference live clears ................ %10d\n", msg->stats.liveClears);
    printf("  Blocks included in snapbits file ..... %10d\n", msg->stats.snapBitBlocks);
    printf("  Blocks not free ...................... %10d\n", msg->stats.notFrees);
    printf("    Live bit set ....................... %10d\n", msg->stats.liveNotFrees);
    printf("    Snap counter > 0 ................... %10d\n", msg->stats.snapNotFrees);
    printf("  Blocks freed after snap remove ....... %10d\n", msg->stats.blockFrees);

    printf("\nrevert:\n"); 
    printf("  Blocks included in snapbits file ..... %10d\n", msg->stats.revertSnapBitBlks);
    printf("  Reference byte mods\n");
    printf("    Live bit set ....................... %10d\n", msg->stats.revertLiveSets);
    printf("    Live bit clear ..................... %10d\n", msg->stats.revertLiveClears);
    printf("    Already live ....................... %10d\n", msg->stats.revertAlreadyLives);
    printf("    Never snap bits .................... %10d\n", msg->stats.revertNeverSnaps);
    printf("  Blocks returned to buddy ............. %10d\n", msg->stats.revertBuddyAdds); 
    printf("  Allocated inodes after snap revert\n");
    printf("    Regular ............................ %10d\n", msg->stats.revertInodes);
    printf("    Directory .......................... %10d\n", msg->stats.revertDInodes);
    printf("  Number of file system blocks ......... %10lld\n", msg->stats.totalBlocks);
    printf("  Number of used blocks ................ %10lld\n", msg->stats.usedBlocks);
    printf("  Number of used inodes ................ %10lld\n", msg->stats.usedInodes);

    return NULL;
}
/*
 * Dump Testing
 */

static fs_MsgReqHdr *
cmd_setup_dump(int argc, char *argv[])
{
	fs_MsgDumpSetupReq_t	*req;
	evm_volId_t					volId;
    uint32                  blobLen = 0;
    int                     level = 0;
    int                     recursiveDescent = 0;
    int                     fileHistory = 0;
    int                     prevSnapTime = 0;
    int                     labelLen = 0;
    int                     prevSnapLen = 0;
    char                    *endp;
    char                    *label = NULL;
    char                    *snapname = "default";
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc >= 7 && argc <= 8);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    level = strtol(argv[2], &endp, 10);
    recursiveDescent = strtol(argv[3], &endp, 10);
    fileHistory = strtol(argv[4], &endp, 10);
    prevSnapTime = strtol(argv[5], &endp, 10);
    if (argc > 7) {
        label = argv[6];
        snapname = argv[7];
    } else {
        snapname = argv[6];
    }

    labelLen = strlen(label);
    prevSnapLen = strlen(snapname);


    blobLen = labelLen + prevSnapLen;
	req = (fs_MsgDumpSetupReq_t *)alloc_msg(FS_REQ_DUMP_SETUP,
									 sizeof (fs_MsgDumpSetupReq_t) +
                                     blobLen - 1, nodeName);

	req->volId = volId;
    req->dump.ndmpSessId = 1;
    req->dump.recursiveDescent = recursiveDescent;
    req->dump.level = level;
    req->dump.fileHistory = fileHistory;
    req->dump.prevSnapTime = prevSnapTime;
    req->dump.cwdLen = 0;
    req->dump.labelLen = labelLen;
    req->dump.prevSnapNameLen = prevSnapLen;
    req->dump.blob[0] = '\0';
    if (labelLen) {
        memcpy(req->dump.blob, label, labelLen);
    }
    memcpy(&req->dump.blob[labelLen], snapname, prevSnapLen);

    printf("Returning message from setup dump");
    printf("Request type = %d\n", req->hdr.requestType);
    return (fs_MsgReqHdr *)req;
}
static fs_MsgReqHdr *
cmd_start_dump(int argc, char *argv[])
{
	fs_MsgDumpStartReq_t	*req;
	evm_volId_t					volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc == 2);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    printf("volid is %lld\n", volId.whole_thing);
	req = (fs_MsgDumpStartReq_t *)alloc_msg(FS_REQ_DUMP_START,
									 sizeof (fs_MsgDumpSetupReq_t), nodeName);
	req->volId = volId;
    req->start.ndmpSessId = 1;
    return (fs_MsgReqHdr *)req;
}
static fs_MsgReqHdr *
cmd_set_dump_path(int argc, char *argv[])
{
	fs_MsgDumpNamesReq_t	    *req;
	evm_volId_t					volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc == 3);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    printf("volid is %lld\n", volId.whole_thing);
	req = (fs_MsgDumpNamesReq_t *)alloc_msg(FS_REQ_DUMP_NAMES,
									 sizeof (fs_MsgDumpNamesReq_t) +
                                     strlen(argv[2]) - 1, nodeName);
	req->volId = volId;
    req->names.ndmpSessId = 1;
    req->names.len = strlen(argv[2]);
    strcpy(req->names.msgBlock, argv[2]);
    return (fs_MsgReqHdr *)req;
}

/*
 * Restore Testing
 */

static fs_MsgReqHdr *
cmd_setup_restore(int argc, char *argv[])
{
	fs_MsgRestoreSetupReq_t	*req;
	evm_volId_t				volId;
    int                     recursiveDescent = 0;
    uint32                  iosize;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc == 4);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;
    iosize = strtoul(argv[2], NULL, 10);
    recursiveDescent = strtol(argv[1], NULL, 10);

	req = (fs_MsgRestoreSetupReq_t *)alloc_msg(FS_REQ_REST_SETUP,
									 sizeof (fs_MsgRestoreSetupReq_t), nodeName);

	req->volId = volId;
    req->restore.ndmpSessId = 1;
    req->restore.recursiveDescent = recursiveDescent;
    req->restore.ioSize = iosize;
    req->restore.cwdLen = 0;
    return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
cmd_start_restore(int argc, char *argv[])
{
	fs_MsgRestoreStartReq_t	*req;
	evm_volId_t					volId;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc == 2);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    printf("volid is %lld\n", volId.whole_thing);
	req = (fs_MsgRestoreStartReq_t *)alloc_msg(FS_REQ_REST_START,
									 sizeof (fs_MsgRestoreSetupReq_t), nodeName);
	req->volId = volId;
    req->start.ndmpSessId = 1;
    return (fs_MsgReqHdr *)req;
}

static fs_MsgReqHdr *
cmd_add_restore_path(int argc, char *argv[])
{
	fs_MsgRestoreNamesReq_t	    *req;
	evm_volId_t					volId;
    char                        *path;
    char                        *newPath = NULL;
    uint32                      size;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

	FS_ASSERT(argc == 3 || argc == 4);

    if (!fs_shell_get_volid(argv[1], &volId, nodeName))
        return NULL;

    printf("volid is %lld\n", volId.whole_thing);
    path = argv[2];
    FS_ASSERT(path != NULL);
    if (argc == 4) {
        newPath = argv[3];
    }
    printf("path is %s newpath is %s\n", path, newPath);
    printf("sizeof restore names is %d\n", sizeof (fs_MsgRestoreNamesReq_t));
    size = sizeof (fs_MsgRestoreNamesReq_t) + strlen(path) - 1;
    size += (newPath ? strlen(newPath) : 0);
    printf("size is %u\n", size);
	req = (fs_MsgRestoreNamesReq_t *)alloc_msg(FS_REQ_REST_NAMES, size, NULL);
	req->volId = volId;
    req->names.ndmpSessId = 1;
    req->names.oldLen = strlen(path);
    req->names.newLen = strlen(newPath);
    memcpy(req->names.msgBlock, path, req->names.oldLen);
    if (newPath) {
        memcpy(req->names.msgBlock + req->names.oldLen, newPath,
               req->names.newLen);
        printf("copied new path\n");
    }
    return (fs_MsgReqHdr *)req;
}
static fs_MsgReqHdr *cmd_localmirror(int argc, char **argv)
{
    return NULL;
}

/*------------------------------------------------------------------------------
 * FSSF command.
 *------------------------------------------------------------------------------
 */

#include "exports/fssf-api.h"

typedef struct
{
#ifndef FS_STANDALONE
    char                nodeName[LNM_MAX_NODE_NAME];
#else
    char                *nodeName;
#endif
    char                objName[FS_PATH_MAX];
    char                pathName[FS_PATH_MAX];  // path name
    char                fileName[FS_NAME_MAX];  // file name
} fssfcmd_cb_t;

/*------------------------------------------------------------------------------
 * Name         : cmd_mcl
 *
 * Description  : Multi component lookup.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 27-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
cmd_mcl(int argc, char *argv[])
    
{
    fssfcmd_cb_t *      cb = NULL;
    evm_volId_t         volId;
    uint32              reqSize;
    fs_MsgGetRootReq *  req = NULL;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    NFX_ASSERT(argc == 3);
    
    /* Initalize the control block;
     */
    cb = (fssfcmd_cb_t *) fs_ramAlloc(FS_RAM_REQ_MSG, sizeof(fssfcmd_cb_t));
    bzero((void *) cb, sizeof(fssfcmd_cb_t));
    strcpy(cb->objName, argv[2]);

    /* Get the volume id.
     */
    if (fs_shell_get_volid(argv[1], &volId, nodeName) == FALSE)
    {
        free(cb);
        return(NULL);
    }

#ifndef FS_STANDALONE
        strcpy (cb->nodeName, nodeName);
#else
        cb->nodeName =  (char *)NULL;
#endif

    /* Get the root handle for a mount point in a filesystem.
     */
    reqSize = sizeof(fs_MsgGetRootReq) + strlen("/") + 1;
    req = (fs_MsgGetRootReq *) alloc_msg (FS_RAM_REQ_MSG, reqSize, nodeName);
    req->hdr.requestType = FS_REQ_GET_ROOT;
	req->hdr.attrType = 0;
	req->hdr.uid = 0;   // root
	req->hdr.gid = 0;   // root
    req->hdr.proto = FS_PROTO_NFS_V3;
    req->hdr.numGids = 0;
    req->hdr.clientPrivate = (uint64) (address_t) cb;
    req->volId = volId;
    req->pathLen = strlen("/");
    strcpy(req->pathName, "/");

    return((fs_MsgReqHdr *) req);
}

/*------------------------------------------------------------------------------
 * Name         : mcl_response
 *
 * Description  : Multi component lookup.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
mcl_response(int status, fs_MsgRespHdr * hdr)
    
{
    fs_Handle   dirHandle;
    fs_Handle   fileHandle;
    uint32 *    msgType = NULL;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    switch(*msgType)
    {
    case FS_REQ_GET_ROOT:
    {
        fs_MsgGetRootResp * rsp = (fs_MsgGetRootResp *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;
        uint32              reqSize;
        fs_MsgMclReq *   req = NULL;
        
        printf("FS_REQ_GET_ROOT status = %d\n", rsp->hdr.fsStatus);
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }


        /* Now that we have the root handle, let's lookup for the file.
         */
        reqSize = sizeof(fs_MsgMclReq) + strlen(cb->objName) + 1;
        req = (fs_MsgMclReq *) alloc_msg (FS_REQ_MCL, reqSize, cb->nodeName);
        req->hdr.attrType = FS_POST_ATTRS;
        req->hdr.uid = 0;   // root
        req->hdr.gid = 0;   // root
        req->hdr.proto = FS_PROTO_NFS_V3;
        req->hdr.numGids = 0;
        req->hdr.clientPrivate = (uint64) (address_t) cb;
        memcpy((void *) &(req->parentHandle), (const void *) &(rsp->dirHandle),
               sizeof(fs_Handle));
        req->objLen = strlen(cb->objName);
        strcpy(req->objName, cb->objName);
        return((fs_MsgReqHdr *) req);
    } // case FS_REQ_GET_ROOT

    case FS_REQ_MCL:
    {
        fs_MsgMclResp * rsp = (fs_MsgMclResp *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;

        printf("FS_REQ_MCL status = %d\n", rsp->hdr.fsStatus);
        printf("handle = 0x%llx/0x%llx/%d/%d/0x%llx\n",
               rsp->handle.volId.whole_thing,
               rsp->handle.inum,
               rsp->handle.gen,
               rsp->handle.mkfsTime,
               rsp->handle.mntPnt);
        printf("parentHandle = 0x%llx/0x%llx/%d/%d/0x%llx\n",
               rsp->parentHandle.volId.whole_thing,
               rsp->parentHandle.inum,
               rsp->parentHandle.gen,
               rsp->parentHandle.mkfsTime,
               rsp->parentHandle.mntPnt);
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }
        NFX_ASSERT(rsp->numAttrs == 1);
        NFX_ASSERT(rsp->attr[0].attrType == FS_POST_ATTRS);
        printf("rsp->attr[0].mode = 0x%x\n", rsp->attr[0].mode);
        printf("Done.\n");
        fs_ramDealloc(FS_RAM_REQ_MSG, cb);
        break;
    } // case FS_REQ_MCL

    default:
    {
        break;
    }    
    } // switch

    return(NULL);
}

/*------------------------------------------------------------------------------
 * Name         : cmd_fssfLookup
 *
 * Description  : Lookup for a specific file on a given filesystem.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
cmd_fssfLookup(int argc, char *argv[])
    
{
    fssfcmd_cb_t *      cb = NULL;
    evm_volId_t         volId;
    uint32              reqSize;
    fs_MsgGetRootReq *  req = NULL;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    NFX_ASSERT(argc == 3);
    
    //printf("volname = %s\n", argv[1]);
    //printf("filename = %s\n", argv[2]);

    /* Initalize the control block;
     */
    cb = (fssfcmd_cb_t *) fs_ramAlloc(FS_RAM_REQ_MSG, sizeof(fssfcmd_cb_t));
    bzero((void *) cb, sizeof(fssfcmd_cb_t));
    NFX_ASSERT(cb != NULL);
    strcpy(cb->pathName, "/");
    strcpy(cb->fileName, argv[2]);

    /* Get the volume id.
     */
    if (fs_shell_get_volid(argv[1], &volId, nodeName) == FALSE)
    {
        free(cb);
        return(NULL);
    }


#ifndef FS_STANDALONE
        strcpy (cb->nodeName, nodeName);
#else
        cb->nodeName = NULL;
#endif

    /* Get the root handle for a mount point in a filesystem.
     */
    reqSize = sizeof(fs_MsgLookupReq) + strlen(cb->pathName);
    //req = (fs_MsgGetRootReq *) calloc(reqSize, 1);
    req = (fs_MsgGetRootReq *) alloc_msg (FS_REQ_GET_ROOT, reqSize, nodeName);
	req->hdr.attrType = 0;
	req->hdr.uid = 0;   // root
	req->hdr.gid = 0;   // root
    req->hdr.proto = FS_PROTO_NFS_V3;
    req->hdr.numGids = 0;
    req->hdr.clientPrivate = (uint64) (address_t) cb;
    req->volId = volId;
    req->pathLen = strlen(cb->pathName);
    strcpy(req->pathName, cb->pathName);

    return((fs_MsgReqHdr *) req);
}

/*------------------------------------------------------------------------------
 * Name         : fssfLookup_response
 *
 * Description  : Lookup for a specific file on a given filesystem.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
fssfLookup_response(int status, fs_MsgRespHdr * hdr)
    
{
    fs_Handle   dirHandle;
    fs_Handle   fileHandle;
    uint32 *    msgType = NULL;

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    switch(*msgType)
    {
    case FS_REQ_GET_ROOT:
    {
        fs_MsgGetRootResp * rsp = (fs_MsgGetRootResp *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;
        uint32              reqSize;
        fs_MsgLookupReq *   req = NULL;
        
        printf("FS_REQ_GET_ROOT status = %d\n", rsp->hdr.fsStatus);
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            //free(cb);
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }

        /* Now that we have the root handle, let's lookup for the file.
         */
        reqSize = sizeof(fs_MsgLookupReq) + strlen(cb->fileName);
        //req = (fs_MsgLookupReq *) calloc(reqSize, 1);
        req = alloc_msg (FS_REQ_LOOKUP, reqSize, NULL);
        req->hdr.attrType = FS_POST_ATTRS;
        req->hdr.uid = 0;   // root
        req->hdr.gid = 0;   // root
        req->hdr.proto = FS_PROTO_NFS_V3;
        req->hdr.numGids = 0;
        req->hdr.clientPrivate = (uint64) (address_t) cb;
        memcpy((void *) &(req->parentHandle), (const void *) &(rsp->dirHandle),
               sizeof(fs_Handle));
        req->nameLen = strlen(cb->fileName);
        strcpy(req->fileName, cb->fileName);
        return((fs_MsgReqHdr *) req);
    } // case FS_REQ_GET_ROOT

    case FS_REQ_LOOKUP:
    {
        fs_MsgLookupResp *  rsp = (fs_MsgLookupResp *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;

        printf("FS_REQ_LOOKUP status = %d\n", rsp->hdr.fsStatus);
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            //free(cb);
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }
        NFX_ASSERT(rsp->numAttrs == 1);
        NFX_ASSERT(rsp->attr[0].attrType == FS_POST_ATTRS);
        printf("rsp->attr[0].mode = 0x%x\n", rsp->attr[0].mode);
        printf("Done.\n");
        //free(cb);
        fs_ramDealloc(FS_RAM_REQ_MSG, cb);
        break;
    } // case FS_REQ_LOOKUP

    default:
    {
        break;
    }    
    } // switch

    return(NULL);
}

/*------------------------------------------------------------------------------
 * Name         : cmd_fssfOpen
 *
 * Description  : Open a file.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
cmd_fssfOpen(int argc, char *argv[])
    
{
    fssfcmd_cb_t *      cb = NULL;
    evm_volId_t         volId;
    uint32              reqSize;
    fs_MsgGetRootReq *  req = NULL;
#ifndef FS_STANDALONE
    char nodeName[LNM_MAX_NODE_NAME];
    memset (nodeName, 0, LNM_MAX_NODE_NAME);
#else
    char *nodeName;
    nodeName = NULL;
#endif

    NFX_ASSERT(argc == 3);
    
    //printf("volname = %s\n", argv[1]);
    //printf("filename = %s\n", argv[2]);

    /* Initalize the control block;
     */
    cb = (fssfcmd_cb_t *) fs_ramAlloc(FS_RAM_REQ_MSG, sizeof(fssfcmd_cb_t));
    bzero((void *) cb, sizeof(fssfcmd_cb_t));
    NFX_ASSERT(cb != NULL);
    strcpy(cb->pathName, "/");
    strcpy(cb->fileName, argv[2]);

    /* Get the volume id.
     */
    if (fs_shell_get_volid(argv[1], &volId, nodeName) == FALSE)
    {
        //free(cb);
        fs_ramDealloc(FS_RAM_REQ_MSG, cb);
        return(NULL);
    }

#ifndef FS_STANDALONE
        strcpy (cb->nodeName, nodeName);
#else
        cb->nodeName =  (char *)NULL;
#endif

    /* Get the root handle for a mount point in a filesystem.
     */
    reqSize = sizeof(fs_MsgLookupReq) + strlen(cb->pathName);
    //req = (fs_MsgGetRootReq *) calloc(reqSize, 1);
    req = (fs_MsgGetRootReq *) alloc_msg (FS_REQ_GET_ROOT, reqSize, nodeName);
	req->hdr.attrType = 0;
	req->hdr.uid = 0;   // root
	req->hdr.gid = 0;   // root
    req->hdr.proto = FS_PROTO_NFS_V3;
    req->hdr.numGids = 0;
    req->hdr.clientPrivate = (uint64) (address_t) cb;
    req->volId = volId;
    req->pathLen = strlen(cb->pathName);
    strcpy(req->pathName, cb->pathName);

    return((fs_MsgReqHdr *) req);
}

/*------------------------------------------------------------------------------
 * Name         : fssfOpen_response
 *
 * Description  : Open a file.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
fssfOpen_response(int status, fs_MsgRespHdr * hdr)
    
{
    fs_Handle   dirHandle;
    fs_Handle   fileHandle;
    uint32 *    msgType = NULL;

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    switch(*msgType)
    {
    case FS_REQ_GET_ROOT:
    {
        fs_MsgGetRootResp * rsp = (fs_MsgGetRootResp *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;
        uint32              reqSize;
        fssf_openReq_t *    req;
        
        printf("FS_REQ_GET_ROOT status = %d\n", rsp->hdr.fsStatus);
        if (rsp->hdr.fsStatus != NFX_OK)
        {
            //free(cb);
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }

        /* Now that we have the root handle, let's open the file.
         */
        reqSize = sizeof(fssf_openReq_t);
        //req = (fssf_openReq_t *) calloc(sizeof(fssf_openReq_t), 1);
        req = (fssf_openReq_t *) alloc_msg (FSSF_OPEN_REQ, reqSize, cb->nodeName);
        req->hdr.protocol = FS_PROTO_CIFS;
        req->hdr.clientPrivate = (uint64) (address_t) cb;
        req->owner = 0xDAD;
        memcpy((void *) &req->parentHandle, (const void *)  &(rsp->dirHandle),
               sizeof(fs_Handle));
        strcpy(req->pathName, "");
        strcpy(req->fileName, cb->fileName);
        req->createFlag = FALSE;
        //req->createHow;
        //req->u;
        req->shareAccess = FSSF_SHARE_ACCESS_BOTH;
        req->shareDeny = FSSF_SHARE_DENY_NONE;
        
        return((fs_MsgReqHdr *) req);
    } // case FS_REQ_GET_ROOT

    case FSSF_OPEN_RSP:
    {
        fssf_openRsp_t *    rsp = (fssf_openRsp_t *) hdr;
        fssfcmd_cb_t * cb = (fssfcmd_cb_t *) (address_t) rsp->hdr.clientPrivate;

        printf("FSSF_OPEN_RSP status = %d\n", rsp->status);
        if (rsp->status != NFX_OK)
        {
            //free(cb);
            fs_ramDealloc(FS_RAM_REQ_MSG, cb);
            break;
        }
        printf("rsp->stateid = 0x%llx\n", rsp->stateid.id);
        printf("rsp->attr.mode = 0x%x\n", rsp->attr.mode);
        printf("Done.\n");
        //free(cb);
        fs_ramDealloc(FS_RAM_REQ_MSG, cb);
        break;
    } // case FS_REQ_LOOKUP

    default:
    {
        break;
    }    
    } // switch
    
    return(NULL);
}

/*------------------------------------------------------------------------------
 * Name         : cmd_fssfClose
 *
 * Description  : Close a file.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
cmd_fssfClose(int argc, char *argv[])

{
    fssf_stateid_t      stateid;
    uint32              reqSize;
    fssf_closeReq_t *   req = NULL;

    NFX_ASSERT(argc == 2);
    
    stateid.id = strtoq(argv[1], NULL, 0);
    //printf("stateid.id = 0x%llx\n", stateid.id);

    /* Prepare the request.
     */
    reqSize = sizeof(fssf_closeReq_t);
    //req = (fssf_closeReq_t *) calloc(reqSize, 1);
    req = (fssf_closeReq_t *) alloc_msg (FSSF_CLOSE_REQ, reqSize, NULL);
    req->hdr.clientPrivate = 0;
    req->hdr.protocol = FS_PROTO_CIFS;
    req->stateid = stateid;

    return((fs_MsgReqHdr *) req);
}

/*------------------------------------------------------------------------------
 * Name         : fssfClose_response
 *
 * Description  : Close a file.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
fssfClose_response(int status, fs_MsgRespHdr * hdr)
    
{
    uint32 *    msgType = NULL;

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    
    switch(*msgType)
    {
    case FSSF_CLOSE_RSP:
    {
        fssf_closeRsp_t *   rsp = (fssf_closeRsp_t *) hdr;

        printf("FSSF_CLOSE_RSP status = %d\n", rsp->status);
        if (rsp->status != NFX_OK)
        {
            break;
        }
        printf("Done.\n");
        break;
    } // case FSSF_CLOSE_RSP

    default:
    {
        break;
    }    
    } // switch
    
    return(NULL);
}

/*------------------------------------------------------------------------------
 * Name         : cmd_fssfGetattr
 *
 * Description  : Get state-full file attributes.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
cmd_fssfGetattr(int argc, char *argv[])

{
    fssf_stateid_t      stateid;
    uint32              reqSize;
    fssf_getattrReq_t * req = NULL;

    NFX_ASSERT(argc == 2);
    
    stateid.id = strtoq(argv[1], NULL, 0);
    //printf("stateid.id = 0x%llx\n", stateid.id);

    /* Prepare the request.
     */
    reqSize = sizeof(fssf_getattrReq_t);
    req = (fssf_getattrReq_t *) alloc_msg (FSSF_GETATTR_REQ, reqSize, NULL);
    req->hdr.clientPrivate = 0;
    req->hdr.protocol = FS_PROTO_CIFS;
    req->stateid = stateid;

    return((fs_MsgReqHdr *) req);
}

/*------------------------------------------------------------------------------
 * Name         : fssfGetattr_response
 *
 * Description  : Get state-full file attributes.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static fs_MsgReqHdr *
fssfGetattr_response(int status, fs_MsgRespHdr * hdr)
    
{
    uint32 *    msgType = NULL;

    NFX_ASSERT(hdr != NULL);
    msgType = (uint32 *) hdr;
    
    switch(*msgType)
    {
    case FSSF_GETATTR_RSP:
    {
        fssf_getattrRsp_t * rsp = (fssf_getattrRsp_t *) hdr;

        printf("FSSF_GETATR_RSP status = %d\n", rsp->status);
        if (rsp->status != NFX_OK)
        {
            break;
        }
        printf("currPos = %lld\n", rsp->currPos);
        printf("shareAccess = 0x%x\n", rsp->shareAccess);
        printf("shareDeny = 0x%x\n", rsp->shareDeny);
        printf("filehandle = 0x%llx/0x%llx/%d/%d/0x%llx\n",
               rsp->filehandle.volId.whole_thing,
               rsp->filehandle.inum,
               rsp->filehandle.gen,
               rsp->filehandle.mkfsTime,
               rsp->filehandle.mntPnt);
        printf("Done.\n");
        break;
    } // case FSSF_GETATR_RSP

    default:
    {
        break;
    }    
    } // switch
    
    return(NULL);
}




/*----------------------------------------------------------------------------*/


#if defined(SSC)

fs_MsgReqHdr *
fscmd_parse_opts(int argc, char **argv)
{
    int     i;
#define FSCMD_ARGS_LINE_MAX     1100
    char    inputLine[FSCMD_ARGS_LINE_MAX];

    // Reconstruct the input line so the standalone shell
    // parser can do its thing.
    
    // Eat 'fscmd'.
    argc--;
    argv++;

    for (   i = 0;
            i < FSCMD_ARGS_LINE_MAX && argc;
            argv++, argc--)
    {
        strncpy(    &inputLine[i],
                    *argv,
                    FSCMD_ARGS_LINE_MAX - i);
        
        i += strlen(*argv);
        inputLine[i++] = ' ';
    }

    if (i >= FSCMD_ARGS_LINE_MAX)
    {
        fprintf(stderr, "input args too long\n");
        return NULL;
    }

    return fs_shell_parsecmd(inputLine, &cmdRespHandler);
}


/**
 * Main routine.
 */
int
main(int argc, char **argv)
{
    fs_MsgReqHdr        *reqHdr;
    fs_MsgRespHdr       *respHdr;
    int                 rc;
    void                *respBuf;
    struct timeval      timeout = {600, 0};
    struct timeval      *clusTimeout;
    ncmMsg_t            *ncmReq;
    uint32              ncmReqSize;

    // This is where the SSC command line interface command for
    // the fs begins.   There's only a single binary, fscmd, which
    // takes arguments to direct its actions.  It is intended that
    // shell scripts (eg. mkfs) will provide the appropriate args
    // to fscmd.

    if (argc <= 1)
    {
        cmd_help (argc, argv);
        exit(0);
    }

    get_applications();

    reqHdr = fscmd_parse_opts(argc, argv);
    if (reqHdr == NULL)
        exit(2);

    do {
        respBuf = malloc(FS_SHELL_RESP_MAX + sizeof (ncmMsg_t));
        if (respBuf == NULL) {
            fprintf(stderr, "failed to allocate response buffer\n");
            exit(2);
        }

        clusTimeout = ((reqHdr->requestType == FS_REQ_EEK) ||
                       (reqHdr->requestType == FS_REQ_EEKCONT) ||
                       (reqHdr->requestType == FS_REQ_QUOTA_ON)) ?
                      NULL : &def_clus_timeout;
        ncmReq = (ncmMsg_t *) CONTAINING_RECORD(reqHdr, ncmMsg_t, appMsg); 

        if (NCM_REQUEST(ncmReq->msgType)) {
            /*
             * We will need to route this request through the ncm rather than 
             * directly to the application on the current node.
             */
            ncmReqSize = reqHdr->len + sizeof (ncmMsg_t);
            rc = sendAgileMsg (-1, ncmReq, ncmReqSize, "ncm", NFX_SSC_UNIX, 
                               SSC_SLOT, MGMTPLANE, clusTimeout, respBuf, 
                               FS_SHELL_RESP_MAX + sizeof (ncmMsg_t)); 
        } else 
            if ((reqHdr->requestType >= ELOG_CONFIGURE_ENABLE) && 
                (reqHdr->requestType <= (ELOG_CONFIGURE_ENABLE+31))) {
            timeout.tv_sec = 0;
            rc = sendAgileMsg(  -1,             // system assigned tmp socket
                            reqHdr,
                            reqHdr->len,
                            "e_log",
                            //NFX_NFP_FP,       // Either cpu
                            NFX_NFP_FP2,        // LINUX_TEST use this fp now
                            1,                  // TODO - identify correct slot
                            MGMTPLANE,          // Mgmt plane
                            &timeout,
                            respBuf,
                            FS_SHELL_RESP_MAX);
        } else if (reqHdr->requestType >= FS_REQ_DUMP_SETUP &&
             reqHdr->requestType <= FS_REQ_REST_IODONE) {
            printf("sending message\n");
            rc = sendAgileMsg(  -1,             // system assigned tmp socket
                            reqHdr,
                            reqHdr->len,
                            "efs",
                            //NFX_NFP_FP,       // Either cpu
                            NFX_NFP_FP2,        // LINUX_TEST use this fp now
                            1,                  // TODO - identify correct slot
                            MGMTPLANE,          // Mgmt plane
                            &timeout,
                            NULL,
                            FS_SHELL_RESP_MAX);
            printf("sent message\n");
        } else if ((reqHdr->requestType == FS_REQ_EEK) ||
             (reqHdr->requestType == FS_REQ_EEKCONT) ||
             (reqHdr->requestType == FS_REQ_QUOTA_ON)) {
            rc = sendAgileMsg(  -1,             // system assigned tmp socket
                            reqHdr,
                            reqHdr->len,
                            "efs",
                            NFX_NFP_FP2,        // LINUX_TEST use this fp now
                            1,                  // TODO - identify correct slot
                            MGMTPLANE,          // Mgmt plane
                            NULL,               // no timeout - eek & quota on
                                                // take a long time
                            respBuf,
                            FS_SHELL_RESP_MAX);
        } else {
            rc = sendAgileMsg(  -1,             // system assigned tmp socket
                            reqHdr,
                            reqHdr->len,
                            "efs",
                            NFX_NFP_FP2,        // LINUX_TEST use this fp now
                            1,                  // TODO - identify correct slot
                            MGMTPLANE,          // Mgmt plane
                            &timeout,
                            respBuf,
                            FS_SHELL_RESP_MAX);
        }

        if (rc <= 0) {
            perror("sendAgileMsg");
            fprintf(stderr, "command failed to complete (rc: %d)\n", rc);
            exit(2);
        }

        if (NCM_REQUEST(ncmReq->msgType)) {
            respHdr = (fs_MsgRespHdr *) ((ncmMsg_t *) respBuf)->appMsg;
        } else {
            respHdr = respBuf;
        }
        if (respHdr->fsStatus != NFX_OK)
        {
            fprintf(stderr, "%s (%u)\n", fs_errCodeToString(respHdr->fsStatus),
                                        respHdr->fsStatus);
            exit(2);
        }

        fs_ramDealloc (FS_RAM_SIM_MISC, ncmReq);

        reqHdr = cmdRespHandler(respHdr->fsStatus, respHdr);

        fs_ramDealloc (FS_RAM_SIM_MISC, respBuf); 

    } while (reqHdr != NULL);

    exit(0);

} /* main */

#endif // SSC
